#include "stdafx.h"

#include <math.h>

#include "angles.h"

#include "basics/definitions.h"


// converts orientation to [0,range) range
float canonical_range (float orientation, float range)
{
  float result;
  if (orientation < 0.0f)
  {
    float factor = floor(orientation/range);
    result = orientation - factor*range;
  }
  else
  {
    float factor = floor(orientation/range);
    result = orientation - factor*range;
  }

  // handle numerical errors, for example when orientation = -0.000000001
  if (result < 0.0f)
  {
    result += range;
  }

  if (result >= range)
  {
    result -= range;
  }

  if ((result < 0.0f) || (result >= range))
  {
    exit_error ("\nerror: canonical_orientation (%f) = %f\n", orientation, result);
  }

  return result;
}


// converts orientation to [0,180) range
float canonical_orientation_half (float orientation)
{
  float result = canonical_range(orientation, 180.0f);
  return result;
}


// converts orientation to [0,360) range
float canonical_orientation (float orientation)
{
  float result = canonical_range(orientation, 360.0f);
  return result;
}


float make_degrees_canonical (float radiant_argument)
{
  float first_result = make_degrees (radiant_argument);
  float result = canonical_orientation (first_result);
  return result;
}


float make_radians_canonical (float degree_argument)
{
  float canonical = canonical_orientation (degree_argument);
  float result = make_radians (canonical);
  return result;
}


float signed_degrees_distance(float first, float second)
{
  return signed_circular_distance(first, second, 360.0f);
}


float signed_radians_distance(float first, float second)
{
  return signed_circular_distance(first, second, M_2PIf);
}


// converts orientation to [0,360) range
float canonical_degrees (float degrees)
{
  return canonical_range(degrees, 360.0f);
}


// converts orientation to [0, 2*pi) range
float canonical_radians (float radians)
{
  return canonical_range(radians, M_2PIf);
}


float degrees_distance(float first_degrees, float second_degrees)
{
  return absolute_circular_distance(first_degrees, second_degrees, 360.0f);
}


float radians_distance(float first_radians, float second_radians)
{
  return absolute_circular_distance(first_radians, second_radians, M_2PIf);
}


float degrees_distance2(float first_degrees, float second_degrees)
{
  return absolute_circular_distance(first_degrees, second_degrees, 180.0f);
}


float radians_distance2(float first_radians, float second_radians)
{
  return absolute_circular_distance(first_radians, second_radians, M_PIf);
}


float orientation_average(float first_radians, float second_radians)
{
  float first_canonical = canonical_radians(first_radians);
  float second_canonical = canonical_radians(second_radians);

  float smaller = function_minimum(first_canonical, second_canonical);
  float larger = function_maximum(first_canonical, second_canonical);
  float result;

  if (larger - smaller <= M_PIf)
  {
    result = (larger + smaller) / 2.0f;
  }
  else
  {
    result = (larger + smaller) / 2.0f + M_PIf;
  }

  result = canonical_radians(result);
  return result;
}

float orientation_average(float first_radians, float second_radians,float wght)
{
  float first_canonical = canonical_radians(first_radians);
  float second_canonical = canonical_radians(second_radians);

  float smaller = function_minimum(first_canonical, second_canonical);
  float larger = function_maximum(first_canonical, second_canonical);
  float result;

  if (larger - smaller <= M_PIf)
  {
    result = (first_canonical*wght + second_canonical*(1-wght));
  }
  else
  {
    result = (first_canonical*wght + second_canonical*(1-wght)); + M_PIf;
  }

  result = canonical_radians(result);
  return result;
}



