#include "stdafx.h"

#include <time.h>

#include "base_interpreter.h"
#include "basics/auxiliaries.h"
#include "local_data.h"
#include "algebra.h"
#include "precomputed.h"
#include "simple_algo.h"
#include "io_aux.h"

#include "basics/definitions.h"


const char * platform_face_detector_path = platform_face_detector_path1;
const char * platform_skin_directory = platform_skin_directory1;
const char * platform_non_skin_directory = platform_non_skin_directory1;
const char * crl_skin_file = crl_skin_file1;
const char * crl_non_skin_file = crl_non_skin_file1;
const char * sequences_directory = sequences_directory1;
char * code_directory = vCopyString(code_directory1);
char * data_directory = vCopyString(data_directory1);
char * nessie = vCopyString(nessie1);
char * nessied = vCopyString(nessied1);



class_interpreter::class_interpreter()
{
  initialize ();
}


class_interpreter::~class_interpreter()
{
  clean_up();
}


long class_interpreter::initialize()
{
  input_file_pointer = 0;

  // The goal here is to instantiate some template functions, so
  // that I can call them while debugging.
  vMatrix<uchar> junk1(1,1);
  vMatrix<long> junk2(1,1);
  vMatrix<float> junk3(1,1);
  vMatrix<double> junk4(1,1);
  long a = 0;
  if (a)
  {
    junk1.Print(0);
    junk1.PrintRange(1,1,1,1,0);
    junk1.PrintInt(0);
    junk1.PrintRangeInt(1,1,1,1,0);
    junk1.PrintRangem(1,1,1,1,0);
    junk1.PrintRangeIntm(1,1,1,1,0);

    junk1.PrintTrans(0);
    junk1.PrintRangeTrans(1,1,1,1,0);
    junk1.PrintIntTrans(0);
    junk1.PrintRangeIntTrans(1,1,1,1,0);
    junk1.PrintRangemTrans(1,1,1,1,0);
    junk1.PrintRangeIntmTrans(1,1,1,1,0);
    junk1.WriteDebug("trash");

    junk2.Print(0);
    junk2.PrintRange(1,1,1,1,0);
    junk2.PrintInt(0);
    junk2.PrintRangeInt(1,1,1,1,0);
    junk2.PrintRangem(1,1,1,1,0);
    junk2.PrintRangeIntm(1,1,1,1,0);

    junk2.PrintTrans(0);
    junk2.PrintRangeTrans(1,1,1,1,0);
    junk2.PrintIntTrans(0);
    junk2.PrintRangeIntTrans(1,1,1,1,0);
    junk2.PrintRangemTrans(1,1,1,1,0);
    junk2.PrintRangeIntmTrans(1,1,1,1,0);
    junk2.WriteDebug("trash");

    junk3.Print(0);
    junk3.PrintRange(1,1,1,1,0);
    junk3.PrintInt(0);
    junk3.PrintRangeInt(1,1,1,1,0);
    junk3.PrintRangem(1,1,1,1,0);
    junk3.PrintRangeIntm(1,1,1,1,0);

    junk3.PrintTrans(0);
    junk3.PrintRangeTrans(1,1,1,1,0);
    junk3.PrintIntTrans(0);
    junk3.PrintRangeIntTrans(1,1,1,1,0);
    junk3.PrintRangemTrans(1,1,1,1,0);
    junk3.PrintRangeIntmTrans(1,1,1,1,0);
    junk3.WriteDebug("trash");

    junk4.Print(0);
    junk4.PrintRange(1,1,1,1,0);
    junk4.PrintInt(0);
    junk4.PrintRangeInt(1,1,1,1,0);
    junk4.PrintRangem(1,1,1,1,0);
    junk4.PrintRangeIntm(1,1,1,1,0);

    junk4.PrintTrans(0);
    junk4.PrintRangeTrans(1,1,1,1,0);
    junk4.PrintIntTrans(0);
    junk4.PrintRangeIntTrans(1,1,1,1,0);
    junk4.PrintRangemTrans(1,1,1,1,0);
    junk4.PrintRangeIntmTrans(1,1,1,1,0);
    junk4.WriteDebug("trash");
  }

  input_file_pointer = 0;
  
  return 1;
}


// An interesting thing with the CleanUp functions of each interpreter
// (i.e. interpreter1, 2, 3), is that each of them gets called by
// the destructor of that class. I don't understand why that happens.
// I would expect that, since the CleanUp functions are all virtual,
// only the CleanUp function of the lowest subclass (interpreter3
// currently) should be called. I don't know if the way things work 
// now is according to the C++ standard or if it is a Virtual Studio bug.
long class_interpreter::clean_up()
{
  vEraseCharpVector(&module_names);
  vDeletePrecomputedValues();

  if (input_file_pointer != 0)
  {
    fclose(input_file_pointer);
  }

  vdelete2(code_directory);
  vdelete2(data_directory);
  vdelete2(nessie);
  vdelete2(nessied);
  
  return 1;
}


long class_interpreter::vScan(const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result;
  if (input_file_pointer == 0)
  {
    vConsoleToForeground ();
    result = vVScan(format, arguments);
  }
  else
  {
    result = vFscan(input_file_pointer, format, arguments);
  }
  va_end(arguments);
  return result;
}


// speech-recognition-friendly version of vprint
long class_interpreter::print (const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result = vVprint(format, arguments);
  va_end(arguments);
  return result;
}


// speech-recognition-friendly version of vScan
long class_interpreter::question (const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result;
  if (input_file_pointer == 0)
  {
    vConsoleToForeground ();
    result = vVScan(format, arguments);
  }
  else
  {
    result = vFscan(input_file_pointer, format, arguments);
  }
  va_end(arguments);
  return result;
}


long class_interpreter::process_command(const char * command)
{
  if (command == 0) return 0;

  if (command[0] == 0) return 0;
  
  if (strcmp(command, "file") == 0)
  {
    process_command_file();
    return 1;
  }

  if (strcmp(command, "system") == 0)
  {
    char system_command[1000];
    vGetNonEmptyLine(system_command, 990);
    system(system_command);
    return 1;
  }

  if (strcmp(command, "x") == 0)
  {
    // 2 signifies we should exit.
    vMainWindowToForeground();
    return 2;
  }

  long number = modules.size();
  long success = 0;
  long i;
  for (i = 0; i < number; i++)
  {
    long success = modules[i]->process_command(command);
    if (success == 1)
    {
      return 1;
    }
  }

  // we only get here if success is 0.
  vPrint("??? %s is not a recognized command\n", command);
  
  return 1;
}


// This command allows us to process text commands that are
// stored in a file.
long class_interpreter::process_command_file()
{
  char buffer[500];
  vPrint("Input name of command file: ");
  vScan("%s", buffer);
  return process_command_file1(buffer);
}


long class_interpreter::process_command_file1(const char * simple_file_name)
{
  // We require that the file is in the directory code_directory/commands 
  // and we add the ".txt" extension to it.
  char * temp = vJoinPaths(code_directory, "commands");
  char * temp2;
  if (vStringEndsIn(simple_file_name, ".txt"))
  {
    temp2 = vCopyString(simple_file_name);
  }
  else
  {
    temp2 = vMergeStrings2(simple_file_name, ".txt");
  }
  char * file = vJoinPaths(temp, temp2);
  vdelete2(temp);
  vdelete2(temp2);

  // Possibly we are already executing commands from another
  // file and one of those asks us to execute this file. Then,
  // we must save the previous FILE * object, process the 
  // new file, and then restore the previous file.
  FILE * previous_input_file_pointer = 0;
  // If we are processing another command file, save the status
  // of that file
  if (input_file_pointer != 0) 
  {
    previous_input_file_pointer = input_file_pointer;
  }
  // Open the requested file, return failure if we can't open it.
  input_file_pointer = fopen(file, vFOPEN_READ);
  if (input_file_pointer == 0)
  {
    vPrint("Can't find input file %s\n", file);
    input_file_pointer = previous_input_file_pointer;
    vdelete2(file);
    return 0;
  }

  vdelete2(file);
  // Process commands from the file until we read all the contents
  // of the file. If there is nothing more to read, we are done
  while(1)
  {
    char command[100];
    long items = vScan("%s", command);
    // If we reached the end of file (or a read error has occured)
    // we are done.
    if (items != 1) break;
    process_command(command);
  }
  fclose(input_file_pointer);
  // Restore the previous file (if any)
  input_file_pointer = previous_input_file_pointer;
  return 1;
}


long class_interpreter::enter_module(class_module * module, char * name)
{
  modules.push_back (module);
  module_names.push_back (function_copy_string (name));
  return 1;
}


class_module * class_interpreter::get_module(char * name)
{
  class_module * result = 0;
  long number = module_names.size ();
  long counter;

  for (counter = 0; counter < number; counter ++)
  {
    if (strcmp (name, module_names [counter]) == 0)
    {
      result = modules [counter];
      break;
    }
  }

  return result;
}


long class_interpreter::check_command_duplicates()
{
  long module_number = modules.size ();
  vector <char*> commands;
  
  long counter;
  for (counter = 0; counter <module_number; counter ++)
  {
    modules [counter]->get_commands (& commands);
  }

  sort (commands.begin (), commands.end (), CharpLess());
  long number = commands.size ();
  vPrint ("%li commands found.\n", number);

  long flag = 0;
  for (counter = 0; counter < number-1; counter ++)
  {
    if (function_compare_strings (commands [counter], commands [counter +1]) == 0)
    {
      vPrint ("duplicate command: %s\n", commands [counter]);
      flag = 1;
    }
  }

  if (flag == 0)
  {
    vPrint ("no duplicates were found\n");
  }

  return 1;
}

  
long class_interpreter::list_commands()
{
  long module_number = modules.size ();
  vector <char*> commands;
  
  long counter;
  for (counter = 0; counter <module_number; counter ++)
  {
    modules [counter]->get_commands (& commands);
  }

  sort (commands.begin (), commands.end (), CharpLess());
  long number = commands.size ();
  vPrint ("%li commands found.\n", number);

  for (counter = 0; counter < number-1; counter ++)
  {
    vPrint ("%s\n", commands[counter]);
  }

  return 1;
}

  
long class_interpreter::main_loop(long argc, char ** argv)
{
  // If there was a command-line argument, we need to just process a script file.
  if (argc > 1)
  {
    if (argc != 2)
    {
      exit_error("usage: %s [script_file]\n", argv[0]);
    }

    const char * script_file = argv[1];
    process_command_file1(script_file);
    return 0;
  }

  // If no arguments were passed, then we just start the interactive
  // loop.
  char command[1000];
  while(1)
  {
    vPrint(">> ");
    vScan("%s", command);
    long return_code = process_command(command);
    
    // if the user decided to exit...
    if (return_code == 2) 
    {
      break;
    }
  }

  return 0;
}


class_module::class_module(class_interpreter * interpreter)
{
  initialize();
  base_interpreter = interpreter;
}


class_module::~class_module()
{
  clean_up();
  initialize();
}

long class_module::initialize()
{
  base_interpreter = 0;
  
  return 1;
}

long class_module::clean_up()
{
  vEraseCharpVector(&commands);
  
  return 1;
}


long class_module::get_commands (std::vector<char*> * strings)
{
  strings->insert (strings->end (), commands.begin (), commands.end ());
  return 1;
}


long class_module::command_index(const char * command)
{
  long result = -1;
  long number = commands.size();
  
  long i;
  for (i = 0; i < number; i++)
  {
    if (strcmp(command, commands[i]) == 0)
    {
      result = i;
      break;
    }
  }
  
  return result;
}


long class_module::vScan(const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result;
  if (base_interpreter->input_file_pointer == 0)
  {
    vConsoleToForeground ();
    result = vVScan(format, arguments);
  }
  else
  {
    result = vFscan(base_interpreter->input_file_pointer, format, arguments);
  }
  va_end(arguments);
  return result;
}


// speech-recognition-friendly version of vprint
long class_module::print (const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result = vVprint(format, arguments);
  va_end(arguments);
  return result;
}


// speech-recognition-friendly version of vScan
long class_module::question (const char * format, ...)
{
  va_list arguments;
  va_start(arguments, format);
  long result;
  if (base_interpreter->input_file_pointer == 0)
  {
    vConsoleToForeground ();
    result = vVScan(format, arguments);
  }
  else
  {
    result = vFscan(base_interpreter->input_file_pointer, format, arguments);
  }
  va_end(arguments);
  return result;
}



//////////////////////////////////////////////////////////
//                                                      //
// start of obligatory member functions for any module  //
//                                                      //
//////////////////////////////////////////////////////////

class_base_module::class_base_module(class_interpreter * interpreter) :
  class_module(interpreter)
{
  initialize();
  sort_commands ();
}


class_base_module::~class_base_module()
{
  class_module::clean_up();
  clean_up();
}

long class_base_module::initialize()
{
  enter_command("c_code_dir", &class_base_module::c_code_dir);
  enter_command("c_data_dir", &class_base_module::c_data_dir);

  enter_command("d_code_dir", &class_base_module::d_code_dir);
  enter_command("d_data_dir", &class_base_module::d_data_dir);

  enter_command("l_code_dir", &class_base_module::l_code_dir);
  enter_command("l_data_dir", &class_base_module::l_data_dir);
  enter_command("list_commands", &class_base_module::list_all_commands);

  enter_command("randomize", &class_base_module::randomize);
  enter_command("r_code_dir", &class_base_module::r_code_dir);
  enter_command("r_data_dir", &class_base_module::r_data_dir);

  enter_command("s_data_dir", &class_base_module::s_data_dir);
  enter_command("set_code_dir", &class_base_module::set_code_dir);
  enter_command("set_data_dir", &class_base_module::set_data_dir);
  enter_command("set_nessie", &class_base_module::set_nessie);
  enter_command("system", &class_base_module::system_function);
  

  return 1;
}

long class_base_module::clean_up()
{
  vEraseCharpVector(&commands);
  
  return 1;
}


long class_base_module::enter_command(char * command, type_procedure procedure)
{
  commands.push_back(function_copy_string (command));
  procedures.push_back(procedure);
  return 1;
}


long class_base_module::process_command(const char * command)
{
  long index = command_index (command);
  if (index < 0)
  {
    return 0;
  }
  type_procedure procedure = procedures [index];
  (this->*procedure)();

  return 1;
}


long class_base_module::sort_commands()
{
  vector <long> indices;
  function_string_ranks (&commands, & indices);
  
  vector <char*> temporary;
  temporary.insert(temporary.end(), commands.begin (), commands.end ());
  vector <type_procedure> temporary_procedures;
  temporary_procedures.insert(temporary_procedures.end(), procedures.begin (), procedures.end ());

  long number = commands.size ();
  long counter;
  for (counter = 0; counter < number; counter ++)
  {
    long index = indices [counter];
    commands[counter] = temporary [index];
    procedures [counter] = temporary_procedures [index];
  }

  return 1;
}


//////////////////////////////////////////////////////////
//                                                      //
//  end of obligatory member functions for any module   //
//                                                      //
//////////////////////////////////////////////////////////

long class_base_module::randomize()
{
  srand(time(0));
  return 1;
}



long class_base_module::c_code_dir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("c:\\users\\athitsos\\code\\source");
  return 1;
}


long class_base_module::d_code_dir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("d:\\users\\athitsos\\code\\source");
  return 1;
}


long class_base_module::r_code_dir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("r:\\code\\source");
  return 1;
}


long class_base_module::l_code_dir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("/home/home7/athitsos/code/source");
  return 1;
}


long class_base_module::c_data_dir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("c:\\users\\athitsos\\data");
  return 1;
}


long class_base_module::d_data_dir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("d:\\users\\athitsos\\data");
  return 1;
}

long class_base_module::r_data_dir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("r:\\nessie\\data");
  return 1;
}


long class_base_module::s_data_dir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("S:\\nessie\\data");
  return 1;
}


long class_base_module::l_data_dir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("/home/home7/athitsos/nessie/data");
  return 1;
}


long class_base_module::set_code_dir()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(code_directory);
  code_directory = vCopyString(buffer);
  return 1;
}


long class_base_module::set_data_dir()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(data_directory);
  data_directory = vCopyString(buffer);
  return 1;
}


long class_base_module::set_nessie()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(nessie);
  nessie = vCopyString(buffer);
  return 1;
}


long class_base_module::system_function ()
{
  char system_command[1000];
  vGetNonEmptyLine(system_command, 990);
  system(system_command);

  return 1;
}


long class_base_module::list_all_commands()
{
  base_interpreter->list_commands();
  return 1;
}
