#ifndef VASSILIS_BASE_INTERPRETER_H
#define VASSILIS_BASE_INTERPRETER_H

#include <stdio.h>
#include <vector>

#include "auxiliaries.h"

class class_module;

class class_interpreter
{
private:
  std::vector<char *> module_names;
  std::vector<class_module *> modules;

public:
  FILE * input_file_pointer;

public:
  class_interpreter();
  ~class_interpreter();

  long initialize();
  long clean_up();
  
  long process_command(const char * command);
  long process_command_file();
  long process_command_file1(const char * filename);
  long vScan(const char * format, ...);
  long print (const char * format, ...);
  long question (const char * format, ...);

  long enter_module(class_module * module, char * name);
  class_module * get_module(char * name);
  long check_command_duplicates();
  long list_commands();
  long main_loop(long argc, char ** argv);
};


class class_module
{
protected:
  class_interpreter * base_interpreter;
  std::vector<char *> commands;
  typedef long (class_module::*type_procedure) ();

protected:

  // the virtual functions should be set to 0, so that class_module is abstract.
  // however, I wanted to provide prototypes for these functions, that I can
  // copy, paste and modify when I define a subclass.
  virtual long initialize();
  virtual long clean_up();
  
public:
  class_module(class_interpreter * interpreter);
  virtual ~class_module();

  long enter_command(char * command, type_procedure procedure);
  long get_commands (std::vector<char*> * strings);
  virtual long process_command(const char * command) = 0;

  long command_index(const char * command);
  long vScan(const char * format, ...);
  long print (const char * format, ...);
  long question (const char * format, ...);
};


class class_base_module : public class_module
{
protected:
  // the next two lines are mandatory for any module
  typedef long (class_base_module::*type_procedure) ();
  std::vector<type_procedure> procedures;

public:

  //////////////////////////////////////////////////////////
  //                                                      //
  // start of obligatory member functions for any module  //
  //                                                      //
  //////////////////////////////////////////////////////////

  class_base_module(class_interpreter * interpreter);
  virtual ~class_base_module();
  
  virtual long initialize();
  virtual long clean_up();
  
  long enter_command(char * command, type_procedure procedure);
  virtual long process_command(const char * command);
  long sort_commands ();

  //////////////////////////////////////////////////////////
  //                                                      //
  // end of obligatory member functions for any module  //
  //                                                      //
  //////////////////////////////////////////////////////////

  long randomize();
  long c_code_dir();
  long d_code_dir();
  long l_code_dir();
  long r_code_dir();

  long c_data_dir();
  long d_data_dir();
  long l_data_dir();
  long r_data_dir();
  long s_data_dir();

  long set_code_dir();
  long set_data_dir();
  long set_nessie();
  long system_function();
  long list_all_commands();
};


#endif // VASSILIS_BASE_INTERPRETER_H
