#include "stdafx.h"

#ifdef VASSILIS_SGI_PLATFORM
#define VASSILIS_PROCESS_FILE
#endif // VASSILIS_SGI_PLATFORM

#ifdef VASSILIS_PROCESS_FILE
#undef VASSILIS_PROCESS_FILE

#include "drawing.h"
#include "drawing_temp.h"
#include "matrix.h"

template<class type>
static vArray2(uchar) S_PolygonMask(const v3dMatrix<type> * source, 
                                    const vPolygon * polygon);

template<class type>
v3dMatrix<type> * vCopyImage(const v3dMatrix<type> * source)
{
  const long rows = source->Rows();
  const long cols = source->Cols();
  const long bands = source->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vCopyImage(result, source);
  return result;
}

template<class type>
ushort vCopyImage(v3dMatrix<type> * target, const v3dMatrix<type> * source)
{
  long byte_size = source->AllBandSize() * sizeof(type);
  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();
  assert(rows == target->Rows());
  assert(cols == target->Cols());
  assert(bands == target->Bands());
  
  vArray(type) source_matrix = source->Matrix();
  vArray(type) target_matrix = target->Matrix();
  memcpy(vData(target_matrix), vData(source_matrix), byte_size);
  
  return 1;
}


template<class type>
v3dMatrix<type> * vCopyBand2(const v3dMatrix<type> * source, const long band)
{
  if ((band < 0) || (band >= source->Bands()))
  {
    return 0;
  }
  long rows = source->Rows();
  long cols = source->Cols();
  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, 1);
  long success = vCopyBand3(result, source, band);
  if (success <= 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}


template<class type>
long vCopyBand3(v3dMatrix<type> * target, const v3dMatrix<type> * source, const long band)
{
  if ((band < 0) || (band >= source->Bands()))
  {
    return 0;
  }
  long size = source->Size();
  if (size != target->Size()) return 0;
  vArray(type) target1 = target->Matrix();
  vArray(type) source1 = source->Matrix(band);
  long byte_size = sizeof(type) * source->Size();
  memcpy(vData(target1), vData(source1), byte_size);
  return 1;
}


template<class type>
long function_copy_band(v3dMatrix<type> * target, const long target_band, 
                        const v3dMatrix<type> * source, const long source_band)
{
  if ((source_band < 0) || (source_band >= source->Bands()))
  {
    return 0;
  }
  if ((target_band < 0) || (target_band >= target->Bands()))
  {
    return 0;
  }
  long size = source->Size();
  if (size != target->Size()) 
  {
    return 0;
  }
  vArray(type) target1 = target->Matrix(target_band);
  vArray(type) source1 = source->Matrix(source_band);
  long byte_size = sizeof(type) * source->Size();
  memcpy(vData(target1), vData(source1), byte_size);
  return 1;
}

template<class type>
v3dMatrix<type> * vCopyAndDrawRectangle(const v3dMatrix<type> * source, 
					                                 const long y1, const long x1, const long y2, const long x2,
					                                 const uchar r, const uchar g, const uchar b)
{
  v3dMatrix<type> * result = vCopyImage(source);
  vDrawRectangle(result, y1, x1, y2, x2, r, g, b);
  return result;
}

template<class type>
ushort vDrawRectangle1(v3dMatrix<type> * the_image,
		                  const long y1, const long x1, const long y2, const long x2,
		                  const type value)
{
  vector<type> values;
  long bands = the_image->Bands();
  long i;
  for (i = 0; i < bands; i++)
    values.push_back(value);
  return vDrawRectangle(the_image, y1, x1, y2, x2, values);
}


template<class type>
ushort vDrawRectangle(v3dMatrix<type>* the_image,
		                 const long y1, const long x1, const long y2, const long x2,
		                 const vector<type> & values)
{
  long i1 = Min(y1, y2);
  long i2 = Max(y1, y2);
  long j1 = Min(x1, x2);
  long j2 = Max(x1, x2);

  // Check that endpoints lie within the image.
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  assert(bands == values.size());
  if (i1 < 0) i1 = 0;
  else if (i1 >= rows) i1 = rows-1;
  
  if (i2 < 0) i2 = 0;
  else if (i2 >= rows) i2 = rows-1;
  
  if (j1 < 0) j1 = 0;
  else if (j1 >= cols) j1 = cols-1;

  if (j2 < 0) j2 = 0;
  else if (j2 >= cols) j2 = cols-1;
	   
  long row, col, band;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix = the_image->Matrix2(band);
    type value = values[band];
    for (row = i1; row <= i2; row++)
    {
      matrix[row][j1] = value;
      matrix[row][j2] = value;
    }

  for (col = j1; col <= j2; col++)
    {
      matrix[i1][col] = value;
      matrix[i2][col] = value;
    }
  }
  return 1;
}


template<class type>
ushort vDrawFilledRectangle1(v3dMatrix<type> * the_image,
			                     const long y1, const long x1, const long y2, const long x2,
			                     const type value)
{
  vector<type> values;
  long i;
  long bands = the_image->Bands();
  for (i = 0; i < bands; i++)
    values.push_back(value);
  return vDrawFilledRectangle(the_image, y1, x1, y2, x2, values);
}

  
template<class type>
ushort vDrawFilledRectangle(v3dMatrix<type> * the_image,
			                     const long y1, const long x1, const long y2, const long x2,
			                     const vector<type> & values)
{
  long i1 = Min(y1, y2);
  long i2 = Max(y1, y2);
  long j1 = Min(x1, x2);
  long j2 = Max(x1, x2);

  // Check that endpoints lie within the image.
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  assert(bands == values.size());
  if (i1 < 0) i1 = 0;
  else if (i1 >= rows) i1 = rows-1;
  
  if (i2 < 0) i2 = 0;
  else if (i2 >= rows) i2 = rows-1;
  
  if (j1 < 0) j1 = 0;
  else if (j1 >= cols) j1 = cols-1;

  if (j2 < 0) j2 = 0;
  else if (j2 >= cols) j2 = cols-1;
	   
  long row, col, band;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix = the_image->Matrix2(band);
    type value = values[band];
    for (row = i1; row <= i2; row++)
      for (col = j1; col <= j2; col++)
	      matrix[row][col] = value;
  }
  return 1;
}


template<class type>
ushort vDrawSquare(v3dMatrix<type> * the_image, const long y, const long x, const double angle, const long size,
		               const vector<type> & values)
{
  angle = -angle; // This adjusts the co-ordinates;
  double diag1 = angle + M_PI / 4;
  double diag2 = angle - M_PI / 4;
  double diag_length = sqrt(2) * ((double) size) / 2.0;
  double x0, y0, x1, y1, x2, y2, x3, y3;
  vLocatePoint(y, x, diag1, diag_length, y0, x0);
  vLocatePoint(y, x, diag2, -diag_length, y1, x1);
  vLocatePoint(y, x, diag1, -diag_length, y2, x2);
  vLocatePoint(y, x, diag2, diag_length, y3, x3);
  
  vDrawLine(the_image, y0, x0, y1, x1, values);
  vDrawLine(the_image, y1, x1, y2, x2, values);
  vDrawLine(the_image, y2, x2, y3, x3, values);
  vDrawLine(the_image, y3, x3, y0, x0, values);
  return 1;
}


template<class type>
ushort vDrawCircle1(v3dMatrix<type> * source, const long y1, const long x1, const long radius,
			              const type value)
{
  vector<type> values;
  long i;
  long bands = source->Bands();
  for (i = 0; i < bands; i++)
    values.push_back(value);
  vDrawCircle(source, y1, x1, radius, values);
  return 1;
}


template<class type>
ushort vDrawCircle(v3dMatrix<type> * source, const long y1, const long x1, const long radius,
	                 const vector<type> & values)
{
  long rows, cols, bands;
  rows = source->Rows();
  cols = source->Cols();
  bands = source->Bands();
  assert(bands == values.size());
  vArray3(type) matrix = source->Matrix3();

  long top, bottom, left, right;

  long row, col, band;
  double row_dist, col_dist;

  top = y1 - radius;
  bottom = y1 + radius;
  if (top < 0) top = 0;
  if (bottom >= rows) bottom = rows - 1;

  for (row = top; row <= bottom; row++)
  {
	  row_dist = row - y1;
	  col_dist = sqrt((double) (radius * radius - row_dist * row_dist));
	  left = (long) (x1 - col_dist);
	  right = (long) (x1 + col_dist);
	  if (left < 0) left = 0;
	  if (right >= cols) right = cols - 1;
	  for (band = 0; band < bands; band++)
	  {
       matrix[band][row][left] = values[band];
       matrix[band][row][right] = values[band];
	  }
  }

  left = x1 - radius;
  right = x1 + radius;
  if (left < 0) left = 0;
  if (right >= cols) right = cols - 1;

  for (col = left; col <= right; col++)
  {
	  col_dist = col - x1;
	  row_dist = sqrt((double) (radius * radius - col_dist * col_dist));
	  top = (long) (y1 - row_dist);
	  bottom = (long) (y1 + row_dist);
	  if (top < 0) top = 0;
	  if (bottom >= rows) bottom = rows- 1;
	  for (band = 0; band < bands; band++)
	  {
       matrix[band][top][col] = values[band];
       matrix[band][bottom][col] = values[band];
	  }
  }
  return 1;
}


template<class type>
ushort vDrawFilledCircle1(v3dMatrix<type> * source, 
			                    const long y1, const long x1, const long radius,
			                    const type value)
{
  vector<type> values;
  long i;
  long bands = source->Bands();
  for (i = 0; i < bands; i++)
    values.push_back(value);
  vDrawFilledCircle(source, y1, x1, radius, values);
  return 1;
}


template<class type>
ushort vDrawFilledCircle(v3dMatrix<type> * source, 
			                   const long y1, const long x1, const long radius,
			                   const vector<type> & values)
{
    long rows, cols, bands;
    rows = source->Rows();
    cols = source->Cols();
    bands = source->Bands();
    assert(bands == values.size());

    long top, bottom, left, right;

    top = y1 - radius;
    bottom = y1 + radius;
    if (top < 0) top = 0;
    if (bottom >= rows) bottom = rows - 1;
    long row, col, band;
    double row_dist, col_dist;

    for (row = top; row <= bottom; row++)
    {
	    row_dist = row - y1;
	    col_dist = sqrt((double) (radius * radius - row_dist * row_dist));
	    left = (long) (x1 - col_dist);
	    right = (long) (x1 + col_dist);
	    if (left < 0) left = 0;
	    if (right >= cols) right = cols - 1;
	    for (band = 0; band < bands; band++)
	    {
	      vArray2(type) matrix = source->Matrix2(band);
	      type value = values[band];
	      for (col = left; col <= right; col++)
	        matrix[row][col] = value;
	    }
    }
    return 1;
}

template<class type>
v3dMatrix<type> * vCopyPolygon(const v3dMatrix<type> * the_image, const vPolygon * polygon)
{
  if (polygon == 0) return 0;
  const long top = vTopEnd(polygon) - 1; 
  const long left = vLeftEnd(polygon) - 1;
  const long right = vRightEnd(polygon) + 1;
  const long bottom = vBottomEnd(polygon) + 1;

  v3dMatrix<type> * result = vCopyRectangle(the_image, left, right, top, bottom);
  vTranslatePolygon(polygon, 0 - left, 0 - top);
  vCropImage(result, polygon);
  return result;
}

// Copies the source region with the specified top left corner and the specified
// rows and cols to the target region whose top left corner is 
// (target_top, target_left)

template <class type>
ushort vCopyRectangle2(const v3dMatrix<type> * source, v3dMatrix<type> * target,
                       const long top, const long left, const long rows, const long cols,
                       const long target_top, const long target_left)
{ 
  // Get the size of the source and target
  long source_rows = source->Rows();
  long source_cols = source->Cols();
  long target_rows = target->Rows();
  long target_cols = target->Cols();
  long min_rows = Min(source_rows, target_rows);
  long min_cols = Min(source_cols, target_cols);

  // Get the boundaries of the source and target regions
  long bottom = top + rows - 1;
  long right = left + cols - 1;
  long target_bottom = target_top + rows - 1;
  long target_right = target_left + cols - 1;

  // Return false if one of the specified regions exceeds the boundaries
  // of its corresponding image
  if (top < 0) return 0;
  if (bottom >= source_rows) return 0;
  if (target_bottom >= target_rows) return 0;
  if (left < 0) return 0;
  if (right >= source_cols) return 0;;
  if (target_right >= target_cols) return 0;

  long bands = source->Bands();
  if (bands != target->Bands()) return 0;
 
  long i, j, row, col, band;
  
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix1 = source->Matrix2(band);
    vArray2(type) matrix2 = target->Matrix2(band);
    for (row = top; row <= bottom; row++)
    {
      i = row - top + target_top;
      for (col = left; col <= right; col++)
      {
	      j = col - left + target_left;
	      matrix2[i][j] = matrix1[row][col];
      }
    }
  }

  return 1;
}

// Returns an image that is a copy of the specified region in the source image
// If the specified region exceeds the boundaries of the source image, 
// the function returns 0
template<class type>
v3dMatrix<type> * vCopyRectangle(const v3dMatrix<type> * source, 
				                         long left, long right, long top, long bottom)
{ 
  long result_rows = bottom - top + 1;
  long result_cols = right - left + 1;
  if (left < 0) return 0;
  if (right >= source->Cols()) right = source->Cols() - 1;
  if (top < 0) top = 0;
  if (bottom >= source->Rows()) bottom = source->Rows() - 1;
  
  long rows = bottom - top + 1;
  long cols = right - left + 1;
  long bands = source->Bands();
  v3dMatrix<type> * target = new v3dMatrix<type>(result_rows, result_cols, bands);

//  ushort vCopyRectangle2(v3dMatrix<type> * source, v3dMatrix<type> * target,
//                       long top, long left, long rows, long cols,
//                       long target_top, long target_left)

  vCopyRectangle2(source, target, top, left, rows, cols, 0, 0);
  long row, col, band;

  // Write zeros to uninitialized pixels of target.
  for (band = 0; band < bands; band++)
  {
    vArray2(type) array = target->Matrix2(band);
    for (row = rows; row < result_rows; row++)
    {
      for (col = 0; col < result_cols; col++)
      {
        array[row][col] = 0;
      }
    }
    for (col = cols; col < result_cols; col++)
    {
      for (row = 0; row < result_rows; row++)
      {
        array[row][col] = 0;
      }
    }
  }

  return target;
}


template<class type>
ushort vCropImage(v3dMatrix<type> * the_image, const vPolygon * polygon)
{
  static const unsigned char OUTSIDE = 2;
  vArray2(uchar) scratch = S_PolygonMask(the_image, polygon);

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  long size = rows * cols;
  long band;

  long i, j;
  for (band = 0; band < bands; band++)
  {
    type ** matrix = the_image->Matrix2(band);
    for (i = 0; i < rows; i++)
      for (j = 0; j < cols; j++)
      {
	      if (scratch[i][j] == OUTSIDE)
	        matrix[i][j] = 0;
      }
  }
   
  for (i = 0; i < rows; i++)
  {
    vdelete2(scratch[i]);
  }

  vdelete2(scratch);
  return 1;
}


// Draws a line between (y0, x0) and (y1, x1). (Coordinates are in
// (row, col) format). If an endpoint is outside the boundary of the
// image, the line is only drawn up to the boundary.
template<class type>
ushort vDrawLine(v3dMatrix<type> * the_image, const long y0, const long x0, const long y1, const long x1,
		             const vector<type> * values)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  assert(bands == values->size());

  long band;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix = the_image->Matrix2(band);
    vDrawArrayLine(matrix, rows, cols, y0, x0, y1, x1, (*values)[band]);
  }
  return 1;
}
template<class type>
ushort vDrawLine1(v3dMatrix<type> * the_image, 
            		  const long y0, const long x0, const long y1, const long x1,
		              const type value)
{
  vector<type> values;
  long i;
  long bands = the_image->Bands();
  for (i = 0; i < bands; i++)
    values.push_back(value);
  return vDrawLine(the_image, y0, x0, y1, x1, &values);
}


// Draws a line starting at (y,x) (y is row, x is col), whose
// slope is specified by angle and has the given length. Notice
// that the image coordinates are not the standard coordinate 
// system (y increases as we go down in image coordinates). Therefore
// a line with an orientation of PI/2 points downwards.
template<class type>
ushort vDrawLineA(v3dMatrix<type> * the_image, 
		              const long y, const long x, const double angle, const long length,
		              const vector<type> * values)
{
  long x1 = (long) ((double) x + (cos(angle) * (double) length));
  long y1 = (long) ((double) y + (sin(angle) * (double) length));
  return vDrawLine(the_image, y, x, y1, x1, values);
}


template<class type>
ushort vDrawLineB(v3dMatrix<type> * the_image, 
		              const long y, const long x, const double angle, const long length, 
                  const type value)
{
  long bands = the_image->Bands();
  vector<type> values(bands);
  long i;
  for (i = 0; i < bands; i++)
  {
    values[i] = value;
  }
  return vDrawLineA(the_image, y, x, angle, length, &values);
}


template<class type>
ushort vDrawCenteredLine(v3dMatrix<type> * the_image, const long center_row, const long center_col,
                         const double angle, const long length, const type value)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  if (the_image == 0) return 0;
  long half_length = length/2;
  double row_diff = half_length * sin(angle);
  double col_diff = half_length * cos(angle);
  long start_row = vRound(center_row - row_diff);
  long start_col = vRound(center_col - col_diff);
  long end_row = vRound(center_row + row_diff);
  long end_col = vRound(center_col + col_diff);
  ushort result = vDrawLine1(the_image, start_row, start_col, end_row, end_col, value);
  return result;
}


template<class type>
ushort vDrawPolygon(v3dMatrix<type> * the_image, const vPolygon * polygon,
		                const vector<type> & values)
{
  long * xs = polygon->x_coordinates;
  long * ys = polygon->y_coordinates;
  long num_vertices = polygon->num_vertices;

  long i;
  long x1, y1, x2, y2;
  for (i = 0; i < num_vertices; i++)
  {
	  x1 = xs[i];
	  y1 = ys[i];
	  x2 = xs[(i+1) % num_vertices];
	  y2 = ys[(i+1) % num_vertices];
	  vDrawLine(the_image, y1, x1, y2, x2, values);
	  vDrawLine(the_image, y1, (x1-3), y1, (x1+3), values);
	  vDrawLine(the_image, y1 - 3, x1, y1 + 3, x1, values);
  }
  return 1;
}    


// Draws an open polygon onto an image. "Open polygon" is my own terminology
// for a polygon in which the line segment connecting the last vertex back
// to the first vertex is not drawn.
// The color of the polygon is specified
// by the "values" argument, which should have one entry for each band
// in the target image.
template<class type>
ushort vDrawOpenPolygon(v3dMatrix<type> * the_image, const vPolygon * polygon,
		                    const vector<type> * values)
{
  const vector<long> & xs = polygon->x_coordinates;
  const vector<long> & ys = polygon->y_coordinates;
  const long num_vertices = polygon->num_vertices;

  long i;
  long x1, y1, x2, y2;
  for (i = 0; i < num_vertices - 1; i++)
  {
	  x1 = xs[i];
	  y1 = ys[i];
	  x2 = xs[i+1];
	  y2 = ys[i+1];
	  vDrawLine(the_image, y1, x1, y2, x2, values);
	  //V_DrawLine(the_image, y1, (x1-3), y1, (x1+3), values);
	  //V_DrawLine(the_image, y1 - 3, x1, y1 + 3, x1, values);
  }
  return 1;
}



template<class type>
v3dMatrix<type> * vMaskFromPolygon(const v3dMatrix<type> * source, 
				   const vPolygon * polygon)
{
  static const unsigned char OUTSIDE = 2;
  vArray2(uchar) scratch = S_PolygonMask(source, polygon);

  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();
  long band, i, j;

  v3dMatrix<type> * the_image = new v3dMatrix<type>(rows, cols, bands);

  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix = the_image->Matrix2(band);
    for (i = 0; i < rows; i++)
      for (j = 0; j < cols; j++)
      {
	      if (scratch[i][j] == OUTSIDE) matrix[i][j] = 0;
	      else matrix[i][j] = 255;
      }
  }

  for (i = 0; i < rows; i++)
  {
    vdelete2(scratch[i]);
  }

  vdelete2(scratch);
  return the_image;
}    


template<class type>
static vArray2(uchar) S_PolygonMask(const v3dMatrix<type> * source, 
                                    const vPolygon * polygon)
{
  static const unsigned char INSIDE  = 0;
  static const unsigned char TO_VISIT = 1;
  static const unsigned char OUTSIDE = 2;
  
  long vertices = polygon->num_vertices;
  long * xs = polygon->x_coordinates;
  long * ys = polygon->y_coordinates;

  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();
  long size = rows * cols;

  vArray2(uchar) scratch = vnew(vArray(uchar), rows);
  vArray(long) to_visit_i = vnew(long, size);
  vArray(long) to_visit_j = vnew(long, size);

  long i, j;
  for (i = 0; i < rows; i++)
  {
    scratch[i] = vnew(uchar, cols);
    bzero(vData(scratch[i]), cols);
  }

  for (i = 0; i < vertices - 1; i++)
  {
    vDrawArrayLine(scratch, rows, cols, 
            		    ys[i], xs[i], ys[i+1], xs[i+1], (uchar) 255);
  }
  
  vDrawArrayLine(scratch, rows, cols,
		              ys[vertices - 1], xs[vertices - 1], ys[0], xs[0], 
                  (uchar) 255);
 
  // We assume that the top left pixel is outside the polygon.
  to_visit_i[0] = 0;
  to_visit_j[0] = 0;
  scratch[0][0] = TO_VISIT;
  long back_index = 0;
  long front_index = 1;

  long row, col;

  while (back_index < front_index)
  {
    row = to_visit_i[back_index];
    col = to_visit_j[back_index];

    scratch[row][col] = OUTSIDE;

    i = row+1;
    j = col;
    if ((i < rows) && (scratch[i][j] == INSIDE))
    {
      to_visit_i[front_index] = i;
      to_visit_j[front_index] = j;
      scratch[i][j] = TO_VISIT;
      front_index++;
    }

    i = row-1;
    j = col;
    if ((i >= 0) && (scratch[i][j] == INSIDE))
    {
      to_visit_i[front_index] = i;
      to_visit_j[front_index] = j;
      scratch[i][j] = TO_VISIT;
      front_index++;
    }

    i = row;
    j = col+1;
    if ((j < cols) && (scratch[i][j] == INSIDE))
    {
      to_visit_i[front_index] = i;
      to_visit_j[front_index] = j;
      scratch[i][j] = TO_VISIT;
      front_index++;
    }

    i = row;
    j = col-1;
    if ((j >= 0) && (scratch[i][j] == INSIDE))
    {
      to_visit_i[front_index] = i;
      to_visit_j[front_index] = j;
      scratch[i][j] = TO_VISIT;
      front_index++;
    }

    back_index++;
  }

  vdelete2(to_visit_i);
  vdelete2(to_visit_j);

  return scratch;
}

template<class type>
ushort vDrawArrayLine(vArray2(type) array, const long rows, const long cols, 
		                  const long y0, const long x0, const long y1, const long x1,
		                  const type value)
{
  double dx = x1 - (double) x0;
  double dy = y1 - (double) y0;

  double abs_dx = fabs(dx);
  double abs_dy = fabs(dy); 
  double times, step_x, step_y;

  if (abs_dx >= abs_dy) times = abs_dx + 1;
  else times = abs_dy + 1;

  if (times == 1)
  {
    array[y0][x0] = value;
    return 1;
  }

  step_x = dx / (times-1);
  step_y = dy / (times-1);
  
  long i;
  double col = x0 - step_x;
  double row = y0 - step_y;
  long int_row, int_col;
  for (i = 0; i < times; i++)
  {
    row = row + step_y;
    col = col + step_x;
    int_row = vRound(row);
    int_col = vRound(col);
    if (int_row >= rows) continue;
    if (int_row < 0) continue;
    if (int_col >= cols) continue;
    if (int_col < 0) continue;
    array[int_row][int_col] = value;
  }
  return 1;
}


template<class type>
ushort vDrawCross(v3dMatrix<type> * the_image, const long row, const long col, 
                  const long width, const long length, const type value)
{
  long bands = the_image->Bands();
  vector<type> values;
  long i;
  for (i = 0; i < bands; i++)
  {
    values.push_back(value);
  }
  //return 1;
  long left, right, top, bottom;
  left = col - width;
  right = col + width;
  top = row - length;
  bottom = row + length;
  long col_index, row_index;
  for (col_index = left; col_index <= right; col_index++)
  {
    vDrawLine(the_image, top, col_index, bottom, col_index, &values);
  }
  left = col - length;
  right = col + length;
  top = row - width;
  bottom = row + width;
  for (row_index = top; row_index <= bottom; row_index++)
  {
    vDrawLine(the_image, row_index, left, row_index, right, &values);
  }
  return 1;
}


template<class type>
ushort vDrawCrossv(v3dMatrix<type> * the_image, const long row, const long col, 
                   const long width, const long length, const vector<type> * values)
{
  long left, right, top, bottom;
  left = col - width;
  right = col + width;
  top = row - length;
  bottom = row + length;
  long col_index;
  for (col_index = left; col_index <= right; col_index++)
  {
    vDrawLine(the_image, top, col_index, bottom, col_index, values);
  }
  left = col - length;
  right = col + length;
  top = row - width;
  bottom = row + width;
  for (long row_index = top; row_index <= bottom; row_index++)
  {
    vDrawLine(the_image, row_index, left, row_index, right, values);
  }
  return 1;
}


template<class type>
v3dMatrix<type> * vRotateRight(const v3dMatrix<type> * fimage)
{
  long rows = fimage->Rows();
  long cols = fimage->Cols();
  long bands = fimage->Bands();
  
  // the result image must have as many rows as fimage->cols, since it
  // will be fimage rotated.

  v3dMatrix<type> * result = new v3dMatrix<type>(cols, rows, bands);
  long i;

  for (i = 0; i < bands; i++)
  {
    vArray2(type) I1 = fimage->Matrix2(i);
    vArray2(type) I2 = result->Matrix2(i);
  
    long row, col;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	      I2[col][rows - 1 - row] = I1[row][col];
      }
  }

  return result;
}


template<class type>
v3dMatrix<type> * vRotateRight(const v3dMatrix<type> * fimage, const long degrees)
{
  long remainder = degrees % 90;
  if (remainder != 0) return 0;
  long real_degrees = degrees % 360;
  while(real_degrees < 0) real_degrees += 360;
  if (real_degrees == 0) return vCopyImage(fimage);
  else if (real_degrees == 90) return vRotateRight(fimage);
  else if (real_degrees == 180) return vUpsideDown(fimage);
  else if (real_degrees == 270) return vRotateLeft(fimage);
  else 
  {
    assert(0);
    return 0;
  }
}


template<class type>
v3dMatrix<type> * vRotateLeft(const v3dMatrix<type> * fimage)
{
  long rows = fimage->Rows();
  long cols = fimage->Cols();
  long bands = fimage->Bands();
  
  // the result image must have as many rows as fimage->cols, since it
  // will be fimage rotated.

  v3dMatrix<type> * result = new v3dMatrix<type>(cols, rows, bands);
  long i;

  for (i = 0; i < bands; i++)
  {
    vArray2(type) I1 = fimage->Matrix2(i);
    vArray2(type) I2 = result->Matrix2(i);
  
    long row, col;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	      I2[cols - 1 - col][row] = I1[row][col];
      }
  }

  return result;
}


template<class type>
v3dMatrix<type> * vUpsideDown(const v3dMatrix<type> * fimage)
{
  long rows = fimage->Rows();
  long cols = fimage->Cols();
  long bands = fimage->Bands();
  
  // the result image must have as many rows as fimage->cols, since it
  // will be fimage rotated.

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  long i;

  for (i = 0; i < bands; i++)
  {
    vArray2(type) I1 = fimage->Matrix2(i);
    vArray2(type) I2 = result->Matrix2(i);
  
    long row, col;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	      I2[rows - 1 - row][cols - 1 - col] = I1[row][col];
      }
  }

  return result;
}


// This function is useful for raster images that are stored on 
// file with their bottom row first. This way I can read them
// using v3dMatrix::ReadInterlaced and then call this function
// to flip them 
template<class type>
v3dMatrix<type> * vSwitchTopBottom(const v3dMatrix<type> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  long row_byte_size = cols * sizeof(type);
  vArray3(type) in_matrix3 = the_image->Matrix3();
  vArray3(type) out_matrix3 = result->Matrix3();
  long band, row;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) in_matrix2 = in_matrix3[band];
    vArray2(type) out_matrix2 = out_matrix3[band];
    for (row = 0; row < rows; row++)
    {
      long in_row = rows - row - 1;
      memcpy(vData(out_matrix2[row]), vData(in_matrix2[in_row]), 
             row_byte_size);
    }
  }
  return result;
}


template<class type>
v3dMatrix<type> * vReverseRowsCols(const v3dMatrix<type> * fimage)
{
  long rows = fimage->Rows();
  long cols = fimage->Cols();
  long bands = fimage->Bands();

  // the result image must have as many rows as fimage->cols, since it
  // will be fimage rotated.

  v3dMatrix<type> * result = new v3dMatrix<type>(cols, rows, bands);
  long i;

  for (i = 0; i < bands; i++)
  {
    vArray2(type) I1 = fimage->Matrix2(i);
    vArray2(type) I2 = result->Matrix2(i);
    
    long row, col;
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
	      I2[col][row] = I1[row][col];
      }
  }

  return result;
}


template<class type>
void vWriteValue(v3dMatrix<type> * the_image, const type value)
{
  vArray(type) matrix = the_image->Matrix();
  long size = the_image->AllBandSize();
  for (long i = 0; i < size; i++)
  {
    matrix[i] = value;
  }
}


template<class type>
void vWriteValue2(v3dMatrix<type> * the_image, 
                  const vector<long> * point_rows, const vector<long> * point_cols, 
                  const type value)
{
  vArray3(type) matrix = the_image->Matrix3();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  long band, i;
  long size = point_rows->size();
  for (i = 0; i < size; i++)
  {
    long row = (*point_rows)[i];
    long col = (*point_cols)[i];
    if (the_image->check_bounds(row, col) <= 0) continue;
    for (band = 0; band < bands; band++)
    {
      matrix[band][row][col] = value;
    }
  }
}


template<class type>
void vWriteValue3(v3dMatrix<type> * the_image, 
                  const vArray(uchar) point_rows, 
                  const vArray(uchar) point_cols, 
                  const long number, const type value)
{
  vArray3(type) matrix = the_image->Matrix3();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  long band, i;
  for (i = 0; i < number; i++)
  {
    long row = point_rows[i];
    long col = point_cols[i];
    if (the_image->check_bounds(row, col) <= 0) continue;
    for (band = 0; band < bands; band++)
    {
      matrix[band][row][col] = value;
    }
  }
}


template<class type>
void vWriteValue3(v3dMatrix<type> * the_image, 
                  const vArray(long) point_rows, 
                  const vArray(long) point_cols, 
                  const long number, const type value)
{
  vArray3(type) matrix = the_image->Matrix3();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  long band, i;
  for (i = 0; i < number; i++)
  {
    long row = point_rows[i];
    long col = point_cols[i];
    if (the_image->check_bounds(row, col) <= 0) continue;
    for (band = 0; band < bands; band++)
    {
      matrix[band][row][col] = value;
    }
  }
}


template<class type>
void vWriteValue4(v3dMatrix<type> * the_image, 
                  const vector<vPoint> * pixels, 
                  const type value)
{
  vArray3(type) matrix = the_image->Matrix3();
  const long rows = the_image->Rows();
  const long cols = the_image->Cols();
  const long bands = the_image->Bands();

  long band, i;
  const long size = pixels->size();
  for (i = 0; i < size; i++)
  {
    const vPoint & point = (*pixels)[i];
    const long row = point.row;
    const long col = point.col;
    if (the_image->check_bounds(row, col) <= 0) continue;
    for (band = 0; band < bands; band++)
    {
      matrix[band][row][col] = value;
    }
  }
}


// Writes the specified value to the border pixels of the image
// (pixels in the top and bottom rows and left and right columns.
template<class type>
ushort vWriteBorderValue(v3dMatrix<type> * the_image, const type value)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bottom = rows - 1;
  long right = cols - 1;
  long bands = the_image->Bands();
  vArray3(type) matrix3 = the_image->Matrix3();
  long band, row, col;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix2 = matrix3[band];

    for (col = 0; col < cols; col++)
    {
      matrix2[0][col] = value;
      matrix2[bottom][col] = value;
    }

    for (row = 0; row < rows; row++)
    {
      matrix2[row][0] = value;
      matrix2[row][right] = value;
    }
  }
  return 1;
}


// Writes the specified value to the border pixels of the image
// (pixels whose distance from the border is <= width).
template<class type>
ushort vWriteBorderValue3(v3dMatrix<type> * the_image, const type value, long width)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (width > rows) width = rows;
  if (width > cols) width = cols;
  long bottom = rows - 1;
  long right = cols - 1;
  long bands = the_image->Bands();
  vArray3(type) matrix3 = the_image->Matrix3();
  long band, row, col, i;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix2 = matrix3[band];

    for (col = 0; col < cols; col++)
    {
      for (i = 0; i < width; i++)
      {
        matrix2[i][col] = value;
        matrix2[bottom-i][col] = value;
      }
    }

    for (row = 0; row < rows; row++)
    {
      for (i = 0; i < width; i++)
      {
        matrix2[row][i] = value;
        matrix2[row][right-i] = value;
      }
    }
  }
  return 1;
}


// result[row][col] will get the values of color_image[row2][col2]
// iff warp_template[row][col] = (row2, col2), with one exception,
// of course, i.e. when two source points map to the same 
// result point, in which case only one of the source points will
// give its value to the result point.
template<class type>
v3dMatrix<type> * vWarpImage(const v3dMatrix<type> * the_image, 
                             const v3dMatrix<short> * warp_template)
{
  // Error checking:
  if (the_image == 0) return 0;
  if (warp_template == 0) return 0;
  if (warp_template->Rows() < the_image->Rows()) return 0;
  if (warp_template->Cols() < the_image->Cols()) return 0;
  if (warp_template->Bands() != 2) return 0;

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) color_image3 = the_image->Matrix3();
  vArray3(type) result3 = result->Matrix3();
  vArray2(short) source_rows = warp_template->Matrix2(0);
  vArray2(short) source_cols = warp_template->Matrix2(1);

  long row, col, band;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long source_row = source_rows[row][col];
      long source_col = source_cols[row][col];
      if (the_image->check_bounds(source_row, source_col) <= 0)
      {
        for (band = 0; band < bands; band++)
        {
          result3[band][row][col] = (type) 0;
        }
      }
      else
      {
        for (band = 0; band < bands; band++)
        {
          result3[band][row][col] = color_image3[band][source_row][source_col];
        }
      }
    }
  }

  return result;
}


// result[row][col] will get the values of color_image[row2][col2]
// iff warp_template[row2][col2] = (row, col).
template<class type>
v3dMatrix<type> * inverse_warp(const v3dMatrix<type> * image, 
                               const v3dMatrix<short> * warp_template)
{
  // Error checking:
  if (image == 0) return 0;
  if (warp_template == 0) return 0;
  if (warp_template->Rows() < image->Rows()) return 0;
  if (warp_template->Cols() < image->Cols()) return 0;
  if (warp_template->Bands() != 2) return 0;

  long rows = image->Rows();
  long cols = image->Cols();
  long bands = image->Bands();

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  function_enter_value(result, (type) 0);
  vArray3(type) image3 = image->Matrix3();
  vArray3(type) result3 = result->Matrix3();
  vArray2(short) target_rows = warp_template->Matrix2(0);
  vArray2(short) target_cols = warp_template->Matrix2(1);

  long row, col, band;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long target_row = target_rows[row][col];
      long target_col = target_cols[row][col];
      if (result->check_bounds(target_row, target_col) > 0)
      {
        for (band = 0; band < bands; band++)
        {
          result3[band][target_row][target_col] = image3[band][row][col];
        }
      }
    }
  }

  return result;
}



// result[row][col] will get the values of color_image[row2][col2]
// iff warp_template[row][col] = (row2, col2), as in vWarpImage,
// with one difference: if a zero source pixel and a non-zero source
// pixel map to the same result pixel, the non-zero pixel takes precedence.
template<class type>
v3dMatrix<type> * vWarpImageSparse(const v3dMatrix<type> * the_image, 
                                   const v3dMatrix<short> * warp_template)
{
  // Error checking:
  if (the_image == 0) return 0;
  if (warp_template == 0) return 0;
  if (warp_template->Rows() < the_image->Rows()) return 0;
  if (warp_template->Cols() < the_image->Cols()) return 0;
  if (warp_template->Bands() != 2) return 0;

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) color_image3 = the_image->Matrix3();
  vArray3(type) result3 = result->Matrix3();
  vArray2(short) source_rows = warp_template->Matrix2(0);
  vArray2(short) source_cols = warp_template->Matrix2(1);
  vWriteValue(result, (type) 0);

  long row, col, band;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long source_row = source_rows[row][col];
      long source_col = source_cols[row][col];
      if (the_image->check_bounds(source_row, source_col) <= 0) continue;
      long zero_flag = 1;
      for (band = 0; band < bands; band++)
      {
        if (color_image3[band][source_row][source_col] != 0) zero_flag = 0;
      }
      if (zero_flag == 1) continue;
      for (band = 0; band < bands; band++)
      {
        result3[band][row][col] = color_image3[band][source_row][source_col];
      }
    }
  }

  return result;
}

/////////////////////////////////////////////////////////////////////////

template<class type>
v3dMatrix<type> * function_copy_image(const v3dMatrix<type> * source)
{
  return vCopyImage(source);
}

template<class type> 
ushort function_copy_image(v3dMatrix<type> * result, const v3dMatrix<type> * source)
{
  return vCopyImage(result, source);
}

template<class type>
v3dMatrix<type> * function_copy_band(const v3dMatrix<type> * source, const long band)
{
  return vCopyBand2(source, band);
}


template<class type>
long function_copy_band(v3dMatrix<type> * target, const v3dMatrix<type> * source, const long band)
{
  return vCopyBand3(target, source, band);
}

// Returns a result that is a copy of the source, except that it
// also has a rectangle drawn on it, with top y1, left x1, bottom y2,
// right x2. The rectangle sides are colored as specified by values.
template<class type>
v3dMatrix<type> * copy_draw_rectangle(const v3dMatrix<type> * source, 
					                            const long y1, const long x1, const long y2, const long x2,
					                            const vector<type> & values)
{
  return vCopyAndDrawRectangle(source, y1, x1, y2, x2, values);
}

// Similar as vCopyAndDrawRectangle, but here we just draw the rectangle 
// directly onto the source.
template<class type>
ushort function_draw_rectangle(v3dMatrix<type> * source, 
 	                             const long y1, const long x1, const long y2, const long x2,
	                             const vector<type> & values)
{
  return vDrawRectangle(source, y1, x1, y2, x2, values);
}

// Similar to vDrawRectangle, but here the color of the rectangle sides is
// specified by value (value is used in all bands of the image).
template<class type>
ushort function_draw_rectangle(v3dMatrix<type> * source, 
  		                         const long y1, const long x1, const long y2, const long x2,
  		                         const type value)
{
  return vDrawRectangle1(source, y1, x1, y2, x2, value);
}

template<class type>
ushort function_draw_square(v3dMatrix<type> * source, 
  		                      const long y, long x, double angle, long size,
		                        const vector<type> & values)
{
  return vDrawSquare(source, y, x, angle, size, values);
}

template<class type>
ushort draw_full_rectangle(v3dMatrix<type> * source, 
			                     const long y1, const long x1, const long y2, const long x2,
			                     const vector<type> & values)
{
  return vDrawFilledRectangle(source, y1, x1, y2, x2, values);
}

template<class type>
ushort draw_full_rectangle(v3dMatrix<type> * source, 
		                       const long y1, const long x1, const long y2, const long x2,
		                       const type value)
{
  return vDrawFilledRectangle1(source, y1, x1, y2, x2, value);
}


template<class type>
ushort function_draw_circle(v3dMatrix<type> * source, const long y1, const long x1, const long radius,
			                      const vector<type> & values)
{
  return vDrawCircle(source, y1, x1, radius, values);
}

template<class type>
ushort function_draw_circle(v3dMatrix<type> * source, const long y1, const long x1, const long radius, const type value)
{                     
  return vDrawCircle1(source, y1, x1, radius, value);
}



template<class type>
ushort draw_full_circle(v3dMatrix<type> * source, 
			                  const long y1, const long x1, const long radius,
			                  const vector<type> & values)
{
  return vDrawFilledCircle(source, y1, x1, radius, values);
}

template<class type>
ushort draw_full_circle(v3dMatrix<type> * source, 
 			                  const long y1, const long x1, const long radius,
			                  const type value)
{
  return vDrawFilledCircle1(source, y1, x1, radius, value);
}



template<class type>
v3dMatrix<type> * function_copy_polygon(const v3dMatrix<type> * the_image, 
				                                const vPolygon * polygon)
{
  return vCopyPolygon(the_image, polygon);
}

template<class type>
v3dMatrix<type> * function_subtract_polygon(const v3dMatrix<type> * the_image, 
                        				            const vPolygon * polygon)
{
  return vSubtractPolygon(the_image, polygon);
}

template<class type>
v3dMatrix<type> * function_copy_rectangle(const v3dMatrix<type> * source, 
				                                  const long left, const long right, 
				                                  const long top, const long bottom)
{
  return vCopyRectangle(source, left, right, top, bottom);
}

template <class type>
ushort function_copy_rectangle(const v3dMatrix<type> * source, v3dMatrix<type> * target,
                               const long left, const long right, const long top, const long bottom)
{
  return vCopyRectangle2(source, target, top, left, bottom-top+1, right-left+1, 0, 0);
}

template <class type>
ushort function_copy_rectangle(const v3dMatrix<type> * source, v3dMatrix<type> * target,
                               const long left, const long right, const long top, const long bottom, 
                               const long target_top, const long target_left)
{
  return vCopyRectangle2(source, target, top, left, bottom-top+1, right-left+1, target_top, target_left);
}

template<class type>
ushort function_crop_image(v3dMatrix<type> * the_image, const vPolygon * polygon)
{
  return vCropImage(the_image, polygon);
}

template<class type>
ushort function_draw_line(v3dMatrix<type> * the_image, const long y0, const long x0, const long y1, const long x1,
             		          const vector<type> * values)
{
  return vDrawLine(the_image, y0, x0, y1, x1, values);
}


template<class type>
ushort function_draw_line(v3dMatrix<type> * the_image, const long y0, const long x0, const long y1, const long x1,
             		          const type red, const type green, const type blue)
{
  if (the_image->channels() != 3)
  {
    exit_error("\nerror in function_draw_line\n");
  }

  vector<type> values;
  values.push_back(red);
  values.push_back(green);
  values.push_back(blue);

  return function_draw_line(the_image, y0, x0, y1, x1, &values);
}




template<class type>
ushort function_draw_cross(v3dMatrix<type> * the_image, const long row, const long col, 
                           const long width, const long length, const type value)
{
  return vDrawCross(the_image, row, col, width, length, value);
}

template<class type>
ushort function_draw_cross(v3dMatrix<type> * the_image, const long row, const long col, 
                           const long width, const long length, const vector<type> * values)
{
  return vDrawCrossv(the_image, row, col, width, length, values);
}



// Draws a line starting at (y,x) (y is row, x is col), whose
// slope is specified by angle and has the given length. Notice
// that the image coordinates are not the standard coordinate 
// system (y increases as we go down in image coordinates). Therefore
// a line with an orientation of PI/2 points downwards.
template<class type>
ushort function_draw_line(v3dMatrix<type> * the_image, 
		                      const long y, const long x, const double angle, const long length,
		                      const vector<type> * values)
{
  return vDrawLineA(the_image, y, x, angle, length, values);
}

template<class type>
ushort function_draw_line(v3dMatrix<type> * the_image, 
		                      const long y, const long x, const double angle, const long length,
		                      const type value)
{
  return vDrawLineB(the_image, y, x, angle, length, value);
}


template<class type>
ushort function_draw_line(v3dMatrix<type> * the_image, const long y0, const long x0, const long y1, const long x1,
                          const type value)
{
  return vDrawLine1(the_image, y0, x0, y1, x1, value);
}

template<class type>
ushort draw_centered_line(v3dMatrix<type> * the_image, const long center_row, const long center_col,
                          const double angle, const long length, type value)
{
  return vDrawCenteredLine(the_image, center_row, center_col, angle, length, value);
}


template<class type>
ushort draw_array_line(type ** array, const long rows, const long cols,
		                   const long y0, const long x0, const long y1, const long x1,
		                   const type value)
{
  return vDrawArrayLine(array, rows, cols, y0, x0, y1, x1, value);
}

// Draws a polygon onto an image. The color of the polygon is specified
// by the "values" argument, which should have one entry for each band
// in the target image.
template<class type>
ushort function_draw_polygon(v3dMatrix<type> * the_image, const vPolygon * polygon,
		                         const vector<type> * values)
{
  return vDrawPolygon(the_image, polygon, values);
}


// Draws an open polygon onto an image. "Open polygon" is my own terminology
// for a polygon in which the line segment connecting the last vertex back
// to the first vertex is not drawn.
// The color of the polygon is specified
// by the "values" argument, which should have one entry for each band
// in the target image.
template<class type>
ushort draw_open_polygon(v3dMatrix<type> * the_image, const vPolygon * polygon,
		                     const vector<type> * values)
{
  return vDrawOpenPolygon(the_image, polygon, values);
}


template<class type>
v3dMatrix<type> * mask_from_polygon(const v3dMatrix<type> * the_image, 
				                            const vPolygon * polygon)
{
  return vMaskFromPolygon(the_image, polygon);
}

template<class type>
v3dMatrix<type> * function_rotate_right(const v3dMatrix<type> * fimage)
{
  return vRotateRight(fimage);
}

// Works only if degrees is a multiple of 90.
template<class type>
v3dMatrix<type> * function_rotate_right(const v3dMatrix<type> * fimage, const long degrees)
{
  return vRotateRight(fimage, degrees);
}

template<class type>
v3dMatrix<type> * function_rotate_left(const v3dMatrix<type> * fimage)
{
  return vRotateLeft(fimage);
}

template<class type>
v3dMatrix<type> * function_upside_down(const v3dMatrix<type> * fimage)
{
  return vUpsideDown(fimage);
}

// This function is useful for raster images that are stored on 
// file with their bottom row first. This way I can read them
// using v3dMatrix::ReadInterlaced and then call this function
// to flip them 
template<class type>
v3dMatrix<type> * switch_top_bottom(const v3dMatrix<type> * image)
{
  return vSwitchTopBottom(image);
}

template<class type>
v3dMatrix<type> * reverse_rows_columns(const v3dMatrix<type> * fimage)
{
  return vReverseRowsCols(fimage);
}


template<class type>
void function_enter_value(v3dMatrix<type> * the_image, const type value)
{
  vWriteValue(the_image, value);
}


template<class type>
void function_enter_values(v3dMatrix<type> * the_image, 
                           const vector<long> * rows, const vector<long> * cols, 
                           const type value)
{
  return vWriteValue2(the_image, rows, cols, value);
}

// Same as vWriteValue2, but point coordinates here are passed
// in in arrays, not in vectors.
template<class type>
void function_enter_values(v3dMatrix<type> * the_image, 
                           const vArray(uchar) point_rows, 
                           const vArray(uchar) point_cols, 
                           const long number, const type value)
{
  return vWriteValue3(the_image, point_rows, point_cols, number, value);
}

template<class type>
void function_enter_values(v3dMatrix<type> * the_image, 
                           const vArray(long) point_rows, 
                           const vArray(long) point_cols, 
                           const long number, const type value)
{
  return vWriteValue3(the_image, point_rows, point_cols, number, value);
}


template<class type>
void function_enter_values(v3dMatrix<type> * the_image, 
                           const vector<vPoint> * points, 
                           const type value)
{
  return vWriteValue4(the_image, points, value);
}


// matrix has two columns, and each row represents a pixel:
// col 0 is pixel row, col1 is pixel col
template<class type>
void function_enter_values(v3dMatrix<type> * the_image, 
                           const long_matrix pixels, const type value)
{
  long number = pixels.vertical();
  long channels = the_image->channels();
  long counter, channel;
  for (counter = 0; counter < number; counter++)
  {
    long vertical = pixels(counter, 0);
    long horizontal = pixels(counter, 1);
    if (the_image->check_bounds(vertical, horizontal) <= 0) 
    {
      continue;
    }

    for (channel = 0; channel < channels; channel++)
    {
      (*the_image)(channel, vertical, horizontal) = value;
    }
  }
}


// Writes the specified value to the border pixels of the image
// (pixels in the top and bottom rows and left and right columns).
template<class type>
ushort enter_border_value(v3dMatrix<type> * the_image, const type value)
{
  return vWriteBorderValue(the_image, value);
}

// Writes the specified value to the border pixels of the image
// (pixels whose distance from the border is <= width).
template<class type>
ushort enter_border_value(v3dMatrix<type> * the_image, const type value, long width)
{
  return vWriteBorderValue3(the_image, value, width);
}


// result[row][col] will get the values of color_image[row2][col2]
// iff warp_template[row][col] = (row2, col2).
template<class type>
v3dMatrix<type> * function_warp_template(const v3dMatrix<type> * the_image, 
                                         const v3dMatrix<short> * warp_template)
{
  return vWarpImage(the_image, warp_template);
}


// here we do bilinear interpolation
template <class type >
v3dMatrix<type> * rotate_image_exact_degrees(const v3dMatrix<type> * the_image, 
                                             const ulong center_row, const ulong center_col,
                                             const double degrees)
{
  const double degrees_to_radians = M_PI / 180.0;
  double radians = degrees * degrees_to_radians;
  return rotate_image_exact(the_image, center_row, center_col, radians);
}

template <class type >
v3dMatrix<type> * rotate_image_exact(const v3dMatrix<type> * the_image, 
                                     const ulong center_row, const ulong center_col,
                                     const double radians)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<float> * sources = vRotationTemplateRf(rows, cols, 
                                                   center_row, center_col, radians);
  if (sources == 0) return 0;
  vArray2(float) source_rows = sources->Matrix2(0);
  vArray2(float) source_cols = sources->Matrix2(1);

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) result3 = result->Matrix3();
  vArray3(type) source3 = the_image->Matrix3();
  vWriteValue(result, (type) 0);

  long row, col;

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      double source_row = source_rows[row][col];
      double source_col = source_cols[row][col];
      if ((!result->check_bounds((long) source_row, (long) source_col)) ||
          (!result->check_bounds((long) (source_row+1), (long) (source_col+1))) ||
          (!result->check_bounds((long) (source_row-1), (long) (source_col-1))))
      {
        continue;
      }

      long band;
      for (band = 0; band < bands; band++)
      {
      result3[band][row][col] = (type)
          vBilinearInterpolationD(source3[band], source_row, source_col);
      }
    }
  }

  vdelete(sources);
  return result;
}


// write a value to a specific band
template<class type>
void function_channel_value(v3dMatrix<type> * image, const long channel, const type value)
{
  if ((image->Bands() >= channel) || (channel < 0))
  {
    return;
  }

  class_pointer(type) data = image->Matrix(channel);
  long size = image->Size();
  long counter = 0;
  for (counter = 0; counter < size; counter ++)
  {
    data [counter] = value;
  }
}





// If the cpp template file is included as part of the header file,
// we must undefine certain things.
#include "undefine.h"

#endif // VASSILIS_PROCESS_FILE
