#include "stdafx.h"

#include "gl_utils.h"

#include "definitions.h"

color_image * vGLReadColor2(long rows, long cols)
{
  color_image * result = new color_image(rows, cols);
  ushort success = vGLReadColor5(result, 0, 0, rows, cols);
  if (success == 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}

color_image * vGLReadColor4(long top, long left, long rows, long cols)
{
  color_image * result = new color_image(rows, cols);
  ushort success = vGLReadColor5(result, top, left, rows, cols);
  if (success == 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}


// Read the GL subwindow specified by top, left, rows, cols,
// and put its contents into color_image.
ushort vGLReadColor5(color_image * the_image, long top, long left,
                     long rows, long cols)
{
  if (color_image->Rows() < rows) return 0;
  if (color_image->Cols() < cols) return 0;

  // Read in pixel values.
  long array_size = rows * cols * 3;
	vArray(uchar) buffer = vnew(uchar, array_size);
  long bottom = top + rows - 1;
  glReadPixels(left, top, cols, rows,
               GL_RGB, GL_UNSIGNED_BYTE, vData(buffer));

  // Convert pixel values to an actual color_image object. Note
  // that pixel values appear in order from bottom col to top col.
	vArray2(uchar) R = color_image->GetR2();
	vArray2(uchar) G = color_image->GetG2();
	vArray2(uchar) B = color_image->GetB2();

	long row, col;
	long index = array_size - 3;
	for (row = 0; row < rows; row++)
  {
    index = (rows - row - 1) * cols * 3;
    vArray(uchar) r_base = R[row];
    vArray(uchar) g_base = G[row];
    vArray(uchar) b_base = B[row];
		for (col = 0; col < cols; col++)
		{
			r_base[col] = buffer[index++];
			g_base[col] = buffer[index++];
			b_base[col] = buffer[index++];
		}
  }
	
	vdelete2(buffer);
  return 1;
}


v3dMatrix<float> * vGLReadDepth2(long rows, long cols)
{
  v3dMatrix<float> * result = new v3dMatrix<float>(rows, cols, 1);
  ushort success = vGLReadDepth5(result, 0, 0, rows, cols);
  if (success == 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}


v3dMatrix<float> * vGLReadDepth4(long top, long left, long rows, long cols)
{
  v3dMatrix<float> * result = new v3dMatrix<float>(rows, cols, 1);
  ushort success = vGLReadDepth5(result, top, left, rows, cols);
  if (success == 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}


// Read in the depth values for the subregion of the GL window 
// specified by top, left, rows, cols, and store then into color_image.
ushort vGLReadDepth5(v3dMatrix<float> * the_image, long top, long left,
                      long rows, long cols)
{
  if (color_image->Rows() < rows) return 0;
  if (color_image->Cols() < cols) return 0;

  // Read in pixel values.
  long array_size = rows * cols;
 	vArray(float) buffer = vnew(float, array_size);
  glReadPixels(left, top, cols, rows,
               GL_DEPTH_COMPONENT, GL_FLOAT, vData(buffer));

	vArray2(float) matrix = color_image->Matrix2(0);

  // Write pixel values into color_image. Note that pixel values
  // appear in order from bottom to top.
	long row, col;
	long index = 0;
	for (row = 0; row < rows; row++)
  {
    index = (rows - row - 1) * cols;
    vArray(float) base = matrix[row];

		for (col = 0; col < cols; col++)
		{
			base[col] = buffer[index++];
		}
  }

  vdelete2(buffer);
  return 1;
}


