#include "stdafx.h"

#include "vplatform.h"

#include <assert.h>
#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#include <fstream>
#else 
#include <iostream>
#include <fstream>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef VASSILIS_WINDOWS
#include <direct.h>
#include <vector> 
#include "pc_aux.h"
#endif // VASSILIS_WINDOWS

#ifdef VASSILIS_SGI_PLATFORM
#include <algo.h>
#include <vector.h>
#endif // VASSILIS_SGI_PLATFORM


#include "basics/auxiliaries.h"
#include "basics/drawing.h"
#include "basics/drawing_temp.h"
#include "local_data.h"
#include "basics/read_avi.h"
#include "basics/video_common.h"
#include "basics/wrapper.h"
#include "basics/io_aux.h"
#include "basics/simple_algo.h"
#include "basics/image_templates.h"
#include "basics/image_algorithms.h"
#include "basics/my_video_format.h"
#include "basics/simple_image.h"
#include "window.h"
#include "avi_file.h"

#include "basics/rle_encoding.h"
#include "basics/video_file.h"
#include "basics/graphical_module.h"

#include "basics/definitions.h"

using namespace std;

template<class type>
static type S_EraseItem(vector<type> * v, long index);


template<class type>
static type S_EraseItem(vector<type> * v, long index)
{
  if ((index < 0) || (index >= (long) v->size()))
  {
    return (type) 0;
  }
  type result = (*v)[index];
  vector<type>::iterator iter = v->begin();
  long i;
  for (i = 0; i < index; i++)
  {
    iter++;
  }
  v->erase(iter);
  return result;
}


class_sequence_information::class_sequence_information(long in_buffer_size)
{
  sequence = 0;
  path_name = 0;
  simple_name = 0;
  image_buffer = vZero(general_image *);
  buffer_size = in_buffer_size;
  buffer_index = 0;
  current_image = 0;

  sequence_step = 1;
  display_top = 0;
  display_left = 0;
  resolution = 0;
  current_frame = 0;

  center_row = 0;
  center_col = 0;
  automatic_centering = 1;
  //magnified_rows = -1;
  //magnified_cols = -1;

  is_camera_input = 0;

  SetImageBufferSize(buffer_size);
}


class_sequence_information::~class_sequence_information()
{
  vCloseVideoFile(sequence);
  vdelete2(path_name);
  vdelete2(simple_name);
  ClearImageBuffer();
}



ushort class_sequence_information::ClearImageBuffer()
{
  if (vIsZero(image_buffer)) return 0;
  long i;
  for (i = 0; i < buffer_size; i++)
  {
    general_image * image = image_buffer[i];
    vdelete(image);
  }
  vdelete2(image_buffer);
  image_buffer = vZero(general_image *);
  buffer_index = 0;
  current_image = 0;
  return 1;
}


ushort class_sequence_information::SetImageBufferSize(long number)
{
  general_image * current_copy = 0;
  if (current_image != 0)
  {
   current_copy = current_image->Copy();
  }
  ClearImageBuffer();
  buffer_size = number;
  image_buffer = vnew(general_image *, buffer_size);
  long i;
  for (i = 0; i < buffer_size; i++)
    image_buffer[i] = 0;
  buffer_index = 0;
  image_buffer[buffer_index] = current_copy;
  current_image = current_copy;
  return 1;
}


long class_sequence_information::CurrentFrame()
{
  return current_frame;
}


ushort class_sequence_information::SetCurrentFrame(long index)
{
  if (index < 0) return 0;
  if (index >= number_of_frames) return 0;
  current_frame = index;
  return 1;
}
  

//////////////////////////////////////////////////////////
//                                                      //
// start of obligatory member functions for any module  //
//                                                      //
//////////////////////////////////////////////////////////


graphical_module::graphical_module(class_interpreter* in_interpreter,
                                   graphical_interface_globals * globals) :
class_module (in_interpreter)
{
  initialize(globals);
  initialize();
  sort_commands ();

#ifndef VASSILIS_AX_DEMO
  vInitializeConsole();
#endif

  input_fp = 0;

  current_input_index = -1;
  number_of_inputs = 0;
  input = 0;
  current_output = -1;
  number_of_outputs = 0;

  filter_blur_size = 5;
  current_directory = 0;
  default_buffer_size = 10;
  mouse_click_response = pixel_info_response;

  histogram_equalization_flag = 0;
  equalization_window_size = 101;
  preserve_current_frame = 0;
  frame_rate = 30;
}


graphical_module::~graphical_module()
{
  class_module::clean_up();
  clean_up();
}


long graphical_module::clean_up()
{
  vdelete(window);
  window = 0;
  vdelete(frame_info_window);
  frame_info_window = 0;
  vdelete(pixel_info_window);
  pixel_info_window = 0;
  if (input_fp != 0)
  {
    fclose(input_fp);
  }
 
  // If there is a camera input, mark it as a non-camera
  // input so that it can be removed.
  long i;
  for (i = 0; i < number_of_inputs; i++)
  {
    class_sequence_information * info = (class_sequence_information *) inputs[i];
    info->is_camera_input = 0;
  }


  ClearInputs();
  ClearOutputs();
  vdelete2(current_directory);

  return 1;
}


long graphical_module::initialize()
{
  enter_command("add_input", AddInput);
  enter_command("add_output", AddOutput);
  enter_command("add_output_stream", add_output_stream);
  enter_command("amplify", AmplifyImage);
  enter_command("average_regions", AverageRegions);

  enter_command("b", PreviousInSequence);
  enter_command("band_max", BandMax0);
  enter_command("band_min", BandMin0);
  enter_command("bb", PreviousInput);
  enter_command("bbox", BoundingBox0);
  enter_command("biggest_component", BiggestComponent);
  enter_command("binary", Binary);
  enter_command("blur", Blur);
  enter_command("browse", BrowseImage0);
  enter_command("buffer_size", set_buffer_size);
  enter_command("buffer_default_size", set_default_buffer_size);

  enter_command("canonical_orientation", test_canonical_orientation);
  enter_command("cd", ChangeDirectory);
  enter_command("center", Center0);
  enter_command("channel_average", channel_average);
  enter_command("channel_total", channel_total);
  enter_command("circular_distance", CircularDistance0);
  enter_command("clear", ClearImage);
  enter_command("clear_inputs", ClearInputs);
  enter_command("color", to_color);
  enter_command("color_bound", to_color_bound);
  enter_command("compare_images", compare_images);
  enter_command("compare_inputs", CompareInputs0);
  enter_command("connect", ConnectedComponents);
  enter_command("connected_size_threshold", connected_size_threshold);
  enter_command("connect8", ConnectedComponents8);
  enter_command("copy_band", CopyBand0);
  enter_command("copy_input", CopyInput0);
  enter_command("count_gequal", CountGequal0);
  enter_command("count_color", CountColor0);
  enter_command("count_lequal", CountLequal0);
  enter_command("count_non_zero", CountNonZero);
  enter_command("count_value", CountValue0);
  enter_command("ct", channel_total);
  enter_command("current", ShowCurrentFrame);
  enter_command("decres", DecreaseResolution);
  enter_command("default", default_input0);
  enter_command("dilate", DilateImage);
  enter_command("double_size", DoubleSize);
  enter_command("double_threshold", double_threshold);
  enter_command("draw_circle", DrawCircle);
  enter_command("draw_fcircle", DrawFilledCircle);
  enter_command("draw_frectangle", DrawFilledRectangle);
  enter_command("draw_line", DrawLine);
  enter_command("draw_line2", DrawLineB);
  enter_command("draw_rectangle", DrawRectangle);
  enter_command("edge_orientations", EdgeOrientations);
  enter_command("edge_orientations2", EdgeOrientations2);
  enter_command("edges", UndirectedEdges);
  enter_command("entry", entry_value);
  enter_command("entry_rank", entry_rank);
  enter_command("entry_threshold_rank", entry_threshold_rank);
  enter_command("er", entry_rank);
  enter_command("etr", entry_threshold_rank);

  enter_command("f", NextInSequence);
  enter_command("ff", NextInput);
  enter_command("file_copy", function_file_copy);
  enter_command("fill", FillRegion0);
  enter_command("fill_up_holes", FillUpHoles);
  enter_command("find_largest", find_largest);
  enter_command("find_smallest", find_smallest);
  enter_command("forward", ForwardImageBuffer0);
  enter_command("frame", JumpToFrame0);

  enter_command("gradient_norms", GradientNorms);
  enter_command("gray", ToGray);
  enter_command("gray_bound", ToGrayBound);

  enter_command("heon", histogram_equalization_on);
  enter_command("heoff", histogram_equalization_off);
  enter_command("histogram", Histogram0);
  enter_command("histogram3", Histogram30);
  enter_command("histogram_equalization", histogram_equalization);
  enter_command("histogram_equalization_offset", histogram_equalization_offset);
  enter_command("histogram_equalization_size", histogram_equalization_size);
  enter_command("histogram_equalization_weights", histogram_equalization_weights_test);

  enter_command("image_absolute", image_absolute);
  enter_command("image_add", image_add);
  enter_command("image_average", image_average);
  enter_command("image_diff", image_diff);
  enter_command("image_divide", image_divide);
  enter_command("image_max", ImageMax0);
  enter_command("image_median", image_median);
  enter_command("image_min", ImageMin0);
  enter_command("image_multiply", image_multiply);
  enter_command("image_subtract", image_subtract);
  enter_command("image_total", image_total);
  enter_command("incres", IncreaseResolution);
  enter_command("ininfo", PrintInputs);
  enter_command("input",  SetCurrentInput);
  enter_command("intensity_maxima", IntensityMaxima);
  enter_command("inverse_threshold_entries", inverse_threshold_entries);
  enter_command("ite", inverse_threshold_entries);
  enter_command("ithr", InverseThresholdImage);

  enter_command("keep_largest", keep_largest);
  enter_command("keep_smallest", keep_smallest);

  enter_command("load", LoadImage);
  enter_command("load_simple_float", load_simple_float);

  enter_command("make_short", make_short);
  enter_command("make_ushort", make_ushort);
  enter_command("make_long", make_long);
  enter_command("make_ulong", make_ulong);
  enter_command("make_float", make_float);
  enter_command("make_double", make_double);
  enter_command("mass_center", CenterOfMass);
  enter_command("max_neighbors", MaxNeighbors);
  enter_command("maximizing_channels", maximizing_channels);
  enter_command("motion", ComputeMotion0);
  enter_command("move_sequence", MoveSequence);
  enter_command("multiply", MultiplyImage);

  enter_command("new_image", NewImage);
  enter_command("next", ShowNextImage);
  enter_command("nn", ShowNextImage);
  enter_command("non_maxima", NonMaximaSuppression);
  enter_command("nonzero_average", nonzero_average);
  enter_command("normalized_coordinates", PrintNormalizedCoordinates0);
  enter_command("normalize_intensity", NormalizeIntensity);
  enter_command("nz", CountNonZero);

  enter_command("overlay", Overlay0);

  enter_command("pad", Pad);
  enter_command("pad2", Pad2);
  enter_command("ph", print_header);
  enter_command("pcf", toggle_preserve_current_frame);
  enter_command("pixel", PrintPixelInfo);
  enter_command("pixel_rank", pixel_rank);
  enter_command("pixel_threshold_rank", pixel_threshold_rank);
  enter_command("pp", ShowPreviousImage);
  enter_command("pr", pixel_rank);
  enter_command("previous", ShowPreviousImage);
  enter_command("print_header", print_header);
  enter_command("process_sequence", process_sequence);
//  enter_command("program", InteractiveExecuteProgram);
  enter_command("ptr", pixel_threshold_rank);

  enter_command("random", RandomFrame);
  enter_command("random_image", random_image);
  enter_command("randomize0", Randomize0);
  enter_command("randomize1", Randomize1);
  enter_command("range", SelectRange0);
  enter_command("redraw", RedrawWindow);
  enter_command("recenter", Recenter);
  enter_command("remove_input", RemoveCurrentInput);
  enter_command("remove_inputs", remove_inputs);
  enter_command("remove_negatives", remove_negatives);
  enter_command("remove_output", RemoveCurrentOutput);
  enter_command("replace_color", ReplaceColor0);
  enter_command("reset_bounds", ResetBounds);
  enter_command("reset_magnified", ResetMagnifiedSize);
  enter_command("resize", resize);
  enter_command("resize_image", ResizeImage0);
  enter_command("rewind", RewindImageBuffer0);
  enter_command("rotate", RotateImage);
  enter_command("rotleft", RotateLeft0);
  enter_command("rotright", RotateRight0);

  enter_command("save_image", SaveImage);
  enter_command("save_filtered_frames", save_filtered_frames);
  enter_command("save_pgm", SavePGM);
  enter_command("scale_bbox", ScaleBoundingBox0);
  enter_command("scale_bbox2", ScaleBoundingBox0b);
  enter_command("scale_down1", ScaleImageA0);
  enter_command("scale_down2", ScaleImageB0);
  enter_command("scale_image", ScaleImage0);
  enter_command("select_bbox", SelectBoundingBox0);
  enter_command("select_color", SelectColor0);
  enter_command("select_component", SelectComponent);
  enter_command("select_components", SelectComponents);
  enter_command("select_pixels", SelectPixels0);
  enter_command("select_rectangle", select_rectangle);
  enter_command("set_bounds", SetBounds);
  enter_command("set_frame_rate", set_frame_rate);
  enter_command("set_magnified", SetMagnifiedSize0);
  enter_command("set_range", set_range);
  enter_command("setres", SetResolution0);
  enter_command("sleep", WaitSeconds0);
  enter_command("sync", SynchronizeSequences);

  enter_command("te", threshold_entries);
  enter_command("threshold", ThresholdCurrentImage);
  enter_command("threshold_entries", threshold_entries);
  
  enter_command("time_precomputed_square_root", time_precomputed_square_root);
  enter_command("time_square_root", time_square_root);
  enter_command("time_precomputed_cosine", time_precomputed_cosine);
  enter_command("time_cosine", time_cosine);
  enter_command("time_precomputed_sine", time_precomputed_sine);
  enter_command("time_sine", time_sine);
  enter_command("time_precomputed_inverse_tangent", time_precomputed_inverse_tangent);
  enter_command("time_inverse_tangent", time_inverse_tangent);
  enter_command("time_canonical_range", time_canonical_range);

  enter_command("top_bottom", TopBottom);
  enter_command("translate", Translate0);
  enter_command("trash", Trash);
  enter_command("tt", ThresholdCurrentImage);

  enter_command("value", SelectValue0);

  enter_command("write_frame", write_frame);
  enter_command("write_frames", write_frames0);
  enter_command("write_range", WriteRange0);
  enter_command(">>", NextInSequence);
  enter_command("<<", PreviousInSequence);

  return 1;
}


long graphical_module::enter_command(char * command, type_procedure procedure)
{
  commands.push_back(function_copy_string (command));
  procedures.push_back(procedure);
  return 1;
}


long graphical_module::process_command(const char * command)
{
  long index = command_index (command);
  if (index < 0)
  {
    return 0;
  }
  type_procedure procedure = procedures [index];
  (this->*procedure)();
  return 1;
}

long graphical_module::sort_commands()
{
  vector <long> indices;
  function_string_ranks (&commands, & indices);
  
  vector <char*> temporary;
  temporary.insert(temporary.end (), commands.begin (), commands.end ());
  vector <type_procedure> temporary_procedures;
  temporary_procedures.insert(temporary_procedures.end(), procedures.begin (), procedures.end ());

  long number = commands.size ();
  long counter;
  for (counter = 0; counter < number; counter ++)
  {
    long index = indices [counter];
    commands[counter] = temporary [index];
    procedures [counter] = temporary_procedures [index];
  }

  return 1;
}


//////////////////////////////////////////////////////////
//                                                      //
//  end of obligatory member functions for any module   //
//                                                      //
//////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////
//                                                      //
// start of event handling for the graphical interface  //
//                                                      //
//////////////////////////////////////////////////////////


// Processing keyboard commands.
// Returns 1 when it is time to exit.
// Processing keyboard commands.
// Returns 1 when it is time to exit.
ushort graphical_module::ProcessKey(char command)
{
 switch(command)
 {
 case 'a':
   this->SetSequenceStep();
   break;

 case 'B':
   this->PreviousInput();
   break;

 case 'b':
   this->PreviousInSequence();
   break;

 case 'c':
   break;

 case 'C':
   break;

 case 'd':
   break;

   // Sometimes we want to use the mouse to define a set of lines on an
   // image. In those cases, 'D' deletes the last vertex.
 case 'D':
   break;

 case 'e':
   break;

 case 'E':
   break;

 case 'f':
   this->NextInSequence();
   break;

 case 'F':
   this->NextInput();
   break;

 case 'g':
   break;

 case 'G':
   break;

 case 'h':
   this->histogram_equalization();
   break;

 case 'i':
   this->InverseThresholdImage();
   break;

 case 'I':
   this->mouse_click_response =
this->pixel_info_response;
   break;

 case 'j':
   this->SetCurrentCounter();
   this->ShowCurrentFrame();
   break;

 case 'J':
   break;

 case 'k':
   break;

 case 'K':
   break;

 case 'l':
   this->ShowCurrentFrame();
   break;

 case 'L':
   break;

 case 'm':
   this->SynchronizeSequences();
   this->NextInput();
   break;

 case 'M':
   break;

 case 'n':
   this->ShowNextImage();
   break;

 case 'N':
   break;

 case 'o':
   break;

 case 'O':
   break;

 case 'p':
   this->ShowPreviousImage();
   break;

 case 'P':
   break;

 case 'q':
    vConsoleToForeground();
    return 1;
   break;

 case 'r':
   this->RedrawWindow();
   break;

 case 's':
   break;

 case 'S':
   break;

 case 't':
   this->ThresholdCurrentImage();
   break;

 case 'T':
   break;

 case 'u':
   break;

 case 'U':
   break;

 case 'v':
   break;

 case 'w':
   if ((this->input != 0) &&
       ((this->input->sequence == 0) ||
       (this->input->sequence->Top() != -1)))
   {
     this->RemoveCurrentInput();
   }
   break;

 case 'z':
   break;

 case '1':
 case '2':
 case '3':
 case '4':
 case '5':
 case '6':
 case '7':
 case '8':
 case '9':
 case '0':
   this->browse_image(command - '0');
   break;

 case '[':
   this->PreviousInput();
   break;

 case ']':
   this->NextInput();
   break;

 case '\\':
   this->ResetBounds();
   break;

 case '{':

 case '!':
   break;

 case ';':
   this->match_window_size();
   break;

 case '\'':
   break;

 case ':':
   break;

 case ',':
   break;

 case '.':
   break;

 case '/':
   break;

 case '`':
   break;

 case '=':
   vConsoleToForeground();
   base_interpreter->main_loop(1, 0);
   break;

 case '+':
   break;

 case '_':
   break;

 default:
   break;

 }
 return 0;
}
// End of processing keyboard commands.


// We want to do different things with the mouse at different times.
// The interpreter has a variable, mouse_click_response, 
// which indicates what should be done with a mouse click.
// event_type:
// - 1: left button down
// - 2: left button up
// - 4: mouse move, no pressed left button
// - 5: mouse move, pressed button
ushort graphical_module::ProcessMouseClick(long event_type, long row, long col)
{
  long normalized_row, normalized_col;
  normalized_row = normalized_col = 0;
  NormalizedCoordinates(row, col, &normalized_row, &normalized_col);

  // In this case we show the color of the pixel on which the user clicked
  if (mouse_click_response == pixel_info_response) 
  {
    ShowPixelInfo(normalized_row, normalized_col);
    return 1;
  }

  // In this case we center the displayed image around the click
  // (useful when we display with resolutions > 1, i.e. when
  // we magnify images)
  else if (mouse_click_response == CenterResponse)
  {
    long result = Center(normalized_row, normalized_col);
    mouse_click_response = previous_mouse_response;
    if (mouse_click_response == CenterResponse)
    {
      mouse_click_response = pixel_info_response;
    }
    return (ushort) result;
  }

  // In this case we set the region where the current sequence is
  // displayed to the region whose top-left corner is the point
  // where the mouse clicked.
  else if (mouse_click_response == MoveSequenceResponse)
  {
    MoveSequence(row, col);
    mouse_click_response = pixel_info_response;
  }

  else if (mouse_click_response == LineResponse)
  {
  }

  else if (mouse_click_response == CutoffLabelResponse)
  {
  }

  else if (mouse_click_response == AddFeaturesResponse)
  {
  }
 
  else if (mouse_click_response == choose_rectangle_response)
  {
  }

  else if (mouse_click_response == spline_response)
  {
    ShowPixelInfo(normalized_row, normalized_col);
  }

  else
  {
    exit_error("Impossible mouse click response\n");
  }

  return 1;
}


//////////////////////////////////////////////////////////
//                                                      //
//  end of event handling for the graphical interface   //
//                                                      //
//////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////
//                                                      //
// start of core functions for the graphical interface  //
//                                                      //
//////////////////////////////////////////////////////////


long graphical_module::initialize(graphical_interface_globals * in_globals)
{
  window = in_globals->window; 

  pixel_info_window = in_globals->pixel_info_window;
  frame_info_window = in_globals->frame_info_window;
  min_width = in_globals->min_width;
  extra_height = in_globals->extra_height; 

  return 1;
}


long graphical_module::store_image(general_image * the_image)
{
  if (the_image == 0) return 0;
  if (input == 0)
  {
    exit_error ("\nerror: show_image called without an input\n");
  }

  // Since images that are shown by calling ShowImage get deleted
  // automatically by the system, we should never call ShowImage
  // if there is no current input sequence. If we did, then
  // we would not be able to display it and save it in an image
  // ring from which it would ultimately get deleted, and we would
  // get a memory leak.

  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + 1) % buffer_size;
  vdelete(input->image_buffer[input->buffer_index]);

  input->image_buffer[input->buffer_index] = the_image;
  input->current_image = the_image;

  return 1;
}


long graphical_module::ShowImage(general_image * the_image)
{
  if (the_image == 0) return 0;
  if (input == 0) 
  {
    print("\ncreating new input\n");
    default_input(50, 50);
    if (input == 0)
    {
      exit_error("\nerror: failed to create default input\n");
    }
  }

  store_image (the_image);
  DrawCurrentImage();
  return 1;
}


long graphical_module::AddInput()
{
  vPrint("Add sequence: ");
  char buffer[400];
  vScan("%s", buffer);
  return AddInput(buffer);
}


long graphical_module::AddInput(const char * filename)
{
  if (filename[0] == 0) return 0;
  vVideoFile * new_video_file;
  new_video_file = vOpenReadVideoFile(filename);
  if (new_video_file != 0)
  {
    print("input: %s\n", filename);
    char * simple = function_simple_name(filename);
    print("simple: %s\n", simple);
    delete_pointer(simple);
  }

  return AddInput(new_video_file);
}


long graphical_module::AddInput(vVideoFile * new_video_file)
{
  if (new_video_file == 0)
  {
    return 0;
  }

  class_sequence_information * info = new class_sequence_information(default_buffer_size);
  info->sequence = new_video_file;
  info->path_name = vCopyString(new_video_file->GetFilename());
  info->simple_name = vSimpleName(info->path_name, platform_directory_slash);
  info->number_of_frames = new_video_file->NumberOfFrames();
  //info->magnified_rows = new_video_file->Rows();
  //info->magnified_cols = new_video_file->Cols();
  inputs.push_back(info);

  number_of_inputs++;

  SetCurrentInput(number_of_inputs - 1);
  long success = AdvanceSequence(0);
  if (success == 0)
  {
    vPrint("Error in reading first frame of file %s\n", info->path_name);
    RemoveInput(number_of_inputs - 1);
    return 0;
  }

  return 1;
}


long graphical_module::default_input0()
{
  vPrint("Input rows and cols for input: ");
  long rows = 0;
  long cols = 0;
  vScan("%li %li", &rows, &cols);
  return default_input(rows, cols);
}


// We can't call ShowImage unless we have a current input.
// However, sometimes we would like to create our own images
// and display them without having opened a sequence first.
// DefaultInput does just that, it creates a class_sequence_information
// object that corresponds to no input video sequence
long graphical_module::default_input(long rows, long cols)
{
  class_sequence_information * info = new class_sequence_information(default_buffer_size);
  info->sequence = 0;
  info->path_name = vCopyString("default input");
  info->simple_name = vCopyString("default input");
  info->number_of_frames = 0;
  inputs.push_back(info);

  number_of_inputs++;
  SetCurrentInput(number_of_inputs - 1);
  NewImage(rows, cols);

  return 1;
}


long graphical_module::RemoveInput(long index)
{
  class_sequence_information * info;
  if (index < 0) return 0;
  if (index >= number_of_inputs) return 0;
  info = (class_sequence_information *) inputs[index];
  if (info->is_camera_input != 0) return 0;

  info = (class_sequence_information *) S_EraseItem(&inputs, index);
  if (info == 0) return 0;
  vdelete(info);
  number_of_inputs--;
  if (number_of_inputs == 0)
  {
    current_input_index = -1;
    input = 0;
  }
  else if ((current_input_index >= index) && 
           (current_input_index >= 1))
           
  {
    SetCurrentInput(current_input_index - 1);
  }
  else
  {
    SetCurrentInput(0);
  }

  return 1;
}


long graphical_module::ClearInputs()
{
  while(number_of_inputs > 0)
  {
    RemoveInput(0);
  }

  return 1;
}


long graphical_module::PrintInputs()
{
  long i; 
  for (i = 0; i < number_of_inputs; i++)
  {
    class_sequence_information * info = (class_sequence_information *) inputs[i];
    long current_frame = info->CurrentFrame();
    long number_of_frames = info->number_of_frames;
    char * filename = info->simple_name;
    vPrint("%li :Frame %li of %li, %s\n", i, current_frame, number_of_frames, filename);
  }

  return 1;
}

// CurrentImage(i) returns the current image of the i-th
// input file, or 0 if it doesn't exist.
general_image * graphical_module::CurrentImage(long index)
{
  if (index < 0) return 0;
  if (index >= number_of_inputs) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[index];
  return info->current_image;
}

  
long graphical_module::AddOutput()
{
  char filename[1000];
  vPrint("Name of new output file: ");
  vScan("%s", filename);
  return AddOutput(filename);
}


// Add an output sequence, to which we can write frames.
// If the specified filename already exists, then we 
// alter the filename, copy into it the original sequence,
// and keep it open for writing.
long graphical_module::AddOutput(const char * in_filename)
{
  if (input == 0) return 0;
  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  char * filename;
  ushort exists = 0;
  if (vFileExists(in_filename))
  {
    char * name = vRemoveEnding(in_filename);
    char * ending = vGetEnding(in_filename);
    filename = vMergeStrings3(name, "2", ending);
    exists = 1;
  }
  else
  { 
    filename = vCopyString(in_filename);
  }
  
  vVideoFile * output;
  vVideoFile * in_sequence = input->sequence;
  if (in_sequence == 0)
  {
    output = vOpenWriteVideoFile(filename, rows, cols);
  }
  else
  {
    long rate = in_sequence->Rate();
    long scale = in_sequence->Scale();
    //long color_bytes = in_sequence->BytesPerPixel();
    long color_bytes = input->current_image->channels() * input->current_image->SizeOfType();
    output = vOpenWriteVideoFile(filename, rows, cols, 10, 1, color_bytes);
  }

  if (exists)
  {
    // Try to write the existing video file into the beginning of 
    // the new video file.
    long success = vWriteVideo(output, in_filename);
    // We could fail, if, for example, the sequence in in_filename
    // has a different frame size (rows and cols) than the current
    // input image.
    if (success == 0)
    {
      vCloseVideoFile(output);
      vPrint("Failed to copy %s to %s\n", in_filename, filename);
      vdelete2(filename);
      return 0;
    }
  }

  vdelete2(filename);
  return AddOutput(output);
}


// Add an output sequence, to which we can write frames.
// If the specified filename already exists, then we 
// alter the filename, copy into it the original sequence,
// and keep it open for writing.
long graphical_module::add_output_stream()
{
  if (input == 0) return 0;
  if (input->current_image == 0)
  {
    return 0;
  }

  char in_filename [1000];
  print ("\nenter filename:\n");
  question ("%s", in_filename);

  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  long bands = input->current_image-> Bands ();
  ::Type type_name = input->current_image->TypeName();

  char * filename;
  ushort exists = 0;
  if (vFileExists(in_filename))
  {
    char * name = vRemoveEnding(in_filename);
    char * ending = vGetEnding(in_filename);
    filename = vMergeStrings3(name, "2", ending);
    delete_pointer(name);
    delete_pointer(ending);
    exists = 1;
  }
  else
  { 
    filename = vCopyString(in_filename);
  }
  
  class_image_stream * output = new class_image_stream (filename, rows, cols, bands, type_name);
  vdelete2(filename);
  print("preparing output: %li rows, %li cols, %li bands, type: ", rows, 
        cols, bands);
  print_type_name(type_name);
  print("\n");
  return AddOutput(output);
}


long graphical_module::AddOutput(vVideoFile * output)
{
  if ((output == 0) || (output->IsValid() == 0)) return 0;
  output_files.push_back(output);
  output_names.push_back(vCopyString(output->GetFilename()));
  number_of_outputs++;
  current_output = number_of_outputs - 1;
  printf("Created output %s\n", output_names.back());

  return 1;
}


long graphical_module::RemoveOutput(long index)
{
  vVideoFile * sequence;
  sequence = (vVideoFile *) S_EraseItem(&output_files, index);
  if (sequence == 0) return 0;
  ushort result = 1;
  ushort success = sequence->FinishWriting();
  char * name = (char *) S_EraseItem(&output_names, index);
  vPrint("Closing %s\n", name);
  if (success == 0) 
  {
    result = 0;
    vPrint("Failed to close %s\n", name);
  }
  vCloseVideoFile(sequence);

  vdelete2(name);
  number_of_outputs--;
  if (current_output >= index) current_output--;

  return result;
}



long graphical_module::RemoveCurrentOutput()
{
  return RemoveOutput(current_output);
}

long graphical_module::ClearOutputs()
{
  while(number_of_outputs > 0)
  {
    RemoveOutput(0);
  }
  return 1;
}


long graphical_module::write_frame()
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  general_image * current = input->current_image;
  ushort result = 0;
  
  if (strcmp(output->Type(), "class_image_stream") == 0)
  {
    class_image_stream* stream = (class_image_stream*) output;

    //    double_image * image = new double_image(current);
    //    result = stream->save_image(image);
    //    vdelete(image);
    result = stream->save_image(current);
  }
  else if (predicate_color_image(current)) 
  {
    result = output->write_frame((color_image *) current);
  }
  else if (predicate_gray_image(current)) 
  {
    result = output->write_frameGray((grayscale_image *) current);
  }
  if (result != 0)
  {
    vPrint("Wrote frame\n");
  }
  else 
  {
    print_header();
    vPrint("failed to save frame\n");
  }

  return result;
}


long graphical_module::write_frames0()
{
  if (current_output == -1) return 0;
  if (input == 0) return 0;
  long number_of_frames = 0;
  vPrint("Input number of frames to write: ");
  vScan("%li", &number_of_frames);
  return graphical_module::write_frames(number_of_frames);
}


long graphical_module::write_frames(long number_of_frames)
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;

  if (number_of_frames <= 0) return 0;
  long buffer_size = input->buffer_size;
  if (number_of_frames > buffer_size) return 0;

  vPrint("Writing frames: \n");
  long i;
  for (i = 0; i < number_of_frames; i++)
  {
    write_frame();
    ShowNextImage();
    vPrint("Wrote frame %li\r", i);
  }
  vPrint("\nFinished writing frames\n");
  return 1;
}


long graphical_module::ChangeDirectory(const char * directory)
{
  long number = chdir(directory);
  ushort result;
  if (number == 0)
  {
    vdelete2(current_directory);
    current_directory = vCopyString(directory);
    vPrint("Changed to directory %s\n", current_directory);
    result = 1;
  }
  else
  {
    vPrint("Failed to change directory to %s.\n", directory);
    result = 0;
  }
  return result;
}


vVideoFile* graphical_module::GetInput(long index)
{
  if (index >= number_of_inputs) return 0;
  if (index < 0) return 0;
  class_sequence_information * info =  (class_sequence_information *) inputs[index];
  return info->sequence;
}


vVideoFile* graphical_module::GetOutput(long index)
{
  if (index >= number_of_outputs) return 0;
  return (vVideoFile *) output_files[index];
}


vWindow * graphical_module::GetWindow()
{
  return window;
}


// Here we can have overlapping images. The strategy is as follows: 
// - Take the rest of the inputs in order, and display their current
//   images, as long as no input preceding in order has the same 
//   display_top and display_left.
// - Draw the current image of the current input
long graphical_module::RedrawImages()
{
  if (input == 0) return 0;
  long current_top, current_left;
  current_top = input->display_top;
  current_left = input->display_left;
  long i, j;
  for (i = 0; i < number_of_inputs; i++)
  {
    if (i == current_input_index) continue;
    class_sequence_information * info1 = (class_sequence_information *) inputs[i];
    if ((info1->display_left == current_left) &&
        (info1->display_top == current_top))
    {
      continue;
    }
    ushort same_coords_flag = 0;
    for (j = 0; j < i; j++)
    {
      class_sequence_information * info2 = (class_sequence_information *) inputs[j];
      if ((info1->display_top == info2->display_top) &&
          (info1->display_left == info2->display_left))
      {
        same_coords_flag = 1;
        break;
      }
    }
    if (same_coords_flag != 0) continue;
    DrawInput(i);
  }
  DrawCurrentImage();

  return 1;
}


long graphical_module::DrawCurrentImage()
{
  if (input == 0) return 0;
  if (window == 0) return 0;
  general_image * the_image = input->current_image;
  if (the_image == 0) return 0;
  long top = input->display_top;
  long left = input->display_left;
  long sampling = 1;

  general_image * image_to_show;
  if ((histogram_equalization_flag) && (the_image->TypeName() == UshortType))
  {
    image_to_show = function_histogram_equalization((ushort_image *) the_image, equalization_window_size);
  }
  else
  {
    image_to_show = the_image;
  }

  if (input->resolution <= 0)
  {
    window->ShowImage(image_to_show, top, left, sampling);
  }

  else // We magnify
  {
    long factor = (long) pow(2, input->resolution);
//    general_image * scaled = vMagnify0(the_image, input->center_row, 
//                                      input->center_col, input->magnified_rows,
//                                      input->magnified_cols, factor);
    general_image * scaled = vMagnify0(image_to_show, input->center_row, 
                                      input->center_col, the_image->Rows() * factor,
                                      the_image->Cols() * factor, factor);

    window->ShowImage(scaled, top, left, sampling);
    vdelete(scaled);
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (input->automatic_centering == 1)
  {
    input->center_row = rows / 2;
    input->center_col = cols / 2;
  }
  if (image_to_show != the_image)
  {
    vdelete(image_to_show);
  }

  return 1;
}


long graphical_module::RedrawWindow()
{
  window->Clear();
  window->RedrawSubwindows();
//  RedrawImages();

  return 1;
}


long graphical_module::SetBounds()
{
  if (input == 0) return 0;
  long top, left, rows, cols;
  top = left = rows = cols = 0;
  vPrint("Type in top, left, rows, columns: ");
  vScan("%li %li %li %li\n", &top, &left, &rows, &cols);
  graphical_module::SetBounds(top, left, rows, cols);

  return 1;
}


long graphical_module::ResetBounds()
{
  if (input == 0) return 0;
  vVideoFile * video_file = input->sequence;
  SetBounds(0, 0, video_file->Rows(), video_file->Cols());

  return 1;
}

long graphical_module::SetBounds(long top, long left, 
		                               long rows, long cols)
{
  if (input == 0) return 0;
  vVideoFile * file = input->sequence;
  if (file == 0) return 0;
  file->SetBounds(top, left, rows, cols);
  graphical_module::ShowCurrentFrame();

  return 1;
}


// We need to normalize them based on the magnification
// factor (2 to the power of resolution), the center_row and
// center_col of the input, and the top and left of the location
// where the input is displayed
long graphical_module::NormalizedCoordinates(long in_row, long in_col,
                                           long * out_row, long * out_col)
{
  if (input == 0) return 0;

  long subwindow_row = in_row - input->display_top;
  long subwindow_col = in_col - input->display_left;
  long factor = (long) pow(2, input->resolution);
  // If the resolution is less than 1, we don't do any scale normalization
  if (factor == 0) factor = 1;

//  long input_rows = input->magnified_rows / factor;
//  long input_cols = input->magnified_cols / factor;

  long input_top, input_left;
  if (input->resolution > 0)
  {
//    input_top = input->center_row - input_rows / 2;  
//    input_left = input->center_col - input_cols / 2;
    input_top = input->display_top;  
    input_left = input->display_left;
  }
  else
  {
    input_top = 0;
    input_left = 0;
  }

  *out_row = subwindow_row / factor;
  *out_col = subwindow_col / factor;
  //vPrint("(%li, %li) ==> (%li, %li)\n", in_row, in_col, *out_row, *out_col);

  return 1;
}


long graphical_module::PrintNormalizedCoordinates0()
{
  if (input == 0) return 0;
  long in_row, in_col;
  in_row = in_col = 0;
  vPrint("\nEnter in_row, in_col:\n");
  vScan("%li %li", &in_row, &in_col);
  return PrintNormalizedCoordinates(in_row, in_col);

}


long graphical_module::PrintNormalizedCoordinates(long in_row, long in_col)
{
  if (input == 0) return 0;
  long out_row, out_col;
  out_row = out_col = 0;
  NormalizedCoordinates(in_row, in_col, &out_row, &out_col);
  vPrint("input->top = (%li, %li)\n", 
          input->display_top, input->display_left);
  vPrint("input->center = (%li, %li)\n", 
          input->center_row, input->center_col);
//  vPrint("input display size = %li x %li\n", 
//          input->magnified_rows, input->magnified_cols);
  vPrint("(%li, %li) ==> (%li, %li)\n", in_row, in_col, out_row, out_col);
  return 1;
}

  
long graphical_module::ShowPixelInfo(long row, long col)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (pixel_info_window == 0) return 0;

  long width = input->current_image->Cols();
  long height = input->current_image->Rows();

  row = Max(row, (long) 0);
  row = Min(row, (long) (height-1));
  col = Max(col, (long) 0);
  col = Min(col, (long) (width-1));
  long bands = input->current_image->Bands();

  char buffer[1000];

  Type type_name = input->current_image->TypeName();
  Type super_type = vSuperType(type_name);
  if ((super_type == LongType) || (super_type == UlongType))
  {
    vector<long> values;
    input->current_image->LongValues(values, row, col);
    long intensity = 0;
    long i;
    for (i = 0; i < bands; i++)
      intensity += values[i];
    sprintf(buffer, "(%li, %li)  (", row, col);
    for (i = 0; i < bands; i++)
    {
      sprintf(buffer, "%s%li", buffer, values[i]);
      if (i < bands - 1) sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)  %li  (", buffer, intensity);

    double divisor;
    if (intensity == 0) divisor = 1000000;
    else divisor = ((double) intensity) / 255.0;
    for (i = 0; i < bands; i++)
    {
      double value = values[i];
      value = value / divisor;
      sprintf(buffer, "%s%li", buffer, (long) value);
      if (i < bands - 1) sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)                                   ", buffer);
  }
  else
  {
    assert(super_type == DoubleType);
    vector<double> values;
    input->current_image->DoubleValues(values, row, col);
    double intensity = 0;
    long i;
    for (i = 0; i < bands; i++)
      intensity += values[i];
    sprintf(buffer, "(%li, %li)  (", row, col);
    for (i = 0; i < bands; i++)
    {
      sprintf(buffer, "%s%lf", buffer, values[i]);
      if (i < bands - 1)
	sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)  %lf                                   ", 
	    buffer, intensity);
  }
  pixel_info_window->DrawString(3, 13, buffer);

  return 1;
}

long graphical_module::ShowFrameInfo()
{
  if (pixel_info_window == 0) return 0;
  if (current_input_index== -1) return 0;
  char buffer[200];
  sprintf(buffer, "%s - Frame %li of %li                        ", 
          input->simple_name,
          input->CurrentFrame(),
          input->number_of_frames);

  frame_info_window->DrawString(3, 13, buffer);

  return 1;
}


long graphical_module::PrintPixelInfo()
{
  if (input == 0) return 0;
  vPrint("Enter row and col: ");
  long row = 0;
  long col = 0;
  vScan("%li %li", &row, &col);
  return PrintPixelInfo(row, col);
}


long graphical_module::PrintPixelInfo(long row, long col)
{
  if (input == 0) return 0;
  vector<double> values;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  if (row >= rows) return 0;
  if (row < 0) return 0;
  if (col >= cols) return 0;
  if (col < 0) return 0;

  current->DoubleValues(values, row, col);
  long i;
  for (i = 0; i < (long) values.size(); i++)
  {
    vPrint("%lf ", values[i]);
  }
  vPrint("\n");
  return 1;
}


long graphical_module::entry_value()
{
  if (input == 0) return 0;
  vPrint("Enter row, col, and band: ");
  long row = 0;
  long col = 0;
  long channel = 0;
  vScan("%li %li %li", &row, &col, & channel);

  vector<double> values;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  if (row >= rows) return 0;
  if (row < 0) return 0;
  if (col >= cols) return 0;
  if (col < 0) return 0;
  if (channel < 0) return 0;
  if (channel >= current->channels()) return 0;


  current->DoubleValues(values, row, col);
  vPrint("%lf\n", values[channel]);
  vPrint("\n");
  return 1;
}



long graphical_module::SynchronizeSequences()
{
  if (input == 0) return 0;
  long old_current_input_index = current_input_index;
  long current_frame = input->CurrentFrame();
  long i;
  for (i = 0; i < number_of_inputs; i++)
  {
    if (i == old_current_input_index) continue;
    SetCurrentInputSilent(i);
    input->SetCurrentFrame(current_frame);
    StoreCurrentFrame();
  }
  SetCurrentInput(old_current_input_index);

  return 1;
}


// Makes the current image all black
long graphical_module::ClearImage()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * the_image = (color_image *) current->ToRGBCopy();
  long all_band_size = the_image->AllBandSize();
  long size_of_type = the_image->SizeOfType();
  long byte_size = all_band_size * size_of_type;
  vArray(uchar) matrix = the_image->Matrix();
  bzero(vData(matrix), byte_size);
  ShowImage(the_image);
  return 1;
}



long graphical_module::ShowPreviousImage()
{
  if (input == 0) return 0;
  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + buffer_size - 1) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + 1) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
  }
  else 
  {
    DrawCurrentImage();
  }

  return 1;
}


long graphical_module::ShowNextImage()
{
  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + 1) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
    input->buffer_index = (input->buffer_index + buffer_size - 1) % buffer_size;
    input->current_image = input->image_buffer[input->buffer_index];
  }
  else 
  {
    DrawCurrentImage();
  }

  return 1;
}


//Subtract from or add a given number to the image buffer index.
long graphical_module::RewindImageBuffer0()
{
  long frames_to_skip = 0;
  vPrint("Input number of frames to skip backwards: ");
  vScan("%li", &frames_to_skip);
  return RewindImageBuffer(frames_to_skip);
}


long graphical_module::ForwardImageBuffer0()
{
  long frames_to_skip = 0;
  vPrint("Input number of frames to skip forward: ");
  vScan("%li", &frames_to_skip);
  return ForwardImageBuffer(frames_to_skip);
}

long graphical_module::RewindImageBuffer(long frames_to_skip)
{
  if (input == 0) return 0;
  if (frames_to_skip < 0) return 0;
  long buffer_size = input->buffer_size;
  if (frames_to_skip >= buffer_size) return 0;

  input->buffer_index = (input->buffer_index + buffer_size - frames_to_skip) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + frames_to_skip) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
      return 0;
  }
  else 
  {
    DrawCurrentImage();
    return 1;
  }
}


long graphical_module::ForwardImageBuffer(long frames_to_skip)
{
  if (input == 0) return 0;
  if (frames_to_skip < 0) return 0;
  long buffer_size = input->buffer_size;
  if (frames_to_skip >= buffer_size) return 0;

  input->buffer_index = (input->buffer_index + frames_to_skip) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + buffer_size - frames_to_skip) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
      return 0;
  }
  else 
  {
    DrawCurrentImage();
    return 1;
  }
}


long graphical_module::ChangeDirectory()
{
  char buffer[1000];
  print("\nenter new directory:\n");
  question("%s", buffer);
  return ChangeDirectory(buffer);
}


long graphical_module::DrawInput(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  long previous_index = current_input_index;
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  DrawCurrentImage();
  current_input_index = previous_index;
  input = (class_sequence_information *) inputs[current_input_index];
  return 1;
}


long graphical_module::SetCurrentInput()
{
  long index = 0;
  vPrint("New current input index: ");
  vScan("%li", &index);
  return SetCurrentInput(index);
}



long graphical_module::SetCurrentInput(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  char * parent = vParentDir(input->path_name, platform_directory_slash);
  ChangeDirectory(parent);
  vdelete2(parent);
  DrawCurrentImage();
  ShowFrameInfo();
  return 1;
}


long graphical_module::NextInput()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ 1) % number_of_inputs;
  long current_frame = input->current_frame;
  if (preserve_current_frame > 0)
  {
    SetCurrentInputSilent(next_index);
    JumpToFrame(current_frame);
  }
  else
  {
    SetCurrentInput(next_index);
  }

  return 1;
}


long graphical_module::PreviousInput()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ number_of_inputs - 1) % number_of_inputs;
  if (preserve_current_frame > 0)
  {
    long current_frame = input->current_frame;
    SetCurrentInputSilent(next_index);
    JumpToFrame(current_frame);
  }
  else
  {
    SetCurrentInput(next_index);
  }


  return 1;
}


long graphical_module::SetCurrentInputSilent(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  char * parent = vParentDir(input->path_name, platform_directory_slash);
  ChangeDirectory(parent);
  vdelete2(parent);
  return 1;
}


long graphical_module::NextInputSilent()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ 1) % number_of_inputs;
  long current_frame = input->current_frame;
  SetCurrentInputSilent(next_index);
  if (preserve_current_frame > 0)
  {
    JumpToFrame(current_frame);
  }

  return 1;
}


long graphical_module::PreviousInputSilent()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ number_of_inputs - 1) % number_of_inputs;
  long current_frame = input->current_frame;
  SetCurrentInputSilent(next_index);
  if (preserve_current_frame > 0)
  {
    JumpToFrame(current_frame);
  }

  return 1;
}


long graphical_module::RemoveCurrentInput()
{
  if (current_input_index== -1) return 0;
  
  RemoveInput(current_input_index);
  RedrawWindow();

  return 1;
}


long graphical_module::remove_inputs()
{
  long number = number_of_inputs;
  long counter;
  for (counter = 0; counter <number; counter++)
  {
    RemoveCurrentInput();
  }
  
  return 1;
}


long graphical_module::MoveSequence()
{
  long top = 0;
  long left = 0;
  vPrint("New top and left of sequence display: ");
  vScan("%li %li", &top, &left);
  return MoveSequence(top, left);
}


long graphical_module::MoveSequenceWithClick()
{
  mouse_click_response = MoveSequenceResponse;
  return 1;
}


long graphical_module::MoveSequence(long top, long left)
{
  if (current_input_index== -1) return 0;
  input->display_top = top;
  input->display_left = left;
  RedrawWindow();
  return 1;
}

  
long graphical_module::Recenter()
{
  if (input == 0) return 0;
  input->automatic_centering = 1;
  general_image * current = input->current_image;
  input->center_row = current->Rows() / 2;
  input->center_col = current->Cols() / 2;
  return 1;
}


long graphical_module::Center0()
{
  if (input == 0) return 0;
  long center_row, center_col;
  center_row = center_col = 0;
  vPrint("Enter center row and center col: ");
  vScan("%li %li", &center_row, &center_col);
  return Center(center_row, center_col);
}


// We center the image around center row and center col. This
// makes a difference only for resolutions > 0, i.e. when 
// we magnify an image. In such cases, (center_row, center_col)
// must be converted back to the corresponding resolution 0 
// coordinates, because centering is done with respect to the
// original image.
long graphical_module::Center(long center_row, long center_col)
{
  if (input == 0) return 0;
  input->automatic_centering = 0;

  input->center_row = center_row;
  input->center_col = center_col;
  DrawCurrentImage();
  return 1;
}


long graphical_module::CenterWithClick()
{
  if (input == 0) return 0;
  previous_mouse_response = mouse_click_response;
  mouse_click_response = CenterResponse;
  return 1;
}

  
long graphical_module::SetMagnifiedSize0()
{
  if (input == 0) return 0;
  long rows, cols;
  rows = cols = 0;
  vPrint("Enter rows and cols for magnified display: ");
  vScan("%li %li", &rows, &cols);
  return SetMagnifiedSize(rows, cols);
}


long graphical_module::ResetMagnifiedSize()
{
  if (input == 0) return 0;
  long factor = (long) pow(2, input->resolution);
  long rows = input->sequence->Height() * factor;
  long cols = input->sequence->Width() * factor;

  return SetMagnifiedSize(rows, cols);
}


long graphical_module::SetMagnifiedSize(long rows, long cols)
{
  if (input == 0) return 0;
  if (rows < 10) return 0;
  if (cols < 10) return 0;
  if (rows > 12000) return 0;
  if (cols > 20000) return 0;
//  input->magnified_rows = rows;
//  input->magnified_cols = cols;
  DrawCurrentImage();
  return 1;
}

  
long graphical_module::SetCurrentCounter()
{
  if (input == 0) return 0;
	printf("Current counter is %li. New one >> ", input->CurrentFrame());
  long current_frame = 0;
	scanf("%li", &current_frame);
  input->SetCurrentFrame(current_frame);

  return 0;
}


long graphical_module::IncreaseResolution()
{
  if (input == 0) return 0;
  long resolution = input->resolution;
  return SetResolution(resolution+1);
}


long graphical_module::DecreaseResolution()
{
  if (input == 0) return 0;
  long resolution = input->resolution;
  return SetResolution(resolution-1);
}


long graphical_module::SetResolution0()
{
  vPrint("Input new resolution: ");
  long new_resolution;
  vScan("%li", &new_resolution);
  return SetResolution(new_resolution);
}


long graphical_module::SetResolution(long new_resolution)
{
  if (input == 0) return 0;
  input->resolution = new_resolution;
  long factor = (long) pow(2, new_resolution);
//  long magnified_rows, magnified_cols;
//  if (input->sequence != 0)
//  {
//    magnified_rows = factor * input->sequence->Height();
//    magnified_cols = factor * input->sequence->Width();
//  }
//  else 
//  {
//    magnified_rows = factor * input->current_image->Rows();
//    magnified_cols = factor * input->current_image->Cols();
//  }
//  if (magnified_rows > 600) magnified_rows = 600;
//  if (magnified_cols > 800) magnified_rows = 800;
//  input->magnified_rows = magnified_rows;
//  input->magnified_cols = magnified_cols;
  DrawCurrentImage();

  vPrint("Resolution = %li\n", new_resolution);
  return 1;
}


long graphical_module::NextInSequence()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  AdvanceSequence(input->sequence_step);

  return 1;
}


long graphical_module::AdvanceSequence(long num_frames)
{
  if (current_input_index == -1) return 0;
  if (input->sequence == 0) return 0;
  if (input->number_of_frames == 0) return 0;
  input->SetCurrentFrame((input->CurrentFrame() + num_frames) % input->number_of_frames);
  long success = ShowCurrentFrame();
  if (!success) input->SetCurrentFrame(input->CurrentFrame() - num_frames);
  return success;
}


long graphical_module::ShowCurrentFrame()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  general_image * the_image = ReadCurrentImage();

  if (the_image == 0) return 0;
  if (input->resolution >= 0)
  {
    ShowImage(the_image);
  }
  else
  {
    general_image * scaled = vPyramidImage0(the_image, -input->resolution);
    ShowImage(scaled);
    vdelete(the_image);
  }

  ShowFrameInfo();
  return 1;
}


long graphical_module::StoreCurrentFrame()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  general_image * the_image = ReadCurrentImage();

  if (the_image == 0) return 0;
  if (input->resolution >= 0)
  {
    store_image(the_image);
  }
  else
  {
    general_image * scaled = vPyramidImage0(the_image, -input->resolution);
    store_image(scaled);
    vdelete(the_image);
  }

  ShowFrameInfo();
  return 1;
}


general_image * graphical_module::ReadCurrentImage()
{
  general_image * the_image;
  if (input == 0) return 0;
  vVideoFile * video_file = input->sequence;
  if (video_file == 0) return 0;
  the_image = video_file->ReadImage(input->CurrentFrame());
  if (the_image == 0)
  {
    vPrint("Video file has %li frames, attempt to access frame %li failed.\n",
	         video_file->NumberOfFrames(), input->CurrentFrame());
  }
  return the_image;
}


long graphical_module::SetCurrentFrame(long index)
{
  if (input == 0) return 0;
  return input->SetCurrentFrame(index);
}


long graphical_module::PreviousInSequence()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  input->SetCurrentFrame(input->CurrentFrame() - input->sequence_step);
  long success = ShowCurrentFrame();
  if (!success) 
  {
    input->SetCurrentFrame(input->CurrentFrame() + input->sequence_step);
  }

  return 1;
}



long graphical_module::BrowseImage0()
{
  vPrint("Enter image number to go to: ");
  long number = 0;
  vScan("%li", &number);
  browse_image(number);
  return 1;
}


long graphical_module::browse_image(long number)
{
  if (input == 0)
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  if ((number < 0) || (number >= buffer_size))
  {
    return 0;
  }
  long temp_index = (input->buffer_index + buffer_size - number) % buffer_size;
  input->current_image = input->image_buffer[temp_index];
  if (input->current_image == 0) 
		input->current_image = input->image_buffer[input->buffer_index];
  else 
  {
    RedrawImages();
  }

  return 1;
}


long graphical_module::SetSequenceStep()
{
  if (input == 0) return 0;
  vPrint("\nPlease enter sequence step: ");
  vScan("%li", &(input->sequence_step));

  return 1;
}


long graphical_module::SetSequenceStep(long step)
{
  if (input == 0) return 0;
  input->sequence_step = step;

  return 1;
}



long graphical_module::JumpToFrame0()
{

  if (input == 0) return 0;
  vPrint("Input frame to jump to: ");
  long frame;
  vScan("%li", &frame);
  return JumpToFrame(frame);
}

  
long graphical_module::JumpToFrame(long frame)
{
  if (input == 0) return 0;
  long success = input->SetCurrentFrame(frame);
  if (success >= 0) 
  {
    success = ShowCurrentFrame();
  }
  return success;
}


long graphical_module::histogram_equalization_on()
{
  histogram_equalization_flag = 1;
  return 1;
}


long graphical_module::histogram_equalization_off()
{
  histogram_equalization_flag = 0;
  return 1;
}


long graphical_module::histogram_equalization_size ()
{
  long window_size = 0;
  print ("\nenter window size to be used for histogram equalization:\n");
  question ("%li", &window_size);
  if (window_size <1)
  {
    print ("the window size should be at least 1\n");
    return 0;
  }

  if (window_size % 2 != 1)
  {
    print ("the window size should be odd\n");
    return 0;
  }

  equalization_window_size = window_size;
  return 1;
}


long graphical_module::histogram_equalization ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ushort_image* current = new ushort_image (input->current_image);
  grayscale_image * result = function_histogram_equalization (current, equalization_window_size);
  function_delete (current); 
  show_image (result);
  return 1;
}


long graphical_module::histogram_equalization_offset ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long vertical_offset = 0, horizontal_offset = 0;
  print ("\nenter vertical offset, horizontal offset:\n");
  question("%li %li", & vertical_offset, & horizontal_offset);

  ushort_image* current = new ushort_image (input->current_image);
  grayscale_image * result = function_histogram_equalization_offset (current, equalization_window_size, 
                                                            vertical_offset, horizontal_offset);
  function_delete (current); 
  show_image (result);
  return 1;
}


long graphical_module::histogram_equalization_weights_test ()
{
  if (input == 0)
  {
    return 0;
  }

  float_image * weights = histogram_equalization_weights(equalization_window_size);
  show_image(weights);
  return 1;
}


long graphical_module::toggle_preserve_current_frame()
{
  preserve_current_frame = 1 - preserve_current_frame;
  return 1;
}


long graphical_module::current_percentage()
{
  if (input == 0)
  {
    return 0;
  }
  if (input->sequence == 0)
  {
    return 0;
  }

  if (input->number_of_frames <= 1)
  {
    return 0;
  }

  float resultf = ((float) input->current_frame) / ((float) input->number_of_frames - 1) * 100.f;
  long result = round_number(resultf);
  return result;
}


long graphical_module::SetCurrentCounterPercent(double percentage)
{
	if (input == 0) return 0;
	double max_frame = input->number_of_frames - 1;
	long frame = round_number((max_frame * percentage) / 100.0);
	input->SetCurrentFrame(frame);

  return 0;
}


long graphical_module::match_window_size()
{
  if ((input == 0) || (input->sequence == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long rows = input->current_image->vertical();
  long cols = input->current_image->horizontal();
  rows = rows + 150;
  cols = Max(cols, (long) 700);

  window->Resize(rows, cols);
  return 1;
}


long graphical_module::AddInputSilent(const char * filename)
{
  if (filename[0] == 0) return 0;
  vVideoFile * new_video_file;
  new_video_file = vOpenReadVideoFile(filename);

  if (new_video_file == 0)
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  class_sequence_information * info = new class_sequence_information(buffer_size);
  info->sequence = new_video_file;
  info->path_name = vCopyString(new_video_file->GetFilename());
  info->simple_name = vSimpleName(info->path_name, platform_directory_slash);
  info->number_of_frames = new_video_file->NumberOfFrames();
  //info->magnified_rows = new_video_file->Rows();
  //info->magnified_cols = new_video_file->Cols();
  inputs.push_back(info);

  number_of_inputs++;

  SetCurrentInputSilent(number_of_inputs - 1);
  return 1;
}


long graphical_module::set_frame_rate()
{
  long rate = 0;
  vPrint("\nenter frame rate:\n");
  vScan("%li", &rate);
  if ((rate < 1) || (rate > 300))
  {
    vPrint("rate must be between 1 and 300\n");
  }
  else
  {
    frame_rate = rate;
  }

  return 1;
}


long graphical_module::get_frame_rate()
{
  return frame_rate;
}


general_image * graphical_module::current_image ()
{
  class_sequence_information * sequence = current_sequence ();
  if (sequence == 0)
  {
    return 0;
  }

  return sequence->current_image;
}


class_sequence_information * graphical_module::current_sequence ()
{
  return input;
}


long graphical_module::set_buffer_size()
{
  if (input == 0)
  {
    return 0;
  }

  long new_size = 0;
  print("\nenter new buffer size:\n");
  question("%li", & new_size);

  if ((new_size <= 0) || (new_size > 50))
  {
    print("acceptable sizes are between 1 and 50\n");
    return 0;
  }

  input->SetImageBufferSize(new_size);
  return 1;
}


long graphical_module::set_default_buffer_size()
{
  long new_size = 0;
  print("\nenter new default buffer size:\n");
  question("%li", & new_size);

  if ((new_size <= 0) || (new_size > 50))
  {
    print("acceptable sizes are between 1 and 50\n");
    return 0;
  }

  default_buffer_size = new_size;
  return 1;
}


long graphical_module::check_input()
{
  if (input == 0)
  {
    default_input(100,100);
  }

  if (input == 0)
  {
    exit_error("\nerror: check_input failed\n");
  }

  return 1;
}


long graphical_module::CopyInput0()
{
  if (input == 0) return 0;
  long input_index = 0;
  vPrint("\nInput index:\n");
  vScan("%li", &input_index);
  return CopyInput(input_index);
}


long graphical_module::CopyInput(long input_index)
{
  if (input == 0) return 0;
  if ((input_index < 0) || (input_index >= number_of_inputs)) return 0;
  general_image * the_image = inputs[input_index]->current_image->Copy();
  ShowImage(the_image);
  return 1;
}


//////////////////////////////////////////////////////////
//                                                      //
//   end of core functions for the graphical interface  //
//                                                      //
//////////////////////////////////////////////////////////



//////////////////////////////////////////////////////////
//                                                      //
//      start of generic image processing functions     //
//                                                      //
//////////////////////////////////////////////////////////


long graphical_module::ConnectedComponents()
{
  if (input->current_image == 0) return 0;
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  color_image * the_image = vLabelsToImage(connected, &labels);
  graphical_module::ShowImage(connected);
  graphical_module::ShowImage(the_image);

  return 1;
}


long graphical_module::BiggestComponent()
{
  if (input->current_image == 0) return 0;
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  graphical_module::ShowImage(connected);
  std::sort(labels.begin(), labels.end(), LabelMore());
  Label biggest = labels[0];
  if ((biggest.id == 0) && (labels.size() > 1))
  {
    biggest = labels[1];
  }
  v3dMatrix<uchar> * selected = vSelectValue(connected, biggest.id);
  ShowImage(selected);

  return 1;
}


long graphical_module::SelectComponent()
{
  if (input->current_image == 0) return 0;
  long component_rank = 0;
  vPrint("\nEnter component rank:\n");
  vScan("%li", &component_rank);
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  graphical_module::ShowImage(connected);

  std::sort(labels.begin(), labels.end(), LabelMore());
  long i;
  long rank = 0;
  long selected_id = -1;
  for (i = 0; i < (long) labels.size(); i++)
  {
    Label current_label = labels[i];
    if (current_label.id == 0) continue;
    if (rank == component_rank)
    {
      selected_id = current_label.id;
      break;
    }
    rank++;
  }

  if (selected_id == -1)
  {
    return 0;
  }

  v3dMatrix<uchar> * selected = vSelectValue(connected, selected_id);
  ShowImage(selected);

  return 1;
}


long graphical_module::SelectComponents()
{
  if (input->current_image == 0) return 0;
  long component_rank1 = 0, component_rank2 = 0;
  vPrint("\nEnter component rank low and high:\n");
  vScan("%li %li", &component_rank1, &component_rank2);
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  graphical_module::ShowImage(connected);
  if ((component_rank1 < 0) || (component_rank2 < 0) ||
      (component_rank1 >= (long) labels.size()) || (component_rank2 >= (long) labels.size()))
  {
    print("only %li labels found\n", labels.size());
    return 1;
  }

  long size = labels.size();
  vector<Label> copied_labels(size);
  long i;
  for (i = 0; i < size; i++)
  {
    copied_labels[i] = labels[i];
  }

  std::sort(labels.begin(), labels.end(), LabelMore());
  long lowest = labels[component_rank2].count;
  long highest = labels[component_rank1].count;

  long rows = connected->Rows();
  long cols = connected->Cols();
  grayscale_image * selected =  new grayscale_image(rows, cols);
  for (i = 0; i < rows * cols; i++)
  {
    long label = connected->Matrix()[i];
    long count = copied_labels[label].count;
    if ((lowest <= count) && (count <= highest))
    {
      selected->Matrix()[i] = 255;
    }
    else
    {
      selected->Matrix()[i] = 0;
    }
  }

  ShowImage(selected);

  return 1;
}


long graphical_module::FrameDifferences()
{
  long num_frames = input->sequence_step;
  if (!predicate_color_image(input->current_image)) return 0;
  color_image * image1 = (color_image *) input->current_image;
  graphical_module::AdvanceSequence(num_frames);
  color_image * image2 = (color_image *) input->current_image;
  v3dMatrix<double> * difference = vImageDifference(image1, image2);
  ShowImage(difference);

  return 1;
}



long graphical_module::SetPixel()
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  vPrint("Enter row, col and value: ");
  long row = 0;
  long col = 0;
  long value = 0;
  vScan("%li %li %li", &row, &col, &value);
  uchar uchar_value = (uchar) (value % 256);
  return SetPixel(row, col, uchar_value);
}


long graphical_module::SetPixel(long row, long col, uchar value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  if (current->TypeName() != UcharType) return 0;
  v3dMatrix<uchar> * the_image = (v3dMatrix<uchar> *) current;
  long bands = the_image->Bands();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (row < 0) return 0;
  if (col < 0) return 0;
  if (row >= rows) return 0;
  if (col >= cols) return 0;
  
  vArray3(uchar) matrix = the_image->Matrix3();
  long i;
  for (i = 0; i < bands; i++)
  {
    matrix[i][row][col] = value;
  }
  DrawCurrentImage();
  return 1;
}


v3dMatrix<double> * graphical_module::ComputeMotion()
{
  if (input->current_image == 0) return 0;
  long num_frames = input->sequence_step;
  general_image * the_image1 = input->current_image;

  input->SetCurrentFrame(input->CurrentFrame() - num_frames);
  general_image * the_image0 = ReadCurrentImage();
  if (the_image0 == 0)
  {
    input->SetCurrentFrame(input->CurrentFrame() + num_frames);
    return 0;
  }

  input->SetCurrentFrame(input->CurrentFrame() + 2 * num_frames);
  general_image * the_image2 = ReadCurrentImage();
  v3dMatrix<double> * difference = 0;
  if (the_image2 != 0)
  {
    difference = vComputeMotion0(the_image0, the_image1, the_image2);
    ShowImage(difference);
  }
  input->SetCurrentFrame(input->CurrentFrame()- num_frames);

  vdelete(the_image0);
  vdelete(the_image2);
  return difference;
}


long graphical_module::ThresholdCurrentImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  
  double threshold = 0;
  vPrint("Threshold: ");
  vScan("%lf", &threshold);

  grayscale_image * threshold_image;
  threshold_image = (grayscale_image*) input->current_image->threshold(threshold);
  graphical_module::ShowImage(threshold_image);
  vector<long> rows, cols;
  vGetNonZeroPoints(threshold_image, &rows, &cols);
  long number = rows.size();
  vPrint("number = %li, percentage = %f%%\n", number, 
         ((float) number) / ((float) threshold_image->Size()) * 100.0f);

  return 1;
}


long graphical_module::ThresholdCurrentImage(double threshold)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  
  v3dMatrix<uchar> * threshold_image;
  threshold_image = input->current_image->threshold(threshold);
  graphical_module::ShowImage(threshold_image);

  return 1;
}


long graphical_module::InverseThresholdImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  double threshold = 0;
  vPrint("Threshold: ");
  vScan("%lf", &threshold);

  v3dMatrix<uchar> * threshold_image;
  threshold_image = input->current_image->threshold(threshold);
  vInverseImage(threshold_image);
  graphical_module::ShowImage(threshold_image);

  return 1;
}


 
 
long graphical_module::UndirectedEdges()
{
  if (input->current_image == 0) return 0;
  general_image * image1 = input->current_image;
  double_image * image2 = new double_image(image1);
  double_image * result_image =  vUndirectedEdges(image2);
  graphical_module::ShowImage(result_image);

  vdelete(image2);

  return 1;
}


long graphical_module::AmplifyImage()
{
  if (input->current_image == 0) return 0;
  general_image * image1 = input->current_image;
  color_image * image2 = input->current_image->ToRGB();
  color_image * result_image = (color_image *) vAmplifyImage(image2);
  graphical_module::ShowImage(result_image);

  if (image2 != image1) vdelete(image2);

  return 1;
}
 

long graphical_module::MultiplyImage()
{
  if (input->current_image == 0) return 0;

  double number = 0;
  vPrint("Multiply image by: ");
  vScan("%lf", &number);

  general_image * image1 = input->current_image;
  double_image image2(input->current_image);
  double_image * result_image = vMultiplyImage(&image2, number);
  graphical_module::ShowImage(result_image);

  return 1;
}
 

long graphical_module::SaveCurrentImage()
{
	if (input->current_image == 0) return 0;
  char filename[1000];
  print("\nSave current image in (q to abort):\n");
  question("%s", filename);
  if (strcmp(filename, "q") == 0) return 0;
  vAdjustEnding3(filename, filename, ".ppm");

  general_image * image1 = input->current_image;
  color_image * image2 = input->current_image->ToRGB();
  image2->save(filename);
  if (image1 != image2) vdelete(image2);
  cout << "Image saved" << endl;
  return 0;
}


long graphical_module::ApplyProcedure(vPixelPred proc)
{
  if(input->current_image == 0) return 0;
  general_image * image1 = input->current_image;
  color_image * image2 = input->current_image->ToRGB();
  color_image * result_image = vProcessPixels(image2, proc);
  ShowImage(result_image);
  
  if (image1 != image2) vdelete(image2);

  return 1;
}


long graphical_module::HeightAdjustment()
{
  if (input->current_image == 0) return 0;
  if (!predicate_color_image(input->current_image)) return 0;
  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  color_image * adjusted = new color_image(rows, cols);
  color_image * image1 = (color_image *) input->current_image;

  graphical_module::AdvanceSequence(1);
  color_image * current = image1;

  vArray2(uchar) R1 = current->GetR2();
  vArray2(uchar) G1 = current->GetG2();
  vArray2(uchar) B1 = current->GetB2();
  vArray2(uchar) R2 = adjusted->GetR2();
  vArray2(uchar) G2 = adjusted->GetG2();
  vArray2(uchar) B2 = adjusted->GetB2();

  long row, col;
  long height_adjustment;
  print("\nenter adjustment:\n");
  question("%li", &height_adjustment);
  for (row = height_adjustment; row < rows; row++)
  {
    long row2 = (rows + row - height_adjustment) % rows;
    for (col = 0; col < cols; col++)
    {
      R2[row2][col] = R1[row][col];
      G2[row2][col] = G1[row][col];
      B2[row2][col] = B1[row][col];
    }    
  }

  current = (color_image *) input->current_image;
  R1 = current->GetR2();
  G1 = current->GetG2();
  B1 = current->GetB2();

  for (row = 0; row < height_adjustment; row++)
  {
    long row2 = (rows + row - height_adjustment) % rows;
    for (col = 0; col < cols; col++)
    {
      R2[row2][col] = R1[row][col];
      G2[row2][col] = G1[row][col];
      B2[row2][col] = B1[row][col];
    }    
  }

  ShowImage(adjusted);

  return 1;
}


long graphical_module::Trash()
{
  //float first = 10.0;
  //float second = first/0.0f;
  //float third = -first/0.0f;
  //if (second > 100000.0f)
  //{
  //  print("1\n");
  //}
  //if (third < -100000.0f)
  //{
  //  print("2\n");
  //}
  //if (second < 100000.0f)
  //{
  //  print("3\n");
  //}
  //if (third >remote -100000.0f)
  //{
  //  print("4\n");
  //}

  return 1;
}


long graphical_module::Trash2()
{
  color_image * padded;
  grayscale_image * thresholded;
  grayscale_image * normalized, * cropped;

  if (!predicate_color_image(input->current_image)) return 0;
  color_image * current = (color_image *) input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  padded = (color_image *) vPad(current, 256, 256);
  ShowImage(padded);

  thresholded = (grayscale_image *) padded->threshold(1);
  ShowImage(thresholded);
  normalized = (grayscale_image *) vNormalizeImage(thresholded, 96.0, 127.5, 127.5);
  ShowImage(normalized);
  cropped = (grayscale_image *) vCopyRectangle(normalized, 0, 255, 0, 255);
  ShowImage(cropped);

  return 1;
}


long graphical_module::Trash3()
{


  return 1;
}


long graphical_module::LoadImage()
{
  char filename[256];
  print("\nLoad image file:\n");
  question("%s", filename);
  general_image * the_image = general_image::Read(filename);
  if (the_image == 0) 
  {
    vPrint("Loading image from %s failed\n", filename);
    return 0;
  }
  
  ShowImage(the_image);

  return 1;
}


long graphical_module::SaveImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  general_image * image = input->current_image;
  char filename[200];
  print("\nOutput file name:\n");
  question("%s", filename);

  long success = 0;
  if (predicate_color_image(image))
  {
    color_image * image2 = (color_image *) image;
    success = image2->save(filename);
  }
  else if (predicate_gray_image(image))
  {
    grayscale_image * image2 = (grayscale_image *) image;
    success = image2->save(filename);
  }
  else
  {
    success = image->store(filename);
  }

  if (success <= 0)
  {
    print("failed to save image to %s\n", filename);
  }
  else
  {
    print("saved image successfully to %s\n", filename);
  }

  return 1;
}


long graphical_module::SavePGM()
{
  if (input->current_image == 0) return 0;
  general_image * image1 = input->current_image;
  grayscale_image * image2 = input->current_image->ToGray();

  char filename[200];
  print("\nOutput file name:\n");
  question("%s", filename);
  image2->save(filename);
  if(image1 != image2) vdelete(image2);

  return 1;
}


long graphical_module::Blur()
{
  if (input == 0) return 0;
	if (input->current_image == 0) return 0;

  general_image * image1 = input->current_image;
  v3dMatrix<double> * image2 = new v3dMatrix<double> (image1);
  v3dMatrix<double> * blurred = vBlur(image2);
  double average = function_image_average (blurred);
//  enter_border_value (blurred, average);
  ShowImage(blurred);

  if (image1 != image2) vdelete(image2);

  return 1;
}


long graphical_module::AverageRegions()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  long window_size;
  print("\nPreparing to average regions. Input window size:\n");
  question ("%li", &window_size);
  general_image * average, * resampled;
  average = input->current_image->AverageRegions(window_size, window_size);
  resampled = average->Resample(input->current_image->Rows(),
				input->current_image->Cols());
  ShowImage(resampled);
  vdelete(average);

  return 1;
}


long graphical_module::WritePPM()
{
  if (input == 0) return 0;
  char buffer[1000];
  print ("\nSpecify filename to save image to:\n");
  question ("%s", buffer);
  ushort success = input->current_image->Write(buffer);
  if (success == 0)
  {
    vPrint("Error in saving image.");
  }
  else 
  {
    vPrint("Saved image successfully");
  }

  return 1;
}


long graphical_module::ProcessRegions()
{
  if (input == 0) return 0;
  color_image * the_image;
  long threshold;
  if (input->current_image == 0) return 0;
  print ("\nThreshold:\n");
  question ("%li", &threshold);
  
  // Threshold the current image
  v3dMatrix<uchar> * threshold_image;
  threshold_image = input->current_image->threshold(threshold);

  vector<Label> labels1, labels2;
  v3dMatrix<long> * connected_background, * connected_regions;
  
  // Get the connected components of the thresholded image
  // connected_background is the labeled image and labels
  // contains Label objects with information about each label
  // used in connected_background
  connected_background = threshold_image->ConnectedComponents(&labels1);

  // Create an RGB image that colors each component with a distinct color
  the_image = vLabelsToImage(connected_background, &labels1);
  ShowImage(the_image);

  // Invert threshold_image
  vInverseImage(threshold_image);
  // Get connected components of threshold_image and their labels
  connected_regions = threshold_image->ConnectedComponents(&labels2);
  // Create an easy-to-look-at color image of the connected components
  // and displayit
  the_image = vLabelsToImage(connected_regions, &labels2);
  ShowImage(the_image);

  v3dMatrix<long> * connected_no_holes = vCopyImage(connected_regions);
  // Fill up the holes in the connected image, and display the result.
  vFillUpHoles(connected_no_holes, connected_background);
  the_image = vLabelsToImage(connected_no_holes, &labels2);
  vdelete(threshold_image);
  vdelete(connected_no_holes);

  ShowImage(connected_regions);
  ShowImage(connected_background);
  ShowImage(the_image);

  return 1;
}


long graphical_module::NormalizeImage(double distance, double center_row, 
                                      double center_col)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * the_image = current->ToGray();
  grayscale_image * normalized = (grayscale_image *) vNormalizeImage(the_image, distance, 
                                                              center_row, center_col);
  ShowImage(normalized);
  if (the_image != current) vdelete(the_image); 
  return 1;
}

long graphical_module::RotateLeft0()
{
  if (input == 0) return 0;
  long turns = 0;
  vPrint("Input left turns: ");
  vScan("%li", &turns);
  return RotateLeft(turns);
}


// Rotates the current image by 90 * right_turns degrees
long graphical_module::RotateLeft(long turns)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * rgb = current->ToRGB();
  color_image * rotated = (color_image *) vRotateRight(rgb, 360 - turns * 90);
  ShowImage(rotated);
  if (rgb != current) vdelete(rgb);
  return 0;
}


long graphical_module::RotateRight0()
{
  if (input == 0) return 0;
  long right_turns = 0;
  vPrint("Input right turns: ");
  vScan("%li", &right_turns);
  return RotateRight(right_turns);
}


// Rotates the current image by 90 * right_turns degrees
long graphical_module::RotateRight(long right_turns)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * rgb = current->ToRGB();
  color_image * rotated = (color_image *) vRotateRight(rgb, right_turns * 90);
  ShowImage(rotated);
  if (rgb != current) vdelete(rgb);
  return 0;
}


long graphical_module::NewImage()
{
  if (input == 0) return 0;
  long rows = 0;
  long cols = 0;
  vPrint("Input rows and cols: ");
  vScan("%li %li", &rows, &cols);
  vPrint("\n");
  return NewImage(rows, cols);
}


long graphical_module::NewImage(long rows, long cols)
{
  if (input == 0) return 0;
  color_image * the_image = new color_image(rows, cols);
  vWriteValue(the_image, (uchar) 0);
  ShowImage(the_image);
  return 1;
}


long graphical_module::DrawLine()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawLine(row1, col1, row2, col2, value);
}


long graphical_module::DrawLine(long row1, long col1, long row2, long col2, long value)
{
  if (input == 0) return 0;
  color_image * current = (color_image *) input->current_image->ToRGBBoundCopy();
  ushort result = vDrawLine1(current, row1, col1, row2, col2, (uchar) value);
  ShowImage(current);
  return result;
}


long graphical_module::DrawLineB()
{
  long row, col, length, value;
  double angle;
  row = col = length = value = 0;
  angle = 0.0;
  vPrint("Input row, col, angle, length, value:\n");
  vScan("%li %li %lf %li %li", &row, &col, &angle, &length, &value);
  return DrawLine2(row, col, angle, length, value);
}


long graphical_module::DrawLine2(long row, long col, double angle, long length, long value)
{
  if (input == 0) return 0;
  color_image * current = (color_image *) input->current_image->ToRGBCopy();
  ushort result = vDrawCenteredLine(current, row, col, angle, length, (uchar) value);
  ShowImage(current);
  return result;
}


long graphical_module::DrawRectangle()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawRectangle(row1, col1, row2, col2, value);
}


long graphical_module::DrawRectangle(long row1, long col1, 
                                    long row2, long col2, long value)
{
  if (input == 0) return 0;

  color_image * current = input->current_image->ToRGBBoundCopy();
  ushort result = vDrawRectangle1(current, row1, col1, row2, col2, (uchar) value);
  show_image(current);
  return result;
}


long graphical_module::DrawCircle()
{
  long row1, col1, radius, value;
  row1 = col1 = radius = value = 0;
  vPrint("Input row, col, radius, value");
  vScan("%li %li %li %li", &row1, &col1, &radius, &value);
  vPrint("\n");
  return DrawCircle(row1, col1, radius, value);
}


long graphical_module::DrawCircle(long row, long col, long radius, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawCircle1(current, row, col, radius, (uchar) value);
  DrawCurrentImage();
  return result;
}


long graphical_module::DrawFilledRectangle()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawFilledRectangle(row1, col1, row2, col2, value);
}


long graphical_module::DrawFilledRectangle(long row1, long col1, 
                                    long row2, long col2, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawFilledRectangle1(current, row1, col1, 
                                         row2, col2, (uchar) value);
  DrawCurrentImage();
  return result;
}


long graphical_module::DrawFilledCircle()
{
  long row1, col1, radius, value;
  row1 = col1 = radius = value = 0;
  vPrint("Input row, col, radius, value");
  vScan("%li %li %li %li", &row1, &col1, &radius, &value);
  vPrint("\n");
  return DrawFilledCircle(row1, col1, radius, value);
}


long graphical_module::DrawFilledCircle(long row, long col, long radius, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawFilledCircle1(current, row, col, radius, (uchar) value);
  DrawCurrentImage();
  return result;
}


long graphical_module::NormalizeLabeledImage(double distance, double center_row, 
                                            double center_col)
{
  if (input == 0) return 0;   
  general_image * current = input->current_image;
  color_image * the_image = current->ToRGB();
  color_image * normalized = (color_image *) vNormalizeLabeledImage3(the_image, distance, 
                                                              center_row, center_col);
  ShowImage(normalized);
  if (the_image != current) vdelete(the_image); 
  return 1;
}


long graphical_module::SelectValue0()
{
  if (input == 0) return 0;
  int value;
  vPrint("Input value to select: ");
  vScan("%i", &value);
  return SelectValue(value);
}


long graphical_module::SelectValue(long value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  v3dMatrix <long> * image = new v3dMatrix <long>(current);
  grayscale_image * result;
  result = (grayscale_image *) vSelectValue(image, value);
  ShowImage(result);
  if (image != current) vdelete(image);
  return 1;
}


long graphical_module::MaxNeighbors()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * max_neighbors = vMaxNeighbors0(current);
  ShowImage(max_neighbors);
  return 1;
}


long graphical_module::Histogram0()
{
  if (input == 0) return 0;

  long number_of_bins = 0;
  vPrint("Enter number of bins for histogram: ");
  vScan("%li", &number_of_bins);
  return Histogram(number_of_bins);
}


long graphical_module::Histogram(long number_of_bins)
{
  if (input == 0) return 0;
  vMatrix<double> * histogram = vPixelHistogram0(input->current_image, 
                                                   number_of_bins);
  if (histogram == 0) return 0;
  histogram->Print();
  vdelete(histogram);
  return 1;
}


long graphical_module::Histogram30()
{
  if (input == 0) return 0;

  long number_of_bins = 0;
  double lower = 0;
  double upper = 0;
  vPrint("Enter number of bins, lower, upper: ");
  vScan("%li %lf %lf", &number_of_bins, &lower, &upper);
  return Histogram3(number_of_bins, lower, upper);
}


long graphical_module::Histogram3(long number_of_bins, double lower, double upper)
{
  if (input == 0) return 0;
  vMatrix<double> * histogram = vBoundedRankHistogram0(input->current_image, 
                                                         number_of_bins, lower, upper);
  if (histogram == 0) return 0;
  histogram->Print();
  vdelete(histogram);
  return 1;
}


long graphical_module::BoundingBox0()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  vBoundingBox result = vBinaryBoundingBox(current);
  vPrint("\ncenter = (%.1lf, %.1lf), top = %li, bottom = %li, left = %li, right = %li, height = %li, width = %li\n",
          (double) (result.top + result.bottom + 0.0) / 2.0,
          (double) (result.left + result.right + 0.0) / 2.0,
          result.top, result.bottom, result.left, result.right, 
          result.bottom - result.top + 1, result.right - result.left + 1);
  return 1;
}

  
long graphical_module::Translate0()
{
  if (input == 0) return 0;
  vPrint("Enter rows and cols to translate by: ");
  long rows, cols;
  rows = cols = 0;
  vScan("%li %li\n", &rows, &cols);
  return Translate(rows, cols);
}


long graphical_module::Translate(long row_diff, long col_diff)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = function_translate_image0(current, row_diff, col_diff);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long graphical_module::ScaleImage0()
{
  if (input == 0) return 0;
  vPrint("Input scaling factor for binary image: ");
  double scaling = 0;
  vScan("%lf", &scaling);
  return ScaleImage(scaling);
}


long graphical_module::ScaleImage(double scaling)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = vScaleImage0(current, scaling);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long graphical_module::ResizeImage0()
{
  if (input == 0) return 0;
  vPrint("Input new rows and cols: ");
  long rows = 0, cols = 0;
  vScan("%li %li", &rows, &cols);
  return ResizeImage(rows, cols);
}


long graphical_module::ResizeImage(long rows, long cols)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = vResizeImage0(current, rows, cols);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long graphical_module::ScaleBoundingBox0()
{
  if (input == 0) return 0;
  vPrint("\nInput new bounding box size for binary image:\n");
  long new_size = 0;
  vScan("%li", &new_size);
  return ScaleBoundingBox(new_size);
}


long graphical_module::ScaleBoundingBox0b()
{
  if (input == 0) return 0;
  vPrint("\nInput new bounding box size, image rows, image cols:\n");
  long new_size = 0;
  long rows = 0;
  long cols = 0;
  vScan("%li %li %li", &new_size, &rows, &cols);
  return ScaleBoundingBox2(new_size, rows, cols);
}


long graphical_module::ScaleBoundingBox(long new_box_size)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * gray = current->ToGray();
  long center = new_box_size / 2;
  long rows = new_box_size;
  long cols = new_box_size;
  general_image * result = vScaleBoundingBox6(gray, new_box_size, center, center,
                                             rows, cols);
  if (gray != current) vdelete(gray);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}



long graphical_module::ScaleBoundingBox2(long new_box_size, long new_rows, long new_cols)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  double_image * the_image = new double_image (current);
  long center_row = (new_rows - 1) / 2;
  long center_col = (new_cols - 1) / 2;
  general_image * result = vScaleBoundingBox6(the_image, new_box_size, center_row, center_col,
                                             new_rows, new_cols);
  if (the_image != current) vdelete(the_image);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long graphical_module::CenterOfMass()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * the_image = current->ToGray();
  double center_row = 0;
  double center_col = 0;

  vCenterOfMass(the_image, center_row, center_col);
  double max_distance = vMaxDistanceFromPoint(the_image, center_row, center_col);
  vPrint("center of mass = (%lf, %lf), max_distance = %lf\n", 
          center_row, center_col, max_distance);
  if (the_image != current) vdelete(the_image);
  return 1;
}


long graphical_module::TopBottom()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  general_image * top_bottom = wrim(the_image).TopBottom();

  ShowImage(top_bottom);
  return 1;
}


long graphical_module::Pad()
{
  if (input == 0) return 0;
  long margin = 0;
  vPrint("\nEnter margin:\n");
  vScan("%li", &margin);

  general_image * current = input->current_image;
  general_image * padded = wrim(current).Pad4(margin, margin, margin, margin);
  ShowImage(padded);
  return 0;
}


long graphical_module::Pad2()
{
  if (input == 0) return 0;
  long new_rows = 0, new_cols = 0;
  vPrint("\nEnter new rows, new cols:\n");
  vScan("%li %li", &new_rows, &new_cols);

  float_image current(input->current_image);
  float_image * padded = vPad(&current, new_rows, new_cols);
  ShowImage(padded);
  return 0;
}


long graphical_module::make_short()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  short_image* image = new short_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::make_long()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long_image* image = new long_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::make_ushort()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ushort_image* image = new ushort_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::make_ulong()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ulong_image* image = new ulong_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::make_float()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  float_image* image = new float_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::make_double()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image* image = new double_image (input->current_image);
  show_image (image);
  return 1;
}


long graphical_module::ImageMax0()
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  double max = function_image_maximum(matrix);
  vPrint("\nmax = %lf\n", max);
  vdelete(matrix);
  return 1;
}


long graphical_module::ImageMin0()
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  double min = function_image_minimum(matrix);
  vPrint("\nmin = %lf\n", min);
  vdelete(matrix);
  return 1;
}


long graphical_module::BandMin0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return BandMin(band);
}


long graphical_module::BandMin(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  double min = function_image_minimum(matrix);
  vPrint("\nmin = %lf\n", min);
  vdelete(matrix);
  vdelete(band_matrix);
  return 1;
}



long graphical_module::BandMax0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return BandMax(band);
}


long graphical_module::BandMax(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  double max = function_image_maximum(matrix);
  vPrint("\nmax = %lf\n", max);
  vdelete(matrix);
  vdelete(band_matrix);
  return 1;
}

  
long graphical_module::CopyBand0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return CopyBand(band);
}



long graphical_module::CopyBand(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  ShowImage(band_matrix);
  vdelete(matrix);
  return 1;
}

  
long graphical_module::CountValue0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountValue(value);
}


long graphical_module::CountValue(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountValue(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long graphical_module::CountLequal0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountLequal(value);
}


long graphical_module::CountLequal(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountLequal(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long graphical_module::CountGequal0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountGequal(value);
}


long graphical_module::CountGequal(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountGequal(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long graphical_module::FillRegion0()
{
  if (input == 0) return 0;
  vPrint("Input row, col, value for region: ");
  long row, col, value;
  row = col = value = 0;
  vScan("%li %li %li", &row, &col, &value);
  return FillRegion(row, col, (char) value);
}


long graphical_module::FillRegion(long row, long col, uchar value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * temp_image = current->ToGray();
  grayscale_image * the_image;
  if (temp_image == current)
  {
    the_image = (grayscale_image *) vCopyImage(temp_image);
  }
  else
  {
    the_image = temp_image;
  }
  long result = vFillRegion(the_image, row, col, value);
  ShowImage(the_image);
  return result;
}


long graphical_module::RandomFrame()
{
  if (input == 0) return 0;
  long number_of_frames = input->number_of_frames;
  if (number_of_frames <= 0) return 0;
  long random_frame = vRandomLong2(0, number_of_frames - 1);
  function_print("\nrandom frame = %li\n", random_frame);
  return JumpToFrame(random_frame);
}


long graphical_module::to_color()
{
  if (input == 0) return 0;
  color_image * image = input->current_image->ToRGBCopy();
  show_image(image);
  return 1;
}


long graphical_module::to_color_bound()
{
  if (input == 0) return 0;
  color_image * image = input->current_image->ToRGBBoundCopy();
  show_image(image);
  return 1;
}


long graphical_module::ToGray()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  ShowImage(gray);
  return 1;
}


long graphical_module::ToGrayBound()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayBoundCopy();
  ShowImage(gray);
  return 1;
}


long graphical_module::ConnectedComponents8()
{
  if (input == 0) return 0;
  grayscale_image * current = input->current_image->ToGrayCopy();
  vector<Label> label_objects;
  long_matrix connected = 
    vConnectedComponents8(current, &label_objects);
  vdelete(current);
  ShowImage(connected.copy());
  color_image * the_image = vLabelsToImage(&connected, &label_objects);
  ShowImage(the_image);
  return 1;
}


long graphical_module::EdgeOrientations()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations(gray);
  ShowImage(orientations);
  color_image * color_orientations = vCircularIntensity(orientations, 0, M_PI, -1);
  ShowImage(color_orientations);
  vdelete(gray);
  return 1;
}


long graphical_module::EdgeOrientations2()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations2(gray);
  ShowImage(orientations);
  color_image * color_orientations = vCircularIntensity(orientations, 0, 2*M_PI, -1);
  ShowImage(color_orientations);
  vdelete(gray);
  return 1;
}


long graphical_module::NonMaximaSuppression()
{
  if (input == 0) return 0;
  v3dMatrix<double> * non_maxima = vNonMaximaSuppression(input->current_image);
  grayscale_image * binary = (grayscale_image *) non_maxima->threshold(1);
  vdelete(non_maxima);
  v3dMatrix<double> * gradients = vGradientNorm0(input->current_image);
  ShowImage(binary);
  v3dMatrix<double> * selected = vSelectPixels(gradients, binary);
  ShowImage(gradients);
  ShowImage(selected);

  return 1;
}

  
long graphical_module::IntensityMaxima()
{
  if (input == 0) return 0;
  v3dMatrix<double> * maxima = vIntensityMaxima(input->current_image);
  v3dMatrix<double> * double_image1 = new v3dMatrix<double>(input->current_image);

  grayscale_image * binary = (grayscale_image *) maxima->threshold(1);
  ShowImage(binary);

  v3dMatrix<double> * selected = vSelectPixels(double_image1, binary);
  ShowImage(selected);

  float_image m2(double_image1);
  float_matrix m3(&m2, 0);
  float_matrix m4 = function_intensity_maxima(m3);
  show_image(m4.copy());

  vdelete(maxima);
  vdelete(double_image1);

  return 1;
}

  
long graphical_module::GradientNorms()
{
  if (input == 0) return 0;
  v3dMatrix<double> * gradient_norms = vGradientNorm0(input->current_image);
  ShowImage(gradient_norms);
  return 1;
}


long graphical_module::Overlay0()
{
  if (input == 0) return 0;
  long input1, input2;
  input1 = input2 = 0;
  vPrint("\nEnter input1 and input2:\n");
  vScan("%li %li", &input1, &input2);
  return Overlay(input1, input2);
}


long graphical_module::Overlay(long input1, long input2)
{
  if (input == 0) return 0;
  if ((input1 < 0) || (input1 >= number_of_inputs) ||
      (input2 < 0) || (input2 >= number_of_inputs))
  {
    return 0;
  }
  general_image * image1 = inputs[input1]->current_image;
  general_image * image2 = inputs[input2]->current_image;
  color_image * overlay = vOverlay0(image1, image2);
  ShowImage(overlay);
  return 1;
}


long graphical_module::Randomize0()
{
  srand(time(0));
  return 1;
}


long graphical_module::Randomize1()
{
  vPrint("\nEnter seed:\n");
  long seed = 0;
  vScan("%li", &seed);
  srand(seed);
  return 1;
}


long graphical_module::DilateImage()
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGray();
  grayscale_image * dilated = vDilateImage(the_image, 1, 255);
  if (the_image != input->current_image) vdelete(the_image);
  ShowImage(dilated);
  return 1;
}


long graphical_module::SelectRange0()
{
  if (input == 0) return 0;
  long band;
  double low, high;
  band = 0;
  low = high = (double) 0;
  vPrint("\nEnter band, low and high\n");
  vScan("%li %lf %lf", &band, &low, &high);
  return SelectRange(band, low, high);
}


long graphical_module::SelectRange(long band, double low, double high)
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  general_image * selected = vSelectRange0(the_image, band, low, high);
  ShowImage(selected);
  return 1;
}



long graphical_module::CircularDistance0()
{
  double x, y, period;
  x = y = period = 0;
  vPrint("\nEnter x, y, period\n");
  vScan("%lf %lf %lf", &x, &y, &period);
  double distance = vCircularDistance(x, y, period);
  vPrint("distance = %lf\n", distance);
  return 1;
}



long graphical_module::CountNonZero()
{
  if (input == 0) return 0;
  grayscale_image * thresholded = (grayscale_image *) input->current_image->threshold(0.000000000000001);
  vector<long> rows, cols;
  vGetNonZeroPoints(thresholded, &rows, &cols);
  long number = rows.size();
  vPrint("number = %li, percentage = %f%%\n", number, ((float) number) / ((float) thresholded->Size()) * 100.0f);
  vdelete(thresholded);
  return 1;
}


long graphical_module::TestLinePoints0()
{
  if (input == 0) return 0;
  long row, col, length;
  double angle;
  vPrint("\nEnter row, col, angle, length:\n");
  vScan("%li %li %lf %li", &row, &col, &angle, &length);
  return TestLinePoints(row, col, angle, length);
}


long graphical_module::TestLinePoints(long row, long col, double angle, long length)
{
  if (input == 0) return 0;
  if ((angle < 0) || (angle >= 2 * M_PI)) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vector<long> rows, cols;
  vLinePoints(row, col, angle, length, &rows, &cols);
  vWriteValue2(the_image, &rows, &cols, (uchar) 255);
  ShowImage(the_image);
  return 1;
}

// SelectPixels uses the current image as a binary mask, and 
// then applies this mask to the image in the ring buffer that
// is at the selected index, so that the product of the mask
// and the buffer image is displayed.
long graphical_module::SelectPixels0()
{
  if (input == 0) return 0;
  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return SelectPixels(index);
}


long graphical_module::SelectPixels(long index)
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  browse_image(index);
  if (input->current_image == 0) return 0;
  v3dMatrix<double> * the_image = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * selected = vSelectPixels(the_image, binary);
  ShowImage(selected);
  vdelete(binary);
  vdelete(the_image);
  return 1;
}


// SelectBBox uses the current image as a binary mask, finds the
// bounding box of the non-zero area, and 
// then selects the corresponding pixels (that belong to that box)
// from the image in the ring buffer that
// is at the selected index.
long graphical_module::SelectBoundingBox0()
{
  if (input == 0) return 0;
  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return SelectBoundingBox(index);
}


long graphical_module::SelectBoundingBox(long index)
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vBoundingBox box = vBinaryBoundingBox(binary);
  browse_image(index);
  v3dMatrix<double> * the_image = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * selected = vCopyRectangle(the_image, box.left, box.right,
                                                   box.top, box.bottom);
  ShowImage(selected);
  vdelete(binary);
  vdelete(the_image);
  return 1;
}




long graphical_module::FillUpHoles()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vFillUpHoles2(binary, binary);
  ShowImage(binary);
  return 1;
}


long graphical_module::RotateImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0)
  {
    return 0;
  }
  long center_row, center_col;
  double degrees;
  center_row = center_col = 0;
  degrees = (double) 0;
  vPrint("\nEnter center_row, center_col, degrees: ");
  vScan("%li %li %lf", &center_row, &center_col, &degrees);
  if ((center_row < 0) || (center_col < 0))
  {
    center_row = input->current_image->Rows() / 2;
    center_col = input->current_image->Cols() / 2;
  }

  double_image * the_image = new double_image (input->current_image);
  v3dMatrix<float> * sources = vRotationTemplatef(the_image->Rows(), the_image->Cols(),
                                                    center_row, center_col, degrees);
  ShowImage(sources);
  v3dMatrix<double> * rotated = rotate_image_exact_degrees(the_image, center_row, center_col, degrees);
  vdelete(the_image);
  ShowImage(rotated);
  return 1;
}


long graphical_module::Binary()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;

//  vImageWrap wrap(the_image);
//  grayscale_image * binary = wrap.Binary();
  grayscale_image * binary = wrim(the_image).Binary();
  ShowImage(binary);
  return 1;
}


long graphical_module::WaitSeconds0()
{
  long seconds = 0;
  vPrint("\nEnter seconds:\n");
  vScan("%li", &seconds);
  return WaitSeconds(seconds);
}


long graphical_module::WaitSeconds(long seconds)
{
  long argument = seconds * 1000;
  Sleep(argument);
  return 1;
}
long graphical_module::DoubleSize()
{
  if (input == 0) return 0;
  v3dMatrix<double> * current = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * double_size = vDoubleSize(current);
  ShowImage(double_size);
  vdelete(current);
  return 1;
}


long graphical_module::SequenceDoubleSize()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vVideoFile * sequence = input->sequence;
  long rows = sequence->Rows();
  long cols = sequence->Cols();
  long frames = sequence->NumberOfFrames();
  const char * filename = sequence->GetFilename();
  char * output_name = vReplaceEnding2(filename, "_double.avi");

  vAVI_Stream * output = new vAVI_Stream(output_name, 2*rows, 2*cols, 3);
  if (output->IsValid() <= 0)
  {
    vPrint("Failed to write to %s\n", output_name);
    vdelete2(output_name);
    vdelete(output);
    return 0;
  }

  long i;
  vPrint("\n");
  for (i = 0; i < frames; i++)
  {
    color_image * the_image = sequence->ReadFrame(i);
    if (the_image == 0) break;
    v3dMatrix<double> * image2 = new v3dMatrix<double>(the_image);
    v3dMatrix<double> * double_image2 = vDoubleSize(image2);
    color_image * double_image = double_image2->ToRGBCopy();
    output->write_frame(double_image);
    vPrint("Done with frame %li of %li\r", i, frames);
    vdelete(image2);
    vdelete(the_image);
    vdelete(double_image2);
    vdelete(double_image);
  }

  vPrint("\n");
  vdelete(output);
  return 1;
}


long graphical_module::ReplaceColor0()
{
  if (input == 0) return 0;
  long r1, g1, b1, r2, g2, b2;
  r1 = g1 = b1 = r2 = g2 = b2 = 0;
  vPrint("\nEnter r1, g1, b1, r2, g2, b2:\n");
  vScan("%li %li %li %li %li %li", &r1, &g1, &b1, &r2, &g2, &b2);
  return ReplaceColor((uchar) r1, (uchar) g1, (uchar) b1, 
                      (uchar) r2, (uchar) g2, (uchar) b2);
}


long graphical_module::ReplaceColor(uchar r1, uchar g1, uchar b1,
                                  uchar r2, uchar g2, uchar b2)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();

  long size = the_image->Size();

  long i;
  for (i = 0; i < size; i++)
  {
    if ((R[i] == r1) && (G[i] == g1) && (B[i] == b1))
    {
      R[i] = r2;
      G[i] = g2;
      B[i] = b2;
    }
  }
  ShowImage(the_image);
  return 1;
}


long graphical_module::NormalizeIntensity()
{
  if (input == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  color_image * normalized = vNormalizeIntensity(current);
  ShowImage(normalized);
  vdelete(current);
  return 1;
}


long graphical_module::WriteRange0()
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  
  long low, high;
  low = high = 0;
  vPrint("\nEnter start, end:\n");
  vScan("%li %li", &low, &high);
  return WriteRange(low, high);
}


long graphical_module::WriteRange(long low, long high)
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vVideoFile * in_sequence = input->sequence;
  long color_bytes = Min(in_sequence->BytesPerPixel(), output->BytesPerPixel());

  long frame;
  vPrint("\n");
  for (frame = low; frame <= high; frame++)
  {
    long success = 0;
    if (strcmp(output->Type(), "class_image_stream") == 0)
    {
      general_image * current = in_sequence->ReadImage(frame);
      if (current == 0)
      {
        break;
      }
      class_image_stream* output_stream = (class_image_stream*) output;
      success = output_stream->save_image(current);
      vdelete(current);
    }
    else if (color_bytes == 1)
    {
      grayscale_image * the_image = in_sequence->ReadFrameGray(frame);
      if (the_image == 0) break;
      success = output->write_frameGray(the_image);
      vdelete(the_image);
    }
    else if (color_bytes == 3)
    {
      color_image * the_image = in_sequence->ReadFrame(frame);
      if (the_image == 0) break;
      success = output->write_frame(the_image);
      vdelete(the_image);
    }
    if (success > 0)
    {
      vPrint("Successfully wrote frame %li\r", frame);
    }
    else
    {
      vPrint("\nfailed to save frame %li\n", frame);
    }
  }
  vPrint("\n");
  return 1;
}


// The next functions are used to test and debug vScaleImageDown8
long graphical_module::ScaleImageA0()
{
  if (input == 0) return 0;
  long bbox_size = 0;
  long image_size = 0;
  vPrint("\nEnter bbox size and image size:\n");
  vScan("%li %li", &bbox_size, &image_size);
  return ScaleImageA(bbox_size, image_size);
}


long graphical_module::ScaleImageA(long bbox_size, long image_size)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vBoundingBox bbox = vBinaryBoundingBox(the_image);
  long center_row = bbox.CenterRow();
  long center_col = bbox.CenterCol();
  long new_center_row = image_size/2;
  long new_center_col = image_size/2;
  double max_side = bbox.MaxSide();
  double scaling = ((double) bbox_size) / max_side;
  grayscale_image * result = (grayscale_image *)
    vScaleImageDown8(the_image, scaling, center_row, center_col,
                      new_center_row, new_center_col, 
                      image_size, image_size);
  ShowImage(result);
  vdelete(the_image);
  return 1;
}


long graphical_module::ScaleImageB0()
{
  if (input == 0) return 0;
  double scaling = 0;
  long center_row, center_col, new_center_row, new_center_col, new_rows, new_cols;
  center_row = center_col = new_center_row = new_center_col = new_rows = new_cols = 0;
  vPrint("\nEnter scaling, center_row, center_col, new_center_row, new_center_col, new_rows, new_cols:\n");
  vScan("%lf %li %li %li %li %li %li", &scaling, &center_row, &center_col,
         &new_center_row, &new_center_col, &new_rows, &new_cols);
  return ScaleImageB(scaling, center_row, center_col, new_center_row,
                     new_center_col, new_rows, new_cols);
}


long graphical_module::ScaleImageB(double scaling, long center_row, long center_col,
                                 long new_center_row, long new_center_col,
                                 long new_rows, long new_cols)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * result = (grayscale_image *)
    vScaleImageDown8(the_image, scaling, center_row, center_col,
                      new_center_row, new_center_col, 
                      new_rows, new_cols);
  ShowImage(result);
  vdelete(the_image);
  return 1;
}



// Selects a color from the current image (assuming it is a color image).
long graphical_module::SelectColor0()
{
  if (input == 0) return 0;
  long r, g, b;
  r = g = b = 0;

  vPrint("\nEnter r, g, b\n");
  vScan("%li %li %li", &r, &g, &b);
  return SelectColor((uchar) r, (uchar) g, (uchar) b);
}


long graphical_module::SelectColor(uchar r, uchar g, uchar b)
{
  if (input == 0) return 0;

  vector<uchar> color(3);
  color[0] = r;
  color[1] = g;
  color[2] = b;

  color_image * current = input->current_image->ToRGBCopy();
  grayscale_image * result = vSelectColor(current, &color);
  ShowImage(result);
  vdelete(current);
  return 1;
}


// Counts the pixels having the specified color in the current image 
// (assuming it is a color image).
long graphical_module::CountColor0()
{
  if (input == 0) return 0;
  long r, g, b;
  r = g = b = 0;

  vPrint("\nEnter r, g, b\n");
  vScan("%li %li %li", &r, &g, &b);
  return CountColor((uchar) r, (uchar) g, (uchar) b);
}


long graphical_module::CountColor(uchar r, uchar g, uchar b)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();

  long size = the_image->Size();

  long counter = 0;
  long i;
  for (i = 0; i < size; i++)
  {
    if ((R[i] == r) && (G[i] == g) && (B[i] == b))
    {
      counter = counter+1;
    }
  }

  vPrint("%li pixels\n", counter);
  vdelete(the_image);
  return 1;
}


long graphical_module::CompareInputs0()
{
  if (input == 0) return 0;
  vPrint("\nEnter input1 and input2\n");
  long input1, input2;
  input1 = input2 = 0;
  vScan("%li %li", &input1, &input2);
  return CompareInputs(input1, input2);
}


long graphical_module::CompareInputs(long input1, long input2)
{
  if ((input1 < 0) || (input2 < 0) ||
      (input1 >= number_of_inputs) || (input2 >= number_of_inputs))
  {
    return 0;
  }

  double_image * image1 = new double_image(inputs[input1]->current_image);
  double_image * image2 = new double_image(inputs[input2]->current_image);
  long result = color_imageEqual(image1, image2);
  if (result > 0)
  {
    vPrint("\nImages are equal\n");
  }
  else
  {
    vPrint("\nImages are different, result = %li\n", result);
  }
  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::compare_images()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;
  double_image * image1 = new double_image(input->image_buffer[temp1]);
  double_image * image2 = new double_image(input->image_buffer[temp2]);
  long result = color_imageEqual(image1, image2);
  if (result > 0)
  {
    vPrint("\nImages are equal\n");
  }
  else
  {
    vPrint("\nImages are different, result = %li\n", result);
  }
  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::function_file_copy ()
{
  char filename1 [1000];
  char filename2 [1000];
  long block_size = 0;
  
  vPrint ("\nenter filename1, filename2, block size in megabytes:\n");
  vScan ("%s %s %li", filename1, filename2, & block_size);

  if ((block_size < 1) || (block_size > 1000))
  {
    vPrint ("block_size should be between 1 and 1000:\n");
    return 0;
  }

  FILE * input_file = fopen (filename1, vFOPEN_READ);
  if (input_file == 0)
  {
    vPrint ("failed to open file %s\n", filename1);
    return 0;
  }

  FILE * output_file = fopen (filename2, vFOPEN_WRITE);
  if (output_file == 0)
  {
    vPrint ("failed to open file %s\n", filename2);
    fclose (input_file);
    return 0;
  }

  long buffer_size = 1000000 * block_size;
  char * buffer = new char[buffer_size];
  if (buffer == 0)
  {
    vPrint (" failed to allocate memory for buffer\n");
    fclose (input_file);
    fclose (output_file);
    return 0;
  }

  long block_count = 0;
  while (1)
  {
    long bytes_read = fread (buffer, sizeof (char), buffer_size, input_file);
    vPrint ("read %8li bytes for block %8li\n", bytes_read, block_count);
    if (bytes_read == 0) 
    {
      break;
    }

    long bytes_written = fwrite (buffer, sizeof (char), bytes_read, output_file);
    vPrint ("wrote %8li bytes for block %8li\n", bytes_written, block_count);
    if (bytes_written != bytes_read) 
    {
      vPrint (" failure in write operation\n");
      break;
    }

    block_count ++;
  }

  fclose (input_file);
  fclose (output_file);
  vdelete2(buffer);
  return 1;
}


long graphical_module::image_average ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = function_image_average (&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long graphical_module::nonzero_average ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = ::nonzero_average (&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long graphical_module::image_median ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = function_image_median (&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long graphical_module::image_absolute ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image * current = new double_image(input->current_image);
  function_image_absolute(current, current);
  show_image (current);

  return 1;
}


long graphical_module::image_add ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal (first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_add(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::image_subtract ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal (first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_subtract(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::image_multiply ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal (first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_multiply(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::image_divide ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal (first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_divide(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}



long graphical_module::image_diff ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal (first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_subtract(image1, image2);
  function_image_absolute(result, result);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long graphical_module::image_total ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = function_image_total(&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long graphical_module::save_filtered_frames()
{
  if ((input == 0) || (input->sequence == 0) || 
      (current_output == -1))
  {
    return 0;
  }

  char buffer [1000];
  long start = 0, finish = 0;
  print ("\nenter name of script file, start frame, end frame:\n");
  question ("%s %li %li", buffer, & start, & finish);

  return save_filtered_frames(buffer, start, finish);
}


long graphical_module::save_filtered_frames(const char * script_name, long start, long finish)
{
  if ((input == 0) || (input->sequence == 0) || 
      (current_output == -1))
  {
    return 0;
  }

  long frames = input->number_of_frames;
  if (start < 0)
  {
    start = 0;
  }
  if ((finish >= frames) || (finish < 0))
  {
    finish = frames -1;
  }

  long frame;
  for (frame = start; frame <= finish; frame ++)
  {
    print("frame = %li of %li\n", frame, frames);
    make_current_frame(frame);
    base_interpreter->process_command_file1 (script_name);
    write_frame();
  }

  return 1;
}


// like save_filtered_frames, but here we just do the processing,
// we don't write to any output sequence.
long graphical_module::process_sequence()
{
  if ((input == 0) || (input->sequence == 0))
  {
    return 0;
  }

  char buffer [1000];
  long start = 0, finish = 0;
  print ("\nenter name of script file, start frame, end frame:\n");
  question ("%s %li %li", buffer, & start, & finish);

  long frames = input->number_of_frames;
  if (start < 0)
  {
    start = 0;
  }
  if ((finish >= frames) || (finish < 0))
  {
    finish = frames -1;
  }

  long frame;
  for (frame = start; frame <= finish; frame ++)
  {
    print("frame = %li\n", frame);
    make_current_frame(frame);
    base_interpreter->process_command_file1 (buffer);
  }

  return 1;
}


long graphical_module::random_image()
{
  if (input == 0)
  {
    default_input(100, 100);
  }

  long rows = 0, cols = 0;
  print("\nenter rows, cols:\n");
  question("%li %li", &rows, &cols);
  color_image * image = vRandomImage(rows, cols);
  show_image(image);

  return 1;
}


long graphical_module::resize()
{
  long width = 0, height = 0;
  print("\nenter height, width:\n");
  question("%li %li", &height, &width);

  window->Resize(height, width);
  return 1;
}


long graphical_module::add_ax_match(const char * filename)
{
  long length = string_length (filename);
  char* ending;
  char* alternative_ending;
  const char* shifted;
  long ending_length, position;
  
  ending = "_input.avi";
  alternative_ending = "_track.avi";
  ending_length = string_length (ending);

  position = length - ending_length;
  shifted = &(filename [position]);
  if (position >= 0)
  {
    if (function_compare_strings (ending, shifted) == 0)
    {
      char buffer [1000];
      memcpy(buffer, filename, position);
      strcpy(&(buffer[position]), alternative_ending);
      AddInputSilent (buffer);
      return 1;
    }
  }

  ending = "_track.avi";
  alternative_ending = "_input.avi";
  ending_length = string_length (ending);

  position = length - ending_length;
  shifted = &(filename [position]);
  if (position >= 0)
  {
    if (function_compare_strings (ending, shifted) == 0)
    {
      char buffer [1000];
      memcpy(buffer, filename, position);
      strcpy(&(buffer[position]), alternative_ending);
      AddInputSilent (buffer);
      return 1;
    }
  }

  return 1;
}


long graphical_module::test_canonical_orientation()
{
  float orientation = 0.0f;
  print ("\nenter orientation:\n");
  question ("%f", & orientation);
  float result = canonical_orientation(orientation);
  print ("result = %f\n", result);
  return 1;
}


// counts the number of pixels with higher intensity than the specified pixel
long graphical_module::pixel_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channels = current->channels ();
  if (channels != 1)
  {
    print ("\nthis image has %li channels, pixel_rank assumes one channel\n");
    return 1;
  }

  long vertical = 0, horizontal = 0;
  print ("\nenter vertical, horizontal:\n");
  question ("%li %li", & vertical, & horizontal);

  double value = current->double_value_safe (0, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_delete (thresholded);

  return 1;
}


// same as pixel_rank, but in addition, this function also shows
// a thresholded image using the intensity of the specified pixel as a threshold.
long graphical_module::pixel_threshold_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channels = current->channels ();
  if (channels != 1)
  {
    print ("\nthis image has %li channels, pixel_rank assumes one channel\n");
    return 1;
  }

  long vertical = 0, horizontal = 0;
  print ("\nenter vertical, horizontal:\n");
  question ("%li %li", & vertical, & horizontal);

  double value = current->double_value_safe (0, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


// here the entry at each band is treated as a separate entry
long graphical_module::entry_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channel = 0, vertical = 0, horizontal = 0;
  print ("\nenter channel, vertical, horizontal:\n");
  question ("%li %li %li", & channel, & vertical, & horizontal);

  double value = current->double_value_safe (channel, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_delete(thresholded);

  return 1;
}


long graphical_module::entry_threshold_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channel = 0, vertical = 0, horizontal = 0;
  print ("\nenter channel, vertical, horizontal:\n");
  question ("%li %li %li", & channel, & vertical, & horizontal);

  double value = current->double_value_safe (channel, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}

  
long graphical_module::threshold_entries ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  double value = 0;
  print ("\nenter value:\n");
  question ("%lf", & value);

  uchar_image* thresholded = ::threshold_entries (current, value);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


long graphical_module::inverse_threshold_entries ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  double value = 0;
  print ("\nenter value:\n");
  question ("%lf", & value);

  uchar_image* thresholded = ::inverse_threshold_entries (current, value);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


long graphical_module::channel_average()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  general_image* result = current->AverageBands();
  show_image (result);

  return 1;
}


long graphical_module::channel_total ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  general_image* result = current->SumBands();
  show_image (result);

  return 1;
}


long graphical_module::select_rectangle()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long top = 0, left = 0, vertical = 0, horizontal = 0;
  print("\nenter top, to left, vertical, horizontal:\n");
  question("%li %li %li %li", & top, & left, & vertical, & horizontal);
  long right = left + horizontal - 1;
  long bottom = top + vertical - 1;

  general_image* result = current->copy(left, right, top, bottom);
  show_image (result);

  return 1;
}


long graphical_module::load_image()
{
  if (input == 0) 
  {
    return 0;
  }

  char buffer[1000];
  print("\nenter filename:\n");
  question("%s", buffer);

  general_image * image = general_image::Read(buffer);
  show_image(image);

  return 1;
}


long graphical_module::double_threshold()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  if (original->channels() != 1)
  {
    print("\nimage has more than one band\n");
    return 1;
  }

  float_matrix image(original);
  float small_threshold = 0, big_threshold = 0;
  print("\nenter small_threshold, big_threshold:\n");
  question("%f %f", & small_threshold, & big_threshold);

  float_matrix result = ::double_threshold(image, small_threshold, big_threshold);
  show_image(result.copy());
  return 1;
}


long graphical_module::connected_size_threshold()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  if (original->channels() != 1)
  {
    print("\nimage has more than one band\n");
    return 1;
  }

  long threshold = 0;
  print("\nenter threshold:\n");
  question("%li", & threshold);

  float_image image(original);
  float_image result = ::connected_size_threshold(image, threshold);
  show_image(result.copy());

  return 1;
}


long graphical_module::find_smallest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  print("\nenter rank:\n");
  question("%li", &rank);

  uchar_image result = find_smallest_entries(&image, rank);
  show_image(result.copy());
  return 1;
}


long graphical_module::keep_smallest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  float replacement = 0.0f;
  print("\nenter rank, replacement:\n");
  question("%li %f", &rank, & replacement);

  float_image result = keep_smallest_entries(&image, rank, replacement);
  show_image(result.copy());
  return 1;
}


long graphical_module::find_largest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  print("\nenter rank:\n");
  question("%li", &rank);

  uchar_image result = find_largest_entries(&image, rank);
  show_image(result.copy());
  return 1;
}


long graphical_module::keep_largest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  float replacement = 0.0f;
  print("\nenter rank, replacement:\n");
  question("%li %f", &rank, & replacement);

  float_image result = keep_largest_entries(&image, rank, replacement);
  show_image(result.copy());
  return 1;
}


long graphical_module::set_range()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float minimum = 0, maximum = 0;
  print("\nenter minimum, maximum:\n");
  question("%f %f", & minimum, & maximum);
  
  float range = maximum - minimum;

  float_matrix image(original);
  float_matrix second_result = image.set_range(minimum, maximum);
  show_image(second_result.copy());

  return 1;
}


long graphical_module::maximizing_channels ()
{
  general_image * current = current_image ();
  if (current == 0)
  {
    return 0;
  }

  double_image image (current);
  long_matrix channels = ::maximizing_channels (& image);
  double_matrix maxima = maximal_image (& image);
  show_image (channels.copy ());
  show_image (maxima.copy ());

  return 1;
}


long graphical_module::remove_negatives()
{
  general_image * current = current_image();
  if ((current == 0) || (current->channels() != 1))
  {
    return 1;
  }
  float_image image(current);
  long counter;
  long size = image.all_channel_size();
  for (counter = 0; counter < size; counter++)
  {
    if (image(counter) < 0.0f)
    {
      image(counter) = 0.0f;
    }
  }

  show_image (image.copy());

  return 1;
}

  
long graphical_module::print_header()
{
  general_image * current = current_image();
  if (current == 0)
  {
    return 1;
  }

  current->print_header();
  return 1;
}


// this function is used to test the simple_float_matrix class.
long graphical_module::load_simple_float()
{
  check_input();
  char input_name[1000];
  char output_name[1010];

  print("\nenter input filename:\n");
  question("%s", input_name);
  sprintf(output_name, "trash_%s", input_name);
  
  long status = 0;
  simple_float_matrix simple_image(input_name, & status);
  if (status <= 0)
  {
    print("\nfailed to load %s, status = %li\n", input_name, status);
    return 0;
  }

  long success = simple_image.save(output_name);
  if (success <= 0)
  {
    print("\nfailed to save to %s, status = %li\n", input_name, status);
    return 0;
  }

  float_matrix converted = convert_simple_float(& simple_image);
  show_image(converted.copy());

  return 1;
}


long graphical_module::time_precomputed_square_root()
{
  static long junk = vPrecomputedSqrtf::PrecomputeValues(100000);

  float argument = 0;
  long times = 0;
  print("\nEnter argument, times:\n");
  question("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = sum + vPrecomputedSqrtf::SqrtfR(sum+.28129f);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for square_root: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }
  return 1;
}


long graphical_module::time_square_root()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = sum + sqrt(sum+argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for square_root: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_precomputed_cosine()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = vPrecomputedCosf::Cos2(sum+argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for precomputed cosine: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_cosine()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = cos(sum+argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for cosine: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_precomputed_sine()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = vPrecomputedSinf::Sin2(sum+argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for precomputed sine: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_sine()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = sum + sin(sum+argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for sine: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_precomputed_inverse_tangent()
{
  static long junk = precomputed_inverse_tangent::precompute_values();

  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = .28129f;
  for (i = 0; i < times; i++)
  {
    sum = sum + precomputed_inverse_tangent::value(sum+.28129f, argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for precomputed inverse tangent: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


long graphical_module::time_inverse_tangent()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

	time_t time_0 = clock();
  long i;
  float sum = .28129f;
  for (i = 0; i < times; i++)
  {
    sum = sum + atan2(sum+.28129f, argument);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for atan2: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


// converts orientation to [0,range) range
float s_canonical_range (float orientation, float range)
{
  float result;
  if (orientation < 0.0f)
  {
    float factor = floor(orientation/range);
    result = orientation - factor*range;
  }
  else
  {
    float factor = floor(orientation/range);
    result = orientation - factor*range;
  }

  // handle numerical errors, for example when orientation = -0.000000001
  if (result < 0.0f)
  {
    result += range;
  }

  if (result >= range)
  {
    result -= range;
  }

  if ((result < 0.0f) || (result >= range))
  {
    exit_error ("\nerror: canonical_orientation (%f) = %f\n", orientation, result);
  }

  return result;
}



long graphical_module::time_canonical_range()
{
  float argument = 0;
  long times = 0;
  vPrint("\nEnter argument, times:\n");
  vScan("%f %li", &argument, &times);

  float argument2 = .28129f;
	time_t time_0 = clock();
  long i;
  float sum = .28129f;
  for (i = 0; i < times; i++)
  {
    sum = s_canonical_range(sum, M_PIf);
  }

  vPrint("sum = %f. Done\n", sum);
	time_t time_1 = clock() - time_0;
  double seconds = (double(time_1))/CLOCKS_PER_SEC;
  print("total time for canonical_range: %lf\n", seconds);
  if (seconds > 0.0)
  {
    print("%lf operations per second\n", ((double) times)/seconds);
  }

  return 1;
}


