#include "stdafx.h"

#include "vplatform.h"
#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#else 
#include <iostream>
#endif
#include <math.h>

#include "basics/algebra.h"
#include "basics/drawing_temp.h"
#include "basics/precomputed.h"
#include "basics/simple_algo.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"

#include "basics/definitions.h" 




vTemplateCallerBase1 * vTemplateCallerBase1::Create(general_image * the_image)
{
  Type type = the_image->TypeName();
  switch(type)
  {
  case ScharType:
    return new vTemplateCaller1<char>;

  case ShortType:
    return new vTemplateCaller1<short>;

  case IntType:
    return new vTemplateCaller1<int>;

  case LongType:
    return new vTemplateCaller1<long>;

  case FloatType:
    return new vTemplateCaller1<float>;

  case DoubleType:
    return new vTemplateCaller1<double>;

  case UcharType:
    return new vTemplateCaller1<uchar>;

  case UshortType:
    return new vTemplateCaller1<ushort>;

  case UintType:
    return new vTemplateCaller1<uint>;

  case UlongType:
    return new vTemplateCaller1<ulong>;

  case OtherType:
    assert(0);
    return 0;

  default:
    assert(0);
    return 0;
  }
  assert(0);
  return 0;
}


vImageWrap * vWrapImage(general_image * the_image)
{
  Type type = the_image->TypeName();
  switch(type)
  {
  case ScharType:
    return new vMatrixWrap<char>(the_image);

  case ShortType:
    return new vMatrixWrap<short>(the_image);

  case IntType:
    return new vMatrixWrap<int>(the_image);

  case LongType:
    return new vMatrixWrap<long>(the_image);

  case FloatType:
    return new vMatrixWrap<float>(the_image);

  case DoubleType:
    return new vMatrixWrap<double>(the_image);

  case UcharType:
    return new vMatrixWrap<uchar>(the_image);

  case UshortType:
    return new vMatrixWrap<ushort>(the_image);

  case UintType:
    return new vMatrixWrap<uint>(the_image);

  case UlongType:
    return new vMatrixWrap<ulong>(the_image);

  case OtherType:
    assert(0);
    return new vMatrixWrap<uchar>(0);

  default:
    assert(0);
    return new vMatrixWrap<uchar>(0);
  }
  assert(0);
  return new vMatrixWrap<uchar>(0);
}


vImageWrap::vImageWrap(general_image * the_image)
{
  type_name = the_image->TypeName();
  switch(type_name)
  {
  case ScharType:
    pointer = new vMatrixWrap<char>(the_image);
    return;

  case ShortType:
    pointer = new vMatrixWrap<short>(the_image);
    return;

  case IntType:
    pointer = new vMatrixWrap<int>(the_image);
    return;

  case LongType:
    pointer = new vMatrixWrap<long>(the_image);
    return;

  case FloatType:
    pointer = new vMatrixWrap<float>(the_image);
    return;

  case DoubleType:
    pointer = new vMatrixWrap<double>(the_image);
    return;

  case UcharType:
    pointer = new vMatrixWrap<uchar>(the_image);
    return;

  case UshortType:
    pointer = new vMatrixWrap<ushort>(the_image);
    return;

  case UintType:
    pointer = new vMatrixWrap<uint>(the_image);
    return;

  case UlongType:
    pointer = new vMatrixWrap<ulong>(the_image);
    return;

  case OtherType:
    assert(0);
    pointer = new vMatrixWrap<uchar>(0);
    return;

  default:
    assert(0);
    pointer = new vMatrixWrap<uchar>(0);
  }
  assert(0);
  pointer = new vMatrixWrap<uchar>(0);
}


vCoordinateNormalizer::vCoordinateNormalizer()
{
  current_center_row = -1;
  current_center_col = -1;
  desired_center_row = -1;
  desired_center_col = -1;
  row_diff = 0;
  col_diff = 0;
  current_max_distance = -1;
  desired_max_distance = -1;
  distance_ratio = 0;
}


void vCoordinateNormalizer::SetCurrentCenter(double in_row, double in_col)
{
  current_center_row = in_row;
  current_center_col = in_col;
  row_diff = desired_center_row - current_center_row;
  col_diff = desired_center_col - current_center_col;
}


void vCoordinateNormalizer::SetDesiredCenter(double in_row, double in_col)
{
  desired_center_row = in_row;
  desired_center_col = in_col;
  row_diff = desired_center_row - current_center_row;
  col_diff = desired_center_col - current_center_col;
}


void vCoordinateNormalizer::SetCurrentMaxDistance(double in_distance)
{
  current_max_distance = in_distance;
  distance_ratio = current_max_distance / desired_max_distance;
}


void vCoordinateNormalizer::SetDesiredMaxDistance(double in_distance)
{
  desired_max_distance = in_distance;
  distance_ratio = current_max_distance / desired_max_distance;
}


void vCoordinateNormalizer::SetRowRange(long in_min, long in_max)
{
  min_row = in_min;
  max_row = in_max;
}


void vCoordinateNormalizer::SetColRange(long in_min, long in_max)
{
  min_col = in_min;
  max_col = in_max;
}

  
void vCoordinateNormalizer::Unnormalize(double row, double col)
{
  double centered_row = row - desired_center_row;
  double centered_col = col - desired_center_col;
  double scaled_row = centered_row * distance_ratio;
  double scaled_col = centered_col * distance_ratio;
  double source_row = scaled_row + current_center_row;
  double source_col = scaled_col + current_center_col;

  valid_pixels = 0;
  long low_row = (long) (floor(source_row));
  long high_row = low_row + 1;
  long low_col = (long) (floor(source_col));
  long high_col = low_col + 1;

  if ((low_row >= min_row) && (low_row <= max_row))
  {
    if ((low_col >= min_col) && (low_col <= max_col))
    {
      rows[valid_pixels] = low_row;
      cols[valid_pixels] = low_col;
      valid_pixels++;
    }
    if ((high_col >= min_col) && (high_col <= max_col))
    {
      rows[valid_pixels] = low_row;
      cols[valid_pixels] = high_col;
      valid_pixels++;
    }
  }
  if ((high_row >= min_row) && (high_row <= max_row))
  {
    if ((low_col >= min_col) && (low_col <= max_col))
    {
      rows[valid_pixels] = high_row;
      cols[valid_pixels] = low_col;
      valid_pixels++;
    }
    if ((high_col >= min_col) && (high_col <= max_col))
    {
      rows[valid_pixels] = high_row;
      cols[valid_pixels] = high_col;
      valid_pixels++;
    }
  }


  double row_side, col_side;
  long i;
  for (i = 0; i < valid_pixels; i++)
  {
    row_side = 1 - vAbs(source_row - rows[i]);
    col_side = 1 - vAbs(source_col - cols[i]);
    weights[i] = row_side * col_side;
  }
}


// vCumulativeCalculations takes in a histogram, that is a matrix of
// number_of_bins rows an four columns. We assume that the second column
// is the number of items thrown into that bin. This function puts into
// each row of the third column (col = 2) 
// the sums of the values in the second column
// from row 0 up to that row. It puts into each row of the fourth column the 
// sums of the values in the second column from the last row down to that row.
// Finally, it normalizes the third and fourth columns so that each entry 
// is converted from raw number of items to percentage of items.
//
// Return 0 if no entries are in the histogram (all entries in column 1 are 0).
ushort vCumulativeCalculations(vMatrix<double> * histogram)
{
  vArray2(double) bins = histogram->Matrix2();
  long number_of_bins = histogram->Rows();

  // Go through the bins and fill up columns 2 and 3 (cumulative histograms
  // from top to bottom and bottom to top respectively)
  bins[0][2] = bins[0][1];
  bins[number_of_bins - 1][3] = bins[number_of_bins - 1][1];
  long i;
  for (i = 1; i < number_of_bins; i++)
  {
    bins[i][2] = bins[i][1] + bins[i-1][2];
  }

  for (i = number_of_bins - 2; i >= 0; i--)
  {
    bins[i][3] = bins[i][1] + bins[i+1][3];
  }

  // If all bins have 0 items, we can't normalize, so we return 0.
  double factor;
  if (bins[0][3] == 0) 
  {
    factor = 0;
  }
  else
  {
    factor = 100 / bins[0][3];
  }

  // Normalize columns 2 and 3 so that they become percentages of pixels.
  for (i = 0; i < number_of_bins; i++)
  {
    bins[i][2] = bins[i][2] * factor;
    bins[i][3] = bins[i][3] * factor;
  }
  if (bins[0][3] == 0) return 0;
  else return 1;
}


// Beginning of wrappers around template functions.
// 
// Each of the following functions has a name that ends in a 0
// is a wrapper around a template function defined in algo_templates.cpp
// that has the same name and but doesn't end in a 0, and its arguments
// are v3dMatrix<type> objects instead of general_image objects.
// 

v3dMatrix<double> * vComputeMotion0(general_image * the_image1, 
                                       general_image * the_image2,
                                       general_image * the_image3)
{
  vCallerHolder1 holder(the_image1);
  return holder.caller->ComputeMotion(the_image1, the_image2, the_image3);
}


vMatrix<double> *
vBoundedRankHistogram0(general_image * input, long number_of_bins,
                        double lower, double upper)
{
  vCallerHolder1 holder(input);
  return holder.caller->BoundedRankHistogram(input, number_of_bins, lower, upper);
}


v3dMatrix<double> *
vImageDifference0(general_image * the_image1, general_image * the_image2)
{
  vCallerHolder1 holder(the_image1);
  return holder.caller->ImageDifference(the_image1, the_image2);
}


general_image * 
vMaxNeighbors0(general_image * the_image)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->MaxNeighbors(the_image);
}


vMatrix<double> *
vPixelBoundedHistogram0(general_image * input, long number_of_bins,
                         double lower, double upper)
{
  vCallerHolder1 holder(input);
  return holder.caller->PixelBoundedHistogram(input, number_of_bins, lower, upper);
}


vMatrix<double> * 
vPixelHistogram0(general_image * input, long number_of_bins)
{
  vCallerHolder1 holder(input);
  return holder.caller->PixelHistogram(input, number_of_bins);
}


general_image * vPyramidImage0(general_image * input, long times)
{
  vCallerHolder1 holder(input);
  return holder.caller->PyramidImage(input, times);
}


general_image * 
function_translate_image0(general_image * the_image, long translation_rows, 
                  long translation_cols)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->TranslateImage(the_image, translation_rows, translation_cols);
}

general_image * 
vScaleImage0(general_image * the_image, double scaling)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->ScaleImage(the_image, scaling, 
                                   the_image->Rows() / 2, the_image->Cols() / 2);
}


general_image * 
vScaleImage0(general_image * the_image, double scaling,
              long center_row, long center_col)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->ScaleImage(the_image, scaling, center_row, center_col);
}


general_image * 
vScaleImage0(general_image * the_image, double scaling,
              long center_row, long center_col,
              long new_rows, long new_cols)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->ScaleImage(the_image, scaling, center_row, center_col,
                                   new_rows, new_cols);
}


general_image * 
vResizeImage0(general_image * the_image, long new_rows, long new_cols)
{
  vCallerHolder1 holder(the_image);
  return holder.caller->ResizeImage(the_image, new_rows, new_cols);
}


vBoundingBox
vBinaryBoundingBox(general_image * input)
{
  vCallerHolder1 holder(input);
  return holder.caller->BinaryBoundingBox(input);
}


general_image * vMagnify0(general_image * input, long center_row, 
                         long center_col, long output_rows, long output_cols,
                         long factor)
{
  vCallerHolder1 holder(input);
  return holder.caller->Magnify(input, center_row, center_col, 
                                output_rows, output_cols, factor);
}


color_image * vCircularIntensity0(general_image * input, 
                             double low, double high, double ignore)
{
  vCallerHolder1 holder(input);
  return holder.caller->CircularIntensity(input, low, high, ignore);
}


general_image * vSelectRange0(general_image * input, long band, 
                             double low, double high)
{
  vCallerHolder1 holder(input);
  return holder.caller->SelectRange(input, band, low, high);
}


// End of wrappers around template functions


// Returns an image that has, at every pixel, the norm of the gradient
// of the intensity change at that pixel at color_image, as long as that pixel
// is a local maximum in the direction of the gradient of intensity.
// Otherwise the pixel value is 0.
// Note: It may seem that I should have a separate function that simply
// does non-maxima suppression and takes in the gradient image as an
// argument. However, that would not work, because we also need to
// calculate the actual direction of the gradient, and that can only
// be obtained by looking at the original image. One potential solution
// would be to pass in, as an argument, a two-band image, that contains the 
// gradient in the x and the y direction.
v3dMatrix<double> * vNonMaximaSuppression(general_image * the_image)
{
  v3dMatrix<double> * one_band = the_image->SumBandsDouble();
  v3dMatrix<double> * gradient_norms = vGradientNorm(one_band);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, 1);

  vArray2(double) result2 = result->Matrix2(0);
  vArray2(double) values2 = one_band->Matrix2(0);
  vArray2(double) gradient2 = gradient_norms->Matrix2(0);

  vWriteBorderValue(result, (double) 0);
  long row, col;
  double horizontal = 0;
  double vertical = 0;

  for (row = 1; row < rows - 1; row++)
  {
    for (col = 1; col < cols - 1; col++)
    {
      vPixelGradient2(values2, row, col, horizontal, vertical);
      double norm_square = horizontal*horizontal + vertical*vertical;
      if (norm_square == 0.0) 
      {
        result2[row][col] = 0;
        continue;
      }
      double norm = gradient2[row][col];
      double row_step = vertical / norm;
      double col_step = horizontal / norm;
      
      double previous_row = row - row_step;
      double previous_col = col - col_step;
      double next_row = row + row_step;
      double next_col = col + col_step;

      double previous_value = 
        vBilinearInterpolation(gradient_norms, previous_row, previous_col);
      double next_value = 
        vBilinearInterpolation(gradient_norms, next_row, next_col);
      if ((norm >= previous_value) && (norm >= next_value))
      {
        result2[row][col] = 255;
      }
      else
      {
        result2[row][col] = 0;
      }
    }
  }
  vdelete(one_band);
  vdelete(gradient_norms);

  return result;
}


// Returns a binary image that is non-zero at local maxima 
// in the direction of the gradient of the intensity in the image. 
v3dMatrix<double> * vIntensityMaxima(general_image * the_image)
{
  v3dMatrix<double> * one_band = the_image->SumBandsDouble();
  v3dMatrix<double> * gradient_norms = vGradientNorm2b(one_band);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, 1);

  vArray2(double) result2 = result->Matrix2(0);
  vArray2(double) values2 = one_band->Matrix2(0);
  vArray2(double) gradient2 = gradient_norms->Matrix2(0);

  vWriteBorderValue(result, (double) 0);
  long row, col;
  double horizontal = 0;
  double vertical = 0;

  for (row = 1; row < rows - 1; row++)
  {
    for (col = 1; col < cols - 1; col++)
    {
      vPixelGradient2b(values2, row, col, horizontal, vertical);
      double norm_square = horizontal*horizontal + vertical*vertical;
      if (norm_square == 0.0) 
      {
        result2[row][col] = 0;
        continue;
      }

      double value = values2[row][col];
      double norm = gradient2[row][col];
      double row_step = vertical / norm;
      double col_step = horizontal / norm;
      
      double previous_row = row - row_step;
      double previous_col = col - col_step;
      double next_row = row + row_step;
      double next_col = col + col_step;

      double previous_value = 
        vBilinearInterpolation(one_band, previous_row, previous_col);
      double next_value = 
        vBilinearInterpolation(one_band, next_row, next_col);
      if ((value >= previous_value) && (value >= next_value))
      {
        result2[row][col] = 255;
      }
      else
      {
        result2[row][col] = 0;
      }
    }
  }
  vdelete(one_band);
  vdelete(gradient_norms);

  return result;
}


// Returns an image that has the intensity of image at local maxima 
// in the direction of the gradient of the intensity in the image,
// and is zero elsewhere
float_matrix function_intensity_maxima(float_matrix image)
{
  const long margin = 2;
  long rows = image.Rows();
  long cols = image.Cols();
  float_matrix result(rows, cols);

  vArray2(float) result2 = result.Matrix2();
  vArray2(float) values2 = image.Matrix2();

  vWriteBorderValue3(&result, (float) 0, margin);
  long row, col;
  float horizontal = 0;
  float vertical = 0;

  for (row = margin; row < rows - margin; row++)
  {
    for (col = margin; col < cols - margin; col++)
    {
      vPixelGradient2b(values2, row, col, horizontal, vertical);
      float norm_square = horizontal*horizontal + vertical*vertical;
      if (norm_square == 0.0) 
      {
        result2[row][col] = 0;
        continue;
      }

      float value = values2[row][col];
      float norm = sqrt(norm_square);
      float row_step = vertical / norm;
      float col_step = horizontal / norm;
      
      float previous_row = row - row_step;
      float previous_col = col - col_step;
      float next_row = row + row_step;
      float next_col = col + col_step;

      float previous_value = vBilinearInterpolation(values2, previous_row, previous_col);
      float next_value = vBilinearInterpolation(values2, next_row, next_col);
      if ((value >= previous_value) && (value >= next_value))
      {
        result2[row][col] = value;
      }
      else
      {
        result2[row][col] = 0.0f;
      }
    }
  }

  return result;
}


// Returns a binary image that is non-zero at local maxima 
// in the direction of the normal to the given orientations. 
float_matrix function_intensity_maxima2(float_matrix image_matrix, float_matrix orientations_matrix,
                                        const float threshold)
{
  const long margin = 2;
  long rows = image_matrix.Rows();
  long cols = image_matrix.Cols();
  float_matrix result_matrix(rows, cols);

  vArray2(float) result = result_matrix.Matrix2();
  vArray2(float) image = image_matrix.Matrix2();
  matrix_pointer(float) orientations = orientations_matrix.Matrix2();

  vWriteValue(&result_matrix, (float) 0);
  long row, col;
  float horizontal = 0;
  float vertical = 0;

  for (row = margin; row < rows - margin; row++)
  {
    for (col = margin; col < cols - margin; col++)
    {
      float value = image[row][col];
      if (value <= threshold)
      {
        continue;
      }
      float orientation = orientations[row][col];
      
      float col_step = -vPrecomputedSinf::Sin2(orientation);
      float row_step = vPrecomputedCosf::Cos2(orientation);
      
      float previous_row = row - row_step;
      float previous_col = col - col_step;
      float next_row = row + row_step;
      float next_col = col + col_step;

      float previous_value = vBilinearInterpolation(image, previous_row, previous_col);
      float next_value = vBilinearInterpolation(image, next_row, next_col);
      if ((value >= previous_value) && (value >= next_value))
      {
        result[row][col] = value;
      }
    }
  }

  return result_matrix;
}


// Scale image up by a factor of two, and compute pixel values
// based on bilinear interpolation.
v3dMatrix<double> * vDoubleSize(v3dMatrix<double> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<double> * result = new v3dMatrix<double>(2*rows, 2*cols, bands);

  vArray3(double) result3 = result->Matrix3();
  vArray3(double) values3 = the_image->Matrix3();
  double row_weights[] = {0, 0, 0.5, 0.5};
  double col_weights[] = {0, 0.5, 0, 0.5};

  long row, col, band;
  for (row = 0; row < rows; row++)
  {
    double pixel_rows[] = {row, row, row+0.5, row+0.5};
    long new_pixel_rows[] = {2*row, 2*row, 2*row+1, 2*row+1};
    long tops[] = {row, row, row, row};
    long bottoms[] = {row+1, row+1, row+1, row+1};
    for (col = 0; col < cols; col++)
    {
      double pixel_cols[] = {col, col+0.5, col, col+0.5};
      long new_pixel_cols[] = {2*col, 2*col+1, 2*col, 2*col+1};
      long lefts[] = {col, col, col, col};
      long rights[] = {col+1, col+1, col+1, col+1};
      long i;
      for (i = 0; i < 4; i++)
      {
        double x = col_weights[i];
        double y = row_weights[i];
        long top = tops[i];
        long bottom = bottoms[i];
        long left = lefts[i];
        long right = rights[i];
        long new_row = new_pixel_rows[i];
        long new_col = new_pixel_cols[i];

        for (band = 0; band < bands; band++)
        {
          // Make sure rectangle is completely inside the image
          if ((top < 0) || (bottom >= rows) || (left < 0) || (right >= cols))
          {
            result3[band][new_row][new_col] = 0;
            continue;
          }

          double top_left = values3[band][top][left];
          double top_right = values3[band][top][right];
          double bottom_left = values3[band][bottom][left];
          double bottom_right = values3[band][bottom][right];
          double value = vBilinearInterpolation(x, y, top_left, top_right,
                                                 bottom_left, bottom_right);
          result3[band][new_row][new_col] = value;
        }
      }
    }
  }

  return result;
}


// Returns an image that has, at every pixel, the norm of the gradient
// of the intensity change at that pixel at color_image. I'm not sure if it
// does the right thing for images that have more than one bands.
v3dMatrix<double> * vGradientNorm0(general_image * the_image)
{
  v3dMatrix<double> * double_color_image = the_image->SumBandsDouble();
  v3dMatrix<double> * result = vGradientNorm(double_color_image);
  vdelete(double_color_image);
  return result;
}


// Returns an image that has, at every pixel, the norm of the gradient
// of the intensity change at that pixel at color_image. I'm not sure if it
// does the right thing for images that have more than one bands.
v3dMatrix<double> * vGradientNorm(v3dMatrix<double> * the_image)
{
  if (the_image->Bands() != 1) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, 1);

  vArray2(double) the_image2 = the_image->Matrix2(0);
  vArray2(double) result2 = result->Matrix2(0);
  vWriteBorderValue(result, (double) 0);

  long row, col, left, right, top, bottom;
  left = 1;
  right = cols - 2;
  top = 1;
  bottom = rows - 2;
  double horizontal = 0;
  double vertical = 0;
  for (row = top; row <= bottom; row++)
  {
    for (col = left; col <= right; col++)
    {
      vPixelGradient2(the_image2, row, col, horizontal, vertical);
      double norm_square = horizontal * horizontal + vertical * vertical;
      result2[row][col] = sqrt(norm_square);
    }
  }
  return result;
}


// here the calculations are non-centered
v3dMatrix<double> * vGradientNorm2b(v3dMatrix<double> * the_image)
{
{
  if (the_image->Bands() != 1) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, 1);

  vArray2(double) the_image2 = the_image->Matrix2(0);
  vArray2(double) result2 = result->Matrix2(0);
  vWriteBorderValue(result, (double) 0);

  long row, col, left, right, top, bottom;
  left = 1;
  right = cols - 2;
  top = 1;
  bottom = rows - 2;
  double horizontal = 0;
  double vertical = 0;
  for (row = top; row <= bottom; row++)
  {
    for (col = left; col <= right; col++)
    {
      vPixelGradient2b(the_image2, row, col, horizontal, vertical);
      double norm_square = horizontal * horizontal + vertical * vertical;
      result2[row][col] = sqrt(norm_square);
    }
  }
  return result;
}
}


// This function puts the locations of non-zero points into row_vector and
// col_vector. 
ushort vGetNonZeroPoints(grayscale_image * binary, vector<long> * row_vector, 
                          vector<long> * col_vector)
{
  // Make sure the row_vector and col_vector are empty to begin with
  row_vector->erase(row_vector->begin(), row_vector->end());
  col_vector->erase(col_vector->begin(), col_vector->end());

  long rows = binary->Rows();
  long cols = binary->Cols();
  long row, col;
  vArray2(uchar) I2 = binary->GetI2();
  for (row = 0; row < rows; row++)
  {
    vArray(uchar) I = I2[row];
    for (col = 0; col < cols; col++)
    {
      if (I[col] != 0)
      {
        row_vector->push_back(row);
        col_vector->push_back(col);
      }
    }
  }
  return 1;
}


// This function puts the locations of non-zero points into *rows_address and
// *cols_address, and the number of those points at *number_address. 
ushort vGetNonZeroPoints4(grayscale_image * binary, vArray(long) * rows_address, 
                           vArray(long) * cols_address, long * number_address)
{
  long image_rows = binary->Rows();
  long image_cols = binary->Cols();
  long row, col;
  vArray2(uchar) I2 = binary->GetI2();
  vArray(uchar) I = binary->GetI();
  long number = 0;
  long size = image_rows * image_cols;
  long index = 0;
  // First scan: Figure out how non-zero points there are.
  for (index = 0; index < size; index++)
  {
    if (I[index] > 0) number++;
  }
  
  // Create rows and cols arrays, and save values into the argument addresses.
  vArray(long) rows = vnew(long, number);
  vArray(long) cols = vnew(long, number);
  *rows_address = rows;
  *cols_address = cols;
  *number_address = number;

  // Second scan: Put the coordinates of the non-zero points into
  // the rows and cols arrays.
  index = 0;
  for (row = 0; row < image_rows; row++)
  {
    vArray(uchar) I = I2[row];
    for (col = 0; col < image_cols; col++)
    {
      if (I[col] != 0)
      {
        rows[index] = row;
        cols[index] = col;
        index++;
      }
    }
  }
  return 1;
}


// This function puts the locations of non-zero points into *rows_address and
// *cols_address, and the number of those points at *number_address. 
ushort vGetNonZeroPoints4(grayscale_image * binary, vArray(double) * rows_address, 
                           vArray(double) * cols_address, long * number_address)
{
  long image_rows = binary->Rows();
  long image_cols = binary->Cols();
  long row, col;
  vArray2(uchar) I2 = binary->GetI2();
  vArray(uchar) I = binary->GetI();
  long number = 0;
  long size = image_rows * image_cols;
  long index = 0;
  // First scan: Figure out how non-zero points there are.
  for (index = 0; index < size; index++)
  {
    if (I[index] > 0) number++;
  }
  
  // Create rows and cols arrays, and save values into the argument addresses.
  vArray(double) rows = vnew(double, number);
  vArray(double) cols = vnew(double, number);
  *rows_address = rows;
  *cols_address = cols;
  *number_address = number;

  // Second scan: Put the coordinates of the non-zero points into
  // the rows and cols arrays.
  index = 0;
  for (row = 0; row < image_rows; row++)
  {
    vArray(uchar) I = I2[row];
    for (col = 0; col < image_cols; col++)
    {
      if (I[col] != 0)
      {
        rows[index] = row;
        cols[index] = col;
        index++;
      }
    }
  }
  return 1;
}


color_image * vLabelsToImage(v3dMatrix<long> * labels_image,
                          vector<Label> * label_objects)
{
  vArray(long) labels = labels_image->Matrix(0);
  long rows = labels_image->Rows();
  long cols = labels_image->Cols();
  color_image * result = new color_image(rows, cols);
  vArray(uchar) R = result->GetR();
  vArray(uchar) G = result->GetG();
  vArray(uchar) B = result->GetB();

  long i;
  for (i = 0; i < (long) label_objects->size(); i++)
  {
    (*label_objects)[i].r = rand() % 255;
    (*label_objects)[i].g = rand() % 255;
    (*label_objects)[i].b = rand() % 255;
  }
  
  for (i = 0; i < labels_image->Size(); i++)
  {
    long label = labels[i];
    uchar r = (*label_objects)[label].r;
    uchar g = (*label_objects)[label].g;
    uchar b = (*label_objects)[label].b;
    R[i] = r;
    G[i] = g;
    B[i] = b;
  }
  return result;
}



void vInverseImage(v3dMatrix<uchar> * the_image)
{
  vArray(uchar) matrix = the_image->Matrix();
  long size = the_image->AllBandSize();
  long i;
  for (i = 0; i < size; i++)
  {
    matrix[i] = 255 - matrix[i];
  }
}

// The assumptions is that regions is a labeled image, where each region has a 
// non-zero label. Regions can contain holes, i.e subregions that belong to 
// the background. A hole is a connected 
// component that has a non-zero label in the regions image (meaning that 
// it is not connected to the top left pixel, which is assumed to be background)
// and has label 1 in the background image.
// We delete the "hole" pixels in regions by replacing their value with the label
// of the region that encloses them.

void vFillUpHoles(v3dMatrix<long> * regions, 
                   v3dMatrix<long> * background)
{
  assert(regions->Bands() == 1);
  assert(background->Bands() == 1);
  long rows = regions->Rows();
  long cols = regions->Cols();
  assert(rows == background->Rows());
  assert(cols == background->Cols());
	
  vArray2(long) region_matrix = regions->Matrix2(0);
  vArray2(long) background_matrix = background->Matrix2(0);
	
  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols-1; col++)
    {
      long current = region_matrix[row][col];
      long right = col+1;
      long & next = region_matrix[row][right];
      long background_label = background_matrix[row][right];

      if (current != 0)
      {
        // If next is background but not connected to the top left pixel
        // then take it (assign it to current label)
        if ((next == 0) && (background_label != 1))
        {
          next = current;
        }
      }
    }
  }
}


// This function assumes that foreground is a binary image
// (values are 0 or 255). A hole is a connected component
// of value 0 that is not connected to the top left corner
// (which is also assumed to be background). In the
// result, the holes are erased (their values are 255).
grayscale_image * vFillUpHoles1(v3dMatrix<uchar> * foreground)
{
  if (foreground->Bands() != 1) return 0;
  long rows = foreground->Rows();
  long cols = foreground->Cols();
  grayscale_image * result = new grayscale_image(rows, cols);
  long success = vFillUpHoles2(foreground, result);
  if (success <= 0)
  {
    vdelete(result);
    return 0;
  }
  else
  {
    return result;
  }
}


long vFillUpHoles2(v3dMatrix<uchar> * foreground, 
                    v3dMatrix<uchar> * result)
{
  // We assume that the top left pixel is background,
  // both images are grayscale (one band) and the
  // image sizes are equal.
  if (foreground->Matrix2(0)[0][0] != 0) return 0;
  if (foreground->Bands() != 1) return 0;
  if (result->Bands() != 1) return 0;
  if (foreground->Rows() != result->Rows()) return 0;
  if (foreground->Cols() != result->Cols()) return 0;

  vector<Label> labels;
  v3dMatrix<uchar> * background = vCopyImage(foreground);
  vInverseImage(background);
  v3dMatrix<long> * background_components = 
    background->ConnectedComponents(&labels);
  vArray(long) background_array = background_components->Matrix();
  vArray(uchar) result_array = result->Matrix();
  long rows = foreground->Rows();
  long cols = foreground->Cols();
  long size = rows * cols;

  long i;
  // Here we calculate the result. We use the fact that in 
  // the background_components the connected component with label 1
  // is the component connected to the top left pixel.
  for (i = 0; i < size; i++)
  {
    if (background_array[i] != 1) result_array[i] = 255;
    else result_array[i] = 0;
  }
  vdelete(background);
  vdelete(background_components);
  return 1;
}


Label vFindClosestLabel(vector<Label> & labels, Label & label,
			 long start, long end)
{
  long length = labels.size();
  if (start < 0) start = 0;
  if (end >= length) end = length - 1;
  if (start > end) return Label();

  double min_distance = vLabelDistance(label, labels[start]);
  long min_index = start;
  long i;
  for (i = start + 1; i <= end; i++)
  {
    double distance = vLabelDistance(label, labels[i]);
    if (distance < min_distance) 
    {
      min_distance = distance;
      min_index = i;
    }
  }
  Label result = labels[min_index];
  return result;
}


Label vLeftmostLabel(vector<Label> & labels, long start, long end)
{
  long length = labels.size();
  if (start < 0) start = 0;
  if (end >= length) end = length - 1;
  if (start > end) return Label();

  long i;
  Label min_label = labels[start];
  for (i = start + 1; i <= end; i++)
  {
    Label label = labels[i];
    if (label.left < min_label.left) min_label = label;
  }
  Label result = min_label;
  return result;
}


Label vRightmostLabel(vector<Label> & labels, long start, long end)
{
  long length = labels.size();
  if (start < 0) start = 0;
  if (end >= length) end = length - 1;
  if (start > end) return Label();

  long i;
  Label max_label = labels[start];
  for (i = start + 1; i <= end; i++)
  {
    Label label = labels[i];
    if (label.right > max_label.right) max_label = label;
  }
  Label result = max_label;
  return result;
}


double vLabelDistance(Label & label1, Label & label2)
{
  double result = 0;
  result = result + vAbs(label1.left - label2.left);
  result = result + vAbs(label1.right - label2.right);
  result = result + vAbs(label1.top - label2.top);
  result = result + vAbs(label1.bottom - label2.bottom);
  return result;
}


// We threshold the input image, so that all non-zero values become 255.
// We also pad the image, if necessary, to make sure it has at least
// as many rows and cols as specified by the "rows" and "cols" arguments
// We call vNormalizeImage which rescales the image, and then we crop
// the image to obtain a result image of size "rows" x "cols".
grayscale_image * vNormalizedBinary(general_image * the_image,
                                 double max_distance, long rows, long cols)
{
  if (the_image == 0) return 0;
  double double_rows = rows;
  double double_cols = cols;
  double center_row = (double_rows - 1) / 2;
  double center_col = (double_cols - 1) / 2;

  grayscale_image * thresholded = (grayscale_image *) the_image->threshold(1);
  grayscale_image * padded = (grayscale_image *) vPad(thresholded, rows, cols);
  grayscale_image * normalized = (grayscale_image *) vNormalizeImage(padded, max_distance, 
                                                              center_row, center_col);
  grayscale_image * cropped = (grayscale_image *) vCopyRectangle(normalized, 0, rows-1, 0, cols-1);
  vdelete(thresholded);
  thresholded = (grayscale_image *) cropped->threshold(128);

  vdelete(padded);
  vdelete(normalized);
  vdelete(cropped);
  return thresholded;
}


// March 17, 2002. I think the description of vScaleBoundingBox is 
// incorrect, it does not treat its input as a binary image.

// vScaleBoundingBox treats its input as a binary image, meaning that
// it treats all non-zero values as if they were equal to 255. It finds
// the top, bottom, left and right-most pixels that are non-zero. We define
// as size of the bounding box of the foreground the maximum value between
// bottom - top + 1 and right - left + 1. We return a binary image (values 
// are 0 or 255) that is a translated and scaled version of the original,
// so that the bounding box of the foreground has size new_box_size and
// the center of that bounding box is the point specified by
// new_center_row and new_center_col.
grayscale_image * vScaleBoundingBox2(grayscale_image * binary, long long_new_box_size,
                                  long new_center_row, long new_center_col)

{
  vBoundingBox box = vBinaryBoundingBox(binary);
  long height = box.bottom - box.top + 1;
  long width = box.right - box.left + 1;
  long long_box_size = Max(width, height);

  // If long_box_size is 0 or less it means the binary input image is all 
  // zeros.
  if (long_box_size <= 0) 
  {
    grayscale_image * result = (grayscale_image *) vCopyImage(binary);
    return result;
  }
  
  // Figure out scaling factor;
  double box_size = (double) long_box_size;
  double new_box_size = (double) long_new_box_size;
  double scaling = new_box_size / box_size;

  // Figure out translation
  double current_center_row = (box.bottom + box.top) / 2;
  double current_center_col = (box.right + box.left) / 2;
  long rows = binary->Rows();
  long cols = binary->Cols();
  long translation_rows = (long) (new_center_row - current_center_row);
  long translation_cols = (long) (new_center_col - current_center_col);
  v3dMatrix<uchar> * translated, * scaled;
  translated = function_translate_image(binary, translation_rows, 
                                translation_cols);
  scaled = vScaleImage4(translated, scaling, 
                         new_center_row, new_center_col);
  vdelete(translated);
  return (grayscale_image *) scaled;
}


grayscale_image * vNormalizeLabeledImage(grayscale_image * labels, double max_distance,
									                    long rows, long cols)
{
  if (labels == 0) return 0;
  double double_rows = rows;
  double double_cols = cols;
  double center_row = (double_rows - 1) / 2;
  double center_col = (double_cols - 1) / 2;

  grayscale_image * padded = (grayscale_image *) vPad(labels, rows, cols);
  grayscale_image * normalized;
  normalized = (grayscale_image *) vNormalizeLabeledImage3(padded, max_distance, 
                                                        center_row, center_col);

  vdelete(padded);
  return normalized;
}


color_image * vProcessPixels(color_image * the_image, vPixelPred predicate)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  
  color_image * result = new color_image(rows, cols);
  long row, col;
  vArray2(uchar) R2 = result->GetR2();
  vArray2(uchar) G2 = result->GetG2();
  vArray2(uchar) B2 = result->GetB2();

  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
    {
      predicate(the_image, row, col, R2[row][col], G2[row][col], B2[row][col]);
    }

  return result;
}


void vRowEdgePred(color_image * the_image, long row, long col,
		   uchar & r, uchar & g, uchar & b)
{
  vArray2(uchar) R;
  vArray2(uchar) G;
  vArray2(uchar) B;
  R = the_image->GetR2();
  G = the_image->GetG2();
  B = the_image->GetB2();

  if (col == 0)
    {
      r = R[row][col];
      g = G[row][col];
      b = B[row][col];
      return;
    }

  else
    {
      r = abs(R[row][col] - R[row][col - 1]);
      g = abs(G[row][col] - G[row][col - 1]);
      b = abs(B[row][col] - B[row][col - 1]);
    }
}


// Find the 8-connected components in the image.
long_matrix vConnectedComponents8(uchar_image * the_image,
                                  vector<Label> * label_objects)
{
  if (the_image->channels() > 1)
  {
    exit_error("error: in vConnectedComponents8, more than one channel in input image\n");
  }

  // We assume that the image is binarized, and gray-scale. 

  vArray2(uchar) R2 = the_image->Matrix2(0);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long number_of_labels = 0;
  long row, col;

  long_matrix labels_image(rows, cols);
  vArray2(long) labels = labels_image.Matrix3()[0];

  uchar previous_value = 0;

  // First pass: assign some initial labels to each pixel. We assign the 
  // same label to consecutive non-zero
  // pixels in the same row.
  for (row = 0; row < rows; row++)
  {
    previous_value = (uchar) 0;
    for (col = 0; col < cols; col++)
    {
      uchar current = R2[row][col];
      if (current != 0)
      {
        if (previous_value == 0) number_of_labels++;
        labels[row][col] = number_of_labels;
      }
      else labels[row][col] = 0;
      previous_value = (uchar) current;
    }
  }


 	  
// Second pass: For each connected region, find the smallest label in
//  that region.
  long * label_refs = new long[number_of_labels+1];
  long i;
  for (i = 0; i <= number_of_labels; i++)
    label_refs[i] = i;

  for (row = 1; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long label1 = labels[row][col];
      if (label1 == 0) continue;
      long left = Max<long>(col-1, 0);
      long right = Min(col+1, cols-1);
      long col2;
      for (col2 = left; col2 <= right; col2++)
      {
        long label2 = labels[row-1][col2];
        if (label2 == 0) continue;
        long ref1 = label_refs[label1];
        long ref2 = label_refs[label2];
        long temp_ref;
        while (ref1 != label_refs[ref1])  
        {
	        temp_ref = ref1;
	        ref1 = label_refs[ref1];
	        label_refs[temp_ref] = label_refs[ref1];
        }

        while (ref2 != label_refs[ref2])  
        {
	        temp_ref = ref2;
	        ref2 = label_refs[ref2];
	        label_refs[temp_ref] = label_refs[ref2];
        }

        if (ref1 < ref2) temp_ref = ref1;
        else temp_ref = ref2;

        label_refs[label2] = temp_ref;
        label_refs[label1] = temp_ref;
        label_refs[ref1] = temp_ref;
        label_refs[ref2] = temp_ref;
        labels[row][col] = temp_ref;
      }
    }
  }

  // Now we can count the number of different connected components in
  // the image. We renumber the labels, so that they have consecutive
  // numbers.
  long * new_refs = new long[number_of_labels+1];
  for (i = 0; i <= number_of_labels; i++)
    new_refs[i] = 0;
  long new_numbers = 1;

  for (i = 1; i <= number_of_labels; i++)
  {
    long temp_label = label_refs[i];
    while(temp_label != label_refs[temp_label])
    {
      assert(temp_label > label_refs[temp_label]);
      long temp_ref = temp_label;
      temp_label = label_refs[temp_label];
      label_refs[temp_ref] = label_refs[temp_label];
    }
    label_refs[i] = temp_label;
    if (temp_label == i)
    {
      new_refs[temp_label] = new_numbers;
      new_numbers++;
    }
    else 
    {
      new_refs[i] = new_refs[temp_label];
    }
  }

  // Third pass: For each pixel, replace the temporary label with the
  // renumbered label;

  // number_of_components is a pass-by-reference argument to the function.

  Label temp_label;
  for (i = 0; i < new_numbers; i++)
  {
    temp_label.id = i;
    label_objects->push_back(temp_label);
  }

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long label = labels[row][col];
      long label_ref = label_refs[label];
      long new_label = new_refs[label_ref];
      labels[row][col] = new_label;
      Label & label_object = (*label_objects)[new_label];
      label_object.count++;
      if (row < label_object.top)
	      label_object.top = row;
      if (row > label_object.bottom)
	      label_object.bottom = row;
      if (col < label_object.left)
	      label_object.left = col;
      if (col > label_object.right)
	      label_object.right = col;

      // Store the value corresponding to the label, if this
      // is the first pixel we found on that label
      if (label_object.count == 1)
      {
        label_object.value = R2[row][col];
      }
    }
  }

  vdelete2(label_refs);
  vdelete2(new_refs);
  return labels_image;
}


// For each 8-connected component in labels it finds the two points
// in that component that are the most distant from each other (in
// terms of the shortest path within that component that connects them).
// In the result matrix, row i looks like 
// (start_row, start_col, end_row, end_col), where those four numbers
// specify the position of the start and endpoint of segment i.
// Of course, it is arbitrary which endpoint we call start and
// which we call end.
// rows and cols contain the location of foreground pixels. This
// way we don't have to scan the whole image to find them.
//
// IMPORTANT NOTE: This function assumes that the top, bottom,
// left and right sides of the image are background. This
// allows us to skip boundary checks and speeds up the function.
//
// How do we find the two most distant points in a component?
// The answer is a bit heuristic here, and is not guaranteed
// to be correct. I don't know if there
// is a non-heuristic and equally efficient method. Basically,
// we pick a random point of the component we are looking at,
// and using breadth-first search we find the most distant 
// point from that point. We use the most distant point as our
// next pick and again do breadth-first search to find the 
// most distant point of that pick. We stop when the distance
// of a pick to its most distant point is the same as the
// distance between that pick and the pick before that.
vMatrix<long> v8ComponentEndpoints(vector<long> * point_rows,
                                     vector<long> * point_cols,
                                     v3dMatrix<long> * labels)
{
  long rows = labels->Rows();
  long cols = labels->Cols();
  long size = labels->Size();
  long number_of_points = point_rows->size();
  assert(number_of_points == point_cols->size());
  vArray2(long) values2 = labels->Matrix2(0);

  // Find out the number of labels (heuristically estimated as
  // the value of the maximum label in the image, plus 1).
  long number_of_labels = 1;
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    long row = (*point_rows)[i];
    long col = (*point_cols)[i];
    long label = values2[row][col];
    if (label >= number_of_labels)
    {
      number_of_labels = label+1;
    }
  }
  
  vArray(vector<long>) component_rows = vnew(vector<long>, number_of_labels);
  vArray(vector<long>) component_cols = vnew(vector<long>, number_of_labels);
  // Now put in component_rows[i] and component_cols[i] the locations
  // of the points in the component labeled with i in labels.
  for (i = 0; i < number_of_points; i++)
  {
    long row = (*point_rows)[i];
    long col = (*point_cols)[i];
    long label = values2[row][col];
    component_rows[label].push_back(row);
    component_cols[label].push_back(col);
  }

  // Create the result matrix
  vMatrix<long> result_matrix(number_of_labels, 4);
  vArray2(long) result = result_matrix.Matrix2();
  v3dMatrix<long> scratch(rows, cols, 1);
  vArray2(long) scratch2 = scratch.Matrix2(0);
  // Now find the endpoint of each component
  for (i = 0; i < number_of_labels; i++)
  {
    long coordinates[4];
    vFindEndpoints(&(component_rows[i]), &(component_cols[i]),
                    labels, &scratch, coordinates);
    long j;
    for (j = 0; j < 4; j++)
    {
      result[i][j] = coordinates[j];
    }
  }
  vdelete2(component_rows);
  vdelete2(component_cols);
  return result_matrix;
}


// This function is an auxiliary function for v8ComponentEndpoints,
// although it can be used by itself as well. It finds the 
// endpoints of a particular component. It is assumed that 
// rows and cols specify the locations of the pixels that belong
// to that component. coordinates is assumed to be an array of
// length 4, and it will contain at the end of the function 
// start_row, start_col, end_row, end_col, in that order.
// scratch is simply a matrix that is used for book-keeping.
// It is passed in as an argument so that if we call
// vFindEndpoints repeatedly we don't have to allocate
// and deallocate scratch inside the function every time.
long vFindEndpoints(vector<long> * point_rows,
                     vector<long> * point_cols,
                     v3dMatrix<long> * labels,
                     v3dMatrix<long> * scratch,
                     long * coordinates)
{
  long i;
  long number_of_points = point_rows->size();
  if (number_of_points == 0)
  {
    for (i = 0; i < 4; i++)
    {
      coordinates[i] = -1;
    }
    return 0;
  }

  long rows = labels->Rows();
  long cols = labels->Cols();
  long max_distance = rows*cols+3;
  long current_max_distance = -1;
  long next_max_distance = 0;
  long end_row = (*point_rows)[0];
  long end_col = (*point_cols)[0];
  long start_row, start_col;
  vArray2(long) labels2 = labels->Matrix2(0);
  vArray2(long) scratch2 = scratch->Matrix2(0);
  long component_label = labels2[end_row][end_col];

  while(next_max_distance > current_max_distance)
  {
    current_max_distance = next_max_distance;
    start_row = end_row;
    start_col = end_col;
    // First write max_distance into all pixel locations
    // in scratch that correspond to points
    // on the component of interest.
    for (i = 0; i < number_of_points; i++)
    {
      long row = (*point_rows)[i];
      long col = (*point_cols)[i];
      scratch2[row][col] = max_distance;
    }
    // to_visit_rows[i] will be the row of the i-th point
    // to visit. to_visit_distance[i] will be the distance
    // of that point.
    vector<long> to_visit_rows;
    vector<long> to_visit_cols;
    long index = 0;
    to_visit_rows.push_back(start_row);
    to_visit_cols.push_back(start_col);
    scratch2[start_row][start_col] = 0;
    while(index < (long) to_visit_rows.size())
    {
      long current_row = to_visit_rows[index];
      long current_col = to_visit_cols[index];
      long current_distance = scratch2[current_row][current_col];
      long next_distance = current_distance + 1;
      index++;
      long row, col;
      for (row = current_row - 1; row <= current_row+1; row++)
      {
        for (col = current_col - 1; col <= current_col+1; col++)
        {
          long label = labels2[row][col];
          if (label != component_label) continue;
          if (scratch2[row][col] <= next_distance) continue;
          scratch2[row][col] = next_distance;
          to_visit_rows.push_back(row);
          to_visit_cols.push_back(col);
        }
      }
    }
    // Now we have calculated distances for every pixel in 
    // the component, and those distances are stored in 
    // the scratch2 array. We just need to go over all pixels
    // in the component and find the most distant pixel.
    long max_distance_found = -1;
    long max_index = -1;
    for (i = 0; i < number_of_points; i++)
    {
      long row = (*point_rows)[i];
      long col = (*point_cols)[i];
      long current_distance = scratch2[row][col];
      if ((current_distance < 0) || (current_distance == max_distance))
      {
//        exit_error("Something is wrong in vFindComponents\n");
      }
      if (current_distance > max_distance_found)
      {
        max_distance_found = current_distance;
        max_index = i;
      }
    }

    // Get the next pick.
    end_row = (*point_rows)[max_index];
    end_col = (*point_cols)[max_index];
    next_max_distance = max_distance_found;
  }
  coordinates[0] = start_row;
  coordinates[1] = start_col;
  coordinates[2] = end_row;
  coordinates[3] = end_col;
  return 1;
}


long vMergeComponents(v3dMatrix<long> * components,
                       vMatrix<long> * endpoints,
                       vector<long> * point_rows, vector<long> * point_cols)
{
  long number_of_labels = endpoints->Rows();
  long rows = components->Rows();
  long cols = components->Cols();
  long size = rows * cols;
  long max_distance = rows * rows + cols * cols + 3;
  long label1, label2;
  long min_label1, min_label2;
  long min_distance = max_distance;
  long point1_row, point1_col, point2_row, point2_col;

  vArray2(long) endpoints2 = endpoints->Matrix2();
  // Find the closest pair of endpoints that belong 
  // to different components.
  for (label1 = 0; label1 < number_of_labels; label1++)
  {
    // Read the endpoints of component label1.
    long label1_rows[2], label1_cols[2];
    label1_rows[0] = endpoints2[label1][0];
    if (label1_rows[0] == -1) continue;
    label1_cols[0] = endpoints2[label1][1];
    label1_rows[1] = endpoints2[label1][2];
    label1_cols[1] = endpoints2[label1][3];

    for (label2 = label1+1; label2 < number_of_labels; label2++)
    {
      // Read the endpoints of label2;
      long label2_rows[2], label2_cols[2];
      label2_rows[0] = endpoints2[label2][0];
      if (label2_rows[0] == -1) continue;
      label2_cols[0] = endpoints2[label2][1];
      label2_rows[1] = endpoints2[label2][2];
      label2_cols[1] = endpoints2[label2][3];

      long i, j;
      for (i = 0; i < 2; i++)
      {
        long row1 = label1_rows[i];
        long col1 = label1_cols[i];
        for (j = 0; j < 2; j++)
        {
          long row2 = label2_rows[j];
          long col2 = label2_cols[j];
          long row_diff = row2 - row1;
          long col_diff = col2 - col1;
          long distance = row_diff * row_diff + col_diff * col_diff;
          if (distance < min_distance)
          {
            min_label1 = label1;
            min_label2 = label2;
            min_distance = distance;
            point1_row = row1;
            point1_col = col1;
            point2_row = row2;
            point2_col = col2;
          }
        }
      }
    }
  }

  vPrint("min_distance = %li, min_label1 = %li, min_label2 = %li\n",
          min_distance, min_label1, min_label2);

  vector<long> line_values;
  line_values.push_back(min_label1);

  vPrint("Drawing line (%li, %li) to (%li, %li)\n",
          point1_row, point1_col, point2_row, point2_col);
  vDrawLine(components, point1_row, point1_col, point2_row, point2_col,
             &line_values);
  return vMergeComponents(components, min_label1, min_label2,
                           point_rows, point_cols);
}


// Replace all occurences of Max(label1, label2) in 
// components with Min(label1, label2), and relabel
// all values > label2 by decrementing their label by
// one.
long vMergeComponents(v3dMatrix<long> * components,
                       long label1, long label2,
                       vector<long> * rows, vector<long> * cols)
{
  vArray2(long) values2 = components->Matrix2(0);
  long number_of_points = rows->size();

  // Make sure label1 < label2;
  if (label1 == label2) return 1;
  if (label1 > label2)
  {
    long temp = label1;
    label1 = label2;
    label2 = temp;
  }

  long i;
  for (i = 0; i < number_of_points; i++)
  {
    long row = (*rows)[i];
    long col = (*cols)[i];
    long label = values2[row][col];
    if (label == label2)
    {
      values2[row][col] = label1;
    }
    else if (label > label2)
    {
      values2[row][col]--;
    }
  }
  return 1;
}


color_image * vOverlay0(general_image * the_image1, general_image * the_image2)
{
  grayscale_image * gray1 = the_image1->ToGray();
  grayscale_image * gray2 = the_image2->ToGray();
  color_image * result = vOverlay(gray1, gray2);
  if (gray1 != the_image1) vdelete(gray1);
  if (gray2 != the_image2) vdelete(gray2);
  return result;
}


// Take two gray scale images, and put them on top of
// each other in a color image, where the red band
// shows the entries in color_image1, and the green band
// shows the entries in color_image2.
color_image * vOverlay(grayscale_image * the_image1, grayscale_image * the_image2)
{
  long rows1 = the_image1->Rows();
  long cols1 = the_image1->Cols();
  long rows2 = the_image2->Rows();
  long cols2 = the_image2->Cols();
  // Get the size of the intersection of the two images.
  long min_rows = Min(rows1, rows2);
  long min_cols = Min(cols1, cols2);

  color_image * result_matrix = new color_image(min_rows, min_cols);
  vArray3(uchar) result = result_matrix->Matrix3();
  vArray2(uchar) values1 = the_image1->GetI2();
  vArray2(uchar) values2 = the_image2->GetI2();

  long row, col;
  for (row = 0; row < min_rows; row++)
  {
    for (col = 0; col < min_cols; col++)
    {
      result[0][row][col] = values1[row][col];
      result[1][row][col] = values2[row][col];
      result[2][row][col] = 0;
    }
  }

  return result_matrix;
}

grayscale_image * vDilateImage(grayscale_image * the_image, uchar foreground_low, 
                            uchar foreground_high)
{
  grayscale_image * result = (grayscale_image *) vCopyImage(the_image);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bottom = rows - 1;
  long right = cols - 1;
  vArray2(uchar) input2 = the_image->Matrix2(0);
  vArray2(uchar) output2 = result->Matrix2(0);

  long row, col;
  for (row = 1; row < bottom; row++)
  {
    vArray(uchar) input = input2[row];
    vArray(uchar) output = output2[row];
    vArray(uchar) top = output2[row-1];
    vArray(uchar) bottom = output2[row+1];

    for (col = 1; col < right; col++)
    {
      if ((input[col] >= foreground_low) &&
          (input[col] <= foreground_high))
      {
        output[col-1] = foreground_high;
        output[col] = foreground_high;
        output[col+1] = foreground_high;
        top[col] = foreground_high;
        bottom[col] = foreground_high;
      }
    }
  }
  return result;
}

color_image * vNormalizeIntensity(color_image * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  color_image * result = new color_image(rows, cols);
  vArray(uchar) R1 = the_image->GetR();
  vArray(uchar) G1 = the_image->GetG();
  vArray(uchar) B1 = the_image->GetB();
  vArray(uchar) R2 = result->GetR();
  vArray(uchar) G2 = result->GetG();
  vArray(uchar) B2 = result->GetB();

  float factor = 255.0;

  long i;
  long size = the_image->Size();
  for (i = 0; i < size; i++)
  {
    float r = R1[i];
    float g = G1[i];
    float b = B1[i];
    float sum = (r+g+b)/factor;
    if (sum == 0)
    {
      R2[i] = 0;
      G2[i] = 0;
      B2[i] = 0;
    }

    else
    {
      R2[i] = (uchar) vRound(r/sum);
      G2[i] = (uchar) vRound(g/sum);
      B2[i] = (uchar) vRound(b/sum);
    }
  }

  return result;
}


void vPixelGradient(grayscale_image * the_image, long row, long col, 
                      double & horizontal, double & vertical)
{
  vArray2(uchar) values = the_image->Matrix2(0);
  vPixelGradient2(values, row, col, horizontal, vertical);
}


// In the result, every pixel receives the angle 
// (between 0 and PI) representing the direction of 
// least change at that pixel.
v3dMatrix<double> * vEdgeOrientations(grayscale_image * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  v3dMatrix<double> * result_image = new v3dMatrix<double>(rows, cols, 1);
  vArray2(double) result = result_image->Matrix2(0);
  vArray2(uchar) intensities = the_image->Matrix2(0);

  long row, col;
  // Write zeros at the borders, because edge 
  // orientation is not defined for them.
  for (row = 0; row < rows; row++)
  {
    result[row][0] = 0;
    result[row][cols-1] = 0;
  }

  for (col = 0; col < cols; col++)
  {
    result[0][col] = 0;
    result[rows-1][col] = 0;
  }

  for (row = 1; row < rows-1; row++)
  {
    for (col = 1; col < cols-1; col++)
    {
      result[row][col] = vPixelOrientation(intensities, row, col);
    }
  }

  return result_image;
}


// In the result, every pixel receives the angle 
// (between 0 and 2*PI) representing the direction of 
// greatest increase (gradient) at that pixel
v3dMatrix<double> * vEdgeOrientations2(grayscale_image * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  v3dMatrix<double> * result_image = new v3dMatrix<double>(rows, cols, 1);
  vArray2(double) result = result_image->Matrix2(0);
  vArray2(uchar) intensities = the_image->Matrix2(0);

  long row, col;
  // Write zeros at the borders, because edge 
  // orientation is not defined for them.
  for (row = 0; row < rows; row++)
  {
    result[row][0] = 0;
    result[row][cols-1] = 0;
  }

  for (col = 0; col < cols; col++)
  {
    result[0][col] = 0;
    result[rows-1][col] = 0;
  }

  for (row = 1; row < rows-1; row++)
  {
    for (col = 1; col < cols-1; col++)
    {
      result[row][col] = vPixelOrientation2(intensities, row, col);
    }
  }

  return result_image;
}


// In the result, every pixel receives the angle 
// (between 0 and 2*PI) representing the direction of 
// greatest increase (gradient) at that pixel
v3dMatrix<double> * vEdgeOrientations2(v3dMatrix<double> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  v3dMatrix<double> * result_image = new v3dMatrix<double>(rows, cols, 1);
  vArray2(double) result = result_image->Matrix2(0);
  vArray2(double) intensities = the_image->Matrix2(0);

  long row, col;
  // Write zeros at the borders, because edge 
  // orientation is not defined for them.
  for (row = 0; row < rows; row++)
  {
    result[row][0] = 0;
    result[row][cols-1] = 0;
  }

  for (col = 0; col < cols; col++)
  {
    result[0][col] = 0;
    result[rows-1][col] = 0;
  }

  for (row = 1; row < rows-1; row++)
  {
    for (col = 1; col < cols-1; col++)
    {
      result[row][col] = vPixelOrientation2(intensities, row, col);
    }
  }

  return result_image;
}


// returns an image that is a copy of the input image except that
// the color specified by old_red, old_green, old_blue 
// is replaced by the color specified by new_red, new_green, new_blue.
color_image * function_copy_replace_color(color_image * input, uchar old_red, 
                                          uchar old_green, uchar old_blue,
                                          uchar new_red, uchar new_green, uchar new_blue)
{
  if (input == 0) return 0;
  color_image * result = input->ToRGBCopy();
  function_replace_color(result, old_red, old_green, old_blue,
                         new_red, new_green, new_blue);
  
  return result;
}


// modifies the argument image by replacing
// the color specified by old_red, old_green, old_blue 
// by the color specified by new_red, new_green, new_blue.
long function_replace_color(color_image * image, uchar old_red, 
                            uchar old_green, uchar old_blue,
                            uchar new_red, uchar new_green, uchar new_blue)
{
  if (image == 0) return 0;
  vArray(uchar) R = image->GetR();
  vArray(uchar) G = image->GetG();
  vArray(uchar) B = image->GetB();

  long size = image->Size();

  long i;
  for (i = 0; i < size; i++)
  {
    if ((R[i] == old_red) && (G[i] == old_green) && (B[i] == old_blue))
    {
      R[i] = new_red;
      G[i] = new_green;
      B[i] = new_blue;
    }
  }
  
  return 1;
}



// this function selects all pixels with intensity greater than
// small_threshold whose 8-connected component contains at least one pixel
// with intensity greater than big_threshold. An example where such a function
// would be useful is for canny edge detection.
float_matrix double_threshold(float_matrix image, float small_threshold, 
                              float big_threshold)
{
  uchar_image * first_threshold = image.threshold(small_threshold);
  vector<class_label> labels_vector;
  long_matrix labels = vConnectedComponents8(first_threshold, & labels_vector);
  function_delete(first_threshold);

  long labels_number = labels_vector.size();
  float_matrix label_maxima = function_label_maxima(image, labels, &labels_vector);

  long vertical = image.vertical();
  long horizontal = image.horizontal();
  float_matrix result(vertical, horizontal);
  long size = result.length();

  long counter;
  for (counter = 0; counter < size; counter++)
  {
    long label = labels(counter);
    float maximum = label_maxima(label);
    if (maximum >= big_threshold)
    {
      result(counter) = image(counter);
    }
    else
    {
      result(counter) = 0;
    }
  }

  return result;
}


float_matrix function_label_maxima(float_matrix image, long_matrix labels,
                                   std::vector<class_label> * labels_vector)
{
  long label_number = labels_vector->size();
  float_matrix result(1, label_number);
  short_matrix labels_visited (1, label_number);
  function_enter_value(& labels_visited, (short) 0);
  
  long size = image.length();
  long counter;
  for (counter = 0; counter < size; counter++)
  {
    long label = labels(counter);
    float value = image(counter);
    if (labels_visited(label) == 0)
    {
      labels_visited(label) = 1;
      result(label) = value;
    }
    else
    {
      if (value > result(label))
      {
        result(label) = value;
      }
    }
  }

  return result;
}


// selects pixels that belong to a connected component with more pixels than
// the specified threshold.
float_image connected_size_threshold(float_image & image, long threshold)
{
  uchar_image * white_pixels = function_choose_value(& image, (float) 0);
  vInverseImage(white_pixels);

  vector<class_label> labels_vector;
  long_matrix labels = vConnectedComponents8(white_pixels, & labels_vector);
  function_delete(white_pixels);

  long channels = image.channels();
  long vertical = image.vertical();
  long horizontal = image.horizontal();
  float_image result(vertical, horizontal, channels);

  long size = labels.length();
  long counter;
  for (counter = 0; counter < size; counter++)
  {
    long label = labels(counter);
    long size = labels_vector[label].count;
    long channel;

    if (size >= threshold)
    {
      for (channel = 0; channel < channels; channel++)
      {
        result(channel, counter) = image(channel, counter);
      }
    }
    else
    {
      for (channel = 0; channel < channels; channel++)
      {
        result(channel, counter) = 0;
      }
    }
  }

  return result;
}
