#include "stdafx.h"

#include "vplatform.h"
#ifdef VASSILIS_SGI_PLATFORM
#define VASSILIS_PROCESS_FILE
#endif // VASSILIS_SGI_PLATFORM

#ifdef VASSILIS_PROCESS_FILE
#undef VASSILIS_PROCESS_FILE


#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#include <algorithm.h>
#else 
#include <iostream>
#include <algorithm>
#endif
#include <math.h>


#include "basics/matrix.h"
#include "basics/algebra.h"
#include "basics/simple_algo_templates.h"

#include "basics/definitions.h"

using namespace std;



template<class type>
static void S_Helper_UndirectedEdges(vArray2(type) band1, 
                                     vArray2(type) ** band2,
                                     long rows, long cols);

template<class type>
static type S_MaxBand(vArray(type) band, long size);

template<class type>
static void S_MultiplyBand(vArray(type) source, vArray(type) target, 
			   long size, double factor);

template<class type1, class type2>
static void S_Convolve(vArray2(type1) source, vArray2(double) result, 
		       long rows, long cols, 
		       v3dMatrix<type2> * filter);


// Start of implementation of class vTemplateCaller1
template<class type>
vMatrix<double> *
vTemplateCaller1<type>::BoundedRankHistogram(general_image * input, long number_of_bins,
                                              double lower, double upper)
{
  if (input->TypeName() != type_name) return 0;
  return vBoundedRankHistogram((v3dMatrix<type> *) input, number_of_bins,
                                lower, upper);
}


template<class type>
v3dMatrix<double> * 
vTemplateCaller1<type>::ComputeMotion(general_image * the_image1, 
                                       general_image * the_image2,
                                       general_image * the_image3)
{
  if (the_image1->TypeName() != type_name) return 0;
  if (the_image2->TypeName() != type_name) return 0;
  if (the_image3->TypeName() != type_name) return 0;
  return vComputeMotion((v3dMatrix<type> *) the_image1,
                         (v3dMatrix<type> *) the_image2,
                         (v3dMatrix<type> *) the_image3);
}


template<class type>
vMatrix<double> *
vTemplateCaller1<type>::PixelHistogram(general_image * input, 
                                        long number_of_bins)
{
  if (input->TypeName() != type_name) return 0;
  return vPixelHistogram((v3dMatrix<type> *) input, number_of_bins);
}


template<class type>
v3dMatrix<double> *
vTemplateCaller1<type>::ImageDifference(general_image * the_image1, 
                                         general_image * the_image2)
{
  if (the_image1->TypeName() != type_name) return 0;
  if (the_image2->TypeName() != type_name) return 0;
  return vImageDifference((v3dMatrix<type> *) the_image1,
                           (v3dMatrix<type> *) the_image2);
}


template<class type>
general_image * 
vTemplateCaller1<type>::MaxNeighbors(general_image * the_image)
{
  if (the_image->TypeName() != type_name) return 0;
  return vMaxNeighbors((v3dMatrix<type> *) the_image);
}

template<class type>
vMatrix<double> *
vTemplateCaller1<type>::PixelBoundedHistogram(general_image * input, long number_of_bins,
                                               double lower, double upper)
{
  if (input->TypeName() != type_name) return 0;
  return vPixelBoundedHistogram((v3dMatrix<type> *) input, number_of_bins,
                                 lower, upper);
}


template<class type>
general_image * 
vTemplateCaller1<type>::PyramidImage(general_image * input, long times)
{
  if (input->TypeName() != type_name) return 0;
  return vPyramidImage((v3dMatrix<type> *) input, times);
}


template<class type>
general_image * 
vTemplateCaller1<type>::TranslateImage(general_image * the_image, long translation_rows, 
                                        long translation_cols)
{
  if (the_image->TypeName() != type_name) return 0;
  return function_translate_image((v3dMatrix<type> *) the_image, translation_rows,
                          translation_cols);
}


template<class type>
general_image * 
vTemplateCaller1<type>::ScaleImage(general_image * the_image, double scaling,
                                    long new_center_row, long new_center_col)
{
  if (the_image->TypeName() != type_name) return 0;
  return vScaleImage4((v3dMatrix<type> *) the_image, scaling,
                       new_center_row, new_center_col);
}


template<class type>
general_image * 
vTemplateCaller1<type>::ScaleImage(general_image * the_image, double scaling,
                                    long new_center_row, long new_center_col,
                                    long new_rows, long new_cols)
{
  if (the_image->TypeName() != type_name) return 0;
  return vScaleImage6((v3dMatrix<type> *) the_image, scaling,
                       new_center_row, new_center_col,
                       new_rows, new_cols);
}


template<class type>
general_image * 
vTemplateCaller1<type>::ResizeImage(general_image * the_image, 
                                     long new_rows, long new_cols)
{
  if (the_image->TypeName() != type_name) return 0;
  return vResizeImage((v3dMatrix<type> *) the_image, new_rows, new_cols);
}


template<class type>
general_image * 
vTemplateCaller1<type>::ScaleBoundingBox(general_image * the_image, long new_box_size,
                                          long new_center_row, long new_center_col)
{
  if (the_image->TypeName() != type_name) return 0;
  return vScaleBoundingBox4((v3dMatrix<type> *) the_image, new_box_size,
                             new_center_row, new_center_col);
}


template<class type>
general_image * 
vTemplateCaller1<type>::ScaleBoundingBox(general_image * the_image, long new_box_size,
                                          long new_center_row, long new_center_col,
                                          long new_rows, long new_cols)
{
  if (the_image->TypeName() != type_name) return 0;
  return vScaleBoundingBox6((v3dMatrix<type> *) the_image, new_box_size,
                             new_center_row, new_center_col,
                             new_rows, new_cols);
}


template<class type>
vBoundingBox
vTemplateCaller1<type>::BinaryBoundingBox(general_image * input)
{
  if (input->TypeName() != type_name) return vBoundingBox();
  return vBinaryBoundingBox((v3dMatrix<type> *) input);
}


template<class type>
general_image *
vTemplateCaller1<type>::Magnify(general_image * input, 
                                 long center_row, long center_col,
                                 long output_rows, long output_cols, 
                                 long factor)
{
  if (input->TypeName() != type_name) return 0;
  return vMagnify((v3dMatrix<type> *) input, center_row, center_col,
                   output_rows, output_cols, factor);
}


template<class type> color_image *
vTemplateCaller1<type>::CircularIntensity(general_image * input, 
                                           double low, double high, double ignore)
{
  if (input->TypeName() != type_name) return 0;
  return vCircularIntensity((v3dMatrix<type> *) input, low, high, ignore);
}

template<class type> general_image *
vTemplateCaller1<type>::SelectRange(general_image * input, long band, 
                                     double low, double high)
{
  if (input->TypeName() != type_name) return 0;
  return vSelectRange((v3dMatrix<type> *) input, band, (type) low, (type) high);
}


// End of implementation of class vTemplateCaller1



// Pads the image with zeros at the borders.
template<class type>
v3dMatrix<type> * vPad(v3dMatrix<type> * image, long left, long right, 
			  long top, long bottom)
{
  if ((left < 0) || (right < 0) || (top < 0) || (bottom < 0)) return 0;
  long new_rows = image->Rows() + top + bottom;
  long new_cols = image->Cols() + left + right;
  long bands = image->Bands();
  v3dMatrix<type> * padded = new v3dMatrix<type>(new_rows, new_cols, 
						     bands);
  vArray2(type) I1;
  vArray2(type) I2;

  long row, col, i;
  for (i = 0; i < bands; i++)
  {
    I1 = image->Matrix2(i);
    I2 = padded->Matrix2(i);

    for (row = 0; row < new_rows; row++)
      for (col = 0; col < new_cols; col++)
	I2[row][col] = 0;

    for (row = 0; row < image->Rows(); row++)
      for (col = 0; col < image->Cols(); col++)
	I2[row+top][col+left] = I1[row][col];
  }
  return padded;
}


// The result image has a top left portion equal to the image,
// and zeros at the bottom and right portion so that its size 
// is new_rows x new_cols.
template<class type>
v3dMatrix<type> * vPad(v3dMatrix<type> * image, long new_rows, long new_cols)
{
  long rows = image->Rows();
  long cols = image->Cols();
  long left = 0;
  long right = Max(new_cols - cols, (long) 0);
  long top = 0;
  long bottom = Max(new_rows - rows, (long) 0);
  return vPad(image, left, right, top, bottom);
}


// The result image has a center portion equal to image,
// and it is padded at the top, bottom, left and right
// so that it has the size specified by new_rows and new_cols.
template<class type>
v3dMatrix<type> * vPadCentered(v3dMatrix<type> * image, 
                                  long new_rows, long new_cols)
{
  long rows = image->Rows();
  long cols = image->Cols();
  if ((rows > new_rows) || (cols > new_cols)) return 0;
  long row_diff = new_rows - rows;
  long col_diff = new_cols - cols;
  long left = col_diff / 2;
  long right = col_diff - left;
  long top = row_diff / 2;
  long bottom = row_diff - top;
  return vPad(image, left, right, top, bottom);
}


// Returns the convolution of the image with the filter. The result has
// the same size as the input image.
template<class type1, class type2>
v3dMatrix<double> * function_convolve(v3dMatrix<type1> * image, 
				                         v3dMatrix<type2> * filter)
{
  assert(image->Rows() >= filter->Rows());
  assert(image->Cols() >= filter->Rows());
  assert(filter->Bands() == 1);
  long bands = image->Bands();

  long horizontal_pad, vertical_pad;
  horizontal_pad = filter->Cols() / 2;
  vertical_pad = filter->Rows() / 2;

  v3dMatrix<type1> * padded = vPad(image, horizontal_pad, horizontal_pad,
				     vertical_pad, vertical_pad);
  v3dMatrix<double> * result = new v3dMatrix<double>(image->Rows(), 
							 image->Cols(),
							 bands);
  long i;
  for (i = 0; i < bands; i++)
    S_Convolve(padded->Matrix2(i), result->Matrix2(i), 
	       image->Rows(), image->Cols(), filter);

  vdelete(padded);
  return result;
}


template<class type1, class type2>
static void S_Convolve(vArray2(type1) source, vArray2(double) result, 
		                   long rows, long cols, v3dMatrix<type2> * filter)
{
  long row, col, i, j;
  assert(filter->Rows() % 2 == 1);
  assert(filter->Cols() % 2 == 1);
  long di = filter->Rows() / 2;
  long dj = filter->Cols() / 2;
  vArray2(type2) I2 = filter->Matrix2(0);

  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
    {
      double temp = 0;
      for (i = -di; i <= di; i++)
	for (j = -dj; j <= dj; j++)
	  temp += source[row+di+i][col+dj+j] * I2[di+i][dj+j];
      result[row][col] = temp;
    }
}


// Returns the convolution of the image with the filter. The result has
// the same size as the input image.
template<class type1, class type2>
v3dMatrix<float> * function_convolve_float(v3dMatrix<type1> * image, 
				                         v3dMatrix<type2> * filter)
{
  assert(image->Rows() >= filter->Rows());
  assert(image->Cols() >= filter->Cols());
  assert(filter->Bands() == 1);
  long bands = image->Bands();

  long horizontal_pad, vertical_pad;
  horizontal_pad = filter->Cols() / 2;
  vertical_pad = filter->Rows() / 2;

  v3dMatrix<type1> * padded = vPad(image, horizontal_pad, horizontal_pad,
				     vertical_pad, vertical_pad);
  v3dMatrix<float> * result = new v3dMatrix<float>(image->Rows(), 
							 image->Cols(),
							 bands);
  long i;
  for (i = 0; i < bands; i++)
    S_Convolve_float(padded->Matrix2(i), result->Matrix2(i), 
	       image->Rows(), image->Cols(), filter);

  vdelete(padded);
  return result;
}


// Returns the convolution of the image with the filter. The result has
// the same size as the input image.
template<class type1, class type2>
vMatrix<float> function_convolve_float(vMatrix<type1> image, 
				                               vMatrix<type2> filter)
{
  assert(image.Rows() >= filter.Rows());
  assert(image.Cols() >= filter.Cols());

  long horizontal_pad, vertical_pad;
  horizontal_pad = filter.Cols() / 2;
  vertical_pad = filter.Rows() / 2;

  v3dMatrix<type1> * padded = vPad(&image, horizontal_pad, horizontal_pad,
				                           vertical_pad, vertical_pad);
  vMatrix<float> result(image.Rows(), image.Cols());

  S_Convolve_float(padded->Matrix2(0), result.Matrix2(), 
            	     image.Rows(), image.Cols(), &filter);

  vdelete(padded);
  return result;
}


// Returns the convolution of the image with the filter. The result has
// the same size as the input image. This is an optimized version,
// no padding is done.
template<class type1, class type2>
vMatrix<float> function_convolve_float2(vMatrix<type1> image, 
				                                vMatrix<type2> filter)
{
  assert(image.Rows() >= filter.Rows());
  assert(image.Cols() >= filter.Cols());
  const long filter_vertical = filter.vertical();
  const long filter_horizontal = filter.horizontal();
  const long half_vertical = filter_vertical / 2;
  const long half_horizontal = filter_horizontal / 2;

  vMatrix<float> result(image.Rows(), image.Cols());
  enter_border_value(& result, (float) 0, function_maximum(half_vertical, half_horizontal));

  if (filter_vertical == 1)
  {
    convolve_float_horizontal(image.Matrix2(), result.Matrix2(), 
                    	        image.Rows(), image.Cols(), filter.Matrix(), filter_horizontal);
  }
  else if (filter_horizontal == 1)
  {
    convolve_float_vertical(image.Matrix2(), result.Matrix2(), 
                    	      image.Rows(), image.Cols(), filter.Matrix(), filter_vertical);
  }
  else
  {
    S_Convolve_float2(image.Matrix2(), result.Matrix2(), 
            	        image.Rows(), image.Cols(), &filter);
  }

  return result;
}


template<class type1, class type2>
static void convolve_float_horizontal(vArray2(type1) source, vArray2(float) result, 
		                                 long rows, long cols, class_pointer(type2) filter, long filter_size)
{
  long row, col, j;
  assert(filter_size % 2 == 1);
  long half_size = filter_size / 2;
  long limit = cols - filter_size + 1;

  for (row = half_size; row < rows - half_size; row++)
  {
    type1 * source_vertical = source[row];
    float * result_vertical = result[row]+ half_size;
    for (col = 0; col < limit; col++)
    {
      float temp = 0.0f;
  	  for (j = 0; j < filter_size; j++)
      {
        temp += (float) (source_vertical[col+j] * filter[j]);
      }
      result_vertical[col] = temp;
    }
  }
}


template<class type1, class type2>
static void convolve_float_vertical(vArray2(type1) source, vArray2(float) result, 
		                                long rows, long cols, class_pointer(type2) filter, long filter_size)
{
  long row, col, j;
  assert(filter_size % 2 == 1);
  long half_size = filter_size / 2;
  long vertical_limit = rows - filter_size + 1;
  long horizontal_limit = cols - half_size;

  for (row = 0; row < vertical_limit; row++)
  {
    type1 * source_vertical = source[row];
    float * result_vertical = result[row + half_size];
    for (col = half_size; col < horizontal_limit; col++)
    {
      float temp = 0.0f;
  	  for (j = 0; j < filter_size; j++)
      {
        temp += (float) (source[row+j][col] * filter[j]);
      }
      result_vertical[col] = temp;
    }
  }
}


template<class type1, class type2>
static void S_Convolve_float(vArray2(type1) source, vArray2(float) result, 
		                   long rows, long cols, v3dMatrix<type2> * filter)
{
  long row, col, i, j;
  assert(filter->Rows() % 2 == 1);
  assert(filter->Cols() % 2 == 1);
  long di = filter->Rows() / 2;
  long dj = filter->Cols() / 2;
  vArray2(type2) I2 = filter->Matrix2(0);

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      float temp = 0;
      for (i = -di; i <= di; i++)
      {
  	    for (j = -dj; j <= dj; j++)
        {
          temp += source[row+di+i][col+dj+j] * I2[di+i][dj+j];
        }
      }
      result[row][col] = temp;
    }
  }
}


// optimized version of S_Convolve_float. we should use this version when 
// the original image has not been padded
template<class type1, class type2>
static void S_Convolve_float2(vArray2(type1) source, vArray2(float) result, 
		                   long rows, long cols, v3dMatrix<type2> * filter)
{
  exit_error("\nerror: S_Convolve_float2 has not been implemented yet\n");
  long row, col, i, j;
  assert(filter->Rows() % 2 == 1);
  assert(filter->Cols() % 2 == 1);
  long di = filter->Rows() / 2;
  long dj = filter->Cols() / 2;
  vArray2(type2) I2 = filter->Matrix2(0);

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      float temp = 0;
      for (i = -di; i <= di; i++)
      {
  	    for (j = -dj; j <= dj; j++)
        {
          temp += source[row+di+i][col+dj+j] * I2[di+i][dj+j];
        }
      }
      result[row][col] = temp;
    }
  }
}


template<class type>
v3dMatrix<double> * vComputeMotion(v3dMatrix<type> * image1, 
				      v3dMatrix<type> * image2, 
				      v3dMatrix<type> * image3)
//template<class type>
//v3dMatrix<double> * vComputeMotion(general_image * in_image1, general_image * in_image2,
//                                      general_image * in_image3)
{
//  v3dMatrix<type> * image1 = (v3dMatrix<type> *) in_image1;
//  v3dMatrix<type> * image2 = (v3dMatrix<type> *) in_image2;
//  v3dMatrix<type> * image3 = (v3dMatrix<type> *) in_image3;

  v3dMatrix<double> * temp1 = vImageDifference(image1, image2);
  v3dMatrix<double> * temp2 = vImageDifference(image2, image3);
  v3dMatrix<double> * result = function_image_minimum(temp1, temp2);
  vdelete(temp1);
  vdelete(temp2);
  return result;
}



template<class type>
v3dMatrix<double> * vImageDifference(v3dMatrix<type> * the_image1, 
					v3dMatrix<type> * the_image2)
{
  long rows = the_image1->Rows();
  long cols = the_image1->Cols();
  long bands = the_image1->Bands();

  if (the_image2->Rows() != rows) return 0;
  if (the_image2->Cols() != cols) return 0;
  if (the_image2->Bands() != bands) return 0;
  
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, bands);
  
  long i, band;
  vArray(type) I1;
  vArray(type) I2;
  vArray(double) I;
  
  for (band = 0; band < bands; band++)
  {
    I1 = the_image1->Matrix(band);
    I2 = the_image2->Matrix(band);
    I = result->Matrix(band);

    for (i = 0; i < rows * cols; i++)
    {
      double diff = I1[i] - I2[i];
      I[i] = (type) abs((long)diff);
    }
  }

  return result;
}


// Blurs an image using a 3x3 Gaussian filter
template<class type>
v3dMatrix<double> * vBlur(v3dMatrix<type> * the_image)
{
  // Create the filter that we should blur the input with before resampling
  vMatrix<double> filter = vMatrix<double>(0, 2, 0, 2);
  filter.WriteRow(0, 0.0625, 0.125, 0.0625);
  filter.WriteRow(1, 0.1250, 0.250, 0.1250);
  filter.WriteRow(2, 0.0625, 0.125, 0.0625);

  // Blur the input
  v3dMatrix<double> * blurred = function_convolve(the_image, &filter);
  return blurred;
}


// Blurs an image using a filter_rows x filter_cols uniform filter
// (uniform meaning that all values in the filter are identical).
template<class type>
v3dMatrix<double> * vBlurUniformly(v3dMatrix<type> * the_image, 
			                                long filter_rows, long filter_cols)
{
  v3dMatrix<double> * hor_filter = new v3dMatrix<double>(1, filter_cols,1);
  v3dMatrix<double> * ver_filter = new v3dMatrix<double>(filter_rows, 1,1);
  vWriteValue(hor_filter, 1.0 / (double) filter_cols);
  vWriteValue(ver_filter, 1.0 / (double) filter_rows);

  v3dMatrix<double> * temp = function_convolve(the_image, hor_filter);
  v3dMatrix<double> * blurred = function_convolve(temp, ver_filter);
  vdelete(hor_filter);
  vdelete(ver_filter);
  vdelete(temp);
  return blurred;
}


// This function creates a half-resolution version of the input by blurring and
// subsampling. If the number of rows (or cols) is odd, the last row (or col) 
// is dropped.
template<class type1>
v3dMatrix<type1> * vHalfScale(v3dMatrix<type1> * input)
{
  // Blur the input
  v3dMatrix<double> * blurred = vBlur(input);

  // Convert blurred image to the same type as the input image
  v3dMatrix<type1> * typed_blurred = new v3dMatrix<type1>(blurred);
  vdelete(blurred);

  // Do the resampling
  long rows = typed_blurred->Rows();
  long cols = typed_blurred->Cols();
  assert(rows == input->Rows());
  assert(cols = input->Cols());
  v3dMatrix<type1> * result;
  result = (v3dMatrix<type1> *) typed_blurred->Resample2(0, 0, rows, cols, 2);
  
  vdelete(typed_blurred);
  return result;
}


// This function scales down an image by half (in each dimension) repeatedly, 
// by calling vHalfScale, as many times as the second argument specifies.
template<class type1>
v3dMatrix<type1> * vPyramidImage(v3dMatrix<type1> * input, long times)
{
  if (times < 0) return 0;
  if (times == 0) return (v3dMatrix<type1> *) input->Copy();
  long i;
  // Do the first downsampling
  v3dMatrix<type1> * current = vHalfScale(input);
  v3dMatrix<type1> * next = 0;
  // Now loop, each time downsampling and deleting the current image, and
  // making it equal to the downsampled version
  for (i = 1; i < times; i++)
  {
    next = vHalfScale(current);
    vdelete(current);
    current = next;
  }
  return current;
}



// This function creates and returns a histogram (with the specified
// number of bins) where each bin shows the number of pixels corresponding
// to the bin. The histogram is a vMatrix object, that has one row
// per bin, and each row has an entry for the lower bound of the range
// of the bin and an entry for the number of pixels.
template<class type>
vMatrix<double> *
vPixelHistogram(v3dMatrix<type> * input, long number_of_bins)
{
  if (number_of_bins <= 1) return 0;
  vMatrix<double> * histogram = new vMatrix<double>(0, number_of_bins - 1, 0, 3);
  vArray2(double) bins = histogram->Matrix2();

  double max_value = function_image_maximum(input);
  double min_value = function_image_minimum(input);
  double range = max_value - min_value;
  // We make the step such that the last bin has max_value as its
  // lower bound. This ensures that max_value will not be too 
  // large for any bin, and also when we print the histogram the
  // lower bound of the last bin shows us the max value.
  double step = range / (number_of_bins - 1);

  // Write the lower bounds of each bin in column 0, and initialize
  // column 1 to 0.
  long i;
  for (i = 0; i < number_of_bins; i++)
  {
    bins[i][0] = min_value + step * i;
    bins[i][1] = 0;
  }

  vArray(type) matrix = input->Matrix();
  long size = input->AllBandSize();
  for (i = 0; i < size; i++)
  {
    double entry = matrix[i];
    double double_bin = (entry - min_value) / step;
    long bin = (long) double_bin;
    bins[bin][1]++;
  }

  vCumulativeCalculations(histogram);
  return histogram;
}


// This function is like vPixelHistogram, but it only considers values
// in the interval [lower, upper] as it builds the histogram.
template<class type>
vMatrix<double> *
vPixelBoundedHistogram(v3dMatrix<type> * input, long number_of_bins,
                        double lower, double upper)
{
  if (number_of_bins <= 1) return 0;
  type temp_min_value = 0;
  type temp_max_value = 0;
  ushort success = function_image_minimumMaxBounded(input, &temp_min_value, &temp_max_value,
                                        lower, upper);
  if (success == 0) return 0;
  double min_value = temp_min_value;
  double max_value = temp_max_value;
  
  vMatrix<double> * histogram = new vMatrix<double>(0, number_of_bins - 1, 0, 3);
  vArray2(double) bins = histogram->Matrix2();

  double range = max_value - min_value;
  // We make the step such that the last bin has max_value as its
  // lower bound. This ensures that max_value will not be too 
  // large for any bin, and also when we print the histogram the
  // lower bound of the last bin shows us the max value.
  double step = range / (number_of_bins - 1);

  // Write the lower bounds of each bin in column 0, and initialize
  // column 1 to 0.
  long i;
  for (i = 0; i < number_of_bins; i++)
  {
    bins[i][0] = min_value + step * i;
    bins[i][1] = 0;
  }

  // Go through the image and through each pixel in the [min_value, max_value]
  // interval into the appropriate bin
  vArray(type) matrix = input->Matrix();
  long size = input->AllBandSize();
  for (i = 0; i < size; i++)
  {
    double entry = matrix[i];
    if (entry < lower) continue;
    if (entry > upper) continue;
    double double_bin = (entry - min_value) / step;
    long bin = (long) double_bin;
    bins[bin][1]++;
  }

  // Put cumulative statistics into columns 2 and 3 (third and fourth column).
  vCumulativeCalculations(histogram);
  return histogram;
}


// vBoundedRankHistogram returns a histogram of the values of all pixels
// in input that are in the [lower, ..., upper] interval. However, it is
// different than vPixelBoundedHistogram, in that here bins don't correspond
// to equal-spaced intervals, but are defined in such a way that the first bin
// gets the smallest 1/number_of_bins fraction of pixels, the second bin gets
// the second smallest, and so on. The returned histogram has 4 columns and
// one row per bin. The first entry in the row is the lower bound of the bin.
// The second entry is the number of pixels in the bin. The third is a 
// cumulative value summing up all bins from the first up to the current.
// The fourth is a cumulative value summing up all bins from the last down
// to the current.
template<class type>
vMatrix<double> *
vBoundedRankHistogram(v3dMatrix<type> * input, long number_of_bins,
                       double lower, double upper)
{
  if (number_of_bins <= 1) return 0;
  type temp_min_value = 0;
  type temp_max_value = 0;
  ushort success = function_image_minimumMaxBounded(input, &temp_min_value, &temp_max_value,
                                        lower, upper);
  if (success == 0) return 0;
  double min_value = temp_min_value;
  double max_value = temp_max_value;
  

  // Collect all values in matrix that are within the [lower, upper] interval.
  vArray(type) matrix = input->Matrix();
  long size = input->AllBandSize();
  vArray(type) values = vnew(type, size+1);
  long counter = 0;
  long i;
  for (i = 0; i < size; i++)
  {
    type entry = matrix[i];
    if (entry < lower) continue;
    if (entry > upper) continue;
    values[counter++] = entry;
  }

  std::sort(&(values[0]), &(values[counter]), less<type>());

  // Create the histogram.
  vMatrix<double> * histogram = new vMatrix<double>(0, number_of_bins - 1, 0, 3);
  vArray2(double) bins = histogram->Matrix2();

  double range = counter;
  // We make the step such that the last bin has max_value as its
  // lower bound. This ensures that max_value will not be too 
  // large for any bin, and also when we print the histogram the
  // lower bound of the last bin shows us the max value.
  double step = range / (number_of_bins - 1);

  // Go through the values array and, every time we count more than step * bin
  // pixels put the number of pixels into the next bin.
  long previous_value = 0;
  bins[0][0] = values[0];
  for (i = 1; i < number_of_bins - 1; i++)
  {
    // Figure out the corresponding index into the values array.
    double double_index = step * (double) i;
    long index = (long) double_index;
    // bins[i][0] is the lower bound of bin i.
    bins[i][0] = values[index];
    // All values between previous_value and index-1 go to bin i - 1.
    bins[i-1][1] = index - previous_value;
    previous_value = index;
  }
  bins[number_of_bins-1][0] = max_value;
  bins[number_of_bins-2][1] = counter - previous_value - 1;
  bins[number_of_bins-1][1] = 1;
  vdelete2(values);

  vCumulativeCalculations(histogram);
  return histogram;
}


// We find the top-most, bottom-most, left-most and right-most coordinate
// values of non-zero pixels in input, and record them into the result's
// member variables. In a multi-band image, a pixel is non-zero if any
// of its bands is non-zero.
template<class type>
vBoundingBox
vBinaryBoundingBox(v3dMatrix<type> * input)
{
  vBoundingBox result;
  long size = input->Size();
  assert(size != 0);
  if (size == 0) return result;

  long top, bottom, left, right;
  long rows = input->Rows();
  long cols = input->Cols();
  long bands = input->Bands();
  vArray3(type) matrix3 = input->Matrix3();
  top = rows;
  bottom = -1;
  left = cols;
  right = -1;

  long band, row, col;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) matrix2 = matrix3[band];
    for (row = 0; row < rows; row++)
    {
      vArray(type) matrix = matrix2[row];
      for (col = 0; col < cols; col++)
      {
        type value = matrix[col];
        if (value == 0) continue;
        if (row < top) top = row;
        if (row > bottom) bottom = row;
        if (col < left) left = col;
        if (col > right) right = col;
      }
    }
  }

  result.top = top;
  result.bottom = bottom;
  result.left = left;
  result.right = right;
  return result;
}


// We return an image in which pixel i,j has the same value as
// the color_image pixel i-translation_rows, j-translation_cols. If
// there is no such pixels within the value of the_image, pixel i,j
// is zero in the result.
template<class type>
v3dMatrix<type> *
function_translate_image(v3dMatrix<type> * the_image, long translation_rows, 
                 long translation_cols)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) in_matrix3 = the_image->Matrix3();
  vArray3(type) out_matrix3 = result->Matrix3();
  
  long row, col, band;
  for (band = 0; band < bands; band++)
  {
    vArray2(type) in_matrix2 = in_matrix3[band];
    vArray2(type) out_matrix2 = out_matrix3[band];
    for (row = 0; row < rows; row++)
    {
      long in_row = row - translation_rows;
      ushort valid_row;
      vArray(type) in_matrix;
      if ((in_row >= 0) && (in_row < rows)) 
      {
        valid_row = 1;
        in_matrix = in_matrix2[in_row];
      }
      else 
      {
        valid_row = 0;
        in_matrix = vZero(type);
      }

      vArray(type) out_matrix = out_matrix2[row];
      for (col = 0; col < cols; col++)
      {
        if (valid_row == 0)
        {
          out_matrix[col] = 0;
          continue;
        }

        long in_col = col - translation_cols;
        if (in_col < 0) 
        {
          out_matrix[col] = 0;
          continue;
        }
        if (in_col >= cols)
        {
          out_matrix[col] = 0;
          continue;
        }
        out_matrix[col] = in_matrix[in_col];
      }
    }
  }
  return result;
}


// We return an image which is the result of scaling color_image by
// multiplying the distance of everything from 
// (new_center_row, new_center_col)
// We don't want non-zero values to get lost in the scaling. This 
// could happen when we scale down. To prevent it, we use two methods,
// one for scaling up (scaling >= 1) and one for scaling down
// (0 < scaling < 1). 
//
// Note: No interpolation is done. Because of that, this function
// is best for binary images, when we really want to ensure
// that intensity values don't change. I defined this function
// in order to apply it to edge images, and take care of 
// the following two cases:
//
// - If the result image is 
// larger, we don't want connected edge components in color_image
// to become disconenected. Therefore for every output pixel
// we should find a corresponding input pixel and copy its
// value. vScaleImageUp does that.
//
// - If the result image is smaller, we want all edge pixels
// in color_image to be represented in the output image. So,
// for each color_image pixel that is non-zero we find its corresponding
// position in the output image and copy its value there.
// The auxiliary function vScaleImageDown does that.
template<class type>
v3dMatrix<type> * 
vScaleImage4(v3dMatrix<type> * the_image, double scaling,
              long new_center_row, long new_center_col)
{
  if (scaling <= 0) 
  {
    return 0;
  }
  if (scaling < 1) 
  {
    return vScaleImageDown(the_image, scaling,
                            new_center_row, new_center_col,
                            the_image->Rows(), the_image->Cols());
  }
  else 
  {
    return vScaleImageUp(the_image, scaling, new_center_row, new_center_col,
                          the_image->Rows(), the_image->Cols());
  }
}


template<class type>
v3dMatrix<type> * 
vScaleImage6(v3dMatrix<type> * the_image, double scaling,
              long new_center_row, long new_center_col,
              long new_rows, long new_cols)
{
  if (scaling <= 0) 
  {
    return 0;
  }
  if (scaling < 1) 
  {
    return vScaleImageDown(the_image, scaling,
                            new_center_row, new_center_col,
                            new_rows, new_cols);
  }
  else 
  {
    return vScaleImageUp(the_image, scaling, new_center_row, new_center_col,
                          new_rows, new_cols);
  }
}


// First we zero out the result, and then for every edge in the input 
// image we find the corresponding pixel in the result image and set it
// to 255. Each pixel is moved so that its new distance from the 
// specified center is the old distance * scaling.
//
// Also see comments on vScaleImage4.
template<class type>
v3dMatrix<type> * 
vScaleImageDown(v3dMatrix<type> * the_image, double scaling,
                 long center_row, long center_col,
                 long new_rows, long new_cols)
{
  if (scaling <= 0) return 0;
  // We do allow scalings above one, in cases where we want each
  // source pixel to map to at most one target pixel.
  //if (scaling > 1) return 0;

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(new_rows, new_cols, bands);
  // Initialize all entries in result to 0.
  vWriteValue(result, (type) 0);

  vArray3(type) in3 = the_image->Matrix3();
  vArray3(type) out3 = result->Matrix3();
  long in_row, in_col;
  for (in_row = 0; in_row < rows; in_row++)
  {
    double row_diff = in_row - center_row;
    double scaled_row_diff = row_diff * scaling;
    long out_row = (long) (center_row + scaled_row_diff);
    
    // if out_row is out of bounds, don't write anything to the
    // output image
    if ((out_row < 0) || (out_row >= new_rows))
    {
      continue;
    }

    for (in_col = 0; in_col < cols; in_col++)
    {
      double col_diff = in_col - center_col;
      double scaled_col_diff = col_diff * scaling;
      long out_col = (long) (center_col + scaled_col_diff);
      if (out_col < 0)
      {
        continue;
      }
      if (out_col >= new_cols)
      {
        continue;
      }
    
      long band;
      for (band = 0; band < bands; band++)
      {
        if (in3[band][in_row][in_col] == 0) continue;
        out3[band][out_row][out_col] = in3[band][in_row][in_col];
      }
    }
  }

  return result;
}


// Once I verify that this works correctly, I should make 
// vScaleImageDown call this function
template<class type>
v3dMatrix<type> * 
vScaleImageDown8(v3dMatrix<type> * the_image, double scaling,
                  long center_row, long center_col,
                  long new_center_row, long new_center_col,
                  long new_rows, long new_cols)
{
//  vPrint("ScaleImageDown8:\n");
//  vPrint("scaling = %lf\n", scaling);
//  vPrint("center = (%li, %li)\n", center_row, center_col);
//  vPrint("new center = (%li, %li)\n", new_center_row, new_center_col);
  if (scaling <= 0) return 0;
  // We do allow scalings above one, in cases where we want each
  // source pixel to map to at most one target pixel.
  //if (scaling > 1) return 0;

  long rows = the_image->Rows();
  long cols = the_image->Cols();
//  vPrint("size = %lix%li\n", rows, cols);
//  vPrint("new_size = %lix%li\n", new_rows, new_cols);
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(new_rows, new_cols, bands);
  // Initialize all entries in result to 0.
  vWriteValue(result, (type) 0);

  vArray3(type) in3 = the_image->Matrix3();
  vArray3(type) out3 = result->Matrix3();
  long in_row, in_col;
  for (in_row = 0; in_row < rows; in_row++)
  {
    double row_diff = in_row - center_row;
    double scaled_row_diff = row_diff * scaling;
    long out_row = (long) (new_center_row + scaled_row_diff);
    
    // if out_row is out of bounds, don't write anything to the
    // output image
    if ((out_row < 0) || (out_row >= new_rows))
    {
      continue;
    }

    for (in_col = 0; in_col < cols; in_col++)
    {
      double col_diff = in_col - center_col;
      double scaled_col_diff = col_diff * scaling;
      long out_col = (long) (new_center_col + scaled_col_diff);
      if (out_col < 0)
      {
        continue;
      }
      if (out_col >= new_cols)
      {
        continue;
      }
    
      long band;
      long zero_flag = 1;

      // Check if we have a non-black pixel (only non-black pixels are 
      // copied to the output image, to make sure that connected components
      // of non-zero pixels map to connected components of non-zero pixels).
      for (band = 0; band < bands; band++)
      {
        if (in3[band][in_row][in_col] != 0) 
        {
          zero_flag = 0;
          break;
        }
      }
      if (zero_flag == 0)
      {
        for (band = 0; band < bands; band++)
        {
          out3[band][out_row][out_col] = in3[band][in_row][in_col];
        }
      }
    }
  }

  return result;
}


// For a pixel i,j in the result image, we find the pixel that it 
// corresponds to in the_image, and put the value of that pixel to i,j.
//
// Also see comments on vScaleImage4.
template<class type>
v3dMatrix<type> * 
vScaleImageUp(v3dMatrix<type> * the_image, double scaling,
               long center_row, long center_col,
               long new_rows, long new_cols)
{
  if (scaling < 1) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(new_rows, new_cols, bands);

  vArray3(type) in3 = the_image->Matrix3();
  vArray3(type) out3 = result->Matrix3();
  long out_row, out_col, band;
  for (out_row = 0; out_row < new_rows; out_row++)
  {
    double row_diff = out_row - center_row;
    double scaled_row_diff = row_diff / scaling;
    long in_row = (long) (center_row + scaled_row_diff);
    
    // if in_row is out of bounds, write zeros to the corresponding out_row
    if ((in_row < 0) || (in_row >= rows))
    {
      for (band = 0; band < bands; band++)
      {
        for (out_col = 0; out_col < new_cols; out_col++)
        {
          out3[band][out_row][out_col] = 0;
        }
      }
      continue;
    }

    for (out_col = 0; out_col < new_cols; out_col++)
    {
      double col_diff = out_col - center_col;
      double scaled_col_diff = col_diff / scaling;
      long in_col = (long) (center_col + scaled_col_diff);
      if (in_col < 0)
      {
        for (band = 0; band < bands; band++)
        {
          out3[band][out_row][out_col] = 0;
        }
        continue;
      }
      if (in_col >= cols)
      {
        for (band = 0; band < bands; band++)
        {
          out3[band][out_row][out_col] = 0;
        }
        continue;
      }
      for (band = 0; band < bands; band++)
      {
        out3[band][out_row][out_col] = in3[band][in_row][in_col];
      }
    }
  }

  return result;
}


// Like vScaleImage4 and vScaleImage6, this function is best
// for binary images, like edge images. If the output image
// is larger, we go through all output pixels and get their
// values from the corresponding input pixels. If the output
// image is smaller, we go through all input pixels and
// write their values on the corresponding output pixels.
// 
// We have three concerns that this function addresses:
// - Input pixels that are non_zero should be represented
// in the output
// - Connected non-zero components in the input should map to connected
// non-zero components in the output;
// - The last row and column of the input should not be lost due
// to rounding errors, and same for last row and column of output.
//
// The first two concerns are addressed by first writing zeros onto
// the result, and then incrementing current row and current col
// (which are different for the input and the output)
// in such a way that no pixel will be skipped in the input,
// and no pixel will be skipped in the output. 
template<class type>
v3dMatrix<type> *
vResizeImage(v3dMatrix<type> * the_image, long new_rows, long new_cols)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(new_rows, new_cols, bands);
  vArray3(type) in3 = the_image->Matrix3();
  vArray3(type) out3 = result->Matrix3();
  double in_drow, in_dcol, out_drow, out_dcol;
  long in_row, in_col, out_row, out_col;
  double in_row_step, out_row_step, in_col_step, out_col_step;
  double row_ratio = ((double) new_rows) / (double) rows;
  double col_ratio = ((double) new_cols) / (double) cols;
  long row_steps, col_steps;
  vWriteValue(result, (type) 0);

  if (new_rows > rows)
  {
    out_row_step = 1.0;
    in_row_step = out_row_step / row_ratio;
    row_steps = new_rows;
  }
  else
  {
    in_row_step = 1.0;
    out_row_step = in_row_step * row_ratio;
    row_steps = rows;
  }

  if (new_cols > cols)
  {
    out_col_step = 1.0;
    in_col_step = out_col_step / col_ratio;
    col_steps = new_cols;
  }
  else
  {
    in_col_step = 1.0;
    out_col_step = in_col_step * col_ratio;
    col_steps = cols;
  }

  in_drow = out_drow = (double) 0.0;
  in_row = (long) in_drow;
  out_row = (long) out_drow;
  for (long i = 0; i < row_steps; i++)
  {
    assert(out_row < new_rows);
    in_dcol = out_dcol = (double) 0.0;
    in_col = (long) in_dcol;
    out_col = (long) out_dcol;
    for (long j = 0; j < col_steps; j++)
    {
      assert(out_col < new_cols);
      long band;

      long zero_flag = 1;
      // Check if it is a pixel with zeros in all bands
      for (band = 0; band < bands; band++)
      {
        type value = in3[band][in_row][in_col];
        if (value != 0)
        {
          zero_flag = 0;
          break;
        }
      }

      // if the pixel is non-zero, write it on the output pixel.
      if (zero_flag == 0)
      {
        for (band = 0; band < bands; band++)
        {
          type value = in3[band][in_row][in_col];
          out3[band][out_row][out_col] = value;
        }
      }
      in_dcol += in_col_step;
      out_dcol += out_col_step;
      in_col = (long) in_dcol;
      out_col = (long) out_dcol;
    }
    assert((in_col == cols) || (in_col == cols - 1));
    assert((out_col == new_cols) || (out_col == new_cols - 1));
    in_drow += in_row_step;
    out_drow += out_row_step;
    in_row = (long) in_drow;
    out_row = (long) out_drow;
  }
  assert((in_row == rows) || (in_row == rows - 1));
  assert((out_row == new_rows) || (out_row == new_rows - 1));
  return result;
}


// this function is better than vResizeImage when we want to 
// preserve image quality. Here we use bilinear interpolation.
// What we don't do (and we should) is, when the result image
// is smaller, first blur color_image with a gaussian of the right
// scale.
template<class type>
v3dMatrix<type> *
vResizeImagef(v3dMatrix<type> * the_image, long new_rows, long new_cols)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(new_rows, new_cols, bands);
  vArray3(type) in3 = the_image->Matrix3();
  vArray3(type) out3 = result->Matrix3();
  float in_drow, in_dcol, out_drow, out_dcol;
  long out_row, out_col;
  float in_row_step, out_row_step, in_col_step, out_col_step;
  float row_ratio = ((float) new_rows) / (float) rows;
  float col_ratio = ((float) new_cols) / (float) cols;
  long row_steps, col_steps;
  vWriteValue(result, (type) 0);

  out_row_step = 1.0;
  in_row_step = out_row_step / row_ratio;
  row_steps = new_rows;

  out_col_step = 1.0;
  in_col_step = out_col_step / col_ratio;
  col_steps = new_cols;

  in_drow = out_drow = (float) 0.0;
  out_row = (long) out_drow;
  for (long i = 0; i < row_steps; i++)
  {
    if (out_row >= new_rows)
    {
      exit_error("error: out_row >= new_rows\n");
    }
    in_dcol = out_dcol = (float) 0.0;
    for (long j = 0; j < col_steps; j++)
    {
      out_col = (long) out_dcol;
      if (out_col >= new_cols)
      {
        exit_error("error: out_col >= new_cols\n");
      }
      long band;

      for (band = 0; band < bands; band++)
      {
        long in_row = (long) in_drow;
        long in_col = (long) in_dcol;
        float top_left = in3[band][in_row][in_col];
        float top_right, bottom_left, bottom_right;
        if ((in_row + 1 == rows) || (in_col + 1 == cols))
        {
          top_right = top_left;
          bottom_left = top_left;
          bottom_right = top_left;
        }
        else
        {
          top_right = in3[band][in_row][in_col+1];
          bottom_left = in3[band][in_row+1][in_col];
          bottom_right = in3[band][in_row+1][in_col+1];
        }
        float value = vBilinearInterpolation(in_dcol - in_col, in_drow - in_row, top_left, top_right, 
                                              bottom_left, bottom_right);
        out3[band][out_row][out_col] = (type) value;
      }
      in_dcol += in_col_step;
      out_dcol += out_col_step;
    }
    in_drow += in_row_step;
    out_drow += out_row_step;
    out_row = (long) out_drow;
  }
  assert((out_row == new_rows) || (out_row == new_rows - 1));
  return result;
}



// The result image has the size of the_image, bounding box (of its non-zero
// values) of size (i.e. longest side) new_box_size and center of 
// the bounding box specified by new_center_row and new_center_col
template<class type>
v3dMatrix<type> * 
vScaleBoundingBox4(v3dMatrix<type> * the_image, long new_box_size,
                   long new_center_row, long new_center_col)
{
  return vScaleBoundingBox6(the_image, new_box_size, new_center_row, 
                             new_center_col, the_image->Rows(), the_image->Cols());
}


// The result image has the size of the_image, bounding box (of its non-zero
// values) of size (i.e. longest side) new_box_size and center of 
// the bounding box specified by new_center_row and new_center_col
template<class type>
v3dMatrix<type> * 
vScaleBoundingBoxf4(v3dMatrix<type> * the_image, long new_box_size,
                    long new_center_row, long new_center_col)
{
  return vScaleBoundingBoxf6(the_image, new_box_size, new_center_row, 
                              new_center_col, the_image->Rows(), the_image->Cols());
}


// Like vScaleBoundingBox4, but here the result image has the size
// specified by new_rows and new_cols.
// There is a bug. The arguments new_center_row and new_center_col
// are ignored.
template<class type>
v3dMatrix<type> * 
vScaleBoundingBox6(v3dMatrix<type> * the_image, long new_box_size,
                    long new_center_row, long new_center_col,
                    long new_rows, long new_cols)
{
  vBoundingBox box = vBinaryBoundingBox(the_image);
  long height = box.bottom - box.top + 1;
  long width = box.right - box.left + 1;
  long long_box_size = Max(width, height);

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result;

  // If long_box_size is 0 or less it means the binary input image is all 
  // zeros.
  if (long_box_size <= 0) 
  {
    result = new v3dMatrix<type>(new_rows, new_cols, bands);
    vWriteValue(result, (type) 0);
    return result;
  }
  
  // Figure out translation
  long current_center_row = (box.bottom + box.top) / 2;
  long current_center_col = (box.right + box.left) / 2;

  v3dMatrix<type> * box_image = 
    vCopyRectangle(the_image, box.left, box.right, box.top, box.bottom);
  
  long box_width = box.right - box.left + 1;
  long box_height = box.bottom - box.top + 1;
  long new_box_width, new_box_height;
  if (box_width > box_height)
  {
    new_box_width = new_box_size;
    double new_box_height_d = 
      ((double) box_height) * ((double) new_box_width) / ((double) box_width);
    new_box_height = (long) new_box_height_d;
  }
  else
  {
    new_box_height = new_box_size;
    double new_box_width_d = 
      ((double) box_width) * ((double) new_box_height) / ((double) box_height);
    new_box_width = (long) new_box_width_d;
  }

  v3dMatrix<type> * resized_box_image = 
    vResizeImage(box_image, new_box_height, new_box_width);

  result = vPadCentered(resized_box_image, new_rows, new_cols);
  vdelete(box_image);
  vdelete(resized_box_image);
  return result;
}


// Like vScaleBoundingBox4, but here the result image has the size
// specified by new_rows and new_cols.
// There is a bug. The arguments new_center_row and new_center_col
// are ignored.
template<class type>
v3dMatrix<type> * 
vScaleBoundingBoxf6(v3dMatrix<type> * the_image, long new_box_size,
                     long new_center_row, long new_center_col,
                     long new_rows, long new_cols)
{
  vBoundingBox box = vBinaryBoundingBox(the_image);
  long height = box.bottom - box.top + 1;
  long width = box.right - box.left + 1;
  long long_box_size = Max(width, height);

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result;

  // If long_box_size is 0 or less it means the binary input image is all 
  // zeros.
  if (long_box_size <= 0) 
  {
    result = new v3dMatrix<type>(new_rows, new_cols, bands);
    vWriteValue(result, (type) 0);
    return result;
  }
  
  // Figure out translation
  long current_center_row = (box.bottom + box.top) / 2;
  long current_center_col = (box.right + box.left) / 2;

  v3dMatrix<type> * box_image = 
    vCopyRectangle(the_image, box.left, box.right, box.top, box.bottom);
  
  long box_width = box.right - box.left + 1;
  long box_height = box.bottom - box.top + 1;
  long new_box_width, new_box_height;
  if (box_width > box_height)
  {
    new_box_width = new_box_size;
    double new_box_height_d = 
      ((double) box_height) * ((double) new_box_width) / ((double) box_width);
    new_box_height = (long) new_box_height_d;
  }
  else
  {
    new_box_height = new_box_size;
    double new_box_width_d = 
      ((double) box_width) * ((double) new_box_height) / ((double) box_height);
    new_box_width = (long) new_box_width_d;
  }

  v3dMatrix<type> * resized_box_image = 
    vResizeImagef(box_image, new_box_height, new_box_width);

  result = vPadCentered(resized_box_image, new_rows, new_cols);
  vdelete(box_image);
  vdelete(resized_box_image);
  return result;
}


// Like vScaleBoundingBox4, but here the result image has the size
// specified by new_rows and new_cols.
template<class type>
v3dMatrix<type> * 
vSparseScaleBoundingBox6(v3dMatrix<type> * the_image, long new_box_size,
                          long new_center_row, long new_center_col,
                          long new_rows, long new_cols)
{
  vBoundingBox box = vBinaryBoundingBox(the_image);
  return vSparseScaleBoundingBox7(the_image, box, new_box_size, new_center_row,
                                   new_center_col, new_rows, new_cols);
}


// Like vScaleBoundingBox6, but here we pass in the bounding box as an
// argument, as opposed to obtaining it directly from color_image. This is 
// useful in the case where we want to scale an image A exactly the same way we 
// scaled another image B, and their bounding boxes are different. By passing
// A as the_image, and the bounding box of B, we make sure that A is scaled the
// way b was scaled, assuming the other parameters are the same.
template<class type>
v3dMatrix<type> * 
vSparseScaleBoundingBox7(v3dMatrix<type> * the_image, vBoundingBox box,
                          long new_box_size,
                          long new_center_row, long new_center_col,
                          long new_rows, long new_cols)
{
  long height = box.bottom - box.top + 1;
  long width = box.right - box.left + 1;
  long long_box_size = Max(width, height);

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  v3dMatrix<type> * result;

  // If long_box_size is 0 or less it means the binary input image is all 
  // zeros.
  if (long_box_size <= 0) 
  {
    result = new v3dMatrix<type>(rows, cols, bands);
    vWriteValue(result, (type) 0);
    return result;
  }
  
  // Figure out translation
  long current_center_row = (box.bottom + box.top) / 2;
  long current_center_col = (box.right + box.left) / 2;

  long box_width = box.right - box.left + 1;
  long box_height = box.bottom - box.top + 1;
  long new_box_width, new_box_height;
  if (box_width > box_height)
  {
    new_box_width = new_box_size;
    double new_box_height_d = 
      ((double) box_height) * ((double) new_box_width) / ((double) box_width);
    new_box_height = (long) new_box_height_d;
  }
  else
  {
    new_box_height = new_box_size;
    double new_box_width_d = 
      ((double) box_width) * ((double) new_box_height) / ((double) box_height);
    new_box_width = (long) new_box_width_d;
  }

  double scaling = ((double) new_box_height) / (double) box_height;
  v3dMatrix<type> * resized_box_image = 
    vScaleImageDown8(the_image, scaling, current_center_row, current_center_col,
                      new_center_row, new_center_col, new_rows, new_cols);

  return resized_box_image;
}


// Fill a connected component (4-connected) whose value is 0 and contains
// the point (row, col) with the specified value.
template<class type>
long vFillRegion(v3dMatrix<type> * the_image, long in_row, long in_col, type value)
{
  // We assume the matrix has one band.
  if (the_image->Bands() != 1) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vArray2(type) matrix2 = the_image->Matrix2(0);
  deque<long> rows_queue;
  deque<long> cols_queue;
  rows_queue.push_back(in_row);
  cols_queue.push_back(in_col); 
  while(rows_queue.size() != 0)
  {
    long row = rows_queue.front();
    long col = cols_queue.front();
    rows_queue.pop_front();
    cols_queue.pop_front();
    if (row < 0) continue;
    if (col < 0) continue;
    if (row >= rows) continue;
    if (col >= cols) continue;
    if (matrix2[row][col] != 0) continue;
      
    matrix2[row][col] = value;
    // Add 4 neighbors to the queue
    rows_queue.push_back(row-1);
    cols_queue.push_back(col);
    rows_queue.push_back(row+1);
    cols_queue.push_back(col);
    rows_queue.push_back(row);
    cols_queue.push_back(col-1);
    rows_queue.push_back(row);
    cols_queue.push_back(col+1);
  }
  return 1;
} 


// Returns an output image which is the result of magnifying a portion 
// of the input image, centered at (center_row, center_col), by a
// factor specified by the factor argument. The corners of the region that
// will be magnified in the input image can be deduced by 
// (output_rows, output_cols) and the magnifying factor.
template<class type>
v3dMatrix<type> * vMagnify(v3dMatrix<type> * source,
                              long center_row, long center_col, 
                              long output_rows, long output_cols,
                              long factor)
{
  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();

  long input_rows = output_rows / factor;
  long input_cols = output_cols / factor;

  long input_top = center_row - input_rows / 2;  
  long input_left = center_col - input_cols / 2;

  v3dMatrix<type> * result = new v3dMatrix<type>(output_rows, output_cols,
                                                     bands);
  long in_row, in_col, out_row, out_col, band;
  vArray3(type) input3 = source->Matrix3();
  vArray3(type) output3 = result->Matrix3();
  
  for (band = 0; band < bands; band++)
  {
    vArray2(type) input2 = input3[band];
    vArray2(type) output2 = output3[band];

    for (out_row = 0; out_row < output_rows; out_row++)
    {
      in_row = input_top + out_row / factor;
      for (out_col = 0; out_col < output_cols; out_col++)
      {
        in_col = input_left + out_col / factor;
        if (source->check_bounds(in_row, in_col) == 0)
        {
          output2[out_row][out_col] = 0;
        }
        else
        {
          output2[out_row][out_col] = input2[in_row][in_col];
        }
      }
    }
  }

  return result;
}


// I wrote this function to visualize edge orientations, but it can
// be used for other visualizations as well. With edge orientations,
// orientation 0 is the same as orientation 2*PI. If I represent
// each orientation value with a proportional intensity level, 
// pixels that look visually very different (with intensity values
// very low and very high) have values that, interpreted as orientations,
// are very similar. vCircularIntensity takes in an image whose values
// are in the range [low, high] and which are circular, in the sense
// that low and high are the same value conceptualy, and it creates
// a color image that lets us visualize the input image. In the color
// image, low is mapped to red. As we move towards high, values change
// to green, and they become green at 1/3 of the way towards high. Then
// they start changing towards blue, and they become blue at 2/3 of the
// way towards high. Then they start changing towards red again. This
// way similar pixel values correspond to conceptually similar 
// values in the input image.
// Intensity values equal to ignore are mapped to black.
template<class type> color_image *
vCircularIntensity(v3dMatrix<type> * input, double low, double high, double ignore)
{
  if (high < low) return 0;
  if (input->Bands() != 1) return 0;
  type image_min = 0, image_max = 0;
  function_image_minimumMax(input, &image_min, &image_max);
  if ((image_min < low) && (image_min != ignore)) return 0;
  if ((image_max > high) && (image_max != ignore)) return 0;
  double period = high - low;
  double third = period / 3;

  vArray(type) values = input->Matrix();
  long rows = input->Rows();
  long cols = input->Cols();
  long size = input->Size();
  color_image * result = new color_image(rows, cols);
  vArray(uchar) R = result->GetR();
  vArray(uchar) G = result->GetG();
  vArray(uchar) B = result->GetB();

  long i;
  for (i = 0; i < size; i++)
  {
    double value = values[i];
    if (value == ignore)
    {
      R[i] = 0;
      G[i] = 0;
      B[i] = 0;
      continue;
    }
    double offset = value - low;
    long category = (long) (offset / third);
    double red = 0, green = 0, blue = 0;
    if (category == 0)
    {
      green = (offset / third) * 255.0;
      red = 255.0 - green;
    }
    else if (category == 1)
    {
      offset = offset - third;
      blue = (offset / third) * 255.0;
      green = 255.0 - blue;
    }
    else
    {
      offset = offset - 2*third;
      red = (offset / third) * 255.0;
      blue = 255.0 - red;
    }

    if (red < 0) red = 0;
    if (green < 0) green = 0;
    if (blue < 0) blue = 0;
    if (red > 255) red = 255;
    if (green > 255) green = 255;
    if (blue > 255) blue = 255;

    R[i] = (uchar) red;
    G[i] = (uchar) green;
    B[i] = (uchar) blue;
  }
  return result;
}


///////////////////////////////////////////////////
//                                               //
//        Definitions for vMatrixWrap           //
//                                               //
///////////////////////////////////////////////////


template<class type>
grayscale_image * vMatrixWrap<type>::Binary()
{
  v3dMatrix<type> * the_image1 = (v3dMatrix<type> *) the_image;
  long bands = the_image1->Bands();
  long rows = the_image1->Rows();
  long cols = the_image1->Cols();
  grayscale_image * result = new grayscale_image(rows, cols);
  vArray(uchar) binary = result->Matrix();
  long size = rows * cols;
    long i;

  if (bands == 1)
  {
    vArray(type) source = the_image1->Matrix();
    for (i = 0; i < size; i++)
    {
      if (source[i] == 0) binary[i] = 0;
      else binary[i] = 255;
    }
  }

  else
  {
    vArray2(type) source = the_image1->Matrix2();
    for (i = 0; i < size; i++)
    {
      long band;
      binary[i] = 0;
      for (band = 0; band < bands; band++)
      {
        if (source[band][i] != 0)
        {
          binary[i] = 255;
          break;
        }
      }
    }
  }

  return result;
}


//template<class type>
//v3dMatrix<type> * vMatrixWrap<type>::NormalizeMEC1_a(long radius)
//{
//  vCircleLong mec = vMinEnclosingCircle(the_image);
//  double factor = ((double) radius) / (double) mec.radius;

//  return vScaleImage4(the_image, factor, mec.center_row, mec.center_col);  
//}


//template<class type>
//v3dMatrix<type> *
//vMatrixWrap<type>::NormalizeMEC3_a(long center_row, long center_col,
//                                    long radius)
//{
//  vCircleLong mec = vMinEnclosingCircle(the_image);
//  double factor = ((double) radius) / (double) mec.radius;
//  long rows = the_image->Rows();
//  long cols = the_image->Cols();
//
//  long temp_center_row = (rows - 1) / 2;
//  long temp_center_col = (cols - 1) / 2;
//  long translation_rows = temp_center_row - mec.center_row;
//  long translation_cols = temp_center_col - mec.center_col;
//  // First we center the non-zero region at the center of the image,
//  // then we scale, and finally we move the non-zero region so that
//  // it is centered at center_row and center_col. The reason
//  // we do not just do two operation (centering at center_row and center_col
//  // preceded or followed by scaling) is that sometimes non-zero pixels 
//  // fall out of bounds either in the translation or in the scaling.
//  // By first centering at the image's center row and center col we
//  // reduce the occurrences of non-zero pixels falling out of bounds.
//  v3dMatrix<type> * translated = function_translate_image(the_image, translation_rows,
//                                                    translation_cols);
//  v3dMatrix<type> * scaled = vScaleImage4(translated, factor,
//                                             temp_center_row, temp_center_col);
//
//  translation_rows = center_row - temp_center_row;
//  translation_cols = center_col - temp_center_col;
//  v3dMatrix<type> * result = function_translate_image(scaled, translation_rows, 
//                                                translation_cols);
//
//  vdelete(translated);
//  vdelete(scaled);
//  return result;
//}


template<class type>
v3dMatrix<type> *
vMatrixWrap<type>::Warp_a(v3dMatrix<short> * sources)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  if (sources == 0) return 0;
  vArray2(short) source_rows = sources->Matrix2(0);
  vArray2(short) source_cols = sources->Matrix2(1);

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) result3 = result->Matrix3();
  vArray3(type) source3 = the_image->Matrix3();
  vWriteValue(result, (type) 0);

  long row, col;

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long source_row = source_rows[row][col];
      long source_col = source_cols[row][col];
      if (!result->check_bounds(source_row, source_col)) continue;

      long band;
      for (band = 0; band < bands; band++)
      {
        result3[band][row][col] = source3[band][source_row][source_col];
      }
    }
  }

  return result;
}


// result[row][col] = color_image[row][col] when color_image[row][col] is
// a valid uchar value (between 0 and 255). Negative values 
// go to 0, values > 255 go to 255.
template<class type>
v3dMatrix<uchar> * vCroppedUchar(v3dMatrix<type> * the_image)
{
  if (the_image == 0) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  long size = rows * cols * bands;

  v3dMatrix<uchar> * cropped = new v3dMatrix<uchar>(rows, cols, bands);
  vArray(type) sources = the_image->Matrix(0);
  vArray(uchar) targets = cropped->Matrix(0);
  long i;
  for (i = 0; i < size; i++)
  {
    long value = vRound(sources[i]);
    if (value > 255) targets[i] = (uchar) 255;
    else if (value < 0) targets[i] = (uchar) 0;
    else targets[i] = (uchar) value;
  }

  return cropped;
}




///////////////////////////////////////////////////
//                                               //
//              End of vMatrixWrap              //
//                                               //
///////////////////////////////////////////////////


// If the images are equal return 1. Else, if they have different dimensions,
// return 0. If they have same dimensions but differ in at least one pixel,
// return 0 - the number of pixels where the images differ.
template<class type>
long color_imageEqual(v3dMatrix<type> * the_image1, v3dMatrix<type> * the_image2)
{
  if (the_image1->Rows() != the_image2->Rows()) return 0;
  if (the_image1->Cols() != the_image2->Cols()) return 0;
  if (the_image1->Bands() != the_image2->Bands()) return 0;

  long size = the_image1->Size();
  long bands = the_image1->Bands();
  long counter = 0;

  vArray2(type) values1 = the_image1->Matrix2();
  vArray2(type) values2 = the_image2->Matrix2();
  long i, band;
  for (i = 0; i < size; i++)
  {
    long equal = 1;
    for (band = 0; band < bands; band++)
    {
      if (values1[band][i] != values2[band][i])
      {
        equal = 0;
        break;
      }
    }
    if (equal == 0) counter++;
  }

  if (counter == 0) return 1;
  else return -counter;
}


// The result is white (255) where color_image has the given color,
// black (0) otherwise.
template<class type>
grayscale_image * vSelectColor(v3dMatrix<type> * the_image, 
                            std::vector<type> * color)
{
  long bands = the_image->Bands();
  if (color->size() != bands) return 0;

  vArray2(type) colors = the_image->Matrix2();
  long size = the_image->Size();
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  grayscale_image * result = new grayscale_image(rows, cols);
  vArray(uchar) selection = result->Matrix();

  long i;
  for (i = 0; i < size; i++)
  {
    long equal = 255;
    long band;
    for (band = 0; band < bands; band++)
    {
      if (colors[band][i] != (*color)[band])
      {
        equal = 0;
        break;
      }
    }
    selection[i] = (uchar) equal;
  }
  return result;
}



template<class type>
static void S_Helper_UndirectedEdges(vArray2(type) band1, vArray2(type) band2,
				     long rows, long cols)
{
  // First take care of sides
  long row, col;
  
  for (row = 0; row < rows; row++)
    {
      band2[row][0] = 0;
      band2[row][cols-1] = 0;
    }

  for (col = 1; col < cols-1; col++)
    {
      band2[0][col] = 0;
      band2[rows-1][col] = 0;
    }

  // Then calculate the interior
  type value;

  for (row = 1; row < rows-1; row++)
    for (col = 1; col < cols-1; col++)
    {
	    value = band1[row][col];
	    band2[row][col] = (abs(band1[row-1][col] - value) +
			       abs(band1[row+1][col] - value) +
			       abs(band1[row][col-1] - value) +
			       abs(band1[row][col+1] - value)) / 4;
    }
}


template<class type>
v3dMatrix<type> * vUndirectedEdges(v3dMatrix<type> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);

  long i;
  for (i = 0; i < bands; i++)
  {
    vArray2(type) values1 = the_image->Matrix3()[i];
    vArray2(type) values2 = result->Matrix3()[i];
    S_Helper_UndirectedEdges(values1, values2, rows, cols);
  }
  return result;
}


// This function calculates for every pixel and each band the maximum
// distance between the pixel and one of its four neighbors
template<class type>
v3dMatrix<type> * vMaxNeighbors(v3dMatrix<type> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bottom = rows - 1;
  long right = cols - 1;

  // Create the result image
  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, 1);
  // Write zeros at the borders
  vWriteBorderValue(result, (type) 0);
  vArray2(type) matrix1 = the_image->Matrix2(0);
  vArray2(type) matrix2 = result->Matrix2(0);

  long row, col;
  for (row = 1; row < bottom; row++)
  {
    for (col = 1; col < right; col++)
    {
      type center = matrix1[row][col];
      // Take care of the case where the pixel is a background pixel
      float max_distance = 0;
      type values[4];
      values[0] = matrix1[row-1][col];
      values[1] = matrix1[row+1][col];
      values[2] = matrix1[row][col-1];
      values[3] = matrix1[row][col+1];
      
      long i;
      for (i = 0; i < 4; i++)
      {
        float distance = vAbs((float) values[i] - (float) center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      matrix2[row][col] = (type) max_distance;
    }
  }
  return result;
}


template<class type>
v3dMatrix<type>  * vAmplifyImage(v3dMatrix<type> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = rows * cols;

  // Find largest value
  long bands = the_image->Bands();
  if (bands == 0) assert(0);

  type biggest = S_MaxBand(the_image->Matrix2()[0], size);
  long i;
  for (i = 1; i < bands; i++)
  {
    type temp = S_MaxBand(the_image->Matrix(i), size);
    if (temp > biggest) biggest = temp;
  }

  type max_possible = vTypeMax(biggest);
  type factor = max_possible / biggest;
  v3dMatrix<type> * result = vMultiplyImage(the_image, factor);
  return result;
}


template<class type>
v3dMatrix<type> * vMultiplyImage(v3dMatrix<type> * the_image, 
				    double factor)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = rows * cols;
  long bands = the_image->Bands();

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  long i;
  for (i = 0; i < bands; i++)
    S_MultiplyBand(the_image->Matrix(i), result->Matrix(i), size, factor);

  return result;
}

template<class type>
static type S_MaxBand(vArray(type) band, long size)
{
  type biggest = vTypeMax(band[0]);
  int i;
  for (i = 0; i < size; i++)
    {
      if (band[i] > biggest) biggest = band[i];
    }
  return biggest;
}

template<class type>
static void S_MultiplyBand(vArray(type) source, vArray(type) target, 
			   long size, double factor)
{
  double result;
  type max1 = vTypeMax(source[0]);

  long i;
  for (i = 0; i < size; i++)
    {
      result = factor * source[i];
      if (result > max1) result = max1;
      target[i] = (type) result;
    }
}


template<class type>
double vInnerProduct(v3dMatrix<type> * fimage1, 
		      long top1, long left1,
		      v3dMatrix<type> * fimage2, 
		      long top2, long left2,
		      long rows, long cols,
		      long band)
{
  long row, col;
  double result = 0.0;
  long i1, j1, i2, j2;

  vArray2(type) I1 = fimage1->Matrix2(band);
  vArray2(type) I2 = fimage2->Matrix2(band);
  
  for (row = 0; row < rows; row++)
    for (col = 0; col < cols; col++)
      {
	i1 = top1 + row;
	j1 = left1 + col;
	i2 = top2 + row;
	j2 = left2 + col;
	result = result + I1[i1][j1] * I2[i2][j2];
      }
  return result;
}


template<class type>
double color_imageMagnitude(v3dMatrix<type> * fimage, long band)
{
  long size = fimage->Size();
  vArray(type) I = fimage->Matrix(band);
  long i;
  double result = 0;
  double value;

  for (i = 0; i < size; i++)
    {
      value = I[i];
      result += value * value;
    }

  result = sqrt(result);
  return result;
}

template<class type>
double vCorrelation(v3dMatrix<type> * fimage1, long top1, long left1,
		     v3dMatrix<type> * fimage2, long top2, long left2,
		     long rows, long cols, long band)
{
  double product = vInnerProduct(fimage1, top1, left1,
				  fimage2, top2, left2, rows, cols, band);
  double m1 = color_imageMagnitude(fimage1);
  double m2 = color_imageMagnitude(fimage2);

  double result;
  if (m1 * m2 == 0) result = 1;
  else result = product / (m1 * m2);
  return result;
}


template<class type>
double vInnerProduct(v3dMatrix<type> * fimage1, 
		      v3dMatrix<type> * fimage2, long band)
{
  double result = vInnerProduct(fimage1, 0, 0, fimage2, 0, 0,
				 fimage1->Rows(), fimage1->Cols(), band);
  return result;
}

template<class type>
double vCorrelation(v3dMatrix<type> * fimage1, v3dMatrix<float> * fimage2,
		     long band)
{
  double result = vCorrelation(fimage1, 0, 0, fimage2, 0, 0,
				fimage1->Rows(), fimage1->Cols(), band);
  return result;
}


template<class type>
v3dMatrix<double> * vGradient_X(v3dMatrix<type> * the_image)
{
    long rows, cols, bands;
    rows = the_image->Rows();
    cols = the_image->Cols();
    bands = the_image->Bands();

    v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, bands);
    long i;
    
    for (i = 0; i < bands; i++)
    {
      type ** gi = the_image->Matrix2(i);
      double ** fi = result->Matrix2(i);
      
      long row, col;
      for (row = 0; row < rows; row++)
      {
	fi[row][0] = 0.0;
	fi[row][cols-1] = 0.0;
      }
      
      for (row = 0; row < rows; row++)
	for (col = 1; col < cols-1; col++)
	  fi[row][col] = (gi[row][col+1] - gi[row][col-1]) / 2.0;
    }

    return result;
}

template<class type>
v3dMatrix<double> * vGradient_Y(v3dMatrix<type> * the_image)
{
  long rows, cols, bands;
  rows = the_image->Rows();
  cols = the_image->Cols();
  bands = the_image->Bands();
    
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols);
  long i;
    
  for (i = 0; i < bands; i++)
  {
    type ** gi = the_image->Matrix2(i);
    double ** fi = result->Matrix2(i);

    long row, col;
    for (col = 0; col < cols; col++)
    {
      fi[0][col] = 0.0;
      fi[rows-1][col] = 0.0;
    }

    for (row = 1; row < rows-1; row++)
      for (col = 0; col < cols; col++)
	fi[row][col] = (gi[row+1][col] - gi[row-1][col]) / 2.0;
  }
  return result;
}

template<class type>
v3dMatrix<uchar> * vImageAND(v3dMatrix<type> * image1, 
				v3dMatrix<type> * image2)
{
  long rows, cols, bands;
  rows = image1->Rows();
  cols = image1->Cols();
  bands = image1->Bands();
  assert(rows == image2->Rows());
  assert(cols == image2->Cols());
  assert(bands == image2->Bands());

  v3dMatrix<uchar> * result = new v3dMatrix<uchar>(rows, cols, bands);
  vArray(type) I1;
  vArray(type) I2;
  vArray(uchar) I3;
    
  long size = rows * cols;
  long i, j;
  for (j = 0; j < bands; j++)
  {
    I1 = image1->Matrix(j);
    I2 = image2->Matrix(j);
    I3 = result->Matrix(j);
    for (i = 0; i < size; i++)
    {
      if ((I1[i] != 0) && (I2[i] != 0)) I3[i] = 255;
      else I3[i] = 0;
    }
  }
  return result;
}

    
template<class type>
v3dMatrix<uchar> * vImageOR(v3dMatrix<type> * image1, 
				v3dMatrix<type> * image2)
{
  long rows, cols, bands;
  rows = image1->Rows();
  cols = image1->Cols();
  bands = image1->Bands();
  assert(rows == image2->Rows());
  assert(cols == image2->Cols());
  assert(bands == image2->Bands());

  v3dMatrix<uchar> * result = new v3dMatrix<uchar>(rows, cols, bands);
  type * I1, I2;
  uchar * I3;
    
  long size = rows * cols;
  long i, j;
  for (j = 0; j < bands; j++)
  {
    I1 = image1->Matrix(j);
    I2 = image2->Matrix(j);
    I3 = result->Matrix(j);
    for (i = 0; i < size; i++)
    {
      if ((I1[i] != 0) || (I2[i] != 0)) I3[i] = 255;
      else I3[i] = 0;
    }
  }
  return result;
}

template<class type>
v3dMatrix<uchar> * vImageNOT(v3dMatrix<type> * image1)
{
  long rows, cols, bands;
  rows = image1->Rows();
  cols = image1->Cols();
  bands = image1->Bands();

  v3dMatrix<uchar> * result = new v3dMatrix<uchar>(rows, cols, bands);
  type * I1;
  uchar * I3;
  long size = rows * cols;
  long i, j;

  for (j = 0; j < bands; j++)
  {
    I1 = image1->Matrix(j);
    I3 = result->Matrix(j);

    for (i = 0; i < size; i++)
    {
      if (I1[i] != 0) I3[i] = 0;
      else I3[i] = 255;
    }
  }
  return result;
}


template<class type>
v3dMatrix<type> * function_image_minimum(v3dMatrix<type> * image1, 
			       v3dMatrix<type> * image2)
{
    long rows, cols, bands;
    rows = image1->Rows();
    cols = image1->Cols();
    bands = image1->Bands();
    assert(rows == image2->Rows());
    assert(cols == image2->Cols());
    assert(bands == image2->Bands());

    v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
    vArray(type) I1;
    vArray(type) I2;
    vArray(type) I3;

    long size = rows * cols;
    long i, j;
    for (j = 0; j < bands; j++)
    {
      I1 = image1->Matrix(j);
      I2 = image2->Matrix(j);
      I3 = result->Matrix(j);

      for (i = 0; i < size; i++)
      {
	if (I1[i] < I2[i]) I3[i] = I1[i];
	else I3[i] = I2[i];
      }
    }

    return result;
}


template<class type>
type function_image_median(v3dMatrix<type> * the_image)
{
  const long number_of_bins = 4000;
  type max_value = function_image_maximum(the_image);
  type min_value = function_image_minimum(the_image);
  type range = max_value - min_value + 1;
  double step = range / (double) number_of_bins;
  long counters[number_of_bins];
  long i;
  for (i = 0; i < number_of_bins; i++)
    counters[i] = 0;

  vArray(type) matrix = the_image->Matrix();
  long size = the_image->AllBandSize();
  for (i = 0; i < size; i++)
  {
    type value = matrix[i];
    long bin = (long) ((value - min_value) / step);
    counters[bin]++;
  }
  long counter = 0;
  long half_size = size / 2;
  long median_bin = -1;
  cout << "Min value = " << min_value << endl;
  cout << "Max value = " << max_value << endl;

  for (i = 0; i < number_of_bins; i++)
  {
    counter += counters[i];
    if (counter >= half_size)
    {
      median_bin = i;
      break;
    }
  }
  cout << "Median bin = " << median_bin << endl;
  assert(median_bin != -1);
  type result = min_value + step * median_bin;
  return result;
}
			      

// Returns an image that has a 255 in all pixels that, in the input image,
// have the given value in their first color band.
template<class type1>
grayscale_image * vSelectValue(v3dMatrix<type1> * the_image, type1 value)
{
  long bands = the_image->Bands();
  if (bands != 1) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = the_image->AllBandSize();
  grayscale_image * result = new grayscale_image(rows, cols);
  vArray(type1) source = the_image->Matrix(0);
  vArray(uchar) target = result->Matrix(0);

  long i;
  for (i = 0; i < size; i++)
  {
    if (source[i] == value) target[i] = 255;
    else target[i] = 0;
  }
  return result;
}


template<class type1>
v3dMatrix<type1> * vSelectRange(v3dMatrix<type1> * the_image, long band, 
                                   type1 low, type1 high)
{
  long bands = the_image->Bands();
  if (bands <= band) return 0;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = the_image->AllBandSize();
  v3dMatrix<type1> * result = new v3dMatrix<type1>(rows, cols, bands);
  vArray3(type1) source = the_image->Matrix3();
  vArray3(type1) target = result->Matrix3();

  long row, col, i;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      type1 value = source[band][row][col];
      if ((value >= low) && (value <= high))
      {
        for (i = 0; i < bands; i++)
        {
          target[i][row][col] = source[i][row][col];
        }
      }
      else
      {
        for (i = 0; i < bands; i++)
        {
          target[i][row][col] = 0;
        }
      }
    }
  }
  return result;
}


// This function assumes that it is passed a one-band image. It treats all
// non-zero values as being equal to 1.
template<class type1>
void vCenterOfMass(v3dMatrix<type1> * the_image, 
                    double & center_row, double & center_col)
{
  if (the_image->Bands() != 1)
  {
    v3dMatrix<double> * sums = the_image->SumBandsDouble();
    vCenterOfMass(sums, center_row, center_col);
    vdelete(sums);
    return;
  }
  
  assert(the_image->Bands() == 1);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vArray2(type1) matrix2 = the_image->Matrix2(0);
  long row_sum = 0;
  long col_sum = 0;
  long value_sum = 0;

  long row, col;
  for (row = 0; row < rows; row++)
  {
    vArray(type1) base = matrix2[row];
    for (col = 0; col < cols; col++)
    {
      long value = (long) (base[col] != 0);
      row_sum += row * value;
      col_sum += col * value;
      value_sum += value;
    }
  }

  double row_sum_double = (double) row_sum;
  double col_sum_double = (double) col_sum;
  double value_sum_double = (double) value_sum;
  center_row = row_sum_double / value_sum_double;
  center_col = col_sum_double / value_sum_double;
}


// This function finds the maximum distance between a non-zero point
// in the image (which is assumed to be a one-band image) and the
// point whose coordinates are center_row and center_col.

template<class type1>
double vMaxDistanceFromPoint(v3dMatrix<type1> * the_image,
                              double center_row, double center_col)
{
  // It makes life a bit easier to handle images with more than one
  // bands separately. For those images, we convert them to a one-band
  // image (using v3dMatrix::SumBandsDouble)
  if (the_image->Bands() != 1)
  {
    v3dMatrix<double> * sums = the_image->SumBandsDouble();
    double result = vMaxDistanceFromPoint(sums, center_row, center_col);
    vdelete(sums);
    return result;
  }

  // From this point on we can assume that the image has only one band.
  assert(the_image->Bands() == 1);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vArray2(type1) matrix = the_image->Matrix2(0);
  long row, col;
  double max_distance = 0.0;
  
  for (row = 0; row < rows; row++)
  {
    vArray(type1) base = matrix[row];    
    double row_diff = center_row - row;
    for (col = 0; col < cols; col++)
    {
      if (base[col] == 0) continue;
      double col_diff = center_col - col;
      double distance = row_diff * row_diff + col_diff * col_diff;
      if (distance > max_distance) max_distance = distance;
    }
  }
  return sqrt(max_distance);
}

template<class type1>
v3dMatrix<type1> * vNormalizeImage(v3dMatrix<type1> * the_image,
                                      double new_max_distance)
{
  double rows = the_image->Rows();
  double cols = the_image->Cols();
  double new_center_row = (rows - 1) / 2;
  double new_center_col = (cols - 1) / 2;

  return vNormalizeImage(the_image, new_max_distance, 
                          new_center_row, new_center_col);
  
}


// Create a "normalized" image based on the input argument "color_image". The 
// center of mass of the normalized image is specified by "new_center_row" and
// "new_center_col". The maximum distance of a non-zero point from the 
// center of mass in the result is specified by "new_max_distance". The result
// image has the same size as the input image. To calculate the center of mass
// of an image we treat the image as if it were binary, i.e. pixels with 0 
// intensity have 0 mass and pixels with non-zero intensity have non-zero mass.
// 
// To determine the value of a pixel in the result image we do the following:
// We consider every pixel to be a unit-sized square, whose center is the 
// coordinates of the pixel. We take the pixel in the result image and warp
// it back to the input image. The coordinates (X, Y) 
// we get are probably not integers
// any more. The unit-sized square whose center is those coordinates overlaps
// with (at most) four pixels of the input image. The pixel in the result image
// gets the value that is the weighted avarage of the values in those four pixels 
// in the input image, where the weight of each pixel in the source image is 
// proportional to its overlap with the unit square centered at (X, Y).
template<class type1>
v3dMatrix<type1> * vNormalizeImage(v3dMatrix<type1> * the_image,
                                      double new_max_distance, 
                                      double new_center_row, double new_center_col)
{
  double center_row, center_col;
  // Put the center of mass of the input image into center_row, center_col.
  vCenterOfMass(the_image, center_row, center_col);
  double max_distance = vMaxDistanceFromPoint(the_image, center_row, center_col);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  // normalizer will hold many variables that are needed by the Unnormalize
  // function. By creating a vCoordinateNormalizer class and making Unnormalize
  // a member function of that class we avoid having to pass all those variables 
  // every time we call Unnormalize.
  vCoordinateNormalizer normalizer;
  normalizer.SetCurrentCenter(center_row, center_col);
  normalizer.SetDesiredCenter(new_center_row, new_center_col);
  normalizer.SetCurrentMaxDistance(max_distance);
  normalizer.SetDesiredMaxDistance(new_max_distance);
  normalizer.SetRowRange(0, rows-1);
  normalizer.SetColRange(0, cols-1);
  long * row_indices = normalizer.rows;
  long * col_indices = normalizer.cols;
  double * weights = normalizer.weights;
  long & valid_pixels = normalizer.valid_pixels;


  v3dMatrix<type1> * result = new v3dMatrix<type1>(rows, cols, bands);
  long band;
  for (band = 0; band < bands; band++)
  {
    vArray2(type1) source = the_image->Matrix2(band);
    vArray2(type1) target = result->Matrix2(band);
    long row, col;
    for (row = 0; row < rows; row++)
    {
      for (col = 0; col < cols; col++)
      {
        normalizer.Unnormalize(row, col);
        long pixel;
        type1 value = (type1) 0;
        for (pixel = 0; pixel < valid_pixels; pixel++)
        {
          long source_row = row_indices[pixel];
          long source_col = col_indices[pixel];
          double weight = weights[pixel];
          value += (type1) (source[source_row][source_col] * weight);
        }
        target[row][col] = value;
      }
    }
  }
  return result;
}


template<class type1>
v3dMatrix<type1> * vNormalizeLabeledImage2(v3dMatrix<type1> * the_image,
                                              double new_max_distance)
{
  double rows = the_image->Rows();
  double cols = the_image->Cols();
  double new_center_row = (rows - 1) / 2;
  double new_center_col = (cols - 1) / 2;

  return vNormalizeLabeledImage3(the_image, new_max_distance, 
                                  new_center_row, new_center_col);
  
}


// See comments on vNormalizeLabeledImage for why this function is useful.
template<class type1>
v3dMatrix<type1> * vNormalizeLabeledImage3(v3dMatrix<type1> * the_image,
                                              double new_max_distance, 
                                              double new_center_row, 
                                              double new_center_col)
{
  double center_row, center_col;
  // Put the center of mass of the input image into center_row, center_col.
  vCenterOfMass(the_image, center_row, center_col);
  double max_distance = vMaxDistanceFromPoint(the_image, center_row, center_col);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();

  // normalizer will hold many variables that are needed by the Unnormalize
  // function. By creating a vCoordinateNormalizer class and making Unnormalize
  // a member function of that class we avoid having to pass all those variables 
  // every time we call Unnormalize.
  vCoordinateNormalizer normalizer;
  normalizer.SetCurrentCenter(center_row, center_col);
  normalizer.SetDesiredCenter(new_center_row, new_center_col);
  normalizer.SetCurrentMaxDistance(max_distance);
  normalizer.SetDesiredMaxDistance(new_max_distance);
  normalizer.SetRowRange(0, rows-1);
  normalizer.SetColRange(0, cols-1);
  long * row_indices = normalizer.rows;
  long * col_indices = normalizer.cols;
  double * weights = normalizer.weights;
  long & valid_pixels = normalizer.valid_pixels;


  v3dMatrix<type1> * result = new v3dMatrix<type1>(rows, cols, bands);
  long band;
  long row, col;
  vArray3(type1) source3 = the_image->Matrix3();
  vArray3(type1) target3 = result->Matrix3();
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      normalizer.Unnormalize(row, col);
      long pixel;
      // Find the source pixel with the highest weight and put its value into
      // the target pixel.
      long max_index = -1;
      double max_weight = 0;
      for (pixel = 0; pixel < valid_pixels; pixel++)
      {
        double weight = weights[pixel];
        if (weight > max_weight)
        {
          max_index = pixel;
          max_weight = weight;
        }
      }
      if (max_index == -1) 
      {
        for (band = 0; band < bands; band++)
        {
          target3[band][row][col] = 0;
        }
      }
      else 
      {
        long source_row = row_indices[max_index];
        long source_col = col_indices[max_index];
        for (band = 0; band < bands; band++)
        {
          target3[band][row][col] = source3[band][source_row][source_col];
        }
      }
    }
  }
  return result;
}


// Returns an image that is 0 where binary is 0 and identical to
// source where binary is non-zero.
template<class type>
v3dMatrix<type> * vSelectPixels(v3dMatrix<type> * source,
                                   grayscale_image * binary)
{
  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();
  if (rows != binary->Rows()) return 0;
  if (cols != binary->Cols()) return 0;

  v3dMatrix<type> * result = new v3dMatrix<type>(rows, cols, bands);
  vArray3(type) source3 = source->Matrix3();
  vArray3(type) result3 = result->Matrix3();
  vArray2(uchar) binary2 = binary->Matrix2(0);

  long row, col, band;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      if (binary2[row][col] != 0)
      {
        for (band = 0; band < bands; band++)
        {
          result3[band][row][col] = source3[band][row][col];
        }
      }
      else
      {
        for (band = 0; band < bands; band++)
        {
          result3[band][row][col] = 0;
        }
      }
    }
  }
  return result;
}


// returns the number of nonzero entries in the image.
// if the image has more than one channel, the entries in each channel
// are counted separately.
template<class type>
long count_non_zero(v3dMatrix<type> * image)
{
  vArray(type) entries = image->Matrix();
  long size = image->all_channel_size();
  long result = 0;

  long counter;
  for (counter = 0; counter < size; counter++)
  {
    if (entries[counter] != (type) 0)
    {
      result++;
    }
  }

  return result;
}


// returns the coordinates of non-zero pixels in the image.
// this function should only be called for images with one channel.
template<class type>
long_matrix non_zero_pixels(v3dMatrix<type> * image)
{
  if (image->channels() != 1)
  {
    exit_error("\nerror: non_zero_pixels called with a multiple-channel image\n");
  }

  vArray2(type) entries = image->Matrix2(0);
  long size = image->all_channel_size();

  long number = count_non_zero(image);
  long_matrix result(number, 2);
  long vertical_size = image->vertical();
  long horizontal_size = image->horizontal();

  long index = 0;
  long vertical, horizontal;
  for (vertical = 0; vertical < vertical_size; vertical++)
  {
    for (horizontal = 0; horizontal < horizontal_size; horizontal++)
    {
      if (entries[vertical][horizontal] != (type) 0)
      {
        result(index, 0) = vertical;
        result(index, 1) = horizontal;
        index++;
      }
    }
  }

  return result;
}




















// If the cpp template file is included as part of the header file,
// we must undefine certain things.
#include "basics/undefine.h"

#endif // VASSILIS_PROCESS_FILE
