#ifndef VASSILIS_IMAGE_TEMPLATES_H
#define VASSILIS_IMAGE_TEMPLATES_H

#include <vector>

#include "basics/matrix.h"
#include "vplatform.h"
#include "basics/drawing_temp.h"


class vTemplateCallerBase1
{
protected:
  Type type_name;

public:
  vTemplateCallerBase1() {}
  ~vTemplateCallerBase1() {}

  static vTemplateCallerBase1 * Create(general_image * the_image);

  virtual vMatrix<double> *
  BoundedRankHistogram(general_image * input, long number_of_bins,
                       double lower, double upper) = 0;

  virtual v3dMatrix<double> *
  ComputeMotion(general_image * the_image1, general_image * the_image2,
                general_image * the_image3) = 0;

  virtual v3dMatrix<double> *
  ImageDifference(general_image * the_image1, general_image * the_image2) = 0;

  virtual general_image * 
  MaxNeighbors(general_image * the_image) = 0;

  virtual vMatrix<double> *
  PixelBoundedHistogram(general_image * input, long number_of_bins,
                        double lower, double upper) = 0;

  virtual vMatrix<double> *
  PixelHistogram(general_image * input, long number_of_bins) = 0;

  virtual general_image * 
  PyramidImage(general_image * input, long times) = 0;

  virtual general_image * 
  TranslateImage(general_image * the_image, long translation_rows, 
                   long translation_cols) = 0;

  virtual general_image * 
  ScaleImage(general_image * the_image, double scaling,
             long new_center_row, long new_center_col) = 0;

  virtual general_image * 
  ScaleImage(general_image * the_image, double scaling,
             long new_center_row, long new_center_col,
             long new_rows, long new_cols) = 0;

  virtual general_image *
  ResizeImage(general_image * the_image, long new_rows, long new_cols) = 0;

  virtual general_image * 
  ScaleBoundingBox(general_image * the_image, long new_box_size,
                   long new_center_row, long new_center_col) = 0;

  virtual general_image * 
  ScaleBoundingBox(general_image * the_image, long new_box_size,
                   long new_center_row, long new_center_col,
                   long new_rows, long new_cols) = 0;

  virtual vBoundingBox
  BinaryBoundingBox(general_image * input) = 0;

  virtual general_image *
  Magnify(general_image * input, long center_row, long center_col,
          long output_rows, long output_cols, long factor) = 0;

  virtual color_image *
  CircularIntensity(general_image * input, double low, double high,
                    double ignore) = 0;

  virtual general_image *
  SelectRange(general_image * input, long band, double low, double high) = 0;
};


template<class type>
class vTemplateCaller1 : public vTemplateCallerBase1
{
public:
  vTemplateCaller1() 
  {
    type_name = vGetType<type>((type) 0);
  }

  ~vTemplateCaller1() {}

  virtual vMatrix<double> *
  BoundedRankHistogram(general_image * input, long number_of_bins,
                       double lower, double upper);

  virtual v3dMatrix<double> * 
  ComputeMotion(general_image * the_image1, general_image * the_image2,
                general_image * the_image3);

  virtual general_image * 
  MaxNeighbors(general_image * the_image);

  virtual vMatrix<double> *
  PixelBoundedHistogram(general_image * input, long number_of_bins,
                        double lower, double upper);

  virtual vMatrix<double> *
  PixelHistogram(general_image * input, long number_of_bins);

  virtual v3dMatrix<double> *
  ImageDifference(general_image * the_image1, general_image * the_image2);

  virtual general_image * PyramidImage(general_image * input, long times);

  virtual general_image * 
  TranslateImage(general_image * the_image, long translation_rows, 
                   long translation_cols);

  virtual general_image * 
  ScaleImage(general_image * the_image, double scaling,
             long new_center_row, long new_center_col);

  virtual general_image * 
  ScaleImage(general_image * the_image, double scaling,
             long new_center_row, long new_center_col,
             long new_rows, long new_cols);

  virtual general_image * 
  ResizeImage(general_image * the_image, long new_rows, long new_cols);

  virtual general_image * 
  ScaleBoundingBox(general_image * the_image, long new_box_size,
                   long new_center_row, long new_center_col);

  virtual general_image * 
  ScaleBoundingBox(general_image * the_image, long new_box_size,
                   long new_center_row, long new_center_col,
                   long new_rows, long new_cols);

  virtual vBoundingBox
  BinaryBoundingBox(general_image * input);

  virtual general_image *
  Magnify(general_image * input, long center_row, long center_col,
          long output_rows, long output_cols, long factor);


  virtual color_image *
  CircularIntensity(general_image * input, double low, double high,
                    double ignore);

  virtual general_image *
  SelectRange(general_image * input, long band, double low, double high);
};


class vCallerHolder1
{
public:
  vTemplateCallerBase1 * caller;

  vCallerHolder1(general_image * the_image)
  {
    caller = vTemplateCallerBase1::Create(the_image);
  }

  ~vCallerHolder1()
  {
    vdelete(caller);
  }
};



class vImageWrap : public vUnique
{
public:
  general_image * the_image_base;
  Type type_name;  
  vImageWrap * pointer;

  vImageWrap()
  {
    pointer = 0;
  }

  vImageWrap(general_image * in_color_image);
  ~vImageWrap()
  {
    DecrementReferences();
  }

  virtual void DeleteUnique()
  {
    vdelete(pointer);
  }

  virtual grayscale_image * Binary()
  {
    return pointer->Binary();
  }

  // normalize so that minimum enclosing circle for non-zero region 
  // has the specified radius
  //virtual general_image * NormalizeMEC1(long radius)
  //{
  //  return pointer->NormalizeMEC1(radius);
  //}

  //// normalize so that minimum enclosing circle for non-zero region 
  //// has the specified center and radius
  //virtual general_image * NormalizeMEC3(long center_row, long center_col, 
  //                                    long radius)
  //{
  //  return pointer->NormalizeMEC3(center_row, center_col, radius);
  //}

  virtual general_image * Warp(v3dMatrix<short> * sources)
  {
    return pointer->Warp(sources);
  };

  virtual general_image * UpsideDown()
  {
    return pointer->UpsideDown();
  }

  virtual general_image * TopBottom()
  {
    return pointer->TopBottom();
  }

  // Pads by adding zeros on each border side, as specified
  virtual general_image * Pad4(long left, long right, long top, long bottom)
  {
    return pointer->Pad4(left, right, top, bottom);
  }

  // Pads by adding zeros on border side, so that the new image has
  // the specified size
  virtual general_image * Pad2(long new_rows, long new_cols)
  {
    return pointer->Pad2(new_rows, new_cols);
  }
};


vImageWrap * vWrapImage(general_image * the_image);

// An abbreviation of vWrapImage.
//inline vImageWrap * wrim(general_image * the_image)
//{
//  return vWrapImage(color_image);
//}

// An abbreviation of calling the vImageWrap constructor
inline vImageWrap wrim(general_image * the_image)
{
  return vImageWrap(the_image);
}

template<class type>
class vMatrixWrap : public vImageWrap
{
public:
  v3dMatrix<type> * the_image;
  
  vMatrixWrap()
  {
    color_image_base = 0;
    color_image = 0;
    pointer = 0;
    type_name = vGetType<type>((type) 0);
  }

  vMatrixWrap(general_image * in_color_image)
  {
    the_image_base = in_color_image;
    pointer = 0;
    type_name = vGetType<type>((type) 0);
    if (the_image_base->TypeName() != type_name)
    {
      exit_error("Error, vMatrixWrap called with bad image type\n");
    }
    the_image = (v3dMatrix<type> *) in_color_image;
  }

  virtual grayscale_image * Binary();

  //virtual general_image * NormalizeMEC1(long radius)
  //{
  //  return NormalizeMEC1_a(radius);
  //}

  //v3dMatrix<type> * NormalizeMEC1_a(long radius);

  //// normalize so that minimum enclosing circle for non-zero region 
  //// has the specified center and radius
  //virtual general_image * NormalizeMEC3(long center_row, long center_col, 
  //                                    long radius)
  //{
  //  return NormalizeMEC3_a(center_row, center_col, radius);
  //}

  //v3dMatrix<type> * NormalizeMEC3_a(long center_row, long center_col, 
  //                                    long radius);

  virtual general_image * Warp(v3dMatrix<short> * sources)
  {
    return Warp_a(sources);
  };

  v3dMatrix<type> * Warp_a(v3dMatrix<short> * sources);

  virtual general_image * UpsideDown()
  {
    return UpsideDown_a();
  }

  v3dMatrix<type> * UpsideDown_a()
  {
    return vUpsideDown(the_image);
  }

  virtual general_image * TopBottom()
  {
    return TopBottom_a();
  }

  v3dMatrix<type> * TopBottom_a()
  {
    return vSwitchTopBottom(the_image);
  }

  // Pads by adding zeros on each border side, as specified
  virtual general_image * Pad4(long left, long right, long top, long bottom)
  {
    return Pad4_a(left, right, top, bottom);
  }

  v3dMatrix<type> * Pad4_a(long left, long right, long top, long bottom)
  {
    return vPad(the_image, left, right, top, bottom);
  }

  // Pads by adding zeros on border side, so that the new image has
  // the specified size
  virtual general_image * Pad2(long new_rows, long new_cols)
  {
    return Pad2_a(new_rows, new_cols);
  }

  v3dMatrix<type> * Pad2_a(long new_rows, long new_cols)
  {
    return vPad(the_image, new_rows, new_cols);
  }
};

template<class type>
inline vMatrixWrap<type> wrma(v3dMatrix<type> * the_image)
{
  return vMatrixWrap<type>(the_image);
}


// Pads by adding the specified rows or columns or zeros at the top,
// bottom, left and right sides.
template<class type>
v3dMatrix<type> * vPad(v3dMatrix<type> * image, long left, long right, 
			  long top, long bottom);

// Pads at the bottom and right 
// to make sure that there are at least as many rows and columns as
// the arguments new_rows, new_cols specify.
template<class type>
v3dMatrix<type> * vPad(v3dMatrix<type> * image, long new_rows, long new_cols);

// The result image has a center portion equal to image,
// and it is padded at the top, bottom, left and right
// so that it has the size specified by new_rows and new_cols.
template<class type>
v3dMatrix<type> * vPadCentered(v3dMatrix<type> * image, 
                                  long new_rows, long new_cols);

template<class type1, class type2>
v3dMatrix<double> * function_convolve(v3dMatrix<type1> * image, 
				 v3dMatrix<type2> * filter);

template<class type1, class type2>
v3dMatrix<float> * function_convolve_float(v3dMatrix<type1> * image, 
				 v3dMatrix<type2> * filter);

// Returns the convolution of the image with the filter. The result has
// the same size as the input image.
template<class type1, class type2>
vMatrix<float> function_convolve_float(vMatrix<type1> image, 
				                               vMatrix<type2> filter);

// Returns the convolution of the image with the filter. The result has
// the same size as the input image. This is an optimized version,
// no padding is done.
template<class type1, class type2>
vMatrix<float> function_convolve_float2(vMatrix<type1> image, 
				                               vMatrix<type2> filter);

template<class type>
v3dMatrix<double> * vComputeMotion(v3dMatrix<type> * image1, 
                          			      v3dMatrix<type> * image2, 
				                              v3dMatrix<type> * image3);
template<class type>
v3dMatrix<double> * vImageDifference(v3dMatrix<type> * the_image1, 
					                              v3dMatrix<type> * the_image2);

// Blurs an image using a 3x3 Gaussian filter
template<class type>
v3dMatrix<double> * vBlur(v3dMatrix<type> * the_image);


// Blurs an image using a filter_rows x filter_cols uniform filter
// (uniform meaning that all values in the filter are identical).
template<class type>
v3dMatrix<double> * vBlurUniformly(v3dMatrix<type> * the_image, 
			                                long filter_rows, long filter_cols);

                                      
// This function calculates for every pixel and each band
// the average of distances in the band value between that pixel and each
// of its four neighbors.
template<class type>
v3dMatrix<type> * vUndirectedEdges(v3dMatrix<type> * the_image);

// This function calculates for every pixel and each band the maximum
// distance between the pixel and one of its four neighbors
template<class type>
v3dMatrix<type> * vMaxNeighbors(v3dMatrix<type> * the_image);

template<class type>
v3dMatrix<type>  * vAmplifyImage(v3dMatrix<type> * the_image);

template<class type>
v3dMatrix<type> * vMultiplyImage(v3dMatrix<type> * the_image, 
				    double factor);


template<class type>
double vInnerProduct(v3dMatrix<type> * fimage1, 
		      long top1, long left1,
		      v3dMatrix<type> * fimage2, 
		      long top2, long left2,
		      long rows, long cols,
		      long band);

template<class type>
double color_imageMagnitude(v3dMatrix<type> * fimage, long band);

template<class type>
double vCorrelation(v3dMatrix<type> * fimage1, long top1, long left1,
		     v3dMatrix<type> * fimage2, long top2, long left2,
		     long rows, long cols, long band);

template<class type>
double vInnerProduct(v3dMatrix<type> * fimage1, 
		      v3dMatrix<type> * fimage2, long band);

template<class type>
double vCorrelation(v3dMatrix<type> * fimage1, v3dMatrix<float> * fimage2,
		     long band);

template<class type>
v3dMatrix<double> * vGradient_X(v3dMatrix<type> * the_image);

template<class type>
v3dMatrix<double> * vGradient_Y(v3dMatrix<type> * the_image);

template<class type>
v3dMatrix<uchar> * vImageAND(v3dMatrix<type> * image1, 
				v3dMatrix<type> * image2);
    
template<class type>
v3dMatrix<uchar> * vImageOR(v3dMatrix<type> * image1, 
				v3dMatrix<type> * image2);

template<class type>
v3dMatrix<uchar> * vImageNOT(v3dMatrix<type> * image1);


template<class type>
v3dMatrix<type> * function_image_minimum(v3dMatrix<type> * image1, 
			       v3dMatrix<type> * image2);

template<class type>
type function_image_median(v3dMatrix<type> * the_image);

template<class type1, class type2>
v3dMatrix<double> * function_image_add(v3dMatrix<type1> * the_image1, 
				 v3dMatrix<type2> * the_image2);

template<class type1, class type2>
v3dMatrix<double> * function_image_subtract(v3dMatrix<type1> * the_image1, 
				      v3dMatrix<type2> * the_image2);

template<class type1, class type2>
v3dMatrix<float> * function_image_subtract_float(v3dMatrix<type1> * the_image1, 
				      v3dMatrix<type2> * the_image2);

template<class type1, class type2>
v3dMatrix<double> * function_image_multiply(v3dMatrix<type1> * the_image1, 
				      v3dMatrix<type2> * the_image2);

template<class type1, class type2>
v3dMatrix<double> * function_image_divide(v3dMatrix<type1> * the_image1, 
				    v3dMatrix<type2> * the_image2);

template<class type>
v3dMatrix<double> * function_image_add(v3dMatrix<type> * the_image1, 
				 double number);

template<class type>
v3dMatrix<double> * function_image_subtract(v3dMatrix<type> * the_image1, 
				      double number);

template<class type>
v3dMatrix<double> * function_image_multiply(v3dMatrix<type> * the_image1, 
				      double number);

template<class type>
v3dMatrix<double> * function_image_divide(v3dMatrix<type> * the_image1, 
				    double number);

template<class type1, class type2, class type3>
void function_image_add(v3dMatrix<type1> * the_image1, 
		v3dMatrix<type2> * the_image2,
		v3dMatrix<type3> * result);

template<class type1, class type2, class type3>
void function_image_subtract(v3dMatrix<type1> * the_image1, 
		     v3dMatrix<type2> * the_image2,
		     v3dMatrix<type3> * result);

template<class type1, class type2, class type3>
void function_image_multiply(v3dMatrix<type1> * the_image1, 
		     v3dMatrix<type2> * the_image2,
		     v3dMatrix<type3> * result);

template<class type1, class type2, class type3>
void function_image_divide(v3dMatrix<type1> * the_image1, 
		   v3dMatrix<type2> * the_image2,
		   v3dMatrix<type3> * result);

template<class type1, class type2>
void function_image_add(v3dMatrix<type1> * the_image1, double number,
		v3dMatrix<type2> * result);

template<class type1, class type2>
void function_image_subtract(v3dMatrix<type1> * the_image1, double number, 
		     v3dMatrix<type2> * result);

template<class type1, class type2>
void function_image_multiply(v3dMatrix<type1> * the_image1, double number,
		     v3dMatrix<type2> * result);

template<class type1, class type2>
void function_image_divide(v3dMatrix<type1> * the_image1, double number,
		   v3dMatrix<type2> * result);

template<class type1>
v3dMatrix<type1> image_square_root(v3dMatrix<type1> * the_image1);

template<class type1, class type2>
void image_square_root(v3dMatrix<type1> * the_image1, v3dMatrix<type2> * result);


//////////////////////////////////////////////////////////////

template<class type1>
grayscale_image * vSelectValue(v3dMatrix<type1> * the_image, type1 value);

template<class type1>
v3dMatrix<type1> * vSelectRange(v3dMatrix<type1> * the_image, long band,
                                   type1 low, type1 high);

template<class type1>
grayscale_image * function_choose_value(v3dMatrix<type1> * the_image, type1 value)
{
  return vSelectValue(the_image, value);
}

template<class type1>
v3dMatrix<type1> * function_choose_range(v3dMatrix<type1> * the_image, long band,
                                         type1 low, type1 high)
{
  return vSelectRange(the_image, band, low, high);
}

template<class type1>
void vCenterOfMass(v3dMatrix<type1> * the_image, 
                    double & center_row, double & center_col);


template<class type1>
double vMaxDistanceFromPoint(v3dMatrix<type1> * the_image,
                              double center_row, double center_col);

template<class type1>
v3dMatrix<type1> * vNormalizeImage(v3dMatrix<type1> * the_image,
                                      double new_max_distance);


// Create a "normalized" image based on the input argument "color_image". The 
// center of mass of the normalized image is specified by "new_center_row" and
// "new_center_col". The maximum distance of a non-zero point from the 
// center of mass in the result is specified by "new_max_distance". The result
// image has the same size as the input image. To calculate the center of mass
// of an image we treat the image as if it were binary, i.e. pixels with 0 
// intensity have 0 mass and pixels with non-zero intensity have non-zero mass.
// 
// To determine the value of a pixel in the result image we do the following:
// We consider every pixel to be a unit-sized square, whose center is the 
// coordinates of the pixel. We take the pixel in the result image and warp
// it back to the input image. The coordinates (X, Y) 
// we get are probably not integers
// any more. The unit-sized square whose center is those coordinates overlaps
// with (at most) four pixels of the input image. The pixel in the result image
// gets the value that is the weighted avarage of the values in those four pixels 
// in the input image, where the weight of each pixel in the source image is 
// proportional to its overlap with the unit square centered at (X, Y).
template<class type1>
v3dMatrix<type1> * vNormalizeImage(v3dMatrix<type1> * the_image,
                                      double new_max_distance, 
                                      double new_center_row, double new_center_col);

template<class type1>
v3dMatrix<type1> * vNormalizeLabeledImage2(v3dMatrix<type1> * the_image,
                                              double new_max_distance);


template<class type1>
v3dMatrix<type1> * vNormalizeLabeledImage3(v3dMatrix<type1> * the_image,
                                              double new_max_distance, 
                                              double new_center_row, 
                                              double new_center_col);

// This function creates a half-resolution version of the input by blurring and
// subsampling. If the number of rows (or cols) is odd, the last row (or col) 
// is dropped.
template<class type1>
v3dMatrix<type1> * vHalfScale(v3dMatrix<type1> * input);


// This function scales down an image by half (in each dimension) repeatedly, 
// by calling vHalfScale, as many times as the second argument specifies.
template<class type1>
v3dMatrix<type1> * vPyramidImage(v3dMatrix<type1> * input, long times);


// This function creates and returns a histogram (with the specified
// number of bins) where each bin shows the number of pixels corresponding
// to the bin. The histogram is a vMatrix object, that has one row
// per bin, and each row has an entry for the lower bound of the range
// of the bin and an entry for the number of pixels.
template<class type>
vMatrix<double> *
vPixelHistogram(v3dMatrix<type> * input, long number_of_bins);

// This function is like vPixelHistogram, but it only considers values
// in the interval [lower, upper] as it builds the histogram.
template<class type>
vMatrix<double> *
vPixelBoundedHistogram(v3dMatrix<type> * input, long number_of_bins,
                        double lower, double upper);


// vBoundedRankHistogram returns a histogram of the values of all pixels
// in input that are in the [lower, ..., upper] interval. However, it is
// different than vPixelBoundedHistogram, in that here bins don't correspond
// to equal-spaced intervals, but are defined in such a way that the first bin
// gets the smallest 1/number_of_bins fraction of pixels, the second bin gets
// the second smallest, and so on. The returned histogram has 4 columns and
// one row per bin. The first entry in the row is the lower bound of the bin.
// The second entry is the number of pixels in the bin. The third is a 
// cumulative value summing up all bins from the first up to the current.
// The fourth is a cumulative value summing up all bins from the last down
// to the current.
template<class type>
vMatrix<double> *
vBoundedRankHistogram(v3dMatrix<type> * input, long number_of_bins,
                       double lower, double upper);


// We find the top-most, bottom-most, left-most and right-most coordinate
// values of non-zero pixels in input, and record them into the result's
// member variables. In a multi-band image, a pixel is non-zero if any
// of its bands is non-zero.
template<class type>
vBoundingBox
vBinaryBoundingBox(v3dMatrix<type> * input);


// We return an image in which pixel i,j has the same value as
// the color_image pixel i-translation_rows, j-translation_cols. If
// there is no such pixels within the value of color_image, pixel i,j
// is zero in the result.
template<class type>
v3dMatrix<type> *
function_translate_image(v3dMatrix<type> * the_image, long translation_rows, 
                 long translation_cols);


// We return an image which is the result of scaling color_image by
// multiplying the distance of everything from 
// (new_center_row, new_center_col)
// We don't want non-zero values to get lost in the scaling. This 
// could happen when we scale down. To prevent it, we use two methods,
// one for scaling up (scaling >= 1) and one for scaling down
// (0 < scaling < 1). 
//
// Note: No interpolation is done. Because of that, this function
// is best for binary images, when we really want to ensure
// that intensity values don't change. I defined this function
// in order to apply it to edge images, and take care of 
// the following two cases:
//
// - If the result image is 
// larger, we don't want connected edge components in color_image
// to become disconenected. Therefore for every output pixel
// we should find a corresponding input pixel and copy its
// value. vScaleImageUp does that.
//
// - If the result image is smaller, we want all edge pixels
// in color_image to be represented in the output image. So,
// for each color_image pixel that is non-zero we find its corresponding
// position in the output image and copy its value there.
// The auxiliary function vScaleImageDown does that.
template<class type>
v3dMatrix<type> * 
vScaleImage4(v3dMatrix<type> * the_image, double scaling,
              long new_center_row, long new_center_col);

template<class type>
v3dMatrix<type> * 
vScaleImage6(v3dMatrix<type> * the_image, double scaling,
              long new_center_row, long new_center_col,
              long new_rows, long new_cols);


// Like vScaleImage4 and vScaleImage6, this function is best
// for binary images, like edge images. If the output image
// is larger, we go through all output pixels and get their
// values from the corresponding input pixels. If the output
// image is smaller, we go through all input pixels and
// write their values on the corresponding output pixels.
template<class type>
v3dMatrix<type> *
vResizeImage(v3dMatrix<type> * the_image, long new_rows, long new_cols);

// The result image has the size of color_image, bounding box (of its non-zero
// values) of size (i.e. longest side) new_box_size and center of 
// the bounding box specified by new_center_row and new_center_col
template<class type>
v3dMatrix<type> * 
vScaleBoundingBox4(v3dMatrix<type> * the_image, long new_box_size,
                   long new_center_row, long new_center_col);

// Like vScaleBoundingBox4, but here the result image has the size
// specified by new_rows and new_cols.
template<class type>
v3dMatrix<type> * 
vScaleBoundingBox6(v3dMatrix<type> * the_image, long new_box_size,
                    long new_center_row, long new_center_col,
                    long new_rows, long new_cols);


// First we zero out the result, and then for every edge in the input 
// image we find the corresponding pixel in the result image and set it
// to 255.
template<class type>
v3dMatrix<type> * 
vScaleImageDown(v3dMatrix<type> * the_image, double scaling,
                 long new_center_row, long new_center_col,
                 long new_rows, long new_cols);


// For a pixel i,j in the result image, we find the pixel that it 
// corresponds to in color_image, and put the value of that pixel to i,j.
template<class type>
v3dMatrix<type> * vScaleImageUp(v3dMatrix<type> * the_image, double scaling,
                                   long new_center_row, long new_center_col,
                                   long new_rows, long new_cols);

// Fill a connected component (4-connected) whose value is 0 and contains
// the point (row, col) with the specified value.
template<class type>
long vFillRegion(v3dMatrix<type> * the_image, long row, long col, type value);


// Returns an output image which is the result of magnifying a portion 
// of the input image, centered at (center_row, center_col), by a
// factor specified by the factor argument. The corners of the region that
// will be magnified in the input image can be deduced by 
// (output_rows, output_cols) and the magnifying factor.
template<class type>
v3dMatrix<type> * vMagnify(v3dMatrix<type> * source,
                              long center_row, long center_col, 
                              long output_rows, long output_cols,
                              long resampling);



// I wrote this function to visualize edge orientations, but it can
// be used for other visualizations as well. With edge orientations,
// orientation 0 is the same as orientation 2*PI. If I represent
// each orientation value with a proportional intensity level, 
// pixels that look visually very different (with intensity values
// very low and very high) have values that, interpreted as orientations,
// are very similar. vCircularIntensity takes in an image whose values
// are in the range [low, high] and which are circular, in the sense
// that low and high are the same value conceptualy, and it creates
// a color image that lets us visualize the input image. In the color
// image, low is mapped to red. As we move towards high, values change
// to green, and they become green at 1/3 of the way towards high. Then
// they start changing towards blue, and they become blue at 2/3 of the
// way towards high. Then they start changing towards red again. This
// way similar pixel values correspond to conceptually similar 
// values in the input image.
template<class type> color_image *
vCircularIntensity(v3dMatrix<type> * input, double low, double high, double ignore);

// source where binary is non-zero.
template<class type>
v3dMatrix<type> * vSelectPixels(v3dMatrix<type> * source,
                                   grayscale_image * binary);


// This function is useful in cases where we perform an image operation
// that doesn't give a meaningful result to border pixels. In such cases,
// it often makes sense to replace the values of those pixels with the values
// of their neighboring pixels that are one unit away from the boundary.
template<class type>
long vSmoothBorders(v3dMatrix<type> * the_image);


// here the derivative calculations are centered
template<class type>
inline void vPixelGradient2(vArray2(type) values, long row, long col,
                             double & horizontal, double & vertical)
{
  double left = values[row][col-1];
  double right = values[row][col+1];
  double top = values[row-1][col];
  double bottom = values[row+1][col];
  vertical = bottom - top;
  horizontal = right - left;
}


// here the derivative calculations are not centered
template<class type>
inline void vPixelGradient2b(vArray2(type) values, long row, long col,
                             double & horizontal, double & vertical)
{
  double left = values[row][col-1];
  double top = values[row-1][col];
  double center = values[row][col];
  vertical = center - top;
  horizontal = center - left;
}


// here the derivative calculations are not centered
template<class type>
inline void vPixelGradient2b(vArray2(type) values, long row, long col,
                             float & horizontal, float & vertical)
{
  float left = (float) values[row][col-1];
  float top = (float) values[row-1][col];
  float center = (float) values[row][col];
  vertical = center - top;
  horizontal = center - left;
}


template<class type>
inline void vPixelGradient2(vArray2(type) values, long row, long col,
                             float & horizontal, float & vertical)
{
  float left = values[row][col-1];
  float right = values[row][col+1];
  float top = values[row-1][col];
  float bottom = values[row+1][col];
  vertical = bottom - top;
  horizontal = right - left;
}


// In some cases, like in the canny edge detector, it may make more sense to 
// look at the absolute values of the horizontal and vertical changes, rather
// than the changes themselves. For example, an edge may look like
/// 00000
/// 88677
/// 00000
// In that case, for the center pixel, vPixelGradient2 would report a gradient
// that's horizontal, whereas we are clearly interested in the fact that
// more rapid changes occur in the vertical direction.
template<class type>
inline void vPixelAbsGradient(vArray2(type) values, long row, long col,
                               double & horizontal, double & vertical)
{
  double center = values[row][col];
  double left = values[row][col-1];
  double right = values[row][col+1];
  double top = values[row-1][col];
  double bottom = values[row+1][col];
  vertical = vAbs(center - bottom) + vAbs(center - top);
  horizontal = vAbs(center - left) + vAbs(center - right);
}


template<class type>
inline double vPixelOrientation(vArray2(type) values, long row, long col)
{
  double left = values[row][col-1];
  double right = values[row][col+1];
  double top = values[row-1][col];
  double bottom = values[row+1][col];
  double vertical = bottom - top;
  double horizontal = right - left;
  //return vAngle(horizontal, vertical) + M_PI / 2;
  // If we wanted to return the direction of the gradient, we would
  // return vAngle(horizontal, vertical). However, we want to 
  // return the direction of least change, because that is 
  // what we consider the edge orientation at that pixel. So, essentially
  // we want to return gradient + M_PI/2. For efficiency, we use in 
  // the next line the property that vAngle(-y,x) = vAngle(x,y) + M_PI/2.
  return vAngle(-vertical, horizontal);
}


template<class type>
inline double vPixelOrientation2(vArray2(type) values, long row, long col)
{
  double left = values[row][col-1];
  double right = values[row][col+1];
  double top = values[row-1][col];
  double bottom = values[row+1][col];
  double vertical = bottom - top;
  double horizontal = right - left;
  // If there is no gradient, return -1.
  if ((top == bottom) && (left == right)) return -1;
  //return vAngle(horizontal, vertical) + M_PI / 2;
  // If we wanted to return the direction of the gradient, we would
  // return vAngle(horizontal, vertical). However, we want to 
  // return the direction of least change, because that is 
  // what we consider the edge orientation at that pixel. So, essentially
  // we want to return gradient + M_PI/2. For efficiency, we use in 
  // the next line the property that vAngle(-y,x) = vAngle(x,y) + M_PI/2.
  return vAngle2(horizontal, vertical);
}


// result[row][col] = color_image[row][col] when color_image[row][col] is
// a valid uchar value (between 0 and 255). Negative values 
// go to 0, values > 255 go to 255.
template<class type>
v3dMatrix<uchar> * vCroppedUchar(v3dMatrix<type> * the_image);


// If the images are equal return 1. Else, if they have different dimensions,
// return 0. If they have same dimensions but differ in at least one pixel,
// return 0 - the number of pixels where the images differ.
template<class type>
long color_imageEqual(v3dMatrix<type> * the_image1, v3dMatrix<type> * the_image2);


// The result is white (255) where color_image has the given color,
// black (0) otherwise.
template<class type>
grayscale_image * vSelectColor(v3dMatrix<type> * the_image, 
                            std::vector<type> * color);

// returns the number of nonzero entries in the image.
// if the image has more than one channel, the entries in each channel
// are counted separately.
template<class type>
long count_non_zero(v3dMatrix<type> * image);

// returns the coordinates of non-zero pixels in the image.
// this function should only be called for images with one channel.
template<class type>
long_matrix non_zero_pixels(v3dMatrix<type> * image);



#ifndef VASSILIS_SGI_PLATFORM
#define VASSILIS_PROCESS_FILE
#include "image_templates.cpp"
#endif // VASSILIS_SGI_PLATFORM


#endif // VASSILIS_IMAGE_TEMPLATES_H
