#include "stdafx.h"

#include "vplatform.h"
#include "read_avi.h"
#ifdef VASSILIS_WINDOWS
#include "avi_file.h"
#else // VASSILIS_WINDOWS
typedef vReadAVI vAVI_Stream;
#endif // VASSILIS_WINDOWS
#include "rle_encoding.h"
#include "video_common.h"
#include "auxiliaries.h"
#include "my_video_format.h"

#ifdef VASSILIS_SIEMENS
#include "ax_project/input_output.h"
#include "general/dicom_sequence.h"
#endif

#include "definitions.h"


vVideoFile * vOpenReadVideoFile(const char * filename)
{
  vVideoFile * result = 0;
  if (vStringCaseEndsIn(filename, ".avi"))
  {
    result = new vReadAVI(filename);
//    result = new vAVI_Stream(filename);
  }
  else if (vStringCaseEndsIn(filename, ".rle"))
  {
    result = new vRLE_Sequence(filename);
  }
  // My own image format, with RGB images
  else if (vStringCaseEndsIn(filename, ".vimseq3"))
  {
    result = new class_image_stream(filename);
  }
  else if (vStringCaseEndsIn(filename, ".vimseq1"))
  {
    result = new class_image_stream(filename);
  }
  else if (vStringCaseEndsIn(filename, ".pgm"))
  {
    result = new vPPM_Sequence(filename);
  }
  else if (vStringCaseEndsIn(filename, ".ppm"))
  {
    result = new vPPM_Sequence(filename);
  }
  else if (vStringCaseEndsIn(filename, ".bmp"))
  {
    result = new vPPM_Sequence(filename);
  }
  else if (vStringCaseEndsIn(filename, ".bmpseq"))
  {
    result = new bitmap_sequence(filename);
  }
  else if (vStringCaseEndsIn(filename, ".aviset"))
  {
    result = new vMFC_AVI_Set(filename);
  }
#ifdef VASSILIS_SIEMENS
  else if (vStringCaseEndsIn(filename, ".medical"))
  {
    result = new class_existing_sequence(filename);
  }
  else if (vStringCaseEndsIn(filename, ".ima"))
  {
    result = new dicom_sequence(filename);
  }
#endif
  else
  {
    return 0;
  }
  if (!result->IsValid())
  {
    vCloseVideoFile(result);
    result = 0;
  }
  return result;
}


vVideoFile * vOpenWriteVideoFile(const char * filename, 
                                       long rows, long cols)
{
  return vOpenWriteVideoFile(filename, rows, cols, 30, 1, 3);
}


vVideoFile * vOpenWriteVideoFile(const char * filename, 
                                       long rows, long cols, 
                                       long rate, long scale, long color_bytes)
{
  vVideoFile * result = 0;
  if (vStringCaseEndsIn(filename, ".avi"))
  {
    result = new vAVI_Stream(filename, rows, cols, rate, scale, color_bytes);
  }
  else if (vStringCaseEndsIn(filename, ".aviset"))
  {
    result = new vMFC_AVI_Set(filename, rows, cols, rate, scale, color_bytes);
  }

  else if (vStringCaseEndsIn(filename, ".rle"))
  {
    result = new vRLE_Sequence(filename, rows, cols);
  }
  else if (vStringCaseEndsIn(filename, ".vimseq3"))
  {
    result = new class_image_stream(filename, rows, cols, 3, DoubleType);
  }
  else if (vStringCaseEndsIn(filename, ".vimseq1"))
  {
    result = new class_image_stream(filename, rows, cols, 1, DoubleType);
  }
  else if (vStringCaseEndsIn(filename, ".pgm"))
  {
    result = new vPPM_Sequence(filename, rows, cols);
  }
  else if (vStringCaseEndsIn(filename, ".ppm"))
  {
    result = new vPPM_Sequence(filename, rows, cols);
  }
  else if (vStringCaseEndsIn(filename, ".bmpseq"))
  {
    result = new bitmap_sequence(filename, rows, cols, color_bytes);
  }
  return result;
}


void vCloseVideoFile(vVideoFile * sequence)
{
  if (sequence == 0) return;
  const char * type = sequence->Type();
  if (strcmp(type, "vReadAVI") == 0)
  {
    vdelete((vReadAVI *&) sequence);
  }
  else if (strcmp(type, "vAVI_Stream") == 0)
  {
    vdelete((vAVI_Stream *&) sequence);
  }
  else if (strcmp(type, "vRLE_Sequence") == 0)
  {
    vdelete((vRLE_Sequence *&) sequence);
  }
  else if (strcmp(type, "class_image_stream") == 0)
  {
    vdelete((class_image_stream *&) sequence);
  }
  else if (strcmp(type, "vPPM_Sequence") == 0)
  {
    vdelete((vPPM_Sequence *&) sequence);
  }
  else if (strcmp(type, "bitmap_sequence") == 0)
  {
    vdelete((bitmap_sequence *&) sequence);
  }
  else if (strcmp(type, "vMFC_AVI_Set") == 0)
  {
    vdelete((vMFC_AVI_Set *&) sequence);
  }
#ifdef VASSILIS_SIEMENS
  else if (strcmp(type, "class_existing_sequence") == 0)
  {
    vdelete((class_existing_sequence *&) sequence);
  }
  else if (strcmp(type, "dicom_sequence") == 0)
  {
    vdelete((dicom_sequence *&) sequence);
  }
#endif // VASSILIS_SIEMENS
  else
  {
    printf("Unexpected type of video sequence: %s\n", type);
    assert(0);
  }
}


// Write the video sequence stored in input_file into output,
// frame by frame.
long vWriteVideo(vVideoFile * output, const char * input_file)
{
  vVideoFile * input = vOpenReadVideoFile(input_file);
  if (input == 0) return 0;
  long in_frames = input->NumberOfFrames();
  long in_rows = input->Rows();
  long in_cols = input->Cols();
  if ((in_rows != output->Rows()) || (in_cols != output->Cols()))
  {
    vCloseVideoFile(input);
    return 0;
  }

  long result = 1;
  long frame;
  for (frame = 0; frame < in_frames; frame++)
  {
    color_image * the_image = input->ReadFrame(frame);
    if (the_image == 0)
    {
      result = 0;
      break;
    }
    long temp = output->write_frame(the_image);
    vdelete(the_image);
    if (temp <= 0)
    {
      result = 0;
      break;
    }
  }
  vCloseVideoFile(input);
  return result;
}
