#ifndef VASSILIS_VIDEO_FILE_H
#define VASSILIS_VIDEO_FILE_H

#include <stdio.h>
#include "auxiliaries.h"
#include "image.h"

// vVideoFile is supposed to be a superclass of all classes that
// I define that can read and write video sequences. By forcing
// all classes to inherit from this one, I can treat them all
// the same way in my code, regardless of what they actually are.
class vVideoFile
{
protected:
  char * filename;
  char * type; // "avi" for AVIs, "rle" for RLEs, etc.
  FILE * fp;
  long rows;
  long cols;
  long number_of_frames;
  // frame rate = rate/scale frames per second.
  long rate;
  long scale;

  long current_frame;
  long current_position;
  
  long top, left, height, width; // Used if only want to read a specific region
  ushort is_readable;
  ushort is_writable;
  ushort is_valid;

public:
  vVideoFile();
  ~vVideoFile();
  inline const char * Type()
  {
    return type;
  }

  inline unsigned short IsValid()
  {
    return is_valid;
  }

  inline unsigned short valid()
  {
    return is_valid;
  }

  inline ushort IsReadable()
  {
    return is_readable;
  }

  inline ushort IsWritable()
  {
    return is_writable;
  }

  inline long Rows()
  {
    return rows;
  }

  inline long Cols()
  {
    return cols;
  }

  inline long vertical()
  {
    return rows;
  }

  inline long horizontal()
  {
    return cols;
  }

  inline long Height()
  {
    if (height == -1)
    {
      return rows;
    }
    return height;
  }

  inline long Width()
  {
    if (width == -1)
    {
      return cols;
    }
    return width;
  }

  inline long Top()
  {
    return top;
  }

  inline long NumberOfFrames()
  {
    return number_of_frames;
  }

  inline long CurrentFrame()
  {
    return current_frame;
  }

  inline long Scale()
  {
    return scale;
  }

  inline long Rate()
  {
    return rate;
  }

  inline const char * GetFilename()
  {
    return filename;
  }

  inline const char * get_filename()
  {
    return filename;
  }

  virtual long SetBounds(long top, long left, long height, long width);

  // ReadFrameGray reads in the specified frame and converts it
  // (if necessary) to a grayscale_image.
  virtual grayscale_image * ReadNextFrameGray() = 0;
  virtual grayscale_image * ReadFrameGray(long number) = 0;

  // ReadFrame reads in the specified frame and converts it
  // (if necessary) to a color_image.
  virtual color_image * ReadNextFrame() = 0;
  virtual color_image * ReadFrame(long number) = 0;
  
  // ReadImage reads in the specified frame and returns it in the
  // format that it considers the best. The default just returns
  // a color_image, but it can be overwritten.
  virtual general_image * ReadImage(long number);
  virtual general_image * ReadNextImage();

  virtual ushort InitializeWriting() = 0;
  virtual ushort write_frame(color_image * image) = 0;
  virtual ushort write_frameGray(grayscale_image * gray) = 0;
  virtual ushort FinishWriting() = 0;
  virtual long BytesPerPixel() = 0;
};


#endif // VASSILIS_VIDEO_FILE_H
