#include "stdafx.h"

#include "vmatlab.h"
#include "algebra.h"

#include "definitions.h"

#ifdef VASSILIS_USE_MATLAB



vMxArray::vMxArray(long in_rows, long in_cols)
{
  mx_array = mxCreateDoubleMatrix(in_rows, in_cols, mxREAL);
  Initialize();
}


vMxArray::vMxArray(mxArray * in_array)
{
  mx_array = in_array;
  Initialize();
}


vMxArray::vMxArray(v3dMatrix<float> * in_array, long order)
{
  long rows = in_array->Rows();
  long cols = in_array->Cols();
  long bands = in_array->Bands();

  if (bands != 1)
  {
    exit_error("error: vMxArray(v3dMatrix<float> * in_array), bands != 1\n");
  }

  mx_array = mxCreateDoubleMatrix(rows, cols, mxREAL);
  Initialize();
  vMatrixToMx3(in_array, order, this);
}


vMxArray::~vMxArray()
{
  if ((dummy == 0) && (mx_array != 0))
  {
    exit_error("error: dummy = 0, mx_array != 0\n");
  }
  else if (dummy == 0)
  {
    return;
  }
  else if (*dummy != 1234567890)
  {
    exit_error("error in ~vMxArray: *dummy = %li\n", *dummy);
  }

  double * current_pointer = mxGetPr(mx_array);
  if (current_pointer != pointer)
  {
    exit_error("error: current_pointer != mxGetPr(mx_array)\n");
  }

  vdelete(dummy);
  mxDestroyArray(mx_array);
  dummy = 0;
  mx_array = 0;
  pointer = 0;
}


long vMxArray::Initialize()
{
  if (mx_array == 0)
  {
    vPrint("out of memory\n");
    pointer = 0;
    dummy = 0;
    return 0;
  }

  pointer = mxGetPr(mx_array);
  dummy = new long;
  *dummy = 1234567890;

  return 1;
}


mxArray * vMxArray::MxArray()
{
  return mx_array;
}


// ArrayPointer returns mxGetPr(mx_array), which is not always the
// same as the pointer member variable (not if mxSetPr has been
// called at some point).
double * vMxArray::ArrayPointer()
{
  if (mx_array == 0)
  {
    return 0;
  }
  return mxGetPr(mx_array);
}


long vMxArray::Rows()
{
  if (mx_array == 0)
  {
    return -1;
  }
  return mxGetM(mx_array);
}


long vMxArray::Cols()
{
  if (mx_array == 0)
  {
    return -1;
  }
  return mxGetN(mx_array);
}


// order should be 0 if we want sourcem(i,j) to equal target(i+1,j+1),
// and 1 if we want sourcem(i,j) to equal target(j+1,i+1). The latter
// is desirable because it preserves the one-dimensional structure,
// i.e. sourcem.Matrix()[i] = mxGetPr(target)[i]. 
vMxArray * vMatrixToMx2(v3dMatrix<float> * sourcem, long order)
{
  long rows = sourcem->Rows();
  long cols = sourcem->Cols();
  long bands = sourcem->Bands();
  if (bands != 1)
  {
    exit_error("error: vMatrixToMx2, bands != 1\n");
  }

  vMxArray * result = new vMxArray(rows, cols);
  vMatrixToMx3(sourcem, order, result);
  return result;
}


long vMatrixToMx3(v3dMatrix<float> * sourcem, long order, vMxArray * vtarget)
{
  long bands = sourcem->Bands();
  if (bands != 1)
  {
    exit_error("error: vMatrixToMx3, bands != 1\n");
  }

  mxArray * target = vtarget->MxArray();
  mxClassID class_id = mxGetClassID(target);
  if (class_id != mxDOUBLE_CLASS)
  {
    exit_error("error, array is not of double type\n");
  }

  long m = mxGetM(target);
  long n = mxGetN(target);
  long size = m * n;
  
  if ((sourcem->Rows() != m) || (sourcem->Cols() != n))
  {
    exit_error("error: vMxToMatrix3 called with bad sizes: %li %li %li %li\n",
                   m, n, sourcem->Rows(), sourcem->Cols());
  }
  vArray(float) matrix = sourcem->Matrix();
  vArray2(float) matrix2 = sourcem->Matrix2(0);

  double * pointer = mxGetPr(target);
  if (order == 0)
  {
    long row, col;
    long i = 0;
    for (col = 0; col < n; col++)
    {
      for (row = 0; row < m; row++)
      {
        pointer[i] = (double) matrix2[row][col];
        i = i+1;
      }
    }
  }
  else if (order == 1)
  {
    long i;
    for (i = 0; i < size; i++)
    {
      pointer[i] = (double) matrix[i];
    }
  }
  else
  {
    exit_error("error, order should be 0 or 1\n");
  }

  return 1;
}


vMatrix<float> vMxToMatrix2(vMxArray * source, long order)
{
  long rows = source->Rows();
  long cols = source->Cols();
  vMatrix<float> result(rows, cols);
  vMxToMatrix3(source, order, result);
  return result;
}


long vMxToMatrix3(vMxArray * vsource, long order, vMatrix<float> targetm)
{
  mxArray * source = vsource->MxArray();
  mxClassID class_id = mxGetClassID(source);
  if (class_id != mxDOUBLE_CLASS)
  {
    exit_error("error, array is not of double type\n");
  }

  long m = mxGetM(source);
  long n = mxGetN(source);
  long size = m * n;
  
  if ((targetm.Rows() != m) || (targetm.Cols() != n))
  {
    exit_error("error: vMxToMatrix3 called with bad sizes: %li %li %li %li\n",
                   m, n, targetm.Rows(), targetm.Cols());
  }
  vArray(float) matrix = targetm.Matrix();
  vArray2(float) matrix2 = targetm.Matrix2();

  double * pointer = mxGetPr(source);
  if (order == 0)
  {
    long row, col;
    long i = 0;
    for (col = 0; col < n; col++)
    {
      for (row = 0; row < m; row++)
      {
        matrix2[row][col] = (float) pointer[i];
        i = i+1;
      }
    }
  }
  else if (order == 1)
  {
    long i;
    for (i = 0; i < size; i++)
    {
      matrix[i] = (float) pointer[i];
    }
  }
  else
  {
    exit_error("error, order should be 0 or 1\n");
  }

  return 1;
}



vMatrix<float> vMlfMultiply(v3dMatrix<float> * matrix1, v3dMatrix<float> * matrix2)
{
  vMxArray array1(matrix1, 0);
  vMxArray array2(matrix2, 0);
  vMxArray temp_result(mlfMtimes(array1.MxArray(), array2.MxArray()));
  vMatrix<float> result = vMxToMatrix2(&temp_result, 0);
  return result;
}


vMatrix<float> vMlfInverse(v3dMatrix<float> * matrix1)
{
  vMxArray array1(matrix1, 0);
  vMxArray temp_result(mlfInv(array1.MxArray()));
  vMatrix<float> result = vMxToMatrix2(&temp_result, 0);
  return result;
}


vMatrix<float> vMlfSvd(v3dMatrix<float> * matrix1)
{
  vMxArray array1(matrix1, 0);
  vMxArray temp_result(mlfSvd(NULL, NULL, array1.MxArray(), NULL));
  vMatrix<float> result = vMxToMatrix2(&temp_result, 0);
  return result;
}


vMatrix<float> vGaussker(long window_size)
{
  vMxArray window_size_array(mlfScalar(window_size));
  vMxArray pascal_w(mlfPascal(window_size_array.MxArray(), NULL));
  vMxArray flipped(mlfFliplr(pascal_w.MxArray()));
  vMxArray diag_array(mlfDiag(flipped.MxArray(), NULL));
  
  vMatrix<float> diag = vMxToMatrix2(&diag_array, 0);
  float factor = pow((float) 2, (float) (1 - window_size));
  vMatrix<float> gm = factor * diag;
  vMatrix<float> gm_trans = gm.Trans();
  vMatrix<float> result = gm * gm_trans;
//  vMatrix<float> result = gm * gm.Trans();
  return result;
}




#else  // VASSILIS_USE_MATLAB


vMxArray::vMxArray(long in_rows, long in_cols)
{
}


vMxArray::vMxArray(v3dMatrix<float> * in_array, long order)
{
}


vMxArray::~vMxArray()
{
}


long vMxArray::Initialize()
{
  return 1;
}


// ArrayPointer returns mxGetPr(mx_array), which is not always the
// same as the pointer member variable (not if mxSetPr has been
// called at some point).
double * vMxArray::ArrayPointer()
{
    return 0;
}


long vMxArray::Rows()
{
  return -1;
}


long vMxArray::Cols()
{
    return -1;
}


// order should be 0 if we want sourcem(i,j) to equal target(i+1,j+1),
// and 1 if we want sourcem(i,j) to equal target(j+1,i+1). The latter
// is desirable because it preserves the one-dimensional structure,
// i.e. sourcem.Matrix()[i] = mxGetPr(target)[i]. 
vMxArray * vMatrixToMx2(v3dMatrix<float> * sourcem, long order)
{
  return 0;
}


long vMatrixToMx3(v3dMatrix<float> * sourcem, long order, vMxArray * vtarget)
{
  return 1;
}


vMatrix<float> vMxToMatrix2(vMxArray * source, long order)
{
  long rows = source->Rows();
  long cols = source->Cols();
  vMatrix<float> result(rows, cols);
   return result;
}


long vMxToMatrix3(vMxArray * vsource, long order, vMatrix<float> targetm)
{
  return 1;
}



vMatrix<float> vMlfMultiply(v3dMatrix<float> * matrix1, v3dMatrix<float> * matrix2)
{
  return vMatrix<float>();
}


vMatrix<float> vMlfInverse(v3dMatrix<float> * matrix1)
{
   return vMatrix<float>();
}


vMatrix<float> vMlfSvd(v3dMatrix<float> * matrix1)
{
  return vMatrix<float>();
}


vMatrix<float> vGaussker(long window_size)
{
  return vMatrix<float>();
}

#endif // VASSILIS_USE_MATLAB
