#ifndef VASSILIS_VMATLAB_H
#define VASSILIS_VMATLAB_H



#ifdef VASSILIS_USE_MATLAB
#include "matlab.h"
#include "algebra.h"

// I define class vMxArray as a wrapper around vMxArray, 
// so that my functions always access mxArrays through this function.
// This function will allocate some dummy memory using new, and 
// this way whenever I create or delete an mxArray, there will be
// a simultaneous call to new and delete. This way I can catch
// occasions where I don't delete an mxArray and a memory leak is caused.
// This function can also verify that the pointer that mxArray holds
// when deleted is the same that was used when it got created.
class vMxArray
{
protected:
  mxArray * mx_array;
  double * pointer;
  long * dummy;

public:
  vMxArray(long in_rows, long in_cols);
  vMxArray(mxArray * in_array);
  vMxArray(v3dMatrix<float> * in_array, long order);
  ~vMxArray();

  long Initialize();
  mxArray * MxArray();

  // ArrayPointer returns mxGetPr(mx_array), which is not always the
  // same as the pointer member variable (not if mxSetPr has been
  // called at some point).
  double * ArrayPointer();
  long Rows();
  long Cols();
};


// order should be 0 if we want sourcem(i,j) to equal target(i+1,j+1),
// and 1 if we want sourcem(i,j) to equal target(j+1,i+1). The latter
// is desirable because it preserves the one-dimensional structure,
// i.e. sourcem.Matrix()[i] = mxGetPr(target)[i]. 
vMxArray * vMatrixToMx2(v3dMatrix<float> * sourcem, long order);

long vMatrixToMx3(v3dMatrix<float> * sourcem, long order, vMxArray * target);


vMatrix<float> vMxToMatrix2(vMxArray * source, long order);

long vMxToMatrix3(vMxArray * source, long order, vMatrix<float> targetm);

vMatrix<float> vMlfMultiply(v3dMatrix<float> * matrix1, v3dMatrix<float> * matrix2);

vMatrix<float> vMlfInverse(v3dMatrix<float> * matrix1);

vMatrix<float> vMlfSvd(v3dMatrix<float> * matrix1);

vMatrix<float> vGaussker(long window_size);


#else // VASSILIS_USE_MATLAB

#include "algebra.h"
//#include "matlab.h"

// I define class vMxArray as a wrapper around vMxArray, 
// so that my functions always access mxArrays through this function.
// This function will allocate some dummy memory using new, and 
// this way whenever I create or delete an mxArray, there will be
// a simultaneous call to new and delete. This way I can catch
// occasions where I don't delete an mxArray and a memory leak is caused.
// This function can also verify that the pointer that mxArray holds
// when deleted is the same that was used when it got created.
class vMxArray
{
public:
  vMxArray(long in_rows, long in_cols);
  vMxArray(v3dMatrix<float> * in_array, long order);
  ~vMxArray();

  long Initialize();

  // ArrayPointer returns mxGetPr(mx_array), which is not always the
  // same as the pointer member variable (not if mxSetPr has been
  // called at some point).
  double * ArrayPointer();
  long Rows();
  long Cols();
};


// order should be 0 if we want sourcem(i,j) to equal target(i+1,j+1),
// and 1 if we want sourcem(i,j) to equal target(j+1,i+1). The latter
// is desirable because it preserves the one-dimensional structure,
// i.e. sourcem.Matrix()[i] = mxGetPr(target)[i]. 
vMxArray * vMatrixToMx2(v3dMatrix<float> * sourcem, long order);

long vMatrixToMx3(v3dMatrix<float> * sourcem, long order, vMxArray * target);


vMatrix<float> vMxToMatrix2(vMxArray * source, long order);

long vMxToMatrix3(vMxArray * source, long order, vMatrix<float> targetm);

vMatrix<float> vMlfMultiply(v3dMatrix<float> * matrix1, v3dMatrix<float> * matrix2);

vMatrix<float> vMlfInverse(v3dMatrix<float> * matrix1);

vMatrix<float> vMlfSvd(v3dMatrix<float> * matrix1);

vMatrix<float> vGaussker(long window_size);

#endif // VASSILIS_USE_MATLAB

#endif // VASSILIS_VMATLAB_H
