#include "stdafx.h"

#include "vplatform.h"
#ifdef VASSILIS_SGI_PLATFORM
#define VASSILIS_PROCESS_FILE
#endif // VASSILIS_SGI_PLATFORM

#ifdef VASSILIS_PROCESS_FILE
#undef VASSILIS_PROCESS_FILE


#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#include <algorithm.h>
#else 
#include <iostream>
#include <algorithm>
#endif
#include <math.h>


#include "basics/matrix.h"
#include "basics/drawing.h"
#include "basics/simple_algo_templates.h"
#include "basics/simple_algo.h"
#include "algorithms.h"
#include "basics/algebra.h"
#include "basics/image.h"
#include "basics/definitions.h"

using namespace std;




template<class type>
void S_ReadTable1(const char * filename, v3dMatrix<type> *& result)
{
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0)
  {
    exit_error("Reading skin histograms: Can't find %s\n", filename);
  }
  long items_read;
  long rows, cols, bands;
  items_read = fread(&bands, sizeof(long), 1, fp);
  assert(items_read == 1);
  items_read = fread(&rows, sizeof(long), 1, fp);
  assert(items_read == 1);
  items_read = fread(&cols, sizeof(long), 1, fp);
  assert(items_read == 1);
  assert(rows == cols);
  assert(rows == bands);

  result = new v3dMatrix<type>(rows, cols, bands);
  result->ReadBandwise(fp);
  fclose(fp);
}


template<class type>
void S_ReadTable2(const char * filename, long dim_size,
                v3dMatrix<type> *& result)
{
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0)
  {
    exit_error("Reading skin histograms: Can't find %s\n", filename);
  }

  result = new v3dMatrix<type>(dim_size, dim_size, dim_size);
  result->ReadBandwise(fp);
  fclose(fp);
}


template<class type>
void S_WriteTable(const char * filename, v3dMatrix<type> * image)
{
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0)
  {
    exit_error("Saving skin histograms: Can't open %s\n", filename);
  }
  long items;
  long rows = image->Rows();
  long cols = image->Cols();
  long bands = image->Bands();
  items = fwrite(&bands, sizeof(long), 1, fp);
  assert(items == 1);
  items = fwrite(&rows, sizeof(long), 1, fp);
  assert(items == 1);
  items = fwrite(&cols, sizeof(long), 1, fp);
  assert(items == 1);
  
  image->WriteBandwise(fp);
}


// This constructor reads my skin histograms
template<class type>
vSkinTable<type>::vSkinTable(const char * filename1, const char * filename2)
{
  S_ReadTable1(filename1, skin_data);
  S_ReadTable1(filename2, non_skin_data);
  dim_size = skin_data->Bands();
  assert(dim_size == skin_data->Bands());
}


// This function reads the CRL skin histograms
template<class type>
vSkinTable<type>::vSkinTable(const char * filename1, const char * filename2,
                               long in_dim_size)
{
  dim_size = in_dim_size;
  S_ReadTable2(filename1, in_dim_size, skin_data);
  S_ReadTable2(filename2, in_dim_size, non_skin_data);
}


template<class type>
vSkinTable<type>::~vSkinTable()
{
  //AfxMessageBox("SkinTable destructor called.");
  DecrementReferences();
}


template<class type>
void vSkinTable<type>::DeleteUnique()
{
  //AfxMessageBox("Skin table delete reference called");
  vdelete(skin_data);
  vdelete(non_skin_data);
}

template<class type>
ushort vSkinTable<type>::Write(const char * filename1, const char * filename2)
{
  S_WriteTable(filename1, skin_data);
  S_WriteTable(filename2, non_skin_data);
  return 1;
}


template<class type>
grayscale_image * vSkinTable<type>::DetectSkin(color_image * the_image)
{
  grayscale_image * result = new grayscale_image(the_image->Rows(), the_image->Cols());
  
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();
  vArray(uchar) I = result->GetI();

  vArray3(type) skin_table = skin_data->Matrix3();
  vArray3(type) non_skin_table = non_skin_data->Matrix3();

  long factor = 256 / dim_size;

  long size = the_image->Size();
  long i;
  for (i = 0; i < size; i++)
  {
    uchar r1 = R[i];
    uchar g1 = G[i];
    uchar b1 = B[i];

    long r_index = r1/factor;
    long g_index = g1/factor;
    long b_index = b1/factor;

    double entry1 = skin_table[r_index][g_index][b_index];
    double entry2 = non_skin_table[r_index][g_index][b_index];

    double value;
    if ((entry1 == 0) && (entry2 == 0)) value = 0.01;
    else value = entry1 / (entry1 + entry2);

    I[i] = (uchar) (value * 255.0);
  }
  return result;
}


// For debugging, return the value stored at the specified bin.
template<class type>
type vSkinTable<type>::ReadSkinValue(long r_index, long g_index,
                                      long b_index)
{
  if (skin_data->check_bounds(g_index, b_index) <= 0) return 0;
  if (r_index < 0) return 0;
  if (r_index >= skin_data->Bands()) return 0;

  vArray3(type) skin_table = skin_data->Matrix3();
  return skin_table[r_index][g_index][b_index];
}

template<class type>
type  vSkinTable<type>::ReadNonSkinValue(long r_index, long g_index, 
                                          long b_index)
{
  if (non_skin_data->check_bounds(g_index, b_index) <= 0) return 0;
  if (r_index < 0) return 0;
  if (r_index >= non_skin_data->Bands()) return 0;

  vArray3(type) non_skin_table = non_skin_data->Matrix3();
  return non_skin_table[r_index][g_index][b_index];
}




// As of 1/1/2004, I am not sure what this function is doing and
// whether it is doing it right.
template<class type>
color_image * vFindMaxSpots(v3dMatrix<type> * source, long number)
{
  long i;
  const long max_colors = 30;
  assert(max_colors % 3 == 0);
  char r[max_colors];
  char g[max_colors];
  char b[max_colors];
  const long third = max_colors / 3;
  const long spacing = 200 / max_colors;
  assert(spacing >= 1);

  // Set the color coding for the spots where the maxima occur. This
  // should be executed only once, in principle, but in practice it
  // doesn't make a difference.
  for (i = 0; i < third; i++)
  {
    r[i] = (uchar) (255 - i * spacing);
    g[i] = 0;
    b[i] = 0;

    r[third+i] = 0;
    g[third+i] = (uchar) (255 - i * spacing);
    b[third+i] = 0;

    r[2*third+i] = 0;
    g[2*third+i] = 0;
    b[2*third+i] = (uchar) (255-i*spacing);
  }

  assert(number <= 30);
  v3dMatrix<double> * temp_image;
  temp_image = (v3dMatrix<double> *) source->AverageBands();
  v3dMatrix<uchar> * gimage = new v3dMatrix<uchar>(temp_image);
  vdelete(temp_image);
  vArray(uchar) I = gimage->Matrix(0);
  long size = gimage->Size();
    
  // First we need to find the cutoff point.
  long values[256];
  for (i = 0; i < 256; i++)
    values[i] = 0;

  for (i = 0; i < size; i++)
  {
    uchar c = I[i];
    values[c]++;
  }

  for (i = 254; i >= 0; i--)
    values[i] = values[i] + values[i+1];

  color_image * result = gimage->ToRGBBound();
  vArray(uchar) R = result->GetR();
  vArray(uchar) G = result->GetG();
  vArray(uchar) B = result->GetB();
  for (i = 0; i < size; i++)
  {
    uchar c = I[i];
    long rank = values[c];
    if (rank < number)
    {
      R[i] = r[rank];
      G[i] = g[rank];
      B[i] = b[rank];
    }
  }

  vdelete(gimage);
  return result;
}

template<class type>
vSequenceStatistics<type>::vSequenceStatistics(long in_rows, long in_cols, 
				       long in_bands)
{
  rows = in_rows;
  cols = in_cols;
  bands = in_bands;
  size = rows * cols;
  valid_means = 0;
  valid_deviations = 0;
  values_processed = 0;
  deviations_processed = 0;
  totals = new v3dMatrix<double>(rows, cols, bands);
  total_deviations = new v3dMatrix<double>(rows, cols, bands);
  means = new v3dMatrix<double>(rows, cols, bands);
  deviations = new v3dMatrix<double>(rows, cols, bands);
  vWriteValue(totals, (double) 0.0);
  vWriteValue(total_deviations, (double) 0.0);
}


template<class type>
vSequenceStatistics<type>::~vSequenceStatistics()
{
  vdelete(totals);
  vdelete(total_deviations);
  vdelete(means);
  vdelete(deviations);
}


template<class type>
void vSequenceStatistics<type>::ComputeMeans()
{
  if (values_processed == 0) return;
  double divider = values_processed;
  function_image_divide(totals, divider, means);
  valid_means = 1;
}

template<class type>
void vSequenceStatistics<type>::ComputeDeviations()
{
  if (deviations_processed <= 1) return;
  double divider = deviations_processed - 1;
  function_image_divide(total_deviations, divider, deviations);
  image_square_root(deviations, deviations);
  valid_deviations = 1;
}

template<class type>
void vSequenceStatistics<type>::ProcessValues(v3dMatrix<type> * the_image)
{
  valid_means = 0;
  valid_deviations = 0;
  assert(rows == the_image->Rows());
  assert(cols == the_image->Cols());
  assert(bands == the_image->Bands()
    );
  
  function_image_add(totals, the_image, totals);
  values_processed++;
}

template<class type>
void vSequenceStatistics<type>::ProcessDeviations(v3dMatrix<type> * the_image)
{
  assert(valid_means == 1);
  valid_deviations = 0;
  assert(rows == the_image->Rows());
  assert(cols == the_image->Cols());
  assert(bands == the_image->Bands());

  v3dMatrix<double> * scratch = function_image_subtract(means, the_image);
  function_image_absolute(scratch, scratch);
  function_image_multiply(scratch, scratch, scratch);
  function_image_add(total_deviations, scratch, total_deviations);
  
  vdelete(scratch);
  deviations_processed++;
}

template<class type>
long vSequenceStatistics<type>::Rows()
{
  return rows;
}

template<class type>
long vSequenceStatistics<type>::Cols()
{
  return cols;
}

template<class type>
long vSequenceStatistics<type>::Bands()
{
  return bands;
}

template<class type>
long vSequenceStatistics<type>::Size()
{
  return size;
}

template<class type>
void vSequenceStatistics<type>::Reset()
{
  vWriteValue(totals, 0.0);
  vWriteValue(total_deviations, 0.0);
  values_processed = 0;
  deviations_processed = 0;
}

template<class type>
v3dMatrix<double> * vSequenceStatistics<type>::GetMeans()
{
  if (!valid_means) ComputeMeans();
  return means;
}

template<class type>
v3dMatrix<double> * vSequenceStatistics<type>::GetDeviations()
{
  if (!valid_deviations) ComputeDeviations();
  return deviations;
}

// Input: A source image, an image assigning labels to each pixel in the source
// image, and a particular label.
// Output: A color grid, where each cell corresponds to an rg color
// (r = R/I, g = G/I, I = R+G+B), and the value of the cell is its distance 
// from the nearest rg color in the pixels of the source image that belong
// to the given label (the labels image is used to check which pixels in 
// the source image belong to the given label). It is computed using
// breadth-first search.
template<class type>
vSearchGrid * vMakeColorGrid(color_image * source, v3dMatrix<type> * labels,
			      Label & label)
{
  vSearchGrid * result = new vSearchGrid(256, 256);
  long initial_value = (result->grid_rows + result->grid_cols) * 3;
  
  // Initialize the color grid to ensure that all interior pixels will be visited
  // and that the border rows and cols (used for padding, and invisible if we use
  // the distances array of the color grid) will never be processed.
  result->WriteBorder(-1);
  result->WriteInterior(initial_value);

  long row, col;
  long ** pixel_labels = labels->Matrix2(0);
  uchar ** R = source->GetR2();
  uchar ** G = source->GetG2();
  uchar ** B = source->GetB2();
  long id = label.id;
  long ** distances = result->distances;
  vNodeList to_visit;

  // Prepare the to_visit list by adding in it
  // all the colors that show up in the source image,
  // and set the distances array entry for those colors
  // to 0.
  for (row = label.top; row <= label.bottom; row++)
    for (col = label.left; col <= label.right; col++)
    {
      type pixel_label = pixel_labels[row][col];
      // If the pixel does not belong to the region specified
      // by label, skip it.
      if (pixel_label == id)
      {
	      uchar r = R[row][col];
	      uchar g = G[row][col];
	      uchar b = B[row][col];
	      long intensity = r+g+b;
	      uchar r_normal = r * 255 / intensity;
	      uchar g_normal = g * 255 /intensity;
	      distances[r_normal][g_normal] = 0;
	      to_visit.push_back(new vPoint(r_normal, g_normal));
      }
    }

  // GridBreadthFirstProc will need to access the color grid, so
  // we add that to the procedure_args.
  vector<void *> procedure_args;
  procedure_args.push_back((void *) result);
  BreadthFirstSearch(&to_visit, GridBreadthFirstProc, &procedure_args);
  return result;
}
			      

// Input: A source image, an image assigning labels to each pixel in the source
// image, and a particular label.
// Output: A color grid, where each cell corresponds to an rg color
// (r = R/I, g = G/I, I = R+G+B), and the value of the cell is its distance 
// from the average rg color in the pixels of the source image that belong
// to the given label (the labels image is used to check which pixels in 
// the source image belong to the given label).
template<class type>
vSearchGrid * vMakeColorGrid2(color_image * source, v3dMatrix<type> * labels,
			      Label & label)
{
  long row, col;
  vArray2(type) pixel_labels = labels->Matrix2(0);
  vArray2(uchar) R = source->GetR2();
  vArray2(uchar) G = source->GetG2();
  vArray2(uchar) B = source->GetB2();

  long id = label.id;
//  vNodeList to_visit;

  long r_sum = 0;
  long g_sum = 0;

  // Find the average color of the pixels corresponding to the
  // given label in the source image
  long count = 0;

  for (row = label.top; row <= label.bottom; row++)
    for (col = label.left; col <= label.right; col++)
    {
      type pixel_label = pixel_labels[row][col];
      // If the pixel label is not the same as the argument label,
      // skip the pixel, otherwise use the source pixel in
      // calculating the average rg color
      if (pixel_label == id)
      {
        count++;
				uchar r = R[row][col];
				uchar g = G[row][col];
				uchar b = B[row][col];
				long intensity = r+g+b;
				if (intensity == 0) intensity = 1;
				uchar r_normal = (uchar) (r * 255 / intensity);
				uchar g_normal = (uchar) (g * 255 /intensity);
				r_sum += r_normal;
				g_sum += g_normal;
      }
    }
  
  // Get the average color
  uchar r_average = (uchar) (r_sum / count);
  uchar g_average = (uchar) (g_sum / count);
  assert(count == label.count);

  // Print the average color (for debugging)
  long long_r = r_average;
  long long_g = g_average;
	vPrint("Average face color: (%li, %li)\n", long_r, long_g);

  // Now that we have the average color, we can fill up the cells
  // of the color grid with the appropriate values.
  // Create a grid that will store scores for each colors in the rg space 
  vSearchGrid * result = new vSearchGrid(256, 256);
  vArray2(long) distances = result->distances;

  for (row = 0; row < result->grid_rows; row++)
		for (col = 0; col < result->grid_cols; col++)
		{
			double distance = abs(row - r_average) + abs(col - g_average);
//      double r_diff = row - r_average;
//      double g_diff = col - g_average;
//      double distance = sqrt(r_diff * r_diff + g_diff * g_diff);
			distances[row][col] = (long) distance;
		}

	result->WriteBorder(255);

  return result;
}



// vSaveMatrixAsUshorts can be used to save disk space when we write a 
// matrix to disk. It is applicable under the following assumptions
// - Suppose low is the min value in source
// - Suppose high is the max value in source
// - Suppose USHORT_MAX is the maximum value attainable by a ushort.
// - Assume that it doesn't make a noticeable difference in our domain 
//   if we replace each value in the matrix with its nearest
//   neighbor in a set of USHORT_MAX uniformly spaced points between low 
//   and high.
// Then, we can save to disk low and high, and for each entry in the matrix
// simply save the index of its nearest neighbor in the set of USHORT_MAX
// points. This information is enough to recover an approximation of the 
// matrix.
template<class type>
long vSaveMatrixAsUshorts(v3dMatrix<type> * source, const char * filename)
{
  const double USHORT_MAX = 65535;

  // Get low and high values of source, figure out the range and spacing 
  // of the USHORT_MAX points.
  double low = source->MinValue();
  double high = source->MaxValue();
  double range = high - low;
  double spacing = range / USHORT_MAX;
  if (spacing == 0) spacing = 1;
  long rows = source->Rows();
  long cols = source->Cols();
  long bands = source->Bands();

  // Create the ushort matrix.
  v3dMatrix<ushort> ushorts_matrix(rows, cols, bands);
  vArray(type) entries = source->Matrix();
  vArray(ushort) ushorts = ushorts_matrix.Matrix();
  long size = source->AllBandSize();

  // Do the conversion
  long i;
  for (i = 0; i < size; i++)
  {
    double entry = entries[i] - low;
    double double_index = entry / spacing;
    ushorts[i] = (ushort) double_index;
  }

  // Save low, high, ushorts into filename.
  
  // Open file
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) 
  {
    vPrint("Failed to open %s\n", filename);
    return 0;
  }
  // Write low, high
  long items = fwrite(&low, sizeof(double), 1, fp);
  items += fwrite(&high, sizeof(double), 1, fp);
  if (items != 2)
  {
    vPrint("Error in writing low and high to %s\n", filename);
    fclose(fp);
    return 0;
  }

  // Write filename
  long success = ushorts_matrix.Write(fp);
  fclose(fp);
  if (success <= 0)
  {
    vPrint("Failed to save ushort matrix into %s\n", filename);
  }
  return success;
}


// vLoadMatrixAsUshorts returns an approximation of a matrix saved as 
// filename by vSaveMatrixAsUshorts.
template<class type>
v3dMatrix<type> * vLoadMatrixAsUshorts(const char * filename)
{
  const double USHORT_MAX = 65535;

  // Read low, high, ushorts from filename.

  // Read low, high;
  double low, high;
  low = high = 0;
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) 
  {
    vPrint("Failed to open %s\n", filename);
    return 0;
  }
  long items = fread(&low, sizeof(double), 1, fp);
  items += fread(&high, sizeof(double), 1, fp);
  if (items != 2)
  {
    vPrint("Error in reading low and high from %s\n", filename);
    fclose(fp);
    return 0;
  }

  v3dMatrix<ushort> * ushorts_matrix = v3dMatrix<ushort>::Read(fp);
  fclose(fp);
  if (ushorts_matrix == 0)
  {
    vPrint("Failed to save read ushorts matrix from %s\n", filename);
  }

  // Figure out spacing.  
  double range = high - low;
  double spacing = range / USHORT_MAX;
  if (spacing == 0) spacing = 1;

  // Create the result matrix.
  long rows = ushorts_matrix->Rows();
  long cols = ushorts_matrix->Cols();
  long bands = ushorts_matrix->Bands();
  long sizeof_type = sizeof(type);

  vPrint("rows = %li, cols = %li, bands = %li, size of type is %li\n",
          rows, cols, bands, sizeof_type);
  vPrint("Memory required = %li\n", rows * cols * bands * sizeof(type));
  v3dMatrix<type> * result_matrix = new v3dMatrix<type>(rows, cols, bands);
  vArray(type) entries = result_matrix->Matrix();
  vArray(ushort) ushorts = ushorts_matrix->Matrix();
  long size = ushorts_matrix->AllBandSize();

  // Do the conversion
  long i;
  for (i = 0; i < size; i++)
  {
    double double_index = ushorts[i];;
    double entry = double_index * spacing + low;
    entries[i] = (type) entry;
  }

  vdelete(ushorts_matrix);
  return result_matrix;
}


// Finds the mean color robustly of the region in color_image that is
// specified with non-zero values in region.
// Stores the mean color in result (the
// result will have as many values as the bands of color_image).
// The result is not necessarily the arithmetic mean of the colors
// in the region. We do some iterations of going through the region,
// finding the mean, rejecting outliers, recalculating the mean,
// re-rejecting outliers, and so on.
template<class type>
long vRegionRobustMean(v3dMatrix<type> * the_image, 
                        v3dMatrix<uchar> * region, 
                        vector<double> * result)
{
  result->erase(result->begin(), result->end());
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = rows * cols;
  long bands = the_image->Bands();
  if (rows != region->Rows()) return 0;
  if (cols != region->Cols()) return 0;
  if (region->Bands() != 1) return 0;
  
  // color_image2 is indexed by band, row and col.
  vArray3(type) color_image3 = the_image->Matrix3();
  long band;
  vArray(double) mean_color = vnew(double, bands);
  vArray(double) new_mean_color = vnew(double, bands);
  vArray(double) current_color = vnew(double, bands);
  for (band = 0; band < bands; band++)
  {
    mean_color[band] = (type) 0;
    new_mean_color[band] = (type) 0;
  }

  vector<double> distances;
  double distance_threshold = vTypeMax((double) 0);

  // Get the locations of the non-zero pixels of region.
  vector<long> row_values;
  vector<long> col_values;
  vGetNonZeroPoints((grayscale_image *) region, &row_values, &col_values);
  long number_of_points = row_values.size();
  double d_number = (double) number_of_points;
  long times, i;
  double d_threshold_index = d_number * 0.5;
  long threshold_index = (long) d_threshold_index;
  
  // Do the main loop some times
  for (times = 0; times < 10; times++)
  {
    // Go through each point, and if it is not an outlier, use
    // it to recompute the mean.
    double inlier_counter = 0;
    for (i = 0; i < number_of_points; i++)
    {
      // Get the point coordinates
      long row = row_values[i];
      long col = col_values[i];
      double distance = 0;
      // Store the color of the point in current_color,
      // and compute its distance from mean_color.
      for (band = 0; band < bands; band++)
      {
        current_color[band] = color_image3[band][row][col];
        distance += vAbs(current_color[band] - mean_color[band]);
      }
      // If the color distance is over the threshold then we 
      // consider the pixel to be an outlier, and we don't 
      // use it in computing the updated mean (new_mean_color).
      if (distance <= distance_threshold)
      {
        inlier_counter++;
        for (band = 0; band < bands; band++)
        {
          new_mean_color[band] += current_color[band];
        }
      }
    }
    // Compute the new average color
    for (band = 0; band < bands; band++)
    {
      mean_color[band] = new_mean_color[band] / inlier_counter;
      new_mean_color[band] = 0;
    }

    // Compute distances from the updated average color,
    // and calculate the new distance threshold.
    
    // First prepare the distances vector
    distances.erase(distances.begin(), distances.end());
    // Go through all points and compute their distances
    // to the mean color
    for (i = 0; i < number_of_points; i++)
    {
      long row = row_values[i];
      long col = col_values[i];
      double distance = 0;
      // Store the color of the point in current_color,
      // and compute its distance from mean_color.
      for (band = 0; band < bands; band++)
      {
        current_color[band] = color_image3[band][row][col];
        distance += vAbs(current_color[band] - mean_color[band]);
      }
      distances.push_back(distance);
    }
    // Sort distances and pick the cutoff point
    std::sort(distances.begin(), distances.end(), less<double>());

    distance_threshold = distances[threshold_index];
    vPrint("distance_threshold = %lf\n", distance_threshold);
    vPrint("mean_color: ");
    for (band = 0; band < bands; band++)
    {
      vPrint("%.1lf ", mean_color[band]);
    }
    vPrint("\n\n");
  }

  for (band = 0; band < bands; band++)
  {
    result->push_back(mean_color[band]);
  }


  vdelete2(mean_color);
  vdelete2(new_mean_color);
  vdelete2(current_color);
  return 1;
}

// Returns a binary image, whose non-zero values specify a 
// region in color_image that is a subset of the non-zero pixels
// in candidates, and whose color in color_image is within the
// specfied distance from the color argument.
template<class type>
grayscale_image * vSelectColorRange(v3dMatrix<type> * the_image,
                                 v3dMatrix<uchar> * candidates,
                                 vector<double> * in_color, 
                                 double distance_threshold)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  if (rows != candidates->Rows()) return 0;
  if (cols != candidates->Cols()) return 0;
  if (candidates->Bands() != 1) return 0;
  if (in_color->size() != bands) return 0;

  vArray(double) color = vnew(double, bands);
  long band;
  for (band = 0; band < bands; band++)
  {
    color[band] = (*in_color)[band];
  }

  grayscale_image * result_image = new grayscale_image(rows, cols);
  vArray(uchar) result1 = result_image->Matrix();
  vWriteValue(result_image, (uchar) 0);
  long size = rows * cols;
  // color_image2 is indexed by band and pixel index
  vArray2(type) color_image2 = the_image->Matrix2();
  // candidates1 is indexed by pixel index;
  vArray(uchar) candidates1 = candidates->Matrix();
  long i;
  for (i = 0; i < size; i++)
  {
    if (candidates1[i] == 0) continue;
    double distance = 0;
    for (band = 0; band < bands; band++)
    {
      distance += vAbs(((double) color_image2[band][i]) - color[band]);
    }
    if (distance < distance_threshold)
    {
      result1[i] = 255;
    }
  }

  vdelete2(color);
  return result_image;
}



// This is like vSelectColorRange, but addresses the problem of selecting
// pixels whose colors are much closer to color than to other_color.
// "much closer" is defined as follows: If a pixel's color has a distance
// d1 to color and d2 to other_color, if d1 < d2-distance then it
// the pixel's color is "much closer" to color than it is to other_color.
template<class type>
grayscale_image * vSelectColorRange5(v3dMatrix<type> * the_image,
                                  v3dMatrix<uchar> * candidates,
                                  vector<double> * in_color, 
                                  vector<double> * in_other_color, double distance)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  if (rows != candidates->Rows()) return 0;
  if (cols != candidates->Cols()) return 0;
  if (candidates->Bands() != 1) return 0;
  if (in_color->size() != bands) return 0;
  if (in_other_color->size() != bands) return 0;

  vArray(double) color = vnew(double, bands);
  vArray(double) other_color = vnew(double, bands);
  long band;
//  double threshold = 0;
  for (band = 0; band < bands; band++)
  {
    color[band] = (*in_color)[band];
    other_color[band] = (*in_other_color)[band];
//    threshold += vAbs(color[band] - other_color[band]);
  }

//  threshold = threshold - 2 * distance;

  grayscale_image * result_image = new grayscale_image(rows, cols);
  vArray(uchar) result1 = result_image->Matrix();
  vWriteValue(result_image, (uchar) 0);
  long size = rows * cols;
  // color_image2 is indexed by band and pixel index
  vArray2(type) color_image2 = the_image->Matrix2();
  // candidates1 is indexed by pixel index;
  vArray(uchar) candidates1 = candidates->Matrix();
  long i;
  for (i = 0; i < size; i++)
  {
    if (candidates1[i] == 0) continue;
    double distance1 = 0;
    double distance2 = 0;
    for (band = 0; band < bands; band++)
    {
      double value = color_image2[band][i];
      distance1 += vAbs(value - color[band]);
      distance2 += vAbs(value - other_color[band]);
    }
    if (distance2 - distance1 > distance)
    {
      result1[i] = 255;
    }
  }

  vdelete2(color);
  vdelete2(other_color);
  return result_image;
}



// This function is useful in cases where we perform an image operation
// that doesn't give a meaningful result to border pixels. In such cases,
// it often makes sense to replace the values of those pixels with the values
// of their neighboring pixels that are one unit away from the boundary.
template<class type>
long vSmoothBorders(v3dMatrix<type> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bands = the_image->Bands();
  if (rows < 2) return 0;
  if (cols < 2) return 0;
  vArray3(type) matrix3 = the_image->Matrix3();
  long row, col, band;
  for (band = 0; band < bands; band++)
  {
    // take care of four corners
    matrix3[band][0][0] = matrix3[band][1][1];
    matrix3[band][0][cols-1] = matrix3[band][1][cols-2];
    matrix3[band][rows-1][0] = matrix3[band][rows-2][1];
    matrix3[band][rows-1][cols-1] = matrix3[band][rows-2][cols-2];

    // Take care of top and bottom side
    for (col = 1; col < cols-1; col++)
    {
      matrix3[band][0][col] = matrix3[band][1][col];
      matrix3[band][rows-1][col] = matrix3[band][rows-2][col];
    }

    // Take care of left and right side
    for (row = 1; row < rows-1; row++)
    {
      matrix3[band][row][0] = matrix3[band][row][1];
      matrix3[band][row][cols-1] = matrix3[band][row][cols-2];
    }
  }
  return 1;
}


template<class type>
v3dMatrix<type> * normalize_enclosing_circle(v3dMatrix<type> * image, long radius)
{
  if (image == 0)
  {
   return 0;
  }

  long vertical = image->vertical();
  long horizontal = image->horizontal();
  long center_vertical = vertical/2;
  long center_horizontal = horizontal/2;

  return normalize_enclosing_circle4(image, radius, center_vertical, center_horizontal);
}


template<class type>
v3dMatrix<type> * normalize_enclosing_circle4(v3dMatrix<type> * image, long radius,
                                             long center_row, long center_col)
{
  vCircleLong mec = vMinEnclosingCircle(image);
  double factor = ((double) radius) / (double) mec.radius;
  long rows = image->Rows();
  long cols = image->Cols();

  long temp_center_row = (rows - 1) / 2;
  long temp_center_col = (cols - 1) / 2;
  long translation_rows = temp_center_row - mec.center_row;
  long translation_cols = temp_center_col - mec.center_col;
  // First we center the non-zero region at the center of the image,
  // then we scale, and finally we move the non-zero region so that
  // it is centered at center_row and center_col. The reason
  // we do not just do two operation (centering at center_row and center_col
  // preceded or followed by scaling) is that sometimes non-zero pixels 
  // fall out of bounds either in the translation or in the scaling.
  // By first centering at the image's center row and center col we
  // reduce the occurrences of non-zero pixels falling out of bounds.
  v3dMatrix<type> * translated = function_translate_image(image, translation_rows,
                                                    translation_cols);
  v3dMatrix<type> * scaled = vScaleImage4(translated, factor,
                                             temp_center_row, temp_center_col);

  // vScaleImage4, because of errors introduced by discretization,
  // oftentimes does not produce an image that is truly centered
  // where it should be. By finding the enclosing circle again
  // and translating appropriately we ensure that we get the desired
  // centering.
  mec = vMinEnclosingCircle(scaled);
  translation_rows = center_row - mec.center_row;
  translation_cols = center_col - mec.center_col;
  v3dMatrix<type> * result = function_translate_image(scaled, translation_rows, 
                                                translation_cols);

  vdelete(translated);
  vdelete(scaled);
  return result;
}




// If the cpp template file is included as part of the header file,
// we must undefine certain things.
#include "basics/undefine.h"

#endif // VASSILIS_PROCESS_FILE
