#ifndef VASSILIS_ANGLE_SAMPLER_H
#define VASSILIS_ANGLE_SAMPLER_H

#include <vector>

#include "quaternion.h"


// Note that for many of these functions there are two versions, an 
// "Old" version and a "Poser" version. This is because in the old version
// viewpoint (0,0) was the north pole, but now it is the equator.


// Suppose we inscribe an n-gon into a circle of given radius.
// How long is each side of the n-gon? This is what this
// function computes
double vVertexDistance(long n, double radius);

// Suppose we want to sample points on the surface of the unit sphere,
// so that all sample points are x degrees away from the north pole,
// the points are spaced
// uniformly, and the distance between successive points is the distance
// between point (0, 0) and point (degrees, 0) on the surface of the unit 
// sphere. By point(A,B) here I mean point with latitude A and 
// B degrees east. This is what this function computes
// If samples is not empty, the previous contents are retained
// and more contents are added to the back. Points are returned as 3D
// coordinates, assuming the sphere is a unit sphere centered at the origin.
long vOldSampleSphereYs(double x, double degrees, std::vector<vPoint3> * samples);
long vPoserSampleSphereYs(double x, double degrees, std::vector<vPoint3> * samples);

// This function samples points more or less evenly spaced along the surface
// of the unit sphere, such that no point on that surface is further than 
// the given number of degrees from a point in the sample. Point A is 
// d degrees away from point B if the smallest rotation that moves A to B
// is d degrees. If samples is not empty, the previous contents are retained
// and more contents are added to the back. Points are returned as 3D
// coordinates, assuming the sphere is a unit sphere centered at the origin.
long vOldSampleSpherePoints(double degrees, std::vector<vPoint3> * samples);
long vPoserSampleSpherePoints(double degrees, std::vector<vPoint3> * samples);

// Suppose we want to sample points on the surface of the unit sphere,
// so that all sample points are x degrees away from the north pole,
// the points are spaced
// uniformly, and the distance between successive points is the distance
// between point (0, 0) and point (degrees, 0) on the surface of the unit 
// sphere. By point(A,B) here I mean point with latitude A (A degrees from
// the north pole) and Y degrees east. This is what this function computes
// If samples is not empty, the previous contents are retained
// and more contents are added to the back. The points are returned with 
// two coordinates, that specify their location on the sphere surface.
long vOldSampleViewpointYs(double x_angle, double degrees, vector<vPoint2> * samples);
long vPoserSampleViewpointYs(double x_angle, double degrees, vector<vPoint2> * samples);

// This function samples points more or less evenly spaced along the surface
// of the unit sphere, such that no point on that surface is further than 
// the given number of degrees from a point in the sample. Point A is 
// d degrees away from point B if the smallest rotation that moves A to B
// is d degrees. If samples is not empty, the previous contents are retained
// and more contents are added to the back. The points are returned with 
// two coordinates, that specify their location on the sphere surface.
long vOldSampleViewpoints(double degrees, std::vector<vPoint2> * samples);
long vPoserSampleViewpoints(double degrees, std::vector<vPoint2> * samples);

// This function is a lot more simple. We simply sample orientations spaced
// apart as specified by degrees, starting at 0 and ending before 360.
long vOldSampleOrientations(double degrees, std::vector<double> * samples);
long vPoserSampleOrientations(double degrees, std::vector<double> * samples);


#endif // VASSILIS_ANGLE_SAMPLER_H
