#include "stdafx.h"

#include <stdlib.h>

#include "basics/drawing_temp.h"
#include "interpreter.h"
#include "matching.h"
#include "gesture_data.h"
#include "basics/video_common.h"
#include "opencv_utils.h"
#include "interface_programs.h"
#include "hausdorff_matching.h"
#include "contour.h"
#include "quaternion.h"
#include "virtual_hand/FrameViewFinder.h"
#include "basics/precomputed.h"
#include "angle_sampler.h"
#include "projections.h"
#include "basics/io_aux.h"

#include "basics/definitions.h"

void  S_OldViewQuaternion2(double x, double y, double z, double * q);



// Processing keyboard commands.
// Returns 1 when it is time to exit.
ushort vInterpreter::ProcessKey(char command)
{
  switch(command)
  {
  case 'a':
    SetSequenceStep();
    break;
						
  case 'B':
    PreviousInput();
    break;

  case 'b':
    PreviousInSequence();
    break;
	      
  case 'c':
    break;
	
  case 'C':
    break;

  case 'd':
    break;

    // Sometimes we want to use the mouse to define a set of lines on an 
    // image. In those cases, 'D' deletes the last vertex.
  case 'D':
    DeleteVertex();
    break;

  case 'e':
    break;

  case 'E':
    DrawLineSet();
    break;
  
  case 'f':
    NextInSequence();
    break;

  case 'F':
    NextInput();
    break;

  case 'g':
    CameraBackground();
    break;

  case 'G':
    break;

  case 'h':
    break;
					
  case 'i':
    InverseThresholdImage();
    break;

  case 'I':
    mouse_click_response = pixel_info_response;
    break;

  case 'j':
    SetCurrentCounter();
    ShowCurrentFrame();
    break;

  case 'J':
    break;

  case 'k':
    break;

  case 'K':
    break;

  case 'l':
    ShowCurrentFrame();
    break;

  case 'L':
    CreateLineSet();
    break;

  case 'm':
    SynchronizeSequences();
    NextInput();
    break;

  case 'M':
    if (mouse_click_response == pixel_info_response)
      mouse_click_response = LineResponse;
    else if (mouse_click_response == LineResponse)
      mouse_click_response = pixel_info_response;
    break;

  case 'n':
    ShowNextImage();
    break;

  case 'N':
    OverlayLineSet();
    break;

  case 'o':
    break;
	      
  case 'O':
    break;

  case 'p':
    ShowPreviousImage();
    break;

  case 'P':
    break;

  case 'q':
     vConsoleToForeground();   
     return 1;
    break;
	      
  case 'r':
    RedrawWindow();
    break;

  case 's':
    break;

  case 'S':
    break;

  case 't':
    ThresholdCurrentImage();
    break;
					
  case 'T':
    vConsoleToForeground();
    base_interpreter->main_loop(1, 0);
    break;

  case 'u':
    break;

  case 'U':
    break;

  case 'v':
    break;

  case 'w':
    if ((input != 0) && 
        ((input->sequence == 0) ||
        (input->sequence->Top() != -1)))
    {
      RemoveCurrentInput();
    }
    break;

  case 'z':
    break;

  case '1':
  case '2':
  case '3':
  case '4':
  case '5':
  case '6':
  case '7':
  case '8':
  case '9':
  case '0':
    browse_image(command - '0');
    break;
	      
  case '[':
    break;

  case ']':
    break;

  case '\\':
    ResetBounds();
    break;

  case '{':

  case '!':
    break;

  case ';':
    break;

  case '\'':
    break;

  case ':':
    break;

  case ',':
    StartCamera();
    break;

  case '.':
    StopCamera();
    //ShowCameraHand();
    OnlineSegmentation();
    break;

  case '/':
    OnlineRanking();
    break;

  case '`':
    break;

  case '=':
    ToGrayBound();
    write_frame();
    break;

  case '+':
    ShowNextTenMatches();
    break;

  case '_':
    ShowPreviousTenMatches();
    break;

  default:
    break;

  }
  return 0;
}
// End of processing keyboard commands.


// We want to do different things with the mouse at different times.
// The interpreter has a variable, mouse_click_response, 
// which indicates what should be done with a mouse click.
ushort vInterpreter::ProcessMouseClick(long lbutton_down, long row, long col)
{
  long normalized_row, normalized_col;
  normalized_row = normalized_col = 0;
  NormalizedCoordinates(row, col, &normalized_row, &normalized_col);

  // In this case we show the color of the pixel on which the user clicked
  if (mouse_click_response == pixel_info_response) 
  {
    ShowPixelInfo(normalized_row, normalized_col);
    return 1;
  }

  // In this case we center the displayed image around the click
  // (useful when we display with resolutions > 1, i.e. when
  // we magnify images)
  else if (mouse_click_response == CenterResponse)
  {
    long result = Center(normalized_row, normalized_col);
    mouse_click_response = previous_mouse_response;
    if (mouse_click_response == CenterResponse)
    {
      mouse_click_response = pixel_info_response;
    }
    return (ushort) result;
  }

  // In this case we set the region where the current sequence is
  // displayed to the region whose top-left corner is the point
  // where the mouse clicked.
  else if (mouse_click_response == MoveSequenceResponse)
  {
    MoveSequence(row, col);
    mouse_click_response = pixel_info_response;
  }

  else if (mouse_click_response == LineResponse)
  {
    AddVertex(row, col);
  }

  else if (mouse_click_response == CutoffLabelResponse)
  {
    ProcessCutoffClick(row, col);
  }

  else if (mouse_click_response == AddFeaturesResponse)
  {
    AddFeature(normalized_row, normalized_col);
  }
 
  else
  {
    exit_error("Impossible mouse click response\n");
  }

  return 1;
}


// See comments on the header file about this function.
long vInterpreter::InteractiveExecuteProgram()
{
  char buffer[1000];
  vector<char *> arguments;
  vPrint("\nEnter program and arguments (--- to finish):\n");
  vScan("%s", buffer);
  while(strcmp(buffer, "---") != 0)
  {
    arguments.push_back(vCopyString(buffer));
    vScan("%s", buffer);
  }

  // We need to make an explicit cast, because otherwise it doesn't
  // compile.
  long result = vExecuteProgram((vector<const char *> *) &arguments);

  vEraseCharpVector(&arguments);
  return result;
}


//////////////////////////////////////////////////////////
//                                                      //
// start of obligatory member functions for any module  //
//                                                      //
//////////////////////////////////////////////////////////


class_gesture_module::class_gesture_module(class_interpreter * interpreter,
                                           vGestureInterfaceGlobals * globals) :
vInterpreter(interpreter)
{
  Initialize (globals);
  initialize();
  sort_commands ();
  print ("common version\n");
}


class_gesture_module::~class_gesture_module()
{
  CleanUp();
  class_module::clean_up();
  clean_up();
}

long class_gesture_module::initialize()
{
  enter_command("acd_approx_loss", ACD_ApproximateLoss0);
  enter_command("acd_approx_rank", ACD_ApproximateRank0);
  enter_command("acd_loss_stats", ACD_LossStats0);
  enter_command("acd_nn", ACD_NN0);
  enter_command("acd_save", ACD_Save0);
  enter_command("acd_save_directed", ACD_SaveDirected0);
  enter_command("acd_set_inputs", ACD_SetInputs0);
  enter_command("acd_test", ACD_Test0);
  enter_command("acd_test_directed", ACD_TestDirected0);
  enter_command("acd_training_distances", AcdTrainingDistances);
  enter_command("add_all_features", AddAllFeatures);
  enter_command("add_feature", AddFeature0);
  enter_command("add_features", AddFeatures);
  enter_command("add_input", AddInput);
  enter_command("add_output", AddOutput);
  enter_command("add_output_stream", add_output_stream);
  enter_command("all_model_lines", AllModelLines0);
  enter_command("alt_hand_links", AltHandLinks0);
  enter_command("alt_hand_points", AltHandPoints0);
  enter_command("alt_hand_polygons", AltHandPolygons0);
  enter_command("amplify", AmplifyImage);
  enter_command("angle_cutoff", TrajectoryAngleCutoff);
  enter_command("articulated_edges", ArticulatedEdges);
  enter_command("average_regions", AverageRegions);
  enter_command("average_score", AverageLineMatchScore0);

  enter_command("background_stats", BackgroundStatistics);
  enter_command("band_max", BandMax0);
  enter_command("band_min", BandMin0);
  enter_command("bbox", BoundingBox0);
  enter_command("bbox_segmentation", BoundingBoxSegmentation);
  enter_command("best_segments", CountBestSegments0);
  enter_command("biggest_component", BiggestComponent);
  enter_command("binary", Binary);
  enter_command("blur", Blur);
  enter_command("bm_allow_lipschitz", BM_AllowLipschitz0);
  enter_command("bm_allow_negative", BM_AllowNegative0);
  enter_command("bm_allow_projections", BM_AllowProjections0);
  enter_command("bm_allow_removals", BM_AllowRemovals0);
  enter_command("bm_load", BM_Load0);
  enter_command("bm_basis_image", BM_BasisImage);
  enter_command("bm_point_image", BM_PointImage);
  enter_command("bm_print_classifier", BM_PrintClassifier);
  enter_command("bm_print_summary", BM_PrintSummary);
  enter_command("bm_print_all", BM_PrintAll);
  enter_command("bm_random", BM_Random0);
  enter_command("bm_scores", BM_Scores);
  enter_command("bm_step", BM_Step0);
  enter_command("bm_steps", BM_Steps0);
  enter_command("bm_training", BM_Training0);
  enter_command("bm_validation", BM_Validation0);
  enter_command("bourgain_index_errors", BourgainIndexErrors);
  enter_command("bourgain_save", BourgainSave);
  enter_command("bourgain_test", BourgainTest);
  enter_command("browse", BrowseImage0);

  enter_command("calculate_std", CalculateSTD);
  enter_command("calculate_stds", CalculateSTDs);
  enter_command("camera_format", SetCameraFormat);
  enter_command("camera_hand", ShowCameraHand);
  enter_command("camera_hand2", ShowCameraHand2);
  enter_command("camera_options", SetCameraOptions);
  enter_command("canonical_sizes", SetCanonicalSizes0);
  enter_command("canny", CannyDetector0a);
  enter_command("canny3", CannyDetector0b);
  enter_command("canny_threshold", CannyThreshold0);
  enter_command("cbi_bm", CBI_BM);
  enter_command("cbi_distance", CBI_Distance0);
  enter_command("cbi_distances", CBI_Distances);
  enter_command("cbi_fm", CBI_FM);
  enter_command("cbi_indices", CBI_Indices);
  enter_command("cbi_loadbm", CBI_LoadBM0);
  enter_command("cbi_random", CBI_Random0);
  enter_command("cbi_sm", CBI_SM0);
  enter_command("cbi_steps", CBI_Steps0);
  enter_command("cbi_training", CBI_Training0);
  enter_command("cbi_triples", CBI_Triples);
  enter_command("cbi_validation", CBI_Validation0);
  enter_command("ccode", CCodeDir);
  enter_command("cdata", CDataDir);
  enter_command("cd", ChangeDirectory);
  enter_command("ceb_embeddings", CEB_Embeddings);
  enter_command("ceb_load", CEB_Load0);
  enter_command("ceb_print", CEB_Print);
  enter_command("ceb_save", CEB_Save0);
  enter_command("ceb_scores", CEB_Scores);
  enter_command("ceb_create", CEB_Create);
  enter_command("center", Center0);
  enter_command("chamfer", ChamferDistances0);
  enter_command("chamfer2", ChamferDistancesSlow0);
  enter_command("chamfer_distances", ChamferDistances);
  enter_command("chamfer_do", Chamfer_DO_Scores);
  enter_command("chamfer_do_mi", Chamfer_DO_MI_Scores);
  enter_command("chamfer_features", TestChamferFeatures0);
  enter_command("chamfer_matching", ChamferMatching0);
  enter_command("chamfer_matching4", ChamferMatching40);
  enter_command("chamfer1", OneWayChamfer);
  enter_command("chamfer_outliers", ChamferIndexOutliers0);
  enter_command("chamfer_rankings", ChamferRankings);
  enter_command("chamfer_scores", ChamferScores0);
  enter_command("chamfer1_top", Chamfer1TopScores0);
  enter_command("chamfer_triangle", ChamferTriangleInequality);
  enter_command("chamfer_triangle2", ChamferTriangleInequality2);
  enter_command("chamfer1b", OneWayChamfer2);
  enter_command("chamfer_index_scores", ChamferIndexScores0);
  enter_command("chamfer_im_scores", ChamferIMIndexScores);
  enter_command("chamfer_index_scoresb", ChamferIndexScoresb0);
  enter_command("check_camera", CheckCamera);
  enter_command("choose_basis1", ChooseBasis1);
  enter_command("choose_basis2", ChooseBasis2);
  enter_command("choose_basis3", ChooseBasis3);
  enter_command("choose_basis4", ChooseBasis4);
  enter_command("choose_basis5", ChooseBasis5);
  enter_command("choose_basis6", ChooseBasis6);
  enter_command("choose_basis7", ChooseBasis7);
  enter_command("circular_distance", CircularDistance0);
  enter_command("clear", ClearImage);
  enter_command("clear_all_dts", DatabaseClearAllDTs);
  enter_command("clear_all_point_sequences", DatabaseClearAllPointSequences);
  enter_command("clear_dt", DatabaseClearDT0);
  enter_command("clear_inputs", ClearInputs);
  enter_command("clear_point_sequence", DatabaseClearPointSequence0);
  enter_command("color_distancse1", ColorDistances0a);
  enter_command("color_distances2", ColorDistances0b);
  enter_command("combine_logs", CombineLogs0a);
  enter_command("combine_logs2", CombineLogs0b);
  enter_command("combine_roots", CombineRoots0);
  enter_command("combine_seq_stats", CombineSequenceStats0);
  enter_command("combine_sums", CombineSums0);
  enter_command("combine_with_logs", CombineWithLogs0);
  enter_command("compare_dolvs", CompareDOLVS0);
  enter_command("compare_edge_histograms", CompareEdgeHistograms0);
  enter_command("compare_embeddings", CompareEmbeddings0);
  enter_command("compare_images", compare_images);
  enter_command("compare_inputs", CompareInputs0);
  enter_command("compare_match_model_lines", CompareMatchModelLines0);
  enter_command("complement", Complement0);
  enter_command("connect", ConnectedComponents);
  enter_command("conference_database_new", conference_database_new);
  enter_command("conference_database_delete", conference_database_delete);
  enter_command("conference_category", conference_category);
  enter_command("conference_filename", conference_filename);
  enter_command("conference_load_images", conference_load_images);
  enter_command("conference_distance_transforms", conference_distance_transforms);
  enter_command("conference_image", conference_image);
  enter_command("conference_transform", conference_transform);
  enter_command("conference_chamfer_features", conference_chamfer_features);
  enter_command("conference_one_chamfer", conference_one_chamfer);
  enter_command("conference_one_Hausdorff", conference_one_Hausdorff);
  enter_command("conference_edges", conference_edges);
  enter_command("conference_chamfer", conference_chamfer);
  enter_command("conference_Hausdorff", conference_Hausdorff);
  enter_command("conference_chamfer2", conference_chamfer2);
  enter_command("conference_Hausdorff2", conference_Hausdorff2);
  enter_command("conference_save", conference_save);
  enter_command("conference_save2", conference_save2);
  enter_command("connect8", ConnectedComponents8);
  enter_command("contour", OuterContour);
  enter_command("contour_line_votes", ContourLineVotes);
  enter_command("contour_s", SmoothOuterContour0);
  enter_command("contour_cuts", ContourCuts0);
  enter_command("contour_maxima", ContourMaxima0);
  enter_command("contour_regions", ContourRegions0);
  enter_command("contour_scores", ContourScores0);
  enter_command("contour_scores_fast", ContourScoresFast0);
  enter_command("convex_hull", ConvexHull);
  enter_command("convert_sequence", ConvertSequence0);
  enter_command("copy_band", CopyBand0);
  enter_command("copy_input", CopyInput0);
  enter_command("correct_match", CorrectMatch0);
  enter_command("correct_mean", SequenceCorrectMean0);
  enter_command("correct_median", SequenceCorrectMedian0);
  enter_command("correct_to_print", CorrectToPrint);
  enter_command("correct_under", SequenceCorrectUnder0);
  enter_command("correct_under2", SequenceCorrectUnder0b);
  enter_command("correct_within", SequenceCorrectWithin0);
  enter_command("count_gequal", CountGequal0);
  enter_command("count_color", CountColor0);
  enter_command("count_trajectories", CountTrajectories);
  enter_command("count_lequal", CountLequal0);
  enter_command("count_model_lines", CountAllModelLines0);
  enter_command("count_non_zero", CountNonZero);
  enter_command("count_segments", CountLineSegments0);
  enter_command("count_value", CountValue0);
  enter_command("create_cartoon_sequence", CreateCartoonSequence);
  enter_command("create_database", CreateDatabase);
  enter_command("create_dt_sequence", CreateDTSequence0);
  enter_command("create_image_sequence", CreateImageSequence);
  enter_command("create_point_sequence", CreatePointSequence0);
  enter_command("cropped_uchar", CroppedUchar);
  enter_command("current", ShowCurrentFrame);
  enter_command("current_dt", DatabaseSetCurrentDT0);
  enter_command("current_point_sequence", DatabaseSetCurrentPointSequence0);
  enter_command("cut_triangle", CutTriangle0);
  enter_command("database_edges3", DatabaseEdges3);
  enter_command("database_edges2", DatabaseEdges2);
  enter_command("database_edge_image", DatabaseEdgeImage0);
  enter_command("database_frame", DatabaseFrame0);
  enter_command("database_image3", DatabaseImage3);
  enter_command("database_image2", DatabaseImage2);
  enter_command("database_inputs", SetDatabaseInputs0);
  enter_command("database_name", DatabaseName);
  enter_command("database_pose2", DatabasePose2);
  enter_command("database_pose", DatabasePose);
  enter_command("database_rotate", DatabaseRotate);
  enter_command("dcode", DCodeDir);
  enter_command("ddata", DDataDir);
  enter_command("debug_skin_table", DebugSkinTable0);
  enter_command("decres", DecreaseResolution);
  enter_command("default", default_input0);
  enter_command("default_moments", DefaultMomentsFile);
  enter_command("delete_bm", DeleteBM);
  enter_command("delete_cbi", DeleteCBI);
  enter_command("delete_dts", DeleteDistanceTransforms);
  enter_command("delete_sbi", DeleteSBI);
  enter_command("depth1", DepthNeighbors);
  enter_command("delete_lines", DestroyLineSet);
  enter_command("diameter", Diameter);
  enter_command("dilate", DilateImage);
  enter_command("direction_changes", DirectionChanges);
  enter_command("direction_histogram", DirectionHistogram);
  enter_command("dolvs_bin", DOLVS_Bin0);
  enter_command("dolvs_entry", DOLVS_Entry0);
  enter_command("dolvs_gequal", DOLVS_Gequal0);
  enter_command("dolvs_lequal", DOLVS_Lequal0);
  enter_command("dolvs_to_pdf", DOLVS_To_PDF);
  enter_command("dolvs_pdf", DOLVS_PDF0);
  enter_command("dolvs_pdf2", DOLVS_PDF2);
  enter_command("dolvs_stats", DOLVS_Stats);
  enter_command("dolvs_stats2", DOLVS_Stats2);
  enter_command("dolvs_value", DOLVS_Value0);
  enter_command("dominant_orientation1", DominantOrientation1);
  enter_command("domt", DominanceTransform);
  enter_command("double_size", DoubleSize);
  enter_command("draw_circle", DrawCircle);
  enter_command("draw_extended_cutoff", DrawExtendedCutoff0);
  enter_command("draw_fcircle", DrawFilledCircle);
  enter_command("draw_frectangle", DrawFilledRectangle);
  enter_command("draw_line", DrawLine);
  enter_command("draw_line2", DrawLineB);
  enter_command("draw_match2", DrawLineMatch);
  enter_command("draw_quad", DrawQuadrilateral0);
  enter_command("draw_quad2", DrawQuadrilateral0b);
  enter_command("draw_rectangle", DrawRectangle);
  enter_command("draw_segment", DrawLineSegment0);
  enter_command("draw_triangle", DrawTriangle0);
  enter_command("dt_slow", DistanceTransformSlow);
  enter_command("dt", DistanceTransform);
  enter_command("edata", RDataDir);
  enter_command("edge_gradients", EdgePixelGradients0);
  enter_command("edge_histogram", EdgeHistogram0a);
  enter_command("edge_histogram_b", EdgeHistogram0b);
  enter_command("edge_hist_scores_mean", MeanEdgeHistogramScores);
  enter_command("edge_hist_scores_min", MinEdgeHistogramScores);
  enter_command("edge_orientations", EdgeOrientations);
  enter_command("edge_orientations2", EdgeOrientations2);
  enter_command("edge_stat_vote1", EdgeStatisticsVote0a);
  enter_command("edge_stat_vote2", EdgeStatisticsVote0b);
  enter_command("edge_stat_vote4", EdgeStatisticsVote0d);
  enter_command("edges", UndirectedEdges);
  enter_command("ellipse0", FitEllipse0);
  enter_command("ellipse", FitEllipse);
  enter_command("elongation_statistics", ElongationStatistics0);
  enter_command("embedding_dts", EmbeddingDTs);
  enter_command("enclosing_circle", EnclosingCircle);
  enter_command("enclosing_circle2", EnclosingCircle2);
  enter_command("enclosing_circle_loop", EnclosingCircleLoop);
  enter_command("erase_features", EraseFeatures);
  enter_command("erase_scores", EraseScores);
  enter_command("estimate_foreground", EstimateForeground);
  enter_command("existential_dolvs", ExistentialDOLVS0);
  enter_command("existential_dolvs2", ExistentialDOLVS02);
  enter_command("extend_cutoffs", ExtendCutoffs);
  enter_command("face_color", SetFaceColor0);
  enter_command("fastmap_filter_refine", FastMapFilterRefine);
  enter_command("fastmap_index_errors", FastMapIndexErrors);
  enter_command("fastmap_knn", FastMapKnn);
  enter_command("fdata", FDataDir);
  enter_command("features_to_track", FeaturesToTrack);
  enter_command("fcombine_logs", FilterCombineLogs0);
  enter_command("file_copy", function_file_copy);
  enter_command("fill", FillRegion0);
  enter_command("fill_up_holes", FillUpHoles);
  enter_command("filter_with_fingers", FilterScoresUsingFingers0);
  enter_command("filter_with_logs", FilterScoresUsingLogs0);
  enter_command("filter_with_moments", FilterScoresUsingMoments0);
  enter_command("find_faces", DetectFaces0);
  enter_command("find_fingers", 	FindFingers0);
  enter_command("find_hands", FindHands);
  enter_command("find_hands2", FindHands2);
  enter_command("find_hands3", FindHands3);
  enter_command("find_segment", FindSegment0);
  enter_command("find_trajectories", FindTrajectories);
  enter_command("finger_contours", FingerContours);
  enter_command("fine_segmentation", FineSegmentation);
  enter_command("finger_edges", FingerEdges0);
  enter_command("finger_lines", FingerLines);
  enter_command("finger_lines2", FingerLines2);
  enter_command("finger_score", FingerScore);
  enter_command("finger_scores", FingerScores);
  enter_command("fit_line", FitLine0);
  enter_command("fm_basis_image", FM_BasisImage);
  enter_command("fm_distances", FM_Distances);
  enter_command("fm_embeddings", FM_TrainingEmbedding);
  enter_command("fm_line_projection", FM_LineProjection0);
  enter_command("fm_load_embedding", FM_LoadEmbedding0);
  enter_command("fm_point_image", FM_PointImage);
  enter_command("fm_pivots", FM_Pivots);
  enter_command("fm_print_info", FM_PrintInfo);
  enter_command("fm_scores", FM_Scores);
  enter_command("fm_test_coords", FM_TestCoords);
  enter_command("fm_test_embedding", FM_TestEmbedding);
  enter_command("fm_training_coords", FM_TrainingCoords0);
  enter_command("fmdelete", FmDelete);
  enter_command("fmembed_test", FmEmbedTest);
  enter_command("fmembed_train", FmEmbedTraining);
  enter_command("fmnew2", FmMake20);
  enter_command("fmnew3", FmMake30);
  enter_command("fmpivots", FmPrintPivots);
  enter_command("fmsample", FmPrintSample0);
  enter_command("fmsave", FmSave0);
  enter_command("fmstep", FmNextStep);
  enter_command("fmsteps", FmNextSteps0);
  enter_command("fmtest", FmTest);
  enter_command("fmtraining", FmPrintTraining0);
  enter_command("foreground", SubtractBackground0);
  enter_command("foreground_hand", ForegroundHand);
  enter_command("foreground_threshold", ForegroundThreshold0);
  enter_command("forward", ForwardImageBuffer0);
  enter_command("frame", JumpToFrame0);
  enter_command("frame_match_stats", FrameMatchStats0);
  enter_command("front_distance", FrontDistance);

  enter_command("gaussj", TestGaussj);
  enter_command("gaussj2", TestGaussj2);
  enter_command("get_dt", DatabaseGetDT0);
  enter_command("giorgos2", DistanceMatrix);
  enter_command("giorgos2_check", CheckDistanceMatrix);
  enter_command("giorgos3", OldGroundDistanceMatrix);
  enter_command("giorgos3_check", CheckOldGroundDistanceMatrix);
  enter_command("goto_match", JumpToMatch);
  enter_command("grab_frame", GrabFrame);
  enter_command("gradient_norms", GradientNorms);
  enter_command("gray", ToGray);
  enter_command("gray_bound", ToGrayBound);
  enter_command("grid_fraction", GridFraction0);
  enter_command("ground_distance", SetGroundTruthDistance0);
  enter_command("ground_match", GroundMatch0);
  enter_command("ground_mean", SequenceGroundMean0);
  enter_command("ground_median", SequenceGroundMedian0);
  enter_command("ground_stats", GroundTruthStatistics0);
  enter_command("ground_view", SetGroundTruthView0);
  enter_command("ground_under", SequenceGroundUnder0);
  enter_command("ground_under2", SequenceGroundUnder0);
  enter_command("ground_truth_distances", GroundTruthDistances);
  enter_command("half_plane", HalfPlane0);
  enter_command("hand_database", SetHandDatabase0);
  enter_command("hand_edge_image", SetHandEdgeImage);
  enter_command("hand_images", HandAndEdgeImages);
  enter_command("hand_links", HandLinks0);
  enter_command("hand_links_slow", HandLinksSlow0);
  enter_command("hand_points", HandPoints0);
  enter_command("hand_points_slow", HandPointsSlow0);
  enter_command("hand_polygons", HandPolygons0);
  enter_command("hand_polygons_slow", HandPolygonsSlow0);
  enter_command("hand_threshold", SetHandThreshold0);
  enter_command("hand_track_1", HandTrackSequence);
  enter_command("hand_track_n", HandTrackSequences);
  enter_command("hand_tracker_propagate", HandTrackerPropagate);
  enter_command("hand_tracker_scores", HandTrackerScores);
  enter_command("heon", histogram_equalization_on);
  enter_command("heoff", histogram_equalization_off);
  enter_command("histogram", Histogram0);
  enter_command("histogram3", Histogram30);
  enter_command ("histogram_equalization", histogram_equalization);
  enter_command ("histogram_equalization_offset", histogram_equalization_offset);
  enter_command ("histogram_equalization_size", histogram_equalization_size);
  enter_command ("histogram_equalization_weights", histogram_equalization_weights_test);
  enter_command("hold_scores", HoldScores);

  enter_command ("image_absolute", image_absolute);
  enter_command("image_add", image_add);
  enter_command ("image_average", image_average);
  enter_command("image_diff", image_diff);
  enter_command("image_divide", image_divide);
  enter_command("image_max", ImageMax0);
  enter_command("image_median", image_median);
  enter_command("image_min", ImageMin0);
  enter_command("image_multiply", image_multiply);
  enter_command("image_subtract", image_subtract);
  enter_command ("image_total", image_total);
  enter_command("incres", IncreaseResolution);
  enter_command("ininfo", PrintInputs);
  enter_command("init_hand_tracker", InitializeHandTracker);
  enter_command("init_skin_tracker", InitializeSkinTracker);
  enter_command("input",  SetCurrentInput);
  enter_command("input_face_color", InputFaceColor0);
  enter_command("ithr", InverseThresholdImage);
  enter_command("keep_palm", KeepPalm);
  enter_command("keep_palm2", KeepPalm2);
  enter_command("keep_palm3", KeepPalm3);
  enter_command("keep_palm4", KeepPalm4);
  enter_command("kth_stats", KthRankStats0);
  enter_command("kth_test", TestKthSmallest0);
  enter_command("left_hand", LeftHand);
  enter_command("leftover_model", ShowLeftoverModelPoints0);
  enter_command("leftover_test", ShowLeftoverTestPoints0);
  enter_command("line_endpoints", LineEndpoints0);
  enter_command("line_match_scores", LineMatchScores0);
  enter_command("line_chamfer", LineChamferScores0);
  enter_command("line_number_stats", LineNumberStats0);
  enter_command("line_scores", LineMatchStats0);
  enter_command("line_segment_images", LineSegmentImages0);
  enter_command("line_segments", GetLineSegments0);
  enter_command("line_segments3", GetLineSegments03);
  enter_command("line_segments4", GetLineSegments04);
  enter_command("line_segment_matches", LineSegmentMatches0);
  enter_command("line_voronoi", LineVoronoi);
  enter_command("link_boundaries", LinkBoundaries);
  enter_command("link_ids", LinkIDs);
  enter_command("link_matrices", LinkMatrices0);
  enter_command("lipschitz_basis", MakeLipschitzBasis);
  enter_command("lipschitz_vectors", LipschitzVectors0);
  enter_command("load_boost_map", LoadBoostMap0);
  enter_command("load_cbi", LoadCBI0);
  enter_command("load_cd", Load_All_CD_Distances);
  enter_command("load_cdp", Load_CD_Pairs);
  enter_command("load_combined_scores", LoadCombinedScores);
  enter_command("load_dolvs", LoadFrameDOLVS0);
  enter_command("load_dt", DatabaseLoadDT0);
  enter_command("load_edge_histograms", LoadEdgeHistograms);
  enter_command("load_edge_statistics", LoadEdgeStatistics0);
  enter_command("load_edge_summary", LoadEdgeSummary);
  enter_command("load_extended_cutoffs", LoadExtendedCutoffs);
  enter_command("load_finger_info", LoadFingerInfo);
  enter_command("load_im", Load_All_IM_Distances);
  enter_command("load_line_match", LoadLineMatchFrameStats0);
  enter_command("load_mi", Load_All_MI_Distances);
  enter_command("load_orientations", LoadEdgeOrientations);
  enter_command("load_model_lines", LoadModelLines0);
  enter_command("load_palm_cutoffs", LoadCutoffs);
  enter_command("load_point_set", LoadPointSet);
  enter_command("load_point_sequence", DatabaseLoadPointSequence0);
  enter_command("load_random_embedding", LoadRandomEmbedding);
  enter_command("load_training_embedding", LoadTrainingEmbedding);
  enter_command("load_scores", LoadMatchScores);
  enter_command("load_seq_stats", LoadSequenceStats0);
  enter_command("local_shadow_score", LocalShadowScore0);
  enter_command("log_score_stats", LogScoreStats0);

  enter_command("make_cbi", MakeCBI0);
  enter_command("make_cdp", Make_CD_Pairs);
  enter_command ("make_short", make_short);
  enter_command ("make_ushort", make_ushort);
  enter_command ("make_long", make_long);
  enter_command ("make_ulong", make_ulong);
  enter_command ("make_float", make_float);
  enter_command ("make_double", make_double);
  enter_command("mass_center", CenterOfMass);
  enter_command("match_distance", MatchDistance0);
  enter_command("match_model_lines", MatchModelLines0);
  enter_command("make_sbi", MakeSBI0);
  enter_command("match_segment", MatchLineSegment0);
  enter_command("match_segment2", MatchLineSegment02);
  enter_command("match_segment3", MatchLineSegment03);
  enter_command("match_segment4", MatchLineSegment04);
  enter_command("match_segment_vs", MatchLineSegmentVS0);
  enter_command("matrix_info", MatrixInfo);
  enter_command("max_neighbors", MaxNeighbors);
  enter_command("maximal_motion", FindMaximalMotion);
  enter_command("mclines", MatchCorrectLines0);
  enter_command("mec", EnclosingCircle);
  enter_command("merge_random_distances", MergeRandomDistances0);
  enter_command("merge_real_poser1", MergeRealPoser1);
  enter_command("merge_vertically_float", MergeVerticallyFloat);
  enter_command("mglines", MatchGroundLines0);
  enter_command("mmlines", MatchMatchLines0);
  enter_command("mmlines2", MatchModelLines0b);
  enter_command("mnist_candidates", MnistCandidates);
  enter_command("mnist_chamfer", MnistChamfer0);
  enter_command("mnist_dataset2", MnistDataset20);
  enter_command("mnist_dataset4", MnistDataset40);
  enter_command("mnist_labels", MnistLabels);
  enter_command("mnist_load", MnistLoad0);
  enter_command("mnist_load3", MnistLoad30);
  enter_command("mnist_show", MnistShow0);
  enter_command("mnist_testtrain", MnistTestTrain);
  enter_command("mnist_training", MnistTraining);
  enter_command("mnist_traintrain", MnistTrainTrain);
  enter_command("mnist_validation", MnistValidation);
  enter_command("mnist_whole", MnistWholeDataset);
  enter_command("moment_index", SetMomentIndex0);
  enter_command("moment_scores", MomentScores);
  enter_command("moments", ImageMoments);
  enter_command("moments_file", InputMomentsFile);
  enter_command("motion", ComputeMotion0);
  enter_command("mouse_features", MouseAddFeatures);
  enter_command("move_sequence", MoveSequence);
  enter_command("multi_pick_scores", MultiPickScores0);
  enter_command("multiply", MultiplyImage);

  enter_command("new_image", NewImage);
  enter_command("next", ShowNextImage);
  enter_command("nn_rank_stats", NN_RankStats0);
  enter_command("non_maxima", NonMaximaSuppression);
  enter_command("normalized_coordinates", PrintNormalizedCoordinates0);
  enter_command("norm_mec1", NormalizeMEC1);
  enter_command("norm_mec3", NormalizeMEC3);
  enter_command("ns", LoadNextScores);
  enter_command("nsc", LoadCurrentScores);
  enter_command("ns1", LoadMatchScoresN0);
  enter_command("normalize_intensity", NormalizeIntensity);

  enter_command("old_fast_pose1", OldTestFastPoseFrame1);
  enter_command("old_fast_pose2", OldTestFastPoseFrame2);
  enter_command("old_line_orientation", OldLineOrientation0);
  enter_command("old_qview_test", OldQuaternionViewTest);
  enter_command("old_rotation_distance", OldRotationDistance0);
  enter_command("old_sphere_points", OldSampleSpherePoints);
  enter_command("old_sphere_points2", OldSampleSpherePoints2);
  enter_command("old_sphere_surface_to_3D", OldSphereSurfaceTo3D0);
  enter_command("old_verify_plane_intersection", OldVerifyPlaneIntersection0);
  enter_command("old_verify_plane_intersection2", OldVerifyPlaneIntersection20);
  enter_command("old_view_distance", OldViewDistance0);
  enter_command("old_view_distance2", OldViewDistance2);
  enter_command("old_viewpoints", OldSampleViewpoints);
  enter_command("old_viewpoints2", OldSampleViewpoints2);
  enter_command("old_viewpoints3", OldSampleViewpoints3);
  enter_command("old_3D_to_sphere_surface", OldThreeDToSphereSurface0);
  enter_command("online_methods", OnlineMethods0);
  enter_command("online_ranking", OnlineRanking);
  enter_command("optical_flow", OpticalFlow);
  enter_command("oriented_lines", OrientedLines0);
  enter_command("oriented_lines2", OrientedLines0b);
  enter_command("oriented_lines3", OrientedLines0c);
  enter_command("oriented_lines4", OrientedLines0d);
  enter_command("oriented_lines_d", OrientedLinesD0);
  enter_command("original_frame", ShowOriginalFrame);
  enter_command("overlay", Overlay0);

  enter_command("pad", Pad);
  enter_command("pair_match", LinePairMatch0);
  enter_command("palm_cutpoint", PalmCutpoint);
  enter_command("palm_cutpoint2", PalmCutpoint2);
  enter_command("palm_scores", PalmScores);
  enter_command("pca_dimensions", InputPCAUsedDimensions);
  enter_command("pca_file", InputPCAFile);
  enter_command("pca_moment_scores", PCAMomentScores);
  enter_command("pca_stats", PCAScoreStats);
  enter_command("pcf", toggle_preserve_current_frame);
  enter_command("pdf_entry", PDF_Entry0);
  enter_command("pdf_prob", PDF_Probability0);
  enter_command("permutation", Permutation0);
  enter_command("pets_classify", PETS_Classify);
  enter_command("pixel", PrintPixelInfo);
  enter_command("pixel_gradient", PixelGradient0);
  enter_command("pixor_test", PixelOrientationsTest);
  enter_command("point_bm", PointBM0);
  enter_command("point_fm", PointFM0);
  enter_command("point_image", DatabasePointImage0);
  enter_command("point_sm", PointSM0);
  enter_command("pose_edges", PoseEdges);
  enter_command("pose_frame", DatabasePoseFrame);
  enter_command("pose_frame1", Database1PoseFrame);
  enter_command("pose_image", PoseImage);
  enter_command("poser_crop", PoserCrop);
  enter_command("poser1_chamfer", Poser1ChamferScores);
  enter_command("poser1_pose", Poser1Pose0);
  enter_command("poser_viewpoints", PoserSampleViewpoints);
  enter_command("poser_viewpoints2", PoserSampleViewpoints2);
  enter_command("poser_viewpoints3", PoserSampleViewpoints3);
  enter_command("positive_weights", PositiveWeights);
  enter_command("precompute_sqrtf", PrecomputeSqrtf);
  enter_command("precomputed_triangle", PrecomputedTriangle0);
  enter_command("preprocess_polygons", PreprocessPolygons);
  enter_command("previous", ShowPreviousImage);
  enter_command("previous_scores", SetPreviousScores);
  enter_command("principal_orientation", PrincipalOrientation);
  enter_command("print_references", PrintReferenceImages);
  enter_command("print_seq_stats", PrintSequenceStats);
  enter_command("program", InteractiveExecuteProgram);
  enter_command("projections", DrawProjections);
  enter_command("psqd", PsQueryDatabase);
  enter_command("psqd_distance", PsQdDistance);
  enter_command("psqq", PsQueryQuery);
  enter_command("psqq_distance", PsQqDistance);
  enter_command("ps_ref2", PsRef2);
  enter_command("ps_test2", PsTest2);
  enter_command("ps_train1", PsTrain1);

  enter_command("quick_chamfer_scores", QuickChamferScores);

  enter_command("random", RandomFrame);
  enter_command("random_chamfer_distances", RandomChamferDistances0);
  enter_command("random_edges", RandomEdges);
  enter_command("random_image", random_image);
  enter_command("randomize0", Randomize0);
  enter_command("randomize1", Randomize1);
  enter_command("random_quad", RandomQuadrilateral);
  enter_command("random_segmentation", RandomSegmentation);
  enter_command("range", SelectRange0);
  enter_command("rank_views", RankViews);
  enter_command("rank_views2", RankViews2);
  enter_command("rank_views3", RankViews3);
  enter_command("rank_views4", RankViews4);
  enter_command("rank_views5", RankViews5);
  enter_command("rank_views6", RankViews6);
  enter_command("rank_views6b", RankViews6b);
  enter_command("rank_views7", RankViews7);
  enter_command("rank_views8", RankViews8);
  enter_command("rank_views9", RankViews9);
  enter_command("rank_views10", RankViews10);
  enter_command("rank_views11", RankViews11);
  enter_command("rank_views12", RankViews11);
  enter_command("rank_views13", RankViews13);
  enter_command("rank_views14", RankViews14);
  enter_command("rank_views15", RankViews15);
  enter_command("rank_views16", RankViews16);
  enter_command("rank_views17", RankViews17);
  enter_command("rank_views18", RankViews18);
  enter_command("rank_views19", RankViews19);
  enter_command("rank_views20", RankViews20);
  enter_command("rank_views21", RankViews21);
  enter_command("rank_views22", RankViews22);
  enter_command("rank_views23", RankViews23);
  enter_command("rank_views24", RankViews24);
  enter_command("rank_views25", RankViews25);
  enter_command("rank_views26", RankViews26);
  enter_command("rank_views27", RankViews27);
  enter_command("rank_views28", RankViews28);
  enter_command("rank_views29", RankViews29);
  enter_command("rank_views30", RankViews30);
  enter_command("rank_views31", RankViews31);
  enter_command("rank_views32", RankViews32);
  enter_command("rank_views33", RankViews33);
  enter_command("rcode", RCodeDir);
  enter_command("rdata", RDataDir);
  enter_command("read_bbox", ReadBoundingBox0);
  enter_command("read_big", ReadBigDistances0);
  enter_command("read_max_entry", ReadMaxEntry);
  enter_command("read_model_lines", ReadModelLines0);
  enter_command("real_sequence", RealSequence);
  enter_command("record_max_entry", RecordMaxEntry);
  enter_command("recreate_old_edge_sequence", RecreateOldEdgeSequence);
  enter_command("redraw", RedrawWindow);
  enter_command("recenter", Recenter);
  enter_command("reload_database", ReloadHandDatabase);
  enter_command("remove_forearm", RemoveForearm);
  enter_command("remove_input", RemoveCurrentInput);
  enter_command("remove_output", RemoveCurrentOutput);
  enter_command("replace_color", ReplaceColor0);
  enter_command("reset_bounds", ResetBounds);
  enter_command("reset_magnified", ResetMagnifiedSize);
  enter_command("resize", resize);
  enter_command("resize_image", ResizeImage0);
  enter_command("resize_holder", ResizeScoresHolder);
  enter_command("rewind", RewindImageBuffer0);
  enter_command("right_hand", RightHand);
  enter_command("root_score_stats", RootScoreStats0);
  enter_command("rotate", RotateImage);
  enter_command("rotleft", RotateLeft0);
  enter_command("rotright", RotateRight0);
  enter_command("rotation_template", DatabaseRotationTemplate0);

  enter_command("sample_segments", SampleLineSegments0);
  enter_command("save_bboxes", SaveBoundingBoxes);
  enter_command("save_cbi", SaveCBI);
  enter_command("save_cdp", Save_CD_Pairs);
  enter_command("save_chamfer1_index", SaveChamfer1Index);
  enter_command("save_combined_scores", SaveCombinedScores);
  enter_command("save_database_bounding_boxes", SaveDatabaseBoundingBoxes);
  enter_command("save_dolvs", SaveDOLVS0);
  enter_command("save_dolvs_stats", SaveDOLVSStats0);
  enter_command("save_dts", SaveDTs);
  enter_command("save_edge_histograms", SaveEdgeHistograms0);
  enter_command("save_edge_histogram_scores", SaveEdgeHistogramScores0);
  enter_command("save_edge_images", SaveEdgeImages0);
  enter_command("save_edge_statistics", SaveEdgeStatistics0);
  enter_command ("save_filtered_frames", save_filtered_frames);
  enter_command("save_finger_info", SaveFingerInfo0);
  enter_command("save_im", Save_All_IM_Distances);
  enter_command("save_mecs", SaveMECs);
  enter_command("save_mi", Save_All_MI_Distances);
  enter_command("save_model_leftovers", SaveLeftoverSet);
  enter_command("save_model_lines", SaveModelLines0);
  enter_command("save_old_format", SaveOldFormat0);
  enter_command("save_orientations", SaveEdgeOrientations);
  enter_command("save_point_set", SavePointSet);
  enter_command("save_points", SaveDatabasePoints);
  enter_command("save_reference_distances", SaveReferenceDistances);
  enter_command("save_transpose", TransposeMatrix);
  enter_command("sbi_bm", SBI_BM);
  enter_command("sbi_bm_stats", SBI_BM_Stats0);
  enter_command("sbi_bm_stats4", SBI_BM_Stats0d);
  enter_command("sbi_bm_hrcm", SBI_BM_AHRCM0d);
  enter_command("sbi_dataset", SbiDataset);
  enter_command("sbi_fm", SBI_FM);
  enter_command("sbi_fm_stats", SBI_FM_Stats0);
  enter_command("sbi_fm_hrcm", SBI_FM_AHRCM0d);
  enter_command("sbi_fm_stats4", SBI_FM_Stats0d);
  enter_command("sbi_hrcm", SBI_HRCM0);
  enter_command("sbi_print", SBI_Print);
  enter_command("sbi_sm", SBI_SM0);
  enter_command("sbi_sm_stats4", SBI_SM_Stats0d);
  enter_command("sbi_sm_hrcm", SBI_SM_AHRCM0d);
  enter_command("sbi_training", SBI_Training0);
  enter_command("scale_bbox", ScaleBoundingBox0);
  enter_command("scale_bbox2", ScaleBoundingBox0b);
  enter_command("scale_down1", ScaleImageA0);
  enter_command("scale_down2", ScaleImageB0);
  enter_command("scale_image", ScaleImage0);
  enter_command("sc_add_classifiers", ScAddClassifiers);
  enter_command("sc_align_index", ScAlignIndexError);
  enter_command("sc_align_index_errors", ScAlignIndexErrors);
  enter_command("sc_check_triple", ScCheckTriple);
  enter_command("sc_classifier_distance", ScClassifierDistance);
  enter_command("sc_classifier_distances", ScClassifierDistances);
  enter_command("sc_delete_alignments", DeleteAlignments);
  enter_command("sc_load_alignments", LoadAlignments);
  enter_command("sc_load_classifiers", BmLoadScClassifiers);
  enter_command("sc_prototype_index_errors", ScPrototypeIndexErrors);
  enter_command("sc_save_classifiers", ScSaveClassifiers);
  enter_command("sc_summary_fr", ScSummaryFilterRefine);
  enter_command("sc_summary_index_errors", ScSummaryIndexErrors);
  enter_command("sdata", SDataDir);
  enter_command("score_base", SetScoreBase0);
  enter_command("score_test_file", SetTestFilename0);
  enter_command("score_test_frame", SetTestFrame0);
  enter_command("scores_to_binary", ScoresToBinary0);
  enter_command("scores_to_use", SetScoresToUse);
  enter_command("segment_hand_a", SegmentHand_A);
  enter_command("segment_hand_b", SegmentHand_B);
  enter_command("segment_hand_c", SegmentHand_B);
  enter_command("select_all", SequenceSelectAll);
  enter_command("select_bbox", SelectBoundingBox0);
  enter_command("select_color", SelectColor0);
  enter_command("select_component", SelectComponent);
  enter_command("select_front_distance", SequenceSelectFrontDistance0);
  enter_command("select_pixels", SelectPixels0);
  enter_command("select_range", SequenceSelectRange0);
  enter_command("select_range_front_distance", SequenceSelectRangeDistance0);
  enter_command("select_segments", SelectLineSegments0);
  enter_command("select_viewpoint", SequenceSelectViewpoint0);
  enter_command("sequence_double_size", SequenceDoubleSize);
  enter_command("sequence_match_stats", SequenceMatchStats0);
  enter_command("sequence_moments_a", SequenceImageMoments_a);
  enter_command("sequence_moments_b", SequenceImageMoments_b);
  enter_command("set_bounds", SetBounds);
  enter_command("set_distance", SetDistance0);
  enter_command("set_distance_index", SetDistanceIndex0);
  enter_command("set_magnified", SetMagnifiedSize0);
  enter_command("set_pixel", SetPixel);
  enter_command("set_random_dimensions", SetRandomDimensions0);
  enter_command("setres", SetResolution0);
  enter_command("set_scale", SetScale0);
  enter_command("set_edge_image", SetEdgeImage);
  enter_command("shadow_score", ShadowScore);
  enter_command("shape_matrices", ShapeMatrices0);
  enter_command("show_background", ShowBackground);
  enter_command("show_cd", Show_CD_Entry);
  enter_command("show_cdp", Show_CD_Pair);
  enter_command("show_cd4", Show_CD_Entry4);
  enter_command("show_embedding", ShowEmbedding);
  enter_command("show_im", Show_IM_Entry);
  enter_command("show_mi", Show_MI_Entry);
  enter_command("show_orientations", ShowOrientations0);
  enter_command("show_trajectory", ShowTrajectory);
  enter_command("show_trajectories", ShowTrajectories);
  enter_command("side_length_stats", SideLengthStats);
  enter_command("single_pick_scores", SinglePickScores0);
  enter_command("skeleton", GrowSkeleton);
  enter_command("skin", DetectSkin);
  enter_command("skin_hand", SkinHand);
  enter_command("skin_segment_a", SkinSegmentHand0a);
  enter_command("skin_segment_b", SkinSegmentHand0b);
  enter_command("skin_segment_c", SkinSegmentHand0c);
  enter_command("skin_segment_sequence_a", SkinSegmentSequence0a);
  enter_command("skin_segment_sequence_b", SkinSegmentSequence0b);
  enter_command("skin_segment_sequence_c", SkinSegmentSequence0c);
  enter_command("skin_segment_sequence_d", SkinSegmentSequence0d);
  enter_command("skin_track_frame", SkinTrackNextFrame);
  enter_command("sleep", WaitSeconds0);
  enter_command("sm_features", SM_Features0);
  enter_command("sm_info", SparseMapInfo);
  enter_command("sm_load_embedding", SM_LoadEmbedding0);
  enter_command("sm_save_embedding", SM_SaveEmbedding0);
  enter_command("sm_set", SM_ReferenceSet0);
  enter_command("sm_scores", SM_Scores);
  enter_command("sm_test_coord", SM_TestCoordinate0);
  enter_command("sm_training_coord", SM_TrainingCoordinate0);
  enter_command("sparsemap_index_errors", SparseMapIndexErrors);
  enter_command("sparsemap_save", SparseMapSave);
  enter_command("start_camera", StartCamera);
  enter_command("start_labeling_cutoffs", StartLabelingCutoffs);
  enter_command("step", SetSequenceStep);
  enter_command("step0_views", SetStep0Views0);
  enter_command("step1_views", SetStep1Views0);
  enter_command("stop_camera", StopCamera);
  enter_command("stop_labeling_cutoffs", StopLabelingCutoffs);
  enter_command("straight_links", StraightLinks);
  enter_command("straight_links2", StraightLinks0);
  enter_command("sum_score_stats", SumScoreStats0);
  enter_command("sync", SynchronizeSequences);

  enter_command("ten_matches", ShowTenMatches0);
  enter_command("test_chamfer1", TestOneWayChamfer);
  enter_command("test_chamfer1_index", TestChamfer1Index);
  enter_command("test_cutoff_centering", TestCutoffCentering);
  enter_command("test_do_dt", Test_DO_Transform);
  enter_command("test_edges_orientations", TestEdgesOrientations);
  enter_command("test_embedding", PrintTestEmbedding);
  enter_command("test_index", TestIndexAdjustment);
  enter_command("test_insert_distance", TestInsertDistance);
  enter_command("test_leftovers", TestLeftoverSet0);
  enter_command("test_line_cols", TestLineCols0);
  enter_command("test_line_grid", TestLineGrid0);
  enter_command ("test_line_pixels", test_line_pixels);
  enter_command("test_line_points", TestLinePoints0);
  enter_command("test_line_points5", TestLinePoints50);
  enter_command("test_point_set", TestPointSet0);
  enter_command("test_prepare_chamfer", TestPrepareChamfer);
  enter_command("test_sample", TestSample0);
  enter_command("test_scale_x", TestScaleX0);
  enter_command("test_search_region", TestSearchRegion0);
  enter_command("test_samples", TestLineSegmentSamples0);
  enter_command("test_trajectory", TrajectorySampleTest);
  enter_command("tetr_chamfer", TestTrainChamfer);
  enter_command("threshold", ThresholdCurrentImage);
  enter_command("time_dt", TimeDT0);
  enter_command("time_exp", TimeExp0);
  enter_command("time_gaussj2", TimeGaussj2);
  enter_command("time_hand_links", TimeHandLinks0);
  enter_command("time_hand_links_slow", TimeHandLinksSlow0);
  enter_command("time_hand_points", TimeHandPoints0);
  enter_command("time_hand_points_slow", TimeHandPointsSlow0);
  enter_command("time_hand_polygons", TimeHandPolygons0);
  enter_command("time_hand_polygons_slow", TimeHandPolygonsSlow0);
  enter_command("time_image", TimeImage0);
  enter_command("time_pexp", TimePrecomputedExp0);
  enter_command("time_precomputed_sqrtf", TimePrecomputedSqrtf);
  enter_command("time_sqrtf", TimeSqrtf);
  enter_command("time_sts", TimeSlowTrajectorySampling);
  enter_command("time_triangle_pixels", TimeTrianglePixels0);
  enter_command("time_triangle_pixels2", TimeTrianglePixels0b);
  enter_command("time_ts", TimeTrajectorySampling);
  enter_command("time_vops", TimeVertexOperations0);
  enter_command("top", TopCircles0);
  enter_command("top_bottom", TopBottom);
  enter_command("top_ten", ShowTopTenMatches);
  enter_command("top_to_print", SetTopToPrint0);
  enter_command("track_match_stats", TrackMatchStats0);
  enter_command("trajectory_cutoff", SetTrajectoryCutoff);
  enter_command("trajectory_scores", TrajectoryScores);
  enter_command("trajectory_scores2", TrajectoryScores2);
  enter_command("translate", Translate0);
  enter_command("trash", Trash);
  enter_command("triangle_points3D", TrianglePoints3D0);
  enter_command("triangle_pixels", TrianglePixels0);
  enter_command("triangle_pixels2", TrianglePixels0b);
  enter_command("trivial_segmentation", TrivialSegmentation);
  enter_command("ts_break_distances", TsBreakDistances);
  enter_command("ts_compile_distances", TsCompileDistances);
  enter_command("ts_permute", TsPermute);
  enter_command("ts_testtest", TsTestTestDistances);
  enter_command("ts_testtrain", TsTestTrainDistances);
  enter_command("ts_traintrain", TsTrainTrainDistances);

  enter_command("undistort_frame", UndistortFrame);
  enter_command("unipen_load_classifiers", UnipenLoadClassifiers);
  enter_command("update_features", UpdateFeatures);
  enter_command("update_hand_tracker", UpdateHandTracker);

  enter_command("value", SelectValue0);
  enter_command("verify_inverse", VerifyInverse);
  enter_command("verify_multiplication", VerifyMultiplication);
  enter_command("verify_orientations", VerifyOrientations);
  enter_command("verify_rotation", VerifyRotation);
  enter_command("verify_rotation_matrix", VerifyRotationMatrix);
  enter_command("vhand_pose", ConvertPose0);
  enter_command("view_params", ViewParams0);
  enter_command("view_rank", ViewRank0);
  enter_command("vs_stats", VS_Stats);

  enter_command("warp_image", TestWarpImage0);
  enter_command("warp_image_sparse", TestWarpImageSparse0);
  enter_command("write_big", WriteBigDistances0);
  enter_command("write_frame", write_frame);
  enter_command("write_frames", write_frames0);
  enter_command("write_range", WriteRange0);
  enter_command(">>", NextInSequence);
  enter_command("<<", PreviousInSequence);

  return 1;
}

long class_gesture_module::clean_up()
{
  return 1;
}


long class_gesture_module::enter_command(char * command, type_procedure procedure)
{
  commands.push_back(function_copy_string (command));
  procedures.push_back(procedure);
  return 1;
}


long class_gesture_module::process_command(const char * command)
{
  long index = command_index (command);
  if (index < 0)
  {
    return 0;
  }
  type_procedure procedure = procedures [index];
  (this->*procedure)();
  return 1;
}

long class_gesture_module::sort_commands()
{
  vector <long> indices;
  function_string_ranks (&commands, & indices);
  
  vector <char*> temporary;
  temporary.insert(temporary.end (), commands.begin (), commands.end ());
  vector <type_procedure> temporary_procedures;
  temporary_procedures.insert(temporary_procedures.end(), procedures.begin (), procedures.end ());

  long number = commands.size ();
  long counter;
  for (counter = 0; counter < number; counter ++)
  {
    long index = indices [counter];
    commands[counter] = temporary [index];
    procedures [counter] = temporary_procedures [index];
  }

  return 1;
}

//////////////////////////////////////////////////////////
//                                                      //
//  end of obligatory member functions for any module   //
//                                                      //
//////////////////////////////////////////////////////////
