#include "stdafx.h"

#include "contour.h"
#include "algo_templates.h"
#include "basics/drawing_temp.h"

#include "basics/definitions.h"


const long vElongation::extra_number = 13;
const long vElongation::number_of_rows = 8;


//////////////////////////////////////////////////////////////
// 
// Start of implementation of class vElongation
//
//////////////////////////////////////////////////////////////


vElongation::vElongation(long columns) : 
vMatrix<double>(number_of_rows, Max(extra_number, columns))
{
  vWriteValue(this, (double) 0);
  Initialize();
}


vElongation::~vElongation()
{
}


long vElongation::Initialize()
{
  long index = 0;
  lengths = data[index++];
  start_rows = data[index++];
  start_cols = data[index++];
  start_indices = data[index++];
  end_rows = data[index++];
  end_cols = data[index++];
  end_indices = data[index++];
  extra = data[index++];
  assert(index == number_of_rows);

  index = 0;
  max_elongation = &(extra[index++]);
  max_length = &(extra[index++]);
  max_width = &(extra[index++]);
  max_index = &(extra[index++]);
  max_start_row = &(extra[index++]);

  max_start_col = &(extra[index++]);
  max_start_index = &(extra[index++]);
  max_end_row = &(extra[index++]);
  max_end_col = &(extra[index++]);
  max_end_index = &(extra[index++]);
  
  starting_point_row = &(extra[index++]);
  starting_point_col = &(extra[index++]);
  starting_point_index = &(extra[index++]);
  assert(index == extra_number);
  return 1;
}


long vElongation::Print()
{
  vMatrix<double> transpose = Trans();
  transpose.Print("Elongation");
  return 1;
}

//////////////////////////////////////////////////////////////
// 
// End of implementation of class vElongation
//
//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////
// 
// Start of implementation of class vContour
//
//////////////////////////////////////////////////////////////



// If we want to immediately create a contour of an image, we can call
// the constructor(s) that take images as arguments. However, we might
// want to specify what values should be considered foreground and 
// what values should be considered background. The other constructors
// treat (by default) 0 as background and non-zero as foreground. By
// using the default constructor we can then set the foreground and 
// background values as desired, and then call the Process member
// function to actually get the contour.
vContour::vContour()
{
  Initialize();
}


vContour::vContour(grayscale_image * the_image)
{
  Initialize();
  OuterContour(the_image);
}


vContour::~vContour()
{
  CleanUp();
}


// Deletes all data stored in this object.
long vContour::CleanUp()
{
  xs.erase(xs.begin(), xs.end());
  ys.erase(ys.begin(), ys.end());

  return 1;
}


vContour * vContour::Copy()
{
  vContour * result = new vContour();
  vector<double> & xs1 = result->xs;
  vector<double> & ys1 = result->ys;

  xs1.insert(xs1.begin(), xs.begin(), xs.end());
  ys1.insert(ys1.begin(), ys.begin(), ys.end());
  result->foreground_low = foreground_low;
  result->foreground_high = foreground_high;
  result->initialized = initialized;
  result->image_rows = image_rows;
  result->image_cols = image_cols;
  return result;
}


vector<long> * vContour::GetXsLong()
{
  vector<long> * result = new vector<long>(number_of_points);
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    (*result)[i] = vRound(xs[i]);
  }
  return result;
}


vector<long> * vContour::GetYsLong()
{
  vector<long> * result = new vector<long>(number_of_points);
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    (*result)[i] = vRound(ys[i]);
  }
  return result;
}


// Sets the range of values that will be considered foreground.
long vContour::SetForegroundRange(uchar low, uchar high)
{
  if (low > high) return 0;
  foreground_low = low;
  foreground_high = high;
  return 1;
}


// Gets the values of foreground_low and foreground_high.
long vContour::GetForegroundRange(uchar * low, uchar * high)
{
  *low = foreground_low;
  *high = foreground_high;
  return 1;
}


// Sets variables to default values.
long vContour::Initialize()
{
  foreground_low = 1;
  foreground_high = 255;
  initialized = 0;
  return 1;
}


long vContour::StartingPoint(grayscale_image * the_image, long * out_row, long * out_col)
{
  vArray2(uchar) matrix2 = the_image->Matrix2(0);
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  long row, col;
  for (row = 0; row < rows; row++)
  {
    vArray(uchar) matrix = matrix2[row];
    for (col = 0; col < cols; col++)
    {
      if (IsForeground(matrix[col]))
      {
        *out_row = row;
        *out_col = col;
        return 1;
      }
    }
  }
  // We only get here if no foreground pixel was found.
  return 0;
}


long S_InitializeNeighbors(long * neighbor_xs, long * neighbor_ys)
{
  long index = -1;

  // Set up neighbors so that the first one is the top neighbor and
  // the rest follow in clockwise order. Note that the first one
  // is index 1, not 0. At indices 0 and 9 we put the top left and top
  // respectively, so that we can easily check common neighbors between
  // any neighor and the center.
  index++;
  neighbor_xs[index] = -1;
  neighbor_ys[index] = -1;

  index++;
  neighbor_xs[index] = 0;
  neighbor_ys[index] = -1;

  index++;
  neighbor_xs[index] = 1;
  neighbor_ys[index] = -1;

  index++;
  neighbor_xs[index] = 1;
  neighbor_ys[index] = 0;

  index++;
  neighbor_xs[index] = 1;
  neighbor_ys[index] = 1;

  index++;
  neighbor_xs[index] = 0;
  neighbor_ys[index] = 1;

  index++;
  neighbor_xs[index] = -1;
  neighbor_ys[index] = 1;

  index++;
  neighbor_xs[index] = -1;
  neighbor_ys[index] = 0;

  index++;
  neighbor_xs[index] = -1;
  neighbor_ys[index] = -1;

  index++;
  neighbor_xs[index] = 0;
  neighbor_ys[index] = -1;
  return 1;
}


// Computes the outer contour of the foreground in an image. A problem
// with this function is that, in going around the contour, represent
// its current position with the pixel location in that position. This
// way there are certain junctions in which there are more than one
// possibilities for the next point, and this function just picks
// one of those arbitrary possibilities. To address this problem
// we need to keep track, for the current position, of the actual
// pixel side that corresponds to the contour (i.e that is between
// the pixel and the background) and the direction in which we 
// moved along that side. That approach is taken in OuterContour2
long vContour::OuterContour2(grayscale_image * the_image)
{
  CleanUp();
  image_rows = the_image->Rows();
  image_cols = the_image->Cols();

  const long number_of_neighbors = 8;
  long neighbor_xs[number_of_neighbors+2];
  long neighbor_ys[number_of_neighbors+2];
  // We set up neighbors so that the first is the top neighbor, and 
  // we move in clockwise order. The first neighbor is index 1. At 
  // indices 0 and 8 we have the top left neighbor and at indices
  // 1 and 9 we have the top neighbor. 
  S_InitializeNeighbors(neighbor_xs, neighbor_ys);
  
  // Make the border 0 so that, as we follow the contour, we never
  // fall out of bounds (so we don't have to check).
  vWriteBorderValue(the_image, (uchar) 0);
  
  // Get a starting point
  long start_row = 0;
  long start_col = 0;
  long success = StartingPoint(the_image, &start_row, &start_col);

  // scratch[i][j] will be set to 1 when the i,j pixel is processed.
  grayscale_image * scratch_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(scratch_image, (uchar) 0);
  vArray2(uchar) scratch = scratch_image->Matrix2(0);
  vArray2(uchar) matrix2 = the_image->Matrix2(0);

//  long last_neighbor = -1;
  long row = start_row;
  long col = start_col;
  long backtrack_index = 0;
  if (success >= 1)
  {
    while(1)
    {
      // Add the (row, col) pixel to the contour points
      xs.push_back(col);
      ys.push_back(row);
      // Mark the (row, col) pixel as already processed.
      scratch[row][col] = 1;

      // Find the next contour point.
      long next_row, next_col;
      ulong found = 0;
      long i;
      for (i = 1; i <= number_of_neighbors; i++)
      {
        next_row = row + neighbor_ys[i];
        next_col = col + neighbor_xs[i];

        uchar value = matrix2[next_row][next_col];
        if (!(IsForeground(value))) continue;
        if (scratch[next_row][next_col] != 0) continue;
        // Check if the neighbor and the center have a common
        // neighbor that is background. This is why I made the neighbors
        // array have 10 elements instead of 8.
        long row1 = row + neighbor_ys[i-1];
        long col1 = col + neighbor_xs[i-1];
        long row2 = row + neighbor_ys[i+1];
        long col2 = col + neighbor_xs[i+1];
        if ((IsForeground(matrix2[row1][col1])) &&
            (IsForeground(matrix2[row2][col2])))
        {
          continue;
        }

        found = 1;
        backtrack_index = xs.size();
        break;
      }
      // If we haven't found a next pixel yet, there are two possibilities:
      // Either we got back to the start, in which case we are done, or 
      // we took a dead-end branch, in which case we must backtrack (go back to
      // the contour point that was last entered and start again from there).
      if (found == 0) 
      {
        // Check if we got back to the start
        long row_diff = vAbs(row - start_row);
        long col_diff = vAbs(col - start_col);
        if ((row_diff <= 1) && (col_diff <= 1))
        {
          // We got back to the start, break.
          break;
        }

        // Check if we can backtrack:
        if (xs.size() < 2) break;
        // Backtrack to the previous point
        backtrack_index--;
        next_col = (long) xs[backtrack_index];
        next_row = (long) ys[backtrack_index];
      }
      row = next_row;
      col = next_col;
    }
  }

  vdelete(scratch_image);
  Pack();
  number_of_points = NumberOfPoints();
  initialized = 1;
  return 1;
}


// The following classes and types are used in OuterContour2.

// S_ContourPosition holds all the information we need to 
// know (given the image) in order to:
// - specify our current position on the contour
// - determine what the next position should be
class S_ContourPosition
{
public:
  // pixel location
  long row, col;
  // side on that pixel: 0=top, 1=right, 2=bottom, 3=left
  long side; 
  // corner on that side: 0=corner on clockwise start, 1= corner on other end
  // For example, for side=0 (top), 0 is left and 1 is right.
  long corner;

  // dummy default constructor
  S_ContourPosition()
  {
  }

  S_ContourPosition(long in_row, long in_col, long in_side, long in_corner)
  {
    row = in_row;
    col = in_col;
    side = in_side;
    corner = in_corner;
  }
};


// PositionNeighbors has slots for the two pixels we need to consider
// in order to determine how to move from one contour position to the next.
// There is a next pixel that we get to by going from the current pixel
// along the direction specified in S_ContourPosition, and a diagonal pixel
// that is 8-connected with the pixel in S_ContourPosition, such that
// the connection point is the corner specified by corner and side
// of S_ContourPosition
class S_PositionNeighbors
{
public:
  // row and col of next pixel
  long next_row, next_col;
  // row and col of diagonal pixel
  long diagonal_row, diagonal_col;

  // dummy default constructor
  S_PositionNeighbors()
  {
  }

  S_PositionNeighbors(long in_next_row, long in_next_col, 
                      long in_diagonal_row, long in_diagonal_col)
  {
    next_row = in_next_row;
    next_col = in_next_col;
    diagonal_row = in_diagonal_row;
    diagonal_col = in_diagonal_col;
  }
};


static long S_InitPositionNeighborsArray(void * result)
{
  S_PositionNeighbors * position_neighbors =
    (S_PositionNeighbors *) result;
  
  long i = 0;
  position_neighbors[i++] = S_PositionNeighbors(0, -1, -1, -1);
  position_neighbors[i++] = S_PositionNeighbors(0, 1, -1, 1);

  position_neighbors[i++] = S_PositionNeighbors(-1, 0, -1, 1);
  position_neighbors[i++] = S_PositionNeighbors(1, 0,  1, 1);
  
  position_neighbors[i++] = S_PositionNeighbors(0, 1, 1, 1);
  position_neighbors[i++] = S_PositionNeighbors(0, -1, 1, -1);

  position_neighbors[i++] = S_PositionNeighbors(1, 0, 1, -1);
  position_neighbors[i++] = S_PositionNeighbors(-1, 0, -1, -1);

  return 1;
}


static long S_InitNextPositionArray(void * result)
{
  S_ContourPosition * next_positions = 
    (S_ContourPosition *) result;

  // Guidelines:
  // If next is background, we stay on the same pixel
  // If next is foreground and diag is background,
  // we move to next
  // If next is foreground and diag is foreground 
  // we move to diag

  long i = 0;
  // side = 0;
  // [0][0][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 3, 0);
  // [0][0][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 3, 0);
  // [0][0][1][0]
  next_positions[i++] = S_ContourPosition(0,-1, 0, 0);
  // [0][0][1][1]
  next_positions[i++] = S_ContourPosition(-1, -1, 1, 0);
  // [0][1][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 1, 1);
  // [0][1][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 1, 1);
  // [0][1][1][0]
  next_positions[i++] = S_ContourPosition(0, 1, 0, 1);
  // [0][1][1][1]
  next_positions[i++] = S_ContourPosition(-1, 1, 3, 1);

  // side = 1;
  // [1][0][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 0, 0);
  // [1][0][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 0, 0);
  // [1][0][1][0]
  next_positions[i++] = S_ContourPosition(-1, 0, 1, 0);
  // [1][0][1][1]
  next_positions[i++] = S_ContourPosition(-1, 1, 2, 0);
  // [1][1][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 2, 1);
  // [1][1][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 2, 1);
  // [1][1][1][0]
  next_positions[i++] = S_ContourPosition(1, 0, 1, 1);
  // [1][1][1][1]
  next_positions[i++] = S_ContourPosition(1, 1, 0, 1);

  // side = 2;
  // [2][0][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 1, 0);
  // [2][0][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 1, 0);
  // [2][0][1][0]
  next_positions[i++] = S_ContourPosition(0, 1, 2, 0);
  // [2][0][1][1]
  next_positions[i++] = S_ContourPosition(1, 1, 3, 0);
  // [2][1][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 3, 1);
  // [2][1][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 3, 1);
  // [2][1][1][0]
  next_positions[i++] = S_ContourPosition(0, -1, 2, 1);
  // [2][1][1][1]
  next_positions[i++] = S_ContourPosition(1, -1, 1, 1);

  // side = 3;
  // [3][0][0][0]
  next_positions[i++] = S_ContourPosition(0, 0, 2, 0);
  // [3][0][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 2, 0);
  // [3][0][1][0]
  next_positions[i++] = S_ContourPosition(1, 0, 3, 0);
  // [3][0][1][1]
  next_positions[i++] = S_ContourPosition(1, -1, 0, 0);
  // [3][1][0][0]T
  next_positions[i++] = S_ContourPosition(0, 0, 0, 1);
  // [3][1][0][1]
  next_positions[i++] = S_ContourPosition(0, 0, 0, 1);
  // [3][1][1][0]
  next_positions[i++] = S_ContourPosition(-1, 0, 3, 1);
  // [3][1][1][1]
  next_positions[i++] = S_ContourPosition(-1, -1, 2, 1);

  return 1;
}


// This function addresses the weakness of OuterContour2 described
// in the comments preceding OuterContour.
long vContour::OuterContour(grayscale_image * the_image)
{
  static S_PositionNeighbors position_neighbors[4][2];
  static S_ContourPosition next_positions[4][2][2][2];
  static long dummy1 = S_InitPositionNeighborsArray(position_neighbors);
  static long dummy2 = S_InitNextPositionArray(&next_positions);
  // Used to identify the bit in the scratch image that corresponds
  // the the side of interest (one of the four sides of the pixel).
  static long side_bits[] = {1, 2, 4, 8};

  CleanUp();
  image_rows = the_image->Rows();
  image_cols = the_image->Cols();

  // Make the border 0 so that, as we follow the contour, we never
  // fall out of bounds (so we don't have to check).
  vWriteBorderValue(the_image, (uchar) 0);
  
  // Get a starting point. It will be the the leftmost point
  // in the topmost row of the object.
  long start_row = 0;
  long start_col = 0;
  long success = StartingPoint(the_image, &start_row, &start_col);
  if (success == 0) return 0;

  // scratch[i][j] will be keep track of the sides of pixel i,j
  // that have been processed. This way we can always check if 
  // we have visited a side twice.
  // I think I have proven that no side will be visited twice
  // except for the starting side (which will be visited when
  // we have gone around the contour, and it is time to stop).
  // However, I am doing this test as a sanity check
  grayscale_image scratch_image = grayscale_image(image_rows, image_cols);
  vWriteValue(&scratch_image, (uchar) 0);
  vArray2(uchar) scratch2 = scratch_image.Matrix2(0);
  vArray2(uchar) matrix2 = the_image->Matrix2(0);

  // 
  S_ContourPosition current(start_row, start_col, 0, 1);
  xs.push_back(start_col);
  ys.push_back(start_row);
  scratch2[start_row][start_col] = scratch2[start_row][start_col] | (uchar) side_bits[0];
  while(1)
  {
    // Get the two neighbors of interest based on the current position. We
    // first get the neighbors' displacement with respect to the current 
    // pixel.
    S_PositionNeighbors neighbors = position_neighbors[current.side][current.corner];
    
    // Get the neighbors' coordinates based on their displacements.
    long next_row = neighbors.next_row + current.row;
    long next_col = neighbors.next_col + current.col;
    long diag_row = neighbors.diagonal_row + current.row;
    long diag_col = neighbors.diagonal_col + current.col;
    
    // Get the intensity values of the neighbors, and determine if they
    // are foreground or not.
    uchar next_value = matrix2[next_row][next_col];
    uchar diag_value = matrix2[diag_row][diag_col];
    long next_class = IsForeground(next_value);
    long diag_class = IsForeground(diag_value);

    // Get the next contour position based on the current pixel
    // position and the foreground/background status of the 
    // two neighbors. The next position's coordinates will
    // be relative to the current position
    S_ContourPosition next_position = 
      next_positions[current.side][current.corner][next_class][diag_class];

    // Update current by moving it to the next position.
    current.row = current.row + next_position.row;
    current.col = current.col + next_position.col;
    current.side = next_position.side;
    current.corner = next_position.corner;
    
    // Check if we have been to that position (same side of same pixel)
    // before. If yes, this means we got back to the start. As a sanity
    // check, I verify that we got indeed back to the start, because
    // otherwise it means that my algorithm for finding contours is wrong.
    long visited = scratch2[current.row][current.col] & side_bits[current.side];
    if (visited != 0)
    {
      if ((current.row == start_row) && (current.col == start_col))
      {
        break;
      }
      else
      {
        exit_error("Error, revisiting same contour position. This should never happen\n");
      }
    }

    // We get here if the current position (pixel coordinates and side)
    // has not been visited in the past. In that case, we mark it
    // now as visited.
    scratch2[current.row][current.col] |= side_bits[current.side];

    xs.push_back(current.col);
    ys.push_back(current.row);
    long size = xs.size();
//    vPrint("Entered %li:  (%li, %li), (%li, %li)\n", 
//             size, current.row, current.col, current.side, current.corner);
  }

  Pack2();
  initialized = 1;
  number_of_points = NumberOfPoints();
  return 1;
}


// This function smooths the contour. It basically creates a new
// contour, on which point i is the average of a number of 
// nearby pixels in the old contour. The number of pixels used
// is specified by neighborhood (it is 2 * neighborhood + 1)
long vContour::Smooth1(long neighborhood)
{
  // Copy over the old contour into arrays. We make the arrays 
  // have some overlap between their beginning portions and their
  // ending portions, so that we can easily find each pixel's neighbors.

  long number_of_points = xs.size();
  if (number_of_points == 0) return 1;
  long buffer_size = number_of_points + 2 * neighborhood;
  long neighborhood_size = 2 * neighborhood + 1;
  vArray(double) x_buffer = vnew(double, buffer_size);
  vArray(double) y_buffer = vnew(double, buffer_size);
  
  // Create old_xs and old_ys, which are displaced versions of x_buffer
  // and y_buffer, such that old_xs[i] corresponds to xs[i] (basically,
  // x_buffer[neighborhood] = old_xs[0]
  vArray(double) old_xs = x_buffer + neighborhood;
  vArray(double) old_ys = y_buffer + neighborhood;
  long i;
  long index = number_of_points - neighborhood;
  while (index < 0) index += number_of_points;

  for (i = -neighborhood; i < number_of_points + neighborhood; i++)
  {
    old_xs[i] = xs[index];
    old_ys[i] = ys[index];
    index = (index + 1) % number_of_points;
  }

  // Find the first point of the smoothed contour.
  double x_sum = 0;
  double y_sum = 0;
  for (i = -neighborhood; i <= neighborhood; i++)
  {
    x_sum += old_xs[i];
    y_sum += old_ys[i];
  }
  xs[0] = x_sum / (double) neighborhood_size;
  ys[0] = y_sum / (double) neighborhood_size;

  // Process the rest of the points.
  for (i = 1; i < number_of_points; i++)
  {
    x_sum = x_sum - old_xs[i - neighborhood - 1];
    x_sum = x_sum + old_xs[i + neighborhood];
    y_sum = y_sum - old_ys[i - neighborhood - 1];
    y_sum = y_sum + old_ys[i + neighborhood];
    xs[i] = x_sum / (double) neighborhood_size;
    ys[i] = y_sum / (double) neighborhood_size;
  }
  vdelete2(x_buffer);
  vdelete2(y_buffer);
  return 1;
}


// This function is like Smooth1, but instead of modifying the 
// object from which it is called, it leaves the object unchanged
// and returns the results in another object.
vContour * vContour::Smooth2(long neighborhood)
{
  vContour * result = Copy();
  long success = result->Smooth1(neighborhood);
  if (success == 0)
  {
    vdelete(result);
    return 0;
  }
  return result;
}


// Pack goes through the contour points and deletes points such that
// their preceding and their next point are neighbors.
/*
long vContour::Pack()
{
  number_of_points = NumberOfPoints();
  
  long items_to_delete = 0;

  // We keep going around and around until we have done a complete
  // circle without deleting any points.
  do
  {
    vPrint("number_of_points = %li\n", number_of_points);
    if (number_of_points < 3) return 1;
    items_to_delete = 0;
    long index1 = 0;
    long index2 = 2;
    do
    {
      double x_diff = vAbs(xs[index1] - xs[index2]);
      double y_diff = vAbs(ys[index1] - ys[index2]);
      if ((x_diff <= 1) && (y_diff <= 1))
      {
        items_to_delete++;
      }
      else 
      {
         index1 = Increment(index1);
      }
    
      long i1 = Increment(index1);
      xs[i1] = xs[index2];
      ys[i1] = ys[index2];
      index2 = Increment(index2);
    }
    while(index2 != 2);

    for (long i = 0; i < items_to_delete; i++)
    {
      xs.pop_back();
      ys.pop_back();
    }
    number_of_points = number_of_points - items_to_delete;   
  }
  while(items_to_delete != 0);
  //while(0);

  return 1;
}
*/


long vContour::Pack()
{
  // We keep going around and around until we have done a complete
  // circle without deleting any points.
  do
  {
    number_of_points = NumberOfPoints();
    vPrint("number_of_points = %li\n", number_of_points);
    if (number_of_points < 3) return 1;
    vector<double> new_xs, new_ys;
    double last_col = xs[0];
    double last_row = ys[0];
    new_xs.push_back(last_col);
    new_ys.push_back(last_row);
    
    long index1 = 2;
    long points_processed = 1;
    while(points_processed < number_of_points)
    {
      double current_col = xs[index1];
      double current_row = ys[index1];
      double x_diff = vAbs(last_col - current_col);
      double y_diff = vAbs(last_row - current_row);
      if ((x_diff > 1.1) || (y_diff > 1.1))
      {
        long previous_index = (index1 + number_of_points - 1) % number_of_points;
        last_col = xs[previous_index];
        last_row = ys[previous_index];
        new_xs.push_back(last_col);
        new_ys.push_back(last_row);
        vPrint("added (%.0lf, %.0lf)\n", last_row, last_col);
      }  
      else
      {
        vPrint("Something is getting dropped\n");
      }
      index1 = Increment(index1);
      points_processed++;
    }

    // replace xs with new_xs and ys with new_ys.
    xs.erase(xs.begin(), xs.end());
    xs.insert(xs.begin(), new_xs.begin(), new_xs.end());
    ys.erase(ys.begin(), ys.end());
    ys.insert(ys.begin(), new_ys.begin(), new_ys.end());
    vPrint("------------------------\n");
  }
  while(number_of_points > (long) xs.size());
  //while(0);

  return 1;
}


// Pack2 goes through the contour points and deletes repeated points.
long vContour::Pack2()
{
  number_of_points = NumberOfPoints();
  if (number_of_points == 0) return 1;
 
  vector<double> new_xs, new_ys;
  // Add the first point
  new_xs.push_back(xs[0]);
  new_ys.push_back(ys[0]);
  
  long i;
  // Add middle points as long as they are different from the 
  // point right before them.
  for (i = 1; i < number_of_points - 1; i++)
  {
    long previous_i = Decrement(i);
    if ((xs[previous_i] != xs[i]) || (ys[previous_i] != ys[i]))
    {
      new_xs.push_back(xs[i]);
      new_ys.push_back(ys[i]);
    }  
  }

  // Add the last point if it's different from the previous one and from 
  // the first one
  if ((number_of_points >= 2) &&
      ((xs[number_of_points-1] != xs[number_of_points-2]) ||
       (ys[number_of_points-1] != ys[number_of_points-2])) &&
      ((xs[number_of_points-1] != xs[0]) ||
       (ys[number_of_points-1] != ys[0])))
  {
    new_xs.push_back(xs[number_of_points-1]);
    new_ys.push_back(ys[number_of_points-1]);
  }

  // replace xs with new_xs and ys with new_ys.
  xs.erase(xs.begin(), xs.end());
  xs.insert(xs.begin(), new_xs.begin(), new_xs.end());
  ys.erase(ys.begin(), ys.end());
  ys.insert(ys.begin(), new_ys.begin(), new_ys.end());
  number_of_points = NumberOfPoints();

  return 1;
}


// Returns an image that is 0 everywhere except for the contour
// points, which are set to "value".
grayscale_image * vContour::ContourImage(uchar value)
{
  if (initialized == 0) return 0;
  grayscale_image * result = new grayscale_image(image_rows, image_cols);
  vWriteValue(result, (uchar) 0);
  long success = ContourImage2(result, value);
  if (success <= 0)
  {
    vdelete(result);
    return 0;
  }
  return result;
}


// Writes the contour onto the image (sets contour points to value).
long vContour::ContourImage2(grayscale_image * the_image, uchar value)
{
  if (initialized == 0) return 0;
  vArray2(uchar) matrix2 = the_image->Matrix2(0);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long number_of_points = xs.size();
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    long row = (long) ys[i];
    long col = (long) xs[i];
    if (row < 0) return 0;
    if (row >= rows) return 0;
    if (col < 0) return 0;
    if (col >= cols) return 0;
    matrix2[row][col] = value;
  }
  return 1;
}


// Returns an image that is 0 everywhere except for the contour points 
// between the start index and the end index (in increasing order), where
// it has the specified value.
grayscale_image * vContour::SegmentImage(long start_index, long end_index, 
                                      uchar value)
{
  if (initialized == 0) return 0;
  grayscale_image * result = new grayscale_image(image_rows, image_cols);
  vWriteValue(result, (uchar) 0);
  long success = SegmentImage2(result, start_index, end_index, value);
  if (success <= 0)
  {
    vdelete(result);
    return 0;
  }
  return result;
}


// Sets the pixel value of points in the segment specified by start_index
// and end_index to value
long vContour::SegmentImage2(grayscale_image * the_image, long start_index, 
                              long end_index, uchar value)
{
  if (initialized == 0) return 0;
  vArray2(uchar) matrix2 = the_image->Matrix2(0);
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long i;
  for (i = start_index; i != end_index; i = Increment(i))
  {
    long row = (long) ys[i];
    long col = (long) xs[i];
    if (row < 1) return 0;
    if (row >= rows-1) return 0;
    if (col < 1) return 0;
    if (col >= cols-1) return 0;
	long row2, col2;
	for (row2 = row-1; row2 <= row+1; row2++)
		for (col2 = col-1; col2 <= col+1; col2++)
		{
			matrix2[row2][col2] = value;
		}
  }
  return 1;
}


// Sets the pixel value of points in the segment specified by start_index
// and end_index to value
long vContour::SegmentImage3(color_image * the_image, long start_index, 
                              long end_index, uchar value1, uchar value2, uchar value3)
{
  number_of_points = xs.size();
  if (initialized == 0) return 0;
  if (start_index < 0) return 0;
  if (start_index >= number_of_points) return 0;
  vArray3(uchar) matrix3 = the_image->Matrix3();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long i;

  for (i = start_index; ; i = Increment(i))
  {
    long row = (long) ys[i];
    long col = (long) xs[i];
//    vPrint("Processing index %li, (%li, %li)\n", i, row, col);
    if (row < 1) return 0;
    if (row >= rows-1) return 0;
    if (col < 1) return 0;
    if (col >= cols-1) return 0;
	  long row2, col2;
	  for (row2 = row-1; row2 <= row+1; row2++)
    {
      for (col2 = col-1; col2 <= col+1; col2++)
      {
//    for (row2 = row; row2 <= row; row2++)
//    {
//      for (col2 = col; col2 <= col; col2++)
//		  {
			  matrix3[0][row2][col2] = value1;
			  matrix3[1][row2][col2] = value2;
			  matrix3[2][row2][col2] = value3;
		  }
    }
    if (i == end_index) break;
  }
  return 1;
}



// Region boundaries are points that segment a contour into segments, such
// that each segment is a (more or less) convex region. The "threshold"
// argument specifies how much we tolerate non-convexities. Basically,
// if we have a cut point A, to find the next, we look at each point in
// order starting at the point after the cut point. First we find a point
// B whose orientation is different than the cut point's by more than the
// threshold. From that point on we know (based on B) the way the next
// segment will turn (left or right). Now we are looking for the endpoint.
// Whenever we find a point whose difference in orientation from A is 
// the most we have seen so far (for point A) we keep track of that. When
// we find a point C whose difference in orientation with respect to A is
// less the max difference we have seen so far minus the threshold, then
// we pick B as the next cut point. 
// 
// The neighborhood argument specifies the neighborhood that we use
// to calculate first and second derivatives, it is passed as is to 
// the vCircularDerivative function.
// We put the indices of the cut points in "indices". The direction
// of the segment starting at (*indices)[i] is saved in (*directions)[i],
// it is either 1 or 0. The length of each curve segment is saved in 
// lengths.
long vContour::RegionBoundaries5(vector<long> * indices, vector<long> * directions,
                          vector<long> * lengths,
                          long neighborhood, double threshold)
{
  vector<double> magnitudes, orientations;
  return RegionBoundaries7(indices, directions, lengths, neighborhood, threshold,
                    &magnitudes, &orientations);
}


long vContour::RegionBoundaries7(vector<long> * indices, vector<long> * directions,
                          vector<long> * lengths,
                          long neighborhood, double threshold,
                          vector<double> * magnitudes, 
                          vector<double> * orientations)
{
  if (neighborhood < 1) return 0;
  if (threshold >= M_PI / 2) return 0;
  vector<double> dx, dy, ddx, ddy;
  long number_of_points = xs.size();
  if (number_of_points <= 1) return 1;
  long result = 1;
  long temp;
  temp = vCircularDerivative(&xs, &ys, &dx, &dy, neighborhood);
  result = (temp > 0);
  temp = vCircularDerivative(&dx, &dy, &ddx, &ddy, neighborhood);
  temp = vMagnitudes(&ddx, &ddy, magnitudes);
  temp = vOrientations(&dx, &dy, orientations);

  // What should be the first cut point? This is the heuristic I use:
  // Start at the global minimum with respect to the magnitude of 
  // the second derivative. We pick that because we expect the region
  // in that area to be pretty uneventful. Treat that temporarily 
  // as a start point,
  // and look for the next cut point. Then, use that cut point as 
  // the real first cut point.
  long current_start = 0;
  double value = 0.0;
  function_vector_minimum(magnitudes, &current_start, &value);
  long previous_direction = 0;
  long current_direction = 0;

  long scanned_points = 1;
  const double period = 2 * M_PI;
  while(scanned_points <= number_of_points)
  {
    double start_orientation = (*orientations)[current_start];
//    vPrint("Start orientation = %lf, (%5.0lf, %5.0lf)\n",
//            start_orientation, ys[current_start], xs[current_start]);
    double angle_diff;
    // First find the direction.
    long current = current_start + 1;
    while(1)
    {
      current = (current + 1) % number_of_points;
      double current_angle = (*orientations)[current];
      angle_diff = vCircularDistance<double>(start_orientation, 
                                               current_angle, period);
//      vPrint("(%5.0lf, %5.0lf) orientation = %lf, diff = %lf\n",
//              ys[current], xs[current], current_angle, angle_diff);
      if (vAbs(angle_diff) > threshold) break;
    }
    // Now find some point past the limit of the region.
    double max_diff = vAbs(angle_diff);
    long max_index = current;
    double cut_threshold = max_diff - threshold;
    previous_direction = current_direction;
    if (angle_diff > 0)
    {
      current_direction = 1;
    }
    else
    {
      current_direction = -1;
    }
    while(1)
    {
      current = (current + 1) % number_of_points;
      double current_angle = (*orientations)[current];
      angle_diff = vDirectedCircularDistance<double>(start_orientation, 
                                                      current_angle, period,
                                                      current_direction);
//      vPrint("(%5.0lf, %5.0lf) orientation = %lf, diff = %lf\n",
//              ys[current], xs[current], current_angle, angle_diff);
      double abs_diff = vAbs(angle_diff);
      if (abs_diff > max_diff) 
      {
        max_diff = abs_diff;
        max_index = current;
        cut_threshold = max_diff - threshold;
      }
      else if (abs_diff < cut_threshold)
      {
        break;
      }
      // If we make a complete circle it means the shape is convex, and
      // there are no cuts.
      if (current == current_start) 
      {
        indices->erase(indices->begin(), indices->end());
        lengths->erase(lengths->begin(), lengths->end());
        directions->erase(directions->begin(), directions->end());
        return 1;
      }
    }
    long length = (max_index + number_of_points - current_start) % number_of_points;
    current_start = max_index;
    indices->push_back(max_index);
    scanned_points += length;

    // If this is not the first segment, it could be that this segment
    // should be merged to the previous segment. This is true when the
    // direction of the two segments is similar, because a segment
    // (unless it is the result of merging) can't have an overall change
    // of orientation that is greater than 180 degrees. To handle such
    // segments, we need to merge consecutive segments with the same 
    // direction.
    // Also, if no segment has been found yet, for the same reasons,
    // we may have to replace our selection of the first cut point.
    // However, we can have contours where we should not find any
    // cut points (a circle is an example). scanned_points, in that
    // case, will get greater than the number of points.
    long segments_so_far = indices->size() - 1;
    if (previous_direction == current_direction)
    {
      // Merge this segment into the previous one, or replace the
      // starting point.
      if (segments_so_far > 1)
      {
        (*lengths)[segments_so_far - 2] += length;
        long t = (*indices)[segments_so_far - 1];
        vPrint("(%5.0lf, %5.0lf), (%5.0lf, %5.0lf)\n",
                ys[current_start], xs[current_start], 
                ys[t], xs[t]);
      }
      indices->pop_back();
      
      // If scanned_points > number of points, then there are two
      // cases: Either we went around once looking for a starting
      // point and failed (which can happen if the contour is convex,
      // like in a circle), or we are done with the segmentation.
      // In both cases the function is done, but if we have been
      // segmenting, 
      if ((scanned_points > number_of_points) &&
          (segments_so_far == 0))
      {
        return result;
      }
    }
    else
    {
      if (segments_so_far == 1)
      {
        // If this is our first segment, update scanned_points, so that
        // it starts measuring the total length of the segmented contour
        // so far (before it was measuring the number of points we went
        // through to find a good start point)
        scanned_points = length + 1;
      }
      if (segments_so_far > 0)
      {
        // Don't merge segments, add the current segment as an individual
        // segment.
        lengths->push_back(length);
        directions->push_back(current_direction);
      }
    }
  }
  long trash1 = indices->size();
  long trash2 = lengths->size();
  long trash3 = directions->size();
  
  // We know we have gone around the contour and have gone past the 
  // first starting point, so we should delete the end point of the 
  // last segment (eventually the last segment will end at the starting
  // point)
  indices->pop_back();

  // Maybe the first and last segment need to be merged.
  long number_of_segments = directions->size();

  long end = number_of_segments - 1;
  if ((*directions)[0] == (*directions)[end])
  {
    // Merge first and last segment
    (*indices)[0] = (*indices)[end];
    indices->pop_back();
    directions->pop_back();
    (*lengths)[0] += (*lengths)[end];
    lengths->pop_back();
    number_of_segments--;
  }

  // Make sure the start point of the first segment is the same as 
  // the end point of the last segment.
  indices->push_back((*indices)[0]);

  return result;
}


// Cut points are like region boundaries, but there is more of 
// them. In particular, whereas a contour segment between two
// boundary points can change rotation by more than 180 degrees,
// CutPoints makes sure that this doesn't happen. This is useful
// in things like locating fingers, where, if we allow a segment
// to go over 180 degrees in rotation we can have two fingertips
// in the same segment.
long vContour::CutPoints5(vector<long> * indices, vector<long> * directions,
                           vector<long> * lengths,
                           long neighborhood, double threshold)
{
  vector<double> magnitudes, orientations;
  return CutPoints7(indices, directions, lengths, neighborhood, threshold,
                    &magnitudes, &orientations);
}


long vContour::CutPoints7(vector<long> * indices, vector<long> * directions,
                           vector<long> * lengths,
                           long neighborhood, double threshold,
                           vector<double> * magnitudes, 
                           vector<double> * orientations)
{
  if (neighborhood < 1) return 0;
  if (threshold >= M_PI / 2) return 0;
  vector<double> dx, dy, ddx, ddy;
  long number_of_points = xs.size();
  if (number_of_points <= 1) return 1;
  long result = 1;
  long temp;
  temp = vCircularDerivative(&xs, &ys, &dx, &dy, neighborhood);
  result = (temp > 0);
  temp = vCircularDerivative(&dx, &dy, &ddx, &ddy, neighborhood);
  temp = vMagnitudes(&ddx, &ddy, magnitudes);
  temp = vOrientations(&dx, &dy, orientations);

  // What should be the first cut point? This is the heuristic I use:
  // Start at the global minimum with respect to the magnitude of 
  // the second derivative. We pick that because we expect the region
  // in that area to be pretty uneventful. Treat that temporarily 
  // as a start point,
  // and look for the next cut point. Then, use that cut point as 
  // the real first cut point.
  long current_start = 0;
  double value = 0.0;
  function_vector_minimum(magnitudes, &current_start, &value);
  long previous_direction = 0;
  long current_direction = 0;

  long scanned_points = 1;
  const double period = 2 * M_PI;
  while(scanned_points <= number_of_points)
  {
    double start_orientation = (*orientations)[current_start];
    double angle_diff;
    // First find the direction.
    long current = current_start + 1;
//    vPrint("Start orientation = %lf, (%5.0lf, %5.0lf)\n",
//            start_orientation, ys[current_start], xs[current_start]);
    while(1)
    {
      current = (current + 1) % number_of_points;
      double current_angle = (*orientations)[current];
      angle_diff = vCircularDistance<double>(start_orientation, 
                                              current_angle, period);
      if (vAbs(angle_diff) > threshold) break;
    }
    // Now find some point past the limit of the region.
    double max_diff = vAbs(angle_diff);
    long max_index = current;
    double cut_threshold = max_diff - threshold;
    previous_direction = current_direction;
    if (angle_diff > 0)
    {
      current_direction = 1;
    }
    else
    {
      current_direction = -1;
    }
    while(1)
    {
      current = (current + 1) % number_of_points;
      double current_angle = (*orientations)[current];
      angle_diff = vDirectedCircularDistance<double>(start_orientation, 
                                                      current_angle, period,
                                                      current_direction);
      double abs_diff = vAbs(angle_diff);
//      vPrint("(%5.0lf, %5.0lf) orientation = %lf, diff = %lf\n",
//              ys[current], xs[current], current_angle, angle_diff);
//      vPrint("abs_diff = %lf\n", abs_diff);
      if (abs_diff > max_diff) 
      {
        max_diff = abs_diff;
        max_index = current;
        cut_threshold = max_diff - threshold;
      }
      else if (abs_diff < cut_threshold)
      {
        break;
      }
      // If we make a complete circle it means the shape is convex, and
      // there are no cuts.
      if (current == current_start) 
      {
        indices->erase(indices->begin(), indices->end());
        lengths->erase(lengths->begin(), lengths->end());
        directions->erase(directions->begin(), directions->end());
        return 1;
      }
      // This forces the segmentation of contour parts that curve more
      // than 180 degrees (plus epsilon)
      if (abs_diff > M_PI) 
      {
        break;
      }
    }
    long length = (max_index + number_of_points - current_start) % number_of_points;
    current_start = max_index;
    indices->push_back(max_index);
    scanned_points += length;

    long segments_so_far = indices->size() - 1;
    if (segments_so_far == 1)
    {
      // If this is our first segment, update scanned_points, so that
      // it starts measuring the total length of the segmented contour
      // so far (before it was measuring the number of points we went
      // through to find a good start point)
      scanned_points = length + 1;
    }
    if (segments_so_far > 0)
    {
      lengths->push_back(length);
      directions->push_back(current_direction);
    }
  }

  // We know we have gone around the contour and have gone past the 
  // first starting point, so we should delete the end point of the 
  // last segment (eventually the last segment will end at the starting
  // point)
  indices->pop_back();
  // Make sure the start point of the first segment is the same as 
  // the end point of the last segment.
  indices->push_back((*indices)[0]);

  return result;
}



// Here we find points of maximal curvature in the contour. The
// arguments "indices", "values" and "directions" are output arguments,
// that's where we put the indices, values of the magnitude of the 
// second derivative, and directions of curve rotation for each maximum.
long vContour::Maxima5(vector<long> * indices, vector<double> * values,
                        vector<long> * directions,
                        long neighborhood, double threshold)
{
  vector<long> cut_points, lengths;
  vector<double> magnitudes, orientations;
  long temp = CutPoints7(&cut_points, directions, &lengths, neighborhood,
                         threshold, &magnitudes, &orientations);
  return Maxima8(indices, values, directions, neighborhood, threshold,
                 &cut_points, &magnitudes, &orientations);
}



// This function is like the previous one, but instead of computing
// the magnitudes of the second derivative and the orientation angles
// at each contour point, it gets them as arguments. It also gets
// as an argument the cut points (as defined in RegionBoundaries()), 
// instead of computing them.
long vContour::Maxima8(vector<long> * indices, vector<double> * values,
                        vector<long> * directions, long neighborhood,
                        double threshold,
                        vector<long> * cuts,
                        vector<double> * magnitudes, 
                        vector<double> * orientations)
{
  if (neighborhood < 1) return 0;
  if (threshold > M_PI / 2) return 0;

  long number_of_cuts = cuts->size() - 1;
  long number_of_points = xs.size();
  long i;
  for (i = 0; i < number_of_cuts; i++)
  {
    long start = (*cuts)[i];
    long end = (*cuts)[i+1];
    double max_magnitude = (*magnitudes)[start];
    double max_index = start;
    long j;
    for (j = start + 1; j != end; j = (j + 1) % number_of_points)
    {
      double mag = (*magnitudes)[j];
      if (mag > max_magnitude)
      {
        max_magnitude = mag;
        max_index = j;
      }
    }
    indices->push_back((long) max_index);
    values->push_back(max_magnitude);
//    vPrint("(%lf, %lf)\n", ys[max_index], xs[max_index]);
  }

  return 1;
}


double vContour::MeasureElongation2(long index, long dp_length, double in_threshold)
{
  number_of_points = NumberOfPoints();
  long left = Decrement(index);
  long right = Increment(index);

  // For efficiency, compute the square of the threshold
  double threshold = in_threshold * in_threshold;
  long length = 3;
  while(1)
  {
    double left_x = xs[left];
    double left_y = ys[left];
    double right_x = xs[right];
    double right_y = ys[right];

    long left1 = Decrement(left);
    long right1 = Increment(right);
    if (left1 == right) break;
    if (right1 == left) break;
    double left1_x = xs[left1];
    double left1_y = ys[left1];
    double right1_x = xs[right1];
    double right1_y = ys[right1];

    double distance1 = S_Distance(left_x, left_y, right1_x, right1_y);
    double distance2 = S_Distance(left1_x, left1_y, right_x, right_y);
    double min_distance = Min(distance1, distance2);
    if (min_distance > threshold)
    {
      break;
    }
    if (min_distance == distance1)
    {
      right = right1;
    }
    else
    {
      left = left1;
    }
    length++;
  }
  double double_length = (double) length;
  return double_length;
}


// This function takes in as argument the index to a point in the contour.
// It returns a matrix whose i-th entry is the maximum length of a contour
// segment that containsthe indexed point and has width i. Widths and 
// lengths are measured in pixels. The length of a contour segment is 
// defined as the number of contour points between the endpoints of the segment.
// The width definition is much trickier, and depends on the "index" argument.
// It is obtained as follows: Match point-by-point
// the contour segment on the left of the 
// indexed point to the segment on the right of the indexed point using
// dynamic programming, so that the maximum distance between any two
// matched points is minimized. That maximum distance is defined as the width.
vElongation
vContour::MeasureElongation(long index, long in_dp_length, double in_threshold)
{
  number_of_points = NumberOfPoints();
  // We simplify our error checking by making certain requirements.
  if (number_of_points < 50) return vElongation();
  if (in_dp_length < 10) return vElongation();

  long dp_length;
  if (in_dp_length * 2 > number_of_points)
  {
    dp_length = number_of_points / 2;
  }
  else
  {
    dp_length = in_dp_length;
  }

  // Set up some useful arrays. We need, for each point on the left and right
  // side of the start point, to have its 2D coordinates and its index.
  vMatrix<long> left_rows_matrix(1, dp_length);
  vArray(long) left_rows = left_rows_matrix.Matrix();
  vMatrix<long> left_cols_matrix(1, dp_length);
  vArray(long) left_cols = left_cols_matrix.Matrix();
  vMatrix<long> left_indices_matrix(1, dp_length);
  vArray(long) left_indices = left_indices_matrix.Matrix();

  vMatrix<long> right_rows_matrix(1, dp_length);
  vArray(long) right_rows = right_rows_matrix.Matrix();
  vMatrix<long> right_cols_matrix(1, dp_length);
  vArray(long) right_cols = right_cols_matrix.Matrix();
  vMatrix<long> right_indices_matrix(1, dp_length);
  vArray(long) right_indices = right_indices_matrix.Matrix();

  // distances[i][j] will be the Euclidean distance between left point
  // i (i.e. point with index left_indices[i]) and right point j.
  v3dMatrix<double> distances_matrix(dp_length, dp_length, 1);
  vArray2(double) distances = distances_matrix.Matrix2(0);
  
  // matches will be the 2D array on which we will actually do
  // the dynamic programming.
  v3dMatrix<double> matches_matrix(dp_length, dp_length, 1);
  vArray2(double) matches = matches_matrix.Matrix2(0);

  long current_left = Decrement(index);
  long current_right = Increment(index);
  long contour_length = 1;

  double threshold = in_threshold * in_threshold;

  // left_length and right_length will denote the actual number 
  // of points on each side that are considered. Based on 
  // the assumptions that I utilize in the beginning of the function,
  // they will be set equal to dp_length, but that may change if
  // I change the way I implement the function.
  long left_length, right_length;
  long left = Increment(current_left);
  long i;
  // Get the coordinates and indices of points on the left side
  for (i = 0; i < dp_length; i++)
  {
    left = Decrement(left);
    if (left == index) 
    {
      i++;
      break;
    }
    left_rows[i] = (long) ys[left];
    left_cols[i] = (long) xs[left];
    left_indices[i] = left;
  }
  left_length = i;

  // Get the coordinates and indices of points on the right side.
  long right = Decrement(current_right);
  for (i = 0; i < dp_length; i++)
  {
    right = Increment(right);
    if (right == index) 
    {
      i++;
      break;
    }
    right_rows[i] = (long) ys[right];
    right_cols[i] = (long) xs[right];
    right_indices[i] = right;
  }
  right_length = i;

  // Compute distances between each left point and each right point
  long j;
  for (i = 0; i < left_length; i++)
  {
    long left_index = left_indices[i];
    long left_row = (long) ys[left_index];
    long left_col = (long) xs[left_index];
    for (j = 0; j < right_length; j++)
    {
      long right_index = right_indices[j];
      long right_row = (long) ys[right_index];
      long right_col = (long) xs[right_index];
      distances[i][j] = S_Distance(left_col, left_row, right_col, right_row);
    }
  }

  // Do dynamic programming.
  // First do first row and first col;
  for (i = 0; i < left_length; i++)
  {
     matches[i][0] = distances[i][0];
  }
  for (j = 0; j < right_length; j++)
  {
    matches[0][j] = distances[0][j];
  }

  // Now fill up the rest of the table. Entry i,j will be the 
  // min width of any path possible by which i and j can
  // be matched.
  for (i = 1; i < left_length; i++)
  {
    for (j = 1; j < right_length; j++)
    {
      double distance1 = matches[i-1][j];
      double distance2 = matches[i][j-1];
      double distance3 = distances[i][j];
      double min_distance = Min(distance1, distance2);
      matches[i][j] = Max(min_distance, distance3);
    }
  }

  // Now, for every left point, figure out how far we can 
  // go for each width.
  // width_limit - 1 is the maximum width we consider.
  long width_limit = (long) (in_threshold + 2); 
  double square_limit = width_limit * width_limit;

  // left_min_lengths[i][j] will have what we consider as the length
  // of the segment attainable from left point i using the given width
  // j. They are called "min" lengths because the length of a segment
  // is defined as the minimum between the length of its left segment
  // and the length of its right segment. left_lenghts[i][j] will
  // contain the actual sum of the lengths of the two segments. We
  // need to keep track of that, so that we know the end point of
  // the best segment for a given left point and width.
  vMatrix<long> left_lengths_matrix(left_length, width_limit);
  vArray2(long) left_lengths = left_lengths_matrix.Matrix2();
  vMatrix<long> left_min_lengths_matrix(left_length, width_limit);
  vArray2(long) left_min_lengths = left_min_lengths_matrix.Matrix2();
  vMatrix<long> * roots_matrix = vPrecalculateRoots((long) square_limit);
  vArray(long) roots = roots_matrix->Matrix();

  long index1, width;
  long index2;
  // Initialize all lengths to 0 at first
  for (index1 = 0; index1 < left_length; index1++)
  {
    for (width = 0; width < width_limit; width++)
    {
      left_lengths[index1][width] = 0;
      left_min_lengths[index1][width] = 0;
    }

    // Go through the distances. Whenever we find a width
    // less than the limit, we record it, and we record
    // the length of the corresponding segment.
    for (index2 = 0; index2 < right_length; index2++)
    {
      long distance = (long) matches[index1][index2];
      if (distance >= square_limit) continue;
      long root = roots[distance];
      long length = index1 + index2 + 1;
      long min_length = Min(index1, index2) + 1;
      left_min_lengths[index1][root] = min_length;
      left_lengths[index1][root] = length;
    }

    // Now, make sure that entries in a given row (corresponding to a 
    // left point) are monotonically increasing. If no matching distance
    // of, say, 50, was found, then in column 50 we should record 
    // the maximum length recorded so far in columns 0-49.
    long previous_min_length = left_min_lengths[index1][0];
    long previous_length = left_lengths[index1][0];
    for (width = 0; width < width_limit; width++)
    {
      // We check min_length as opposed to length, because otherwise
      // the best elongation found often starts at the fingertip
      // and ignores totally one of the sides around the fingertip.
      if (previous_min_length > left_min_lengths[index1][width])
//      if (previous_length > left_lengths[index1][width])
      {
        left_min_lengths[index1][width] = previous_min_length;
        left_lengths[index1][width] = previous_length;
      }
      else
      {
        previous_min_length = left_min_lengths[index1][width];
        previous_length = left_lengths[index1][width];
      }
    }
  }

  // Figure out, for each width, the longest attainable segment
  // (between a left and a right point and going through the starting
  // point.
  vElongation elongation(width_limit);
  vArray(double) lengths = elongation.lengths;
  double max_elongation = -1;
  long max_elongation_index = -1;
  long max_width = -1;
  for (width = 1; width < width_limit; width++)
  {
    long max_min_length = left_min_lengths[0][width];
    long max_length = left_lengths[0][width];
    long max_index = left_indices[0];
    for (index1 = 1; index1 < left_length; index1++)
    {
      long value = left_min_lengths[index1][width];
      if (value > max_min_length) 
      //long value = left_lengths[index1][width];
      //if (value > max_length)
      {
        max_min_length = value;
        //max_length = value;
        max_length = left_lengths[index1][width];
        //max_min_length = left_min_lengths[index1][width];
        max_index = left_indices[index1];
      }
    }
    //lengths[width] = max_min_length;
    lengths[width] = max_length;
    elongation.start_rows[width] = ys[max_index];
    elongation.start_cols[width] = xs[max_index];
    elongation.start_indices[width] = max_index;

    long end_index = (max_index + max_length + number_of_points - 1) % number_of_points;
    elongation.end_rows[width] = ys[end_index];
    elongation.end_cols[width] = xs[end_index];
    elongation.end_indices[width] = end_index;
    double elongation = ((double) max_length) / ((double) width);
    if (elongation >= max_elongation)
    {
      max_elongation = elongation;
      max_elongation_index = max_index;
      max_width = width;
    }

    //vPrint("lenghts[%li] = %li\n", thr, max_length);
  }
  *(elongation.max_elongation) = max_elongation;
  *(elongation.max_index) = max_elongation_index;
  *(elongation.max_width) = max_width;
  *(elongation.max_length) = elongation.lengths[max_width];

  *(elongation.max_start_row) = elongation.start_rows[max_width];
  *(elongation.max_start_col) = elongation.start_cols[max_width];
  *(elongation.max_start_index) = elongation.start_indices[max_width];
  *(elongation.max_end_row) = elongation.end_rows[max_width];
  *(elongation.max_end_col) = elongation.end_cols[max_width];
  *(elongation.max_end_index) = elongation.end_indices[max_width];
  
  *(elongation.starting_point_row) = ys[index];
  *(elongation.starting_point_col) = xs[index];
  *(elongation.starting_point_index) = index;

  vdelete(roots_matrix);
  return elongation;
}


vBoundingBox vContour::BoundingBox()
{
  vBoundingBox result;
  long left = image_cols;
  long right = -1;
  long top = image_rows;
  long bottom = -1;

  long i;
  number_of_points = NumberOfPoints();
  for (i = 0; i < number_of_points; i++)
  {
    long row = (long) ys[i];
    long col = (long) xs[i];
    if (row < top) top = row;
    if (row > bottom) bottom = row;
    if (col < left) left = col;
    if (col > right) right = col;
  }
  result.left = left;
  result.right = right;
  result.top = top;
  result.bottom = bottom;
  return result;
}


// This function basically calls ElongationStatistics3, and supplies
// to it reasonable default arguments for dp_length and threshold.
vElongation vContour::ElongationStatistics1(long index)
{
  vBoundingBox bounding_box = BoundingBox();
  long max_side = bounding_box.MaxSide();
//  double min_side = bounding_box.MinSide();
  long dp_length = max_side / 2;
  double threshold = max_side / 4.0;
  return ElongationStatistics3(index, dp_length, threshold);
}


// This function behaves like MeasureElongation, takes the same arguments
// and returns the same type of result. The only difference is that, in the
// result, the unit size
// in which widths and lengths are measured is 1% of dp_length 
vElongation vContour::ElongationStatistics3(long index, long dp_length, 
                                              double threshold)
{
  vElongation temp_result = MeasureElongation(index, dp_length, 
                                               threshold);
  // Create a result matrix, which is the result of converting temp_result
  // to our new units for measuring length and width (where 1 is
  // 1% of dp_length)
  long rows = temp_result.Rows();
  long cols = temp_result.Cols();
  vArray2(double) input2 = temp_result.Matrix2();

  double unit = ((double) dp_length) / 100.0;
  double double_max_width = ((double) cols) / unit;
  long max_width = (long) double_max_width;
    
  vElongation result = vElongation(max_width);
  // For debugging.
  vElongation * elongation = &result;
  vArray2(double) output2 = result.Matrix2();

  // Go through temp_results and normalize widths and lengths.
  long width_index;
  double max_elongation = -1;
  long max_index = -1;
  for (width_index = 0; width_index < max_width; width_index++)
  {
    double double_old_width = ((double) width_index) * unit;
    long old_width = (long) double_old_width;
    double old_length = (double) input2[0][old_width];
    double new_length = old_length / unit;
    output2[0][width_index] = new_length;
    long i;
    for (i = 1; i < rows - 1; i++)
    {
      output2[i][width_index] = input2[i][old_width];
    }
  }

  *(result.max_elongation) = *(temp_result.max_elongation);
  *(result.max_length) = *(temp_result.max_length) / unit;
  *(result.max_width) = *(temp_result.max_width) / unit;
  *(result.max_index) = *(temp_result.max_index);
  *(result.max_start_row) = *(temp_result.max_start_row);

  *(result.max_start_col) = *(temp_result.max_start_col);
  *(result.max_start_index) = *(temp_result.max_start_index);
  *(result.max_end_row) = *(temp_result.max_end_row);
  *(result.max_end_col) = *(temp_result.max_end_col);
  *(result.max_end_index) = *(temp_result.max_end_index);
  
  *(result.starting_point_row) = *(temp_result.starting_point_row);
  *(result.starting_point_col) = *(temp_result.starting_point_col);
  *(result.starting_point_index) = *(temp_result.starting_point_index);

  return result;
}


// This function was used to try out an idea of mine about how
// to get a skeleton-like feature efficiently. It ends up
// getting stuck if some local orientation is not cooperating.
vMatrix<double> vContour::GrowSkeleton(long index, long in_dp_length, 
                                         grayscale_image * silhouette)
{
  number_of_points = NumberOfPoints();

  if (number_of_points < 50) return vMatrix<double>();
  if (in_dp_length < 10) return vMatrix<double>();

  long dp_length;
  if (in_dp_length * 2 > number_of_points)
  {
    dp_length = (number_of_points - 2) / 2;
  }
  else
  {
    dp_length = in_dp_length;
  }

  vMatrix<double> result_matrix(3 * dp_length, 8);
  vArray2(double) result2 = result_matrix.Matrix2();
  vWriteValue(&result_matrix, (double) 0);

  // Set up some useful arrays. We need, for each point on the left and right
  // side of the start point, to have its 2D coordinates and its index.
  vMatrix<long> left_rows_matrix(1, dp_length);
  vArray(long) left_rows = left_rows_matrix.Matrix();
  vMatrix<long> left_cols_matrix(1, dp_length);
  vArray(long) left_cols = left_cols_matrix.Matrix();
  vMatrix<long> left_indices_matrix(1, dp_length);
  vArray(long) left_indices = left_indices_matrix.Matrix();

  vMatrix<long> right_rows_matrix(1, dp_length);
  vArray(long) right_rows = right_rows_matrix.Matrix();
  vMatrix<long> right_cols_matrix(1, dp_length);
  vArray(long) right_cols = right_cols_matrix.Matrix();
  vMatrix<long> right_indices_matrix(1, dp_length);
  vArray(long) right_indices = right_indices_matrix.Matrix();

  long current_left = Decrement(index);
  long current_right = Increment(index);
  long contour_length = 1;

  // left_length and right_length will denote the actual number 
  // of points on each side that are considered. Based on 
  // the assumptions that I utilize in the beginning of the function,
  // they will be set equal to dp_length, but that may change if
  // I change the way I implement the function.
  long left_length, right_length;
  long left = Increment(current_left);
  long i;
  // Get the coordinates and indices of points on the left side
  for (i = 0; i < dp_length; i++)
  {
    left = Decrement(left);
    if (left == index) 
    {
      i++;
      break;
    }
    left_rows[i] = (long) ys[left];
    left_cols[i] = (long) xs[left];
    left_indices[i] = left;
  }
  left_length = i;

  // Get the coordinates and indices of points on the right side.
  long right = Decrement(current_right);
  for (i = 0; i < dp_length; i++)
  {
    right = Increment(right);
    if (right == index) 
    {
      i++;
      break;
    }
    right_rows[i] = (long) ys[right];
    right_cols[i] = (long) xs[right];
    right_indices[i] = right;
  }
  right_length = i;

  long step = 0;
  long current_row = (long) ys[index];
  long current_col = (long) xs[index];
  long left_index = 0;
  long right_index = 0;
  vArray2(uchar) interior = silhouette->Matrix2(0);
  double min_distance = 0;

  while(1)
  {
    // Record information on current step (position of left, current and right points,
    // and the distance).
    long left_row = left_rows[left_index];
    long left_col = left_cols[left_index];
    long right_row = right_rows[right_index];
    long right_col = right_cols[right_index];
    result2[step][0] = left_row;
    result2[step][1] = left_col;
    result2[step][2] = right_row;
    result2[step][3] = right_col;
    result2[step][4] = current_row;
    result2[step][5] = current_col;
    result2[step][6] = min_distance;

    // Now we must advance at least one of left_index, right_index, and, 
    // optionally, we can advance the contour point.
    long left_next_index = left_index + 1;
    long right_next_index = right_index + 1;
    if ((left_next_index >= dp_length) &&
        (right_next_index >= dp_length))
    {
      break;
    }

    if (left_next_index >= dp_length) left_next_index = left_index;
    if (right_next_index >= dp_length) right_next_index = right_index;

    min_distance = 100 * dp_length * dp_length;
    long min_left_index, min_right_index, min_row, min_col;
    long row, col;
    for (row = current_row - 1; row <= current_row + 1; row++)
    {
      for (col = current_col - 1; col <= current_col + 1; col++)
      {
        // Don't consider (row, col) if it is out of bounds or
        // it is not a point in the interior of the contour.
        if (silhouette->check_bounds(row, col) <= 0) continue;
        if (interior[row][col] == 0) continue;

        double distance1 = DoubleDistance(row, col, 
                                          left_indices[left_next_index], 
                                          right_indices[right_index]);
        double distance2 = DoubleDistance(row, col, 
                                          left_indices[left_index], 
                                          right_indices[right_next_index]);
        long best_left, best_right;
        if (distance1 < distance2) 
        {
          best_left = left_next_index;
          best_right = right_index;
        }
        else
        {
          best_left = left_index;
          best_right = right_next_index;
        }
        double current_distance = Min(distance1, distance2);
        if (current_distance < min_distance)
        {
          min_distance = current_distance;
          min_row = row;
          min_col = col;
          min_left_index = best_left;
          min_right_index = best_right;
        }
      }
    }
    if ((min_left_index == left_index) && (min_right_index == right_index))
    {
      break;
    }

    left_index = min_left_index;
    right_index = min_right_index;
    current_row = min_row;
    current_col = min_col;
    step++;
  }
  return result_matrix;
}


// FingerFilter takes as input an elongation that can possibly be
// a finger. It trims the endpoints of the max elongation, using 
// the following rule: Consider the line connecting the two finger
// endpoints. If we start at one of the endpoints and we start following
// that line, the next pixel after the endpoint should be an interior 
// pixel of the hand. In most cases that will also mean that the line
// is entirely in the interior of the hand (so it can possibly
// be the base line of the finger).
long vContour::FingerFilter(vElongation * elongation)
{
  long left_index, right_index;
  left_index = right_index = 0;
  elongation->MaxSegmentIndices(&left_index, &right_index);
  long size = NumberOfPoints();
  // Used for the derivative.
  const long neighborhood = 10;
  static const double two_pi = 2 * M_PI;

  // Compare the angle between the baseline and the first derivative
  // at each endpoint. If first derivative at an endpoint points up, 
  // the baseline should point to the right. So, half of the angles
  // are acceptable (between first derivative and its opposite, but
  // rotating clockwise from first derivative). If the baseline is 
  // acceptable from both endpoints, we are done, otherwise we trim
  // one endpoint (alternatingly at the left and right ends) and
  // we check again.
  while(1)
  {
    long left_row, left_col, right_row, right_col;
    long current, previous, next;
    double dx, dy;
    long base_dx, base_dy;
    double base_direction, contour_direction, angle;
    long continue_flag = 0;

    // Break if we reached the right index.
    if (left_index == right_index) break;

    // Compute baseline direction starting at the left point
    left_row = (long) ys[left_index];
    left_col = (long) xs[left_index];
    right_row = (long) ys[right_index];
    right_col = (long) xs[right_index];
    base_dx = right_col - left_col;
    base_dy = right_row - left_row;

    // Compute derivative at left endpoint.
    current = left_index;
    previous = (current + size - neighborhood) % size;
    next = (current + neighborhood) % size;
    dx = xs[next] - xs[previous];
    dy = ys[next] - ys[previous];

    // Compute angle between baseline and derivative at left endpoint
    base_direction = vAngle2(base_dx, base_dy);
    contour_direction = vAngle2(dx, dy);
    angle = vCircularDistance(contour_direction, base_direction, two_pi);
    if (angle < 0) 
    {
      left_index++;
      continue_flag = 1;
    }

    // Break if we reached the right index.
    if (left_index == right_index) break;

    // Compute baseline direction starting at the left point
    left_row = (long) ys[left_index];
    left_col = (long) xs[left_index];
    right_row = (long) ys[right_index];
    right_col = (long) xs[right_index];
    base_dx = right_col - left_col;
    base_dy = right_row - left_row;

    // Compute derivative at right endpoint.
    current = right_index;
    previous = (current + size - neighborhood) % size;
    next = (current + neighborhood) % size;
    dx = xs[next] - xs[previous];
    dy = ys[next] - ys[previous];

    // Compute angle between baseline and derivative at right endpoint
    base_direction = vAngle2(base_dx, base_dy);
    contour_direction = vAngle2(dx, dy);
    angle = vCircularDistance(contour_direction, base_direction, two_pi);
    if (angle > 0) 
    {
      right_index--;
      continue_flag = 1;
    }
    if (continue_flag == 0) break;
  }

  *(elongation->max_start_row) = ys[left_index];
  *(elongation->max_start_col) = xs[left_index];
  *(elongation->max_start_index) = left_index;

  *(elongation->max_end_row) = ys[right_index];
  *(elongation->max_end_col) = xs[right_index];
  *(elongation->max_end_index) = right_index;

  *(elongation->max_length) = vDirectedCircularDistance(left_index, right_index, size, 1);
  *(elongation->max_elongation) = *(elongation->max_length) / *(elongation->max_width);
  
  return 1;
}

  
// A quick and dirty finger detector. dp_length and width_threshold
// are the parameters passed to ElongationStatistics3. neighborhood
// and curvature_threshold are the arguments passed to 
// ContourMaxima. Fingers whose contour length is less than 
// length_threshold are rejected, and if two fingers share a 
// contour segment whose length is more than overlap_threshold,
// then only the longest of them is retained.
long vContour::FindFingers9(vector<vElongation> * results, long dp_length,
                             double width_threshold, long neighborhood,
                             double curvature_threshold, 
                             double elongation_threshold,
                             long length_threshold, long min_width, 
                             long overlap_threshold)
{
  vector<long> indices, directions;
  vector<double> maxima;
  // Get local maxima in contour curvature
  Maxima5(&indices, &maxima, &directions, neighborhood, curvature_threshold);
  long number_of_points = directions.size();

  long i;
  vector<vElongation> temp_results;

  // Check each local maximum point for the possibility of being a 
  // fingertip.
  for (i = 0; i < number_of_points; i++)
  {
    double value = maxima[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) ys[raw_index];
    long col = (long) xs[raw_index];
    // For the current local maximum, reject it if 
    // its curvature is negative 
    // (turning the opposite way than the contour overall).
    long direction = directions[i];
    if (direction == 1)
    {
      // Get the elongation statistics of the local maximum 
      // (contour segment of the possible finger).
      vElongation elongation_statistics = 
        ElongationStatistics3(index, dp_length, width_threshold);
      // Trim endpoints of contour segment, so that the potential
      // finger baseline goes from each endpoint towards the
      // interior of the finger.
      FingerFilter(&elongation_statistics);
      
      // Reject the elongation if it is not elongated enough 
      // (elongation is width / contour_length) or if
      // is not long enough (in terms of contour length)
	    double elongation = elongation_statistics.MaxElongation();
	    if (elongation < elongation_threshold) continue;
      if (*(elongation_statistics.max_length) < length_threshold) continue;
      if (*(elongation_statistics.max_width) < min_width) continue;
      
      // Accept the elongation temporarily
      temp_results.push_back(elongation_statistics);
    }
  }

  // Now find overlapping elongations, and reject the least elongated
  // of them.
  long j;
  number_of_points = temp_results.size();
  for (i = 0; i < number_of_points; i++)
  {
    vElongation elongation1 = temp_results[i];
    long start1, end1;
    elongation1.MaxSegmentIndices(&start1, &end1);
    double longness1 = elongation1.MaxElongation();
    // accept_flag will be set to zero if elongation1 overlaps
    // too much with a longer elongation (in terms of contour points) .
    long accept_flag = 1;
    for (j = 0; j < number_of_points; j++)
    {
      if (i == j) continue;
      vElongation elongation2 = temp_results[j];
      long start2, end2;
      elongation2.MaxSegmentIndices(&start2, &end2);
      long overlap = vCircularSegmentOverlap(start1, end1, start2, end2, 
                                              number_of_points);
      if (overlap >= overlap_threshold)
      {
        double longness2 = elongation2.MaxElongation();
        if ((longness2 > longness1) ||
            ((longness2 == longness1) && (j > i)))
        {
          accept_flag = 0;
        }
      }
    }
    if (accept_flag > 0)
    {
      results->push_back(elongation1);
    }
  }
  
  return 1;
}


// Like FindFingers9. It calls FindFingers9, but it first picks some 
// default values for the arguments.
long vContour::FindFingers1(vector<vElongation> * results)
{
  vBoundingBox bounding_box = BoundingBox();
  long max_side = bounding_box.MaxSide();
//  double min_side = bounding_box.MinSide();
  long dp_length = max_side / 2;
  double search_width = max_side / 4.0;
  return FindFingers9(results, dp_length, search_width,
                      10, 0.3, 2, 15, 15, 15);
}

  
// Like FindFingers1, but it lets the caller also specify the 
// elongation threshold.
long vContour::FindFingers2(vector<vElongation> * results, 
                             double elongation_threshold)
{
  vBoundingBox bounding_box = BoundingBox();
  long max_side = bounding_box.MaxSide();
//  double min_side = bounding_box.MinSide();
  long dp_length = max_side / 2;
  double width_threshold = max_side / 4.0;
  return FindFingers9(results, dp_length, width_threshold,
                      10, 0.3, elongation_threshold, 15, 15, 15);
}

  
//////////////////////////////////////////////////////////////
// 
// End of implementation of class vContour
//
//////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////
// 
// Start of implementation of class vElongationBins
//
//////////////////////////////////////////////////////////////

// As of 10/8/2001 I have never used this class
vElongationBins::vElongationBins(const char * filename)
{
  is_valid = 0;
  finger_bins = 0;
  non_finger_bins = 0;
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return;

  long items;
  items = vReadPCLongs(fp, &width_bins, 1);
  if (items != 1) goto exit;
  items = vReadPCLongs(fp, &length_bins, 1);
  if (items != 1) goto exit;
  items = vReadPCLongs(fp, &width_factor, 1);
  if (items != 1) goto exit;
  items = vReadPCLongs(fp, &length_factor, 1);
  if (items != 1) goto exit;

  finger_bins = v3dMatrix<long>::Read(fp);
  if (finger_bins == 0) goto exit;
  non_finger_bins = v3dMatrix<long>::Read(fp);
  if (finger_bins == 0) goto exit;
  is_valid = 1;

exit:
  fclose(fp);
}


// As of 10/8/2001 I have never used this class
vElongationBins::vElongationBins(long in_width_bins, long in_length_bins,
                                   long in_width_factor, long in_length_factor)
{
  width_bins = in_width_bins;
  length_bins = in_length_bins;
  width_factor = in_width_factor;
  length_factor = in_length_factor;
  finger_bins = new v3dMatrix<long>(width_bins, length_bins, 1);
  non_finger_bins = new v3dMatrix<long>(width_bins, length_bins, 1);
  vWriteValue(finger_bins, (long) 0);
  vWriteValue(non_finger_bins, (long) 0);
  is_valid = 1;
}


// As of 10/8/2001 I have never used this class
vElongationBins::~vElongationBins()
{
  CleanUp();
}


// As of 10/8/2001 I have never used this class
long vElongationBins::CleanUp()
{
  vdelete(finger_bins);
  vdelete(non_finger_bins);
  return 1;
}


// As of 10/8/2001 I have never used this class
long vElongationBins::ProcessElongation(vElongation * elongation, 
                                         v3dMatrix<long> * bins_matrix)
{
  vArray(double) lengths = elongation->lengths;
  long number_of_lengths = elongation->Cols();

  vArray2(long) bins = bins_matrix->Matrix2(0);
  long width;
  for (width = 1; width < number_of_lengths; width++)
  {
    long width_index = WidthIndex(width);;
    double length = lengths[width] / (double) width;
    if (length < 0) return 0;
    long length_index = LengthIndex(length);
    bins[width_index][length_index]++;
  }
  return 1;
}


// As of 10/8/2001 I have never used this class
long vElongationBins::ProcessFingerElongation(vElongation * elongation)
{
  return ProcessElongation(elongation, finger_bins);
}


// As of 10/8/2001 I have never used this class
long vElongationBins::ProcessNonFingerElongation(vElongation * elongation)
{
  return ProcessElongation(elongation, non_finger_bins);
}


// As of 10/8/2001 I have never used this class
long vElongationBins::Save(const char * filename)
{
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) return 0;
  long result = 0;

  long items;
  items = vWritePCLongs(fp, &width_bins, 1);
  if (items != 1) goto exit;
  items = vWritePCLongs(fp, &length_bins, 1);
  if (items != 1) goto exit;
  items = vWritePCLongs(fp, &width_factor, 1);
  if (items != 1) goto exit;
  items = vWritePCLongs(fp, &length_factor, 1);
  if (items != 1) goto exit;

  long success;
  success = finger_bins->Write(fp);
  if (success <= 0) goto exit;
  success = non_finger_bins->Write(fp);
  if (success <= 0) goto exit;
  result = 1;

exit:
  fclose(fp);
  return result;
}


// As of 10/8/2001 I have never used this class
long vElongationBins::IsValid()
{
  return is_valid;
}


// As of 10/8/2001 I have never used this class
double vElongationBins::FingerProbability(double width, double length)
{
  if (width < 0) return 0;
  if (length < 0) return 0;
  long width_index = WidthIndex(width);
  long length_index = LengthIndex(length);
  vArray2(long) fingers = finger_bins->Matrix2(0);
  vArray2(long) non_fingers = non_finger_bins->Matrix2(0);
  double finger_count = fingers[width_index][length_index];
  double non_finger_count = non_fingers[width_index][length_index];
  double result = finger_count / (finger_count + non_finger_count);
  return result;
}


//////////////////////////////////////////////////////////////
// 
// End of implementation of class vElongationBins
//
//////////////////////////////////////////////////////////////


// For each frame in sequence_file, it finds fingers using 
// vContour::FindFingers2, and saves the results in a .fingers
// file. The format of the file is:
// frame1 ... frame_n
// where each frame is:
// left_start_row
// left_start_col
// right_start_row
// right_start_col
// tip_row
// tip_col
// width
// elongation
// orientation
// If end_frame is negative, it is set to the last frame in the sequence.
// It ignores fingers whose finger_tip has the value ignore_value, because
// in the training data we want to reject detected protrusions coming from 
// the palm. If ignore_value is not a valid uchar value, then no value
// gets ignored.
// For each frame read, we actually save information for that frame, and 
// three rotations of that frame (clockwise 90 degrees, 180 degrees and 270
// degrees respectively).
long vSaveFingerInfo(const char * sequence_file, double threshold,
                      long start_frame, long end_frame, long ignore_value)
{
  vPrint("\n");
  vVideoFile * input = vOpenReadVideoFile(sequence_file);
  if (input == 0) 
  {
    vPrint("Failed to open %s\n", sequence_file);
    return 0;
  }
  char * target_file = vReplaceEnding2(sequence_file, ".fingers");
  FILE * fp = fopen(target_file, vFOPEN_WRITE);
  int fseek_result;
  if (fp != 0)
  {
    fseek_result = fseek(fp, 32, SEEK_SET);
  }
  if ((fp == 0) || (fseek_result != 0))
  {
    vPrint("Failed to create %s\n", target_file);
    vCloseVideoFile(input);
    vdelete2(target_file);
    return 0;
  }

  long frames = input->NumberOfFrames();
  if ((end_frame < 0) || (end_frame >= frames))
  {
    end_frame = frames - 1;
  }

  long frame;
  long error_flag = 0;
  long saved_frames = 4 * (end_frame - start_frame + 1);
  long items;
  for (frame = start_frame; frame <= end_frame; frame++)
  {
    grayscale_image * rotated[4];
    rotated[0] = input->ReadFrameGray(frame);
    if (rotated[0] == 0)
    {
      vPrint("Error in reading frame %li\n", frame);
      error_flag = 1;
      break;
    }

    long j;
    for (j = 1; j < 4; j++)
    {
      rotated[j] = (grayscale_image *) vRotateRight(rotated[j-1]);
    }

    // For each rotation, calculate and save finger info
    for (j = 0; j < 4; j++)
    {
      grayscale_image * gray = rotated[j];
      vArray2(uchar) matrix2 = gray->Matrix2(0);
      vContour contour(gray);
      vector<double> * xs = contour.GetXs();
      vector<double> * ys = contour.GetYs();
      // We smooth the contour, but we must keep the original,
      // to look up the pixel value at the fingertip (possibly
      // the fingertip position in the smooth contour is actually
      // a background pixel).
      vContour * smoothed = contour.Smooth2(3);

      vector<vElongation> temp_elongations;
      smoothed->FindFingers2(&temp_elongations, threshold);
      vdelete(smoothed);
      vFingerInfo info;
      long tip_index = 0;
      long number = temp_elongations.size();
      vector<vElongation> elongations;
      // Reject elongations whose fingertip has the value ignore_value.
      long i;
      for (i = 0; i < number; i++)
      {
        vElongation elongation = temp_elongations[i];
        elongation.StartingPoint(&(info.tip_row), &(info.tip_col), &tip_index);
        info.tip_row = (long) (*ys)[tip_index];
        info.tip_col = (long) (*xs)[tip_index];
        long value = matrix2[info.tip_row][info.tip_col];
        if (value != ignore_value) 
        {
          elongations.push_back(elongation);
        }
      }

      // Save finger info..
      // First, right how many potential fingers we have detected.
      number = elongations.size();
      items = vWritePCLongs(fp, &number, 1);
      if (items != 1)
      {
        vPrint("Error in saving info for frame %li, rotation %li\n",
                frame, j);
        error_flag = 1;
        break;
      }
      for (i = 0; i < number; i++)
      {
        vElongation elongation = elongations[i];
        elongation.StartingPoint(&(info.tip_row), &(info.tip_col), 
                                 &tip_index);
        elongation.MaxSegment(&(info.left_row), &(info.left_col), 
                              &(info.right_row), &(info.right_col));
        elongation.StartingPoint(&(info.tip_row), &(info.tip_col), 
                                 &tip_index);
        info.elongation = elongation.MaxElongation();
        info.width = elongation.MaxElongationWidth();
        double base_row = ((double) (info.left_row + info.right_row)) / (double) 2.0;
        double base_col = ((double) (info.left_col + info.right_col)) / (double) 2.0;
        double vector_row = ((double) info.tip_row) - base_row;
        double vector_col = ((double) info.tip_col) - base_col;
        info.orientation = vAngle2(vector_col, vector_row);
        items = 0;
        items += vWritePCLongs(fp, &(info.left_row), 1);
        items += vWritePCLongs(fp, &(info.left_col), 1);
        items += vWritePCLongs(fp, &(info.right_row), 1);
        items += vWritePCLongs(fp, &(info.right_col), 1);
        items += vWritePCLongs(fp, &(info.tip_row), 1);
        items += vWritePCLongs(fp, &(info.tip_col), 1);
        items += fwrite(&(info.width), sizeof(double), 1, fp);
        items += fwrite(&(info.elongation), sizeof(double), 1, fp);
        items += fwrite(&(info.orientation), sizeof(double), 1, fp);
        if (items != 9)
        {
          vPrint("Error in writing frame %li, finger info %li, rotation %li\n",
                  frame, i, j);
          error_flag = 1;
          break;
        }
      }
      if (error_flag != 0)
      {
        break;
      }
    }
    for (j = 0; j < 4; j++)
    {
      vdelete(rotated[j]);
    }
    vPrint("Processed frame %li\r", frame);
    if (error_flag != 0) break;
  }

  vCloseVideoFile(input);
  fseek_result = fseek(fp, 0, SEEK_SET);
  if (fseek_result == 0)
  {
    items = vWritePCLongs(fp, &saved_frames, 1);
  }
  fclose(fp);
  vdelete2(target_file);
  vPrint("\n");
  if ((fseek_result != 0) || (items != 1)) 
  {
    vPrint("Error in writing number of frames to result file\n");
    return 0;
  }
  return 1;
}


vFingerInfo::vFingerInfo(vElongation in_elongation)
{
  long tip_index = 0;
  in_elongation.StartingPoint(&(tip_row), &(tip_col), 
                              &tip_index);
  in_elongation.MaxSegment(&(left_row), &(left_col), 
                        &(right_row), &(right_col));
  in_elongation.StartingPoint(&(tip_row), &(tip_col), 
                           &tip_index);
  elongation = in_elongation.MaxElongation();
  width = in_elongation.MaxElongationWidth();
  base_row = ((double) (left_row + right_row)) / (double) 2.0;
  base_col = ((double) (left_col + right_col)) / (double) 2.0;
  double vector_row = ((double) tip_row) - base_row;
  double vector_col = ((double) tip_col) - base_col;
  orientation = vAngle2(vector_col, vector_row);
}


long vFingerInfo::Print()
{
  vPrint("left (%li %li), right (%li %li), tip (%li %li)\n",
          left_row, left_col, right_row, right_col,
          tip_row, tip_col);
  vPrint("           width = %lf, elongation = %lf, orientation = %lf\n",
          width, elongation, orientation);
  return 1;
}


vSequenceFingerInfo::vSequenceFingerInfo(const char * fingers_file)
{
  numbers = 0;
  indices = 0;
  is_valid = 0;
  FILE * fp = fopen(fingers_file, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("can't open %s\n", fingers_file);
    return;
  }
  long items;
  int fseek_result;
  items = vReadPCLongs(fp, &number_of_frames, 1);
  if (items != 1) 
  {
    fclose(fp);
    return;
  }

  indices = new long[number_of_frames];
  numbers = new long[number_of_frames];

  fseek_result = fseek(fp, 32, SEEK_SET);
  if (fseek_result != 0)
  {
    fclose(fp);
    return;
  }
  long index = 0;
  long i;
  for (i = 0; i < number_of_frames; i++)
  {
    long elongations = 0;
    vReadPCLongs(fp, &elongations, 1);
    numbers[i] = elongations;
    indices[i] = index;
    long j;
    long error_flag = 0;
    for (j = 0; j < elongations; j++)
    {
      vFingerInfo info;
      items = vReadPCLongs(fp, &(info.left_row), 1);
      items += vReadPCLongs(fp, &(info.left_col), 1);
      items += vReadPCLongs(fp, &(info.right_row), 1);
      items += vReadPCLongs(fp, &(info.right_col), 1);
      items += vReadPCLongs(fp, &(info.tip_row), 1);
      items += vReadPCLongs(fp, &(info.tip_col), 1);
      items += fread(&(info.width), sizeof(double), 1, fp);
      items += fread(&(info.elongation), sizeof(double), 1, fp);
      items += fread(&(info.orientation), sizeof(double), 1, fp);
      info.base_row = ((double) (info.left_row + info.right_row)) / 2.0;
      info.base_col = ((double) (info.left_col + info.right_col)) / 2.0;
      frame_fingers.push_back(info);
      index++;
      if (items != 9) 
      {
        error_flag = 1;
        break;
      }
    }
    if (error_flag != 0) break;
  }
  fclose(fp);
  is_valid = 1;
}


vSequenceFingerInfo::~vSequenceFingerInfo()
{
  vdelete2(indices);
  vdelete2(numbers);
}


long vSequenceFingerInfo::Print(long start_frame, long end_frame)
{
  long i;
  if (start_frame < 0) start_frame = 0;
  if (start_frame >= number_of_frames) start_frame = number_of_frames - 1;
  if (end_frame < 0) end_frame = number_of_frames - 1;
  if (end_frame >= number_of_frames) end_frame = number_of_frames - 1;
  for (i = start_frame; i < end_frame; i++)
  {
    vPrint("\n\nFrame %li\n", i);
    long number = numbers[i];
    long j;
    for (j = 0; j < number; j++)
    {
      vFingerInfo & info = Get(i, j);
      vPrint("Finger info %li: ", j);
      info.Print();
    }
  }
  return 1;
}


vMatrix<double> vSequenceFingerInfo::Distances(vector<vElongation> * elongations)
{
  long number_of_fingers = elongations->size();
  vFingerInfo * in_fingers = new vFingerInfo[number_of_fingers];
  long i;
  for (i = 0; i < number_of_fingers; i++)
  {
    in_fingers[i] = vFingerInfo((*elongations)[i]);
  }
  vMatrix<double> result_matrix(1, number_of_frames);
  vArray(double) result = result_matrix.Matrix();

  long frame;
  for (frame = 0; frame < number_of_frames; frame++)
  {
    long train_number = numbers[frame];
    long index = indices[frame];
    long index_limit = index + train_number - 1;

    double max_distance = 0;
    // for each test and each frame finger, match them.
    long test_i, train_i;
    // First, match required test fingers to potential training fingers
    for (test_i = 0; test_i < number_of_fingers; test_i++)
    {
      vFingerInfo & test = in_fingers[test_i];
      if (test.elongation < 1.8) continue;
      double test_base_row = test.base_row;
      double test_base_col = test.base_col;
      double test_tip_row = test.tip_row;
      double test_tip_col = test.tip_col;
      double min_base_distance = 1000000;
      double min_tip_distance = 1000000;
      for (train_i = index; train_i <= index_limit; train_i++)
      {
        vFingerInfo & train = frame_fingers[train_i];
        if (train.elongation < 1.1) continue;
        double base_row_diff = test_base_row - train.base_row;
        double base_col_diff = test_base_col - train.base_col;
        double base_distance = base_row_diff * base_row_diff + 
                               base_col_diff * base_col_diff;
        if (base_distance < min_base_distance)
        {
          min_base_distance = base_distance;
        }
        double tip_row_diff = test_tip_row - train.tip_row;
        double tip_col_diff = test_tip_col - train.tip_col;
        double tip_distance = tip_row_diff * tip_row_diff + 
                               tip_col_diff * tip_col_diff;
        if (tip_distance < min_tip_distance)
        {
          min_tip_distance = tip_distance;
        }
      }
      max_distance = Max(max_distance, 
                         Max(min_base_distance, min_tip_distance));
    } 

    // Second, match required training fingers to potential test fingers
    for (train_i = index; train_i <= index_limit; train_i++)
    {
      vFingerInfo & train = frame_fingers[train_i];
      if (train.elongation < 1.8) continue;
      double train_base_row = train.base_row;
      double train_base_col = train.base_col;
      double train_tip_row = train.tip_row;
      double train_tip_col = train.tip_col;
      double min_base_distance = 1000000;
      double min_tip_distance = 1000000;
      for (test_i = 0; test_i < number_of_fingers; test_i++)
      {
        vFingerInfo & test = in_fingers[test_i];
        if (test.elongation < 1.1) continue;
        double base_row_diff = train_base_row - test.base_row;
        double base_col_diff = train_base_col - test.base_col;
        double base_distance = base_row_diff * base_row_diff + 
                               base_col_diff * base_col_diff;
        if (base_distance < min_base_distance)
        {
          min_base_distance = base_distance;
        }
        double tip_row_diff = train_tip_row - test.tip_row;
        double tip_col_diff = train_tip_col - test.tip_col;
        double tip_distance = tip_row_diff * tip_row_diff + 
                               tip_col_diff * tip_col_diff;
        if (tip_distance < min_tip_distance)
        {
          min_tip_distance = tip_distance;
        }
      }
      max_distance = Max(max_distance, 
                         Max(min_base_distance, min_tip_distance));
    } 
    result[frame] = max_distance;
  }
  vdelete2(in_fingers);
  return result_matrix;
}


double vSequenceFingerInfo::Distance(long frame, 
                                      vector<vElongation> * elongations)
{
  long number_of_fingers = elongations->size();
  vFingerInfo * in_fingers = new vFingerInfo[number_of_fingers];
  long i;
  for (i = 0; i < number_of_fingers; i++)
  {
    in_fingers[i] = vFingerInfo((*elongations)[i]);
  }

  long train_number = numbers[frame];
  long index = indices[frame];
  long index_limit = index + train_number - 1;

  double max_distance = 0;
  // for each test and each frame finger, match them.
  long test_i, train_i;
  // First, match required test fingers to potential training fingers
  for (test_i = 0; test_i < number_of_fingers; test_i++)
  {
    vFingerInfo & test = in_fingers[test_i];
    if (test.elongation < 1.8) continue;
    double test_base_row = test.base_row;
    double test_base_col = test.base_col;
    double test_tip_row = test.tip_row;
    double test_tip_col = test.tip_col;
    double min_base_distance = 1000000;
    double min_tip_distance = 1000000;
    for (train_i = index; train_i <= index_limit; train_i++)
    {
      vFingerInfo & train = frame_fingers[train_i];
      if (train.elongation < 1.1) continue;
      double base_row_diff = test_base_row - train.base_row;
      double base_col_diff = test_base_col - train.base_col;
      double base_distance = base_row_diff * base_row_diff + 
                             base_col_diff * base_col_diff;
      if (base_distance < min_base_distance)
      {
        min_base_distance = base_distance;
      }
      double tip_row_diff = test_tip_row - train.tip_row;
      double tip_col_diff = test_tip_col - train.tip_col;
      double tip_distance = tip_row_diff * tip_row_diff + 
                             tip_col_diff * tip_col_diff;
      if (tip_distance < min_tip_distance)
      {
        min_tip_distance = tip_distance;
      }
    }
    max_distance = Max(max_distance, 
                       Max(min_base_distance, min_tip_distance));
  } 

  // Second, match required training fingers to potential test fingers
  for (train_i = index; train_i <= index_limit; train_i++)
  {
    vFingerInfo & train = frame_fingers[train_i];
    if (train.elongation < 1.8) continue;
    double train_base_row = train.base_row;
    double train_base_col = train.base_col;
    double train_tip_row = train.tip_row;
    double train_tip_col = train.tip_col;
    double min_base_distance = 1000000;
    double min_tip_distance = 1000000;
    for (test_i = 0; test_i < number_of_fingers; test_i++)
    {
      vFingerInfo & test = in_fingers[test_i];
      if (test.elongation < 1.1) continue;
      double base_row_diff = train_base_row - test.base_row;
      double base_col_diff = train_base_col - test.base_col;
      double base_distance = base_row_diff * base_row_diff + 
                             base_col_diff * base_col_diff;
      if (base_distance < min_base_distance)
      {
        min_base_distance = base_distance;
      }
      double tip_row_diff = train_tip_row - test.tip_row;
      double tip_col_diff = train_tip_col - test.tip_col;
      double tip_distance = tip_row_diff * tip_row_diff + 
                             tip_col_diff * tip_col_diff;
      if (tip_distance < min_tip_distance)
      {
        min_tip_distance = tip_distance;
      }
    }
    max_distance = Max(max_distance, 
                       Max(min_base_distance, min_tip_distance));
  } 
  vdelete2(in_fingers);
  return max_distance;
}


// BorderContour is used when the contour we are interested in
// can possibly be non-circular, meaning that the two endpoints
// are distinct. A little complication that is introduced by
// non-circularity is that we can't start tracing the contour
// from any point, we need to first (starting from any point)
// trace the contour till we get to a dead-end, and that 
// gives us a starting point from which we can trace the
// entire contour
// At the time of calling this function, the scratch image is 15 
// for points that we do not wish to consider as possible contour
// points. More points are marked on the scratch image as this
// function progresses.
// start_row and start_col tell us the point at which we should
// start looking for the contour.
long vContour::BorderContour(grayscale_image * the_image, 
                              long start_row, long start_col,
                              grayscale_image * scratch_image)
{
  static S_PositionNeighbors position_neighbors[4][2];
  static S_ContourPosition next_positions[4][2][2][2];
  static long dummy1 = S_InitPositionNeighborsArray(position_neighbors);
  static long dummy2 = S_InitNextPositionArray(&next_positions);
  // Used to identify the bit in the scratch image that corresponds
  // the the side of interest (one of the four sides of the pixel).
  static long side_bits[] = {1, 2, 4, 8};

  CleanUp();
  image_rows = the_image->Rows();
  image_cols = the_image->Cols();

  // Make the border 0 so that, as we follow the contour, we never
  // fall out of bounds (so we don't have to check).
  vWriteBorderValue(the_image, (uchar) 0);
  
  // scratch[i][j] will be keep track of the sides of pixel i,j
  // that have been processed. This way we can always check if 
  // we have visited a side twice.
  // I think I have proven that no side will be visited twice
  // except for the starting side (which will be visited when
  // we have gone around the contour, and it is time to stop).
  // However, I am doing this test as a sanity check
  vArray2(uchar) scratch2 = scratch_image->Matrix2(0);
  vArray2(uchar) matrix2 = the_image->Matrix2(0);

  S_ContourPosition current(start_row, start_col, 0, 1);
  S_ContourPosition previous = current;

  // We start tracing the contour in one direction, to find the first endpoint
  xs.push_back(start_col);
  ys.push_back(start_row);
  scratch2[start_row][start_col] = scratch2[start_row][start_col] | (uchar) side_bits[0];
  while(1)
  {
    // Get the two neighbors of interest based on the current position. We
    // first get the neighbors' displacement with respect to the current 
    // pixel.
    S_PositionNeighbors neighbors = position_neighbors[current.side][current.corner];
    
    // Get the neighbors' coordinates based on their displacements.
    long next_row = neighbors.next_row + current.row;
    long next_col = neighbors.next_col + current.col;
    long diag_row = neighbors.diagonal_row + current.row;
    long diag_col = neighbors.diagonal_col + current.col;
    
    // Get the intensity values of the neighbors, and determine if they
    // are foreground or not.
    uchar next_value = matrix2[next_row][next_col];
    uchar diag_value = matrix2[diag_row][diag_col];
    long next_class = IsForeground(next_value);
    long diag_class = IsForeground(diag_value);

    // Get the next contour position based on the current pixel
    // position and the foreground/background status of the 
    // two neighbors. The next position's coordinates will
    // be relative to the current position
    S_ContourPosition next_position = 
      next_positions[current.side][current.corner][next_class][diag_class];

    previous = current;
    // Update current by moving it to the next position.
    current.row = current.row + next_position.row;
    current.col = current.col + next_position.col;
    current.side = next_position.side;
    current.corner = next_position.corner;
    
    // Check if we have been to that position (same side of same pixel)
    // before. 
    long visited = scratch2[current.row][current.col] & side_bits[current.side];
    if (visited != 0)
    {
        break;
    }

    // We get here if the current position (pixel coordinates and side)
    // has not been visited in the past. In that case, we mark it
    // now as visited.
    scratch2[current.row][current.col] |= side_bits[current.side];

    xs.push_back(current.col);
    ys.push_back(current.row);
  }

  long size = xs.size();

  // Reset the scratch2 array, since we will start tracing 
  // the contour again, from the endpoint that we found.
  long i;
  for (i = 0; i < size; i++)
  {
    long row = (long) ys[i];
    long col = (long) xs[i];
    scratch2[row][col] = 0;
  }

  // previous still has the value of the endpoint we found. In order
  // to make the contour tracing go in the reverse direction, we 
  // need to set current to previous and change the corner (i.e.
  // reverse direction)
  current = previous;
  current.corner = 1 - current.corner;

  // erase points from xs and ys, since we will reenter them starting
  // at the new endpoint.
  xs.erase(xs.begin(), xs.end());
  ys.erase(ys.begin(), ys.end());

  // We start tracing the contour from the first endpoint, to find
  // the other endpoint.
  xs.push_back(current.col);
  ys.push_back(current.row);
  scratch2[current.row][current.col] = scratch2[current.row][current.col] | (uchar) side_bits[current.side];
  while(1)
  {
    // Get the two neighbors of interest based on the current position. We
    // first get the neighbors' displacement with respect to the current 
    // pixel.
    S_PositionNeighbors neighbors = position_neighbors[current.side][current.corner];
    
    // Get the neighbors' coordinates based on their displacements.
    long next_row = neighbors.next_row + current.row;
    long next_col = neighbors.next_col + current.col;
    long diag_row = neighbors.diagonal_row + current.row;
    long diag_col = neighbors.diagonal_col + current.col;
    
    // Get the intensity values of the neighbors, and determine if they
    // are foreground or not.
    uchar next_value = matrix2[next_row][next_col];
    uchar diag_value = matrix2[diag_row][diag_col];
    long next_class = IsForeground(next_value);
    long diag_class = IsForeground(diag_value);

    // Get the next contour position based on the current pixel
    // position and the foreground/background status of the 
    // two neighbors. The next position's coordinates will
    // be relative to the current position
    S_ContourPosition next_position = 
      next_positions[current.side][current.corner][next_class][diag_class];

    // Update current by moving it to the next position.
    current.row = current.row + next_position.row;
    current.col = current.col + next_position.col;
    current.side = next_position.side;
    current.corner = next_position.corner;
    
    // Check if we have been to that position (same side of same pixel)
    // before. If yes, this means we got back to the start, or to a dead end.
    long visited = scratch2[current.row][current.col] & side_bits[current.side];
    if (visited != 0)
    {
        break;
    }

    // We get here if the current position (pixel coordinates and side)
    // has not been visited in the past. In that case, we mark it
    // now as visited.
    scratch2[current.row][current.col] |= side_bits[current.side];

    xs.push_back(current.col);
    ys.push_back(current.row);
  }

  Pack2();
  initialized = 1;
  number_of_points = NumberOfPoints();
  return 1;
}


