#ifndef VASSILIS_INTERFACE_PROGRAMS_H
#define VASSILIS_INTERFACE_PROGRAMS_H


// In this file and the corresponding CPP file I define lots of 
// miscellaneous functions. This was meant as an alternative
// to implementing many standalone programs, each of which 
// would perform those functions. From interface or text_interface
// I can type
//
// program aaaaa ---
//
// and the function vInterpreter::InteractiveExecuteProgram() will be 
// called, which will call vExecuteProgram, which will call  the 
// appropriate function. Note that a --- must always come at 
// the end, to signify the end of "command line" arguments that
// should be passed to the program.
// The functions defined here can also be called from a command line
// invocation of text_interface,from the DOS prompt.
// For example, instead of doing 
//
// program aaaaa arg1 arg2 arg3 ---
//
// from the prompt while running interface (or text_interface), I can invoke
// text interface from the DOS prompt with the command 
//
// text_interface aaaaa arg1 arg2 arg3
//
// and that will do the same thing, except that it will exit after
// executing that program.


// This function takes in a vector of arguments, the first
// of which is the name of the "program" to be executed. The
// function figures out which function corresponds to the 
// particular program, and passes the arguments to that 
// function.
long vExecuteProgram(vector<const char *> * arguments);

long vConvertBenStory0(vector<const char *> * arguments);
long vConvertBenStory(vector<voidp> * filenames);

// This program is used from python scripts in Poser, when
// they save images corresponding to each viewpoint of 
// each frame of each transition. After finishing with a transition,
// it is useful to go through all the BMP files and save them images
// in a single RLE sequence, in order to keep the total storage low,
// and the number of files also low. The only argument is a string
// that is the base of the filename of all the BMP files.
long vPoserBMPToRLE(vector<const char *> * arguments);

// This program is used from python scripts in Poser, when
// they save images corresponding to each viewpoint of 
// each frame of each transition. After finishing with a transition,
// it is useful to go through all the BMP files and save them images
// in a single AVI sequence, in order to keep the total storage low,
// and the number of files also low. The only argument is a string
// that is the base of the filename of all the BMP files.
// The assumption is that the image files are named as 
// output_base_[a]_[b].bmp, where a and b are counters.
long vPoserBMPToAVI(vector<const char *> * arguments);

// This program is used to determine, given current camera params,
// link world matrix, link orientation, and a list of viewpoints,
// a list of camera parameters under which the link will be seen having
// the orientations described by the viewpoints. We assume that the 
// camera is pointing at the link, so that we can ignore the actual
// 3D positions of the link and the camera.
long vPoserLinkParams(vector<const char *> * arguments);


// This program simply goes through all the files matching all the arguments
// and prints out the frame rates for the video sequences stored in those
// files.
long vFrameRate(vector<const char *> * arguments);


// This program reads video files and saves copies of those files
// in a specified directory. The frame rate in the copies is
// set to the specified rate and scale. It is assumed that
// the video files are of format either AVI or AVISET.
long vSetFrameRate(vector<const char *> * arguments);


#endif // VASSILIS_INTERFACE_PROGRAMS_H
