#include "stdafx.h"

#include "vplatform.h"

#include <assert.h>
#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#include <fstream>
#else 
#include <iostream>
#include <fstream>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef VASSILIS_WINDOWS
#include <direct.h>
#include <vector> 
#include "pc_aux.h"
#include "opencv_utils.h"
#endif // VASSILIS_WINDOWS

#ifdef VASSILIS_SGI_PLATFORM
#include <algo.h>
#include <vector.h>
#endif // VASSILIS_SGI_PLATFORM


#include "basics/auxiliaries.h"
#include "basics/drawing.h"
#include "basics/drawing_temp.h"
#include "gesture_data.h"
#include "basics/read_avi.h"
#include "basics/video_common.h"
#include "gestures/matching.h"
#include "basics/wrapper.h"
#include "basics/io_aux.h"
#include "window.h"

#include "basics/rle_encoding.h"
#include "basics/video_file.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "algorithms.h"
#include "algo_templates.h"
#include "interpreter.h"
#include "volumetric.h"
#include "chamfer_matching.h"

#ifndef VASSILIS_LINUX_PLATFORM
#include "../leon/SkinTracker.h"
#endif
#include "basics/definitions.h"

using namespace std;

const long NUMBER_OF_SKIN_TABLES = 7;

template<class type>
static type S_EraseItem(vector<type> * v, long index);


template<class type>
static type S_EraseItem(vector<type> * v, long index)
{
  if ((index < 0) || (index >= (long) v->size()))
  {
    return (type) 0;
  }
  type result = (*v)[index];
  vector<type>::iterator iter = v->begin();
  long i;
  for (i = 0; i < index; i++)
  {
    iter++;
  }
  v->erase(iter);
  return result;
}


class_sequence_information::class_sequence_information(long in_buffer_size)
{
  sequence = 0;
  path_name = 0;
  simple_name = 0;
  image_buffer = vZero(general_image *);
  buffer_size = in_buffer_size;
  buffer_index = 0;
  current_image = 0;

  sequence_step = 1;
  display_top = 0;
  display_left = 0;
  resolution = 0;
  current_frame = 0;

  center_row = 0;
  center_col = 0;
  automatic_centering = 1;
  //magnified_rows = -1;
  //magnified_cols = -1;

  face_color = 0;
  is_camera_input = 0;

  SetImageBufferSize(buffer_size);
}


class_sequence_information::~class_sequence_information()
{
  vCloseVideoFile(sequence);
  vdelete2(path_name);
  vdelete2(simple_name);
  vdelete(face_color);
  ClearImageBuffer();
}



ushort class_sequence_information::ClearImageBuffer()
{
  if (vIsZero(image_buffer)) return 0;
  long i;
  for (i = 0; i < buffer_size; i++)
  {
    general_image * image = image_buffer[i];
    vdelete(image);
  }
  vdelete2(image_buffer);
  image_buffer = vZero(general_image *);
  buffer_index = 0;
  current_image = 0;
  return 1;
}


ushort class_sequence_information::SetImageBufferSize(long number)
{
  general_image * current_copy = 0;
  if (current_image != 0)
  {
   current_copy = current_image->Copy();
  }
  ClearImageBuffer();
  buffer_size = number;
  image_buffer = vnew(general_image *, buffer_size);
  long i;
  for (i = 0; i < buffer_size; i++)
    image_buffer[i] = 0;
  buffer_index = 0;
  image_buffer[buffer_index] = current_copy;
  current_image = current_copy;
  return 1;
}


long class_sequence_information::CurrentFrame()
{
  return current_frame;
}


ushort class_sequence_information::SetCurrentFrame(long index)
{
  if (index < 0) return 0;
  if (index >= number_of_frames) return 0;
  current_frame = index;
  return 1;
}
  

vInterpreter1::vInterpreter1(class_interpreter* in_interpreter) :
class_module (in_interpreter)
{
#ifndef VASSILIS_AX_DEMO
  vInitializeConsole();
#endif
  window = 0;
  pixel_info_window = 0;
  frame_info_window = 0;
  input_fp = 0;

  current_input_index = -1;
  number_of_inputs = 0;
  input = 0;
  current_output = -1;
  number_of_outputs = 0;
  number_of_images = 0;

  skin_data_initialized = 0;
  skin_tables = vZero(vSkinTableBase *);
  current_table_index = 6;
  skin_directory = platform_skin_directory;
  non_skin_directory = platform_non_skin_directory;

  background_model = 0;
  calibration = 0;
  #ifndef VASSILIS_LINUX_PLATFORM
  skin_tracker = 0;
  Informedia_DetectorPath = platform_face_detector_path;
  #endif
  match_scores = 0;
  score_base = 0;
  score_frame = 0;
  test_score_file = 0;
  ground_truth_distance = 30;

  // Valid score_order values are 0 and 1, this value T gets ignored.
  score_order = 10;

  filter_blur_size = 5;
  hand_threshold = 15;

  current_directory = 0;
  default_buffer_size = 10;
  mouse_click_response = pixel_info_response;
  line_set = 0;
  left_hand = 0;

  histogram_equalization_flag = 0;
  equalization_window_size = 101;
  preserve_current_frame = 0;
  frame_rate = 30;
}


vInterpreter1::~vInterpreter1()
{
  CleanUp();
}


void vInterpreter1::Initialize(vGestureInterfaceGlobals * in_globals)
{
  window = in_globals->window; 

  pixel_info_window = in_globals->pixel_info_window;
  frame_info_window = in_globals->frame_info_window;
  min_width = in_globals->min_width;
  extra_height = in_globals->extra_height; 
  ground_distances = 0;
}



void vInterpreter1::CleanUp()
{
  vdelete(window);
  window = 0;
  vdelete(frame_info_window);
  frame_info_window = 0;
  vdelete(pixel_info_window);
  pixel_info_window = 0;
  if (input_fp != 0)
  {
    fclose(input_fp);
  }
 
  // If there is a camera input, mark it as a non-camera
  // input so that it can be removed.
  long i;
  for (i = 0; i < number_of_inputs; i++)
  {
    class_sequence_information * info = (class_sequence_information *) inputs[i];
    info->is_camera_input = 0;
  }


  ClearInputs();
  ClearOutputs();
  ClearImages();

  DeleteSkinData();

  vdelete(background_model);
  vdelete(calibration);
  #ifndef VASSILIS_LINUX_PLATFORM
  vdelete(skin_tracker);
  #endif 
  vdelete(match_scores);
  vdelete2(score_base);
  vdelete2(test_score_file);

  vdelete2(current_directory);
  vdelete(line_set);
  vdelete(ground_distances);
}


long vInterpreter1::DeleteSkinData()
{
  if (skin_data_initialized != 0)
  {	
    long i;
    for (i = 0; i < NUMBER_OF_SKIN_TABLES - 1; i++)
    {
      vdelete((vSkinTable<long> *&) skin_tables[i]);
    }
    vdelete((vSkinTable<double> *&) skin_tables[i]);
    vdelete2(skin_tables);

    vSetZero(skin_tables);
    skin_data_initialized = 0;
  }

  return 1;
}



long vInterpreter1::vGetNonEmptyLine(char * buffer, long max_length)
{
  if (input_fp == 0)
  {
    long result = ::vGetNonEmptyLine(buffer, max_length);
    return result;
  }
  else
  {
    long result = vFileGetNonEmptyLine(input_fp, buffer, max_length);
    return result;
  }
}

  
long vInterpreter1::store_image(general_image * the_image)
{
  if (the_image == 0) return 0;
  if (input == 0)
  {
    exit_error ("\nerror: show_image called without an input\n");
  }

  // Since images that are shown by calling ShowImage get deleted
  // automatically by the system, we should never call ShowImage
  // if there is no current input sequence. If we did, then
  // we would not be able to display it and save it in an image
  // ring from which it would ultimately get deleted, and we would
  // get a memory leak.

  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + 1) % buffer_size;
  vdelete(input->image_buffer[input->buffer_index]);

  input->image_buffer[input->buffer_index] = the_image;
  input->current_image = the_image;

  return 1;
}


long vInterpreter1::ShowImage(general_image * the_image)
{
  if (the_image == 0) return 0;
  if (input == 0) 
  {
    print("\ncreating new input\n");
    default_input(50, 50);
    if (input == 0)
    {
      exit_error("\nerror: failed to create default input\n");
    }
  }

  store_image(the_image);
  DrawCurrentImage();
  return 1;
}


long vInterpreter1::AddInput()
{
  vPrint("Add sequence: ");
  char buffer[400];
  vScan("%s", buffer);
  return AddInput(buffer);
}


long vInterpreter1::AddInput(const char * filename)
{
  if (filename[0] == 0) return 0;
  vVideoFile * new_video_file;
  new_video_file = vOpenReadVideoFile(filename);

  return AddInput(new_video_file);
}


long vInterpreter1::AddInput(vVideoFile * new_video_file)
{
  if (new_video_file == 0)
  {
    return 0;
  }

  class_sequence_information * info = new class_sequence_information(default_buffer_size);
  info->sequence = new_video_file;
  info->path_name = vCopyString(new_video_file->GetFilename());
  info->simple_name = vSimpleName(info->path_name, platform_directory_slash);
  info->number_of_frames = new_video_file->NumberOfFrames();
  //info->magnified_rows = new_video_file->Rows();
  //info->magnified_cols = new_video_file->Cols();
  inputs.push_back(info);

  number_of_inputs++;

  SetCurrentInput(number_of_inputs - 1);
  long success = AdvanceSequence(0);
  if (success == 0)
  {
    vPrint("Error in reading first frame of file %s\n", info->path_name);
    RemoveInput(number_of_inputs - 1);
    return 0;
  }

  return 1;
}


long vInterpreter1::default_input0()
{
  vPrint("Input rows and cols for input: ");
  long rows = 0;
  long cols = 0;
  vScan("%li %li", &rows, &cols);
  return default_input(rows, cols);
}


// We can't call ShowImage unless we have a current input.
// However, sometimes we would like to create our own images
// and display them without having opened a sequence first.
// DefaultInput does just that, it creates a class_sequence_information
// object that corresponds to no input video sequence
long vInterpreter1::default_input(long rows, long cols)
{
  class_sequence_information * info = new class_sequence_information(default_buffer_size);
  info->sequence = 0;
  info->path_name = vCopyString("default input");
  info->simple_name = vCopyString("default input");
  info->number_of_frames = 0;
  inputs.push_back(info);

  number_of_inputs++;
  SetCurrentInput(number_of_inputs - 1);
  NewImage(rows, cols);

  return 1;
}


long vInterpreter1::RemoveInput(long index)
{
  class_sequence_information * info;
  if (index < 0) return 0;
  if (index >= number_of_inputs) return 0;
  info = (class_sequence_information *) inputs[index];
  if (info->is_camera_input != 0) return 0;

  info = (class_sequence_information *) S_EraseItem(&inputs, index);
  if (info == 0) return 0;
  vdelete(info);
  number_of_inputs--;
  if (number_of_inputs == 0)
  {
    current_input_index = -1;
    input = 0;
  }
  else if ((current_input_index >= index) && 
           (current_input_index >= 1))
           
  {
    SetCurrentInput(current_input_index - 1);
  }
  else
  {
    SetCurrentInput(0);
  }

  return 1;
}


long vInterpreter1::ClearInputs()
{
  while(number_of_inputs > 0)
  {
    RemoveInput(0);
  }

  return 1;
}


long vInterpreter1::PrintInputs()
{
  long i; 
  for (i = 0; i < number_of_inputs; i++)
  {
    class_sequence_information * info = (class_sequence_information *) inputs[i];
    long current_frame = info->CurrentFrame();
    long number_of_frames = info->number_of_frames;
    char * filename = info->simple_name;
    vPrint("%li :Frame %li of %li, %s\n", i, current_frame, number_of_frames, filename);
  }

  return 1;
}

// CurrentImage(i) returns the current image of the i-th
// input file, or 0 if it doesn't exist.
general_image * vInterpreter1::CurrentImage(long index)
{
  if (index < 0) return 0;
  if (index >= number_of_inputs) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[index];
  return info->current_image;
}

  
long vInterpreter1::AddOutput()
{
  char filename[1000];
  vPrint("Name of new output file: ");
  vScan("%s", filename);
  return AddOutput(filename);
}


// Add an output sequence, to which we can write frames.
// If the specified filename already exists, then we 
// alter the filename, copy into it the original sequence,
// and keep it open for writing.
long vInterpreter1::AddOutput(const char * in_filename)
{
  if (input == 0) return 0;
  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  char * filename;
  ushort exists = 0;
  if (vFileExists(in_filename))
  {
    char * name = vRemoveEnding(in_filename);
    char * ending = vGetEnding(in_filename);
    filename = vMergeStrings3(name, "2", ending);
    exists = 1;
  }
  else
  { 
    filename = vCopyString(in_filename);
  }
  
  vVideoFile * output;
  vVideoFile * in_sequence = input->sequence;
  if (in_sequence == 0)
  {
    output = vOpenWriteVideoFile(filename, rows, cols);
  }
  else
  {
    long rate = in_sequence->Rate();
    long scale = in_sequence->Scale();
    long color_bytes = in_sequence->BytesPerPixel();
    output = vOpenWriteVideoFile(filename, rows, cols, 30, 1, color_bytes);
  }

  if (exists)
  {
    // Try to write the existing video file into the beginning of 
    // the new video file.
    long success = vWriteVideo(output, in_filename);
    // We could fail, if, for example, the sequence in in_filename
    // has a different frame size (rows and cols) than the current
    // input image.
    if (success == 0)
    {
      vCloseVideoFile(output);
      vPrint("Failed to copy %s to %s\n", in_filename, filename);
      vdelete2(filename);
      return 0;
    }
  }

  vdelete2(filename);
  return AddOutput(output);
}
// Add an output sequence, to which we can write frames.
// If the specified filename already exists, then we 
// alter the filename, copy into it the original sequence,
// and keep it open for writing.
long vInterpreter1::add_output_stream()
{
  if (input == 0) return 0;
  if (input->current_image == 0)
  {
    return 0;
  }

  char in_filename [1000];
  print ("\nenter filename:\n");
  question ("%s", in_filename);

  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  long bands = input->current_image-> Bands ();
  ::Type type_name = input->current_image->TypeName();

  char * filename;
  ushort exists = 0;
  if (vFileExists(in_filename))
  {
    char * name = vRemoveEnding(in_filename);
    char * ending = vGetEnding(in_filename);
    filename = vMergeStrings3(name, "2", ending);
    exists = 1;
  }
  else
  { 
    filename = vCopyString(in_filename);
  }
  
  class_image_stream * output = new class_image_stream (filename, rows, cols, bands, type_name);
  vdelete2(filename);
  return AddOutput(output);
}


long vInterpreter1::AddOutput(vVideoFile * output)
{
  if ((output == 0) || (output->IsValid() == 0)) return 0;
  output_files.push_back(output);
  output_names.push_back(vCopyString(output->GetFilename()));
  number_of_outputs++;
  current_output = number_of_outputs - 1;
  printf("Created output %s\n", output_names.back());

  return 1;
}


long vInterpreter1::RemoveOutput(long index)
{
  vVideoFile * sequence;
  sequence = (vVideoFile *) S_EraseItem(&output_files, index);
  if (sequence == 0) return 0;
  ushort result = 1;
  ushort success = sequence->FinishWriting();
  char * name = (char *) S_EraseItem(&output_names, index);
  vPrint("Closing %s\n", name);
  if (success == 0) 
  {
    result = 0;
    vPrint("Failed to close %s\n", name);
  }
  vCloseVideoFile(sequence);

  vdelete2(name);
  number_of_outputs--;
  if (current_output >= index) current_output--;

  return result;
}



long vInterpreter1::RemoveCurrentOutput()
{
  return RemoveOutput(current_output);
}

long vInterpreter1::ClearOutputs()
{
  while(number_of_outputs > 0)
  {
    RemoveOutput(0);
  }
  return 1;
}


long vInterpreter1::write_frame()
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  general_image * current = input->current_image;
  ushort result = 0;
  
  if (strcmp(output->Type(), "class_image_stream") == 0)
  {
    class_image_stream* stream = (class_image_stream*) output;

    //    double_image * image = new double_image(current);
    //    result = stream->save_image(image);
    //    vdelete(image);
    result = stream->save_image(current);
  }
  else if (predicate_color_image(current)) 
  {
    result = output->write_frame((color_image *) current);
  }
  else if (predicate_gray_image(current)) 
  {
    result = output->write_frameGray((grayscale_image *) current);
  }
  if (result != 0) vPrint("Wrote frame\n");
  else vPrint("failed to save frame\n");

  return result;
}


long vInterpreter1::write_frames0()
{
  if (current_output == -1) return 0;
  if (input == 0) return 0;
  long number_of_frames = 0;
  vPrint("Input number of frames to write: ");
  vScan("%li", &number_of_frames);
  return vInterpreter1::write_frames(number_of_frames);
}


long vInterpreter1::write_frames(long number_of_frames)
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;

  if (number_of_frames <= 0) return 0;
  long buffer_size = input->buffer_size;
  if (number_of_frames > buffer_size) return 0;

  vPrint("Writing frames: \n");
  long i;
  for (i = 0; i < number_of_frames; i++)
  {
    write_frame();
    ShowNextImage();
    vPrint("Wrote frame %li\r", i);
  }
  vPrint("\nFinished writing frames\n");
  return 1;
}


long vInterpreter1::AddImage(long sequence_index, long frame)
{
  vVideoFile * sequence = GetInput(sequence_index);
  if (sequence == 0) return 0;  
  color_image * the_image = sequence->ReadFrame(frame);
  images.push_back(the_image);
  number_of_images++;
  return 1;
}


long vInterpreter1::RemoveImage(long index)
{
  general_image * image;
  image = (general_image *) S_EraseItem(&images, index);
  if (image == 0) return 0;
  vdelete(image);
  number_of_images--;
  return 1;
}


long vInterpreter1::ClearImages()
{
  while(number_of_images > 0)
  {
    RemoveImage(0);
  }
  return 1;
}



long vInterpreter1::ChangeDirectory(const char * directory)
{
  long number = chdir(directory);
  ushort result;
  if (number == 0)
  {
    vdelete2(current_directory);
    current_directory = vCopyString(directory);
    vPrint("Changed to directory %s\n", current_directory);
    result = 1;
  }
  else
  {
    vPrint("Failed to change directory to %s.\n", directory);
    result = 0;
  }
  return result;
}


vVideoFile* vInterpreter1::GetInput(long index)
{
  if (index >= number_of_inputs) return 0;
  if (index < 0) return 0;
  class_sequence_information * info =  (class_sequence_information *) inputs[index];
  return info->sequence;
}


vVideoFile* vInterpreter1::GetOutput(long index)
{
  if (index >= number_of_outputs) return 0;
  return (vVideoFile *) output_files[index];
}


general_image * vInterpreter1::GetImage(long index)
{
  if (index >= number_of_images) return 0;
  return (general_image *) images[index];
}


color_image * vInterpreter1::GetColorImage(long index)
{
  general_image * base = GetImage(index);
  if (base == 0) return 0;
  if (base->TypeName() != UcharType) return 0;
  if (base->Bands() != 3) return 0;
  return (color_image *) base;
}


grayscale_image * vInterpreter1::GetGrayImage(long index)
{
  general_image * base = GetImage(index);
  if (base == 0) return 0;
  if (base->TypeName() != UcharType) return 0;
  if (base->Bands() != 1) return 0;
  return (grayscale_image *) base;
}


vWindow * vInterpreter1::GetWindow()
{
  return window;
}


long vInterpreter1::CompareContours(long frame1, long frame2)
{
  ushort result = 0;
  double score = 0;
  long current = current_input_index;

  general_image * base1 = CurrentImage(frame1);
  general_image * base2 = CurrentImage(frame2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * binary1 = (grayscale_image *) base1->threshold(1);
  SetCurrentInput(frame1);
  ShowImage(binary1);
  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  SetCurrentInput(frame2);
  ShowImage(binary2);
  SetCurrentInput(current);
  score = vCompareContours(binary1, binary2);
  vPrint("Score = %lf\n", score);
  return 1;
}


long vInterpreter1::CompareNormalizedContours(long frame1, long frame2)
{
  ushort result = 0;
  double score = 0;
  long current = current_input_index;

  general_image * base1 = CurrentImage(frame1);
  general_image * base2 = CurrentImage(frame2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * binary1 = vNormalizedBinary(base1, 96.0, 256, 256);
//  grayscale_image * binary1 = base1->ToGray();
  SetCurrentInput(frame1);
  ShowImage(binary1);
  //grayscale_image * binary2 = vNormalizedBinary(base2, 96.0, 256, 256);
  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  SetCurrentInput(frame2);
  ShowImage(binary2);
  SetCurrentInput(current);
  score = vCompareContours(binary1, binary2);
  vPrint("Score = %lf\n", score);
  //delete(binary1);
  //delete(binary2);
  return 1;
}


long vInterpreter1::CompareRescaledContours(long frame1, long frame2)
{
  ushort result = 0;
  double score = 0;
  long current = current_input_index;

  general_image * base1 = CurrentImage(frame1);
  general_image * base2 = CurrentImage(frame2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * gray = base1->ToGray();
  grayscale_image * binary1 = (grayscale_image *) vScaleBoundingBox6(gray, 192, 128, 128, 
                                                              256, 256);
  if (gray != base1) vdelete(gray);
  SetCurrentInput(frame1);
  ShowImage(binary1);

  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  SetCurrentInput(frame2);
  ShowImage(binary2);
  SetCurrentInput(current);
  score = vCompareContours(binary1, binary2);
  vPrint("Score = %lf\n", score);

  return 1;
}


long vInterpreter1::CompareRescaledContoursFast(long frame1, long frame2, long parameter)
{
  ushort result = 0;
  double score = 0;
  long current = current_input_index;

  general_image * base1 = CurrentImage(frame1);
  general_image * base2 = CurrentImage(frame2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * gray = base1->ToGray();
  grayscale_image * binary1 = (grayscale_image *) vScaleBoundingBox6(gray, 192, 128, 128, 256, 256);
  if (gray != base1) vdelete(gray);
  SetCurrentInput(frame1);
  ShowImage(binary1);

  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  SetCurrentInput(frame2);
  ShowImage(binary2);
  SetCurrentInput(current);
  score = vCompareContoursFast(binary1, binary2, parameter);
  vPrint("Score = %lf\n", score);

  return 1;
}


long vInterpreter1::CompareFrames(long frame1, long frame2)
{
  ushort result = 0;
  long score = 0;

  general_image * base1 = CurrentImage(frame1);
  general_image * base2 = CurrentImage(frame2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * binary1 = vNormalizedBinary(base1, 96.0, 256, 256);
  grayscale_image * binary2 = vNormalizedBinary(base2, 96.0, 256, 256);
  score = vMatchBinaryImages(binary1, binary2);
  vPrint("Score = %li\n", score);
  vdelete(binary1);
  vdelete(binary2);
  return 1;
}


long vInterpreter1::CompareFrames(const char * filename1, long frame1,
                                    const char * filename2, long frame2)
{
  ushort result = 0;
  long score = 0;
  vVideoFile * sequence1 = 0;
  vVideoFile * sequence2 = 0;
  grayscale_image * gray1, * gray2, * thresholded, * normalized, * cropped;
  gray1 = gray2 = thresholded = normalized = cropped = 0;

  sequence1 = vOpenReadVideoFile(filename1);
  if (sequence1 == 0) goto exit;
  if (!sequence1->IsValid()) goto exit;
  sequence2 = vOpenReadVideoFile(filename2);
  if (sequence2 == 0) goto exit;
  if (!sequence2->IsValid()) goto exit;

  gray1 = sequence1->ReadFrameGray(frame1);
  if (gray1 == 0) goto exit;
  gray2 = sequence2->ReadFrameGray(frame2);
  if (gray2 == 0) goto exit;

  thresholded = (grayscale_image *) gray1->threshold(1);
  normalized = (grayscale_image *) vNormalizeImage(thresholded, 96.0, 127.5, 127.5);
  cropped = (grayscale_image *) vCopyRectangle(normalized, 0, 255, 0, 255);
  vdelete(thresholded);
  thresholded = (grayscale_image *) normalized->threshold(128);

  score = vMatchBinaryImages(thresholded, gray2);
  cout << "matching score = " << score << endl << endl;
  result = 1;

exit:
  vdelete(sequence1);
  vdelete(sequence2);
  vdelete(gray1);
  vdelete(gray2);
  vdelete(thresholded);
  vdelete(normalized);
  vdelete(cropped);
  return result;
}


long vInterpreter1::MatchingScores(const char * test_filename, 
                                          long start, long end,
                                          const char * training_filename, 
                                          const char * results_base_name)
{
  ushort result = 0;
  long score = 0;
  vVideoFile * test_sequence = 0;
  vVideoFile * training_sequence = 0;
  grayscale_image * test_gray, * training_gray, * thresholded;
  grayscale_image * normalized, * cropped, * temp_gray, * padded;
  test_gray = training_gray = thresholded = padded = 0;
  normalized = cropped = temp_gray = 0;
  long test_frame, training_frame;
  test_frame = training_frame = 0;
  long number_of_training_frames;
  number_of_training_frames = 0;
  char * current_results = 0;
  long results_name_length = strlen(results_base_name);
  long orientation = 0;
  FILE * results_fp = 0;

  test_sequence = vOpenReadVideoFile(test_filename);
  if (test_sequence == 0) 
  {
    vPrint("Can't open test sequence %s\n", test_filename);
    goto exit;
  }
  if (!test_sequence->IsValid()) 
  {
    vPrint("Can't open test sequence %s\n", test_filename);
    goto exit;
  }
  training_sequence = vOpenReadVideoFile(training_filename);
  if (training_sequence == 0)
  {
    vPrint("Can't open training sequence %s\n", training_filename);
    goto exit;
  }
  if (!training_sequence->IsValid()) goto exit;
  number_of_training_frames = training_sequence->NumberOfFrames();
  vPrint("%15s %15s\n", "test_frame", "training_frame");

  for (test_frame = start; test_frame <= end; test_frame++)
  {
    // Prepare test image before matching
    test_gray = test_sequence->ReadFrameGray(test_frame);
    if (test_gray == 0) goto exit;

    long rows = test_gray->Rows();
    long cols = test_gray->Cols();
    if ((rows < 256) || (cols < 256))
    {
      padded = (grayscale_image *) vPad(test_gray, 256, 256);
      vdelete(test_gray);
      test_gray = 0;
    }
    else
    {
      padded = test_gray;
      test_gray = 0;
    }

    thresholded = (grayscale_image *) padded->threshold(1);
    normalized = (grayscale_image *) vNormalizeImage(thresholded, 96.0, 127.5, 127.5);
    cropped = (grayscale_image *) vCopyRectangle(normalized, 0, 255, 0, 255);
    vdelete(thresholded);
    thresholded = (grayscale_image *) normalized->threshold(128);
    vdelete(normalized);
    vdelete(cropped);
    vdelete(padded);
    normalized = cropped = test_gray = 0;

    current_results = new char[results_name_length + 10];
    sprintf(current_results, "%s.%li.txt", results_base_name, test_frame);
    results_fp = fopen(current_results, vFOPEN_WRITE);
    if (results_fp == 0)
    {
      vPrint("Error, can't open results file %s\n", current_results);
      goto exit;
    }
    vdelete2(current_results);
    current_results = 0;

    // Do the matching
    for (training_frame = 0; training_frame < number_of_training_frames; 
         training_frame++)
    {
      training_gray = training_sequence->ReadFrameGray(training_frame);
      if (training_gray == 0) goto exit;
      
      for (long orientation = 0; orientation < 4; orientation++)
      {
        score = vMatchBinaryImages(thresholded, training_gray);
        long items = fprintf(results_fp, "%li %li %li\r\n", 
                             training_frame, orientation, score);
        if (items < 0)
        {
          vPrint("Error in writing to results file, items = %li\n", items);
        }
        vPrint("%15li %15li\r", test_frame, training_frame);
        if (orientation < 3)
        {
          temp_gray = (grayscale_image *) vRotateRight(training_gray);
          vdelete(training_gray);
          training_gray = temp_gray;
        }
        else
        {
          vdelete(training_gray);
        }
      }
    }

    fclose(results_fp);
    results_fp = 0;
    vdelete(thresholded);
    thresholded = training_gray = 0;
  }
  result = 1;
  vPrint("\n");

exit:
  if (results_fp != 0) fclose(results_fp);
  vdelete2(current_results);
  vCloseVideoFile(test_sequence);
  vCloseVideoFile(training_sequence);
  vdelete(test_gray);
  vdelete(padded);
  vdelete(training_gray);
  vdelete(thresholded);
  vdelete(normalized);
  vdelete(cropped);
  return result;
}


long vInterpreter1::MakeSequence(const char * sequence_name, 
                                        const char * pattern)
{
  vector<voidp> filenames;
  ushort success = vFilesMatching(pattern, &filenames);
  if (success == 0) return success;
  if (filenames.size() == 0) return 0;

  char * filename = (char *) filenames[0];
  general_image * base = general_image::Read(filename);
  if (base == 0) return 0;
  color_image * the_image = base->ToRGB();
  if (the_image != base) vdelete(base);
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  vVideoFile * video_file = vOpenWriteVideoFile(sequence_name, 
                                                      rows, cols);
  vdelete(the_image);
  if (video_file == 0)
  {
    vEraseCharpVector(&filenames);
    return 0;
  }

  long frame;
  long number_of_frames = filenames.size();

  for (frame = 0; frame < number_of_frames; frame++)
  {
    char * filename = (char *) filenames[frame];
    general_image * base = general_image::Read(filename);
    if (base == 0)
    {
      vPrint("Couldn't open image file %s\n", filename);
      continue;
    }
    color_image * the_image = base->ToRGB();
    if (the_image != base) vdelete(base);

    video_file->write_frame(the_image);
    vPrint("Processed %s (%li/%li)        \r", 
            filename, frame+1, number_of_frames);
    delete the_image;
  }
  vPrint("\n");

  video_file->FinishWriting();
  vCloseVideoFile(video_file);
  vEraseCharpVector(&filenames);
  return 1;
}


long vInterpreter1::SkinSegmentSequence0a()
{
  char input[400];
  char output[400];
  vPrint("Enter input and output for hand detection: ");
  vScan("%s %s", input, output);
  return SkinSegmentSequence7(input, output, -1, -1, -1, -1, -1);
}


long vInterpreter1::SkinSegmentSequence0b()
{
  char input[400];
  char output[400];
  long start, end;
  start = end = 0;
  vPrint("Enter input and output for hand detection, start and end: ");
  vScan("%s %s %li %li", input, output, &start, &end);
  return SkinSegmentSequence7(input, output, start, end, -1, -1, -1);
}


long vInterpreter1::SkinSegmentSequence0c()
{
  char input[400];
  char output[400];
  long start, end, threshold;
  start = end = threshold = 0;
  vPrint("Enter input file, output_file, start, end, threshold: ");
  vScan("%s %s %li %li %li", input, output, &start, &end, &threshold);
  return SkinSegmentSequence7(input, output, start, end, threshold, -1, -1);
}


long vInterpreter1::SkinSegmentSequence0d()
{
  char input[400];
  char output[400];
  long start, end, threshold, row, col;
  start = end = threshold = row = col = 0;
  vPrint("Enter input file, output_file, start, end, threshold, row, col: ");
  vScan("%s %s %li %li %li %li %li", input, output, &start, &end, &threshold, &row, &col);
  return SkinSegmentSequence7(input, output, start, end, threshold, row, col);
}


long vInterpreter1::SkinSegmentSequence2(const char * input_file, 
                                            const char * output_file)
{
  return SkinSegmentSequence7(input_file, output_file, -1, -1, -1, -1, -1);
}


long vInterpreter1::SkinSegmentSequence4(const char * input_file, 
                                            const char * output_file,
                                            long start, long end)
{
  return SkinSegmentSequence7(input_file, output_file, start, end, 
                              -1, -1, -1);
}


long vInterpreter1::SkinSegmentSequence5(const char * input_file, 
                                            const char * output_file,
                                            long start, long end, 
                                            long threshold)
{
  return SkinSegmentSequence7(input_file, output_file, start, end, 
                              threshold, -1, -1);
}


long vInterpreter1::SkinSegmentSequence7(const char * input_file, 
                                            const char * output_file,
                                            long start, long end, 
                                            long threshold, long row, long col)
{
  vVideoFile * input_video = vOpenReadVideoFile(input_file);
  if (input_video == 0) return 0;
  if (input_video->IsValid() == 0) 
  {
    vCloseVideoFile(input_video);
    return 0;
  }

  long rows = input_video->Rows();
  long cols = input_video->Cols();
  vVideoFile * output_video= vOpenWriteVideoFile(output_file, rows, cols);
  if ((output_video== 0) || (!output_video->IsValid()))
  {
    vCloseVideoFile(input_video);
    vCloseVideoFile(output_video);
    return 0;
  }

  long number_of_frames = input_video->NumberOfFrames();

  // Reset start and end to default values, if they are -1.
  if (start == -1) start = 0;
  if (end == -1) end = number_of_frames - 1;

  long frame_counter;
  for (frame_counter = start; frame_counter <= end; frame_counter++)
  {
    color_image * source_image = input_video->ReadFrame(frame_counter);
    if (source_image == 0) 
    {
      vPrint("\nFailed to read frame %li\n", frame_counter);
      break;
    }
    color_image * target_image = SkinSegmentHand4(source_image, threshold, row, col);
    output_video->write_frame(target_image);
    //vdelete(source_image);
    //vdelete(target_image);
    vPrint("Frame %li. Processed frames %li of %li\r", 
            frame_counter, frame_counter - start + 1, end - start + 1);
  }
  vPrint("\n");
  vCloseVideoFile(input_video);
  output_video->FinishWriting();
  vCloseVideoFile(output_video);

  return 1;
}


long vInterpreter1::ConnectedComponents()
{
  if (input->current_image == 0) return 0;
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  color_image * the_image = vLabelsToImage(connected, &labels);
  vInterpreter1::ShowImage(connected);
  vInterpreter1::ShowImage(the_image);

  return 1;
}


long vInterpreter1::BiggestComponent()
{
  if (input->current_image == 0) return 0;
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  vInterpreter1::ShowImage(connected);
  std::sort(labels.begin(), labels.end(), LabelMore());
  Label biggest = labels[0];
  if ((biggest.id == 0) && (labels.size() > 1))
  {
    biggest = labels[1];
  }
  v3dMatrix<uchar> * selected = vSelectValue(connected, biggest.id);
  ShowImage(selected);

  return 1;
}


long vInterpreter1::SelectComponent()
{
  if (input->current_image == 0) return 0;
  long component_rank = 0;
  vPrint("\nEnter component rank:\n");
  vScan("%li", &component_rank);
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  vInterpreter1::ShowImage(connected);

  std::sort(labels.begin(), labels.end(), LabelMore());
  long i;
  long rank = 0;
  long selected_id = -1;
  for (i = 0; i < (long) labels.size(); i++)
  {
    Label current_label = labels[i];
    if (current_label.id == 0) continue;
    if (rank == component_rank)
    {
      selected_id = current_label.id;
      break;
    }
    rank++;
  }

  if (selected_id == -1)
  {
    return 0;
  }

  v3dMatrix<uchar> * selected = vSelectValue(connected, selected_id);
  ShowImage(selected);

  return 1;
}


long vInterpreter1::SelectComponents()
{
  if (input->current_image == 0) return 0;
  long component_rank1 = 0, component_rank2 = 0;
  vPrint("\nEnter component rank low and high:\n");
  vScan("%li %li", &component_rank1, &component_rank2);
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = input->current_image->ConnectedComponents(&labels);
  vInterpreter1::ShowImage(connected);
  if ((component_rank1 < 0) || (component_rank2 < 0) ||
      (component_rank1 >= (long) labels.size()) || (component_rank2 >= (long) labels.size()))
  {
    print("only %li labels found\n", labels.size());
    return 1;
  }

  long size = labels.size();
  vector<Label> copied_labels(size);
  long i;
  for (i = 0; i < size; i++)
  {
    copied_labels[i] = labels[i];
  }

  std::sort(labels.begin(), labels.end(), LabelMore());
  long lowest = labels[component_rank2].count;
  long highest = labels[component_rank1].count;

  long rows = connected->Rows();
  long cols = connected->Cols();
  grayscale_image * selected =  new grayscale_image(rows, cols);
  for (i = 0; i < rows * cols; i++)
  {
    long label = connected->Matrix()[i];
    long count = copied_labels[label].count;
    if ((lowest <= count) && (count <= highest))
    {
      selected->Matrix()[i] = 255;
    }
    else
    {
      selected->Matrix()[i] = 0;
    }
  }

  ShowImage(selected);

  return 1;
}


long vInterpreter1::DetectFaces0()
{
  FaceLocation * locs;
  long result = DetectFaces(&locs);
  free(locs);
  return result;
}


long vInterpreter1::DetectFaces(FaceLocation ** locs)
{
  if (input->current_image == 0) 
  {
    *locs = 0;
    return 0;
  }

  color_image * result_image;
  color_image * current = input->current_image->ToRGB();
  if (current != input->current_image)
  {
    result_image = current;
  }
  else
  {
    result_image = (color_image *) vCopyImage(current);
  }

  cout << "Looking for faces..." << endl;
  #ifndef VASSILIS_LINUX_PLATFORM
  long result = vFindFaces(result_image, locs);
	char buffer[100];
	sprintf(buffer, "result = %i", result);
  cout << buffer << endl;

  long i;
  for (i = 0; i < result; i++)
  {
    vDrawRectangle1(result_image, (*locs)[i].y1, (*locs)[i].x1,
									   (long) (*locs)[i].y2, (*locs)[i].x2, (uchar) 255);
  }
  
  vInterpreter1::ShowImage(result_image);
  return result;
  #endif
  return 1;
}

long vInterpreter1::PrintImage(color_image * the_image)
{
  vPrint("rows = %li, cols = %li\n", the_image->Rows(), the_image->Cols());
  fflush(stdout);

  return 1;
}


// Here we can have overlapping images. The strategy is as follows: 
// - Take the rest of the inputs in order, and display their current
//   images, as long as no input preceding in order has the same 
//   display_top and display_left.
// - Draw the current image of the current input
long vInterpreter1::RedrawImages()
{
  if (input == 0) return 0;
  long current_top, current_left;
  current_top = input->display_top;
  current_left = input->display_left;
  long i, j;
  for (i = 0; i < number_of_inputs; i++)
  {
    if (i == current_input_index) continue;
    class_sequence_information * info1 = (class_sequence_information *) inputs[i];
    if ((info1->display_left == current_left) &&
        (info1->display_top == current_top))
    {
      continue;
    }
    ushort same_coords_flag = 0;
    for (j = 0; j < i; j++)
    {
      class_sequence_information * info2 = (class_sequence_information *) inputs[j];
      if ((info1->display_top == info2->display_top) &&
          (info1->display_left == info2->display_left))
      {
        same_coords_flag = 1;
        break;
      }
    }
    if (same_coords_flag != 0) continue;
    DrawInput(i);
  }
  DrawCurrentImage();

  return 1;
}


long vInterpreter1::DrawCurrentImage()
{
  if (input == 0) return 0;
  if (window == 0) return 0;
  general_image * the_image = input->current_image;
  if (the_image == 0) return 0;
  long top = input->display_top;
  long left = input->display_left;
  long sampling = 1;

  general_image * image_to_show;
  if ((histogram_equalization_flag) && (the_image->TypeName() == UshortType))
  {
    image_to_show = function_histogram_equalization((ushort_image *) the_image, equalization_window_size);
  }
  else
  {
    image_to_show = the_image;
  }

  if (input->resolution <= 0)
  {
    window->ShowImage(image_to_show, top, left, sampling);
  }

  else // We magnify
  {
    long factor = (long) pow(2, input->resolution);
//    general_image * scaled = vMagnify0(color_image, input->center_row, 
//                                      input->center_col, input->magnified_rows,
//                                      input->magnified_cols, factor);
    general_image * scaled = vMagnify0(image_to_show, input->center_row, 
                                      input->center_col, the_image->Rows() * factor,
                                      the_image->Cols() * factor, factor);

    window->ShowImage(scaled, top, left, sampling);
    vdelete(scaled);
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (input->automatic_centering == 1)
  {
    input->center_row = rows / 2;
    input->center_col = cols / 2;
  }
  if (image_to_show != the_image)
  {
    vdelete(image_to_show);
  }

  return 1;
}


long vInterpreter1::RedrawWindow()
{
  window->Clear();
  window->RedrawSubwindows();
//  RedrawImages();

  return 1;
}


long vInterpreter1::SetBounds()
{
  if (input == 0) return 0;
  long top, left, rows, cols;
  top = left = rows = cols = 0;
  vPrint("Type in top, left, rows, columns: ");
  vScan("%li %li %li %li\n", &top, &left, &rows, &cols);
  vInterpreter1::SetBounds(top, left, rows, cols);

  return 1;
}


long vInterpreter1::ResetBounds()
{
  if (input == 0) return 0;
  vVideoFile * video_file = input->sequence;
  SetBounds(0, 0, video_file->Rows(), video_file->Cols());

  return 1;
}

long vInterpreter1::SetBounds(long top, long left, 
		                               long rows, long cols)
{
  if (input == 0) return 0;
  vVideoFile * file = input->sequence;
  if (file == 0) return 0;
  file->SetBounds(top, left, rows, cols);
  vInterpreter1::ShowCurrentFrame();

  return 1;
}


// We need to normalize them based on the magnification
// factor (2 to the power of resolution), the center_row and
// center_col of the input, and the top and left of the location
// where the input is displayed
long vInterpreter1::NormalizedCoordinates(long in_row, long in_col,
                                           long * out_row, long * out_col)
{
  if (input == 0) return 0;

  long subwindow_row = in_row - input->display_top;
  long subwindow_col = in_col - input->display_left;
  long factor = (long) pow(2, input->resolution);
  // If the resolution is less than 1, we don't do any scale normalization
  if (factor == 0) factor = 1;

//  long input_rows = input->magnified_rows / factor;
//  long input_cols = input->magnified_cols / factor;

  long input_top, input_left;
  if (input->resolution > 0)
  {
//    input_top = input->center_row - input_rows / 2;  
//    input_left = input->center_col - input_cols / 2;
    input_top = input->display_top;  
    input_left = input->display_left;
  }
  else
  {
    input_top = 0;
    input_left = 0;
  }

  *out_row = subwindow_row / factor;
  *out_col = subwindow_col / factor;
  //vPrint("(%li, %li) ==> (%li, %li)\n", in_row, in_col, *out_row, *out_col);

  return 1;
}


long vInterpreter1::PrintNormalizedCoordinates0()
{
  if (input == 0) return 0;
  long in_row, in_col;
  in_row = in_col = 0;
  vPrint("\nEnter in_row, in_col:\n");
  vScan("%li %li", &in_row, &in_col);
  return PrintNormalizedCoordinates(in_row, in_col);

}


long vInterpreter1::PrintNormalizedCoordinates(long in_row, long in_col)
{
  if (input == 0) return 0;
  long out_row, out_col;
  out_row = out_col = 0;
  NormalizedCoordinates(in_row, in_col, &out_row, &out_col);
  vPrint("input->top = (%li, %li)\n", 
          input->display_top, input->display_left);
  vPrint("input->center = (%li, %li)\n", 
          input->center_row, input->center_col);
//  vPrint("input display size = %li x %li\n", 
//          input->magnified_rows, input->magnified_cols);
  vPrint("(%li, %li) ==> (%li, %li)\n", in_row, in_col, out_row, out_col);
  return 1;
}

  
long vInterpreter1::ShowPixelInfo(long row, long col)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (pixel_info_window == 0) return 0;

  long width = input->current_image->Cols();
  long height = input->current_image->Rows();

  row = Max(row, (long) 0);
  row = Min(row, (long) (height-1));
  col = Max(col, (long) 0);
  col = Min(col, (long) (width-1));
  long bands = input->current_image->Bands();

  char buffer[1000];

  Type type_name = input->current_image->TypeName();
  Type super_type = vSuperType(type_name);
  if (super_type == LongType)
  {
    vector<long> values;
    input->current_image->LongValues(values, row, col);
    long intensity = 0;
    long i;
    for (i = 0; i < bands; i++)
      intensity += values[i];
    sprintf(buffer, "(%li, %li)  (", row, col);
    for (i = 0; i < bands; i++)
    {
      sprintf(buffer, "%s%li", buffer, values[i]);
      if (i < bands - 1) sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)  %li  (", buffer, intensity);

    double divisor;
    if (intensity == 0) divisor = 1000000;
    else divisor = ((double) intensity) / 255.0;
    for (i = 0; i < bands; i++)
    {
      double value = values[i];
      value = value / divisor;
      sprintf(buffer, "%s%li", buffer, (long) value);
      if (i < bands - 1) sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)                                   ", buffer);
  }
  else
  {
    assert(super_type == DoubleType);
    vector<double> values;
    input->current_image->DoubleValues(values, row, col);
    double intensity = 0;
    long i;
    for (i = 0; i < bands; i++)
      intensity += values[i];
    sprintf(buffer, "(%li, %li)  (", row, col);
    for (i = 0; i < bands; i++)
    {
      sprintf(buffer, "%s%lf", buffer, values[i]);
      if (i < bands - 1)
	sprintf(buffer, "%s, ", buffer);
    }
    sprintf(buffer, "%s)  %lf                                   ", 
	    buffer, intensity);
  }
  pixel_info_window->DrawString(3, 13, buffer);

  return 1;
}

long vInterpreter1::ShowFrameInfo()
{
  if (pixel_info_window == 0) return 0;
  if (current_input_index== -1) return 0;
  char buffer[200];
  sprintf(buffer, "%s - Frame %li of %li                        ", 
          input->simple_name,
          input->CurrentFrame(),
          input->number_of_frames);

  frame_info_window->DrawString(3, 13, buffer);

  return 1;
}


long vInterpreter1::PrintPixelInfo()
{
  if (input == 0) return 0;
  vPrint("Enter row and col: ");
  long row = 0;
  long col = 0;
  vScan("%li %li", &row, &col);
  return PrintPixelInfo(row, col);
}


long vInterpreter1::PrintPixelInfo(long row, long col)
{
  if (input == 0) return 0;
  vector<double> values;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  if (row >= rows) return 0;
  if (row < 0) return 0;
  if (col >= cols) return 0;
  if (col < 0) return 0;

  current->DoubleValues(values, row, col);
  long i;
  for (i = 0; i < (long) values.size(); i++)
  {
    vPrint("%lf ", values[i]);
  }
  vPrint("\n");
  return 1;
}


long vInterpreter1::entry_value()
{
  if (input == 0) return 0;
  vPrint("Enter row, col, and band: ");
  long row = 0;
  long col = 0;
  long channel = 0;
  vScan("%li %li %li", &row, &col, & channel);

  vector<double> values;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  if (row >= rows) return 0;
  if (row < 0) return 0;
  if (col >= cols) return 0;
  if (col < 0) return 0;
  if (channel < 0) return 0;
  if (channel >= current->channels()) return 0;


  current->DoubleValues(values, row, col);
  vPrint("%lf\n", values[channel]);
  vPrint("\n");
  return 1;
}


long vInterpreter1::FrameDifferences()
{
  long num_frames = input->sequence_step;
  if (!predicate_color_image(input->current_image)) return 0;
  color_image * the_image1 = (color_image *) input->current_image;
  vInterpreter1::AdvanceSequence(num_frames);
  color_image * the_image2 = (color_image *) input->current_image;
  v3dMatrix<double> * difference = vImageDifference(the_image1, the_image2);
  ShowImage(difference);

  return 1;
}


long vInterpreter1::SynchronizeSequences()
{
  if (input == 0) return 0;
  long old_current_input_index = current_input_index;
  long current_frame = input->CurrentFrame();
  long i;
  for (i = 0; i < number_of_inputs; i++)
  {
    if (i == old_current_input_index) continue;
    SetCurrentInputSilent(i);
    input->SetCurrentFrame(current_frame);
    StoreCurrentFrame();
  }
  SetCurrentInput(old_current_input_index);

  return 1;
}


// Makes the current image all black
long vInterpreter1::ClearImage()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * the_image = (color_image *) current->ToRGBCopy();
  long all_band_size = the_image->AllBandSize();
  long size_of_type = the_image->SizeOfType();
  long byte_size = all_band_size * size_of_type;
  vArray(uchar) matrix = the_image->Matrix();
  bzero(vData(matrix), byte_size);
  ShowImage(the_image);
  return 1;
}


long vInterpreter1::SetPixel()
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  vPrint("Enter row, col and value: ");
  long row = 0;
  long col = 0;
  long value = 0;
  vScan("%li %li %li", &row, &col, &value);
  uchar uchar_value = (uchar) (value % 256);
  return SetPixel(row, col, uchar_value);
}


long vInterpreter1::SetPixel(long row, long col, uchar value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  if (current->TypeName() != UcharType) return 0;
  v3dMatrix<uchar> * the_image = (v3dMatrix<uchar> *) current;
  long bands = the_image->Bands();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (row < 0) return 0;
  if (col < 0) return 0;
  if (row >= rows) return 0;
  if (col >= cols) return 0;
  
  vArray3(uchar) matrix = the_image->Matrix3();
  long i;
  for (i = 0; i < bands; i++)
  {
    matrix[i][row][col] = value;
  }
  DrawCurrentImage();
  return 1;
}



v3dMatrix<double> * vInterpreter1::ComputeMotion()
{
  if (input->current_image == 0) return 0;
  long num_frames = input->sequence_step;
  general_image * the_image1 = input->current_image;

  input->SetCurrentFrame(input->CurrentFrame() - num_frames);
  general_image * the_image0 = ReadCurrentImage();
  if (the_image0 == 0)
  {
    input->SetCurrentFrame(input->CurrentFrame() + num_frames);
    return 0;
  }

  input->SetCurrentFrame(input->CurrentFrame() + 2 * num_frames);
  general_image * the_image2 = ReadCurrentImage();
  v3dMatrix<double> * difference = 0;
  if (the_image2 != 0)
  {
    difference = vComputeMotion0(the_image0, the_image1, the_image2);
    ShowImage(difference);
  }
  input->SetCurrentFrame(input->CurrentFrame()- num_frames);

  vdelete(the_image0);
  vdelete(the_image2);
  return difference;
}


v3dMatrix<double> * vInterpreter1::ComputeMotion(long win_rows, long win_cols)
{
  if (input->current_image == 0) return 0;
  long num_frames = input->sequence_step;
  if (!predicate_color_image(input->current_image)) return 0;
  general_image * the_image1 = (color_image *) input->current_image;

  input->SetCurrentFrame(input->CurrentFrame() - num_frames);
  general_image * the_image0 = ReadCurrentImage();
  if (the_image0 == 0)
  {
    input->SetCurrentFrame(input->CurrentFrame() + num_frames);
    return 0;
  }

  input->SetCurrentFrame(input->CurrentFrame() + 2 * num_frames);
  general_image * the_image2 = ReadCurrentImage();
  v3dMatrix<double> * difference = 0;
  if (the_image2 != 0)
  {
    general_image * small0, * small1, * small2;
    small0 = (general_image *) the_image0->AverageRegions(win_rows, win_cols);
    small1 = (general_image *) the_image1->AverageRegions(win_rows, win_cols);
    small2 = (general_image *) the_image2->AverageRegions(win_rows, win_cols);
    difference = vComputeMotion0(small0, small1, small2);
    vdelete(small0);
    vdelete(small1);
    vdelete(small2);
  }
  input->SetCurrentFrame(input->CurrentFrame() - num_frames);

  vdelete(the_image0);
  vdelete(the_image2);
  return difference;
}


long vInterpreter1::ThresholdCurrentImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  
  double threshold = 0;
  vPrint("Threshold: ");
  vScan("%lf", &threshold);

  grayscale_image * threshold_color_image;
  threshold_color_image = (grayscale_image*) input->current_image->threshold(threshold);
  vInterpreter1::ShowImage(threshold_color_image);
  vector<long> rows, cols;
  vGetNonZeroPoints(threshold_color_image, &rows, &cols);
  long number = rows.size();
  vPrint("number = %li, percentage = %f%%\n", number, 
         ((float) number) / ((float) threshold_color_image->Size()) * 100.0f);

  return 1;
}


long vInterpreter1::ThresholdCurrentImage(double threshold)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  
  v3dMatrix<uchar> * threshold_color_image;
  threshold_color_image = input->current_image->threshold(threshold);
  vInterpreter1::ShowImage(threshold_color_image);

  return 1;
}


long vInterpreter1::InverseThresholdImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  double threshold = 0;
  vPrint("Threshold: ");
  vScan("%lf", &threshold);

  v3dMatrix<uchar> * threshold_color_image;
  threshold_color_image = input->current_image->threshold(threshold);
  vInverseImage(threshold_color_image);
  vInterpreter1::ShowImage(threshold_color_image);

  return 1;
}


long vInterpreter1::ShowPreviousImage()
{
  if (input == 0) return 0;
  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + buffer_size - 1) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + 1) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
  }
  else 
  {
    DrawCurrentImage();
  }

  return 1;
}


long vInterpreter1::ShowNextImage()
{
  long buffer_size = input->buffer_size;
  input->buffer_index = (input->buffer_index + 1) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
    input->buffer_index = (input->buffer_index + buffer_size - 1) % buffer_size;
    input->current_image = input->image_buffer[input->buffer_index];
  }
  else 
  {
    DrawCurrentImage();
  }

  return 1;
}


//Subtract from or add a given number to the image buffer index.
long vInterpreter1::RewindImageBuffer0()
{
  long frames_to_skip = 0;
  vPrint("Input number of frames to skip backwards: ");
  vScan("%li", &frames_to_skip);
  return RewindImageBuffer(frames_to_skip);
}


long vInterpreter1::ForwardImageBuffer0()
{
  long frames_to_skip = 0;
  vPrint("Input number of frames to skip forward: ");
  vScan("%li", &frames_to_skip);
  return ForwardImageBuffer(frames_to_skip);
}

long vInterpreter1::RewindImageBuffer(long frames_to_skip)
{
  if (input == 0) return 0;
  if (frames_to_skip < 0) return 0;
  long buffer_size = input->buffer_size;
  if (frames_to_skip >= buffer_size) return 0;

  input->buffer_index = (input->buffer_index + buffer_size - frames_to_skip) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + frames_to_skip) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
      return 0;
  }
  else 
  {
    DrawCurrentImage();
    return 1;
  }
}


long vInterpreter1::ForwardImageBuffer(long frames_to_skip)
{
  if (input == 0) return 0;
  if (frames_to_skip < 0) return 0;
  long buffer_size = input->buffer_size;
  if (frames_to_skip >= buffer_size) return 0;

  input->buffer_index = (input->buffer_index + frames_to_skip) % buffer_size;
  input->current_image = input->image_buffer[input->buffer_index];
  if (input->current_image == 0)
  {
      input->buffer_index = (input->buffer_index + buffer_size - frames_to_skip) % buffer_size;
      input->current_image = input->image_buffer[input->buffer_index];
      return 0;
  }
  else 
  {
    DrawCurrentImage();
    return 1;
  }
}
 
 
long vInterpreter1::UndirectedEdges()
{
  if (input->current_image == 0) return 0;
  general_image * the_image1 = input->current_image;
  double_image * the_image2 = new double_image(the_image1);
  double_image * result_image =  vUndirectedEdges(the_image2);
  vInterpreter1::ShowImage(result_image);

  vdelete(the_image2);

  return 1;
}


long vInterpreter1::AmplifyImage()
{
  if (input->current_image == 0) return 0;
  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();
  color_image * result_image = (color_image *) vAmplifyImage(the_image2);
  vInterpreter1::ShowImage(result_image);

  if (the_image2 != the_image1) vdelete(the_image2);

  return 1;
}
 

long vInterpreter1::MultiplyImage()
{
  if (input->current_image == 0) return 0;

  double number = 0;
  vPrint("Multiply image by: ");
  vScan("%lf", &number);

  general_image * the_image1 = input->current_image;
  double_image color_image2(input->current_image);
  double_image * result_image = vMultiplyImage(&color_image2, number);
  vInterpreter1::ShowImage(result_image);

  return 1;
}
 
// Commented out on 6/13/2001
//long vInterpreter1::FindSkin2()
//{
//  if(input->current_image == 0) return;
//  vInterpreter1::InitializeSkinData();
//  general_image * the_image1 = input->current_image;
//  color_image * the_image2 = input->current_image->ToRGB();
//  color_image * result_image = vProcessPixels(color_image2, vSkinPredicate2);
//  ShowImage(result_image);
//
// if (color_image2 != color_image1) vdelete(color_image2);
//}

//void SkinPredicate(color_image * the_image, long row, long col,
//		               uchar & r, uchar & g, uchar & b)
//{
//  uchar r1 = color_image->GetR2()[row][col];
//  uchar g1 = color_image->GetG2()[row][col];
//  uchar b1 = color_image->GetB2()[row][col];
//
//  double intensity = r1 + g1 + b1;
//  double r2 = r1 / intensity;
//  double g2 = g1 / intensity;
//  
//  if ((r2 <= 0.55) && (r2 >= 0.40) && (g2 <= 0.35) && (g2 >= 0.25))
//    {
//      r = r1;
//      g = g1;
//      b = b1;
//    }
//
//  else 
//    {
//      r = 0;
//      g = 0;
//      b = 100;
//    }
//}

//void vSkinPredicate2(color_image * the_image, long row, long col,
//		                uchar & r, uchar & g, uchar & b)
//{
// uchar value;
//  uchar r1 = color_image->GetR2()[row][col];
//  uchar g1 = color_image->GetG2()[row][col];
//  uchar b1 = color_image->GetB2()[row][col];
//
//  double intensity = r1 + g1 + b1;
//  double r2 = r1 / intensity;
//  double g2 = g1 / intensity;
//  
//  if ((r2 <= 0.55) && (r2 >= 0.40) && (g2 <= 0.31) && (g2 >= 0.25))
//    value = 255;
//  else value = 0;
//
//  r = value;
//  g = value;
//  b = value;
//}


long vInterpreter1::ProcessPolygon()
{
#ifndef VASSILIS_WINDOWS
  vPrint("Getting polygon...\n");
  vPolygon * polygon = vGetPolygonFromMouse(window);
  if (polygon == 0)
    {
      vPrint("Cancelled\n\n");
      return;
    }

  if (polygon->num_vertices == 0)
    {
      vdelete(polygon);
      vPrint("The polygon had no data\n\n");
      return;
    }

  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();
  color_image * result = (color_image *) vMaskFromPolygon(color_image2, polygon);
  vdelete(polygon);

  if (result == 0)
    {
      vPrint("Polygon gets outside of image region\n");
    }
  else
    {
      window->ClearArea(0, 0, input->current_image->Rows(), input->current_image->Cols());
      ShowImage(result);
    }
  if (color_image1 != color_image2) vdelete(color_image2);

  vPrint("\n");
#endif // VASSILIS_WINDOWS

  return 1;
}


long vInterpreter1::InitializeSkinData()
{
  if (skin_data_initialized != 0) return 0;
  vPrint("Getting skin data...\n");
 
  skin_tables = vnew(vSkinTableBase *, NUMBER_OF_SKIN_TABLES);

  char filename1[500], filename2[500];
  long i;
  long size = 2;
  for (i = 0; i < NUMBER_OF_SKIN_TABLES - 1; i++)
  {
    sprintf(filename1, "%s%stable.%li.pc", 
            skin_directory, platform_directory_slash, size);
    sprintf(filename2, "%s%stable.%li.pc", 
            non_skin_directory, platform_directory_slash, size);
    skin_tables[i] = new vSkinTable<long>(filename1, filename2);
    size = size * 2;
  }

  vSkinTable<double> * crl_skin_table;
  crl_skin_table = new vSkinTable<double>(crl_skin_file,
                                           crl_non_skin_file, 32);
  skin_tables[NUMBER_OF_SKIN_TABLES-1] = crl_skin_table;

  vPrint("Read skin data\n\n");

  skin_data_initialized = 1;
  skin_table = skin_tables[current_table_index];
  vPrint("Done\n\n");

  return 1;
}


long vInterpreter1::PickSkinTables()
{
  do
  {
    vPrint("Select skin tables (0 - %li): ", NUMBER_OF_SKIN_TABLES);
    question("%li", &current_table_index);
  }

  while((current_table_index >= NUMBER_OF_SKIN_TABLES) ||
        (current_table_index < 0));

  if (skin_data_initialized != 0)
  {
    skin_table = skin_tables[current_table_index];
  }

  return 1;
}


long vInterpreter1::DetectSkin()
{
  if(input->current_image == 0) return 0;

  vInterpreter1::InitializeSkinData();
  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();
  grayscale_image * result_image = skin_table->DetectSkin(the_image2);
  ShowImage(result_image);

  if (the_image1 != the_image2) vdelete(the_image2);

  return 1;
}


long vInterpreter1::SaveCurrentImage()
{
	if (input->current_image == 0) return 0;
  char filename[1000];
  print("\nSave current image in (q to abort):\n");
  question("%s", filename);
  if (strcmp(filename, "q") == 0) return 0;
  vAdjustEnding3(filename, filename, ".ppm");

  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();
  the_image2->Save(filename);
  if (the_image1 != the_image2) vdelete(the_image2);
  cout << "Image saved" << endl;
  return 0;
}


long vInterpreter1::ApplyProcedure(vPixelPred proc)
{
  if(input->current_image == 0) return 0;
  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();
  color_image * result_image = vProcessPixels(the_image2, proc);
  ShowImage(result_image);
  
  if (the_image1 != the_image2) vdelete(the_image2);

  return 1;
}


long vInterpreter1::HeightAdjustment()
{
  if (input->current_image == 0) return 0;
  if (!predicate_color_image(input->current_image)) return 0;
  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  color_image * adjusted = new color_image(rows, cols);
  color_image * image1 = (color_image *) input->current_image;

  vInterpreter1::AdvanceSequence(1);
  color_image * current = image1;

  vArray2(uchar) R1 = current->GetR2();
  vArray2(uchar) G1 = current->GetG2();
  vArray2(uchar) B1 = current->GetB2();
  vArray2(uchar) R2 = adjusted->GetR2();
  vArray2(uchar) G2 = adjusted->GetG2();
  vArray2(uchar) B2 = adjusted->GetB2();

  long row, col;
  long height_adjustment;
  print("\nenter adjustment:\n");
  question("%li", &height_adjustment);
  for (row = height_adjustment; row < rows; row++)
  {
    long row2 = (rows + row - height_adjustment) % rows;
    for (col = 0; col < cols; col++)
    {
      R2[row2][col] = R1[row][col];
      G2[row2][col] = G1[row][col];
      B2[row2][col] = B1[row][col];
    }    
  }

  current = (color_image *) input->current_image;
  R1 = current->GetR2();
  G1 = current->GetG2();
  B1 = current->GetB2();

  for (row = 0; row < height_adjustment; row++)
  {
    long row2 = (rows + row - height_adjustment) % rows;
    for (col = 0; col < cols; col++)
    {
      R2[row2][col] = R1[row][col];
      G2[row2][col] = G1[row][col];
      B2[row2][col] = B1[row][col];
    }    
  }

  ShowImage(adjusted);

  return 1;
}


long vInterpreter1::Trash()
{
  // vassilis, for debugging
  unsigned char value;
  unsigned char converted;
  long counter;
  for (counter = 21; counter < 256; counter++)
  {
    value =  (unsigned char) counter;
    converted = (unsigned char) (0.114*value + 0.587*value + 0.299*value);
    printf("%5li, %5li\n", (long) value, (long) converted);
  }

  return 1;
}


long vInterpreter1::Trash2()
{
  color_image * padded;
  grayscale_image * thresholded;
  grayscale_image * normalized, * cropped;

  if (!predicate_color_image(input->current_image)) return 0;
  color_image * current = (color_image *) input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  padded = (color_image *) vPad(current, 256, 256);
  ShowImage(padded);

  thresholded = (grayscale_image *) padded->threshold(1);
  ShowImage(thresholded);
  normalized = (grayscale_image *) vNormalizeImage(thresholded, 96.0, 127.5, 127.5);
  ShowImage(normalized);
  cropped = (grayscale_image *) vCopyRectangle(normalized, 0, 255, 0, 255);
  ShowImage(cropped);

  return 1;
}


long vInterpreter1::Trash3()
{


  return 1;
}


long vInterpreter1::LoadImage()
{
  char filename[256];
  print("\nLoad image file:\n");
  question("%s", filename);
  general_image * the_image = general_image::Read(filename);
  if (the_image == 0) 
  {
    vPrint("Loading image from %s failed\n", filename);
    return 0;
  }
  
  ShowImage(the_image);

  return 1;
}


long vInterpreter1::ChangeDirectory()
{
  char buffer[1000];
  print("\nenter new directory:\n");
  question("%s", buffer);
  return ChangeDirectory(buffer);
}


long vInterpreter1::DrawInput(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  long previous_index = current_input_index;
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  DrawCurrentImage();
  current_input_index = previous_index;
  input = (class_sequence_information *) inputs[current_input_index];
  return 1;
}


long vInterpreter1::SetCurrentInput()
{
  long index = 0;
  vPrint("New current input index: ");
  vScan("%li", &index);
  return SetCurrentInput(index);
}



long vInterpreter1::SetCurrentInput(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  char * parent = vParentDir(input->path_name, platform_directory_slash);
  ChangeDirectory(parent);
  vdelete2(parent);
  DrawCurrentImage();
  ShowFrameInfo();
  return 1;
}


long vInterpreter1::NextInput()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ 1) % number_of_inputs;
  long current_frame = input->current_frame;
  if (preserve_current_frame > 0)
  {
    SetCurrentInputSilent(next_index);
    JumpToFrame(current_frame);
  }
  else
  {
    SetCurrentInput(next_index);
  }

  return 1;
}


long vInterpreter1::PreviousInput()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ number_of_inputs - 1) % number_of_inputs;
  if (preserve_current_frame > 0)
  {
    long current_frame = input->current_frame;
    SetCurrentInputSilent(next_index);
    JumpToFrame(current_frame);
  }
  else
  {
    SetCurrentInput(next_index);
  }


  return 1;
}


long vInterpreter1::SetCurrentInputSilent(long index)
{
  if ((index >= number_of_inputs) || (index < 0))
  {
    vPrint("Invalid index, must be between 0 and %li\n", number_of_inputs - 1);
    return 0;
  }
  current_input_index = index;
  input = (class_sequence_information *) inputs[current_input_index];
  char * parent = vParentDir(input->path_name, platform_directory_slash);
  ChangeDirectory(parent);
  vdelete2(parent);
  return 1;
}


long vInterpreter1::NextInputSilent()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ 1) % number_of_inputs;
  long current_frame = input->current_frame;
  SetCurrentInputSilent(next_index);
  if (preserve_current_frame > 0)
  {
    JumpToFrame(current_frame);
  }

  return 1;
}


long vInterpreter1::PreviousInputSilent()
{
  if (number_of_inputs == 0) return 0;
  long next_index = (current_input_index+ number_of_inputs - 1) % number_of_inputs;
  long current_frame = input->current_frame;
  SetCurrentInputSilent(next_index);
  if (preserve_current_frame > 0)
  {
    JumpToFrame(current_frame);
  }

  return 1;
}


long vInterpreter1::RemoveCurrentInput()
{
  if (current_input_index== -1) return 0;
  
  RemoveInput(current_input_index);
  RedrawWindow();

  return 1;
}


long vInterpreter1::remove_inputs()
{
  long number = number_of_inputs;
  long counter;
  for (counter = 0; counter <number; counter++)
  {
    RemoveCurrentInput();
  }
  
  return 1;
}


long vInterpreter1::MoveSequence()
{
  long top = 0;
  long left = 0;
  vPrint("New top and left of sequence display: ");
  vScan("%li %li", &top, &left);
  return MoveSequence(top, left);
}


long vInterpreter1::MoveSequenceWithClick()
{
  mouse_click_response = MoveSequenceResponse;
  return 1;
}


long vInterpreter1::MoveSequence(long top, long left)
{
  if (current_input_index== -1) return 0;
  input->display_top = top;
  input->display_left = left;
  RedrawWindow();
  return 1;
}

  
long vInterpreter1::Recenter()
{
  if (input == 0) return 0;
  input->automatic_centering = 1;
  general_image * current = input->current_image;
  input->center_row = current->Rows() / 2;
  input->center_col = current->Cols() / 2;
  return 1;
}


long vInterpreter1::Center0()
{
  if (input == 0) return 0;
  long center_row, center_col;
  center_row = center_col = 0;
  vPrint("Enter center row and center col: ");
  vScan("%li %li", &center_row, &center_col);
  return Center(center_row, center_col);
}


// We center the image around center row and center col. This
// makes a difference only for resolutions > 0, i.e. when 
// we magnify an image. In such cases, (center_row, center_col)
// must be converted back to the corresponding resolution 0 
// coordinates, because centering is done with respect to the
// original image.
long vInterpreter1::Center(long center_row, long center_col)
{
  if (input == 0) return 0;
  input->automatic_centering = 0;

  input->center_row = center_row;
  input->center_col = center_col;
  DrawCurrentImage();
  return 1;
}


long vInterpreter1::CenterWithClick()
{
  if (input == 0) return 0;
  previous_mouse_response = mouse_click_response;
  mouse_click_response = CenterResponse;
  return 1;
}

  
long vInterpreter1::SetMagnifiedSize0()
{
  if (input == 0) return 0;
  long rows, cols;
  rows = cols = 0;
  vPrint("Enter rows and cols for magnified display: ");
  vScan("%li %li", &rows, &cols);
  return SetMagnifiedSize(rows, cols);
}


long vInterpreter1::ResetMagnifiedSize()
{
  if (input == 0) return 0;
  long factor = (long) pow(2, input->resolution);
  long rows = input->sequence->Height() * factor;
  long cols = input->sequence->Width() * factor;

  return SetMagnifiedSize(rows, cols);
}


long vInterpreter1::SetMagnifiedSize(long rows, long cols)
{
  if (input == 0) return 0;
  if (rows < 10) return 0;
  if (cols < 10) return 0;
  if (rows > 12000) return 0;
  if (cols > 20000) return 0;
//  input->magnified_rows = rows;
//  input->magnified_cols = cols;
  DrawCurrentImage();
  return 1;
}

  
long vInterpreter1::SetCurrentCounter()
{
  if (input == 0) return 0;
	printf("Current counter is %li. New one >> ", input->CurrentFrame());
  long current_frame = 0;
	scanf("%li", &current_frame);
  input->SetCurrentFrame(current_frame);

  return 0;
}

long vInterpreter1::SaveImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  general_image * the_image1 = input->current_image;
  color_image * the_image2 = input->current_image->ToRGB();

  char filename[200];
  print("\nOutput file name:\n");
  question("%s", filename);
  the_image2->Save(filename);
  if(the_image1 != the_image2) vdelete(the_image2);

  return 1;
}


long vInterpreter1::SavePGM()
{
  if (input->current_image == 0) return 0;
  general_image * the_image1 = input->current_image;
  grayscale_image * the_image2 = input->current_image->ToGray();

  char filename[200];
  print("\nOutput file name:\n");
  question("%s", filename);
  the_image2->Save(filename);
  if(the_image1 != the_image2) vdelete(the_image2);

  return 1;
}


long vInterpreter1::NextInSequence()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  AdvanceSequence(input->sequence_step);

  return 1;
}


long vInterpreter1::AdvanceSequence(long num_frames)
{
  if (current_input_index == -1) return 0;
  if (input->sequence == 0) return 0;
  if (input->number_of_frames == 0) return 0;
  input->SetCurrentFrame((input->CurrentFrame() + num_frames) % input->number_of_frames);
  long success = ShowCurrentFrame();
  if (!success) input->SetCurrentFrame(input->CurrentFrame() - num_frames);
  return success;
}


long vInterpreter1::ShowCurrentFrame()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  general_image * the_image = ReadCurrentImage();

  if (the_image == 0) return 0;
  if (input->resolution >= 0)
  {
    ShowImage(the_image);
  }
  else
  {
    general_image * scaled = vPyramidImage0(the_image, -input->resolution);
    ShowImage(scaled);
    vdelete(the_image);
  }

  ShowFrameInfo();
  return 1;
}


long vInterpreter1::StoreCurrentFrame()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  general_image * the_image = ReadCurrentImage();

  if (the_image == 0) return 0;
  if (input->resolution >= 0)
  {
    store_image(the_image);
  }
  else
  {
    general_image * scaled = vPyramidImage0(the_image, -input->resolution);
    store_image(scaled);
    vdelete(the_image);
  }

  ShowFrameInfo();
  return 1;
}


general_image * vInterpreter1::ReadCurrentImage()
{
  general_image * the_image;
  if (input == 0) return 0;
  vVideoFile * video_file = input->sequence;
  if (video_file == 0) return 0;
  the_image = video_file->ReadImage(input->CurrentFrame());
  if (the_image == 0)
  {
    vPrint("Video file has %li frames, attempt to access frame %li failed.\n",
	         video_file->NumberOfFrames(), input->CurrentFrame());
  }
  return the_image;
}


long vInterpreter1::SetCurrentFrame(long index)
{
  if (input == 0) return 0;
  return input->SetCurrentFrame(index);
}


long vInterpreter1::PreviousInSequence()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  input->SetCurrentFrame(input->CurrentFrame() - input->sequence_step);
  long success = ShowCurrentFrame();
  if (!success) 
  {
    input->SetCurrentFrame(input->CurrentFrame() + input->sequence_step);
  }

  return 1;
}


long vInterpreter1::FindMaximalMotion()
{
  v3dMatrix<double> * difference = ComputeMotion();
  if (difference == 0) return 0;
  v3dMatrix<double> * blurred;
  blurred = (v3dMatrix<double> *) difference->AverageRegions(4, 4);
  ShowImage(blurred);
  color_image * max_spots = vFindMaxSpots(blurred, 30);
  ShowImage(max_spots);
  color_image * resampled_maxes;
  resampled_maxes = (color_image*) max_spots->Resample(difference->Rows(),
						   difference->Cols());
  ShowImage(resampled_maxes);

  return 1;
}


long vInterpreter1::BackgroundStatistics()
{
  if (input->current_image == 0) return 0;
  if(input == 0) return 0;
  vSequenceStatistics<uchar> stats(input->current_image->Rows(), 
				                            input->current_image->Cols(), 3);
  long num_frames = 0;
  vPrint("\nType in the number of frames to process:\n");
  vScan("%li", &num_frames);
  long start = input->CurrentFrame();
  long finish = input->CurrentFrame() + num_frames - 1;

  long current_frame;
  vPrint("\n");
  for (current_frame = start; 
       current_frame <= finish; 
       current_frame++)
  {
    long success = SetCurrentFrame(current_frame);
    if (success == 0) break;
    general_image * current = vInterpreter1::ReadCurrentImage();
    stats.ProcessValues((color_image *) current);
    vPrint("%li\r", input->CurrentFrame());
    vdelete(current);
  }

  vPrint("\n");
  v3dMatrix<double> * means = stats.GetMeans();
  ShowImage(vCopyImage(means));

  for (current_frame = start; 
       current_frame <= finish; 
       current_frame++)
  {
    long success = SetCurrentFrame(current_frame);
    if (success == 0) break;
    general_image * current = vInterpreter1::ReadCurrentImage();
    stats.ProcessDeviations((color_image *) current);
    vPrint("%li\r", input->CurrentFrame());
    vdelete(current);
  }
  vPrint("\n");
  v3dMatrix<double> * deviations = stats.GetDeviations();
  ShowImage(vCopyImage(deviations));
  vdelete(background_model);
  background_model = new vBackgroundModel(&stats);

  return 1;
}


long vInterpreter1::Blur()
{
  if (input == 0) return 0;
	if (input->current_image == 0) return 0;

  general_image * the_image1 = input->current_image;
  v3dMatrix<double> * the_image2 = new v3dMatrix<double> (the_image1);
  v3dMatrix<double> * blurred = vBlur(the_image2);
  double average = function_image_average (blurred);
//  enter_border_value (blurred, average);
  ShowImage(blurred);

  if (the_image1 != the_image2) vdelete(the_image2);

  return 1;
}

// The general strategy to find hands is like this:
// - Find the face (as a rectangle, the way the face detector does)
// - Find skin in the face
// - Look for pixels elsewhere in the image that have color similar to 
// the skin of the face
long vInterpreter1::FindHands()
{
  // Do some error checking
  if (input == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }
  
  if (input->current_image == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }

  general_image * the_image1 = input->current_image;

  // Check if we know the color of the face. If we don't, try to find a face
  // and calculate its color. If no face is found then we fail.
  if (input->face_color == 0)
  {
    long success = SetFaceColor0();
    if (success == 0)
    {
      vPrint("Couldn't find face, can't proceed with finding hands\n");
      return 0;
    }
  }
    
  color_image * current_image = the_image1->ToRGB();

  // Find regions in the image where color is similar to face skin color.
  v3dMatrix<long> * color_distances;
  color_distances = vGridDistances(current_image, input->face_color);

  ShowImage(color_distances);
  grayscale_image * binary = (grayscale_image *) color_distances->threshold((double) hand_threshold);
  ShowImage(binary);

  if (current_image != the_image1) vdelete(current_image);
  return 1;
}


long vInterpreter1::BrowseImage0()
{
  vPrint("Enter image number to go to: ");
  long number = 0;
  vScan("%li", &number);
  browse_image(number);
  return 1;
}


long vInterpreter1::browse_image(long number)
{
  if (input == 0)
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  if ((number < 0) || (number >= buffer_size))
  {
    return 0;
  }
  long temp_index = (input->buffer_index + buffer_size - number) % buffer_size;
  input->current_image = input->image_buffer[temp_index];
  if (input->current_image == 0) 
		input->current_image = input->image_buffer[input->buffer_index];
  else 
  {
    RedrawImages();
  }

  return 1;
}


long vInterpreter1::SetSequenceStep()
{
  if (input == 0) return 0;
  vPrint("\nPlease enter sequence step: ");
  vScan("%li", &(input->sequence_step));

  return 1;
}


long vInterpreter1::SetSequenceStep(long step)
{
  if (input == 0) return 0;
  input->sequence_step = step;

  return 1;
}


long vInterpreter1::AverageRegions()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  long window_size;
  print("\nPreparing to average regions. Input window size:\n");
  question ("%li", &window_size);
  general_image * average, * resampled;
  average = input->current_image->AverageRegions(window_size, window_size);
  resampled = average->Resample(input->current_image->Rows(),
				input->current_image->Cols());
  ShowImage(resampled);
  vdelete(average);

  return 1;
}


long vInterpreter1::InputCalibration()
{
  char buffer[1000];

  vPrint("Name of file with calibration info: ");
  scanf("%s", buffer);
  vdelete(calibration);
  calibration = new vCalibrationInfo(buffer);
  if (calibration->IsValid() == 0) 
  {
    vPrint("Failed to load calibration file.\n");
    return 0;
  }

  vPrint("Loaded calibration file.\n");

  return 1;
}


long vInterpreter1::UndistortFrame()
{
  if (calibration == 0) 
  {
    vPrint("No calibration info has been loaded.\n");
    return 0;
  }

  v3dMatrix<uchar> * undistorted;

  undistorted = calibration->Undistort((color_image *) input->current_image);
  
  ShowImage(undistorted);

//    double row, col;
//    row = 0;
//    while(row != 999)
//    {
//      cout << "Type row and col: ";
//      cin >> row >> col;
//      double out_row1, out_row2, out_col1, out_col2;
//      cal_info.UndistortedCoords(row, col, out_row1, out_col1);
//      cal_info.UndistortedCoords2(row, col, out_row2, out_col2);
//      vPrint("(%12.10f, %12.10f)\n", out_row1, out_col1);
//      vPrint("(%12.10f, %12.10f)\n", out_row2, out_col2);
//    }

  return 1;
}


long vInterpreter1::EstimateForeground()
{
  if (background_model == 0) return 0;
  if (!background_model->IsValid()) return 0;
  ushort delete_flag = 0;
  
  color_image * current = input->current_image->ToRGB();
  if (current != input->current_image) delete_flag = 1;
  v3dMatrix<double> * foreground;
  foreground = background_model->EstimateForeground(current);
  ShowImage(foreground);
  if (delete_flag != 0) vdelete(current);

  return 1;
}


long vInterpreter1::WritePPM()
{
  if (input == 0) return 0;
  char buffer[1000];
  print ("\nSpecify filename to save image to:\n");
  question ("%s", buffer);
  ushort success = input->current_image->Write(buffer);
  if (success == 0)
  {
    vPrint("Error in saving image.");
  }
  else 
  {
    vPrint("Saved image successfully");
  }

  return 1;
}


long vInterpreter1::ProcessRegions()
{
  if (input == 0) return 0;
  color_image * the_image;
  long threshold;
  if (input->current_image == 0) return 0;
  print ("\nThreshold:\n");
  question ("%li", &threshold);
  
  // Threshold the current image
  v3dMatrix<uchar> * threshold_color_image;
  threshold_color_image = input->current_image->threshold(threshold);

  vector<Label> labels1, labels2;
  v3dMatrix<long> * connected_background, * connected_regions;
  
  // Get the connected components of the thresholded image
  // connected_background is the labeled image and labels
  // contains Label objects with information about each label
  // used in connected_background
  connected_background = threshold_color_image->ConnectedComponents(&labels1);

  // Create an RGB image that colors each component with a distinct color
  the_image = vLabelsToImage(connected_background, &labels1);
  ShowImage(the_image);

  // Invert threshold_image
  vInverseImage(threshold_color_image);
  // Get connected components of threshold_color_image and their labels
  connected_regions = threshold_color_image->ConnectedComponents(&labels2);
  // Create an easy-to-look-at color image of the connected components
  // and displayit
  the_image = vLabelsToImage(connected_regions, &labels2);
  ShowImage(the_image);

  v3dMatrix<long> * connected_no_holes = vCopyImage(connected_regions);
  // Fill up the holes in the connected image, and display the result.
  vFillUpHoles(connected_no_holes, connected_background);
  the_image = vLabelsToImage(connected_no_holes, &labels2);
  vdelete(threshold_color_image);
  vdelete(connected_no_holes);

  ShowImage(connected_regions);
  ShowImage(connected_background);
  ShowImage(the_image);

  return 1;
}


// Here we assume that the current image is a mask showing what we want to
// track, and the current frame in the avi file corresponds to that mask.
long vInterpreter1::InitializeSkinTracker()
{
  if (input == 0) return 0;
  general_image * base = input->current_image;

  Type type_name = base->TypeName();
  if (type_name != UcharType) return 0;
  if (base->Bands() != 1) return 0;

  vPrint("Initializing skin tracker...\n");
  v3dMatrix<uchar> * mask = (v3dMatrix<uchar> *) base;
  vArray(uchar) matrix = mask->Matrix();
  long size = mask->Size();
  long i;
  for (i = 0; i < size; i++)
  {
    if (matrix[i] == 255) matrix[i] = 1;
  }

#ifndef VASSILIS_LINUX_PLATFORM
  vdelete(skin_tracker);
  skinTrackerParams params(base->Cols(), 
			   base->Rows(), 3);
  skin_tracker = new skinTracker(params);


  ShowCurrentFrame();
  color_image * the_image = (color_image *) input->current_image;
  vArray(uchar) buffer = vImageToRGB(the_image);
  vArray(uchar) mask_buffer = mask->Matrix();
  skin_tracker->initialize(vData(buffer), vData(mask_buffer));
  skin_tracker->initialize(vData(buffer));

  vPrint("Skin tracker is now initialized.\n");
  vdelete2(buffer);
#endif

  return 1;
}


long vInterpreter1::SkinTrackNextFrame()
{

#ifndef VASSILIS_LINUX_PLATFORM
  NextInSequence();
  color_image * the_image = (color_image *) input->current_image;
  
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = the_image->Size();
  uchar * result_buffer;

  vArray(uchar) buffer = vImageToRGB(the_image);
  skin_tracker->run(vData(buffer), result_buffer);
  vArray(uchar) result_array = vToArray(size, result_buffer);
  color_image * result = vRGBToImage(result_array, rows, cols);
  ShowImage(result);

  vPrint("Done tracking next frame.\n");
  vdelete2(buffer);
  vdelete2(result_array);
#endif

  return 1;
}


long vInterpreter1::HandTrackSequences()
{
  char directory[2000];
  print ("\nType in the directory whose sequences must be processed:\n");
	question ("%s", directory);
  vector<voidp> filenames;
  assert(0);
  // I should write a PC function for this, and uncomment the next line.
  //vDirectoryFiles(directory, filenames);

  long length = filenames.size();
  vPrint("length = %li\n", length);
  long i;
  for (i = 0; i < length; i++)
  {
    const char * filename = (const char *) filenames[i];
    vPrint("i = %li, filename = %s\n", i, filename);
    vInterpreter1::HandTrackSequence(filename);
  }
  vEraseCharpVector(&filenames);

  return 1;
}


long vInterpreter1::HandTrackSequence()
{
  if (input == 0) return 0;
  HandTrackSequence(input->path_name);

  return 1;
}


long vInterpreter1::HandTrackSequence(const char * filename)
{
  if (!vStringCaseEndsIn(filename, ".avi")) return 0;
  long success = AddInput(filename);
  if (success == 0) return 0;
  input->SetCurrentFrame(0);
  ShowCurrentFrame();

  char buffer[2000];
  sprintf(buffer, "%s.boundaries", filename);
  FILE * boundary_fp = fopen(buffer, vFOPEN_WRITE);
  assert(boundary_fp != 0);

  vVideoFile * video_file = input->sequence;
  long frames = input->number_of_frames;
  long hands_found = FindHands();
  if (hands_found == 0) return 0;

  v3dMatrix<uchar> * threshold_color_image;
  threshold_color_image = input->current_image->threshold(17);
  vInverseImage(threshold_color_image);
  ShowImage(threshold_color_image);
  
  InitializeSkinTracker();
  Label body_parts[3]; // Face, left hand, right hand.
  ushort first_frame_flag = 1;

  while (input->CurrentFrame() < frames - 1)
  {
    SkinTrackNextFrame();
    v3dMatrix<long> * connected;
    v3dMatrix<uchar> * thresholded;
    color_image * current = input->current_image->ToRGB();
    current->Save(buffer);
    if (current != input->current_image) vdelete(current);
    thresholded = input->current_image->threshold(1);
    vector<Label> labels;
    connected = thresholded->ConnectedComponents(&labels);
    std::sort(labels.begin(), labels.end(), LabelMore());

    long length = labels.size();
    body_parts[0] = labels[1];

    if (first_frame_flag)
    {
      body_parts[1] = vLeftmostLabel(labels, 2, 3);
      body_parts[2] = vRightmostLabel(labels, 2, 3);
      first_frame_flag = 0;
    }
    else 
    {
      body_parts[1] = vFindClosestLabel(labels, body_parts[1], 2, 3);
      body_parts[2]  = vFindClosestLabel(labels, body_parts[2], 2, 3);
    }

    if (length > 4) length = 4;
    long i;
    for (i = 0; i < length - 1; i++)
    {
      Label & label = body_parts[i];
      if (label.count == 0) continue;

      v3dMatrix<long> * box = vCopyRectangle(connected, 
						label.left, label.right,
						label.top, label.bottom);
      v3dMatrix<uchar> * selected = vSelectValue(box, label.id);
      color_image * the_image = selected->ToRGB();
      sprintf(buffer, "%s.box%li.%li.ppm", 
	      filename, i, input->CurrentFrame());
      the_image->Save(buffer);
      fprintf(boundary_fp, "%li %li %li %li %li %li\n", 
	      input->CurrentFrame(), i, label.left, label.right, 
	      label.top, label.bottom);
      vPrint("%li %li %li %li %li %li\n", 
	     input->CurrentFrame(), i, label.left, label.right, 
	     label.top, label.bottom);
      vdelete(box);
      vdelete(selected);
      vdelete(the_image);
    }
    vdelete(connected);
    vdelete(thresholded);
  }
  fclose(boundary_fp);

  return 1;
}


long vInterpreter1::JumpToFrame0()
{

  if (input == 0) return 0;
  vPrint("Input frame to jump to: ");
  long frame;
  vScan("%li", &frame);
  return JumpToFrame(frame);
}

  
long vInterpreter1::JumpToFrame(long frame)
{
  if (input == 0) return 0;
  long success = input->SetCurrentFrame(frame);
  if (success >= 0) ShowCurrentFrame();
  return success;
}


long vInterpreter1::SkinSegmentHand0a()
{
  if (input == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  SkinSegmentHand1(current);
  return 1;
}


long vInterpreter1::SkinSegmentHand0b()
{
  if (input == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  long threshold = 0;
  vPrint("Input threshold: ");
  vScan("%li", &threshold);
  SkinSegmentHand2(current, threshold);
  return 1;
}


long vInterpreter1::SkinSegmentHand0c()
{
  if (input == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  long threshold = 0;
  long row = 0;
  long col = 0;
  vPrint("Input threshold, row, col: ");
  vScan("%li %li %li", &threshold, &row, &col);
  SkinSegmentHand4(current, threshold, row, col);
  return 1;
}


color_image * vInterpreter1::SkinSegmentHand1(color_image * source_image)
{
  return SkinSegmentHand4(source_image, -1, -1, -1);
}


color_image * vInterpreter1::SkinSegmentHand2(color_image * source_image, 
                                           long threshold)
{
  return SkinSegmentHand4(source_image, threshold, -1, -1);
}


// Note that this function duplicates a lot of the work of
// vRefineHandSegmentation.
color_image * vInterpreter1::SkinSegmentHand4(color_image * source_image, 
                                           long threshold,
                                           long row, long col)
{
  // Check if row and col have valid values. If both are -1, then
  // they are not used. Otherwise, they must specify a valid pixel
  // in the image.
  long rows = source_image->Rows();
  long cols = source_image->Cols();

  if ((row != -1) && (col != -1))
  {
    if ((row < 0) || (col < 0) || 
        (row >= source_image->Rows()) || (col >= source_image->Cols()))
    {
      vPrint("Error, (%li, %li) is outside the image bounds\n", row, col);
      vdelete(source_image);
      return 0;
    }
  }

  // Set default value for threshold, if threshold = -1
  if (threshold == -1) threshold = 20;

  vInterpreter1::InitializeSkinData();
  const long BLUR_WINDOW = 5;
  ShowImage(source_image);

  // Blur source image.
  v3dMatrix<double> * blurred = vBlur(source_image);
  ShowImage(blurred);
  color_image * blurred_rgb = blurred->ToRGB();
 
  // Detect skin and threshold scores to get a binary skin image
  grayscale_image * skin_scores = skin_table->DetectSkin(blurred_rgb);
  ShowImage(skin_scores);
  grayscale_image * skin_binary = (grayscale_image *) skin_scores->threshold(threshold);
  ShowImage(skin_binary);

  // Do connected component analysis on the binary skin image and find the
  // largest component
  vector<Label> labels;
  v3dMatrix<long> * connected = skin_binary->ConnectedComponents(&labels);
  ShowImage(connected);

  // Fill up holes in the connected components.
  // First we need to produce a label image where the background is label 1.
  // To do that, we invert the binary skin image and do connected component
  // analysis on the result.
  grayscale_image * inverse_skin_binary = skin_binary->ToGrayCopy();
  vInverseImage(inverse_skin_binary);
  vector<Label> temp_labels;
  v3dMatrix<long> * background_component = 
    inverse_skin_binary->ConnectedComponents(&temp_labels);
  ShowImage(background_component);

  connected = new v3dMatrix<long>(connected);
  vFillUpHoles(connected, background_component);
  ShowImage(connected);

  vdelete(inverse_skin_binary);
  //vdelete(background_component);

  // Filling up holes may have connected components that were previously
  // not connected (this happens if one component was fully enclosed by
  // another but having background pixels separating them). So, we should
  // do connected component analysis again.
  grayscale_image * component_binary = (grayscale_image *) connected->threshold(1);
  labels.erase(labels.begin(), labels.end());
  connected = component_binary->ConnectedComponents(&labels);
  vdelete(component_binary);
  ShowImage(connected);

  std::sort(labels.begin(), labels.end(), LabelMore());
  Label largest = labels[1];

  // Select the connected component that corresponds to the hand. If
  // row and col are valid pixel coordinates, select the component
  // that contains that pixel. If (row, col) is not a valid pixel,
  // then we choose as hand the component that contains the 
  // point (row, col), then we choose the largest component.
  long hand_id;
  if ((row == -1) && (col == -1))
  {
    // Some hacks to increase likelihood of finding the hand.
    long index = 1;
    while(1)
    {
      const long margin = 2;
      if ((largest.top - margin >= 0) &&
          (largest.bottom + margin < rows) &&
          (largest.left - margin >= 0) &&
          (largest.right + margin < cols))
      {
        hand_id = largest.id;
        break;
      }
      else if (index == labels.size() - 1)
      {
        hand_id = largest.id;
        break;
      }
      else
      {
        index++;
        largest = labels[index];
      }
    }
  }
  else 
  {
    vArray2(long) connected_matrix = connected->Matrix2(0);
    hand_id = connected_matrix[row][col];
  }

  v3dMatrix<uchar> * hand_binary_temp1 = vSelectValue(connected, hand_id);
  v3dMatrix<uchar> * hand_binary_temp2 = new v3dMatrix<uchar>(source_image->Rows(), 
                                                                  source_image->Cols(), 1);
  function_image_divide(hand_binary_temp1, 255, hand_binary_temp2);
  v3dMatrix<uchar> * hand_binary = hand_binary_temp2->ToRGB();
  vdelete(hand_binary_temp1);
  vdelete(hand_binary_temp2);
  color_image * hand_image = new color_image(source_image->Rows(), source_image->Cols());
  function_image_multiply(hand_binary, source_image, hand_image);

  // Finally, we should make sure that there is no pixel that is marked as
  // hand in hand_binary that has RGB value (0,0,0), because in that case
  // that pixel may be considered a background pixel in later processing
  // of the hand (like contour finding). So, to every pixel that is 
  // marked as hand in hand_binary we will make sure that it has 
  // intensity more than 1 in every band.

  vArray(uchar) binary = hand_binary->Matrix(0);
  vArray(uchar) R = hand_image->GetR();
  vArray(uchar) G = hand_image->GetR();
  vArray(uchar) B = hand_image->GetR();
  long size = hand_binary->Size();

  long i;
  for (i = 0; i < size; i++)
  {
    if (binary[i] == 0) continue;
    if (R[i] < 2) R[i] = 2;
    if (G[i] < 2) G[i] = 2;
    if (B[i] < 2) B[i] = 2;
  }

  vdelete(blurred_rgb);
  vdelete(hand_binary);

  
  ShowImage(hand_image);
  return hand_image;
}


// Sets certain variables in the current match scores, using variables stored
// in the interpreter object.
long vInterpreter1::UpdateMatchScores(long frame_number)
{
  if (match_scores == 0) return 0;
  score_frame = frame_number;
  JumpToFrame(score_frame);
  SetCurrentInput(0);
  JumpToMatch(0);
  match_scores->SetGroundDistances(ground_distances);
  if (test_score_file != 0)
  {
    match_scores->SetTestFilename(test_score_file);
  }
  match_scores->SetTestFrame(frame_number);
  if (ground_truth_distance != (double) -1)
  {
    match_scores->SetGroundTruthDistance(ground_truth_distance);
  }
  return 1;
}


long vInterpreter1::LoadMatchScores()
{
  char filename[1000];
  vPrint("Filename to load scores from: ");
  vScan("%s", filename);
  return LoadMatchScores(filename);
}


long vInterpreter1::LoadMatchScores(char * filename)
{
  vdelete(match_scores);
  match_scores = new vMatchScores(filename);
  if (!match_scores->IsValid()) 
  {
    vPrint("Could not open scores file %s\n", filename);
    vdelete(match_scores);
    match_scores = 0;

    return 0;
  }

  else
  {
    // Set the current score base
    //char base[500];
    //sscanf(filename, "%s.%li.txt", base, &score_frame);
    //vPrint("base = %s, frame_number = %li\n", base, score_frame);
    //vdelete2(score_base);
    //score_base = vCopyString(base);
    SetScoreOrder(score_order);
    
    return 1;
  }
}


long vInterpreter1::SetScoreBase0()
{
  char temp_base[500];
  long temp_frame = 0;
  vPrint("Input new score base and frame: ");
  vScan("%s %li", temp_base, &temp_frame);
  return SetScoreBase(temp_base, temp_frame);
}

// In general, in SetScoreBase we assume that matching scores
// pertaining to matches between test frame i (from some video file)
// and a training set are saved in score_base.i.txt. SetScoreBase and 
// LoadNextScores make it a bit easier to load scores pertaining
// to the same experiment but different frames.
//
// In SetScoreBase and LoadNextScores we assume that input 0
// is the training set and input 1 is the test set.
long vInterpreter1::SetScoreBase(const char * temp_base, long temp_frame)
{
  if (input == 0) return 0;
  char * score_path = vJoinPaths(sequences_directory, "glove_data\\results");
  char * new_score_base = vJoinPaths(score_path, temp_base);

  char * temp_filename = new char[strlen(new_score_base) + 40];
  sprintf(temp_filename, "%s.%li", new_score_base, temp_frame);
  vdelete2(score_path);
  // Check if the new score base and score frame are valid

  long success = LoadMatchScores(temp_filename);
  if (success != 0) 
  {
    // If they score base and score frame are valid, store them
    vdelete2(score_base);
    score_base = vCopyString(new_score_base);
    score_frame = temp_frame;
  
    UpdateMatchScores(input->current_frame);
  }
  else
  {
    vPrint("Can't set score base to %s and %li\n", temp_base, temp_frame);
  }
  vdelete2(temp_filename);
  vdelete2(new_score_base);
  return success;
}


long vInterpreter1::LoadCurrentScores()
{
  long new_frame_number = input->current_frame;
  return LoadMatchScoresN(new_frame_number);
}


long vInterpreter1::LoadNextScores()
{
  long new_frame_number = score_frame + 1;
  return LoadMatchScoresN(new_frame_number);
}


long vInterpreter1::LoadMatchScoresN0()
{
  long frame_number = 0;
  vPrint("Input frame number: ");
  vScan("%li", &frame_number);
  return LoadMatchScoresN(frame_number);
}


long vInterpreter1::LoadMatchScoresN(long frame_number)
{
  if (score_base == 0) return 0;
  char * temp_filename = new char[strlen(score_base) + 40];
  sprintf(temp_filename, "%s.%li", score_base, frame_number);
  long success = LoadMatchScores(temp_filename);
  if (success == 0) 
  {
    vPrint("Error, can't load scores from %s\n", temp_filename);
  }
  else
  {
    // Also, move test sequence to score_frame and training sequence
    // to the best match.
/*    score_frame = frame_number;
    SetCurrentInput(1);
    JumpToFrame(score_frame);
    SetCurrentInput(0);
    JumpToMatch(0);
    if (test_score_file != 0)
    {
      match_scores->SetTestFilename(test_score_file);
    }
    match_scores->SetTestFrame(frame_number);
    if (ground_truth_distance != (double) -1)
    {
      match_scores->SetGroundTruthDistance(ground_truth_distance);
    }
*/
    UpdateMatchScores(frame_number);
    vPrint("Scores loaded from %s\n", temp_filename);
  }
  vdelete2(temp_filename);
  return success;
}


long vInterpreter1::JumpToMatch()
{
  long rank = 0;
  vPrint("Input rank (highest is 0): ");
  vScan("%li", &rank);
  return JumpToMatch(rank);
}


long vInterpreter1::JumpToMatch(long index)
{
  if (match_scores == 0)
  {
    vPrint("No match scores are loaded\n");
    return 0;
  }

  vMatchScore score = match_scores->GetScore(index);
  if (score.is_valid == 0)
  {
    vPrint("Unable to get score of rank %li\n", index);
    return 0;
  }
  
  score.Print();
  DatabaseFrame(score.frame, score.view);

  return 1;
}


long vInterpreter1::ViewRank0()
{
  long frame = 0;
  long view = 0;
  vPrint("Input frame and view: ");
  vScan("%li %li", &frame, &view);
  return ViewRank(frame, view);
}


long vInterpreter1::ViewRank(long frame, long view)
{
  DatabaseFrame(frame, view);
  if (match_scores == 0)
  {
    vPrint("No match scores are loaded\n");
    return 0;
  }

  vMatchScore score = match_scores->GetRank(frame, view);
  if (score.is_valid <= 0)
  {
    vPrint("Error, no valid score found.\n");
    return 0;
  }

  score.Print();
  return 1;
}

  
long vInterpreter1::SetScoreOrder(ushort order)
{
  if ((order < 0) || (order > 1))
  {
    return 0;
  }
  score_order = order;
  if (match_scores != 0)
  {
    match_scores->SetOrder(score_order);
  }
  return 1;
}


long vInterpreter1::ScoresToBinary0()
{
  if (match_scores == 0)
  {
    vPrint("Can't convert files, no scores have been loaded yet.\n");
    return 0;
  }

  long limit;
  vScan("Input limit: %li", &limit);

  ushort success = match_scores->ToBinary(limit);
  if (success == 0)
  {
    vPrint("Failed to convert scores to binary\n");
  }
  else
  {
    vPrint("Finished converting scores to binary\n");
  }
  return success;
}


long vInterpreter1::SetTestFilename0()
{
  long input_index = 0;
  vPrint("Input input index for test filename: ");
  vScan("%li", &input_index);
  return SetTestFilename(input_index);
}


long vInterpreter1::GroundMatch0()
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vPrint("Input rank of ground truth match: ");
  long rank = 0;
  vScan("%li", &rank);
  return GroundMatch(rank);
}


long vInterpreter1::CorrectMatch0()
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vPrint("Input rank of correct match: ");
  long rank = 0;
  vScan("%li", &rank);
  return CorrectMatch(rank);
}
  

long vInterpreter1::CorrectMatch(long rank)
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vMatchScore score = match_scores->CorrectMatch(rank);
  if (score.is_valid == 0)
  {
    vPrint("Failed to get correct truth match of rank %li\n", rank);
    return 0;
  }
  else
  {
    score.Print();
    DatabaseFrame(score.frame, score.view);
    return 1;
  }
}


long vInterpreter1::SetTestFrame0()
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vPrint("Input test frame: ");
  long frame = 0;
  vScan("%li", &frame);
  return SetTestFrame(frame);
}


long vInterpreter1::GroundTruthStatistics0()
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vPrint("Input frame limit: ");
  long frame = 0;
  vScan("%li", &frame);
  return GroundTruthStatistics(frame);
}


long vInterpreter1::SetTestFilename(long input_index)
{
  vVideoFile * video = GetInput(input_index);
  if (video == 0)
  {
    vPrint("No input found with index %li\n", input_index);
    return 0;
  }
  const char * in_test_filename = video->GetFilename();
  vdelete2(test_score_file);
  test_score_file = vCopyString(in_test_filename);

  char * ground_distances_file = vReplaceEnding2(in_test_filename, ".gt_distances");
  vdelete(ground_distances);
  ground_distances = v3dMatrix<ushort>::Read(ground_distances_file);
  vdelete2(ground_distances_file);
  if (ground_distances == 0)
  {
    vPrint("Failed to load ground distances for file %s\n", in_test_filename);
    return 0;
  }

  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  ushort success = match_scores->SetTestFilename(in_test_filename);
  if (success == 0)
  {
    vPrint("Failed to set test filename to %s\n", in_test_filename);
    return 0;
  }

  else
  {
    vPrint("Set test filename to %s\n", in_test_filename);
    match_scores->SetGroundDistances(ground_distances);
    return 1;
  }
}

long vInterpreter1::GroundMatch(long rank)
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  vMatchScore score = match_scores->GroundMatch(rank);
  if (score.is_valid == 0)
  {
    vPrint("Failed to get ground truth match of rank %li\n", rank);
    return 0;
  }
  else
  {
    score.Print();
    DatabaseFrame(score.frame, score.view);
    return 1;
  }
}


long vInterpreter1::SetTestFrame(long in_test_frame)
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  ushort success = match_scores->SetTestFrame(in_test_frame);
  if (success == 0)
  {
    vPrint("Failed to set test frame to %li\n", in_test_frame);
    return 0;
  }
  else
  {
    vPrint("Set test frame to %li\n", in_test_frame);
    return 1;
  }
}


long vInterpreter1::GroundTruthStatistics(long limit)
{
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }

  ushort success = match_scores->GroundTruthStatistics(limit);
  if (success == 0)
  {
    vPrint("Failed to get ground truth statistics\n");
    return 0;
  }
  else
  {
    vPrint("Finished getting ground truth statistics\n");
    return 1;
  }
}




// End of functions interfacing with vMatchScores.


long vInterpreter1::CenterOfMass()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * the_image = current->ToGray();
  double center_row = 0;
  double center_col = 0;

  vCenterOfMass(the_image, center_row, center_col);
  double max_distance = vMaxDistanceFromPoint(the_image, center_row, center_col);
  vPrint("center of mass = (%lf, %lf), max_distance = %lf\n", 
          center_row, center_col, max_distance);
  if (the_image != current) vdelete(the_image);
  return 1;
}


long vInterpreter1::NormalizeImage(double distance, double center_row, 
                                      double center_col)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * the_image = current->ToGray();
  grayscale_image * normalized = (grayscale_image *) vNormalizeImage(the_image, distance, 
                                                              center_row, center_col);
  ShowImage(normalized);
  if (the_image != current) vdelete(the_image); 
  return 1;
}

long vInterpreter1::MatchWithRLESequence()
{
  assert(0); // Need to modify a couple of things, this is not runnable.
  vReadAVI avi(0);
  if (avi.IsValid() == 0) 
  {
    vPrint("Bad avi\n");
    return 0;
  }
  vRLE_Sequence rle(0);
  if (rle.IsValid() == 0) 
  {
    vPrint("Bad rle\n");
    return 0;
  }
  long rows = rle.Rows();
  long cols = rle.Cols();

  grayscale_image * temp_gray1 = avi.ReadFrameGray(0);
  grayscale_image * temp_gray2 = (grayscale_image *) vPad(temp_gray1, 256, 256);
  grayscale_image * temp_gray3 = (grayscale_image *) vNormalizeImage(temp_gray2, 96.0, 127.5, 127.5);
  grayscale_image * gray1 = (grayscale_image *) temp_gray3->threshold(1);
  vdelete(temp_gray1);
  vdelete(temp_gray2);
  vdelete(temp_gray3);
  vRLE_Binary * rle1 = new vRLE_Binary(gray1);
  vRLE_Binary * rle2 = (vRLE_Binary *) rle.ReadRLEFrame(0);
  grayscale_image * gray2 = rle2->ToGrayImage();

  long score1, score2, score3, score4;
  score1 = vMatchBinaryImages(gray1, gray2);
  score2 = vMatchBinaryImages(gray2, gray1);
  score3 = vFastMatch(rle1, rle2, rows, cols);
  score4 = vFastMatch(rle2, rle1, rows, cols);
  vPrint("score1 = %li, score2 = %li\n", score1, score2);
  vPrint("score3 = %li, score4 = %li\n", score3, score4);

  vdelete(gray1);
  vdelete(rle1);
  vdelete(gray2);
  vdelete(rle2);
  return 1;
}


long vInterpreter1::RotateLeft0()
{
  if (input == 0) return 0;
  long turns = 0;
  vPrint("Input left turns: ");
  vScan("%li", &turns);
  return RotateLeft(turns);
}


// Rotates the current image by 90 * right_turns degrees
long vInterpreter1::RotateLeft(long turns)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * rgb = current->ToRGB();
  color_image * rotated = (color_image *) vRotateRight(rgb, 360 - turns * 90);
  ShowImage(rotated);
  if (rgb != current) vdelete(rgb);
  return 0;
}


long vInterpreter1::RotateRight0()
{
  if (input == 0) return 0;
  long right_turns = 0;
  vPrint("Input right turns: ");
  vScan("%li", &right_turns);
  return RotateRight(right_turns);
}


// Rotates the current image by 90 * right_turns degrees
long vInterpreter1::RotateRight(long right_turns)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  color_image * rgb = current->ToRGB();
  color_image * rotated = (color_image *) vRotateRight(rgb, right_turns * 90);
  ShowImage(rotated);
  if (rgb != current) vdelete(rgb);
  return 0;
}


long vInterpreter1::NewImage()
{
  if (input == 0) return 0;
  long rows = 0;
  long cols = 0;
  vPrint("Input rows and cols: ");
  vScan("%li %li", &rows, &cols);
  vPrint("\n");
  return NewImage(rows, cols);
}


long vInterpreter1::NewImage(long rows, long cols)
{
  if (input == 0) return 0;
  color_image * the_image = new color_image(rows, cols);
  vWriteValue(the_image, (uchar) 0);
  ShowImage(the_image);
  return 1;
}


long vInterpreter1::DrawLine()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawLine(row1, col1, row2, col2, value);
}


long vInterpreter1::DrawLine(long row1, long col1, long row2, long col2, long value)
{
  if (input == 0) return 0;
  color_image * current = (color_image *) input->current_image->ToRGBCopy();
  ushort result = vDrawLine1(current, row1, col1, row2, col2, (uchar) value);
  ShowImage(current);
  return result;
}


long vInterpreter1::DrawLineB()
{
  long row, col, length, value;
  double angle;
  row = col = length = value = 0;
  angle = 0.0;
  vPrint("Input row, col, angle, length, value:\n");
  vScan("%li %li %lf %li %li", &row, &col, &angle, &length, &value);
  return DrawLine2(row, col, angle, length, value);
}


long vInterpreter1::DrawLine2(long row, long col, double angle, long length, long value)
{
  if (input == 0) return 0;
  color_image * current = (color_image *) input->current_image->ToRGBCopy();
  ushort result = vDrawCenteredLine(current, row, col, angle, length, (uchar) value);
  ShowImage(current);
  return result;
}


long vInterpreter1::DrawRectangle()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawRectangle(row1, col1, row2, col2, value);
}


long vInterpreter1::DrawRectangle(long row1, long col1, 
                                    long row2, long col2, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawRectangle1(current, row1, col1, row2, col2, (uchar) value);
  DrawCurrentImage();
  return result;
}


long vInterpreter1::DrawCircle()
{
  long row1, col1, radius, value;
  row1 = col1 = radius = value = 0;
  vPrint("Input row, col, radius, value");
  vScan("%li %li %li %li", &row1, &col1, &radius, &value);
  vPrint("\n");
  return DrawCircle(row1, col1, radius, value);
}


long vInterpreter1::DrawCircle(long row, long col, long radius, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawCircle1(current, row, col, radius, (uchar) value);
  DrawCurrentImage();
  return result;
}


long vInterpreter1::DrawFilledRectangle()
{
  long row1, col1, row2, col2, value;
  row1 = col1 = row2 = col2 = value = 0;
  vPrint("Input row1, col1, row2, col2, value");
  vScan("%li %li %li %li %li", &row1, &col1, &row2, &col2, &value);
  vPrint("\n");
  return DrawFilledRectangle(row1, col1, row2, col2, value);
}


long vInterpreter1::DrawFilledRectangle(long row1, long col1, 
                                    long row2, long col2, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawFilledRectangle1(current, row1, col1, 
                                         row2, col2, (uchar) value);
  DrawCurrentImage();
  return result;
}


long vInterpreter1::DrawFilledCircle()
{
  long row1, col1, radius, value;
  row1 = col1 = radius = value = 0;
  vPrint("Input row, col, radius, value");
  vScan("%li %li %li %li", &row1, &col1, &radius, &value);
  vPrint("\n");
  return DrawFilledCircle(row1, col1, radius, value);
}


long vInterpreter1::DrawFilledCircle(long row, long col, long radius, long value)
{
  if (input == 0) return 0;
  if (input->current_image->TypeName() != UcharType) return 0;
  color_image * current = (color_image *) input->current_image;
  ushort result = vDrawFilledCircle1(current, row, col, radius, (uchar) value);
  DrawCurrentImage();
  return result;
}


long vInterpreter1::NormalizeLabeledImage(double distance, double center_row, 
                                            double center_col)
{
  if (input == 0) return 0;   
  general_image * current = input->current_image;
  color_image * the_image = current->ToRGB();
  color_image * normalized = (color_image *) vNormalizeLabeledImage3(the_image, distance, 
                                                              center_row, center_col);
  ShowImage(normalized);
  if (the_image != current) vdelete(the_image); 
  return 1;
}


long vInterpreter1::SelectValue0()
{
  if (input == 0) return 0;
  int value;
  vPrint("Input value to select: ");
  vScan("%i", &value);
  return SelectValue(value);
}


long vInterpreter1::SelectValue(long value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  v3dMatrix <long> * image = new v3dMatrix <long>(current);
  grayscale_image * result;
  result = (grayscale_image *) vSelectValue(image, value);
  ShowImage(result);
  if (image != current) vdelete(image);
  return 1;
}


long vInterpreter1::CreateLineSet()
{
  if (line_set != 0)
  {
    vPrint("Can't create line set, a line set already exists\n");
    return 0;
  }
  line_set = new vLineSet();
  mouse_click_response = LineResponse;
  vPrint("Created new line set\n");
  return 1;
}


long vInterpreter1::DestroyLineSet()
{
  vdelete(line_set);
  line_set = 0;
  mouse_click_response = pixel_info_response;
  vPrint("Destroyed line set\n");
  return 1;
}


long vInterpreter1::AddVertex(long row, long col)
{
  if (input == 0) return 0;
  if ((predicate_color_image(input->current_image) == 0) &&
      (predicate_gray_image(input->current_image) == 0))
  {
    return 0;
  }
  if (line_set == 0) 
  {
    vPrint("No line set exists, can't add vertex\n");
    return 0;
  }
  line_set->AddVertex(row, col);
  OverlayLineSet();

  return 1;
}


long vInterpreter1::DeleteVertex()
{
  if (input == 0) return 0;
  if ((predicate_color_image(input->current_image) == 0) &&
      (predicate_gray_image(input->current_image) == 0))
  {
    return 0;
  }
  if (line_set == 0) 
  {
    vPrint("No line set exists, can't add vertex\n");
    return 0;
  }
  line_set->DeleteVertex();
  ShowCurrentFrame();
  OverlayLineSet();
  
  return 1;
}

long vInterpreter1::StartPolygon()
{
  if (line_set == 0) return 0;
  mouse_click_response = LineResponse;
  return line_set->StartPolygon();
}


long vInterpreter1::DrawLineSet()
{
  if (input == 0) return 0;
  if (line_set == 0) return 0;
  long rows = input->current_image->Rows();
  long cols = input->current_image->Cols();
  grayscale_image * gray = new grayscale_image(rows, cols);
  vWriteValue(gray, (uchar) 0);
  ushort success = line_set->Draw(gray);
  if (success != 0) ShowImage(gray);
  return success;
}


long vInterpreter1::OverlayLineSet()
{
  if (input == 0) return 0;
  if (line_set == 0) return 0;
  general_image * image_with_edges = input->current_image->Copy();
  ushort success  = line_set->Draw(image_with_edges);
  if (success != 0) ShowImage(image_with_edges);
  return success;
}


long vInterpreter1::IncreaseResolution()
{
  if (input == 0) return 0;
  long resolution = input->resolution;
  return SetResolution(resolution+1);
}


long vInterpreter1::DecreaseResolution()
{
  if (input == 0) return 0;
  long resolution = input->resolution;
  return SetResolution(resolution-1);
}


long vInterpreter1::SetResolution0()
{
  vPrint("Input new resolution: ");
  long new_resolution;
  vScan("%li", &new_resolution);
  return SetResolution(new_resolution);
}


long vInterpreter1::SetResolution(long new_resolution)
{
  if (input == 0) return 0;
  input->resolution = new_resolution;
  long factor = (long) pow(2, new_resolution);
//  long magnified_rows, magnified_cols;
//  if (input->sequence != 0)
//  {
//    magnified_rows = factor * input->sequence->Height();
//    magnified_cols = factor * input->sequence->Width();
//  }
//  else 
//  {
//    magnified_rows = factor * input->current_image->Rows();
//    magnified_cols = factor * input->current_image->Cols();
//  }
//  if (magnified_rows > 600) magnified_rows = 600;
//  if (magnified_cols > 800) magnified_rows = 800;
//  input->magnified_rows = magnified_rows;
//  input->magnified_cols = magnified_cols;
  DrawCurrentImage();

  vPrint("Resolution = %li\n", new_resolution);
  return 1;
}

// Start of section using the OpenCV library
long vInterpreter1::CannyDetector0a()
{
  if (input == 0) return 0;
  double low = 0;
  vPrint("Input low threshold for Canny detector: ");
  vScan("%lf", &low);
  long success = CannyDetector(low);
  if (success == 0)
  {
    vPrint("Failed to perform Canny edge detection with given input and parameters\n");
  }
  return success;
}


long vInterpreter1::CannyDetector0b()
{
  if (input == 0) return 0;
  double low = 0;
  double high = 0;
  int aperture = 0;
  vPrint("Input low, high, aperture for Canny detector: ");
  vScan("%lf %lf %i", &low, &high, &aperture);
  long success = CannyDetector3(low, high, aperture);
  if (success == 0)
  {
    vPrint("Failed to perform Canny edge detection with given input and parameters\n");
  }
  return success;
}


long vInterpreter1::CannyDetector(double low)
{
  return CannyDetector3(low, 2*low, 3);
}


long vInterpreter1::CannyDetector3(double low, double high, int aperture)
{
  // Make sure no errors occur
  if (input == 0) return 0;

  // The cvCanny function will crash the program if the following conditions
  // are violated
  if (low < 0) return 0;
  if (high < 0) return 0;
  if (low > 255) return 0;
  if (high > 255) high = 255;
  if (aperture < 1) return 0;
  if ((aperture % 2) != 1) return 0;

  grayscale_image * source = input->current_image->ToGray();
  grayscale_image * edges = vCannyDetector(source, (uchar) low, (uchar) high, aperture);
  if (source != input->current_image)
  {
    vdelete(source);
  }
  if (edges == 0) return 0;
  ShowImage(edges);
  return 1;
}


// End of section using the OpenCV library


// In every video sequence, we may want to store and retrieve
// the color of the face 
long vInterpreter1::SetFaceColor0()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  FaceLocation * locs;
  long i;
  // Get the current input image
  general_image * the_image1 = input->current_image;
  color_image * current_image = input->current_image->ToRGB();

  // Find the face.
  long result = vInterpreter1::DetectFaces(&locs);
  
  if (result != 1)
  {
    vPrint("Can't proceed, failed to find face.\n");
    return 0;
  }

  assert(locs[0].x1 < locs[0].x2);
  assert(locs[0].y1 < locs[0].y2);
  face_box.top = locs[0].y1;
  face_box.bottom = locs[0].y2;
  face_box.left = locs[0].x1;
  face_box.right = locs[0].x2;

  // Get the image of the face rectangle
  color_image * face_image;
  face_image = (color_image *) vCopyRectangle(current_image, locs[0].x1+1, 
	                              				   locs[0].x2-1, locs[0].y1+1, 
					                                 locs[0].y2-1);
  free(locs);
  if (current_image != the_image1) vdelete(current_image);

  // This is the fraction of face pixels that we want to be classified
  // as skin with the threshold that we will find. The higher the fraction,
  // the more likely it is that non-skin pixels will be included
  const float FACE_SKIN_FRACTION = 0.50;
//  const float FACE_SKIN_FRACTION = 1.00;

  // Detect skin in the face image (the face image is a rectangle, parts of which
  // do not belong to the actual face).
  vInterpreter1::InitializeSkinData();
  grayscale_image * face_skin = skin_table->DetectSkin(face_image);

  // Make a histogram, where, for each value, we count the number of pixels
  // to which skin detection assigned a score equal to that value.
  long face_pixels = face_skin->Size();
  vArray(uchar) skin_values = face_skin->Matrix();

  // First initialize the histogram entries to 0
  long values[256];
  for (i = 0; i < 256; i++) 
  {
    values[i] = 0;
  }

  for (i = 0; i < face_pixels; i++)
  {
    values[(uchar) skin_values[i]]++;
  }

  // Convert the histogram to a cumulative histogram, where for each value
  // we store the number of pixels that had a score greater than or equal
  // to that value (score means skin detection score).
  for (i = 254; i >= 0; i--)
  {
    values[i] = values[i] + values[i+1];
  }

  // Find the appropriate threshold so that the fraction of pixels classified
  // as skin is at least equal to FACE_SKIN_FRACTION
  long required_pixels = (long) (((float) face_pixels) * FACE_SKIN_FRACTION);
  long threshold = 0;
  long skin_pixels = -1;
  for (i = 255; i >= 0; i--)
  {
    if (values[i] >= required_pixels)
    {
      threshold = i;
      skin_pixels = values[i];
      break;
    }
  }
  
  // Threshold the face skin image to obtain a binary classification of pixels
  // (value 255 is skin, value 0 is non-skin)
  v3dMatrix<uchar> * labels = face_skin->threshold(threshold);
  ShowImage(labels);

  // Initialize the label that we will pass to vMakeColorGrid2. That label
  // will specify which pixels should be used in building the color grid
  Label label;
  label.top = 0;
  label.left = 0;
  label.bottom = labels->Rows() - 1;
  label.right = labels->Cols() - 1;
  label.id = 255;
  label.count = skin_pixels;

  // Build the grid, which for each rg color will contain the distance
  // of that color (using the block metric) from the average color
  // of the facial skin
  vSearchGrid * grid = vMakeColorGrid2(face_image, labels, label);
  vdelete(input->face_color);
  input->face_color = grid;
  vdelete(face_image);
  vdelete(face_skin);

  return 1;
//  ShowImage(grid);
}


long vInterpreter1::SetHandThreshold0()
{
  long new_threshold;
  vPrint("Input new hand threshold: ");
  vScan("%li", &new_threshold);
  return SetHandThreshold(new_threshold);
}


long vInterpreter1::SetHandThreshold(long new_threshold)
{
  hand_threshold = new_threshold;
  return 1;
}


// This function is used to look at the frames that represent
// the depth values for pixels of the hand (rendered by 
// HandInSpace). The question I want to solve by looking at 
// this stuff is: What edge values indicate that the pixels
// belong to different links? I need to answer that to 
// find a better way to calculate finger edges. Currently,
// finger edges ignore edges between two neighboring links,
// but they should only ignore (sometimes) the edges on
// on the actual joint locations.
//
// The function assumes that the sequence with the previous
// index has an image that contains for each pixel an encoding
// of the ID of the link to which that pixel belongs.
//
// In this particular function, we create and display an image
// which, for every pixel, shows the distance between that pixel
// and the most distant 4-neighbor of the pixel that belongs to
// the same link as the pixel. We write -1 in pixels that are 
// background or (hard to imagine) if they have no neighbor that
// belongs to the same link as they.
long vInterpreter1::DepthNeighbors()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (input->current_image->TypeName() != FloatType) return 0;
  if (input->current_image->Bands() != 1) return 0;

  // Get the image with the link ID information
  general_image * link_base = CurrentImage(current_input_index - 1);
  if (link_base == 0) return 0;
  if (link_base->TypeName() != UcharType) return 0;
  v3dMatrix<uchar> * link_image = (v3dMatrix<uchar> *) link_base;
  vArray2(uchar) links = link_image->Matrix2(0);

  char * neighbors_file = vJoinPaths(sequences_directory, 
                                      "glove_data\\training\\neighbors.bin");
  v3dMatrix<ushort> * adjacencies = v3dMatrix<ushort>::Read(neighbors_file);
  vdelete2(neighbors_file);
  if (adjacencies == 0) return 0;
  vArray2(ushort) neighbors = adjacencies->Matrix2(0);
  
  v3dMatrix<float> * the_image = (v3dMatrix<float> *) input->current_image;

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long bottom = rows - 1;
  long right = cols - 1;

  // Create the result image
  v3dMatrix<float> * result = new v3dMatrix<float>(rows, cols, 1);
  // Write zeros at the borders
  vWriteBorderValue(result, (float) 0);
  vArray2(float) matrix1 = the_image->Matrix2(0);
  vArray2(float) matrix2 = result->Matrix2(0);
  

  long row, col;
  for (row = 1; row < bottom; row++)
  {
    for (col = 1; col < right; col++)
    {
      // The encoding I use for link IDs is that they are guaranteed
      // to be the pixel value of the first band % 70 (see 
      // hand_renderer.cpp).
      long center_id = links[row][col] % 70;
      if (center_id == 0) 
      {
        matrix2[row][col] = 0;
        continue;
      }
      
      float center = matrix1[row][col];
      // Take care of the case where the pixel is a background pixel
      float max_distance = 0;

      long next_id;
      float next;

      // Check top pixel
      next_id = links[row-1][col] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row-1][col];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check bottom pixel
      next_id = links[row+1][col] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row+1][col];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check left pixel
      next_id = links[row][col-1] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row][col-1];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check right pixel
      next_id = links[row][col+1] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row][col+1];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      matrix2[row][col] = max_distance;
    }
  }
  ShowImage(result);
  Histogram3(100, 0.0000001, 0.9999999);
  vdelete(adjacencies);
  return 1;
}


long vInterpreter1::MaxNeighbors()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * max_neighbors = vMaxNeighbors0(current);
  ShowImage(max_neighbors);
  return 1;
}


long vInterpreter1::Histogram0()
{
  if (input == 0) return 0;

  long number_of_bins = 0;
  vPrint("Enter number of bins for histogram: ");
  vScan("%li", &number_of_bins);
  return Histogram(number_of_bins);
}


long vInterpreter1::Histogram(long number_of_bins)
{
  if (input == 0) return 0;
  vMatrix<double> * histogram = vPixelHistogram0(input->current_image, 
                                                   number_of_bins);
  if (histogram == 0) return 0;
  histogram->Print();
  vdelete(histogram);
  return 1;
}


long vInterpreter1::Histogram30()
{
  if (input == 0) return 0;

  long number_of_bins = 0;
  double lower = 0;
  double upper = 0;
  vPrint("Enter number of bins, lower, upper: ");
  vScan("%li %lf %lf", &number_of_bins, &lower, &upper);
  return Histogram3(number_of_bins, lower, upper);
}


long vInterpreter1::Histogram3(long number_of_bins, double lower, double upper)
{
  if (input == 0) return 0;
  vMatrix<double> * histogram = vBoundedRankHistogram0(input->current_image, 
                                                         number_of_bins, lower, upper);
  if (histogram == 0) return 0;
  histogram->Print();
  vdelete(histogram);
  return 1;
}


long vInterpreter1::FingerEdges0()
{
  long link_input = 0;
  long depth_input = 0;
  char output[500];

  vPrint("Input link sequence, depth sequence, output filename: ");
  vScan("%li %li %s", &link_input, &depth_input, output);
  return FingerEdges(link_input, depth_input, output);
}


// Calls vArticulatedEdges on each frame of input video file with index
// link_input, using for depth the corresponding frame of the video file
// with index depth_input. It saves the results into the output
// video sequence named with output.
long vInterpreter1::FingerEdges(long link_input, long depth_input, 
                                   const char * output)
{
  vVideoFile * links = GetInput(link_input);
  if (links == 0) return 0;
  vVideoFile * temp_depths = GetInput(depth_input);
  if (temp_depths == 0) return 0;
  // The depths video file should be of type class_image_stream
  // with image type FloatType, since that is the only
  // video format I have right now that can save float images.
  if (strcmp(temp_depths->Type(), "class_image_stream") != 0) return 0;
  class_image_stream * depths = (class_image_stream *) temp_depths;
  if (depths->ImageType() != FloatType) return 0;

  return vSequenceArticulatedEdges(links, depths, output);
}


long vInterpreter1::ContourScores0()
{
  long test_input, training_input, low_frame, high_frame;
  test_input = training_input = low_frame = high_frame = 0;
  char scores_file[500];
  vPrint("Enter test index, training_index, low frame, high frame, scores file.\n");
  vScan("%li %li %li %li %s\n", &test_input, &training_input, 
         &low_frame, &high_frame, scores_file);
  return ContourScores(test_input, training_input, low_frame, high_frame,
                       scores_file);
}


long vInterpreter1::ContourScoresFast0()
{
  long test_input, training_input, low_frame, training_frames, high_frame;
  long grid_size, subsampling;
  test_input = training_input = low_frame = high_frame = 0;
  grid_size = subsampling = 0;
  char scores_file[500];
  vPrint("Enter test index, training_index, low frame, high frame,\n");
  vPrint("   training_frames, grid_size, subsampling, scores file.\n");
  vScan("%li %li %li %li %li %li %li %s\n", &test_input, &training_input, 
         &low_frame, &high_frame, &training_frames, &grid_size, 
         &subsampling, scores_file);
  return ContourScoresFast(test_input, training_input, low_frame, high_frame,
                           training_frames, grid_size, subsampling, scores_file);
}


long vInterpreter1::ContourScores(long test_input, long training_input, 
                                    long low_frame, long high_frame, 
                                    const char * temp_scores_file)
{
  // Get the inputs specified by the indices
  vVideoFile * temp_test_input = GetInput(test_input);
  if (temp_test_input == 0) return 0;
  vVideoFile * temp_training_input = GetInput(training_input);
  if (temp_training_input == 0) return 0;

  const char * test_file = temp_test_input->GetFilename();
  const char * training_file = temp_training_input->GetFilename();

  // Create a filename for the scores that is under the right directory
  char * scores_dir = vJoinPaths(sequences_directory, "glove_data\\results");
  char * scores_file = vJoinPaths(scores_dir, temp_scores_file);
  vdelete2(scores_dir);

  ushort success = vContourScores(test_file, low_frame, high_frame, 
                                   training_file, scores_file);
  vdelete2(scores_file);
  if (success == 0)
  {
    vPrint("\nError in processing contour scores of %s with %s\n",
            test_file, training_file);
  }
  else
  {
    vPrint("\nFinished calculating contour scores successfully.\n");
  }

  return success;
}


long vInterpreter1::ContourScoresFast(long test_input, long training_input, 
                                         long low_frame, long high_frame, 
                                         long training_frames, long grid_size,
                                         long subsampling,
                                         const char * temp_scores_file)
{
  // Get the inputs specified by the indices
  vVideoFile * temp_test_input = GetInput(test_input);
  if (temp_test_input == 0) return 0;
  vVideoFile * temp_training_input = GetInput(training_input);
  if (temp_training_input == 0) return 0;

  const char * test_file = temp_test_input->GetFilename();
  const char * training_file = temp_training_input->GetFilename();

  // Create a filename for the scores that is under the right directory
  char * scores_dir = vJoinPaths(sequences_directory, "glove_data\\trash");
  char * scores_file = vJoinPaths(scores_dir, temp_scores_file);
  vdelete2(scores_dir);

  ushort success = vContourScoresFast(test_file, low_frame, high_frame, training_frames,
                                       training_file, scores_file, grid_size, subsampling);
  vdelete2(scores_file);
  if (success == 0)
  {
    vPrint("\nError in processing contour scores of %s with %s\n",
            test_file, training_file);
  }
  else
  {
    vPrint("\nFinished calculating contour scores successfully.\n");
  }

  return success;
}


long vInterpreter1::ChamferScores0()
{
  long test_input, training_input, low_frame, high_frame;
  test_input = training_input = low_frame = high_frame = 0;
  char scores_file[500];
  vPrint("Enter test index, training_index, low frame, high frame, scores file.\n");
  vScan("%li %li %li %li %s\n", &test_input, &training_input, 
         &low_frame, &high_frame, scores_file);
  return ChamferScores(test_input, training_input, low_frame, high_frame,
                         scores_file);
}


long vInterpreter1::ChamferScores(long test_input, long training_input, 
                                      long low_frame, long high_frame, 
                                      const char * temp_scores_file)
{
  // Get the inputs specified by the indices
  vVideoFile * temp_test_input = GetInput(test_input);
  if (temp_test_input == 0) return 0;
  vVideoFile * temp_training_input = GetInput(training_input);
  if (temp_training_input == 0) return 0;

  const char * test_file = temp_test_input->GetFilename();
  const char * training_file = temp_training_input->GetFilename();

  // Create a filename for the scores that is under the right directory
  char * scores_dir = vJoinPaths(sequences_directory, "glove_data\\results");
  char * scores_file = vJoinPaths(scores_dir, temp_scores_file);
  vdelete2(scores_dir);

  ushort success = vMultipleChamferScores(test_file, low_frame, high_frame, 
                                             training_file, scores_file);
  vdelete2(scores_file);
  if (success == 0)
  {
    vPrint("\nError in processing contour scores of %s with %s\n",
            test_file, training_file);
  }
  else
  {
    vPrint("\nFinished calculating contour scores successfully.\n");
  }

  return success;
}


long vInterpreter1::BoundingBox0()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  vBoundingBox result = vBinaryBoundingBox(current);
  vPrint("\ncenter = (%.1lf, %.1lf), top = %li, bottom = %li, left = %li, right = %li, height = %li, width = %li\n",
          (double) (result.top + result.bottom + 0.0) / 2.0,
          (double) (result.left + result.right + 0.0) / 2.0,
          result.top, result.bottom, result.left, result.right, 
          result.bottom - result.top + 1, result.right - result.left + 1);
  return 1;
}

  
long vInterpreter1::Translate0()
{
  if (input == 0) return 0;
  vPrint("Enter rows and cols to translate by: ");
  long rows, cols;
  rows = cols = 0;
  vScan("%li %li\n", &rows, &cols);
  return Translate(rows, cols);
}


long vInterpreter1::Translate(long row_diff, long col_diff)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = function_translate_image0(current, row_diff, col_diff);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long vInterpreter1::ScaleImage0()
{
  if (input == 0) return 0;
  vPrint("Input scaling factor for binary image: ");
  double scaling = 0;
  vScan("%lf", &scaling);
  return ScaleImage(scaling);
}


long vInterpreter1::ScaleImage(double scaling)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = vScaleImage0(current, scaling);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long vInterpreter1::ResizeImage0()
{
  if (input == 0) return 0;
  vPrint("Input new rows and cols: ");
  long rows = 0, cols = 0;
  vScan("%li %li", &rows, &cols);
  return ResizeImage(rows, cols);
}


long vInterpreter1::ResizeImage(long rows, long cols)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  general_image * result = vResizeImage0(current, rows, cols);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long vInterpreter1::ScaleBoundingBox0()
{
  if (input == 0) return 0;
  vPrint("\nInput new bounding box size for binary image:\n");
  long new_size = 0;
  vScan("%li", &new_size);
  return ScaleBoundingBox(new_size);
}


long vInterpreter1::ScaleBoundingBox0b()
{
  if (input == 0) return 0;
  vPrint("\nInput new bounding box size, image rows, image cols:\n");
  long new_size = 0;
  long rows = 0;
  long cols = 0;
  vScan("%li %li %li", &new_size, &rows, &cols);
  return ScaleBoundingBox2(new_size, rows, cols);
}


long vInterpreter1::ScaleBoundingBox(long new_box_size)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * gray = current->ToGray();
  long center = new_box_size / 2;
  long rows = new_box_size;
  long cols = new_box_size;
  general_image * result = vScaleBoundingBox6(gray, new_box_size, center, center,
                                             rows, cols);
  if (gray != current) vdelete(gray);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}



long vInterpreter1::ScaleBoundingBox2(long new_box_size, long new_rows, long new_cols)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  double_image * the_image = new double_image (current);
  long center_row = (new_rows - 1) / 2;
  long center_col = (new_cols - 1) / 2;
  general_image * result = vScaleBoundingBox6(the_image, new_box_size, center_row, center_col,
                                             new_rows, new_cols);
  if (the_image != current) vdelete(the_image);
  if (result == 0) return 0;
  ShowImage(result);
  return 1;
}


long vInterpreter1::DistanceTransformSlow()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  v3dMatrix<long> * transform = vDistanceTransformSlow(current);
  ShowImage(transform);
  return 1;
}


long vInterpreter1::DistanceTransform()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  //v3dMatrix<long> * transform = vDistanceTransform(current);
  v3dMatrix<float> * transform = vCVDistanceTransform(current);
  ShowImage(transform);
  return 1;
}


long vInterpreter1::SetDistance0()
{
  if (input == 0) return 0;

  long input1 = 0;
  long input2 = 0;
  vPrint("Input indices for set distance: ");
  vScan("%li %li", &input1, &input2);
  return SetDistance(input1, input2);
}


long vInterpreter1::SetDistance(long input1, long input2)
{
  general_image * base1 = CurrentImage(input1);
  general_image * base2 = CurrentImage(input2);
  if (base1 == 0) return 0;
  if (base2 == 0) return 0;
  v3dMatrix<long> * distance = vSetDistance(base1, base2);
  ShowImage(distance);
  return 1;
}


long vInterpreter1::ChamferDistances0()
{
  vPrint("Test and training indices for Chamfer matching: ");
  long input1 = 0;
  long input2 = 0;
  vScan("%li %li", &input1, &input2);
  return ChamferDistances(input1, input2);
}


long vInterpreter1::ChamferDistances(long input1, long input2)
{
  general_image * base1 = CurrentImage(input1);
  general_image * base2 = CurrentImage(input2);
  if (base1 == 0) return 0;
  if (base2 == 0) return 0;
  grayscale_image * test = base1->ToGray();
  grayscale_image * training = base2->ToGray();
  grayscale_image * scaled_test = (grayscale_image *) vScaleBoundingBox6(test, 192, 128, 128, 256, 256);
  ShowImage(scaled_test);
    
  double distance = 0;
  long rows = 0;
  long cols = 0;
  double scaling = 0;

  vChamferDistances(scaled_test, training, &distance, &rows, &cols, &scaling);
  vPrint("distance = %lf, rows = %li, cols = %li, scaling = %lf\n",
          distance, rows, cols, scaling);
  long current = current_input_index;
  SetCurrentInput(input1);
  ScaleImage(scaling);
  Translate(rows, cols);
  SetCurrentInput(current);
  if (test != base1) vdelete(test);
  if (training != base2) vdelete(training);
  return 1;
}



long vInterpreter1::ChamferDistancesSlow0()
{
  vPrint("Test and training indices for Chamfer matching: ");
  long input1 = 0;
  long input2 = 0;
  vScan("%li %li", &input1, &input2);
  return ChamferDistancesSlow(input1, input2);
}


long vInterpreter1::ChamferDistancesSlow(long input1, long input2)
{
  general_image * base1 = CurrentImage(input1);
  general_image * base2 = CurrentImage(input2);
  if (base1 == 0) return 0;
  if (base2 == 0) return 0;
  grayscale_image * gray1 = base1->ToGray();
  grayscale_image * gray2 = base2->ToGray();

  double distance = 0;
  long rows = 0;
  long cols = 0;
  double scaling = 0;

  vChamferDistancesSlow(gray1, gray2, &distance, &rows, &cols, &scaling);
  vPrint("distance = %lf, rows = %li, cols = %li, scaling = %lf\n",
          distance, rows, cols, scaling);
  long current = current_input_index;
  SetCurrentInput(input1);
  ScaleImage(scaling);
  Translate(rows, cols);
  SetCurrentInput(current);
  if (gray1 != base1) vdelete(gray1);
  if (gray2 != base2) vdelete(gray2);
  return 1;
}


long vInterpreter1::ImageMoments()
{
  if (input == 0) return 0;

  general_image * current = input->current_image;
  long number;
  double * moments = vImageMoments(current, &number);
  long i;
  vPrint("Moments:\n");
  for (i = 0; i < number / 2; i++)
  {
    vPrint("%.7f ", moments[i]);
  }
  vPrint("\n");
  for (i = number / 2; i < number; i++)
  {
    vPrint("%.7f ", moments[i]);
  }
  vPrint("\n");

  vPrint("\n");
  vdelete2(moments);
  return 1;
}


long vInterpreter1::SequenceImageMoments_a()
{
  if (input == 0) return 0;
  const char * input_file = input->simple_name;
  char * simple_output_file = vMergeStrings2(input_file, ".moments");
  char * output_file = vJoinPaths4(sequences_directory, "glove_data", 
                                    "training", simple_output_file);
  vdelete2(simple_output_file);
  long success = SequenceImageMoments1(output_file);
  vdelete2(output_file);
  return success;
}


long vInterpreter1::SequenceImageMoments_b()
{
  if (input == 0) return 0;
  char buffer[500];
  vPrint("Input filename in which to save moments: ");
  vScan("%s", buffer);
  return SequenceImageMoments1(buffer);
}


long vInterpreter1::SequenceImageMoments1(const char * output_file)
{
  if (input == 0) return 0;
  const char * input_file = input->path_name;
  short success = vSequenceImageMoments_b(input_file, output_file);
  if (success > 0)
  {
    vPrint("Finished saving moments from %s to %s successfully", 
            input_file, output_file);
    return 1;
  }
  else
  {
    vPrint("Error in saving moments from %s to %s successfully", 
            input_file, output_file);
    return 0;
  }
}


long vInterpreter1::LeftHand()
{
  left_hand = 1;
  return 1;
}


long vInterpreter1::RightHand()
{
  left_hand = 0;
  return 1;
}


long vInterpreter1::TopBottom()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  general_image * top_bottom = wrim(the_image).TopBottom();

  ShowImage(top_bottom);
  return 1;
}


long vInterpreter1::DatabaseFrame0()
{
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view\n");
  vScan("%li %li", &frame, &view);
  return DatabaseFrame(frame, view);
}


// This one is for the old database
color_image * vInterpreter1::Training7Frame(long frame, long view)
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  color_image * the_image = input->sequence->ReadFrame(frame);
  if (the_image == 0) return 0;
  color_image * rotated = (color_image *) vRotateRight(the_image, view * 90);
  vdelete(the_image);

  return rotated;
}


// This one is for the poser1 database
color_image * vInterpreter1::Poser1Frame(long score_frame, long score_view)
{
  vFrameViewFinder finder;
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  long frame = 0, rotation = 0;
  float angle = 0;
  finder.ConvertToPoser1Pose(score_frame, score_view, 
                             &frame, &rotation, &angle);

  color_image * the_image = input->sequence->ReadFrame(frame);
  if (the_image == 0)
  {
    vPrint("failed to load frame %li\n", frame);
    return 0;
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  color_image * rotated = (color_image *) rotate_image_exact_degrees(the_image, rows/2, cols/2, angle);
  color_image * scaled = (color_image *) vScaleBoundingBoxf6(rotated, 192, 128, 128, 256, 256);
  vdelete(the_image);
  vdelete(rotated);
  return scaled;
}

  
color_image * vInterpreter1::ReadDatabaseFrame(long frame, long view)
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  color_image * rotated = 0;
  if (input->number_of_frames == 107328 / 4)
  {
    rotated = Training7Frame(frame, view);
  }
  else if (input->number_of_frames == 107328 / 48)
  {
    rotated = Poser1Frame(frame, view);
  }
  else 
  {
    vPrint("unexpected number of frames in database sequence: %li\n",
           input->number_of_frames);
    return 0;
  }
  return rotated;
}

  
long vInterpreter1::DatabaseFrame(long frame, long view)
{
  color_image * rotated = ReadDatabaseFrame(frame, view);
  if (rotated == 0)
  {
    return 0;
  }

  if (left_hand == 0)
  {
    ShowImage(rotated);
  }
  else
  {
    color_image * flipped = (color_image *) wrma(rotated).TopBottom_a();
    vdelete(rotated);
    ShowImage(flipped);
  }
  return 1;
}


long vInterpreter1::Pad()
{
  if (input == 0) return 0;
  long margin = 0;
  vPrint("\nEnter margin:\n");
  vScan("%li", &margin);

  general_image * current = input->current_image;
  general_image * padded = wrim(current).Pad4(margin, margin, margin, margin);
  ShowImage(padded);
  return 0;
}


long vInterpreter1::make_short()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  short_image* image = new short_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::make_long()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long_image* image = new long_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::make_ushort()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ushort_image* image = new ushort_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::make_ulong()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ulong_image* image = new ulong_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::make_float()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  float_image* image = new float_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::make_double()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image* image = new double_image (input->current_image);
  show_image (image);
  return 1;
}


long vInterpreter1::histogram_equalization_on()
{
  histogram_equalization_flag = 1;
  return 1;
}


long vInterpreter1::histogram_equalization_off()
{
  histogram_equalization_flag = 0;
  return 1;
}


long vInterpreter1::histogram_equalization_size ()
{
  long window_size = 0;
  print ("\nenter window size to be used for histogram equalization:\n");
  question ("%li", &window_size);
  if (window_size <1)
  {
    print ("the window size should be at least 1\n");
    return 0;
  }

  if (window_size % 2 != 1)
  {
    print ("the window size should be odd\n");
    return 0;
  }

  equalization_window_size = window_size;
  return 1;
}


long vInterpreter1::histogram_equalization ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  ushort_image* current = new ushort_image (input->current_image);
  grayscale_image * result = function_histogram_equalization (current, equalization_window_size);
  function_delete (current); 
  show_image (result);
  return 1;
}


long vInterpreter1::histogram_equalization_offset ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long vertical_offset = 0, horizontal_offset = 0;
  print ("\nenter vertical offset, horizontal offset:\n");
  question("%li %li", & vertical_offset, & horizontal_offset);

  ushort_image* current = new ushort_image (input->current_image);
  grayscale_image * result = function_histogram_equalization_offset (current, equalization_window_size, 
                                                            vertical_offset, horizontal_offset);
  function_delete (current); 
  show_image (result);
  return 1;
}


long vInterpreter1::histogram_equalization_weights_test ()
{
  if (input == 0)
  {
    return 0;
  }

  float_image * weights = histogram_equalization_weights(equalization_window_size);
  show_image(weights);
  return 1;
}


long vInterpreter1::toggle_preserve_current_frame()
{
  preserve_current_frame = 1 - preserve_current_frame;
  return 1;
}


long vInterpreter1::current_percentage()
{
  if (input == 0)
  {
    return 0;
  }
  if (input->sequence == 0)
  {
    return 0;
  }

  if (input->number_of_frames <= 1)
  {
    return 0;
  }

  float resultf = ((float) input->current_frame) / ((float) input->number_of_frames - 1) * 100.f;
  long result = round_number(resultf);
  return result;
}


long vInterpreter1::SetCurrentCounterPercent(double percentage)
{
	if (input == 0) return 0;
	double max_frame = input->number_of_frames - 1;
	long frame = round_number((max_frame * percentage) / 100.0);
	input->SetCurrentFrame(frame);

  return 0;
}


long vInterpreter1::match_window_size()
{
  if ((input == 0) || (input->sequence == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long rows = input->current_image->vertical();
  long cols = input->current_image->horizontal();
  rows = rows + 150;
  cols = Max(cols, (long) 700);

  window->Resize(rows, cols);
  return 1;
}


long vInterpreter1::AddInputSilent(const char * filename)
{
  if (filename[0] == 0) return 0;
  vVideoFile * new_video_file;
  new_video_file = vOpenReadVideoFile(filename);

  if (new_video_file == 0)
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  class_sequence_information * info = new class_sequence_information(buffer_size);
  info->sequence = new_video_file;
  info->path_name = vCopyString(new_video_file->GetFilename());
  info->simple_name = vSimpleName(info->path_name, platform_directory_slash);
  info->number_of_frames = new_video_file->NumberOfFrames();
  //info->magnified_rows = new_video_file->Rows();
  //info->magnified_cols = new_video_file->Cols();
  inputs.push_back(info);

  number_of_inputs++;

  SetCurrentInputSilent(number_of_inputs - 1);
  return 1;
}


long vInterpreter1::set_frame_rate()
{
  long rate = 0;
  vPrint("\nenter frame rate:\n");
  vScan("%li", &rate);
  if ((rate < 1) || (rate > 300))
  {
    vPrint("rate must be between 1 and 300\n");
  }
  else
  {
    frame_rate = rate;
  }

  return 1;
}


long vInterpreter1::get_frame_rate()
{
  return frame_rate;
}


general_image * vInterpreter1::current_image ()
{
  class_sequence_information * sequence = current_sequence ();
  if (sequence == 0)
  {
    return 0;
  }

  return sequence->current_image;
}


class_sequence_information * vInterpreter1::current_sequence ()
{
  return input;
}


long vInterpreter1::set_buffer_size()
{
  if (input == 0)
  {
    return 0;
  }

  long new_size = 0;
  print("\nenter new buffer size:\n");
  question("%li", & new_size);

  if ((new_size <= 0) || (new_size > 50))
  {
    print("acceptable sizes are between 1 and 50\n");
    return 0;
  }

  input->SetImageBufferSize(new_size);
  return 1;
}


long vInterpreter1::set_default_buffer_size()
{
  long new_size = 0;
  print("\nenter new default buffer size:\n");
  question("%li", & new_size);

  if ((new_size <= 0) || (new_size > 50))
  {
    print("acceptable sizes are between 1 and 50\n");
    return 0;
  }

  default_buffer_size = new_size;
  return 1;
}


long vInterpreter1::check_input()
{
  if (input == 0)
  {
    default_input(100,100);
  }

  if (input == 0)
  {
    exit_error("\nerror: check_input failed\n");
  }

  return 1;
}

