#ifndef VASSILIS_INTERPRETER2_H
#define VASSILIS_INTERPRETER2_H


#include "interpreter1.h"
#include "contour.h"
#include "Camera.h"
#include "particle_tracker.h"
#include "fingers.h"
#include "hand_database.h"
#include "trajectory.h"

class vInterpreter2 : public vInterpreter1
{
protected:
  // moment_index is the index of the shape moment that we want to
  // use for matching.
  long moment_index;
  long reference_index;
  long reference_start;
  long embedding_dimensions;
  long ten_matches_index;
  // default threshold to use for the Canny edge detector in 
  // finding similar hand images.
  long canny_threshold;

  // step1_views indicates how many database views we should keep 
  // after the first step in the retrieval process (used in RankViews)
  long step0_views;
  long step1_views;

  long database_input;
  long database_edge_input;
  long dolvs_start;
  long dolvs_period;
  vLineSegmentLong palm_cutoff;
  vLineSegmentLong centered_palm_cutoff;

  // segmentation_method is 0 if we want to segment just the hand bounding
  // box, or 1 if we want to segment the hand in more fine detail.
  long segmentation_method;
  
  // online_segmentation specifies what type of segmentation should be 
  // done for the online demo. online_ranking specifies how to do the
  // retrieval.
  long online_segmentation;
  long online_ranking;
  long skin_threshold;
  long top_to_print;

  double grid_fraction;

  char * moments_file;

  vMatrix<double> * training_moments;
  vMatrix<double> * training_edge_histograms;

  v3dMatrix<float> * training_embeddings;
  v3dMatrix<float> * chamfer_index_im;
  v3dMatrix<float> * reference_images;
  v3dMatrix<float> * im_reference_images;
  v3dMatrix<float> * test_embedding;

  vPCA * pca;
  vSequenceMatchStats * sequence_match_stats;
  vSequenceFingerInfo * finger_info;
  vCombinedScores * combined_scores;
  vEdgeStatistics * edge_statistics;

  CCamera camera;
  class_sequence_information * camera_input;
  vHandShapeTracker * hand_tracker;
  vMatchScores * tracking_scores;

  std::vector<vStraightLinks> * model_straight_links;
  std::vector<vLineSegmentMatch> * line_segments;
  v3dMatrix<uchar> * all_model_lines;
  vDOLVS_Histogram * existential_DOLVS;
  vMatrix<long> dolvs_pdf;
  vLineMatchStats * line_match_stats;
  vLineMatchFrameStats * line_match_frame_stats;
  vPointSetSequence * point_sequence;
  
  v3dMatrix<uchar> * palm_cutoffs;
  v3dMatrix<uchar> * extended_cutoffs;
  vLineGrid * line_grid;

  v3dMatrix<uchar> * all_im_distances;
  v3dMatrix<uchar> * all_mi_distances;
  v3dMatrix<uchar> * all_cd_distances;

  char * im_distances_file;
  char * mi_distances_file;
  char * cd_distances_file;
  char * cd_pair_file;

  v3dMatrix<long> * cd_pairs;
  v3dMatrix<double> * cd_pair_distances;
  vOldDatabase * database;

  vMatchScores * previous_scores;
  long previous_score_frame;
  // In generating possible trajectories, trajectory_cutoff specifies
  // how many estimates from each vMatchScores we should use. The 
  // total number of trajectories is at most the square of
  // trajectory_cutoff.
  long trajectory_cutoff;
  long scores_to_hold;
  long scores_to_use;
  vScoresHolder * scores_holder;
  vTrajectoryScores * trajectory_scores;
  vector<vPoint2> features_to_track;

  // The original image, corrected so that the hand is a right hand,
  // before hand segmentation. This is useful
  // when we want to extract the edge image. We want to keep 
  // edges from the original image, and we don't want artificial
  // edges that show up at the boundary between the segmented image
  // and the background.
  color_image * right_hand_frame;
  
  // original_video_frame is the same as original_frame when 
  // left_hand is 0 (i.e. when we are dealing with a right-hand
  // image). When left_hand is 1 then original_video_frame is 
  // the frame as loaded from the video file (or captured from
  // the camera) and original_frame is the upside-down mirror version
  // of original_video frame.
  color_image * original_video_frame;

  // The segmented hand image, corrected so that the hand is a right hand.
  color_image * right_hand_image;

  // As in original_video_frame, this is the hand image as seen
  // in the actual movie file or camera.
  color_image * original_hand_image;

  // The mask of the segmented hand image
  grayscale_image * hand_mask;

  grayscale_image * hand_edge_image;
  v3dMatrix<double> * hand_orientation_image;
  grayscale_image * hand_edge_scaled;
  v3dMatrix<double> * hand_orientation_scaled;
  vArray(v3dMatrix<float> *) distance_transforms;

  // last file to which we saved or from which we loaded sequence_match_stats
  char * seq_stats_file;

public:
  vInterpreter2(class_interpreter* in_interpreter);
  ~vInterpreter2();

  virtual void Initialize(vGestureInterfaceGlobals * in_globals);
  virtual void CleanUp();

  long ImageMax0();
  long ImageMin0();
  long BandMin0();
  long BandMin(long band);
  long BandMax0();
  long BandMax(long band);
  long CopyBand0();
  long CopyBand(long band);

  long CountValue0();
  long CountValue(double value);
  long CountLequal0();
  long CountLequal(double value);
  long CountGequal0();
  long CountGequal(double value);

  long OuterContour();
  long SmoothOuterContour0();
  long SmoothOuterContour(long neighborhood);
  long ContourRegions0();
  long ContourRegions(long neighborhood, double threshold);
  long ContourCuts0();
  long ContourCuts(long neighborhood, double threshold);
  long ContourMaxima0();
  long ContourMaxima(long neighborhood, double threshold);
  long ElongationStatistics0();
  long GrowSkeleton();
  long FindFingers0();
  long FindFingers1(double elongation_threshold);
  long FindFingers2(double elongation_threshold, vector<vElongation> * elongations);
  
  long SaveFingerInfo0();
  long SaveFingerInfo(long start_frame, long end_frame);
  long LoadFingerInfo();
  long FingerScores();
  long FingerScore();
  long FilterScoresUsingFingers0();
  // We go through scores and filter them using another set
  // of scores, obtained by calling FingerScores. We discard
  // scores in the original set of scores whose rank is after
  // the given threshold.
  long FilterScoresUsingFingers(long threshold);
  long FilterScoresUsingLogs0();
  long FilterScoresUsingLogs(long threshold, double w0, double w1, 
                             double w2, double w3, long add);


  long FillRegion0();
  long FillRegion(long row, long col, uchar value);
  long RandomFrame();
  long ToGray();
  long ToGrayBound();

  long DefaultMomentsFile();
  long InputMomentsFile();
  long SetMomentsFile(const char * new_file);

  long MomentScores();
  long SetMomentIndex0();
  long SetMomentIndex(long index);

  long InputPCAFile();
  long SetPCAFile(const char * new_file);
  long InputPCAUsedDimensions();
  long SetPCAUsedDimensions(long new_dimensions);
  long PCAMomentScores();
  long PCAScoreStats();

  long FilterScoresUsingMoments0();
  // We go through scores and filter them using another set
  // of scores, obtained by calling PCAMomentScores. We discard
  // scores in the original set of scores whose rank is after
  // the given threshold.
  long FilterScoresUsingMoments(long threshold);

  // Draws a line in the direction of the pixel gradient.
  long PixelGradient0();
  long PixelGradient(long row, long col);

  // Takes the current image, finds the edges, and for each
  // out of every subsampling pixels in the edge image it 
  // shows the gradient direction in the original images.
  // Gradients are drawn onto the edge image.
  long EdgePixelGradients0();
  long EdgePixelGradients(long subsampling);
  long EdgeHistogram0a();
  long EdgeHistogram0b();
  long EdgeHistogram3a(long bins, long low, long high);
  long EdgeHistogram3b(long bins, long source_seq, long edge_seq);
  
  // Used to compare histogram of a test image with histogram of a training
  // image.
  long CompareEdgeHistograms0();
  long CompareEdgeHistograms(long bins, long test_input, 
                             long training_input, long training_edges);
  
  // Calls vEdgeHistogram::SaveHistograms, with the files corresponding
  // to the inputs indexed by original and edges.
  long SaveEdgeHistograms0();
  long SaveEdgeHistograms(long bins, long original, long edges);

  long LoadEdgeHistograms();
  long MinEdgeHistogramScores();
  long MeanEdgeHistogramScores();
  long SaveEdgeHistogramScores0();
  long SaveEdgeHistogramScores(const char * base, long start, long end);

  long FrameMatchStats0();
  long FrameMatchStats1(long correct, long ground);
  long FrameMatchStats2(long start, long end, long period);
  long SequenceMatchStats0();
  long SequenceMatchStats(const char * score_script, long correct, 
                          long ground, long start_frame, long end_frame, long period);
  long TrackMatchStats0();
  long TrackMatchStats(const char * score_script, long correct, 
                       long ground, long start_frame, long end_frame, long period);
  long SequenceCorrectMean0();
  long SequenceCorrectMean(long rank);
  long SequenceCorrectMedian0();
  long SequenceCorrectMedian(long rank);
  long SequenceGroundMean0();
  long SequenceGroundMean(long rank);
  long SequenceGroundMedian0();
  long SequenceGroundMedian(long rank);
  long SequenceCorrectUnder0();
  long SequenceCorrectUnder(long rank);
  long SequenceCorrectUnder0b();
  long SequenceCorrectUnder2(long correct_index, long rank);
  long SequenceCorrectWithin0();
  long SequenceCorrectWithin(long index, long low_rank, long high_rank,
                             long low_distance, long high_distance);
  long SequenceGroundUnder0();
  long SequenceGroundUnder0b();
  long SequenceGroundUnder(long rank);
  long SequenceGroundUnder2(long ground_index, long rank);
  long SequenceSelectAll();
  long SequenceSelectViewpoint0();
  long SequenceSelectViewpoint(double x, double y, double tolerance);
  long SequenceSelectFrontDistance0();
  long SequenceSelectFrontDistance(double distance, double tolerance);
  long SequenceSelectRange0();
  long SequenceSelectRange(long start, long end);
  long SequenceSelectRangeDistance0();
  long SequenceSelectRangeDistance(long start, long end, 
                                   double distance, double tolerance);

  long PrintSequenceStats();
  // Prints statistics obtained with SequenceCorrectWithin.
  long PrintGroundDistanceStats();
  long SaveSequenceStats0();
  long SaveSequenceStats(const char * filename);
  long LoadSequenceStats0();
  long LoadSequenceStats(const char * filename);

  long LogScoreStats0();
  long LogScoreStats(long correct, long ground, long start, long end,
                     double w0, double w1, double w2, double w3, long add);

  long RootScoreStats0();
  long RootScoreStats(long correct, long ground, long start, long end,
                      double w0, double w1, double w2, double w3);

  long SumScoreStats0();
  long SumScoreStats(long correct, long ground, long start, long end,
                     double w0, double w1, double w2, double w3);

  long GroundTruthDistances();
  long SaveCombinedScores();
  long SaveCombinedScoresOld();
  long LoadCombinedScores();
  long LoadCombinedScoresOld();
  long CombineLogs0a();
  long CombineLogs(double w0, double w1, double w2, double w3, double w4, double w5);
  long FilterCombineLogs0();
  long FilterCombineLogs(long number, double w0, double w1, double w2, double w3, double w4, double w5);
  long CombineLogs0b();
  long CombineLogs2(double w0, double w1, double w2, double w3, long add);
  long CombineRoots0();
  long CombineRoots(double w0, double w1, double w2, double w3);
  long CombineSums0();
  long CombineSums(double w0, double w1, double w2, double w3);
  long CombineWithLogs0();
  long CombineWithLogs(double w0, double w1, double w2, double w3, double w4);

  // Save chamfer distances from all database frames to random database frames
  // into a file
  long RandomChamferDistances0();
  long RandomChamferDistances(long number, 
								              long training_frames);
  // Merges together individual chamfer distance files created by
  // random_chamfer_distances
  long MergeRandomDistances0();
  long MergeRandomDistances(long training_input, long start, long end);
  // Loads in a big file generated by MergeRandomDistances (and 
  // therefore by RandomChamferDistances)
  long LoadRandomEmbedding();

  // Loads in chamfer distances between reference images and all
  // database images, assuming that reference_images has been
  // initialized.
  long LoadTrainingEmbedding();

  // Computes distances between input and all database images by
  // computing the chamfer distance between input and picks, and
  // comparing the resulting vector with the vectors of the database
  // images
  long ChamferDistances();
  // Sets distance index for single pick scores
  long SetDistanceIndex0();
  long SetDistanceIndex(long new_index);
  long SinglePickScores0();
  // Sets the starting index and number of frames to use for 
  // the database picks. This way, we can save, say, scores
  // for 1000 picks, but only use 200 of them
  long SetRandomDimensions0();
  long SetRandomDimensions(long start, long dimensions);
  // Compute database scores, assuming chamfer distances have
  // already been computed for the current input frame
  long MultiPickScores0();
  // Combines ChamferDistances and MultiPickScores
  long ChamferIndexScores0();
  long ChamferIMIndexScores();

  // RankViews ranks the views of the database in order of similarity
  // to the current image. This is done online, no precomputed info
  // about the current image is used.
  long RankViews();
  long RankViews2();
  long RankViews3();
  long RankViews4();
  long RankViews5();
  long RankViews6();
  long RankViews6b();
  long RankViews7();
  long RankViews8();
  long RankViews9();
  long RankViews10();
  long RankViews11();
  long RankViews12();
  long RankViews13();
  long RankViews14();
  long RankViews15();
  long RankViews16();
  long RankViews17();
  long RankViews18();
  long RankViews19();
  long RankViews20();
  long RankViews21();
  long RankViews22();
  long RankViews23();
  long RankViews24();
  long RankViews25();
  long RankViews26();
  long RankViews27();
  long RankViews28();
  long RankViews29();
  long RankViews30();
  long RankViews31();

  // For GW2003
  long RankViews32();
  long RankViews33();

  long ShowTopTenMatches();
  long ShowTenMatches0();
  // Shows the ten matches ranked start, start+1, start+2 etc.
  long ShowTenMatches(long start);
  long ShowNextTenMatches();
  long ShowPreviousTenMatches();
  long ViewParams0();
  long ViewParams(long frame, long rotation);
  long SetStep0Views0();
  long SetStep0Views(long in_step1_views);
  long SetStep1Views0();
  long SetStep1Views(long in_step1_views);

  long IsCameraInitialized();
  long IsCameraRunning();
  long CheckCamera();
  long StartCamera();
  long StopCamera();
  long GrabFrame();
  long SetCameraFormat();
  long SetCameraOptions();
  long CameraInputIndex();

  v3dMatrix<long> * CameraHand();
  long ShowCameraHand();
  long ShowCameraHand2();

  long SegmentHand_A();
  long SegmentHand_B();
  long SegmentHand_C();
  long PalmCutpoint();
  long KeepPalm();
  long PalmCutpoint2();
  long KeepPalm2();
  long KeepPalm3();
  long KeepPalm4();
  
  long ColorDistances0a();
  long ColorDistances0b();
  long ColorDistances1a(long row, long col);
  long ColorDistances1b(uchar r, uchar g, uchar b);
  long ConnectedComponents8();
  long ShadowScore();
  long LocalShadowScore0();
  long LocalShadowScore(long row, long col);

  long EdgeOrientations();
  long EdgeOrientations2();
  long NonMaximaSuppression();
  long IntensityMaxima();
  long GradientNorms();
  long PrincipalOrientation();
  long PETS_Classify();
  long OldViewDistance0();
  long OldViewDistance(double x1, double y1, double z1, 
                    double x2, double y2, double z2);
  long OldViewDistance2();
    
  long SaveEdgeStatistics0();
  long SaveEdgeStatistics(long original_input, long edge_input,
                          long spatial_subsampling, long pixel1_bins,
                          long pixel2_bins, long pixel1_start,
                          long pixel1_end, long frames_to_process);
  long LoadEdgeStatistics0();
  long EdgeStatisticsVote0a();
  long EdgeStatisticsVote(long original_input, long edge_input);
  long EdgeStatisticsVote0b();
  long EdgeStatisticsVote2(long original_input, long edge_input);
  long EdgeStatisticsVote0d();
  long EdgeStatisticsVote4(long original_input, long edge_input);

  // Uses the current match scores to initialize hand tracker
  long InitializeHandTracker();

  long HandTrackerPropagate();
    
  // Uses the current match scores to update hand tracker
  long UpdateHandTracker();

  // Gets match scores based on the current tracker estimates.
  long HandTrackerScores();

  // Save non-zero image points in a MATLAB-readable file
  long SavePoints0();
  long SavePoints(long number, const char * name, const char * file);
  long CannyThreshold0();
  long CannyThreshold(long new_threshold);
  long SetGroundTruthView0();
  long SetGroundTruthView(long frame, long view);

  // Compute the distance in camera parameter space between 
  // database images (frame1, view1) and (frame2, view2)
  long MatchDistance0();
  long MatchDistance(long frame1, long view1, long frame2, long view2);
  long SetGroundTruthDistance0();
  long SetGroundTruthDistance(double in_ground_truth_distance);

  long OldRotationDistance0();
  long OldRotationDistance(double theta1, double x1, double y1,
                        double theta2, double x2, double y2);

  long ChamferIndexScoresb0();
  long ChamferIndexScoresb(long dimensions, long times);

  long SetDatabaseInputs0();
  long SetDatabaseInputs(long in_database_input, long in_database_edge_input);

  long CopyInput0();
  long CopyInput(long input);
  long Overlay0();
  long Overlay(long input1, long input2);

  long Randomize0();
  long Randomize1();

  long OrientedLines0();
  long OrientedLines0b();
  long OrientedLines(double orientation, long length, long threshold);
  long OrientedLines2(double orientation, long length, long threshold);
  long OrientedLines0c();
  long OrientedLines3(double orientation, long length, long 
                      threshold, double tolerance);
  long OrientedLines0d();
  long OrientedLines4(double orientation, long length, long 
                      threshold, double tolerance);

  long DilateImage();
  long LinkBoundaries();
  long FingerContours();
  long ContourLineVotes();
  long FingerLines();
  long FingerLines2();

  long SelectRange0();
  long SelectRange(long band, double low, double high);
  long LineEndpoints0();
  long LineEndpoints(long row, long col, double orientation);
  long FitEllipse0();
  long FitEllipse();
  long FitEllipse3(long band, long low, long high);
  long LinkIDs();
  long StraightLinks();
  // The next version of straight links (the next two functions)
  // is better than the previous function.
  long StraightLinks0();
  long StraightLinks2(long frame, long view);
  vLineSegment GetModelLine(long link_id, long line);
  long DrawModelStraightLinks();

  long CircularDistance0();
  long HalfPlane0();

  long FitLine0();
  long FitLine(long method);
  long LineSegmentMatches0();
  long LineSegmentMatches(double angle, long length, 
                          long threshold, double tolerance);

  long CountNonZero();
  long TestLinePoints0();
  long TestLinePoints(long row, long col, double angle, long length);
  long TestSearchRegion0();
  long TestSearchRegion(double orientation, double high_length, 
                        double angle_tolerance, long distance_tolerance);
  long OrientedLinesD0();
  long OrientedLinesD(double orientation, double length, 
                      long distance_tolerance, double angle_tolerance);

  // A better version of GetLineSegments2 (bugs corrected)
  long GetLineSegments0();
  long GetLineSegments(long orientations, double low_length, double high_length,
                       long lengths, double angle_tolerance, long distance_tolerance);

  // A modified version of GetLineSegments. Here, the segments are normalized,
  // according to a scaling and translation that moves the hand image to the central
  // 192x192 square of a 256x256 image.
  long GetLineSegments03();
  long GetLineSegments3(long orientations, double low_length, double high_length,
                       long lengths, double angle_tolerance, long distance_tolerance);

  // Yet another modified version of GetLineSegments. This is almost like
  // GetLineSegments3, but adjusted for the fact that sometimes we only care
  // about a subarea of the image. For example, this comes up when we want to
  // segment a bounding box corresponding to the hand from the rest of the image.
  // If we just put zeros for the background, then the sides of the bounding box
  // will yield strong edge pixels, but that will just be an artifact of the 
  // segmentation.
  long GetLineSegments04();
  long GetLineSegments4(long orientations, double low_length, double high_length,
                        long lengths, double angle_tolerance, long distance_tolerance);

  // Here we order matches by sorting them separately based on their length, V, S,
  // and based on their D, O, L with respect to the specified model line.
  long MatchLineSegment0();
  long MatchLineSegment(long link_id, long line);

  // Here we order matches based on their frequency in a DOLVS histogram built
  // using the specified model line.
  long MatchLineSegment02();
  long MatchLineSegment2(long link_id, long line);

  // Here we order matches based on their existential frequencies in 
  // the existential_DOLVS histogram which has been computed using
  // a sampling of possible model lines.
  long MatchLineSegment03();
  long MatchLineSegment3(long link_id, long line);

  // Here we order matches based on their existential frequencies in 
  // the existential_DOLVS histogram which has been computed using
  // a sampling of possible model lines. However, the score of each
  // segment is not the frequency, but its corresponding probability
  // in the dolvs_pdf table.
  long MatchLineSegment04();
  long MatchLineSegment4(long link_id, long line);

  // Produce the two images of the top k line segments 
  // (one image shows all the info, another shows the rank).
  long LineSegmentImages0();
  long LineSegmentImages(long k);

  // Count line segments whose dolvs are strictly better (or worse) than 
  // the ones given here
  long CountLineSegments0();
  long CountLineSegments(long score, long d, long o, long l, 
                         long v, long s, long direction);

  // Same as before (CountLineSegments), but here we erase all 
  // line segments that do not fit the criteria.
  long SelectLineSegments0();
  long SelectLineSegments(long score, long d, long o, 
                          long l, long v, long s, long direction);

  long DrawLineSegment0();
  long DrawLineSegment(long rank);

  long AllModelLines0();
  long AllModelLines(long database_images, long database_edges);

  long SaveModelLines0();
  long SaveModelLines(long database_images, long database_edges);

  long LoadModelLines0();
  long LoadModelLines(long database_input);

  long ReadModelLines0();
  long ReadModelLines(long frame, long view);
  long CountAllModelLines0();
  long CountAllModelLines(long start, long period);

  long ExistentialDOLVS0();
  long ExistentialDOLVS(long start, long period);

  long ExistentialDOLVS02();
  long ExistentialDOLVS2(long start, long period);

  long DOLVS_PDF0();
  long DOLVS_PDF2();
  long DOLVS_To_PDF();

  long DOLVS_Stats();
  long DOLVS_Stats2();
  long SaveDOLVSStats0();
  long SaveDOLVSStats(long start, long period, long start2, long period2, long matches);

  long SaveDOLVSFrameStats0();
  long SaveDOLVSFrameStats(long start, long period, long matches);

  long SaveDOLVS0();

  long SaveLineMatchFrameStats0();
  long SaveLineMatchFrameStats(long start, long period, long matches);

  long LoadFrameDOLVS0();
  long LoadFrameDOLVS(long start, long period);

  // Compares a DOLVS saved to disk to existential_DOLVS. Used to verify
  // that there are no bugs in the saving/loading process.
  long CompareDOLVS0();
  long CompareDOLVS(long start, long period);

  long LoadLineMatchFrameStats0();
  long LoadLineMatchFrameStats(long start, long period);

  long MatchModelLines0();
  long MatchModelLines(long frame, long view);

  long MatchModelLines0b();
  long MatchModelLines2(long frame, long view, double fraction);

  // Match model lines for the model image that is found
  // by CorrectMatch(rank).
  long MatchCorrectLines0();
  long MatchCorrectLines(long rank);

  // Match model lines for the model image that is found
  // by GroundMatch(rank).
  long MatchGroundLines0();
  long MatchGroundLines(long rank);

  // Match model lines for the model image that is found
  // by GotoMatch(rank).
  long MatchMatchLines0();
  long MatchMatchLines(long rank);

  long CompareMatchModelLines0();
  long CompareMatchModelLines(long start, long period, long frame, long view);

  long LineMatchStats0();
  long LineMatchStats(long number);
  long LineMatchScores0();

  long DOLVS_Bin0();
  long DOLVS_Bin(long d, long o, long l, long v, long s);

  long DOLVS_Entry0();
  long DOLVS_Entry(long d, long o, long l, long v, long s);

  long DOLVS_Value0();
  long DOLVS_Value(long value);

  long DOLVS_Lequal0();
  long DOLVS_Lequal(long value);

  long DOLVS_Gequal0();
  long DOLVS_Gequal(long value);

  long PDF_Entry0();
  long PDF_Entry(long frequency);
  long PDF_Probability0();
  long PDF_Probability(long frequency);

  long TestLineSegmentSamples0();
  long TestLineSegmentSamples(long location_diff, long orientation_diff, 
                              long length_diff);
  long SampleLineSegments0();
  long SampleLineSegments(long location_diff, long orientation_diff, 
                          long length_diff);


  long FindSegment0();
  long FindSegment(long row, long col, long o, long l);

  long AverageLineMatchScore0();
  long AverageLineMatchScore(long tries);

  long CountBestSegments0();

  long SavePointSet();
  long LoadPointSet();
  long TestPointSet0();
  long TestPointSet(long frame);
  long SaveLeftoverSet();
  long TestLeftoverSet0();
  long TestLeftoverSet(long frame);


  long ChamferTriangleInequality();
  long ChamferTriangleInequality2();
  long OneWayChamfer();
  long OneWayChamfer2();
  long TestOneWayChamfer();
  long Chamfer1TopScores0();
  long Chamfer1TopScores(long number);

  // The next functions are used to compute and display the 
  // model leftover points, i.e. points that were not used
  // for the fitting of lines.
  long ShowLeftoverModelPoints0();
  long ShowLeftoverModelPoints(long frame, long view);
  long ShowLeftoverTestPoints0();
  long ShowLeftoverTestPoints(vector<long> * segments);

  long LineChamferScores0();
  long LineChamferScores(long number);

  // ChamferIndexOutliers is used to give results to George Kollios 
  // for a publication. He wants the results of non-hand images
  // using the chamfer indexing scheme.
  long ChamferIndexOutliers0();
  long ChamferIndexOutliers(long start);

  long SubtractBackground0();
  long SubtractBackground(double threshold);

  // SelectPixels uses the current image as a binary mask, and 
  // then applies this mask to the image in the ring buffer that
  // is at the selected index, so that the product of the mask
  // and the buffer image is displayed.
  long SelectPixels0();
  long SelectPixels(long index);

  // SelectBBox uses the current image as a binary mask, finds the
  // bounding box of the non-zero area, and 
  // then selects the corresponding pixels (that belong to that box)
  // from the image in the ring buffer that
  // is at the selected index.
  long SelectBoundingBox0();
  long SelectBoundingBox(long index);

  long FillUpHoles();
  long DrawProjections();

  long SaveChamfer1Index();
  long ChamferMatching0();
  long ChamferMatching(long input2);
  long TestChamfer1Index();

  // Sets the prefered method of hand segmentation to
  // bounding box segmentation
  long BoundingBoxSegmentation();
  // Sets the prefered method of hand segmentation to
  // fine segmentation
  long FineSegmentation();

  // This function assumes that the currently displayed imagea
  // is a binary image indicating the location of the segmented 
  // hand. Based on that image, it uses the current frame
  // of the video sequence to extract the hand pixels (producing
  // a segmented hand image), and also to extract the edge image
  // of the hand. In the edge image it artificially adds a line
  // for the palm cutoff point (which, this function assumes, is
  // stored in the palm_cutoff member variable of vInterpreter2).
  long HandAndEdgeImages();

  long SetEdgeImage();

  // A way to find the hand based on skin detection
  long SkinHand();

  long LineNumberStats0();
  long LineNumberStats(long number_of_views);

  long StartLabelingCutoffs();
  long StopLabelingCutoffs();
  long ProcessCutoffClick(long row, long col);
  long LoadCutoffs();
  long DrawCutoffLine();
  long DrawCurrentCutoff();
  long NextCutoff();
  long PreviousCutoff();

  long Diameter();
  long ExtendCutoffs();
  long LoadExtendedCutoffs();
  long DrawExtendedCutoff0();
  long DrawExtendedCutoff(long view);
  long PalmScores();
  long TestCutoffCentering();

  long TestLineGrid0();
  long TestLineGrid(long d_samples, long o_samples, long l_samples);
  long GridFraction0();
  long GridFraction(double new_grid_fraction);

  long DrawLineMatch();
  long FrontDistance();

  // The following functions are used to write the GW2003 paper
  long Save_All_CD_Distances();
  long Save_All_IM_Distances();
  long Save_All_MI_Distances();
  long Load_All_CD_Distances();
  long Load_All_IM_Distances();
  long Load_All_MI_Distances();
  long Show_CD_Entry();
  long Show_IM_Entry();
  long Show_MI_Entry();
  long TestIndexAdjustment();

  long Show_CD_Entry4();
  long ChamferMatching40();
  long ChamferMatching4(long frame1, long view1, long frame2, long view2);

  long Make_CD_Pairs();
  long Save_CD_Pairs();
  long Load_CD_Pairs();
  long Show_CD_Pair();

  long CalculateSTD();
  long CalculateSTDs();
  long ChooseBasis1();
  long ChooseBasis2();
  long ChooseBasis3();
  long ChooseBasis4();
  long ChooseBasis5();
  long ChooseBasis6();
  long ChooseBasis7();

  long MakeLipschitzBasis();
  long SaveReferenceDistances();
  // End of functions used for GW2003

  // Samples points from the surface of a sphere, shows their number.
  // Points are represented by 3 coordinates (in 3D).
  long OldSampleSpherePoints();

  // OldSampleSpherePoints2 points from the surface of a sphere, prints the points
  // and their number.
  // Points are represented by 3 coordinates (in 3D).
  long OldSampleSpherePoints2();

  // OldSampleViewpoints points from the surface of a sphere, shows their number.
  // Points are represented by 2 coordinates (on sphere surface).
  long OldSampleViewpoints();

  // OldSampleViewpoints2 points from the surface of a sphere, prints the points
  // and their number.
  // Points are represented by 2 coordinates (on sphere surface).
  long OldSampleViewpoints2();

  // OldSampleViewpoints3 points from the surface of a sphere, prints the points.
  // Points are represented by 2 coordinates (on sphere surface).
  // Same as OldSampleViewpoints2, but the printing format here is 
  // different (appropriate to be saved as a text file, to be used
  // by save_views).
  long OldSampleViewpoints3();

  // PoserSampleViewpoints points from the surface of a sphere, shows their number.
  // Points are represented by 2 coordinates (on sphere surface).
  long PoserSampleViewpoints();

  // PoserSampleViewpoints2 points from the surface of a sphere, prints the points
  // and their number.
  // Points are represented by 2 coordinates (on sphere surface).
  long PoserSampleViewpoints2();

  // PoserSampleViewpoints3 points from the surface of a sphere, prints the points.
  // Points are represented by 2 coordinates (on sphere surface).
  // Same as PoserSampleViewpoints2, but the printing format here is 
  // different (appropriate to be saved as a text file, to be used
  // by save_views).
  long PoserSampleViewpoints3();

  // My algorithm
  long EnclosingCircle();
  // OpenCV's buggy algorithm, do not rely on it.
  long EnclosingCircle2();
  // Used to measure the speed of enclosing circle.
  long EnclosingCircleLoop();

  // Draw crosses on points that are vertices of the convex hull
  // of the non-zero region in the image.
  long ConvexHull();

  // Rotate image by an arbitrary angle.
  long RotateImage();

  // Save bounding box information for each frame in a video.
  // This is probably a one-use routine, I need it to check
  // bounding boxes on a sequence of synthetic hand images
  // (viewpoints_2.5.rle)
  long SaveBoundingBoxes();

  // Save minimum enclosign circle information for each frame in a video.
  // This is probably a one-use routine, I need it to check
  // MECs on a sequence of synthetic hand images
  // (viewpoints_2.5.rle)
  long SaveMECs();
  
  long NormalizeMEC1();

  long NormalizeMEC3();

  // Converts current image to a gray scale image that
  // is 0 where the current image is 0 and 255 where
  // the current image is non-zero.
  long Binary();

  long OldQuaternionViewTest();

  long TrajectorySampleTest();
  // Measure how fast generation of linear trajectories is.
  long TimeSlowTrajectorySampling();
  long TimeTrajectorySampling();

  long CreateDatabase();
  long PoseImage();
  long DatabaseImage3();
  long DatabaseImage2();
  long PoseEdges();
  long DatabaseEdges3();
  long DatabaseEdges2();
  long DatabaseRotate();
  long DatabasePose2();
  long DatabasePose();
  long DatabasePoseFrame();
  long Database1PoseFrame();

  long SetTrajectoryCutoff();
  long SetPreviousScores();
  // This is a first attempt at generating trajectories. Assuming that
  // previous_scores and previous_frame holds some previous match scores
  // and the frame at which they were generated, and that match_scores
  // holds scores for the current frame, we generate trajectories that
  // join the top estimates of the previous frame with the top estimates
  // of the current frame.
  long CountTrajectories();

  long OldTestFastPoseFrame1();
  long OldTestFastPoseFrame2();

  long ResizeScoresHolder();
  long HoldScores();
  long TrajectoryScores();
  long SetScoresToUse();
  long EraseScores();

  // Get a vTrajectoryScores object.
  long TrajectoryScores2();
  // Show a trajectory (from vTrajectoryScores) specified by its rank.
  long ShowTrajectory();
  long ShowTrajectories();
  // Find trajectories (from vTrajectoryScores) that end in the specified
  // frame/view.
  long FindTrajectories();
  long TrajectoryAngleCutoff();
  long FindHands2();
  long FindHands3();

  long FeaturesToTrack();
  long OpticalFlow();
  long EraseFeatures();
  long AddFeature0();
  long AddFeature(long row, long col);
  long AddFeatures();
  long UpdateFeatures();
  long MouseAddFeatures();
  long AddAllFeatures();

  // This function is called to adjust palm cutoff in the case
  // where left_hand is 1, and we flip the whole image upside down.
  vLineSegmentLong LineTopBottom(vLineSegmentLong line, 
                                  long rows, long cols);
  long OnlineMethods0();
  long OnlineMethods(long in_segmentation, long in_ranking);
  long SkinThreshold0();
  long SkinThreshold(long threshold);
  long SetTopToPrint0();
  long SetTopToPrint(long number);
  long DeleteDistanceTransforms();
};






#endif // VASSILIS_INTERPRETER2_H

