#include "stdafx.h"

#include "interpreter4.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "gestures/algorithms.h"
#include "gestures/algo_templates.h"
#include "gestures/hausdorff_matching.h"
#include "opencv_utils.h"
#include "gestures/chamfer_matching.h"
#include "gestures/projections.h"
#include "shape_context/shape_context.h"
#include "shape_context/sc_learning.h"
#include "datasets/protein.h"
#include "basics/wrapper.h"
#include "datasets/time_series.h"
#include "avi_file.h"
#include "gestures/features.h"
#include "basics/new_code.h"
#include "basics/new_templates.h"

#include "basics/definitions.h"

extern float canonical_orientation(float orientation);


vInterpreter4::vInterpreter4(class_interpreter* in_interpreter) :
vInterpreter3(in_interpreter)
{
  sc_mnist = 0;

  training_alignments = vZero(vScAlignmentc);
  test_alignments = vZero(vScAlignmentc);

  vp_tree = 0;
  line_drawings = 0;


}


vInterpreter4::~vInterpreter4()
{
  CleanUp();
}


void vInterpreter4::Initialize(vGestureInterfaceGlobals * in_globals)
{
  vInterpreter3::Initialize(in_globals);

  // The goal here is to instantiate some template functions, so
  // that I can call them while debugging.
  vMatrix<uchar> junk1(1,1);
  vMatrix<long> junk2(1,1);
  vMatrix<float> junk3(1,1);
  vMatrix<double> junk4(1,1);
  long a = 0;
  if (a)
  {
    junk1.Print(0);
    junk1.PrintRange(1,1,1,1,0);
    junk1.PrintInt(0);
    junk1.PrintRangeInt(1,1,1,1,0);
    junk1.PrintRangem(1,1,1,1,0);
    junk1.PrintRangeIntm(1,1,1,1,0);

    junk1.PrintTrans(0);
    junk1.PrintRangeTrans(1,1,1,1,0);
    junk1.PrintIntTrans(0);
    junk1.PrintRangeIntTrans(1,1,1,1,0);
    junk1.PrintRangemTrans(1,1,1,1,0);
    junk1.PrintRangeIntmTrans(1,1,1,1,0);
    junk1.WriteDebug("trash");

    junk2.Print(0);
    junk2.PrintRange(1,1,1,1,0);
    junk2.PrintInt(0);
    junk2.PrintRangeInt(1,1,1,1,0);
    junk2.PrintRangem(1,1,1,1,0);
    junk2.PrintRangeIntm(1,1,1,1,0);

    junk2.PrintTrans(0);
    junk2.PrintRangeTrans(1,1,1,1,0);
    junk2.PrintIntTrans(0);
    junk2.PrintRangeIntTrans(1,1,1,1,0);
    junk2.PrintRangemTrans(1,1,1,1,0);
    junk2.PrintRangeIntmTrans(1,1,1,1,0);
    junk2.WriteDebug("trash");

    junk3.Print(0);
    junk3.PrintRange(1,1,1,1,0);
    junk3.PrintInt(0);
    junk3.PrintRangeInt(1,1,1,1,0);
    junk3.PrintRangem(1,1,1,1,0);
    junk3.PrintRangeIntm(1,1,1,1,0);

    junk3.PrintTrans(0);
    junk3.PrintRangeTrans(1,1,1,1,0);
    junk3.PrintIntTrans(0);
    junk3.PrintRangeIntTrans(1,1,1,1,0);
    junk3.PrintRangemTrans(1,1,1,1,0);
    junk3.PrintRangeIntmTrans(1,1,1,1,0);
    junk3.WriteDebug("trash");

    junk4.Print(0);
    junk4.PrintRange(1,1,1,1,0);
    junk4.PrintInt(0);
    junk4.PrintRangeInt(1,1,1,1,0);
    junk4.PrintRangem(1,1,1,1,0);
    junk4.PrintRangeIntm(1,1,1,1,0);

    junk4.PrintTrans(0);
    junk4.PrintRangeTrans(1,1,1,1,0);
    junk4.PrintIntTrans(0);
    junk4.PrintRangeIntTrans(1,1,1,1,0);
    junk4.PrintRangemTrans(1,1,1,1,0);
    junk4.PrintRangeIntmTrans(1,1,1,1,0);
    junk4.WriteDebug("trash");
  }
}


// An interesting thing with the CleanUp functions of each interpreter
// (i.e. interpreter1, 2, 3), is that each of them gets called by
// the destructor of that class. I don't understand why that happens.
// I would expect that, since the CleanUp functions are all virtual,
// only the CleanUp function of the lowest subclass (interpreter3
// currently) should be called. I don't know if the way things work 
// now is according to the C++ standard or if it is a Virtual Studio bug.
void vInterpreter4::CleanUp()
{
  vdelete(sc_mnist);
  vdelete2(training_alignments);
  vdelete2(test_alignments);

  vdelete(vp_tree);
  vdelete (line_drawings);
}


long vInterpreter4::CCodeDir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("c:\\users\\athitsos\\code\\source");
  return 1;
}


long vInterpreter4::DCodeDir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("d:\\users\\athitsos\\code\\source");
  return 1;
}


long vInterpreter4::RCodeDir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("r:\\code\\source");
  return 1;
}


long vInterpreter4::LCodeDir()
{
  vdelete2(code_directory);
  code_directory = vCopyString("/home/home7/athitsos/code/source");
  return 1;
}


long vInterpreter4::CDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("c:\\users\\athitsos\\data");
  return 1;
}


long vInterpreter4::DDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("d:\\users\\athitsos\\data");
  return 1;
}

long vInterpreter4::EDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("e:\\users\\athitsos\\data");
  return 1;
}

long vInterpreter4::FDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("f:\\users\\athitsos\\data");
  return 1;
}

long vInterpreter4::RDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("r:\\nessie\\data");
  return 1;
}


long vInterpreter4::SDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("S:\\nessie\\data");
  return 1;
}


long vInterpreter4::LDataDir()
{
  vdelete2(data_directory);
  data_directory = vCopyString("/home/home7/athitsos/nessie/data");
  return 1;
}


long vInterpreter4::SetCodeDir()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(code_directory);
  code_directory = vCopyString(buffer);
  return 1;
}


long vInterpreter4::SetDataDir()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(data_directory);
  data_directory = vCopyString(buffer);
  return 1;
}


long vInterpreter4::SetNessie()
{
  char buffer[1000];
  vPrint("\nenter new directory:\n");
  vScan("%s", buffer);
  vdelete2(nessie);
  nessie = vCopyString(buffer);
  return 1;
}


long vInterpreter4::TestGaussj()
{
  long rows = 0;
  vPrint("\nenter size:\n");
  vScan("%li", &rows);
  vMatrix<float> original(rows, rows);
  long size = original.Size();
  long i;
  for (i = 0; i < size; i++)
  {
    vPrint("\nenter entry %li:\n", i);
    float entry = 0;
    vScan("%f", &entry);
    original(i) = entry;
  }

  original.Print("original");
  vMatrix<float> inverse = original.Inv();
  inverse.Print("inverse");
  return 1;
}


long vInterpreter4::TestGaussj2()
{
#ifdef VASSILIS_OPENCV_LIBRARY
  long rows = 0;
  vPrint("\nenter size:\n");
  vScan("%li", &rows);
  vMatrix<float> original(rows, rows);
  long size = original.Size();
  long i;
  for (i = 0; i < size; i++)
  {
    vPrint("\nenter entry %li:\n", i);
    float entry = 0;
    vScan("%f", &entry);
    original(i) = entry;
  }

  original.Print("original");
  CvMat mat1;
  mat1.rows = rows;
  mat1.cols = rows;
  mat1.type = CV_MAT32F;
  mat1.step = rows * sizeof(float);
  mat1.data.fl = actual_pointer (original.Matrix()); 

  
  vMatrix<float> inverse(&original, 0);
  CvMat mat2;
  mat2.rows = rows;
  mat2.cols = rows;
  mat2.type = CV_MAT32F;
  mat2.step = rows * sizeof(float);
  mat2.data.fl = actual_pointer (inverse.Matrix()); 
  cvmInvert(&mat1, &mat2);
  
  //  gaussj2(inverse.Matrix2(), rows, (float **) 0, 0);


  inverse.Print("inverse");
 
#endif  // VASSILIS_OPENCV_LIBRARY
     
  return 1;
}


long vInterpreter4::TimeGaussj2()
{
#ifdef VASSILIS_OPENCV_LIBRARY  

  long rows = 0, times = 0;
  vPrint("\nenter size, times:\n");
  vScan("%li %li", &rows, &times);
  vMatrix<float> original(rows, rows);
  vArray(float) source = original.Matrix();
  long size = original.Size();
  long i;
  for (i = 0; i < size; i++)
  {
    float something = (float) vRandomLong2(1, 100);
    source[i] = ((float) 1.0) / something;
  }
  vPrint("starting\n");
  CvMat mat1;
  mat1.rows = rows;
  mat1.cols = rows;
  mat1.type = CV_MAT32F;
  mat1.step = rows * sizeof(float);
  mat1.data.fl = actual_pointer (original.Matrix()); 

  
  vMatrix<float> inverse(&original, 0);
  CvMat mat2;
  mat2.rows = rows;
  mat2.cols = rows;
  mat2.type = CV_MAT32F;
  mat2.step = rows * sizeof(float);
  mat2.data.fl = actual_pointer (inverse.Matrix()); 

  for (i = 0; i < times; i++)
  {
    cvmInvert(&mat1, &mat2);  
  }
  vPrint("done\n");

#endif      // VASSILIS_OPENCV_LIBRARY

  return 1;
}


long vInterpreter4::PrecomputeSqrtf()
{
  long start = 0;
  vPrint("\nEnter start:\n");
  vScan("%li", &start);

  vPrecomputedSqrtf::PrecomputeValues(start);

  return 1;
}


long vInterpreter4::TestSqrtf()
{
  float number = 0;
  vPrint("\nEnter number:\n");
  vScan("%f", &number);
  float precomputed_root = vPrecomputedSqrtf::SqrtfR(number);
  float root = sqrt(number);
  float diff = vAbs(root - precomputed_root);
  float fraction = diff / root;

  vPrint("precomputed root = %f\n", precomputed_root);
  vPrint("root = %f\n", root);
  vPrint("diff = %f\n", diff);
  vPrint("fraction = %f\n", fraction);
  
  return 1;
}


long vInterpreter4::TimePrecomputedSqrtf()
{
  float number = 0;
  long times = 0;
  vPrint("\nEnter number, times:\n");
  vScan("%f, %li", &number, &times);

  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = sum + vPrecomputedSqrtf::SqrtfR(number);
  }

  vPrint("sum = %f. Done\n", sum);
  return 1;
}


long vInterpreter4::TimeSqrtf()
{
  float number = 0;
  long times = 0;
  vPrint("\nEnter number, times:\n");
  vScan("%f, %li", &number, &times);

  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    sum = sum + log(number);
  }

  vPrint("sum = %f. Done\n", sum);
  return 1;
}


// It prints out header information for the matrix, and information
// about that particular entry.
long vInterpreter4::MatrixInfo()
{
  char filename[1000];
  long row = 0;
  long col = 0;
  vPrint("\nEnter filename, row, col\n");
  vScan("%s %li %li", filename, &row, &col);
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("failed to open %s\n", filename);
    return 0;
  }

  vector<long> header;
  v3dMatrix<float>::ReadHeader(fp, &header);

  vPrint("%li rows, %li cols, %li bands, %li type\n",
         header[1], header[2], header[3], header[0]);

  long type_number = header[0];
  if ((type_number != vTypeToLong(FloatType)) &&
      (type_number != vTypeToLong(LongType)))
  {
    vPrint("type is not FloatType (%li) or LongType (%li)\n", 
           vTypeToLong(FloatType), vTypeToLong(LongType));
    fclose(fp);
    return 0;
  }

  long rows = header[1];
  long cols = header[2];
  long bands = header[3];
  vector<float> float_values;
  vector<long> long_values;
  long i;
  for (i = 0; i < bands; i++)
  {
    vint8 offset = sizeof(long) * 4;
    offset = offset + ((vint8) sizeof(float)) * (((vint8) row) * ((vint8) cols) + ((vint8) col));
    offset = offset + ((vint8) i) * ((vint8) sizeof(float)) * ((vint8) rows) * ((vint8) cols);
    long success = function_seek(fp, offset, SEEK_SET);
    if (success <= 0)
    {
      vPrint("failed to access offset\n");
      fclose(fp);
      return 0;
    }
    
    long items;
    
    if (type_number == vTypeToLong(FloatType))
    {
      float value = 0;
      items = fread(&value, sizeof(float), 1, fp);
      float_values.push_back(value);
      vPrint("band %li: %f\n", i, float_values[i]);
    }
    else if (type_number == vTypeToLong(LongType))
    {
      long value = 0;
      items = fread(&value, sizeof(long), 1, fp);
      long_values.push_back(value);
      vPrint("band %li: %li\n", i, long_values[i]);
    }
    else
    {
      exit_error("error: we should never get here\n");
    }

    if (items != 1)
    {
      vPrint("failed to read item, items = %li\n", items);
      fclose(fp);
      return 0;
    }
  }
  
  fclose(fp);
  return 1;
}


// TransposeMatrix allows a user to load a matrix from a file
// (assuming for the time being that it is in float format) and
// save its transpose into another file.
long vInterpreter4::TransposeMatrix()
{
  char file1[1000], file2[1000];
  vPrint("\nenter input filename, output filename:\n");
  vScan("%s %s", file1, file2);
  vMatrix<float> in_matrix = vMatrix<float>::Read(file1);
  if (in_matrix.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", file1);
    return 0;
  }
  
  vMatrix<float> transpose = in_matrix.Trans();
  long success = transpose.Write(file2);
  if (success <= 0)
  {
    vPrint("failed to save to %s\n", file2);
  }
  else
  {
    vPrint("saved transpose to %s\n", file2);
  }
  return success;
}


// this function takes in the pathnames for two matrices (assumed
// to have the same number of columns, and to be in float format)
// and the pathname of an output file. A new matrix is constructed
// by putting the first matrix on top of the second matrix, and 
// the new matrix is saved on disk.
long vInterpreter4::MergeVerticallyFloat()
{
  char file1[1000], file2[1000], file3[1000];
  vPrint("\nenter filename1, filename2, output filename:\n");
  vScan("%s %s %s", file1, file2, file3);
  vMatrix<float> in_matrix1 = vMatrix<float>::Read(file1);
  if (in_matrix1.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", file1);
    return 0;
  }
  vPrint("read first matrix\n");
  
  vMatrix<float> in_matrix2 = vMatrix<float>::Read(file2);
  if (in_matrix2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", file2);
    return 0;
  }
  vPrint("read second matrix\n");
  
  long rows1 = in_matrix1.Rows();
  long rows2 = in_matrix2.Rows();
  long cols1 = in_matrix1.Cols();
  long cols2 = in_matrix2.Cols();
  long rows = rows1 + rows2;
  if (cols1 != cols2)
  {
    vPrint("can't merge, cols1 = %li, cols2 = %li\n", cols1, cols2);
    return 0;
  }
  long cols = cols1;

  FILE * fp = fopen(file3, vFOPEN_WRITE);
  if (fp == 0)
  {
    vPrint("failed to open %s\n", file3);
    return 0;
  }

  long success;
  success = vWriteHeader(fp, FloatType, rows, cols, 1);
  if (success <= 0)
  {
    exit_error("error: could not write header\n");
  }

  success = in_matrix1.WriteBandwise(fp);
  if (success <= 0)
  {
    exit_error("error: could not write in_matrix1\n");
  }
  vPrint("saved first matrix\n");

  success = in_matrix2.WriteBandwise(fp);
  if (success <= 0)
  {
    exit_error("error: could not write in_matrix2\n");
  }
  vPrint("saved second matrix\n");

  fclose(fp);
  vPrint("saved merged matrix to %s\n", file3);

  return success;
}


long vInterpreter4::ScmNew()
{
  ScmDelete();
  sc_mnist = new vScMnist();
  if (sc_mnist->IsValid() == 0)
  {
    vPrint("failed to load valid sc_mnist\n");
    ScmDelete();
    return 0;
  }

  vPrint("loaded sc_mnist\n");
  return 1;
}


long vInterpreter4::ScmDelete()
{
  vdelete(sc_mnist);
  sc_mnist = 0;
  vPrint("deleted sc_mnist\n");
  return 1;
}


long vInterpreter4::ScmTrainingData()
{
  if (sc_mnist == 0)
  {
    return 0;
  }
  if (input == 0) 
  {
    return 0;
  }

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  if ((index < 0) || (index >= sc_mnist->TrainingNumber()))
  {
    vPrint("bad index\n");
    return 0;
  }

  v3dMatrix<uchar> training_set = sc_mnist->TrainingSet();
  vMatrix<long> training_permutation = sc_mnist->TrainingPermutation();
  vMatrix<uchar> training_labels = sc_mnist->TrainingLabels();

  v3dMatrix<uchar> * digit = vCopyBand2(&training_set, index);
  ShowImage(digit);
  long permutation = training_permutation(index);
  long label = training_labels(index);
  vPrint("index = %li, permutation = %li, label = %li\n",
         index, permutation, label);

  return 1;
}


long vInterpreter4::ScmTestData()
{
  if (sc_mnist == 0)
  {
    return 0;
  }
  if (input == 0) 
  {
    return 0;
  }

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  if ((index < 0) || (index >= sc_mnist->TestNumber()))
  {
    vPrint("bad index\n");
    return 0;
  }

  v3dMatrix<uchar> test_set = sc_mnist->TestSet();
  vMatrix<long> test_permutation = sc_mnist->TestPermutation();
  vMatrix<uchar> test_labels = sc_mnist->TestLabels();

  v3dMatrix<uchar> * digit = vCopyBand2(&test_set, index);
  ShowImage(digit);
  long permutation = test_permutation(index);
  long label = test_labels(index);
  vPrint("index = %li, permutation = %li, label = %li\n",
         index, permutation, label);

  return 1;
}


long vInterpreter4::ScmTrainingBoundary()
{
  if (sc_mnist == 0)
  {
    return 0;
  }

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  if ((index < 0) || (index >= sc_mnist->TrainingNumber()))
  {
    vPrint("bad index\n");
    return 0;
  }

  v3dMatrix<float> training_boundaries = sc_mnist->TrainingBoundaries();
  v3dMatrix<float> * boundarym = vCopyBand2(&training_boundaries, index);
  vArray2(float) boundary = boundarym->Matrix2(0);

  long i;
  for (i = 0; i < boundarym->Cols(); i++)
  {
    float x = boundary[0][i];
    float y = boundary[1][i];
    float z = boundary[2][i];
    vPrint("%5li %10.5f %10.5f %10.5f\n", i, x, y, z);
  }

  vdelete(boundarym);
  return 1;
}


long vInterpreter4::ScmTestBoundary()
{
  if (sc_mnist == 0)
  {
    return 0;
  }

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  if ((index < 0) || (index >= sc_mnist->TestNumber()))
  {
    vPrint("bad index\n");
    return 0;
  }

  v3dMatrix<float> test_boundaries = sc_mnist->TestBoundaries();
  v3dMatrix<float> * boundarym = vCopyBand2(&test_boundaries, index);
  vArray2(float) boundary = boundarym->Matrix2(0);

  long i;
  for (i = 0; i < boundarym->Cols(); i++)
  {
    float x = boundary[0][i];
    float y = boundary[1][i];
    float z = boundary[2][i];
    vPrint("%5li %10.5f %10.5f %10.5f\n", i, x, y, z);
  }

  vdelete(boundarym);
  return 1;
}


long vInterpreter4::ScTestInterpolate()
{
  if (sc_mnist == 0)
  {
    return 0;
  }
  if (input == 0) 
  {
    return 0;
  }
  vShapeContext sc;

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  if ((index < 0) || (index >= sc_mnist->TrainingNumber()))
  {
    vPrint("bad index\n");
    return 0;
  }

  v3dMatrix<uchar> training_set = sc_mnist->TrainingSet();

  vMatrix<uchar> digit(&training_set, index);
  v3dMatrix<uchar> * copy = new v3dMatrix<uchar>(&digit);
  ShowImage(copy);

  vMatrix<float> samples = sc.ReshapeSamples();
  vMatrix<float> scaled = vInterpolate(samples, digit);
  v3dMatrix<float> * copy_scaled = new v3dMatrix<float>(&scaled);
  ShowImage(copy_scaled);

  return 1;
}


long vInterpreter4::ScmOriginalSet()
{
  if (sc_mnist == 0)
  {
    return 0;
  }

  char name[1000];
  long test_start = 0, test_size = 0, training_start = 0, training_size = 0;
  vPrint("\nenter name, test start, test size, training start, training size:\n");
  vScan("%s %li %li %li %li", name, &test_start, 
        &test_size, &training_start, &training_size);
  
  long success = sc_mnist->MakeOriginalSet(name, test_start, test_size, 
                                           training_start, training_size);
  if (success > 0)
  {
    vPrint("created original set successfully\n");
  }
  else
  {
    vPrint("failed to create original set\n");
  }
  return success;
}


long vInterpreter4::ScmValidationSet()
{
  if (sc_mnist == 0)
  {
    return 0;
  }

  char name[1000];
  long validation_start = 0, validation_size = 0, training_start = 0, training_size = 0;
  vPrint("\nenter name, validation_start, validation_size, training start, training size:\n");
  vScan("%s %li %li %li %li", name, &validation_start, &validation_size,
        &training_start, &training_size);
  
  long success = sc_mnist->MakeValidationSet(name, validation_start, validation_size,
                                             training_start, training_size);
  if (success > 0)
  {
    vPrint("created validation set successfully\n");
  }
  else
  {
    vPrint("failed to create validation set\n");
  }
  return success;
}


// Protein dataset
long vInterpreter4::PsQueryDatabase()
{
  return vProteinSet::MakeQueryDatabaseDistances();
}


long vInterpreter4::PsQueryQuery()
{
  return vProteinSet::MakeQueryQueryDistances();
}


long vInterpreter4::PsQdDistance()
{
  long query = 0, object = 0;
  vPrint("\nenter query, object:\n");
  vScan("%li %li", &query, &object);

  float distance1 = vProteinSet::QueryDatabaseDistance1(query, object);
  float distance2 = vProteinSet::QueryDatabaseDistance2(query, object);
  float distance3 = vProteinSet::QueryDatabaseDistance3(query, object);
  vPrint("query = %li, object = %li\ndistance1 = %f, distance2 = %f, distance3 = %f\n",
         query, object, distance1, distance2, distance3);
  return 1;
}


long vInterpreter4::PsQqDistance()
{
  long query = 0, object = 0;
  vPrint("\nenter query, object:\n");
  vScan("%li %li", &query, &object);

  float distance1 = vProteinSet::QueryQueryDistance1(query, object);
  float distance2 = vProteinSet::QueryQueryDistance2(query, object);
  float distance3 = vProteinSet::QueryQueryDistance3(query, object);
  vPrint("query = %li, object = %li\ndistance1 = %f, distance2 = %f, distance3 = %f\n",
         query, object, distance1, distance2, distance3);
  return 1;
}



// Creates a traintrain_distances file appropriate for
// creating boostmap training datasets.
long vInterpreter4::PsTrain1()
{
  long result = vProteinSet::TrainTrainDistances1();
  vPrint("\nresult = %li\n", result);
  return 1;
}


// create a dataset appropriate for measuring index errors
// (for proteins)
long vInterpreter4::PsTest2()
{
  return vProteinSet::TestTrainDistances2();
}


long vInterpreter4::PsRef2()
{
  return vProteinSet::RefTestDistances2();
}


// time series dataset
long vInterpreter4::TsCompileDistances()
{
  long thousands = 0;
  vPrint("\nenter thousands:\n");
  vScan("%li", &thousands);
  vSeries series;
  series.CompileDistances(thousands);
  return 1;
}


long vInterpreter4::TsBreakDistances()
{
  long start = 0, finish = 0;
  vPrint("\nenter start thousands, end thousands:\n");
  vScan("%li %li", &start, &finish);
  vSeries series;
  
  long success = series.BreakDistances(start, finish);
  if (success <= 0)
  {
    vPrint("errors were found\n");
  }
  else
  {
    vPrint("distances broken successfully\n");
  }
  return success;
}


long vInterpreter4::TsTestTrainDistances()
{
  vSeries series;
  series.TestTrainDistances();
  return 1;
}


long vInterpreter4::TsTestTestDistances()
{
  vSeries series;
  series.TestTestDistances();
  return 1;
}


long vInterpreter4::TsTrainTrainDistances()
{
  vSeries series;
  series.TrainTrainDistances();
  return 1;
}


long vInterpreter4::TsPermute()
{
  Randomize0();
  BoostMap_dataset::PermuteTraining("2ts", "2ats");
  return 1;
}


// Creates a giant file with distances between all pairs of
// images from the 107,328 hand poses in the database.
long vInterpreter4::AcdTrainingDistances()
{
  if (acd == 0) return 0;

  char dataset[1000];
  vPrint("\nenter dataset name:\n");
  vScan("%s", dataset);

  char * pathname = BoostMap_dataset::TrainTrainDistancesPath(dataset);
  long objects = acd->NumberOfObjects();
  FILE * fp = fopen(pathname, vFOPEN_WRITE);
  if (fp == 0)
  {
    vPrint("could not open %s\n", pathname);
    vdelete2(pathname);
    return 0;
  }

  long success;
  success = vWriteHeader(fp, FloatType, objects, objects, 1);
  if (success <= 0)
  {
    exit_error("could not write header to %s\n", pathname);
  }

  const long views = 4;
  long frames = objects / views;
  long frame, view;
  vPrint("\n");
  for (frame = 0; frame < frames; frame++)
  {
    for (view = 0; view < views; view++)
    {
      v3dMatrix<float> * distances = acd->Distances(frame, view);
      if (distances == 0)
      {
        exit_error("\nerror: failed to read distances for %li and %li\n",
                       frame, view);
      }
      success = distances->WriteBandwise(fp);
      if (success == 0)
      {
        exit_error("\nerror: failed to read distances for %li and %li\n",
                       frame, view);
      }
      vdelete(distances);
    }
    vPrint("processed frame %li of %li\r", frame+1, frames);
    
  }

  fclose(fp);
  vdelete2(pathname);
  vPrint("\n");
  return 1;
}


// creates match scores based on the chamfer distance. It assumes
// that the current image is an edge image.
long vInterpreter4::ChamferRankings()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;

  vVideoFile * database_seq = inputs[database_edge_input]->sequence;
  if (database_seq == 0) return 0;
  if (strcmp(database_seq->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }
  vRLE_Sequence * database = (vRLE_Sequence *) inputs[database_edge_input]->sequence;

  long subsampling = 0;
  vPrint("\nEnter subsampling:\n");
  vScan("%li", &subsampling);

  grayscale_image * edges = input->current_image->ToGrayCopy();
  long frames = database->NumberOfFrames();
  vMatrix<double> distances = vDatabaseChamfer(edges, database, 10, subsampling, frames);

  vdelete(match_scores);
  match_scores = new vMatchScores(&distances);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();

  vdelete(edges);
  return 1;
}


// saves chamfer distances from all frames in current input to
// all database poses.
long vInterpreter4::TestTrainChamfer()
{
  if (input == 0) 
  {
    return 0;
  }
  if (input->current_image == 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;

  vVideoFile * database_seq = inputs[database_edge_input]->sequence;
  if (database_seq == 0) return 0;
  if (strcmp(database_seq->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }
  vRLE_Sequence * database = (vRLE_Sequence *) inputs[database_edge_input]->sequence;

  char script[1000];
  char output_name[1000];
  vPrint("\nenter script, output file:\n");
  vScan("%s %s", script, output_name);

  char * directory = BoostMap_dataset::Directory1("hands");
  char * pathname = vJoinPaths(directory, output_name);
  FILE * fp = fopen(pathname, vFOPEN_WRITE);
  if (fp == 0)
  {
    vPrint("\nerror, could not open %s\n", pathname);
    vdelete2(pathname);
    vdelete2(directory);
    return 0;
  }

  long test_size = input->number_of_frames;
  long training_size = database->NumberOfFrames() * 4;
  long success = vWriteHeader(fp, FloatType, test_size, training_size, 1);

  if (success <= 0)
  {
    exit_error("error: couldn't write header\n");
  }

  long frame;
  for (frame = 0; frame < test_size; frame++)
  {
    JumpToFrame(frame);
    base_interpreter->process_command_file1(script);
    grayscale_image * edges = input->current_image->ToGrayCopy();
    long frames = database->NumberOfFrames();
    vMatrix<double> temp_distances = vDatabaseChamfer(edges, database, 10, 1, frames);
    v3dMatrix<float> distances(&temp_distances);
    success = distances.WriteBandwise(fp);
    if (success <= 0)
    {
      exit_error("error: couldn't write distances for frame %li\n", frame);
    }

    vdelete(edges);
    vPrint("processed frame %li of %li\n", frame + 1, test_size);
  }
  fclose(fp);

  return 1;
}


long vInterpreter4::DirectionChanges()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  long row = 0, col = 0, length = 0;
  float direction = 0;
  vPrint("\nenter row, col, length, direction (in degrees):\n");
  vScan("%li %li %li %f\n", &row, &col, &length, &direction);
  direction = direction * (float) M_PI / (float) 180.0;
  vPrint("%f radians\n", direction);

  v3dMatrix<float> color_image(input->current_image);
  float sum = vDirectionChanges(&color_image, row, col, length, direction);
  vPrint("sum = %f", sum);
  DrawLine2(row, col, direction, length, 255);
  return 1;
}


long vInterpreter4::DominantOrientation1()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  long row = 0, col = 0, length = 0;
  vPrint("\nenter row, col, length:\n");
  vScan("%li %li %li\n", &row, &col, &length);

  v3dMatrix<float> color_image(input->current_image);
  vMatrix<float> result = vDominantOrientation(&color_image, row, col, length);
  result.Print("result");
  float min_direction = result(0);
  float max_direction = result(2);
  DrawLine2(row, col, min_direction, length, 255);
  DrawLine2(row, col, max_direction, length/2, 200);
  return 1;
}


long vInterpreter4::DirectionHistogram()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);

  v3dMatrix<float> the_image(input->current_image);
  vMatrix<float> result = vImageDirectionChanges(&the_image, number);
  vMatrix<float> directions = vSampleDirections(number);

  long max_index = 0, junk = 0;
  float max_score = function_image_maximum3(&result, &junk, &max_index);
  long min_index = 0;
  float min_score = function_image_minimum3(&result, &junk, &min_index);
  if (max_score <= 0)
  {
    vPrint("no changes were found, max_score = %f\n", max_score);
  }
  
  color_image * image_copy = input->current_image->ToRGBCopy();
  long rows = image_copy->Rows();
  long cols = image_copy->Cols();
  long min_side= Min(rows, cols);
  float max_length = (float) Min(min_side, (long) 100);
  float factor = max_length / max_score;
  long crow = rows / 2;
  long ccol = cols / 2;
  vPrint("min direction = %f, min_value = %f, max_direction = %f, max_value = %f\n",
         directions(min_index) * 180.0 / M_PI, min_score, 
         directions(max_index) * 180.0 / M_PI, max_score);

  long i;
  for (i = 0; i < number; i++)
  {
    float direction = directions(i);
    float score = result(i);
    long length = vRound(score * factor);
    uchar value = 180;
    if (i == min_index)
    {
      value = 255;
    }
    vDrawCenteredLine(image_copy, crow, ccol, direction, length, value);
  }

  ShowImage(image_copy);
  return 1;
}


long vInterpreter4::PoserCrop()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  color_image * the_image = input->current_image->ToRGBCopy();
  color_image * cropped = vPoserExtractHand(the_image, 380, 360);
  if (cropped == 0)
  {
    vPrint("extraction failed\n");
  }
  else
  {
    ShowImage(cropped);
  }
  vdelete(the_image);
  return 1;
}


// this function merges the 26 subsequences of real poser 1 
// (like the poser1 database, but with realistic renderings).
long vInterpreter4::MergeRealPoser1()
{
  char * base = vJoinPaths3(sequences_directory, "databases", 
                           "real_poser1");
  long start = 0;
  long end = 25;
  long i;
  char * output = vJoinPaths(base, "real_poser1.avi");
  vAVI_Stream * out_video = 0;
  for (i = start; i <= end; i++)
  {
    char simple_name[100];
    if (i < 10)
    {
      sprintf(simple_name, "0%li.avi", i);
    }
    else
    {
      sprintf(simple_name, "%li.avi", i);
    }
    char * input = vJoinPaths(base, simple_name);
    vVideoFile * in_video = vOpenReadVideoFile(input);
    if (in_video == 0)
    {
      vPrint("failed to open %s\n", input);
      vdelete2(input);
      continue;
    }
    vdelete2(input);
    
    if (i == start)
    {
      long rows = in_video->Rows();
      long cols = in_video->Cols();
      out_video = new vAVI_Stream(output, rows, cols, 3);
      if (out_video->IsValid() <= 0)
      {
        vPrint("failed to open %s\n", output);
        vdelete(out_video);
        vdelete(in_video);
        break;
      }
    }

    long frames = in_video->NumberOfFrames();
    long frame;
    for (frame = 0; frame < frames; frame++)
    {
      color_image * the_image = in_video->ReadFrame(frame);
      if (the_image == 0)
      {
        vPrint("%li sequence, failed to read frame %li\n", 
               i, frame);
      }
      long success = out_video->write_frame(the_image);
      if (success <= 0)
      {
        vPrint("%li sequence, failed to write frame %li\n", i, frame);
      }
      vPrint("%li sequence, processed %li of %li frames\r", 
             i, frame+1, frames);
      vdelete(the_image);
    }
    vCloseVideoFile(in_video);
  }
  long success = out_video->FinishWriting();
  if (success <= 0)
  {
    vPrint("failed to close output video file\n");
  }
  vdelete(out_video);
  vdelete2(base);
  vdelete2(output);

  return 1;
}


// We create an input based on the "real" sequence of the
// hand database.
long vInterpreter4::RealSequence()
{
  if (hand_database == 0)
  {
    vPrint("no hand database, use hand_database\n");
    return 0;
  }

  vVideoFile * sequence = hand_database->RealImages();
  AddInput(sequence);
  return 1;
}


long vInterpreter4::PixelOrientationsTest()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  long min_scale = 10;
  long max_scale = 100;
  long scale_steps = 10;
  long orientation_number = 40;

  long row = 0, col = 0;
  long orientation_index = 0;
  vPrint("input row, col, orientation_index\n");
  vScan("%li %li %li", &row, &col, &orientation_index);

  if ((orientation_index >= orientation_number) || (orientation_index < 0))
  {
    vPrint("bad orientation index\n");
    return 0;
  }
  if (input->current_image->check_bounds(row, col) <= 0)
  {
    vPrint("out of bounds\n");
    return 0;
  }

  color_image * the_image = input->current_image->ToRGBCopy();
  v3dMatrix<float> color_imagef(input->current_image);

  vImageOrientations imor(the_image, orientation_number, min_scale, max_scale, scale_steps);
  vMatrix<float> pixel_values = imor.PixelValues(row, col);
  vMatrix<long> scales = imor.Scales();
  vMatrix<float> orientations = imor.Orientations();

  v3dMatrix<float> * displacements = imor.Displacements(orientations, scales);
  vMatrix<float> pixel_values2 = imor.DirectionChanges(the_image, row, col, displacements);

  long i;
  for (i = 0; i < scale_steps; i++)
  {
    float direction = orientations(orientation_index);
    long length = scales(i);
    float sum = vDirectionChanges(&color_imagef, row, col, length, direction);
    vPrint("%3li: %6.2f %6.2f %6.2f\n", length, sum, 
           pixel_values2(orientation_index, i), pixel_values(orientation_index, i));
  }

  vdelete(the_image);
  vdelete(displacements);
  return 1;
}


long vInterpreter4::BourgainTest()
{
  long number = 0, objects_to_use = 0;
  vPrint("\nEnter training number, objects to use:\n");
  vScan("%li %li", &number, &objects_to_use);

  class_Bourgain bourgain(number, objects_to_use);
  bourgain.Print();
  return 1;
}


long vInterpreter4::BourgainSave()
{
  // We could, in principle, instead of asking the user to type the
  // number of training objects, look up the number from the dataset.
  // However, the MNIST dataset with the Shape Context distance is 
  // an example where we need to type a number less than the training
  // number, because we don't have the full training-to-training
  // matrix of distances computed. In that dataset,
  // we do know the distances from all objects to the first 5,000
  // objects, so we should enter 5,000 as the number.
  long number = 0, objects_to_use = 0;
  char dataset[1000];
  vPrint("\nenter dataset, number, objects to use:\n");
  vScan("%s %li %li", dataset, &number, &objects_to_use);
  class_Bourgain bourgain(number, objects_to_use);
  bourgain.SaveEmbeddings(dataset);
  vPrint("bourgain embedding of %s saved\n", dataset);
  return 1;
}


long vInterpreter4::BourgainIndexErrors()
{
  long dimensions = 0, number = 0, max_k = 0;
  char dataset[1000];
  vPrint("\nenter dataset, dimensions, experiment number, max_k:\n");
  vScan("%s %li %li %li", dataset, &dimensions, &number, &max_k);

  vMatrix<float> test_embedding = class_Bourgain::LoadTestEmbedding(dataset, dimensions, number);
  vMatrix<float> training_embedding = class_Bourgain::LoadTrainingEmbedding(dataset, dimensions, number);

  char * distances_file = BoostMap_dataset::TestTrainDistancesPath(dataset);
  class_file * fp = BoostMap_dataset::OpenObjectDistancesFile(distances_file);
  vdelete2(distances_file);
  if (fp == 0)
  {
    vPrint("failed to open distances file\n");
    return 0;
  }

  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 1.0);

  vMatrix<float> result = BoostMap_dataset::IndexErrors6L2(test_embedding, training_embedding,
                                                     weights, fp, 0, max_k);
  fclose(fp);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
    v3dMatrix<float> temp_ranksk = copy_vertical_line(&result, max_k);

    vector<float> ranks1;
    vector<float> ranksk;
    vector_from_matrix(&temp_ranks1, &ranks1);
    vector_from_matrix(&temp_ranksk, &ranksk);
    std::sort(ranks1.begin(), ranks1.end(), less<float>());
    std::sort(ranksk.begin(), ranksk.end(), less<float>());
    long size = ranks1.size();
    float number = 100;
    float i;
    for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
    {
      float percentile = i * ((float) 100.0 / number);
      long index = vRound(((float) size) / number * i);
      if (index == size)
      {
        index--;
      }
      float entry1 = ranks1[index];
      float entryk = ranksk[index];

      vPrint("%6li: %6.2f, %10.1f, %10.1f\n", vRound(index), percentile, entry1, entryk);
    }
  }

  char * output = BoostMap_dataset::IndexErrorOutputPath2(dataset, "bourgain", 
                                                    dimensions);

  long success = result.Write(output);
  if (success <= 0)
  {
    vPrint("failed to save result to %s\n", output);
  }
  else
  {
    vPrint("saved result to %s\n", output);
  }
  vdelete2(output);

  return 1;
}


long vInterpreter4::SparseMapSave()
{
  // We could, in principle, instead of asking the user to type the
  // number of training objects, look up the number from the dataset.
  // However, the MNIST dataset with the Shape Context distance is 
  // an example where we need to type a number less than the training
  // number, because we don't have the full training-to-training
  // matrix of distances computed. In that dataset,
  // we do know the distances from all objects to the first 5,000
  // objects, so we should enter 5,000 as the number.
  long number = 0, objects_to_use = 0;
  char dataset[1000];
  vPrint("\nenter dataset, number, objects to use:\n");
  vScan("%s %li %li", dataset, &number, &objects_to_use);
  class_SparseMap sparsemap(number, objects_to_use);
  sparsemap.SaveEmbeddings(dataset);
  vPrint("SparseMap embedding of %s saved\n", dataset);
  return 1;
}


long vInterpreter4::SparseMapIndexErrors()
{
  long dimensions = 0, number = 0, max_k = 0;
  char dataset[1000];
  vPrint("\nenter dataset, dimensions, experiment number, max_k:\n");
  vScan("%s %li %li %li", dataset, &dimensions, &number, &max_k);

  vMatrix<float> test_embedding = class_SparseMap::LoadTestEmbedding(dataset, dimensions, number);
  vMatrix<float> training_embedding = class_SparseMap::LoadTrainingEmbedding(dataset, dimensions, number);

  char * distances_file = BoostMap_dataset::TestTrainDistancesPath(dataset);
  class_file * fp = BoostMap_dataset::OpenObjectDistancesFile(distances_file);
  vdelete2(distances_file);
  if (fp == 0)
  {
    vPrint("failed to open distances file\n");
    return 0;
  }

  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 1.0);

  vMatrix<float> result = BoostMap_dataset::IndexErrors6L2(test_embedding, training_embedding,
                                                     weights, fp, 0, max_k);
  fclose(fp);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
    v3dMatrix<float> temp_ranksk = copy_vertical_line(&result, max_k);

    vector<float> ranks1;
    vector<float> ranksk;
    vector_from_matrix(&temp_ranks1, &ranks1);
    vector_from_matrix(&temp_ranksk, &ranksk);
    std::sort(ranks1.begin(), ranks1.end(), less<float>());
    std::sort(ranksk.begin(), ranksk.end(), less<float>());
    long size = ranks1.size();
    float number = 100;
    float i;
    for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
    {
      float percentile = i * ((float) 100.0 / number);
      long index = vRound(((float) size) / number * i);
      if (index == size)
      {
        index--;
      }
      float entry1 = ranks1[index];
      float entryk = ranksk[index];

      vPrint("%6li: %6.2f, %10.1f, %10.1f\n", vRound(index), percentile, entry1, entryk);
    }
  }

  char * output = BoostMap_dataset::IndexErrorOutputPath2(dataset, "sparsemap", 
                                                    dimensions);

  long success = result.Write(output);
  if (success <= 0)
  {
    vPrint("failed to save result to %s\n", output);
  }
  else
  {
    vPrint("saved result to %s\n", output);
  }
  vdelete2(output);

  return 1;
}


long vInterpreter4::BmsetOnSparseMap()
{
  long dimensions = 0, number = 0, max_k = 0;
  char original[1000], dataset1[1000], dataset2[1000];
  
  vPrint("\nEnter original, dataset1, dataset2, dimensions, experiment number:\n");
  vScan("%s %s %s %li %li", original, dataset1, dataset2, 
        &dimensions, &number);

  vMatrix<float> training_embedding = class_SparseMap::LoadTrainingEmbedding(original, dimensions, number);
  if (training_embedding.IsValid() <= 0)
  {
    vPrint("failed to load sparsemap embedding\n");
    return 0;
  }

  BoostMap_dataset bmset(dataset1);
  if (bmset.IsValid() <= 0)
  {
    vPrint("failed to load dataset1\n");
    return 1;
  }

  bmset.OnTopDataset(dataset2, training_embedding);
  return 1;
}


long vInterpreter4::BmsmIndexErrors()
{
  long dimensions = 0, number = 0, max_k = 0;
  char dataset[1000], boostmap_file[1000];
  vPrint("\nenter dataset, boostmap file, dimensions, experiment number, max_k:\n");
  vScan("%s %s %li %li %li", dataset, boostmap_file, &dimensions, &number, &max_k);

  vMatrix<float> test_embedding = class_SparseMap::LoadTestEmbedding(dataset, dimensions, number);
  vMatrix<float> training_embedding = class_SparseMap::LoadTrainingEmbedding(dataset, dimensions, number);
  if ((test_embedding.IsValid() <= 0) || (training_embedding.IsValid() <= 0))
  {
    vPrint("failed to load embeddings\n");
    return 0;
  }

  char * bm_path = class_BoostMap::Pathname(boostmap_file);
  vMatrix<float> all_classifiers = vMatrix<float>::ReadText(bm_path);
  if (all_classifiers.IsValid() <= 0)
  {
    vPrint("failed to load boostmap embedding from %s\n", bm_path);
    vdelete2(bm_path);
    return 0;
  }
  
  vdelete2(bm_path);
  
  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 0);
  long i;
  for (i = 0; i < all_classifiers.Rows(); i++)
  {
    float index = all_classifiers(i, 1);
    float weight = all_classifiers(i, 3);
    if ((index < 0) || (index >= dimensions))
    {
      vPrint("bad index = %li, dimensions = %li\n", index, dimensions);
      return 0;
    }
    weights((long) index) += weight;
  }

  char * distances_file = BoostMap_dataset::TestTrainDistancesPath(dataset);
  class_file * fp = BoostMap_dataset::OpenObjectDistancesFile(distances_file);
  vdelete2(distances_file);
  if (fp == 0)
  {
    vPrint("failed to open distances file\n");
    return 0;
  }

  vMatrix<float> result = BoostMap_dataset::IndexErrors6L2(test_embedding, training_embedding,
                                                     weights, fp, 0, max_k);
  fclose(fp);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
    v3dMatrix<float> temp_ranksk = copy_vertical_line(&result, max_k);

    vector<float> ranks1;
    vector<float> ranksk;
    vector_from_matrix(&temp_ranks1, &ranks1);
    vector_from_matrix(&temp_ranksk, &ranksk);
    std::sort(ranks1.begin(), ranks1.end(), less<float>());
    std::sort(ranksk.begin(), ranksk.end(), less<float>());
    long size = ranks1.size();
    float number = 100;
    float i;
    for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
    {
      float percentile = i * ((float) 100.0 / number);
      long index = vRound(((float) size) / number * i);
      if (index == size)
      {
        index--;
      }
      float entry1 = ranks1[index];
      float entryk = ranksk[index];

      vPrint("%6li: %6.2f, %10.1f, %10.1f\n", vRound(index), percentile, entry1, entryk);
    }
  }

  char * output = BoostMap_dataset::IndexErrorOutputPath2(dataset, "bmsm", 
                                                    dimensions);

  long success = result.Write(output);
  if (success <= 0)
  {
    vPrint("failed to save result to %s\n", output);
  }
  else
  {
    vPrint("saved result to %s\n", output);
  }
  vdelete2(output);

  return 1;
}


long vInterpreter4::LoadAlignments()
{
  long training_number = 20000;
  long test_number = 10000;
  long class_id = 0, number = 0;
  vPrint("\nEnter class_id, number:\n");
  vScan("%li %li", &class_id, &number);

  if (vIsZero(training_alignments))
  {
    training_alignments = vnew(vScAlignmentc, training_number);
  }
  if (vIsZero(test_alignments))
  {
    test_alignments = vnew(vScAlignmentc, test_number);
  }

  char * filename;
  FILE * fp;
  long i;

  filename = vScAlignmentc::TrainingAlignmentPath(class_id, number);
  fp = fopen(filename, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("failed to open %s\n", filename);
    vdelete2(filename);
    return 0;
  }
  vdelete2(filename);
  vPrint("\n");
  for (i = 0; i < training_number; i++)
  {
    long success = training_alignments[i].Load(fp);
    if (success <= 0)
    {
      vPrint("\nfailed to load training alignment %li\n", i);
      fclose(fp);
      return 0;
    }
    vPrint("loaded alignment %li of %li\r", i+1, training_number);
  }
  vPrint("\n");
  
  filename = vScAlignmentc::TestAlignmentPath(class_id, number);
  fp = fopen(filename, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("failed to open %s\n", filename);
    vdelete2(filename);
    return 0;
  }
  vdelete2(filename);
  vPrint("\n");
  for (i = 0; i < test_number; i++)
  {
    long success = test_alignments[i].Load(fp);
    if (success <= 0)
    {
      vPrint("\nfailed to load test alignment %li\n", i);
      fclose(fp);
      return 0;
    }
    vPrint("loaded alignment %li of %li\r", i+1, test_number);
  }
  vPrint("\n");
  return 1;
}


long vInterpreter4::DeleteAlignments()
{
  vdelete2(training_alignments);
  vdelete2(test_alignments);
  return 1;
}


long vInterpreter4::CheckAlignments()
{
  if (vIsZero(training_alignments))
  {
    return 0;
  }
  if (vIsZero(test_alignments))
  {
    return 0;
  }

  return 1;
}


long vInterpreter4::ScAlignIndexError()
{
  if (CheckAlignments() <= 0)
  {
    return 0;
  }

  long max_k = 0, query = 0;
  vPrint("\nEnter max_k, query index:\n");
  vScan("%li %li", &max_k, &query);

  if ((query < 0) || (query >= 10000))
  {
    vPrint("bad query number\n");
    return 0;
  }

  vScAlignmentc query_alignment = test_alignments[query];
  vMatrix<float> approx_distances = vAlignmentDistances(&query_alignment,
                                                        training_alignments, 
                                                        20000);
  vMatrix<float> exact_distances = BoostMap_dataset::TestTrainDistance("sc", query);
  vMatrix<float> result = BoostMap_dataset::IndexErrors3(approx_distances, exact_distances, max_k);
  result.PrintInt("ranks:");
  return 1;
}


long vInterpreter4::ScAlignIndexErrors()
{
  if (CheckAlignments() <= 0)
  {
    return 0;
  }

  long max_k = 0;
  vPrint("\nEnter max_k:\n");
  vScan("%li", &max_k);

  vMatrix<float> result = vScAlignmentIndexErrors(test_alignments, training_alignments,
                                                   10000, 20000, max_k);

  v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
  v3dMatrix<float> temp_ranks2 = copy_vertical_line(&result, max_k);

  vector<float> ranks1;
  vector<float> ranks2;
  vector_from_matrix(&temp_ranks1, &ranks1);
  vector_from_matrix(&temp_ranks2, &ranks2);
  std::sort(ranks1.begin(), ranks1.end(), less<float>());
  std::sort(ranks2.begin(), ranks2.end(), less<float>());
  long size = ranks1.size();
  float number = 100;
  float i;
  for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
  {
    float percentile = i * ((float) 100.0 / number);
    long index = vRound(((float) size) / number * i);
    if (index == size)
    {
      index--;
    }
    float entry1 = ranks1[index];
    float entry2 = ranks2[index];

    vPrint("%5li: %6.2f, %10.2f %10.2f\n", 
           vRound(index), percentile, entry1, entry2);
  }

  return 1;
}


long vInterpreter4::ScSaveClassifiers()
{
  char training_dataset[100];
  char triples_file[100];
  long first = 0, last = 0;
  
  vPrint("\nEnter dataset, triples file, first, last:\n");
  vScan("%s %s %li %li", training_dataset, triples_file, &first, &last);

  DeleteAlignments();
  long success = vScSaveClassifiers(training_dataset, triples_file, 
                                    first, last);
  if (success <= 0)
  {
    vPrint("\nfailed to save classifiers\n");
  }
  else
  {
    vPrint("\nsaved classifiers successfully\n");
  }

  return 1;
}


// This function is useful when we save the results of weak classifiers
// to disk, and then we want to normalize them so that they are all
// in the range [-1, 1].
long vInterpreter4::RecordMaxEntry()
{
  char training_dataset[100];
  char triples_file[100];
  long first = 0, last = 0;
  
  vPrint("\nEnter dataset, triples file, first, last:\n");
  vScan("%s %s %li %li", training_dataset, triples_file, &first, &last);
  class_BoostMap::RecordMaxEntry(training_dataset, triples_file, first, last);

  return 1;
}


long vInterpreter4::ReadMaxEntry()
{
  char training_dataset[100];
  char triples_file[100];
  long first = 0, last = 0;
  
  vPrint("\nEnter dataset, triples file, first, last:\n");
  vScan("%s %s %li %li", training_dataset, triples_file, &first, &last);
  float max_entry = class_BoostMap::ReadMaxEntry(training_dataset, triples_file, 
                                            first, last);
  vPrint("max_entry = %f\n", max_entry);

  return 1;
}


long vInterpreter4::BmLoadScClassifiers()
{
  if (bm == 0)
  {
    return 0;
  }

  long file_first = 0, file_last = 0, feature_first = 0, feature_last = 0;
  long first = 0, last = 0;

  vPrint("\nenter file first and last, feature first and last, first, last:\n");
  vScan("%li %li %li %li %li %li", &file_first, &file_last, &feature_first, 
        &feature_last, &first, &last);

  vScLoadClassifiers(bm, file_first, file_last, feature_first, feature_last,
                     first, last);

  return 1;
}


// BmTripleResult and ScCheckTriple are used to verify that the 
// results of weak classifiers saved in directories like
// sc1_data are correct.
long vInterpreter4::BmTripleResult()
{
  if (bm == 0)
  {
    return 0;
  }

  long triple = 0, classifier = 0;
  vPrint("\nEnter triple, classifier:\n");
  vScan("%li %li", &triple, &classifier);

  bm->TripleResult(triple, classifier);

  return 1;
}


long vInterpreter4::ScCheckTriple()
{
  if (sc_mnist == 0)
  {
    vPrint("use scm_new first\n");
    return 0;
  }
  if (bm == 0)
  {
    return 0;
  }

  long triple = 0, classifier_id = 0;
  vPrint("\nEnter triple, classifier_id:\n");
  vScan("%li %li", &triple, &classifier_id);
  return ScCheckTriple(triple, classifier_id);
}


long vInterpreter4::ScCheckTriple(long triple, long classifier_id)
{
  if (sc_mnist == 0)
  {
    vPrint("use scm_new first\n");
    return 0;
  }
  if (bm == 0)
  {
    return 0;
  }

  if (triple < 0) 
  {
    return 0;
  }

  bm->TripleResult(triple, classifier_id);

  vMatrix<long> training_triples = bm->TrainingTriples();
  vMatrix<long> validation_triples = bm->ValidationTriples();
  vMatrix<long> training_ids = bm->TrainingIds();
  vMatrix<long> validation_ids = bm->ValidationIds();

  if (triple < training_triples.Rows())
  {
    long q = training_ids(training_triples(triple, 0));
    long a = training_ids(training_triples(triple, 1));
    long b = training_ids(training_triples(triple, 2));

    // First check the distance, as saved in training distances.
    char * pathname = BoostMap_dataset::TrainTrainDistancesPath("sc");
    vMatrix<float> distances = BoostMap_dataset::ObjectDistances(pathname, q);
    vdelete2(pathname);
    if (distances.IsValid() == 0)
    {
      vPrint("failed to read object distances\n");
      return 0;
    }
    float qa = distances(a);
    float qb = distances(b);
    float diff = qb - qa;
    vPrint("qa = %f, qb = %f, diff = %f\n", qa, qb, diff);

    // Figure out prototype and feature corresponding to classifier_id.
    long prototype = classifier_id / 400;
    long point = (classifier_id % 400) / 4;
    long feature = classifier_id % 4;
    long class_id = prototype % 10;
    long number = prototype / 10;

    long prototype_id = vPrototypeIndex(class_id, number);
    vPrint("prototype_id = %li\n", prototype_id);

    // Load these objects.
    vMatrix<uchar> q_image = sc_mnist->Digit(q, 0);
    vMatrix<uchar> a_image = sc_mnist->Digit(a, 0);
    vMatrix<uchar> b_image = sc_mnist->Digit(b, 0);
    vMatrix<uchar> p_image = sc_mnist->Digit(prototype_id, 0);

    vMatrix<float> q_boundary = sc_mnist->Boundary(q, 0);
    vMatrix<float> a_boundary = sc_mnist->Boundary(a, 0);
    vMatrix<float> b_boundary = sc_mnist->Boundary(b, 0);
    vMatrix<float> p_boundary = sc_mnist->Boundary(prototype_id, 0);

    // Compute alignments.
    vScAlignment q_align(q_image, p_image, q_boundary, p_boundary);
    vScAlignment a_align(a_image, p_image, a_boundary, p_boundary);
    vScAlignment b_align(b_image, p_image, b_boundary, p_boundary);

    // directly compute feature-based distances.
    float qa_point = vScPointDistance(q_align.points, a_align.points, point);
    float qb_point = vScPointDistance(q_align.points, b_align.points, point);
    float diff_point = qb_point - qa_point;
    vPrint("qa_point = %f, qb_point = %f, diff = %f\n", qa_point, qb_point, diff_point);

    float qa_orientation = vScOrientationDistance(q_align.orientations, a_align.orientations, point);
    float qb_orientation = vScOrientationDistance(q_align.orientations, b_align.orientations, point);
    float diff_orientation = qb_orientation - qa_orientation;
    vPrint("qa_orientation = %f, qb_orientation = %f, diff = %f\n", qa_orientation, qb_orientation, diff_orientation);

    float qa_sc = vChiSquare5(q_align.sc_features, q_align.orientations, 
                              a_align.sc_features, a_align.orientations, point);
    float qb_sc = vChiSquare5(q_align.sc_features, q_align.orientations, 
                              b_align.sc_features, b_align.orientations, point);
    float diff_sc = qb_sc - qa_sc;
    vPrint("qa_sc = %f, qb_sc = %f, diff = %f\n", qa_sc, qb_sc, diff_sc);

    float qa_ssd = vSsd5(q_align.image, q_align.points, 
                         a_align.image, a_align.points, point);
    float qb_ssd = vSsd5(q_align.image, q_align.points, 
                         b_align.image, b_align.points, point);
    float diff_ssd = qb_ssd - qa_ssd;
    vPrint("qa_ssd = %f, qb_ssd = %f, diff = %f\n\n", qa_ssd, qb_ssd, diff_ssd);
  }

  if (triple < validation_triples.Rows())
  {
    long q = validation_ids(validation_triples(triple, 0));
    long a = validation_ids(validation_triples(triple, 1));
    long b = validation_ids(validation_triples(triple, 2));

    // First check the distance, as saved in training distances.
    char * pathname = BoostMap_dataset::TrainTrainDistancesPath("sc");
    vMatrix<float> distances = BoostMap_dataset::ObjectDistances(pathname, q);
    vdelete2(pathname);
    if (distances.IsValid() == 0)
    {
      vPrint("failed to read object distances\n");
      return 0;
    }
    float qa = distances(a);
    float qb = distances(b);
    float diff = qb - qa;
    vPrint("qa = %f, qb = %f, diff = %f\n", qa, qb, diff);

    // Figure out prototype and feature corresponding to classifier_id.
    long prototype = classifier_id / 400;
    long point = (classifier_id % 400) / 4;
    long feature = classifier_id % 4;
    long class_id = prototype % 10;
    long number = prototype / 10;

    long prototype_id = vPrototypeIndex(class_id, number);
    vPrint("prototype_id = %li\n", prototype_id);

    // Load these objects.
    vMatrix<uchar> q_image = sc_mnist->Digit(q, 0);
    vMatrix<uchar> a_image = sc_mnist->Digit(a, 0);
    vMatrix<uchar> b_image = sc_mnist->Digit(b, 0);
    vMatrix<uchar> p_image = sc_mnist->Digit(prototype_id, 0);

    vMatrix<float> q_boundary = sc_mnist->Boundary(q, 0);
    vMatrix<float> a_boundary = sc_mnist->Boundary(a, 0);
    vMatrix<float> b_boundary = sc_mnist->Boundary(b, 0);
    vMatrix<float> p_boundary = sc_mnist->Boundary(prototype_id, 0);

    // Compute alignments.
    vScAlignment q_align(q_image, p_image, q_boundary, p_boundary);
    vScAlignment a_align(a_image, p_image, a_boundary, p_boundary);
    vScAlignment b_align(b_image, p_image, b_boundary, p_boundary);

    // directly compute feature-based distances.
    float qa_point = vScPointDistance(q_align.points, a_align.points, point);
    float qb_point = vScPointDistance(q_align.points, b_align.points, point);
    float diff_point = qb_point - qa_point;
    vPrint("qa_point = %f, qb_point = %f, diff = %f\n", qa_point, qb_point, diff_point);

    float qa_orientation = vScOrientationDistance(q_align.orientations, a_align.orientations, point);
    float qb_orientation = vScOrientationDistance(q_align.orientations, b_align.orientations, point);
    float diff_orientation = qb_orientation - qa_orientation;
    vPrint("qa_orientation = %f, qb_orientation = %f, diff = %f\n", qa_orientation, qb_orientation, diff_orientation);

    float qa_sc = vChiSquare5(q_align.sc_features, q_align.orientations, 
                              a_align.sc_features, a_align.orientations, point);
    float qb_sc = vChiSquare5(q_align.sc_features, q_align.orientations, 
                              b_align.sc_features, b_align.orientations, point);
    float diff_sc = qb_sc - qa_sc;
    vPrint("qa_sc = %f, qb_sc = %f, diff = %f\n", qa_sc, qb_sc, diff_sc);

    float qa_ssd = vSsd5(q_align.image, q_align.points, 
                         a_align.image, a_align.points, point);
    float qb_ssd = vSsd5(q_align.image, q_align.points, 
                         b_align.image, b_align.points, point);
    float diff_ssd = qb_ssd - qa_ssd;
    vPrint("qa_ssd = %f, qb_ssd = %f, diff = %f\n", qa_ssd, qb_ssd, diff_ssd);
  }
  return 1;
}


long vInterpreter4::ScSummaryIndexErrors()
{
  char buffer[200];
  long dimensions = 0, max_k = 0;
  vPrint("\nEnter filename, dimensions, max_k:\n");
  vScan("%s %li %li", buffer, &dimensions, &max_k);

  vMatrix<float> result = vScSummaryIndexErrors2(buffer, dimensions, max_k);
  if (result.IsValid() <= 0)
  {
    vPrint("failed to get results\n");
    return 0;
  }

  v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
  v3dMatrix<float> temp_ranks2 = copy_vertical_line(&result, max_k);

  vector<float> ranks1;
  vector<float> ranks2;
  vector_from_matrix(&temp_ranks1, &ranks1);
  vector_from_matrix(&temp_ranks2, &ranks2);
  std::sort(ranks1.begin(), ranks1.end(), less<float>());
  std::sort(ranks2.begin(), ranks2.end(), less<float>());
  long size = ranks1.size();
  float number = 100;
  float i;
  for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
  {
    float percentile = i * ((float) 100.0 / number);
    long index = vRound(((float) size) / number * i);
    if (index == size)
    {
      index--;
    }
    float entry1 = ranks1[index];
    float entry2 = ranks2[index];

    vPrint("%5li: %6.2f, %10.2f %10.2f\n", 
           vRound(index), percentile, entry1, entry2);
  }

  return 1;
}


// here we make up classifiers consisting of features 2-3 of 
// specified prototypes, with equal weights.
long vInterpreter4::ScPrototypeIndexErrors()
{
  long first = 0, last = 0, max_k = 0;
  vPrint("\nenter first, last, max_k:\n");
  vScan("%li %li %li", &first, &last, &max_k);

  if ((first < 0) || (last >= 10) || (first > last))
  {
    return 0;
  }
  long prototypes = last - first + 1;
  long total_features = 200 * prototypes;

  vMatrix<long> classifier_ids(1, total_features/2);
  vMatrix<float> classifier_weights(1, total_features/2);
  vWriteValue(&classifier_weights, (float) 1);

  long k, j;
  long index = 0;
  for (k = first; k <= last; k++)
  {
    for (j = 0; j < 100; j+=2)
    {
      long id = 400 * k + 4 * j + 2;
      classifier_ids(index) = id;
      index++;
      classifier_ids(index) = id + 1;
      index++;
    }
  }

  class_pointer (vScSummary) training_summaries = vScSummaries(classifier_ids, 20000, 0);
  class_pointer (vScSummary) test_summaries = vScSummaries(classifier_ids, 10000, 1);

  vMatrix<float> result = vScSummaryIndexErrors(test_summaries, training_summaries,
                                                classifier_weights, 10000, 20000,
                                                max_k);

  vdelete2(training_summaries);
  vdelete2(test_summaries);
  v3dMatrix<float> temp_ranks1 = copy_vertical_line(&result, 1);
  v3dMatrix<float> temp_ranks2 = copy_vertical_line(&result, max_k);

  vector<float> ranks1;
  vector<float> ranks2;
  vector_from_matrix(&temp_ranks1, &ranks1);
  vector_from_matrix(&temp_ranks2, &ranks2);
  std::sort(ranks1.begin(), ranks1.end(), less<float>());
  std::sort(ranks2.begin(), ranks2.end(), less<float>());
  long size = ranks1.size();
  float number = 100;
  float i;
  for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
  {
    float percentile = i * ((float) 100.0 / number);
    long index = vRound(((float) size) / number * i);
    if (index == size)
    {
      index--;
    }
    float entry1 = ranks1[index];
    float entry2 = ranks2[index];

    vPrint("%5li: %6.2f, %10.2f %10.2f\n", 
           vRound(index), percentile, entry1, entry2);
  }

  return 1;
}


// here we manually add classifiers to see the classification error we
// get
long vInterpreter4::ScAddClassifiers()
{
  if (bm == 0)
  {
    return 0;
  }

  long prototype = 0, feature = 0;
  vPrint("\nenter prototype, feature:\n");
  vScan("%li %li", &prototype, &feature);

  long i;
  for (i = 0; i < 100; i++)
  {
    long id = prototype * 400 + i * 4 + feature;
    bm->AddClassifier4b(id, 1);
    bm->PrintSummary();
  }

  return 1;
}


// This function is used for debugging: for two objects, and a specified
// weak classifier (corresponding to prototype, point, and feature), we
// compare the corresponding distance between those two objects, using
// vScAlignment and using vScSummary. The two distances should be the same.
long vInterpreter4::ScClassifierDistance()
{
  if (sc_mnist == 0)
  {
    vPrint("first use scm_new\n");
    return 0;
  }

  long object1 = 0, object2 = 0, test_flag = 0, id = 0;
  vPrint("\nEnter object1, test_flag, object2, id:\n");
  vScan("%li %li %li %li", &object1, &test_flag, &object2, &id);

  vScClassifier scc(id);
  long class_id = scc.prototype % 10;
  long round = scc.prototype / 10;
  long prototype = vPrototypeIndex(class_id, round);

  vMatrix<uchar> image1 = sc_mnist->Digit(object1, test_flag);
  if (image1.IsValid() <= 0)
  {
    return 0;
  }
  vMatrix<float> boundary1 = sc_mnist->Boundary(object1, test_flag);
  if (boundary1.IsValid() <= 0)
  {
    return 0;
  }

  vMatrix<uchar> image2 = sc_mnist->Digit(object2, 0);
  if (image1.IsValid() <= 0)
  {
    return 0;
  }
  vMatrix<float> boundary2 = sc_mnist->Boundary(object2, 0);
  if (boundary1.IsValid() <= 0)
  {
    return 0;
  }

  vMatrix<uchar> imagep = sc_mnist->Digit(prototype, 0);
  if (image1.IsValid() <= 0)
  {
    return 0;
  }
  vMatrix<float> boundaryp = sc_mnist->Boundary(prototype, 0);
  if (boundary1.IsValid() <= 0)
  {
    return 0;
  }

  vScAlignment align1(image1, imagep, boundary1, boundaryp);
  vScAlignment align2(image2, imagep, boundary2, boundaryp);
  vScAlignmentc alignc1(image1, imagep, boundary1, boundaryp);
  vScAlignmentc alignc2(image2, imagep, boundary2, boundaryp);

  float distance1 = vScFeatureDistance(&align1, &align2, scc.point, scc.feature);
  vScSummary::number_of_points = 0;
  vScSummary::number_of_orientations = 0;
  vScSummary::number_of_shape_contexts = 0;
  vScSummary::number_of_images = 0;

  if (scc.feature == 0)
  {
    vScSummary::number_of_points = 1;
  }
  else if (scc.feature == 1)
  {
    vScSummary::number_of_orientations = 1;
  }
  else if (scc.feature == 2)
  {
    vScSummary::number_of_shape_contexts = 1;
  }
  else if (scc.feature == 3)
  {
    vScSummary::number_of_images = 1;
  }

  vMatrix<long> classifier_ids(1, 1);
  vMatrix<float> weights(1, 1);
  classifier_ids(0) = id;
  weights(0) = 1;
  
  vScSummary summary1, summary2;
  summary1.AddFeatures(&alignc1, scc.prototype, classifier_ids);
  summary2.AddFeatures(&alignc2, scc.prototype, classifier_ids);

  float distance2 = summary1.Compare(&summary2, weights);
  vPrint("distance1 = %f, distance2 = %f\n", distance1, distance2);

  return 1;
}


// Here we manually enter a classifier
long vInterpreter4::ScClassifierDistances()
{
  if (sc_mnist == 0)
  {
    vPrint("first use scm_new\n");
    return 0;
  }

  long object1 = 0, object2 = 0, test_flag = 0;
  vPrint("\nEnter object1, test_flag, object2:\n");
  vScan("%li %li %li", &object1, &test_flag, &object2);

  vector<long> idsv;
  vector<float> weightsv;

  while(1)
  {
    long id = 0;
    float weight = 0;
    vPrint("\nEnter classifier id, negative to end\n");
    vScan("%li", &id);
    if (id < 0)
    {
      break;
    }

    idsv.push_back(id);
    vPrint("\nEnter weight:\n");
    vScan("%f", &weight);
    weightsv.push_back(weight);
  }

  long number = idsv.size();
  vMatrix<long> classifier_ids(1, number);
  vMatrix<float> weights(1, number);
  vector<vScAlignment *> alignments1;
  vector<vScAlignment *> alignments2;
  vector<vScAlignmentc *> alignmentsc1;
  vector<vScAlignmentc *> alignmentsc2;

  vMatrix<uchar> image1 = sc_mnist->Digit(object1, test_flag);
  if (image1.IsValid() <= 0)
  {
    return 0;
  }
  vMatrix<float> boundary1 = sc_mnist->Boundary(object1, test_flag);
  if (boundary1.IsValid() <= 0)
  {
    return 0;
  }

  vMatrix<uchar> image2 = sc_mnist->Digit(object2, 0);
  if (image1.IsValid() <= 0)
  {
    return 0;
  }
  vMatrix<float> boundary2 = sc_mnist->Boundary(object2, 0);
  if (boundary1.IsValid() <= 0)
  {
    return 0;
  }

  long i;
  for (i = 0; i < number; i++)
  {
    classifier_ids(i) = idsv[i];
    weights(i) = weightsv[i];
  }

  weights = vScSortWeights(classifier_ids, weights);
  classifier_ids = vScSortClassifiers(classifier_ids);

  float distance1 = 0;
  for (i = 0; i < number; i++)
  {
    long id = classifier_ids(i);
    vScClassifier scc(id);
    long class_id = scc.prototype % 10;
    long round = scc.prototype / 10;
    long prototype = vPrototypeIndex(class_id, round);

    vMatrix<uchar> imagep = sc_mnist->Digit(prototype, 0);
    if (image1.IsValid() <= 0)
    {
      return 0;
    }
    vMatrix<float> boundaryp = sc_mnist->Boundary(prototype, 0);
    if (boundary1.IsValid() <= 0)
    {
      return 0;
    }

    vScAlignment * align1 = new vScAlignment(image1, imagep, boundary1, boundaryp);
    vScAlignment * align2 = new vScAlignment(image2, imagep, boundary2, boundaryp);
    vScAlignmentc * alignc1 = new vScAlignmentc(image1, imagep, boundary1, boundaryp);
    vScAlignmentc * alignc2 = new vScAlignmentc(image2, imagep, boundary2, boundaryp);
    alignments1.push_back(align1);
    alignments2.push_back(align2);
    alignmentsc1.push_back(alignc1);
    alignmentsc2.push_back(alignc2);
    float temp_distance = vScFeatureDistance(alignments1[i], alignments2[i],
                                             scc.point, scc.feature);
    vPrint("distance %3li: %f\n", i, temp_distance);
    distance1 += weights(i) * temp_distance;
  }

  vScSummary::number_of_points = vScCountPoints(classifier_ids);
  vScSummary::number_of_orientations = vScCountOrientations(classifier_ids);
  vScSummary::number_of_shape_contexts = vScCountScs(classifier_ids);
  vScSummary::number_of_images = vScCountImages(classifier_ids);

  vScSummary summary1, summary2;

  long current_prototype = -1;
  for (i = 0; i < number; i++)
  {
    long id = classifier_ids(i);
    vScClassifier scc(id);
    
    if (current_prototype != scc.prototype)
    {
      current_prototype = scc.prototype;
      summary1.AddFeatures(alignmentsc1[i], scc.prototype, classifier_ids);
      summary2.AddFeatures(alignmentsc2[i], scc.prototype, classifier_ids);
    }
  }

  vMatrix<float> reordered_weights = vScReorderWeights(classifier_ids, weights);
  float distance2 = summary1.Compare(&summary2, reordered_weights);
  vPrint("distance1 = %f, distance2 = %f\n", distance1, distance2);

  return 1;
}


long vInterpreter4::ScSummaryFilterRefine()
{
  char buffer[200];
  long dimensions = 0, to_keep = 0, max_k = 0;
  vPrint("\nEnter filename, dimensions, to_keep, max_k:\n");
  vScan("%s %li %li %li", buffer, &dimensions, &to_keep, &max_k);

  vMatrix<float> result = vScSummaryFilterRefine(buffer, dimensions, to_keep, max_k);
  if (result.IsValid() <= 0)
  {
    vPrint("failed to get results\n");
    return 0;
  }

  else
  {
    long i;
    long min_index = -1;
    float min_error = (float) 1000;
    for (i = 0; i <= max_k; i++)
    {
      float error = result(i);
      vPrint("%li-nn error: %f\n", i, error);
      if (i != 0)
      { 
        if (error < min_error)
        {
          min_error = error;
          min_index = i;
        }
      }
    }
    vPrint("1-nn error: %f\n", result(1));
    vPrint("\nbest k: %li. best knn-error: %f\n", min_index, min_error);
  }

  return 1;
}


long vInterpreter4::UnipenLoadClassifiers()
{
  if (bm == 0) 
  {
    return 0;
  }

  bm->ClearClassifierResults();
  vMatrix<float> training_results = vMatrix<float>::Read("d:\\users\\athitsos\\data\\experiments\\bm_datasets\\uni1_data\\training_features.bin");
  if (training_results.IsValid() <= 0)
  {
    vPrint("failed to load training results\n");
  }

  vMatrix<float> validation_results = vMatrix<float>::Read("d:\\users\\athitsos\\data\\experiments\\bm_datasets\\uni1_data\\validation_features.bin");
  if (validation_results.IsValid() <= 0)
  {
    vPrint("failed to load validation results\n");
  }

  if (training_results.Rows() != validation_results.Rows())
  {
    vPrint("training rows != validation rows\n");
  }

  long rows = training_results.Rows();
  vMatrix<long> classifier_ids = vRangeMatrix(1, rows);
  float max_entry = function_image_maximum(&training_results);
  bm->SetClassifiers(training_results, validation_results, classifier_ids, max_entry);

  return 1;
}


long vInterpreter4::BmSetSubset()
{
  char original[1000], subset[1000];
  long test_start = 0, test_size = 0, training_start = 0, training_size = 0;

  vPrint("\nenter original, subset, test_start, test_size, training_start, training_size:\n");
  vScan("%s %s %li %li %li %li", original, subset, &test_start, &test_size,
        &training_start, &training_size);

  long success = BoostMap_dataset::OriginalSubdataset(original, subset, 
                                                test_start, test_size, 
                                                training_start, training_size);

  if (success <= 0)
  {
    vPrint("failed to save subset\n");
  }
  else
  {
    vPrint("saved subset successfully\n");
  }

  return 1;
}


long vInterpreter4::VpNew()
{
  char dataset[1000];
  vPrint("\nEnter dataset:\n");
  vScan("%s", dataset);

  VpDelete();
  vp_tree = new vVpTree(dataset);
  vPrint("vp_tree ready\n");
  return 1;
}


long vInterpreter4::VpDelete()
{
  vdelete(vp_tree);
  vp_tree = 0;

  return 1;
}


long vInterpreter4::VpPrint()
{
  if (vp_tree == 0)
  {
    return 0;
  }

  long levels = 0;
  vPrint("\nEnter levels:\n");
  vScan("%li", &levels);
  vp_tree->Print(levels);

  return 1;
}


long vInterpreter4::VpLoad()
{
  char filename[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", filename);

  VpDelete();
  vp_tree = new vVpTree();
  long success = vp_tree->Load(filename);
  if (success <= 0)
  {
    vPrint("failed to load vp tree from %s\n", filename);
    VpDelete();
  }

  vPrint("loaded vp tree successfully\n");
  return 1;
}


long vInterpreter4::VpSave()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  char filename[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", filename);

  long success = vp_tree->Save(filename);
  if (success <= 0)
  {
    vPrint("failed to save vp tree to %s\n", filename);
  }

  vPrint("saved vp tree successfully\n");

  return 1;
}


long vInterpreter4::VpTestRange()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  char dataset[1000];
  long index = 0;
  float radius = (float) 0;
  vPrint("\nEnter dataset, index, radius:\n");
  vScan("%s %li %f", dataset, &index, &radius);

  vp_tree->TestRangeSearch(dataset, index, radius);
  return 1;
}


long vInterpreter4::VpTestNn()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  char dataset[1000];
  long index = 0;
  vPrint("\nEnter dataset, index:\n");
  vScan("%s %li", dataset, &index);

  vp_tree->TestNnSearch(dataset, index);
  return 1;
}


long vInterpreter4::VpTestKnn()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  char dataset[1000];
  long index = 0, number = 0;
  vPrint("\nEnter dataset, index, number:\n");
  vScan("%s %li %li", dataset, &index, & number);

  vp_tree->TestKnnSearch(dataset, index, number);
  return 1;
}


long vInterpreter4::VpNnStats()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  char dataset[1000];
  vPrint("\nEnter dataset:\n");
  vScan("%s", dataset);

  float average_distances = (float) 0;
  vp_tree->NnSearchStats(dataset, &average_distances);
  return 1;
}


long vInterpreter4::VpKnnStats()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  long number = 0;
  char dataset[1000];
  vPrint("\nEnter dataset, number:\n");
  vScan("%s %li", dataset, & number);

  float average_distances = (float) 0;
  vp_tree->KnnSearchStats(dataset, &average_distances, number);
  return 1;
}


long vInterpreter4::VpKnnClassification()
{
  if (vp_tree == 0)
  { 
    return 0;
  }
  
  long number = 0;
  char dataset[1000];
  vPrint("\nEnter dataset, number:\n");
  vScan("%s %li", dataset, & number);

  float average_distances = (float) 0;
  vMatrix<float> result = vp_tree->KnnClassificationStats(dataset, 
                                                          &average_distances, number);

  if (result.IsValid() <0)
  {
    vPrint("failed to estimate classification error\n");
    return 0;
  }

  PrintClassificationError(result);
  return 1;
}


long vInterpreter4::TestInsertDistance()
{
  vMatrix<long> neighbors(1, 10);
  vMatrix<float> distances(1, 10);

  long index;
  for (index = 0; index < 10; index++)
  {
    neighbors(index) = 10 * index;
    distances (index) = (float) (100 -10*index);
  }

  neighbors.Print ("neighbors");
  distances.Print("distances");

  while (1)
  {
    float distance = 0;
    long neighbor = 0;
    vPrint("\nenter distance, neighbor:\n");
    vScan ("%f, %li", & distance, & neighbor);
    if (distance <0)
    {
      break;
    }

    vVpTree::InsertDistance(distance, neighbor, distances, neighbors);
    neighbors.Print ("neighbors");
    distances.Print("distances");
  }

  return 1;
}


long vInterpreter4::conference_database_new()
{
  char buffer [1000];
  vPrint("\nenter database name:\n");
  vScan("%s", buffer);

  vdelete (line_drawings);
  line_drawings = new conference_database (buffer);
  if (line_drawings->check_valid () == 0)
  {
    vPrint ("failed to load line_drawings from %s\n", buffer);
    vdelete (line_drawings);
    line_drawings = 0;
  }
  else
  {
    vPrint ("successfully loaded line_drawings\n");
  }

  return 1;
}

long vInterpreter4::conference_database_delete ()
{
  vdelete (line_drawings);
  line_drawings = 0;
  return 1;
}


long vInterpreter4::conference_category()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  vPrint ("\nenter index:\n");
  vScan ("%li", & index);
  const char* category = line_drawings->get_category (index);
  vPrint ("the category is %s\n", category);
  
  return 1;
}

long vInterpreter4::conference_filename()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  vPrint ("\nenter index:\n");
  vScan ("%li", & index);
  const char* filename = line_drawings->get_filename (index);
  vPrint ("the filename is %s\n", filename);
  
  return 1;
}

long vInterpreter4::conference_load_images()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  line_drawings->load_images ();
  return 1;
}

long vInterpreter4::conference_distance_transforms()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  line_drawings->compute_distance_transforms ();
  return 1;
}

long vInterpreter4::conference_image()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  vPrint ("\nenter index:\n");
  vScan ("%li", & index);

  class_grayscale_image * image = line_drawings->get_image(index);
  ShowImage (vCopyImage(image));
  return 1;
}


long vInterpreter4::conference_transform()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  vPrint ("\nenter index:\n");
  vScan ("%li", & index);

  class_grayscale_image * image = line_drawings->get_distance_transform(index);
  ShowImage (vCopyImage(image));
  return 1;
}


long vInterpreter4::conference_chamfer_features()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  vPrint ("\nenter index:\n");
  vScan ("%li", & index);

  class_grayscale_image * image = line_drawings->get_image(index);
  ShowImage (vCopyImage(image));
  if (image == 0)
  {
    return 0;
  }
  
  class_chamfer_features * features = line_drawings->make_chamfer_features(image);
  long number = features->point_sequence->NumberOfFrames();
  
  long current;
  for (current = 0; current < number; current ++)
  {
//    class_grayscale_image * image = features->distance_transforms [current];
    class_grayscale_image * image = features->point_sequence->PointImage(current);
    ShowImage (image);
  }

  long i;
  for (i = 0; i < number; i++)
  {
    vdelete(features->distance_transforms[i]);
  }
  vdelete2(features->distance_transforms);
  vdelete(features->point_sequence);
  vdelete (features);
  return 1;
}


long vInterpreter4::conference_one_chamfer()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index_one = 0, index_two = 0;
  vPrint ("\nenter index_one, index_two:\n");
  vScan ("%li %li", & index_one, & index_two);

  class_grayscale_image * image = line_drawings->get_image(index_one);
  if (image == 0)
  {
    return 0;
  }
  ShowImage (vCopyImage(image));
  class_grayscale_image * image2 = line_drawings->get_image(index_two);
  if (image2 == 0)
  {
    return 0;
  }
  ShowImage (vCopyImage(image2));

  long rotation_index = 0;
  class_chamfer_features * features = line_drawings->make_chamfer_features(image);
  long number = features->point_sequence->NumberOfFrames();
  float distance = line_drawings->chamfer_distance(features, index_two, & rotation_index);

  long i;
  for (i = 0; i < number; i++)
  {
    vdelete(features->distance_transforms[i]);
  }
  vdelete2(features->distance_transforms);
  vdelete(features->point_sequence);
  vdelete (features);

  vPrint ("distance = %f, rotation_index = %li\n", distance, rotation_index);
  return 1;
}


long vInterpreter4::conference_one_Hausdorff()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index_one = 0, index_two = 0;
  vPrint ("\nenter index_one, index_two:\n");
  vScan ("%li %li", & index_one, & index_two);

  class_grayscale_image * image = line_drawings->get_image(index_one);
  if (image == 0)
  {
    return 0;
  }
  ShowImage (vCopyImage(image));
  class_grayscale_image * image2 = line_drawings->get_image(index_two);
  if (image2 == 0)
  {
    return 0;
  }
  ShowImage (vCopyImage(image2));

  long rotation_index = 0;
  class_chamfer_features * features = line_drawings->make_chamfer_features(image);
  long number = features->point_sequence->NumberOfFrames();
  float distance = line_drawings->Hausdorff_distance(features, index_two, & rotation_index);

  long i;
  for (i = 0; i < number; i++)
  {
    vdelete(features->distance_transforms[i]);
  }
  vdelete2(features->distance_transforms);
  vdelete(features->point_sequence);
  vdelete (features);

  vPrint ("distance = %f, rotation_index = %li\n", distance, rotation_index);
  return 1;
}


long vInterpreter4::conference_edges()
{
  if (input == 0) return 0;
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  grayscale_image * image1 = input->current_image->ToGrayCopy();
  class_grayscale_image * image = line_drawings->extract_edges(image1);
  ShowImage (vCopyImage(image));
  vdelete(image1);
  return 1;
}


long vInterpreter4::conference_chamfer2 ()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0, number = 0;
  vPrint ("\nenter index, number:\n");
  vScan ("%li %li", & index, & number);
  class_matrix_float distances = line_drawings->compute_chamfer_scores(index);
  class_matrix_long accuracy = line_drawings->get_accuracy (index, distances, number);
  accuracy.Print("accuracy");
  return 1;
}


long vInterpreter4::conference_Hausdorff2()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0, number = 0;
  vPrint ("\nenter index, number:\n");
  vScan ("%li %li", & index, & number);
  class_matrix_float distances = line_drawings->compute_Hausdorff_scores(index);
  class_matrix_long accuracy = line_drawings->get_accuracy (index, distances, number);
  accuracy.Print("accuracy");
  return 1;
}

long vInterpreter4::conference_chamfer ()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0, number = 0;
  vPrint ("\nenter number:\n");
  vScan ("%li", & number);

  vPrint("\n\n");
  for (index = 0; index < line_drawings->get_number_images(); index++)
  {
    class_matrix_float distances = line_drawings->compute_chamfer_scores(index);
    class_matrix_long accuracy = line_drawings->get_accuracy (index, distances, number);

    long current;
    vPrint("%li ", index);
    for (current = 0; current < number; current++)
    {
      vPrint("%li ", accuracy (current, 0));
    }
    vPrint("\n");
  }

  vPrint("\n");
  return 1;
}


long vInterpreter4::conference_Hausdorff()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0, number = 0;
  vPrint ("\nenter number:\n");
  vScan ("%li", & number);

  vPrint("\n\n");
  for (index = 0; index < line_drawings->get_number_images(); index++)
  {
    class_matrix_float distances = line_drawings->compute_Hausdorff_scores(index);
    class_matrix_long accuracy = line_drawings->get_accuracy (index, distances, number);

    long current;
    vPrint("%li ", index);
    for (current = 0; current < number; current++)
    {
      vPrint("%li ", accuracy (current, 0));
    }
    vPrint("\n");
  }

  vPrint("\n");
  return 1;
}


long vInterpreter4::conference_save()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;
  char buffer [1000];
  vPrint ("\nenter filename:\n");
  vScan ("%s", buffer);
  char* pathname = vJoinPaths(data_directory, buffer);
  vVideoFile * output = vOpenWriteVideoFile(pathname, 150, 150);
  if (output->IsValid() <= 0)
  {
    vPrint("failed to open %s\n", pathname);
    vdelete2 (pathname);
    return 0;
  }

  vPrint("\n");
  for (index = 0; index < line_drawings->get_number_images(); index++)
  {
    class_grayscale_image * image = line_drawings->get_image(index);
    long success = output->write_frameGray(image);
    if (success <= 0)
    {
      vPrint ("failed to save frame %li\n", index);
    }
    else
    {
      vPrint ("process frame %li of %li\r", index + 1, 
              line_drawings->get_number_images());
    }
  }
  vPrint("\n");

  long success = output->FinishWriting();
    if (success <= 0)
    {
      vPrint ("failed to finish writing\n", index);
    }
    else
    {
      vPrint ("finished writing sequence\n", index + 1, 
              line_drawings->get_number_images());
    }
  
  vCloseVideoFile(output);
  vdelete2 (pathname);
  vPrint("\n");
  return 1;
}


long vInterpreter4::conference_save2()
{
  if (line_drawings == 0)
  {
    vPrint ("no line drawings have been loaded\n");
    return 0;
  }

  long index = 0;

  vPrint("\n");
  for (index = 0; index < line_drawings->get_number_images(); index++)
  {
    class_grayscale_image * image = line_drawings->get_image(index);
    char* pathname2 = vJoinPaths3(data_directory, "temp1", line_drawings->get_filename (index));
    char* pathname = vMergeStrings2(pathname2, ".pgm");
    long success = image->Save(pathname);
    if (success <= 0)
    {
      vPrint ("failed to save %s", pathname);
    }
    else
    {
      vPrint ("process frame %li of %li\r", index + 1, 
              line_drawings->get_number_images());
    }
    vdelete2 (pathname);
    vdelete2 (pathname2);
  }
  vPrint("\n");

  vPrint("\n");
  return 1;
}


long vInterpreter4::function_file_copy ()
{
  char filename1 [1000];
  char filename2 [1000];
  long block_size = 0;
  
  vPrint ("\nenter filename1, filename2, block size in megabytes:\n");
  vScan ("%s %s %li", filename1, filename2, & block_size);

  if ((block_size < 1) || (block_size > 1000))
  {
    vPrint ("block_size should be between 1 and 1000:\n");
    return 0;
  }

  FILE * input_file = fopen (filename1, vFOPEN_READ);
  if (input_file == 0)
  {
    vPrint ("failed to open file %s\n", filename1);
    return 0;
  }

  FILE * output_file = fopen (filename2, vFOPEN_WRITE);
  if (output_file == 0)
  {
    vPrint ("failed to open file %s\n", filename2);
    fclose (input_file);
    return 0;
  }

  long buffer_size = 1000000 * block_size;
  char * buffer = new char[buffer_size];
  if (buffer == 0)
  {
    vPrint (" failed to allocate memory for buffer\n");
    fclose (input_file);
    fclose (output_file);
    return 0;
  }

  long block_count = 0;
  while (1)
  {
    long bytes_read = fread (buffer, sizeof (char), buffer_size, input_file);
    vPrint ("read %8li bytes for block %8li\n", bytes_read, block_count);
    if (bytes_read == 0) 
    {
      break;
    }

    long bytes_written = fwrite (buffer, sizeof (char), bytes_read, output_file);
    vPrint ("wrote %8li bytes for block %8li\n", bytes_written, block_count);
    if (bytes_written != bytes_read) 
    {
      vPrint (" failure in write operation\n");
      break;
    }

    block_count ++;
  }

  fclose (input_file);
  fclose (output_file);
  vdelete2(buffer);
  return 1;
}


long vInterpreter4::image_average ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = function_image_average (&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long vInterpreter4::image_median ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image current(input->current_image);
  double result = function_image_median (&current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long vInterpreter4::image_absolute ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image * current = new double_image(input->current_image);
  function_image_absolute(current, current);
  show_image (current);

  return 1;
}


long vInterpreter4::image_add ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal(first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_add(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter4::image_subtract ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal(first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_subtract(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter4::image_multiply ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal(first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_multiply(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter4::image_divide ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal(first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_divide(image1, image2);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}



long vInterpreter4::image_diff ()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;

  general_image* first_image = input->image_buffer[temp1];
  general_image* second_image = input->image_buffer[temp2];
  if (image_dimensions_equal(first_image, second_image) == 0)
  {
    print ("\nimage dimensions are not equal\n");
    return 0;
  }

  double_image * image1 = new double_image(first_image);
  double_image * image2 = new double_image(second_image);
  double_image * result = function_image_subtract(image1, image2);
  function_image_absolute(result, result);

  show_image(result);

  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter4::image_total ()
{
  if ((input == 0) || (input->current_image == 0))
  {
    return 0;
  }

  double_image * current = new double_image(input->current_image);
  double result = function_image_total (current);
  print ("\nresult = %lf\n", result);
  return 1;
}


long vInterpreter4::save_filtered_frames()
{
  if ((input == 0) || (input->sequence == 0) || 
      (current_output == -1))
  {
    return 0;
  }

  char buffer [1000];
  long start = 0, finish = 0;
  print ("\nenter name of script file, start frame, end frame:\n");
  question ("%s %li %li", buffer, & start, & finish);

  long frames = input->number_of_frames;
  if (start < 0)
  {
    start = 0;
  }
  if ((finish >= frames) || (finish < 0))
  {
    finish = frames -1;
  }

  long frame;
  for (frame = start; frame <= finish; frame ++)
  {
    print("frame = %li\n", frame);
    make_current_frame(frame);
    base_interpreter->process_command_file1 (buffer);
    write_frame();
  }

  return 1;
}


// like save_filtered_frames, but here we just do the processing,
// we don't write to any output sequence.
long vInterpreter4::process_sequence()
{
  if ((input == 0) || (input->sequence == 0))
  {
    return 0;
  }

  char buffer [1000];
  long start = 0, finish = 0;
  print ("\nenter name of script file, start frame, end frame:\n");
  question ("%s %li %li", buffer, & start, & finish);

  long frames = input->number_of_frames;
  if (start < 0)
  {
    start = 0;
  }
  if ((finish >= frames) || (finish < 0))
  {
    finish = frames -1;
  }

  long frame;
  for (frame = start; frame <= finish; frame ++)
  {
    print("frame = %li\n", frame);
    make_current_frame(frame);
    base_interpreter->process_command_file1 (buffer);
  }

  return 1;
}


long vInterpreter4::random_image()
{
  if (input == 0)
  {
    default_input(100, 100);
  }

  long rows = 0, cols = 0;
  print("\nenter rows, cols:\n");
  question("%li %li", &rows, &cols);
  color_image * image = vRandomImage(rows, cols);
  show_image(image);

  return 1;
}


long vInterpreter4::resize()
{
  long width = 0, height = 0;
  print("\nenter height, width:\n");
  question("%li %li", &height, &width);

  window->Resize(height, width);
  return 1;
}


long vInterpreter4::add_ax_match(const char * filename)
{
  long length = string_length (filename);
  char* ending;
  char* alternative_ending;
  const char* shifted;
  long ending_length, position;
  
  ending = "_input.avi";
  alternative_ending = "_track.avi";
  ending_length = string_length (ending);

  position = length - ending_length;
  shifted = &(filename [position]);
  if (position >= 0)
  {
    if (function_compare_strings (ending, shifted) == 0)
    {
      char buffer [1000];
      memcpy(buffer, filename, position);
      strcpy(&(buffer[position]), alternative_ending);
      AddInputSilent (buffer);
      return 1;
    }
  }

  ending = "_track.avi";
  alternative_ending = "_input.avi";
  ending_length = string_length (ending);

  position = length - ending_length;
  shifted = &(filename [position]);
  if (position >= 0)
  {
    if (function_compare_strings (ending, shifted) == 0)
    {
      char buffer [1000];
      memcpy(buffer, filename, position);
      strcpy(&(buffer[position]), alternative_ending);
      AddInputSilent (buffer);
      return 1;
    }
  }

  return 1;
}


long vInterpreter4::test_canonical_orientation()
{
  float orientation = 0.0f;
  print ("\nenter orientation:\n");
  question ("%f", & orientation);
  float result = canonical_orientation(orientation);
  print ("result = %f\n", result);
  return 1;
}


// counts the number of pixels with higher intensity than the specified pixel
long vInterpreter4::pixel_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channels = current->channels ();
  if (channels != 1)
  {
    print ("\nthis image has %li channels, pixel_rank assumes one channel\n");
    return 1;
  }

  long vertical = 0, horizontal = 0;
  print ("\nenter vertical, horizontal:\n");
  question ("%li %li", & vertical, & horizontal);

  double value = current->double_value_safe (0, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_delete (thresholded);

  return 1;
}


// same as pixel_rank, but in addition, this function also shows
// a thresholded image using the intensity of the specified pixel as a threshold.
long vInterpreter4::pixel_threshold_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channels = current->channels ();
  if (channels != 1)
  {
    print ("\nthis image has %li channels, pixel_rank assumes one channel\n");
    return 1;
  }

  long vertical = 0, horizontal = 0;
  print ("\nenter vertical, horizontal:\n");
  question ("%li %li", & vertical, & horizontal);

  double value = current->double_value_safe (0, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


// here the entry at each band is treated as a separate entry
long vInterpreter4::entry_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channel = 0, vertical = 0, horizontal = 0;
  print ("\nenter channel, vertical, horizontal:\n");
  question ("%li %li %li", & channel, & vertical, & horizontal);

  double value = current->double_value_safe (channel, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_delete(thresholded);

  return 1;
}


long vInterpreter4::entry_threshold_rank ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long channel = 0, vertical = 0, horizontal = 0;
  print ("\nenter channel, vertical, horizontal:\n");
  question ("%li %li %li", & channel, & vertical, & horizontal);

  double value = current->double_value_safe (channel, vertical, horizontal);
  print("value = %lf\n", value);
  uchar_image* thresholded = ::threshold_entries (current, value);
  long count = count_positive_entries (thresholded);
  long size = current->Size ();

  float fraction = ((float) count) / (float) size;
  print ("\n%li pixels, %f %%\n", count, 100.0f*fraction);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}

  
long vInterpreter4::threshold_entries ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  double value = 0;
  print ("\nenter value:\n");
  question ("%lf", & value);

  uchar_image* thresholded = ::threshold_entries (current, value);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


long vInterpreter4::inverse_threshold_entries ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  double value = 0;
  print ("\nenter value:\n");
  question ("%lf", & value);

  uchar_image* thresholded = ::inverse_threshold_entries (current, value);
  function_image_multiply (thresholded, (uchar) 255, thresholded);
  show_image(thresholded);

  return 1;
}


long vInterpreter4::channel_total ()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  general_image* result = current->SumBands();
  show_image (result);

  return 1;
}


long vInterpreter4::select_rectangle()
{
  general_image* current = current_image ();
  if (current == 0)
  {
    return 1;
  }

  long top = 0, left = 0, vertical = 0, horizontal = 0;
  print("\nenter top, to left, vertical, horizontal:\n");
  question("%li %li %li %li", & top, & left, & vertical, & horizontal);
  long right = left + horizontal - 1;
  long bottom = top + vertical - 1;

  general_image* result = current->copy(left, right, top, bottom);
  show_image (result);

  return 1;
}


long vInterpreter4::load_image()
{
  if (input == 0) 
  {
    return 0;
  }

  char buffer[1000];
  print("\nenter filename:\n");
  question("%s", buffer);

  general_image * image = general_image::Read(buffer);
  show_image(image);

  return 1;
}


long vInterpreter4::double_threshold()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  if (original->channels() != 1)
  {
    print("\nimage has more than one band\n");
    return 1;
  }

  float_matrix image(original);
  float small_threshold = 0, big_threshold = 0;
  print("\nenter small_threshold, big_threshold:\n");
  question("%f %f", & small_threshold, & big_threshold);

  float_matrix result = ::double_threshold(image, small_threshold, big_threshold);
  show_image(result.copy());
  return 1;
}


long vInterpreter4::connected_size_threshold()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  if (original->channels() != 1)
  {
    print("\nimage has more than one band\n");
    return 1;
  }

  long threshold = 0;
  print("\nenter threshold:\n");
  question("%li", & threshold);

  float_image image(original);
  float_image result = ::connected_size_threshold(image, threshold);
  show_image(result.copy());

  return 1;
}


long vInterpreter4::find_smallest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  print("\nenter rank:\n");
  question("%li", &rank);

  uchar_image result = find_smallest_entries(&image, rank);
  show_image(result.copy());
  return 1;
}


long vInterpreter4::keep_smallest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  float replacement = 0.0f;
  print("\nenter rank, replacement:\n");
  question("%li %f", &rank, & replacement);

  float_image result = keep_smallest_entries(&image, rank, replacement);
  show_image(result.copy());
  return 1;
}


long vInterpreter4::find_largest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  print("\nenter rank:\n");
  question("%li", &rank);

  uchar_image result = find_largest_entries(&image, rank);
  show_image(result.copy());
  return 1;
}


long vInterpreter4::keep_largest()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float_matrix image(original);
  long rank = 0;
  float replacement = 0.0f;
  print("\nenter rank, replacement:\n");
  question("%li %f", &rank, & replacement);

  float_image result = keep_largest_entries(&image, rank, replacement);
  show_image(result.copy());
  return 1;
}


long vInterpreter4::set_range()
{
  general_image * original = current_image();
  if (original == 0)
  {
    return 0;
  }

  float minimum = 0, maximum = 0;
  print("\nenter minimum, maximum:\n");
  question("%f %f", & minimum, & maximum);
  
  float range = maximum - minimum;

  float_matrix image(original);
  float image_minimum = function_image_minimum(&image);
  float image_maximum = function_image_maximum(&image);
  float image_range = image_maximum - image_minimum;
  image = image - image_minimum;
  if (image_range != 0.0f)
  {
    image = image / image_range;
  }
  image = image * range;
  image = image + minimum;
  show_image(image.copy());

  return 1;
}
