#ifndef VASSILIS_INTERPRETER4_H
#define VASSILIS_INTERPRETER4_H


#include "interpreter3.h"
#include "shape_context/sc_mnist.h"
#include "shape_context/sc_alignment.h"
#include "algorithms/vp_tree.h"
#include "datasets/cikm.h"

class vInterpreter4 : public vInterpreter3
{
protected:
  vScMnist * sc_mnist;

  // alignments of 20,000 training objects to a single prototype.
  vArray(vScAlignmentc) training_alignments;

  // alignments of 10,000 test objects to a single prototype.
  vArray(vScAlignmentc) test_alignments;

  vVpTree * vp_tree;
  conference_database * line_drawings;

public:
  vInterpreter4(class_interpreter* in_interpreter);
  ~vInterpreter4();

  virtual void Initialize(vGestureInterfaceGlobals * in_globals);

  // An interesting thing with the CleanUp functions of each interpreter
  // (i.e. interpreter1, 2, 3), is that each of them gets called by
  // the destructor of that class. I don't understand why that happens.
  // I would expect that, since the CleanUp functions are all virtual,
  // only the CleanUp function of the lowest subclass (interpreter4
  // currently) should be called. I don't know if the way things work 
  // now is according to the C++ standard or if it is a Virtual Studio bug.
  virtual void CleanUp();

  long CCodeDir();
  long DCodeDir();
  long RCodeDir();
  long LCodeDir();

  long CDataDir();
  long DDataDir();
  long EDataDir();
  long FDataDir();
  long RDataDir();
  long SDataDir();
  long LDataDir();

  long SetCodeDir();
  long SetDataDir();
  long SetNessie();


  ////////////////////////////////////////////////////////////////////////
  //                                                                    //
  // beginning of functions written for submissions to NIPS 2004        //
  // and ICDE 2005                                                      //
  //                                                                    //
  ////////////////////////////////////////////////////////////////////////

  // I currently have a bug in the gaussj function. The original version 
  // assumed that matrix indices start at 1, and I made a modified version
  // called gaussj2 that assumes that indices start at 0. The new function
  // gives different results. These functions here are useful for 
  // finding the bug.
  long TestGaussj();
  long TestGaussj2();
  long TimeGaussj2();

  long PrecomputeSqrtf();
  long TestSqrtf();
  long TimePrecomputedSqrtf();
  long TimeSqrtf();

  // this function takes in a pathname (where a matrix is saved)
  // and a (row,col) specification.
  // It prints out header information for the matrix, and information
  // about that particular entry.
  long MatrixInfo();
    
  // TransposeMatrix allows a user to load a matrix from a file
  // (assuming for the time being that it is in float format) and
  // save its transpose into another file.
  long TransposeMatrix();

  // this function takes in the pathnames for two matrices (assumed
  // to have the same number of columns, and to be in float format)
  // and the pathname of an output file. A new matrix is constructed
  // by putting the first matrix on top of the second matrix, and 
  // the new matrix is saved on disk.
  long MergeVerticallyFloat();

  long ScmNew();
  long ScmDelete();
  long ScmTrainingData();
  long ScmTestData();
  long ScmTrainingBoundary();
  long ScmTestBoundary();
  long ScmOriginalSet();
  long ScmValidationSet();

  long ScTestInterpolate();

  // Protein dataset
  long PsQueryDatabase();
  long PsQueryQuery();

  long PsQdDistance();
  long PsQqDistance();

  // Creates a traintrain_distances file appropriate for
  // creating boostmap training datasets.
  long PsTrain1();

  // create a dataset appropriate for measuring index errors
  // (for proteins)
  long PsTest2();
  long PsRef2();

  // time series dataset
  long TsCompileDistances();
  long TsBreakDistances();
  long TsTestTrainDistances();
  long TsTestTestDistances();
  long TsTrainTrainDistances();
  long TsPermute();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  // end of functions written for submissions to NIPS 2004        //
  // and ICDE 2005                                                //
  //                                                              //
  //////////////////////////////////////////////////////////////////


  //////////////////////////////////////////////////////////////////
  //                                                              //
  // beginning of functions written for applying BoostMap and     //
  // Boost-NN to hands                                            //
  //                                                              //
  //////////////////////////////////////////////////////////////////
  
  // Creates a giant file with distances between all pairs of
  // images from the 107,328 hand poses in the database.
  long AcdTrainingDistances();

  // creates match scores based on the chamfer distance. It assumes
  // that the current image is an edge image.
  long ChamferRankings();

  // saves chamfer distances from all frames in current input to
  // all database poses.
  long TestTrainChamfer();

  long DirectionChanges();

  long DominantOrientation1();

  long DirectionHistogram();

  long PoserCrop();

  // this function merges the 26 subsequences of real poser 1 
  // (like the poser1 database, but with realistic renderings).
  long MergeRealPoser1();

  // We create an input based on the "real" sequence of the
  // hand database.
  long RealSequence();

  long PixelOrientationsTest();

  long BnnqsCreate0();
  long BnnqsCreate(long training, long validation);
  long BnnqsPrintSummary();
  long BnnqsPrintTraining();
  long BnnqsPrintValidation();
  long BnnqsTestTriples();
  long BnnqsTripleStats();
  long BnnqsConfusions();
  long BnnqsClassifier();
  long BnnqsTriple();
  long BnnqsWeight();
  long BnnqsWeights();
  long BnnqsSomeWeights();
  long BnnqsWeightSum();
  long BnnqsSortWeights();
  long BnnqsZContributions();
  long BnnqsKnnErrors();
  long BnnqsRankInfo();
  long BnnqsKnnInfo();

  long BnnqsStep0();
  long BnnqsStep(const char * buffer);
  long BnnqsSteps();
  long BnnqsNewBNN0();
  long BnnqsNewBNN(long max_k);
  long BnnqsNewBNN2();
//  long BnnqsBacktrack();
  long BnnqsLoad();
  long BnnqsSave();
  long BnnqsAllowNegative();

  float BnnqsTrain0();
  float BnnqsTrain(long triples);
  float BnnqsTrain2();
  float BnnqsTrain3();
  long BnnqsTrainLoop();
  long BnnqsTrainLoop2();

  // The right way to cross-validate.
  long BnnqsCrossValidate();
  long BnnqsAddClassifier();
  long BnnqsAddClassifiers0();
  long BnnqsAddClassifiers(long start, long end, float weight);
  long BnnqsAnalyzeTriples();
  long BnnqsAnalyzeTestTriples();
  long BnnqsBadTriples();
  long BnnqsBadTriples2();
  long BnnqsBadNumberHistogram();
  long BnnqsQNumbers();
  long BnnqsQNumbers2();

  long BnnqsRanges();
  long BnnqsStds();
  long BnnqsNormalizeRanges();
  long BnnqsNormalizeRanges2();
  long BnnqsNormalizeStds();
  long BnnqsKNNTraining();
  long BnnqsKNNTest();

  long BnnqsStepqs0();
  long BnnqsStepqs(long number);
  long BnnqsStepsqs();
  long BnnqsCheck();
  long BnnqsCheckTr();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  // end of functions written for applying BoostMap and           //
  // Boost-NN to hands                                            //
  //                                                              //
  //////////////////////////////////////////////////////////////////
  

  //////////////////////////////////////////////////////////////////
  //                                                              //
  // beginning of functions written for the PAMI paper on         //
  // BoostMap                                                     //
  //                                                              //
  //////////////////////////////////////////////////////////////////
  
  long BourgainTest();
  long BourgainSave();
  long BourgainIndexErrors();

  long SparseMapSave();
  long SparseMapIndexErrors();

  long BmsetOnSparseMap();
  long BmsmIndexErrors();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  // end of functions written for the PAMI paper on               //
  // BoostMap                                                     //
  //                                                              //
  //////////////////////////////////////////////////////////////////
  
  //////////////////////////////////////////////////////////////////
  //                                                              //
  // beginning of functions written for Joni's CVPR 2005          //
  // paper                                                        //
  //                                                              //
  //////////////////////////////////////////////////////////////////

  long LoadAlignments();
  long DeleteAlignments();
  long CheckAlignments();
  long ScAlignIndexError();
  long ScAlignIndexErrors();
  long ScSaveClassifiers();

  // This function is useful when we save the results of weak classifiers
  // to disk, and then we want to normalize them so that they are all
  // in the range [-1, 1].
  long RecordMaxEntry();

  long ReadMaxEntry();

  long BmLoadScClassifiers();

  // BmTripleResult and ScCheckTriple are used to verify that the 
  // results of weak classifiers saved in directories like
  // sc1_data are correct.
  long BmTripleResult();

  long ScCheckTriple();
  long ScCheckTriple(long triple, long classifier_id);
  long ScSummaryIndexErrors();

  // here we make up classifiers consisting of features 2-3 of 
  // specified prototypes, with equal weights.
  long ScPrototypeIndexErrors();

  // here we manually add classifiers to see the classification error we
  // get
  long ScAddClassifiers();

  // This function is used for debugging: for two objects, and a specified
  // weak classifier (corresponding to prototype, point, and feature), we
  // compare the corresponding distance between those two objects, using
  // vScAlignment and using vScSummary. The two distances should be the same.
  long ScClassifierDistance();

  long ScClassifierDistances();

  long ScSummaryFilterRefine();

  long UnipenLoadClassifiers();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  //       end of functions written for Joni's CVPR 2005          //
  //       paper                                                  //
  //                                                              //
  //////////////////////////////////////////////////////////////////

  long BmSetSubset();

  long VpNew();
  long VpDelete();
  long VpPrint();
  long VpLoad();
  long VpSave();
  long VpTestRange();
  long VpTestNn();
  long VpTestKnn();
  long VpNnStats();
  long VpKnnStats();
  long VpKnnClassification();

  long TestInsertDistance();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  //       start of functions written for CIKM 2005 paper         //
  //                                                              //
  //////////////////////////////////////////////////////////////////

  long conference_database_new();
  long conference_database_delete ();
  long conference_category();
  long conference_filename();
  long conference_load_images();
  long conference_distance_transforms();
  long conference_image();
  long conference_transform();
  long conference_chamfer_features();
  long conference_one_chamfer();
  long conference_one_Hausdorff();
  long conference_edges();

  long conference_chamfer2 ();
  long conference_Hausdorff2 ();
  long conference_chamfer ();
  long conference_Hausdorff ();
  long conference_save();
  long conference_save2();

  //////////////////////////////////////////////////////////////////
  //                                                              //
  //       end of functions written for CIKM 2005 paper           //
  //                                                              //
  //////////////////////////////////////////////////////////////////

  long function_file_copy();
  long image_average();
  long image_median();
  long image_absolute();
  long image_add();
  long image_subtract();
  long image_multiply();
  long image_divide();
  long image_diff();
  long image_total();

  // this function adds all the frames of the current input sequence
  // to the current output sequence, after it processes each frame
  // by running the commands in the given script file

  long save_filtered_frames();
  long process_sequence();
  long random_image();
  long resize();
  long add_ax_match(const char * filename);
  long test_canonical_orientation();

  // counts the number of pixels with higher intensity than the specified pixel
  long pixel_rank ();

  // same as pixel_rank, but in addition, this function also shows
  // a thresholded image using the intensity of the specified pixel as a threshold.
  long pixel_threshold_rank ();

  // here the entry at each band is treated as a separate entry
  long entry_rank ();
  long entry_threshold_rank ();
  
  long threshold_entries ();
  long inverse_threshold_entries ();
  long channel_total ();
  long select_rectangle();
  long load_image();

  long double_threshold();
  long connected_size_threshold();

  long find_smallest();
  long keep_smallest();
  long find_largest();
  long keep_largest();

  long set_range();
};













#endif // VASSILIS_INTERPRETER4_H
