#include "stdafx.h"

#include "basics/image.h"
#include "basics/rle_encoding.h"
#include "basics/drawing_temp.h"
#include "matching.h"

long vMatchBinaryImages(grayscale_image * gray1, grayscale_image * gray2)
{
  long score = 0;
  long rows1 = gray1->Rows();
  long rows2 = gray2->Rows();
  long cols1 = gray1->Cols();
  long cols2 = gray2->Cols();

  long rows = Min(rows1, rows2);
  long cols = Min(cols1, cols2);

  vArray2(uchar) I1 = gray1->GetI2();
  vArray2(uchar) I2 = gray2->GetI2();
  long row, col;
  for (row = 0; row < rows; row++)
  {
    vArray(uchar) base1 = I1[row];
    vArray(uchar) base2 = I2[row];
    for (col = 0; col < cols; col++)
    {
      if (base1[col] == base2[col]) score++;
     else
      {
        score = score + 0;
      }
    }
  }

  return score;
}


long vFastMatch(vRLE_Binary * rle1, vRLE_Binary * rle2, long rows, long cols)
{
  long result = 0;
  long row;
  vector<ushort> * data1 = rle1->GetChanges();
  vector<ushort> * data2 = rle2->GetChanges();
	for (row = 0; row < rows; row++)
	{
		long index = 1;
		vector<ushort> & entries1 = data1[row];
		vector<ushort> & entries2 = data2[row];
    long left_col1, right_col1, left_col2, right_col2;
    left_col1 = right_col1 = left_col2 = right_col2 = 0;
		ushort changes1 = entries1[0];
		ushort changes2 = entries1[0];
    long index2 = 1;
    long index1;
    long size1 = entries1.size();
    long change1, change2;
    change2 = entries2[index2++];
    right_col2 = change2;
    long value1 = 1;
    long value2 = 0;
    for (index1 = 1; index1 < size1; index1++)
    {
      change1 = entries1[index1];
      right_col1 += change1;
      value1 = 1 - value1;
    }    
  }
  return result;
}


long * vFastBinaryMatch(vRLE_Binary * rle1, vRLE_Sequence * sequence)
{
  long number_of_frames = sequence->NumberOfFrames();
  long * result = new long[number_of_frames*4];
  long rows = sequence->Rows();
  long cols = sequence->Cols();
  grayscale_image * gray1 = rle1->ToGrayImage();
  grayscale_image * gray2 = (grayscale_image *) vRotateRight(gray1);
  grayscale_image * gray3 = (grayscale_image *) vRotateRight(gray2);
  grayscale_image * gray4 = (grayscale_image *) vRotateRight(gray3);
  vRLE_Binary * rle2 = new vRLE_Binary(gray2);
  vRLE_Binary * rle3 = new vRLE_Binary(gray3);
  vRLE_Binary * rle4 = new vRLE_Binary(gray4);
  long i = 0;
  long index = 0;
  for (i = 0; i < 5; i++)
  {
    vRLE_Binary * image2 = (vRLE_Binary *) sequence->ReadRLEFrame(i);
    result[index++] = vFastMatch(rle1, image2, rows, cols);
    result[index++] = vFastMatch(rle2, image2, rows, cols);
    result[index++] = vFastMatch(rle3, image2, rows, cols);
    result[index++] = vFastMatch(rle4, image2, rows, cols);
    vdelete(image2);
    vPrint("Processed %li    \r", i);
  }
  
  vPrint("\n");

  vdelete(gray1);
  vdelete(gray2);
  vdelete(gray3);
  vdelete(gray4);
  vdelete(rle2);
  vdelete(rle3);
  vdelete(rle4);
  return result;
}


static const long frames_per_view = 12;

// The macro vStepDisplacement returns the bin displacement
// that occurs when we move from view 0 to view 1.
#define vStepDisplacement(bins) ((bins) - ((bins) / (frames_per_view * 2)))

// vNormalizedAngle(view) returns the number of stepwise
// counterclockwise rotations we need to apply to view 0 to
// obtain the view indexed by view (this is applicable to 
// views generated by the program save_views) Views 0-11 map
// to themselves, views 12-23 go to 36-47, views 24-35 go to
// themselves and views 36-47 go to 12-23.
#define vNormalizedAngle(i) \
(((i) % frames_per_view + (4 - (i)/frames_per_view) * frames_per_view) % (4 * frames_per_view))

// The macro vViewDisplacement returns the bin displacement that
// occurs when we move from view 0 to view i (i is between 0 and 48).
// So, the j-th bin of view 0 is equal to the bin indexed by
// (j + vViewDisplacement(bins, i)) % bins for frame i, 
// provided the number of
// bins in the histogram is the bins variable.
#define vViewDisplacement(bins, i) \
((vNormalizedAngle(i) * vStepDisplacement(bins)) % (bins))


vMatchScores *
vEdgeOrientationRanks(vMatrix<double> * training,
                       vMatrix<double> * test,
                       vMatchScores * prefiltered,
                       long number_to_keep)
{
  long bins = training->Cols();
  long saved_frames = training->Rows();
  long frames = saved_frames * frames_per_view * 4;
  if (bins != test->Cols()) return 0;
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) matrix = vnew(double, bins);
  vArray2(double) matrix2 = training->Matrix2();
  vArray(double) other_matrix = test->Matrix();

  long frame = 0;
  vMatrix<double> result_matrix(1, frames);
  vArray(double) result_array = result_matrix.Matrix();
  vArray(vMatchScore) scores = vnew(vMatchScore, number_to_keep + 1);

  long prefiltered_frame;

  for (prefiltered_frame = 0; prefiltered_frame < number_to_keep; prefiltered_frame++)
  {
    vMatchScore score = prefiltered->GetScore(prefiltered_frame);
    long database_frame = score.frame;
    long database_view = score.view;
    long viewpoint = database_frame / frames_per_view;
    vArray(double) training_histogram = matrix2[viewpoint];
    // view is the number, between 0-11, identifying the image plane
    // rotation of database_frame.
    long view = database_frame % frames_per_view;
    long rotation = database_view % 2;

    long displacement = vViewDisplacement(bins, view + rotation * 12);
    long bin;
    for (bin = 0; bin < bins; bin++)
    {
      matrix[(bin+displacement) % bins] = training_histogram[bin];
    }

    // Now, calculate the distance between rotated histogram 
    // (stored in matrix) and test histogram (stored in 
    // other_matrix).
    long start;
    double min_distance = bins * 10;
    long min_index = -1;

    // Calculate all cumulative distances, select the min.
    for (start = 0; start < bins; start++)
    {
      // Convert histograms into cumulative histograms with given start
      array1[0] = matrix[start];
      array2[0] = other_matrix[start];
      for (bin = 1; bin < bins; bin++)
      {
        long index = (start + bin) % bins;
        array1[bin] = array1[bin - 1] + matrix[index];
        array2[bin] = array2[bin - 1] + other_matrix[index];
      }

      // Calculate distance;
      double distance = 0;
      for (bin = 0; bin < bins; bin++)
      {
//          distance += vAbs(array1[bin] - array2[bin]);
        distance += 1 - Min(array1[bin], array2[bin]);
      }
      // Check if this distance is minimum found so far.
      if (distance < min_distance)
      {
        min_distance = distance;
        min_index = start;
      }
    }
    score.score = (float) min_distance;
    scores[prefiltered_frame] = score;
//    vPrint("290\n");
//    vScan("%li", &trash);
  }

  vdelete2(array1);
  vdelete2(array2);
  vdelete2(matrix);
  vMatchScores * result = new vMatchScores(scores, number_to_keep);
  return result;
}


// Here, instead of trying all possible starts to make the cumulative histogram,
// we only try a couple.
vMatchScores *
vEdgeOrientationRanksFast(vMatrix<double> * training,
                           vMatrix<double> * test,
                           vMatchScores * prefiltered,
                           long number_to_keep)
{
  long bins = training->Cols();
  vPrint("Bins = %li\n", bins);
  long saved_frames = training->Rows();
  long frames = saved_frames * frames_per_view * 4;
  if (bins != test->Cols()) return 0;
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) matrix = vnew(double, bins);
  vArray2(double) matrix2 = training->Matrix2();
  vArray(double) other_matrix = test->Matrix();

  long frame = 0;
  vMatrix<double> result_matrix(1, frames);
  vArray(double) result_array = result_matrix.Matrix();
  vArray(vMatchScore) scores = vnew(vMatchScore, number_to_keep + 1);

  long prefiltered_frame;

  for (prefiltered_frame = 0; prefiltered_frame < number_to_keep; prefiltered_frame++)
  {
    vMatchScore score = prefiltered->GetScore(prefiltered_frame);
    long database_frame = score.frame;
    long database_view = score.view;
    long viewpoint = database_frame / frames_per_view;
    vArray(double) training_histogram = matrix2[viewpoint];
    // view is the number, between 0-11, identifying the image plane
    // rotation of database_frame.
    long view = database_frame % frames_per_view;
    long rotation = database_view % 2;

    long displacement = vViewDisplacement(bins, view + rotation * 12);
    long bin;
    for (bin = 0; bin < bins; bin++)
    {
      matrix[(bin+displacement) % bins] = training_histogram[bin];
    }

    // Now, calculate the distance between rotated histogram 
    // (stored in matrix) and test histogram (stored in 
    // other_matrix).
    long start;
    double min_distance = bins * 10;
    long min_index = -1;

    // Calculate all cumulative distances, select the min.
    for (start = 0; start < bins; start+= bins/3)
    {
      // Convert histograms into cumulative histograms with given start
      array1[0] = matrix[start];
      array2[0] = other_matrix[start];
      for (bin = 1; bin < bins; bin++)
      {
        long index = (start + bin) % bins;
        array1[bin] = array1[bin - 1] + matrix[index];
        array2[bin] = array2[bin - 1] + other_matrix[index];
      }

      // Calculate distance;
      double distance = 0;
      for (bin = 0; bin < bins; bin++)
      {
//          distance += vAbs(array1[bin] - array2[bin]);
        distance += 1 - Min(array1[bin], array2[bin]);
      }
      // Check if this distance is minimum found so far.
      if (distance < min_distance)
      {
        min_distance = distance;
        min_index = start;
      }
    }
    score.score = (float) min_distance;
    scores[prefiltered_frame] = score;
//    vPrint("290\n");
//    vScan("%li", &trash);
  }

  vdelete2(array1);
  vdelete2(array2);
  vdelete2(matrix);
  vMatchScores * result = new vMatchScores(scores, number_to_keep);
  return result;
}
