#include "stdafx.h"


#include "particle_tracker.h"
#include "quaternion.h"
#include "gesture_data.h"

#include "basics/definitions.h"




vTrackingEstimate::vTrackingEstimate()
{
  probability = 0;
}


vTrackingEstimate::~vTrackingEstimate()
{
}
  

vTrackingObservation::vTrackingObservation()
{
  type = 0;
}


vTrackingObservation::~vTrackingObservation()
{
}



vParticleTracker::vParticleTracker()
{
  number_of_samples = 0;
  particles = vZero(vTrackingEstimate *);
}


vParticleTracker::~vParticleTracker()
{
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    vdelete(particles[i]);
  }
  vdelete2(particles);
}
  

vHandShapeEstimate::vHandShapeEstimate()
{
}


vHandShapeEstimate::vHandShapeEstimate(long in_database_frame, long in_database_view)
{
  static vFrameViewFinder frame_view_finder;
  database_frame = in_database_frame;
  database_view = in_database_view;
  frame_view_finder.GetViewParams(database_frame, database_view,
                                  &shape_id, &x_angle, &y_angle, &z_angle);
}


vHandShapeEstimate::~vHandShapeEstimate()
{
}


vHandShapeObservation::vHandShapeObservation()
{
}


vHandShapeObservation::vHandShapeObservation(vMatchScores * in_scores)
{
  type = ClassType();
  scores = in_scores;
}


vHandShapeObservation::~vHandShapeObservation()
{
}


long vHandShapeObservation::ClassType()
{
  return 1001;
}

/////////////////////////////////////////////////////////////////
//                                                             //
// Beginning of implementation of the class vHandShapeTracker //
//                                                             //
/////////////////////////////////////////////////////////////////



vHandShapeTracker::vHandShapeTracker()
{
  number_of_shapes = 26;
  number_of_viewpoints = 86;
  number_of_orientations = 48;
  number_of_poses = number_of_viewpoints * number_of_orientations;
  number_of_hand_views = number_of_poses * number_of_shapes;
  ZeroAll();

  SetNumberOfSamples(107328);
  PrecomputeAngleDistances();
}


vHandShapeTracker::~vHandShapeTracker()
{
  ClearAll();
}


long vHandShapeTracker::ZeroAll()
{
  particles = vZero(vTrackingEstimate *);
  number_of_samples = 0;
  return 1;
}


long vHandShapeTracker::ClearAll()
{
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    vHandShapeEstimate * estimate = (vHandShapeEstimate *) particles[i];
    vdelete(estimate);
  }
  vdelete2(particles);
  ZeroAll();
  return 1;
}


long vHandShapeTracker::PrecomputeAngleDistances()
{
  // Precompute the z_angle indices.
  long i;
  for (i = 0; i < 12; i++)
  {
    z_angle_index[i][0] = i;
    z_angle_index[i][1] = i+36;
    z_angle_index[i][2] = i+24;
    z_angle_index[i][3] = i+12;
  }
  
  // Precompute the angle distances, unless it is already saved in a file
  angle_distances_matrix = vMatrix<uchar>(number_of_viewpoints, number_of_poses);
  angle_distances = angle_distances_matrix.Matrix2();
  char * angle_distances_file = 
    vJoinPaths(sequences_directory, "glove_data\\training\\training7.angle_distances");
  FILE * fp = fopen(angle_distances_file, vFOPEN_READ);
  if (fp != 0)
  {
    long success = angle_distances_matrix.ReadBandwise(fp);
    if (success <= 0)
    {
      exit_error("can't read angle distances file %s\n", angle_distances_file);
    }
    fclose(fp);
  }
  else
  {
    vPrint("precomputing angle distances\n");
    double x_angle1, y_angle1, x_angle2, y_angle2;
    long angle_index1, angle_index2, rotation2, z_orientation2;

    long number_of_angles = view_finder.NumberOfAngles();
    long number_of_orientations = view_finder.NumberOfOrientations();
    vPrint("%15s %15s\n", "angle1", "angle2");
    for (angle_index1 = 0; angle_index1 < number_of_angles; angle_index1++)
    {
      // frame2 will be the current frame in the database (between 0 and 1032)
      // index2_base will be the second index to use in angle_distances 
      // (between 0 and 4128), after we add to it the displacement corresponding
      // to a given frame and rotation
      long frame2 = -1;
      long index2_base = -number_of_orientations;
      x_angle1 = view_finder.GetXAngle(angle_index1);
      y_angle1 = view_finder.GetYAngle(angle_index1);

      for (angle_index2 = 0; angle_index2 < number_of_angles; angle_index2++)
      {
        vPrint("%15li %15li\r", angle_index1, angle_index2);
        index2_base += number_of_orientations;
        x_angle2 = view_finder.GetXAngle(angle_index2);
        y_angle2 = view_finder.GetYAngle(angle_index2);

        for (z_orientation2 = 0; z_orientation2 < number_of_orientations; z_orientation2++)
        {
          frame2++;
          for (rotation2 = 0; rotation2 < 4; rotation2++)
          {
            long z_index = z_angle_index[z_orientation2][rotation2];
            long index2 = index2_base + z_index;
            double z_angle2 = (360.0 / (double) number_of_orientations) * z_index;
            double distance = vOldViewDistance(x_angle1, y_angle1, 0, 
                                                x_angle2, y_angle2, z_angle2);
            if (distance > 181.0) 
            {
              exit_error("\nImpossible, angle distance over 180\n");
            }
            angle_distances[angle_index1][index2] = (uchar) distance;
            //vPrint("frame2 = %10li, index2=%10li, z_index = %10li\r",
            //         frame2, index2, z_index);
          }
        }
      }
    }
    vPrint("\n");
    fp = fopen(angle_distances_file, vFOPEN_WRITE);
    vPrint("Saving angle distances into %s\n", angle_distances_file);
    long success = angle_distances_matrix.WriteBandwise(fp);
    if (success <= 0)
    {
      exit_error("failed to save angle distances into %s\n", angle_distances_file);
    }
    fclose(fp);
  }
  
  // Precompute the angle probabilities
  for (i = 0; i < 360; i++)
  {
    angle_probabilities[i] = AngleDistanceProbability(i);
    //vPrint("angle_probabilities[%li] = %lf\n", i, angle_probabilities[i]);
  }

  vdelete2(angle_distances_file);
  return 1;
}


double vHandShapeTracker::RankProbability(long rank)
{
  const double rank0_probability = 0.1;
  const double log2 = log(2.0);
  const double last_rank_log = log((float) number_of_samples) / log((float) 2.0);
  const double A = rank0_probability;
  const double C = (1-A)/last_rank_log;
  static vMatrix<double> logs_matrix = vPrecalculateLogs2(number_of_samples * 2);
  static vArray(double) logs = logs_matrix.Matrix();

  if (rank < 0) return 0;
  if (rank == 0) return A;
  double result = C / (double) (rank+1);
  return result;
}


double vHandShapeTracker::AngleDistanceProbability(double distance)
{
  const double first_limit = 36;
  const double second_limit = 108;
  const double third_limit = 180;

  const double first_prob = 0.8 / first_limit;
  const double third_prob = 0.05 / (third_limit - second_limit);

  if (distance <= first_limit) return first_prob;
  if (distance >= second_limit) return third_prob;
  double difference = second_limit - first_limit;
  double weight2 = (distance - first_limit) / difference;
  double weight1 = 1 - weight2;
  double probability = weight1 * first_prob + weight2 * third_prob;
  return probability;
}



double vHandShapeTracker::CameraParamsProbability(double current_x, double current_y, 
                                                   double current_z,                              
                                                   double next_x, double next_y, 
                                                   double next_z)
{
  double distance = vOldViewDistance(current_x, current_y, current_z,
                                      next_x, next_y, next_z);
  return AngleDistanceProbability(distance);
}


double vHandShapeTracker::ShapeProbability(long current_shape_id, 
                                            long next_shape_id)
{
  if (current_shape_id == next_shape_id)
  {
    return 0.85;
  }
  else
  {
    return 0.15;
  }
}

  
long vHandShapeTracker::SetNumberOfSamples(long in_number_of_samples)
{
  ClearAll();
  if (number_of_hand_views != in_number_of_samples)
  {
    vPrint("Error: vHandShapeTracker::SetNumberOfSamples, %li\n",
            in_number_of_samples);
  }

  number_of_samples = in_number_of_samples;
  typedef vTrackingEstimate * vTrackingEstimatep;
  particles = vnew(vTrackingEstimatep, number_of_samples);
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    particles[i] = new vHandShapeEstimate();
  }

  return 1;
}


long vHandShapeTracker::Propagate()
{
  // count weights on every shape, use them for a first round of weight estimation
  // Then, for each estimate, find nearby estimates (within 108 degrees) of the 
  // same shape and use that for second round of estimates. Then, for every
  // estimate find all estimates with same camera params and different shapes,
  // and use that for third round of estimates.

  // Compute the probability of each shape, 
  long scores_per_pose = view_finder.ScoresPerPose();

  vArray(double) estimates0 = vnew(double, number_of_samples);
  vArray(double) estimates1 = vnew(double, number_of_samples);
  vArray(double) estimates2 = vnew(double, number_of_samples);

  // Put all estimate probabilities into the estimates0 array, 
  // to optimize speed
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    estimates0[i] = particles[i]->probability;
  }

  const long frames_per_pose = view_finder.FramesPerPose();
  const long number_of_orientations = view_finder.NumberOfOrientations();
  // For each estimate, go through all other estimates that belong to the same
  // shape id, and update its probability
  long shape_id, shape_base, pose1, pose2;
  vArray(double) weight_sums = vnew(double, number_of_shapes);
  
  // Go through each pair of camera parameters (pose1 and pose2
  // each represent a set of camera parameters)
  for (pose1 = 0; pose1 < scores_per_pose; pose1++)
  {
    if (pose1 % 100 == 0) vPrint("pose1 = %10li\r", pose1);
    // weight_sums[i] will accumulate the weight for shape = i
    // and pose = pose1 from all other poses (range for pose2)
    // from the same shape i.
    for (i = 0; i < number_of_shapes; i++)
    {
      weight_sums[i] = 0;
    }

    long frame1 = pose1/4;
    long view1 = pose1%4;
    long angle_index1 = frame1 / number_of_orientations;
    long temp_z_index1 = frame1 % number_of_orientations;
    long z_index1 = z_angle_index[temp_z_index1][view1];

    for (pose2 = 0; pose2 < scores_per_pose; pose2++)
    {
      long frame2 = pose2/4;
      long view2 = pose2%4;

      long angle_index2 = frame2 / number_of_orientations;
      long temp_z_index2 = frame2 % number_of_orientations;
      long z_index2 = z_angle_index[temp_z_index2][view2];

      long index2_base = angle_index2 * number_of_orientations;
      long displacement = z_index2 - z_index1;
      if (displacement < 0) displacement += number_of_orientations;
      //return angle_distances[angle_index1][index2_base + displacement];
      long angle_distance = angle_distances[angle_index1][index2_base + displacement];
      double probability = angle_probabilities[angle_distance];
//      if (pose1 == 0)
//      {
//        vPrint("pose1 = %5li, pose2 = %5li, distance = %4li, prob = %.10lf\n",
//                pose1, pose2, angle_distance, probability);
//      }

      // Increment the weight for each (pose = pose1, shape = shape_id) particle.
      for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
      {
        shape_base = shape_id * scores_per_pose;
        weight_sums[shape_id] += probability * estimates0[shape_base + pose2];
//        if ((pose1 == 0) && (shape_id == 10))
//        {
//          vPrint("shape_base = %li, estimates0[pose2] = %.12lf, weight_sum = %.12lf\n",
//                  shape_base, estimates0[shape_base + pose2], weight_sums[shape_id]);
//        }
      }
    }

    for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
    {
      shape_base = shape_id * scores_per_pose;
      estimates1[pose1 + shape_base] = weight_sums[shape_id];
    }
  }

  vdelete2(weight_sums);
  vPrint("\n");
  const double shapes_minus_one = number_of_shapes - 1;
  const double same_shape_probability = 0.9;
  const double other_shape_probability = (1-same_shape_probability)/shapes_minus_one;
  // Now, for each estimate, go through all other estimates that have identical
  // camera parameters, and add their influence, too
  for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
  {
    shape_base = shape_id * scores_per_pose;
    long shape_limit = shape_base + scores_per_pose;
    for (pose1 = shape_base; pose1 < shape_limit; pose1++)
    {
      if (pose1 % 1000 == 0)
      {
        vPrint("%15li %15li\r", shape_id, pose1);
      }
      long pose_index = pose1 % scores_per_pose;
      long pose2;
      double weight_sum = 0;
      for (pose2 = pose_index; pose2 < number_of_samples; pose2 += scores_per_pose)
      {
        long shape_id2 = pose2/scores_per_pose;
        double shape_probability = ShapeProbability(shape_id, shape_id2);
//        if (shape_id == pose2/scores_per_pose) shape_probability = same_shape_probability;
//        else shape_probability = other_shape_probability;
        double weight = estimates1[pose2];
        weight_sum += shape_probability * weight;
      }
      estimates2[pose1] = weight_sum;
    }
  }
  vPrint("\n");

  // Update the weight of each estimate
  for (i = 0; i < number_of_samples; i++)
  {
    particles[i]->probability = estimates2[i];
  }

  vdelete2(estimates0);
  vdelete2(estimates1);
  vdelete2(estimates2);

  vPrint("Done with propagating\n");
  return 1;
}



long vHandShapeTracker::Resample()
{
  // The minimum we should do is make sure weights (probabilities)
  // sum up to 1.

  double weight_sum = 0;
  long i;
  const double min_probability = 0.00000001; // (1 over ten million)
  for (i = 0; i < number_of_samples; i++)
  {
    weight_sum += particles[i]->probability;
  }

  vPrint("Weight sum = %lf\n", weight_sum);
  for (i = 0; i < number_of_samples; i++)
  {
    double probability = particles[i]->probability;
    probability /= weight_sum;

    if (probability < min_probability)
    {
      probability = min_probability;
    }

    if (i < 3)
    {
      vPrint("before: probability = %lf, after: probability = %lf\n", 
              particles[i]->probability, probability);
    }

    particles[i]->probability = probability;
  }

  return 1;
}


long vHandShapeTracker::InitializeEstimates(vTrackingObservation * in_observation)
{
  if (in_observation->Type() != vHandShapeObservation().ClassType())
  {
    return 0;
  }

  vHandShapeObservation * observation = (vHandShapeObservation *) in_observation;
  vMatchScores * scores = observation->GetScores();
  long number_of_scores = scores->NumberOfScores();
  if (number_of_scores != number_of_samples) return 0;
  vArray(vMatchScore) frame_sorted = scores->GetFrameSorted();
  long i;
  for (i = 0; i < number_of_scores; i++)
  {
    vMatchScore & score = frame_sorted[i];
    vHandShapeEstimate * estimate = (vHandShapeEstimate *) particles[i];
    estimate->SetEstimate(&view_finder, score.frame, score.view);
    double probability = RankProbability(score.rank);
    estimate->SetProbability(probability);
  }

  return 1;
}

long vHandShapeTracker::ProcessObservation(vTrackingObservation * in_observation)
{
  if (in_observation->Type() != vHandShapeObservation().ClassType())
  {
    return 0;
  }

  vHandShapeObservation * observation = (vHandShapeObservation *) in_observation;
  vMatchScores * scores = observation->GetScores();
  vArray(vMatchScore) frame_sorted = scores->GetFrameSorted();
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    vTrackingEstimate * estimate = particles[i];
    vMatchScore & score = frame_sorted[i];
    double probability = estimate->GetProbability();
    double rank_probability = RankProbability(score.rank);
    // The next line is an application of Bayes rule: 
    // P(A|B) = p(B|A) * p(A) / p(B), where
    // A is estimate, B is observation, and p(B) is 1/number_of_samples.
    double new_probability = number_of_samples * probability * rank_probability;
    estimate->SetProbability(new_probability);
  }

  Resample();
  return 1;
}


vMatchScores * vHandShapeTracker::ParticleScores()
{
  v3dMatrix<double> probabilities_matrix(1, number_of_samples, 1);
  vArray(double) probabilities = probabilities_matrix.Matrix();
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    // We negate probabilities, because the vMatchScores constructor
    // that we will use assumes that high scores indicate large
    // distances in ground truth space. 
    // In our case, high probabilities indicate small
    // distances.
    probabilities[i] = -particles[i]->GetProbability();
  }
  vMatchScores * scores = new vMatchScores(&probabilities_matrix);
  return scores;
}


/////////////////////////////////////////////////////////////////
//                                                             //
// End of implementation of the class vHandShapeTracker       //
//                                                             //
/////////////////////////////////////////////////////////////////


vHandShapeEstimate2::vHandShapeEstimate2()
{
}


vHandShapeEstimate2::vHandShapeEstimate2(long in_database_frame, 
                                           long in_database_view,
                                           long number_of_speeds)
{
  static vFrameViewFinder frame_view_finder;
  database_frame = in_database_frame;
  database_view = in_database_view;
  frame_view_finder.GetViewParams(database_frame, database_view,
                                  &shape_id, &x_angle, &y_angle, &z_angle);
  speed_probabilities = vnew(double, number_of_speeds);
  long i;
  for (i = 0; i < number_of_speeds; i++)
  {
    speed_probabilities[i] = 0;
  }
}


vHandShapeEstimate2::~vHandShapeEstimate2()
{
  vdelete2(speed_probabilities);
}


//////////////////////////////////////////////////////////////////
//                                                              //
// Beginning of implementation of the class vHandShapeTracker2 //
//                                                              //
//////////////////////////////////////////////////////////////////



vHandShapeTracker2::vHandShapeTracker2()
{
  max_distance = 60;
  // Note that ZeroAll() and vHandShape::SetNumberOfSamples() are 
  // called by the constructor
  // vHandShapeTracker(). Here we need to call vHandShapeTracker::ClearAll()
  // to clear up whatever was created by that constructor.
  
  vHandShapeTracker::ClearAll();
  ZeroAll();
    
  PrecomputeAngleDistances();
  SetNumberOfSamples(number_of_hand_views * max_neighbors);
}


vHandShapeTracker2::~vHandShapeTracker2()
{
  ClearAll();
}


long vHandShapeTracker2::ZeroAll()
{
  particles = vZero(vTrackingEstimate *);
  number_of_samples = 0;
  return 1;
}


long vHandShapeTracker2::ClearAll()
{
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    vHandShapeEstimate2 * estimate = (vHandShapeEstimate2 *) particles[i];
    vdelete(estimate);
  }
  vdelete2(particles);
  ZeroAll();
  return 1;
}


long vHandShapeTracker2::PrecomputeAngleDistances()
{
  vHandShapeTracker::PrecomputeAngleDistances();
  
  // First compute max_neighbors.
  max_neighbors = 0;
  long frame1, view1, frame2, view2;
  long frame_limit = number_of_poses / 4;
  for (frame1 = 0; frame1 < frame_limit; frame1++)
  {
    for (view1 = 0; view1 < 4; view1++)
    {
      long number_of_neighbors = 0;
      // Compute number of neighbors within max_distance for given frame1/view1
      for (frame2 = 0; frame2 < frame_limit; frame2++)
      {
        for (view2 = 0; view2 < 4; view2++)
        {
          long distance = CameraParamsDistance(frame1, view1, frame2, view2);
          if (distance <= max_distance) number_of_neighbors++;
        }
      }
      // Update max_neighbors if necessary
      if (number_of_neighbors > max_neighbors) max_neighbors = number_of_neighbors;
    }
  }

  
  neighbors_matrix = v3dMatrix<long>(number_of_hand_views, max_neighbors+1, 1);
  neighbors = neighbors_matrix.Matrix2(0);
  neighbor_angles_matrix = v3dMatrix<ushort>(number_of_hand_views, max_neighbors+1, 1);
  neighbor_angles = neighbor_angles_matrix.Matrix2(0);;

  // Now fill in the entries at the neighbors and neighbor_angles arrays.
  for (frame1 = 0; frame1 < frame_limit; frame1++)
  {
    for (view1 = 0; view1 < 4; view1++)
    {
      long pose1_index = frame1 * 4 + view1;
      long pose2_index = 0;
      // Find all neighbors within max_distance from frame1/view1
      for (frame2 = 0; frame2 < frame_limit; frame2++)
      {
        for (view2 = 0; view2 < 4; view2++)
        {
          long distance = CameraParamsDistance(frame1, view1, frame2, view2);
          if (distance > max_distance) continue;
          neighbors[pose1_index][pose2_index] = frame2 * 4 + view2;
          neighbor_angles[pose1_index][pose2_index] = (ushort) distance;
          pose2_index++;
        }
      }
      // Put a -1 at the end of neighbors[pose1_index] to signal that there
      // are no mor neighbors for frame1/view1.
      neighbors[pose1_index][pose2_index] = -1;
      neighbor_angles[pose1_index][pose2_index] = -1;
    }
  }

  return 1;
}


long vHandShapeTracker2::SetNumberOfSamples(long in_number_of_samples)
{
  ClearAll();
  number_of_samples = in_number_of_samples;
  if (number_of_samples != number_of_hand_views * max_neighbors)
  {
    exit_error("vHandShapeTracker2::SetNumberOfSamples error: %li\n", 
                    number_of_samples);
  }
  
  particles = vnew(vTrackingEstimate *, number_of_samples);
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    particles[i] = new vHandShapeEstimate2();
  }

  return 1;
}


long vHandShapeTracker2::Propagate()
{
  // count weights on every shape, use them for a first round of weight estimation
  // Then, for each estimate, find nearby estimates (within 108 degrees) of the 
  // same shape and use that for second round of estimates. Then, for every
  // estimate find all estimates with same camera params and different shapes,
  // and use that for third round of estimates.

  // Compute the probability of each shape, 
  long scores_per_pose = view_finder.ScoresPerPose();

  vArray(double) estimates0 = vnew(double, number_of_samples);
  vArray(double) estimates1 = vnew(double, number_of_samples);
  vArray(double) estimates2 = vnew(double, number_of_samples);

  // Put all estimate probabilities into the estimates0 array, 
  // to optimize speed
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    estimates0[i] = particles[i]->probability;
  }

  const long frames_per_pose = view_finder.FramesPerPose();
  const long number_of_orientations = view_finder.NumberOfOrientations();
  // For each estimate, go through all other estimates that belong to the same
  // shape id, and update its probability
  long shape_id, shape_base, pose1, pose2;
  vArray(double) weight_sums = vnew(double, number_of_shapes);
  
  // Go through each pair of camera parameters (pose1 and pose2
  // each represent a set of camera parameters)
  for (pose1 = 0; pose1 < scores_per_pose; pose1++)
  {
    if (pose1 % 100 == 0) vPrint("pose1 = %10li\r", pose1);
    // weight_sums[i] will accumulate the weight for shape = i
    // and pose = pose1 from all other poses (range for pose2)
    // from the same shape i.
    for (i = 0; i < number_of_shapes; i++)
    {
      weight_sums[i] = 0;
    }

    long frame1 = pose1/4;
    long view1 = pose1%4;
    long angle_index1 = frame1 / number_of_orientations;
    long temp_z_index1 = frame1 % number_of_orientations;
    long z_index1 = z_angle_index[temp_z_index1][view1];

    for (pose2 = 0; pose2 < scores_per_pose; pose2++)
    {
      long frame2 = pose2/4;
      long view2 = pose2%4;

      long angle_index2 = frame2 / number_of_orientations;
      long temp_z_index2 = frame2 % number_of_orientations;
      long z_index2 = z_angle_index[temp_z_index2][view2];

      long index2_base = angle_index2 * number_of_orientations;
      long displacement = z_index2 - z_index1;
      if (displacement < 0) displacement += number_of_orientations;
      //return angle_distances[angle_index1][index2_base + displacement];
      long angle_distance = angle_distances[angle_index1][index2_base + displacement];
      double probability = angle_probabilities[angle_distance];
//      if (pose1 == 0)
//      {
//        vPrint("pose1 = %5li, pose2 = %5li, distance = %4li, prob = %.10lf\n",
//                pose1, pose2, angle_distance, probability);
//      }

      // Increment the weight for each (pose = pose1, shape = shape_id) particle.
      for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
      {
        shape_base = shape_id * scores_per_pose;
        weight_sums[shape_id] += probability * estimates0[shape_base + pose2];
//        if ((pose1 == 0) && (shape_id == 10))
//        {
//          vPrint("shape_base = %li, estimates0[pose2] = %.12lf, weight_sum = %.12lf\n",
//                  shape_base, estimates0[shape_base + pose2], weight_sums[shape_id]);
//        }
      }
    }

    for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
    {
      shape_base = shape_id * scores_per_pose;
      estimates1[pose1 + shape_base] = weight_sums[shape_id];
    }
  }

  vPrint("\n");
  vdelete2(weight_sums);
  const double shapes_minus_one = number_of_shapes - 1;
  const double same_shape_probability = 0.9;
  const double other_shape_probability = (1-same_shape_probability)/shapes_minus_one;
  // Now, for each estimate, go through all other estimates that have identical
  // camera parameters, and add their influence, too
  for (shape_id = 0; shape_id < number_of_shapes; shape_id++)
  {
    shape_base = shape_id * scores_per_pose;
    long shape_limit = shape_base + scores_per_pose;
    for (pose1 = shape_base; pose1 < shape_limit; pose1++)
    {
      if (pose1 % 1000 == 0)
      {
        vPrint("%15li %15li\r", shape_id, pose1);
      }
      long pose_index = pose1 % scores_per_pose;
      long pose2;
      double weight_sum = 0;
      for (pose2 = pose_index; pose2 < number_of_samples; pose2 += scores_per_pose)
      {
        long shape_id2 = pose2/scores_per_pose;
        double shape_probability = ShapeProbability(shape_id, shape_id2);
//        if (shape_id == pose2/scores_per_pose) shape_probability = same_shape_probability;
//        else shape_probability = other_shape_probability;
        double weight = estimates1[pose2];
        weight_sum += shape_probability * weight;
      }
      estimates2[pose1] = weight_sum;
    }
  }
  vPrint("\n");

  // Update the weight of each estimate
  for (i = 0; i < number_of_samples; i++)
  {
    particles[i]->probability = estimates2[i];
  }

  vdelete2(estimates0);
  vdelete2(estimates1);
  vdelete2(estimates2);

  vPrint("Done with propagating\n");
  return 1;
}



long vHandShapeTracker2::Resample()
{
  // The minimum we should do is make sure weights (probabilities)
  // sum up to 1.

  double weight_sum = 0;
  long i;
  const double min_probability = 0.00000001; // (1 over ten million)
  for (i = 0; i < number_of_samples; i++)
  {
    weight_sum += particles[i]->probability;
  }

  vPrint("Weight sum = %lf\n", weight_sum);
  for (i = 0; i < number_of_samples; i++)
  {
    double probability = particles[i]->probability;
    probability /= weight_sum;

    if (probability < min_probability)
    {
      probability = min_probability;
    }

    if (i < 3)
    {
      vPrint("before: probability = %lf, after: probability = %lf\n", 
              particles[i]->probability, probability);
    }

    particles[i]->probability = probability;
  }

  return 1;
}


long vHandShapeTracker2::InitializeEstimates(vTrackingObservation * in_observation)
{
  if (in_observation->Type() != vHandShapeObservation().ClassType())
  {
    return 0;
  }

  vHandShapeObservation * observation = (vHandShapeObservation *) in_observation;
  vMatchScores * scores = observation->GetScores();
  long number_of_scores = scores->NumberOfScores();
  if (number_of_scores != number_of_samples) return 0;
  vArray(vMatchScore) frame_sorted = scores->GetFrameSorted();
  long i;
  for (i = 0; i < number_of_scores; i++)
  {
    vMatchScore & score = frame_sorted[i];
    vHandShapeEstimate * estimate = (vHandShapeEstimate *) particles[i];
    estimate->SetEstimate(&view_finder, score.frame, score.view);
    double probability = RankProbability(score.rank);
    estimate->SetProbability(probability);
  }

  return 1;
}

long vHandShapeTracker2::ProcessObservation(vTrackingObservation * in_observation)
{
  if (in_observation->Type() != vHandShapeObservation().ClassType())
  {
    return 0;
  }

  vHandShapeObservation * observation = (vHandShapeObservation *) in_observation;
  vMatchScores * scores = observation->GetScores();
  vArray(vMatchScore) frame_sorted = scores->GetFrameSorted();
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    vTrackingEstimate * estimate = particles[i];
    vMatchScore & score = frame_sorted[i];
    double probability = estimate->GetProbability();
    double rank_probability = RankProbability(score.rank);
    // The next line is an application of Bayes rule: 
    // P(A|B) = p(B|A) * p(A) / p(B), where
    // A is estimate, B is observation, and p(B) is 1/number_of_samples.
    double new_probability = number_of_samples * probability * rank_probability;
    estimate->SetProbability(new_probability);
  }

  Resample();
  return 1;
}


vMatchScores * vHandShapeTracker2::ParticleScores()
{
  v3dMatrix<double> probabilities_matrix(1, number_of_samples, 1);
  vArray(double) probabilities = probabilities_matrix.Matrix();
  long i;
  for (i = 0; i < number_of_samples; i++)
  {
    // We negate probabilities, because the vMatchScores constructor
    // that we will use assumes that high scores indicate large
    // distances in ground truth space. 
    // In our case, high probabilities indicate small
    // distances.
    probabilities[i] = -particles[i]->GetProbability();
  }
  vMatchScores * scores = new vMatchScores(&probabilities_matrix);
  return scores;
}


/////////////////////////////////////////////////////////////////
//                                                             //
// End of implementation of the class vHandShapeTracker2      //
//                                                             //
/////////////////////////////////////////////////////////////////



