#include "stdafx.h"

#include "simple_chamfer.h"
#include "basics/drawing_temp.h"
#include "opencv_utils.h"
#include "algorithms.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "algo_templates.h"
#include "basics/drawing.h"

#include "basics/definitions.h"


class S_GridDistance
{
public:
  ushort grid;
  long distance;
};


class S_GridDistanceLess
{
public:
  ushort operator() (const S_GridDistance & item1, const S_GridDistance & item2);
};



ushort S_GridDistanceLess::operator() (const S_GridDistance & item1, 
                                       const S_GridDistance & item2)
{
  return (item1.distance < item2.distance);
}




// Finds all non-zero points in the rle image (assuming that its top left
// pixel is non-zero) and stores their coordinates in arrays that it
// creates. It stores those arrays in rows and cols.
// number_address receives the number of non-zero points.
long vRLENonZeroPoints(vRLE_Gray * rle, vArray(long) rows, vArray(long) cols, 
                        long * number_address){
  vector<ushort> * changes_array = rle->GetChanges();
  vector<uchar> * values_array = rle->GetValues();
  long number = 0;

  long rle_rows = rle->Rows();
  long rle_cols = rle->Cols();

  long row;
	for (row = 0; row < rle_rows; row++)
	{
		long index = 1;
		vector<ushort> & changes = changes_array[row];
    vector<uchar> & values = values_array[row];
    long col = 0;
    long changes_size = changes.size();
    while(index < changes_size)
		{
			ushort times = changes[index];
      uchar value = values[index];
      index++;
      // The value alternates between 0 and 255.
 			ushort i;
      if (value > 0)
      {
  			for (i = 0; i < times; i++)
			  {
				  rows[number] = row;
          cols[number] = col++;
          number++;
			  }
      }
      else
      {
        col = col + times;
      }
		}
	}
  
  *number_address = number;
  return 1;
}


// This function puts the locations of binary edges into row_vector and
// col_vector. A binary edge is defined to be a pixel that is non-zero and
// that is different than at least one of its four neighbors. If the image
// is binary this will give us its edges, that's why the function is called
// "vGetBinaryEdges". Obviously, if we pass in a gray scale photograph,
// the results will not be very useful.
ushort vGetBinaryEdges(grayscale_image * binary, vector<long> * row_vector, 
                        vector<long> * col_vector)
{
  // Make sure the row_vector and col_vector are empty to begin with
  row_vector->erase(row_vector->begin(), row_vector->end());
  col_vector->erase(col_vector->begin(), col_vector->end());

  long rows = binary->Rows();
  long cols = binary->Cols();
  long row, col;
  vArray2(uchar) I = binary->GetI2();
  for (row = 1; row < rows - 1; row++)
  {
    vArray(uchar) previous = I[row-1];
    vArray(uchar) base = I[row];
    vArray(uchar) next = I[row+1];
    for (col = 1; col < cols - 1; col++)
    {
      uchar center = base[col];
      // Pixels with value 0 are considered to be background
      if (center == 0) continue;
      // If the pixel is non-zero and at least one of its
      // four neighbors has a different value, then it is
      // a binary edge.
      if ((center != previous[col]) || (center != next[col]) ||
          (center != base[col-1]) || (center != base[col+1]))
      {
        row_vector->push_back(row);
        col_vector->push_back(col);
      }
    }
  }
  return 1;
}


// Returns the index of the closest point in the set described by rows2/cols2
// (the i-th point is (*rows2[i], *cols2[i])) to the point (row, col), and
// the square distance between (row, col) and the closest point. The return
// values are stored in the arguments square_distance and index.
ushort vClosestPoint(long row, long col, vector<long> * rows2, 
                      vector<long> * cols2, long * square_distance, long * index)
{
  long min_distance = 2000000000; // 2 billion
  long size2 = rows2->size();
  long i;
  long min_index = -1;
  for (i = 0; i < size2; i++)
  {
    long row2 = (*rows2)[i];
    long col2 = (*cols2)[i];
    long row_diff = row2 - row;
    long col_diff = col2 - col;
    long distance = row_diff * row_diff + col_diff * col_diff;
    if (distance < min_distance)
    {
      min_index = i;
      min_distance = distance;
    }
  }
  *square_distance = min_distance;
  *index = min_index;
  return 1;
}


// Note that the Chamfer distance is not symmetric.
double vChamferDistance(vector<long> * rows1, vector<long> * cols1,
                           vector<long> * rows2, vector<long> * cols2)
{
  long size1 = rows1->size();
  assert(size1 == cols1->size());

  long size2 = rows2->size();
//  long k;
//  for (k = 0; k < Min(size1, size2); k++)
//  {
//    vPrint("%7li: (%7li, %7li)  (%7li, %7li)\n", k, 
//            (*rows1)[k], (*cols1)[k], (*rows2)[k], (*cols2)[k]);
//  }
  
  long i;
  double result = 0;
  for (i = 0; i < size1; i++)
  {
    long row = (*rows1)[i];
    long col = (*cols1)[i];
    long index, square_distance;
    vClosestPoint(row, col, rows2, cols2, &square_distance, &index);
    result = result + sqrt((double) square_distance);
  }

  result = result / (double) size1;
  return result;
}


// Like vChamferDistance but it applies some optimizing
// heuristics (the degree of utilization of those heuristics depends
// on the "parameter" passed).
double vChamferDistanceFast(vArray(long) rows1, vArray(long) cols1, long number1,
                               vArray(long) rows2, vArray(long) cols2, long number2,
                               long parameter)
{
  double result = 0;
  long i;
  for (i = 0; i < number1; i++)
  {
    long row = rows1[i];
    long col = cols1[i];

    // Find closest point.
    long j;
    long min_distance = 1000000000;
    for (j = 0; j < number2; j++)
    {
      long row2 = rows2[j];
      long col2 = cols2[j];
      long row_diff = row2 - row;
      long col_diff = col2 - col;
      long distance = row_diff * row_diff + col_diff * col_diff;
      if (distance < min_distance) min_distance = distance;
    }

    result = result + sqrt((double) min_distance);
  }

  result = result / (double) number1;
  return result;
}


// Like vChamferDistance but instead of getting the list of edge
// points for the second image, it gets the distance transform of the
// second image.
double vChamferFromDT(grayscale_image * the_image, v3dMatrix<float> * dt_image)
{
  vArray(long) rows;
  vArray(long) cols;
  long number = 0;
  vGetNonZeroPoints4(the_image, 
    &rows, &cols, &number);
  double result = vChamferFromDT(rows, cols, number, dt_image);
  vdelete2(rows);
  vdelete2(cols);
  return result;
}


double vChamferFromDT(vArray(long) rows1, vArray(long) cols1, 
                       long number1, v3dMatrix<float> * dt_image)
{
  if (number1 == 0) return dt_image->Rows() + dt_image->Cols();
  vArray2(float) distances = dt_image->Matrix2(0);
  long i;
  double total = 0;
  for (i = 0; i < number1; i++)
  {
    long row = rows1[i];
    long col = cols1[i];
    total = total + distances[row][col];
  }
  double result = total / (double) number1;
  return result;
}


double vChamferFromDT(vArray(uchar) rows1, vArray(uchar) cols1, 
                      long number1, v3dMatrix<float> * dt_image)
{
  if (number1 == 0) return dt_image->Rows() + dt_image->Cols();
  vArray2(float) distances = dt_image->Matrix2(0);
  long i;
  double total = 0;
  for (i = 0; i < number1; i++)
  {
    long row = rows1[i];
    long col = cols1[i];
    total = total + distances[row][col];
  }
  double result = total / (double) number1;
  return result;
}


double vChamferFromDT(vArray(uchar) rows1, vArray(uchar) cols1, 
                         long number1, v3dMatrix<uchar> * dt_image)
{
  if (number1 == 0) return dt_image->Rows() + dt_image->Cols();
  vArray2(uchar) distances = dt_image->Matrix2(0);
  long i;
  float total = 0;
  for (i = 0; i < number1; i++)
  {
    long row = rows1[i];
    long col = cols1[i];
    total = total + distances[row][col];
  }
  double result = total / (double) number1;
  return result;
}


double vChamferFromDT(vector<long> * rows1, vector<long> * cols1, 
                         v3dMatrix<uchar> * dt_image)
{
  vArray2(uchar) distances = dt_image->Matrix2(0);
  long number = rows1->size();
  long i;
  float total = 0;
  for (i = 0; i < number; i++)
  {
    long row = (*rows1)[i];
    long col = (*cols1)[i];
    total = total + distances[row][col];
  }
  double result = total / (double) number;
  return result;
}


double vChamferFromDT(vector<long> * rows1, vector<long> * cols1, 
                         v3dMatrix<float> * dt_image)
{
  vArray2(float) distances = dt_image->Matrix2(0);
  long number = rows1->size();
  long i;
  float total = 0;
  for (i = 0; i < number; i++)
  {
    long row = (*rows1)[i];
    long col = (*cols1)[i];
    total = total + distances[row][col];
  }
  double result = total / (double) number;
  return result;
}


// Like vChamferDistance but it searches nearest neighbors in a more
// efficient way. It only works, though, if the points come from images
// of size at most 256x256 (row < 256, col < 256).
double vOptimizedChamfer(vArray(long) rows1, vArray(long) cols1, long number1,
                            vArray(long) rows2, vArray(long) cols2, long number2,
                            long grid_size)
{
  if (grid_size == -1) grid_size = 10;
  const long max_min_distance = 132000;
  static vMatrix<double> roots_matrix = vPrecalculateRoots2(max_min_distance);
  static long previous_grid_size = grid_size;
  // cell_order[i][j] is the index of the j-th closest cell
  // to cell i in the grid.
  static v3dMatrix<ushort> cell_order_matrix = 
    vGridCellOrder(256, 256, previous_grid_size);

  // cell_distance[i][j] is the min square distance 
  // between any point on cell i
  // and any point on cell j (if a cell has a 2D index x,y, 
  // its one-dimensional index is x * 256/grid_size = y / grid_size).
  static v3dMatrix<ushort> cell_distance_matrix = 
    vGridCellDistance(256, 256, previous_grid_size);

//  static vMatrix<long> cell_distance_matrix2 =
//    vGridCellDistance2(256, 256, previous_grid_size);

  static v3dMatrix<uchar> grid = vMakeGrid(256, 256, grid_size);

  static v3dMatrix<ushort> grid_stats_matrix = 
    vMakeGridStats(256, 256, grid_size);

  if (grid_size != previous_grid_size)
  {
    previous_grid_size = grid_size;
    cell_order_matrix = vGridCellOrder(256, 256, previous_grid_size);
    cell_distance_matrix = vGridCellDistance(256, 256, previous_grid_size);
//    cell_distance_matrix2 = vGridCellDistance2(256, 256, previous_grid_size);    
    grid = vMakeGrid(256, 256, grid_size);
    grid_stats_matrix = vMakeGridStats(256, 256, grid_size);
  }
  
  long grid_rows = (256 - 1)/grid_size + 1;
  long grid_cols = (256 - 1)/grid_size + 1;
  vArray2(ushort) cell_order = cell_order_matrix.Matrix2(0);
  vArray2(ushort) cell_distances = cell_distance_matrix.Matrix2(0);
//  vArray2(long) cell_distances2 = cell_distance_matrix2.GetData();
  vArray2(uchar) grid_points = grid.Matrix2(0);
  vArray(double) roots = roots_matrix.Matrix();
  vArray2(ushort) grid_stats = grid_stats_matrix.Matrix2(0);
  vArray(ushort) grid_counters = grid_stats[0];
  
  // Put points of second set into the grid.
  vPutPointsToGrid(&grid, &grid_stats_matrix, grid_rows, grid_cols,
                    grid_size, rows2, cols2, number2);

//  double comparisons = 0;
//  double total_grids = 0;
//  double empty_grids = 0;
  long square_grid_size = grid_size * grid_size;
  
  double result = 0;
  long i;
  for (i = 0; i < number1; i++)
  {
    long row = rows1[i];
    long col = cols1[i];

    long grid_row = row / grid_size;
    long grid_col = col / grid_size;
    long grid_index = row * grid_cols + grid_col;
    long grid_counter = 0;
    long min_distance = max_min_distance;
    // Find closest point.
    while(1)
    {
      if (grid_counter == grid_rows * grid_cols) break;
      long grid_index = grid_row * grid_cols + grid_col;
      long next_index = cell_order[grid_index][grid_counter];
      grid_counter++;
//      long next_row = next_index / grid_cols;
//      long next_col = next_index % grid_cols;
//      long min_next_distance = cell_distances2[grid_row - next_row][grid_col - next_col];

      long min_next_distance = cell_distances[grid_index][next_index];

      // If we can't find any points closer than min distance anymore,
      // we are done.
      if (min_next_distance >= min_distance)
      {
        break;
      }
//      total_grids++;
      long number_of_points = grid_counters[next_index];
//      if (number_of_points == 0) empty_grids++;
      if (number_of_points == 0) continue;
      vArray(uchar) next_points = grid_points[next_index];
      long point = 0;
      long index_limit = 2 * number_of_points;
      while(point < index_limit)
      {
        long row2 = next_points[point++];
        long col2 = next_points[point++];
        long row_diff = row2 - row;
        long col_diff = col2 - col;
        long square_distance = row_diff * row_diff + col_diff * col_diff;
//        comparisons++;
        if (square_distance < min_distance)
        {
          min_distance = square_distance;
        }
      }
    }

    result = result + roots[min_distance];
  }

  result = result / (double) number1;
//  vPrint("grid size = %li, number1 = %li, number2 = %li\n",
//          grid_size, number1, number2);
//  vPrint("comparisons = %lf, total_grids = %lf, empty_grids = %lf\n",
//          comparisons / (double) number1, total_grids / (double) number1, 
//          empty_grids / (double) number1);
  vRemovePointsFromGrid(&grid, &grid_stats_matrix);
  return result;
}


// Like vChamferDistance but it searches nearest neighbors in a more
// efficient way. It only works, though, if the points come from images
// of size at most max_sizexmax_size (row < max_size, col < max_size).
double vOptimizedChamfer2(vArray(long) rows1, vArray(long) cols1, long number1,
                             vArray(long) rows2, vArray(long) cols2, long number2,
                             long grid_size, long subsampling)
{
  const long max_min_distance = 30000;
  const long max_size = 256;
  static vMatrix<double> roots_matrix = vPrecalculateRoots2(max_min_distance);
  static long previous_grid_size = grid_size;
  // cell_order[i][j] is the index of the j-th closest cell
  // to cell i in the grid.
  static v3dMatrix<ushort> cell_order_matrix = 
    vGridCellOrder(max_size, max_size, previous_grid_size);

  // cell_distance[i][j] is the min square distance 
  // between any point on cell i
  // and any point on cell j (if a cell has a 2D index x,y, 
  // its one-dimensional index is x * max_size/grid_size = y / grid_size).
  static v3dMatrix<ushort> cell_distance_matrix = 
    vGridCellDistance(max_size, max_size, previous_grid_size);

  static v3dMatrix<uchar> grid = vMakeGrid(max_size, max_size, grid_size);

  static v3dMatrix<ushort> grid_stats_matrix = 
    vMakeGridStats(max_size, max_size, grid_size);

  static vMatrix<ulong> random_matrix = vRandomVector(1000);

  if (number1 == 0) return 2 * max_size;
  if (number2 == 0) return 2 * max_size;

  if (grid_size != previous_grid_size)
  {
    previous_grid_size = grid_size;
    cell_order_matrix = vGridCellOrder(max_size, max_size, previous_grid_size);
    cell_distance_matrix = vGridCellDistance(max_size, max_size, previous_grid_size);
    grid = vMakeGrid(max_size, max_size, grid_size);
    grid_stats_matrix = vMakeGridStats(max_size, max_size, grid_size);
  }
  
  long grid_rows = (max_size - 1)/grid_size + 1;
  long grid_cols = (max_size - 1)/grid_size + 1;
  vArray2(ushort) cell_order = cell_order_matrix.Matrix2(0);
  vArray2(ushort) cell_distances = cell_distance_matrix.Matrix2(0);
  vArray2(uchar) grid_points = grid.Matrix2(0);
  vArray(double) roots = roots_matrix.Matrix();
  vArray2(ushort) grid_stats = grid_stats_matrix.Matrix2(0);
  vArray(ushort) grid_counters = grid_stats[0];
  vArray(ulong) randoms = random_matrix.Matrix();
  
  // Put points of second set into the grid.
  vPutPointsToGrid(&grid, &grid_stats_matrix, grid_rows, grid_cols,
                    grid_size, rows2, cols2, number2);

//  double comparisons = 0;
//  double total_grids = 0;
// double empty_grids = 0;
  long points_checked = 0;
  
  double result = 0;
  long i;
  for (i = 0; i < number1; i++)
  {
    ulong random = randoms[i % 1000];
    if (random % subsampling != 0) continue;
    points_checked++;
    long row = rows1[i];
    long col = cols1[i];

    long grid_row = row / grid_size;
    long grid_col = col / grid_size;
    long grid_index = row * grid_cols + grid_col;
    long grid_counter = 0;
    long min_distance = max_min_distance;
    // Find closest point.
    while(1)
    {
      if (grid_counter == grid_rows * grid_cols) break;
      long grid_index = grid_row * grid_cols + grid_col;
      long next_index = cell_order[grid_index][grid_counter];
      grid_counter++;
      long min_next_distance = cell_distances[grid_index][next_index];
      // If we can't find any points closer than min distance anymore,
      // we are done.
      if (min_next_distance >= min_distance)
      {
        break;
      }
//      total_grids++;
      long number_of_points = grid_counters[next_index];
//      if (number_of_points == 0) empty_grids++;
      if (number_of_points == 0) continue;
      vArray(uchar) next_points = grid_points[next_index];
      long point = 0;
      long index_limit = 2 * number_of_points;
      while(point < index_limit)
      {
        long row2 = next_points[point++];
        long col2 = next_points[point++];
        long row_diff = row2 - row;
        long col_diff = col2 - col;
        long square_distance = row_diff * row_diff + col_diff * col_diff;
//        comparisons++;
        if (square_distance < min_distance)
        {
          min_distance = square_distance;
        }
      }
    }

    result = result + roots[min_distance];
  }

  result = result / (double) points_checked;
//  vPrint("grid size = %li, number1 = %li, number2 = %li\n",
//          grid_size, number1, number2);
//  vPrint("comparisons = %lf, total_grids = %lf, empty_grids = %lf\n",
//          comparisons / (double) number1, total_grids / (double) number1, 
//          empty_grids / (double) number1);
  vRemovePointsFromGrid(&grid, &grid_stats_matrix);
  return result;
}


double vTwoWayChamferFast(vArray(long) rows1, vArray(long) cols1, long number1,
                             vArray(long) rows2, vArray(long) cols2, long number2,
                             long parameter, double * first, double * second)
{
  double result1 = vOptimizedChamfer(rows1, cols1, number1, 
                                        rows2, cols2, number2, parameter);
  double result2 = vOptimizedChamfer(rows2, cols2, number2, 
                                        rows1, cols1, number1, parameter);
  if (first != 0)
  {
    *first = result1;
  }
  if (second != 0)
  {
    *second = result2;
  }
  return result1 + result2;
}


ushort vChamferDistancesSlow(grayscale_image * in_binary1, grayscale_image * binary2,
                                double * out_distance, long * out_rows, long * out_cols,
                                double * out_scaling)
{
  grayscale_image * binary1 = (grayscale_image *) vCopyRectangle(in_binary1, 0, 255, 0, 255);
  long rows = binary1->Rows();
  long cols = binary1->Cols();

  v3dMatrix<float> * distances2 = vCVDistanceTransform(binary2);
  vector<long> rows1, cols1, rows2, cols2;
  vGetNonZeroPoints(binary2, &rows2, &cols2);
  long points2 = rows2.size();
  
  vector<long> train_rows(points2);
  vector<long> train_cols(points2);
  long translation_rows, translation_cols;
  double scaling = 0;
  long min_rows, min_cols;
  double min_distance, min_scaling;
  min_rows = -1;
  min_cols = -1;
  min_distance = rows * rows + cols * cols;

  // We want to search over horizontal translation, vertical translation and
  // scaling. We want to first sample the space, find a peak, then sample around
  // the peak, find a new peak, and then sample again.

  // First search using the following sampling. (6x6x7 points)
  long step = 5;
  //vPrint("Enter row/col step: ");
  //vScan("%li", &step);
  long small_rows = -20;
  long big_rows = 20;
  long row_step = step;
  long small_cols = -20;
  long big_cols = 20;
  long col_step = step;
  double small_scaling = 0.85;
  double big_scaling = 1.15;
  double scaling_step = 0.05;
//  vPrint("Scaling step: ");
//  vScan("%lf", &scaling_step);
  long counter = 0;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    v3dMatrix<uchar> * scaled_test = vScaleImage4(binary1, scaling, 128, 128);
    v3dMatrix<float> * distances1 = vCVDistanceTransform(scaled_test);
    vGetNonZeroPoints(binary1, &rows1, &cols1);
    long points1 = rows1.size();
    //vPrint("points1 = %li, points2 = %li\n", points1, points2);
    vector<long> test_rows(points1);
    vector<long> test_cols(points1);

    //vPrint("\rcounter = %li", counter++);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        
        vTranslatePoints(&rows1, &cols1, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        vTranslatePoints(&rows2, &cols2, &train_rows, &train_cols,
                          -translation_rows, -translation_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
    vdelete(scaled_test);
    vdelete(distances1);
  }

  
  // Now look closer. (5x5x5 points)
  small_rows = min_rows - 4;
  big_rows = min_rows + 4;
  row_step = 2;
  small_cols = min_cols - 4;
  big_cols = min_cols + 4;
  col_step = 2;
  small_scaling = min_scaling - 0.04;
  big_scaling = min_scaling + 0.04;
  scaling_step = 0.02;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    v3dMatrix<uchar> * scaled_test = vScaleImage4(binary1, scaling, 128, 128);
    v3dMatrix<float> * distances1 = vCVDistanceTransform(scaled_test);
    vGetNonZeroPoints(binary1, &rows1, &cols1);
    long points1 = rows1.size();
    vector<long> test_rows(points1);
    vector<long> test_cols(points1);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&rows1, &cols1, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        vTranslatePoints(&rows2, &cols2, &train_rows, &train_cols,
                          -translation_rows, -translation_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
      }
    }
    vdelete(scaled_test);
    vdelete(distances1);
  }
  
  // Now look even closer (final sampling) (5x5x5 points)
  small_rows = min_rows - 2;
  big_rows = min_rows + 2;
  row_step = 1;
  small_cols = min_cols - 2;
  big_cols = min_cols + 2;
  col_step = 1;
  small_scaling = min_scaling - 0.02;
  big_scaling = min_scaling + 0.02;
  scaling_step = 0.01;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    v3dMatrix<uchar> * scaled_test = vScaleImage4(binary1, scaling, 128, 128);
    v3dMatrix<float> * distances1 = vCVDistanceTransform(scaled_test);
    vGetNonZeroPoints(binary1, &rows1, &cols1);
    long points1 = rows1.size();
    vector<long> test_rows(points1);
    vector<long> test_cols(points1);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&rows1, &cols1, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        vTranslatePoints(&rows2, &cols2, &train_rows, &train_cols,
                          -translation_rows, -translation_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
      }
    }
    vdelete(scaled_test);
    vdelete(distances1);
  }
  
  vdelete(distances2);
  vdelete(binary1);
  *out_rows = min_rows;
  *out_cols = min_cols;
  *out_scaling = min_scaling;
  *out_distance = min_distance;
  return 1;
}


// This function behaves correctly if the input images are binary. It
// finds their edges, takes the two Chamfer distances between the 
// images and returns their sum. If the input images are not binary,
// one needs to look at vGetBinaryEdges to understand what will happen.
double vCompareContours(grayscale_image * binary1, grayscale_image * binary2)
{
  vector<long> rows1, cols1, rows2, cols2;
  vGetNonZeroPoints(binary1, &rows1, &cols1);
  vGetNonZeroPoints(binary2, &rows2, &cols2);
  double result1 = vChamferDistance(&rows1, &cols1, &rows2, &cols2);
  double result2 = vChamferDistance(&rows2, &cols2, &rows1, &cols1);
  double result = result1 + result2;
  return result;
}


// returns the total chamfer distance, and stores in output arguments
// the two directional distances
double function_chamfer_distance(grayscale_image * binary1, grayscale_image * binary2,
                                 double * first_distance, double * second_distance)
{
  if ((binary1 == 0) || (binary2 == 0))
  {
    return -1;
  }

  vector<long> rows1, cols1, rows2, cols2;
  vGetNonZeroPoints(binary1, &rows1, &cols1);
  vGetNonZeroPoints(binary2, &rows2, &cols2);
  double result1 = vChamferDistance(&rows1, &cols1, &rows2, &cols2);
  double result2 = vChamferDistance(&rows2, &cols2, &rows1, &cols1);
  double result = result1 + result2;

  if (first_distance != 0)
  {
    *first_distance = result1;
  }
  if (second_distance != 0)
  {
    *second_distance = result2;
  }

  return result;
}



double vCompareContoursFast(grayscale_image * binary1, grayscale_image * binary2, 
                             long parameter)
{
  vArray(long) binary1_rows;
  vArray(long) binary1_cols;
  vArray(long) binary2_rows;
  vArray(long) binary2_cols;
  long binary1_number, binary2_number;
  binary1_rows = binary1_cols = binary2_rows = binary2_cols = (vArray(long)) (vZero(long));
  binary1_number = binary2_number = 0;
  vGetNonZeroPoints4(binary1, &binary1_rows, &binary1_cols, &binary1_number);
  vGetNonZeroPoints4(binary2, &binary2_rows, &binary2_cols, &binary2_number);
  double result = vTwoWayChamferFast(binary1_rows, binary1_cols, binary1_number,
                                        binary2_rows, binary2_cols, binary2_number,
                                        parameter);
  vdelete2(binary1_rows);
  vdelete2(binary1_cols);
  vdelete2(binary2_rows);
  vdelete2(binary2_cols);
  return result;
}


// returns the total chamfer distance, and stores in output arguments
// the two directional distances
double fast_chamfer_distance(grayscale_image * binary1, grayscale_image * binary2,
                             long parameter,
                             double * first_distance, double * second_distance)
{
  if ((binary1 == 0) || (binary2 == 0))
  {
    return -1;
  }

  vArray(long) binary1_rows;
  vArray(long) binary1_cols;
  vArray(long) binary2_rows;
  vArray(long) binary2_cols;
  long binary1_number, binary2_number;
  binary1_rows = binary1_cols = binary2_rows = binary2_cols = (vArray(long)) (vZero(long));
  binary1_number = binary2_number = 0;
  vGetNonZeroPoints4(binary1, &binary1_rows, &binary1_cols, &binary1_number);
  vGetNonZeroPoints4(binary2, &binary2_rows, &binary2_cols, &binary2_number);
  double result = vTwoWayChamferFast(binary1_rows, binary1_cols, binary1_number,
                                        binary2_rows, binary2_cols, binary2_number,
                                        parameter, first_distance, second_distance);
  vdelete2(binary1_rows);
  vdelete2(binary1_cols);
  vdelete2(binary2_rows);
  vdelete2(binary2_cols);

  return result;
}


ushort vContourScores(const char * test_filename, 
                       long start, long end,
                       const char * training_filename, 
                       const char * results_base_name)
{
  // Declare and initialize certain useful variables.
  ushort result = 0;
  long score = 0;
  vVideoFile * test_sequence = 0;
  vVideoFile * training_sequence = 0;
  grayscale_image * binary0, * binary1, * binary2, * binary3;
  binary1 = binary2 = binary3 = binary0 = 0;
  grayscale_image * test_gray, * training_gray;
  test_gray = training_gray = 0;
  double scores[4];

  long test_frame, training_frame;
  test_frame = training_frame = 0;
  long number_of_training_frames;
  number_of_training_frames = 0;
  char * current_results = 0;
  long results_name_length = strlen(results_base_name);
  long orientation = 0;
  FILE * results_fp = 0;

  // Open test and training sequences
  test_sequence = vOpenReadVideoFile(test_filename);
  if (test_sequence == 0) 
  {
    vPrint("Can't open test sequence %s\n", test_filename);
    goto exit;
  }

  training_sequence = vOpenReadVideoFile(training_filename);
  if (training_sequence == 0)
  {
    vPrint("Can't open training sequence %s\n", training_filename);
    goto exit;
  }

  number_of_training_frames = training_sequence->NumberOfFrames();
  //number_of_training_frames = 3;
  vPrint("%15s %15s\n", "test_frame", "training_frame");

  // The for loop will read each test frame, and will find
  // its scores against all training frames
  for (test_frame = start; test_frame <= end; test_frame++)
  {
    // Prepare test image before matching. We read a gray-scale
    // frame, and we convert it to a normalized binary. We then rotate it
    // by 90, 180 and 270 degrees. We compare each of those 
    // views to each of the test frames.
    test_gray = test_sequence->ReadFrameGray(test_frame);
    if (test_gray == 0) goto exit;
//    binary0 = (grayscale_image *) vNormalizedBinary(test_gray, 96.0, 256, 256);
    binary0 = (grayscale_image *) vScaleBoundingBox6(test_gray, 192, 128, 128, 
                                                  256, 256);
    vdelete(test_gray);
    binary1 = (grayscale_image *) vRotateLeft(binary0);
    binary2 = (grayscale_image *) vRotateLeft(binary1);
    binary3 = (grayscale_image *) vRotateLeft(binary2);
    test_gray = 0;

    // Create the file in which we will save the results.
    current_results = new char[results_name_length + 15];
    sprintf(current_results, "%s.%li.txt", results_base_name, test_frame);
    results_fp = fopen(current_results, vFOPEN_WRITE);
    if (results_fp == 0)
    {
      vPrint("Error, can't open results file %s\n", current_results);
      goto exit;
    }
    vdelete2(current_results);
    current_results = 0;

    // Do the matching
    for (training_frame = 0; training_frame < number_of_training_frames; 
         training_frame++)
    {
      // Read in the training frame.
      training_gray = training_sequence->ReadFrameGray(training_frame);
      if (training_gray == 0) goto exit;
      grayscale_image * training_binary = (grayscale_image *) training_gray->threshold(1);
      vdelete(training_gray);
      training_gray = 0;
      
      // Compare training frame to each of the four test views
      scores[0] = vCompareContours(binary0, training_binary);
      scores[1] = vCompareContours(binary1, training_binary);
      scores[2] = vCompareContours(binary2, training_binary);
      scores[3] = vCompareContours(binary3, training_binary);
      // Write the score results into the results file
      long i;
      for (i = 0; i < 4; i++)
      {
        long items = fprintf(results_fp, "%li %li %lf\r\n", 
                             training_frame, i, scores[i]);
        if (items < 0)
        {
          vPrint("Error in writing to results file, items = %li\n", items);
        }
      }
      vdelete(training_binary);
      training_binary = 0;
      vPrint("%15li %15li               \r", test_frame, training_frame);
    }

    fclose(results_fp);
    results_fp = 0;
    vdelete(binary0);
    vdelete(binary1);
    vdelete(binary2);
    vdelete(binary3);
    binary0 = binary1 = binary2 = binary3 = 0;
  }
  result = 1;
  vPrint("\n");

exit:
  if (results_fp != 0) fclose(results_fp);
  vdelete2(current_results);
  vCloseVideoFile(test_sequence);
  vCloseVideoFile(training_sequence);
  vdelete(test_gray);
  vdelete(training_gray);
  vdelete(binary0);
  vdelete(binary1);
  vdelete(binary2);
  vdelete(binary3);
  return result;
}


// Auxiliary functions used by vOptimizedChamfer
v3dMatrix<ushort> vGridCellOrder(long image_rows, long image_cols,
                                    long grid_size)
{
  long grid_rows = (image_rows - 1) / grid_size + 1;
  long grid_cols = (image_cols - 1) / grid_size + 1;
  long number_of_grids = grid_rows * grid_cols;
  v3dMatrix<ushort> result_matrix(number_of_grids, number_of_grids, 1);
  vArray2(ushort) result = result_matrix.Matrix2(0);
  long i;
  for (i = 0; i < number_of_grids; i++)
  {
    long row1 = i / grid_cols;
    long col1 = i % grid_cols;
    vector<S_GridDistance> distances(number_of_grids);
    long trash = distances.size();
    long j;
    // Get the distances from all grids to grid i.
    for (j = 0; j < number_of_grids; j++)
    {
      long row2 = j / grid_cols;
      long col2 = j % grid_cols;

      long row_diff = grid_size * vAbs(row2 - row1);
      long col_diff = grid_size * vAbs(col1 - col2);
      if (row_diff > 0) row_diff = row_diff - grid_size;
      if (col_diff > 0) col_diff = col_diff - grid_size;

      distances[j].grid = (ushort) j;
      if ((row1 == row2) && (col1 == col2))
      {
        distances[j].distance = -1;
      }
      else
      {
        distances[j].distance = row_diff * row_diff + col_diff * col_diff;
      }
    }

    // Sort grids based on min distance to grid i.
    std::sort(distances.begin(), distances.end(), S_GridDistanceLess());

    // Store the order of the grids into result[i].
    vArray(ushort) current_row = result[i];
    for (j = 0; j < number_of_grids; j++)
    {
      current_row[j] = distances[j].grid;
    }
  }

  return result_matrix;
}


v3dMatrix<ushort> vGridCellDistance(long image_rows, long image_cols,
                                       long grid_size)
{
  long grid_rows = (image_rows - 1) / grid_size + 1;
  long grid_cols = (image_cols - 1) / grid_size + 1;
  long number_of_cells = grid_rows * grid_cols;
  v3dMatrix<ushort> result_matrix(number_of_cells, number_of_cells, 1);
  long cell1, cell2;
  vArray2(ushort) result2 = result_matrix.Matrix2(0);
  for (cell1 = 0; cell1 < number_of_cells; cell1++)
  {
    ushort row1 = (ushort) (cell1 / grid_cols);
    ushort col1 = (ushort) (cell1 % grid_cols);
    vArray(ushort) result = result2[cell1];
    for (cell2 = 0; cell2 < number_of_cells; cell2++)
    {
      long row2 = cell2 / grid_cols;
      long col2 = cell2 % grid_cols;
      long row_diff = grid_size * vAbs(row2 - row1);
      long col_diff = grid_size * vAbs(col1 - col2);
      if (row_diff > 0) row_diff = row_diff - grid_size;
      if (col_diff > 0) col_diff = col_diff - grid_size;
      long long_square_distance = row_diff * row_diff + col_diff * col_diff;
      ushort square_distance;
      if (long_square_distance < 65535) 
      {
        square_distance = (ushort) long_square_distance;
      }
      else square_distance = 65535;
      result[cell2] = square_distance;
    }
  }
  return result_matrix;
}

vMatrix<long> vGridCellDistance2(long image_rows, long image_cols,
                                        long grid_size)
{
  long grid_rows = (image_rows - 1) / grid_size + 1;
  long grid_cols = (image_cols - 1) / grid_size + 1;
  long number_of_cells = grid_rows * grid_cols;
  vMatrix<long> result_matrix(-number_of_cells, number_of_cells, 
                               -number_of_cells, number_of_cells);
  long square_grid_size = grid_size * grid_size;
  vArray2(long) result2 = result_matrix.GetData();
  long row, col;
  for (row = 0; row < grid_rows; row++)
  {
    long row1;
    if (row > 0) row1 = row - 1;
    else row1 = 0;
    for (col = 0; col < grid_cols; col++)
    {
      long col1;
      if (col > 0) col1 = col - 1;
      else col1 = 0;

      long square_distance = (row1 * row1 + col1 * col1) * square_grid_size;
      result2[row][col] = square_distance;
      result2[row][-col] = square_distance;
      result2[-row][col] = square_distance;
      result2[-row][-col] = square_distance;
    }
  }
  return result_matrix;
}

v3dMatrix<uchar> vMakeGrid(long image_rows, long image_cols,
                              long grid_size)
{
  if ((image_rows < 0) || (image_rows > 256)) 
  {
    return v3dMatrix<uchar>(1, 1, 1);
  }
  if ((image_cols < 0) || (image_cols > 256)) 
  {
    return v3dMatrix<uchar>(1, 1, 1);
  }
  long grid_rows = (image_rows - 1) / grid_size + 1;
  long grid_cols = (image_cols - 1) / grid_size + 1;
  long number_of_cells = grid_rows * grid_cols;
  long max_points_per_cell = grid_size * grid_size;

  v3dMatrix<uchar> result(number_of_cells, max_points_per_cell * 2, 1);
  vWriteValue(&result, (uchar) 0);
  return result;
}


// Grid stats are a 2x(number_of_cells+1) matrix. First row contains
// number of points in each grid cell. Second row contains the 
// number of cells that are non-empty, and the indices of those cells.
v3dMatrix<ushort> vMakeGridStats(long image_rows, long image_cols,
                                    long grid_size)
{
  if ((image_rows < 0) || (image_rows > 256)) 
  {
    return v3dMatrix<ushort>(1, 1, 1);
  }
  if ((image_cols < 0) || (image_cols > 256)) 
  {
    return v3dMatrix<ushort>(1, 1, 1);
  }

  long grid_rows = (image_rows - 1) / grid_size + 1;
  long grid_cols = (image_cols - 1) / grid_size + 1;
  long number_of_cells = grid_rows * grid_cols;

  v3dMatrix<ushort> result(2, number_of_cells + 1, 1);
  vWriteValue(&result, (ushort) 0);
  return result;
}


long vPutPointsToGrid(v3dMatrix<uchar> * grid_matrix, 
                       v3dMatrix<ushort> * grid_log_matrix,
                       long grid_rows, long grid_cols,
                       long grid_size, vArray(long) rows, vArray(long) cols, long number)
{
  vArray2(uchar) grid2 = grid_matrix->Matrix2(0);
  vArray2(ushort) grid_log2 = grid_log_matrix->Matrix2(0);
  vArray(ushort) counters = grid_log2[0];
  vArray(ushort) indices = grid_log2[1];

  long i;
  for (i = 0; i < number; i++)
  {
    long row = rows[i];
    long col = cols[i];
    long grid_row = row / grid_size;
    long grid_col = col / grid_size;
    assert(grid_row < 256);
    assert(grid_col < 256);
    long grid_index = grid_row * grid_cols + grid_col;
    long current_number = counters[grid_index];
    counters[grid_index]++;
    long index = 2 * current_number;
    grid2[grid_index][index] = (uchar) row;
    grid2[grid_index][index+1] = (uchar) col;
    if (current_number == 0)
    {
      indices[0]++;
      long current_grid_indices = indices[0];
      indices[current_grid_indices] = (ushort) grid_index;
    }
  }

  return 1;
}


long vRemovePointsFromGrid(v3dMatrix<uchar> * grid_matrix,
                            v3dMatrix<ushort> * grid_log_matrix)
{
  vArray2(ushort) grid_log2 = grid_log_matrix->Matrix2(0);
  vArray(ushort) counters = grid_log2[0];
  vArray(ushort) indices = grid_log2[1];
  long number_of_indices = indices[0];
  long i;
  for (i = 1; i <= number_of_indices; i++)
  {
    long index = indices[i];
    counters[index] = 0;
  }
  indices[0] = 0;
  return 1;
}


ushort vContourScoresFast(const char * test_filename, 
                           long start, long end, long training_frames,
                           const char * training_filename, 
                           const char * results_base_name,
                           long grid_size, long subsampling)
{
  // Declare and initialize certain useful variables.
  ushort result = 0;
  long score = 0;
  long i = 0;
  vVideoFile * test_sequence = 0;
  vRLE_Sequence * training_sequence = 0;
  grayscale_image * binary0, * binary1, * binary2, * binary3;
  v3dMatrix<float> * dt_0, * dt_1, * dt_2, * dt_3;
  binary1 = binary2 = binary3 = binary0 = 0;
  dt_0 = dt_1 = dt_2 = dt_3 = 0;
  grayscale_image * test_gray, * training_gray;
  test_gray = training_gray = 0;
  double scores[4];
  vArray(long) binary0_rows;
  vArray(long) binary0_cols;
  vArray(long) binary1_rows;
  vArray(long) binary1_cols;
  vArray(long) binary2_rows;
  vArray(long) binary2_cols;
  vArray(long) binary3_rows;
  vArray(long) binary3_cols;
  binary0_rows = binary0_cols = binary1_rows = binary1_cols = vZero(long);
  binary2_rows = binary2_cols = binary3_rows = binary3_cols = vZero(long);

  long test_frame, training_frame;
  test_frame = training_frame = 0;
  long number_of_training_frames;
  number_of_training_frames = 0;
  char * current_results = 0;
  long results_name_length = strlen(results_base_name);
  long orientation = 0;
  FILE * results_fp = 0;
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  long training_rows, training_cols, training_pixels;
  training_rows = training_cols = training_pixels = 0;
  long number_of_test_frames = 0;

  // Open test and training sequences
  test_sequence = vOpenReadVideoFile(test_filename);
  if (test_sequence == 0) 
  {
    vPrint("Can't open test sequence %s\n", test_filename);
    goto exit;
  }

  training_sequence = new vRLE_Sequence(training_filename);
  if (training_sequence == 0)
  {
    vPrint("Can't open training sequence %s\n", training_filename);
    goto exit;
  }

  number_of_training_frames = training_sequence->NumberOfFrames();
  training_rows = training_sequence->Rows();
  training_cols = training_sequence->Cols();
  training_pixels = training_rows * training_cols;
  rows = vnew(long, training_pixels);
  cols = vnew(long, training_pixels);
  number_of_test_frames = test_sequence->NumberOfFrames();
  // Set start, end, training_frames, parameter, to default_values, if they are -1,
  // reject them otherwise.
  if (start == -1) start = 0;
  if (end == -1) end = number_of_test_frames - 1;
  if (training_frames == -1) training_frames = number_of_training_frames;
  if (grid_size == -1) grid_size = 10;
  if (subsampling < 0) subsampling = 1;
  if ((start < 0) || (start >= number_of_test_frames)) goto exit;
  if ((end < 0) || (end >= number_of_test_frames)) goto exit;
  if ((training_frames < 0) || 
      (training_frames > number_of_training_frames)) goto exit;
  if ((grid_size < 4) || (grid_size > 256)) goto exit;

  //number_of_training_frames = 3;
  vPrint("%15s %15s\n", "test_frame", "training_frame");
  // The for loop will read each test frame, and will find
  // its scores against all training frames
  for (test_frame = start; test_frame <= end; test_frame++)
  {
    // Prepare test image before matching. We read a gray-scale
    // frame, and we convert it to a normalized binary. We then rotate it
    // by 90, 180 and 270 degrees. We compare each of those 
    // views to each of the test frames.
    test_gray = test_sequence->ReadFrameGray(test_frame);
    if (test_gray == 0) goto exit;
//    binary0 = (grayscale_image *) vNormalizedBinary(test_gray, 96.0, 256, 256);
    grayscale_image * binary0 = 
      (grayscale_image *) vScaleBoundingBox6(test_gray, 192, 128, 128, 256, 256);
    vdelete(test_gray);
    binary1 = (grayscale_image *) vRotateLeft(binary0);
    binary2 = (grayscale_image *) vRotateLeft(binary1);
    binary3 = (grayscale_image *) vRotateLeft(binary2);
    test_gray = 0;

    dt_0 = vCVDistanceTransform(binary0);
    dt_1 = vCVDistanceTransform(binary1);
    dt_2 = vCVDistanceTransform(binary2);
    dt_3 = vCVDistanceTransform(binary3);

    long binary0_number, binary1_number, binary2_number, binary3_number;
    binary0_number = binary1_number = binary2_number = binary3_number = 0;
    vGetNonZeroPoints4(binary0, &binary0_rows, &binary0_cols, &binary0_number);
    vGetNonZeroPoints4(binary1, &binary1_rows, &binary1_cols, &binary1_number);
    vGetNonZeroPoints4(binary2, &binary2_rows, &binary2_cols, &binary2_number);
    vGetNonZeroPoints4(binary3, &binary3_rows, &binary3_cols, &binary3_number);

    grayscale_image * binaries[4] = {binary0, binary1, binary2, binary3};
    v3dMatrix<float> * dts[4] = {dt_0, dt_1, dt_2, dt_3};
    vArray(long) binary_rows[4] = {binary0_rows, binary1_rows, 
                             binary2_rows, binary3_rows};
    vArray(long) binary_cols[4] = {binary0_cols, binary1_cols, 
                             binary2_cols, binary3_cols};
    long binary_numbers[4] = {binary0_number, binary1_number, 
                              binary2_number, binary3_number};

    // Create the file in which we will save the results.
    current_results = new char[results_name_length + 15];
    sprintf(current_results, "%s.%li.txt", results_base_name, test_frame);
    results_fp = fopen(current_results, vFOPEN_WRITE);
    if (results_fp == 0)
    {
      vPrint("Error, can't open results file %s\n", current_results);
      goto exit;
    }
    vdelete2(current_results);
    current_results = 0;

    // Do the matching
    for (training_frame = 0; training_frame < training_frames; 
         training_frame++)
    {
//      training_gray = training_sequence->ReadFrameGray(training_frame);
//      if (training_gray == 0) goto exit;

     // Read in the training frame.
      vRLE_Gray * training_rle = (vRLE_Gray *) training_sequence->ReadRLEFrame(training_frame);
      if (training_rle == 0) goto exit;
      long number = 0;
      vRLENonZeroPoints(training_rle, rows, cols, &number);

      for (i = 0; i < 4; i++)
      {
        double distance1 = vOptimizedChamfer2(binary_rows[i], binary_cols[i], 
                                                 binary_numbers[i], rows, cols,
                                                 number, grid_size, subsampling);
        double distance2 = vChamferFromDT(rows, cols, number, dts[i]);
//        double distance1 = 0;
//        double distance2 = 0;
        scores[i] = distance1 + distance2;
      }
      

      // Compare training frame to each of the four test views
//      scores[0] = vTwoWayChamferFast(binary0_rows, binary0_cols, binary0_number,
//                                        rows, cols, number, parameter);
//      scores[1] = vTwoWayChamferFast(binary1_rows, binTTary1_cols, binary1_number,
//                                        rows, cols, number, parameter);
//      scores[2] = vTwoWayChamferFast(binary2_rows, binary2_cols, binary2_number,
//                                        rows, cols, number, parameter);
//      scores[3] = vTwoWayChamferFast(binary3_rows, binary3_cols, binary3_number,
//                                        rows, cols, number, parameter);
      // Write the score results into the results file
      for (i = 0; i < 4; i++)
      {
        long items = fprintf(results_fp, "%li %li %lf\r\n", 
                             training_frame, i, scores[i]);
        if (items < 0)
        {
          vPrint("Error in writing to results file, items = %li\n", items);
        }
      }
      vdelete(training_rle);
      training_rle = 0;
      vPrint("%15li %15li               \r", test_frame, training_frame);
    }

    fclose(results_fp);
    results_fp = 0;

    long i;
    for (i = 0; i < 4; i++)
    {
      vdelete(binaries[i]);
      vdelete(dts[i]);
    }
    
    vdelete2(binary0_rows);
    vdelete2(binary0_cols);
    vdelete2(binary1_rows);
    vdelete2(binary1_cols);
    vdelete2(binary2_rows);
    vdelete2(binary2_cols);
    vdelete2(binary3_rows);
    vdelete2(binary3_cols);
    binary0_rows = binary0_cols = binary1_rows = binary1_cols = vZero(long);
    binary2_rows = binary2_cols = binary3_rows = binary3_cols = vZero(long);
    binary0 = binary1 = binary2 = binary3 = 0;
    dt_0 = dt_1 = dt_2 = dt_3 = 0;
  }
  result = 1;
  vPrint("\n");

exit:
  if (results_fp != 0) fclose(results_fp);
  vdelete2(current_results);
  vCloseVideoFile(test_sequence);
  vCloseVideoFile(training_sequence);
  vdelete(test_gray);
  vdelete(training_gray);
  vdelete(binary0);
  vdelete(binary1);
  vdelete(binary2);
  vdelete(binary3);
  vdelete(dt_0);
  vdelete(dt_1);
  vdelete(dt_2);
  vdelete(dt_3);
  vdelete2(rows);
  vdelete2(cols);
  return result;
}


ushort vMultipleChamferScores(const char * test_filename, 
                                 long start, long end,
                                 const char * training_filename, 
                                 const char * results_base_name)
{
  // Declare and initialize certain useful variables.
  ushort result = 0;
  long score = 0;
  vVideoFile * test_sequence = 0;
  vVideoFile * training_sequence = 0;
  grayscale_image * binary0, * binary1, * binary2, * binary3;
  binary1 = binary2 = binary3 = binary0 = 0;
  grayscale_image * test_gray, * training_gray;
  test_gray = training_gray = 0;
  double scores[4];

  long test_frame, training_frame;
  test_frame = training_frame = 0;
  long number_of_training_frames;
  number_of_training_frames = 0;
  char * current_results = 0;
  long results_name_length = strlen(results_base_name);
  long orientation = 0;
  FILE * results_fp = 0;

  // Open test and training sequences
  test_sequence = vOpenReadVideoFile(test_filename);
  if (test_sequence == 0) 
  {
    vPrint("Can't open test sequence %s\n", test_filename);
    goto exit;
  }

  training_sequence = vOpenReadVideoFile(training_filename);
  if (training_sequence == 0)
  {
    vPrint("Can't open training sequence %s\n", training_filename);
    goto exit;
  }

  number_of_training_frames = training_sequence->NumberOfFrames();
  //number_of_training_frames = 3;
  vPrint("%15s %15s\n", "test_frame", "training_frame");

  // The for loop will read each test frame, and will find
  // its scores against all training frames
  for (test_frame = start; test_frame <= end; test_frame++)
  {
    // Prepare test image before matching. We read a gray-scale
    // frame, and we convert it to a normalized binary. We then rotate it
    // by 90, 180 and 270 degrees. We compare each of those 
    // views to each of the test frames.
    test_gray = test_sequence->ReadFrameGray(test_frame);
    if (test_gray == 0) goto exit;
//    binary0 = (grayscale_image *) vNormalizedBinary(test_gray, 96.0, 256, 256);
    binary0 = (grayscale_image *) vScaleBoundingBox6(test_gray, 192, 128, 128, 
                                                   256, 256);
    vdelete(test_gray);
    binary1 = (grayscale_image *) vRotateLeft(binary0);
    binary2 = (grayscale_image *) vRotateLeft(binary1);
    binary3 = (grayscale_image *) vRotateLeft(binary2);
    test_gray = 0;
//    binary0->Save("D:\\vassilis\\mine\\sequences\\binary0.pgm");
    binary0->save("G:\\users\\athitsos\\sequences\\binary0.pgm");
    binary1->save("G:\\users\\athitsos\\sequences\\binary1.pgm");
    binary2->save("G:\\users\\athitsos\\sequences\\binary2.pgm");
    binary3->save("G:\\users\\athitsos\\sequences\\binary3.pgm");

    // Create the file in which we will save the results.
    current_results = new char[results_name_length + 15];
    sprintf(current_results, "%s.%li.txt", results_base_name, test_frame);
    results_fp = fopen(current_results, vFOPEN_WRITE);
    if (results_fp == 0)
    {
      vPrint("Error, can't open results file %s\n", current_results);
      goto exit;
    }
    vdelete2(current_results);
    current_results = 0;

    // Do the matching
    for (training_frame = 0; training_frame < number_of_training_frames; 
         training_frame++)
    {
      // Read in the training frame.
      training_gray = training_sequence->ReadFrameGray(training_frame);
      if (training_gray == 0) goto exit;
      grayscale_image * training_binary = (grayscale_image *) training_gray->threshold(1);
      vdelete(training_gray);
      training_gray = 0;
      
      // Compare training frame to each of the four test views
      double scaling;
      long translation_rows, translation_cols;
      scaling = translation_rows = translation_cols = 0;
      vChamferDistances(binary0, training_binary, &scores[0], &translation_rows,
                           &translation_cols, &scaling);
      vChamferDistances(binary1, training_binary, &scores[1], &translation_rows,
                           &translation_cols, &scaling);
      vChamferDistances(binary2, training_binary, &scores[2], &translation_rows,
                           &translation_cols, &scaling);
      vChamferDistances(binary3, training_binary, &scores[3], &translation_rows,
                           &translation_cols, &scaling);
      // Write the score results into the results file
      long i;
      for (i = 0; i < 4; i++)
      {
        long items = fprintf(results_fp, "%li %li %lf\r\n", 
                             training_frame, i, scores[i]);
        if (items < 0)
        {
          vPrint("Error in writing to results file, items = %li\n", items);
        }
      }
      vdelete(training_binary);
      training_binary = 0;
      vPrint("%15li %15li               \r", test_frame, training_frame);
    }

    fclose(results_fp);
    results_fp = 0;
    vdelete(binary0);
    vdelete(binary1);
    vdelete(binary2);
    vdelete(binary3);
    binary0 = binary1 = binary2 = binary3 = 0;
  }
  result = 1;
  vPrint("\n");

exit:
  if (results_fp != 0) fclose(results_fp);
  vdelete2(current_results);
  vCloseVideoFile(test_sequence);
  vCloseVideoFile(training_sequence);
  vdelete(test_gray);
  vdelete(training_gray);
  vdelete(binary0);
  vdelete(binary1);
  vdelete(binary2);
  vdelete(binary3);
  return result;
}


// This function returns an image i, which at pixel i,j has the value 0
// if image1(i,j) is 0, and 1 + the distance between i,j and the closest
// non-zero pixel in image2 otherwise. The reason we add 1 to the distance
// is so that we can separate non-zero points of base1 with zero set
// distance to base2 from zero points in base1. In general, I use the
// term "set distance of a point A to the set B" as the smallest distance
// between A and a point of the set B.
v3dMatrix<long> * vSetDistance(general_image * base1, general_image * base2)
{
  // First binarize the two images.
  v3dMatrix<uchar> * binary1 = base1->threshold(1);
  v3dMatrix<uchar> * binary2 = base2->threshold(1);

  vector<long> rows2, cols2;
  vGetNonZeroPoints((grayscale_image *) binary2, &rows2, &cols2);
  long number_of_points = rows2.size();
  vdelete(binary2);
  long rows = binary1->Rows();
  long cols = binary1->Cols();

  v3dMatrix<double> * distances = new v3dMatrix<double>(rows, cols, 1);
  vArray2(double) distances2 = distances->Matrix2(0);
  vArray2(uchar) input2 = binary1->Matrix2(0);
  long row, col;
  for (row = 0; row < rows; row++)
  {
    vArray(double) distances1 = distances2[row];
    vArray(uchar) input1 = input2[row];
    for (col = 0; col < cols; col++)
    {
      if (input1[col] == 0) 
      {
        distances1[col] = 0;
        continue;
      }
      long index, square_distance;
      vClosestPoint(row, col, &rows2, &cols2, &square_distance, &index);
      distances1[col] = 1.0 + sqrt((double) square_distance);
    }
  }
  vdelete(binary1);
  // Add 0.5 to distances, so that when we convert the pixel values
  // to long they will be correctly rounded.
  function_image_add(distances, 0.5, distances);
  v3dMatrix<long> * result = new v3dMatrix<long>(distances);
  vdelete(distances);
  return result;
}


ushort vChamferDistances2(grayscale_image * in_binary1, grayscale_image * binary2,
                              double * out_distance, long * out_rows, long * out_cols,
                              double * out_scaling)
{
  grayscale_image * binary1 = (grayscale_image *) vCopyRectangle(in_binary1, 0, 255, 0, 255);
  long rows = binary1->Rows();
  long cols = binary1->Cols();

  v3dMatrix<float> * distances1 = vCVDistanceTransform(binary1);
  v3dMatrix<float> * distances2 = vCVDistanceTransform(binary2);
  vector<long> rows1, cols1, rows2, cols2;
  vGetNonZeroPoints(binary1, &rows1, &cols1);
  long points1 = rows1.size();
  vGetNonZeroPoints(binary2, &rows2, &cols2);
  long points2 = rows2.size();
  
  vector<long> scaled_test_rows(points1);
  vector<long> scaled_test_cols(points1);
  vector<long> test_rows(points1);
  vector<long> test_cols(points1);

  vector<long> scaled_train_rows(points2);
  vector<long> scaled_train_cols(points2);
  vector<long> train_rows(points2);
  vector<long> train_cols(points2);
  long translation_rows, translation_cols;
  double scaling = 0;
  long min_rows, min_cols;
  double min_distance, min_scaling;
  min_rows = -1;
  min_cols = -1;
  min_distance = rows * rows + cols * cols;

  // We want to search over horizontal translation, vertical translation and
  // scaling. We want to first sample the space, find a peak, then sample around
  // the peak, find a new peak, and then sample again.

  // First search using the following sampling. (9x9x7 points)
  long step = 5;
  //vPrint("Enter row/col step: ");
  //vScan("%li", &step);
  long small_rows = -20;
  long big_rows = 20;
  long row_step = step;
  long small_cols = -20;
  long big_cols = 20;
  long col_step = step;
  double small_scaling = 0.85;
  double big_scaling = 1.15;
  double scaling_step = 0.05;
//  vPrint("Scaling step: ");
//  vScan("%lf", &scaling_step);
  long counter = 0;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }

  
  // Now look closer. (5x5x5 points)
  small_rows = min_rows - 4;
  big_rows = min_rows + 4;
  row_step = 2;
  small_cols = min_cols - 4;
  big_cols = min_cols + 4;
  col_step = 2;
  small_scaling = min_scaling - 0.04;
  big_scaling = min_scaling + 0.04;
  scaling_step = 0.02;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }
  
  // Now look even closer (final sampling) (5x5x5 points)
  small_rows = min_rows - 2;
  big_rows = min_rows + 2;
  row_step = 1;
  small_cols = min_cols - 2;
  big_cols = min_cols + 2;
  col_step = 1;
  small_scaling = min_scaling - 0.02;
  big_scaling = min_scaling + 0.02;
  scaling_step = 0.01;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }
  
  vdelete(distances1);
  vdelete(distances2);
  vdelete(binary1);
  *out_rows = min_rows;
  *out_cols = min_cols;
  *out_scaling = min_scaling;
  *out_distance = min_distance;
  return 1;
}


ushort S_InitializeRandoms(long * randoms, long size)
{
  for (long i = 0; i < size; i++)
  {
    randoms[i] = rand();
  }
  return 1;
}

ushort vChamferDistances(grayscale_image * in_binary1, grayscale_image * binary2,
                            double * out_distance, long * out_rows, long * out_cols,
                            double * out_scaling)
{
  static long randoms[65536];
  // The next line just initializes randoms;
  static ushort trash = S_InitializeRandoms(randoms, 65536);
  long point_sampling = 5;
  grayscale_image * binary1 = (grayscale_image *) vCopyRectangle(in_binary1, 0, 255, 0, 255);
  long rows = binary1->Rows();
  long cols = binary1->Cols();

  v3dMatrix<float> * distances1 = vCVDistanceTransform(binary1);
  v3dMatrix<float> * distances2 = vCVDistanceTransform(binary2);
  vector<long> temp_rows1, temp_cols1, temp_rows2, temp_cols2;
  vector<long> rows1, cols1, rows2, cols2;
  vGetNonZeroPoints(binary1, &temp_rows1, &temp_cols1);
  long temp_points1 = temp_rows1.size();
  vGetNonZeroPoints(binary2, &temp_rows2, &temp_cols2);
  long temp_points2 = temp_rows2.size();

  long i;
  // Sample one out of every point_sampling points (to speed up the matching).
  for (i = 0; i < temp_points1; i++)
  {
    if (randoms[i] % point_sampling == 0)
    {
      rows1.push_back(temp_rows1[i]);
      cols1.push_back(temp_cols1[i]);
    }
  }
  
  for (i = 0; i < temp_points2; i++)
  {
    if (randoms[i] % point_sampling == 0)
    {
      rows2.push_back(temp_rows2[i]);
      cols2.push_back(temp_cols2[i]);
    }
  }

  long points1 = rows1.size();
  long points2 = rows2.size();
//  vPrint("points1 = %li, %li, points2 = %li, %li\n",
//          temp_points1, points1, temp_points2, points2);

  vector<long> scaled_test_rows(points1);
  vector<long> scaled_test_cols(points1);
  vector<long> test_rows(points1);
  vector<long> test_cols(points1);

  vector<long> scaled_train_rows(points2);
  vector<long> scaled_train_cols(points2);
  vector<long> train_rows(points2);
  vector<long> train_cols(points2);
  long translation_rows, translation_cols;
  double scaling = 0;
  long min_rows, min_cols;
  double min_distance, min_scaling;
  min_rows = -1;
  min_cols = -1;
  min_distance = rows * rows + cols * cols;

  // We want to search over horizontal translation, vertical translation and
  // scaling. We want to first sample the space, find a peak, then sample around
  // the peak, find a new peak, and then sample again.

  // First search using the following sampling. (9x9x7 points)
  long step = 5;
  //vPrint("Enter row/col step: ");
  //vScan("%li", &step);
  long small_rows = -20;
  long big_rows = 20;
  long row_step = step;
  long small_cols = -20;
  long big_cols = 20;
  long col_step = step;
  double small_scaling = 0.85;
  double big_scaling = 1.15;
  double scaling_step = 0.05;
//  vPrint("Scaling step: ");
//  vScan("%lf", &scaling_step);
  long counter = 0;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }

  
  // Now look closer. (5x5x5 points)
  small_rows = min_rows - 4;
  big_rows = min_rows + 4;
  row_step = 2;
  small_cols = min_cols - 4;
  big_cols = min_cols + 4;
  col_step = 2;
  small_scaling = min_scaling - 0.04;
  big_scaling = min_scaling + 0.04;
  scaling_step = 0.02;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }
  
  // Now look even closer (final sampling) (5x5x5 points)
  small_rows = min_rows - 2;
  big_rows = min_rows + 2;
  row_step = 1;
  small_cols = min_cols - 2;
  big_cols = min_cols + 2;
  col_step = 1;
  small_scaling = min_scaling - 0.02;
  big_scaling = min_scaling + 0.02;
  scaling_step = 0.01;

  for (scaling = small_scaling; scaling <= big_scaling; scaling += scaling_step)
  {
    vScalePoints(&rows1, &cols1, &scaled_test_rows, &scaled_test_cols, 
                  scaling, 128, 128);
    vScalePoints(&rows2, &cols2, &scaled_train_rows, &scaled_train_cols, 
                  1.0 / scaling, 128, 128);
    for (translation_rows = small_rows; translation_rows <= big_rows; translation_rows += row_step)
    {
      for (translation_cols = small_cols; translation_cols <= big_cols; translation_cols += col_step)
      {
        vTranslatePoints(&scaled_test_rows, &scaled_test_cols, &test_rows, &test_cols,
                          translation_rows, translation_cols);
        double double_rows = (double) translation_rows;
        double double_cols = (double) translation_cols;
        double_rows = double_rows / scaling;
        double_cols = double_cols / scaling;
        long scaled_rows = (long) (double_rows);
        long scaled_cols = (long) (double_cols);
        vTranslatePoints(&scaled_train_rows, &scaled_train_cols, &train_rows, &train_cols,
                          -scaled_rows, -scaled_cols);
        double distance1 = vSetDistance(&test_rows, &test_cols, distances2);
        double distance2 = vSetDistance(&train_rows, &train_cols, distances1);
        double distance = distance1 + distance2 * scaling;
        if (distance < min_distance)
        {
          min_distance = distance;
          min_rows = translation_rows;
          min_cols = translation_cols;
          min_scaling = scaling;
        }
        counter++;
        //vPrint("\r%li         ", counter);
      }
    }
    //vPrint("\n");
  }
  
  vdelete(distances1);
  vdelete(distances2);
  vdelete(binary1);
  *out_rows = min_rows;
  *out_cols = min_cols;
  *out_scaling = min_scaling;
  *out_distance = min_distance;
  return 1;
}


