#ifndef VASSILIS_SIMPLE_CHAMFER_H
#define VASSILIS_SIMPLE_CHAMFER_H


#include "basics/image.h"
#include "basics/algebra.h"
#include "basics/rle_encoding.h"


long vRLENonZeroPoints(vRLE_Gray * rle, vArray(long) rows, vArray(long) cols, 
                        long * number);

ushort vGetBinaryEdges(grayscale_image * binary, vector<long> * rows, 
                        vector<long> * cols);

ushort vClosestPoint(long row, long col, vector<long> * rows2, 
                      vector<long> * cols2, long * square_distance, long * index);

// Calculates the directed Chamfer distance between the points in
// rows1, cols1 and the points in rows2, cols2.
double vChamferDistance(vector<long> * rows1, vector<long> * cols1,
                         vector<long> * rows2, vector<long> * cols2);

// Like vChamferDistance but it applies some optimizing
// heuristics (the degree of utilization of those heuristics depends
// on the "parameter" passed).
double vChamferDistanceFast(vArray(long) rows1, vArray(long) cols1, long number1,
                               vArray(long) rows2, vArray(long) cols2, long number2,
                               long parameter);

// Like vChamferDistance but instead of getting the list of edge
// points for the second image, it gets the distance transform of the
// second image.
double vChamferFromDT(grayscale_image * the_image, v3dMatrix<float> * dt_image);

double vChamferFromDT(vArray(long) rows1, vArray(long) cols1, 
                       long number1, v3dMatrix<float> * dt_image);

// Like vChamferDistance but instead of getting the list of edge
// points for the second image, it gets the distance transform of the
// second image.
double vChamferFromDT(vArray(uchar) rows1, vArray(uchar) cols1, 
                       long number1, v3dMatrix<float> * dt_image);

// Like vChamferDistance but instead of getting the list of edge
// points for the second image, it gets the distance transform of the
// second image.
double vChamferFromDT(vArray(uchar) rows1, vArray(uchar) cols1, 
                       long number1, v3dMatrix<uchar> * dt_image);

// Like vChamferDistance but instead of getting the list of edge
// points for the second image, it gets the distance transform of the
// second image.
double vChamferFromDT(vector<long> * rows1, vector<long> * cols1, 
                         v3dMatrix<uchar> * dt_image);
double vChamferFromDT(vector<long> * rows1, vector<long> * cols1, 
                         v3dMatrix<float> * dt_image);

// Like vChamferDistance but it searches nearest neighbors in a more
// efficient way. It only works, though, if the points come from images
// of size at most 256x256 (row < 256, col < 256).
double vOptimizedChamfer(vArray(long) rows1, vArray(long) cols1, long number1,
                            vArray(long) rows2, vArray(long) cols2, long number2,
                            long grid_size);

double vOptimizedChamfer2(vArray(long) rows1, vArray(long) cols1, long number1,
                             vArray(long) rows2, vArray(long) cols2, long number2,
                             long grid_size, long subsampling);

// Auxiliary functions used by vOptimizedChamfer
v3dMatrix<ushort> vGridCellOrder(long image_rows, long image_cols,
                                    long grid_size);

v3dMatrix<ushort> vGridCellDistance(long image_rows, long image_cols,
                                       long grid_size);

vMatrix<long> vGridCellDistance2(long image_rows, long image_cols,
                                     long grid_size);

v3dMatrix<uchar> vMakeGrid(long image_rows, long image_cols,
                              long grid_size);

v3dMatrix<ushort> vMakeGridStats(long image_rows, long image_cols,
                                    long grid_size);

long vPutPointsToGrid(v3dMatrix<uchar> * grid_matrix, 
                       v3dMatrix<ushort> * grid_log_matrix,
                       long grid_rows, long grid_cols,
                       long grid_size, vArray(long) rows, vArray(long) cols, long number);

long vRemovePointsFromGrid(v3dMatrix<uchar> * grid_matrix,
                            v3dMatrix<ushort> * grid_log_matrix);


double vTwoWayChamferFast(vArray(long) rows1, vArray(long) cols1, long number1,
                             vArray(long) rows2, vArray(long) cols2, long number2,
                             long parameter, double * first = 0, double * second = 0);

// This function returns an image i, which at pixel i,j has the value 0
// if image1(i,j) is 0, and the distance between i,j and the closest
// non-zero pixel in image2 otherwise.
v3dMatrix<long> * vSetDistance(general_image * image1, general_image * image2);


double vCompareContours(grayscale_image * binary1, grayscale_image * binary2);

// returns the total chamfer distance, and stores in output arguments
// the two directional distances
double function_chamfer_distance(grayscale_image * binary1, grayscale_image * binary2,
                                 double * first_distance = 0, double * second_distance = 0);


double vCompareContoursFast(grayscale_image * binary1, grayscale_image * binary2,
                             long parameter);

// returns the total chamfer distance, and stores in output arguments
// the two directional distances
double fast_chamfer_distance(grayscale_image * binary1, grayscale_image * binary2,
                             long parameter,
                             double * first_distance = 0, double * second_distance = 0);


// vContourScores computes similarity based on the chamfer distance between 
// test images (saved in test_filename) and training images (saved in 
// training_filename). For each test image, its distance to all training
// images is computed, and the results are saved in files whose name 
// starts with results_base_name.
ushort vContourScores(const char * test_filename, 
                       long start, long end, 
                       const char * training_filename, 
                       const char * results_base_name);

// Like vContourScores, but using heuristics (specified by grid_size
// and subsampling) to speed up the process.
ushort vContourScoresFast(const char * test_filename, 
                           long start, long end, long training_frames,
                           const char * training_filename, 
                           const char * results_base_name,
                           long grid_size, long subsampling);


ushort vMultipleChamferScores(const char * test_filename, 
                                 long start, long end,
                                 const char * training_filename, 
                                 const char * results_base_name);

ushort vChamferDistances2(grayscale_image * in_binary1, grayscale_image * binary2,
                              double * out_distance, long * out_rows, long * out_cols,
                              double * out_scaling);

ushort vChamferDistancesSlow(grayscale_image * binary1, grayscale_image * binary2,
                                double * out_distance, long * out_rows, long * out_cols,
                                double * out_scaling);

ushort vChamferDistances(grayscale_image * binary1, grayscale_image * binary2,
                            double * out_distance, long * out_rows, long * out_cols,
                            double * out_scaling);


// test_gray is supposed to be a binary image of feature points (edges 
// in my 
// current application). database is a sequence of model images (also
// represented as images of feature points). If the database has K
// images, the result will have 4*K values, each of them being the chamfer
// distance between test_gray and a model image. For each image in the
// database we actually use 4 images, the results of rotating that image
// by 0, 90, 180 and 270 degrees.
vMatrix<double> 
vDatabaseChamfer(grayscale_image * test_gray, vRLE_Sequence * database,
                  long grid_size, long subsampling, long training_frames);


// test_gray is supposed to be a binary image of feature points (edges 
// in my 
// current application). database is a sequence of model images (also
// represented as images of feature points). frames and views specify
// the database frames that we want to compare test_gray with, and 
// the orientation from which we want to compare them (there are four
// possible orientations, corresponding to rotating the training image
// to the right by 0, 90, 180 or 270 degrees). start and dimensions specify
// that we will use "dimension" frames and views, starting at start.
class vPointSetSequence;

vMatrix<double> 
vDatabasePicksChamfer(grayscale_image * test_gray, vPointSetSequence * database,
                       v3dMatrix<long> * frames, v3dMatrix<long> * views,
                       long start, long dimensions,
                       long grid_size, long subsampling);

vMatrix<double> 
vDatabasePicksChamfer(grayscale_image * test_gray, vRLE_Sequence * database,
                       v3dMatrix<long> * frames, v3dMatrix<long> * views,
                       long start, long dimensions,
                       long grid_size, long subsampling);

vMatrix<double> 
vDatabasePicksChamfer(grayscale_image * test_gray, vRLE_Sequence * database,
                       v3dMatrix<float> * reference_images,
                       long start, long dimensions,
                       long grid_size, long subsampling);

vMatrix<double> 
vDatabasePicksChamferIM(grayscale_image * test_gray, vPointSetSequence * database,
                         v3dMatrix<long> * frames, v3dMatrix<long> * views,
                         long start, long dimensions,
                         long grid_size, long subsampling);

vMatrix<double> 
vDatabasePicksChamferIM(grayscale_image * test_gray, vPointSetSequence * database,
                         v3dMatrix<float> * reference_images,
                         long start, long dimensions,
                         long grid_size, long subsampling);

                       
// vRandomChamferDistances picks some random views from the database, and
// for each picked view it calculates the chamfer distances between
// that view and the entire database and saves the results on a file.
// A separate file is created for each random pick. 
// vMergeRandomChamferDistances can be used to merge those files together.
long vRandomChamferDistances(vRLE_Sequence * database, long number,
                              const char * target, long training_frames);


// vMergeRandomChamferDistances takes a sequence of files created by
// vRandomChamferDistances and puts them together in one big file.
// The individual files have random matrices of type v3dMatrix<double>.
// The result is saved as v3dMatrix<ushort> to save disk space.
long vMergeRandomChamferDistances(const char * base, long start, long end);

// dt is the distance transform of the input image.
// database is a sequence of model images (also
// represented as images of feature points). frames and views specify
// the database frames that we want to compare test_gray with, and 
// the orientation from which we want to compare them (there are four
// possible orientations, corresponding to rotating the training image
// to the right by 0, 90, 180 or 270 degrees).
// We calculate the one-way chamfer distance from model to image.
vMatrix<double> 
vOneWayChamfer(v3dMatrix<float> * dt, vRLE_Sequence * database);

vMatrix<double> 
vOneWayChamfer(v3dMatrix<float> * dt, vRLE_Sequence * database,
                v3dMatrix<long> * frames, v3dMatrix<long> * views);

class vPointSetSequence;

// dt is the distance transform of the input image.
// database is a sequence of model images (also
// represented as images of feature points). frames and views specify
// the database frames that we want to compare test_gray with, and 
// the orientation from which we want to compare them (there are four
// possible orientations, corresponding to rotating the training image
// to the right by 0, 90, 180 or 270 degrees).
// We calculate the one-way chamfer distance from model to image.
vMatrix<double> 
vOneWayChamfer2(v3dMatrix<float> * dt, vPointSetSequence * database,
                 v3dMatrix<long> * frames, v3dMatrix<long> * views);

vMatrix<double> 
vOneWayChamfer2(v3dMatrix<float> * dt, vPointSetSequence * database);

// database_edges is a video file with edge images of the model.
// point_sequence is a structure that contains, for every database frame,
// the list of all non-zero pixels in the corresponding edge image in 
// database_edges. (frames[i], views[i]) specifies the i-th base image.
// For each of those base images, we compute the one way chamfer distances
// from each database image to them (the order matters). The result is
// a matrix whose rows are equal to the number of database images, and
// whose cols are equal to the number of base images, so that entry i,j
// is the one-way distance from the i-th database image to the j-th base image.
// This matrix is saved into a file that is obtained by replacing the ending
// of the filename of database_edges with ".chamfer1_index".
long vSaveChamferIndexIM(vRLE_Sequence * database_edges,
                          vPointSetSequence * point_sequence, 
                          v3dMatrix<long> * frames,
                          v3dMatrix<long> * views);

// Saves all directed chamfer distances between every 
// shape/viewpoint combination in the database and every 
// database image. The direction is from database image
// to shape viewpoint combination.
v3dMatrix<uchar> * vAll_MI_Distances(vVideoFile * database_edges, 
                                        vPointSetSequence * points);

// Saves all directed chamfer distances between every 
// shape/viewpoint combination in the database and every 
// database image. The direction is from shape/viewpoint combination
// to database image.
v3dMatrix<uchar> * vAll_IM_Distances(vVideoFile * database_edges, 
                                        vPointSetSequence * points);






















#endif // VASSILIS_SIMPLE_CHAMFER_H
