#include "stdafx.h"

#include "vplatform.h"

#include <assert.h>
#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#include <fstream>
#else 
#include <iostream>
#include <fstream>
#endif
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef VASSILIS_WINDOWS
#include <direct.h>
#include <vector> 
#include "pc_aux.h"
#include "opencv_utils.h"
#include "avi_file.h"
#endif // VASSILIS_WINDOWS

#ifdef VASSILIS_SGI_PLATFORM
#include <algo.h>
#include <vector.h>
#endif // VASSILIS_SGI_PLATFORM


#include "basics/auxiliaries.h"
#include "basics/drawing.h"
#include "basics/drawing_temp.h"
#include "local_data.h"
#include "basics/read_avi.h"
#include "basics/video_common.h"
#include "basics/wrapper.h"
#include "basics/io_aux.h"
#include "window.h"

#include "basics/rle_encoding.h"
#include "basics/video_file.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "algorithms.h"
#include "algo_templates.h"
#include "simple_chamfer.h"
#include "chamfer_distances.h"
#include "thesis_module.h"


const long NUMBER_OF_SKIN_TABLES = 7;


//////////////////////////////////////////////////////////
//                                                      //
// start of obligatory member functions for any module  //
//                                                      //
//////////////////////////////////////////////////////////


thesis_module::thesis_module(class_interpreter * interpreter) :
  class_module(interpreter)
{
  initialize();
  sort_commands ();
}


thesis_module::~thesis_module()
{
  class_module::clean_up();
  clean_up();
}

long thesis_module::initialize()
{
  graphics_module = 0;
  skin_data_initialized = 0;
  skin_tables = vZero(vSkinTableBase *);
  current_table_index = 6;
  skin_directory = platform_skin_directory;
  non_skin_directory = platform_non_skin_directory;

  database = 0;

  enter_command("canny", CannyDetector0a);
  enter_command("canny3", CannyDetector0b);
  enter_command("chamfer", chamfer_distance0);
  enter_command("chamfer_fast", chamfer_distance_fast);
  enter_command("chamfer_features", chamfer_feature_test);
  enter_command("chamfer_feature_distance", chamfer_feature_distance);
  enter_command("chamfer_rank", chamfer_rank);
  enter_command("convert", convert);

  enter_command("database_create", database_create);
  enter_command("database_destroy", database_destroy);
  enter_command("database_edges", database_edges);
  enter_command("database_pose", database_pose);
  enter_command("database_load_distances", database_load_distances);
  enter_command("database_clear_distances", database_clear_distances);
  enter_command("database_distances", database_distances);
  enter_command("database_save_pixels", database_save_pixels);
  enter_command("database_load_pixels", database_load_pixels);
  enter_command("database_pixels", database_pixels);
  enter_command("database_rotation", database_rotation);
  enter_command("database_print", database_print);
  enter_command("database_normalize", database_normalize);
  enter_command("database_chamfer_distances", database_chamfer_distances);

  enter_command("database_chamfer", database_chamfer);
  enter_command("database_chamfer2", database_chamfer2);

  enter_command("dt", DistanceTransform);
  enter_command("distance_transform", DistanceTransform);
  enter_command("dt_slow", DistanceTransformSlow);
  enter_command("distance_transform_slow", DistanceTransformSlow);
  
  enter_command("enclosing_circle",  enclosing_circle);
  enter_command("enclosing_circle_normalize",  enclosing_circle_normalize);
  enter_command("enclosing_circle_normalize3", enclosing_circle_normalize3);

  enter_command("load_chamfer_distances", load_chamfer_distances);

  enter_command("poser_cartoon_sequence", poser_cartoon_sequence);
  enter_command("poser_color_sequence", poser_color_sequence);
  enter_command("poser_first_edges", poser_first_edges);
  enter_command("poser_normalized_edges", poser_normalized_edges);

  enter_command("save_full_distances", save_full_distances);
  enter_command("sequence_chamfer_distances", sequence_chamfer_distances);

  enter_command("skin", DetectSkin);
  enter_command("skin_hand", SkinHand);

  return 1;
}

long thesis_module::clean_up()
{
  DeleteSkinData();
  database_destroy();

  return 1;
}


long thesis_module::enter_command(char * command, type_procedure procedure)
{
  commands.push_back(function_copy_string (command));
  procedures.push_back(procedure);
  return 1;
}


long thesis_module::process_command(const char * command)
{
  long index = command_index (command);
  if (index < 0)
  {
    return 0;
  }
  type_procedure procedure = procedures [index];
  (this->*procedure)();
  return 1;
}

long thesis_module::sort_commands()
{
  vector <long> indices;
  function_string_ranks (&commands, & indices);
  
  vector <char*> temporary;
  temporary.insert(temporary.end (), commands.begin (), commands.end ());
  vector <type_procedure> temporary_procedures;
  temporary_procedures.insert(temporary_procedures.end(), procedures.begin (), procedures.end ());

  long number = commands.size ();
  long counter;
  for (counter = 0; counter < number; counter ++)
  {
    long index = indices [counter];
    commands[counter] = temporary [index];
    procedures [counter] = temporary_procedures [index];
  }

  return 1;
}

//////////////////////////////////////////////////////////
//                                                      //
//  end of obligatory member functions for any module   //
//                                                      //
//////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////
//                                                      //
//  start of functions accessing the graphical module   //
//                                                      //
//////////////////////////////////////////////////////////


long thesis_module::show_image(general_image * image)
{
  return graphics_module->show_image(image);
}


long thesis_module::default_input(long vertical, long horizontal)
{
  return graphics_module->default_input(vertical, horizontal);
}


long thesis_module::check_input()
{
  return graphics_module->check_input();
}


general_image * thesis_module::current_image ()
{
  class_sequence_information * sequence = current_sequence ();
  if (sequence == 0)
  {
    return 0;
  }

  return sequence->current_image;
}


class_sequence_information * thesis_module::current_sequence()
{
  return graphics_module->input;
}



//////////////////////////////////////////////////////////
//                                                      //
//    end of functions accessing the graphical module   //
//                                                      //
//////////////////////////////////////////////////////////


long thesis_module::DeleteSkinData()
{
  if (skin_data_initialized != 0)
  {	
    long i;
    for (i = 0; i < NUMBER_OF_SKIN_TABLES - 1; i++)
    {
      vdelete((vSkinTable<long> *&) skin_tables[i]);
    }
    vdelete((vSkinTable<double> *&) skin_tables[i]);
    vdelete2(skin_tables);

    vSetZero(skin_tables);
    skin_data_initialized = 0;
  }

  return 1;
}


long thesis_module::InitializeSkinData()
{
  if (skin_data_initialized != 0) return 0;
  vPrint("Getting skin data...\n");
 
  skin_tables = vnew(vSkinTableBase *, NUMBER_OF_SKIN_TABLES);

  char filename1[500], filename2[500];
  long i;
  long size = 2;
  for (i = 0; i < NUMBER_OF_SKIN_TABLES - 1; i++)
  {
    sprintf(filename1, "%s%stable.%li.pc", 
            skin_directory, platform_directory_slash, size);
    sprintf(filename2, "%s%stable.%li.pc", 
            non_skin_directory, platform_directory_slash, size);
    skin_tables[i] = new vSkinTable<long>(filename1, filename2);
    size = size * 2;
  }

  vSkinTable<double> * crl_skin_table;
  crl_skin_table = new vSkinTable<double>(crl_skin_file,
                                           crl_non_skin_file, 32);
  skin_tables[NUMBER_OF_SKIN_TABLES-1] = crl_skin_table;

  vPrint("Read skin data\n\n");

  skin_data_initialized = 1;
  skin_table = skin_tables[current_table_index];
  vPrint("Done\n\n");

  return 1;
}


long thesis_module::PickSkinTables()
{
  do
  {
    vPrint("Select skin tables (0 - %li): ", NUMBER_OF_SKIN_TABLES);
    question("%li", &current_table_index);
  }

  while((current_table_index >= NUMBER_OF_SKIN_TABLES) ||
        (current_table_index < 0));

  if (skin_data_initialized != 0)
  {
    skin_table = skin_tables[current_table_index];
  }

  return 1;
}


long thesis_module::DetectSkin()
{
  if (current_image() == 0) return 0;

  thesis_module::InitializeSkinData();
  general_image * the_image1 = current_image();
  color_image * the_image2 = the_image1->ToRGB();
  grayscale_image * result_image = skin_table->DetectSkin(the_image2);
  show_image(result_image);

  if (the_image1 != the_image2) vdelete(the_image2);

  return 1;
}



// Start of section using the OpenCV library
long thesis_module::CannyDetector0a()
{
  if (current_image() == 0) return 0;
  double low = 0;
  vPrint("\nInput low threshold for Canny detector:\n");
  vScan("%lf", &low);
  long success = CannyDetector(low);
  if (success == 0)
  {
    vPrint("Failed to perform Canny edge detection with given input and parameters\n");
  }
  return success;
}


long thesis_module::CannyDetector0b()
{
  if (current_image() == 0) return 0;
  double low = 0;
  double high = 0;
  int aperture = 0;
  vPrint("\nInput low, high, aperture for Canny detector:\n");
  vScan("%lf %lf %i", &low, &high, &aperture);
  long success = CannyDetector3(low, high, aperture);
  if (success == 0)
  {
    vPrint("Failed to perform Canny edge detection with given input and parameters\n");
  }
  return success;
}


long thesis_module::CannyDetector(double low)
{
  return CannyDetector3(low, 2*low, 3);
}


long thesis_module::CannyDetector3(double low, double high, int aperture)
{
  // Make sure no errors occur
  if (current_image() == 0) return 0;

  // The cvCanny function will crash the program if the following conditions
  // are violated
  if (low < 0) return 0;
  if (high < 0) return 0;
  if (low > 255) return 0;
  if (high > 255) high = 255;
  if (aperture < 1) return 0;
  if ((aperture % 2) != 1) return 0;

  grayscale_image * source = current_image()->ToGray();
  grayscale_image * edges = vCannyDetector(source, (uchar) low, (uchar) high, aperture);
  if (source != current_image())
  {
    vdelete(source);
  }
  if (edges == 0) return 0;
  show_image(edges);
  return 1;
}


long thesis_module::DistanceTransformSlow()
{
  if (current_image() == 0) return 0;
  general_image * current = current_image();
  v3dMatrix<long> * transform = vDistanceTransformSlow(current);
  show_image(transform);
  return 1;
}


long thesis_module::DistanceTransform()
{
  if (current_image() == 0) return 0;
  general_image * current = current_image();
  //v3dMatrix<long> * transform = vDistanceTransform(current);
  v3dMatrix<float> * transform = vCVDistanceTransform(current);
  show_image(transform);
  return 1;
}


// My algorithm
long thesis_module::enclosing_circle()
{
  if (current_image() == 0) return 0;
  grayscale_image * gray = current_image()->ToGrayCopy();
  color_image * color = current_image()->ToRGBCopy();
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  long number = 0;
  vGetNonZeroPoints4(gray, &rows, &cols, &number);
  vCircleLong circle = vMinEnclosingCircle3(rows, cols, number);
  show_image(color);
  graphics_module->DrawCircle(circle.center_row, circle.center_col, circle.radius, 255);
  vPrint("center = (%li, %li), radius = %li\n", 
          circle.center_row, circle.center_col, circle.radius);
  vdelete(gray);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


long thesis_module::enclosing_circle_normalize()
{
  if (current_image() == 0) 
  {
    return 0;
  }
  long radius = 0;
  print("\nEnter radius:\n");
  question("%li", &radius);

  double_image current(current_image());
  double_image * normalized = normalize_enclosing_circle(& current, radius);
  show_image(normalized);

  return 1;
}


long thesis_module::enclosing_circle_normalize3()
{
  if (current_image() == 0) 
  {
    return 0;
  }
  long radius = 0, center_row = 0, center_col = 0;
  print("\nEnter radius, center_row, center_col:\n");
  question("%li %li %li", &radius, &center_row, &center_col);

  double_image current(current_image());
  double_image * normalized = normalize_enclosing_circle4(& current, radius, 
                                                          center_row, center_col);
  show_image(normalized);

  return 1;
}


// End of section using the OpenCV library



// A way to find the hand based on skin detection
long thesis_module::SkinHand()
{
  if (current_image() == 0) return 0;

  DetectSkin();
  graphics_module->ThresholdCurrentImage(100);
  graphics_module->Blur();
  graphics_module->Blur();
  graphics_module->Blur();
  graphics_module->BiggestComponent();
  graphics_module->FillUpHoles();
  KeepPalm2();
  graphics_module->SelectPixels(10);

  return 1;
}


long thesis_module::KeepPalm2()
{
  if (current_image() == 0) return 0;
  color_image * color = current_image()->ToRGBCopy();
  grayscale_image * binary = current_image()->ToGrayCopy();
  vLineSegmentLong palm_cutoff;
  grayscale_image * palm = vKeepPalm2(binary, &palm_cutoff);
  v3dMatrix<uchar> * color_palm = vSelectPixels(color, palm);
  show_image(palm);
  show_image(color_palm);
  vdelete(color);
  vdelete(binary);
  return 1;
}


// this are one-shot procedures, that are used once to combine
// the BMP outputs of poser (in cartoon format and in color format)
// into AVI sequences, and to then generate the edge images
long thesis_module::poser_cartoon_sequence()
{
  long shapes = 20;
  long viewpoints = 84;
  char * directory = function_join_paths_three(sequences_directory, "poser", "cartoons");
  char * base_name = function_join_paths(directory, "selected_asl");
  char * output_name = function_join_paths_three(sequences_directory, "poser", 
                                                 "selected_asl_cartoons.rle");

//  long shapes = 1;
//  long viewpoints = 84;
//  char * directory = function_join_paths_three(sequences_directory, "poser", "trash");
//  char * base_name = function_join_paths(directory, "one");
//  char * output_name = function_join_paths_three(sequences_directory, "poser", "trash.avi");

  vRLE_Sequence output(output_name, 700, 700);
  if (output.valid() <= 0)
  {
    exit_error("\nerror: failed to open output file %s\n", output_name);
  }

  print("\n");
  long shape, viewpoint;
  for (shape = 0; shape < shapes; shape++)
  {
    for (viewpoint = 0; viewpoint < viewpoints; viewpoint++)
    {
      char input[1000]; 
      sprintf(input, "%s_%li_%li.bmp", base_name, shape, viewpoint);
      color_image * image = color_image::load(input);
      if (image == 0)
      {
        exit_error("error: failed to load input %s\n", input);
      }
      long success = output.write_frame(image);
      if (success <= 0)
      {
        exit_error("error: failed to save frame\n");
      }
      print("processed shape %li, viewpoint %li     \r", shape, viewpoint);
      function_delete(image);
    }
  }
  print("\n");
  output.FinishWriting();

  delete_pointer(directory);
  delete_pointer(base_name);
  delete_pointer(output_name);
  
  return 1;
}


long thesis_module::poser_color_sequence()
{
  long shapes = 20;
  long viewpoints = 84;
  char * directory = function_join_paths_three(sequences_directory, "poser", "colors");
  char * base_name = function_join_paths(directory, "selected_asl");
  char * output_name = function_join_paths_three(sequences_directory, "poser", 
                                                 "selected_asl_colors.rle");

//  long shapes = 1;
//  long viewpoints = 84;
//  char * directory = function_join_paths_three(sequences_directory, "poser", "trash");
//  char * base_name = function_join_paths(directory, "one");
//  char * output_name = function_join_paths_three(sequences_directory, "poser", "trash.avi");

  vRLE_Sequence output(output_name, 700, 700);
  if (output.valid() <= 0)
  {
    exit_error("\nerror: failed to open output file %s\n", output_name);
  }

  print("\n");
  long shape, viewpoint;
  for (shape = 0; shape < shapes; shape++)
  {
    for (viewpoint = 0; viewpoint < viewpoints; viewpoint++)
    {
      char input[1000]; 
      sprintf(input, "%s_%li_%li.bmp", base_name, shape, viewpoint);
      color_image * image = color_image::load(input);
      if (image == 0)
      {
        exit_error("error: failed to load input %s\n", input);
      }
      long success = output.write_frame(image);
      if (success <= 0)
      {
        exit_error("error: failed to save frame\n");
      }
      print("processed shape %li, viewpoint %li     \r", shape, viewpoint);
      function_delete(image);
    }
  }
  print("\n");
  output.FinishWriting();

  delete_pointer(directory);
  delete_pointer(base_name);
  delete_pointer(output_name);

  return 1;
}


long thesis_module::poser_first_edges() // here the edge images are not normalized
{
  char * cartoon_name = function_join_paths_three(sequences_directory, "poser", "selected_asl_cartoons.rle");
  char * color_name = function_join_paths_three(sequences_directory, "poser", "selected_asl_colors.rle");
  char * output_name = function_join_paths_three(sequences_directory, "poser", "selected_asl_first_edges.rle");

  class_sequence * cartoon_input = open_read_sequence(cartoon_name);
  if (cartoon_input == 0)
  {
    exit_error("\nerror: failed to open file %s\n", cartoon_name);
  }

  class_sequence * color_input = open_read_sequence(color_name);
  if (color_input == 0)
  {
    exit_error("\nerror: failed to open file %s\n", color_name);
  }

  long vertical_size = cartoon_input->vertical();
  long horizontal_size = cartoon_input->horizontal();

  vRLE_Sequence output(output_name, vertical_size, horizontal_size);
  if (output.valid() <= 0)
  {
    exit_error("\nerror: failed to open output file %s\n", output);
  }

  long frames = cartoon_input->NumberOfFrames();

  print("\n");
  long frame;
  for (frame = 0; frame < frames; frame++)
  {
    color_image * cartoon = cartoon_input->ReadFrame(frame);
    if (cartoon == 0)
    {
      exit_error("error: failed to load cartoon\n");
    }

    color_image * color = color_input->ReadFrame(frame);
    if (color == 0)
    {
      exit_error("error: failed to load color image\n");
    }

    grayscale_image * edges = poser_edge_image(cartoon, color);

    long success = output.write_frameGray2(edges, 0, 0);
    if (success <= 0)
    {
      exit_error("error: failed to save edges\n");
    }
    print("processed %li of %li frames\r", frame+1, frames);
    function_delete(cartoon);
    function_delete(color);
    function_delete(edges);
  }

  print("\n");
  output.FinishWriting();
  close_sequence(cartoon_input);
  close_sequence(color_input);

  delete_pointer(cartoon_name);
  delete_pointer(color_name);
  delete_pointer(output_name);

  return 1;
}
  

// here the edge images are normalized so that their enclosing
// circle has fixed center and radius, and they are also flipped
// so that they correspond to the right hand.
long thesis_module::poser_normalized_edges() 
{
  char * input_name = function_join_paths_three(sequences_directory, "poser", "selected_asl_first_edges.rle");
  char * output_name = function_join_paths_three(sequences_directory, "poser", "selected_asl_edges.rle");

  class_sequence * input = open_read_sequence(input_name);
  if (input == 0)
  {
    exit_error("\nerror: failed to open file %s\n", input_name);
  }

  long vertical_size = 256;
  long horizontal_size = 256;
  long radius = 120;

  vRLE_Sequence output(output_name, vertical_size, horizontal_size);
  if (output.valid() <= 0)
  {
    exit_error("\nerror: failed to open output file %s\n", output);
  }

  long frames = input->NumberOfFrames();

  print("\n");
  long frame;
  for (frame = 0; frame < frames; frame++)
  {
    grayscale_image * first_edges = input->ReadFrameGray(frame);
    if (first_edges == 0)
    {
      exit_error("error: failed to load first edges\n");
    }

    grayscale_image * edges = ::poser_normalized_edges(first_edges, radius, vertical_size, horizontal_size);

    long success = output.write_frameGray2(edges, 0, 0);
    if (success <= 0)
    {
      exit_error("error: failed to save edges\n");
    }
    print("processed %li of %li frames\r", frame+1, frames);
    function_delete(first_edges);
    function_delete(edges);
  }

  print("\n");
  output.FinishWriting();
  close_sequence(input);

  delete_pointer(input_name);
  delete_pointer(output_name);

  return 1;
}


long thesis_module::chamfer_distance0()
{
  if (current_image() == 0) return 0;

  long input1 = 0;
  long input2 = 0;
  print("\nenter input1, input2:\n");
  question("%li %li", &input1, &input2);
  return chamfer_distance(input1, input2);
}

  
long thesis_module::chamfer_distance(long input1, long input2)
{
  ushort result = 0;
  double score = 0;
  long current = graphics_module->current_input_index;

  general_image * base1 = graphics_module->CurrentImage(input1);
  general_image * base2 = graphics_module->CurrentImage(input2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * binary1 = (grayscale_image *) base1->threshold(1);
  graphics_module->SetCurrentInput(input1);
  show_image(binary1);
  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  graphics_module->SetCurrentInput(input2);
  show_image(binary2);
  graphics_module->SetCurrentInput(current);

  double first = 0, second = 0;
  score = function_chamfer_distance(binary1, binary2, & first, & second);
  print("\nfirst distance = %lf, second distance = %lf\n", first, second);
  print("Score = %lf\n", score);
  return 1;
}


long thesis_module::chamfer_distance_fast()
{
  if (current_image() == 0) return 0;

  long input1 = 0;
  long input2 = 0;
  long parameter = 0;
  print("\nenter input1, input2, parameter:\n");
  question("%li %li %li", &input1, &input2, & parameter);

  ushort result = 0;
  double score = 0;
  long current = graphics_module->current_input_index;

  general_image * base1 = graphics_module->CurrentImage(input1);
  general_image * base2 = graphics_module->CurrentImage(input2);
  if ((base1 == 0) || (base2 == 0)) return 0;

  grayscale_image * binary1 = (grayscale_image *) base1->threshold(1);
  graphics_module->SetCurrentInput(input1);
  show_image(binary1);
  grayscale_image * binary2 = (grayscale_image *) base2->threshold(1);
  graphics_module->SetCurrentInput(input2);
  show_image(binary2);
  graphics_module->SetCurrentInput(current);
  double first = 0, second = 0;
  score = fast_chamfer_distance(binary1, binary2, parameter, & first, & second);
  print("\nfirst distance = %lf, second distance = %lf\n", first, second);
  print("Score = %lf\n", score);
  return 1;
}

  
long thesis_module::database_create()
{
  database_destroy();
  
  char * name = "selected_asl";
  database = new class_hand_database(name);
  if (database->valid() <= 0)
  {
    print("\nfailed to initialize database %s\n", name);
    database_destroy();
    return 0;
  }

  print("\ncreated database %s\n", name);
  return 1;
}


long thesis_module::database_destroy()
{
  if (database == 0)
  {
    print("\nno database available to delete\n");
    return 0;
  }

  function_delete(database);
  database = 0;
  print("\ndeleted database\n");
  return 1;
}


long thesis_module::database_edges()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long frame = 0, orientation = 0;
  print("\nenter frame, orientation:\n");
  question("%li %li", & frame, & orientation);

  grayscale_image * edges = database->get_edges_two(frame, orientation);
  show_image(edges);
  return 1;
}


// like database_edges, but here the user only provides
// one number to specify the pose, as opposed to two numbers in
// database_edges
long thesis_module::database_pose()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long pose = 0;
  print("\nenter pose:\n");
  question("%li", & pose);

  return database_pose(pose);
}

  
long thesis_module::database_pose(long pose)
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  grayscale_image * edges = database->get_edges(pose);
  show_image(edges);
  return 1;
}

  
long thesis_module::database_load_distances()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long success = database->load_distances();
  if (success <= 0)
  {
    print("\nfailed to load distances\n");
  }
  else
  {
    print("\nloaded distances successfully \n");
  }

  return 1;
}


long thesis_module::database_clear_distances()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  database->clear_distances();
  return 1;
}


long thesis_module::database_distances()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long frame = 0, orientation = 0;
  print("\nenter frame, orientation:\n");
  question("%li %li", & frame, & orientation);

  grayscale_image * distances = database->distance_transform(frame, orientation);
  show_image(distances);
  return 1;
}


long thesis_module::database_save_pixels()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long success = database->create_pixel_sequence();
  if (success <= 0)
  {
    print("\nfailed to create pixel sequence\n");
  }
  else
  {
    print("\ncreated pixel sequence successfully \n");
  }
  return 1;
}


long thesis_module::database_load_pixels()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long success = database->load_pixel_sequence();
  if (success <= 0)
  {
    print("\nfailed to load pixel sequence\n");
  }
  else
  {
    print("\nloaded distances pixel sequence \n");
  }

  return 1;
}


long thesis_module::database_pixels()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long frame = 0, orientation = 0;
  print("\nenter frame, orientation:\n");
  question("%li %li", & frame, & orientation);

  grayscale_image * image = database->pixel_image(frame, orientation);
  show_image(image);
  return 1;
}


long thesis_module::database_rotation()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long rotation = 0;
  print("\nenter rotation:\n");
  question("%li", & rotation);

  short_image * rotation_template = database->rotation_template(rotation);
  show_image(rotation_template);
  return 1;
}


long thesis_module::database_print()
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  database->print();
  return 1;
}


long thesis_module::database_chamfer() // we specify poses
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long first_pose = 0, second_pose = 0;
  print("\nenter first_pose, second_rose\n");
  question("%li %li", & first_pose, & second_pose);

  grayscale_image * first = database->get_edges(first_pose);
  grayscale_image * second = database->get_edges(second_pose);
  show_image(first);
  show_image(second);
  
  double first_distance = 0, second_distance = 0;
  double score = fast_chamfer_distance(first, second, 50, & first_distance, & second_distance);
  print("\nfirst distance = %lf, second distance = %lf\n", first_distance, second_distance);
  print("Score = %lf\n", score);

  return 1;
}


long thesis_module::database_chamfer2() // we specify frames and orientations
{
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long first_frame = 0, first_orientation = 0, second_frame = 0, second_orientation = 0;
  print("\nenter first_frame, first_orientation, second_frame, second_orientation\n");
  question("%li %li %li %li", & first_frame, & first_orientation, 
           & second_frame, & second_orientation);

  grayscale_image * first = database->pixel_image(first_frame, first_orientation);
  grayscale_image * second = database->pixel_image(second_frame, second_orientation);
  show_image(first);
  show_image(second);

  double first_distance = 0, second_distance = 0;
  double score = fast_chamfer_distance(first, second, 50, & first_distance, & second_distance);
  print("\nfirst distance = %lf, second distance = %lf\n", first_distance, second_distance);
  print("Score = %lf\n", score);

  return 1;
}


// treats the current image as a query image and normalizes
// that image as required by the database.
long thesis_module::database_normalize()
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  grayscale_image * query = current_image()->grayscale_bound_copy();
  grayscale_image * normalized = database->normalize_query(query);
  show_image(normalized);
  function_delete(query);

  return 1;
}


// creates chancery features for the current query image
// and displays a specific orientation.
long thesis_module::chamfer_feature_test()
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long orientation = 0;
  print("\nenter orientation:\n");
  question("%li", & orientation);

  if ((orientation < 0) || (orientation >= database->get_orientation_number()))
  {
    print("illegal orientation index %li\n", orientation);
    return 1;
  }

  grayscale_image * query = current_image()->grayscale_bound_copy();
  class_chamfer_features features(query, database);
  
  grayscale_image * distances = (grayscale_image *) new uchar_image(features.distance_transforms[orientation]);
  show_image(distances);
  grayscale_image * pixels = features.pixel_sequence->pixel_image(orientation);
  show_image(pixels);

  function_delete(query);
  return 1;
}


// computes the chamfer distance between the specified orientation
// of the query image and the specified database frame,
// by first creating chamfer feature objects for both the query
// and the database
long thesis_module::chamfer_feature_distance()
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  long frame = 0, orientation = 0;
  print("\nenter frame, orientation:\n");
  question("%li %li", &frame, & orientation);

  if ((orientation < 0) || (orientation >= database->get_orientation_number()))
  {
    print("illegal orientation index %li\n", orientation);
    return 1;
  }

  if ((frame < 0) || (frame >= database->get_frame_number()))
  {
    print("illegal frame index %li\n", frame);
    return 1;
  }

  class_chamfer_features * database_features = database->database_chamfer_features();
  if (database_features == 0)
  {
    print("\nfailed to extract database features\n");
    return 1;
  }

  grayscale_image * query = current_image()->grayscale_bound_copy();
  class_chamfer_features query_features(query, database);
  float query_model = 0, model_query = 0;
  float distance = fast_chamfer_distance(& query_features, orientation, database_features, frame,
                                         & query_model, & model_query);

  print("\nquery-model distance = %f, model-query distance = %f\n", query_model, model_query);
  print("distance = %f\n", distance);

  function_delete(query);
  function_delete(database_features);
  return 1;
}


// normalizes the current query image and
// computes the chamfer distances between the query
// and every pose represented in the database.
long thesis_module::database_chamfer_distances()
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  grayscale_image * query = current_image()->grayscale_bound_copy();
  query_distances = chamfer_distances(query, database);
  chamfer_rank(1);
  
  function_delete(query);
  return 1;
}


// displays information about the rank-th nearest neighbor
// of the query, starting with the first neighbor.
long thesis_module::chamfer_rank()
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  if (query_distances.valid() <= 0)
  {
    print("\nno chamfer distances have been computed\n");
    return 0;
  }

  long rank = 0;
  print("\nenter rank:\n");
  question("%li", & rank);
  return chamfer_rank(rank);
}


long thesis_module::chamfer_rank(long rank)
{
  if (current_image() == 0)
  {
    return 0;
  }
  
  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  if (query_distances.valid() <= 0)
  {
    print("\nno chamfer distances have been computed\n");
    return 0;
  }

  long index = 0;
  float distance = find_rank_ascending(rank, & query_distances, & index);
  print("\ndistance = %f\n", distance);

  long orientation_number = database->get_orientation_number();
  long frame = index/orientation_number;
  long orientation = index%orientation_number;
  print("pose = %li, frame = %li, orientation = %li\n", index, frame, orientation);
  database_pose(index);

  float minimum = function_image_minimum(& query_distances);
  print("minimum = %f\n", minimum);

  return 1;
}


// save a matrix of distances between every frame of the 
// current sequence and every pose represented in the database.
long thesis_module::sequence_chamfer_distances()
{
  if ((current_sequence() == 0) || 
      (current_sequence()->sequence == 0))
  {
    print("\nno current video sequence available\n");
    return 0;
  }

  if (database == 0)
  {
    print("\nno database available\n");
    return 0;
  }

  class_sequence * sequence = current_sequence()->sequence;
  float_matrix result = ::sequence_chamfer_distances(sequence, database);
  if (result.valid() <= 0)
  {
    print("\nfailed to compute distances\n");
    return 0;
  }
  print("\ncomputed distances successfully\n");
  return 1;
}


// for specified frame and orientation of the current input
// we load from a file the distances from the frame to all database poses
// and we adjust those distances based on the orientation
long thesis_module::load_chamfer_distances()
{
  if ((current_sequence() == 0) || 
      (current_sequence()->sequence == 0))
  {
    print("\nno current video sequence available\n");
    return 0;
  }

  long frame = 0, orientation_index = 0;
  print("\nenter frame, orientation_index:\n");
  question("%li %li", & frame, & orientation_index);
  
  class_sequence * sequence = current_sequence()->sequence;
  query_distances = load_pose_chamfer_distances(sequence, frame, orientation_index);

  if (query_distances.valid() <= 0)
  {
    print("\nfailed to load distances\n");
  }
  else
  {
    print("\nloaded distances successfully\n");
  }

  return 1;
}


long thesis_module::save_full_distances()
{
  if ((current_sequence() == 0) || 
      (current_sequence()->sequence == 0))
  {
    print("\nno current video sequence available\n");
    return 0;
  }

  class_sequence * sequence = current_sequence()->sequence;
  char output_file[1000];
  print("\nenter output file:\n");
  question("%s", output_file);

  long success = full_distance_matrix(sequence, output_file);
  if (success > 0)
  {
    print("\nsaved distances successfully\n");
  }
  else
  {
    print("\nfailed to save distances\n");
  }

  return success;
}


long  thesis_module::convert()
{
  char input_name[1000], output_name[1000];
  print("\nenter input name, output name\n");
  question("%s %s", input_name, output_name);

  vAVI_Stream input(input_name);
  if (input.valid() <= 0)
  {
    print("failed to open input sequence\n");
    return 0;
  }
  long vertical = input.vertical();
  long horizontal = input.horizontal();
  class_sequence * output = open_write_sequence(output_name, vertical, horizontal);
  
  long counter = 0;
  while(1)
  {
    color_image * image = input.ReadNextFrame();
    if (image == 0)
    {
      print("failed to read frame %li\n", counter);
      break;
    }
    long success = output->write_frame(image);
    function_delete(image);
    if (success == 0)
    {
      print("failed to save frame %li\n", counter);
      break;
    }

    print("saved frame %li\n", counter);
    counter++;
  }

  output->FinishWriting();
  close_sequence(output);

  return 1;
}






















