#ifndef VASSILIS_VOLUMETRIC_H
#define VASSILIS_VOLUMETRIC_H

#include "basics/auxiliaries.h"
#include "basics/image.h"
#include "algo_templates.h"


// The class vBackgroundModel is used to store the representation of
// a background. Currently a background is represented by its
// pixelwise means and stds. However, this is kept transparent from
// other parts of the code. The constructor takes as argument a
// file, of the same format as used for
// vBackgroundInfo. Alternatively, the user can type in the name of
// the file. The function SubtractBackground returns a foreground
// image, in which every pixel receives a score, that decreases as the
// pixel becomes similar to the background.

class vBackgroundModel
{
private:
  v3dMatrix<double> * background_mean;
  v3dMatrix<double> * background_std;
  long is_valid;

public:
  vBackgroundModel(vSequenceStatistics<uchar> * stats);
  ~vBackgroundModel();
  v3dMatrix<double> * BackgroundMean();
  v3dMatrix<double> * BackgroundStd();
  v3dMatrix<double> * EstimateForeground(v3dMatrix<uchar> * the_image);
  color_image * SubtractBackground(color_image * the_image, double threshold);
  ushort IsValid();
};


class vVolume;

// An object of this class contains the calibration information that
// can be obtained from Matheen's calibration software. Its
// constructor takes as argument a file produced by the calibration software.

class vCalibrationInfo
{
private:
  double a;
  double s;
  double b;
  double u;
  double v;
  double kappa1;
  double kappa2;

  // Note that the indices of the matrices start at 1, not at 0.
  // There is a different set of matrices for each calibration
  // frame. Some calibration frames have no matrix. So, basically, at
  // each point, one of the frames will be selected, and its matrices
  // will be R, T, A, RT and P. At any point we can choose some other
  // frame. 
  double ** R;
  double ** T;
  double ** A;
  double ** RT;
  double ** P;

  // The following variables save all the matrices of all frames, as
  // well as the original frame number that corresponds to each set of
  // matrices. 



  ushort valid;

public:
  vCalibrationInfo(const char * filename);
  ~vCalibrationInfo();

  vVolume * GetVolume(v3dMatrix<uchar> * the_image);
  void Print();
  v3dMatrix<uchar> * Undistort(v3dMatrix<uchar> * distorted);
  void UndistortedCoords(double in_row, double in_col, 
			 double & out_row, double & out_col);
  void UndistortedCoords2(double in_row, double in_col, 
			 double & out_row, double & out_col);
  void DistortedCoords(double in_row, double in_col, 
		       double & out_row, double & out_col);
  ushort IsValid();
};


class vVolume
{
private:
  double x_min, x_max, y_min, y_max, z_min, z_max;

public:
  vVolume(vector<double> & bounds);
  ~vVolume();
  void Intersect(vVolume * volume2);
};


class vVolumetricInfo
{
private:
  vector<voidp> calibration_results;

public:
  vVolumetricInfo(const char * filename);
  ~vVolumetricInfo();
};



#endif // VASSILIS_VOLUMETRIC_H
