#include "stdafx.h"

#include "vplatform.h"
#include <assert.h>
#ifdef VASSILIS_SGI_PLATFORM
#include <fstream>
#else 
#include <fstream>
#endif

#include "basics/auxiliaries.h"
#include "avi_file.h"
#include "basics/definitions.h"

using namespace std;

BITMAPINFO * S_BitmapInfo(long rows, long cols, long color_bytes,
                          long * bitmap_info_size);

vAVI_File_Init vAVI_File::avi_file_init = vAVI_File_Init();


vAVI_File_Init::vAVI_File_Init()
{
  AVIFileInit();
}


vAVI_File_Init::~vAVI_File_Init()
{
  AVIFileExit();
}


vAVI_File::vAVI_File()
{
  file_open = 0;
  write_flag = 0;
  filename = 0;
}


vAVI_File::~vAVI_File()
{
  if (file_open != 0)
    AVIFileRelease(file);
  vdelete2(filename);
}


vAVI_File * vAVI_File::OpenWrite(const char * filename)
{
  remove(filename);
  vAVI_File * result = new vAVI_File();

  int temp_result;
  temp_result = AVIFileOpen(&(result->file), filename,
                            OF_CREATE, 0);
  long error = 0;
  if (temp_result != 0) 
  {
    // At some point I got error -2147221008
    
    //2147205019
	  switch(temp_result)
	  {
	  case AVIERR_UNSUPPORTED: 
      error = 1;
      break;
	  case AVIERR_MEMORY:     
      error = 2;
		  break;
	  case AVIERR_INTERNAL:       
      error = 3;
		  break;
	  case AVIERR_BADFLAGS:     
      error = 4;
		  break;
	  case AVIERR_BADPARAM:      
      error = 5;
		  break;
	  case AVIERR_BADSIZE:     
      error = 6;
		  break;
	  case AVIERR_BADHANDLE:        
      error = 7;
		  break;
	  case AVIERR_FILEREAD:
      error = 8;
		  break;
	  case AVIERR_FILEWRITE:    
      error = 9;
		  break;
	  case AVIERR_FILEOPEN:
      error = 10;
		  break;
	  case AVIERR_COMPRESSOR:     
      error = 11;
		  break;
	  case AVIERR_NOCOMPRESSOR:   
      error = 12;
		  break;
	  case AVIERR_READONLY:
      error = 13;
		  break;
	  case AVIERR_NODATA:
      error = 14;
		  break;
	  case AVIERR_BUFFERTOOSMALL:
      error = 15;
		  break;
	  case AVIERR_CANTCOMPRESS:
      error = 16;
		  break;
	  case AVIERR_USERABORT:
      error = 17;
		  break;
	  case AVIERR_ERROR:
      error = 18;
		  break;
	  case AVIERR_BADFORMAT:
      error = 19;
		  break;

    case REGDB_E_CLASSNOTREG:
      error = 20;
      break;
	  }
    delete result;
    return 0;
  }
 
  result->write_flag = 1;
  result->file_open = 1;
  result->filename = vCopyString(filename);

  return result;
}



vAVI_File * vAVI_File::OpenRead(const char * filename)
{
  vAVI_File * result = new vAVI_File();

  int temp_result;
  temp_result = AVIFileOpen(&(result->file), filename, 
                            OF_READ, 0);
  if (temp_result != 0) 
  {
    delete result;
    return 0;
  }
 
  result->write_flag = 0;
  result->file_open = 1;
  result->filename = vCopyString(filename);

  return result;
}

void vAVI_File::CloseFile()
{
  AVIFileRelease(file);
  delete [] filename;
  filename = 0;
  file_open = 0;
}


unsigned short vAVI_File::WriteFlag()
{
  return write_flag;
}

char * vAVI_File::GetFilename()
{
  return filename;
}


PAVIFILE * vAVI_File::GetHandle()
{
  return &file;
}


void vAVI_Stream::Initialize(vAVI_File * avi_file)
{
  filename = vCopyString(avi_file->GetFilename());
  is_valid = 0;
  exclusive_file_flag = 0;
  file = avi_file;
  buffer = 0;
  frame_data = 0;
 
  long temp_result;
  temp_result = AVIFileGetStream(*(file->GetHandle()), 
                                 &stream,
                                 streamtypeVIDEO, 0);
  if (temp_result != 0) return;
  
  AVISTREAMINFO stream_info;
  temp_result = AVIStreamInfo(stream, &stream_info,
                              sizeof(stream_info));
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  rate = stream_info.dwRate;
  scale = stream_info.dwScale;

  temp_result = AVIStreamReadFormat(stream, 0, 0, 
                                    &bitmap_info_size);
  
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  buffer = new unsigned char[bitmap_info_size];
  bitmap_info = (BITMAPINFO *) buffer;
  bitmap_header = (BITMAPINFOHEADER *) buffer;
  bitmap_header_size = sizeof(BITMAPINFOHEADER);
  palette_size = bitmap_info_size - bitmap_header_size;
  palette = (RGBQUAD *) &(buffer[bitmap_header_size]);


  long temp = bitmap_info_size;
  temp_result = AVIStreamReadFormat(stream, 0, bitmap_info, 
                                    &bitmap_info_size);
  

  if ((temp_result != 0) || (temp != bitmap_info_size))
  {
    AVIStreamRelease(stream);
    return;
  }
  
  cols = bitmap_header->biWidth;
  rows = bitmap_header->biHeight;
  color_bits = bitmap_header->biBitCount;
  bytes_per_pixel = color_bits / 8;
  bytes_per_frame = rows * cols * bytes_per_pixel;
  frame_data = new unsigned char[bytes_per_frame];

  current_frame = -1;
  is_writable = 0;
  is_valid = 1;
  SetBounds(0, 0, rows, cols);
}


vAVI_Stream::vAVI_Stream(vAVI_File * avi_file)
{
  type = vCopyString("vAVI_Stream");
  Initialize(avi_file);
}



vAVI_Stream::vAVI_Stream(const char * in_filename)
{
  filename = 0;
  is_valid = 0;
  exclusive_file_flag = 0;
  buffer = 0;
  frame_data = 0;
  type = vCopyString("vAVI_Stream");
 
  file = vAVI_File::OpenRead(in_filename);
  if (file == 0) return;

  Initialize(file);
  exclusive_file_flag = 1;
}


void vAVI_Stream::Initialize(vAVI_File * avi_file, 
                              long rows_p, long cols_p, 
                              long rate_p, long scale_p,
                              long color_bytes_p)
{
  filename = vCopyString(file->GetFilename());
  is_valid = 0;
  exclusive_file_flag = 0;
  file = avi_file;
  buffer = 0;
  frame_data = 0;

  AVISTREAMINFO stream_info;
  int temp_result;
  rows = rows_p;
  cols = cols_p;
  rate = rate_p;
  scale = scale_p;
  bytes_per_pixel = color_bytes_p;
  if ((bytes_per_pixel != 1) && (bytes_per_pixel != 3)) return;

  memset(&stream_info, 0, sizeof(stream_info));
  stream_info.fccType = streamtypeVIDEO;
  stream_info.fccHandler = 0;
  stream_info.dwScale = scale;
  stream_info.dwRate = rate;
  SetRect(&(stream_info.rcFrame), 0, 0, cols, rows);

  temp_result = AVIFileCreateStream(*(file->GetHandle()), &stream,
                                    &stream_info);
  if (temp_result != 0) 
  {
     return;
  }

  bitmap_info = S_BitmapInfo(rows, cols, bytes_per_pixel,
                             &bitmap_info_size);
  buffer = (unsigned char *) bitmap_info;
  bitmap_header = (BITMAPINFOHEADER *) buffer;
  bitmap_header_size = sizeof(BITMAPINFOHEADER);
  palette_size = bitmap_info_size - bitmap_header_size;
  palette = (RGBQUAD *) &(buffer[bitmap_header_size]);

  temp_result = AVIStreamSetFormat(stream, 0, bitmap_info,
                                   bitmap_info_size);
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  bytes_per_frame = rows * cols * bytes_per_pixel;
  frame_data = new unsigned char[bytes_per_frame];
  current_frame = -1;
  is_writable = 1;
  is_valid = 1;
  SetBounds(0, 0, rows, cols);
}



vAVI_Stream::vAVI_Stream(vAVI_File * avi_file, 
                           long rows_p, long cols_p, 
                           long rate_p, long scale_p,
                           long color_bytes_p)
{
  type = vCopyString("vAVI_Stream");
  Initialize(avi_file, rows_p, cols_p, rate_p, scale_p, color_bytes_p);
}


vAVI_Stream::vAVI_Stream(const char * in_filename, long rows_p, long cols_p, 
                           long color_bytes_p)
{
  is_valid = 0;
  filename = 0;
  exclusive_file_flag = 0;
  buffer = 0;
  frame_data = 0;
  type = vCopyString("vAVI_Stream");
 
  remove(filename);
  file = vAVI_File::OpenWrite(in_filename);
  if (file == 0) return;

  Initialize(file, rows_p, cols_p, 30, 1, color_bytes_p);
  exclusive_file_flag = 1;
}


vAVI_Stream::vAVI_Stream(const char * in_filename, 
                           long rows_p, long cols_p,
                           long rate_p, long scale_p,
                           long color_bytes_p)
{
  is_valid = 0;
  filename = 0;
  exclusive_file_flag = 0;
  buffer = 0;
  frame_data = 0;
  type = vCopyString("vAVI_Stream");

  remove(in_filename);
  file = vAVI_File::OpenWrite(in_filename);
  if (file == 0) return;

  Initialize(file, rows_p, cols_p, rate_p, scale_p, color_bytes_p);
  exclusive_file_flag = 1;
}


BITMAPINFO * S_BitmapInfo(long rows, long cols, long color_bytes,
                          long * bitmap_info_size)
{
  long bitmap_header_size = sizeof(BITMAPINFOHEADER);
  if (color_bytes == 3) *bitmap_info_size = bitmap_header_size;
  else
  {
    assert(color_bytes == 1);
    *bitmap_info_size = bitmap_header_size + sizeof(RGBQUAD) * 256;
  }

  unsigned char * buffer = new unsigned char[*bitmap_info_size];
 
  BITMAPINFO * result = (BITMAPINFO *) buffer;
  BITMAPINFOHEADER * info_header = (BITMAPINFOHEADER *) result;

  memset(info_header, 0, bitmap_header_size);
  info_header->biSize = bitmap_header_size;
  info_header->biWidth = cols;
  info_header->biHeight = rows;
  info_header->biPlanes = 1;
  info_header->biBitCount = (unsigned short) color_bytes * 8;
  info_header->biCompression = BI_RGB;

  if (color_bytes == 1)
  {
    long i;
    long position = bitmap_header_size;
    for (i = 0; i < 256; i++)
    {
      buffer[position++] = (unsigned char) i;
      buffer[position++] = (unsigned char) i;
      buffer[position++] = (unsigned char) i;
      buffer[position++] = 0;
    }
  }
  return result;
}


vAVI_Stream::~vAVI_Stream()
{
  vdelete2(type);
  if (exclusive_file_flag != 0)
    delete file;
  if (is_valid != 0)
    AVIStreamRelease(stream);
  delete [] frame_data;
  delete [] buffer;
}


unsigned short vAVI_Stream::ReadFrame()
{
  if (is_writable != 0) return 0;
  current_frame++;
  long temp_result;
  temp_result = AVIStreamRead(stream, current_frame, 1, 
                              frame_data, bytes_per_frame,
                              0, 0);
  return (temp_result == 0);  
}


unsigned short vAVI_Stream::write_frame()
{
  return write_frame(frame_data);
}


unsigned short vAVI_Stream::write_frame(unsigned char * data)
{
  if (is_writable == 0) return 0;
  long temp_result;
  current_frame++;
  temp_result = AVIStreamWrite(stream, current_frame, 1, 
                               data,
                               bytes_per_frame,
                               AVIIF_KEYFRAME, 0, 0);
  return (temp_result == 0);
}

unsigned char * vAVI_Stream::GetFrame()
{
  return frame_data;
}


char * vAVI_Stream::GetFilename()
{
  return file->GetFilename();
}


long vAVI_Stream::CurrentFrameID()
{
  return current_frame;
}


long vAVI_Stream::GetRows()
{
  return rows;
}


long vAVI_Stream::GetCols()
{
  return cols;
}


long vAVI_Stream::GetWidth()
{
  return cols;
}


long vAVI_Stream::GetHeight()
{
  return rows;
}


long vAVI_Stream::GetFrameBytes()
{
  return bytes_per_frame;
}


long vAVI_Stream::GetRate()
{
  return rate;
}


long vAVI_Stream::GetScale()
{
  return scale;
}


long vAVI_Stream::GetColorBytes()
{
  return bytes_per_pixel;
}


RGBQUAD * vAVI_Stream::GetPalette()
{
  return palette;
}


unsigned short vAVI_Stream::IsValid()
{
  return is_valid;
}


color_image * vAVI_Stream::To_color_image(long left, long right, 
                                  long top, long bottom)
{
  assert(left >= 0);
  assert(right < cols);
  assert(top >= 0);
  assert(bottom < rows);
  long image_rows = bottom - top + 1;
  long image_cols = right - left + 1;
  color_image * image = new color_image(image_rows, image_cols);
  long row, col;
  long index = 0;
  vArray2(uchar) R2 = image->GetR2();
  vArray2(uchar) G2 = image->GetG2();
  vArray2(uchar) B2 = image->GetB2();

  if (bytes_per_pixel == 3)
  {
    for (row = rows - 1; row >= 0; row--)
    {
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
          B2[row-top][col-left] = frame_data[index++];
      	  G2[row-top][col-left] = frame_data[index++];
      	  R2[row-top][col-left] = frame_data[index++];
        }
        else index += 3;
      }
    }
  }
  else
  {
    assert(bytes_per_pixel == 1);
    for (row = rows - 1; row >= 0; row--)
    {
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
          uchar value = frame_data[index++];
      	  B2[row-top][col-left] = value;
      	  G2[row-top][col-left] = value;
      	  R2[row-top][col-left] = value;
        }
        else index++;
      }
    }
  }
  return image;
}


v3dMatrix<uchar> * vAVI_Stream::ToGrayImage(long left, long right, 
																							 long top, long bottom)
{
  assert(left >= 0);
  assert(right < cols);
  assert(top >= 0);
  assert(bottom < rows);
  long image_rows = bottom - top + 1;
  long image_cols = right - left + 1;
  v3dMatrix<uchar> * image = new v3dMatrix<uchar>(image_rows, image_cols, 1);
  long row, col;
  long index = 0;
  vArray2(uchar) I2 = image->Matrix2(0);

  if (bytes_per_pixel == 3)
  {
    for (row = rows - 1; row >= 0; row--)
    {
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
					long sum = frame_data[index++] + frame_data[index++] + frame_data[index++];
					long value = sum / 3;
					I2[row-top][col-left] = (uchar) value;
        }
        else index += 3;
      }
    }
  }
  else
  {
    assert(bytes_per_frame== 1);
    for (row = rows - 1; row >= 0; row--)
    {
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
          uchar value = frame_data[index++];
      	  I2[row-top][col-left] = value;
        }
        else index++;
      }
    }
  }
  return image;
}


color_image * vAVI_Stream::To_color_image()
{
  return To_color_image(0, cols - 1, 0, rows - 1);
}


v3dMatrix<uchar> * vAVI_Stream::ToGrayImage()
{
  return ToGrayImage(0, cols - 1, 0, rows - 1);
}


void vAVI_Stream::From_color_image(color_image * image,
                                long left, long right, 
                                long top, long bottom)
{
  long row, col;
  long index = 0;
	vArray2(uchar) R2 = image->GetR2();
	vArray2(uchar) G2 = image->GetG2();
	vArray2(uchar) B2 = image->GetB2();

  if (bytes_per_pixel == 3)
  {
    for (row = rows - 1; row >= 0; row--)
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
          frame_data[index++] = B2[row-top][col-left];
	        frame_data[index++] = G2[row-top][col-left] ;
	        frame_data[index++] = R2[row-top][col-left];
        }
        else index += 3;
      }
  }
  else
  {
    assert(bytes_per_pixel == 1);
    for (row = rows - 1; row >= 0; row--)
		{
			long new_row = row - top;
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
				{
					long new_col = col - left;
					ulong value = (R2[new_row][new_col] + G2[new_row][new_col] + B2[new_row][new_col]);
					value = value / 3;
  	      frame_data[index++] = (uchar) value;
				}
        else index++;
      }
		}
  }
}


void vAVI_Stream::From_color_image(color_image * image)
{
  assert(image->Rows() == rows);
  assert(image->Cols() == cols);

  From_color_image(image, 0, cols - 1, 0, rows - 1);
}


void vAVI_Stream::FromGrayImage(v3dMatrix<uchar> * image,
                                 long left, long right, 
                                 long top, long bottom)
{
  long row, col;
  long index = 0;
	vArray2(uchar) I2 = image->Matrix2(0);

  if (bytes_per_pixel == 3)
  {
    for (row = rows - 1; row >= 0; row--)
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
        {
					uchar value = I2[row-top][col-left];
          frame_data[index++] = value;
	        frame_data[index++] = value;
	        frame_data[index++] = value;
        }
        else index += 3;
      }
  }
  else
  {
    assert(bytes_per_pixel == 1);
    for (row = rows - 1; row >= 0; row--)
      for (col = 0; col < cols; col++)
      {
        if ((row >= top) && (row <= bottom) &&
            (col >= left) && (col <= right))
  	      frame_data[index++] = I2[row-top][col-left];
        else index++;
      }
  }
}


void vAVI_Stream::FromGrayImage(v3dMatrix<uchar> * image)
{
  assert(image->Rows() == rows);
  assert(image->Cols() == cols);

  FromGrayImage(image, 0, cols - 1, 0, rows - 1);
}


void vAVI_Stream::Print(char * filename)
{
  ofstream out(filename);
  assert(out);
  out << rows << " " << cols << endl << endl;
  long row, col;
  long index = bytes_per_frame - 1;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      out << (int) frame_data[index--] << " ";
      if (bytes_per_pixel == 3)
      {
        out << (int) frame_data[index--] << " ";
        out << (int) frame_data[index--] << " ";
      }
    }
    out << endl;
  }
}


// Beginning of functions required by vVideoSequence
grayscale_image * vAVI_Stream::ReadNextFrameGray()
{
  ushort success = ReadFrame();
  if (success == 0) return 0;
   
  grayscale_image * result;
  long right = left + width - 1;
  long bottom = top + height - 1;
  result = (grayscale_image *) ToGrayImage(left, right, top, bottom);
  return result;
}


grayscale_image * vAVI_Stream::ReadFrameGray(long number)
{
  printf("Random access of frames not implemented in vAVI_Stream.\n");
  return 0;
}


color_image * vAVI_Stream::ReadNextFrame()
{
  ushort success = ReadFrame();
  if (success == 0) return 0;
   
  color_image * result;
  long right = left + width - 1;
  long bottom = top + height - 1;
  result = To_color_image(left, right, top, bottom);
  return result;
}


color_image * vAVI_Stream::ReadFrame(long number)
{
  printf("Random access of frames not implemented in vAVI_Stream.\n");
  return 0;
}


ushort vAVI_Stream::InitializeWriting()
{
  return 1;
}


ushort vAVI_Stream::write_frame(color_image * image)
{
  long right = left + width - 1;
  long bottom = top + height - 1;
  From_color_image(image, left, right, top, bottom);
  ushort success = write_frame();
  return success;
}


ushort vAVI_Stream::write_frameGray(grayscale_image * gray)
{
  long right = left + width - 1;
  long bottom = top + height - 1;
  FromGrayImage(gray, left, right, top, bottom);
  ushort success = write_frame();
  return success;
}


ushort vAVI_Stream::FinishWriting()
{
  return 1;
}

// End of functions required by vVideoSequence


void vAVI_Text_Stream::Initialize(vAVI_File * avi_file)
{
  valid_stream = 0;
  exclusive_file_flag = 0;
  file = avi_file;
  frame_data = 0;
 
  long temp_result;
  temp_result = AVIFileGetStream(*(file->GetHandle()), 
                                 &stream,
                                 streamtypeTEXT, 0);
  if (temp_result != 0) return;
  
  AVISTREAMINFO stream_info;
  temp_result = AVIStreamInfo(stream, &stream_info,
                              sizeof(stream_info));
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  left = stream_info.rcFrame.left;
  right = stream_info.rcFrame.right;
  top = stream_info.rcFrame.top;
  bottom = stream_info.rcFrame.bottom;
  scale = stream_info.dwScale;
  rate = stream_info.dwRate;
  frame_size = stream_info.dwSampleSize;

  long format_size;
  temp_result = AVIStreamReadFormat(stream, 0, 0, 
                                    &format_size);
  
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  char * buffer = new char[format_size];
  long temp = format_size;
  temp_result = AVIStreamReadFormat(stream, 0, buffer, 
                                    &format_size);
  delete [] buffer;

  if ((temp_result != 0) || (temp != format_size))
  {
    AVIStreamRelease(stream);
    return;
  }
  
  
  frame_data = new unsigned char[frame_size];

  current_frame = -1;
  write_flag = 0;
  valid_stream = 1;
}

  
vAVI_Text_Stream::vAVI_Text_Stream(vAVI_File * avi_file)
{
  Initialize(avi_file);
}



vAVI_Text_Stream::vAVI_Text_Stream(char * filename)
{
  valid_stream = 0;
  exclusive_file_flag = 0;
  frame_data = 0;
 
  file = vAVI_File::OpenRead(filename);
  if (file == 0) return;

  Initialize(file);
  exclusive_file_flag = 1;
}


void vAVI_Text_Stream::Initialize(vAVI_File * avi_file, 
                                   long rate_p, long scale_p,
                                   long size,
                                   long left_p, long right_p, 
                                   long top_p, long bottom_p)
{
  valid_stream = 0;
  exclusive_file_flag = 0;
  file = avi_file;
  frame_data = 0;

  long text_stream_format = sizeof(long);
  AVISTREAMINFO stream_info;
  int temp_result;
  left = left_p;
  right = right_p;
  top = top_p;
  bottom = bottom_p;
  scale = scale_p;
  rate = rate_p;
  frame_size = size;
 
  memset(&stream_info, 0, sizeof(stream_info));
  stream_info.fccType = streamtypeTEXT;
  stream_info.fccHandler = 0;
  stream_info.dwScale = scale;
  stream_info.dwRate = rate;
  SetRect(&(stream_info.rcFrame), left, top, right, bottom);
  stream_info.dwSampleSize = size;

  temp_result = AVIFileCreateStream(*(file->GetHandle()), &stream,
                                    &stream_info);
  if (temp_result != 0) 
  {
     return;
  }

  temp_result = AVIStreamSetFormat(stream, 0, &text_stream_format,
                                   text_stream_format);
  if (temp_result != 0)
  {
    AVIStreamRelease(stream);
    return;
  }

  frame_data = new unsigned char[frame_size];
  current_frame = -1;
  write_flag = 1;
  valid_stream = 1;
}



vAVI_Text_Stream::vAVI_Text_Stream(vAVI_File * avi_file, 
                                     long rate_p, long scale_p,
                                     long size,
                                     long left_p, long right_p, 
                                     long top_p, long bottom_p)
{
  Initialize(avi_file, rate_p, scale_p, size,
             left_p, right_p, top_p, bottom_p);
}

vAVI_Text_Stream::vAVI_Text_Stream(char * filename, 
                                     long rate_p, long scale_p,
                                     long size,
                                     long left_p, long right_p, 
                                     long top_p, long bottom_p)
{
  valid_stream = 0;
  exclusive_file_flag = 0;
  frame_data = 0;
 
  file = vAVI_File::OpenWrite(filename);
  if (file == 0) return;

  Initialize(file, rate_p, scale_p, size,
             left_p, right_p, top_p, bottom_p);
  exclusive_file_flag = 1;
}


vAVI_Text_Stream::~vAVI_Text_Stream()
{
  if (exclusive_file_flag != 0)
    delete file;
  if (valid_stream != 0)
    AVIStreamRelease(stream);
  delete [] frame_data;
}


unsigned short vAVI_Text_Stream::ReadFrame()
{
  if (write_flag != 0) return 0;
  current_frame++;
  long temp_result;
  temp_result = AVIStreamRead(stream, current_frame, 1, 
                              frame_data, frame_size,
                              0, 0);
  return (temp_result == 0);  
}


unsigned short vAVI_Text_Stream::write_frame()
{
  if (write_flag == 0) return 0;
  long temp_result;
  current_frame++;
  temp_result = AVIStreamWrite(stream, current_frame, 1, 
                               frame_data,
                               frame_size,
                               AVIIF_KEYFRAME, 0, 0);
  return (temp_result == 0);
}

unsigned char * vAVI_Text_Stream::GetFrame()
{
  return frame_data;
}


char * vAVI_Text_Stream::GetFilename()
{
  return file->GetFilename();
}


long vAVI_Text_Stream::CurrentFrameID()
{
  return current_frame;
}


unsigned short vAVI_Text_Stream::IsValid()
{
  return valid_stream;
}


vMFC_AVI_Set::vMFC_AVI_Set(const char * filename_p)
{
  Initialize();
  filename = vCopyString(filename_p);
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return;
  base_filename = BaseFilename(filename);
  long items = vReadPCLongs(fp, &number_of_frames, 1);
  if (items != 1) return;
  items = vReadPCLongs(fp, &rows, 1);
  if (items != 1) return;
  items = vReadPCLongs(fp, &cols, 1);
  if (items != 1) return;
  items = vReadPCLongs(fp, &bytes_per_pixel, 1);
  if (items != 1) return;
  items = vReadPCLongs(fp, &rate, 1);
  if (items != 1) return;
  items = vReadPCLongs(fp, &scale, 1);
  if (items != 1) return;

  items = vReadPCLongs(fp, &frames_per_file, 1);
  if (items != 1) return;
  fclose(fp);
  is_readable = 1;
  SetBounds(0, 0, rows, cols);
  long success = OpenNumber(0);
  if (success >= 1) is_valid = 1;
}


vMFC_AVI_Set::vMFC_AVI_Set(const char * filename_p, long rows_p, long cols_p, 
                             long color_bytes_p, long frames_per_file_p)
{
  Initialize();
  filename = vCopyString(filename_p);
  rows = rows_p;
  cols = cols_p;
  bytes_per_pixel = color_bytes_p;
  long max_frames = MaxFramesPerFile(rows, cols, bytes_per_pixel);
  if (max_frames < 0) return;
  frames_per_file = frames_per_file_p;
  if (frames_per_file > max_frames) 
  {
    frames_per_file = max_frames;
  }
  rate = 30;
  scale = 1;
  if (vWritableFileExists(filename) == 0) return;
  MakeDirectory();
  base_filename = BaseFilename(filename);
  number_of_frames = 0;
  is_writable = 1;
  long success = OpenNumber(0);
  if (success >= 1) is_valid = 1;
  else is_valid = 0;
}


vMFC_AVI_Set::vMFC_AVI_Set(const char * filename_p, long rows_p, long cols_p,
                             long rate_p, long scale_p, long color_bytes_p)
{
  Initialize();
  filename = vCopyString(filename_p);
  rows = rows_p;
  cols = cols_p;
  bytes_per_pixel = color_bytes_p;
  long max_frames = MaxFramesPerFile(rows, cols, bytes_per_pixel);
  if (max_frames < 0) return;
  frames_per_file = max_frames;
  rate = rate_p;
  scale = scale_p;
  if (vWritableFileExists(filename) == 0) return;
  MakeDirectory();
  base_filename = BaseFilename(filename);
  number_of_frames = 0;
  is_writable = 1;
  long success = OpenNumber(0);
  if (success >= 1) is_valid = 1;
  else is_valid = 0;
}


vMFC_AVI_Set::vMFC_AVI_Set(const char * filename_p, long rows_p, long cols_p,
                             long rate_p, long scale_p, long color_bytes_p,
                             long frames_per_file_p)
{
  Initialize();
  filename = vCopyString(filename_p);
  rows = rows_p;
  cols = cols_p;
  bytes_per_pixel = color_bytes_p;
  long max_frames = MaxFramesPerFile(rows, cols, bytes_per_pixel);
  if (max_frames < 0) return;
  frames_per_file = frames_per_file_p;
  if (frames_per_file > max_frames) 
  {
    frames_per_file = max_frames;
  }
  rate = rate_p;
  scale = scale_p;
  if (vWritableFileExists(filename) == 0) return;
  MakeDirectory();
  base_filename = BaseFilename(filename);
  number_of_frames = 0;
  is_writable = 1;
  long success = OpenNumber(0);
  if (success >= 1) is_valid = 1;
  else is_valid = 0;
}


vMFC_AVI_Set::~vMFC_AVI_Set()
{
  CleanUp();
}


long vMFC_AVI_Set::Initialize()
{
  filename = 0;
  type = vCopyString("vMFC_AVI_Set"); // "avi" for AVIs, "rle" for RLEs, etc.
  fp = 0;
  rows = -1;
  cols = -1;
  number_of_frames = -1;
  // frame rate = rate/scale frames per second.
  rate = 30;
  scale = 1;

  current_frame = -1;
  current_position = 0;
  
  top = left = height = width = -1;
  is_readable = is_writable = is_valid = 0;
  frames_per_file = 0;
  current_avi = 0;
  current_read_avi = 0;
  current_number = -1;
  base_filename = 0;
  return 1;
}


long vMFC_AVI_Set::CleanUp()
{
  vdelete2(type);
  vdelete2(base_filename);
  vdelete(current_avi);
  vdelete(current_read_avi);
  return 1;
}


long vMFC_AVI_Set::SetBounds(long in_top, long in_left, long in_height, long in_width)
{
  vVideoFile::SetBounds(in_top, in_left, in_height, in_width);
  if (current_avi != 0) 
  {
    current_avi->SetBounds(top, left, height, width);
  }
  if (current_read_avi != 0)
  {
    current_read_avi->SetBounds(top, left, height, width);
  }
  return 1;
}


char * vMFC_AVI_Set::Directory()
{
  char * title = vRemoveEnding(filename);
  char * parent = vParentDir(title, platform_directory_slash);
  char * simple = vSimpleName(title, platform_directory_slash);
  char * directory = vJoinPaths(parent, "avi_sets");

  vdelete2(title);
  vdelete2(parent);
  vdelete2(simple);
  return directory;
}


long vMFC_AVI_Set::MakeDirectory()
{
  char * directory = Directory();  
  char * buffer = new char[strlen(directory) + 20];
  sprintf(buffer, "mkdir %s", directory);
  system(buffer);
  vdelete2(directory);
  vdelete2(buffer);
  return 1;
}


// Returns the filename to which we must add _n.avi to get the actual
// n-th file in the set.
char * vMFC_AVI_Set::BaseFilename(const char * filename)
{
  char * title = vRemoveEnding(filename);
  char * simple = vSimpleName(title, platform_directory_slash);
  char * directory = Directory();

  char * result = vJoinPaths(directory, simple);
  vdelete2(title);
  vdelete2(simple);
  vdelete2(directory);
  return result;
}


// Returns the filename of the n-th AVI file in the set of
// AVI files that makes up this sequence.
char * vMFC_AVI_Set::NumberedFile(long n)
{
  char * n_string = vLongToString(n);
  char * result = vMergeStrings4(base_filename, "_", n_string, ".avi");
  vdelete2(n_string);
  return result;
}


// Opens the n-th AVI file, returns a result > 0 if successful.
long vMFC_AVI_Set::OpenNumber(long n)
{
  char * filename = NumberedFile(n);
  if (is_writable != 0)
  {
    // Open the file for writing
    vAVI_Stream * next_avi = new vAVI_Stream(filename, rows, cols, 
                                               rate, scale, bytes_per_pixel);
    vdelete2(filename);
    if (next_avi->IsValid() == 0)
    {
      vdelete(next_avi);
      return 0;
    }
    else
    {
      if (current_avi != 0)
      {
        long success = current_avi->FinishWriting();
        if (success <= 0) return 0;
      }
      vdelete(current_avi);
      current_avi = next_avi;
      current_avi->SetBounds(top, left, height, width);
      current_number = n;
      return 1;
    }
  }

  else if (is_readable != 0)
  {
    // Open the file for reading, and verify that its parameters
    // are what we expect them to be.
    vReadAVI * next_avi = new vReadAVI(filename);
    vdelete2(filename);
    long error = 0;
    if (next_avi->IsValid() == 0) error = 1;
    else if (rows != next_avi->Rows()) error = 2;
    else if (cols != next_avi->Cols()) error = 3;
    else if (bytes_per_pixel != next_avi->BytesPerPixel()) error = 4;
    else if (rate != next_avi->Rate()) error = 5;
    else if (scale != next_avi->Scale()) error = 6;
    if (error != 0)
    {
      vdelete(next_avi);
      vPrint("error = %li\n", error);
      return 0;
    }
    else
    {
      vdelete(current_read_avi);
      current_read_avi = next_avi;
      current_read_avi->SetBounds(top, left, height, width);
      current_number = n;
      return 1;
    }
  }
  else
  {
    vdelete2(filename);
    return 0;
  }
}


// ReadFrameGray reads in the specified frame and converts it
// (if necessary) to a grayscale_image.
grayscale_image * vMFC_AVI_Set::ReadNextFrameGray()
{
  long number = current_frame + 1;
  long file_number = FrameFile(number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  grayscale_image * result = current_read_avi->ReadNextFrameGray();
  if (result != 0) current_frame++;
  return result;
}


grayscale_image * vMFC_AVI_Set::ReadFrameGray(long number)
{
  long file_number = FrameFile(number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  long file_frame_number = FrameFileIndex(number);
  grayscale_image * result = current_read_avi->ReadFrameGray(file_frame_number);
  if (result != 0) current_frame = number;
  return result;
}


color_image * vMFC_AVI_Set::ReadNextFrame()
{
  long number = current_frame + 1;
  long file_number = FrameFile(number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  color_image * result = current_read_avi->ReadNextFrame();
  if (result != 0) current_frame++;
  return result;
}


color_image * vMFC_AVI_Set::ReadFrame(long number)
{
  long file_number = FrameFile(number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  long file_frame_number = FrameFileIndex(number);
  color_image * result = current_read_avi->ReadFrame(file_frame_number);
  if (result != 0) current_frame = number;
  return result;
}


ushort vMFC_AVI_Set::InitializeWriting()
{
  return 1;
}


ushort vMFC_AVI_Set::write_frame(color_image * image)
{
  long frame_number = number_of_frames;
  long file_number = FrameFile(frame_number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  success = current_avi->write_frame(image);
  if (success > 0)
  {
    number_of_frames++;
  }
  return (ushort) success;
}


ushort vMFC_AVI_Set::write_frameGray(grayscale_image * gray)
{
  long frame_number = number_of_frames;
  long file_number = FrameFile(frame_number);
  long success;
  if (file_number != current_number)
  {
    success = OpenNumber(file_number);
    if (success <= 0) return 0;
  }
  success = current_avi->write_frameGray(gray);
  if (success > 0)
  {
    number_of_frames++;
  }
  return (ushort) success;
}


ushort vMFC_AVI_Set::FinishWriting()
{
  long success;
  if (current_avi == 0) return 0;
  success = current_avi->FinishWriting();
  if (success <= 0) return 0;
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) return 0;

  long items = vWritePCLongs(fp, &number_of_frames, 1);
  if (items != 1) return 0;
  items = vWritePCLongs(fp, &rows, 1);
  if (items != 1) return 0;
  items = vWritePCLongs(fp, &cols, 1);
  if (items != 1) return 0;
  items = vWritePCLongs(fp, &bytes_per_pixel, 1);
  if (items != 1) return 0;
  items = vWritePCLongs(fp, &rate, 1);
  if (items != 1) return 0;
  items = vWritePCLongs(fp, &scale, 1);
  if (items != 1) return 0;

  items = vWritePCLongs(fp, &frames_per_file, 1);
  if (items != 1) return 0;
  fclose(fp);
  return 1;
}
