// Notes by Vassilis Athitsos, 2/3/2003
// This file is an abridged and modified version of the Intel OpenCV file 
// cvshapedescr.cpp, and the main goal is to use the function 
// cvMinEnclosingCircle. In order to use that function we need to use
// the cumbersome CvSeq and CvMemStorage classes. Here I rewrite 
// cvMinEnclosingCircle, as vcvMinEnclosingCircle, so that it can take
// as input arrays of coordinates. 
//
// The end result is that the function cvMinEnclosingCircle is buggy. It
// uses a heuristic iterative approach with a max number of iterations,
// and often it does not return a circle that is large enough. I wrote
// the function vMinEnclosingCircle in algorithms.cpp as a correct
// alternative.

/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "stdafx.h"

//#include  <limits.h>
//#include  <float.h>

//#include "../_include/_cv.h"
#ifdef VASSILIS_OPENCV_LIBRARY

#include "../include/cv.h"

#include  "../_include/_cvgeom.h"
//#include  "_cvgeom.h"

#include "gestures/algorithms.h"



int
icvIntersectLines( double x1, double dx1, double y1, double dy1,
                   double x2, double dx2, double y2, double dy2, double *t2 )
{
    double d = dx1 * dy2 - dx2 * dy1;
    int result = -1;

    if( d != 0 )
    {
        *t2 = ((x2 - x1) * dy1 - (y2 - y1) * dx1) / d;
        result = 0;
    }
    return result;
}


CvStatus
icvFindCircle( CvPoint2D32f pt0, CvPoint2D32f pt1,
               CvPoint2D32f pt2, CvPoint2D32f * center, float *radius )
{
    double x1 = (pt0.x + pt1.x) * 0.5;
    double dy1 = pt0.x - pt1.x;
    double x2 = (pt1.x + pt2.x) * 0.5;
    double dy2 = pt1.x - pt2.x;
    double y1 = (pt0.y + pt1.y) * 0.5;
    double dx1 = pt1.y - pt0.y;
    double y2 = (pt1.y + pt2.y) * 0.5;
    double dx2 = pt2.y - pt1.y;
    double t = 0;

    CvStatus result = CV_OK;

    if( icvIntersectLines( x1, dx1, y1, dy1, x2, dx2, y2, dy2, &t ) >= 0 )
    {
        center->x = (float) (x2 + dx2 * t);
        center->y = (float) (y2 + dy2 * t);
        *radius = (float) icvDistanceL2_32f( *center, pt0 );
    }
    else
    {
        center->x = center->y = 0.f;
        radius = 0;
        result = CV_NOTDEFINED_ERR;
    }

    return result;
}



int
icvFindEnslosingCicle4pts_32f( CvPoint2D32f * pts, CvPoint2D32f * _center, float *_radius )
{
    int shuffles[4][4] = { {0, 1, 2, 3}, {0, 1, 3, 2}, {2, 3, 0, 1}, {2, 3, 1, 0} };

    int idxs[4] = { 0, 1, 2, 3 };
    int i, j, k = 1, mi = 0;
    float max_dist = 0;
    CvPoint2D32f center;
    CvPoint2D32f min_center;
    float radius, min_radius = FLT_MAX;
    CvPoint2D32f res_pts[4];

    center = min_center = pts[0];
    radius = 1.f;

    for( i = 0; i < 4; i++ )
        for( j = i + 1; j < 4; j++ )
        {
            float dist = icvDistanceL2_32f( pts[i], pts[j] );

            if( max_dist < dist )
            {
                max_dist = dist;
                idxs[0] = i;
                idxs[1] = j;
            }
        }

    if( max_dist == 0 )
        goto function_exit;

    k = 2;
    for( i = 0; i < 4; i++ )
    {
        for( j = 0; j < k; j++ )
            if( i == idxs[j] )
                break;
        if( j == k )
            idxs[k++] = i;
    }

    center = icvMidPoint( pts[idxs[0]], pts[idxs[1]] );
    radius = (float) (icvDistanceL2_32f( pts[idxs[0]], center ) * (1 + 0.03));
    if( radius < 1.f )
        radius = 1.f;

    if( icvIsPtInCircle( pts[idxs[2]], center, radius ) &&
        icvIsPtInCircle( pts[idxs[3]], center, radius ))
    {
        k = 2;
    }
    else
    {
        mi = -1;
        for( i = 0; i < 4; i++ )
        {
            if( icvFindCircle( pts[shuffles[i][0]], pts[shuffles[i][1]],
                               pts[shuffles[i][2]], &center, &radius ) >= 0 )
            {
                radius *= 1.03f;
                if( radius < 2.f )
                    radius = 2.f;

                if( icvIsPtInCircle( pts[shuffles[i][3]], center, radius ) &&
                    min_radius > radius )
                {
                    min_radius = radius;
                    min_center = center;
                    mi = i;
                }
            }
        }
        assert( mi >= 0 );
        if( mi < 0 )
            mi = 0;
        k = 3;
        center = min_center;
        radius = min_radius;
        for( i = 0; i < 4; i++ )
            idxs[i] = shuffles[mi][i];
    }

  function_exit:

    *_center = center;
    *_radius = radius;

    /* reorder output points */
    for( i = 0; i < 4; i++ )
    {
        res_pts[i] = pts[idxs[i]];
    }

    for( i = 0; i < 4; i++ )
    {
        pts[i] = res_pts[i];
    }

    return k;
}


// I should fix this, to guarantee correctness.
// Then, I should also 
vCircle vcvMinEnclosingCircle(vArray(double) rows, vArray(double) cols, 
                                long count)
{
  if (count <= 2) return vCircle();
  const int max_iters = 20;
  int i, k;
  // Find the extreme left, right, top and bottom points.
  CvPoint2D32f center = {0, 0};
  CvPoint2D32f pts[8];
  float radius = 0;

  // Initialize extreme points.
  CvPoint pt_left = {(int) cols[0], (int) rows[0]};
  CvPoint pt_right = {(int) cols[0], (int) rows[0]};
  CvPoint pt_top = {(int) cols[0], (int) rows[0]};
  CvPoint pt_bottom = {(int) cols[0], (int) rows[0]};
  // Find extreme points.
  for( i = 1; i < count; i++ )
  {
    double row = rows[i];
    double col = cols[i];
    
    if(col < pt_left.x) 
    {
      pt_left.x = (int) col;
      pt_left.y = (int) row;
    }
    if(col > pt_right.x) 
    {
      pt_right.x = (int) col;
      pt_right.y = (int) row;
    }
    if(row < pt_top.y)
    {
      pt_top.x = (int) col;
      pt_top.y = (int) row;
    }
    if(row > pt_bottom.y)
    {
      pt_bottom.x = (int) col;
      pt_bottom.y = (int) row;
    }
  }

  pts[0] = icvCvtPoint32s_32f(pt_left);
  pts[1] = icvCvtPoint32s_32f(pt_right);
  pts[2] = icvCvtPoint32s_32f(pt_top);
  pts[3] = icvCvtPoint32s_32f(pt_bottom);

//  pts[0] = cvPointTo32f(pt_left);
//  pts[1] = cvPointTo32f(pt_right);
//  pts[2] = cvPointTo32f(pt_top);
//  pts[3] = cvPointTo32f(pt_bottom);

  for( k = 0; k < max_iters; k++ )
  {
    // Find enclosing circle for extreme points.
    icvFindEnslosingCicle4pts_32f( pts, &center, &radius );

    // Find a point that is not included.
    for( i = 0; i < count; i++ )
    {
      CvPoint2D32f ptfl = {(float) cols[i], (float) rows[i]};
      if( !icvIsPtInCircle(ptfl, center, radius))
      {
          pts[3] = ptfl;
          break;
      }
    }
    if( i == count )
        break;
  }

  return vCircle(center.y, center.x, radius);
}


#else    //  VASSILIS_OPENCV_LIBRARY

#include "gestures/algorithms.h"

vCircle vcvMinEnclosingCircle(vArray(double) rows, vArray(double) cols, 
                                long count)
{
  vPrint("can't run vcvMinEnclosingCircle. No OpenCV at compile time.");
  return vCircle();
}

#endif   //  VASSILIS_OPENCV_LIBRARY

/* End of file. */
