#include "stdafx.h"

#include <string>

#include "basics/drawing.h"
#include "basics/video_common.h"
#include "basics/wrapper.h"
#include "interface_programs.h"
#include "avi_file.h"
#include "basics/rle_encoding.h"
#include "gestures/hand_database.h"
#include "basics/auxiliaries.h"
#include "gestures/projections.h"
#include "gestures/algo_templates.h"

#include "basics/definitions.h"

using namespace std;


// This function takes in a vector of arguments, the first
// of which is the name of the "program" to be executed. The
// function figures out which function corresponds to the 
// particular program, and passes the arguments to that 
// function.
long vExecuteProgram(vector<const char *> * arguments)
{
  if (arguments == 0) return 0;
  if (arguments->size() == 0) return 0;
  const char * command = (*arguments)[0];
  if (command == 0) return 0;
  char first_char = command[0];
  long result = -1;

  switch(first_char)
  {
  case 'a':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'b':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'c':
    if (strcmp(command, "convert_ben_story") == 0)
    {
      vConvertBenStory0(arguments);
    }

    else
    {
      vPrint("??? program %s does not exist.\n", command);
    }
    break;

  case 'd':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'e':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'f':
    if (strcmp(command, "frame_rate") == 0)
    {
      vFrameRate(arguments);
    }

    else
    {
      vPrint("??? program %s does not exist.\n", command);
    }

    break;

  case 'g':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'h':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'i':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'j':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'k':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'l':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'm':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'n':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'o':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'p':
    if (strcmp(command, "poser_bmp_to_rle") == 0)
    {
      vPoserBMPToRLE(arguments);
    }

    if (strcmp(command, "poser_bmp_to_avi") == 0)
    {
      vPoserBMPToAVI(arguments);
    }

    else if (strcmp(command, "poser_link_params") == 0)
    {
      vPoserLinkParams(arguments);
    }

    else
    {
      vPrint("??? program %s does not exist.\n", command);
    }
    break;

  case 'q':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'r':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 's':
    if (strcmp(command, "set_frame_rate") == 0)
    {
      vSetFrameRate(arguments);
    }

    else
    {
      vPrint("??? program %s does not exist.\n", command);
    }
    break;

  case 't':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'u':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'v':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'w':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'x':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'y':
    vPrint("??? program %s does not exist.\n", command);
    break;

  case 'z':
    vPrint("??? program %s does not exist.\n", command);
    break;

  default:
    vPrint("??? program %s does not exist.\n", command);
    break;

  }

  return result;
}


// The arguments are not used here.
long vConvertBenStory0(vector<const char *> * arguments)
{
  vector<voidp> filenames;
  char input[1000];
  input[0] = 0;
  vPrint("Type in filenames, and type - at the end: ");
  while(strcmp(input, "-") != 0)
  {
    vScan("%s", input);
    vMoreFilesMatching(input, &filenames);
  }
  long result = vConvertBenStory(&filenames);
  vEraseCharpVector(&filenames);
  return result;
}


long vConvertBenStory(vector<voidp> * filenames)
{
  // Figure out name of new sequence
  long number_end_index = 9;
  if (filenames->size() != 2) return 0;
  char * filename = (char *) (*filenames)[0];
  long id = 0;
  long length = strlen(filename);
  long id_index = length - 9;
  sscanf(&(filename[id_index]), "%li", &id);
  long machine_id = 0;
  sscanf(&(filename[id_index + 4]), "%li", &machine_id);
  char sequence_name[200];
  sprintf(sequence_name, "ben_story_%li_%li.aviset", id, machine_id);

  // Merge time stamps, getting rid of last frame of first file (if 2 files);
  char * new_time_file = vReplaceEnding2(sequence_name, ".time");
  char old_time_file1[1000];
  sprintf(old_time_file1, "%s.time", filename);
  long file_size = vFilesize(old_time_file1);
  const long time_frame_length = 24;
  FILE * fp = fopen(old_time_file1, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("Can't open %s\n", old_time_file1);
    return 0;
  }

  FILE * new_fp = fopen(new_time_file, vFOPEN_WRITE);
  if (new_fp == 0)
  {
    vPrint("Can't open %s\n", new_time_file);
    return 0;
  }

  char * buffer = new char[file_size];
  long items_read = fread(buffer, sizeof(char), file_size, fp);
  fclose(fp);
  if (items_read != file_size)
  {
    vPrint("Read only %li out of %li bytes of %s\n", 
            items_read, file_size, old_time_file1);
    return 0;
  }

  long items_written = fwrite(buffer, sizeof(char), 
                              file_size - time_frame_length, new_fp);
  if (items_written != file_size - time_frame_length)
  {
    vPrint("Wrote only %li of %li bytes to %s\n", items_written,
            file_size - time_frame_length, new_time_file);
    return 0;
  }

  // Process second time file
  // Merge time stamps, getting rid of last frame of first file (if 2 files);
  char * filename2 = (char *) (*filenames)[1];
  sprintf(old_time_file1, "%s.time", filename2);
  file_size = vFilesize(old_time_file1);
  fp = fopen(old_time_file1, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("Can't open %s\n", old_time_file1);
    return 0;
  }

  vdelete2(buffer);
  buffer = new char[file_size];
  items_read = fread(buffer, sizeof(char), file_size, fp);
  fclose(fp);
  if (items_read != file_size)
  {
    vPrint("Read only %li out of %li bytes of %s\n", 
            items_read, file_size, old_time_file1);
    return 0;
  }

  items_written = fwrite(buffer, sizeof(char), 
                              file_size, new_fp);
  if (items_written != file_size)
  {
    vPrint("Wrote only %li of %li bytes to %s\n", items_written,
            new_time_file);
    return 0;
  }
  vdelete2(buffer);
  fclose(new_fp);
  vPrint("Created %s successfully\n", new_time_file);


  // Create an AVI set file
  vAVI_Stream * stream = new vAVI_Stream(filename);
  if (stream->IsValid() == 0) 
  {
    vPrint("Couldn't open %s\n", filename);
    return 0;
  }

  long rows = stream->Rows();
  long cols = stream->Cols();
  long rate = stream->Rate();
  long scale = stream->Scale();
  long color_bytes = stream->GetColorBytes();
  vdelete(stream);

  // Write the first AVI file (get rid of its last frame)
  vReadAVI * read_avi = new vReadAVI(filename);
  if (read_avi->IsValid() == 0)
  {
    vPrint("Couldn't open %s\n", filename);
    return 0;
  }
  long frames1 = read_avi->NumberOfFrames() - 1;
  vMFC_AVI_Set * avi_set = new vMFC_AVI_Set(sequence_name, rows, cols, 
                                              rate, scale, color_bytes, 
                                              frames1);
  long frame;
  for (frame = 0; frame < frames1; frame++)
  {
    color_image * image = read_avi->ReadFrame(frame);
    if (image == 0)
    {
      vPrint("Error in reading image %li\n", frame);
      return 0;
    }
    long result = avi_set->write_frame(image);
    if (result <= 0)
    {
      vPrint("Error in writing frame %li\n", frame);
      return 0;
    }
    vPrint("Wrote frame %li\r", frame);
    vdelete(image);
  }
  vdelete(read_avi);
  vPrint("\nProcessed %s\n", filename);

  // Process second AVI file.
  read_avi = new vReadAVI(filename2);
  if (read_avi->IsValid() == 0)
  {
    vPrint("Couldn't open %s\n", filename2);
    return 0;
  }
  long frames2 = read_avi->NumberOfFrames();
  for (frame = 0; frame < frames2; frame++)
  {
    color_image * image = read_avi->ReadFrame(frame);
    if (image == 0)
    {
      vPrint("Error in reading image %li\n", frame);
      return 0;
    }
    long result = avi_set->write_frame(image);
    if (result <= 0)
    {
      vPrint("Error in writing frame %li\n", frame+frames1);
      return 0;
    }
    vPrint("Wrote frame %li\r", frames1 + frame);
    vdelete(image);
  }
  vdelete(read_avi);
  vPrint("\nProcessed %s\n", filename2);

  long success = avi_set->FinishWriting();
  if (success <= 0)
  {
    vPrint("Error in finishing %s\n", sequence_name);
    return 0;
  }

  vdelete2(new_time_file);
  vdelete(avi_set);
  return 1;
}



// This program is used from python scripts in Poser, when
// they save images corresponding to each viewpoint of 
// each frame of each transition. After finishing with a transition,
// it is useful to go through all the BMP files and save them images
// in a single RLE sequence, in order to keep the total storage low,
// and the number of files also low. The only argument is a string
// that is the base of the filename of all the BMP files.
// The assumption is that the image files are named as 
// output_base_[a]_[b].bmp, where a and b are counters.
long vPoserBMPToRLE(vector<const char *> * arguments)
{
  long number_of_arguments = arguments->size();
  if (number_of_arguments != 2)
  {
    vPrint("\nusage: %s output_base\n", (*arguments)[0]);
    return -2;
  }

  // Figure out the filename, rows and cols for the output
  // sequence.
  const char * output_base = (*arguments)[1];
  vPrint("output base = %s\n", output_base);
  char buffer[1000];
  // Open the first image in the sequence, to see if it exists
  // and to check its size.
  sprintf(buffer, "%s_0_0.bmp", output_base);
  color_image * the_image = vReadBMP(buffer);
  if (the_image == 0)
  {
    vPrint("\nno images were found\n");
    return -3;
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vdelete(the_image);
  sprintf(buffer, "%s.rle", output_base);

  vRLE_Sequence output_sequence(buffer, rows, cols);
  if (output_sequence.IsValid() <= 0)
  {
    vPrint("\nFailed to open output sequence %s\n", buffer);
    return 0;
  }

  long frame_counter, viewpoint_counter;
  frame_counter = viewpoint_counter = 0;

  long break_flag = 0;
  long success = 1;
  long frames = 0;
  // Loop over transition frames
  while(break_flag == 0)
  {
    // Loop over viewpoints.
    while(1)
    {
      sprintf(buffer, "%s_%li_%li.bmp", output_base, 
             frame_counter, viewpoint_counter);
      color_image * the_image = vReadBMP(buffer);
      // If no image was read, we either need to move on to the next
      // transition frame, or we are done. We are done if the viewpoint
      // counter is still at 0.
      if (the_image == 0)
      {
        if (viewpoint_counter == 0)
        {
          break_flag = 1;
        }
        else
        {
          viewpoint_counter = 0;
          frame_counter++;
        }
        break;
      }

      success = output_sequence.write_frame(the_image);
      vdelete(the_image);
      if (success <= 0) break;
      vPrint("processed frame %li, %s\n", frames++, buffer);
      viewpoint_counter++;

      // erase the image file.
      char * command = vMergeStrings2("erase ", buffer);
      vPrint("system %s\n\n", command);
      system(command);
      vdelete2(command);
    }
  }
  if (success > 0)
  {
    success = output_sequence.FinishWriting();
    if (success <= 0)
    {
      vPrint("failed to finish writing the output sequence\n");
    }
  }
  return success;
}


// This program is used from python scripts in Poser, when
// they save images corresponding to each viewpoint of 
// each frame of each transition. After finishing with a transition,
// it is useful to go through all the BMP files and save them images
// in a single AVI sequence, in order to keep the total storage low,
// and the number of files also low. The only argument is a string
// that is the base of the filename of all the BMP files.
// The assumption is that the image files are named as 
// output_base_[a]_[b].bmp, where a and b are counters.
long vPoserBMPToAVI(vector<const char *> * arguments)
{
  const long result_size = 380;
  const long bbox_size = 360;
  long number_of_arguments = arguments->size();
  if (number_of_arguments != 2)
  {
    vPrint("\nusage: %s output_base\n", (*arguments)[0]);
    return -2;
  }

  // Figure out the filename, rows and cols for the output
  // sequence.
  const char * output_base = (*arguments)[1];
  vPrint("output base = %s\n", output_base);
  char buffer[1000];
  // Open the first image in the sequence, to see if it exists
  // and to check its size.
  sprintf(buffer, "%s_0_0.bmp", output_base);
  color_image * the_image = vReadBMP(buffer);
  if (the_image == 0)
  {
    vPrint("\nno images were found\n");
    return -3;
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vdelete(the_image);
  sprintf(buffer, "%s.avi", output_base);

  // This is a hack, if Poser exported to RAM disk, to 
  // save the AVI file to the regular disk.
  if ((buffer[0] == 'Y') || (buffer[0] == 'y'))
  {
    buffer[0] = sequences_directory[0];
  }

  vAVI_Stream output_sequence(buffer, result_size, result_size, 3);
  if (output_sequence.IsValid() <= 0)
  {
    vPrint("\nFailed to open output sequence %s\n", buffer);
    return 0;
  }

  long frame_counter, viewpoint_counter;
  frame_counter = viewpoint_counter = 0;

  long break_flag = 0;
  long success = 1;
  long frames = 0;
  // Loop over transition frames
  while(break_flag == 0)
  {
    // Loop over viewpoints.
    while(1)
    {
      sprintf(buffer, "%s_%li_%li.bmp", output_base, 
             frame_counter, viewpoint_counter);
      color_image * the_image = vReadBMP(buffer);
      // If no image was read, we either need to move on to the next
      // transition frame, or we are done. We are done if the viewpoint
      // counter is still at 0.
      if (the_image == 0)
      {
        if (viewpoint_counter == 0)
        {
          break_flag = 1;
        }
        else
        {
          viewpoint_counter = 0;
          frame_counter++;
        }
        break;
      }

      color_image * the_image2 = vPoserExtractHand(the_image, result_size, bbox_size);
      if (the_image2 == 0)
      {
        function_warning("could not extract hand\n");
        vdelete2(the_image);
        output_sequence.FinishWriting();
        return 0;
      }
   
      color_image * the_image3 = (color_image *) vSwitchTopBottom(the_image2);
      success = output_sequence.write_frame(the_image3);
      vdelete(the_image);
      vdelete(the_image2);
      vdelete(the_image3);
      if (success <= 0) break;
      vPrint("processed frame %li, %s\n", frames++, buffer);
      viewpoint_counter++;

      // erase the image file.
      char * command = vMergeStrings2("erase ", buffer);
      //V_Print("system %s\n\n", command);
      system(command);
      vdelete2(command);
    }
  }
  if (success > 0)
  {
    success = output_sequence.FinishWriting();
    if (success <= 0)
    {
      vPrint("failed to finish writing the output sequence\n");
    }
  }
  Sleep(3000);

  return success;
}


// This program is used to determine, given current camera params,
// link world matrix, link orientation, and a list of viewpoints,
// a list of camera parameters under which the link will be seen having
// the orientations described by the viewpoints. We assume that the 
// camera is pointing at the link, so that we can ignore the actual
// 3D positions of the link and the camera.
long vPoserLinkParams(vector<const char *> * arguments)
{
  long number_of_arguments = arguments->size();
  if (number_of_arguments != 2)
  {
    vPrint("usage: %s info_filename\n", (*arguments)[0]);
    Sleep(5000);
    return -2;
  }

  const char * info_filename = (*arguments)[1];
  FILE * fp = fopen(info_filename, vFOPEN_READ);
  if (fp == 0)
  {
    vPrint("failed to open %s\n", info_filename);
    Sleep(5000);
    return -3;
  }

  long success = 1;
  vMatrix<double> world_matrix(4, 4);
  long temp_success = world_matrix.ReadTextBandwise(fp);
  if (temp_success <= 0) success = 0;
  vMatrix<double> orientation(1, 3);
  temp_success = orientation.ReadTextBandwise(fp);
  if (temp_success <= 0) success = 0;

  char * viewpoints_file = vReadString(fp);
  if (viewpoints_file == 0) success = 0;
  char * output_file = vReadString(fp);
  if (output_file == 0) success = 0;
  fclose(fp);

  if (success > 0)
  {
    vMatrix<double> viewpoints = vLoadViewpoints(viewpoints_file);
    if (viewpoints.Rows() == 0)
    {
      success = 0;
    }
    else
    {
      vMatrix<double> camera_params = 
        vLinkCameraParams(world_matrix, orientation, viewpoints);
      temp_success = vSaveCameraParams(output_file, camera_params);
      if (temp_success <= 0) success = 0;
    }
  }

  vdelete2(viewpoints_file);
  vdelete2(output_file);
  if (success <= 0)
  {
    Sleep(3000);
  }
  return success;
}


// This program simply goes through all the files matching all the arguments
// and prints out the frame rates for the video sequences stored in those
// files.
long vFrameRate(vector<const char *> * arguments)
{
  vector<voidp> filenames;
  long i;
  for (i = 1; i < (long) arguments->size(); i++)
  {
    vMoreFilesMatching((*arguments)[i], &filenames);
  }

  long number = filenames.size();
  for (i = 0; i < number; i++)
  {
    const char * filename = (const char *) filenames[i];
    vVideoFile * video = vOpenReadVideoFile(filename);
    long frames, scale;
    if (video != 0)
    {
      frames = video->Rate();
      scale = video->Scale();
      vCloseVideoFile(video);
    }
    else
    {
      frames = -1;
      scale = 0;
    }

    vPrint("%li/%li: %s: %li frames per %li sec\n", i+1, number, filename, frames, scale);
  }

  vEraseCharpVector(&filenames);
  return 1;
}


// This program reads video files and saves copies of those files
// in a specified directory. The frame rate in the copies is
// set to the specified rate and scale. It is assumed that
// the video files are of format either AVI or AVISET.
long vSetFrameRate(vector<const char *> * arguments)
{
  long arg_number = arguments->size();
  if (arg_number < 5)
  {
    vPrint("usage: %s rate scale directory file_1 ... file_n\n",
            (*arguments)[0]);
    exit(1);
  }

  long arg_counter = 1;
  long rate, scale;
  rate = scale = 0;
  sscanf((*arguments)[arg_counter++], "%li", &rate);
  sscanf((*arguments)[arg_counter++], "%li", &scale);
  const char * directory = (*arguments)[arg_counter++];
  vPrint("rate = %li, scale = %li\n", rate, scale);

  vector<voidp> filenames;
  long i;
  for (i = arg_counter; i < arg_number; i++)
  {
    vMoreFilesMatching((*arguments)[i], &filenames);
  }

  long number = filenames.size();
  for (i = 0; i < number; i++)
  {
    const char * filename = (const char *) filenames[i];
    vVideoFile * video_in = vOpenReadVideoFile(filename);
    if (video_in == 0) continue;
    vPrint("\nprocessing %s: file %li of %li\n", filename, i+1, number);

    char * parent = vParentDir(filename, platform_directory_slash);
    char * simple = vSimpleName(filename, platform_directory_slash);
    char * new_file = vJoinPaths3(parent, directory, simple);

    long rows = video_in->Rows();
    long cols = video_in->Cols();
    long old_rate = video_in->Rate();
    long old_scale = video_in->Scale();
    long color_bytes = video_in->BytesPerPixel();

    vVideoFile * video_out = vOpenWriteVideoFile(new_file, rows, cols, 
                                                   rate, scale, color_bytes);

    vdelete2(parent);
    vdelete2(simple);
    vdelete2(new_file);

    if (video_out == 0) 
    {
      vPrint("Failed to open %s\n", new_file);
      continue;
    }

    long j;
    long frames = video_in->NumberOfFrames();
    for (j = 0; j < frames; j++)
    {
      long success;
      if (color_bytes == 3)
      {
        color_image * the_image = video_in->ReadFrame(j);
        success = video_out->write_frame(the_image);
        vdelete(the_image);
      }
      else
      {
        grayscale_image * the_image = video_in->ReadFrameGray(j);
        success = video_out->write_frameGray(the_image);
        vdelete(the_image);
      }
      if (success <= 0)
      {
        vPrint("\nFailed to write frame %li\n", j+1);
      }
      else 
      {
        vPrint("Processed frame %li of %li\r", j+1, frames);
      }
    }
    video_out->FinishWriting();
    vCloseVideoFile(video_out);
  }

  return 1;
}

