// Comments by Vassilis: Calling cvLineFit2D would crash the program very often.
// Here I made minimal changes, so that when the iterations fail to converge
// it just returns something without throwing an exception. This is a subset
// of the Intel OpenCV file cvgeometry.cpp


/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/
#include "stdafx.h"

#ifdef VASSILIS_OPENCV_LIBRARY

#include "../_include/_cv.h"
//#include "../_include/_cvwrap.h"

//#include "_cv.h"
//#include "_cvwrap.h"

#include "../_include/_cvutils.h"
//#include "_cvutils.h"

#include <math.h>

#define _FP double

#if _MSC_VER >= 1200
#pragma warning (disable:4701)
#endif

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvFree_
//    Purpose:
//      Free allocated memory
//    Context:
//    Parameters:
//      pPointer - pointer to memory to be deallocated
//    Returns:
//      CV_OK if all Ok or error code
//    Notes:
//      The memory block must be allocated by icvAlloc_ function
//F*/
void vicvFree(float ** pptr)
{
    /* Checking for bad pointer */
    if( pptr && *pptr )
    {
        long ptr = (long)*pptr;
    
        /* Pointer must be align by 32 */
        assert( (ptr & 31) == 0 );
        *pptr = 0;
    
        free( *(void**)(ptr - 4) );
    }
} /* _cvFree */

void* vicvAlloc(int size)
{
    long ptr = (long)malloc( size + 64 + 4);

    if( !ptr ) return 0;
    *(long*)(((ptr + 32 + 4) & ~31) - 4) = (long)ptr;
    return (void*)((ptr + 32 + 4) & ~31);
} /* icvAlloc_ */


CvStatus
icvFitLine2D_wods( CvPoint2D32f * points, int _count, float *weights, float *line )
{
    _FP x = 0, y = 0, x2 = 0, y2 = 0, xy = 0, w = 0;
    _FP dx2, dy2, dxy;
    int i;
    int count = _count;
    float t;

    /* Calculating the average of x and y... */

    if( weights == 0 )
    {
        for( i = 0; i < count; i += 1 )
        {
            x += points[i].x;
            y += points[i].y;
            x2 += points[i].x * points[i].x;
            y2 += points[i].y * points[i].y;
            xy += points[i].x * points[i].y;
        }
        w = (float) count;
    }
    else
    {
        for( i = 0; i < count; i += 1 )
        {
            x += weights[i] * points[i].x;
            y += weights[i] * points[i].y;
            x2 += weights[i] * points[i].x * points[i].x;
            y2 += weights[i] * points[i].y * points[i].y;
            xy += weights[i] * points[i].x * points[i].y;
            w += weights[i];
        }
    }

    x /= w;
    y /= w;
    x2 /= w;
    y2 /= w;
    xy /= w;

    dx2 = x2 - x * x;
    dy2 = y2 - y * y;
    dxy = xy - x * y;

    t = (float) atan2( 2 * dxy, dx2 - dy2 ) / 2;
    line[0] = (float) cos( t );
    line[1] = (float) sin( t );

    line[2] = (float) x;
    line[3] = (float) y;

    return CV_NO_ERR;
}


void
icvCalcDist2D( CvPoint2D32f * points, int count, float *_line, float *dist )
{
    int j;
    float px = _line[2], py = _line[3];
    float nx = _line[1], ny = -_line[0];

    for( j = 0; j < count; j++ )
    {
        float x, y;

        x = points[j].x - px;
        y = points[j].y - py;

        dist[j] = (float) fabs( nx * x + ny * y );
    }
}

void
icvCalcDist3D( CvPoint3D32f * points, int count, float *_line, float *dist )
{
    int j;
    float px = _line[3], py = _line[4], pz = _line[5];
    float vx = _line[0], vy = _line[1], vz = _line[2];

    for( j = 0; j < count; j++ )
    {
        float x, y, z;

        x = points[j].x - px;
        y = points[j].y - py;
        z = points[j].z - pz;

        dist[j] =
            (float) sqrt( (vy * z - vz * y) * (vy * z - vz * y) +
                          (vz * x - vx * z) * (vz * x - vx * z) + (vx * y - vy * x) * (vx * y -
                                                                                       vy *
                                                                                       x) );
    }
}

void
icvWeightL1( float *d, int count, float *w )
{
    int i;

    for( i = 0; i < count; i++ )
    {
        w[i] = 1 / (float) fabs( (double) d[i] );
    }
}

void
icvWeightL12( float *d, int count, float *w )
{
    int i;

    for( i = 0; i < count; i++ )
    {
        w[i] = 1.0f / (float) sqrt( 1 + (double) (d[i] * d[i] / 2) );
    }
}

void
icvWeightFair( float *d, int count, float *w, float _c )
{
    int i;
    const float c = _c == 0 ? 1 / 1.3998f : 1 / _c;

    for( i = 0; i < count; i++ )
    {
        w[i] = 1 / (1 + d[i] * c);
    }
}

void
icvWeightWelsch( float *d, int count, float *w, float _c )
{
    int i;
    const float c = _c == 0 ? 1 / 2.9846f : 1 / _c;

    for( i = 0; i < count; i++ )
    {
        w[i] = (float) exp( -d[i] * d[i] * c * c );
    }
}


/* Takes an array of 2D points, type of distance (including user-defined 
distance specified by callbacks, fills the array of four floats with line 
parameters A, B, C, D, where (A, B) is the normalized direction vector, 
(C, D) is the point that belongs to the line. */

CvStatus vicvFitLine2D(CvPoint2D32f * points, int count, CvDisType dist,
                        void *param, float reps, float aeps, float *line)
{
    void (*calc_weights) (float *, int, float *) = 0;
    void (*calc_weights_param) (float *, int, float *, float) = 0;
    float *w;                   /* weights */
    float *r;                   /* square distances */
    int i;
    float _line[6], _lineprev[6];
    int first = 1;
    float rdelta = reps != 0 ? reps : 1.0f;
    float adelta = aeps != 0 ? aeps : 0.01f;
    CvStatus ret;
    union
    {
        void *p;
        void (*fp) (float *, int, float *);
    }
    _PFP;
    float _param;

    if( points == 0 || (dist == CV_DIST_USER && param == 0) || line == 0 )
    {
        return CV_NULLPTR_ERR;
    }

    if( count < 0 || reps < 0 || aeps < 0 )
    {
        return CV_BADFACTOR_ERR;
    }

    switch (dist)
    {
    case CV_DIST_L2:
        return icvFitLine2D_wods( points, count, 0, line );

    case CV_DIST_L1:
        calc_weights = icvWeightL1;
        break;

    case CV_DIST_L12:
        calc_weights = icvWeightL12;
        break;

    case CV_DIST_FAIR:
        calc_weights_param = icvWeightFair;
        _param = param ? *(float *) param : 0;
        break;

    case CV_DIST_WELSCH:
        calc_weights_param = icvWeightWelsch;
        _param = param ? *(float *) param : 0;
        break;

    case CV_DIST_USER:
        _PFP.p = param;
        calc_weights = (void ( * )(float *, int, float *)) _PFP.fp;
        break;

    default:
        return CV_BADFACTOR_ERR;
    }

    w = (float *) vicvAlloc( count * sizeof( float ));
    r = (float *) vicvAlloc( count * sizeof( float ));

    for( i = 0; i < count; i++ )
        w[i] = 1.0f;

    ret = icvFitLine2D_wods( points, count, 0, _line );
    for( i = 0; i < 100; i++ )
    {
        if( ret != CV_NO_ERR )
        {
            vicvFree( &w );
            vicvFree( &r );
            return ret;
        }

        if( first )
        {
            first = 0;
        }
        else
        {
            double verror1 = (_line[0] * _lineprev[0] + _line[1] * _lineprev[1]);
            if ((verror1 > 1) && (verror1 < 1.001)) verror1 = 1;
            double verror2 = acos(verror1);
            double verror3 = fabs(verror2);
            double verror4 = fabs( acos( (float) (_line[0] * _lineprev[0] + _line[1] * _lineprev[1]) ));
//            if( fabs( acos( (float) (_line[0] * _lineprev[0] + _line[1] * _lineprev[1]) ))
//                < adelta )
            if (verror3 < adelta)
            {
                float x, y, d;

                x = (float) fabs( _line[2] - _lineprev[2] );
                y = (float) fabs( _line[3] - _lineprev[3] );

                d = x > y ? x : y;
                if( d < rdelta )
                {
                    // Return...
                    memcpy( line, _line, 4 * sizeof( float ));

                    vicvFree( &w );
                    vicvFree( &r );
                    return CV_NO_ERR;
                }

            }
        }
        /* calculate distances */
        icvCalcDist2D( points, count, _line, r );

        /* calculate weights */
        if( calc_weights )
        {
            calc_weights( r, count, w );
        }
        else if( calc_weights_param )
        {
            calc_weights_param( r, count, w, _param );
        }
        else
        {
            assert( 0 );
            return CV_BADFACTOR_ERR;
        }


        /* save the line parameters */
        memcpy( _lineprev, _line, 4 * sizeof( float ));

        /* Run again... */
        ret = icvFitLine2D_wods( points, count, w, _line );
    }

    vicvFree( &w );
    vicvFree( &r );
    return CV_BADCONVERGENCE_ERR;
}


#endif // VASSILIS_OPENCV_LIBRARY
