#include "stdafx.h"

#include "vplatform.h"
#include "mfc_aux.h"
#include "basics/auxiliaries.h"

#include "basics/definitions.h"

using std::vector;

CString vGetFilenameRead(const char * init_name, const char * filter)
{
  int flags = 0;
  CFileDialog file_dialog(TRUE, NULL, init_name,
                          flags, filter, NULL);
  int success = file_dialog.DoModal();
  if (success != IDOK) return "";

  CString result = file_dialog.GetPathName();
  return result;
}


CString vGetFilenameWrite(const char * init_name, const char * filter)
{
  int flags = OFN_HIDEREADONLY & OFN_OVERWRITEPROMPT;
  CFileDialog file_dialog(TRUE, NULL, init_name,
                          flags, filter, NULL);
 
  int success = file_dialog.DoModal();
  if (success != IDOK) return "";

  CString result = file_dialog.GetPathName();
  return result;
}


void vGetFilenames(const char * init_dir, const char * filter,
									  vector<CString> * filenames)
{
  filenames->erase(filenames->begin(), filenames->end());
	vGetMoreFilenames(init_dir, filter, filenames);
}


void vGetMoreFilenames(const char * init_dir, const char * filter,
												vector<CString> * filenames)
{
  char filename_buffer[20000];
  filename_buffer[0] = 0;
  int flags = OFN_ALLOWMULTISELECT;
  CFileDialog file_dialog(TRUE, NULL, 0,
                          flags, filter, NULL);
  file_dialog.m_ofn.lpstrFile = filename_buffer;
  file_dialog.m_ofn.nMaxFile = 20000;
  file_dialog.m_ofn.lpstrInitialDir = init_dir;
 
  int success = file_dialog.DoModal();
  if (success != IDOK) return;

  POSITION position = file_dialog.GetStartPosition();
  while(position != 0)
  {
	  CString filename = file_dialog.GetNextPathName(position);
	  filenames->push_back(filename);
  }
}




CString vVectorToCString(vector<CString> * strings)
{
	long length = strings->size();
	long string_length = 0;
	long i;
	for (i = 0; i < length; i++)
		string_length += strlen((*strings)[i].GetBuffer(0)) + 3;
	if (string_length == 0) return "";

	char * input_string = new char[string_length];
	input_string[0] = 0;
	long current_index = 0;
	for (i = 0; i < length; i++)
	{
		CString current = (*strings)[i];
		long current_length = strlen(current.GetBuffer(0));
		sprintf(&(input_string[current_index]), "%s ", current.GetBuffer(0));
		current_index += current_length + 1;
	}

	CString result = input_string; 
	vdelete2(input_string);
	return result;
}


// Starts a new thread and returns a handle to it. The thread ID
// is stored in thread_id. The thread function must be a function
// that returns an unsigned int and takes in a void * argument.
// args is exactly the argument that will be passed to the thread 
// function.
// The return value is 0 if it was not successful.
HANDLE vNewThread(AFX_THREADPROC thread_function, void * args,
                   DWORD * thread_id)
{
	CWinThread * hThread; 
	hThread = AfxBeginThread(thread_function, args,
							 THREAD_PRIORITY_NORMAL, 0, 0,
							 NULL);

	// Check the return value for success. 

	if (hThread == NULL) 
		return 0; 

	else 
  {
    if (thread_id != 0) *thread_id = hThread->m_nThreadID; 
    return hThread->m_hThread;
  }
}


// Sends a mouse click event to the window represented by the handle.
void vSendMouseClick(HWND handle, int x, int y)
{
	LPARAM lparam = x + 65536 * y;
	::SendMessage(handle, WM_LBUTTONDOWN, 0, lparam);
	::SendMessage(handle, WM_LBUTTONUP, 0, lparam);
}


// Returns in a 64-bit integer the free disk space on the drive
// that corresponds to the given filename.
ULONGLONG vFreeDiskSpace(char * filename)
{
  char * name_copy = vCopyString(filename);
  long length = strlen(name_copy);

  int i;
  for (i = length - 1; i > 0; i--)
  { 
    if (name_copy[i] == '\\')
    {
      name_copy[i] = 0;
      break;
    }
  }

  ULARGE_INTEGER bytes_available, total_bytes, bytes_free;
  GetDiskFreeSpaceEx(name_copy, &bytes_available,
                     &total_bytes, &bytes_free);

  delete [] name_copy;
  return bytes_available.QuadPart;
}


char * vFileTitlePC(const char * filename)
{
  CFileFind finder;
  long success = finder.FindFile(filename);
  if (success == 0) return 0;
  finder.FindNextFile();
  CString directory = finder.GetFileTitle();
  char * result = vCopyString(directory.GetBuffer(0));
  vAdjustDirectoryName(result);
  return result;
}

