#include "stdafx.h"

#include <direct.h>

#include "basics/auxiliaries.h"
#include "basics/wrapper.h"


using namespace std;

void S_MatchesUsingCFileFind(const char * pattern, 
                             vector<voidp> * filenames);

void S_InitialMatches(const char * component, vector<voidp> * matches);

void S_ExpandMatches(const char * component, 
                     vector<voidp> * current_matches, 
                     vector<voidp> * new_matches);

ushort S_DriveExists(const char * drive);

ushort S_DirectoryExists(const char * drive);

ushort S_DriveExists(const char * drive)
{
  long length = strlen(drive);
  if ((length != 2) && (length != 3)) return 0;
  if (drive[1] != ':') return 0;
  if ((length == 3) && (drive[2] != '/') && (drive[2] != '\\')) return 0;

  return S_DirectoryExists(drive);
}


ushort S_DirectoryExists(const char * directory)
{
  // First get current directory
  CFileFind finder;
  BOOL more_files = finder.FindFile(".");
  if (more_files == 0) return 0;

  finder.FindNextFile();
  CString current_dir = finder.GetFilePath();
  
  // We check if the directory exists by trying to cd to it. If it
  // works, it exists.
  long dir_length = strlen(directory);
  char * buffer = new char[dir_length + 3];
  sprintf(buffer, "%s/", directory);
  long result = chdir(buffer);
  long result2 = chdir(current_dir.GetBuffer(0));
  assert(result2 == 0);
  vdelete2(buffer);
  if (result == 0) return 1;
  else return 0;
}


void S_MatchesUsingCFileFind(const char * pattern, 
                             vector<voidp> * filenames)
{
  // CFileFind does the wrong thing when passed things like "D:". Then, instead of
  // returning just "D:", it returns the current directory in the D drive (same 
  // behavior as when typing "D:" at the DOS prompt. So, in that case I will
  // not use CFileFind.             
  if (S_DriveExists(pattern))
  {
    char * pattern_copy = vCopyString(pattern);
    vAdjustDirectoryName(pattern_copy);
    filenames->push_back(pattern_copy);
    return;
  }

  // If the pattern is not a drive name, then we can use CFileFind.
  CFileFind finder;
  BOOL more_files = finder.FindFile(pattern);
  while(more_files)
  {
    more_files = finder.FindNextFile();
    CString filename = finder.GetFilePath();
    char * filename_array = vCopyString(filename.GetBuffer(0));
    vAdjustDirectoryName(filename_array);
    filenames->push_back(filename_array);
  }
}


char * vFileName(const char * filename)
{
  CFileFind finder;
  long success = finder.FindFile(filename);
  if (success == 0) return 0;
  finder.FindNextFile();
  CString directory = finder.GetFileName();
  char * result = vCopyString(directory.GetBuffer(0));
  vAdjustDirectoryName(result);
  return result;
}


char * full_pathname(const char * filename)
{
  CFileFind finder;
  long success = finder.FindFile(filename);
  if (success == 0) return 0;
  finder.FindNextFile();
  CString directory = finder.GetFilePath();
  char * result = vCopyString(directory.GetBuffer(0));
  vAdjustDirectoryName(result);
  return result;
}


ushort vDirectoryFiles(const char * dir, vector<voidp> * filenames)
{
  // Filenames should contain nothing at the beginning, so that the result
  // is mathematically correct (otherwise at the end of the function, in 
  // a mathematical sense, we are not sure that filenames are just the 
  // directory files of the given directory.

  if (filenames->size() != 0) return 0;

  long length = strlen(dir);
  long new_length = length + 10;
  char * pattern = new char[new_length];
  memcpy(pattern, dir, length);
  if ((pattern[length-1] == '/') ||
      (pattern[length-1] == '\\'))
  {
    length--;
    pattern[length] = 0;
  }
  memcpy(&(pattern[length]), "/*", 3);

  S_MatchesUsingCFileFind(pattern, filenames);

  return 1;
}


void S_InitialMatches(const char * component, vector<voidp> * matches)
{
  char * copy = vCopyString("");
  vector<voidp> current_matches;
  current_matches.push_back(copy);
  S_ExpandMatches(component, &current_matches, matches);
  vEraseCharpVector(&current_matches);

//  CFileFind finder;
//
//  BOOL more_files = finder.FindFile(component);
//  while(more_files)
//  {
//    more_files = finder.FindNextFile();
//    CString filename = finder.GetFilePath();
//    matches.push_back(vCopyString(filename.GetBuffer(0)));
//  }
}


void S_ExpandMatches(const char * component, 
                     vector<voidp> * current_matches,
                     vector<voidp> * new_matches)
{
  long current_length = current_matches->size();
  long i;
  // There are two special cases for the pattern, i.e. "." and "..".
  // We don't allow those to be matched with wildcards.

  if ((strcmp(component, ".") == 0) ||
      (strcmp(component, "..") == 0))
  {
    for (i = 0; i < current_length; i++)
    {
      char * current_match = (char *) ((*current_matches)[i]);
      long match_length = strlen(current_match);
      char * new_match = new char[match_length+5];
      if (match_length == 0) strcpy(new_match, component);
      else sprintf(new_match, "%s/%s", current_match, component);
      vector<voidp> temp_filenames;
      S_MatchesUsingCFileFind(new_match, &temp_filenames);
      vdelete2(new_match);
      new_matches->insert(new_matches->begin(), 
                          temp_filenames.begin(), temp_filenames.end());
      temp_filenames.erase(temp_filenames.begin(), temp_filenames.end());
    }
    return;
  }
 
  long component_length = strlen(component);
  
  // For efficiency, instead of getting all directory entries for each current match,
  // we only get the ones that match the maximum initial segment of component that
  // contains no wildcards.
  long temp = 0;
  while((temp < component_length) && (component[temp] != '*') && 
        (component[temp] != '?'))
  {
    temp++;
  }

  long first_card_index = temp;
  long first_part_length = first_card_index + 1;
  char * first_part = new char[first_part_length + 1];
  memcpy(first_part, component, first_card_index);
  strcpy(&(first_part[first_card_index]), "*");
  long trash1 = strlen(first_part);
  assert(trash1 == first_part_length);
    

  
  for (i = 0; i < current_length; i++)
  {
    const char * old_match = (const char *) (*current_matches)[i];
    long match_length = strlen(old_match);
    long pattern_length = match_length + first_part_length + 1;
    char * pattern = new char[pattern_length+1];
    if (match_length != 0)
    {
      sprintf(pattern, "%s/%s", old_match, first_part);
    }
    else 
    {
      sprintf(pattern, "%s", first_part);
      pattern_length--;
    }

    trash1 = strlen(pattern);
    assert(trash1 == pattern_length);
    vector<voidp> potential_matches;
    S_MatchesUsingCFileFind(pattern, &potential_matches);
    vdelete2(pattern);
    long j;
    for (j = 0; j < (long) potential_matches.size(); j++)
    {
      // This [i] should be changed to [j];
      char * potential_match = (char *) (potential_matches[j]);
      char * potential_component  = vFileName(potential_match);

      if (vFilePatternsMatch(component, potential_component))
      {
        new_matches->push_back(potential_match);
      }
      else 
      {
        vdelete2(potential_match);
      }
      vdelete2(potential_component);
    }
    potential_matches.erase(potential_matches.begin(), 
                            potential_matches.end());

  }
  vdelete2(first_part);
}

   
ushort vFilesMatching(const char * pattern, vector<voidp> * filenames)
{
  // To guarantee that at the end the vector "filenames" will contain nothing but
  // the correct matches, we impose that the vector should be empty at the 
  // beginning of the function.
  if (filenames->size() != 0) return 0;

  return vMoreFilesMatching(pattern, filenames);
}


ushort vMoreFilesMatching(const char * pattern, vector<voidp> * filenames)
{
  if (pattern == 0) return 1;
  if (strlen(pattern) == 0) return 1;

  // A special case is drive names, like "D:", because they don't get handled 
  // correctly (at least the way I would consider to be correct) by CFileFind.
  // I take care of that case here.
  if (S_DriveExists(pattern))
  {
    char * pattern_copy = vCopyString(pattern);
    vAdjustDirectoryName(pattern_copy);
    filenames->push_back(pattern_copy);
    return 1;
  }
  
  
  vector<voidp> components;
  vPathComponents(pattern, &components);

  long length = components.size();
  if (length == 0) 
  {
    return 0;
  }
  
  vector<voidp> current_matches;
  S_InitialMatches((const char *) (components[0]), &current_matches);
  long i;
  for (i = 1; i < length; i++)
  {
    vector<voidp> new_matches;
    S_ExpandMatches((const char *) (components[i]), 
                    &current_matches, &new_matches);
    vEraseCharpVector(&current_matches);
    current_matches.insert(current_matches.begin(), 
                           new_matches.begin(), new_matches.end());
    new_matches.erase(new_matches.begin(), new_matches.end());
  }
  filenames->insert(filenames->end(), 
                    current_matches.begin(), current_matches.end());
  current_matches.erase(current_matches.begin(), current_matches.end());
  vEraseCharpVector(&components);
  return 1;
}


ushort vMatchArguments(long argc, char ** argv, long start, 
                        vector<voidp> * filenames)
{
  if (filenames->size() != 0) return 0;
  long counter = start;
  long success_flag = 1;
  while(counter < argc)
  {
    const char * filename = argv[counter++];
    vector<voidp> current_names;
    ushort success = vFilesMatching(filename, &current_names);
    if (success == 0)
    {
      printf("Unexpected error in matching file %s\n", filename);
      success_flag = 0;
    }
    filenames->insert(filenames->end(), current_names.begin(), 
                     current_names.end());
    current_names.erase(current_names.begin(), current_names.end());
  }
  return (ushort) success_flag;
}

