#include "stdafx.h"

#include <assert.h>

#include "net_utils.h"
#include "auxiliaries.h"

#include "definitions.h"

static const int LONG_SIZE = 4;


long vWrite_N_Bytes(CSocket * socket, char * buffer, long n)
{
	long result = 0;
	long left = n;
	long bytes_sent = 0;
	char * current_position = buffer;
	while (left > 0)
	{
		bytes_sent = socket->Send(current_position, left);
		if (bytes_sent <= 0) break;
		current_position += bytes_sent;
		left -= bytes_sent;
	}
	result = n - left;
	return result;
}


long vRead_N_Bytes(CSocket * socket, char * buffer, long n)
{
	long result = 0;
	long left = n;
	long bytes_read = 0;
	char * current_position = buffer;
	while (left > 0)
	{
		bytes_read = socket->Receive(current_position, left);
		if (bytes_read <= 0) break;
		current_position += bytes_read;
		left -= bytes_read;
	}
	result = n - left;
	return result;
}


void vForce_Write_N(CSocket * socket, char * buffer, long n)
{
	long result = vWrite_N_Bytes(socket, buffer, n);
	assert(result == n);
}


void vForce_Read_N(CSocket * socket, char * buffer, long n)
{
	long result = vRead_N_Bytes(socket, buffer, n);
	assert(result == n);
}


void vWriteLong(CSocket * socket, long number, ushort & success)
{
	unsigned char buffer[LONG_SIZE];	
	int i;
	for (i = 0; i < LONG_SIZE; i++)
	{
		buffer[i] = (number >> (8 * i)) & 0xFF;
	}
  
  int bytes_sent;
  bytes_sent = vWrite_N_Bytes(socket, (char *) buffer, LONG_SIZE);
  if (bytes_sent == LONG_SIZE) success = 1;
  else success = 0;
}


long vReadLong(CSocket * socket, ushort & success)
{
	unsigned char buffer[LONG_SIZE];
	int i;

  int bytes_read;
	bytes_read = vRead_N_Bytes(socket, (char *) buffer, LONG_SIZE);
	long result = 0;
	for (i = LONG_SIZE - 1; i >= 0; i--)
	{
		result = 256 * result + buffer[i];
	}

  if (bytes_read == LONG_SIZE) success = 1;
  else success = 0;
	return result;
}

void vForceWriteLong(CSocket * socket, long number)
{
  ushort success;
	V_WriteLong(socket, number, success);
  assert(success == 1);
}


long vForceReadLong(CSocket * socket)
{
  ushort success;
  long result = vReadLong(socket, success);
  assert(success == 1);
  return result;
}


void vWriteString(CSocket * socket, char * string, ushort & success)
{
	long length = strlen(string);

 	V_WriteLong(socket, length, success);
  if (success == 0) return;
	
  int bytes_sent;
  bytes_sent = vWrite_N_Bytes(socket, string, length);
  if (bytes_sent == length) success = 1;
  else success = 0;
}


char * vReadString(CSocket * socket, ushort & success)
{
	long length = vReadLong(socket, success);
  if (success == 0) return 0;

	char * buffer = new char[length+1];
	int bytes_read = vRead_N_Bytes(socket, buffer, length);
	buffer[length] = 0;
  if (bytes_read != length) 
  {
    success = 0;
    delete [] buffer;
    return 0;
  }
  else 
  {
    success = 1;
    return buffer;
  }
}


void vForceWriteString(CSocket * socket, char * string)
{
  ushort success;
  vWriteString(socket, string, success);
  assert(success == 1);
}


char * vForceReadString(CSocket * socket)
{
  ushort success;
  char * result = vReadString(socket, success);
  assert(success == 1);
  return result;
}


// Returns 0 if it fails to connect, a valid CSocket pointer 
// otherwise.
CSocket * vMakeServer(int tcp_port)
{
	int temp_result;
	CSocket * server_socket = new CSocket;

	temp_result = vMakeServer(server_socket, tcp_port);
	if (temp_result == 0) 
	{
		delete server_socket;
		return 0;
	}
	else return server_socket;
}


// Returns 0 if it fails to connect, 1 otherwise;
unsigned short vMakeServer(CSocket * server_socket, int tcp_port)
{
	int temp_result;

	temp_result = server_socket->Create(tcp_port);
	if (temp_result == 0) return 0;
	
	temp_result = server_socket->Listen();
	if (temp_result == 0) return 0;
	
	return 1;
}

// Returns 0 if it fails to connect, a CSocket pointer otherwise.
CSocket * vMakeClient(char * server_address, int tcp_port)
{
  CSocket * client_socket = new CSocket;
  int temp_result;

  temp_result = vMakeClient(client_socket, 
							 server_address, tcp_port);
  if (temp_result == 0)
  {
	  delete client_socket;
	  return 0;
  }
  else return client_socket;
}

// Returns 0 if it fails to connect, 1 otherwise.
unsigned short vMakeClient(CSocket * client_socket,
						    char * server_address, int tcp_port)
{
  int temp_result;

  temp_result = client_socket->Create();
  if (temp_result == 0)
  {
    DWORD last_error = client_socket->GetLastError();
    char buffer[100];
    sprintf(buffer, "%li", last_error);
    AfxMessageBox(buffer);
    return 0;
  }
   
  temp_result = client_socket->Connect(server_address, tcp_port);
  if (temp_result == 0) 
  {
	  client_socket->Close();
	  return 0;
  }
 
  return 1;
}



