#ifndef VASSILIS_OPENCV_UTILS_H
#define VASSILIS_OPENCV_UTILS_H


#include "basics/v_types.h"
#include "basics/image.h"
#include "basics/drawing.h"


// If the OpenCV library is installed, then include the following 
// declarations.
#ifdef VASSILIS_OPENCV_LIBRARY

#include <cv.h>



#else  // VASSILIS_OPENCV_LIBRARY

class IplImage
{
public:
  long depth, nChannels, width, height, nSize;
  long widthStep;
  void * imageData;
};

typedef long CvDisType;
static CvDisType CV_DIST_WELSCH = 0;


typedef long CvPoint2D32f;


#endif // VASSILIS_OPENCV_LIBRARY


// Creates a 3-band (3-channel) image with the given dimensions
IplImage * IplColorImage(long rows, long cols);

// Creates a 1-band (grayscale) image with the given dimensions
IplImage * IplGrayImage(long rows, long cols);

// Creates a 1-band ipl image containing the specified band 
// of color_image (0 = red, 1 = green, 2 = blue)
IplImage * vBandToIplImage(v3dMatrix<uchar> * the_image, long band);

// Copies over the pixel values from the source image to the
// specified band of the target image (0 = red, 1 = green, 2 = blue),
// assuming that the source is a one-band 8 bits/pixel image.
ushort vIplImageToBand(IplImage * source, v3dMatrix<uchar> * target, long band);

// This one works on float images.
ushort vIplFloatImageToBand(IplImage * source, v3dMatrix<float> * target, 
                             long band);

// Converts the given ipl image to a color_image, assuming that the ipl image
// is a 3-channel 8-bits/pixel/band image.
color_image * vIplTocolor_image(IplImage * ipl_image);

// Converts from color_image to IplImage.
IplImage * vvImageToIpl(color_image * the_image);

// Converts the given ipl image to a grayscale_image, assuming that the ipl image
// is a 1-channel 8-bits/pixel/band image
grayscale_image * vIplToGray(IplImage * ipl_image);

// Converts from a grayscale_image to an IplImage
IplImage * vGrayToIpl(grayscale_image * gray);

// Returns the results of Canny edge detection 
grayscale_image * vCannyDetector(grayscale_image * source, uchar low);

grayscale_image * vCannyDetector(grayscale_image * source, uchar low, 
                              uchar high, long aperture);


ushort vSequenceCannyEdges(const char * input_file, const char * output_file,
                            uchar low, uchar high, long aperture);


// Returns the results of distance transform using the Borgefors algorithm
v3dMatrix<float> * vCVDistanceTransform(general_image * input);

inline float_image * function_distance_transform (general_image * input)
{
  return vCVDistanceTransform(input);
}



// Returns the 7 normalized central moments and the 7 Hu moments for 
// the image. Puts the number 14 in the argument "number".
double * vImageMoments(general_image * input, long * number);

// Saves the 14 moments (7 normalized central moments, 7 Hu moments)
// of the given sequence into the given file. Returns a number > 0
// if successful.
short vSequenceImageMoments(const char * sequence_file, 
                             const char * moments_file);


// Saves the 14 moments (7 normalized central moments, 7 Hu moments)
// of the given sequence into the given file. Returns a number > 0
// if successful. The difference with vSequenceImageMoments is that
// here, for every frame, we also rotate it by 90, 180 and 270 degrees,
// and save the moments of those frames, too.
short vSequenceImageMoments_b(const char * sequence_file, 
                               const char * moments_file);



long vConvexHull(vArray(long) rows, vArray(long) cols, int number_of_points, 
                  vArray(long) * hull_rowsp, vArray(long) * hull_colsp, 
                  long * hull_sizep);


long vGoodFeaturesToTrack(grayscale_image * the_image, vector<vPoint2> * locations,
                           double quality, double min_distance);

long vOpticalFlowPyrLK(grayscale_image * the_image1, grayscale_image * the_image2,
                        vector<vPoint2> * locations1, vector<vPoint2> * locations2,
                        long window_size, long pyramids,
                        vector<long> * status, vector<double> * error);

vArray(CvPoint2D32f) vCvPoints2D32f(vector<vPoint2> * points);




#endif // VASSILIS_OPENCV_UTILS_H
