#include "stdafx.h"

#include "opencv_utils.h"
#include "opencv_utils2.h"
#include "basics/video_common.h"
#include "basics/rle_encoding.h"
#include "basics/image_algorithms.h"
#include "basics/drawing_temp.h"
#include "basics/image_templates.h"


// If the OpenCV library has been installed, use the regular 
// definitions of the functions
//
// Note on OpenCV installation: In addition to the OpenCV libraries
// and DLLs, we need the IPL (image processing library) DLLs. I 
// just copy them into the same directory as the OpenCV DLLs 
// (Intel/OpenCV/lib).
#ifdef VASSILIS_OPENCV_LIBRARY


CvStatus vicvFitLine2D(CvPoint2D32f * points, int count, CvDisType dist,
                        void *param, float reps, float aeps, float *line);


long vFitLine2D(vector<double> * rows, vector<double> * cols, 
                 double * row, double * col, double * angle, CvDisType method)
{
  long number = rows->size();
  if (number < 5) return 0;
  vArray(CvPoint2D32f) points = vnew(CvPoint2D32f, number);
  long i;
  for (i = 0; i < number; i++)
  {
    points[i].y = (float) (*rows)[i];
    points[i].x = (float) (*cols)[i];
  }

  float line[4];
  vicvFitLine2D(vData(points), (int) number, method, 0, 0, 0, line);
  vdelete2(points);

  *angle = vAngle2(line[0], line[1]);
  *row = line[3];
  *col = line[2];
  return 1;
}


#else

long vFitLine2D(vector<double> * rows, vector<double> * cols, 
                 double * row, double * col, double * angle, CvDisType method)
{
  vPrint("Can't compute vFitLine2D. OpenCV not installed\n");
  return 0;
}

#endif // VASSILIS_OPENCV_LIBRARY


vEllipse vFitEllipse1(grayscale_image * the_image)
{
  vector<long> rows, cols;
  vGetNonZeroPoints(the_image, &rows, &cols);
  return vFitEllipse2(&rows, &cols);
}



