#ifndef VASSILIS_OPENCV_UTILS2_H
#define VASSILIS_OPENCV_UTILS2_H

#include "local_data.h"

#include "basics/v_types.h"
#include "basics/image.h"
#include "gestures/algorithms.h"
#include "opencv_utils.h"


// If the OpenCV library is installed, then include the following 
// declarations.
#ifdef VASSILIS_OPENCV_LIBRARY

#include <cv.h>

template<class type>
vEllipse vFitEllipse2(vector<type> * rows, vector<type> * cols)
{
  long number = rows->size();
  if (number < 30) return vEllipse();
  vArray(CvPoint2D32f) points = vnew(CvPoint2D32f, number);
  long i;
  for (i = 0; i < number; i++)
  {
    points[i].y = (float) (*rows)[i];
    points[i].x = (float) (*cols)[i];
  }

  CvBox2D32f box;
  cvFitEllipse(vData(points), (int) number, &box);
  vdelete2(points);
  vEllipse result;
  result.center_row = box.center.y;
  result.center_col = box.center.x;
  long width = (long) box.size.width;
  long height = (long) box.size.height;
  if (width < height)
  {
    result.width = box.size.width;
    result.length = box.size.height;
    result.angle = box.angle * M_PI / 180 + M_PI/2;
  }
  else
  {
    result.width = box.size.height;
    result.length = box.size.width;
    result.angle = box.angle * M_PI / 180;
  }

  if (result.angle >= M_PI) result.angle -= M_PI;

  return result;
}


#else  // VASSILIS_OPENCV_LIBRARY


template<class type>
vEllipse vFitEllipse2(vector<type> * rows, vector<type> * cols)
{
  vPrint("Can't compute vFitEllipse2. OpenCV not installed\n");
  return vEllipse();
}



#endif // VASSILIS_OPENCV_LIBRARY


vEllipse vFitEllipse1(grayscale_image * the_image);

long vFitLine2D(vector<double> * rows, vector<double> * cols, 
                 double * row, double * col, double * angle, CvDisType method);



#endif // VASSILIS_OPENCV_UTILS2_H
