#include "stdafx.h"

#include "window.h"


#include "vplatform.h"

#include "basics/definitions.h"

vWindow::vWindow()
{
	assert(0);
}

vWindow::vWindow(color_image * the_image)
{
	assert(0);
}


vWindow::vWindow(color_image * the_image, long rows, long cols)
{
	assert(0);
}


vWindow::vWindow(CWnd * in_window)
{
	window = in_window;
	text = 0;
	parent_dialog = 0;
	CalculateBitsPerPixel();
  no_resize_called_yet = 0;
}


vWindow::vWindow(CWnd * in_window, CString * in_text, CDialog * in_dialog)
{
	window = in_window;
	text = in_text;
	parent_dialog = in_dialog;
	bits_per_pixel = 32;
  no_resize_called_yet = 0;
}



vWindow::~vWindow()
{
}


color_image * vWindow::GetImage()
{
  assert((bits_per_pixel == 32) || (bits_per_pixel == 24));
  long bytes_per_pixel = bits_per_pixel / 8;

	// Calculate size of window.
	RECT rectangle;
	window->GetClientRect(&rectangle);
	long rows = rectangle.bottom;
	long cols = rectangle.right;
	uchar * buffer = new uchar[rows * cols * bytes_per_pixel];

	// Create a memory bitmap and a memory device context.
	CDC * window_dc = window->GetDC();
	CBitmap memory_bitmap;
	BOOL bitmap_success = memory_bitmap.CreateCompatibleBitmap(window_dc, cols, rows);
	assert(bitmap_success);

	CDC memory_dc;
	BOOL memory_dc_success = memory_dc.CreateCompatibleDC(window_dc);
	assert(memory_dc_success);
	CBitmap * old_memory_bitmap = memory_dc.SelectObject(&memory_bitmap);


	BOOL bit_blt_success = memory_dc.BitBlt(0, 0, cols, rows, window_dc, 0, 0, SRCCOPY);
	assert(bit_blt_success);

	// Copy the bitmap into the buffer, and then into the image.
	memory_bitmap.GetBitmapBits(rows*cols*bytes_per_pixel, buffer);

	color_image * the_image = new color_image(rows, cols);
	vArray2(uchar) R = the_image->GetR2();
	vArray2(uchar) G = the_image->GetG2();
	vArray2(uchar) B = the_image->GetB2();

	long row, col;
	long index = 0;

  if (bits_per_pixel == 32)
  {
	  for (row = 0; row < rows; row++)
		  for (col = 0; col < cols; col++)
		  {
			  B[row][col] = buffer[index++];
			  G[row][col] = buffer[index++];
			  R[row][col] = buffer[index++];
			  index++;
		  }
  }
  else if (bits_per_pixel == 24)
  {
	  for (row = 0; row < rows; row++)
		  for (col = 0; col < cols; col++)
		  {
			  B[row][col] = buffer[index++];
			  G[row][col] = buffer[index++];
			  R[row][col] = buffer[index++];
			  //index++;
      }
  }

  else assert(0);
  
	
	delete [] buffer;

	memory_dc.SelectObject(old_memory_bitmap);
	window->ReleaseDC(window_dc);

	return the_image;
}


grayscale_image * vWindow::GetImageGray()
{
  assert((bits_per_pixel == 32) || (bits_per_pixel == 24));
  long bytes_per_pixel = bits_per_pixel / 8;

	// Calculate size of window.
	RECT rectangle;
	window->GetClientRect(&rectangle);
	long rows = rectangle.bottom;
	long cols = rectangle.right;
	uchar * buffer = new uchar[rows * cols * bytes_per_pixel];

	// Create a memory bitmap and a memory device context.
	CDC * window_dc = window->GetDC();
	CBitmap memory_bitmap;
	BOOL bitmap_success = memory_bitmap.CreateCompatibleBitmap(window_dc, cols, rows);
	assert(bitmap_success);

	CDC memory_dc;
	BOOL memory_dc_success = memory_dc.CreateCompatibleDC(window_dc);
	assert(memory_dc_success);
	CBitmap * old_memory_bitmap = memory_dc.SelectObject(&memory_bitmap);


	BOOL bit_blt_success = memory_dc.BitBlt(0, 0, cols, rows, window_dc, 0, 0, SRCCOPY);
	assert(bit_blt_success);

	// Copy the bitmap into the buffer, and then into the image.
	memory_bitmap.GetBitmapBits(rows*cols*bytes_per_pixel, buffer);

	grayscale_image * the_image = new grayscale_image(rows, cols);
	vArray2(uchar) I = the_image->Matrix2(0);

	long row, col;
	long index = 0;
	for (row = 0; row < rows; row++)
		for (col = 0; col < cols; col++)
		{
			I[row][col] = buffer[index];
			index += bytes_per_pixel;
		}

	
	delete [] buffer;

	memory_dc.SelectObject(old_memory_bitmap);
	window->ReleaseDC(window_dc);

	return the_image;
}


void vWindow::CalculateBitsPerPixel()
{
	CDC * window_dc = window->GetDC();
	CBitmap memory_bitmap;
	BOOL bitmap_success = memory_bitmap.CreateCompatibleBitmap(window_dc, 16, 16);
	assert(bitmap_success);
  BITMAP bitmap;
  memory_bitmap.GetBitmap(&bitmap);
  
  if (bitmap.bmBitsPixel == 16)
  {
    bits_per_pixel = bitmap.bmBitsPixel;
  }
  else if (bitmap.bmBitsPixel == 24)
  {
    bits_per_pixel = bitmap.bmBitsPixel;
  }
  else if (bitmap.bmBitsPixel == 32)
  {
    bits_per_pixel = bitmap.bmBitsPixel;
  }
  else
  {
    assert(0);
  }

  window->ReleaseDC(window_dc);
}


long vWindow::BitsPerPixel()
{
	return bits_per_pixel;
}


void vWindow::ShowImage(uchar * bgr_buffer, long top, long left, 
                         long rows, long cols)
{
  // Not implemented yet.
  assert(0);
}


void vWindow::ShowImage(general_image * in_color_image)
{
  ShowImage(in_color_image, 0, 0, 1);
}


void vWindow::ShowImage(general_image * the_image, long top, long left)
{
  ShowImage(the_image, top, left, 1);
}


void vWindow::ShowImage(general_image * in_color_image, long top, long left, long sampling)
{
  if (bits_per_pixel == 16)
  {
    ShowImage16(in_color_image, top, left, 1);
    return;
  }
  else assert((bits_per_pixel == 32) || (bits_per_pixel = 24));
  long bytes_per_pixel = bits_per_pixel / 8;

  if (in_color_image == 0) return;
  color_image * the_image = in_color_image->ToRGBBound();

  long rows, cols;
  rows = the_image->Rows();
  cols = the_image->Cols();
  char * buffer = new char[rows * cols * bytes_per_pixel];

  // Create a memory bitmap and a memory device context.
  CDC * window_dc = window->GetDC();
  CBitmap memory_bitmap;
  BOOL bitmap_success = memory_bitmap.CreateCompatibleBitmap(window_dc, cols, rows);
  assert(bitmap_success);

  CDC memory_dc;
  BOOL memory_dc_success = memory_dc.CreateCompatibleDC(window_dc);
  assert(memory_dc_success);
  CBitmap * old_memory_bitmap = memory_dc.SelectObject(&memory_bitmap);

  // Copy the image into the memory bitmap, and copy
  // that into the window.
  vArray2(uchar) R;
  vArray2(uchar) G;
  vArray2(uchar) B;

  R = the_image->GetR2();
  G = the_image->GetG2();
  B = the_image->GetB2();

  long row, col;
  long index = 0;
  if (bits_per_pixel == 32)
  {
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
        buffer[index++] = B[row][col];
        buffer[index++] = G[row][col];
        buffer[index++] = R[row][col];
        index++;
      }
  }
  else if (bits_per_pixel == 24)
  {
    for (row = 0; row < rows; row++)
      for (col = 0; col < cols; col++)
      {
        buffer[index++] = B[row][col];
        buffer[index++] = G[row][col];
        buffer[index++] = R[row][col];
        //index++;
      }
  }
  else assert(0);
	
  if (the_image != in_color_image) vdelete(the_image);
  memory_bitmap.SetBitmapBits(rows*cols*bytes_per_pixel, buffer);

  BOOL bit_blt_success = window_dc->BitBlt(left, top, cols, rows, &memory_dc, 0, 0, SRCCOPY);
  if (bit_blt_success == 0)
  {
    function_print("\nfailed to display image\n");
  }

  vdelete2(buffer);
  memory_dc.SelectObject(old_memory_bitmap);
  window->ReleaseDC(window_dc);
  //// experimental, from here to end
  //RECT rect;
  //rect.top = top;
  //rect.left = left;
  //rect.bottom = top + rows - 1;
  //rect.right = left + cols - 1;
  //window->InvalidateRect(&rect, FALSE);
}


  
void vWindow::ShowImage16(general_image * in_color_image)
{
  ShowImage16(in_color_image, 0, 0, 1);
}


void vWindow::ShowImage16(general_image * the_image, long top, long left)
{
  ShowImage16(the_image, top, left, 1);
}


void vWindow::ShowImage16(general_image * in_color_image, long top, long left, long sampling)
{
  if (in_color_image == 0) return;
  color_image * the_image = in_color_image->ToRGBBound();

//  AfxMessageBox("Only 16 used.");

  long rows, cols;
  rows = the_image->Rows();
  cols = the_image->Cols();
  ushort * buffer = new ushort[rows * cols];

  // Create a memory bitmap and a memory device context.
  CDC * window_dc = window->GetDC();
  CBitmap memory_bitmap;
  BOOL bitmap_success = memory_bitmap.CreateCompatibleBitmap(window_dc, cols, rows);
  assert(bitmap_success);

  CDC memory_dc;
  BOOL memory_dc_success = memory_dc.CreateCompatibleDC(window_dc);
  assert(memory_dc_success);
  CBitmap * old_memory_bitmap = memory_dc.SelectObject(&memory_bitmap);

  // Copy the image into the memory bitmap, and copy
  // that into the window.
  vArray2(uchar) R;
  vArray2(uchar) G;
  vArray2(uchar) B;
  R = the_image->GetR2();
  G = the_image->GetG2();
  B = the_image->GetB2();

  vArray(uchar) r_base;
  vArray(uchar) g_base;
  vArray(uchar) b_base;

  long row, col;
  long index = 0;
  for (row = 0; row < rows; row++)
  {
    r_base = R[row];
    g_base = G[row];
    b_base = B[row];

    for (col = 0; col < cols; col++)
    {
      ushort value = b_base[col] / 8;
			value += (g_base[col] / 4) << 5;
			value += (r_base[col] / 8) << 11;
      buffer[index++] = value;
    }
	}

  if (the_image != in_color_image) vdelete(the_image);
  memory_bitmap.SetBitmapBits(rows*cols*2, buffer);

  BOOL bit_blt_success = window_dc->BitBlt(top, left, cols, rows, &memory_dc, 0, 0, SRCCOPY);
  assert(bit_blt_success);

  vdelete2(buffer);
  memory_dc.SelectObject(old_memory_bitmap);
  window->ReleaseDC(window_dc);
}


void vWindow::RedrawSubwindows()
{
  RepositionSubwindows(height, width);
}


void vWindow::RepositionSubwindows(long new_height, long new_width)
{
  RECT rectangle1, rectangle2;
  long height_diff = new_height - height;
  CWnd * first_child = window->GetWindow(GW_CHILD);
  if (first_child == 0) return;
  first_child->GetWindowRect(&rectangle1);

  // First we must figure out the origin of the client area of the main
  // window, with respect to the screen.
  first_child->SetWindowPos(0, 0, 0, 0, 0, SWP_NOSIZE | SWP_NOZORDER);
  first_child->GetWindowRect(&rectangle2);
  long client_top = rectangle2.top;
  long client_left = rectangle2.left;
  
  // The following four variables refer to the child's position with respect
  // to the client area of the parent window.
  long new_left, new_top;
  new_left = rectangle1.left - client_left;
  new_top = rectangle1.top - client_top + height_diff;
  first_child->SetWindowPos(0, new_left, new_top, 
                            0, 0, SWP_NOSIZE | SWP_NOZORDER);

  // Now we can do the same thing for all siblings of the first child.
  CWnd * child = first_child->GetNextWindow();
  while(child != 0)
  {
    child->GetWindowRect(&rectangle1);
    new_left = rectangle1.left - client_left;
    new_top = rectangle1.top - client_top + height_diff;
    child->SetWindowPos(0, new_left, new_top, 
                              0, 0, SWP_NOSIZE | SWP_NOZORDER);
    child = child->GetNextWindow();
  }
  
  // this line was used for years
  window->RedrawWindow();

  // this line is experimental
  //window->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW);
//  window->RedrawWindow(NULL, NULL, RDW_INVALIDATE | RDW_UPDATENOW | RDW_ALLCHILDREN | RDW_ERASE);
  //window->Invalidate(FALSE);
  //window->UpdateWindow();
}


void vWindow::GetSize()
{
  RECT rectangle;
  window->GetWindowRect(&rectangle);
  frame_top = rectangle.top;
  frame_left = rectangle.left;
  frame_width = rectangle.right - frame_left;
  frame_height = rectangle.bottom - frame_top;

  window->GetClientRect(&rectangle);
  top = rectangle.top;
  left = rectangle.left;
  long new_width, new_height;
  new_height = rectangle.bottom;
  new_width = rectangle.right;

  // The first time we get to this function, 
  // the buttons will be positioned by the operating
  // system anyway.
  if (no_resize_called_yet == 0)
  {
    no_resize_called_yet = 1;
    height = new_height;
    width = new_width;
    return;
  }

  if (height != new_height)
  {
    RepositionSubwindows(new_height, new_width);
  }
  height = new_height;
  width = new_width;
}


// Resize the window so that the client area has the specified height and width.					
void vWindow::Resize(long in_height, long in_width)
{
  // Get the area of the whole window, and figure out how much the width and height
  // of the whole window should be in order for the client area to have the 
  // desired width and height.
  long width_diff = frame_width - width;
  long height_diff = frame_height - height;
  long new_width = in_width + width_diff;
  long new_height = in_height + height_diff;
  window->SetWindowPos(0, 0, 0, new_width, new_height, SWP_NOZORDER | SWP_NOMOVE | SWP_SHOWWINDOW);
  GetSize();
}


void vWindow::DrawString(long x, long y, char * string)
{
	if (text != 0) 
	{ 
		*text = string;
		parent_dialog->UpdateData(FALSE);
		window->RedrawWindow();
	}
	else 
  {
	  CFrameWnd * the_frame = (CFrameWnd *) window;
	  CView * the_view = the_frame->GetActiveView();

	  CClientDC dc(the_view);
	  dc.SetTextColor(::GetSysColor(COLOR_WINDOWTEXT));

	  RECT rect1, rect2;
	  the_view->GetWindowRect(&rect1);
	  long width = rect1.right - rect1.left + 1;
	  long height = rect1.bottom - rect1.top + 1;
	  y = min(y, height - 1);
	  rect2.left = 0;
	  rect2.right = width - 1;
	  rect2.top = max(0, y - 5);
	  rect2.bottom = min(height - 1, y + 30);

	  the_view->RedrawWindow(&rect2);
	  dc.TextOut(x, y, string);
  }
}


void vWindow::DrawCenteredString(char * string, long background)
{
	if (text != 0) 
	{ 
		*text = string;
		parent_dialog->UpdateData(FALSE);
	}
	else assert(0);
}


void vWindow::DrawLine(long x0, long y0, long x1, long y1, long color)
{
	assert(0);
}


void vWindow::DrawCross(long row, long col, long color)
{
	assert(0);
}

vWindow * vWindow::AddButton(long row, long col, long height, long width,
			       long border_color, long background_color)
{
	assert(0);
	return 0;
}


vWindow * vWindow::AddTextWindow(long row, long col, long height, long width,
				   long border_color, long background_color, 
				   long border)
{
	assert(0);
	return 0;
}


void vWindow::Clear()
{
  ClearArea(0, 0, height, width);
}


void vWindow::ClearArea(long top, long left, long rows, long cols)
{
	CDC * window_dc = window->GetDC();
	window_dc->BitBlt(left, top, cols, rows, 0, 0, 0, BLACKNESS);
	window->ReleaseDC(window_dc);	
}


CWnd * vWindow::GetWindow()
{
	return window;
}
