#ifndef VASSILIS_WINDOW_H
#define VASSILIS_WINDOW_H

#include "vplatform.h"

#ifdef VASSILIS_SGI_PLATFORM
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <X11/Xatom.h>
#endif // VASSILIS_SGI_PLATFORM

#include "basics/image.h"


class vWindow
{
public:
  vWindow();
  vWindow(color_image * the_image);
  vWindow(color_image * the_image, long rows, long cols);
	~vWindow();
  color_image * GetImage();
  grayscale_image * GetImageGray();
  void ShowImage(general_image * the_image);
  void ShowImage(general_image * the_image, long top, long left);
  void ShowImage(general_image * the_image, long top, long left, long sampling);
  void ShowImage(uchar * bgr_buffer, long top, long left, long rows, long cols);

  void GetSize();
  void Resize(long rows, long cols);
  void DrawString(long x, long y, char * string);
  void DrawCenteredString(char * string, long background);
  void DrawLine(long x0, long y0, long x1, long y1, long color);
  void DrawCross(long row, long col, long color = 0xFF);
  vWindow * AddButton(long row, long col, long height, long width,
		       long border_color, long background_color);
  vWindow * AddTextWindow(long row, long col, long height, long width,
			   long border_color, long background_color, 
			   long border);
  void Clear();
  void ClearArea(long top, long left, long rows, long cols);
  void RedrawSubwindows();
  void RepositionSubwindows(long new_height, long new_width);
  
  // UNIX specific:
#ifdef VASSILIS_SGI_PLATFORM
public:
  Window GetWindow();
public: // temporarily
  static Display * display;
  Window window;
#endif // VASSILIS_SGI_PLATFORM

	//Windows specific
#ifdef VASSILIS_WINDOWS
private:
	CWnd * window;
	CString * text;
	CDialog * parent_dialog;
  
	long bits_per_pixel;	
  void CalculateBitsPerPixel();
  long frame_top, frame_left, frame_width, frame_height;
  // The next four variables refer to the client area.
  long top, left, height, width;
  long no_resize_called_yet;

public:
	CWnd * GetWindow();
	vWindow(CWnd * in_window);
	vWindow(CWnd * in_window, CString * in_text, CDialog * dialog);
	void ShowImage16(general_image * the_image);
  void ShowImage16(general_image * the_image, long top, long left);
  void ShowImage16(general_image * the_image, long top, long left, long sampling);
	long BitsPerPixel();
  long Width() 
  {
    return width;
  }
  long Height() 
  {
    return height;
  }

#endif // VASSILIS_WINDOWS_MFC
};





#endif // VASSILIS_WINDOW_H
