/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "stdwin.h"
#include "HighGUI.h"
#include <math.h>
#include <assert.h>
#include <stdlib.h>

IMAGE get_image(IPLIMAGE in)
{
    IMAGE image;
    HDC dc;
    char buffer[sizeof(BITMAPINFOHEADER) + 256*sizeof(RGBQUAD)];
    BITMAPINFO* binfo = (BITMAPINFO*)buffer;
    char* pValues = 0;
    char* iplData = 0;
    int i;
    int iUsage;
    int w, h;
    int step;

    if(!in)
    {
        return 0;
    }
    
    if(in->roi)
    {
        w = in->roi->width;
        h = in->roi->height;
        iplData = in->imageData + in->widthStep*in->roi->yOffset + in->roi->xOffset;
    }
    else
    {
        w = in->width;
        h = in->height;
        iplData = in->imageData;
    }


    dc = CreateCompatibleDC(0);
    binfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    binfo->bmiHeader.biWidth = w;
    binfo->bmiHeader.biHeight = h;
    binfo->bmiHeader.biPlanes = 1;
    binfo->bmiHeader.biCompression = BI_RGB;
    binfo->bmiHeader.biBitCount = 24;
    iUsage = DIB_RGB_COLORS;
        
    image = CreateDIBSection(dc, binfo, iUsage, (void**)&pValues, NULL, 0);


    step = (w*3+3) & -4;
    if(in->origin == IPL_ORIGIN_BL)
    {
        for(i = 0; i < h; i++)
        {
            int j;
            switch(in->nChannels)
            {
            case 1:
                for(j = 0; j < w; j++)
                {
                    pValues[step*i+j*3] = iplData[in->widthStep*i+j];
                    pValues[step*i+j*3+1] = iplData[in->widthStep*i+j];
                    pValues[step*i+j*3+2] = iplData[in->widthStep*i+j];
                }
                break;

            case 3:
                memcpy(pValues + step*i, iplData + in->widthStep*i, w*3);
                break;
            }
            
        }
    }
    else
    {
        for(i = 0; i < h; i++)
        {
            int j;
            switch(in->nChannels)
            {
            case 1:
                for(j = 0; j < w; j++)
                {
                    pValues[step*(h-i-1)+j*3] = iplData[in->widthStep*i+j];
                    pValues[step*(h-i-1)+j*3+1] = iplData[in->widthStep*i+j];
                    pValues[step*(h-i-1)+j*3+2] = iplData[in->widthStep*i+j];
                }
                break;

            case 3:
                memcpy(pValues + step*(h - i - 1), iplData + in->widthStep*i, w*in->nChannels);
                break;
            }
        }
    }
    DeleteDC(dc);
    
    return image;
}

IPLIMAGE get_iplimage(IMAGE in)
{
    char buffer[sizeof(BITMAPINFOHEADER) + 256*sizeof(RGBQUAD)];
    BITMAPINFO* binfo = (BITMAPINFO*)buffer;
    HDC dc;
    char* bits;
    IPLIMAGE image;
    int w, h;
    int error;
    int step;
    int i;
    

    dc = CreateCompatibleDC(NULL);
    binfo->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
    binfo->bmiHeader.biBitCount = 0;
    GetDIBits(dc, (HBITMAP)in, 0, 0, NULL, binfo, DIB_RGB_COLORS);

    assert(binfo->bmiHeader.biHeight > 0);
    w = binfo->bmiHeader.biWidth;
    h = abs(binfo->bmiHeader.biHeight);
    /* binfo->bmiHeader.biHeight *= -1;*/
    step = (w*3+3) & -4;
    bits = (char*)malloc(step*h);
    error = GetDIBits(dc, (HBITMAP)in, 0, h, bits, binfo, DIB_RGB_COLORS);
    DeleteDC(dc);

    image = iplCreateImageHeader(3, 0, IPL_DEPTH_8U, "RGB", "BGR", IPL_DATA_ORDER_PIXEL, 
        IPL_ORIGIN_TL, IPL_ALIGN_DWORD, w, h, NULL, NULL, NULL, NULL);
    iplAllocateImage(image, 0, 0);
    // GetDIBBits returned the positive bitmap height; it should be negative
    for(i = 0; i < h; i++)
    {
        /* We know that the bitmap is top-left in spite of the positive height ... */
        memcpy(image->imageData + image->widthStep*i, bits + i*step, 3*w);
    }

    free(bits);
    return image;
}

int show_iplimage(const char* window_name, IPLIMAGE image)
{
    HWND hwnd = get_hwnd_byname(window_name);
    if(!hwnd)
    {
        return HG_BADNAME;
    }

    IMAGE im = get_image(image);
    if(!im)
    {
        return -1;
    }
    return attach_image(window_name, im);
}

IPLIMAGE load_iplimage(const char* filename)
{
    IMAGE image = load_bitmap(filename);
    IPLIMAGE iplimage;
    if(!image)
    {
        return NULL;
    }
    iplimage = get_iplimage(image);
    destroy_image(image);
    return iplimage;
}

int save_iplimage(const char* filename, IPLIMAGE iplimage)
{
    IMAGE image = get_image(iplimage);
    int ret;
    if(!image)
    {
        return -1;
    }
    ret = save_bitmap(filename, image);
    destroy_image(image);
    return ret;
}
