/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef __CV_H_
#define __CV_H_

#ifndef __CV_DEFINE_USER_LIST__

#if _MSC_VER >= 1200
    #pragma warning( disable: 4711 )
#endif

typedef unsigned char uchar;
typedef unsigned short ushort;

#ifndef IPCVAPI
#define IPCVAPI(type,name,arg)                                \
    CV_EXTERN_C type CV_STDCALL name##_f arg;                 \
    /* function pointer */                                    \
    typedef type (CV_STDCALL* name##_t) arg;                  \
    extern name##_t name;
#endif

#ifndef IPCVAPI_IMPL
#define IPCVAPI_IMPL(type,name,arg)                           \
    /*typedef type (CV_STDCALL* name##_t) arg;*/              \
    CV_EXTERN_C type CV_STDCALL name##_f arg;                 \
    name##_t name = name##_f;                                 \
    CV_EXTERN_C type CV_STDCALL name##_f arg
#endif

#include <ipl.h>

#ifndef _CVTYPES_H_
#include "cvtypes.h"
#endif
#include "cverror.h"


#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

typedef enum {
   cv1u,
   cv8u, cv8s,
   cv16u, cv16s, cv16sc,
   cv32u, cv32s, cv32sc,
   cv32f, cv32fc,
   cv64u, cv64s, cv64sc,
   cv64f, cv64fc
} CvDataType;

#define sizeof_float ((int)sizeof(float))
#define sizeof_short ((int)sizeof(float))

#include "_cvutils.h"

#ifdef __cplusplus
extern "C" {
#endif



#define  CV_ORIGIN_TL  0
#define  CV_ORIGIN_BL  1

#define  _CV_POS_INF       0x7f800000
#define  _CV_NEG_INF       _CV_TOGGLE_FLT(0xff800000)
#define  _CV_TOGGLE_FLT(x) (((x)&0x7fffffff)^(((int)(x))>>31))

#define  _CV_PI   3.1415926535897932384626433832795

/* min & max without jumps */
#define _CV_MIN2(a, b)  ((a) ^ (((a)^(b)) & (((a) < (b)) - 1)))
#define _CV_MAX2(a, b)  ((a) ^ (((a)^(b)) & (((a) > (b)) - 1)))

/* absolute value without jumps */
#define _CV_ABS(a)    (( (a) ^ ((a) >> 31) ) - ((a) >> 31))

#define _CV_SGN(a)    ((((a) >= 0) - 1) | ((a) > 0))

#if _MSC_VER >= 1200
    #pragma warning(disable : 4514) /* unreferenced inline function has been */
                                    /* removed                               */
#endif

#ifndef min
#define min(a,b) (a) > (b) ? (b) : (a)
#endif
#ifndef max
#define max(a,b) (a) < (b) ? (b) : (a)
#endif

typedef struct _CvConvState
{
    char*    buffer;      /* pointer to ring buffer */
    int      BufferStep;  /* BufferStep */
    int      kerType;   /* symmetry of the X kernel */
    int      initialized; /* scale value for result of convolution */
    char*    lines[16];   /* pointers to ring buffer lines */
    char     KerX[256];   /* horizontal kernel */
    char     KerY[256];   /* vertical kernel */
}
_CvConvState;

struct CvMorphState;

typedef struct
{
    uchar  b, g, r;
}
CvRGB8u;


typedef struct
{
    uchar  b, g, r, a;
}
CvRGBA8u;


typedef struct
{
    int  b, g, r;
}
CvRGB32s;


typedef struct
{
    int  b, g, r, a;
}
CvRGBA32s;

#endif __CV_DEFINE_USER_LIST__

#include "_ipcv.h"

/****************************************************************************************/
/*                                  Pyramid segmentation                                */
/****************************************************************************************/

IPCVAPI( CvStatus,  icvUpdatePyrLinks_8u_C1, (
                               int     layer,
                               void*   layer_data,
                               CvSize  size,
                               void*   parent_layer,
                               void*   writer,  
                               float   threshold,
                               int     is_last_iter,
                               void*   stub,
                               void (*writeNode)(void*,void*)))

IPCVAPI( CvStatus,  icvUpdatePyrLinks_8u_C3, (
                               int     layer,
                               void*   layer_data,
                               CvSize  size,
                               void*   parent_layer,
                               void*   writer,  
                               float   threshold,
                               int     is_last_iter,
                               void*   stub,
                               void (*writeNode)(void*,void*)))

/****************************************************************************************/
/*                               Optical Flow primitives                                */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcOpticalFlowBM_8u32fR
//    Purpose: calculate Optical flow for 2 images using block matching algorithm
//    Context:
//    Parameters:
//            imgA,         // pointer to first frame ROI
//            imgB,         // pointer to second frame ROI
//            imgStep,      // full width of input images in bytes
//            imgSize,      // size of the image
//            blockSize,    // size of basic blocks which are compared
//            shiftSize,    // coordinates increments.
//            maxRange,     // size of the scanned neighborhood.
//            usePrevious,  // use previous (input) velocity field.
//            velocityX,    //  pointer to ROI of horizontal and
//            velocityY,    //  vertical components of optical flow
//            velStep);     //  full width of velocity frames in bytes
//    Returns: IPP_NO_ERR or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvCalcOpticalFlowBM_8u32fR,( uchar*   imgA,
                                                    uchar*   imgB,
                                                    int      imgStep,
                                                    CvSize imgSize,
                                                    CvSize blockSize,
                                                    CvSize shiftSize,
                                                    CvSize maxRange,
                                                    int      usePrevious,
                                                    float*   velocityX,
                                                    float*   velocityY,
                                                    int      velStep) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcOpticalFlowLK_8u32fR ( Lucas & Kanade method )
//    Purpose: calculate Optical flow for 2 images using Lucas & Kanade algorithm
//    Context:
//    Parameters:
//            imgA,         // pointer to first frame ROI
//            imgB,         // pointer to second frame ROI
//            imgStep,      // width of single row of source images in bytes
//            imgSize,      // size of the source image ROI
//            winSize,      // size of the averaging window used for grouping
//            velocityX,    // pointer to horizontal and
//            velocityY,    // vertical components of optical flow ROI
//            velStep       // width of single row of velocity frames in bytes
//
//    Returns: IPP_NO_ERR         - all ok
//             IPP_OUTOFMEM_ERR  - insufficient memory for function work
//             IPP_NULLPTR_ERR - if one of input pointers is NULL
//             IPP_BADSIZE_ERR   - wrong input sizes interrelation
//
//    Notes:  1.Optical flow to be computed for every pixel in ROI
//            2.For calculating spatial derivatives we use 3x3 Sobel operator.
//            3.We use the following border mode.
//              The last row or column is replicated for the border
//              ( IPL_BORDER_REPLICATE in IPL ).
//
//F*/
IPCVAPI( CvStatus , icvCalcOpticalFlowLK_8u32fR,( uchar*   imgA,
                                                    uchar*   imgB,
                                                    int      imgStep,
                                                    CvSize imgSize,
                                                    CvSize winSize,
                                                    float*   velocityX,
                                                    float*   velocityY,
                                                    int      velStep  ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcOpticalFlowHS_8u32fR (Horn & Schunck method )
//    Purpose: calculate Optical flow for 2 images using Horn & Schunck algorithm
//    Context:
//    Parameters:
//            imgA,         // pointer to first frame ROI
//            imgB,         // pointer to second frame ROI
//            imgStep,      // width of single row of source images in bytes
//            imgSize,      // size of the source image ROI
//            int usePrevious, // use previous (input) velocity field.
//            velocityX,    // pointer to horizontal and
//            velocityY,    // vertical components of optical flow ROI
//            velStep       // width of single row of velocity frames in bytes
//            float lambda, // Lagrangian multiplier
//            criteria    , // criteria of termination processmaximum number of iterations
//
//    Returns: IPP_NO_ERR         - all ok
//             IPP_OUTOFMEM_ERR  - insufficient memory for function work
//             IPP_NULLPTR_ERR - if one of input pointers is NULL
//             IPP_BADSIZE_ERR   - wrong input sizes interrelation
//
//    Notes:  1.Optical flow to be computed for every pixel in ROI
//            2.For calculating spatial derivatives we use 3x3 Sobel operator.
//            3.We use the following border mode.
//              The first and last rows and columns are replicated for the border
//              ( IPL_BORDER_REPLICATE in IPL ).
//F*/
IPCVAPI( CvStatus , icvCalcOpticalFlowHS_8u32fR,( uchar*  imgA,
                                                  uchar*  imgB,
                                                  int     imgStep,
                                                  CvSize imgSize,
                                                  int     usePrevious,
                                                  float*  velocityX,
                                                  float*  velocityY,
                                                  int     velStep,
                                                  float   lambda,
                                                  CvTermCriteria criteria ) )


/****************************************************************************************/
/*                               Thresholding functions                                 */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvThresh_8uC1R
//    Purpose: Thresholding the source array
//    Context:
//    Parameters:
//      Src     - source array
//      roi     - size of picture in elements
//      srcStep - lenght of string
//      Thresh  - threshold parameter
//      Type    - thresholding type, must be one of
//                  IPPI_THRESH_BINARY       - val = (val > Thresh ? MAX    : 0)
//                  IPPI_THRESH_BINARY_INV   - val = (val > Thresh ? 0      : MAX)
//                  IPPI_THRESH_TRUNC        - val = (val > Thresh ? Thresh : val)
//                  IPPI_THRESH_TOZERO       - val = (val > Thresh ? val    : 0)
//                  IPPI_THRESH_TOZERO_INV   - val = (val > Thresh ? 0      : val)
//    Returns:
//    Notes:
//      The MAX constant for uchar is 255, for char is 127
//F*/
IPCVAPI(CvStatus , icvThresh_8u_C1R, ( const uchar*  src, int  src_step,
                                       uchar*  dst, int  dst_step,
                                       CvSize  roi, int  thresh,
                                       uchar   max_val, CvThreshType type ))

IPCVAPI(CvStatus , icvThresh_8s_C1R, ( const char*   src, int  src_step,
                                       char*   dst, int  dst_step,
                                       CvSize  roi, int  thresh,
                                       char    max_val, CvThreshType type ))

IPCVAPI(CvStatus , icvThresh_32f_C1R,( const float*  src, int    src_step,
                                       float*  dst, int    dst_step,
                                       CvSize  roi, float  thresh,
                                       float   max_val, CvThreshType type))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvAdaptThreshold8uC1R
//    Purpose: Adaptive Thresholding the source array
//    Context:
//    Parameters:
//      Src     - source array
//      roi     - size of picture in elements
//      srcStep - lenght of string
//      type    - thresholding type, must be one of
//                  CV_THRESH_BINARY       - val = (val > Thresh ? MAX    : 0)
//                  CV_THRESH_BINARY_INV   - val = (val > Thresh ? 0      : MAX)
//                  CV_THRESH_TOZERO       - val = (val > Thresh ? val    : 0)
//                  CV_THRESH_TOZERO_INV   - val = (val > Thresh ? 0      : val)
//      size     - size of thresholding neighborhood  ( 1 - 3x3 , 2 - 5x5, 3 - 7x7 )
//    Returns:
//    Notes:
//      The MAX constant for uchar is 255, for char is 127
//F*/
IPCVAPI(CvStatus , icvAdaptiveThreshold8uC1R, ( uchar*  src, int  src_step,
                                                uchar* dst,  int  dst_step,
                                                CvSize roi,  uchar  maxValue,
                                                CvThreshType type,
                                                int size, uchar minDisp))

IPCVAPI(CvStatus , icvFindInitialThreshold,( uchar*  src, int  src_step,
                                             uchar*  dst, int  dst_step,
                                             CvSize  roi, int  size,
                                             int minDisp, int* thresh ))

/****************************************************************************************/
/*                                  Detection functions                                 */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:     icvPreCornerDetect**C1R
//    Purpose:  Calculating constraint image for corner detection
//             Dx^2 * Dyy + Dxx * Dy^2 - 2 * Dx * Dy * Dxy
//    Context:
//    Parameters:
//              pSrc       - pointer to the source image
//              srcStep    - width of the full Src image in bytes
//              corner     - pointer to the image to be filled using expr. (1)
//              cornerStep - it's step in bytes
//              roi        - roi size in pixels
//              apertureSize     - Sobel operator aperture size - 1
//    Returns:
//              IPP_NO_ERR if all ok or error code
//F*/
IPCVAPI(CvStatus ,icvPreCornerDetect8uC1R,( uchar*   src,    int  srcStep,
                                              float*   corner, int  cornerStep,
                                              CvSize roi,    int  apertureSize ))

IPCVAPI(CvStatus ,icvPreCornerDetect8sC1R,( char*    src,    int  srcStep,
                                              float*   corner, int  cornerStep,
                                              CvSize roi,    int  apertureSize ))

IPCVAPI(CvStatus ,icvPreCornerDetect32fC1R,( float*   src,    int  srcStep,
                                               float*   corner, int  cornerStep,
                                               CvSize roi,    int  apertureSize ))



/****************************************************************************************\
*                                     Line sampling                                      *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name[s]:    icvGetRectSubPix_8u32f_C1R,
//                icvGetRectSubPix_8s32f_C1R,
//                icvGetRectSubPix_32f_C1R
//    Purpose:
//      Extracts rectangle from the byte-depth image with sub-pixel resolution
//      (it is done with bilinear interpolation) and writes the rectangle into
//      floating point buffer.
//    Context:
//    Parameters:
//      src      - source image
//      src_step - full width of source image row in bytes
//      src_size - width and height of copied ROI in pixels
//      dst      - destination image ROI, which is able to fit resultant rectangle 
//                 (win_size.width*2 + 1) by (win_size.height*2 + 1) pixels.
//      dst_step - full width of destination image row in bytes
//      win_size - size of destination buffer.
//                 the following rectangle is extracted:
//                     (center.x - win_size.x/2 .. center.x + win_size.x/2,
//                      center.y - win_size.y/2 .. center.y + win_size.y/2)
//                 width and height must be odd (non-even).
//      center   - center of rectangle
//    Returns:
//    Notes:
//      If the extracted rectangle is partially outside the image
//      then the missing pixels are taken from corresponding boundary pixels
//      (that is first and last row and first and last column are propagated (virtually)
//      beyond the image.
//F*/
IPCVAPI( CvStatus, icvGetRectSubPix_8u32f_C1R, ( const uchar* src, int src_step,
                                                 CvSize src_size,
                                                 float* dst, int dst_step,
                                                 CvSize win_size, CvPoint2D32f center ))

IPCVAPI( CvStatus, icvGetRectSubPix_8s32f_C1R, ( const char* src, int src_step,
                                                 CvSize src_size,
                                                 float* dst, int dst_step,
                                                 CvSize win_size, CvPoint2D32f center ))

IPCVAPI( CvStatus, icvGetRectSubPix_32f_C1R, ( const float* src, int src_step,
                                               CvSize src_size,
                                               float* dst, int dst_step,
                                               CvSize win_size, CvPoint2D32f center ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name[s]:    icvGetQuadrangleSubPix_8u32f_C1R,
//                icvGetQuadrangleSubPix_8s32f_C1R,
//                icvGetQuadrangleSubPix_32f_C1R
//    Purpose:
//      Extracts rectangle from the byte-depth image with sub-pixel resolution
//      (it is done with bilinear interpolation) and writes the rectangle into
//      floating point buffer.
//    Context:
//    Parameters:
//      src      - source image
//      src_step - full width of source image row in bytes
//      src_size - width and height of copied ROI in pixels
//      dst      - destination image ROI, which is able to fit resultant rectangle 
//                 (win_size.width*2 + 1) by (win_size.height*2 + 1) pixels.
//      dst_step - full width of destination image row in bytes
//      win_size - size of destination buffer.
//                 the quadrange with center point <center> is extracted from the source image:
//                 width and height must be odd (non-even).
//      center   - center of rectangle
//      A        - affine transform matrix.
//    Returns:
//    Notes:
//      If the extracted quadrangle is partially outside the image
//      then the missing pixels are taken from corresponding boundary pixels
//      (that is first and last row and first and last column are propagated (virtually)
//      beyond the image.
//F*/
IPCVAPI( CvStatus, icvGetQuadrangleSubPix_8u32f_C1R, (
                         const uchar* src, int src_step, CvSize src_size,
                         float* dst, int dst_step, CvSize win_size,
                         CvPoint2D32f center, const float* A ))

IPCVAPI( CvStatus, icvGetQuadrangleSubPix_8s32f_C1R, (
                         const char* src, int src_step, CvSize src_size,
                         float* dst, int dst_step, CvSize win_size,
                         CvPoint2D32f center, const float* A ))

IPCVAPI( CvStatus, icvGetQuadrangleSubPix_32f_C1R, (
                         const float* src, int src_step, CvSize src_size,
                         float* dst, int dst_step, CvSize win_size,
                         CvPoint2D32f center, const float* A ))


/****************************************************************************************/
/*                            Snakes                                                    */
/****************************************************************************************/
IPCVAPI( CvStatus, icvSnakeImage8uC1R, ( uchar*    src,  int  srcStep,
                                       CvSize  roi,  CvPoint* points,
                                       int  length, float* alpha,
                                       float* beta, float* gamma,
                                       CvCoeffType coeffUsage, CvSize  win,
                                       CvTermCriteria criteria ))

IPCVAPI( CvStatus, icvSnakeImageGrad8uC1R,( uchar*    src, int  srcStep,
                                          CvSize  roi, CvPoint* points,
                                          int  length,  float* alpha,
                                          float* beta,  float* gamma,
                                          CvCoeffType coeffUsage, CvSize win,
                                          CvTermCriteria criteria ))

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Names: icvUnDistort1_8uC1R, icvUnDistort1_8uC3R
//    Purpose: The functions correct radial image distortion using known matrix of the
//             camera intrinsic parameters and distortion coefficients
//    Context:
//    Parameters:  srcImage    - source (distorted) image
//                 dstImage    - output (undistorted) image
//                 step        - full width of each image in bytes
//                 size        - ROI size of each image
//                 intrMatrix  - matrix of the camera intrinsic parameters
//                 distCoeffs  - vector of two distortion coefficients (k1 and k2)
//                 interToggle - interpolation toggle
//
//    Returns: CV_NO_ERR or error code
//
//    Notes:   If interToggle=0, interpolation disabled;
//             else bilinear interpolation is used.
//F*/
IPCVAPI( CvStatus, icvUnDistort1_8u_C1R, ( uchar* srcImage, uchar* dstImage, int step,
                                           CvSize size, float* intrMatrix,
                                           float* distCoeffs, int interToggle ) )

IPCVAPI( CvStatus, icvUnDistort1_8u_C3R, ( uchar* srcImage, uchar* dstImage, int step,
                                           CvSize size, float* intrMatrix,
                                           float* distCoeffs, int interToggle ) )

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Names: icvUnDistortEx_8uC1R, icvUnDistortEx_8uC3R
//    Purpose: The functions correct radial and tangential image distortions using known
//             matrix of the camera intrinsic parameters and distortion coefficients
//    Context:
//    Parameters:  srcImage    - source (distorted) image
//                 dstImage    - output (undistorted) image
//                 step        - full width of each image in bytes
//                 size        - ROI size of each image
//                 intrMatrix  - matrix of the camera intrinsic parameters
//                 distCoeffs  - vector of the distortion coefficients
//                               (k1, k2, p1, p2)
//                 interToggle - interpolation toggle
//
//    Returns: CV_NO_ERR or error code
//
//    Notes:   If interToggle=0, interpolation disabled;
//             else bilinear interpolation is used.
//F*/
IPCVAPI( CvStatus, icvUnDistortEx_8u_C1R, ( uchar* srcImage, uchar* dstImage, int step,
                                            CvSize size, float* intrMatrix,
                                            float* distCoeffs, int interToggle ) )

IPCVAPI( CvStatus, icvUnDistortEx_8u_C3R, ( uchar* srcImage, uchar* dstImage, int step,
                                            CvSize size, float* intrMatrix,
                                            float* distCoeffs, int interToggle ) )

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Names: icvUnDistortInit_8uC1R, icvUnDistortInit_8uC3R, 
//    Purpose: The functions calculate arrays of distorted points indices and
//             interpolation coefficients for cvUnDistort function
//    Context:
//    Parameters:  size        - size of each image or of its ROI
//                 step        - full width of each image in bytes
//                 intrMatrix  - matrix of the camera intrinsic parameters
//                 distCoeffs  - vector of the distortion coefficients (k1, k2, p1 and p2)
//                 interToggle - interpolation toggle
//                 data        - distortion data array
//
//    Returns: CV_NO_ERR or error code
//
//    Notes:  1. If interToggle=0, interpolation disabled;
//               else bilinear interpolation is used.
//            2. Array data must be allocated before. If interToggle = 0, its length
//               must be size.width*size.height elements; else 3*size.width*size.height.
//F*/
IPCVAPI( CvStatus, icvUnDistortInit_8u_C1R, ( CvSize size, int step, float* intrMatrix,
                                              float*  distCoeffs, int interToggle,
                                              int* data ) )

IPCVAPI( CvStatus, icvUnDistortInit_8u_C3R, ( CvSize size, int step, float* intrMatrix,
                                              float*  distCoeffs, int interToggle,
                                              int* data ) )

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Names: icvUnDistort_8u_C1R, icvUnDistort_8u_C3R
//    Purpose: The functions correct radial and tangential distortion in the frame
//             using previousely calculated arrays of distorted points indices and
//             undistortion coefficients
//    Context:
//    Parameters:  srcImage    - source (distorted) image
//                 dstImage    - output (undistorted) image
//                 step        - full width of each image in bytes
//                 size        - size of each image or of its ROI
//                 interToggle - interpolation toggle
//                 data        - distortion data array
//
//    Returns: CV_NO_ERR or error code
//
//    Notes:  1. Either icvUnDistortInit_8u_C1R or icvUnDistortInit_8u_C3R function
//               must be used previously
//            2. See Notes to the icvUnDistortInit_8u_C1R, icvUnDistortInit_8u_C3R
//               functions
//F*/
IPCVAPI( CvStatus, icvUnDistort_8u_C1R, ( uchar* srcImage, uchar* dstImage,
                                          int step, CvSize size, int interToggle,
                                          int* data ) )

IPCVAPI( CvStatus, icvUnDistort_8u_C3R, ( uchar* srcImage, uchar* dstImage,
                                          int step, CvSize size, int interToggle,
                                          int* data ) )


/****************************************************************************************/
/*                                Hough Transform                                       */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvHoughLines_8uC1R
//    Purpose:
//      Function detects lines on a binary raster image
//    Context:
//    Parameters:
//      image       - image data
//      step        - distance between image lines in bytes
//      size        - width and height of the image in pixels
//      rho         - resolution in rho (the minimum difference between two values)
//      theta       - resolution in theta (the minimum difference between two values)
//      threshold   - the pixels number which is enough to plot a line through
//      lines       - output parameters of a line
//                      i line is rho = lines[2*i], theta = lines[2*i + 1]
//      linesNumber - 2*linesNumber is the size of the lines buffer
//    Returns:
//      IPP_NO_ERR if all Ok or error code
//    Notes:
//    the Standard Hough Transform is used in the function
//F*/
IPCVAPI( int, icvHoughLines_8uC1R,( uchar* image, int   step,  CvSize size, 
                                    float  rho,   float theta, int threshold, 
                                    float* lines, int   linesNumber))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvHoughLinesSDiv_8uC1R
//    Purpose:
//      Function detects lines on a binary raster image
//    Context:
//    Parameters:
//      image       - image data
//      step        - distance between image lines in bytes
//      size        - width and height of the image in pixels
//      rho         - rough resolution in rho (the minimum difference between two values)
//      srn         - the scale factor of a rough rho resolution to a high one
//      theta       - rough resolution in theta (the minimum difference between two values)
//      stn         - the scale factor of a rough theta resolution to a high one
//      threshold   - the pixels number which is enough to plot a line through
//      lines       - output parameters of a line
//                      i line is rho = lines[2*i], theta = lines[2*i + 1]
//      linesNumber - 2*linesNumber is the size of the lines buffer
//    Returns:
//      the number of lines found
//    Notes:
//    the Standard Hough Transform is used in the function
//F*/
IPCVAPI( int, icvHoughLinesSDiv_8uC1R,( uchar* image_src, int step, CvSize size, 
                                       float rho, int srn, float theta, int stn,
                                       int threshold, float* lines, int lines_number ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvHoughLinesP_8uC1R
//    Purpose:
//      Function detects lines on a binary raster image
//    Context:
//    Parameters:
//      image       - image data
//      step        - distance between image lines in bytes
//      size        - width and height of the image in pixels
//      rho         - resolution in rho (the minimum difference between two values)
//      theta       - resolution in theta (the minimum difference between two values)
//      threshold   - the pixels number which is enough to plot a line through
//      lineLength  - the minimum accepted length of lines
//      lineGap     - the maximum accepted gap in a line (in pixels)
//      lines       - output parameters of a line
//                      the i line starts in x1 = lines[4*i], y1 = lines[4*i + 1] and
//                      finishes in x2 = lines[4*i + 2], y2 = lines[4*i + 3]
//      linesNumber - 4*linesNumber is the size of lines buffer
//      linesToFind - the maximum number of lines to detect
//    Returns:
//      IPP_NO_ERR if all Ok or error code
//    Notes:
//    The Progressive Probabilistic Hough Transform is implemented in the function. It 
//      searches for linesToFind number of lines, taking only those that contain more than
//      lineLength pixels and return. Effectively detects long lines on an image with 
//      strong noise.
//F*/
IPCVAPI( int, icvHoughLinesP_8uC1R,( uchar* image,   int   step,  CvSize size, 
                                     float  rho,     float theta, int threshold, 
                                     int lineLength, int lineGap, int*  lines, int linesNumber ))

/****************************************************************************************/
/*                                Image Statistics                                      */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCountNonZero
//    Purpose:  Counts non-zero pixels in the image
//    Context:
//    Parameters:  
//      img  - source image
//      step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//    Returns:
//      number of non-zero pixels or negative value (if error)
//    Notes:
//F*/
IPCVAPI( CvStatus, icvCountNonZero_8u_C1R, ( const uchar* img, int step,
                                             CvSize size, int *_count ))
IPCVAPI( CvStatus, icvCountNonZero_8u_C3CR, ( const uchar* img, int step,
                                              CvSize size, int coi, int *_count ))

IPCVAPI( CvStatus, icvCountNonZero_32f_C1R, ( const float* img, int step,
                                              CvSize size, int *_count ))
IPCVAPI( CvStatus, icvCountNonZero_32f_C3CR, ( const float* img, int step,
                                               CvSize size, int coi, int *_count ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: cvSumPixels
//    Purpose: Calculates sum of all pixels
//    Context:
//    Parameters:
//      img  - source image
//      step - its step
//      size - its size
//      coi  - channel index (1..3) (if color image)
//    Returns:
//      sum of pixels values or large negative number (if error)
//    Notes:
//F*/
IPCVAPI( CvStatus, icvSumPixels_8u_C1R, ( const uchar* img, int step,
                                          CvSize size, double* sum ))
IPCVAPI( CvStatus, icvSumPixels_8u_C3CR, ( const uchar* img, int step,
                                           CvSize size, int coi, double* sum ))
IPCVAPI( CvStatus, icvSumPixels_8s_C1R, ( const char* img, int step,
                                          CvSize size, double* sum ))
IPCVAPI( CvStatus, icvSumPixels_8s_C3CR, ( const char* img, int step,
                                           CvSize size, int coi, double* sum ))
IPCVAPI( CvStatus, icvSumPixels_32f_C1R, ( const float* img, int step,
                                           CvSize size, double* sum ))
IPCVAPI( CvStatus, icvSumPixels_32f_C3CR, ( const float* img, int step,
                                            CvSize size, int coi, double* sum ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMomentsGray
//    Purpose:  Find all neccessary values for calculating spatial/central/normalized
//              moments up to third order
//    Context:
//    Parameters:
//      img  - source image
//      step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//      moments - structure containing resultant values.
//                The structure is input for icvGetSpatialMoment, 
//                icvGetCentralMoment and icvGetNormalizedCentralMoment functions
//    Returns:
//      IPP_NO_ERR if success or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMomentsGray_8u_C1R, ( const uchar* img, int step, CvSize size, 
                                             CvMoments* moments))
IPCVAPI( CvStatus , icvMomentsGray_8u_C3CR, ( const uchar* img, int step, CvSize size, 
                                              int coi, CvMoments* moments ))
IPCVAPI( CvStatus , icvMomentsGray_8s_C1R, ( const char* img, int step, CvSize size, 
                                             CvMoments* moments))
IPCVAPI( CvStatus , icvMomentsGray_8s_C3CR, ( const char* img, int step, CvSize size, 
                                              int coi, CvMoments* moments ))
IPCVAPI( CvStatus , icvMomentsGray_32f_C1R, ( const float* img, int step, CvSize size, 
                                              CvMoments* moments))
IPCVAPI( CvStatus , icvMomentsGray_32f_C3CR, ( const float* img, int step, CvSize size, 
                                               int coi, CvMoments* moments ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMomentsBinary
//    Purpose:  Find all neccessary values for calculating spatial/central/normalized
//              moments up to third order. Input image is considered as binary image
//              that is each non-zero pixel is treated as "1" and 
//              each zero pixel is treated as "0".
//    Context:
//    Parameters:
//      img  - source image
//      step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//      moments - structure containing resultant values.
//                The structure is input for icvGetSpatialMoment, 
//                icvGetCentralMoment and icvGetNormalizedCentralMoment functions
//    Returns:
//      IPP_NO_ERR if success or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMomentsBinary_8u_C1R, ( const uchar* img, int step, CvSize size,
                                               CvMoments* moments ))
IPCVAPI( CvStatus , icvMomentsBinary_8u_C3CR, ( const uchar* img, int step, CvSize size,
                                                int coi, CvMoments* moments ))
IPCVAPI( CvStatus , icvMomentsBinary_32f_C1R, ( const float* img, int step, CvSize size,
                                                CvMoments* moments ))
IPCVAPI( CvStatus , icvMomentsBinary_32f_C3CR, ( const float* img, int step, CvSize size,
                                                 int coi, CvMoments* moments ))

/****************************************************************************************/
/*                                Norms                                                 */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvNorm
//    Purpose: Calculates norm, difference norm and relative difference norm of images
//    Context:
//    Parameters:
//      imgA,  imgB  - source images,
//      stepA, stepB - their steps,
//      size or roi  - their size or ROI size,
//      coi          - channel index (1..3) (if color image)
//      normType     - norm type (see notes).
//    Returns:
//      value of norm or large negative number (if error)
//    Notes:
//      If imgB is NULL, norm of the image A is calculated, and norm type may be
//      IPPI_C, IPPI_L1 or IPPI_L2. If imgB is not NULL, difference norm or relative
//      difference norm is calculated. In this event norm type may be also
//      IPPI_RELATIVEC, IPPI_RELATIVEL1 and IPPI_RELATIVEL2.
//F*/
IPCVAPI( CvStatus, icvNorm_8u_C1R, ( const uchar* imgA, int stepA,
                                     const uchar* imgB, int stepB,
                                     CvSize roi,  int normType, double* norm ))

IPCVAPI( CvStatus, icvNorm_8s_C1R, ( const char*  imgA, int stepA,
                                     const char*  imgB, int stepB,
                                     CvSize roi,  int normType, double* norm ))

IPCVAPI( CvStatus, icvNorm_32f_C1R,( const float* imgA, int stepA,
                                     const float* imgB, int stepB,
                                     CvSize roi,  int normType, double* norm ))

IPCVAPI( CvStatus, icvNorm_8u_C3CR, ( const uchar* imgA, int stepA,
                                      const uchar* imgB, int stepB,
                                      CvSize roi,  int coi, int normType, double* norm ))

IPCVAPI( CvStatus, icvNorm_8s_C3CR, ( const char*  imgA, int stepA,
                                      const char*  imgB, int stepB,
                                      CvSize roi,  int coi, int normType, double* norm ))

IPCVAPI( CvStatus, icvNorm_32f_C3CR,( const float* imgA, int stepA,
                                      const float* imgB, int stepB,
                                      CvSize roi,  int coi, int normType, double* norm ))


/****************************************************************************************/
/*                                Additional MinMaxLoc functions                        */
/****************************************************************************************/

IPCVAPI(CvStatus, icvMinMaxIndx_16s_C1R, ( const short* img, int step, CvSize size,
                                           float* minVal, float* maxVal,
                                           CvPoint* minLoc, CvPoint* maxLoc ))

IPCVAPI(CvStatus, icvMinMaxIndx_16s_C3CR, ( const short* img, int step,
                                            CvSize size, int coi,
                                            float* minVal, float* maxVal,
                                            CvPoint* minLoc, CvPoint* maxLoc ))

/****************************************************************************************/
/*                                     Utilities                                        */
/****************************************************************************************/

/****************************************************************************************/
/*                            Math routines and RNGs                                    */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbInvSqrt_32f
//    Purpose: block version of icvInvSqrt32f
//    Context:
//    Parameters:
//      src - source vector
//      dst - destination vector
//      len - vectors length
//    Returns:
//      IPP_NO_ERR or IPP_BADFACTOR_ERR if some pointers are NULL or len < 0
//    Notes:
//F*/
IPCVAPI( CvStatus , icvbInvSqrt_32f, ( const float* src, float* dst, int len ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbSqrt_32f
//    Purpose: block version of CvSqrt32f
//    Context:
//    Parameters:
//      src - source vector
//      dst - destination vector
//      len - vectors length
//    Returns:
//      IPP_NO_ERR or IPP_BADFACTOR_ERR if some pointers are NULL or len < 0
//    Notes:
//F*/
IPCVAPI( CvStatus , icvbSqrt_32f, ( const float* src, float* dst, int len ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbRand32f
//    Purpose: block version of the previous function
//    Context:
//    Parameters:
//      state - state of random generator
//      dst   - destination vector
//      len   - vector length
//    Returns:
//      IPP_NO_ERR or error code if state or dst pointer are NULL or len < 0
//    Notes:
//F*/
IPCVAPI( CvStatus , icvbRand32f, ( CvRandState *state, float* dst, int len ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbCartToPolar32f
//    Purpose: fast inaccurate arctangent approximation (processes whole vector)
//    Context:
//    Parameters:
//      x       - abscissas vector
//      y       - ordinates vector
//      mag     - magnitudes vector
//      angle   - angles vector
//      length  - vectors length
//    Returns:
//      IPP_NO_ERR or IPP_BADFACTOR_ERR if x or y or angle pointers are NULL or
//      len is negative.
//    Notes:
//      if x[i] == 0 && y[i] == 0 for some i -> angle[i] is 0 too.
//      Current version accurancy: ~0.3 degrees
//F*/
IPCVAPI( CvStatus , icvbCartToPolar32f, ( const float* x, const float* y, float* mag,
                                          float* phase, int length ))



/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbLog_64f32f
//    Purpose: fast logarithm approximation (processes whole vector)
//    Context:
//    Parameters:
//      x       - input (single-precision) vector
//      y       - output (double-precision) vector
//      n       - number of elements
//    Returns:
//      CV_NO_ERR or error code
//    Notes:
//      Accuracy: 2e-7 - quadratic interpolation
//                1e-5 - linear interpolation
//      Special values and overflow are not handled.
//F*/
IPCVAPI( CvStatus, icvbLog_64f32f, ( const double *x, float *y, int n ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbExp_32f64f
//    Purpose: fast exponent approximation (processes whole vector)
//    Context:
//    Parameters:
//      x       - input (single-precision) vector
//      y       - output (double-precision) vector
//      n       - number of elements
//    Returns:
//      CV_OK or error code
//    Notes:
//      Accuracy: 6e-9 - quadratic interpolation
//                7e-6 - linear interpolation
//      Special values and overflow are not handled.
//F*/
IPCVAPI( CvStatus, icvbExp_32f64f, ( const float *x, double *y, int n ) )


/****************************************************************************************/
/*                                Conversion functions                                  */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCvt_XY_C1R
//    Purpose:
//      Converts image from one format to another with linear
//      transformation of pixel values
//    Context:
//    Parameters:
//      src        - source image ROI
//      src_step   - full width of the source image (in bytes)
//      dst        - destination image ROI
//      dst_step   - full width of the destination image (in bytes)
//      size       - width and height of ROI in pixels
//      scale, shift - linear transform parameters
//                     dst(x,y) = src(x,y)*scale + shift
//    Returns:
//      Nothing       
//    Notes:
F*/
IPCVAPI( CvStatus, icvCvt_8u32f_C1R, ( const uchar* src, int src_step,
                                       float* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_32f8u_C1R, ( const float* src, int src_step,
                                       uchar* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_8s32f_C1R, ( const char* src, int src_step,
                                       float* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_32f8s_C1R, ( const float* src, int src_step,
                                       char* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_16s32f_C1R, ( const short* src, int src_step,
                                        float* dst, int dst_step,
                                        CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_32f16s_C1R, ( const float* src, int src_step,
                                       short* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_32s32f_C1R, ( const int* src, int src_step,
                                        float* dst, int dst_step,
                                        CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_32f32s_C1R, ( const float* src, int src_step,
                                        int* dst, int dst_step,
                                        CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_8u16s_C1R, ( const uchar* src, int src_step,
                                       short* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_16s8u_C1R, ( const short* src, int src_step,
                                       uchar* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_8s16s_C1R, ( const char* src, int src_step,
                                       short* dst, int dst_step,
                                       CvSize size, double scale, double shift ))

IPCVAPI( CvStatus, icvCvt_16s8s_C1R, ( const short* src, int src_step,
                                       char* dst, int dst_step,
                                       CvSize size, double scale, double shift ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvRGBToGray_8uC3C1R
//    Purpose:
//      Converts RGB image into grayscale format
//    Context:
//    Parameters:
//      rgb_img    - source image
//      rgb_step   - full width of the source image (in bytes)
//      gray_img   - grayscale image
//      gray_step  - full width of the grayscale image (in bytes)
//      size       - width and height of the images in pixels
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
F*/
IPCVAPI( CvStatus , icvRGBToGray_8uC3C1R, ( uchar* rgb_img, int rgb_step,
                                            uchar* gray_img, int gray_step,
                                            CvSize size ))

/****************************************************************************************\
*                                      Eigen objects                                     *
\****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcCovarMatrixEx_8u32fR
//    Purpose: The function calculates a covariance matrix for a group of input objects
//             (images, vectors, etc.). ROI supported.
//    Context:
//    Parameters:  nObjects    - number of source objects
//                 objects     - array of pointers to ROIs of the source objects
//                 objStep     - full width of each source object row in bytes
//                 avg         - pointer to averaged object
//                 avgStep     - full width of averaged object row in bytes
//                 size        - ROI size of each source and averaged objects
//                 covarMatrix - covariance matrix (output parameter; must be allocated
//                               before call)
//
//    Returns: IPP_NO_ERR or error code
//
//    Notes:   
//F*/
IPCVAPI( CvStatus, icvCalcCovarMatrixEx_8u32fR, ( int nObjects, void* input, int objStep,
                                                 int ioFlags, int ioBufSize,
                                                 uchar* buffer, void* userData,
                                                 float* avg, int avgStep, CvSize size,
                                                 float* covarMatrix ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcEigenObjects_8u32fR
//    Purpose: The function calculates an orthonormal eigen basis and a mean (averaged)
//             object for a group of input objects (images, vectors, etc.). ROI supported.
//    Context:
//    Parameters: nObjects  - number of source objects
//                input     - pointer either to array of pointers to input objects
//                            or to read callback function (depending on ioFlags)
//                imgStep   - full width of each source object row in bytes
//                output    - pointer either to array of pointers to output eigen objects
//                            or to write callback function (depending on ioFlags)
//                eigStep   - full width of each eigenobject row in bytes
//                size      - ROI size of each source object
//                ioFlags   - input/output flags (see Notes)
//                ioBufSize - input/output buffer size
//                userData  - pointer to the structure which contains all necessary
//                            data for the callback functions
//                calcLimit - determines the calculation finish conditions
//                avg       - pointer to averaged object (has the same size as ROI)
//                avgStep   - full width of averaged object row in bytes
//                eigVals   - pointer to corresponding eigenvalues (array of <nObjects>
//                            elements in descending order)
//
//    Returns: CV_NO_ERR or error code
//
//    Notes: 1. input/output data (that is, input objects and eigen ones) may either
//              be allocated in the RAM or be read from/written to the HDD (or any
//              other device) by read/write callback functions. It depends on the
//              value of ioFlags paramater, which may be the following:
//                  CV_EIGOBJ_NO_CALLBACK, or 0;
//                  CV_EIGOBJ_INPUT_CALLBACK;
//                  CV_EIGOBJ_OUTPUT_CALLBACK;
//                  CV_EIGOBJ_BOTH_CALLBACK, or
//                            CV_EIGOBJ_INPUT_CALLBACK | CV_EIGOBJ_OUTPUT_CALLBACK.
//              The callback functions as well as the user data structure must be
//              developed by the user.
//
//           2. If ioBufSize = 0, or it's too large, the function dermines buffer size
//              itself.
//
//           3. Depending on calcLimit parameter, calculations are finished either if
//              eigenfaces number comes up to certain value or the relation of the
//              current eigenvalue and the largest one comes down to certain value
//              (or any of the above conditions takes place). The calcLimit->type value
//              must be CV_TERMCRIT_NUMB, CV_TERMCRIT_EPS or
//              CV_TERMCRIT_NUMB | CV_TERMCRIT_EPS. The function returns the real
//              values calcLimit->maxIter and calcLimit->epsilon.
//
//           4. eigVals may be equal to NULL (if you don't need eigen values in further).
//
//F*/
IPCVAPI( CvStatus, icvCalcEigenObjects_8u32fR, ( int nObjects, void* input, int objStep,
                                                   void* output, int eigStep,
                                                   CvSize size, int ioFlags,
                                                   int ioBufSize, void* userData,
                                                   CvTermCriteria* calcLimit,
                                                   float* avg, int avgStep,
                                                   float* eigVals ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcDecompCoeff_8u32fR
//    Purpose: The function calculates one decomposition coefficient of input object
//             using previously calculated eigen object and the mean (averaged) object
//    Context:
//    Parameters:  obj     - input object
//                 objStep - its step (in bytes)
//                 eigObj  - pointer to eigen object
//                 eigStep - its step (in bytes)
//                 avg     - pointer to averaged object
//                 avgStep - its step (in bytes)
//                 size    - ROI size of each source object
//
//    Returns: decomposition coefficient value or large negative value (if error)
//F*/
IPCVAPI(float, icvCalcDecompCoeff_8u32fR, ( uchar* obj,    int objStep,
                                            float* eigObj, int eigStep,
                                            float* avg,    int avgStep, CvSize size ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvEigenDecomposite_8u32fR
//    Purpose: The function calculates all decomposition coefficients for input object
//             using previously calculated eigen objects basis and the mean (averaged)
//             object
//    Context:
//    Parameters:  obj         - input object
//                 objStep     - its step (in bytes)
//                 nEigObjs    - number of eigen objects
//                 eigInput - pointer either to array of pointers to eigen objects
//                               or to read callback function (depending on ioFlags)
//                 eigStep     - eigen objects step (in bytes)
//                 ioFlags     - input/output flags
//                 iserData    - pointer to the structure which contains all necessary
//                               data for the callback function
//                 avg         - pointer to averaged object
//                 avgStep     - its step (in bytes)
//                 size        - ROI size of each source object
//                 coeffs      - calculated coefficients (output data)
//
//    Returns: icv status
//
//    Notes:   see notes for icvCalcEigenObjects_8u32fR function
//F*/
IPCVAPI(CvStatus , icvEigenDecomposite_8u32fR, ( uchar* obj, int objStep, int nEigObjs,
                                                   void* eigInput, int eigStep,
                                                   int ioFlags, void* userData,
                                                   float* avg, int avgStep,
                                                   CvSize size, float* coeffs ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvEigenProjection_8u32fR, icvEigenProjection_8u32f
//    Purpose: The function calculates object projection to the eigen sub-space (restores
//             an object) using previously calculated eigen objects basis, mean (averaged)
//             object and decomposition coefficients of the restored object
//    Context:
//    Parameters:  neigens - number of eigen objects
//                 eigens  - array of pointers to eigen objects
//                 eigStep  - eigen objects step (in bytes)
//                 coeffs   - previousely calculated decomposition coefficients
//                 avg      - pointer to averaged object
//                 avgStep  - its step (in bytes)
//                 rest     - pointer to restored object
//                 restStep - its step (in bytes)
//                 size     - ROI size of each object
//
//    Returns: Cv status
//
//    Notes:
//F*/
IPCVAPI(CvStatus, icvEigenProjection_8u32fR,(int nEigObjs, void* eigInput, int eigStep,
                                             int ioFlags, void* userData,
                                             float* coeffs, float* avg, int avgStep,
                                             uchar* rest, int restStep, CvSize size))


/****************************************************************************************\
*                                      Histograms                                        *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvCalcEMD_32f
//    Purpose:    Computes Earth mover distance (and/or lower bound of it) for given pair
//                of signatures. Ground distance can calculated as
//                L1, L2 or C distance between features' coordinate vectors or
//                using user-defined distance function.
//    Context:   
//    Parameters: 
//      signature1  - first signature - array of size1 * (dims + 1) elements
//      signature2  - second signature - array of size2 * (dims + 1) elements
//      dims        - number of dimensions in feature space. If 0, then
//                    signature1 and signature2 are considered as simple 1D histograms,
//                    else both signatures must look as follows:
//                    (weight_i0, x0_i0, x1_i0, ..., x(dims-1)_i0,
//                     weight_i1, x0_i1, x1_i1, ..., x(dims-1)_i1,
//                     ...
//                     weight_(size1-1),x0_(size1-1),x1_(size1-1,...,x(dims-1)_(size1-1))
//
//                     where weight_ik - weight of ik cluster.
//                     x0_ik,...,x(dims-1)_ik - coordinates of ik cluster.
//
//      dist_type   - IPPI_DIST_L1, IPPI_DIST_L2, IPPI_DIST_C mean one of standard metrics.
//                    ((CvDisType)-1) means user-defined distance function, which is
//                    passes two coordinate vectors and user parameter, and which returns
//                    distance between those feature points.
//      emd         - pointer to calculated emd distance
//      lower_bound - pointer to calculated lower bound.
//                    if 0, this quantity is not calculated (only emd is calculated).
//                    else if calculated lower bound is greater or equal to the value,
//                    stored at this pointer, then the true emd is not calculated, but
//                    is set to that lower_bound.
//    Returns:
//      IPP_NO_ERR if ok, error code else
//    Notes:
//F*/
IPCVAPI( CvStatus , icvCalcEMD_32f, ( float* signature1, int size1,
                                      float* signature2, int size2,
                                      int dims, CvDisType dist_type,
                                      float (*dist_func)( float* f1, float* f2,
                                                          void* user_param),
                                      float* emd, float* lower_bound,
                                      void* user_param ))


/****************************************************************************************\
*                                   Histogram of contrast calculation                    *
\****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvCalcContrastHist8uC1R
//    Purpose:    Calculating the histogram of contrast from array of the one-channel images
//    Context:
//    Parameters:
//    Returns:
//    Notes:      if dont_clear parameter is NULL then histogram clearing before
//                calculating (all values sets to NULL)
//F*/
IPCVAPI(CvStatus, icvCalcContrastHist8uC1R, ( uchar** img, int step, CvSize size,
                                              CvHistogram* hist, int dont_clear ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvCalcContrastHistMask8uC1R
//    Purpose:    Calculating the mask histogram of contrast from array of the one-channel images
//    Context:
//    Parameters:
//    Returns:
//    Notes:      if dont_clear parameter is NULL then histogram clearing before
//                calculating (all values sets to NULL)
//F*/
IPCVAPI(CvStatus, icvCalcContrastHistMask8uC1R, ( uchar** img, int step, 
                                                  uchar*  mask, int mask_step, CvSize size,
                                                  CvHistogram* hist, int dont_clear ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvFitEllipse_32f
//    Purpose: The function calculates ellips fitted to set of points by least squares
//    Context:
//    Parameters:  points  - array of float 2D points
//                 n       - number of points
//                 box     - pointer to box which describes ellipse (return parameter)
//
//    Returns: Cv status
//    Notes: number of points must be greater than 5 
//           
//F*/
IPCVAPI( CvStatus , icvFitEllipse_32f, ( CvPoint2D32f* points, int n, CvBox2D32f* box))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvFitLine2D
//    Purpose: The function approximates a given set of points with a line by m-estimators
//    Context:
//    Parameters:  points  - array of float 2D points
//                 count       - number of points
//                 dist        - type of distance
//                 user_weight - user-defined distance 
                                (pointer to function calculating weights, see manual)
//                 reps        - bearable distance error
//                 aeps        - bearable direction angle error
//                 line        - output: (line[0], line[1]) -- normalized direction
//                                  of the line, (line[2], line[3]) -- the point that
//                                  belongs to the line
//
//    Returns: Cv status
//    Notes:
//           
//F*/
IPCVAPI( CvStatus, icvFitLine2D, (CvPoint2D32f* points, int count, CvDisType dist, 
                                  void *param, float reps, float aeps, float* line ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvFitLine3D
//    Purpose: The function approximates a given set of points with a line by m-estimators
//    Context:
//    Parameters:  points  - array of float 3D points
//                 count       - number of points
//                 dist        - type of distance
//                 user_weight - user-defined distance 
                                (pointer to function calculating weights, see manual)
//                 reps        - bearable distance error
//                 aeps        - bearable direction angle error
//                 line        - output: (line[0], line[1], line[2]) -- normalized direction
//                                  of the line, (line[3], line[4], line[5]) -- the point that
//                                  belongs to the line
//
//    Returns: Cv status
//    Notes: 
//           
//F*/
IPCVAPI( CvStatus, icvFitLine3D, (CvPoint3D32f* points, int count, CvDisType dist, 
                                  void *param, float reps, float aeps, float* line ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvProject3D
//    Purpose: Projects every point in the array points3D on the plane. given by the index -- yz for 0, 
   zx for 1 and xy for 2. If swap flag is set to nonzero, then the coordinates exchange places, 
   that is planes zy, xz, yx correspondingly.
//    Context:
//    Parameters:  points3D  - input array of float 3D points
//                 count     - number of points
//                 points2D  - output array of 2D points
//                 index     - defines the plane (0 is yz, 1 -- zx, 2 -- xy) 
//                 swap      - if nonzero then in the output array x and y exchange values
//
//    Returns: Cv status
//    Notes: 
//           
//F*/
//IPCVAPI( CvStatus,  icvProject3D, (CvPoint3D32f* points3D, int count, CvPoint2D32f* points2D,
//                                   int index, int swap ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvFindHandRegion
//    Purpose: The function finds center of points array in 3D on line
//    Context:
//    Parameters:  points  - array of float 3D points
//                 count       - number of points
//                 line        - (line[0], line[1], line[2]) -- normalized direction
//                                  of the line, (line[3], line[4], line[5]) -- the point that
//                                  belongs to the line
//                 size        - input size of region
//                 center      - output center of region
//                 storage     - pointer to the memory storage
//                 numbers     - pointer to the output number's sequence  
//
//    Returns: Cv status
//    Notes: number of points must be greater than 5 
//           
//F*/
IPCVAPI( CvStatus, icvFindHandRegion, (CvPoint3D32f* points, int count,
                                         CvSeq* indexs,
                                         float* line, CvSize2D32f size, int flag, 
                                         CvPoint3D32f* center,
                                         CvMemStorage* storage, CvSeq **numbers))
IPCVAPI( CvStatus, icvFindHandRegionA, (CvPoint3D32f* points, int count,
                                       CvSeq* indexs,
                                       float* line, CvSize2D32f size, int jc, 
                                       CvPoint3D32f* center,
                                       CvMemStorage* storage, CvSeq **numbers))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvCreateHandMask8uC1R,
             icvCreateHandMask
//    Purpose: Create hand mask
//    Context:
//    Parameters:  indexs  - pointer to the sequence of indexs
//                 numbers       - pointer to the sequence of numbers points
//                 image_mask    - pointer to the output image mask
//                 step       - width in bytes of the output image
//                 size      -  ROI size of the output image
//
//    Returns: Cv status
//    Notes: number of points must be greater than 5 
//           
//F*/
IPCVAPI ( CvStatus, icvCreateHandMask8uC1R, (CvSeq* numbers, 
                                              uchar* image_mask, int step, CvSize size,
                                              CvRect *roi))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvCalcImageHomography
//    Purpose: Calculate image Homography matrix for transform hand mask
//    Context:
//    Parameters: 
                   line        - (line[0], line[1], line[2]) -- normalized direction
//                                  of the line, (line[3], line[4], line[5]) -- the point that
//                                  belongs to the lineindexs  - pointer to the sequence of indexs
//                 center      - input center of hand region
//                 intrinsic   - 3*3 matrix of the intrinsic parameter
//                 homography  - output 3*3 homography matrix 
//
//    Returns: Cv status
//    Notes: number of points must be greater than 5 
//           
//F*/
IPCVAPI ( CvStatus, icvCalcImageHomography, (float *line, CvPoint3D32f* center,
                                            float intrinsic[3][3], float homography[3][3]))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvMaxRect
//    Purpose: Calculate max of the two rectangles
//    Context:
//    Parameters: 
//      rect1    - first input and output max rectangle
//      rect2    - second input rectangle
//      max_rect - output max rectangle
//
//    Returns: CvStatus
//F*/
IPCVAPI (CvStatus,  icvMaxRect, ( CvRect *rect1, CvRect *rect2, CvRect *max_rect ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvCalcProbDensity8u,
//           icvCalcProbDensity
//    Purpose: Calculate probability density for the two 2D histograms
//    Context:
//    Parameters: 
//                 hist   - first input histogram
//                 hist_mask   - second input histogram
//                 cur_rect   - second input rectangle
//                 prob_dens -  result probability density 
//                 step - output width  in byte
//
//    Returns: Cv status
//           
//F*/
IPCVAPI (CvStatus, icvCalcProbDensity, (CvHistogram* hist, CvHistogram* hist_mask,
                                        CvHistogram* hist_dens))

/****************************************************************************************/
/*                             HMM (Hidden Markov Models)                               */
/****************************************************************************************/


#define IPPI_COUNT_OBS(r, o, d, n)                           \
{                                                                        \
    (n)->width  = ((r)->width  - (o)->width  + (d)->width) / (d)->width; \
    (n)->height = ((r)->height - (o)->height + (d)->height)/ (d)->height;\
}

//*F//////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCreate2DHMM
//    Purpose: The function allocates memory for 2-dimensional embedded HMM model 
//             after you finish work with created HMM you must free memory
//             by calling icvRelease2DHMM function 
//    Context:
//    Parameters: hmm - addres of pointer to CvEHMM structure
//                state_number - array of hmm sizes (size of array == state_number[0]+1 )
//                num_mix - number of gaussian mixtures in low-level HMM states 
//                          size of array is defined by previous array values
//                obs_size - length of observation vectors
//
//    Returns: Error status. Parameter hmm is filled.
//
//    Notes: state_number[0] - number of states in external HMM.
//           state_number[i] - number of states in embedded HMM
//           
//           example for face recognition: state_number = { 5 3 6 6 6 3 },
//                                         length of num_mix array = 3+6+6+6+3 = 24
// 
//F*/                       
IPCVAPI( CvStatus, icvCreate2DHMM, ( CvEHMM** hmm, 
                                     int* state_number, 
                                     int* num_mix, int obs_size ))

//*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvRelease2DHMM
//    Purpose: The function free memory used by CvEHMM structure 
//    Context:
//    Parameters: hmm - address of pointer to CvEHMM structure
//    Returns: Error status.
//    Notes:  function set *hmm = 0          
//F*/                       
IPCVAPI( CvStatus, icvRelease2DHMM, ( CvEHMM** hmm ))


//*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCreateObsInfo
//    Purpose: The function allocates memory for CvImgObsInfo structure 
//             after you finish working with allocated structure - destroy it 
//             by  icvReleaseObsInfo
//              
//    Context:
//    Parameters: obs_info - addres of pointer to CvImgObsInfo structure
//                num_obs  - number of horizontal and vertical observations.
//                           Total number of allocated observation vectors 
//                           will be   num_obs.width*num_obs.height
//                obs_size - length of observation vector
//
//    Returns: Error status. Parameter obs_info is filled.
//
//    Notes: If you extracts observations from image use IPPI_COUNT_OBS macros 
//           for computing "num_obs" parameter:
// 
//           IPPI_COUNT_OBS( &roi, &obs, &delta, &num_obs) 
//                    
//                          here CvSize roi   - image ROI  
//                               CvSize obs   - size of image block 
//                               CvSize delta - horizontal and vertical shift
//                               CvSize num_obs - output parameter to be computed
//
//           macros IPPI_COUNT_OBS do the following:
//             
//             num_obs.width  = floor((roi.width -  obs.width +  delta.width )/ delta.width);
//             num_obs.height = floor((roi.height - obs.height + delta.height)/delta.height);
//
//
//F*/                       
IPCVAPI( CvStatus, icvCreateObsInfo, ( CvImgObsInfo** obs_info, 
                                       CvSize num_obs, int obs_size ) ) 

//*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvReleaseObsInfo
//    Purpose: The function free memory used by CvImgObsInfo structure 
//              
//    Context:
//    Parameters: obs_info - address of pointer to CvImgObsInfo structure
//    Returns: Error status.
//    Notes:  function sets *obs_info = 0           
//F*/                       
IPCVAPI( CvStatus, icvReleaseObsInfo, ( CvImgObsInfo** obs_info ) )


//*F//////////////////////////////////////////////////////////////////////////////////////
//    Name: icvImgToObs_DCT_8u32f_C1R
//    Purpose: The function takes as input an image and returns the sequnce of observations
//             to be used with an embedded HMM; Each observation is top-left block of DCT
//             coefficient matrix.
//    Context:
//    Parameters: img     - pointer to the original image ROI
//                imgStep - full row width of the image in bytes
//                roi     - width and height of ROI in pixels
//                obs     - pointer to resultant observation vectors
//                dctSize - size of the block for which DCT is calculated
//                obsSize - size of top-left block of DCT coeffs matrix, which is treated
//                          as observation. Each observation vector consists of
//                          obsSize.width * obsSize.height floats.
//                          The following conditions should be satisfied:
//                          0 < objSize.width <= dctSize.width,
//                          0 < objSize.height <= dctSize.height.
//                delta   - dctBlocks are overlapped and this parameter specifies horizontal
//                          and vertical shift.
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//      The algorithm is following:
//          1. First, number of observation vectors per row and per column are calculated:
//
//             Nx = floor((roi.width - dctSize.width + delta.width)/delta.width);
//             Ny = floor((roi.height - dctSize.height + delta.height)/delta.height);
//
//             So, total number of observation vectors is Nx*Ny, and total size of
//             array obs must be >= Nx*Ny*obsSize.width*obsSize.height*sizeof(float).
//          2. Observation vectors are calculated in the following loop
//               ( actual implementation may be different ), where 
//               I[x1:x2,y1:y2] means block of pixels from source image with
//               x1 <= x < x2, y1 <= y < y2,
//               D[x1:x2,y1:y2] means sub matrix of DCT matrix D.
//               O[x,y] means observation vector that corresponds to position
//               (x*delta.width,y*delta.height) in the source image
//               ( all indices are counted from 0 ).
//
//               for( y = 0; y < Ny; y++ )
//               {
//                   for( x = 0; x < Nx; x++ )
//                   {
//                       D = DCT(I[x*delta.width : x*delta.width + dctSize.width,
//                                  y*delta.height : y*delta.height + dctSize.height]);
//                       O[x,y] = D[0:obsSize.width, 0:obsSize.height];
//                   }
//               }
//F*/
IPCVAPI( CvStatus, icvImgToObs_DCT_8u32f_C1R, ( uchar* img, int imgStep, CvSize roi,
                                                float* obs, CvSize dctSize,
                                                CvSize obsSize, CvSize delta ))


//*F//////////////////////////////////////////////////////////////////////////////////////
//    Name: icvUniformImgSegm
//    Purpose: The uniformly segments all observation vectors extracted from image
//    Context:
//    Parameters: obs_info - observations structure
//                hmm      - 2D embedded HMM structure
//
//    Returns: error status
//    Notes: 
//F*/                       
IPCVAPI( CvStatus, icvUniformImgSegm,( CvImgObsInfo* obs_info, CvEHMM* ehmm ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: InitMixSegm
//    Purpose: The function implements the mixture segmentation of the states 
//             of the embedded HMM
//
//    Context: used with the Viterbi training of the embedded HMM
//             Function uses K-Means algorithm for clustering.
//
//    Parameters:  obs_info_array - array of pointers to image observations 
//                 num_img - length of above array
//                 hmm - pointer to HMM structure   
//     
//    Returns: error status                                   
//    Notes: 
//F*/
IPCVAPI( CvStatus, icvInitMixSegm, ( CvImgObsInfo** obs_info_array, 
                                     int num_img, CvEHMM* hmm  ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvEstimHMMStateParams
//    Purpose: function calculates means, variances, weights of every Gaussian mixture
//             of every low-level state of embedded HMM
//    Context:
//    Parameters:  obs_info_array - array of pointers to observation structures
//                 num_img  - length of above array
//                 hmm      - hmm structure
//                 
//    Returns:
//
//    Notes: 
//F*/
IPCVAPI( CvStatus, icvEstimateHMMStateParams, ( CvImgObsInfo** obs_info_array,
                                                int num_img, CvEHMM* hmm ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvEstimateTransProb
//    Purpose: function computes transition probability matrices of embedded HMM
//             given observations segmentation 
//
//    Context:
//    Parameters:  obs_info_array - array of pointers to observation structures
//                 num_img  - length of above array
//                 hmm      - hmm structure
//                 
//    Returns:
//
//    Notes: 
//F*/                
IPCVAPI( CvStatus, icvEstimateTransProb, ( CvImgObsInfo** obs_info_array, 
                                           int num_img, CvEHMM* hmm ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvEstimateTransProb
//    Purpose: function computes probabilities of appearing observations at any state
//             ( i.e. compute P(obs|state) for every pair (obs,state) )
//    Context:
//    Parameters:  obs_info - observations structure
//                 hmm      - hmm structure
//                 
//    Returns: error status
//    Notes: 
//F*/                      
IPCVAPI( CvStatus, icvEstimateObsProb, ( CvImgObsInfo* obs_info, 
                                         CvEHMM* hmm )) 

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvEViterbi   (Embedded Viterbi)
//    Purpose: The function calculates the embedded Viterbi algorithm
//    Context:
//    Parameters:  
//                obs_info - observations structure
//                hmm      - hmm structure
//                 
//    Returns: the Embedded Viterbi probability.
//             Observations, stored in of obs_info structure are segmented 
//             ( but new segmentation does not affect mixture segmentation or 
//               states parameters )
//    Notes: 
//F*/
IPCVAPI( float, icvEViterbi, (CvImgObsInfo* obs_info, CvEHMM* hmm )) 


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMixSegmL2
//    Purpose: function clusters observation vectors from several images 
//             given observations segmentation.
//             Euclidean distance used for clustering vectors.
//             Centers of clusters are given means of every mixture 
//
//    Context: in HMM face recognition used after Viterbi segmentation 
//    Parameters:  obs_info_array - array of pointers to observation structures
//                 num_img  - length of above array
//                 hmm      - hmm structure
//                 
//    Returns: error status
//    Notes: 
//F*/                
IPCVAPI( CvStatus, icvMixSegmL2, ( CvImgObsInfo** obs_info_array, 
                                   int num_img, CvEHMM* hmm  ))



IPCVAPI( float, icvSquareDistance, ( CvVect32f v1, CvVect32f v2, int len ) )
                                   
IPCVAPI( CvStatus, icvViterbiSegmentation, ( int num_states, int num_obs,
                                    CvMatr32f transP,
                                    CvMatr32f B, /*muDur[0], */
                                    int start_obs, int prob_type,
                                    int** q, int min_num_obs, int max_num_obs,
                                    float* prob  ) )

/* end of HMM functions*/  

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: KMeans
//    Purpose: The function implements the K means algorithm, to cluster an array of sample
//             vectors in a number of classes specified by numClusters
//    Context:
//    Parameters:  numClunster - the number of clusters
//                 samples - the array of sample vectors
//                 numSamples - the number of samples
//                 VecSize - the size of each sample vector
//				   termcrit.eps - the convergence error; the iterations to find the best cluster 
//                         centers will stop, when the value of the cost function at consecutive 
//                         iterations falls below this threshold
//                 cluster - characteristic array. for every input vector indicates cluster
//                 
//    Returns: error code
//
//    Notes: 
//F*/
IPCVAPI( CvStatus, icvKMeans, ( int num_clusters, CvVect32f* samples, int num_samples, 
                                int vec_size, CvTermCriteria termcrit, int* cluster ))


/* //////////////////////////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////////////////// */

/*    IPPM Group Functions                                                              */

/* //////////////////////////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////////////////// */
/* //////////////////////////////////////////////////////////////////////////////////// */


/****************************************************************************************/
/*            Create and Delete Functions                                               */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvCreateVect
//   Purpose:
//     Creates new  vector
//   Context:
//   Parameters:
//     length - size of new vector (in elements)
//   Returns:
//     New vector or NULL if No Memory
//   Notes:
//F*/
IPCVAPI(CvVect32f, icvCreateVector_32f,(int length))
IPCVAPI(CvVect64d, icvCreateVector_64d,(int length))
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvCreateMatrix
//   Purpose:
//     Creates new matrix 
//   Context:
//   Parameters:
//     width - number of columns in  matrix
//     height - number of rows in new matrix  
//   Returns:
//     New matrix or NULL if No Memory
//   Notes:
//F*/
IPCVAPI(CvMatr32f, icvCreateMatrix_32f,(int width, int height))
IPCVAPI(CvMatr64d, icvCreateMatrix_64d,(int width, int height))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvDeleteVect
//    Purpose:
//      Deletes vector
//    Context:
//    Parameters:
//      pPointer - pointer to vector to delete
//    Returns:
//      IPP_NO_ERR if all Ok or error code
//    Notes:
//      The vector must be created by  by icvsCreateVector function
//F*/
IPCVAPI(CvStatus, icvDeleteVector, (void* Pointer))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvDeleteMatr
//    Purpose:
//      Deletes matrix
//    Context:
//    Parameters:
//      pPointer - pointer to matrix to delete
//    Returns:
//      IPP_NO_ERR if all Ok or error code
//    Notes:
//      The memory block must be allocated by icvsCreateMatrix function
//F*/
IPCVAPI(CvStatus, icvDeleteMatrix, (void* Pointer))
/****************************************************************************************/
/*            Get and Set Functions                                                     */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvsSetElement
//   Purpose:
//     Fills new matrix with data from single row
//   Context:
//   Parameters:
//     Matrix - matrix  
//     width - number of columns in  matrix
//     height - number of rows in  matrix
//     x      - column number
//     y      - row number
//     Value  - value to set   
//     Source - pointer to data   
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvSetElement_32f,(CvMatr32f Matrix,
                          int width,
                          int height,
                          int x,
                          int y,  
                          float Value))
IPCVAPI(CvStatus, icvSetElement_64d,( CvMatr64d Matrix,
                                      int width,
                                      int height,
                                      int x,
                                      int y,  
                                      double Value))
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvGetElement32f
//   Purpose:
//     Fills new matrix with data from single row
//   Context:
//   Parameters:
//     Matrix - matrix  
//     width - number of columns in  matrix
//     height - number of rows in  matrix
//     x      - column number
//     y      - row number
//     Value  - value to set   
//     Source - pointer to data   
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvGetElement_32f,(CvMatr32f Matrix,
                          int width,
                          int height,
                          int x,
                          int y,  
                          float* Value))
IPCVAPI(CvStatus, icvGetElement_64d,(CvMatr64d Matrix,
                                        int width,
                                        int height,
                                        int x,
                                        int y,  
                                        double* VALUE))
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvsSetZero
//   Purpose:
//     Sets all elements of the Matrix to 0
//   Context:
//   Parameters:
//     Matr  - Matrix to Set 
//     width - number of columns in  matrix
//     height - number of rows in new matrix
//     
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvSetZero_32f,(CvMatr32f Matr, int width, int height))
IPCVAPI(CvStatus, icvSetZero_64d,(CvMatr64d Matr, int width, int height))
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvSetIdentity32f
//   Purpose:
//     Sets matrix to Identity operator
//   Context:
//   Parameters:
//     Matr  - Matrix to Set 
//     width - number of columns in  matrix
//     height - number of rows in new matrix
//     
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvSetIdentity_32f,(CvMatr32f Matr, int width, int height))
IPCVAPI(CvStatus, icvSetIdentity_64d,(CvMatr64d Matr, int width, int height))
/****************************************************************************************/
/*            Copy  Functions                                                           */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvsCopyVector
//   Purpose:
//     Copies one vector to the another
//   Context:
//   Parameters:
//     Source - source vector 
//     length - length of the vector in elements
//     Dest   - destination vector   
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvCopyVector_32f,(CvVect32f Source,
                                    int length,
                                    CvVect32f Dest))
IPCVAPI(CvStatus, icvCopyVector_64d,(CvVect64d Source,
                                       int length,
                                       CvVect64d Dest))
/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvsCopyMatrix
//   Purpose:
//     Copies one matrix to the another
//   Context:
//   Parameters:
//     Source - Soudce matrix 
//     width - number of columns in  matrix
//     height - number of rows in new matrix
//     Dest -   destination Matrix   
//   Returns:
//     IPP_NO_ERR or error code
//   Notes:
//F*/
IPCVAPI(CvStatus, icvCopyMatrix_32f,(CvMatr32f Source,
                                    int width,
                                    int height,
                                    CvMatr32f Dest))
IPCVAPI(CvStatus, icvCopyMatrix_64d,(CvMatr64d Source,
                                       int width,
                                       int height,
                                       CvMatr64d Dest))
/****************************************************************************************/
/*                                Vector operations                                     */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsAddVect   
//    Purpose:    Sum of 2 vectors
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                dest - destination vector
//                length - size of vectors
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvAddVector_32f, ( CvVect32f src1, CvVect32f src2,
                                    CvVect32f dest, int length) )
IPCVAPI( CvStatus, icvAddVector_64d, ( CvVect64d src1, CvVect64d src2,
                                         CvVect64d dest, int length) )
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvSubVect   
//    Purpose:    Subtract one vector from other
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                dest - destination vector
//                length - size of vectors
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvSubVector_32f, ( CvVect32f src1, CvVect32f src2,
                                         CvVect32f dest, int length) )
IPCVAPI( CvStatus, icvSubVector_64d, ( CvVect64d src1, CvVect64d src2,
                                         CvVect64d dest, int length) )
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvDotProduct_32f   
//    Purpose:    Computes dot product of 2 vectors
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                length - size of vectors
//                value  - result of dot product (output parameter)
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvDotProduct_32f, ( CvVect32f src1, CvVect32f src2,
                                          int length, float* value) )
IPCVAPI( CvStatus, icvDotProduct_64d, ( CvVect64d src1, CvVect64d src2,
                                          int length, double* value) )                                        
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsCrossProduct2L_32f, icvsCrossProduct2L_64d
//    Purpose:    Computes cross product of two 3D vectors
//                if A and B - input vectors, then
//                C = A x B is cross product. Its components are
//
//                Cx = AyBz - AzBy 
//                Cy = AzBx - AxBz 
//                Cz = AxBy - AyBx 
//
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                dest - cross product 
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvCrossProduct2L_32f, ( CvVect32f src1,
                                            CvVect32f src2,
                                            CvVect32f dest ))
                                           
IPCVAPI( CvStatus, icvCrossProduct2L_64d, ( CvVect64d src1,
                                            CvVect64d src2,
                                            CvVect64d dest ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvCrossProduct3P32f   
//    Purpose:    Computes cross product of three 4D vectors
//                if A,B,C - input vectors, then
//                D = A x B x C is cross product. Its components are...
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                dest - cross product 
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvCrossProduct3P_32f, ( CvVect32f src1,
                                            CvVect32f src2,
                                            CvVect32f src3,
                                            CvVect32f dest )) 
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsScaleVect   
//    Purpose:    Multiplies  the Vector by a value.
//    Context:   
//    Parameters: src - source vector
//                dest - destination vector
//                length - size of vectors
//                value  - multiplier
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvScaleVector_32f, ( CvVect32f src, CvVect32f dest,
                                         int length, float value ))

IPCVAPI( CvStatus, icvScaleVector_64d, ( CvVect64d src, CvVect64d dest,
                                         int length, double value ))
/****************************************************************************************/
/*                             Matrix-Vector operations                                 */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvTransformVect
//    Purpose:    Calculates linear transform of a vector by a matrix.
//    Context:   
//    Parameters: matr - matrix of transformation
//                srcVect - vector
//                destVect - result of matrix-vector product
//                matrWidth
//                matrHeight - sizes of matrix               
//    Returns:   
//    Notes:    
//F*/
IPCVAPI( CvStatus, icvTransformVector_32f, ( CvMatr32f matr,
                                             CvVect32f srcVec,
                                             CvVect32f destVec,
                                             int matrWidth,
                                             int matrHeight ))
IPCVAPI(CvStatus, icvTransformVector_64d,( CvMatr64d matr,
                                             CvVect64d srcVec, 
                                             CvVect64d destVec,
                                             int matrWidth,    
                                             int matrHeight ))                                        
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvTransform4x1Vect32f
//    Purpose:    Calculates linear transform of a vector 4x1 by a matrix 4x4.
//    Context:   
//    Parameters: matr - matrix of transformation
//                srcVect - vector
//                destVect - result of matrix-vector product
        
//    Returns:   
//    Notes:    
//F*/
IPCVAPI( CvStatus, icvTransformVector4x1_32f, (CvMatr32f matr,
                                            CvVect32f srcVec,
                                            CvVect32f destVec
                                            ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsTransform3x1Vect32f
//    Purpose:    Calculates linear transform of a vector 3x1 by a matrix 3x3.
//    Context:   
//    Parameters: matr - matrix of transformation
//                srcVect - vector
//                destVect - result of matrix-vector product
        
//    Returns:   
//    Notes:    
//F*/
IPCVAPI( CvStatus, icvTransformVector3x1_32f, (CvMatr32f matr,
                                            CvVect32f srcVec,
                                            CvVect32f destVec
                                            ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvbTransformVector3x1P_32f
//    Purpose:    Calculates a product of a matrix 4x4 and an array of vectors (x, y, z, 1)
//                  and then divides the first three components of the result by the fourth.
//                  If the fourth component is zero then the output is (0, 0, 0, 0).
//    Context:   
//    Parameters: matr - matrix of transformation (4x4)
//                srcVect - input array of vectors (x, y, z)
//                count - number of input vectors
//                destVect - array of vectors (x, y, z)
//    Returns:   
//    Notes:    
//F*/
IPCVAPI( CvStatus, icvbTransformVector3x1P_32f, ( CvMatr32f matr,   
                                   CvVect32f srcVec, 
                                   int count,
                                   CvVect32f destVec ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsCalcQuadraticForm
//    Purpose:    Calculates the quadratic form on the two vectors defined by matrix.
//    Context:   
//    Parameters: matr - matrix of quadratic form
//                srcVec1 - left vector
//                srcVec2 - right vector
//                matrWidth
//                matrHeight - sizes of matrix               
//                mean - result of vector-matrix-vector product
//    Returns:   
//    Notes:     
//F*/
IPCVAPI(CvStatus, icvCalcQuadraticForm_32f,
                            ( 
                             CvVect32f srcVec1,
                             CvVect32f srcVec2,  
                             CvMatr32f matr,
                             int matrWidth,
                             int matrHeight,
                             float* mean     
                            ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvMahalanobis32f
//    Purpose:    Calculates mahalanobis distance. 
//    Context:   
//    Parameters: matr - covariance matrix
//                srcVec1 - vector
//                srcVec2 - vector
//                matrSize - sizes of matrix               
//                distance - mahalanobis distance
//    Returns:   
//    Notes:     
//F*/
IPCVAPI( CvStatus, icvMahalanobis_32f,
                          (  CvVect32f srcVec1,
                             CvVect32f srcVec2,  
                             CvMatr32f matr,
                             int matrSize,
                             float* distance
                           ))
/****************************************************************************************/
/*                             Matrix-Matrix operations                                 */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvAddMatrix   
//    Purpose:    Sum of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first vector
//                srcMatr2 - second vector
//                destMatr - destination vector
//                matrWidth
//                matrHeight - sizes of matrix       
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI (CvStatus, icvAddMatrix_32f,( 
                          CvMatr32f srcMatr1,
                          CvMatr32f srcMatr2,    
                          CvMatr32f destMatr,
                          int matrWidth,
                          int matrHeight))
IPCVAPI(CvStatus, icvAddMatrix_64d, ( CvMatr64d src1,
                                        CvMatr64d src2,
                                        CvMatr64d dest,
                                        int width,
                                        int height))                          
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvSubMatrix   
//    Purpose:    Difference of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//                matrWidth
//                matrHeight - sizes of matrix       
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvSubMatrix_32f,
                        ( CvMatr32f srcMatr1,
                          CvMatr32f srcMatr2,    
                          CvMatr32f destMatr,
                          int matrWidth,
                          int matrHeight))
IPCVAPI(CvStatus, icvSubMatrix_64d,
                        ( CvMatr64d src1,
                          CvMatr64d src2,
                          CvMatr64d dest,
                          int width,
                          int height))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvScaleMatrix   
//    Purpose:    Multiplies the Matrix by a value
//    Context:   
//    Parameters: srcMatr -  source  matrix
//                destMatr - destination matrix
//                matrWidth
//                matrHeight - sizes of matrix
//                value - factor       
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvScaleMatrix_32f,( CvMatr32f srcMatr,
                                         CvMatr32f destMatr,
                                         int matrWidth,
                                         int matrHeight,
                                         float value))
IPCVAPI(CvStatus, icvScaleMatrix_64d,( CvMatr64d srcMatr,
                                         CvMatr64d destMatr,
                                         int matrWidth,
                                         int matrHeight,
                                         double value))                                   
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvMulMatrix   
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                matrWidth1
//                matrHeight1 - sizes of srcMatr1
//                srcMatr2 - second matrix
//                matrWidth2
//                matrHeight2 - sizes of srcMatr2       
//
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/
IPCVAPI(CvStatus,icvMulMatrix_32f,
                             (  CvMatr32f srcMatr1,
                                int matrWidth1,
                                int matrHeight1,
                                CvMatr32f srcMatr2,
                                int matrWidth2,
                                int matrHeight2,
                                CvMatr32f destMatr
                              ))
IPCVAPI(CvStatus,icvMulMatrix_64d,
                             (  CvMatr64d srcMatr1,
                                int matrWidth1,
                                int matrHeight1,
                                CvMatr64d srcMatr2,
                                int matrWidth2,
                                int matrHeight2,
                                CvMatr64d destMatr
                              ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       ippmMulTransposed_32f   
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                matrWidth1
//                matrHeight1 - sizes of srcMatr1
//                srcMatr2 - second matrix
//                matrWidth2
//                matrHeight2 - sizes of srcMatr2       
//
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/
IPCVAPI(CvStatus, icvMulTransposed_32f,
							   ( CvMatr32f srcMatr1,
								int matrWidth1,
								int matrHeight1,
								CvMatr32f srcMatr2,
								int matrWidth2,
								int matrHeight2,
								CvMatr32f destMatr
								))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvTransposeMatrix   
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                matrWidth
//                matrHeight - sizes of srcMatr
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvTransposeMatrix_32f,
                              (  CvMatr32f srcMatr,
                                 int matrWidth,
                                 int matrHeight,
                                 CvMatr32f destMatr
                               ))
IPCVAPI(CvStatus, icvTransposeMatrix_64d,
                              (  CvMatr64d srcMatr,
                                 int matrWidth,
                                 int matrHeight,
                                 CvMatr64d destMatr
                               ))                              
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvMulTransMatrixL, icvMulTransMatrixR
//    Purpose:    Multiply a matrix by transposed itself
//    Context:   
//    Parameters: srcMatr (matrHeight, matrWidth) - source matrix
//                                (matrHeight - rows number, matrWidth - columns number)
//                destMatr(matrHeight, matrWidth) - destination matrix
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes: 1) The function icvMulTransMatrixL calculates product A At,
//              where A - source matrix, At - its transposition.
//              The function icvMulTransMatrixR calculates product At A.
//           2) Performs more quickly then ordinary multiplication.
//F*/

IPCVAPI( CvStatus, icvMulTransMatrixL_32f, ( CvMatr32f srcMatr,
                    int matrWidth, int matrHeight, CvMatr32f destMatr ) )

IPCVAPI( CvStatus, icvMulTransMatrixR_32f, ( CvMatr32f srcMatr,
                    int matrWidth, int matrHeight, CvMatr32f destMatr ) )
IPCVAPI( CvStatus, icvMulTransMatrixL_64d, ( CvMatr64d srcMatr,
                    int matrWidth, int matrHeight, CvMatr64d destMatr ) )

IPCVAPI( CvStatus, icvMulTransMatrixR_64d, ( CvMatr64d srcMatr,
                    int matrWidth, int matrHeight, CvMatr64d destMatr ) )                 
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvInvertMatrix   
//    Purpose:    Invertes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                matrWidth
//                matrHeight - sizes of srcMatr
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/
IPCVAPI ( CvStatus, icvInvertMatrix_32f,(  CvMatr32f srcMatr,
                                              int matrSize,
                                              CvMatr32f destMatr
                                          ))
IPCVAPI ( CvStatus, icvInvertMatrix_64d,(  CvMatr64d srcMatr,
                                             int matrSize,
                                             CvMatr64d destMatr
                                          ))                                          
/****************************************************************************************/
/*                             Matrix 4x4 matrix operations                             */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvAdd4x4Matr32f   
//    Purpose:    Sum of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI(CvStatus, icvAddMatrix4x4_32f,( CvMatr32f srcMatr1,
                                     CvMatr32f srcMatr2,
                                     CvMatr32f destMatr))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvSub4x4Matr32f   
//    Purpose:    Difference of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvSubMatrix4x4_32f,
                        ( CvMatr32f srcMatr1,
                          CvMatr32f srcMatr2,    
                          CvMatr32f destMatr
                        ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvScale4x4Matr32f   
//    Purpose:    Multiplies each element of matrix 4x4 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvScaleMatrix4x4_32f,
                            ( CvMatr32f srcMatr,
                              CvMatr32f destMatr,
                              float value
                             ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsMultiplyMatr4x4   
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvMulMatrix4x4_32f,
                             (  CvMatr32f srcMatr1,
                                CvMatr32f srcMatr2,
                                CvMatr32f destMatr
                             ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsTranspose4x4Matr32f   
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvTransposeMatrix4x4_32f,
                              (  CvMatr32f srcMatr,
                                 CvMatr32f destMatr
                              ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icv4x4Determinant32f  
//    Purpose:    Calculates Determinant of 4x4 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                Det - pointer to the value where determinant must be saved
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvDet4x4_32f,(  CvMatr32f srcMatr,
                                     float*    Det ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvInvertMatrix4x4_32f   
//    Purpose:    Invertes  4x4 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/

IPCVAPI (CvStatus, icvInvertMatrix4x4_32f, ( CvMatr32f srcMatr, CvMatr32f dstMatr ))

/****************************************************************************************/
/*                             Matrix 3x3 matrix operations                             */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsAdd3x3Matr32f   
//    Purpose:    Sum of 2 matrix 3x3
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI(CvStatus, icvAddMatrix3x3_32f,( CvMatr32f srcMatr1,
                                     CvMatr32f srcMatr2,
                                     CvMatr32f destMatr))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsSubMatr3x3   
//    Purpose:    Difference of 2 matrix 3x3
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvSubMatrix3x3_32f,
                        ( CvMatr32f srcMatr1,
                          CvMatr32f srcMatr2,    
                          CvMatr32f destMatr
                        ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsScale3x3Matr32f   
//    Purpose:    Multiplies each element of matrix 3x3 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvScaleMatrix3x3_32f,
                            ( CvMatr32f srcMatr,
                              CvMatr32f destMatr,
                              float value
                             ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsMulMatr3x3_32f, icvsMulMatr3x3_64d
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvMulMatrix3x3_32f,
                             (  CvMatr32f srcMatr1,
                                CvMatr32f srcMatr2,
                                CvMatr32f destMatr
                             ))
IPCVAPI(CvStatus, icvMulMatrix3x3_64d,
                             (  CvMatr64d srcMatr1,
                                CvMatr64d srcMatr2,
                                CvMatr64d destMatr
                             ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsTransposeMatr3x3   
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvTransposeMatrix3x3_32f,
                              (  CvMatr32f srcMatr,
                                 CvMatr32f destMatr
                              ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvsDeterminant3x3   
//    Purpose:    Calculates Determinant of 3x3 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                Det - pointer to the value where determinant must be saved
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI(CvStatus, icvDet3x3_32f,(  CvMatr32f srcMatr,
                                     float*    Det ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvInvertMatrix3x3_32f   
//    Purpose:    Inverses 3x3 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI (CvStatus, icvInvertMatrix3x3_32f,(CvMatr32f srcMatr,CvMatr32f dstMatr ))                                     
/****************************************************************************************/
/*                             Matrix 6x6 matrix operations                             */
/****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvAddMatrix6x6_32f   
//    Purpose:    Sum of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvAddMatrix6x6_32f,( CvMatr32f srcMatr1,
                                        CvMatr32f srcMatr2,
                                        CvMatr32f destMatr
                                        ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvSubMatrix6x6_32f   
//    Purpose:    Difference of 2 matrix 6x6
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI(CvStatus, icvSubMatrix6x6_32f,( CvMatr32f srcMatr1,
                                          CvMatr32f srcMatr2,
                                          CvMatr32f destMatr
                                        ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvMulMatrix6x6_32f   
//    Purpose:    Difference of 2 matrixes 6x6
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/

IPCVAPI(CvStatus, icvMulMatrix6x6_32f,( CvMatr32f srcMatr1,
                                          CvMatr32f srcMatr2,
                                          CvMatr32f destMatr
                                        ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvTransposeMatr6x6_32f  
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//F*/


IPCVAPI(CvStatus, icvTransposeMatrix6x6_32f,(  CvMatr32f srcMatr,
                                                 CvMatr32f destMatr
                                              ))                                        
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvScaleMatrix6x6_32f  
//    Purpose:    Multiplies each element of matrix 4x4 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI(CvStatus, icvScaleMatrix6x6_32f,( CvMatr32f srcMatr,
                                            CvMatr32f destMatr,
                                            float value
                                           ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvInvertMatrix6x6_32f   
//    Purpose:    Invertes  6x6 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/

IPCVAPI (CvStatus, icvInvertMatrix6x6_32f, (CvMatr32f srcMatr,CvMatr32f dstMatr))                                           


/****************************************************************************************/
/*                           Eigenvalues & eigenvectors problem                         */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names:      icvJacobiEigens_32f, icvJacobiEigens_64d
//    Purpose:    Eigenvalues & eigenvectors calculation of a symmetric matrix:
//                A Vi  =  Ei Vi
//    Context:   
//    Parameters: A(n, n) - source symmetric matrix (n - rows & columns number),
//                V(n, n) - matrix of its eigenvectors 
//                          (i-th row is an eigenvector Vi),
//                E(n)    - vector of its eigenvalues
//                          (i-th element is an eigenvalue Ei),
//                eps     - accuracy of diagonalization.
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//        1. The functions destroy source matrix A, so if you need it after
//           running icvJacobiEigens_???, you must copy it before.
//        2. Eigenvalies and eigenvectors are sorted in Ei absolute value descending.
//        3. Calculation time depends on eps value. If the time isn't very important,
//           we recommend to set eps = 0.
//F*/
IPCVAPI(CvStatus, icvJacobiEigens_32f, ( CvMatr32f A,
                                           CvMatr32f V,
                                           CvVect32f E,
                                           int n,
                                           float eps ) )

IPCVAPI(CvStatus, icvJacobiEigens_64d, ( CvMatr64d A,
                                           CvMatr64d V,
                                           CvVect64d E,
                                           int n,
                                           double eps ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvCalcCovarMatrix_8u32f, icvCalcCovarMatrix_32f
//    Purpose: The function calculates a covariance matrix for a group of input objects
//             (images, vectors, etc.)
//    Context:
//    Parameters:  nObjects    - number of source objects
//                 objects     - array of pointers to the source objects
//                 avg         - pointer to averaged object
//                 length      - number of elements in each source and averaged objects
//                 covarMatrix - covariance matrix (output parameter; must be allocated
//                               before call)
//
//    Returns: IPPM_NO_ERR or error code
//
//    Notes:   
//F*/
IPCVAPI(CvStatus, icvCalcCovarMatrix_8u32f, ( int nObjects, uchar** objects,
                                            float* avg, int length, float* covarMatrix ) )

IPCVAPI(CvStatus, icvCalcCovarMatrix_32f, ( int nObjects, float** objects,
                                            float* avg, int length, float* covarMatrix ) )


/****************************************************************************************/
/*                           Singular Value Decomposition (SVD)                         */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names:      icvSVDecomposition_32f, icvSVDecomposition_64d
//    Purpose:    Compute Singular Value Decomposition of matrix A = [u][w][vT]
//               
//    Context:   
//    Parameters: A(n, m) - source matrix (n - columns, m - rows number),
//                          and result u - matrix
//                w(n, n) - result diagonal w matrix
//                v(n, n) - result v matrix
//               
//    Returns:
//    IPP_NO_ERROR or error code     
//    Notes:
//        1. The functions destroy source matrix A, so if you need it after
//           running icvSVDecomposition, you must copy it before.
//        2. Values in diagonal matrix are sorted.
//F*/
IPCVAPI(CvStatus, icvSVDecomposition_32f, ( CvMatr32f A,
                                              int         n,
                                              int         m,
                                              CvMatr32f w,
                                              CvMatr32f v ))
                                              
IPCVAPI(CvStatus, icvSVDecomposition_64d, ( CvMatr64d    A,
                                              int         n,
                                              int         m,
                                              CvMatr64d    w,
                                              CvMatr64d    v ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvSVD_64f
//    Purpose: Calculates singular value decomposition for rectangular
//             matrix a (m rows x n columns), such that u'*a*v = w,
//             u and v are ortogonal, w is diagonal and only the diagonal elements
//             are stored.
//    Context:
//    Parameters:  
//      m    - number of rows in matrix a
//      n    - number of columns in matrix a
//      a    - source matrix
//      lda  - step of the matrix a (full number of columns)
//      w    - singular values of the matrix a. array of min(m,n) elements.
//             it is ordered in decreasing order.
//      u    - left singular vectors - ortogonal matrix (m x m). Can be 0,
//             if is not required (function returns transposed matrix).
//      ldu  - step of the matrix u (full number of columns)
//      v    - right singular vectors - ortogonal matrix (n x n). Can be 0,
//             if is not required.
//      ldv  - step of the matrix v (full number of columns)
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//      Function damages source matrix a
//F*/
IPCVAPI( CvStatus, icvSVD_64f, ( int m, int n, double* a, int lda,
                                 double* w, double* u, int ldu, double* v, int ldv ))
                                        

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvPseudoInverse_64f
//    Purpose: Calculates pseudo-inverse matrix.
//    Context:
//    Parameters:  
//      m    - number of rows in matrix a
//      n    - number of columns in matrix a
//      a    - source matrix
//      lda  - step of the matrix a (full number of columns)
//      buffer - temporary buffer. Must have size of m*m + n*n + min(m,n) doubles.
//    Returns:
//      CV_OK or error code
//    Notes:
//      Function substitutes <a> matrix with its pseudo-inverse.
//      while source matrix is m x n, the inverse is n x m.
//F*/
IPCVAPI( CvStatus, icvPseudoInverse_64f, ( int m, int n, double* a, int lda,
                                           double eps, double* buffer ))

/****************************************************************************************/
/*                              Functions, needed for calibration                       */
/****************************************************************************************/

IPCVAPI(CvStatus, icvDetMatrix_32f,  ( CvMatr32f matr,
                                       int       matrSize,
                                       double*   det))

IPCVAPI(CvStatus, icvDetMatrix_64d,  ( CvMatr64d matr,
                                       int       matrSize,
                                       double*   det))

IPCVAPI(CvStatus, icvNormVector_32f, (   CvVect32f    vect,
                                int         length,
                                float*      norm))

IPCVAPI(CvStatus, icvNormVector_64d, (   double*    vect,
                                int         length,
                                double*      norm))


IPCVAPI(CvStatus, icvTrace_32f, (    CvMatr32f    matr,
                            int         width,
                            int         height,
                            float*      trace))

IPCVAPI(CvStatus, icvMulVectors_32f, (   CvVect32f    src1,
                                CvVect32f    src2,
                                CvVect32f    dest,
                                int         length))


IPCVAPI( CvStatus,  icvComplexMult_32f, (  CvMatr32f srcMatr,
                                CvMatr32f dstMatr,
                                int      width,
                                int      height))

IPCVAPI( CvStatus,  icvComplexMult_64d, (  double*  srcMatr,
                                double*  dstMatr,
                                int      width,
                                int      height))
                                
IPCVAPI(CvStatus, icvTrace_64d, (    CvMatr64d    matr,
                            int         width,
                            int         height,
                            double*      trace))
                            
IPCVAPI(CvStatus, icvMulVectors_64d, (   CvVect64d   src1,
                                CvVect64d    src2,
                                CvVect64d    dest,
                                int         length))

IPCVAPI(CvStatus, icvRodrigues, (CvMatr32f       rotMatr,
                                CvVect32f       rotVect,
                                CvMatr32f       Jacobian,
                                CvRodriguesType convType))

IPCVAPI(CvStatus, icvRodrigues_64d, (double*         rotMatr,
                                    double*         rotVect,
                                    double*         Jacobian,
                                    CvRodriguesType convType))



/****************************************************************************************\
*                                  Memory allocation functions                          *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvAlloc
//   Purpose:
//     Allocating memory
//   Context:
//   Parameters:
//     lSize - size of needed memory (in bytes)
//   Returns:
//     Pointer to allocated memory or NULL if No Memory
//   Notes:
//F*/
IPCVAPI( void*, p_cvAlloc_, ( int size, const char* file, int line ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvFree_
//    Purpose:
//      Free allocated memory
//    Context:
//    Parameters:
//      pPointer - pointer to memory to be deallocated
//    Returns:
//      CV_OK if all Ok or error code
//    Notes:
//      The memory block must be allocated by icvAlloc_ function
//F*/
IPCVAPI( void, p_cvFree_, ( void** pptr, const char* file, int line ) )

/****************************************************************************************\
*                                  Error handling functions                              *
\****************************************************************************************/
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvErrorFromStatus
//    Purpose:
//             converts CvStatus error codes to CVStatus
//    Context:
//    Parameters:
//      status - CvStatus
//    Returns:
//      CVStatus
//    Notes:
//      It can't accept CV_OK
//F*/                        
IPCVAPI( CVStatus, icvErrorFromStatus,( CvStatus status ) )



#ifndef __CV_DEFINE_USER_LIST__

#ifdef __cplusplus
}
#endif

#endif /*__CV_DEFINE_USER_LIST__*/

#ifdef IPCVAPI_DEFINED
    #undef IPCVAPI
    #undef IPCVAPI_DEFINED
#endif

#endif /*__CV_H_*/
