/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef _CVARRAY_H_
#define _CVARRAY_H_

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icv1DCopy
//    Purpose:
//      Copying 1D array from source to destination array
//    Context:
//    Parameters:
//      pSrc - source array
//      pDst - destination array
//      lLen - lenght of arrays
//    Returns:
//    Notes:
//F*/
void icv1DCopy( void* pSrc, void* pDst, int lLen );


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icv1ucSet
//    Purpose:
//      Setting unsigned char value to every element of destination value
//    Context:
//    Parameters:
//      ucValue - source value
//      pDst    - destination array
//      lLen    - lenght of destination array
//    Returns:
//    Notes:
//F*/
void icv1ucSet( unsigned char ucValue, void* pDst, long lLen );


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCopy8uC1R
//    Purpose:
//      Copies content of ROI of source image into destination image
//    Context:
//    Parameters:
//      src      - source image
//      src_step - full width of source image row in bytes
//      dst      - destination image
//      dst_step - full width of destination image row in bytes
//      size     - width and height of copied ROI in pixels
//    Returns:
//      none
//    Notes:
//F*/
void  icvCopy8uC1R( const uchar* src, int src_step,
                    uchar* dst, int dst_step,
                    CvSize size );


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCopyToBuf8uC1R
//    Purpose:
//      Copies content of ROI of source image into destination buffer
//    Context:
//    Parameters:
//      src      - source image
//      src_step - full width of source image row in bytes
//      dst      - destination buffer
//      size     - width and height of copied ROI in pixels
//    Returns:
//      none
//    Notes:
//F*/
void  icvCopyToBuf8uC1R( const uchar* src, int src_step,
                         uchar* dst, CvSize size );

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSet
//    Purpose:
//      Fill image by value
//    Context:
//    Parameters:
//      img  - source array
//      value - fill value
//    Returns:
//    Notes:
//F*/
void icvSet(IplImage* img, float value);

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCopy
//    Purpose:
//      Copying from one image to another
//    Context:
//    Parameters:
//      src - source
//      dst - destination
//    Returns:
//    Notes:
//F*/
void icvCopy( IplImage* src, IplImage* dst );

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSubtract
//    Purpose:
//      Subs one image from another and puts result to destination image dst = srcA - srcB
//    Context:
//    Parameters:
//      srcA - first source
//      srcB - second source
//      dst  - destination
//    Returns:
//    Notes:
//F*/
void icvSubtract( IplImage* srcA, IplImage* srcB, IplImage* dst );

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSubtractS
//    Purpose:
//      The functions change the image intensity as follows:
//      If flip is false, the value is subtracted from the image pixel values.
//      If flip is true, the image pixel values are subtracted from the value.
//    Context:
//    Parameters:
//      src - source
//      dst  - destination
//    Returns:
//    Notes:
//F*/
void icvSubtractS(IplImage* src, IplImage* dst, float value, int flip);

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSquare
//    Purpose:
//      The function increases the intensity of an image by squaring each pixel value.
//    Context:
//    Parameters:
//      src - first source
//      dst - destination
//    Returns:
//    Notes:
//F*/
void icvSquare( IplImage* src, IplImage* dst );

#endif /* _CVARRAY_H_ */

/* End of file. */
