/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef _CVCONV_H_
#define _CVCONV_H_

#include "_cv.h"


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSepConvSmall3_32f
//    Purpose:
//      Performs separable convolution given 3x3, 3x1 or 1x3 kernel
//    Context:
//    Parameters:
//      src        - source image ROI
//      src_step   - full width of source image (in bytes)
//      dst        - destination image ROI
//      dst_step   - full width of destination image (in bytes)
//      src_size   - size of source image ROI (width & height)
//      kx         - horizontal kernel - array of 3 numbers. anchor is in the middle.
//                   if NULL then only vertical covolution is applied.
//      ky         - vertical kernel - array of 3 numbers. anchor is in the middle.
//                   if NULL then only horizontal covolution is applied.
//      buffer     - temporary buffer. Used only if both kernels are not NULL.
//                   In the last case, it must be, at least,
//                   (src_size - 2)*sizeof(float) bytes.
//    Returns:
//      Nothing       
//    Notes:
//      Size of destination ROI is not passed into the function, because
//      it assumes dst ROI size:
//      = (src_size.width - 2, src_size.height - 2) if both kernels are used
//      = (src_size.width - 2, src_size.height)     if horizontal kernel != 0 only.
//      = (src_size.width, src_size.height - 2)     if vertical kernel != 0 only.
F*/
void icvSepConvSmall3_32f( float*  src, int src_step,
                             float*  dst, int dst_step,
                             CvSize src_size,
                             const float* kx, const float* ky,
                             float*  buffer );

#endif /* __ */

/* End of file. */
