/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef _CVCONVERT_H_
#define _CVCONVERT_H_

typedef struct
{
    int    size;
    int    step;
    int    count;
    char*  top;
    char*  first;
    char*  last;
    char*  bottom;
}
_CvRingBuffer;


#define _CV_RB_ROTATE( ptr, buf_step, buf_bottom, buf_size ) \
    ((ptr) += (buf_step), (ptr) -= (((ptr) < (buf_bottom))-1) & (buf_size))

#define _CV_RB_ROTATE2( ptr, rb ) \
    _CV_RB_ROTATE( ptr, (rb)->step, (rb)->bottom, (rb)->size )


#define _CV_CVT_PX2PL_FUNC( name )       \
        void name( char* img, int img_step,\
                   CvSize size,           \
                   _CvRingBuffer* buf0,  \
                   _CvRingBuffer* buf1,  \
                   _CvRingBuffer* buf2,  \
                   _CvRingBuffer* buf3 )

#define _CV_CVT_PL2PX_FUNC   _CV_CVT_PX2PL_FUNC

typedef _CV_CVT_PL2PX_FUNC( (*icvCvtPixToPlaneFunc) );
typedef _CV_CVT_PX2PL_FUNC( (*icvCvtPlaneToPixFunc) );

/* convertion pix->plane */
_CV_CVT_PX2PL_FUNC( icvCvtPixToPlane8uC3 );
_CV_CVT_PX2PL_FUNC( icvCvtPixToPlane8uAC4 );
_CV_CVT_PX2PL_FUNC( icvCvtPixToPlane32fC3 );
_CV_CVT_PX2PL_FUNC( icvCvtPixToPlane32fAC4 );

/* convertion plane->pix */
_CV_CVT_PL2PX_FUNC( icvCvtPlaneToPix8uC3 );
_CV_CVT_PL2PX_FUNC( icvCvtPlaneToPix8uAC4 );
_CV_CVT_PL2PX_FUNC( icvCvtPlaneToPix32fC3 );
_CV_CVT_PL2PX_FUNC( icvCvtPlaneToPix32fAC4 );

/* extract color channel to the separate image */
#define _CV_EXTR_COI_FUNC( name )        \
        void name( char* img, int img_step,\
                   CvSize size,           \
                   _CvRingBuffer* buf0,  \
                   int coi  )

typedef _CV_EXTR_COI_FUNC( (*icvExtractCOIFunc) );

_CV_EXTR_COI_FUNC( icvExtractCOI8uC3 );
_CV_EXTR_COI_FUNC( icvExtractCOI8uAC4 );
_CV_EXTR_COI_FUNC( icvExtractCOI32fC3 );
_CV_EXTR_COI_FUNC( icvExtractCOI32fAC4 );


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvExtractROI_8u32f
//    Purpose:
//      Extracts ROI from byte-depth image and stores it in single continuous block.
//    Context:
//    Parameters:
//      src        - source image (not pointer to ROI, but to origin of the whole image
//      src_step   - full width of the source image (in bytes)
//      dst_buffer - destination buffer
//      src_rect   - ROI rectangle.
//    Returns:
//      Nothing       
//    Notes:
F*/
void icvExtractROI_8u32f( uchar* src, int src_step, 
                          float* dst_buffer, CvRect src_rect );


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCvt_32f_64d
//    Purpose:
//      Convert array of float to double.
//    Context:
//    Parameters:
//      src        - source array of float
//      dst        - destination array of double
//      size       - size of array in elements
//    Returns:
//      CV_NULLPTR_ERR  if bad pointers
//      CV_BADRANGE_ERR if bad size
//      CV_OK           if no errors
//    Notes:
F*/
CvStatus  icvCvt_32f_64d(   float*  src,
                            double* dst,
                            int     size);

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvCvt_32f_64d
//    Purpose:
//      Convert array of double to float.
//    Context:
//    Parameters:
//      src        - source array of double
//      dst        - destination array of float
//      size       - size of array in elements
//    Returns:
//      CV_NULLPTR_ERR  if bad pointers
//      CV_BADRANGE_ERR if bad size
//      CV_OK           if no errors
//    Notes:
F*/
CvStatus  icvCvt_64d_32f( double* src, float*  dst, int size );

#endif/*_CVCONVERT_H_*/

/* End of file. */
