/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef __IPCV_H_
#define __IPCV_H_

/****************************************************************************************/
/*                                BackGround Differencing                               */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: CvAcc8u32f, CvAcc8s32f, CvAcc32f32f
//    Purpose: Accumulating the mean: pDst(x,y)=pSrc(x,y)+pDst(x,y)
//    Context:
//    Parameters:
//              pSrc - source image
//              pDst - mean statistic destination image
//              len - length of pixel array
//    Returns:
//              IPP_NO_ERR if all ok or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvAdd_8u32f_C1IR, ( const unsigned char* pSrc, int srcStep,
                                           float* pSrcDst, int srcDstStep,
                                           CvSize roiSize ))

IPCVAPI( CvStatus , icvAdd_8s32f_C1IR, ( const char* pSrc, int srcStep,
                                           float* pSrcDst, int srcDstStep,
                                           CvSize roiSize ))

IPCVAPI( CvStatus , icvAdd_32f_C1IR,   ( const float* pSrc, int srcStep,
                                           float* pSrcDst, int srcDstStep,
                                           CvSize roiSize ))

/*F/////////////////////////////////////////////////////////////////////////////////////
//    Name: CvAccMask8u32f, CvAccMask8s32f, CvAccMask32f32f                          
//    Purpose: Accumulating the mean considering the Mask:                              
//              pDst(x,y)=  pSrc(x,y)+pDst(x,y) if Mask!=0 or pDst(x,y) else            
//    Context:                                                                          
//    Parameters:                                                                       
//              pSrc - source image                                                     
//              pDst - mean statistic destination image                                 
//              len - length of pixel array                                             
//              Mask - pointer to the Mask array                                        
//    Returns:                                                                          
//              IPP_NO_ERR if all ok or error code                                      
//    Notes:                                                                            
//F*/
IPCVAPI( CvStatus, icvAdd_8u32f_C1IMR, ( const unsigned char*  pSrc, int srcStep,
                                         const unsigned char*  pMask, int maskStep,
                                         float* pSrcDst, int srcDstStep,
                                         CvSize roiSize ))

IPCVAPI( CvStatus, icvAdd_8s32f_C1IMR, ( const char*  pSrc, int srcStep,
                                         const unsigned char*  pMask, int maskStep,
                                         float* pSrcDst, int srcDstStep,
                                         CvSize roiSize ))

IPCVAPI( CvStatus, icvAdd_32f_C1IMR,   ( const float*  pSrc, int srcStep,
                                         const unsigned char*  pMask, int maskStep,
                                         float* pSrcDst, int srcDstStep,
                                         CvSize roiSize ))

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Name: CvSquareAccMask8u32f, CvSquareAccMask8s32f, CvSquareAccMask32f32f        
//    Purpose: Accumulating the mean square considering the Mask:                       
//              pDst(x,y)=  pSrc(x,y)*pSrc(x,y)+pDst(x,y) if Mask!=0 or pDst(x,y) else  
//    Parameters:                                                                       
//              pSrc - source image                                                     
//              pDst - mean square statistic destination image                          
//              len - length of pixel array                                             
//              Mask - pointer to the Mask array                                        
//    Returns:                                                                          
//              IPP_NO_ERR if all ok or error code                                      
//    Notes:                                                                            
//F*/
IPCVAPI( CvStatus, icvAddSquare_8u32f_C1IR, (const unsigned char*  pSrc, int srcStep,
                                             float* pSrcDst, int srcDstStep,
                                             CvSize roiSize))

IPCVAPI( CvStatus, icvAddSquare_8s32f_C1IR, (const char*  pSrc, int srcStep,
                                             float* pSrcDst, int srcDstStep,
                                             CvSize roiSize))

IPCVAPI( CvStatus, icvAddSquare_32f_C1IR,   (const float*  pSrc, int srcStep,
                                             float* pSrcDst, int srcDstStep,
                                             CvSize roiSize))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: CvAcc8u32f, CvAcc8s32f, CvAcc32f32f
//    Purpose: If Mask is not equal to zero
//             Accumulating the mean: pDst(x,y)=pSrc(x,y)+pDst(x,y)
//    Context:
//    Parameters:
//              pSrc - source image
//              pDst - mean statistic destination image
//              len - length of pixel array
//    Returns:
//              IPP_NO_ERR if all ok or error code
//    Notes:
//F*/
IPCVAPI( CvStatus, icvAddSquare_8u32f_C1IMR, (const unsigned char* pSrc, int srcStep,
                                              const unsigned char* pMask, int maskStep,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

IPCVAPI( CvStatus, icvAddSquare_8s32f_C1IMR, (const char* pSrc, int srcStep,
                                              const unsigned char* pMask, int maskStep,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

IPCVAPI( CvStatus, icvAddSquare_32f_C1IMR,   (const float* pSrc, int srcStep,
                                              const unsigned char* pMask, int maskStep,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMultiplyAccMask8u32f, icvMultiplyAccMask8s32f, icvMultiplyAccMask32f  
//    Purpose: Accumulating the image covariance considering the Mask:                  
//              pDst(x,y)=  pSrcA(x,y)*pSrcB+pDst(x,y) if Mask!=0 or pDst(x,y) else     
//    Context:                                                                          
//    Parameters:                                                                       
//              pSrcA, pSrcB - source images                                            
//              pDst - covariance statistic destination image                           
//              len - length of pixel array                                             
//              Mask - pointer to the Mask array                                        
//    Returns:                                                                          
//              IPP_NO_ERR if all ok or error code                                      
//    Notes:                                                                            
//F*/
IPCVAPI( CvStatus, icvAddProduct_8u32f_C1IR, (const unsigned char* pSrc1, int src1Step,
                                              const unsigned char* pSrc2, int src2Step,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

IPCVAPI( CvStatus, icvAddProduct_8s32f_C1IR, (const char* pSrc1, int src1Step,
                                              const char* pSrc2, int src2Step,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

IPCVAPI( CvStatus, icvAddProduct_32f_C1IR,   (const float* pSrc1, int src1Step,
                                              const float* pSrc2, int src2Step,
                                              float* pSrcDst, int srcDstStep,
                                              CvSize roiSize))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMultiplyAcc8u32f, icvMultiplyAcc8s32f, icvMultiplyAcc32f32f
//    Purpose: If Mask is not equal to zero
//             Accumulating the images covariance:
//                        pDst(x,y)=pSrcA(x,y)*pSrcA(x,y)+pDst(x,y)
//    Context:
//    Parameters:
//              pSrcA, pSrcB - source images
//              pDst - covariance statistic destination image
//              len - length of pixel array
//    Returns:
//              IPP_NO_ERR if all ok or error code
//    Notes:
//F*/
IPCVAPI( CvStatus, icvAddProduct_8u32f_C1IMR, (const unsigned char* pSrc1, int src1Step,
                                               const unsigned char* pSrc2, int src2Step,
                                               const unsigned char* pMask, int maskStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize))

IPCVAPI( CvStatus, icvAddProduct_8s32f_C1IMR, (const char* pSrc1, int src1Step,
                                               const char* pSrc2, int src2Step,
                                               const unsigned char* pMask, int maskStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize))

IPCVAPI( CvStatus, icvAddProduct_32f_C1IMR,   (const float* pSrc1, int src1Step,
                                               const float* pSrc2, int src2Step,
                                               const unsigned char* pMask, int maskStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: CvRunningAvg8u32f, CvRunningAvg8s32f, CvRunningAvg32f32f
//    Purpose: Adaptive update of statistical image
//             uImg(x,y,t) = alpha*yImg(x,y) + (1-alpha)*uImg(x,y,t-1)
//    Context:
//    Parameters:
//            uImg  -  image for update
//            yImg  -  what to add
//            len   -  length in pixels
//            alpha -  coefficient
//    Returns:  IPP_NO_ERR if all ok or error code
//    Notes:
//F*/
IPCVAPI( CvStatus, icvAddWeighted_8u32f_C1IR, (const unsigned char*  pSrc, int srcStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize, float alpha))

IPCVAPI( CvStatus, icvAddWeighted_8s32f_C1IR, (const char*  pSrc, int srcStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize, float alpha))

IPCVAPI( CvStatus, icvAddWeighted_32f_C1IR,   (const float*  pSrc, int srcStep,
                                               float* pSrcDst, int srcDstStep,
                                               CvSize roiSize, float alpha))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvRunAvgMask8u32f, icvRunAvgMask8s32f, icvRunAvgMask32f32f              
//    Purpose: Calculating the linear combination considering the Mask:                 
//             pDst(x,y)=  alpha * pSrc(x,y)+ (1-alpha)*pDst(x,y) if Mask!=0           
//                          or pDst(x,y) else                                           
//    Parameters:                                                                       
//              pSrc - source image                                                     
//              pDst - destination image                                                
//              len - length of pixel array                                             
//              Mask - pointer to the Mask array                                        
//              alpha - coefficient                                                     
//    Returns:                                                                          
//              IPP_NO_ERR if all ok or error code                                      
//    Notes:                                                                            
//F*/
IPCVAPI( CvStatus, icvAddWeighted_8u32f_C1IMR, (const unsigned char*  pSrc, int srcStep,
                                                const unsigned char* pMask, int maskStep,
                                                float* pSrcDst, int srcDstStep,
                                                CvSize roiSize, float alpha))

IPCVAPI( CvStatus, icvAddWeighted_8s32f_C1IMR, (const char*  pSrc, int srcStep,
                                                const unsigned char* pMask, int maskStep,
                                                float* pSrcDst, int srcDstStep,
                                                CvSize roiSize, float alpha))

IPCVAPI( CvStatus, icvAddWeighted_32f_C1IMR,   (const float*  pSrc, int srcStep,
                                                const unsigned char* pMask, int maskStep,
                                                float* pSrcDst, int srcDstStep,
                                                CvSize roiSize, float alpha))


/****************************************************************************************\
*                                        Pyramids                                        *
\****************************************************************************************/


/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:       icvPyrUpGetSize_Gauss5x5, icvPyrDownGetSize_Gauss5x5
//  Purpose:    Calculates cyclic buffer size for pyramids.
//  Return:
//    ippStsNoErr              Ok
//    ippStsDepthErr           Specified depth is invalid or unsupported
//    ippStsNumChannensErr     Specified number of channels is invalid or unsupported
//    ippStsSizeErr            The width or height of images is less or equal zero
//                             For icvPyrDown*** if width or height are odd
//                             (non-divisible by 2).
//    ippStsStepErr            The steps in images are less or equal zero
//
//  Arguments:
//    roiWidth                 width of image ROI in pixels
//    depth                    image depth
//    channels                 number of image channels
//    bufSize                  output parameter. Size of cyclic buffer for pyramids.
*/
IPCVAPI( CvStatus, icvPyrUpGetBufSize_Gauss5x5, ( int roiWidth, CvDataType dataType,
                                                  int channels, int* bufSize))

IPCVAPI( CvStatus, icvPyrDownGetBufSize_Gauss5x5, (int roiWidth, CvDataType dataType,
                                                   int channels, int* bufSize))


/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:       icvPyrDown_Gauss5x5_8u_C1R, icvPyrDown_Gauss5x5_8u_C3R,
//              icvPyrDown_Gauss5x5_8s_C1R, icvPyrDown_Gauss5x5_8s_C3R,
//              icvPyrDown_Gauss5x5_32f_C1R, icvPyrDown_Gauss5x5_32f_C3R,
//
//              icvPyrUp_Gauss5x5_8u_C1R, icvPyrUp_Gauss5x5_8u_C3R,
//              icvPyrUp_Gauss5x5_8s_C1R, icvPyrUp_Gauss5x5_8s_C3R,
//              icvPyrUp_Gauss5x5_32f_C1R, icvPyrUp_Gauss5x5_32f_C3R,
//
//  Purpose:    Perform downsampling/upsampling of the image with 5x5 gaussian.
//
//  Return:
//    ippStsNoErr              Ok
//    ippStsNullPtrErr         One of pointers is NULL
//    ippStsSizeErr            The width or height of images is less or equal zero.
//                             For icvPyrDown*** if width or height are odd
//                             (non-divisible by 2).
//    ippStsStepErr            Step is too small to fit image.
//
//  Arguments:
//    pSrc                     pointer to source image
//    srcStep                  step in source image
//    pDst                     pointer to destination image
//    dstStep                  step in source image
//    roiSize                  source image ROI size. Destination image width and
//                             height will be twice large (PyrUp)
//                             or twice smaller (PyrDown)
//    pBuffer                  preallocated temporary cyclic buffer,
//                             which size is calculated using
//                             ippPyrUpGetSize_Gauss_5x5 or ippPyrDownGetSize_Gauss_5x5
*/
IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_8u_C1R,  (const uchar* pSrc, int srcStep,
                                               uchar* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_8u_C3R,  (const uchar* pSrc, int srcStep,
                                               uchar* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_8s_C1R,  (const char* pSrc, int srcStep,
                                               char* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_8s_C3R,  (const char* pSrc, int srcStep,
                                               char* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_32f_C1R, (const float* pSrc, int srcStep,
                                               float* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrUp_Gauss5x5_32f_C3R, (const float* pSrc, int srcStep,
                                               float* pDst, int dstStep,
                                               CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_8u_C1R,  (const uchar* pSrc, int srcStep,
                                                 uchar* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_8u_C3R,  (const uchar* pSrc, int srcStep,
                                                 uchar* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_8s_C1R,  (const char* pSrc, int srcStep,
                                                 char* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_8s_C3R,  (const char* pSrc, int srcStep,
                                                 char* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_32f_C1R, (const float* pSrc, int srcStep,
                                                 float* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

IPCVAPI( CvStatus, icvPyrDown_Gauss5x5_32f_C3R, (const float* pSrc, int srcStep,
                                                 float* pDst, int dstStep,
                                                 CvSize roiSize, void* pBuffer ))

/****************************************************************************************/
/*                              Morphological primitives                                */
/****************************************************************************************/

/****************************************************************************************/
/*                                  Erosion primitives                                  */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//
//  Name(s):   icvErode..., icvDilate
//  Purpose:
//    perform the erosion of the source image ( or its ROI )
//    with the specified structuring element and write result to
//    the destination image.
//  Context:
//  Parameters:
//    pSrc     - source image data
//    srcStep  - width of single row of source image in bytes
//    pDst     - destination image data
//    dstStep  - width of single row of destination image in bytes
//    srcSize  - sizes of ROI of source image in pixels (width and height)
//    B        - structuring element. It contains array of values <-1> & <0> when
//               the corresponded point should be or not be used in calculations.
//  Returns:
//    CV_OK -  success
//    CV_BADFACTOR_ERR - bad arguments: null pointer to source, destination or element.
//    CV_BADSIZE_ERR   - wrong sizes: width or height are non positive,
//                         srcStep or dstStep are less then width of ROI or
//                         then source equals destination but srcStep and dstStep are
//                         different. Also when element has null pointer to data or
//                         its anchor point is out of element rectangle.
//    CV_OUTOFMEM_ERR  - function is unable to allocate interlnal buffers.
//  Notes:
//    Functions support inplace operations. Points of the source image
//    which have an empty intersection of element and image left unchanged.
//F*/
IPCVAPI(CvStatus, icvErodeStrip_Rect_8u_C1R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Rect_8u_C3R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Rect_8u_C4R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Rect_32f_C1R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Rect_32f_C3R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Rect_32f_C4R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_8u_C1R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_8u_C3R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_8u_C4R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_32f_C1R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_32f_C3R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvErodeStrip_Cross_32f_C4R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvErodeStrip_8u_C1R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_8u_C3R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_8u_C4R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvErodeStrip_32f_C1R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_32f_C3R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvErodeStrip_32f_C4R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

/* /////////// Dilation //////////// */
IPCVAPI(CvStatus, icvDilateStrip_Rect_8u_C1R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Rect_8u_C3R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Rect_8u_C4R, ( const uchar* pSrc, int srcStep,
                                            uchar* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Rect_32f_C1R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Rect_32f_C3R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Rect_32f_C4R, ( const float* pSrc, int srcStep,
                                             float* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_8u_C1R, ( const uchar* pSrc, int srcStep,
                                             uchar* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_8u_C3R, ( const uchar* pSrc, int srcStep,
                                             uchar* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_8u_C4R, ( const uchar* pSrc, int srcStep,
                                             uchar* pDst, int dstStep,
                                             CvSize* roiSize,
                                             struct CvMorphState* state,
                                             int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_32f_C1R, ( const float* pSrc, int srcStep,
                                              float* pDst, int dstStep,
                                              CvSize* roiSize,
                                              struct CvMorphState* state,
                                              int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_32f_C3R, ( const float* pSrc, int srcStep,
                                              float* pDst, int dstStep,
                                              CvSize* roiSize,
                                              struct CvMorphState* state,
                                              int stage ))

IPCVAPI(CvStatus, icvDilateStrip_Cross_32f_C4R, ( const float* pSrc, int srcStep,
                                              float* pDst, int dstStep,
                                              CvSize* roiSize,
                                              struct CvMorphState* state,
                                              int stage ))

IPCVAPI(CvStatus, icvDilateStrip_8u_C1R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvDilateStrip_8u_C3R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvDilateStrip_8u_C4R, ( const uchar* pSrc, int srcStep,
                                           uchar* pDst, int dstStep,
                                           CvSize* roiSize,
                                           struct CvMorphState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvDilateStrip_32f_C1R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvDilateStrip_32f_C3R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvDilateStrip_32f_C4R, ( const float* pSrc, int srcStep,
                                            float* pDst, int dstStep,
                                            CvSize* roiSize,
                                            struct CvMorphState* state,
                                            int stage ))

IPCVAPI(CvStatus, icvMorphologyInitAlloc, ( int roiWidth,
                                            CvDataType dataType, int channels,
                                            CvSize elSize, CvPoint elAnchor,
                                            CvElementShape elShape, const int* elData,
                                            struct CvMorphState** morphState ))

IPCVAPI(CvStatus, icvMorphologyFree, ( struct CvMorphState** morphState ))

/****************************************************************************************/
/*                                  Fixed Filter functions                              */
/****************************************************************************************/
#define CV_WHOLE   0
#define CV_START   1
#define CV_END     2
#define CV_MIDDLE  4
/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:   icvBlurInitAlloc
//
//  Purpose:  Allocate buffers and initialize internal state of convolution operation.
//  Return:
//    ippStsNoErr              Ok
//    ippStsDepthErr           Specified depth is invalid or unsupported
//    ippStsSizeErr            The width of image or width or height of structuring
//                             element is less or equal zero.
//    ippStsNullPtrErr         Null pointer to element data when shape is custom
//
//  Arguments:
//    roiWidth                 width of image ROI in pixels
//    depth                    image depth
//    kerSize                  size of kernel element
//    dx                       order of horizontal derivative
//    dy                       order of vertical derivative     
//    state                    output parameter. Double pointer to convolution's state.
*/
IPCVAPI(CvStatus, icvBlurInitAlloc, ( int roiWidth,
									  int depth,
                                      int kerSize,
                                      struct _CvConvState** state ))

/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:   icvSobelInitAlloc
//
//  Purpose:  Allocate buffers and initialize internal state of convolution operation.
//  Return:
//    ippStsNoErr              Ok
//    ippStsDepthErr           Specified depth is invalid or unsupported
//    ippStsSizeErr            The width of image or width or height of structuring
//                             element is less or equal zero.
//    ippStsNullPtrErr         Null pointer to element data when shape is custom
//
//  Arguments:
//    roiWidth                 width of image ROI in pixels
//    depth                    image depth
//    kerSize                  size of structuring element
//    dx                       order of horizontal derivative
//    dy                       order of vertical derivative     
//    state                    output parameter. Double pointer to convolution's state.
*/
IPCVAPI(CvStatus, icvSobelInitAlloc, ( int roiwidth,
                                       int depth,
                                       int kerSize,
                                       int origin,
                                       int  dx,
                                       int  dy,
                                       struct _CvConvState** state ))
/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:   icvLaplaceInitAlloc
//
//  Purpose:  Allocate buffers and initialize internal state of convolution operation.
//  Return:
//    ippStsNoErr              Ok
//    ippStsDepthErr           Specified depth is invalid or unsupported
//    ippStsSizeErr            The width of image or width or height of structuring
//                             element is less or equal zero.
//    ippStsNullPtrErr         Null pointer to element data when shape is custom
//
//  Arguments:
//    roiWidth                 width of image ROI in pixels
//    depth                    image depth
//    kerSize                  size of structuring element
//    direct                   direction of derivative(vertical or horizontal)  
//    state                    output parameter. Double pointer to convolution's state.
*/
IPCVAPI(CvStatus, icvLaplaceInitAlloc, ( int roiWidth,
                                         int depth,
                                         int kerSize,
                                         struct _CvConvState** state ))

/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:   icvConvolFree
//
//  Purpose:  Releases buffers, allocated by icvXXXInitAlloc
//  Return:
//    ippStsNoErr              Ok
//    ippStsNullPtrErr         Null pointer to pointer to convolution's state.
//
//  Arguments:
//    state               double pointer to convolution's state.
*/
IPCVAPI(CvStatus, icvConvolFree, ( struct _CvConvState** state ))


/* ///////////////////////////////////////////////////////////////////////////////////////
//  Name:   icvBlur_8u16s_C1R
//          icvBlur_8s16s_C1R
//          icvBlur_32f_C1R
//          icvLaplace_8u16s_C1R
//          icvLaplace_8s16s_C1R
//          icvLaplace_32f_C1R
//          icvLaplace3x3_8u16s_C1R
//          icvLaplace3x3_8s16s_C1R
//          icvLaplace3x3_32f_C1R
//          icvSobel_8u16s_C1R,
//          icvSobel_8s16s_C1R,
//          icvSobel_32f_C1R,
//          icvScharr_Dx_8u16s_C1R,
//          icvScharr_Dx_8s16s_C1R,
//          icvScharr_Dx_32f_C1R,
//          icvScharr_Dy_8u16s_C1R,
//          icvScharr_Dy_8s16s_C1R,
//          icvScharr_Dy_32f_C1R,
//          icvSobel3x3_Dx_8u16s_C1R,
//          icvSobel3x3_Dx_8s16s_C1R,
//          icvSobel3x3_Dx_32f_C1R,
//          icvSobel3x3_Dy_8u16s_C1R,
//          icvSobel3x3_Dy_8s16s_C1R,
//          icvSobel3x3_Dy_32f_C1R,
//          icvSobel3x3_D2x_8u16s_C1R,
//          icvSobel3x3_D2x_8s16s_C1R,
//          icvSobel3x3_D2x_32f_C1R,
//          icvSobel3x3_D2y_8u16s_C1R,
//          icvSobel3x3_D2y_8s16s_C1R,
//          icvSobel3x3_D2y_32f_C1R,
//          icvSobel3x3_DxDy_8u16s_C1R,
//          icvSobel3x3_DxDy_8s16s_C1R,
//          icvSobel3x3_DxDy_32f_C1R
//
//  Purpose:    Perform convolution operation on image using various separable kernels
//  Return:
//    ippStsNoErr              Ok
//    ippStsDepthErr           Specified depth is invalid or unsupported
//    ippStsSizeErr            The width or height of images is less or equal zero
//                             For icvPyrDown*** if width or height are odd
//                             (non-divisible by 2).
//    ippStsStepErr            The steps in images are less or equal zero
//
//  Arguments:
//    pSrc                     The pointer to source image
//    srcStep                  The step in source image
//    pDst                     The pointer to destination image
//    dstStep                  The step in destination image
//    roiSize                      ROI size
//    state                    Pointer to convolution operation state
//    stage                    Can take of the following values:
//                                0       - function processes a whole image
//                                IPP_START - function initializes cyclic buffer and
//                                            process top part of the large image.
//                                IPP_END - function clears cyclic buffer and
//                                          process bottom part of the large image.
//                                IPP_MIDDLE - function processes middle part
//                                             of the large image.
*/
IPCVAPI(CvStatus, icvBlur_8u16s_C1R, ( const unsigned char* pSrc, int srcStep,
                                        short* pDst, int dstStep,
                                        CvSize* roiSize, struct _CvConvState* state,
                                        int stage ))

IPCVAPI(CvStatus, icvBlur_8s16s_C1R, ( const char* pSrc, int srcStep,
                                        short* pDst, int dstStep,
                                        CvSize* roiSize, struct _CvConvState* state,
                                        int stage ))

IPCVAPI(CvStatus, icvBlur_32f_C1R, ( const float* pSrc, int srcStep,
                                      float* pDst, int dstStep,
                                      CvSize* roiSize, struct _CvConvState* state,
                                      int stage ))

IPCVAPI(CvStatus, icvLaplace_8u16s_C1R, ( const unsigned char* pSrc, int srcStep,
                                           short* pDst, int dstStep,
                                           CvSize* roiSize, struct _CvConvState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvLaplace_8s16s_C1R, ( const char* pSrc, int srcStep,
                                           short* pDst, int dstStep,
                                           CvSize* roiSize, struct _CvConvState* state,
                                           int stage ))

IPCVAPI(CvStatus, icvLaplace_32f_C1R, ( const float* pSrc, int srcStep,
                                         float* pDst, int dstStep,
                                         CvSize* roiSize, struct _CvConvState* state,
                                         int stage ))


IPCVAPI(CvStatus, icvSobel_8u16s_C1R, ( const unsigned char* pSrc, int srcStep,
                                         short* pDst, int dstStep,
                                         CvSize* roiSize, struct _CvConvState* state,
                                         int stage ))

IPCVAPI(CvStatus, icvSobel_8s16s_C1R, ( const char* pSrc, int srcStep,
                                         short* pDst, int dstStep,
                                         CvSize* roiSize, struct _CvConvState* state,
                                         int stage ))

IPCVAPI(CvStatus, icvSobel_32f_C1R, ( const float* pSrc, int srcStep,
                                       float* pDst, int dstStep,
                                       CvSize* roiSize, struct _CvConvState* state,
                                       int stage ))



/****************************************************************************************/
/*                                  Motion Template                                     */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvUpdateMotionHistory_8u32f_C1IR
//    Purpose: The function updates motion history image.
//    Context:
//    Parameters:
//        silIm         - silhouette image
//        silStep       - its step
//        mhiIm         - motion history image
//        mhiStep       - its step
//        size          - size of both images (in pixels)
//        timestamp     - current system time (in seconds)
//        mhi_duration  - maximal duration of motion track before it will
//                        be removed (in seconds too)
//    Returns:
//        IPP_NO_ERR or error code:
//           IPP_NULLPTR_ERR - silIm or mhiIm pointer are null
//           IPP_BADSIZE_ERR   - width or height is negative or steps less than width
//           IPP_BADFACTOR_ERR - mhi_duration is non positive
//    Notes:
//F*/
IPCVAPI( CvStatus , icvUpdateMotionHistory_8u32f_C1IR, ( const uchar* silIm, int silStep,
                                                         float* mhiIm, int mhiStep,
                                                         CvSize size,
                                                         float  timestamp,
                                                         float  mhi_duration ))

/****************************************************************************************/
/*                               Flood Filling functions                                */
/****************************************************************************************/


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: CvFloodFillGetSize, CvFloodFillGetSize_Grad
//    Purpose: The functions calculate size of buffer necessary for calling CvFloodFill
//    Context:
//    Parameters:  
//                 roi        - size of image ROI,
//                 bufferSize - size of buffer in bytes output parameter.
//
//    Returns: CV_NO_ERR or error code
//    Notes:   
//F*/

IPCVAPI(CvStatus,  icvFloodFillGetSize,(CvSize roi, int* bufferSize))

IPCVAPI(CvStatus,   icvFloodFillGetSize_Grad,(CvSize roi, int* bufferSize))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names: icvFloodFill_8uC1R,  icvFloodFill_32fC1R,
//           icvFloodFill8_8u_C1R, icvFloodFill8_32f_C1R
//    Purpose: The functions fill the seed pixel environs inside which all pixel
//             values are not far from each other. 
//    Context:
//    Parameters:  img        - pointer to ROI of initial image (in the beginning)
//                              which is "repainted" during the function action,
//                 step       - full string length of initial image (in bytes),
//                 roi        - size of image ROI,
//                 initPoint  - coordinates of the seed point inside image ROI,
//                 nv         - new value of repainted area pixels,
//                 d_lw, d_up - maximal lower and upper differences of the values of
//                              appurtenant to repainted area pixel and one of its
//                              neighbour,
//                 comp       - pointer to connected component structure of the
//                              repainted area
//
//    Notes:   The 1 and 2 functions look for 4-connected environs,
//             the 3 and 4 ones - 8-connected.
//F*/
/*--------------------------------------------------------------------------------------*/

IPCVAPI(CvStatus, icvFloodFill_4Con_8u_C1IR, ( uchar*  pImage, int step,
                                     CvSize roi,    CvPoint seed,
                                     int newVal, CvConnectedComp* region,
                                     void* pBuffer ))

IPCVAPI(CvStatus,   icvFloodFill_4Con_32f_C1IR, ( float*   pImage, int step,
                                                 CvSize  roi,CvPoint seed,
                                                 float    newVal,
                                                 CvConnectedComp* region,
                                                 void* pBuffer))

IPCVAPI(CvStatus, icvFloodFill_8Con_8u_C1IR, ( uchar*  pImage, int step,
                                     CvSize roi,    CvPoint seed,
                                     int newVal, CvConnectedComp* region,
                                     void* pBuffer ))

IPCVAPI(CvStatus, icvFloodFill_8Con_32f_C1IR,(float*   pImage,
                                            int      step,
                                            CvSize  roi,
                                            CvPoint seed,
                                            float      newVal,
                                            CvConnectedComp* region,
                                            void* pBuffer))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvFloodFill_8uC1R, icvFloodFill_32fC1R
//    Purpose: The function fills the seed pixel enewValirons inside which all pixel
//             values are not far from each other. 
//    Context:
//    Parameters:  pImage     - pointer to ROI of initial image (in the beginning)
//                              which is "repainted" during the function action,
//                 step       - full string length of initial image (in bytes),
//                 roi        - size of image ROI,
//                 seed  - coordinates of the seed point inside image ROI,
//                 newVal         - new value of repainted area pixels,
//                 d_lw, d_up - maximal lower and upper differences of the values of
//                              appurtenant to repainted area pixel and one of its
//                              neighbour,
//                 pPainted   - pointer to counter of repainted pixels number,
//                 rect       - pointer to rectangle circumscribed about the
//                              repainted area
//
//    Returns: CV_NO_ERR or error code
//    Notes:   This function uses a rapid non-recursive algorithm.
//F*/

IPCVAPI(CvStatus, icvFloodFill_Grad4Con_8u_C1IR, ( uchar*   pImage,
                                                   int      step,
                                                   CvSize  roi,
                                                   CvPoint seed,
                                                   int      newVal,
                                                   int      d_lw,
                                                   int      d_up,
                                                   CvConnectedComp* region,
                                                    void* pBuffer))

IPCVAPI(CvStatus, icvFloodFill_Grad4Con_32f_C1IR, ( float*   pImage,
                                                    int      step,
                                                    CvSize  roi,
				                                    CvPoint seed,
                                                    float      newVal,
                                                    float      d_lw,
                                                    float      d_up,
                                                    CvConnectedComp* region,
                                                    void* pBuffer))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvFloodFill8_8u_C1R, icvFloodFill8_32f_C1R
//    Purpose: The function fills the seed pixel enewValirons inside which all pixel
//             values are not far from each other (8-connected variant). 
//    Context:
//    Parameters:  pImage     - pointer to ROI of initial image (in the beginning)
//                              which is "repainted" during the function action,
//                 step       - full string length of initial image (in bytes),
//                 roi        - size of image ROI,
//                 seed  - coordinates of the seed point inside image ROI,
//                 newVal         - new value of repainted area pixels,
//                 d_lw, d_up - maximal lower and upper differences of the values of
//                              appurtenant to repainted area pixel and one of its
//                              neighbour,
//                 pPainted   - pointer to counter of repainted pixels number.
//
//    Returns: CV_NO_ERR or error code
//    Notes:   This function uses a rapid non-recursive algorithm.
//F*/
IPCVAPI(CvStatus, icvFloodFill_Grad8Con_8u_C1IR, ( uchar*   pImage,
                                                   int      step,
                                                   CvSize  roi,
                                                   CvPoint seed,
                                                   int      newVal,
                                                   int      d_lw,
                                                   int      d_up,
                                                   CvConnectedComp* region,
                                                   void* pBuffer))
IPCVAPI(CvStatus, icvFloodFill_Grad8Con_32f_C1IR, (float*   pImage,
                                                   int      step,
                                                   CvSize  roi,
                                                   CvPoint seed,
                                                   float      newVal,
                                                   float      d_lw,
                                                  float      d_up,
                                                   CvConnectedComp* region,
                                                   void* pBuffer))



/****************************************************************************************\
*                                      Template matching                                 *
\****************************************************************************************/

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_SqDiff, ( CvSize roiSize,
                                                        CvSize templSize,
                                                        CvDataType dataType,
                                                        int* bufferSize ))

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_SqDiffNormed, ( CvSize roiSize,
                                                              CvSize templSize,
                                                              CvDataType dataType,
                                                              int* bufferSize))

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_Corr, ( CvSize roiSize,
                                                      CvSize templSize,
                                                      CvDataType dataType,
                                                      int* bufferSize ))

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_CorrNormed, ( CvSize roiSize,
                                                            CvSize templSize,
                                                            CvDataType dataType,
                                                            int* bufferSize ))

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_Coeff, ( CvSize roiSize,
                                                       CvSize templSize,
                                                       CvDataType dataType,
                                                       int* bufferSize ))

IPCVAPI( CvStatus, icvMatchTemplateGetBufSize_CoeffNormed, ( CvSize roiSize,
                                                             CvSize templSize,
                                                             CvDataType dataType,
                                                             int* bufferSize ))


/*  processing functions */
IPCVAPI( CvStatus, icvMatchTemplate_SqDiff_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_SqDiffNormed_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Corr_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CorrNormed_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Coeff_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CoeffNormed_8u32f_C1R,
                                (const uchar* pImage, int imageStep, CvSize roiSize,
                                 const uchar* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_SqDiff_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_SqDiffNormed_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Corr_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CorrNormed_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Coeff_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CoeffNormed_8s32f_C1R,
                                (const char* pImage, int imageStep, CvSize roiSize,
                                 const char* pTemplate, int templStep, CvSize templSize,
                                 float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_SqDiff_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_SqDiffNormed_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Corr_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CorrNormed_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_Coeff_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))

IPCVAPI( CvStatus, icvMatchTemplate_CoeffNormed_32f_C1R,
                               (const float* pImage, int imageStep, CvSize roiSize,
                                const float* pTemplate, int templStep, CvSize templSize,
                                float* pResult, int resultStep, void* pBuffer ))


/****************************************************************************************/
/*                                Distance Transform                                    */
/****************************************************************************************/

IPCVAPI(CvStatus, icvDistanceTransform_3x3_8u32f_C1R, ( const uchar* pSrc, int srcStep,
                                                         float* pDst, int dstStep,
                                                         CvSize roiSize, float* pMetrics ))

IPCVAPI(CvStatus, icvDistanceTransform_5x5_8u32f_C1R, ( const uchar* pImage, int imgStep,
                                                        float* pDist, int distStep,
                                                        CvSize roiSize, float* pMetrics ))

IPCVAPI( CvStatus, icvGetDistanceTransformMask, ( int maskType, float* pMetrics ))

/****************************************************************************************\
*                                   Conversion                                           *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvCopy_8u_C3P3R, icvCopy_8u_C4P4R
//           icvCopy_32f_C3P3R, icvCopy_32f_C4P4R
//
//  Purpose: Convert pixel image to planar image or vice versa.
//           The functions support mode when only a single output plane is
//           specified (non-zero). In this case, the appropriate plane is
//           extracted from source image.
//
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width or height of images is less or equal zero
//    ippStsStepErr       The steps in images are too small
//
//  Parameters:
//    pSrc                source image
//    srcStep             its step
//    pDst                destination images
//    distStep            their step
//    roiSize                 ROI size
//F*/
IPCVAPI(CvStatus, icvCopy_8u_C3P3R, ( const uchar* pSrc, int srcStep,
                                      uchar* pDst[3], int dstStep,
                                      CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_8u_C4P4R, ( const uchar* pSrc, int srcStep,
                                      uchar* pDst[4], int dstStep,
                                      CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_32f_C3P3R, ( const float* pSrc, int srcStep,
                                       float* pDst[3], int dstStep,
                                       CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_32f_C4P4R, ( const float* pSrc, int srcStep,
                                       float* pDst[4], int dstStep,
                                       CvSize roiSize ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvConvert_8u_P3C3R, icvConvert_8u_P4C4R,
//           icvConvert_32f_P3C3R, icvConvert_32f_P4C4R,
//
//  Purpose: Convert planar image to pixel image.
//
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width or height of images is less or equal zero
//    ippStsStepErr       The steps in images are too small
//
//  Parameters:
//    pSrc                source images
//    srcStep             their step
//    pDst                destination image
//    dstStep             its step
//F*/
IPCVAPI(CvStatus, icvCopy_8u_P3C3R, ( const uchar* pSrc[3], int srcStep,
                                      uchar* pDst, int dstStep, CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_8u_P4C4R, ( const uchar* pSrc[4], int srcStep,
                                      uchar* pDst, int dstStep, CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_32f_P3C3R, ( const float* pSrc[3], int srcStep,
                                       float* pDst, int dstStep, CvSize roiSize ))

IPCVAPI(CvStatus, icvCopy_32f_P4C4R, ( const float* pSrc[4], int srcStep,
                                       float* pDst, int dstStep, CvSize roiSize ))

/****************************************************************************************\
*                                 Absolute difference                                    *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvAbsDiff
//    Purpose: calculates absolute difference between two images
//    Context:
//    Parameters:
//      src       - pointer to the first source image or its ROI
//      src_step  - full width of source image
//      dst       - pointer to the second source and (at the same time)
//                  destination image or its ROI
//      dst_step  - full width of destination image
//      size      - width and height of images in pixels
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvAbsDiff_8u_C1R, ( uchar* src1, int src1_step,
                                         uchar* src2, int src2_step,
                                         uchar* dst,  int dst_step,
                                         CvSize size ))

IPCVAPI( CvStatus , icvAbsDiff_32f_C1R, ( float* src1, int src1_step,
                                          float* src2, int src2_step,
                                          float* dst,  int dst_step,
                                          CvSize size ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvAbsDiffC
//    Purpose: calculates absolute difference between image and a scalar value
//    Context:
//    Parameters:
//      src       - pointer to the source image or its ROI
//      src_step  - full width of source image
//      dst       - pointer to the destination image or its ROI
//      dst_step  - full width of destination image
//      size      - width and height of images in pixels
//      value     - scalar value
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvAbsDiffC_8u_C1R, ( uchar* src, int src_step,
                                          uchar* dst, int dst_step,
                                          CvSize size, int value ))

IPCVAPI( CvStatus , icvAbsDiffC_32f_C1R, ( float* src, int src_step,
                                           float* dst, int dst_step,
                                           CvSize size, float value ))


/****************************************************************************************\
*                                     Line sampling                                      *
\****************************************************************************************/

/*F//////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvSampleLine...
//    Purpose:
//      Scans the 8-connected straight line,
//      connecting two specified points and stores the points on the
//      line to the buffer.
//    Context:
//    Parameters:
//      img  - pointer to top-left corner of image ROI
//      step - full width of the image (in bytes)
//      size - width and height of the ROI
//      pt1  - starting point
//      pt2  - ending point
//      buffer - where to store the line points.
//               it must have size enough to store
//               max(abs(pt1.x - pt2.x)+1,abs(pt1.y - pt2.y)+1) pixels.
//    Returns:
//      number of points on the line or nagative number if error
//    Notes:
//F*/
IPCVAPI( int, icvSampleLine_8u_C1R, ( const uchar* img, int step, CvSize size,
                                      uchar* buffer, CvPoint pt1, CvPoint pt2 ))

IPCVAPI( int, icvSampleLine_8u_C3R, ( const uchar* img, int step, CvSize size,
                                      uchar* buffer, CvPoint pt1, CvPoint pt2 ))

IPCVAPI( int, icvSampleLine_32f_C1R, ( const float* img, int step, CvSize size,
                                       float* buffer, CvPoint pt1, CvPoint pt2 ))

IPCVAPI( int, icvSampleLine_32f_C3R, ( const float* img, int step, CvSize size,
                                       float* buffer, CvPoint pt1, CvPoint pt2 ))


/****************************************************************************************/
/*                                Image Statistics                                      */
/****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMinMaxIndx
//    Purpose:  Finds minimum and maximum values in the image and their coordinates
//    Context:
//    Parameters:
//      img  - source image
//      step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//      minVal - pointer to minimum value
//      maxVal - pointer to maximum value
//      minIndx - minimum coordinates
//      maxIndx - maximum coordinates
//    Returns:
//      IPP_NO_ERR if success or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMinMaxIndx_8u_C1R, ( const uchar* img, int step, CvSize size,
                                            float* minVal, float* maxVal,
                                            CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8u_C3CR, ( const uchar* img, int step, CvSize size, int coi,
                                             float* minVal, float* maxVal,
                                             CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8s_C1R, ( const char* img, int step, CvSize size,
                                            float* minVal, float* maxVal,
                                            CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8s_C3CR, ( const char* img, int step, CvSize size, int coi,
                                             float* minVal, float* maxVal,
                                             CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_32f_C1R, ( const float* img, int step, CvSize size,
                                             float* minVal, float* maxVal,
                                             CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_32f_C3CR, ( const float* img, int step, CvSize size, int coi,
                                              float* minVal, float* maxVal,
                                              CvPoint* minIndx, CvPoint* maxIndx ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMinMaxIndx
//    Purpose:  Finds minimum and maximum values in the selected 
//              region of the image and their coordinates
//    Context:
//    Parameters:
//      img  - source image
//      step - its step
//      mask - mask image ( non-zero values indicate region )
//      mask_step - its step
//      size - image size (width and height)
//      coi  - index of channel (if color image)
//      minVal - pointer to minimum value
//      maxVal - pointer to maximum value
//      minIndx - minimum coordinates
//      maxIndx - maximum coordinates
//    Returns:
//      IPP_NO_ERR if success or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMinMaxIndx_8u_C1MR, ( const uchar* img, int step,
                                             const uchar* mask, int mask_step, 
                                             CvSize size,
                                             float* minVal, float* maxVal,
                                             CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8u_C3CMR, ( const uchar* img, int step,
                                              const uchar* mask, int mask_step, 
                                              CvSize size, int coi,
                                              float* minVal, float* maxVal,
                                              CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8s_C1MR, ( const char* img, int step,
                                             const uchar* mask, int mask_step, 
                                             CvSize size,
                                             float* minVal, float* maxVal,
                                             CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_8s_C3CMR, ( const char* img, int step,
                                              const uchar* mask, int mask_step, 
                                              CvSize size, int coi,
                                              float* minVal, float* maxVal,
                                              CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_32f_C1MR, ( const float* img, int step,
                                              const uchar* mask, int mask_step, 
                                              CvSize size,
                                              float* minVal, float* maxVal,
                                              CvPoint* minIndx, CvPoint* maxIndx ))

IPCVAPI( CvStatus , icvMinMaxIndx_32f_C3CMR, ( const float* img, int step,
                                               const uchar* mask, int mask_step, 
                                               CvSize size, int coi,
                                               float* minVal, float* maxVal,
                                               CvPoint* minIndx, CvPoint* maxIndx ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMean
//    Purpose:  Calculates mean value over the pixels 
//              in the selected (by mask) region
//    Context:
//    Parameters:  
//      img  - source image
//      step - its step
//      mask - mask image ( non-zero values indicate region )
//      mask_step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//    Returns:
//      mean value
//    Notes:
//F*/
IPCVAPI( CvStatus, icvMean_8u_C1MR, ( const uchar* img, int step, const uchar* mask, 
                                     int mask_step, CvSize size, double* mean ))

IPCVAPI( CvStatus, icvMean_8u_C3CMR, ( const uchar* img, int step, const uchar* mask,
                                       int mask_step, CvSize size, int coi, double* mean ))

IPCVAPI( CvStatus, icvMean_8s_C1MR, ( const char* img, int step, const uchar* mask, 
                                      int mask_step, CvSize size, double* mean ))

IPCVAPI( CvStatus, icvMean_8s_C3CMR, ( const char* img, int step, const uchar* mask,
                                       int mask_step, CvSize size, int coi, double* mean ))

IPCVAPI( CvStatus, icvMean_32f_C1MR, ( const float* img, int step, const uchar* mask, 
                                       int mask_step, CvSize size, double* mean ))

IPCVAPI( CvStatus, icvMean_32f_C3CMR, ( const float* img, int step, const uchar* mask,
                                        int mask_step, CvSize size, int coi, double* mean ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMean_StdDev
//    Purpose:  Calculates mean value over the pixels in the ROI
//    Context:
//    Parameters:  
//      img  - source image
//      step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//      mean - pointer to mean value
//      stddev - pointer to standard deviation
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMean_StdDev_8u_C1R, ( const uchar* img, int step, CvSize size,
                                             double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8u_C3CR, ( const uchar* img, int step,
                                              CvSize size, int coi,
                                              double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8s_C1R, ( const char* img, int step, CvSize size,
                                             double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8s_C3CR, ( const char* img, int step,
                                              CvSize size, int coi,
                                              double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_32f_C1R, ( const float* img, int step, CvSize size,
                                              double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_32f_C3CR, ( const float* img, int step,
                                               CvSize size, int coi,
                                               double* mean, double* stddev ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: icvMean_StdDev_...M
//    Purpose:  Calculates mean value over the pixels in the selected region
//    Context:
//    Parameters:  
//      img  - source image
//      step - its step
//      mask - mask image ( non-zero values indicate region )
//      mask_step - its step
//      size - its size (width and height)
//      coi  - index of channel (if color image)
//      mean - pointer to mean value
//      stddev - pointer to standard deviation
//    Returns:
//      IPP_NO_ERR or error code
//    Notes:
//F*/
IPCVAPI( CvStatus , icvMean_StdDev_8u_C1MR, ( const uchar* img, int step,
                                              const uchar* mask, int mask_step,
                                              CvSize size,
                                              double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8u_C3CMR, ( const uchar* img, int step,
                                               const uchar* mask, int mask_step,
                                               CvSize size, int coi,
                                               double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8s_C1MR, ( const char* img, int step,
                                              const uchar* mask, int mask_step,
                                              CvSize size,
                                              double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_8s_C3CMR, ( const char* img, int step,
                                               const uchar* mask, int mask_step,
                                               CvSize size, int coi,
                                               double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_32f_C1MR, ( const float* img, int step,
                                               const uchar* mask, int mask_step,
                                               CvSize size,
                                               double* mean, double* stddev ))
IPCVAPI( CvStatus , icvMean_StdDev_32f_C3CMR, ( const float* img, int step,
                                                const uchar* mask, int mask_step,
                                                CvSize size, int coi,
                                                double* mean, double* stddev ))


/****************************************************************************************/
/*                                Norms with mask                                       */
/****************************************************************************************/
/****************************************************************************************/
/*                                Norm types defines                                          */
/****************************************************************************************/

#define _CV_C            1
#define _CV_L1           2
#define _CV_L2           4
#define _CV_RELATIVE     8

#define _CV_RELATIVE_C   (_CV_RELATIVE | _CV_C)
#define _CV_RELATIVE_L1  (_CV_RELATIVE | _CV_L1)
#define _CV_RELATIVE_L2  (_CV_RELATIVE | _CV_L2)

#define _CV_DIFF        16

#define _CV_DIFF_C   (_CV_DIFF | _CV_C)
#define _CV_DIFF_L1  (_CV_DIFF | _CV_L1)
#define _CV_DIFF_L2  (_CV_DIFF | _CV_L2)

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Names:   icvNormDiff_Inf_8u_C1MR,  icvNormDiff_Inf_8s_C1MR,  icvNormDiff_Inf_32f_C1MR, 
//             icvNormDiff_Inf_8u_C3CMR, icvNormDiff_Inf_8s_C3CMR, icvNormDiff_Inf_32f_C3CMR,
//             icvNormDiff_L1_8u_C1MR,   icvNormDiff_L1_8s_C1MR,   icvNormDiff_L1_32f_C1MR, 
//             icvNormDiff_L1_8u_C3CMR,  icvNormDiff_L1_8s_C3CMR,  icvNormDiff_L1_32f_C3CMR, 
//             icvNormDiff_L2_8u_C1MR,   icvNormDiff_L2_8s_C1MR,   icvNormDiff_L2_32f_C1MR, 
//             icvNormDiff_L2_8u_C3CMR,  icvNormDiff_L2_8s_C3CMR,  icvNormDiff_L2_32f_C3CMR
// 
//    Purpose: Calculate infinite, 1st power or 2nd power norm of an image, as well as
//             difference or relative norm of two images, with mask
//
//    Parameters:
//         pSrc    (pSrc1, pSrc2),       pMask    - pointers to source image(s)
//                                                  and mask array,
//         srcStep (src1Step, src2Step), maskStep - their steps,
//         roiSize                                - their size or ROI size,
//         coi                                    - COI index (1..3) (if 3-channel images)
//
//    Returns:
//         Cv status
//
//    Notes:
//F*/
IPCVAPI( CvStatus, icvNorm_Inf_8u_C1MR, ( const uchar* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_Inf_8s_C1MR, ( const  char* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_Inf_32f_C1MR, ( const float* pSrc,  int srcStep,
                                          const uchar* pMask, int maskStep,
                                          CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_Inf_8u_C3CMR, ( const uchar* pSrc,  int srcStep,
                                          const uchar* pMask, int maskStep,
                                          CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_Inf_8s_C3CMR, ( const  char* pSrc,  int srcStep,
                                          const uchar* pMask, int maskStep,
                                          CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_Inf_32f_C3CMR, ( const float* pSrc,  int srcStep,
                                           const uchar* pMask, int maskStep,
                                           CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                             const uchar* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                             const char* pSrc2,  int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                              const float* pSrc2, int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                              const uchar* pSrc2, int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                              const char* pSrc2,  int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_Inf_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                               const float* pSrc2, int srcStep2,
                                               const uchar* pMask, int maskStep,
                                               CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                            const uchar* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                            const char* pSrc2,  int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                             const float* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                             const uchar* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                             const char* pSrc2,  int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_Inf_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                              const float* pSrc2, int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize, int coi, double* norm ) )
/*......................................................................................*/
IPCVAPI( CvStatus, icvNorm_L1_8u_C1MR, ( const uchar* pSrc,  int srcStep,
                                        const uchar* pMask, int maskStep,
                                        CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L1_8s_C1MR, ( const  char* pSrc,  int srcStep,
                                        const uchar* pMask, int maskStep,
                                        CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L1_32f_C1MR, ( const float* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L1_8u_C3CMR, ( const uchar* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_L1_8s_C3CMR, ( const  char* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_L1_32f_C3CMR, ( const float* pSrc,  int srcStep,
                                          const uchar* pMask, int maskStep,
                                          CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                            const uchar* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                            const char* pSrc2,  int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                             const float* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                             const uchar* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                             const char* pSrc2,  int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L1_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                              const float* pSrc2, int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                           const uchar* pSrc2, int srcStep2,
                                           const uchar* pMask, int maskStep,
                                           CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                           const char* pSrc2,  int srcStep2,
                                           const uchar* pMask, int maskStep,
                                           CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                            const float* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                            const uchar* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                            const char* pSrc2,  int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L1_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                             const float* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )
/*......................................................................................*/
IPCVAPI( CvStatus, icvNorm_L2_8u_C1MR, ( const uchar* pSrc,  int srcStep,
                                        const uchar* pMask, int maskStep,
                                        CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L2_8s_C1MR, ( const  char* pSrc,  int srcStep,
                                        const uchar* pMask, int maskStep,
                                        CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L2_32f_C1MR, ( const float* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNorm_L2_8u_C3CMR, ( const uchar* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_L2_8s_C3CMR, ( const  char* pSrc,  int srcStep,
                                         const uchar* pMask, int maskStep,
                                         CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNorm_L2_32f_C3CMR, ( const float* pSrc,  int srcStep,
                                          const uchar* pMask, int maskStep,
                                          CvSize roiSize,     int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                            const uchar* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                            const char* pSrc2,  int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                             const float* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                             const uchar* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                             const char* pSrc2,  int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormDiff_L2_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                              const float* pSrc2, int srcStep2,
                                              const uchar* pMask, int maskStep,
                                              CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_8u_C1MR, ( const uchar* pSrc1, int srcStep1,
                                           const uchar* pSrc2, int srcStep2,
                                           const uchar* pMask, int maskStep,
                                           CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_8s_C1MR, ( const char* pSrc1,  int srcStep1,
                                           const char* pSrc2,  int srcStep2,
                                           const uchar* pMask, int maskStep,
                                           CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_32f_C1MR, ( const float* pSrc1, int srcStep1,
                                            const float* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize,     double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_8u_C3CMR, ( const uchar* pSrc1, int srcStep1,
                                            const uchar* pSrc2, int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_8s_C3CMR, ( const char* pSrc1,  int srcStep1,
                                            const char* pSrc2,  int srcStep2,
                                            const uchar* pMask, int maskStep,
                                            CvSize roiSize, int coi, double* norm ) )

IPCVAPI( CvStatus, icvNormRel_L2_32f_C3CMR, ( const float* pSrc1, int srcStep1,
                                             const float* pSrc2, int srcStep2,
                                             const uchar* pMask, int maskStep,
                                             CvSize roiSize, int coi, double* norm ) )

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvbFastAcrtan32f
//    Purpose: block version of the previous function
//    Context:
//    Parameters:
//      y     - ordinate vector
//      x     - abscissa vector
//      angle - resultant vector
//      len   - vectors length
//    Returns:
//      IPP_NO_ERR or IPP_BADFACTOR_ERR if x or y or angle pointers are NULL or
//      len is negative.
//    Notes:
//      if x[i] == 0 && y[i] == 0 for some i -> angle[i] is 0 too.
//F*/
IPCVAPI( CvStatus , icvbFastArctan_32f, ( const float* y, const float* x,
                                          float* angle, int len ))
/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvMinEigenValGetSize
//
//  Purpose: Calculates size of temporary buffer, required to run one of MinEigenVal***
//           functions.
//
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width is less or equal zero
//    ippStsBadRange      Aperture size or average window size are less than 3 or even.
//                        Or depth is invalid or unsupported.
//  Parameters:
//    roiWidth            width of source image ROI
//    depth               source image depth
//    apertureSize        linear size of derivative filter aperture
//    avgWindow           linear size of averaging window
//    bufferSize          output parameter. Calculated buffer size.
//F*/
IPCVAPI(CvStatus, icvMinEigenValGetSize, ( int roiWidth,
                                           int apertureSize, int avgWindow,
                                           int* bufferSize ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvMinEigenVal_8u_C1R, icvMinEigenVal_8s_C1R, icvMinEigenVal_32f_C1R
//
//  Purpose: Calculate minimal eigen value of 2x2 autocorrelation gradient matrix
//           for every pixel. Pixels with relatively large minimal eigen values
//           are strong corners on the picture.
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width or height of images is less or equal zero
//    ippStsStepErr       The steps in images are too small
//    ippStsBadRange      Aperture size or average window size are less than 3 or even.
//
//  Parameters:
//    pSrc                source image
//    srcStep             its step
//    pMinEigenVal        image, filled with minimal eigen values for every pixel
//    minValStep          its step
//    roiSize                 ROI size
//    apertureSize        linear size of derivative filter aperture
//    avgWindow           linear size of averaging window
//    pBuffer             preallocated temporary buffer, which size can be calculated
//                        using icvMinEigenValGetSize function
//F*/
IPCVAPI(CvStatus, icvMinEigenVal_8u32f_C1R, ( const unsigned char* pSrc, int srcStep,
                                               float* pMinEigenVal, int minValStep,
                                               CvSize roiSize, int apertureSize,
                                               int avgWindow, void* pBuffer ))

IPCVAPI(CvStatus, icvMinEigenVal_8s32f_C1R, ( const char* pSrc, int srcStep,
                                               float* pMinEigenVal, int minValStep,
                                               CvSize roiSize, int apertureSize,
                                               int avgWindow, void* pBuffer ))

IPCVAPI(CvStatus, icvMinEigenVal_32f_C1R, ( const float* pSrc, int srcStep,
                                             float* pMinEigenVal, int minValStep,
                                             CvSize roiSize, int apertureSize,
                                             int avgWindow, void* pBuffer ))


/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvEigenValsVecsGetSize
//
//  Purpose: Calculates size of temporary buffer, required to run one of EigenValsVecs***
//           functions.
//
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width is less or equal zero
//    ippStsBadRange      Aperture size or average window size are less than 3 or even.
//                        Or depth is invalid or unsupported.
//  Parameters:
//    roiWidth            width of source image ROI
//    depth               source image depth
//    apertureSize        linear size of derivative filter aperture
//    avgWindow           linear size of averaging window
//    bufferSize          output parameter. Calculated buffer size.
//F*/
IPCVAPI(CvStatus, icvEigenValsVecsGetSize, ( int roiWidth,int apertureSize,
                                              int avgWindow, int* bufferSize ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvEigenValsVecs_8u_C1R, icvEigenValsVecs_8s_C1R, icvEigenValsVecs_32f_C1R
//
//  Purpose: Calculate both eigen values and eigen vectors of 2x2 autocorrelation
//           gradient matrix for every pixel. Can be used for sophisticated
//           edge and corner detection
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width or height of images is less or equal zero
//    ippStsStepErr       The steps in images are too small
//    ippStsBadRange      Aperture size or average window size are less than 3 or even.
//
//  Parameters:
//    pSrc                source image
//    srcStep             its step
//    pEigenVV            image, which is 6 times wider that source image,
//                        filled with 6-tuples:
//                        (eig_val1, eig_val2, eig_vec1_x, eig_vec1_y,
//                         eig_vec2_x, eig_vec2_y)
//    eigStep             output image step 
//    roiSize                 ROI size
//    apertureSize        linear size of derivative filter aperture
//    avgWindow           linear size of averaging window
//    pBuffer             preallocated temporary buffer, which size can be calculated
//                        using icvEigenValsVecsGetSize function
//F*/
IPCVAPI(CvStatus, icvEigenValsVecs_8u32f_C1R, ( const unsigned char* pSrc, int srcStep,
                                                 float* pEigenVV, int eigStep,
                                                 CvSize roiSize, int apertureSize,
                                                 int avgWindow, void* pBuffer ))

IPCVAPI(CvStatus, icvEigenValsVecs_8s32f_C1R, ( const char* pSrc, int srcStep,
                                                 float* pEigenVV, int eigStep,
                                                 CvSize roiSize, int apertureSize,
                                                 int avgWindow, void* pBuffer ))

IPCVAPI(CvStatus, icvEigenValsVecs_32f_C1R, ( const float* pSrc, int srcStep,
                                               float* pEigenVV, int eigStep,
                                               CvSize roiSize, int apertureSize,
                                               int avgWindow, void* pBuffer ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvCannyGetSize
//
//  Purpose: Calculates size of temporary buffer, required to run Canny function.
//
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width is less or equal zero
//  Parameters:
//    roiSize                 source image ROI
//    bufferSize          output parameter. Calculated buffer size.
//F*/
IPCVAPI( CvStatus, icvCannyGetSize, ( CvSize roiSize, int* bufferSize ))

/*F///////////////////////////////////////////////////////////////////////////////////////
//  Name:    icvCanny_16s8u_C1IR
//
//  Purpose: Creates binary image of source's image edges,
//           using derivatives of the first order.
//  Return:
//    ippStsNoErr         Ok
//    ippStsNullPtrErr    One of pointers is NULL
//    ippStsSizeErr       The width or height of images is less or equal zero
//    ippStsStepErr       The steps in images are too small
//
//  Parameters:
//    pDx                 derivative of the source image with respect to x
//    xStep               step of the array of x derivatives
//    pDy                 derivative of the source image with respect to y
//    yStep               step of the array of y derivatives
//    roiSize                 ROI size
//    lowThresh           low threshold for hysteresis thresholding
//    highThresh          high threshold for hysteresis thresholding
//    edges               pointerto the output array
//    edgestep            its step
//    pBuffer             preallocated temporary buffer, which size can be calculated
//                        using icvEigenValsVecsGetSize function
//F*/
IPCVAPI(CvStatus, icvCanny_16s8u_C1R, ( short* pSrcDx, int srcDxStep,
                                          short* pSrcDy, int srcDyStep,
                                          unsigned char*  pDstEdges, int dstEdgeStep, 
                                          CvSize roiSize,
                                          float  lowThresh,
                                          float  highThresh,
                                          void*   pBuffer ))


/*//////////////////////////////////////////////////////////////////////////////*\
**********************************************************************************
\*//////////////////////////////////////////////////////////////////////////////*/
IPCVAPI(CvStatus, icvMomentsBinary_8s_C1R, (const char *img, int step, CvSize size,
                                            CvMoments * moments) )

IPCVAPI(CvStatus, icvMomentsBinary_8s_C3CR, (const char *img, int step, CvSize size,
                                             int coi, CvMoments * moments) )

IPCVAPI(CvStatus, icvMinAreaRect, (CvPoint * points, int n,
                                   int left, int bottom, int right, int top,
                                   CvPoint2D32f * anchor,
                                   CvPoint2D32f * vect1, CvPoint2D32f * vect2) )

IPCVAPI(CvStatus, icvCalcHist8uC1R, (uchar ** img, int step, CvSize size,
                                     CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcHist8sC1R, (char **img, int step, CvSize size,
                                     CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcHist32fC1R, (float **img, int step, CvSize size,
                                      CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcHistMask8uC1R, (uchar ** img, int step,
                                         uchar * mask, int mask_step,
                                         CvSize size,
                                         CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcHistMask8sC1R, (char **img, int step,
                                          uchar * mask, int mask_step,
                                          CvSize size,
                                          CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcHistMask32fC1R, (float **img, int step,
                                          uchar * mask, int mask_step,
                                          CvSize size,
                                          CvHistogram * hist, int dont_clear) )

IPCVAPI(CvStatus, icvCalcBackProject8uC1R, (uchar ** img, int step,
                                            uchar * dst, int dst_step,
                                            CvSize size, CvHistogram * hist) )

IPCVAPI(CvStatus, icvCalcBackProject8sC1R, (char **img, int step,
                                            char *dst, int dst_step,
                                            CvSize size, CvHistogram * hist) )

IPCVAPI(CvStatus, icvCalcBackProject32fC1R, (float **img, int step,
                                             float *dst, int dst_step,
                                             CvSize size, CvHistogram * hist) )

IPCVAPI(CvStatus, icvCalcBackProjectPatch8uC1R, (uchar ** src, int src_step,
                                                 float *dst, int dst_step,
                                                 CvSize roi,
                                                 CvSize range,
                                                 CvHistogram * hist,
                                                 CvCompareMethod method,
                                                 float norm_factor) )

IPCVAPI(CvStatus, icvCalcBackProjectPatch8sC1R, (char **src, int src_step,
                                                 float *dst, int dst_step,
                                                 CvSize roi,
                                                 CvSize range,
                                                 CvHistogram * hist,
                                                 CvCompareMethod method,
                                                 float norm_factor) )

IPCVAPI(CvStatus, icvCalcBackProjectPatch32fC1R, (float **src, int src_step,
                                                  float *dst, int dst_step,
                                                  CvSize roi,
                                                  CvSize range,
                                                  CvHistogram * hist,
                                                  CvCompareMethod method,
                                                  float norm_factor) )

IPCVAPI(CvStatus, icvSegmentMotion_32f_C1R, (float *mhiIm,
                                             int mhiStep,
                                             CvSize size,
                                             float timestamp,
                                             float SegThresh,
                                             float *SegMask,
                                             int SMStep,
                                             CvMemStorage * storage,
                                             CvSeq ** components) )

IPCVAPI(CvStatus, icvCalcOpticalFlowPyrLK_8uC1R, (uchar * imgA,
                                                  uchar * imgB,
                                                  int imgStep,
                                                  CvSize imgSize,
                                                  uchar * pyrA,
                                                  uchar * pyrB,
                                                  CvPoint2D32f * featuresA,
                                                  CvPoint2D32f * featuresB,
                                                  int count,
                                                  CvSize winSize,
                                                  int level,
                                                  char *status,
                                                  float *error,
                                                  CvTermCriteria criteria, int flags) )

IPCVAPI(CvStatus, icvCalcAffineFlowPyrLK_8uC1R, (uchar * imgA, uchar * imgB,
                                                 int imgStep, CvSize imgSize,
                                                 uchar * pyrA, uchar * pyrB,
                                                 CvPoint2D32f * featuresA,
                                                 CvPoint2D32f * featuresB,
                                                 float *matrices, int count,
                                                 CvSize winSize, int level,
                                                 char *status, float *error,
                                                 CvTermCriteria criteria, int flags) )

IPCVAPI(CvStatus, icvCalibrateCamera, (int numImages,
                                       int *numPoints,
                                       CvSize imageSize,
                                       CvPoint2D32f * imagePoints32f,
                                       CvPoint3D32f * objectPoints32f,
                                       CvVect32f distortion32f,
                                       CvMatr32f cameraMatrix32f,
                                       CvVect32f transVects32f,
                                       CvMatr32f rotMatrs32f, int useIntrinsicGuess) )

IPCVAPI(CvStatus, icvFindExtrinsicCameraParams, (int numPoints,
                                                  CvSize imageSize,
                                                  CvPoint2D32f * imagePoints32f,
                                                  CvPoint3D32f * objectPoints32f,
                                                  CvVect32f focalLength32f,
                                                  CvPoint2D32f principalPoint32f,
                                                  CvVect32f distortion32f,
                                                  CvVect32f rotVect32f,
                                                  CvVect32f transVect32f) )

#endif /*__CV_H_*/

