/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#ifndef _CVTYPES_H_
#define _CVTYPES_H_

#include <assert.h>

#ifndef WIN32
    #define CV_CDECL
    #define CV_STDCALL
#else
    #define CV_CDECL __cdecl
    #define CV_STDCALL __stdcall
#endif

#ifndef CV_EXTERN_C
    #ifdef __cplusplus
        #define CV_EXTERN_C extern "C"
        #define CV_DEFAULT(val) = val
    #else
        #define CV_EXTERN_C
        #define CV_DEFAULT(val)
    #endif
#endif

#if defined WIN32 && defined CV
#define CV_DLL_ENTRY __declspec(dllexport)
#else
#define CV_DLL_ENTRY
#endif

#ifndef OPENCVAPI
    #define OPENCVAPI CV_EXTERN_C CV_DLL_ENTRY
#endif

#ifndef CV_INLINE
    #if defined _MSC_VER || defined __GNUC__
        #define CV_INLINE __inline
    #elif defined __ICL
        #define CV_INLINE static
    #else
        #define CV_INLINE __inline
    #endif
#endif /* CV_INLINE */

#define CV_SWAP(a,b,t) ( (t) = (a), (a) = (b), (b) = (t))
#define CV_DIM(array)  (sizeof(array)/sizeof((array)[0]))

#ifndef MIN
#define MIN(a,b)  ((a) > (b) ? (b) : (a))
#endif

#ifndef MAX
#define MAX(a,b)  ((a) < (b) ? (b) : (a))
#endif

typedef enum CvStatus
{         
    CV_INPLACE_NOT_SUPPORTED_ERR= -112,
    CV_UNMATCHED_ROI_ERR        = -111,
    CV_NOTFOUND_ERR             = -110,
    CV_BADCONVERGENCE_ERR       = -109,

    CV_BADDEPTH_ERR             = -107,
    CV_BADROI_ERR               = -106,
    CV_BADHEADER_ERR            = -105,
    CV_UNMATCHED_FORMATS_ERR    = -104,
    CV_UNSUPPORTED_COI_ERR      = -103,
    CV_UNSUPPORTED_CHANNELS_ERR = -102,
    CV_UNSUPPORTED_DEPTH_ERR    = -101,
    CV_UNSUPPORTED_FORMAT_ERR   = -100,

                       
    CV_BADARG_ERR      = -49,  //ipp comp
    CV_NOTDEFINED_ERR  = -48,  //ipp comp

    CV_BADCHANNELS_ERR = -47,  //ipp comp
    CV_BADRANGE_ERR    = -44,  //ipp comp
    CV_BADSTEP_ERR     = -29,  //ipp comp
   
    CV_BADFLAG_ERR     =  -12,
    CV_DIV_BY_ZERO_ERR =  -11, //ipp comp
    CV_BADCOEF_ERR     =  -10,
    CV_BADQUAD_ERR     =  -9,
    CV_BADINTER_ERR    =  -8,

    CV_BADFACTOR_ERR   =  -7,
    CV_BADPOINT_ERR    =  -6,
    CV_BADSCALE_ERR    =  -4,
    CV_OUTOFMEM_ERR    =  -3,
    CV_NULLPTR_ERR     =  -2,
    CV_BADSIZE_ERR     =  -1,
    CV_NO_ERR          =   0,
    CV_OK              =   CV_NO_ERR
} CvStatus;

typedef enum CvMatType
{
    CV_MAT32F     = 32,
    CV_MAT3x1_32F = 35,
    CV_MAT4x1_32F = 36,
    CV_MAT3x3_32F = 41,
    CV_MAT4x4_32F = 48,
    CV_MAT64D     = 64,
    CV_MAT3x1_64D = 67,
    CV_MAT4x1_64D = 68,
    CV_MAT3x3_64D = 73,
    CV_MAT4x4_64D = 80
} CvMatType;

typedef struct CvMat
{
    int rows;      
    int cols; 
    CvMatType type;
    int step;
    union
    {
        float* fl;
        double* db;
    }data;
} CvMat;

typedef struct CvMatArray
{
    int rows;
    int cols;
    int type;
    int step;
    int count;
    union
    {
        float* fl;
        double* db;
    } data;
} CvMatArray;

typedef struct CvRect
{
    int x;
    int y;
    int width;
    int height;
} CvRect;

typedef struct CvPoint
{
    int x;
    int y;
} CvPoint;

typedef struct {
    int width;
    int height;
} CvSize;


typedef float*   CvVect32f;
typedef float*   CvMatr32f;
typedef double*  CvVect64d;
typedef double*  CvMatr64d;

#ifdef _MSC_VER
typedef __int64 int64;
typedef unsigned __int64 uint64;
#else
typedef long long int64;
typedef unsigned long long uint64;
typedef long long _int64;
#endif

#define CV_TERMCRIT_ITER    1
#define CV_TERMCRIT_NUMB    CV_TERMCRIT_ITER
#define CV_TERMCRIT_EPS     2

typedef struct CvTermCriteria
{
    int   type;  /* may be combination of
                 CV_TERMCRIT_ITER
                 CV_TERMCRIT_EPS */
    int   maxIter;
    float epsilon;
} CvTermCriteria;


typedef struct CvMatrix3
{
    float m[3][3];
} CvMatrix3;

typedef struct CvMatrix4
{
    float m[4][4];
} CvMatrix4;

typedef struct CvPoint2D32f
{
    float x;
    float y;
} CvPoint2D32f;

typedef struct CvPoint3D32f
{
    float x;
    float y;
    float z;
} CvPoint3D32f;

typedef struct CvPoint2D64d
{
    double x;
    double y;
} CvPoint2D64d;

typedef struct CvPoint3D64d
{
    double x;
    double y;
    double z;
} CvPoint3D64d;

typedef struct CvSize2D32f
{
    float width;
    float height;
} CvSize2D32f;

typedef struct CvBox2D
{
    CvPoint2D32f center;
    CvSize2D32f  size;
    float angle;
} CvBox2D;

#define CvBox2D32f CvBox2D

typedef enum CvCoeffType
{
    CV_VALUE = 1,
    CV_ARRAY = 2
} CvCoeffType;

typedef struct CvSlice
{
    int  startIndex, endIndex;
} CvSlice;

#define CV_CALIPERS_MAXHEIGHT      0
#define CV_CALIPERS_MINAREARECT    1
#define CV_CALIPERS_MAXDIST        2

#define CV_CLOCKWISE         1
#define CV_COUNTER_CLOCKWISE 2


CV_INLINE CvMat cvMat (int rows,int cols, CvMatType type, void* data)
{
    CvMat m;

    m.cols = cols;
    m.rows = rows;
    m.type = type;
    m.step = 0;
    m.data.fl =(float*)data;

    return m; 
}


CV_INLINE CvMatArray cvMatArray (int rows,int cols, CvMatType type, int number, void* data)
{
    CvMatArray ma;

    ma.cols = cols;
    ma.rows =rows;
    ma.type = type;
    ma.count = number;
    ma.step = 0;
    ma.data.fl =(float*)data;

    return ma; 
}


CV_INLINE double cvmGet(CvMat* mat,int i, int j)
{
    double elem;

    if( (int)mat->type < 64 )
        elem = (double)mat->data.fl[i*mat->cols+j];
    else
        elem = mat->data.db[i*mat->cols+j];

    return elem;
}


CV_INLINE void cvmSet(CvMat* mat, int i, int j, double val)
{
    if( (int)mat->type < 64 )
        mat->data.fl[i*mat->cols+j] = (float)val;
    else
        mat->data.db[i*mat->cols+j] = val;
}


CV_INLINE CvMat* cvmGetMatArrayElem( CvMatArray* array, int index, CvMat* elem )
{
    assert( elem->rows == array->rows && elem->cols == array->cols &&
            elem->type == array->type && index >= 0 && index < array->count );

    if( array->type < CV_MAT64D )
        elem->data.fl = array->data.fl + array->rows*array->cols*index;
    else
        elem->data.db = array->data.db + array->rows*array->cols*index;

    return elem;
}


CV_INLINE CvRect cvRect( int x, int y, int width, int height )
{
    CvRect r;

    r.x = x;
    r.y = y;
    r.width = width;
    r.height = height;

    return r;
}


CV_INLINE CvSize cvSize( int width, int height )
{
    CvSize s;

    s.width = width;
    s.height = height;

    return s;
}


CV_INLINE CvPoint cvPoint( int x, int y )
{
    CvPoint p;

    p.x = x;
    p.y = y;

    return p;
}


CV_INLINE CvPoint2D32f cvPoint2D32f( float x, float y )
{
    CvPoint2D32f p;

    p.x = x;
    p.y = y;

    return p;
}


CV_INLINE CvTermCriteria cvTermCriteria( int type, int maxIter, double epsilon )
{
    CvTermCriteria t;

    t.type = type;
    t.maxIter = maxIter;
    t.epsilon = (float)epsilon;

    return t;
}


CV_INLINE CvSlice cvSlice( int start, int end )
{
    CvSlice slice;
    slice.startIndex = start;
    slice.endIndex = end;

    return slice;
}

#define CV_WHOLE_SEQ(seq)  cvSlice(0, 0x3fffffff)

/* substitutions for (int)floor(x+.49999), floor(x), ceil(x) */
CV_INLINE int cvRound( double val )
{
    static const int round_delta = 0x59C00000;
    double temp = (val + 1e-7) + *((float*)&round_delta);
    return *((int*)&temp);
}


CV_INLINE int cvFloor( double val )
{
    static const int round_delta = 0x59C00000;
    double temp = (val - 0.4999999) + *((float*)&round_delta);
    return *((int*)&temp);
}


CV_INLINE int cvCeil( double val )
{
    static const int round_delta = 0x59C00000;
    double temp = (val + 0.4999999) + *((float*)&round_delta);
    return *((int*)&temp);
}

#define CV_SQRT_MAGIC  0xbe6f0000

CV_INLINE float cvInvSqrt( float arg )
{
    float x, y;
    unsigned iarg = *((unsigned*)&arg);
    *((unsigned*)&x) = (CV_SQRT_MAGIC - iarg)>>1;

    y = arg*0.5f;
    x*= 1.5f - y*x*x;
    x*= 1.5f - y*x*x;

    return x;
}


CV_INLINE float cvSqrt( float arg )
{
    float x, y;
    unsigned iarg = *((unsigned*)&arg);
    *((unsigned*)&x) = (CV_SQRT_MAGIC - iarg)>>1;

    y = arg*0.5f;
    x*= 1.5f - y*x*x;
    x*= 1.5f - y*x*x;

    return x*arg;
}


/* RNG state */
typedef struct CvRandState
{
    unsigned  seed;    /* current 32-bit pseudo-random number */
    unsigned  carry;   /* carry (upper half of 64-bit random number) */
    float     factor;  /* (upper - lower) */
    float     delta;   /* lower - (upper - lower) */
} CvRandState;


/* Defines for Threshold functions */
typedef enum CvThreshType
{
    CV_THRESH_BINARY     = 0,  /* val = (val>thresh? MAX:0)      */
    CV_THRESH_BINARY_INV = 1,  /* val = (val>thresh? 0:MAX)      */
    CV_THRESH_TRUNC      = 2,  /* val = (val>thresh? thresh:val) */
    CV_THRESH_TOZERO     = 3,  /* val = (val>thresh? val:0)      */
    CV_THRESH_TOZERO_INV = 4   /* val = (val>thresh? 0:val)      */
} CvThreshType;

typedef enum CvAdaptiveThreshMethod
{
    CV_STDDEV_ADAPTIVE_THRESH  = 0   /*  method for the defining local adaptive threshold  */
}
CvAdaptiveThreshMethod;

/* Defines for Distance Transform */
typedef enum CvDisType
{
    CV_DIST_USER         = -1, /* User defined distance */
    CV_DIST_L1           = 1, /* distance = |x1-x2| + |y1-y2| */
    CV_DIST_L2           = 2, /* the simple euclidean distance */
    CV_DIST_C            = 3, /* distance = max(|x1-x2|,|y1-y2|) */
    CV_DIST_L12          = 4, /* L1-L2 metric: distance = 2(sqrt(1+x*x/2) - 1)) */
    CV_DIST_FAIR         = 5, /* distance = c^2(|x|/c-log(1+|x|/c)), c = 1.3998 */
    CV_DIST_WELSCH       = 6 /* distance = c^2/2(1-exp(-(x/c)^2)), c = 2.9846 */
} CvDisType;

typedef enum CvDisMaskType
{
    CV_DIST_MASK_3 = 0,
    CV_DIST_MASK_5 = 1
} CvDisMaskType;

/* Filters used in pyramid decomposition */
typedef enum CvFilter
{
    CV_GAUSSIAN_5x5 = 7
} CvFilter;

typedef enum CvMorphOp
{
    CV_MOP_OPEN = 2,
    CV_MOP_CLOSE = 3,
    CV_MOP_GRADIENT = 4,
    CV_MOP_TOPHAT = 5,
    CV_MOP_BLACKHAT = 6
} CvMorphOp;

typedef enum CvElementShape
{
    CV_SHAPE_RECT    = 0,
    CV_SHAPE_CROSS   = 1,
    CV_SHAPE_ELLIPSE = 2,
    CV_SHAPE_CUSTOM  = 100
}
CvElementShape;


/* POSIT structure */
typedef struct CvPOSITObject
{
    int N;
    float* inv_matr;
    float* obj_vecs;
    float* img_vecs;
} CvPOSITObject;

typedef struct CvMoments
{
    double  m00, m10, m01, m20, m11, m02, m30, m21, m12, m03;
    double  mu20, mu11, mu02, mu30, mu21, mu12, mu03;
    double  inv_sqrt_m00;
} CvMoments;


typedef struct CvHuMoments
{
    double hu1; /* First Hu moment (reflection changes its sign) */
    double hu2, hu3, hu4, hu5, hu6, hu7; /* Other, "unchangeable" moments */
} CvHuMoments;


/* method for comparing two images */
typedef enum CvTemplMatchMethod
{
    CV_TM_SQDIFF        = 0,
    CV_TM_SQDIFF_NORMED = 1,
    CV_TM_CCORR         = 2,
    CV_TM_CCORR_NORMED  = 3,
    CV_TM_CCOEFF        = 4,
    CV_TM_CCOEFF_NORMED = 5
}
CvTemplMatchMethod;

typedef enum CvFontFace
{
    CV_FONT_VECTOR0 = 0
} CvFontFace;

typedef struct CvFont
{
    const int*  data; /* font faces and other characteristics */
    CvSize      size; /* horizontal and vertical scale factors (8:8) fix-point nums */
    int         italic_scale; /* slope coefficient: 0 - normal, >0 - italic */
    int         thickness; /* letters thickness */
    int         dx; /* horizontal interval between letters */
} CvFont;

#define CV_RGB( r, g, b )  ( (((r)&255) << 16)|(((g)&255) << 8)|((b)&255))
#define CV_FILLED -1

typedef struct CvLineIterator
{
    uchar* ptr;
    int  err;
    int  plus_delta;
    int  minus_delta;
    int  plus_step;
    int  minus_step;
} CvLineIterator;


#define CV_NEXT_LINE_POINT( iterator )                                       \
{                                                                            \
    int mask = ((iterator).err -= (iterator).minus_delta) >> 31;             \
    (iterator).err += (iterator).plus_delta & mask;                          \
    (iterator).ptr += (iterator).minus_step + ((iterator).plus_step & mask); \
}


/* Rodrigues transform direction */
typedef enum CvRodriguesType
{
    CV_RODRIGUES_M2V = 0,
    CV_RODRIGUES_V2M = 1
}
CvRodriguesType;


/****************  segmented component's structure  *************************************/
typedef struct CvConnectedComp
{
    double area;          /*  area of the segmented component  */
    double value;          /*  gray scale value of the segmented component  */
    CvRect rect;        /*  ROI of the segmented component  */
} CvConnectedComp;

/*********************************** HMM structures *************************************/
typedef struct CvEHMMState
{
    int num_mix;      /*number of mixtures in this state*/
    float* mu;        /*mean vectors corresponding to each mixture*/
    float* inv_var; /* square root of inversed variances corresp. to each mixture*/
    float* log_var_val; /* sum of 0.5 (LN2PI + ln(variance[i]) ) for i=1,n */
    float* weight;   /*array of mixture weights. Summ of all weights in state is 1. */

} CvEHMMState;

typedef struct CvEHMM
{
    int level; /* 0 - lowest(i.e its states are real states), ..... */
    int num_states; /* number of HMM states */
    float*  transP;/*transition probab. matrices for states */
    float** obsProb; /* if level == 0 - array of brob matrices corresponding to hmm
                        if level == 1 - martix of matrices */
    union
    {
        CvEHMMState* state; /* if level == 0 points to real states array,
                                 if not - points to embedded hmms */
        struct CvEHMM* ehmm; /* pointer to embedded models (if NULL then no embedded models) */
    } u;

} CvEHMM;

typedef struct CvImgObsInfo
{
    int obs_x;
    int obs_y;
    int obs_size;
    float* obs;//consequtive observations

    int* state;/* array of pairs superstate/state to which observation belong */
    int* mix;  /* number of mixture to which observation belong */

} CvImgObsInfo;/*struct for 1 image*/

typedef struct 
{
    int MP;
    int DP;
    float* PosterState;          // Vector of State of the System in k-th step  
    float* PriorState;           // Vector of State of the System in (k-1)-th step 
    float* DynamMatr;            // Matrix of the linear Dynamics system 
                                 // (Must be updated by LinearizedDynamics function on each step
                                 //  for nonlinear systems)
    float* MeasurementMatr;      // Matrix of linear measurement (Must be updated by 
                                 // LinearizedMeasurement function on each step
                                 // for nonlinear measurements)
    float* MNCovariance;         // Matrix of measurement noice covariance
                                 // Initializes to Zero matrix, or sets by SetMeasureNoiseCov
                                 // method  
    float* PNCovariance;         // Matrix of process noice covariance
                                 // Initializes to Identity matrix, or sets by SetProcessNoiseCov
                                 // method 
    float* KalmGainMatr;         // Kalman Gain Matrix
    float* PriorErrorCovariance; //Prior Error Covariance matrix
    float* PosterErrorCovariance;//Poster Error Covariance matrix
    float* Temp1;                 // Temporary Matrix 
    float* Temp2; 

} CvKalman;

typedef struct 
{
    int MP;
    int DP;
    float* DynamMatr;       // Matrix of the linear Dynamics system 
    float* State;           // Vector of State
    int SamplesNum;         // Number of the Samples 
    float** flSamples;      // array of the Sample Vectors  
    float** flNewSamples;   // temporary array of the Sample Vectors
    float* flConfidence;    // Confidence for each Sample
    float* flCumulative;    // Cumulative confidence
    float* Temp;            // Temporary vector
    float* RandomSample;    // RandomVector to update sample set 
    CvRandState* RandS;  // Array of structures to generate random vectors

} CvConDensation;

#define  CV_LKFLOW_PYR_A_READY       1
#define  CV_LKFLOW_PYR_B_READY       2
#define  CV_LKFLOW_INITIAL_GUESSES   4

#define CV_EIGOBJ_NO_CALLBACK     0
#define CV_EIGOBJ_INPUT_CALLBACK  1
#define CV_EIGOBJ_OUTPUT_CALLBACK 2
#define CV_EIGOBJ_BOTH_CALLBACK   3

typedef CvStatus (*CvCallback)( int index, void* buffer, void* userData );

typedef union
{
    CvCallback callback;
    void* data;
}
CvInput;

#define CV_SCHARR -1

/****************************************************************************************/
/*                                Norm types defines                                          */
/****************************************************************************************/

#define CV_C            1
#define CV_L1           2
#define CV_L2           4
#define CV_RELATIVE     8
#define CV_DIFF         16

#define CV_DIFF_C       (CV_DIFF | CV_C)
#define CV_DIFF_L1      (CV_DIFF | CV_L1)
#define CV_DIFF_L2      (CV_DIFF | CV_L2)
#define CV_RELATIVE_C   (CV_RELATIVE | CV_C)
#define CV_RELATIVE_L1  (CV_RELATIVE | CV_L1)
#define CV_RELATIVE_L2  (CV_RELATIVE | CV_L2)


/****************************************************************************************\
*                         Histogram structures & defines                                 *
\****************************************************************************************/
#define CV_HIST_MAX_DIM 16

typedef enum CvHistType {
    CV_HIST_ARRAY = 0,
    CV_HIST_TREE  = 1
} CvHistType;

typedef enum CvCompareMethod {
    CV_COMP_CORREL = 0,
    CV_COMP_CHISQR = 1,
    CV_COMP_INTERSECT  = 2
} CvCompareMethod;

typedef enum CvHistFlag {
    CV_HIST_MEMALLOCATED    = 1,
    CV_HIST_HEADERALLOCATED = 2,
    CV_HIST_UNIFORM         = 4,
    CV_HIST_THRESHALLOCATED = 8
} CvHistFlag;

typedef struct CvHistogram
{
    int     header_size;    /* header's size */
    CvHistType type;           /* type of histogram */
    int     flags;
    int     c_dims;
    int     dims[CV_HIST_MAX_DIM];
    int     mdims[CV_HIST_MAX_DIM]; /* coefficients for fast calculation of number of element   */
                            /* &m[a,b,c] = m + a*mdims[0] + b*mdims[1] + c*mdims[2] */
    float*  thresh[CV_HIST_MAX_DIM];
    float*  array; /* all the histogram data, expanded into the single row */

    struct  CvNode* root;
    struct  CvSet*  set;
    int*    chdims[CV_HIST_MAX_DIM];
} CvHistogram;


/****************************************************************************************/
/*                                   Data structures                                    */
/****************************************************************************************/

/******************************** Memory storage ****************************************/

typedef struct CvMemBlock
{
    struct CvMemBlock*  prev;
    struct CvMemBlock*  next;
} CvMemBlock;

typedef struct CvMemStorage
{
    CvMemBlock* bottom;/* first allocated block */
    CvMemBlock* top;   /* current memory block - top of the stack */
    struct  CvMemStorage* parent; /* borrows new blocks from */
    int     block_size;  /* block size */
    int     free_space;  /* free space in the current block */
} CvMemStorage;


typedef struct CvMemStoragePos
{
    CvMemBlock* top;
    int  free_space;
}
CvMemStoragePos;


/*********************************** Sequence *******************************************/

typedef struct CvSeqBlock
{
    struct CvSeqBlock*  prev; /* previous sequence block */
    struct CvSeqBlock*  next; /* next sequence block */
    int    start_index;       /* index of the first element in the block +
                                 sequence->first->start_index */
    int    count;             /* number of elements in the block */
    char*  data;              /* pointer to the first element of the block */
} CvSeqBlock;


/*
   Read/Write sequence.
   Elements can be dynamically inserted to or deleted from the sequence.
*/
#define CV_SEQUENCE_FIELDS()                                    \
    int       header_size;   /* size of sequence header */      \
    struct    CvSeq* h_prev; /* previous sequence */            \
    struct    CvSeq* h_next; /* next sequence */                \
    struct    CvSeq* v_prev; /* 2nd previous sequence */        \
    struct    CvSeq* v_next; /* 2nd next sequence */            \
    int       flags;          /* micsellaneous flags */                \
    int       total;          /* total number of elements */           \
    int       elem_size;      /* size of sequence element in bytes */  \
    char*     block_max;      /* maximal bound of the last block */    \
    char*     ptr;            /* current write pointer */              \
    int       delta_elems;    /* how many elements allocated when the seq grows */  \
    CvMemStorage* storage;  /* where the seq is stored */       \
    CvSeqBlock* free_blocks;  /* free blocks list */       \
    CvSeqBlock* first; /* pointer to the first sequence block */


typedef struct CvSeq
{
    CV_SEQUENCE_FIELDS()
} CvSeq;


/*************************************** Set ********************************************/
/*
  Set.
  Order isn't keeped. There can be gaps between sequence elements.
  After the element has been inserted it stays on the same place all the time.
  The LSB(least-significant bit) of the first field is 0 iff the element exists.
*/
#define CV_SET_ELEM_FIELDS()    \
    int*  aligned_ptr;

typedef struct CvSetElem
{
    CV_SET_ELEM_FIELDS()
}
CvSetElem;

#define CV_SET_FIELDS()      \
    CV_SEQUENCE_FIELDS()     \
    CvMemBlock* free_elems;

typedef struct CvSet
{
    CV_SET_FIELDS()
}
CvSet;

/************************************* Graph ********************************************/

/*
  Graph is represented as a set of vertices.
  Vertices contain their adjacency lists (more exactly, pointers to first incoming or
  outcoming edge (or 0 if isolated vertex)). Edges are stored in another set.
  There is a single-linked list of incoming/outcoming edges for each vertex.

  Each edge consists of:
    two pointers to the starting and the ending vertices (vtx[0] and vtx[1],
    respectively). Graph may be oriented or not. In the second case, edges between
    vertex i to vertex j are not distingueshed (during the search operations).

    two pointers to next edges for the starting and the ending vertices.
    next[0] points to the next edge in the vtx[0] adjacency list and
    next[1] points to the next edge in the vtx[1] adjacency list.
*/
#define CV_GRAPH_EDGE_FIELDS()    \
    struct CvGraphEdge* next[2]; \
    struct CvGraphVertex* vtx[2];

#define CV_GRAPH_VERTEX_FIELDS()  \
    struct CvGraphEdge* first;

typedef struct CvGraphEdge
{
    CV_GRAPH_EDGE_FIELDS()
}
CvGraphEdge;

typedef struct CvGraphVertex
{
    CV_GRAPH_VERTEX_FIELDS()
}
CvGraphVtx;

typedef struct CvGraphVtx2D
{
    CV_GRAPH_VERTEX_FIELDS()
    CvPoint2D32f* ptr;
}
CvGraphVtx2D;

/*
   Graph is "derived" from the set (this is set a of vertices)
   and includes another set (edges)
*/
#define  CV_GRAPH_FIELDS()   \
    CV_SET_FIELDS()          \
    CvSet* edges;

typedef struct CvGraph
{
    CV_GRAPH_FIELDS()
}
CvGraph;

/*********************************** Chain/Countour *************************************/

typedef struct CvChain
{
    CV_SEQUENCE_FIELDS()
    CvPoint  origin;
} CvChain;

typedef struct CvContour
{
    CV_SEQUENCE_FIELDS()
    CvRect rect;
} CvContour;


typedef struct CvContourTree
{
    CV_SEQUENCE_FIELDS()
    CvPoint p1;            /*the start point of the binary tree root*/
    CvPoint p2;            /*the end point of the binary tree root*/

} CvContourTree;

/****************************************************************************************\
*                                    Sequence types                                      *
\****************************************************************************************/

#define CV_SEQ_ELTYPE_BITS         5
#define CV_SEQ_ELTYPE_MASK         ((1 << CV_SEQ_ELTYPE_BITS) - 1)

#define CV_SEQ_ELTYPE_POINT          1  /* (x,y) */
#define CV_SEQ_ELTYPE_CODE           2  /* freeman code: 0..7 */
#define CV_SEQ_ELTYPE_PPOINT         3  /* &(x,y) */
#define CV_SEQ_ELTYPE_INDEX          4  /* #(x,y) */
#define CV_SEQ_ELTYPE_GRAPH_EDGE     5  /* &next_o, &next_d, &vtx_o, &vtx_d */
#define CV_SEQ_ELTYPE_GRAPH_VERTEX   6  /* first_edge, &(x,y) */
#define CV_SEQ_ELTYPE_TRIAN_ATR      7  /* vertex of the binary tree   */
#define CV_SEQ_ELTYPE_CONNECTED_COMP 8  /* connected component  */
#define CV_SEQ_ELTYPE_POINT3D        9  /* (x,y,z)  */

#define CV_SEQ_KIND_BITS      5
#define CV_SEQ_KIND_MASK       (((1 << CV_SEQ_KIND_BITS) - 1)<<CV_SEQ_ELTYPE_BITS)

#define CV_SEQ_KIND_SET        (0 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_CURVE      (1 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_BIN_TREE   (2 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_GRAPH      (3 << CV_SEQ_ELTYPE_BITS)
#define CV_SEQ_KIND_SUBDIV2D   (4 << CV_SEQ_ELTYPE_BITS)

#define CV_SEQ_FLAG_SHIFT      (CV_SEQ_KIND_BITS + CV_SEQ_ELTYPE_BITS)

/* flags for curves */
#define CV_SEQ_FLAG_CLOSED     (1 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_SIMPLE     (2 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_CONVEX     (4 << CV_SEQ_FLAG_SHIFT)
#define CV_SEQ_FLAG_HOLE       (8 << CV_SEQ_FLAG_SHIFT)

/* flags for graphs */
#define CV_GRAPH_FLAG_ORIENTED (1 << CV_SEQ_FLAG_SHIFT)

/* point sets */
#define CV_SEQ_POINT_SET       (CV_SEQ_KIND_SET    | CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POINT3D_SET     (CV_SEQ_KIND_SET    | CV_SEQ_ELTYPE_POINT3D)
#define CV_SEQ_POLYLINE        (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_POINT)
#define CV_SEQ_POLYGON         (CV_SEQ_FLAG_CLOSED | CV_SEQ_POLYLINE )
#define CV_SEQ_CONTOUR         CV_SEQ_POLYGON
#define CV_SEQ_SIMPLE_POLYGON  (CV_SEQ_FLAG_SIMPLE | CV_SEQ_POLYGON  )

/* chain-coded curves */
#define CV_SEQ_CHAIN           (CV_SEQ_KIND_CURVE  | CV_SEQ_ELTYPE_CODE)
#define CV_SEQ_CHAIN_CONTOUR   (CV_SEQ_FLAG_CLOSED | CV_SEQ_CHAIN)

/* binary tree for the contour */
#define CV_SEQ_POLYGON_TREE    (CV_SEQ_KIND_BIN_TREE  | CV_SEQ_ELTYPE_TRIAN_ATR)

/* sequence of the connected components */
#define CV_SEQ_CONNECTED_COMP  (CV_SEQ_KIND_SET  | CV_SEQ_ELTYPE_CONNECTED_COMP)

/* sequence of the integer numbers */
#define CV_SEQ_INDEX  (CV_SEQ_KIND_SET  | CV_SEQ_ELTYPE_INDEX)

/* type retrieving macros */
#define CV_SEQ_TYPE_BITS       CV_SEQ_FLAG_SHIFT
#define CV_SEQ_TYPE_MASK       ((1 << CV_SEQ_TYPE_BITS) - 1)

#define CV_SEQ_SUBTYPE_BITS    (CV_SEQ_TYPE_BITS + 10)
#define CV_SEQ_SUBTYPE_MASK    ((1 << (CV_SEQ_SUBTYPE_BITS) - 1)

#define CV_GET_SEQ_ELTYPE( seq )    ((seq)->flags & CV_SEQ_ELTYPE_MASK)
#define CV_GET_SEQ_KIND( seq )      ((seq)->flags & CV_SEQ_KIND_MASK )
#define CV_GET_SEQ_TYPE( seq )      ((seq)->flags & CV_SEQ_TYPE_MASK )
#define CV_GET_SEQ_SUBTYPE( seq )   ((seq)->flags & CV_SEQ_SUBTYPE_MASK )

/* flag checking */
#define CV_IS_SEQ_INDEX( seq )      ((CV_GET_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_INDEX) && \
                                       (CV_GET_SEQ_KIND(seq) == CV_SEQ_KIND_SET))

#define CV_IS_SEQ_CURVE( seq )      (CV_GET_SEQ_KIND(seq) == CV_SEQ_KIND_CURVE)
#define CV_IS_SEQ_CLOSED( seq )     (((seq)->flags & CV_SEQ_FLAG_CLOSED) != 0)
#define CV_IS_SEQ_CONVEX( seq )     (((seq)->flags & CV_SEQ_FLAG_CONVEX) != 0)
#define CV_IS_SEQ_HOLE( seq )       (((seq)->flags & CV_SEQ_FLAG_HOLE) != 0)
#define CV_IS_SEQ_SIMPLE( seq )     ((((seq)->flags & CV_SEQ_FLAG_SIMPLE) != 0) || \
                                         CV_IS_CONVEX(seq)

/* type checking macros */
#define CV_IS_SEQ_POINT_SET( seq ) \
(CV_GET_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_POINT)

#define CV_IS_SEQ_POINT_SUBSET( seq ) \
(CV_GET_SEQ_ELTYPE(seq) == CV_SEQ_ELTYPE_PPOINT)

#define CV_IS_SEQ_POLYLINE( seq )   \
(CV_GET_SEQ_TYPE(seq) == CV_SEQ_POLYLINE)

#define CV_IS_SEQ_POLYGON( seq )   \
(CV_IS_SEQ_POLYLINE(seq) && CV_IS_SEQ_CLOSED(seq))

#define CV_IS_SEQ_CHAIN( seq )   \
(CV_GET_SEQ_TYPE(seq) == CV_SEQ_CHAIN)

#define CV_IS_SEQ_CONTOUR( seq )   \
(CV_IS_SEQ_CLOSED(seq) && (CV_IS_SEQ_POLYLINE(seq) || CV_IS_SEQ_CHAIN(seq)))

#define CV_IS_SEQ_CHAIN_CONTOUR( seq ) \
(CV_IS_SEQ_CHAIN( seq ) && CV_IS_SEQ_CLOSED( seq ))

#define CV_IS_SEQ_POLYGON_TREE( seq ) \
    (CV_GET_SEQ_ELTYPE (seq) ==  CV_SEQ_ELTYPE_TRIAN_ATR &&    \
    CV_GET_SEQ_KIND( seq ) ==  CV_SEQ_KIND_BIN_TREE )

#define CV_IS_GRAPH( seq )    \
    ( CV_GET_SEQ_KIND(seq) == CV_SEQ_KIND_GRAPH )

#define CV_IS_GRAPH_ORIENTED( seq )   \
    ( CV_IS_GRAPH( seq ) && ((seq)->flags & CV_GRAPH_FLAG_ORIENTED) != 0 )

#define CV_IS_SUBDIV2D( seq )  \
    ( CV_GET_SEQ_KIND(seq) == CV_SEQ_KIND_SUBDIV2D )

/****************************************************************************************/
/*                            Sequence writer & reader                                  */
/****************************************************************************************/

#define CV_SEQ_WRITER_FIELDS()                                     \
    int          header_size;                                      \
    CvSeq*       seq;        /* sequence, beign written */         \
    CvSeqBlock*  block;      /* current block */                   \
    char*        ptr;        /* pointer to free space */           \
    char*        block_min;  /* address of end of block */         \
    char*        block_max;  /* address of end of block */

typedef struct CvSeqWriter
{
    CV_SEQ_WRITER_FIELDS()
    int  reserved[7]; /* some reserved fields */
} CvSeqWriter;


#define CV_SEQ_READER_FIELDS()                                      \
    int          header_size;                                       \
    CvSeq*       seq;        /* sequence, beign read */             \
    CvSeqBlock*  block;      /* current block */                    \
    char*        ptr;        /* pointer to element be read next */  \
    char*        block_max;  /* upper boundary of the block */      \
    char*        block_min;  /* lower boundary of the block */      \
    int          delta_index;/* = seq->first->start_index   */      \
    char*        prev_elem;  /* pointer to previous element */


typedef struct CvSeqReader
{
    CV_SEQ_READER_FIELDS()
    int  reserved[9];
} CvSeqReader;


typedef struct CvChainPtReader
{
    CV_SEQ_READER_FIELDS()
    char      code;
    CvPoint  pt;
    char      deltas[8][2];
    int       reserved[2];
} CvChainPtReader;

/*
Internal structure that is used for sequental retrieving contours from the image.
It supports both hierarchical and plane variants of Suzuki algorithm.
*/
typedef struct _CvContourScanner* CvContourScanner;


typedef enum CvContourRetrievalMode
{
    CV_RETR_EXTERNAL = 0,
    CV_RETR_LIST     = 1,
    CV_RETR_CCOMP    = 2,
    CV_RETR_TREE     = 3
}
CvContourRetrievalMode;


/* approximation method:
DIRECT - keeps all the digital curve points
SIMPLE - substitutes vertical, horizontal and diagonal segments by two ending points
TC89_L1- Teh-Chin algorithm with L1 curvature (difference of successive chain codes)
TC89_KCOS - Teh-Chin algorithm with k-cosine curvature. */
typedef enum CvChainApproxMethod
{
    CV_CHAIN_CODE             = 0,
    CV_CHAIN_APPROX_NONE      = 1,
    CV_CHAIN_APPROX_SIMPLE    = 2,
    CV_CHAIN_APPROX_TC89_L1   = 3,
    CV_CHAIN_APPROX_TC89_KCOS = 4
} CvChainApproxMethod;

typedef enum CvContoursMatchMethod
{
    CV_CONTOURS_MATCH_I1 = 1,
    CV_CONTOURS_MATCH_I2 = 2,
    CV_CONTOURS_MATCH_I3 = 3
} CvContoursMatchMethod;

typedef enum CvContourTreesMatchMethod
{
    CV_CONTOUR_TREES_MATCH_I1 = 1
} CvContourTreesMatchMethod;

typedef enum CvPolyApproxMethod
{
    CV_POLY_APPROX_DP = 0
}
CvPolyApproxMethod;

#define CV_DOMINANT_IPAN 1

typedef struct CvConvexityDefect
{
    CvPoint* start;
    CvPoint* end;
    CvPoint* depth_point;
    float depth;
} CvConvexityDefect;


/****************************************************************************************/
/*                                Operations on sequences                               */
/****************************************************************************************/

#define  CV_GET_SEQ_ELEM( elem_type, seq, index )                \
/* assert gives some guarantee that <seq> parameter is valid */  \
(   assert(sizeof((seq)->first[0]) == sizeof(CvSeqBlock) &&      \
    (seq)->elem_size == sizeof(elem_type)),                      \
    (elem_type*)((seq)->first && (unsigned)index <               \
    (unsigned)((seq)->first->count) ?                            \
    (seq)->first->data + (index) * sizeof(elem_type) :           \
    cvGetSeqElem( (CvSeq*)(seq), (index), 0 )))


/* macro that adds element to sequence */
#define CV_WRITE_SEQ_ELEM( elem, writer )             \
{                                                     \
    assert( (writer).seq->elem_size == sizeof(elem)); \
    if( (writer).ptr >= (writer).block_max )          \
    {                                                 \
        cvCreateSeqBlock( &writer);                   \
    }                                                 \
    memcpy((writer).ptr, &elem, sizeof(elem));        \
    (writer).ptr += sizeof(elem);                     \
}


/* move reader position forward */
#define CV_NEXT_SEQ_ELEM( elem_size, reader )                 \
{                                                             \
    if( ((reader).ptr += (elem_size)) >= (reader).block_max ) \
    {                                                         \
        cvChangeSeqBlock( &(reader), 1 );                     \
    }                                                         \
}


/* move reader position backward */
#define CV_PREV_SEQ_ELEM( elem_size, reader )                \
{                                                            \
    if( ((reader).ptr -= (elem_size)) < (reader).block_min ) \
    {                                                        \
        cvChangeSeqBlock( &(reader), -1 );                   \
    }                                                        \
}

/* read element and move read position forward */
#define CV_READ_SEQ_ELEM( elem, reader )                       \
{                                                              \
    assert( (reader).seq->elem_size == sizeof(elem));          \
    memcpy( &(elem), (reader).ptr, sizeof((elem)));            \
    CV_NEXT_SEQ_ELEM( sizeof(elem), reader )                   \
}

/* read element and move read position backward */
#define CV_REV_READ_SEQ_ELEM( elem, reader )                     \
{                                                                \
    assert( (reader).seq->elem_size == sizeof(elem));            \
    memcpy(&(elem), (reader).ptr, sizeof((elem)));               \
    CV_PREV_SEQ_ELEM( sizeof(elem), reader )                     \
}

#define CV_READ_CHAIN_POINT( _pt, reader )                              \
{                                                                       \
    (_pt) = (reader).pt;                                                \
    if( (reader).ptr )                                                  \
    {                                                                   \
        CV_READ_SEQ_ELEM( (reader).code, (*((CvSeqReader*)&(reader)))); \
        assert( ((reader).code & ~7) == 0 );                            \
        (reader).pt.x += (reader).deltas[(reader).code][0];             \
        (reader).pt.y += (reader).deltas[(reader).code][1];             \
    }                                                                   \
}

#define CV_CURRENT_POINT( reader )  (*((CvPoint*)((reader).ptr)))
#define CV_PREV_POINT( reader )     (*((CvPoint*)((reader).prev_elem)))

#define CV_READ_EDGE( pt1, pt2, reader )               \
{                                                      \
    assert( sizeof(pt1) == sizeof(CvPoint) &&          \
            sizeof(pt2) == sizeof(CvPoint) &&          \
            reader.seq->elem_size == sizeof(CvPoint)); \
    (pt1) = CV_PREV_POINT( reader );                   \
    (pt2) = CV_CURRENT_POINT( reader );                \
    (reader).prev_elem = (reader).ptr;                 \
    CV_NEXT_SEQ_ELEM( sizeof(CvPoint), (reader));      \
}

/************ Set macros **************/
#define CV_IS_SET_ELEM_EXISTS( ptr )  (((*(int*)(ptr)) & 1) == 0)

#define CV_REMOVE_SET_ELEM( set, index, elem )                                 \
    (((CvMemBlock*)(elem))->prev = (CvMemBlock*)((int)((set)->free_elems)|1),  \
    ((CvMemBlock*)(elem))->next = (CvMemBlock*)(index),                        \
    (set)->free_elems = (CvMemBlock*)(elem))

/************ Graph macros ************/

/* returns next graph edge for given vertex */
#define  CV_NEXT_GRAPH_EDGE( edge, vertex )                              \
     (assert((edge)->vtx[0] == (vertex) || (edge)->vtx[1] == (vertex)),  \
      (edge)->next[(edge)->vtx[1] == (vertex)])

/****************************************************************************************/
/*                                  Planar subdivisions                                 */
/****************************************************************************************/

/************ Data structures and related enumerations ************/

typedef int CvSubdiv2DEdge;

#define CV_QUADEDGE2D_FIELDS()     \
    struct CvSubdiv2DPoint* pt[4]; \
    CvSubdiv2DEdge  next[4];

#define CV_SUBDIV2D_VIRTUAL_POINT  2

#define CV_SUBDIV2D_POINT_FIELDS()    \
    int            is_virtual; \
    CvSubdiv2DEdge first;      \
    CvPoint2D32f   pt;

typedef struct CvQuadEdge2D
{
    CV_QUADEDGE2D_FIELDS()
}
CvQuadEdge2D;

typedef struct CvSubdiv2DPoint
{
    CV_SUBDIV2D_POINT_FIELDS()
}
CvSubdiv2DPoint;

#define CV_SUBDIV2D_FIELDS()    \
    CV_GRAPH_FIELDS()           \
    int  quad_edges;            \
    int  is_geometry_valid;     \
    CvSubdiv2DEdge recent_edge; \
    CvPoint2D32f  topleft;      \
    CvPoint2D32f  bottomright;
    
typedef struct CvSubdiv2D
{
    CV_SUBDIV2D_FIELDS()
}
CvSubdiv2D;


typedef enum CvSubdiv2DPointLocation
{
    CV_PTLOC_ERROR = -2,
    CV_PTLOC_OUTSIDE_RECT = -1,
    CV_PTLOC_INSIDE = 0,
    CV_PTLOC_VERTEX = 1,
    CV_PTLOC_ON_EDGE = 2
}
CvSubdiv2DPointLocation;

typedef enum CvNextEdgeType
{
    CV_NEXT_AROUND_ORG   = 0x00,
    CV_NEXT_AROUND_DST   = 0x22,
    CV_PREV_AROUND_ORG   = 0x11,
    CV_PREV_AROUND_DST   = 0x33,
    CV_NEXT_AROUND_LEFT  = 0x13,
    CV_NEXT_AROUND_RIGHT = 0x31,
    CV_PREV_AROUND_LEFT  = 0x20,
    CV_PREV_AROUND_RIGHT = 0x02
}
CvNextEdgeType;


#define  CV_SUBDIV2D_NEXT_EDGE( edge )  (((CvQuadEdge2D*)((edge) & ~3))->next[(edge)&3])


/************ Basic quad-edge navigation and operations ************/

CV_INLINE  CvSubdiv2DEdge  cvSubdiv2DNextEdge( CvSubdiv2DEdge edge )
{
    return  CV_SUBDIV2D_NEXT_EDGE(edge);
}


CV_INLINE  CvSubdiv2DEdge  cvSubdiv2DRotateEdge( CvSubdiv2DEdge edge, int rotate )
{
    return  (edge & ~3) + ((edge + rotate) & 3);
}


CV_INLINE  CvSubdiv2DEdge  cvSubdiv2DSymEdge( CvSubdiv2DEdge edge )
{
    return edge ^ 2;
}


CV_INLINE  CvSubdiv2DEdge  cvSubdiv2DGetEdge( CvSubdiv2DEdge edge, CvNextEdgeType type )
{
    CvQuadEdge2D* e = (CvQuadEdge2D*)(edge & ~3);
    edge = e->next[(edge + (int)type) & 3];
    return  (edge & ~3) + ((edge + ((int)type >> 4)) & 3);
}


CV_INLINE  CvSubdiv2DPoint*  cvSubdiv2DEdgeOrg( CvSubdiv2DEdge edge )
{
    CvQuadEdge2D* e = (CvQuadEdge2D*)(edge & ~3);
    return e->pt[edge & 3];
}


CV_INLINE  CvSubdiv2DPoint*  cvSubdiv2DEdgeDst( CvSubdiv2DEdge edge )
{
    CvQuadEdge2D* e = (CvQuadEdge2D*)(edge & ~3);
    return e->pt[(edge + 2) & 3];
}


CV_INLINE  double  cvTriangleArea( CvPoint2D32f a, CvPoint2D32f b, CvPoint2D32f c )
{
    return (b.x - a.x) * (c.y - a.y) - (b.y - a.y) * (c.x - a.x);
}

typedef struct
{
    const char* file;
    int         line;
}CvStackRecord;                               

#endif /*_CVTYPES_H_*/

/* End of file. */
