/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "_cv.h"
#include <stdlib.h>

/* This string uses as control string for valid memory block */
#define _CV_MEMORY_BLOCK_CONTROL_STRING "OpenCV"

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvAlloc
//   Purpose:
//     Allocating memory
//   Context:
//   Parameters:
//     lSize - size of needed memory (in bytes)
//   Returns:
//     Pointer to allocated memory or NULL if No Memory
//   Notes:
//F*/
IPCVAPI_IMPL( void*, p_cvAlloc_, ( int size, const char* file, int line ) )
{
    return p_cvAlloc( size, file, line );
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    icvAlloc_
//   Purpose:
//     Allocating memory
//   Context:
//   Parameters:
//     lSize - size of needed memory (in bytes)
//   Returns:
//     Pointer to allocated memory or NULL if No Memory
//   Notes:
//F*/
void* icvAlloc_( int size, const char*, int)
{
    long ptr = (long)malloc( size + 64 + 4);

    if( !ptr ) return 0;
    *(long*)(((ptr + 32 + 4) & ~31) - 4) = (long)ptr;
    return (void*)((ptr + 32 + 4) & ~31);
} /* icvAlloc_ */


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvFree_
//    Purpose:
//      Free allocated memory
//    Context:
//    Parameters:
//      pPointer - pointer to memory to be deallocated
//    Returns:
//      CV_OK if all Ok or error code
//    Notes:
//      The memory block must be allocated by icvAlloc_ function
//F*/
IPCVAPI_IMPL( void, p_cvFree_, ( void** pptr, const char* file, int line ) )
{
    p_cvFree( pptr, file, line );
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvFree_
//    Purpose:
//      Free allocated memory
//    Context:
//    Parameters:
//      pPointer - pointer to memory to be deallocated
//    Returns:
//      CV_OK if all Ok or error code
//    Notes:
//      The memory block must be allocated by icvAlloc_ function
//F*/
void icvFree_( void** pptr, const char*,int )
{
    /* Checking for bad pointer */
    if( pptr && *pptr )
    {
        long ptr = (long)*pptr;
    
        /* Pointer must be align by 32 */
        assert( (ptr & 31) == 0 );
        *pptr = 0;
    
        free( *(void**)(ptr - 4) );
    }
} /* _cvFree */


/* End of file. */
