/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "_cv.h"

#include <assert.h>
#include <limits.h>
#include "_cvconvert.h"

/****************************************************************************************\
*                             Conversion pixel -> plane                                  *
\****************************************************************************************/

_CV_CVT_PX2PL_FUNC( icvCpy_8u_C3P3R )
{
    int i, j;
    uchar *plane0;
    uchar *plane1;
    uchar *plane2;
    uchar *buf0_bottom;
    int buf_step, buf_size;

    buf3 = buf3;
    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step;
    buf0_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane0 = (uchar *) (buf0->last);
    plane1 = (uchar *) (buf1->last);
    plane2 = (uchar *) (buf2->last);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            uchar t0 = img[j * 3];
            uchar t1 = img[j * 3 + 1];
            uchar t2 = img[j * 3 + 2];

            plane0[j] = t0;
            plane1[j] = t1;
            plane2[j] = t2;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
    }

    buf0->last = (char *) plane0;
    buf1->last = (char *) plane1;
    buf2->last = (char *) plane2;

    buf2->count = buf1->count = buf0->count += size.height;
}


_CV_EXTR_COI_FUNC( icvExtractCOI8uC3 )
{
    int i, j;
    uchar *plane;
    uchar *buf_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && (1 <= coi && coi <= 3) );

    buf_step = buf0->step;
    buf_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane = (uchar *) (buf0->last);
    img += coi - 1;

    for( i = 0; i < size.height; i++, img += img_step )
    {
        for( j = 0; j < size.width; j++ )
            plane[j] = img[j * 3];
        plane += buf_step;
        plane -= ((plane < buf_bottom) - 1) & buf_size;
    }

    buf0->last = (char *) plane;
    buf0->count += size.height;
}



_CV_CVT_PX2PL_FUNC( icvCpy_8u_C4P4R )
{
    int i, j;
    uchar *plane0;
    uchar *plane1;
    uchar *plane2;
    uchar *plane3;
    uchar *buf0_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step && buf2->step == buf3->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step;
    buf0_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane0 = (uchar *) (buf0->last);
    plane1 = (uchar *) (buf1->last);
    plane2 = (uchar *) (buf2->last);
    plane3 = (uchar *) (buf3->last);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            uchar t0 = img[j * 4];
            uchar t1 = img[j * 4 + 1];
            uchar t2 = img[j * 4 + 2];
            uchar t3 = img[j * 4 + 3];

            plane0[j] = t0;
            plane1[j] = t1;
            plane2[j] = t2;
            plane3[j] = t3;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
        _CV_RB_ROTATE( plane3, buf_step, (uchar *) (buf3->bottom), buf3->size );
    }

    buf0->last = (char *) plane0;
    buf1->last = (char *) plane1;
    buf2->last = (char *) plane2;
    buf3->last = (char *) plane3;

    buf2->count = buf1->count = buf0->count += size.height;
    buf3->count += size.height;
}


_CV_EXTR_COI_FUNC( icvExtractCOI8uAC4 )
{
    int i, j;
    uchar *plane;
    uchar *buf_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && (1 <= coi && coi <= 4) );

    buf_step = buf0->step;
    buf_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane = (uchar *) (buf0->last);
    img += coi - 1;

    for( i = 0; i < size.height; i++, img += img_step )
    {
        for( j = 0; j < size.width; j++ )
            plane[j] = img[j * 4];
        plane += buf_step;
        plane -= ((plane < buf_bottom) - 1) & buf_size;
    }

    buf0->last = (char *) plane;
    buf0->count += size.height;
}


_CV_CVT_PX2PL_FUNC( icvCpy_32f_C3P3R )
{
    int i, j;
    int *plane0;
    int *plane1;
    int *plane2;
    int *buf0_bottom;
    int buf_step, buf_size;

    buf3 = buf3;
    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step / 4;
    buf0_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;

    plane0 = (int *) (buf0->last);
    plane1 = (int *) (buf1->last);
    plane2 = (int *) (buf2->last);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            int t0 = ((int *) img)[j * 3];
            int t1 = ((int *) img)[j * 3 + 1];
            int t2 = ((int *) img)[j * 3 + 2];

            plane0[j] = t0;
            plane1[j] = t1;
            plane2[j] = t2;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
    }

    buf0->last = (char *) plane0;
    buf1->last = (char *) plane1;
    buf2->last = (char *) plane2;

    buf2->count = buf1->count = buf0->count += size.height;
}


_CV_EXTR_COI_FUNC( icvExtractCOI32fC3 )
{
    int i, j;
    int *plane;
    int *buf_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && (1 <= coi && coi <= 3) );

    buf_step = buf0->step / 4;
    buf_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;

    plane = (int *) (buf0->last);
    img += (coi - 1) * sizeof( int );

    for( i = 0; i < size.height; i++, img += img_step )
    {
        for( j = 0; j < size.width; j++ )
            plane[j] = ((int *) img)[j * 3];
        plane += buf_step;
        plane -= ((plane < buf_bottom) - 1) & buf_size;
    }

    buf0->last = (char *) plane;
    buf0->count += size.height;
}


_CV_CVT_PX2PL_FUNC( icvCpy_32f_C4P4R )
{
    int i, j;
    int *plane0;
    int *plane1;
    int *plane2;
    int *plane3;
    int *buf0_bottom;
    int buf_step, buf_size, buf3_size;

    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step && buf2->step == buf3->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step / 4;
    buf0_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;
    buf3_size = buf3->size / 4;

    plane0 = (int *) (buf0->last);
    plane1 = (int *) (buf1->last);
    plane2 = (int *) (buf2->last);
    plane3 = (int *) (buf3->last);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            int t0 = ((int *) img)[j * 4];
            int t1 = ((int *) img)[j * 4 + 1];
            int t2 = ((int *) img)[j * 4 + 2];
            int t3 = ((int *) img)[j * 4 + 3];

            plane0[j] = t0;
            plane1[j] = t1;
            plane2[j] = t2;
            plane3[j] = t3;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
        _CV_RB_ROTATE( plane3, buf_step, (int *) (buf3->bottom), buf3_size );
    }

    buf0->last = (char *) plane0;
    buf1->last = (char *) plane1;
    buf2->last = (char *) plane2;
    buf3->last = (char *) plane3;

    buf2->count = buf1->count = buf0->count += size.height;
    buf3->count += size.height;
}


_CV_EXTR_COI_FUNC( icvExtractCOI32fAC4 )
{
    int i, j;
    int *plane;
    int *buf_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && (1 <= coi && coi <= 4) );

    buf_step = buf0->step / 4;
    buf_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;

    plane = (int *) (buf0->last);
    img += (coi - 1) * sizeof( int );

    for( i = 0; i < size.height; i++, img += img_step )
    {
        for( j = 0; j < size.width; j++ )
            plane[j] = ((int *) img)[j * 4];
        plane += buf_step;
        plane -= ((plane < buf_bottom) - 1) & buf_size;
    }

    buf0->last = (char *) plane;
    buf0->count += size.height;
}


/****************************************************************************************\
*                             Conversion plane -> pixel                                  *
\****************************************************************************************/

_CV_CVT_PX2PL_FUNC( icvCpy_8u_P3C3R )
{
    int i, j;
    uchar *plane0;
    uchar *plane1;
    uchar *plane2;
    uchar *buf0_bottom;
    int buf_step, buf_size;

    buf3 = buf3;
    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step;
    buf0_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane0 = (uchar *) (buf0->first);
    plane1 = (uchar *) (buf1->first);
    plane2 = (uchar *) (buf2->first);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            uchar t0 = plane0[j];
            uchar t1 = plane1[j];
            uchar t2 = plane2[j];

            img[j * 3] = t0;
            img[j * 3 + 1] = t1;
            img[j * 3 + 2] = t2;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
    }

    buf0->first = (char *) plane0;
    buf1->first = (char *) plane1;
    buf2->first = (char *) plane2;

    buf2->count = buf1->count = buf0->count -= size.height;
}


_CV_CVT_PX2PL_FUNC( icvCpy_8u_P4C4R )
{
    int i, j;
    uchar *plane0;
    uchar *plane1;
    uchar *plane2;
    uchar *plane3;
    uchar *buf0_bottom;
    int buf_step, buf_size;

    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step && buf2->step == buf3->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step;
    buf0_bottom = (uchar *) (buf0->bottom);
    buf_size = buf0->size;

    plane0 = (uchar *) (buf0->first);
    plane1 = (uchar *) (buf1->first);
    plane2 = (uchar *) (buf2->first);
    plane3 = (uchar *) (buf3->first);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            uchar t0 = plane0[j];
            uchar t1 = plane1[j];
            uchar t2 = plane2[j];
            uchar t3 = plane3[j];

            img[j * 4] = t0;
            img[j * 4 + 1] = t1;
            img[j * 4 + 2] = t2;
            img[j * 4 + 3] = t3;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
        _CV_RB_ROTATE( plane3, buf_step, (uchar *) (buf3->bottom), buf3->size );
    }

    buf0->first = (char *) plane0;
    buf1->first = (char *) plane1;
    buf2->first = (char *) plane2;
    buf3->first = (char *) plane3;

    buf2->count = buf1->count = buf0->count -= size.height;
    buf3->count -= size.height;
}


_CV_CVT_PX2PL_FUNC( icvCpy_32f_P3C3R )
{
    int i, j;
    int *plane0;
    int *plane1;
    int *plane2;
    int *buf0_bottom;
    int buf_step, buf_size;

    buf3 = buf3;
    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step / 4;
    buf0_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;

    plane0 = (int *) (buf0->first);
    plane1 = (int *) (buf1->first);
    plane2 = (int *) (buf2->first);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            int t0 = plane0[j];
            int t1 = plane1[j];
            int t2 = plane2[j];

            ((int *) img)[j * 3] = t0;
            ((int *) img)[j * 3 + 1] = t1;
            ((int *) img)[j * 3 + 2] = t2;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
    }

    buf0->first = (char *) plane0;
    buf1->first = (char *) plane1;
    buf2->first = (char *) plane2;

    buf2->count = buf1->count = buf0->count += size.height;
}


_CV_CVT_PX2PL_FUNC( icvCpy_32f_P4C4R )
{
    int i, j;
    int *plane0;
    int *plane1;
    int *plane2;
    int *plane3;
    int *buf0_bottom;
    int buf_step, buf_size, buf3_size;

    assert( img && buf0 && buf1 && buf2 );

    assert( buf0->step == buf1->step && buf1->step == buf2->step && buf2->step == buf3->step );

    assert( buf0->last - buf0->top == buf1->last - buf1->top &&
            buf1->last - buf1->top == buf2->last - buf2->top );

    assert( buf0->size == buf1->size && buf1->size == buf2->size );

    assert( buf0->count == buf1->count && buf1->count == buf2->count );

    buf_step = buf0->step / 4;
    buf0_bottom = (int *) (buf0->bottom);
    buf_size = buf0->size / 4;
    buf3_size = buf3->size / 4;

    plane0 = (int *) (buf0->first);
    plane1 = (int *) (buf1->first);
    plane2 = (int *) (buf2->first);
    plane3 = (int *) (buf3->first);

    for( i = 0; i < size.height; i++, img += img_step )
    {
        int step;

        for( j = 0; j < size.width; j++ )
        {
            int t0 = plane0[j];
            int t1 = plane1[j];
            int t2 = plane2[j];
            int t3 = plane3[j];

            ((int *) img)[j * 4] = t0;
            ((int *) img)[j * 4 + 1] = t1;
            ((int *) img)[j * 4 + 2] = t2;
            ((int *) img)[j * 4 + 3] = t3;
        }

        step = buf_step - (((plane0 + buf_step < buf0_bottom) - 1) & buf_size);
        plane0 += step;
        plane1 += step;
        plane2 += step;
        _CV_RB_ROTATE( plane3, buf_step, (int *) (buf3->bottom), buf3_size );
    }

    buf0->first = (char *) plane0;
    buf1->first = (char *) plane1;
    buf2->first = (char *) plane2;
    buf3->first = (char *) plane3;

    buf2->count = buf1->count = buf0->count += size.height;
    buf3->count += size.height;
}


/****************************************************************************************\
*                                 Convertion functions                                   *
\****************************************************************************************/

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: ippiCvtPixToPlane
//    Purpose: divides pixel image into separate planes
//    Context:
//    Parameters:
//      src       - pointer to source image or its ROI
//      src_step  - full width of source image
//      dst0      - first destination plane
//      dst1      - second destination plane
//      dst2      - third destination plane
//      dst3      - fourth destination plane
//      dst_step  - full width of planes
//      size      - width and height of images in pixels
//    Returns:
//      CV_OK or error code
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvCopy_8u_C3P3R, (const uchar * pSrc, int srcStep,
                                           uchar * pDst[3], int dstStep, CvSize roiSize) )
{
    _CvRingBuffer dst_rb[3];
    int i, planes = 0, coi = 0;

    if( pDst[0] )
    {
        planes++;
        coi = 1;
    }
    if( pDst[1] )
    {
        planes++;
        coi = 2;
    }
    if( pDst[2] )
    {
        planes++;
        coi = 3;
    }

    if( !pSrc || (planes != 1 && planes != 3) )
        return CV_NULLPTR_ERR;

    if( roiSize.width < 0 || roiSize.height < 0 ||
        srcStep < roiSize.width * 3 || dstStep < roiSize.width )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < planes; i++ )
    {
        dst_rb[i].size = 0;
        dst_rb[i].count = 0;
        dst_rb[i].step = dstStep;

        dst_rb[i].top = (char *) (pDst[coi - planes + i]);
        dst_rb[i].bottom = dst_rb[i].top + roiSize.height * dstStep;
        dst_rb[i].first = dst_rb[i].last = dst_rb[i].top;
    }

    if( planes == 1 )
        icvExtractCOI8uC3( (char *) pSrc, srcStep, roiSize, dst_rb + 0, coi );
    else
        icvCpy_8u_C3P3R( (char *) pSrc, srcStep, roiSize, dst_rb + 0,
                         dst_rb + 1, dst_rb + 2, 0 );

    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_8u_C4P4R, (const uchar * pSrc, int srcStep,
                                           uchar * pDst[4], int dstStep, CvSize roiSize) )
{
    _CvRingBuffer dst_rb[4];
    int i, planes = 0, coi = 0;

    if( pDst[0] )
    {
        planes++;
        coi = 1;
    }
    if( pDst[1] )
    {
        planes++;
        coi = 2;
    }
    if( pDst[2] )
    {
        planes++;
        coi = 3;
    }
    if( pDst[3] )
    {
        planes++;
        coi = 4;
    }

    if( !pSrc || (planes != 1 && planes != 4) )
        return CV_NULLPTR_ERR;

    if( roiSize.width < 0 || roiSize.height < 0 ||
        srcStep < roiSize.width * 4 || dstStep < roiSize.width )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < planes; i++ )
    {
        dst_rb[i].size = 0;
        dst_rb[i].count = 0;
        dst_rb[i].step = dstStep;

        dst_rb[i].top = (char *) (pDst[coi - planes + i]);
        dst_rb[i].bottom = dst_rb[i].top + roiSize.height * dstStep;
        dst_rb[i].first = dst_rb[i].last = dst_rb[i].top;
    }

    if( planes == 1 )
        icvExtractCOI8uAC4( (char *) pSrc, srcStep, roiSize, dst_rb + 0, coi );
    else
        icvCpy_8u_C4P4R( (char *) pSrc, srcStep, roiSize, dst_rb + 0,
                         dst_rb + 1, dst_rb + 2, dst_rb + 3 );

    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_32f_C3P3R, (const float *pSrc, int srcStep,
                                            float *pDst[3], int dstStep, CvSize roiSize) )
{
    _CvRingBuffer dst_rb[3];
    int i, planes = 0, coi = 0;

    if( pDst[0] )
    {
        planes++;
        coi = 1;
    }
    if( pDst[1] )
    {
        planes++;
        coi = 2;
    }
    if( pDst[2] )
    {
        planes++;
        coi = 3;
    }

    if( !pSrc || (planes != 1 && planes != 3) )
        return CV_NULLPTR_ERR;

    if( roiSize.width <= 0 || roiSize.height <= 0 ||
        srcStep < roiSize.width * 12 || dstStep < roiSize.width * 4 )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < planes; i++ )
    {
        dst_rb[i].size = 0;
        dst_rb[i].count = 0;
        dst_rb[i].step = dstStep;

        dst_rb[i].top = (char *) (pDst[coi - planes + i]);
        dst_rb[i].bottom = dst_rb[i].top + roiSize.height * dstStep;
        dst_rb[i].first = dst_rb[i].last = dst_rb[i].top;
    }

    if( planes == 1 )
        icvExtractCOI32fC3( (char *) pSrc, srcStep, roiSize, dst_rb + 0, coi );
    else
        icvCpy_32f_C3P3R( (char *) pSrc, srcStep, roiSize, dst_rb + 0,
                          dst_rb + 1, dst_rb + 2, 0 );

    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_32f_C4P4R, (const float *pSrc, int srcStep,
                                            float *pDst[4], int dstStep, CvSize roiSize) )
{
    _CvRingBuffer dst_rb[4];
    int i, planes = 0, coi = 0;

    if( pDst[0] )
    {
        planes++;
        coi = 1;
    }
    if( pDst[1] )
    {
        planes++;
        coi = 2;
    }
    if( pDst[2] )
    {
        planes++;
        coi = 3;
    }
    if( pDst[3] )
    {
        planes++;
        coi = 4;
    }

    if( !pSrc || (planes != 1 && planes != 4) )
        return CV_NULLPTR_ERR;

    if( roiSize.width < 0 || roiSize.height < 0 ||
        srcStep < roiSize.width * 4 || dstStep < roiSize.width * 4 )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < planes; i++ )
    {
        dst_rb[i].size = 0;
        dst_rb[i].count = 0;
        dst_rb[i].step = dstStep;

        dst_rb[i].top = (char *) (pDst[coi - planes + i]);
        dst_rb[i].bottom = dst_rb[i].top + roiSize.height * dstStep;
        dst_rb[i].first = dst_rb[i].last = dst_rb[i].top;
    }

    if( planes == 1 )
        icvExtractCOI32fAC4( (char *) pSrc, srcStep, roiSize, dst_rb + 0, coi );
    else
        icvCpy_32f_C4P4R( (char *) pSrc, srcStep, roiSize, dst_rb + 0,
                          dst_rb + 1, dst_rb + 2, dst_rb + 3 );

    return CV_OK;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name: ippiCopy_PXCX
//    Purpose: divides pixel image into separate planes
//    Context:
//    Parameters:
//      pSrc       - pointer to source image or its ROI
//      srcStep  - full width of source image
//      dst0      - first destination plane
//      dst1      - second destination plane
//      dst2      - third destination plane
//      dst3      - fourth destination plane
//      dstStep  - full width of planes
//      roiSize      - width and height of images in pixels
//    Returns:
//      CV_OK or error code
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvCopy_8u_P3C3R, (const uchar * src[3], int srcStep,
                                           uchar * dst, int dstStep, CvSize roiSize) )
{
    _CvRingBuffer src_rb[3];
    int i;

    if( !src[0] || !src[1] || !src[2] || !dst )
        return CV_NULLPTR_ERR;

    if( roiSize.width <= 0 || roiSize.height <= 0 ||
        dstStep < roiSize.width * 3 || srcStep < roiSize.width )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < 3; i++ )
    {
        src_rb[i].size = 0;
        src_rb[i].step = srcStep;
        src_rb[i].count = roiSize.height;

        src_rb[i].top = (char *) src[i];
        src_rb[i].bottom = src_rb[i].top + roiSize.height * srcStep;
        src_rb[i].first = src_rb[i].last = src_rb[i].top;
    }

    icvCpy_8u_P3C3R( (char *) dst, dstStep, roiSize, src_rb, src_rb + 1, src_rb + 2, 0 );
    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_8u_P4C4R, (const uchar * src[4], int srcStep,
                                           uchar * dst, int dstStep, CvSize roiSize) )
{
    _CvRingBuffer src_rb[4];
    int i;

    if( !src[0] || !src[1] || !src[2] || !src[3] || !dst )
        return CV_NULLPTR_ERR;

    if( roiSize.width <= 0 || roiSize.height <= 0 ||
        dstStep < roiSize.width * 4 || srcStep < roiSize.width )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < 4; i++ )
    {
        src_rb[i].size = 0;
        src_rb[i].step = srcStep;
        src_rb[i].count = roiSize.height;

        src_rb[i].top = (char *) src[i];
        src_rb[i].bottom = src_rb[i].top + roiSize.height * srcStep;
        src_rb[i].first = src_rb[i].last = src_rb[i].top;
    }

    icvCpy_8u_P4C4R( (char *) dst, dstStep, roiSize,
                     src_rb, src_rb + 1, src_rb + 2, src_rb + 3 );
    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_32f_P3C3R, (const float *src[3], int srcStep,
                                            float *dst, int dstStep, CvSize roiSize) )
{
    _CvRingBuffer src_rb[3];
    int i;

    if( !src[0] || !src[1] || !src[2] || !dst )
        return CV_NULLPTR_ERR;

    if( roiSize.width <= 0 || roiSize.height <= 0 )
        return CV_BADSIZE_ERR;

    if( dstStep < roiSize.width * 12 || srcStep < roiSize.width * 4 )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < 3; i++ )
    {
        src_rb[i].size = 0;
        src_rb[i].step = srcStep;
        src_rb[i].count = roiSize.height;

        src_rb[i].top = (char *) src[i];
        src_rb[i].bottom = src_rb[i].top + roiSize.height * srcStep;
        src_rb[i].first = src_rb[i].last = src_rb[i].top;
    }

    icvCpy_32f_P3C3R( (char *) dst, dstStep, roiSize, src_rb, src_rb + 1, src_rb + 2, 0 );
    return CV_OK;
}


IPCVAPI_IMPL( CvStatus, icvCopy_32f_P4C4R, (const float *src[4], int srcStep,
                                            float *dst, int dstStep, CvSize roiSize) )
{
    _CvRingBuffer src_rb[4];
    int i;

    if( !src[0] || !src[1] || !src[2] || !src[3] || !dst )
        return CV_NULLPTR_ERR;

    if( roiSize.width <= 0 || roiSize.height <= 0 )
        return CV_BADSIZE_ERR;

    if( dstStep < roiSize.width * 12 || srcStep < roiSize.width * 4 )
        return CV_BADSIZE_ERR;

    /* prepare ring buffers */
    for( i = 0; i < 4; i++ )
    {
        src_rb[i].size = 0;
        src_rb[i].step = srcStep;
        src_rb[i].count = roiSize.height;

        src_rb[i].top = (char *) src[i];
        src_rb[i].bottom = src_rb[i].top + roiSize.height * srcStep;
        src_rb[i].first = src_rb[i].last = src_rb[i].top;
    }

    icvCpy_32f_P4C4R( (char *) dst, dstStep, roiSize,
                      src_rb, src_rb + 1, src_rb + 2, src_rb + 3 );
    return CV_OK;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvExtractROI_8u32f
//    Purpose:
//      Extracts ROI from byte-depth image and stores it in single continuous block.
//    Context:
//    Parameters:
//      src        - source image (not pointer to ROI, but to origin of the whole image
//      src_step   - full width of the source image (in bytes)
//      dst_buffer - destination buffer
//      src_rect   - ROI rectangle.
//    Returns:
//      Nothing       
//    Notes:
F*/
void
icvExtractROI_8u32f( uchar * src, int src_step, float *dst_buffer, CvRect src_rect )
{
    int x, y;

    assert( src && dst_buffer );
    assert( (src_rect.x | src_rect.y | src_rect.width | src_rect.height) >= 0 &&
            src_step >= src_rect.x + src_rect.width );

    src += src_rect.y * src_step + src_rect.x;

    for( y = 0; y < src_rect.height; y++, src += src_step, dst_buffer += src_rect.width )
    {
        for( x = 0; x < src_rect.width; x++ )
            dst_buffer[x] = src[x];
    }
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvRGBToGray8u_C3C1R
//    Purpose:
//      Converts RGB image into grayscale format
//    Context:
//    Parameters:
//      rgb_img    - source image
//      rgb_step   - full width of the source image (in bytes)
//      gray_img   - grayscale image
//      gray_step  - full width of the grayscale image (in bytes)
//      size       - width and height of the images in pixels
//    Returns:
//      CV_OK or error code
//    Notes:
F*/
IPCVAPI_IMPL( CvStatus, icvRGBToGray_8uC3C1R, (uchar * rgb_img, int rgb_step,
                                               uchar * gray_img, int gray_step, CvSize size) )
{
    int j, j0 = -size.width;
    uchar *gray_bottom = gray_img + size.height * gray_step + size.width;

    if( !rgb_img || !gray_img )
        return CV_NULLPTR_ERR;
    if( size.width <= 0 || size.height <= 0 ||
        rgb_step < size.width * 3 || gray_step < size.width )
        return CV_BADSIZE_ERR;

    rgb_step -= size.width * 3;

    for( gray_img += size.width; gray_img < gray_bottom; gray_img += gray_step,
         rgb_img += rgb_step )
        for( j = j0; j < 0; j++, rgb_img += 3 )
        {
            gray_img[j] = (uchar) ((rgb_img[0] + rgb_img[1] + rgb_img[2]) / 3);
        }

    return CV_OK;
}


extern const float icv8x32fTab[] = { 
-128.f, -127.f, -126.f, -125.f, -124.f, -123.f, -122.f, -121.f,
-120.f, -119.f, -118.f, -117.f, -116.f, -115.f, -114.f, -113.f,
-112.f, -111.f, -110.f, -109.f, -108.f, -107.f, -106.f, -105.f,
-104.f, -103.f, -102.f, -101.f, -100.f,  -99.f,  -98.f,  -97.f,
 -96.f,  -95.f,  -94.f,  -93.f,  -92.f,  -91.f,  -90.f,  -89.f,
 -88.f,  -87.f,  -86.f,  -85.f,  -84.f,  -83.f,  -82.f,  -81.f,
 -80.f,  -79.f,  -78.f,  -77.f,  -76.f,  -75.f,  -74.f,  -73.f,
 -72.f,  -71.f,  -70.f,  -69.f,  -68.f,  -67.f,  -66.f,  -65.f,
 -64.f,  -63.f,  -62.f,  -61.f,  -60.f,  -59.f,  -58.f,  -57.f,
 -56.f,  -55.f,  -54.f,  -53.f,  -52.f,  -51.f,  -50.f,  -49.f,
 -48.f,  -47.f,  -46.f,  -45.f,  -44.f,  -43.f,  -42.f,  -41.f,
 -40.f,  -39.f,  -38.f,  -37.f,  -36.f,  -35.f,  -34.f,  -33.f,
 -32.f,  -31.f,  -30.f,  -29.f,  -28.f,  -27.f,  -26.f,  -25.f,
 -24.f,  -23.f,  -22.f,  -21.f,  -20.f,  -19.f,  -18.f,  -17.f,
 -16.f,  -15.f,  -14.f,  -13.f,  -12.f,  -11.f,  -10.f,   -9.f,
  -8.f,   -7.f,   -6.f,   -5.f,   -4.f,   -3.f,   -2.f,   -1.f,
   0.f,    1.f,    2.f,    3.f,    4.f,    5.f,    6.f,    7.f,
   8.f,    9.f,   10.f,   11.f,   12.f,   13.f,   14.f,   15.f,
  16.f,   17.f,   18.f,   19.f,   20.f,   21.f,   22.f,   23.f,
  24.f,   25.f,   26.f,   27.f,   28.f,   29.f,   30.f,   31.f,
  32.f,   33.f,   34.f,   35.f,   36.f,   37.f,   38.f,   39.f,
  40.f,   41.f,   42.f,   43.f,   44.f,   45.f,   46.f,   47.f,
  48.f,   49.f,   50.f,   51.f,   52.f,   53.f,   54.f,   55.f,
  56.f,   57.f,   58.f,   59.f,   60.f,   61.f,   62.f,   63.f,
  64.f,   65.f,   66.f,   67.f,   68.f,   69.f,   70.f,   71.f,
  72.f,   73.f,   74.f,   75.f,   76.f,   77.f,   78.f,   79.f,
  80.f,   81.f,   82.f,   83.f,   84.f,   85.f,   86.f,   87.f,
  88.f,   89.f,   90.f,   91.f,   92.f,   93.f,   94.f,   95.f,
  96.f,   97.f,   98.f,   99.f,  100.f,  101.f,  102.f,  103.f,
 104.f,  105.f,  106.f,  107.f,  108.f,  109.f,  110.f,  111.f,
 112.f,  113.f,  114.f,  115.f,  116.f,  117.f,  118.f,  119.f,
 120.f,  121.f,  122.f,  123.f,  124.f,  125.f,  126.f,  127.f,
 128.f,  129.f,  130.f,  131.f,  132.f,  133.f,  134.f,  135.f,
 136.f,  137.f,  138.f,  139.f,  140.f,  141.f,  142.f,  143.f,
 144.f,  145.f,  146.f,  147.f,  148.f,  149.f,  150.f,  151.f,
 152.f,  153.f,  154.f,  155.f,  156.f,  157.f,  158.f,  159.f,
 160.f,  161.f,  162.f,  163.f,  164.f,  165.f,  166.f,  167.f,
 168.f,  169.f,  170.f,  171.f,  172.f,  173.f,  174.f,  175.f,
 176.f,  177.f,  178.f,  179.f,  180.f,  181.f,  182.f,  183.f,
 184.f,  185.f,  186.f,  187.f,  188.f,  189.f,  190.f,  191.f,
 192.f,  193.f,  194.f,  195.f,  196.f,  197.f,  198.f,  199.f,
 200.f,  201.f,  202.f,  203.f,  204.f,  205.f,  206.f,  207.f,
 208.f,  209.f,  210.f,  211.f,  212.f,  213.f,  214.f,  215.f,
 216.f,  217.f,  218.f,  219.f,  220.f,  221.f,  222.f,  223.f,
 224.f,  225.f,  226.f,  227.f,  228.f,  229.f,  230.f,  231.f,
 232.f,  233.f,  234.f,  235.f,  236.f,  237.f,  238.f,  239.f,
 240.f,  241.f,  242.f,  243.f,  244.f,  245.f,  246.f,  247.f,
 248.f,  249.f,  250.f,  251.f,  252.f,  253.f,  254.f,  255.f};


/* conversion byte->float: dst(x,y) = scale*src(x,y) + shift */
IPCVAPI_IMPL( CvStatus,
icvCvt_8u32f_C1R, (const uchar *src_data, int src_step,
                   float *dst_data, int dst_step,
                   CvSize size, double scale, double shift) )
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                float t0 = CV_8TO32F(src_data[x]);
                float t1 = CV_8TO32F(src_data[x + 1]);

                dst_data[x] = t0;
                dst_data[x + 1] = t1;
                t0 = CV_8TO32F(src_data[x + 2]);
                t1 = CV_8TO32F(src_data[x + 3]);
                dst_data[x + 2] = t0;
                dst_data[x + 3] = t1;
            }

            for( ; x < size.width; x++ )
            {
                dst_data[x] = CV_8TO32F(src_data[x]);
            }
        }
    }
    else
    {
        if( size.width*size.height > 256 )
        {
            float lut[256];
            double val = shift;

            for( x = 0; x < 256; x++ )
            {
                lut[x] = (float)val;
                val += scale;
            }

            for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
            {
                for( x = 0; x <= size.width - 4; x += 4 )
                {
                    float t0 = lut[src_data[x]];
                    float t1 = lut[src_data[x + 1]];

                    dst_data[x] = t0;
                    dst_data[x + 1] = t1;
                    t0 = lut[src_data[x + 2]];
                    t1 = lut[src_data[x + 3]];
                    dst_data[x + 2] = t0;
                    dst_data[x + 3] = t1;
                }

                for( ; x < size.width; x++ )
                {
                    dst_data[x] = lut[src_data[x]];
                }
            }
        }
        else
        {
            for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
            {
                for( x = 0; x <= size.width - 4; x += 4 )
                {
                    float t0 = (float)(CV_8TO32F(src_data[x]) * scale + shift);
                    float t1 = (float)(CV_8TO32F(src_data[x + 1]) * scale + shift);

                    dst_data[x] = t0;
                    dst_data[x + 1] = t1;
                    t0 = (float)(CV_8TO32F(src_data[x + 2]) * scale + shift);
                    t1 = (float)(CV_8TO32F(src_data[x + 3]) * scale + shift);
                    dst_data[x + 2] = t0;
                    dst_data[x + 3] = t1;
                }

                for( ; x < size.width; x++ )
                {
                    dst_data[x] = (float)(CV_8TO32F(src_data[x]) * scale + shift);
                }
            }
        }
    }

    return CV_OK;
}


/* conversion char->float: dst(x,y) = scale*src(x,y) + shift */
IPCVAPI_IMPL( CvStatus,
icvCvt_8s32f_C1R, ( const char *src_data, int src_step,
                    float *dst_data, int dst_step,
                    CvSize size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                float t0 = CV_8TO32F(src_data[x]);
                float t1 = CV_8TO32F(src_data[x + 1]);

                dst_data[x] = t0;
                dst_data[x + 1] = t1;
                t0 = CV_8TO32F(src_data[x + 2]);
                t1 = CV_8TO32F(src_data[x + 3]);
                dst_data[x + 2] = t0;
                dst_data[x + 3] = t1;
            }

            for( ; x < size.width; x++ )
            {
                dst_data[x] = CV_8TO32F(src_data[x]);
            }
        }
    }
    else
    {
        if( size.width*size.height > 256 )
        {
            float lut[256];
            double val = shift - 127*scale;

            for( x = 0; x < 256; x++ )
            {
                lut[x] = (float)val;
                val += scale;
            }

            for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
            {
                for( x = 0; x <= size.width - 4; x += 4 )
                {
                    float t0 = lut[src_data[x] + 128];
                    float t1 = lut[src_data[x + 1] + 128];

                    dst_data[x] = t0;
                    dst_data[x + 1] = t1;
                    t0 = lut[src_data[x + 2] + 128];
                    t1 = lut[src_data[x + 3] + 128];
                    dst_data[x + 2] = t0;
                    dst_data[x + 3] = t1;
                }

                for( ; x < size.width; x++ )
                {
                    dst_data[x] = lut[src_data[x] + 128];
                }
            }
        }
        else
        {
            for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
            {
                for( x = 0; x <= size.width - 4; x += 4 )
                {
                    float t0 = (float)(CV_8TO32F(src_data[x]) * scale + shift);
                    float t1 = (float)(CV_8TO32F(src_data[x + 1]) * scale + shift);

                    dst_data[x] = t0;
                    dst_data[x + 1] = t1;
                    t0 = (float)(CV_8TO32F(src_data[x + 2]) * scale + shift);
                    t1 = (float)(CV_8TO32F(src_data[x + 3]) * scale + shift);
                    dst_data[x + 2] = t0;
                    dst_data[x + 3] = t1;
                }

                for( ; x < size.width; x++ )
                {
                    dst_data[x] = (float)(CV_8TO32F(src_data[x]) * scale + shift);
                }
            }
        }
    }

    return CV_OK;
}


/* conversion float->byte: dst(x,y) = saturate(scale*src(x,y) + shift) */
IPCVAPI_IMPL( CvStatus,
icvCvt_32f8u_C1R, (const float *src_data, int src_step,
                   uchar *dst_data, int dst_step,
                   CvSize size, double scale, double shift) )
{
    int y, x;

    if( !src_data || !dst_data )
        return CV_NULLPTR_ERR;

    if( size.width <= 0 || size.height <= 0 || src_step < size.width * 4 ||
        (src_step & 3) != 0 || dst_step < size.width )
        return CV_BADSIZE_ERR;

    if( scale == 0 )
        return CV_BADRANGE_ERR;
    src_step /= 4;

    /* check for continuous array case */
    if( src_step == size.width && dst_step == size.width )
    {
        size.width *= size.height;
        size.height = 1;
    }

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                int t0 = cvRound( src_data[x] ), t1 = cvRound( src_data[x + 1] );

                dst_data[x] = CV_SATURATE_8U( t0 );
                dst_data[x + 1] = CV_SATURATE_8U( t1 );
                t0 = cvRound( src_data[x + 2] );
                t1 = cvRound( src_data[x + 3] );
                dst_data[x + 2] = CV_SATURATE_8U( t0 );
                dst_data[x + 3] = CV_SATURATE_8U( t1 );
            }

            for( ; x < size.width; x++ )
            {
                int t0 = cvRound( src_data[x] );

                dst_data[x] = CV_SATURATE_8U( t0 );
            }
        }
    }
    else
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                int t0 = cvRound( src_data[x] * scale + shift ),

                    t1 = cvRound( src_data[x + 1] * scale + shift );
                dst_data[x] = CV_SATURATE_8U( t0 );
                dst_data[x + 1] = CV_SATURATE_8U( t1 );
                t0 = cvRound( src_data[x + 2] * scale + shift );
                t1 = cvRound( src_data[x + 3] * scale + shift );
                dst_data[x + 2] = CV_SATURATE_8U( t0 );
                dst_data[x + 3] = CV_SATURATE_8U( t1 );
            }

            for( ; x < size.width; x++ )
            {
                int t0 = cvRound( src_data[x] * scale + shift );

                dst_data[x] = CV_SATURATE_8U( t0 );
            }
        }
    }

    return CV_OK;
}


/* conversion float->byte: dst(x,y) = saturate(scale*src(x,y) + shift) */
IPCVAPI_IMPL( CvStatus,
icvCvt_32f8s_C1R, ( const float *src_data, int src_step,
                    char *dst_data, int dst_step,
                    CvSize size, double scale, double shift ))
{
    int y, x;

    if( !src_data || !dst_data )
        return CV_NULLPTR_ERR;

    if( size.width <= 0 || size.height <= 0 || src_step < size.width * 4 ||
        (src_step & 3) != 0 || dst_step < size.width )
        return CV_BADSIZE_ERR;

    if( scale == 0 )
        return CV_BADRANGE_ERR;
    src_step /= 4;

    /* check for continuous array case */
    if( src_step == size.width && dst_step == size.width )
    {
        size.width *= size.height;
        size.height = 1;
    }

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                int t0 = cvRound( src_data[x] ), t1 = cvRound( src_data[x + 1] );

                dst_data[x] = CV_SATURATE_8S( t0 );
                dst_data[x + 1] = CV_SATURATE_8S( t1 );
                t0 = cvRound( src_data[x + 2] );
                t1 = cvRound( src_data[x + 3] );
                dst_data[x + 2] = CV_SATURATE_8S( t0 );
                dst_data[x + 3] = CV_SATURATE_8S( t1 );
            }

            for( ; x < size.width; x++ )
            {
                int t0 = cvRound( src_data[x] );

                dst_data[x] = CV_SATURATE_8S( t0 );
            }
        }
    }
    else
    {
        for( y = 0; y < size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= size.width - 4; x += 4 )
            {
                int t0 = cvRound( src_data[x] * scale + shift ),

                    t1 = cvRound( src_data[x + 1] * scale + shift );
                dst_data[x] = CV_SATURATE_8S( t0 );
                dst_data[x + 1] = CV_SATURATE_8S( t1 );
                t0 = cvRound( src_data[x + 2] * scale + shift );
                t1 = cvRound( src_data[x + 3] * scale + shift );
                dst_data[x + 2] = CV_SATURATE_8S( t0 );
                dst_data[x + 3] = CV_SATURATE_8S( t1 );
            }

            for( ; x < size.width; x++ )
            {
                int t0 = cvRound( src_data[x] * scale + shift );

                dst_data[x] = CV_SATURATE_8S( t0 );
            }
        }
    }

    return CV_OK;
}


/* Type conversion float -> double */
CvStatus
icvCvt_32f_64d( float *src, double *dst, int size )
{
    int t;

    if( !src || !dst )
        return CV_NULLPTR_ERR;
    if( size <= 0 )
        return CV_BADRANGE_ERR;

    for( t = 0; t < size; t++ )
    {
        dst[t] = (double) (src[t]);
    }

    return CV_OK;
}

/*======================================================================================*/
/* Type conversion double -> float */
CvStatus
icvCvt_64d_32f( double *src, float *dst, int size )
{
    int t;

    if( !src || !dst )
        return CV_NULLPTR_ERR;
    if( size <= 0 )
        return CV_BADRANGE_ERR;

    for( t = 0; t < size; t++ )
    {
        dst[t] = (float) (src[t]);
    }

    return CV_OK;
}


/* conversion float->int with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_32f32s_C1R, ( const float *src_data, int src_step,
                     int *dst_data, int dst_step,
                     CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                dst_data[x] = cvRound( src_data[x] );
            }
    }
    else
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                dst_data[x] = cvRound( src_data[x] * scale + shift );
            }
    }

    return CV_OK;
}


/* conversion int->float with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_32s32f_C1R, ( const int *src_data, int src_step,
                     float *dst_data, int dst_step,
                     CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                dst_data[x] = (float)src_data[x];
            }
    }
    else
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                dst_data[x] = (float)(src_data[x] * scale + shift);
            }
    }

    return CV_OK;
}


/* conversion float->short with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_32f16s_C1R, ( const float *src_data, int src_step,
                     short *dst_data, int dst_step,
                     CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                int t = cvRound( src_data[x] );
                dst_data[x] = CV_SATURATE_16S(t);
            }
    }
    else
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            for( x = 0; x < src_size.width; x++ )
            {
                int t = cvRound( src_data[x]*scale + shift );
                dst_data[x] = CV_SATURATE_16S(t);
            }
    }

    return CV_OK;
}


/* conversion short->float with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_16s32f_C1R, ( const short *src_data, int src_step,
                     float *dst_data, int dst_step,
                     CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                float t0 = (float)src_data[x];
                float t1 = (float)src_data[x + 1];

                dst_data[x] = t0;
                dst_data[x + 1] = t1;

                t0 = (float)src_data[x + 2];
                t1 = (float)src_data[x + 3];

                dst_data[x + 2] = t0;
                dst_data[x + 3] = t1;
            }

            for( ; x < src_size.width; x++ )
            {
                dst_data[x] = (float)src_data[x];
            }
        }
    }
    else
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                float t0 = (float)(src_data[x] * scale + shift);
                float t1 = (float)(src_data[x + 1] * scale + shift);

                dst_data[x] = t0;
                dst_data[x + 1] = t1;

                t0 = (float)(src_data[x + 2] * scale + shift);
                t1 = (float)(src_data[x + 3] * scale + shift);

                dst_data[x + 2] = t0;
                dst_data[x + 3] = t1;
            }

            for( ; x < src_size.width; x++ )
            {
                dst_data[x] = (float)(src_data[x] * scale + shift);
            }
        }
    }

    return CV_OK;
}


#define  SCALE_SHIFT  14

/* conversion short->byte with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_16s8u_C1R, ( const short *src_data, int src_step,
                    uchar *dst_data, int dst_step,
                    CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                int t0 = src_data[x];
                int t1 = src_data[x + 1];

                dst_data[x] = CV_SATURATE_8U( t0 );
                dst_data[x + 1] = CV_SATURATE_8U( t1 );

                t0 = src_data[x + 2];
                t1 = src_data[x + 3];

                dst_data[x + 2] = CV_SATURATE_8U( t0 );
                dst_data[x + 3] = CV_SATURATE_8U( t1 );
            }
            
            for( ; x < src_size.width; x++ )
            {
                int t = src_data[x];
                dst_data[x] = CV_SATURATE_8U( t );
            }
        }
    }
    else
    {
        if( fabs(scale) > 1 || fabs(shift) > 32767 )
        {
            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = cvRound( src_data[x]*scale + shift );
                    dst_data[x] = CV_SATURATE_8U( t );
                }
        }
        else
        {
            int iscale = CV_FLT_TO_FIX( scale, SCALE_SHIFT );
            int ishift = CV_FLT_TO_FIX( shift, SCALE_SHIFT );

            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
            {
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = CV_DESCALE( src_data[x]*iscale + ishift, SCALE_SHIFT );
                    dst_data[x] = CV_SATURATE_8U( t );
                }
            }
        }
    }

    return CV_OK;
}


/* conversion short->byte with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_8u16s_C1R, ( const uchar *src_data, int src_step,
                    short *dst_data, int dst_step,
                    CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                int t0 = src_data[x];
                int t1 = src_data[x + 1];

                dst_data[x] = (short)t0;
                dst_data[x + 1] = (short)t1;

                t0 = src_data[x + 2];
                t1 = src_data[x + 3];

                dst_data[x + 2] = (short)t0;
                dst_data[x + 3] = (short)t1;
            }

            for( ; x < src_size.width; x++ )
            {
                int t = src_data[x];
                dst_data[x] = (short)t;
            }
        }
    }
    else
    {
        if( fabs(scale) >= 256 || fabs(shift) > 32767 )
        {
            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = cvRound( src_data[x]*scale + shift );
                    dst_data[x] = CV_SATURATE_16S( t );
                }
        }
        else
        {
            int iscale = CV_FLT_TO_FIX( scale, SCALE_SHIFT );
            int ishift = CV_FLT_TO_FIX( shift, SCALE_SHIFT );

            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = CV_DESCALE( src_data[x]*iscale + ishift, SCALE_SHIFT );
                    dst_data[x] = CV_SATURATE_16S( t );
                }
        }
    }

    return CV_OK;
}


/* conversion short->byte with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_16s8s_C1R, ( const short *src_data, int src_step,
                    char *dst_data, int dst_step,
                    CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                int t0 = src_data[x];
                int t1 = src_data[x + 1];

                dst_data[x] = CV_SATURATE_8S( t0 );
                dst_data[x + 1] = CV_SATURATE_8S( t1 );

                t0 = src_data[x + 2];
                t1 = src_data[x + 3];

                dst_data[x + 2] = CV_SATURATE_8S( t0 );
                dst_data[x + 3] = CV_SATURATE_8S( t1 );
            }

            for( x = 0; x < src_size.width; x++ )
            {
                int t = src_data[x];
                dst_data[x] = CV_SATURATE_8S( t );
            }
        }
    }
    else
    {
        if( fabs(scale) > 1 || fabs(shift) > 32767 )
        {
            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = cvRound( src_data[x]*scale + shift );
                    dst_data[x] = CV_SATURATE_8S( t );
                }
        }
        else
        {
            int iscale = CV_FLT_TO_FIX( scale, SCALE_SHIFT );
            int ishift = CV_FLT_TO_FIX( shift, SCALE_SHIFT );

            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = CV_DESCALE( src_data[x]*iscale + ishift, SCALE_SHIFT );
                    dst_data[x] = CV_SATURATE_8S( t );
                }
        }
    }

    return CV_OK;
}


/* conversion short->byte with scaling */
IPCVAPI_IMPL( CvStatus,
icvCvt_8s16s_C1R, ( const char *src_data, int src_step,
                    short *dst_data, int dst_step,
                    CvSize src_size, double scale, double shift ))
{
    int x, y;

    src_step /= sizeof(src_data[0]);
    dst_step /= sizeof(dst_data[0]);

    if( scale == 1 && shift == 0 )
    {
        for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
        {
            for( x = 0; x <= src_size.width - 4; x += 4 )
            {
                int t0 = src_data[x];
                int t1 = src_data[x + 1];

                dst_data[x] = (short)t0;
                dst_data[x + 1] = (short)t1;

                t0 = src_data[x + 2];
                t1 = src_data[x + 3];

                dst_data[x + 2] = (short)t0;
                dst_data[x + 3] = (short)t1;
            }

            for( x = 0; x < src_size.width; x++ )
            {
                int t = src_data[x];
                dst_data[x] = (short)t;
            }
        }
    }
    else
    {
        if( fabs(scale) >= 256 || fabs(shift) > 32767 )
        {
            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = cvRound( src_data[x]*scale + shift );
                    dst_data[x] = CV_SATURATE_16S( t );
                }
        }
        else
        {
            int iscale = CV_FLT_TO_FIX( scale, SCALE_SHIFT );
            int ishift = CV_FLT_TO_FIX( shift, SCALE_SHIFT );

            for( y = 0; y < src_size.height; y++, src_data += src_step, dst_data += dst_step )
                for( x = 0; x < src_size.width; x++ )
                {
                    int t = CV_DESCALE( src_data[x]*iscale + ishift, SCALE_SHIFT );
                    dst_data[x] = CV_SATURATE_16S( t );
                }
        }
    }

    return CV_OK;
}


/* End of file. */
