/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "_cv.h"

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    cvCreateVect32f
//   Purpose:
//     Creates new float vector
//   Context:
//   Parameters:
//     length - size of new vector (in elements)
//   Returns:
//     New vector or NULL if No Memory
//   Notes:
//F*/
IPCVAPI_IMPL( CvVect32f, icvCreateVector_32f, (int length) )
{
    CvVect32f Vect;

    if( length <= 0 )
        return NULL;
    Vect = (CvVect32f) icvAlloc( length * 4 );
    return Vect;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    cvCreateVect64d
//   Purpose:
//     Creates new double vector
//   Context:
//   Parameters:
//     length - size of new vector (in elements)
//   Returns:
//     New vector or NULL if No Memory
//   Notes:
//F*/
IPCVAPI_IMPL( CvVect64d, icvCreateVector_64d, (int length) )
{
    CvVect64d Vect;

    if( length <= 0 )
        return NULL;
    Vect = (CvVect64d) icvAlloc( length * 8 );
    return Vect;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    cvCreateMatrix32f
//   Purpose:
//     Creates new matrix with float elements
//   Context:
//   Parameters:
//     width - number of columns in  matrix
//     height - number of rows in new matrix  
//   Returns:
//     New matrix or NULL if No Memory
//   Notes:
//F*/
IPCVAPI_IMPL( CvMatr32f, icvCreateMatrix_32f, (int width, int height) )
{
    CvMatr32f Matr;

    if( (width <= 0) || (height <= 0) )
        return NULL;
    Matr = (CvMatr32f) icvAlloc( height * width * 4 );
    return Matr;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//   Name:    cvCreateMatrix64d
//   Purpose:
//     Creates new matrix with double elements
//   Context:
//   Parameters:
//     width - number of columns in  matrix
//     height - number of rows in new matrix  
//   Returns:
//     New matrix or NULL if No Memory
//   Notes:
//F*/
IPCVAPI_IMPL( CvMatr64d, icvCreateMatrix_64d, (int width, int height) )
{
    CvMatr64d Matr;

    if( (width <= 0) || (height <= 0) )
        return NULL;
    Matr = (CvMatr64d) icvAlloc( height * width * 8 );
    return Matr;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvDeleteVect
//    Purpose:
//      Deletes vector
//    Context:
//    Parameters:
//      pPointer - pointer to vector to delete
//    Returns:
//      CV_NO_ERR if all Ok or error code
//    Notes:
//      The vector must be created by  by cvsCreateVector function
//F*/
IPCVAPI_IMPL( CvStatus, icvDeleteVector, (void *Pointer) )
{
    icvFree( &Pointer );
    return CV_OK;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:    icvDeleteMatr
//    Purpose:
//      Deletes matrix
//    Context:
//    Parameters:
//      pPointer - pointer to matrix to delete
//    Returns:
//      CV_NO_ERR if all Ok or error code
//    Notes:
//      The memory block must be allocated by cvsCreateMatrix function
//F*/
IPCVAPI_IMPL( CvStatus, icvDeleteMatrix, (void *Pointer) )
{
    icvFree( &Pointer );
    return CV_OK;
}

/* End of file */
