/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/
#include "_cv.h"
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvAddMatrix3x3_32f  
//    Purpose:    Sum of 2 matrix 3x3
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvAddMatrix3x3_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{

    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] + srcMatr2[0];
    destMatr[1] = srcMatr1[1] + srcMatr2[1];
    destMatr[2] = srcMatr1[2] + srcMatr2[2];
    destMatr[3] = srcMatr1[3] + srcMatr2[3];
    destMatr[4] = srcMatr1[4] + srcMatr2[4];
    destMatr[5] = srcMatr1[5] + srcMatr2[5];
    destMatr[6] = srcMatr1[6] + srcMatr2[6];
    destMatr[7] = srcMatr1[7] + srcMatr2[7];
    destMatr[8] = srcMatr1[8] + srcMatr2[8];

    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvSub3x3Matr32f  
//    Purpose:    Difference of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvSubMatrix3x3_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{

    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] - srcMatr2[0];
    destMatr[1] = srcMatr1[1] - srcMatr2[1];
    destMatr[2] = srcMatr1[2] - srcMatr2[2];
    destMatr[3] = srcMatr1[3] - srcMatr2[3];
    destMatr[4] = srcMatr1[4] - srcMatr2[4];
    destMatr[5] = srcMatr1[5] - srcMatr2[5];
    destMatr[6] = srcMatr1[6] - srcMatr2[6];
    destMatr[7] = srcMatr1[7] - srcMatr2[7];
    destMatr[8] = srcMatr1[8] - srcMatr2[8];
    return CV_NO_ERR;

}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvScaleMatrix3x3_32f  
//    Purpose:    Multiplies each element of matrix 3x3 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvScaleMatrix3x3_32f, (CvMatr32f srcMatr,
                                                CvMatr32f destMatr, float value) )
{
    /* check bad arguments */
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = value * srcMatr[0];
    destMatr[1] = value * srcMatr[1];
    destMatr[2] = value * srcMatr[2];
    destMatr[3] = value * srcMatr[3];
    destMatr[4] = value * srcMatr[4];
    destMatr[5] = value * srcMatr[5];
    destMatr[6] = value * srcMatr[6];
    destMatr[7] = value * srcMatr[7];
    destMatr[8] = value * srcMatr[8];
    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvMulMatrix3x3_32f   
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvMulMatrix3x3_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] * srcMatr2[0] +
        srcMatr1[1] * srcMatr2[3] + srcMatr1[2] * srcMatr2[6];
    destMatr[1] = srcMatr1[0] * srcMatr2[1] +
        srcMatr1[1] * srcMatr2[4] + srcMatr1[2] * srcMatr2[7];
    destMatr[2] = srcMatr1[0] * srcMatr2[2] +
        srcMatr1[1] * srcMatr2[5] + srcMatr1[2] * srcMatr2[8];
    destMatr[3] = srcMatr1[3] * srcMatr2[0] +
        srcMatr1[4] * srcMatr2[3] + srcMatr1[5] * srcMatr2[6];
    destMatr[4] = srcMatr1[3] * srcMatr2[1] +
        srcMatr1[4] * srcMatr2[4] + srcMatr1[5] * srcMatr2[7];
    destMatr[5] = srcMatr1[3] * srcMatr2[2] +
        srcMatr1[4] * srcMatr2[5] + srcMatr1[5] * srcMatr2[8];
    destMatr[6] = srcMatr1[6] * srcMatr2[0] +
        srcMatr1[7] * srcMatr2[3] + srcMatr1[8] * srcMatr2[6];
    destMatr[7] = srcMatr1[6] * srcMatr2[1] +
        srcMatr1[7] * srcMatr2[4] + srcMatr1[8] * srcMatr2[7];
    destMatr[8] = srcMatr1[6] * srcMatr2[2] +
        srcMatr1[7] * srcMatr2[5] + srcMatr1[8] * srcMatr2[8];
    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:      cvTransposeMatrix3x3_32f  
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/

IPCVAPI_IMPL( CvStatus, icvTransposeMatrix3x3_32f, (CvMatr32f srcMatr, CvMatr32f destMatr) )
{
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr[0];
    destMatr[1] = srcMatr[3];
    destMatr[2] = srcMatr[6];
    destMatr[3] = srcMatr[1];
    destMatr[4] = srcMatr[4];
    destMatr[5] = srcMatr[7];
    destMatr[6] = srcMatr[2];
    destMatr[7] = srcMatr[5];
    destMatr[8] = srcMatr[8];
    return CV_NO_ERR;
}

/* End of file */

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvDet3x3_32f   
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvDet3x3_32f, (CvMatr32f srcMatr, float *Det) )
{
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( Det == NULL )
        return CV_NULLPTR_ERR;
    *Det = srcMatr[0] * (srcMatr[4] * srcMatr[8] - srcMatr[5] * srcMatr[7]) +
        srcMatr[1] * (srcMatr[5] * srcMatr[6] - srcMatr[3] * srcMatr[8]) +
        srcMatr[2] * (srcMatr[3] * srcMatr[7] - srcMatr[4] * srcMatr[6]);
    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvInvertMatrix3x3_32f   
//    Purpose:    Inverses 3x3 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvInvertMatrix3x3_32f, (CvMatr32f srcMatr, CvMatr32f dstMatr) )
{
    float Mult = 1.0f / (srcMatr[0] * (srcMatr[4] * srcMatr[8] - srcMatr[5] * srcMatr[7]) +
                         srcMatr[1] * (srcMatr[5] * srcMatr[6] - srcMatr[3] * srcMatr[8]) +
                         srcMatr[2] * (srcMatr[3] * srcMatr[7] - srcMatr[4] * srcMatr[6]));

    dstMatr[0] = (srcMatr[4] * srcMatr[8] - srcMatr[5] * srcMatr[7]) * Mult;
    dstMatr[1] = (srcMatr[2] * srcMatr[7] - srcMatr[1] * srcMatr[8]) * Mult;
    dstMatr[2] = (srcMatr[1] * srcMatr[5] - srcMatr[2] * srcMatr[4]) * Mult;
    dstMatr[3] = (srcMatr[5] * srcMatr[6] - srcMatr[3] * srcMatr[8]) * Mult;
    dstMatr[4] = (srcMatr[0] * srcMatr[8] - srcMatr[2] * srcMatr[6]) * Mult;
    dstMatr[5] = (srcMatr[2] * srcMatr[3] - srcMatr[0] * srcMatr[5]) * Mult;
    dstMatr[6] = (srcMatr[3] * srcMatr[7] - srcMatr[4] * srcMatr[6]) * Mult;
    dstMatr[7] = (srcMatr[1] * srcMatr[6] - srcMatr[0] * srcMatr[7]) * Mult;
    dstMatr[8] = (srcMatr[0] * srcMatr[4] - srcMatr[1] * srcMatr[3]) * Mult;

    return CV_NO_ERR;
}

/* End of file */
