/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/
#include "_cv.h"
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvAddMatrix4x4_32f   
//    Purpose:    Sum of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvAddMatrix4x4_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] + srcMatr2[0];
    destMatr[1] = srcMatr1[1] + srcMatr2[1];
    destMatr[2] = srcMatr1[2] + srcMatr2[2];
    destMatr[3] = srcMatr1[3] + srcMatr2[3];
    destMatr[4] = srcMatr1[4] + srcMatr2[4];
    destMatr[5] = srcMatr1[5] + srcMatr2[5];
    destMatr[6] = srcMatr1[6] + srcMatr2[6];
    destMatr[7] = srcMatr1[7] + srcMatr2[7];
    destMatr[8] = srcMatr1[8] + srcMatr2[8];
    destMatr[9] = srcMatr1[9] + srcMatr2[9];
    destMatr[10] = srcMatr1[10] + srcMatr2[10];
    destMatr[11] = srcMatr1[11] + srcMatr2[11];
    destMatr[12] = srcMatr1[12] + srcMatr2[12];
    destMatr[13] = srcMatr1[13] + srcMatr2[13];
    destMatr[14] = srcMatr1[14] + srcMatr2[14];
    destMatr[15] = srcMatr1[15] + srcMatr2[15];
    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvSubMatrix4x4_32f   
//    Purpose:    Difference of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvSubMatrix4x4_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] - srcMatr2[0];
    destMatr[1] = srcMatr1[1] - srcMatr2[1];
    destMatr[2] = srcMatr1[2] - srcMatr2[2];
    destMatr[3] = srcMatr1[3] - srcMatr2[3];
    destMatr[4] = srcMatr1[4] - srcMatr2[4];
    destMatr[5] = srcMatr1[5] - srcMatr2[5];
    destMatr[6] = srcMatr1[6] - srcMatr2[6];
    destMatr[7] = srcMatr1[7] - srcMatr2[7];
    destMatr[8] = srcMatr1[8] - srcMatr2[8];
    destMatr[9] = srcMatr1[9] - srcMatr2[9];
    destMatr[10] = srcMatr1[10] - srcMatr2[10];
    destMatr[11] = srcMatr1[11] - srcMatr2[11];
    destMatr[12] = srcMatr1[12] - srcMatr2[12];
    destMatr[13] = srcMatr1[13] - srcMatr2[13];
    destMatr[14] = srcMatr1[14] - srcMatr2[14];
    destMatr[15] = srcMatr1[15] - srcMatr2[15];
    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvScaleMatrix4x4_32f  
//    Purpose:    Multiplies each element of matrix 4x4 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvScaleMatrix4x4_32f, (CvMatr32f srcMatr,
                                                CvMatr32f destMatr, float value) )
{
    /* check bad arguments */
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = value * srcMatr[0];
    destMatr[1] = value * srcMatr[1];
    destMatr[2] = value * srcMatr[2];
    destMatr[3] = value * srcMatr[3];
    destMatr[4] = value * srcMatr[4];
    destMatr[5] = value * srcMatr[5];
    destMatr[6] = value * srcMatr[6];
    destMatr[7] = value * srcMatr[7];
    destMatr[8] = value * srcMatr[8];
    destMatr[9] = value * srcMatr[9];
    destMatr[10] = value * srcMatr[10];
    destMatr[11] = value * srcMatr[11];
    destMatr[12] = value * srcMatr[12];
    destMatr[13] = value * srcMatr[13];
    destMatr[14] = value * srcMatr[14];
    destMatr[15] = value * srcMatr[15];
    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvMulMatrix4x4_32f  
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvMulMatrix4x4_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] * srcMatr2[0] +
        srcMatr1[1] * srcMatr2[4] + srcMatr1[2] * srcMatr2[8] + srcMatr1[3] * srcMatr2[12];
    destMatr[1] = srcMatr1[0] * srcMatr2[1] +
        srcMatr1[1] * srcMatr2[5] + srcMatr1[2] * srcMatr2[9] + srcMatr1[3] * srcMatr2[13];
    destMatr[2] = srcMatr1[0] * srcMatr2[2] +
        srcMatr1[1] * srcMatr2[6] + srcMatr1[2] * srcMatr2[10] + srcMatr1[3] * srcMatr2[14];
    destMatr[3] = srcMatr1[0] * srcMatr2[3] +
        srcMatr1[1] * srcMatr2[7] + srcMatr1[2] * srcMatr2[11] + srcMatr1[3] * srcMatr2[15];
    destMatr[4] = srcMatr1[4] * srcMatr2[0] +
        srcMatr1[5] * srcMatr2[4] + srcMatr1[6] * srcMatr2[8] + srcMatr1[7] * srcMatr2[12];
    destMatr[5] = srcMatr1[4] * srcMatr2[1] +
        srcMatr1[5] * srcMatr2[5] + srcMatr1[6] * srcMatr2[9] + srcMatr1[7] * srcMatr2[13];
    destMatr[6] = srcMatr1[4] * srcMatr2[2] +
        srcMatr1[5] * srcMatr2[6] + srcMatr1[6] * srcMatr2[10] + srcMatr1[7] * srcMatr2[14];
    destMatr[7] = srcMatr1[4] * srcMatr2[3] +
        srcMatr1[5] * srcMatr2[7] + srcMatr1[6] * srcMatr2[11] + srcMatr1[7] * srcMatr2[15];
    destMatr[8] = srcMatr1[8] * srcMatr2[0] +
        srcMatr1[9] * srcMatr2[4] + srcMatr1[10] * srcMatr2[8] + srcMatr1[11] * srcMatr2[12];
    destMatr[9] = srcMatr1[8] * srcMatr2[1] +
        srcMatr1[9] * srcMatr2[5] + srcMatr1[10] * srcMatr2[9] + srcMatr1[11] * srcMatr2[13];
    destMatr[10] = srcMatr1[8] * srcMatr2[2] +
        srcMatr1[9] * srcMatr2[6] + srcMatr1[10] * srcMatr2[10] + srcMatr1[11] * srcMatr2[14];
    destMatr[11] = srcMatr1[8] * srcMatr2[3] +
        srcMatr1[9] * srcMatr2[7] + srcMatr1[10] * srcMatr2[11] + srcMatr1[11] * srcMatr2[15];
    destMatr[12] = srcMatr1[12] * srcMatr2[0] +
        srcMatr1[13] * srcMatr2[4] + srcMatr1[14] * srcMatr2[8] + srcMatr1[15] * srcMatr2[12];
    destMatr[13] = srcMatr1[12] * srcMatr2[1] +
        srcMatr1[13] * srcMatr2[5] + srcMatr1[14] * srcMatr2[9] + srcMatr1[15] * srcMatr2[13];
    destMatr[14] = srcMatr1[12] * srcMatr2[2] +
        srcMatr1[13] * srcMatr2[6] + srcMatr1[14] * srcMatr2[10] + srcMatr1[15] * srcMatr2[14];
    destMatr[15] = srcMatr1[12] * srcMatr2[3] +
        srcMatr1[13] * srcMatr2[7] + srcMatr1[14] * srcMatr2[11] + srcMatr1[15] * srcMatr2[15];

    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvTranspose4x4Matr32f  
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvTransposeMatrix4x4_32f, (CvMatr32f srcMatr, CvMatr32f destMatr) )
{
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr[0];
    destMatr[1] = srcMatr[4];
    destMatr[2] = srcMatr[8];
    destMatr[3] = srcMatr[12];
    destMatr[4] = srcMatr[1];
    destMatr[5] = srcMatr[5];
    destMatr[6] = srcMatr[9];
    destMatr[7] = srcMatr[13];
    destMatr[8] = srcMatr[2];
    destMatr[9] = srcMatr[6];
    destMatr[10] = srcMatr[10];
    destMatr[11] = srcMatr[14];
    destMatr[12] = srcMatr[3];
    destMatr[13] = srcMatr[7];
    destMatr[14] = srcMatr[11];
    destMatr[15] = srcMatr[15];
    return CV_NO_ERR;
}

IPCVAPI_IMPL( CvStatus, icvDet4x4_32f, (CvMatr32f srcMatr, float *Det) )
{

    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( Det == NULL )
        return CV_NULLPTR_ERR;
    *Det = srcMatr[0] * (srcMatr[5] * (srcMatr[10] * srcMatr[15] - srcMatr[11] * srcMatr[14]) +
                         srcMatr[6] * (srcMatr[11] * srcMatr[13] - srcMatr[9] * srcMatr[15]) +
                         srcMatr[7] * (srcMatr[9] * srcMatr[14] - srcMatr[10] * srcMatr[13])) +
        srcMatr[1] * (srcMatr[6] * (srcMatr[8] * srcMatr[15] - srcMatr[11] * srcMatr[12]) +
                      srcMatr[7] * (srcMatr[10] * srcMatr[12] - srcMatr[8] * srcMatr[14]) +
                      srcMatr[4] * (srcMatr[11] * srcMatr[14] - srcMatr[10] * srcMatr[15])) +
        srcMatr[2] * (srcMatr[7] * (srcMatr[8] * srcMatr[13] - srcMatr[9] * srcMatr[12]) +
                      srcMatr[4] * (srcMatr[9] * srcMatr[15] - srcMatr[11] * srcMatr[13]) +
                      srcMatr[5] * (srcMatr[11] * srcMatr[12] - srcMatr[8] * srcMatr[15])) +
        srcMatr[3] * (srcMatr[4] * (srcMatr[10] * srcMatr[13] - srcMatr[9] * srcMatr[14]) +
                      srcMatr[5] * (srcMatr[8] * srcMatr[14] - srcMatr[10] * srcMatr[12]) +
                      srcMatr[6] * (srcMatr[9] * srcMatr[12] - srcMatr[8] * srcMatr[13]));



    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvInvertMatrix4x4_32f   
//    Purpose:    Invertes  4x4 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/
IPCVAPI_IMPL( CvStatus, icvInvertMatrix4x4_32f, (CvMatr32f srcMatr, CvMatr32f dstMatr) )
{
    float di = srcMatr[0];
    float d = 1.0f / di;

    dstMatr[0] = d;
    dstMatr[4] = srcMatr[4] * -d;
    dstMatr[8] = srcMatr[8] * -d;
    dstMatr[12] = srcMatr[12] * -d;
    dstMatr[1] = srcMatr[1] * d;
    dstMatr[2] = srcMatr[2] * d;
    dstMatr[3] = srcMatr[3] * d;
    dstMatr[5] = srcMatr[5] + dstMatr[4] * dstMatr[1] * di;
    dstMatr[6] = srcMatr[6] + dstMatr[4] * dstMatr[2] * di;
    dstMatr[7] = srcMatr[7] + dstMatr[4] * dstMatr[3] * di;
    dstMatr[9] = srcMatr[9] + dstMatr[8] * dstMatr[1] * di;
    dstMatr[10] = srcMatr[10] + dstMatr[8] * dstMatr[2] * di;
    dstMatr[11] = srcMatr[11] + dstMatr[8] * dstMatr[3] * di;
    dstMatr[13] = srcMatr[13] + dstMatr[12] * dstMatr[1] * di;
    dstMatr[14] = srcMatr[14] + dstMatr[12] * dstMatr[2] * di;
    dstMatr[15] = srcMatr[15] + dstMatr[12] * dstMatr[3] * di;
    di = dstMatr[5];
    dstMatr[5] = d = 1.0f / di;
    dstMatr[1] *= -d;
    dstMatr[9] *= -d;
    dstMatr[13] *= -d;
    dstMatr[4] *= d;
    dstMatr[6] *= d;
    dstMatr[7] *= d;
    dstMatr[0] += dstMatr[1] * dstMatr[4] * di;
    dstMatr[2] += dstMatr[1] * dstMatr[6] * di;
    dstMatr[3] += dstMatr[1] * dstMatr[7] * di;
    dstMatr[8] += dstMatr[9] * dstMatr[4] * di;
    dstMatr[10] += dstMatr[9] * dstMatr[6] * di;
    dstMatr[11] += dstMatr[9] * dstMatr[7] * di;
    dstMatr[12] += dstMatr[13] * dstMatr[4] * di;
    dstMatr[14] += dstMatr[13] * dstMatr[6] * di;
    dstMatr[15] += dstMatr[13] * dstMatr[7] * di;
    di = dstMatr[10];
    dstMatr[10] = d = 1.0f / di;
    dstMatr[2] *= -d;
    dstMatr[6] *= -d;
    dstMatr[14] *= -d;
    dstMatr[8] *= d;
    dstMatr[9] *= d;
    dstMatr[11] *= d;
    dstMatr[0] += dstMatr[2] * dstMatr[8] * di;
    dstMatr[1] += dstMatr[2] * dstMatr[9] * di;
    dstMatr[3] += dstMatr[2] * dstMatr[11] * di;
    dstMatr[4] += dstMatr[6] * dstMatr[8] * di;
    dstMatr[5] += dstMatr[6] * dstMatr[9] * di;
    dstMatr[7] += dstMatr[6] * dstMatr[11] * di;
    dstMatr[12] += dstMatr[14] * dstMatr[8] * di;
    dstMatr[13] += dstMatr[14] * dstMatr[9] * di;
    dstMatr[15] += dstMatr[14] * dstMatr[11] * di;
    di = dstMatr[15];
    dstMatr[15] = d = 1.0f / di;
    dstMatr[3] *= -d;
    dstMatr[7] *= -d;
    dstMatr[11] *= -d;
    dstMatr[12] *= d;
    dstMatr[13] *= d;
    dstMatr[14] *= d;
    dstMatr[0] += dstMatr[3] * dstMatr[12] * di;
    dstMatr[1] += dstMatr[3] * dstMatr[13] * di;
    dstMatr[2] += dstMatr[3] * dstMatr[14] * di;
    dstMatr[4] += dstMatr[7] * dstMatr[12] * di;
    dstMatr[5] += dstMatr[7] * dstMatr[13] * di;
    dstMatr[6] += dstMatr[7] * dstMatr[14] * di;
    dstMatr[8] += dstMatr[11] * dstMatr[12] * di;
    dstMatr[9] += dstMatr[11] * dstMatr[13] * di;
    dstMatr[10] += dstMatr[11] * dstMatr[14] * di;
    return CV_NO_ERR;
}

/* End of file */
