/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/
#include "_cv.h"
/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvAdd6x6Matr32f   
//    Purpose:    Sum of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvAddMatrix6x6_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    /* first row */
    destMatr[0] = srcMatr1[0] + srcMatr2[0];
    destMatr[1] = srcMatr1[1] + srcMatr2[1];
    destMatr[2] = srcMatr1[2] + srcMatr2[2];
    destMatr[3] = srcMatr1[3] + srcMatr2[3];
    destMatr[4] = srcMatr1[4] + srcMatr2[4];
    destMatr[5] = srcMatr1[5] + srcMatr2[5];

    /* second row */
    destMatr[6] = srcMatr1[6] + srcMatr2[6];
    destMatr[7] = srcMatr1[7] + srcMatr2[7];
    destMatr[8] = srcMatr1[8] + srcMatr2[8];
    destMatr[9] = srcMatr1[9] + srcMatr2[9];
    destMatr[10] = srcMatr1[10] + srcMatr2[10];
    destMatr[11] = srcMatr1[11] + srcMatr2[11];

    /* third row */
    destMatr[12] = srcMatr1[12] + srcMatr2[12];
    destMatr[13] = srcMatr1[13] + srcMatr2[13];
    destMatr[14] = srcMatr1[14] + srcMatr2[14];
    destMatr[15] = srcMatr1[15] + srcMatr2[15];
    destMatr[16] = srcMatr1[16] + srcMatr2[16];
    destMatr[17] = srcMatr1[17] + srcMatr2[17];

    /* fourth row */
    destMatr[18] = srcMatr1[18] + srcMatr2[18];
    destMatr[19] = srcMatr1[19] + srcMatr2[19];
    destMatr[20] = srcMatr1[20] + srcMatr2[20];
    destMatr[21] = srcMatr1[21] + srcMatr2[21];
    destMatr[22] = srcMatr1[22] + srcMatr2[22];
    destMatr[23] = srcMatr1[23] + srcMatr2[23];

    /* fifth row */
    destMatr[24] = srcMatr1[24] + srcMatr2[24];
    destMatr[25] = srcMatr1[25] + srcMatr2[25];
    destMatr[26] = srcMatr1[26] + srcMatr2[26];
    destMatr[27] = srcMatr1[27] + srcMatr2[27];
    destMatr[28] = srcMatr1[28] + srcMatr2[28];
    destMatr[29] = srcMatr1[29] + srcMatr2[29];

    /* sixth row */
    destMatr[30] = srcMatr1[30] + srcMatr2[30];
    destMatr[31] = srcMatr1[31] + srcMatr2[31];
    destMatr[32] = srcMatr1[32] + srcMatr2[32];
    destMatr[33] = srcMatr1[33] + srcMatr2[33];
    destMatr[34] = srcMatr1[34] + srcMatr2[34];
    destMatr[35] = srcMatr1[35] + srcMatr2[35];

    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvSub4x4Matr32f   
//    Purpose:    Difference of 2 matrix 4x4
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvSubMatrix6x6_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    /* first row */
    destMatr[0] = srcMatr1[0] - srcMatr2[0];
    destMatr[1] = srcMatr1[1] - srcMatr2[1];
    destMatr[2] = srcMatr1[2] - srcMatr2[2];
    destMatr[3] = srcMatr1[3] - srcMatr2[3];
    destMatr[4] = srcMatr1[4] - srcMatr2[4];
    destMatr[5] = srcMatr1[5] - srcMatr2[5];

    /* second row */
    destMatr[6] = srcMatr1[6] - srcMatr2[6];
    destMatr[7] = srcMatr1[7] - srcMatr2[7];
    destMatr[8] = srcMatr1[8] - srcMatr2[8];
    destMatr[9] = srcMatr1[9] - srcMatr2[9];
    destMatr[10] = srcMatr1[10] - srcMatr2[10];
    destMatr[11] = srcMatr1[11] - srcMatr2[11];

    /* third row */
    destMatr[12] = srcMatr1[12] - srcMatr2[12];
    destMatr[13] = srcMatr1[13] - srcMatr2[13];
    destMatr[14] = srcMatr1[14] - srcMatr2[14];
    destMatr[15] = srcMatr1[15] - srcMatr2[15];
    destMatr[16] = srcMatr1[16] - srcMatr2[16];
    destMatr[17] = srcMatr1[17] - srcMatr2[17];

    /* fourth row */
    destMatr[18] = srcMatr1[18] - srcMatr2[18];
    destMatr[19] = srcMatr1[19] - srcMatr2[19];
    destMatr[20] = srcMatr1[20] - srcMatr2[20];
    destMatr[21] = srcMatr1[21] - srcMatr2[21];
    destMatr[22] = srcMatr1[22] - srcMatr2[22];
    destMatr[23] = srcMatr1[23] - srcMatr2[23];

    /* fifth row */
    destMatr[24] = srcMatr1[24] - srcMatr2[24];
    destMatr[25] = srcMatr1[25] - srcMatr2[25];
    destMatr[26] = srcMatr1[26] - srcMatr2[26];
    destMatr[27] = srcMatr1[27] - srcMatr2[27];
    destMatr[28] = srcMatr1[28] - srcMatr2[28];
    destMatr[29] = srcMatr1[29] - srcMatr2[29];

    /* sixth row */
    destMatr[30] = srcMatr1[30] - srcMatr2[30];
    destMatr[31] = srcMatr1[31] - srcMatr2[31];
    destMatr[32] = srcMatr1[32] - srcMatr2[32];
    destMatr[33] = srcMatr1[33] - srcMatr2[33];
    destMatr[34] = srcMatr1[34] - srcMatr2[34];
    destMatr[35] = srcMatr1[35] - srcMatr2[35];

    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvScale4x4Matr32f  
//    Purpose:    Multiplies each element of matrix 4x4 by a value
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                value    - factor
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvScaleMatrix6x6_32f, (CvMatr32f srcMatr,
                                                CvMatr32f destMatr, float value) )
{
    /* check bad arguments */
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    /* first row */
    destMatr[0] = value * srcMatr[0];
    destMatr[1] = value * srcMatr[1];
    destMatr[2] = value * srcMatr[2];
    destMatr[3] = value * srcMatr[3];
    destMatr[4] = value * srcMatr[4];
    destMatr[5] = value * srcMatr[5];

    /* second row */
    destMatr[6] = value * srcMatr[6];
    destMatr[7] = value * srcMatr[7];
    destMatr[8] = value * srcMatr[8];
    destMatr[9] = value * srcMatr[9];
    destMatr[10] = value * srcMatr[10];
    destMatr[11] = value * srcMatr[11];

    /* third row */
    destMatr[12] = value * srcMatr[12];
    destMatr[13] = value * srcMatr[13];
    destMatr[14] = value * srcMatr[14];
    destMatr[15] = value * srcMatr[15];
    destMatr[16] = value * srcMatr[16];
    destMatr[17] = value * srcMatr[17];

    /* fourth row */
    destMatr[18] = value * srcMatr[18];
    destMatr[19] = value * srcMatr[19];
    destMatr[20] = value * srcMatr[20];
    destMatr[21] = value * srcMatr[21];
    destMatr[22] = value * srcMatr[22];
    destMatr[23] = value * srcMatr[23];

    /* fifth row */
    destMatr[24] = value * srcMatr[24];
    destMatr[25] = value * srcMatr[25];
    destMatr[26] = value * srcMatr[26];
    destMatr[27] = value * srcMatr[27];
    destMatr[28] = value * srcMatr[28];
    destMatr[29] = value * srcMatr[29];

    /* sixth row */
    destMatr[30] = value * srcMatr[30];
    destMatr[31] = value * srcMatr[31];
    destMatr[32] = value * srcMatr[32];
    destMatr[33] = value * srcMatr[33];
    destMatr[34] = value * srcMatr[34];
    destMatr[35] = value * srcMatr[35];

    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvMulMatrix6x6_32f   
//    Purpose:    Difference of 2 matrixes 6x6
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//                destMatr - destination matrix
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvMulMatrix6x6_32f, (CvMatr32f srcMatr1,
                                              CvMatr32f srcMatr2, CvMatr32f destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    /* first row */
    destMatr[0] = srcMatr1[0] * srcMatr2[0] +
        srcMatr1[1] * srcMatr2[6] +
        srcMatr1[2] * srcMatr2[12] +
        srcMatr1[3] * srcMatr2[18] + srcMatr1[4] * srcMatr2[24] + srcMatr1[5] * srcMatr2[30];
    destMatr[1] = srcMatr1[0] * srcMatr2[1] +
        srcMatr1[1] * srcMatr2[7] +
        srcMatr1[2] * srcMatr2[13] +
        srcMatr1[3] * srcMatr2[19] + srcMatr1[4] * srcMatr2[25] + srcMatr1[5] * srcMatr2[31];
    destMatr[2] = srcMatr1[0] * srcMatr2[2] +
        srcMatr1[1] * srcMatr2[8] +
        srcMatr1[2] * srcMatr2[14] +
        srcMatr1[3] * srcMatr2[20] + srcMatr1[4] * srcMatr2[26] + srcMatr1[5] * srcMatr2[32];
    destMatr[3] = srcMatr1[0] * srcMatr2[3] +
        srcMatr1[1] * srcMatr2[9] +
        srcMatr1[2] * srcMatr2[15] +
        srcMatr1[3] * srcMatr2[21] + srcMatr1[4] * srcMatr2[27] + srcMatr1[5] * srcMatr2[33];
    destMatr[4] = srcMatr1[0] * srcMatr2[4] +
        srcMatr1[1] * srcMatr2[10] +
        srcMatr1[2] * srcMatr2[16] +
        srcMatr1[3] * srcMatr2[22] + srcMatr1[4] * srcMatr2[28] + srcMatr1[5] * srcMatr2[34];
    destMatr[5] = srcMatr1[0] * srcMatr2[5] +
        srcMatr1[1] * srcMatr2[11] +
        srcMatr1[2] * srcMatr2[17] +
        srcMatr1[3] * srcMatr2[23] + srcMatr1[4] * srcMatr2[29] + srcMatr1[5] * srcMatr2[35];
    /* second row */
    destMatr[6] = srcMatr1[6] * srcMatr2[0] +
        srcMatr1[7] * srcMatr2[6] +
        srcMatr1[8] * srcMatr2[12] +
        srcMatr1[9] * srcMatr2[18] + srcMatr1[10] * srcMatr2[24] + srcMatr1[11] * srcMatr2[30];
    destMatr[7] = srcMatr1[6] * srcMatr2[1] +
        srcMatr1[7] * srcMatr2[7] +
        srcMatr1[8] * srcMatr2[13] +
        srcMatr1[9] * srcMatr2[19] + srcMatr1[10] * srcMatr2[25] + srcMatr1[11] * srcMatr2[31];
    destMatr[8] = srcMatr1[6] * srcMatr2[2] +
        srcMatr1[7] * srcMatr2[8] +
        srcMatr1[8] * srcMatr2[14] +
        srcMatr1[9] * srcMatr2[20] + srcMatr1[10] * srcMatr2[26] + srcMatr1[11] * srcMatr2[32];
    destMatr[9] = srcMatr1[6] * srcMatr2[3] +
        srcMatr1[7] * srcMatr2[9] +
        srcMatr1[8] * srcMatr2[15] +
        srcMatr1[9] * srcMatr2[21] + srcMatr1[10] * srcMatr2[27] + srcMatr1[11] * srcMatr2[33];
    destMatr[10] = srcMatr1[6] * srcMatr2[4] +
        srcMatr1[7] * srcMatr2[10] +
        srcMatr1[8] * srcMatr2[16] +
        srcMatr1[9] * srcMatr2[22] + srcMatr1[10] * srcMatr2[28] + srcMatr1[11] * srcMatr2[34];
    destMatr[11] = srcMatr1[6] * srcMatr2[5] +
        srcMatr1[7] * srcMatr2[11] +
        srcMatr1[8] * srcMatr2[17] +
        srcMatr1[9] * srcMatr2[23] + srcMatr1[10] * srcMatr2[29] + srcMatr1[11] * srcMatr2[35];
    /* third row */
    destMatr[12] = srcMatr1[12] * srcMatr2[0] +
        srcMatr1[13] * srcMatr2[6] +
        srcMatr1[14] * srcMatr2[12] +
        srcMatr1[15] * srcMatr2[18] +
        srcMatr1[16] * srcMatr2[24] + srcMatr1[17] * srcMatr2[30];
    destMatr[13] = srcMatr1[12] * srcMatr2[1] +
        srcMatr1[13] * srcMatr2[7] +
        srcMatr1[14] * srcMatr2[13] +
        srcMatr1[15] * srcMatr2[19] +
        srcMatr1[16] * srcMatr2[25] + srcMatr1[17] * srcMatr2[31];
    destMatr[14] = srcMatr1[12] * srcMatr2[2] +
        srcMatr1[13] * srcMatr2[8] +
        srcMatr1[14] * srcMatr2[14] +
        srcMatr1[15] * srcMatr2[20] +
        srcMatr1[16] * srcMatr2[26] + srcMatr1[17] * srcMatr2[32];
    destMatr[15] = srcMatr1[12] * srcMatr2[3] +
        srcMatr1[13] * srcMatr2[9] +
        srcMatr1[14] * srcMatr2[15] +
        srcMatr1[15] * srcMatr2[21] +
        srcMatr1[16] * srcMatr2[27] + srcMatr1[17] * srcMatr2[33];
    destMatr[16] = srcMatr1[12] * srcMatr2[4] +
        srcMatr1[13] * srcMatr2[10] +
        srcMatr1[14] * srcMatr2[16] +
        srcMatr1[15] * srcMatr2[22] +
        srcMatr1[16] * srcMatr2[28] + srcMatr1[17] * srcMatr2[34];
    destMatr[17] = srcMatr1[12] * srcMatr2[5] +
        srcMatr1[13] * srcMatr2[11] +
        srcMatr1[14] * srcMatr2[17] +
        srcMatr1[15] * srcMatr2[23] +
        srcMatr1[16] * srcMatr2[29] + srcMatr1[17] * srcMatr2[35];
    /* fourth row */
    destMatr[18] = srcMatr1[18] * srcMatr2[0] +
        srcMatr1[19] * srcMatr2[6] +
        srcMatr1[20] * srcMatr2[12] +
        srcMatr1[21] * srcMatr2[18] +
        srcMatr1[22] * srcMatr2[24] + srcMatr1[23] * srcMatr2[30];
    destMatr[19] = srcMatr1[18] * srcMatr2[1] +
        srcMatr1[19] * srcMatr2[7] +
        srcMatr1[20] * srcMatr2[13] +
        srcMatr1[21] * srcMatr2[19] +
        srcMatr1[22] * srcMatr2[25] + srcMatr1[23] * srcMatr2[31];
    destMatr[20] = srcMatr1[18] * srcMatr2[2] +
        srcMatr1[19] * srcMatr2[8] +
        srcMatr1[20] * srcMatr2[14] +
        srcMatr1[21] * srcMatr2[20] +
        srcMatr1[22] * srcMatr2[26] + srcMatr1[23] * srcMatr2[32];
    destMatr[21] = srcMatr1[18] * srcMatr2[3] +
        srcMatr1[19] * srcMatr2[9] +
        srcMatr1[20] * srcMatr2[15] +
        srcMatr1[21] * srcMatr2[21] +
        srcMatr1[22] * srcMatr2[27] + srcMatr1[23] * srcMatr2[33];
    destMatr[22] = srcMatr1[18] * srcMatr2[4] +
        srcMatr1[19] * srcMatr2[10] +
        srcMatr1[20] * srcMatr2[16] +
        srcMatr1[21] * srcMatr2[22] +
        srcMatr1[22] * srcMatr2[28] + srcMatr1[23] * srcMatr2[34];
    destMatr[23] = srcMatr1[18] * srcMatr2[5] +
        srcMatr1[19] * srcMatr2[11] +
        srcMatr1[20] * srcMatr2[17] +
        srcMatr1[21] * srcMatr2[23] +
        srcMatr1[22] * srcMatr2[29] + srcMatr1[23] * srcMatr2[35];
    /* fifth row */
    destMatr[24] = srcMatr1[24] * srcMatr2[0] +
        srcMatr1[25] * srcMatr2[6] +
        srcMatr1[26] * srcMatr2[12] +
        srcMatr1[27] * srcMatr2[18] +
        srcMatr1[28] * srcMatr2[24] + srcMatr1[29] * srcMatr2[30];
    destMatr[25] = srcMatr1[24] * srcMatr2[1] +
        srcMatr1[25] * srcMatr2[7] +
        srcMatr1[26] * srcMatr2[13] +
        srcMatr1[27] * srcMatr2[19] +
        srcMatr1[28] * srcMatr2[25] + srcMatr1[29] * srcMatr2[31];
    destMatr[26] = srcMatr1[24] * srcMatr2[2] +
        srcMatr1[25] * srcMatr2[8] +
        srcMatr1[26] * srcMatr2[14] +
        srcMatr1[27] * srcMatr2[20] +
        srcMatr1[28] * srcMatr2[26] + srcMatr1[29] * srcMatr2[32];
    destMatr[27] = srcMatr1[24] * srcMatr2[3] +
        srcMatr1[25] * srcMatr2[9] +
        srcMatr1[26] * srcMatr2[15] +
        srcMatr1[27] * srcMatr2[21] +
        srcMatr1[28] * srcMatr2[27] + srcMatr1[29] * srcMatr2[33];
    destMatr[28] = srcMatr1[24] * srcMatr2[4] +
        srcMatr1[25] * srcMatr2[10] +
        srcMatr1[26] * srcMatr2[16] +
        srcMatr1[27] * srcMatr2[22] +
        srcMatr1[28] * srcMatr2[28] + srcMatr1[29] * srcMatr2[34];
    destMatr[29] = srcMatr1[24] * srcMatr2[5] +
        srcMatr1[25] * srcMatr2[11] +
        srcMatr1[26] * srcMatr2[17] +
        srcMatr1[27] * srcMatr2[23] +
        srcMatr1[28] * srcMatr2[29] + srcMatr1[29] * srcMatr2[35];
    /* sixth row */
    destMatr[30] = srcMatr1[30] * srcMatr2[0] +
        srcMatr1[31] * srcMatr2[6] +
        srcMatr1[32] * srcMatr2[12] +
        srcMatr1[33] * srcMatr2[18] +
        srcMatr1[34] * srcMatr2[24] + srcMatr1[35] * srcMatr2[30];
    destMatr[31] = srcMatr1[30] * srcMatr2[1] +
        srcMatr1[31] * srcMatr2[7] +
        srcMatr1[32] * srcMatr2[13] +
        srcMatr1[33] * srcMatr2[19] +
        srcMatr1[34] * srcMatr2[25] + srcMatr1[35] * srcMatr2[31];
    destMatr[32] = srcMatr1[30] * srcMatr2[2] +
        srcMatr1[31] * srcMatr2[8] +
        srcMatr1[32] * srcMatr2[14] +
        srcMatr1[33] * srcMatr2[20] +
        srcMatr1[34] * srcMatr2[26] + srcMatr1[35] * srcMatr2[32];
    destMatr[33] = srcMatr1[30] * srcMatr2[3] +
        srcMatr1[31] * srcMatr2[9] +
        srcMatr1[32] * srcMatr2[15] +
        srcMatr1[33] * srcMatr2[21] +
        srcMatr1[34] * srcMatr2[27] + srcMatr1[35] * srcMatr2[33];
    destMatr[34] = srcMatr1[30] * srcMatr2[4] +
        srcMatr1[31] * srcMatr2[10] +
        srcMatr1[32] * srcMatr2[16] +
        srcMatr1[33] * srcMatr2[22] +
        srcMatr1[34] * srcMatr2[28] + srcMatr1[35] * srcMatr2[34];
    destMatr[35] = srcMatr1[30] * srcMatr2[5] +
        srcMatr1[31] * srcMatr2[11] +
        srcMatr1[32] * srcMatr2[17] +
        srcMatr1[33] * srcMatr2[23] +
        srcMatr1[34] * srcMatr2[29] + srcMatr1[35] * srcMatr2[35];


    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvTransposeMatr6x6_32f  
//    Purpose:    Transposes matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvTransposeMatrix6x6_32f, (CvMatr32f srcMatr, CvMatr32f destMatr) )
{
    if( srcMatr == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    /* First row */
    destMatr[0] = srcMatr[0];
    destMatr[1] = srcMatr[6];
    destMatr[2] = srcMatr[12];
    destMatr[3] = srcMatr[18];
    destMatr[4] = srcMatr[24];
    destMatr[5] = srcMatr[30];

    /* Second row */
    destMatr[6] = srcMatr[1];
    destMatr[7] = srcMatr[7];
    destMatr[8] = srcMatr[13];
    destMatr[9] = srcMatr[19];
    destMatr[10] = srcMatr[25];
    destMatr[11] = srcMatr[31];

    /* Third row */
    destMatr[12] = srcMatr[2];
    destMatr[13] = srcMatr[8];
    destMatr[14] = srcMatr[14];
    destMatr[15] = srcMatr[20];
    destMatr[16] = srcMatr[26];
    destMatr[17] = srcMatr[32];

    /* Fourth row */
    destMatr[18] = srcMatr[3];
    destMatr[19] = srcMatr[9];
    destMatr[20] = srcMatr[15];
    destMatr[21] = srcMatr[21];
    destMatr[22] = srcMatr[27];
    destMatr[23] = srcMatr[33];

    /* Fifth row */
    destMatr[24] = srcMatr[4];
    destMatr[25] = srcMatr[10];
    destMatr[26] = srcMatr[16];
    destMatr[27] = srcMatr[22];
    destMatr[28] = srcMatr[28];
    destMatr[29] = srcMatr[34];

    /* Sixth row */
    destMatr[30] = srcMatr[5];
    destMatr[31] = srcMatr[11];
    destMatr[32] = srcMatr[17];
    destMatr[33] = srcMatr[23];
    destMatr[34] = srcMatr[29];
    destMatr[35] = srcMatr[35];

    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       cvInvertMatrix6x6_32f   
//    Purpose:    Invertes  6x6 matrix
//    Context:   
//    Parameters: srcMatr - source matrix
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//    matrWidth1 mus be equal to matrHeight2      
//F*/
IPCVAPI_IMPL( CvStatus, icvInvertMatrix6x6_32f, (CvMatr32f srcMatr, CvMatr32f dstMatr) )
{
    float di = srcMatr[0];
    float d = 1.0f / di;

    dstMatr[0] = d;
    dstMatr[6] = srcMatr[6] * -d;
    dstMatr[12] = srcMatr[12] * -d;
    dstMatr[18] = srcMatr[18] * -d;
    dstMatr[24] = srcMatr[24] * -d;
    dstMatr[30] = srcMatr[30] * -d;
    dstMatr[1] = srcMatr[1] * d;
    dstMatr[2] = srcMatr[2] * d;
    dstMatr[3] = srcMatr[3] * d;
    dstMatr[4] = srcMatr[4] * d;
    dstMatr[5] = srcMatr[5] * d;
    dstMatr[7] = srcMatr[7] + dstMatr[6] * dstMatr[1] * di;
    dstMatr[8] = srcMatr[8] + dstMatr[6] * dstMatr[2] * di;
    dstMatr[9] = srcMatr[9] + dstMatr[6] * dstMatr[3] * di;
    dstMatr[10] = srcMatr[10] + dstMatr[6] * dstMatr[4] * di;
    dstMatr[11] = srcMatr[11] + dstMatr[6] * dstMatr[5] * di;
    dstMatr[13] = srcMatr[13] + dstMatr[12] * dstMatr[1] * di;
    dstMatr[14] = srcMatr[14] + dstMatr[12] * dstMatr[2] * di;
    dstMatr[15] = srcMatr[15] + dstMatr[12] * dstMatr[3] * di;
    dstMatr[16] = srcMatr[16] + dstMatr[12] * dstMatr[4] * di;
    dstMatr[17] = srcMatr[17] + dstMatr[12] * dstMatr[5] * di;
    dstMatr[19] = srcMatr[19] + dstMatr[18] * dstMatr[1] * di;
    dstMatr[20] = srcMatr[20] + dstMatr[18] * dstMatr[2] * di;
    dstMatr[21] = srcMatr[21] + dstMatr[18] * dstMatr[3] * di;
    dstMatr[22] = srcMatr[22] + dstMatr[18] * dstMatr[4] * di;
    dstMatr[23] = srcMatr[23] + dstMatr[18] * dstMatr[5] * di;
    dstMatr[25] = srcMatr[25] + dstMatr[24] * dstMatr[1] * di;
    dstMatr[26] = srcMatr[26] + dstMatr[24] * dstMatr[2] * di;
    dstMatr[27] = srcMatr[27] + dstMatr[24] * dstMatr[3] * di;
    dstMatr[28] = srcMatr[28] + dstMatr[24] * dstMatr[4] * di;
    dstMatr[29] = srcMatr[29] + dstMatr[24] * dstMatr[5] * di;
    dstMatr[31] = srcMatr[31] + dstMatr[30] * dstMatr[1] * di;
    dstMatr[32] = srcMatr[32] + dstMatr[30] * dstMatr[2] * di;
    dstMatr[33] = srcMatr[33] + dstMatr[30] * dstMatr[3] * di;
    dstMatr[34] = srcMatr[34] + dstMatr[30] * dstMatr[4] * di;
    dstMatr[35] = srcMatr[35] + dstMatr[30] * dstMatr[5] * di;
    di = dstMatr[7];
    dstMatr[7] = d = 1.0f / di;
    dstMatr[1] *= -d;
    dstMatr[13] *= -d;
    dstMatr[19] *= -d;
    dstMatr[25] *= -d;
    dstMatr[31] *= -d;
    dstMatr[6] *= d;
    dstMatr[8] *= d;
    dstMatr[9] *= d;
    dstMatr[10] *= d;
    dstMatr[11] *= d;
    dstMatr[0] += dstMatr[1] * dstMatr[6] * di;
    dstMatr[2] += dstMatr[1] * dstMatr[8] * di;
    dstMatr[3] += dstMatr[1] * dstMatr[9] * di;
    dstMatr[4] += dstMatr[1] * dstMatr[10] * di;
    dstMatr[5] += dstMatr[1] * dstMatr[11] * di;
    dstMatr[12] += dstMatr[13] * dstMatr[6] * di;
    dstMatr[14] += dstMatr[13] * dstMatr[8] * di;
    dstMatr[15] += dstMatr[13] * dstMatr[9] * di;
    dstMatr[16] += dstMatr[13] * dstMatr[10] * di;
    dstMatr[17] += dstMatr[13] * dstMatr[11] * di;
    dstMatr[18] += dstMatr[19] * dstMatr[6] * di;
    dstMatr[20] += dstMatr[19] * dstMatr[8] * di;
    dstMatr[21] += dstMatr[19] * dstMatr[9] * di;
    dstMatr[22] += dstMatr[19] * dstMatr[10] * di;
    dstMatr[23] += dstMatr[19] * dstMatr[11] * di;
    dstMatr[24] += dstMatr[25] * dstMatr[6] * di;
    dstMatr[26] += dstMatr[25] * dstMatr[8] * di;
    dstMatr[27] += dstMatr[25] * dstMatr[9] * di;
    dstMatr[28] += dstMatr[25] * dstMatr[10] * di;
    dstMatr[29] += dstMatr[25] * dstMatr[11] * di;
    dstMatr[30] += dstMatr[31] * dstMatr[6] * di;
    dstMatr[32] += dstMatr[31] * dstMatr[8] * di;
    dstMatr[33] += dstMatr[31] * dstMatr[9] * di;
    dstMatr[34] += dstMatr[31] * dstMatr[10] * di;
    dstMatr[35] += dstMatr[31] * dstMatr[11] * di;
    di = dstMatr[14];
    dstMatr[14] = d = 1.0f / di;
    dstMatr[2] *= -d;
    dstMatr[8] *= -d;
    dstMatr[20] *= -d;
    dstMatr[26] *= -d;
    dstMatr[32] *= -d;
    dstMatr[12] *= d;
    dstMatr[13] *= d;
    dstMatr[15] *= d;
    dstMatr[16] *= d;
    dstMatr[17] *= d;
    dstMatr[0] += dstMatr[2] * dstMatr[12] * di;
    dstMatr[1] += dstMatr[2] * dstMatr[13] * di;
    dstMatr[3] += dstMatr[2] * dstMatr[15] * di;
    dstMatr[4] += dstMatr[2] * dstMatr[16] * di;
    dstMatr[5] += dstMatr[2] * dstMatr[17] * di;
    dstMatr[6] += dstMatr[8] * dstMatr[12] * di;
    dstMatr[7] += dstMatr[8] * dstMatr[13] * di;
    dstMatr[9] += dstMatr[8] * dstMatr[15] * di;
    dstMatr[10] += dstMatr[8] * dstMatr[16] * di;
    dstMatr[11] += dstMatr[8] * dstMatr[17] * di;
    dstMatr[18] += dstMatr[20] * dstMatr[12] * di;
    dstMatr[19] += dstMatr[20] * dstMatr[13] * di;
    dstMatr[21] += dstMatr[20] * dstMatr[15] * di;
    dstMatr[22] += dstMatr[20] * dstMatr[16] * di;
    dstMatr[23] += dstMatr[20] * dstMatr[17] * di;
    dstMatr[24] += dstMatr[26] * dstMatr[12] * di;
    dstMatr[25] += dstMatr[26] * dstMatr[13] * di;
    dstMatr[27] += dstMatr[26] * dstMatr[15] * di;
    dstMatr[28] += dstMatr[26] * dstMatr[16] * di;
    dstMatr[29] += dstMatr[26] * dstMatr[17] * di;
    dstMatr[30] += dstMatr[32] * dstMatr[12] * di;
    dstMatr[31] += dstMatr[32] * dstMatr[13] * di;
    dstMatr[33] += dstMatr[32] * dstMatr[15] * di;
    dstMatr[34] += dstMatr[32] * dstMatr[16] * di;
    dstMatr[35] += dstMatr[32] * dstMatr[17] * di;
    di = dstMatr[21];
    dstMatr[21] = d = 1.0f / di;
    dstMatr[3] *= -d;
    dstMatr[9] *= -d;
    dstMatr[15] *= -d;
    dstMatr[27] *= -d;
    dstMatr[33] *= -d;
    dstMatr[18] *= d;
    dstMatr[19] *= d;
    dstMatr[20] *= d;
    dstMatr[22] *= d;
    dstMatr[23] *= d;
    dstMatr[0] += dstMatr[3] * dstMatr[18] * di;
    dstMatr[1] += dstMatr[3] * dstMatr[19] * di;
    dstMatr[2] += dstMatr[3] * dstMatr[20] * di;
    dstMatr[4] += dstMatr[3] * dstMatr[22] * di;
    dstMatr[5] += dstMatr[3] * dstMatr[23] * di;
    dstMatr[6] += dstMatr[9] * dstMatr[18] * di;
    dstMatr[7] += dstMatr[9] * dstMatr[19] * di;
    dstMatr[8] += dstMatr[9] * dstMatr[20] * di;
    dstMatr[10] += dstMatr[9] * dstMatr[22] * di;
    dstMatr[11] += dstMatr[9] * dstMatr[23] * di;
    dstMatr[12] += dstMatr[15] * dstMatr[18] * di;
    dstMatr[13] += dstMatr[15] * dstMatr[19] * di;
    dstMatr[14] += dstMatr[15] * dstMatr[20] * di;
    dstMatr[16] += dstMatr[15] * dstMatr[22] * di;
    dstMatr[17] += dstMatr[15] * dstMatr[23] * di;
    dstMatr[24] += dstMatr[27] * dstMatr[18] * di;
    dstMatr[25] += dstMatr[27] * dstMatr[19] * di;
    dstMatr[26] += dstMatr[27] * dstMatr[20] * di;
    dstMatr[28] += dstMatr[27] * dstMatr[22] * di;
    dstMatr[29] += dstMatr[27] * dstMatr[23] * di;
    dstMatr[30] += dstMatr[33] * dstMatr[18] * di;
    dstMatr[31] += dstMatr[33] * dstMatr[19] * di;
    dstMatr[32] += dstMatr[33] * dstMatr[20] * di;
    dstMatr[34] += dstMatr[33] * dstMatr[22] * di;
    dstMatr[35] += dstMatr[33] * dstMatr[23] * di;
    di = dstMatr[28];
    dstMatr[28] = d = 1.0f / di;
    dstMatr[4] *= -d;
    dstMatr[10] *= -d;
    dstMatr[16] *= -d;
    dstMatr[22] *= -d;
    dstMatr[34] *= -d;
    dstMatr[24] *= d;
    dstMatr[25] *= d;
    dstMatr[26] *= d;
    dstMatr[27] *= d;
    dstMatr[29] *= d;
    dstMatr[0] += dstMatr[4] * dstMatr[24] * di;
    dstMatr[1] += dstMatr[4] * dstMatr[25] * di;
    dstMatr[2] += dstMatr[4] * dstMatr[26] * di;
    dstMatr[3] += dstMatr[4] * dstMatr[27] * di;
    dstMatr[5] += dstMatr[4] * dstMatr[29] * di;
    dstMatr[6] += dstMatr[10] * dstMatr[24] * di;
    dstMatr[7] += dstMatr[10] * dstMatr[25] * di;
    dstMatr[8] += dstMatr[10] * dstMatr[26] * di;
    dstMatr[9] += dstMatr[10] * dstMatr[27] * di;
    dstMatr[11] += dstMatr[10] * dstMatr[29] * di;
    dstMatr[12] += dstMatr[16] * dstMatr[24] * di;
    dstMatr[13] += dstMatr[16] * dstMatr[25] * di;
    dstMatr[14] += dstMatr[16] * dstMatr[26] * di;
    dstMatr[15] += dstMatr[16] * dstMatr[27] * di;
    dstMatr[17] += dstMatr[16] * dstMatr[29] * di;
    dstMatr[18] += dstMatr[22] * dstMatr[24] * di;
    dstMatr[19] += dstMatr[22] * dstMatr[25] * di;
    dstMatr[20] += dstMatr[22] * dstMatr[26] * di;
    dstMatr[21] += dstMatr[22] * dstMatr[27] * di;
    dstMatr[23] += dstMatr[22] * dstMatr[29] * di;
    dstMatr[30] += dstMatr[34] * dstMatr[24] * di;
    dstMatr[31] += dstMatr[34] * dstMatr[25] * di;
    dstMatr[32] += dstMatr[34] * dstMatr[26] * di;
    dstMatr[33] += dstMatr[34] * dstMatr[27] * di;
    dstMatr[35] += dstMatr[34] * dstMatr[29] * di;
    di = dstMatr[35];
    dstMatr[35] = d = 1.0f / di;
    dstMatr[5] *= -d;
    dstMatr[11] *= -d;
    dstMatr[17] *= -d;
    dstMatr[23] *= -d;
    dstMatr[29] *= -d;
    dstMatr[30] *= d;
    dstMatr[31] *= d;
    dstMatr[32] *= d;
    dstMatr[33] *= d;
    dstMatr[34] *= d;
    dstMatr[0] += dstMatr[5] * dstMatr[30] * di;
    dstMatr[1] += dstMatr[5] * dstMatr[31] * di;
    dstMatr[2] += dstMatr[5] * dstMatr[32] * di;
    dstMatr[3] += dstMatr[5] * dstMatr[33] * di;
    dstMatr[4] += dstMatr[5] * dstMatr[34] * di;
    dstMatr[6] += dstMatr[11] * dstMatr[30] * di;
    dstMatr[7] += dstMatr[11] * dstMatr[31] * di;
    dstMatr[8] += dstMatr[11] * dstMatr[32] * di;
    dstMatr[9] += dstMatr[11] * dstMatr[33] * di;
    dstMatr[10] += dstMatr[11] * dstMatr[34] * di;
    dstMatr[12] += dstMatr[17] * dstMatr[30] * di;
    dstMatr[13] += dstMatr[17] * dstMatr[31] * di;
    dstMatr[14] += dstMatr[17] * dstMatr[32] * di;
    dstMatr[15] += dstMatr[17] * dstMatr[33] * di;
    dstMatr[16] += dstMatr[17] * dstMatr[34] * di;
    dstMatr[18] += dstMatr[23] * dstMatr[30] * di;
    dstMatr[19] += dstMatr[23] * dstMatr[31] * di;
    dstMatr[20] += dstMatr[23] * dstMatr[32] * di;
    dstMatr[21] += dstMatr[23] * dstMatr[33] * di;
    dstMatr[22] += dstMatr[23] * dstMatr[34] * di;
    dstMatr[24] += dstMatr[29] * dstMatr[30] * di;
    dstMatr[25] += dstMatr[29] * dstMatr[31] * di;
    dstMatr[26] += dstMatr[29] * dstMatr[32] * di;
    dstMatr[27] += dstMatr[29] * dstMatr[33] * di;
    dstMatr[28] += dstMatr[29] * dstMatr[34] * di;

    return CV_NO_ERR;
}

/* End of file */
