/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "_cv.h"

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvAddMatrix_64d   
//    Purpose:    Sum of 2 matrices
//    Context:   
//    Parameters: src1 - first matrix
//                src2 - second matrix
//                dest - destination matrix
//                width - width of matrices
//                height - height of matrices   
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvAddMatrix_64d, (CvMatr64d src1,
                                           CvMatr64d src2,
                                           CvMatr64d dest, int width, int height) )
{
    int i;

    /* check bad arguments */
    if( src1 == 0 )
        return CV_NULLPTR_ERR;
    if( src2 == 0 )
        return CV_NULLPTR_ERR;
    if( dest == 0 )
        return CV_NULLPTR_ERR;
    if( width <= 0 )
        return CV_BADSIZE_ERR;
    if( height <= 0 )
        return CV_BADSIZE_ERR;
    for( i = 0; i < width * height; i++ )
    {
        dest[i] = src1[i] + src2[i];
    }

    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvSubMatrix_64d   
//    Purpose:    Difference of 2 matrices
//    Context:   
//    Parameters: src1 - first matrix
//                src2 - second matrix(subtrahend)
//                dest - destination matrix
//                width - width of matrices
//                height - height of matrices   
//               
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvSubMatrix_64d, (CvMatr64d src1,
                                           CvMatr64d src2,
                                           CvMatr64d dest, int width, int height) )
{
    int i;

    /* check bad arguments */
    if( src1 == 0 )
        return CV_NULLPTR_ERR;
    if( src2 == 0 )
        return CV_NULLPTR_ERR;
    if( dest == 0 )
        return CV_NULLPTR_ERR;
    if( width <= 0 )
        return CV_BADSIZE_ERR;
    if( height <= 0 )
        return CV_BADSIZE_ERR;

    for( i = 0; i < width * height; i++ )
    {
        dest[i] = src1[i] - src2[i];
    }

    return CV_NO_ERR;
}



/*======================================================================================*/
IPCVAPI_IMPL( CvStatus, icvInvertMatrix_64d, (CvMatr64d srcMatr,
                                              int matrSize, CvMatr64d dstMatr) )
{

    double *Temp;
    double *tmpVect;
    double *tmpLineVect;
    double *tmpDstLineVect;
    int i, j;
    double Mult;

/*    int         ti,tj; */
    int change, st;
    int change_sign;

/*    int         t; */


#ifdef TRACE_TO_FILE
    FILE *file;

    file = fopen( "e:\\test_matrix\\Inv_matr.txt", "w" );
#endif

    change_sign = 0;

    Temp = (double *) icvAlloc( matrSize * matrSize * sizeof( double ));
    tmpVect = (double *) icvAlloc( matrSize * sizeof( double ));
    tmpLineVect = (double *) icvAlloc( matrSize * sizeof( double ));
    tmpDstLineVect = (double *) icvAlloc( matrSize * sizeof( double ));
    memset( Temp, 0, matrSize * matrSize * sizeof( double ));
    memset( tmpVect, 0, matrSize * sizeof( double ));
    memset( tmpLineVect, 0, matrSize * sizeof( double ));
    memset( tmpDstLineVect, 0, matrSize * sizeof( double ));

    icvCopyMatrix_64d( srcMatr, matrSize, matrSize, Temp );

    icvSetIdentity_64d( dstMatr, matrSize, matrSize );

    for( i = 0; i < matrSize; i++ )
    {

        /* Sort line for non zero element */

        change = i;


        /* !!! In change when test !=0 if all values == 0 then DET=0 and INV not exist */

        for( st = i; st < matrSize; st++ )
        {

            if( Temp[st * matrSize + i] != 0 )
            {

                change = st;
                break;
            }
        }

        if( change != i )
        {
            icvCopyVector_64d( &Temp[(i) * matrSize], matrSize, tmpVect );

            icvCopyVector_64d( &Temp[st * matrSize], matrSize, &Temp[(i) * matrSize] );

            icvCopyVector_64d( tmpVect, matrSize, &Temp[st * matrSize] );

            icvCopyVector_64d( &dstMatr[(i) * matrSize], matrSize, tmpVect );

            icvCopyVector_64d( &dstMatr[st * matrSize], matrSize, &dstMatr[(i) * matrSize] );

            icvCopyVector_64d( tmpVect, matrSize, &dstMatr[st * matrSize] );
            change_sign++;

        }
        else
        {
            if( Temp[i * matrSize + i] == 0 )
            {
                /* Inv matrix does not exist det == 0 */
                return CV_BADCOEF_ERR;
            }
        }

        Mult = 1.0 / Temp[i * matrSize + i];
#ifdef TRACE_TO_FILE

        if( Temp[i * matrSize + i] < 0.001 )
        {
            fprintf( file, "\nElement too small: %f\n", Temp[i * matrSize + i] );
        }

        if( Temp[i * matrSize + i] > 1000 )
        {
            fprintf( file, "\nElement too big: %f\n", Temp[i * matrSize + i] );
        }
#endif
        Temp[i * matrSize + i] = 1.0;

        /* Test !!! */

        icvScaleVector_64d( &Temp[i * matrSize + i + 1],
                             &Temp[i * matrSize + i + 1], matrSize - i - 1, Mult );

        icvScaleVector_64d( &dstMatr[i * matrSize], &dstMatr[i * matrSize], matrSize, Mult );

        /* Subtructing */

        icvCopyVector_64d( &Temp[i * matrSize + i + 1], matrSize - i - 1, tmpLineVect );

        icvCopyVector_64d( &dstMatr[i * matrSize], matrSize, tmpDstLineVect );

        for( j = i + 1; j < matrSize; j++ )
        {

            Mult = Temp[j * matrSize + i];

            Temp[j * matrSize + i] = 0.0;

            icvScaleVector_64d( tmpLineVect, tmpVect, matrSize - i - 1, Mult );

            icvSubVector_64d( &Temp[j * matrSize + i + 1],
                               tmpVect, &Temp[j * matrSize + i + 1], matrSize - i - 1 );


            icvScaleVector_64d( tmpDstLineVect, tmpVect, matrSize, Mult );

            icvSubVector_64d( &dstMatr[j * matrSize],
                               tmpVect, &dstMatr[j * matrSize], matrSize );

        }
    }

    /* Down to top direct */

    for( i = matrSize - 1; i > 0; i-- )
    {

        /* Subtructing */

        icvCopyVector_64d( &Temp[i * matrSize + i + 1], matrSize - i - 1, tmpLineVect );

        icvCopyVector_64d( &dstMatr[i * matrSize], matrSize, tmpDstLineVect );

        for( j = i - 1; j >= 0; j-- )
        {

            Mult = Temp[j * matrSize + i];

            Temp[j * matrSize + i] = 0.0;

            icvScaleVector_64d( tmpLineVect, tmpVect, matrSize - i - 1, Mult );

            icvSubVector_64d( &Temp[j * matrSize + i + 1],
                               tmpVect, &Temp[j * matrSize + i + 1], matrSize - i - 1 );


            icvScaleVector_64d( tmpDstLineVect, tmpVect, matrSize, Mult );

            icvSubVector_64d( &dstMatr[j * matrSize],
                               tmpVect, &dstMatr[j * matrSize], matrSize );


        }
    }

    if( change_sign & 1 == 1 )
    {
        icvScaleMatrix_64d( dstMatr, dstMatr, matrSize, matrSize, -1.0 );
    }

    /* free allocated memory */

    icvFree( &Temp );
    icvFree( &tmpVect );
    icvFree( &tmpLineVect );
    icvFree( &tmpDstLineVect );

#ifdef TRACE_TO_FILE
    fclose( file );
#endif
    return CV_NO_ERR;
}                               /* icvInvertMatrix_64d */

/*=====================================================================================*/
IPCVAPI_IMPL( CvStatus, icvScaleMatrix_64d, (CvMatr64d srcMatr,
                                             CvMatr64d destMatr,
                                             int matrWidth, int matrHeight, double value) )
{
    int step, i, j;
    double *src;
    double *dst;

    /* check bad arguments */
    if( srcMatr == 0 )
        return CV_NULLPTR_ERR;
    if( destMatr == 0 )
        return CV_NULLPTR_ERR;
    if( matrWidth <= 0 )
        return CV_BADSIZE_ERR;
    if( matrHeight <= 0 )
        return CV_BADSIZE_ERR;

    step = matrWidth;

    src = srcMatr;
    dst = destMatr;

    for( i = 0; i < matrHeight; i++ )
    {
        for( j = 0; j < matrWidth; j++ )
        {
            dst[j] = value * src[j];
        }
        dst += step;
        src += step;
    }
    return CV_NO_ERR;
}


IPCVAPI_IMPL( CvStatus, icvMulMatrix_64d, (CvMatr64d srcMatr1,
                                           int matrWidth1,
                                           int matrHeight1,
                                           CvMatr64d srcMatr2,
                                           int matrWidth2,
                                           int matrHeight2, CvMatr64d destMatr) )
{
    int i, j, k;
    double sum = 0;
    double *first = srcMatr1;
    double *second = srcMatr2;
    double *dest = destMatr;
    int Step1 = matrWidth1;
    int Step2 = matrWidth2;

    /* check bad arguments */
    if( srcMatr1 == 0 )
        return CV_NULLPTR_ERR;
    if( destMatr == 0 )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == 0 )
        return CV_NULLPTR_ERR;
    if( matrWidth1 <= 0 )
        return CV_BADSIZE_ERR;
    if( matrHeight1 <= 0 )
        return CV_BADSIZE_ERR;
    if( matrWidth2 <= 0 )
        return CV_BADSIZE_ERR;
    if( matrHeight2 <= 0 )
        return CV_BADSIZE_ERR;
    if( matrHeight2 != matrWidth1 )
        return CV_BADSIZE_ERR;
    for( j = 0; j < matrHeight1; j++ )
    {
        for( i = 0; i < matrWidth2; i++ )
        {
            sum = 0;
            second = srcMatr2 + i;
            for( k = 0; k < matrWidth1; k++ )
            {
                sum += first[k] * (*second);
                second += Step2;
            }
            dest[i] = sum;
        }
        first += Step1;
        dest += Step2;
    }
    return CV_NO_ERR;
}



/*=================================================================================*/
IPCVAPI_IMPL( CvStatus, icvMulTransMatrixR_64d, (CvMatr64d srcMatr,
                                                 int matrWidth,
                                                 int matrHeight, CvMatr64d destMatr) )
{
    int stepm = matrWidth;
    int i, j;
    double *C = destMatr, *col_i;

    if( srcMatr == 0 || C == 0 || srcMatr == C )
        return CV_NULLPTR_ERR;
    if( matrHeight <= 0 || matrWidth <= 0 )
        return CV_BADSIZE_ERR;

    col_i = (double *) icvAlloc( sizeof( double ) * matrHeight );

    for( i = 0; i < matrWidth; i++, C += stepm )
    {
        for( j = 0; j < matrHeight; j++ )
            col_i[j] = srcMatr[j * stepm + i];
        for( j = i; j < matrWidth; j++ )
        {
            int st = stepm, k = 0;
            double s = 0.0f, *A = srcMatr;

            for( ; k < matrHeight; k++, A += st )
                s += col_i[k] * A[j];
            C[j] = s;
        }
    }
    icvFree( (void **) &col_i );
    for( i = 1; i < matrWidth; i++ )
        for( j = 0; j < i; j++ )
            destMatr[j + i * stepm] = destMatr[i + j * stepm];
    return CV_NO_ERR;
}

/*=================================================================================*/
IPCVAPI_IMPL( CvStatus, icvMulTransMatrixL_64d, (CvMatr64d srcMatr,
                                                 int matrWidth,
                                                 int matrHeight, CvMatr64d destMatr) )
{
    int i, j, k;
    double Result;

    for( j = 0; j < matrHeight; j++ )
        for( i = j; i < matrHeight; i++ )
        {
            Result = 0.;
            for( k = 0; k < matrWidth; k++ )
                Result += srcMatr[i * matrWidth + k] * srcMatr[j * matrWidth + k];
            destMatr[j * matrHeight + i] = Result;
        }
    for( j = 0; j < matrHeight; j++ )
        for( i = j; i < matrHeight; i++ )
            destMatr[i * matrHeight + j] = destMatr[j * matrHeight + i];
    return CV_NO_ERR;
}






static float
square_32f( float x )
{
    return x * x;
}

/*====================================================================================*/


IPCVAPI_IMPL( CvStatus, icvComplexMult_32f, (CvMatr32f srcMatr,
                                             CvMatr32f dstMatr, int width, int height) )
{
    CvVect32f tmpMatr;
    CvVect32f tmpMatr2;
    CvMatr32f srcTrans;

    float *pMatr;
    float sum;
    int i, j, k;
    float *ps;
    float *pt;
    float *pti;
    float *ptj;
    float *ptk;
    float d;
    float *srcBase;
    float *dstBase;
    float *ptkBase;
    float *begRow;
    float *dstBase_i;
    int colShift;
    float coef;
    int shift;
    int shift_i, shift_j;
    int di, dj;
    int delta;
    int matrSize;

    matrSize = width;

#ifdef TRACE_TO_FILE
    int t;
    FILE *file;

    file = fopen( "E:\\test_matrix\\comples_mult.txt", "w" );

    fprintf( file, "\nSource matrix\n" );
    for( t = 0; t < width * height; t++ )
    {
        fprintf( file, "srcMatr[%d] = %f\n", t, srcMatr[t] );

    }
#endif
    tmpMatr = icvCreateVector_32f( (matrSize * (matrSize + 1)) / 2 );
    tmpMatr2 = icvCreateVector_32f( (matrSize * (matrSize + 1)) / 2 );
    srcTrans = icvCreateMatrix_32f( max( height, width ), width );

    /* Compute (AtA)-1 * At */

    /* Compute AtA and store in low triangular matrix */

    icvTransposeMatrix_32f( srcMatr, width, height, srcTrans );

#ifdef TRACE_TO_FILE
    fprintf( file, "\nSource matrix\n" );
    for( t = 0; t < width * height; t++ )
    {
        fprintf( file, "srcMatr[%d] = %f\n", t, srcMatr[t] );

    }

    fprintf( file, "\ntranspose source matrix\n" );
    for( t = 0; t < width * height; t++ )
    {
        fprintf( file, "srcTrans[%d] = %f\n", t, srcTrans[t] );
    }
#endif

    sum = 0;

    pMatr = tmpMatr;

    for( i = 0; i < width; i++ )
    {
        for( j = 0; j < i + 1; j++ )
        {
            icvDotProduct_32f( srcTrans + i * height,
                                srcTrans + j * height, height, pMatr++ );
        }
    }


#ifdef TRACE_TO_FILE
    fprintf( file, "\nMult matrix\n" );
    for( t = 0; t < (matrSize * (matrSize + 1)) / 2; t++ )
    {
        fprintf( file, "tmpMatr[%d] = %f\n", t, tmpMatr[t] );
    }
#endif
    /* Apply cholesky decomposition */


    ps = tmpMatr;
    pt = tmpMatr2;
    pti = pt;

    for( i = 0; i < matrSize; i++ )
    {

        ptj = pt;               /* Get begin of the matrix */

        for( j = 0; j < i; j++ )
        {                       /*  By all elements in line of source */
            ptk = pti;          /* ti - current element of the matrix */
            sum = 0.0;
            k = j;

            while( k-- )
            {
                sum += (*ptj++) * (*ptk++);
            }
            *ptk = ((*ps++) - sum) / (*ptj++);
        }

        sum = 0.0;
        k = i;
        while( k-- )
        {
            sum += square_32f( *pti++ );
        }
        d = (*ps++) - sum;
        *pti++ = (float) sqrt( d );
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nCholesky matrix (tmpMatr2)\n" );
    for( t = 0; t < (matrSize * (matrSize + 1)) / 2; t++ )
    {
        fprintf( file, "tmpMatr2[%d] = %f\n", t, tmpMatr2[t] );
    }
#endif

    /*  Find inverse matrix for triangular tmpMatr2 and strore in tmpMatr */


    icvSetZero_32f( tmpMatr, 1, (matrSize * (matrSize + 1)) / 2 );

    dstBase = tmpMatr;
    shift = 2;

    for( i = 0; i < matrSize; i++ )
    {
        *dstBase = 1.0;
        dstBase += shift;
        shift++;
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nIdentical matrix (tmpMatr)\n" );
    for( t = 0; t < (matrSize * (matrSize + 1)) / 2; t++ )
    {
        fprintf( file, "tmpMatr[%d] = %f\n", t, tmpMatr[t] );
    }
#endif

    srcBase = tmpMatr2;
    dstBase = tmpMatr;
    ptk = tmpMatr;
    ptkBase = tmpMatr;

    for( i = 0; i < matrSize; i++ )
    {                           /* For each row */

        ptj = tmpMatr;

        for( j = 0; j < i; j++ )
        {                       /* for each element in row */
            coef = *srcBase++;  /* Get current coefficient */
            k = j + 1;

            ptk = ptkBase;
            while( k-- )
            {                   /* Multiplacate elements and subtruct */
                *(ptk++) -= coef * (*ptj++);
            }
        }

        k = i + 1;
        coef = 1.0f / (*srcBase++);

        while( k-- )
        {
            *ptkBase++ *= coef;
        }
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nInv Cholesky matrix (tmpMatr)\n" );
    for( t = 0; t < (matrSize * (matrSize + 1)) / 2; t++ )
    {
        fprintf( file, "tmpMatr[%d] = %f\n", t, tmpMatr[t] );
    }
#endif

    /* Transpose this matrix for suitable computation tmpMatr -> tmpMatr2 */


    dstBase = tmpMatr2;
    srcBase = tmpMatr;
    begRow = tmpMatr;           /* begin of row for each cycle */

    shift = 2;

    for( i = 0; i < matrSize; i++ )
    {

        colShift = i + 1;


        ptj = begRow;

        for( j = 0; j < matrSize - i; j++ )
        {

            *dstBase++ = *ptj;

            ptj += colShift;
            colShift++;
        }

        begRow += shift;
        shift++;
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nTranspose Inv Cholesky matrix (tmpMatr2)\n" );
    for( t = 0; t < (matrSize * (matrSize + 1)) / 2; t++ )
    {
        fprintf( file, "tmpMatr2[%d] = %f\n", t, tmpMatr2[t] );
    }
#endif

    /* multiplacate to itself (quadr) matrix tmpMatr2  */
    /* and store result in the srcTrans (quadr) */

    dstBase_i = srcTrans;

    shift_i = matrSize;

    pti = tmpMatr2;

    for( i = 0; i < matrSize; i++ )
    {

        dstBase = dstBase_i;
        dstBase_i += matrSize;

        ptj = tmpMatr2;

        shift_j = matrSize;

        for( j = 0; j < i + 1; j++ )
        {

            if( j < i )
            {
                dj = i - j;
                di = 0;
                delta = i;
            }
            else
            {
                dj = 0;
                di = j - i;
                delta = j;
            }

            icvDotProduct_32f( pti + di, ptj + dj, matrSize - delta, dstBase++ );
            ptj += shift_j;
            shift_j--;
        }

        pti += shift_i;
        shift_i--;

    }

    /* copy elements to up the diagonal */

    for( i = 0; i < matrSize - 1; i++ )
    {
        for( j = i + 1; j < matrSize; j++ )
        {
            srcTrans[i * matrSize + j] = srcTrans[j * matrSize + i];
        }
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nMultiplacate matix not diagonal(srcTrans)\n" );
    for( i = 0; i < matrSize; i++ )
    {
        for( j = 0; j < matrSize; j++ )
        {
            fprintf( file, "srcTrans[%d,%d] = %f\n", i, j, srcTrans[i * matrSize + j] );
        }
        fprintf( file, "\n" );
    }
#endif


    /* Multiplicate this triangular matrix and transpose source */
    /* for suitable use not transpose matrix srcMatr */

    dstBase = dstMatr;

    pti = srcTrans;

    for( i = 0; i < matrSize; i++ )
    {

        ptj = srcMatr;

        for( j = 0; j < height; j++ )
        {

            icvDotProduct_32f( pti, ptj, matrSize, dstBase++ );

            ptj += width;
        }

        pti += matrSize;
    }

#ifdef TRACE_TO_FILE
    fprintf( file, "\nResult matrix (dstMatr)\n" );
    for( i = 0; i < width; i++ )
    {
        for( j = 0; j < height; j++ )
        {
            fprintf( file, "dstMatr[%d,%d] = %f\n", i, j, dstMatr[i * height + j] );
        }
        fprintf( file, "\n" );
    }
#endif


    icvDeleteMatrix( tmpMatr );
    icvDeleteMatrix( tmpMatr2 );
    icvDeleteMatrix( srcTrans );



#ifdef TRACE_TO_FILE
    fclose( file );
#endif

    return CV_NO_ERR;
}


/*======================================================================================*/
IPCVAPI_IMPL( CvStatus, icvComplexMult_64d, (CvMatr64d srcMatr,
                                             CvMatr64d dstMatr, int width, int height) )
{
    double *tmpMatr;
    double *tmpMatr2;
    double *srcTrans;

    double *pMatr;
    double sum;
    int i, j, k;
    double *ps;
    double *pt;
    double *pti;
    double *ptj;
    double *ptk;
    double d;
    double *srcBase;
    double *dstBase;
    double *ptkBase;
    double *begRow;
    double *dstBase_i;
    int colShift;
    double coef;
    int shift;

/*    int         t; */
    int shift_i, shift_j;
    int di, dj;
    int delta;
    int matrSize;

#ifdef TRACE_TO_FILE
    FILE *file;
#endif
    matrSize = width;
/*
    file = fopen("E:\\test_matrix\\comples_mult.txt","w");

    fprintf(file,"\nSource matrix\n");
    for (t = 0; t < (matrSize * (matrSize )) ; t++) {
        fprintf(file,"srcMatr[%d] = %f\n",t,srcMatr[t]);
        
    }
*/

    tmpMatr = (double *) icvAlloc( (matrSize * (matrSize + 1)) / 2 * sizeof( double ));
    tmpMatr2 = (double *) icvAlloc( (matrSize * (matrSize + 1)) / 2 * sizeof( double ));
    srcTrans = (double *) icvAlloc( max( height, width ) * width * sizeof( double ));
    memset( tmpMatr, 0, (matrSize * (matrSize + 1)) / 2 * sizeof( double ));
    memset( tmpMatr2, 0, (matrSize * (matrSize + 1)) / 2 * sizeof( double ));
    memset( srcTrans, 0, max( height, width ) * width * sizeof( double ));

    /* Compute (AtA)-1 * At */

    /* Compute AtA and store in low triangular matrix */

    icvTransposeMatrix_64d( srcMatr, width, height, srcTrans );
/*
    fprintf(file,"\ntranspose source matrix\n");
    for (t = 0; t < width * height  ; t++) {
        fprintf(file,"srcTrans[%d] = %f\n",t,srcTrans[t]);
    }
*/
    sum = 0;

    pMatr = tmpMatr;

    for( i = 0; i < width; i++ )
    {
        for( j = 0; j < i + 1; j++ )
        {
            icvDotProduct_64d( srcTrans + i * height,
                                srcTrans + j * height, height, pMatr++ );
        }
    }

/*
    fprintf(file,"\nMult matrix\n");
    for (t = 0; t < (matrSize * (matrSize + 1)) / 2; t++) {
        fprintf(file,"tmpMatr[%d] = %f\n",t,tmpMatr[t]);
    }
*/
    /* Apply cholesky decomposition */


    ps = tmpMatr;
    pt = tmpMatr2;
    pti = pt;

    for( i = 0; i < matrSize; i++ )
    {

        ptj = pt;               /* Get begin of the matrix */

        for( j = 0; j < i; j++ )
        {                       /*  By all elements in line of source */
            ptk = pti;          /* ti - current element of the matrix */
            sum = 0.0;
            k = j;

            while( k-- )
            {
                sum += (*ptj++) * (*ptk++);
            }
            *ptk = ((*ps++) - sum) / (*ptj++);
        }

        sum = 0.0;
        k = i;
        while( k-- )
        {
            sum += ((*pti) * (*pti));
            pti++;
        }
        d = (*ps++) - sum;
/*
        if (d <= 0.0) {
            return CV_BADCOEF_ERR;
        }
*/
        *pti++ = sqrt( d );
    }

/*  
    fprintf(file,"\nCholesky matrix (tmpMatr2)\n");
    for (t = 0; t < (matrSize * (matrSize + 1)) / 2; t++) {
        fprintf(file,"tmpMatr2[%d] = %f\n",t,tmpMatr2[t]);
    }
*/

    /*  Find inverse matrix for triangular tmpMatr2 and strore in tmpMatr */


    icvSetZero_64d( tmpMatr, 1, (matrSize * (matrSize + 1)) / 2 );

    dstBase = tmpMatr;
    shift = 2;

    for( i = 0; i < matrSize; i++ )
    {
        *dstBase = 1.0;
        dstBase += shift;
        shift++;
    }
/*
    fprintf(file,"\nIdentical matrix (tmpMatr)\n");
    for (t = 0; t < (matrSize * (matrSize + 1)) / 2; t++) {
        fprintf(file,"tmpMatr[%d] = %f\n",t,tmpMatr[t]);
    }
*/
    srcBase = tmpMatr2;
    dstBase = tmpMatr;
    ptk = tmpMatr;
    ptkBase = tmpMatr;

    for( i = 0; i < matrSize; i++ )
    {                           /* For each row */

        ptj = tmpMatr;

        for( j = 0; j < i; j++ )
        {                       /* for each element in row */
            coef = *srcBase++;  /* Get current coefficient */
            k = j + 1;

            ptk = ptkBase;
            while( k-- )
            {                   /* Multiplacate elements and subtruct */
                *(ptk++) -= coef * (*ptj++);
            }
        }

        k = i + 1;
        coef = 1.0 / (*srcBase++);

        while( k-- )
        {
            *ptkBase++ *= coef;
        }
    }
/*
    fprintf(file,"\nInv Cholesky matrix (tmpMatr)\n");
    for (t = 0; t < (matrSize * (matrSize + 1)) / 2; t++) {
        fprintf(file,"tmpMatr[%d] = %f\n",t,tmpMatr[t]);
    }
*/

    /* Transpose this matrix for suitable computation tmpMatr -> tmpMatr2 */


    dstBase = tmpMatr2;
    srcBase = tmpMatr;
    begRow = tmpMatr;           /* begin of row for each cycle */

    shift = 2;

    for( i = 0; i < matrSize; i++ )
    {

        colShift = i + 1;


        ptj = begRow;

        for( j = 0; j < matrSize - i; j++ )
        {

            *dstBase++ = *ptj;

            ptj += colShift;
            colShift++;
        }

        begRow += shift;
        shift++;
    }
/*
    fprintf(file,"\nTranspose Inv Cholesky matrix (tmpMatr2)\n");
    for (t = 0; t < (matrSize * (matrSize + 1)) / 2; t++) {
        fprintf(file,"tmpMatr2[%d] = %f\n",t,tmpMatr2[t]);
    }
*/
    /* multiplacate to itself (quadr) matrix tmpMatr2  */
    /* and store result in the srcTrans (quadr) */

    dstBase_i = srcTrans;

    shift_i = matrSize;

    pti = tmpMatr2;

    for( i = 0; i < matrSize; i++ )
    {

        dstBase = dstBase_i;
        dstBase_i += matrSize;

        ptj = tmpMatr2;

        shift_j = matrSize;

        for( j = 0; j < i + 1; j++ )
        {

            if( j < i )
            {
                dj = i - j;
                di = 0;
                delta = i;
            }
            else
            {
                dj = 0;
                di = j - i;
                delta = j;
            }

            icvDotProduct_64d( pti + di, ptj + dj, matrSize - delta, dstBase++ );
            ptj += shift_j;
            shift_j--;
        }

        pti += shift_i;
        shift_i--;

    }

    /* copy elements to up the diagonal */

    for( i = 0; i < matrSize - 1; i++ )
    {
        for( j = i + 1; j < matrSize; j++ )
        {
            srcTrans[i * matrSize + j] = srcTrans[j * matrSize + i];
        }
    }

/*
    fprintf(file,"\nMultiplacate matix not diagonal(srcTrans)\n");
    for(i = 0; i < matrSize; i++) {
        for (j = 0; j < matrSize; j++) {
            fprintf(file,"srcTrans[%d,%d] = %f\n",i,j,srcTrans[i * matrSize + j]);
        }
        fprintf(file,"\n");
    }
*/


    /* Multiplicate this triangular matrix and transpose source */
    /* for suitable use not transpose matrix srcMatr */

    dstBase = dstMatr;

    pti = srcTrans;

    for( i = 0; i < matrSize; i++ )
    {

        ptj = srcMatr;

        for( j = 0; j < height; j++ )
        {

            icvDotProduct_64d( pti, ptj, matrSize, dstBase++ );

            ptj += width;
        }

        pti += matrSize;
    }

/*
    fprintf(file,"\nResult matrix (dstMatr)\n");
    for(i = 0; i < width; i++) {
        for (j = 0; j < height; j++) {
            fprintf(file,"dstMatr[%d,%d] = %f\n",i,j,dstMatr[i * height + j]);
        }
        fprintf(file,"\n");
    }
*/

    icvFree( &tmpMatr );
    icvFree( &tmpMatr2 );
    icvFree( &srcTrans );

/*    fclose(file); */
    return CV_NO_ERR;
}

/*=====================================================================================*/
IPCVAPI_IMPL( CvStatus, icvTransposeMatrix_64d, (CvMatr64d srcMatr,
                                                 int matrWidth,
                                                 int matrHeight, CvMatr64d destMatr) )
{
    int i, j;

    double *dst = destMatr;
    double *src = srcMatr;

    int Decrem = matrWidth * matrHeight - 1;

    if( srcMatr == 0 )
        return CV_NULLPTR_ERR;
    if( destMatr == 0 )
        return CV_NULLPTR_ERR;
    if( matrWidth <= 0 )
        return CV_BADSIZE_ERR;
    if( matrHeight <= 0 )
        return CV_BADSIZE_ERR;
    for( j = 0; j < matrHeight; j++ )
    {
        for( i = 0; i < matrWidth; i++ )
        {
            *dst = src[i];
            dst += matrHeight;
        }
        dst -= Decrem;
        src += matrWidth;
    }
    return CV_NO_ERR;
}


/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvMulMatrix3x3_64d   
//    Purpose:    Product of 2 matrix
//    Context:   
//    Parameters: srcMatr1 - first matrix
//                srcMatr2 - second matrix
//
//                destMatr - destination matrix
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvMulMatrix3x3_64d, (CvMatr64d srcMatr1,
                                              CvMatr64d srcMatr2, CvMatr64d destMatr) )
{
    /* check bad arguments */
    if( srcMatr1 == NULL )
        return CV_NULLPTR_ERR;
    if( destMatr == NULL )
        return CV_NULLPTR_ERR;
    if( srcMatr2 == NULL )
        return CV_NULLPTR_ERR;
    destMatr[0] = srcMatr1[0] * srcMatr2[0] +
        srcMatr1[1] * srcMatr2[3] + srcMatr1[2] * srcMatr2[6];
    destMatr[1] = srcMatr1[0] * srcMatr2[1] +
        srcMatr1[1] * srcMatr2[4] + srcMatr1[2] * srcMatr2[7];
    destMatr[2] = srcMatr1[0] * srcMatr2[2] +
        srcMatr1[1] * srcMatr2[5] + srcMatr1[2] * srcMatr2[8];
    destMatr[3] = srcMatr1[3] * srcMatr2[0] +
        srcMatr1[4] * srcMatr2[3] + srcMatr1[5] * srcMatr2[6];
    destMatr[4] = srcMatr1[3] * srcMatr2[1] +
        srcMatr1[4] * srcMatr2[4] + srcMatr1[5] * srcMatr2[7];
    destMatr[5] = srcMatr1[3] * srcMatr2[2] +
        srcMatr1[4] * srcMatr2[5] + srcMatr1[5] * srcMatr2[8];
    destMatr[6] = srcMatr1[6] * srcMatr2[0] +
        srcMatr1[7] * srcMatr2[3] + srcMatr1[8] * srcMatr2[6];
    destMatr[7] = srcMatr1[6] * srcMatr2[1] +
        srcMatr1[7] * srcMatr2[4] + srcMatr1[8] * srcMatr2[7];
    destMatr[8] = srcMatr1[6] * srcMatr2[2] +
        srcMatr1[7] * srcMatr2[5] + srcMatr1[8] * srcMatr2[8];
    return CV_NO_ERR;
}

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvTrace_64d   
//    Purpose:    Compute the trace of matrix, sum of diagonal elements
//    Context:   
//    Parameters: matr - source matrix
//                width
//                height - sizes of matr
//                trace - result
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvTrace_64d, (CvMatr64d matr, int width, int height, double *trace) )
{
    int t;
    int size;
    double tr;

    size = min( width, height );
    tr = 0;
    for( t = 0; t < size; t++ )
    {
        tr += matr[t * width + t];
    }
    *trace = tr;

    return CV_NO_ERR;
}                               /* icvTrace_64d */

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvDetMatrix_64d   
//    Purpose:    Compute determinate of matrix
//    Context:   
//    Parameters: matr - source matrix
//                matrSize - sizes of srcMatr
//                det - result
//               
//    Returns:
//    CV_NO_ERROR or error code     
//    Notes:
//F*/
IPCVAPI_IMPL( CvStatus, icvDetMatrix_64d, (CvMatr64d matr, int matrSize, double *det) )
{
    CvMatr64d matrWork;
    CvVect64d tmpLine;
    int i, j, t;
    int change_sign;
    double coef;
    double mul;
    double norm = 0, eps, eps0 = 1.0e-40;

    if( matr == NULL || det == NULL ) return CV_NULLPTR_ERR;
    if( matrSize < 1 ) return CV_BADFACTOR_ERR;

    if( matrSize == 1 )
    {
        *det = *matr;
        return CV_NO_ERR;
    }

    if( matrSize == 2 )
    {
        *det = matr[0]*matr[3] - matr[1]*matr[2];
        return CV_NO_ERR;
    }

    if( matrSize == 3 )
    {
        *det = matr[0] * ( matr[4]*matr[8] - matr[5]*matr[7] )
             + matr[3] * ( matr[2]*matr[7] - matr[1]*matr[8] )
             + matr[6] * ( matr[1]*matr[5] - matr[2]*matr[4] );
        return CV_NO_ERR;
    }

    if( matrSize == 4 )
    {
        *det = matr[0] * (matr[5] * (matr[10]*matr[15] - matr[11]*matr[14])
                       +  matr[6] * (matr[11]*matr[13] - matr[9] *matr[15])
                       +  matr[7] * (matr[9] *matr[14] - matr[10]*matr[13]) );

        *det+= matr[1] * (matr[6] * (matr[8] *matr[15] - matr[11]*matr[12])
                       + matr[7]  * (matr[10]*matr[12] - matr[8] *matr[14])
                       + matr[4]  * (matr[11]*matr[14] - matr[10]*matr[15]) );

        *det+= matr[2] * (matr[7] * (matr[8] *matr[13] - matr[9] *matr[12])
                       + matr[4]  * (matr[9] *matr[15] - matr[11]*matr[13])
                       + matr[5]  * (matr[11]*matr[12] - matr[8] *matr[15]) );

        *det+= matr[3] * (matr[4] * (matr[10]*matr[13] - matr[9] *matr[14])
                       + matr[5]  * (matr[8] *matr[14] - matr[10]*matr[12])
                       + matr[6]  * (matr[9] *matr[12] - matr[8] *matr[13]) );
        return CV_NO_ERR;
    }

    change_sign = 0;

    for( i=0; i<matrSize*matrSize; i++ )
        norm += fabs( matr[i] );

    norm /= matrSize*matrSize;
    eps = eps0 * norm;

    matrWork = icvCreateMatrix_64d( matrSize, matrSize );
    tmpLine  = icvCreateVector_64d( matrSize );

    icvCopyMatrix_64d( matr, matrSize, matrSize, matrWork );

    /* Make matrix is up triangular */
    for( i = 0; i < matrSize - 1; i++ )
    {
        int     in = i * matrSize;
        double  mx = fabs( matrWork[in+i] );
        int change = i;

        /* find maximal element */

        for( t = i+1; t < matrSize; t++ )
        {
            double w = fabs( matrWork[t*matrSize + i] );
            if( w > mx )
            {
                mx     = w;
                change = t;
            }
        }

        if( mx < eps ) /* Element != 0 not found;  det = 0 */
        {
            *det = 0.0;
            icvDeleteMatrix( matrWork );
            icvDeleteVector( tmpLine );
            return CV_NO_ERR;
        }

        if( change != i ) /* change lines */
        {
            int nchange = change * matrSize;

            icvCopyVector_64d( matrWork + nchange, matrSize,  tmpLine );
            icvCopyVector_64d( matrWork + in,      matrSize,  matrWork + nchange );
            icvCopyVector_64d( tmpLine,            matrSize,  matrWork + in );

            change_sign++;
        }

        /* Virtual set elements to zero */

        coef = 1.0e0 / matrWork[in+i];

        for( j = i+1; j < matrSize; j++ )
        {
            int jn = j * matrSize;

            if( matrWork[jn + i] )
            {
                mul = coef * matrWork[jn + i];

                icvScaleVector_64d( matrWork+in+i+1, tmpLine, matrSize-i-1, mul );
                icvSubVector_64d  ( matrWork+jn+i+1, tmpLine, matrWork+jn+i+1,
                                                                   matrSize-i-1 );
            }
        }

    }  /* i */

    *det = ( (change_sign & 1) == 1 ) ? -1.0e0 : 1.0e0;

    for( t=0, j=0; t < matrSize; t++, j += matrSize+1 )
        *det *= matrWork[j];

    icvDeleteMatrix( matrWork );
    icvDeleteVector( tmpLine );
    return CV_NO_ERR;
}                               /* icvDetMatrix_64d */

/*F///////////////////////////////////////////////////////////////////////////////////////
//    Name:       icvCrossProduct2L_64d  
//    Purpose:    Computes cross product of two 3D vectors
//                if A and B - input vectors, then
//                C = A x B is cross product. Its components are
//
//                Cx = AyBz - AzBy 
//                Cy = AzBx - AxBz 
//                Cz = AxBy - AyBx 
//
//    Context:   
//    Parameters: src1 - first vector
//                src2 - second vector
//                dest - cross product 
//    Returns:   
//    Notes:     
//F*/
IPCVAPI_IMPL( CvStatus, icvCrossProduct2L_64d, (CvVect64d src1,
                                                CvVect64d src2, CvVect64d dest) )
{
    /* check bad arguments */
    if( src1 == NULL )
        return CV_NULLPTR_ERR;
    if( src2 == NULL )
        return CV_NULLPTR_ERR;
    if( dest == NULL )
        return CV_NULLPTR_ERR;

    dest[2] = src1[0] * src2[1] - src1[1] * src2[0];
    dest[0] = src1[1] * src2[2] - src1[2] * src2[1];
    dest[1] = src1[2] * src2[0] - src1[0] * src2[2];
    /*
       dest[2] = src1[0]*src2[1]-src1[1]*src2[0];
       dest[1] = src1[2]*src2[0]-src1[0]*src2[2];
       dest[0] = src1[1]*src2[2]-src1[2]*src2[1];
     */

    return CV_NO_ERR;


}
