/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/
#include "_cv.h"
#include "_cvwrap.h"

CV_IMPL void
cvmAdd( CvMat * srcA, CvMat * srcB, CvMat * dst )
{

    CV_FUNCNAME( "cvmAdd" );

    __BEGIN__;


    if( srcA->type != srcB->type )
        CV_ERROR( IPL_StsBadArg, "sources have different types" );

    if( srcA->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different sizes" );


    switch (srcA->type)
    {
    case CV_MAT32F:
        IPPI_CALL( icvAddMatrix_32f
                   ( srcA->data.fl, srcB->data.fl, dst->data.fl, srcA->cols, srcA->rows ));
        break;
    case CV_MAT3x1_32F:
        IPPI_CALL( icvAddMatrix_32f( srcA->data.fl, srcB->data.fl, dst->data.fl, 1, 3 ));
        break;
    case CV_MAT4x1_32F:
        IPPI_CALL( icvAddMatrix_32f( srcA->data.fl, srcB->data.fl, dst->data.fl, 1, 4 ));
        break;
    case CV_MAT3x3_32F:
        IPPI_CALL( icvAddMatrix3x3_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        break;
    case CV_MAT4x4_32F:
        IPPI_CALL( icvAddMatrix4x4_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        break;
    case CV_MAT64D:
        IPPI_CALL( icvAddMatrix_64d
                   ( srcA->data.db, srcB->data.db, dst->data.db, srcA->cols, srcA->rows ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmSub( CvMat * srcA, CvMat * srcB, CvMat * dst )
{

    CV_FUNCNAME( "cvmSub" );

    __BEGIN__;


    if( srcA->type != srcB->type )
        CV_ERROR( IPL_StsBadArg, "sources have different types" );

    if( srcA->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different sizes" );


    switch (srcA->type)
    {
    case CV_MAT32F:
        IPPI_CALL( icvSubMatrix_32f
                   ( srcA->data.fl, srcB->data.fl, dst->data.fl, srcA->cols, srcA->rows ));
        break;
    case CV_MAT3x1_32F:
        IPPI_CALL( icvSubMatrix_32f( srcA->data.fl, srcB->data.fl, dst->data.fl, 1, 3 ));
        break;
    case CV_MAT4x1_32F:
        IPPI_CALL( icvSubMatrix_32f( srcA->data.fl, srcB->data.fl, dst->data.fl, 1, 4 ));
        break;
    case CV_MAT3x3_32F:
        IPPI_CALL( icvSubMatrix3x3_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        break;
    case CV_MAT4x4_32F:
        IPPI_CALL( icvSubMatrix4x4_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        break;
    case CV_MAT64D:
        IPPI_CALL( icvSubMatrix_64d
                   ( srcA->data.db, srcB->data.db, dst->data.db, srcA->cols, srcA->rows ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmScale( CvMat * src, CvMat * dst, double alpha )
{

    CV_FUNCNAME( "cvmScale" );

    __BEGIN__;

    if( src->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different sizes" );


    switch (src->type)
    {
    case CV_MAT32F:
        IPPI_CALL( icvScaleMatrix_32f
                   ( src->data.fl, dst->data.fl, src->cols, src->rows, (float) alpha ));
        break;
    case CV_MAT3x1_32F:
        IPPI_CALL( icvScaleMatrix_32f( src->data.fl, dst->data.fl, 1, 3, (float) alpha ));
        break;
    case CV_MAT4x1_32F:
        IPPI_CALL( icvScaleMatrix_32f( src->data.fl, dst->data.fl, 1, 4, (float) alpha ));
        break;
    case CV_MAT3x3_32F:
        IPPI_CALL( icvScaleMatrix3x3_32f( src->data.fl, dst->data.fl, (float) alpha ));
        break;
    case CV_MAT4x4_32F:
        IPPI_CALL( icvScaleMatrix4x4_32f( src->data.fl, dst->data.fl, (float) alpha ));
        break;
    case CV_MAT64D:
        IPPI_CALL( icvScaleMatrix_64d
                   ( src->data.db, dst->data.db, src->cols, src->rows, alpha ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL double
cvmDotProduct( CvMat * srcA, CvMat * srcB )
{

    CV_FUNCNAME( "cvmDotProduct" );

    __BEGIN__;

    float fl;
    double db;


    if( (srcA->cols != 1) || (srcB->cols != 1) )
        CV_ERROR( IPL_StsBadArg, "dst has inappropriate size" );
    if( srcA->rows != srcB->rows )
        CV_ERROR( IPL_StsBadArg, " sources have different sizes" );


    if( (int) srcA->type < 64 )
    {
        IPPI_CALL( icvDotProduct_32f( srcA->data.fl, srcB->data.fl, srcA->rows, &fl ));
        db = (double) fl;

    }
    else
    {
        IPPI_CALL( icvDotProduct_64d( srcA->data.db, srcB->data.db, srcA->rows, &db ));

    }
    return db;

    __CLEANUP__;
    __END__;

}
CV_IMPL void
cvmCrossProduct( CvMat * srcA, CvMat * srcB, CvMat * dst )
{
    CV_FUNCNAME( "cvmCrossProduct" );
    __BEGIN__;
    if( (srcA->type != CV_MAT3x1_32F) || (srcB->type != CV_MAT3x1_32F) ||
        (dst->type != CV_MAT3x1_32F) )
        CV_ERROR( IPL_StsBadArg, "source and destination have different sizes" );
    IPPI_CALL( icvCrossProduct2L_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmMul( CvMat * srcA, CvMat * srcB, CvMat * dst )
{

    CV_FUNCNAME( "cvmMul" );

    __BEGIN__;


    if( srcA->cols != srcB->rows )
        CV_ERROR( IPL_StsBadArg, "sources have inappropriate sizes" );
    if( (dst->cols != srcB->cols) || (dst->rows != srcA->rows) )
        CV_ERROR( IPL_StsBadArg, " dst has inappropriate size" );

    switch (srcA->type)
    {
    case CV_MAT32F:
        if( srcB->cols != 1 )
        {
            IPPI_CALL( icvMulMatrix_32f( srcA->data.fl,
                                          srcA->cols,
                                          srcA->rows,
                                          srcB->data.fl,
                                          srcB->cols, srcB->rows, dst->data.fl ));
        }
        else
            IPPI_CALL( icvTransformVector_32f( srcA->data.fl,
                                                srcB->data.fl,
                                                dst->data.fl, srcA->cols, srcA->rows ));
        break;
    case CV_MAT3x3_32F:
        if( srcB->type == CV_MAT3x1_32F )
        {
            IPPI_CALL( icvTransformVector3x1_32f( srcA->data.fl,
                                                   srcB->data.fl, dst->data.fl ));
        }
        else if( srcB->type == CV_MAT3x3_32F )
        {
            IPPI_CALL( icvMulMatrix3x3_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        }
        else
            IPPI_CALL( icvMulMatrix_32f( srcA->data.fl,
                                          3, 3, srcB->data.fl, srcB->cols, 3, dst->data.fl ));
        break;
    case CV_MAT4x4_32F:
        if( srcB->type == CV_MAT4x1_32F )
        {
            IPPI_CALL( icvTransformVector4x1_32f( srcA->data.fl,
                                                   srcB->data.fl, dst->data.fl ));
        }
        else if( srcB->type == CV_MAT4x4_32F )
        {
            IPPI_CALL( icvMulMatrix4x4_32f( srcA->data.fl, srcB->data.fl, dst->data.fl ));
        }
        else
        {
            IPPI_CALL( icvMulMatrix_32f( srcA->data.fl,
                                          4, 4, srcB->data.fl, srcB->cols, 4, dst->data.fl ));
        }
        break;
    case CV_MAT64D:
        if( srcB->cols != 1 )
        {
            IPPI_CALL( icvMulMatrix_64d( srcA->data.db,
                                          srcA->cols,
                                          srcA->rows,
                                          srcB->data.db,
                                          srcB->cols, srcB->rows, dst->data.db ));
        }
        else
            IPPI_CALL( icvTransformVector_64d( srcA->data.db,
                                                srcB->data.db,
                                                dst->data.db, srcA->cols, srcA->rows ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmMulTransposed( CvMat * src, CvMat * dst, int order )
{

    CV_FUNCNAME( "cvmMulTransposed" );

    __BEGIN__;


    if( order )
    {
        if( (dst->cols != src->cols) || (dst->rows != src->cols) )
            CV_ERROR( IPL_StsBadArg, " dst has inappropriate size" );

        if( (int) src->type < 64 )
        {
            IPPI_CALL( icvMulTransMatrixR_32f( src->data.fl,
                                                src->cols, src->rows, dst->data.fl ));

        }
        else
        {
            IPPI_CALL( icvMulTransMatrixR_64d( src->data.db,
                                                src->cols, src->rows, dst->data.db ));
        }
    }
    else
    {
        if( (dst->cols != src->rows) || (dst->rows != src->rows) )
            CV_ERROR( IPL_StsBadArg, " dst has inappropriate size" );

        if( (int) src->type < 64 )
        {
            IPPI_CALL( icvMulTransMatrixL_32f( src->data.fl,
                                                src->cols, src->rows, dst->data.fl ));

        }
        else
        {
            IPPI_CALL( icvMulTransMatrixL_64d( src->data.db,
                                                src->cols, src->rows, dst->data.db ));
        }
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmTranspose( CvMat * src, CvMat * dst )
{

    CV_FUNCNAME( "cvmTranspose" );

    __BEGIN__;

    if( src->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different sizes" );


    switch (src->type)
    {
    case CV_MAT32F:
        IPPI_CALL( icvTransposeMatrix_32f
                   ( src->data.fl, src->cols, src->rows, dst->data.fl ));
        break;
    case CV_MAT3x3_32F:
        IPPI_CALL( icvTransposeMatrix3x3_32f( src->data.fl, dst->data.fl ));
        break;
    case CV_MAT4x4_32F:
        IPPI_CALL( icvTransposeMatrix4x4_32f( src->data.fl, dst->data.fl ));
        break;
    case CV_MAT64D:
        IPPI_CALL( icvTransposeMatrix_64d
                   ( src->data.db, src->cols, src->rows, dst->data.db ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;
}

CV_IMPL void
cvmInvert( CvMat * src, CvMat * dst )
{

    CV_FUNCNAME( "cvmInvert" );

    __BEGIN__;

    if( src->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different sizes" );
    if( src->cols != src->rows )
        CV_ERROR( IPL_StsBadArg, "src is not quadratic matrix" );


    switch (src->type)
    {
    case CV_MAT32F:
        IPPI_CALL( icvInvertMatrix_32f( src->data.fl, src->cols, dst->data.fl ));
        break;
    case CV_MAT3x3_32F:
        IPPI_CALL( icvInvertMatrix3x3_32f( src->data.fl, dst->data.fl ));
        break;
    case CV_MAT4x4_32F:
        IPPI_CALL( icvInvertMatrix4x4_32f( src->data.fl, dst->data.fl ));
        break;
    case CV_MAT64D:
        IPPI_CALL( icvInvertMatrix_64d( src->data.db, src->cols, dst->data.db ));
        break;
    default:
        CV_ERROR( IPL_BadDepth, icvUnsupportedFormat );
    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmSVD( CvMat * src, CvMat * w, CvMat * v )
{

    CV_FUNCNAME( "cvmSVD" );

    __BEGIN__;


    if( (w->cols != src->cols) || (w->rows != src->cols) )
        CV_ERROR( IPL_StsBadArg, " dst has inappropriate size" );
    if( (v->cols != src->cols) || (v->rows != src->cols) )
        CV_ERROR( IPL_StsBadArg, " dst has inappropriate size" );


    if( (int) src->type < 64 )
    {
        IPPI_CALL( icvSVDecomposition_32f( src->data.fl,
                                            src->cols, src->rows, w->data.fl, v->data.fl ));

    }
    else
    {
        IPPI_CALL( icvSVDecomposition_64d( src->data.db,
                                            src->cols, src->rows, w->data.db, v->data.db ));

    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmEigenVV( CvMat * src, CvMat * evects, CvMat * evals, double eps )
{

    CV_FUNCNAME( "cvmEigenVV" );

    __BEGIN__;

    if( src->cols != src->rows )
        CV_ERROR( IPL_StsBadArg, " source is not quadratic matrix" );
    if( (evects->cols != src->cols) || (evects->rows != src->rows) )
        CV_ERROR( IPL_StsBadArg, "evects has inappropriate size" );
    if( (evals->rows != src->rows) )
        CV_ERROR( IPL_StsBadArg, " evals has inappropriate size" );


    if( (int) src->type < 64 )
    {
        IPPI_CALL( icvJacobiEigens_32f( src->data.fl,
                                         evects->data.fl,
                                         evals->data.fl, src->cols, (float) eps ));

    }
    else
    {
        IPPI_CALL( icvJacobiEigens_64d( src->data.db,
                                         evects->data.db,
                                         evals->data.db, src->cols, (float) eps ));

    }

    __CLEANUP__;
    __END__;

}

CV_IMPL void
cvmCopy( CvMat * src, CvMat * dst )
{

    CV_FUNCNAME( "cvmCopy" );

    __BEGIN__;

    if( (src->cols != dst->cols) || (src->rows != dst->rows) )
        CV_ERROR( IPL_StsBadArg, " source is not quadratic matrix" );
    if( src->type != dst->type )
        CV_ERROR( IPL_StsBadArg, "src and dst have different types" );

    if( (int) src->type < 64 )
    {
        IPPI_CALL( icvCopyMatrix_32f( src->data.fl, src->cols, src->rows, dst->data.fl ));

    }
    else
    {
        IPPI_CALL( icvCopyMatrix_64d( src->data.db, src->cols, src->rows, dst->data.db ));

    }

    __CLEANUP__;
    __END__;

}

CV_IMPL double
cvmTrace( CvMat * mat )
{

    float fl;
    double db;

    CV_FUNCNAME( "cvmTrace" );

    __BEGIN__;


    if( (int) mat->type < 64 )
    {
        IPPI_CALL( icvTrace_32f( mat->data.fl, mat->cols, mat->rows, &fl ));
        db = (double) fl;

    }
    else
    {
        IPPI_CALL( icvTrace_64d( mat->data.db, mat->cols, mat->rows, &db ));
    }
    return db;

    __CLEANUP__;
    __END__;

}
CV_IMPL double
cvmDet( CvMat* mat )
{
    double det = 0.0;

    CV_FUNCNAME( "cvmDet" );
    __BEGIN__;
    if( mat->cols != mat->rows )
        CV_ERROR( IPL_StsBadArg, "sources have unsupported format" );

    if( (int)mat->type < 64 )
    {
        IPPI_CALL( icvDetMatrix_32f( mat->data.fl, mat->cols, &det ));
    }
    else
    {
        IPPI_CALL( icvDetMatrix_64d( mat->data.db, mat->cols, &det ));
    }

    return det;

    __CLEANUP__;
    __END__;
}

CV_IMPL void
cvmSetZero( CvMat * mat )
{

    CV_FUNCNAME( "cvmSetZero" );

    __BEGIN__;


    if( (int) mat->type < 64 )
    {
        IPPI_CALL( icvSetZero_32f( mat->data.fl, mat->cols, mat->rows ));

    }
    else
    {
        IPPI_CALL( icvSetZero_64d( mat->data.db, mat->cols, mat->rows ));

    }

    __CLEANUP__;
    __END__;

}
CV_IMPL void
cvmSetIdentity( CvMat * mat )
{

    CV_FUNCNAME( "cvmSetIdentity" );

    __BEGIN__;


    if( (int) mat->type < 64 )
    {
        IPPI_CALL( icvSetIdentity_32f( mat->data.fl, mat->cols, mat->rows ));

    }
    else
    {
        IPPI_CALL( icvSetIdentity_64d( mat->data.db, mat->cols, mat->rows ));

    }

    __CLEANUP__;
    __END__;

}

CV_IMPL double
cvmMahalanobis( CvMat * srcA, CvMat * srcB, CvMat * mat )
{
    float dist;

    CV_FUNCNAME( "cvmMahalanobis" );

    __BEGIN__;

    if( ((int) mat->type > 63) || ((int) srcA->type > 63) || ((int) srcB->type > 63) )
        CV_ERROR( IPL_StsBadArg, "sources have unsupported format" );
    if( mat->cols != mat->rows )
        CV_ERROR( IPL_StsBadArg, "sources have unsupported format" );
    if( srcA->cols != 1 )
        CV_ERROR( IPL_StsBadArg, "source have inappropriate size" );
    if( srcB->cols != 1 )
        CV_ERROR( IPL_StsBadArg, "source have inappropriate size" );

    IPPI_CALL( icvMahalanobis_32f( srcA->data.fl,
                                    srcB->data.fl, mat->data.fl, mat->cols, &dist ));
    return ( double ) dist;
    __CLEANUP__;
    __END__;


}

CV_IMPL void
cvmPerspectiveProject( CvMat * mat, CvMatArray * src, CvMatArray * dst )
{

    CV_FUNCNAME( "cvmPerspectiveProject" );

    __BEGIN__;

    if( ((int) mat->type != CV_MAT4x4_32F) || ((int) src->type != CV_MAT3x1_32F) ||
        ((int) dst->type != CV_MAT3x1_32F) )
        CV_ERROR( IPL_StsBadArg, "sources have unsupported format" );
    if( src->count != dst->count )
        CV_ERROR( IPL_StsBadArg, "source and destination  have different sizes" );

    IPPI_CALL( icvbTransformVector3x1P_32f( mat->data.fl,
                                             src->data.fl, src->count, dst->data.fl ));

    __CLEANUP__;
    __END__;


}

/*======================================================================================*/

CV_IMPL void
cvmRodrigues( CvMat * rotMat, CvMat * rotVect, CvMat * Jacobian, CvRodriguesType convType )
{
    CV_FUNCNAME( "cvmRodrigues" );
    __BEGIN__;
    if( (rotMat->cols != 3) || (rotMat->rows != 3) )
        CV_ERROR( IPL_StsBadArg, "Bad Rotation Matrix" );
    if( (rotVect->cols != 1) || (rotMat->rows != 3) )
        CV_ERROR( IPL_StsBadArg, "Bad Rotation Vector" );
    if( (rotMat->cols != 9) || (rotMat->rows != 3) )
        CV_ERROR( IPL_StsBadArg, "Bad Jacobian" );
    if( (int) rotMat->type < 64 )
    {
        IPPI_CALL( icvRodrigues( rotMat->data.fl,
                                  rotVect->data.fl, Jacobian->data.fl, convType ));
    }
    else
        IPPI_CALL( icvRodrigues_64d( rotMat->data.db,
                                      rotVect->data.db, Jacobian->data.db, convType ));
    __CLEANUP__;
    __END__;
}


CV_IMPL void
cvmAlloc( CvMat * mat )
{
    CV_FUNCNAME( "cvmAlloc" );
    __BEGIN__;

    if( !mat )
        CV_ERROR( IPL_StsBadArg, "Bad matrix pointer" );
    mat->data.fl = (float *) icvAlloc( mat->cols * mat->rows *
                                       (mat->type < 64 ? sizeof( float ) : sizeof( double )));

    __CLEANUP__;
    __END__;
}


CV_IMPL void
cvmFree( CvMat * mat )
{
    CV_FUNCNAME( "cvmFree" );
    __BEGIN__;

    if( !mat )
        CV_ERROR( IPL_StsBadArg, "Bad matrix pointer" );
    icvFree( &mat->data.fl );

    __CLEANUP__;
    __END__;
}


CV_IMPL void
cvmAllocArray( CvMatArray * mat )
{
    CV_FUNCNAME( "cvmAllocArray" );
    __BEGIN__;

    if( !mat )
        CV_ERROR( IPL_StsBadArg, "Bad matrix pointer" );
    mat->data.fl = (float *) icvAlloc( mat->count * mat->cols * mat->rows *
                                       (mat->type < 64 ? sizeof( float ) : sizeof( double )));

    __CLEANUP__;
    __END__;
}

CV_IMPL void
cvmFreeArray( CvMatArray * mat )
{
    CV_FUNCNAME( "cvmFreeArray" );
    __BEGIN__;

    if( !mat )
        CV_ERROR( IPL_StsBadArg, "Bad matrix pointer" );

    icvFree( &mat->data.fl );

    __CLEANUP__;
    __END__;
}
