/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/


/****************************************************************************************/
/*                This part of code is used to generate DLL function body               */
/****************************************************************************************/

/* get all the function prototypes as in static library */
#include "_cv.h"
#include "_cvwrap.h"

#if _MSC_VER >= 1200
#pragma warning( disable: 4115 )        /* type definition in () */
#pragma warning( disable: 4100 )        /* unreferenced formal paramter */
#endif

#ifdef WIN32
#include <windows.h>
#else
#include <dlfcn.h>
#include <string.h>
#endif

#include <stdio.h>

#define __CV_DEFINE_USER_LIST__

/****************************************************************************************/
/*              Implement all dll functions via static library calls                    */
/****************************************************************************************/

/****************************************************************************************/
/*                               Make functions descriptions                            */
/****************************************************************************************/

typedef void (CV_CDECL * ipp_func_addr) (void);

#undef IPCVAPI
#define GETAPI(addr,name) { (ipp_func_addr*)&addr, #name, 0 },
#define IPCVAPI(type,name,arg) GETAPI( name, name )

struct func_desc
{
    ipp_func_addr *addr;
    const char *name;
    int loaded;
};

static func_desc ipp_func_desc[] =
{
#undef __IPCV_H_
#undef __CV_H_
#include "_cv.h"
#undef __CV_H_
#undef __IPCV_H_
    {
    0, 0}
};


#define PROC_PX     0
#define PROC_M5     ((1<<23)|5) /* mmx */
#define PROC_M6     ((1<<15)|(1<<23)|6) /* cmov + mmx */
#define PROC_A6     ((1<<25)|PROC_M6)   /* --||-- + xmm */
#define PROC_W7     ((1<<26)|PROC_A6|(1<<3)|1)   /* --||-- + emm */

/*
   determine processor type
*/
static int
get_processor_type( void )
{
#ifdef WIN32
    int proc_type = PROC_PX;
    int version = 0, features = 0, family;

    __asm
    {
        /* check CPUID support */
        pushfd
        pushfd
        pop eax
        mov edx, eax
        xor eax, (1 << 21)
        push eax
        popfd
        pushfd
        pop eax
        xor ecx, ecx
        cmp eax, edx
        setne cl
        jz no_cpuid
            /* do CPUID and save version and features information */
        mov eax, 1
        push ebx
        push esi
        push edi
        _emit 0x0f
        _emit 0xa2
        pop edi
        pop esi
        pop ebx
        mov version, eax
        mov features, edx
no_cpuid:
        popfd
    }

    family = (version >> 8) & 15;
    if( family >= 5 && (features & (1 << 23)) != 0 )    /* Pentium MMX or higher */
    {
        proc_type = (features & PROC_W7 & -256) | family;
    }
    return proc_type;

#else
    int proc_type = PROC_PX;

    char buffer[1000] = "empty";

    //reading /proc/cpuinfo file (proc file system must be supported)
    FILE *file = fopen( "/proc/cpuinfo", "r" );

    if( !file )
        return proc_type;

    if( !fread( buffer, 1, 1000, file ))
        return proc_type;

    // ok, we read it :)

    // may be it is MMX ?
    if( strstr( buffer, "mmx" ))
        proc_type = PROC_M5;    // yes, it is MMX
    else
        return proc_type;       // no, it isn't MMX (may be Pentium PRO, but we not supported
    // optimization under it, sorry)

    // may be it is PII ?
    if( strstr( buffer, "cmov" ))
        proc_type = PROC_M6;    // yes, it is PII
    else
        return proc_type;       // ok, it is MMX, not bad :)

    // may be it is PIII ?
    if( strstr( buffer, "xmm" ) ||  strstr( buffer, "sse" ))
        proc_type = PROC_A6;    // wery well, it is PIII
    else
        return proc_type;       // ok, it is PII, good processor :)

    // may be it is PIV ?
    if( strstr( buffer, "emm" ))
        proc_type = PROC_W7;    // wow, you have willamette B)
    else
        return proc_type;       // ok, it is PIII, very well :)

    return proc_type;           // cool, you have PIV :)... or this stupid program failed
#endif
}


/*
   write error message and exit program
*/
/*
static void abort_program( const char* msg )
{
    MessageBeep(MB_ICONSTOP);
    MessageBox(0, msg, "CVL Library Error", MB_ICONSTOP | MB_OK | MB_TASKMODAL);
    exit(-1);
}
*/
/*
   determine processor type, load appropriate dll and
   initialize all function pointers
*/
#ifdef WIN32
#define DLL_NAME1 "ippcv"
#define DLL_NAME2 "optcv"
#define DLL_EXTENTION ".dll"
#else
#define DLL_NAME1 "libippcv"
#define DLL_NAME2 "liboptcv"
#define DLL_EXTENTION ".so"
#endif

//#define PRINT_LOADED_STATUS

static int loaded = 0;
static char dll_name[100] = DLL_NAME1;

static int
icvLoadPrimitives( char *dllName, char *processor_type )
{
#ifdef PRNT_LOADED_STATUS
    printf("Loading...\n");
#endif
    static int number = 0;

    /*char    msg[64]; */
    int proc_type = get_processor_type();

#ifdef WIN32
    HMODULE hdll = 0;
#else
    void *hdll = 0;

#define LoadLibrary(name) dlopen(name, RTLD_LAZY)
#define GetProcAddress dlsym
#endif

    do{
        if( dllName )
            strcpy( dll_name, dllName );
        else
            strcpy( dll_name, DLL_NAME1 );
        
        if( !processor_type )
            switch (proc_type)
            {
            case PROC_W7:
                strcat( dll_name, "w7" );
                proc_type = PROC_A6;
                break;
            case PROC_A6:
                strcat( dll_name, "a6" );
                proc_type = PROC_M6;
                break;
            case PROC_M6:
                strcat( dll_name, "m6" );
                proc_type = PROC_M5;
                break;
            case PROC_M5:
                strcat( dll_name, "m5" );
                proc_type = PROC_PX;
                break;
            default:
                strcat( dll_name, "px" );
                proc_type = PROC_PX;
                break;
            }
        else
        {
            strcat( dll_name, processor_type );
            proc_type = PROC_PX;
        }
        
#if 1                           /*defined NDEBUG */
        strcat( dll_name, DLL_EXTENTION );
#else
        strcat( dll_name, "d" );
        strcat( dll_name, DLL_EXTENTION );
#endif
        
        hdll = LoadLibrary( dll_name );
    }while(proc_type != PROC_PX && !hdll);

    if( !hdll )
    {
#ifdef PRINT_LOADED_STATUS
        printf("Can't load %s\n", dll_name);
#endif
        return 0;
    }
    else
    {
#ifdef PRINT_LOADED_STATUS
        printf("Loaded %s\n", dll_name);
#endif
        loaded++;

        //printf("Loading...\n");

        int i;

        for( i = 0; ipp_func_desc[i].name != 0; i++ )
        {
#if _MSC_VER >= 1200
#pragma warning( disable: 4054 4055 )   /* converting pointers to code<->data */
#endif
            if( ipp_func_desc[i].loaded )
                continue;
            char name_ippi[100];
            char name_ippm[100];

            strcpy( name_ippi, "ippi" );
            strcat( name_ippi, ipp_func_desc[i].name + 3 );
            strcpy( name_ippm, "ippm" );
            strcat( name_ippm, ipp_func_desc[i].name + 3 );

            //printf("%s... %s... \n", ipp_func_desc[i].name, name_ippi );

            uchar *addr = (uchar *) GetProcAddress( hdll, name_ippi );

            if( !addr )
            {
                uchar *addr = (uchar *) GetProcAddress( hdll, name_ippm );

                if( !addr )
                {
                    //printf("no\n");
                    //sprintf(msg, "Unable to find function address %s(%s or %s)\n",
                    //             ipp_func_desc[i].name, name_ippi, name_ippm );
                    //fprintf(stderr, "Unable to find function address %s\n",
                    //             ipp_func_desc[i].name, name_ippi, name_ippm );
                    //MessageBox(0, msg, "Error", MB_OK);
                    //abort_program( msg );
                }
                else
                {
                    //printf("yes\n");
                    while( *addr == 0xE9 )
                        addr += 5 + *((int *) (addr + 1));
                    *ipp_func_desc[i].addr = (ipp_func_addr) addr;
                    ipp_func_desc[i].loaded = loaded;
                    number++;
                }
            }
            else
            {
                //printf("yes\n");
                while( *addr == 0xE9 )
                    addr += 5 + *((int *) (addr + 1));
                *ipp_func_desc[i].addr = (ipp_func_addr) addr;
                ipp_func_desc[i].loaded = loaded;
                number++;
            }
#if _MSC_VER >= 1200
#pragma warning( default: 4054 4055 )
#endif
        }
#ifdef PRINT_LOADED_STATUS
        printf( ">>>>>>>>>>>>>>>>>>>>>>>> loaded in %s: <<<<<<<<<<<<<<<<<<<<<<<<<\n",
                dll_name );
        for( i = 0; ipp_func_desc[i].name != 0; i++ )
            if( ipp_func_desc[i].loaded == loaded )
                printf( "%s\n", ipp_func_desc[i].name );
#endif
    }
    return number;
}

CV_IMPL int
cvLoadPrimitives()
{
    icvLoadPrimitives( DLL_NAME1, 0 );
    icvLoadPrimitives( DLL_NAME2, 0 );

#ifdef PRINT_LOADED_STATUS
    printf( ">>>>>>>>>>>>>>>>>>>>>>>> didn't loaded: <<<<<<<<<<<<<<<<<<<<<<<<<\n" );
    for( int i = 0; ipp_func_desc[i].name != 0; i++ )
        if( !ipp_func_desc[i].loaded )
            printf( "%s\n", ipp_func_desc[i].name );
#endif
    return 0;
}

static char *version = __DATE__;

void
cvGetLibraryInfo( char **_version, int *_loaded, char **_dll_name )
{
    if( _version )
        *_version = version;
    if( _loaded )
        *_loaded = loaded;
    if( _dll_name )
        *_dll_name = loaded ? dll_name : 0;
}

static int number = cvLoadPrimitives();


CV_IMPL int
cvFillInetrnalFuncsTable(void* tbl)
{
    int loaded = 0;

    func_desc* table = (func_desc*)tbl;
    assert(table);

    for( int i = 0; table[i].name; i++ )
    {
        int j;
        for( j = 0;
             ipp_func_desc[j].name && strcmp( ipp_func_desc[j].name, table[i].name );
             j++ );
        if( ipp_func_desc[j].name )
        {
            *table[i].addr = *ipp_func_desc[j].addr;
            table[i].loaded = 1;
            loaded++;
        }
    }

    return loaded;
}


/* End of file. */
