/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "precomp.h"
#include "_cvaux.h"


CvGrRunSt* cvAllocateGrRuns( int stringcount, int runcount )
{
    int i = 0;
    CvGrRunSt* strings = (CvGrRunSt*)malloc(stringcount*sizeof(CvGrRunSt));
    for( i = 0; i < stringcount; i++ )
    {
        strings[i].count = 0;
        strings[i].runs = (CvGrRun*)malloc(runcount*sizeof(CvGrRun));
    }

    return strings;
}

void cvReleaseGrRuns( CvGrRunSt* strings, int stringcount )
{
    int i = 0;    
    for( i = 0; i < stringcount; i++ )
    {        
        free(strings[i].runs);
    }
    free(strings);
}

void cvCreateSimpleGradientRuns( IplImage*  src,                          
                                 CvGrRunSt* hRuns, 
                                 CvGrRunSt* vRuns )
{
    int      i, j;
    uchar*   src_data = 0;    
    int      img_step = 0;    
    CvSize   img_size;
    int      count;  
    CvGrRun* runs;                

    cvGetImageRawData( src, &src_data, &img_step, &img_size );        

    //horizontal runs 
    for( i = 0; i < img_size.height; i++ )        
    {   
        count = 0;        
        runs = hRuns[i].runs;            
        //first run
        runs->x1 = 0;
        runs->z1 = src_data[i*img_step];        

        for( j = 1; j < img_size.width; j++ )
        {
            //form run
            runs[count+1].x1 = runs[count].x2 = j;
            runs[count+1].z1 = runs[count].z2 = src_data[i*img_step+j];
            count++;
            
        }//j
        //last run                
        hRuns[i].count = count;
    }//i     

    //vertical runs 
    for( i = 0; i < img_size.width; i++ )        
    {        
        count = 0;        
        runs = vRuns[i].runs;            
        //first run
        runs->x1 = 0;
        runs->z1 = src_data[i];        

        for( j = 1; j < img_size.height; j++ )
        {
            //form run
            runs[count+1].x1 = runs[count].x2 = j;
            runs[count+1].z1 = runs[count].z2 = src_data[i+j*img_step];
            count++;            
        }//j
        //last run                
        vRuns[i].count = count;
    }//i                
}


#define RUN_LEN_2(r)((r.x2-r.x1)*(r.x2-r.x1)+(r.z2-r.z1)*(r.z2-r.z1))
#define RUNS_SMUL(r1,r2)((r1.x2-r1.x1)*(r2.x2-r2.x1)+(r1.z2-r1.z1)*(r2.z2-r2.z1))
#define RUNS_COS_2(r1,r2)((RUNS_SMUL(r1,r2)*RUNS_SMUL(r1,r2))/(double)(RUN_LEN_2(r1)*RUN_LEN_2(r2)))

void cvCreateGradientRunsWithApprox( IplImage*  src,                          
                                     double     intThresh, 
                                     double     cosThresh, 
                                     CvGrRunSt* hRuns, 
                                     CvGrRunSt* vRuns )
{
    int      i, j, x1, x2, h1, h2 = 0;
    uchar*   src_data = 0;
    int      img_step = 0;    
    CvSize   img_size;
    int      currint, count, prev;  
    CvGrRun* runs;            
    int      max, min, maxX = 0, minX = 0;

    cvGetImageRawData( src, &src_data, &img_step, &img_size );        

    //horizontal runs 
    for( i = 0; i < img_size.height; i++ )        
    {        
        currint = h1 = src_data[i*img_step];
        count = 0;
        x1 = 0;
        runs = hRuns[i].runs;    
        max = 0; min = 255;          
        //first run
        runs->x1 = 0;
        runs->z1 = h1;        

        for( j = 1; j < img_size.width; j++ )
        {
            h2 = src_data[i*img_step+j];
            currint += h2;
            x2 = j;
            if( h2 >= max ) 
            {
                maxX = j;
                max  = h2;
            }
            if( h2 <= min )
            {
                minX = j;
                min  = h2;
            }
            if( labs((long)((x2-x1+1)*((float)(h1+h2)*0.5)) - currint ) > intThresh )
            {      
                prev = h2;
                if( maxX > minX )
                {
                    if( maxX == x2 && minX != x1 )
                    {
                        x2 = minX;
                        h2 = min;
                    }
                    else
                    {
                        x2 = maxX;
                        h2 = max;
                    }
                }
                else
                {
                    if( minX == x2 && maxX != x1 )
                    {
                        x2 = maxX;
                        h2 = max;
                    }
                    else
                    {
                        x2 = minX;
                        h2 = min;
                    }
                    
                }                

                //form run
                runs[count].x2 = x2;
                runs[count].z2 = h2;                
                
                x1 = x2;
                h1 = currint = h2;

                //check angle
                if( count > 1 )
                {
                    /*if( x2 <= runs[count].x1 + 2 && labs(h2 - runs[count].z1) <= 2 )
                    {
                        runs[count-1].x2 = runs[count].x2;
                        runs[count-1].z2 = runs[count].z2;
                        count--;
                    }
                    else*/
                    {
                        long prod = RUNS_SMUL(runs[count-1],runs[count]);                    
                        if( prod > 0 )
                        {
                            double angle = prod * prod /
                                (double)(RUN_LEN_2(runs[count-1]) * RUN_LEN_2(runs[count]));
                            if( angle > cosThresh )
                            {
                                runs[count-1].x2 = runs[count].x2;
                                runs[count-1].z2 = runs[count].z2;
                                count--;
                            }        
                        }                    
                    }
                }

                count++;
                runs[count].x1 = x2;
                runs[count].z1 = h2;                

                max = 0;
                min = 255;
            }            
        }//j
        //last run        
        runs[count].x2 = img_size.width - 1;        
        runs[count].z2 = h2;
        hRuns[i].count = count+1;
    }//i     

    //vertical runs 
    for( i = 0; i < img_size.width; i++ )        
    {        
        currint = h1 = src_data[i];
        count = 0;
        x1 = 0;
        runs = vRuns[i].runs;        
        max = 0; min = 255;          
        //first run
        runs->x1 = 0;
        runs->z1 = h1;   
        
        for( j = 1; j < img_size.height; j++ )
        {
            h2 = src_data[i+j*img_step];                     
            currint += h2;            
            x2 = j;
            if( h2 >= max ) 
            {
                maxX = j;
                max  = h2;
            }
            if( h2 <= min )
            {
                minX = j;
                min  = h2;
            }            
            if( labs((long)((x2-x1+1)*((float)(h1+h2)*0.5)) - currint ) > intThresh )
            {   
                prev = h2;
                if( maxX > minX )
                {
                    if( maxX == x2 && minX != x1 )
                    {
                        x2 = minX;
                        h2 = min;
                    }
                    else
                    {
                        x2 = maxX;
                        h2 = max;
                    }
                }
                else
                {
                    if( minX == x2 && maxX != x1 )
                    {
                        x2 = maxX;
                        h2 = max;
                    }
                    else
                    {
                        x2 = minX;
                        h2 = min;
                    }
                    
                }
                
                //form run
                runs[count].x2 = x2;
                runs[count].z2 = h2;                
                                
                x1 = x2;
                h1 = currint = h2;                

                //check angle
                if( count > 1 )
                {
                    /*if( x2 <= runs[count].x1 + 2 && labs(h2 - runs[count].z1) <= 2 )
                    {
                        runs[count-1].x2 = runs[count].x2;
                        runs[count-1].z2 = runs[count].z2;
                        count--;
                    }
                    else*/
                    {
                        long prod = RUNS_SMUL(runs[count-1],runs[count]);                    
                        if( prod > 0 )
                        {
                            double angle = prod * prod /
                                (double)(RUN_LEN_2(runs[count-1]) * RUN_LEN_2(runs[count]));
                            if( angle > cosThresh )
                            {
                                runs[count-1].x2 = runs[count].x2;
                                runs[count-1].z2 = runs[count].z2;
                                count--;
                            }        
                        }                    
                    }
                }

                count++;
                runs[count].x1 = x2;
                runs[count].z1 = h2;                
                
                max = 0;
                min = 255;
            }            
        }//j
        //last run
        runs[count].x1 = x1;
        runs[count].x2 = img_size.width - 1;
        runs[count].z1 = h1;
        runs[count].z2 = h2;                            
        vRuns[i].count = count+1;
    }//i                
}


//approx by deviation and cos^2
void  cvGrRunsApprox( CvGrRunSt* grRuns, 
                       int        stcount,
                       double     thresh,
                       double     cos_thresh )
{
    int      i, j, x1, x2;        
    CvGrRun* runs;    
    double   thresh2 = thresh * thresh;
    long     len, tmp;
    double   h;
    
    for( i = 0; i < stcount; i++ )
    {        
        int count = 0;
        runs = grRuns[i].runs;
        for( x1 = 0; x1 < grRuns[i].count;  )
        {   
            if(x1 < count) runs[x1] = runs[count-1];
            for( x2 = x1 + 1; x2 < grRuns[i].count; x2++ )
            {
                len = (runs[x1].x1 - runs[x2].x2)*(runs[x1].x1 - runs[x2].x2) + 
                           (runs[x1].z1 - runs[x2].z2)*(runs[x1].z1 - runs[x2].z2);
                tmp = (runs[x1].z1 + runs[x2].z2) * (runs[x2].x2 - runs[x1].x1);

                for( j = x1 + 1; j <= x2; j++)
                {   
                    h = (double)( tmp - 
                                 (runs[x1].z1 + runs[j].z1)*(runs[j].x1 - runs[x1].x1) - 
                                 (runs[x2].z2 + runs[j].z1)*(runs[x2].x2 - runs[j].x1) );
                    if( h * h > thresh2 * len ) break;
                }

                if( j != x2 + 1 ) break;
            }
            runs[count].x1 = runs[x1].x1;
            runs[count].z1 = runs[x1].z1;
            if( x2 == grRuns[i].count )
            {                
                runs[count].x2 = runs[x2-1].x2;
                runs[count].z2 = runs[x2-1].z2;
            }
            else
            {
                runs[count].x2 = runs[x2].x1;
                runs[count].z2 = runs[x2].z1;
            }
            x1 = x2;
            //check previous 
            for( j = count - 1; j >= 0; )            
            {
                len = (runs[j].x1 - runs[count].x2)*(runs[j].x1 - runs[count].x2) + 
                           (runs[j].z1 - runs[count].z2)*(runs[j].z1 - runs[count].z2);
                tmp = (runs[j].z1 + runs[count].z2) * (runs[count].x2 - runs[j].x1);

                h = (double)( tmp - 
                                 (runs[j].z1 + runs[count].z1)*(runs[count].x1 - runs[j].x1) - 
                                 (runs[count].z2 + runs[count].z1)*(runs[count].x2 - runs[count].x1) );
                if( h * h <= thresh2 * len )
                {                       
                    runs[j].x2 = runs[count].x2;
                    runs[j].z2 = runs[count].z2;
                    count--;
                    break;
                }
                else
                {   
                    break;
                }
            }
            count++; 
            //ASSERT(count<grRuns[i].count);
        }//x1        

        //additional approximation by cos^2
        grRuns[i].count = count;
        count = 0;
        for( x1 = 0; x1 < grRuns[i].count-1;  )
        {
            for( x2 = x1 + 1; x2 < grRuns[i].count; x2++ )
            {
                double angle = RUNS_COS_2(runs[x1], runs[x2]);
                if( fabs(angle) < cos_thresh )
                {
                    break;
                }                
            }            
            x2--;
            runs[x1].x2 = runs[x2].x2;
            runs[x1].z2 = runs[x2].z2;
            count++;
            x1 = x2+1;
        }
        grRuns[i].count = count;
    }    
}


void  cvGrRunsFiltering( CvGrRunSt* grRuns, 
                         int        stcount,
                         double     thresh )
{
    int      i, j;        
    CvGrRun* runs;    
    double   k;
    
    for( i = 0; i < stcount; i++ )
    {
        int count = 0;
        runs = grRuns[i].runs;                
        for( j = 0; j < grRuns[i].count;  )
        {  
            k = (double)(runs[j].z2 - runs[j].z1)/(runs[j].x2 - runs[j].x1 + 1);            
            //ASSERT(runs[j].x2 - runs[j].x1>0);
            runs[count].x1 = runs[j].x1;
            runs[count].z1 = runs[j].z1;
            if( fabs(k) < thresh )
            {// square                   
                for(; j < grRuns[i].count; j++ )
                {
                    k = (double)(runs[j].z2 - runs[j].z1) / (runs[j].x2 - runs[j].x1 + 1);
                    if( fabs(k) >= thresh) break;
                }                                
            }
            else if( runs[j].z2 - runs[j].z1 > 0 )
            {
                for(; j < grRuns[i].count; j++ )
                {
                    //ASSERT(runs[j].x2 - runs[j].x1>0);
                    k = (double)(runs[j].z2 - runs[j].z1) / (runs[j].x2 - runs[j].x1 + 1);
                    if( runs[j].z2 - runs[j].z1 < 0 || k < thresh ) break;
                }
            }
            else
            {
                for(; j < grRuns[i].count; j++ )
                {
                    //ASSERT(runs[j].x2 - runs[j].x1>0);
                    k = (double)(runs[j].z1 - runs[j].z2) / (runs[j].x2 - runs[j].x1 + 1);
                    if( runs[j].z2 - runs[j].z1 > 0 || k < thresh ) break;
                }
            }
            if( j < grRuns[i].count )
            {
                runs[count].x2 = runs[j].x1;
                runs[count].z2 = runs[j].z1;
            }
            else
            {
                runs[count].x2 = runs[grRuns[i].count-1].x2;
                runs[count].z2 = runs[grRuns[i].count-1].z2;
            }
            //ASSERT(count<j);
            count++;
        }//j
        grRuns[i].count = count;
    }
}

void ImgWithFrame(IplImage* img)
{
    int     x,y;
    int     height, width;
    uchar   *src_data = 0;
    int     step = 0;
    CvSize  img_size;


    cvGetImageRawData( img, &src_data, &step, &img_size );        
    width = img_size.width;
    height = img_size.height;    

    for( x = 0; x < width; x++ )
    { 
        src_data[x] = 255;
        src_data[step*(height-1)+x] = 255;
    }
    for( y = 0; y < height; y++ )
    { 
        src_data[step*y] = 255;
        src_data[step*y+width-1] = 255;
    }
}

CvSeq* cvGetSegmentsFromGradientRuns( CvGrRunSt*     hRuns, 
                                      CvGrRunSt*     vRuns,
                                      int            imgH,
                                      int            imgW,
                                      double         thresh,                               
                                      CvMemStorage*  storage,
                                      IplImage*      tmp )
{   
    int      i, j, x;    
    CvSeq*   first = NULL;    
    uchar*   tmp_data = 0;    
    int      img_step = 0;    
    CvSize   img_size;                       

    cvGetImageRawData( tmp, &tmp_data, &img_step, &img_size );            

    for( i = 0; i < imgH; i++ )
    {        
        for( j = 0; j<hRuns[i].count; j++ )
        {             
            double k = (double)(labs((hRuns[i].runs[j].z2 - hRuns[i].runs[j].z1))) / 
                        (hRuns[i].runs[j].x2 - hRuns[i].runs[j].x1 + 1);                        
            if( k < thresh )  continue;
            for( x = hRuns[i].runs[j].x1; x <= hRuns[i].runs[j].x2; x++  )
            {
                tmp_data[i*img_step+x] = 255;
            }            
        }
    } 

    for( i = 0; i < imgW; i++ )
    {        
        for( int j=0; j<vRuns[i].count; j++ )
        { 
            double k = (double)(labs((vRuns[i].runs[j].z2 - vRuns[i].runs[j].z1))) / 
                        (vRuns[i].runs[j].x2 - vRuns[i].runs[j].x1 + 1);                        
            if( k < thresh ) continue;
            for( x = vRuns[i].runs[j].x1; x <= vRuns[i].runs[j].x2; x++  )
            {
                tmp_data[i+x*img_step] = 255;
            }
        }
    }     

    ImgWithFrame(tmp);

    cvFindContours( tmp, storage, &first, sizeof(CvContour), 
                         CV_RETR_LIST, CV_CHAIN_APPROX_SIMPLE );    

    return first;
}
