/*M///////////////////////////////////////////////////////////////////////////////////////
//
//  IMPORTANT: READ BEFORE DOWNLOADING, COPYING, INSTALLING OR USING.
//
//  By downloading, copying, installing or using the software you agree to this license.
//  If you do not agree to this license, do not download, install,
//  copy or use the software.
//
//
//                        Intel License Agreement
//                For Open Source Computer Vision Library
//
// Copyright (C) 2000, Intel Corporation, all rights reserved.
// Third party copyrights are property of their respective owners.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
//   * Redistribution's of source code must retain the above copyright notice,
//     this list of conditions and the following disclaimer.
//
//   * Redistribution's in binary form must reproduce the above copyright notice,
//     this list of conditions and the following disclaimer in the documentation
//     and/or other materials provided with the distribution.
//
//   * The name of Intel Corporation may not be used to endorse or promote products
//     derived from this software without specific prior written permission.
//
// This software is provided by the copyright holders and contributors "as is" and
// any express or implied warranties, including, but not limited to, the implied
// warranties of merchantability and fitness for a particular purpose are disclaimed.
// In no event shall the Intel Corporation or contributors be liable for any direct,
// indirect, incidental, special, exemplary, or consequential damages
// (including, but not limited to, procurement of substitute goods or services;
// loss of use, data, or profits; or business interruption) however caused
// and on any theory of liability, whether in contract, strict liability,
// or tort (including negligence or otherwise) arising in any way out of
// the use of this software, even if advised of the possibility of such damage.
//
//M*/

#include "precomp.h"
#include "_cvaux.h"

#include <float.h>
#include "_cvutils.h"
#include "_cvwrap.h"

CV_IMPL void icvInitSeqTreeIterator( CvSeqTreeIterator* seqIterator,
                             CvSeq* first, int maxLevel )
{
    seqIterator->seq = first;
    seqIterator->level = 0;
    seqIterator->maxLevel = maxLevel;
}


CV_IMPL CvSeq* icvNextSeq( CvSeqTreeIterator* seqIterator )
{
    CvSeq* seq = seqIterator->seq;
    CvSeq* prevSeq = seq;
    int level = seqIterator->level;
    
    if( seq )
    {
        if( seq->v_next && level < seqIterator->maxLevel )
        {
            seq = seq->v_next;
            level++;
        }
        else
        {
            while( seq->h_next == 0 )
            {
                seq = seq->v_prev;
                if( --level < 0 )
                {
                    seq = 0;
                    break;
                }
            }
            if( seq )
                seq = seq->h_next;
        }
    }

    seqIterator->seq = seq;
    seqIterator->level = level;

    return prevSeq;
}


CV_IMPL CvSeq* icvPrevSeq( CvSeqTreeIterator* seqIterator )
{
    CvSeq* seq = seqIterator->seq;
    CvSeq* prevSeq = seq;
    int level = seqIterator->level;
    
    if( seq )
    {
        if( !seq->h_prev )
        {
            seq = seq->v_prev;
            if( --level < 0 )
                seq = 0;
        }
        else
        {
            seq = seq->h_prev;

            while( seq->v_next && level < seqIterator->maxLevel )
            {
                seq = seq->v_next;
                level++;

                while( seq->h_next )
                    seq = seq->h_next;
            }
        }
    }

    seqIterator->seq = seq;
    seqIterator->level = level;

    return prevSeq;
}

/* End of file. */
