#include "stdafx.h"

#include "vplatform.h"
#ifdef VASSILIS_SGI_PLATFORM
#include <iostream>
#else 
#include <iostream>
#endif
#include <math.h>

#include "basics/image.h"
#include "basics/drawing.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "gestures/algorithms.h"
#include "gestures/algo_templates.h"
#include "basics/drawing_temp.h"
#include "local_data.h"
#include "opencv_utils.h"
#include "opencv_utils2.h"
#include "basics/precomputed.h"
#include "gestures/contour.h"

#include "basics/definitions.h" 




long vPoint3::Print(const char * text)
{
  vPrint("%s", text);
  vPrint("(%.2lf, %.2lf, %.2lf)\n", x, y, z);
  return 1;
}


long vPoint3f::Print(const char * text)
{
  vPrint("%s", text);
  vPrint("(%.2f, %.2f, %.2f)\n", x, y, z);
  return 1;
}



// Below this line starts the BreadthFirstSearch implementation.

// BreadthFirstSearch is the function that performs the breadth first search. 
// It is very simple, it just provides a framework. The caller needs to supply
// three arguments that specify how the search will work, what space it will
// run on and what it's looking for.
//
// The way it is specified, it allows for easy implementations, in which 
// the procedure (i.e. the function specified by the second argument)
// just looks at the passed node, does something to it, and
// adds its neighbors to the to_visit list. However, the procedure in 
// general is free to do whatever it wants with the node and the to_visit
// list. It is free to visit other nodes in the to_visit list, it is free
// to even complete the whole search before returning. Such implementations
// will be more efficient, since they will avoid the overhead of a function call
// per visited node.
void BreadthFirstSearch(vNodeSet * to_visit, BreadthFirstProc procedure, 
			vector<void *> * procedure_args)
{
  while(1)
  {
    if (to_visit->IsEmpty()) return;
    void * node = to_visit->Pop();
    procedure(node, to_visit, procedure_args);
  }
}


vNodeList::vNodeList()
{
}


vNodeList::~vNodeList()
{
}


ushort vNodeList::IsEmpty()
{
  return empty();
}


void * vNodeList::Pop()
{
  void * first = front();
  pop_front();
  return first;
}


void GridBreadthFirstProc(void * nodep, vNodeSet * to_visit_set,
			                    vector<void *> * procedure_args)
{
  // Convert the arguments to the appropriate types for this function
  // The grid is the space on which we make the search.
  vSearchGrid * grid = (vSearchGrid *) (*procedure_args)[0];

  // This is the set of nodes that should be visited. A node is 
  // "visited" when we call GridBreadthFirstProc with that node as
  // the nodep argument.
  vNodeList & to_visit = *((vNodeList *) to_visit_set);

  // This is the last point that we have updated.
  vPoint * point = (vPoint *) nodep;
  long row = point->row;
  long col = point->col;
  vdelete(point);

  vArray2(long) distances = grid->distances;

  // We will update the neighbors of the last point
  // we updated. If the distance to that point (from
  // whatever initial seed) was x, the distance to the
  // neighbors from that seed is at most x+1 (if the shortest
  // path goes through the previous point).
  long distance = distances[row][col];
  long next_distance = distance + 1;
  long new_row, new_col;

  // Check each of the four neighbors of the pixel to see if
  // they need to be updated. If the distance stored in them is
  // larger than next_distance, we update their distance and put
  // them in the to_visit list.
  new_row = row-1;
  new_col = col;
  if (distances[new_row][new_col] > next_distance)
  {
    distances[new_row][new_col] = next_distance;
    to_visit.push_back(new vPoint(new_row, new_col));
  }

  new_row = row+1;
  new_col = col;
  if (distances[new_row][new_col] > next_distance)
  {
    distances[new_row][new_col] = next_distance;
    to_visit.push_back(new vPoint(new_row, new_col));
  }

  new_row = row;
  new_col = col-1;
  if (distances[new_row][new_col] > next_distance)
  {
    distances[new_row][new_col] = next_distance;
    to_visit.push_back(new vPoint(new_row, new_col));
  }

  new_row = row;
  new_col = col+1;
  if (distances[new_row][new_col] > next_distance)
  {
    distances[new_row][new_col] = next_distance;
    to_visit.push_back(new vPoint(new_row, new_col));
  }
}


// Beginning of implementation of class vSearchGrid

// Note the trick I'm doing here: In general, to make the breadth
// first search more efficient. Basically, I make it possible to
// access something like distances[-1][-1], and I initialize all
// border values of distances to -1.  This way, breadth first search
// doesn't have to check for every pixel to see if its neighbors are
// inside the grid. Since it will never visit the border pixels, it
// will always stay inside the grid.

vSearchGrid::vSearchGrid(long in_rows, long in_cols) :
  v3dMatrix<long>(in_rows + 2, in_cols + 2, 1)
{
  grid_rows = in_rows;
  grid_cols = in_cols;
  distance_array = Matrix2(0);
  
  temp_array = vnew(vArray(long), rows);
  long i;
  for (i = 0; i < rows; i++)
    temp_array[i] = distance_array[i] + 1;
  
  distances = temp_array + 1;
}


vSearchGrid::~vSearchGrid()
{
  vdelete2(temp_array);
}
  

// Returns the number of pixels in distances where the two grids
// differ. It only looks into the distances array, it doesn't look
// at the padding (which is accessible via distance_array);
long vSearchGrid::Compare(vSearchGrid * grid1, vSearchGrid * grid2)
{
  long grid_rows = grid1->grid_rows;
  long grid_cols = grid1->grid_cols;
  if (grid_rows != grid2->grid_rows) return -1;
  if (grid_cols != grid2->grid_cols) return -1;
  vArray2(long) distances1 = grid1->distances;
  vArray2(long) distances2 = grid2->distances;

  long result = 0;
  long row, col;
  for (row = 0; row < grid_rows; row++)
  {
    for (col = 0; col < grid_cols; col++)
    {
      if (distances1[row][col] != distances2[row][col])
      {
        result++;
      }
    }
  }
  return result;
}

void vSearchGrid::WriteBorder(long value)
{
  long row, col;
  for (row = 0; row < rows; row++)
  {
    distance_array[row][0] = value;
    distance_array[row][cols-1] = value;
  }
  for (col = 0; col < cols; col++)
  {
    distance_array[0][col] = value;
    distance_array[rows-1][col] = value;
  }
}


void vSearchGrid::WriteInterior(long value)
{
  long row, col;
  for (row = 1; row < rows-1; row++)
    for (col = 1; col < cols-1; col++)
      distance_array[row][col] = value;
}


// Makes every cell of the grid contain the block metric distance
// of that cell to the center that is specified here.
void vSearchGrid::SetCenter(ushort r, ushort g)
{
  long row, col;
  for (row = 0; row < grid_rows; row++)
		for (col = 0; col < grid_cols; col++)
		{
			double distance = abs(row - r) + abs(col - g);
			distances[row][col] = (long) distance;
		}

	WriteBorder(255);
}


// End of implementation of class vSearchGrid


// Beginning of implementation of class vSearchGridDouble

// Note the trick I'm doing here: In general, to make the breadth
// first search more efficient. Basically, I make it possible to
// access something like distances[-1][-1], and I initialize all
// border values of distances to -1.  This way, breadth first search
// doesn't have to check for every pixel to see if its neighbors are
// inside the grid. Since it will never visit the border pixels, it
// will always stay inside the grid.

vSearchGridDouble::vSearchGridDouble(long in_rows, long in_cols) :
  v3dMatrix<double>(in_rows + 2, in_cols + 2, 1)
{
  grid_rows = in_rows;
  grid_cols = in_cols;
  distance_array = Matrix2(0);
  
  temp_array = vnew(vArray(double), rows);
  long i;
  for (i = 0; i < rows; i++)
    temp_array[i] = distance_array[i] + 1;
  
  distances = temp_array + 1;
}


vSearchGridDouble::~vSearchGridDouble()
{
  vdelete2(temp_array);
}
  

// Returns the number of pixels in distances where the two grids
// differ. It only looks into the distances array, it doesn't look
// at the padding (which is accessible via distance_array);
long vSearchGridDouble::Compare(vSearchGridDouble * grid1, vSearchGridDouble * grid2)
{
  long grid_rows = grid1->grid_rows;
  long grid_cols = grid1->grid_cols;
  if (grid_rows != grid2->grid_rows) return -1;
  if (grid_cols != grid2->grid_cols) return -1;
  vArray2(double) distances1 = grid1->distances;
  vArray2(double) distances2 = grid2->distances;

  long result = 0;
  long row, col;
  for (row = 0; row < grid_rows; row++)
  {
    for (col = 0; col < grid_cols; col++)
    {
      if (distances1[row][col] != distances2[row][col])
      {
        result++;
      }
    }
  }
  return result;
}

void vSearchGridDouble::WriteBorder(double value)
{
  long row, col;
  for (row = 0; row < rows; row++)
  {
    distance_array[row][0] = value;
    distance_array[row][cols-1] = value;
  }
  for (col = 0; col < cols; col++)
  {
    distance_array[0][col] = value;
    distance_array[rows-1][col] = value;
  }
}


void vSearchGridDouble::WriteInterior(double value)
{
  long row, col;
  for (row = 1; row < rows-1; row++)
    for (col = 1; col < cols-1; col++)
      distance_array[row][col] = value;
}


// End of implementation of class vSearchGridDouble


// This is the end of the implementation of breadth first search.

v3dMatrix<long> * vGridDistances(color_image * the_image, vSearchGrid * grid)
{
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = the_image->Size();
  v3dMatrix<long> * result = new v3dMatrix<long>(rows, cols, 1);
  vArray(long) matrix = result->Matrix(0);
  vArray2(long) distances = grid->distances;
  
  long i;
  for (i = 0; i < size; i++)
  {
    long r = R[i];
    long g = G[i];
    long b = B[i];
    double intensity = r + g + b;
    uchar r_normal = (uchar) (r * 255 / intensity);
    uchar g_normal = (uchar) (g * 255 /intensity);
    matrix[i] = distances[r_normal][g_normal];
  }
  return result;
}


void vAverageLabelColor(color_image * source, v3dMatrix<long> * labels,
			 Label & label, uchar & r, uchar & g, uchar &
			 b)
{
  long row, col;
  vArray2(long) pixel_labels = labels->Matrix2(0);
  vArray2(uchar) R = source->GetR2();
  vArray2(uchar) G = source->GetG2();
  vArray2(uchar) B = source->GetB2();
  long id = label.id;

  long r_sum = 0;
  long g_sum = 0;
  long b_sum = 0;
  long count = label.count;

  for (row = label.top; row <= label.bottom; row++)
    for (col = label.left; col <= label.right; col++)
    {
      long pixel_label = pixel_labels[row][col];
      if (pixel_label == id)
      {
	r_sum += R[row][col];
	g_sum += G[row][col];
	b_sum += B[row][col];
      }
    }

  r = (uchar) (r_sum / count);
  g = (uchar) (g_sum / count);
  b = (uchar) (b_sum / count);
}


v3dMatrix<long> * vColorDistances(color_image * the_image, 
				     uchar r, uchar g, uchar b)
{
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = the_image->Size();
  v3dMatrix<long> * result = new v3dMatrix<long>(rows, cols, 1);
  vArray(long) matrix = result->Matrix(0);
  
  long i;
  for (i = 0; i < size; i++)
  {
    uchar r1 = R[i];
    uchar g1 = G[i];
    uchar b1 = B[i];
    matrix[i] = abs(r - r1) + abs(g - g1) + abs(b - b1);
  }
  return result;  
}

#ifndef VASSILIS_LINUX_PLATFORM
long vFindFaces(color_image * the_image, FaceLocation ** locations)
{
  vArray(uchar) buffer = vImageToRGB(the_image);
//  long faces_found = Informedia_FindFaces(the_image->Cols(), the_image->Rows(), 
//					  vData(buffer), locations);
  long faces_found  = 0;
  vdelete2(buffer);
  return faces_found;
}
#endif 



void vReadFaceLocation(vBoundingBox * box, FaceLocation & loc)
{
  box->top = loc.y1;
  box->bottom = loc.y2;
  box->left = loc.x1;
  box->right = loc.x2;
}


// This function just loads up a specific image file where we 
// encode the adjacency list for an articulated hand, as 
// defined in hand_renderer.cpp. By saving that list as an 
// image file and loading it up here, we avoid the need of 
// having to include all the VirtualHand related files and  
// libraries in a project. This is even more important if we
// consider the fact that the VirtualHand library only works
// on windows, and needs to be installed.

v3dMatrix<ushort> * vLoadFingerAdjacencies()
{
  char * adjacencies_file = vJoinPaths(sequences_directory, 
                                        "glove_data\\training\\neighbors.bin");
  v3dMatrix<ushort> * adjacencies = v3dMatrix<ushort>::Read(adjacencies_file);
  vdelete2(adjacencies_file);
  return adjacencies;
}


// vDepthNeighbors is very similar to vMaxNeighbors (applied to
// depth), but it assumes that the pixels belong to specific links,
// as specified by the links argument. For background pixels we write
// 0 to the result. For a pixel in a link A, we consider only neighbors
// belonging also to link A.
v3dMatrix<float> * 
vDepthNeighbors(v3dMatrix<uchar> * link_image, v3dMatrix<float> * depth,
                 v3dMatrix<ushort> * adjacencies)
{
  vArray2(uchar) links = link_image->Matrix2(0);
  vArray2(ushort) neighbors = adjacencies->Matrix2(0);

  long rows = depth->Rows();
  long cols = depth->Cols();
  long bottom = rows - 1;
  long right = cols - 1;

  // Create the result image
  v3dMatrix<float> * result = new v3dMatrix<float>(rows, cols, 1);
  // Write zeros at the borders
  vWriteBorderValue(result, (float) 0);
  vArray2(float) matrix1 = depth->Matrix2(0);
  vArray2(float) matrix2 = result->Matrix2(0);
  
  long row, col;
  for (row = 1; row < bottom; row++)
  {
    for (col = 1; col < right; col++)
    {
      // The encoding I use for link IDs is that they are guaranteed
      // to be the pixel value of the first band % 70 (see 
      // hand_renderer.cpp).
      long center_id = links[row][col] % 70;
      if (center_id == 0) 
      {
        matrix2[row][col] = 0;
        continue;
      }
      
      float center = matrix1[row][col];
      // Take care of the case where the pixel is a background pixel
      float max_distance = 0;

      long next_id;
      float next;

      // Check top pixel
      next_id = links[row-1][col] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row-1][col];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check bottom pixel
      next_id = links[row+1][col] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row+1][col];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check left pixel
      next_id = links[row][col-1] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row][col-1];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      // Check right pixel
      next_id = links[row][col+1] % 70;
      if (next_id == center_id)
      {
        next = matrix1[row][col+1];
        float distance = vAbs(next - center);
        if (vAbs(distance) > max_distance)
        {
          max_distance = distance;
        }
      }

      matrix2[row][col] = max_distance;
    }
  }
  return result;
}



// See description of the function in the DESCRIPTION.TXT file
// of the interface project.
grayscale_image * 
vArticulatedEdges(v3dMatrix<uchar> * object, v3dMatrix<float> * depth,
                   v3dMatrix<ushort> * adjacencies)
{
  // First, we need to figure out the threshold for depth discrepancies
  // that we will use to decide if an edge between two pixels of neighboring
  // links should be recorded as an edge or not.

  // Get an image that has, for each pixel, the max neighbor value for
  // that pixel among neighboring pixels of the same link.
  v3dMatrix<float> * max_depths = vDepthNeighbors(object, depth, adjacencies);

  long number_of_bins = 100;
  double lower = 0.0000001;
  double upper = 0.9999999;
  vMatrix<double> * histogram = vBoundedRankHistogram(max_depths, number_of_bins, 
                                                        lower, upper);
  // We choose as threshold the value such that 2% of the hand pixels have 
  // a higher value in max_depths;
  double threshold = (*histogram)(number_of_bins - 3, 0);

  vdelete(max_depths);
  vdelete(histogram);

  vArray2(float) depth_values = depth->Matrix2(0);
  
  long rows = object->Rows();
  long cols = object->Cols();
  grayscale_image * result = new grayscale_image(rows, cols);
  vArray2(ushort) neighbors = adjacencies->Matrix2(0);

  // red will be the red band for a 3-band image, but it will simply
  // be the intensity band for a one-band (grayscale) image.
  vArray2(uchar) red = object->Matrix2(0);
  vArray2(uchar) edges = result->GetI2();

  long row, col;
  vWriteBorderValue(result, (uchar) 0);
  
  // Go through the object image and decide, for each pixel, if it is 
  // an edge.
  for (row = 1; row < rows - 1; row++)
  {
    for (col = 1; col < cols - 1; col++)
    {
      // Read the values of the center and the 4 neighbors.
      uchar center = red[row][col];
      if (center == 0) 
      {
        edges[row][col] = 0;
        continue;
      }
      // The id of the link of the center is obtained by
      // taking the remainder of dividing center by 70.
      uchar center_id = center % 70;
      long near_rows[4];
      long near_cols[4];
      near_rows[0] = row;
      near_cols[0] = col-1;
      near_rows[1] = row;
      near_cols[1] = col+1;
      near_rows[2] = row-1;
      near_cols[2] = col;
      near_rows[3] = row+1;
      near_cols[3] = col;
      double depth_difference;

      // Now check if center is a "finger edge" pixel. Compare
      // center to each neighbor.
      long i;
      long edge_flag = 0;
      for (i = 0; i < 4; i++)
      {
        long near_row = near_rows[i];
        long near_col = near_cols[i];
        uchar around = red[near_row][near_col];

        // If neighbor is same as center, then no edge has been
        // found so far.
        if (center == around) continue;
        {
          // If neighbor is not same as center, check if neighbor
          // belongs to an anatomically neighboring link. If not,
          // then center is a "finger edge".
          uchar neighbor_id = around % 70;
          double depth1 = depth_values[row][col];
          double depth2 = depth_values[near_row][near_col];
          depth_difference = depth2 - depth1;
          if (neighbors[center_id][neighbor_id] == 1)
          {
            // If we define as difference vAbs(depth1 - depth2), then 
            // for each pair of adjacent pixels that belong to different
            // links we get two edge pixels. By considering the 
            // difference depth2 - depth1 we only get one edge pixel,
            // the one whose depth is lower (nearer to the image).
            // double depth_difference = vAbs(depth1 - depth2);
            if (depth_difference >= threshold)
            {
              edge_flag = 1;
              break;
            }
          }
          else
          {
            if (depth_difference >= 0)
            {
              edge_flag = 1;
              break;
            }
          }
        }
      }

      if (edge_flag == 1)
      {
        // Mark the pixel with the id of the center.
        edges[row][col] = center_id;
      }
      else 
      {
        // center is not an edge, we mark the pixel with 0.
        edges[row][col] = 0;
      }
    }
  }
  return result;
}
                   
                   
// This is a simpler version of vArticulatedEdges, where we don't consider
// depth information at all.
grayscale_image * 
vSimpleArticulatedEdges(v3dMatrix<uchar> * object, 
                         v3dMatrix<ushort> * adjacencies)
{
  long rows = object->Rows();
  long cols = object->Cols();
  grayscale_image * result = new grayscale_image(rows, cols);
  vArray2(ushort) neighbors = adjacencies->Matrix2(0);

  // red will be the red band for a 3-band image, but it will simply
  // be the intensity band for a one-band (grayscale) image.
  vArray2(uchar) red = object->Matrix2(0);
  vArray2(uchar) edges = result->GetI2();

  long row, col;
  vWriteBorderValue(result, (uchar) 0);
  
  for (row = 1; row < rows - 1; row++)
  {
    for (col = 1; col < cols - 1; col++)
    {
      // Read the values of the center and the 4 neighbors.
      uchar center = red[row][col];
      if (center == 0) 
      {
        edges[row][col] = 0;
        continue;
      }
      // The id of the link of the center is obtained by
      // taking the remainder of dividing center by 70.
      uchar center_id = center % 70;
      uchar around[4];
      around[0] = red[row][col-1];
      around[1] = red[row][col+1];
      around[2] = red[row-1][col];
      around[3] = red[row+1][col];
 
      // Now check if center is a "finger edge" pixel. Compare
      // center to each neighbor.
      long i;
      long edge_flag = 0;
      for (i = 0; i < 4; i++)
      {
        // If neighbor is same as center, then no edge has been
        // found so far.
        if (center == around[i]) continue;
        {
          // If neighbor is not same as center, check if neighbor
          // belongs to an anatomically neighboring link. If not,
          // then center is a "finger edge".
          uchar neighbor_id = around[i] % 70;
          if (neighbors[center_id][neighbor_id] == 1) continue;
          else
          {
            edge_flag = 1;
            break;
          }
        }
      }

      if (edge_flag == 1)
      {
        // We have a finger edge. Mark it with the id of the center.
        edges[row][col] = center_id;
      }
      else 
      {
        // center is not an edge, we mark the pixel with 0.
        edges[row][col] = 0;
      }
    }
  }
  return result;
}  


// Applies vArticulatedEdges to every frame of links and depths, and
// saves the results to the RLE sequence named by output.
ushort vSequenceArticulatedEdges(vVideoFile * links, class_image_stream * depths,
                                  const char * output)
{
  long rows = links->Rows();
  long cols = links->Cols();

  if (rows != depths->Rows()) return 0;
  if (cols != depths->Cols()) return 0;
  if (depths->ImageType() != FloatType) return 0;

  v3dMatrix<ushort> * adjacencies = vLoadFingerAdjacencies();
  if (adjacencies == 0) return 0;

  vRLE_Sequence edges(output, rows, cols);
  if (edges.IsValid() == 0)
  {
    vPrint("Can't open %s\n", output);
    return 0;
  }

  long frame = 0;
  ushort result = 1;
  vPrint("\n");
  while(1)
  {
    color_image * input = links->ReadFrame(frame);
    general_image * temp_depth = depths->ReadImage(frame);

    if (input == 0)
    {
      vdelete(temp_depth);
      break;
    }
    if (temp_depth == 0)
    {
      vdelete(input);
      break;
    }

    if (temp_depth->TypeName() != FloatType)
    {
      // This should never happen.
      assert(0);
    }

    v3dMatrix<float> * depth = (v3dMatrix<float> *) temp_depth;

    grayscale_image * edge_image = vArticulatedEdges(input, depth, adjacencies);
    vdelete(input);
    vdelete(depth);
    if (edge_image == 0)
    {
      vPrint("Error in getting edge image for frame %li\n", frame);
      result = 0;
    }

    ushort success = edges.write_frameGray2(edge_image, 1, 0);
    if (success == 0)
    {
      vPrint("Error in writing edge image %li\n", frame);
      result = 0;
    }

    vdelete(edge_image);
    // If there was a failure in processing the current frame, break;
    if (result == 0) break;

    vPrint("processed frame %li\r", frame);
    frame++;
  }

  vPrint("\n");
  vdelete(adjacencies);
  ushort success = edges.FinishWriting();
  if (success == 0)
  {
    vPrint("Error in closing sequence %s\n", output);
    result = 0;
  }

  if (result == 0)
  {
    vPrint("Errors have occurred in saving articulated edges\n");
  }

  return result;
}




// S_DistanceNode is used to represent nodes for the distance transform 
// computation (which is implemented as breadth-first search).
class S_DistanceNode
{
public:
  // row, col are the coordinates of the pixel that corresponds to the node
  // (i.e. the coordinates of the current location that the search is 
  // focusing on)
  long row, col;

  // edge_row, edge_col are the coordinates of an edge pixel. To process this
  // node, we will compute the distance of (row, col) from (edge_row, edge_col),
  // and check if it is less than the smallest distance found so far between 
  // (row, col) and an edge pixel. If it is less than that, we enter nodes
  // in the to_visit list for the 8 neighbors of (row, col), using the 
  // same edge_row and edge_col.
  long edge_row, edge_col;

  S_DistanceNode(long in_row, long in_col, 
                 long in_edge_row, long in_edge_col)
  {
    row = in_row;
    col = in_col;
    edge_row = in_edge_row;
    edge_col = in_edge_col;
  }
};


void vDistanceTransformProcSlow(void * void_node, vNodeSet * to_visit_set, 
                                 vector<void *> * procedure_args)
{
  vSearchGrid * grid = (vSearchGrid *) (*procedure_args)[0];
  vNodeList * to_visit = (vNodeList *)  to_visit_set;
  vArray2(long) distances = grid->distances;
  S_DistanceNode * node = (S_DistanceNode *) void_node;
  long row = node->row;
  long col = node->col;
  long edge_row = node->edge_row;
  long edge_col = node->edge_col;

  long row_diff = row - edge_row;
  long col_diff = col - edge_col;
  long square_distance = row_diff * row_diff + col_diff * col_diff;
  if (square_distance < distances[row][col])
  {
    distances[row][col] = square_distance;
    to_visit->push_back(new S_DistanceNode(row-1, col-1, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row, col-1, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row+1, col-1, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row-1, col, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row+1, col, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row-1, col+1, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row, col+1, edge_row, edge_col));
    to_visit->push_back(new S_DistanceNode(row+1, col+1, edge_row, edge_col));
  }
  vdelete(node);
}


void vDistanceTransformProc(void * void_node, vNodeSet * to_visit_set, 
                             vector<void *> * procedure_args)
{
  vSearchGrid * grid = (vSearchGrid *) (*procedure_args)[0];
  vArray2(long) distances = grid->distances;
  vNodeList * to_visit = (vNodeList *)  to_visit_set;
  S_DistanceNode * node = (S_DistanceNode *) void_node;
  while(1)
  {
    long row = node->row;
    long col = node->col;
    long edge_row = node->edge_row;
    long edge_col = node->edge_col;

    long row_diff = row - edge_row;
    long col_diff = col - edge_col;
    long square_distance = row_diff * row_diff + col_diff * col_diff;
//    double distance = sqrt(square_distance);
    if (square_distance < distances[row][col])
    {
      distances[row][col] = square_distance;
      to_visit->push_back(new S_DistanceNode(row-1, col-1, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row, col-1, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row+1, col-1, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row-1, col, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row+1, col, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row-1, col+1, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row, col+1, edge_row, edge_col));
      to_visit->push_back(new S_DistanceNode(row+1, col+1, edge_row, edge_col));
    }
   vdelete(node);
   if (to_visit->IsEmpty()) break;
   node = (S_DistanceNode *) (to_visit->Pop());
  }
}


v3dMatrix<long> * S_DistanceTransform(general_image * the_image_base, ushort fast)
{
  grayscale_image * the_image = the_image_base->ToGrayBound();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vSearchGrid * grid = new vSearchGrid(rows, cols);
  grid->WriteBorder(-1);
  long initial_value_root = 2 * (rows + cols);
  long initial_value = initial_value_root * initial_value_root;
  grid->WriteInterior(initial_value);
  vArray2(uchar) edges = the_image->Matrix2(0);
  vArray2(long) distances = grid->distances;
  vNodeList to_visit;
  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      if (edges[row][col] != 0)
      {
        to_visit.push_back(new S_DistanceNode(row, col, row, col));
      }
    }
  }
  vector<voidp> search_args;
  search_args.push_back(grid);

  if (fast == 0)
  {
    BreadthFirstSearch(&to_visit, vDistanceTransformProcSlow, &search_args);
  }
  else
  {
    BreadthFirstSearch(&to_visit, vDistanceTransformProc, &search_args);
  }

  for (row = 0; row < rows; row++)
  {
    vArray(long) distances1 = distances[row];
    for (col = 0; col < cols; col++)
    {
      long distance = distances1[col];
      distances1[col] = (long) sqrt((float) distance);
    }
  }
  if (the_image != the_image_base) 
  {
    vdelete(the_image);
  }
  return grid;
}


v3dMatrix<long> * vDistanceTransformSlow(general_image * the_image_base)
{
  return S_DistanceTransform(the_image_base, 0);
}


v3dMatrix<long> * vDistanceTransform(general_image * the_image_base)
{
  return S_DistanceTransform(the_image_base, 1);
}


double vSetDistance(vector<long> * rowsp, vector<long> * colsp, 
                     v3dMatrix<float> * distances_image)
{
  long size = rowsp->size();
  long rows = distances_image->Rows();
  long cols = distances_image->Cols();
  if (size == 0) return rows * rows + cols * cols;
  long i;
  vArray2(float) distances = distances_image->Matrix2(0);

  double result = 0;
  for (i = 0; i < size; i++)
  {
    long row = (*rowsp)[i];
    long col = (*colsp)[i];
    if (row < 0)
    {
      result = result - row;
      row = 0;
    }
    else if (row >= rows)
    {
      result = result + (row - rows - 1);
      row = rows - 1;
    }
      
    if (col < 0)
    {
      result = result - col;
      col = 0;
    }
    else if (col >= cols)
    {
      result = result + (col - cols - 1);
      col = cols - 1;
    }

    result = result + distances[row][col];
  }

  result = result / size;
  return result;
}


ushort vTranslatePoints(vector<long> * source_rowsp, vector<long> * source_colsp,
                         vector<long> * target_rowsp, vector<long> * target_colsp,
                         long translation_rows, long translation_cols)
{
  vector<long> & source_rows = *source_rowsp;
  vector<long> & source_cols = *source_colsp;
  vector<long> & target_rows = *target_rowsp;
  vector<long> & target_cols = *target_colsp;

  long size = source_rows.size();
  long i;
  for (i = 0; i < size; i++)
  {
    target_rows[i] = source_rows[i] + translation_rows;
    target_cols[i] = source_cols[i] + translation_cols;
  }
  return 1;
}


ushort vScalePoints(vector<long> * source_rowsp, vector<long> * source_colsp,
                     vector<long> * target_rowsp, vector<long> * target_colsp,
                     double scaling, long center_row, long center_col)
{
  vector<long> & source_rows = *source_rowsp;
  vector<long> & source_cols = *source_colsp;
  vector<long> & target_rows = *target_rowsp;
  vector<long> & target_cols = *target_colsp;

  long size = source_rows.size();
  long i;
  for (i = 0; i < size; i++)
  {
    long source_row = source_rows[i];
    long source_col = source_cols[i];
    float centered_row = (float) (source_row - center_row);
    float centered_col = (float) (source_col - center_col);
    float scaled_row = centered_row * (float) scaling;
    float scaled_col = centered_col * (float) scaling;
    target_rows[i] = (long) scaled_row + center_row;
    target_cols[i] = (long) scaled_col + center_col;
  }
  return 1;
}



// This function assumes that x and y describe a function from N/x.size()
// to R2. It computes the centered derivative at each point. The last
// point is assumed to be adjacent to the first.
short vCircularDerivative(vector<double> * x, vector<double> * y,
                           vector<double> * dx, vector<double> * dy,
                           long neighborhood)
{
  // Check the lengths of x and y, make sure that dx and dy have the 
  // same length
  long size = x->size();
  if (y->size() != size) return 0;
  dx->erase(dx->begin(), dx->end());
  dy->erase(dy->begin(), dy->end());
  dx->insert(dx->begin(), size, 0.0);
  dy->insert(dy->begin(), size, 0.0);

  long previous, current, next;
  for (current = 0; current < size; current++)
  {
    previous = (current + size - neighborhood) % size;
    next = (current + neighborhood) % size;
//    double dx1 = (*x)[current] - (*x)[previous];
//    double dx2 = (*x)[next] - (*x)[current];
//    (*dx)[current] = (dx1 + dx2) / 2.0;

//    double dy1 = (*y)[current] - (*y)[previous];
//    double dy2 = (*y)[next] - (*y)[current];
//    (*dy)[current] = (dy1 + dy2) / 2.0;
    (*dx)[current] = ((*x)[next] - (*x)[previous]) / 2.0;
    (*dy)[current] = ((*y)[next] - (*y)[previous]) / 2.0;

  }

  return 1;
}


// At the end of the function, (*out_norm)[i] = norm((*x)[i], (*y)[i]).
short vMagnitudes(std::vector<double> * x, std::vector<double> * y,
                   std::vector<double> * out_norm)
{
  // Check the size of y and set the size of out_norm
  long size = x->size();
  if (y->size() != size) return 0;
  out_norm->erase(out_norm->begin(), out_norm->end());
  out_norm->insert(out_norm->begin(), size, 0.0);

  long i;
  for (i = 0; i < size; i++)
  {
    double x1 = (*x)[i];
    double y1 = (*y)[i];
    double square_norm = x1 * x1 + y1 * y1;
    double norm = sqrt(square_norm);
    (*out_norm)[i] = norm;
  }
  return 1;
}


// vLocalMaxima takes in a vector of values, and stores the values of 
// the local maxima of the input into maxima, and the corresponding
// indices for those maxima into indices. It assumes that the last
// point in input is adjacent to the first point in input.
short vLocalMaxima(std::vector<double> * input, 
                    std::vector<double> * maxima, 
                    std::vector<long> * indices)
{
  long size = input->size();
  maxima->erase(maxima->begin(), maxima->end());
  indices->erase(indices->begin(), indices->end());
  long i1;
  for (i1 = 0; i1 < size; i1++)
  {
    long i0 = (i1 + size - 1) % size;
    long i2 = (i1 + 1) % size;
    double previous = (*input)[i0];
    double current = (*input)[i1];
    double next = (*input)[i2];
    if ((current > previous) && (current > next))
    {
      maxima->push_back(current);
      indices->push_back(i1);
    }
  }
  return 1;
}


// At the end of the function, (*orientations)[i] is the angle (in radians)
// between the vector specified by (*x)[i] and (*y)[i] and the vector (1, 0)
// (coordinate notation is (x, y)).
short vOrientations(std::vector<double> * x, std::vector<double> * y,
                     std::vector<double> * orientations)
{
  long size = x->size();
  orientations->erase(orientations->begin(), orientations->end());
  orientations->insert(orientations->begin(), size, 0.0);
  long i;
  for (i = 0; i < size; i++)
  {
    double x_value = (*x)[i];
    double y_value = (*y)[i];
    (*orientations)[i] = vAngle2(x_value, y_value);   
  }
  return 1;
}





// For the time being, this function assumes that we have
// a periodic space, between 0 and period - 1. start1,
// end1, start2, end2, are in that space. To find the overlap
// between two segments, we need to take care of the case
// where one or both segments straddle the "period" point
// and go back to the beginning. In those cases, we
// need to break them down into a segment starting at 0
// and a segment ending at period - 1.
long vCircularSegmentOverlap(long start1, long end1, long start2, long end2,
                              long period)
{
  long start1a, start1b, end1a, end1b;
  long start2a, start2b, end2a, end2b;
  long overlapaa, overlapab, overlapba, overlapbb;

  // Break up segment 1 if necessary
  start1a = start1;
  start1b = 0;
  if (end1 < start1)
  {
    // break up segment
    end1a = period - 1;
    end1b = end1;
  }
  else
  {
    // segment doesn't need to be broken
    end1a = end1;
    end1b = -1;
  }

  // Break up segment 2 if necessary
  start2a = start2;
  start2b = 0;
  if (end2 < start2)
  {
    // break up segment
    end2a = period - 1;
    end2b = end2;
  }
  else
  {
    // segment doesn't need to be broken
    end2a = end2;
    end2b = -1;
  }

  overlapaa = vSegmentOverlap(start1a, end1a, start2a, end2a);
  overlapab = vSegmentOverlap(start1a, end1a, start2b, end2b);
  overlapba = vSegmentOverlap(start1b, end1b, start2a, end2a);
  overlapbb = vSegmentOverlap(start1b, end1b, start2b, end2b);

  long overlap = overlapaa + overlapab + overlapba + overlapbb;
  return overlap;
}


// The length of the result is square_limit, and
// the i-th entry in result will be the square root of i.
vMatrix<long> * vPrecalculateRoots(long square_limit)
{
  vMatrix<long> * result = new vMatrix<long>(1, square_limit + 1);
  vArray(long) matrix = result->Matrix();
  long i = 0;
  long square_i = 0;
  while(square_i <= square_limit)
  {
    long next = i + 1;
    long next_square = next * next;
    for (square_i = i * i; square_i < next_square; square_i++)
    {
      if (square_i > square_limit) break;
      matrix[square_i] = i;
    }
    i++;
  }
  return result;
}


// The length of the result is square_limit, and
// the i-th entry in result will be the square root of i.
vMatrix<double> vPrecalculateRoots2(long square_limit)
{
  vMatrix<double> result = vMatrix<double>(1, square_limit + 1);
  vArray(double) matrix = result.Matrix();
  long i;
  for (i = 0; i <= square_limit; i++)
  {
    matrix[i] = sqrt((double)i);
  }
  return result;
}


// The length of the result is limit, and
// the i-th entry in result will be the log base 10 of i.
vMatrix<double> vPrecalculateLogs2(long limit)
{
  vMatrix<double> result = vMatrix<double>(1, limit + 1);
  vArray(double) matrix = result.Matrix();
  long i;
  // log(0) is undefined.
  matrix[0] = 0;
  for (i = 1; i <= limit; i++)
  {
    matrix[i] = log10((double)i);
  }
  return result;
}


// The length of the result is limit, and
// the i-th entry in result will be the log base 10 of i.
// The difference between this function and vPrecalculateLogs2
// is that here the function keeps track of the highest
// argument with which it was ever called, and the value
// it returned for that argument. As long as the current
// argument is not larger than the highest argument ever, 
// the value returned for the highest argument is also returned
// here. 
vMatrix<double> vPrecalculateLogs3(long limit)
{
  static long largest_argument = 0;
  static vMatrix<double> result;
  if (limit < largest_argument) return result;
  else
  {
    largest_argument = limit;
    result = vMatrix<double>(1, limit + 1);
    vArray(double) matrix = result.Matrix();
    long i;
    // log(0) is undefined.
    matrix[0] = 0;
    for (i = 1; i <= limit; i++)
    {
      matrix[i] = log10((double)i);
    }
    return result;
  }
}



/////////////////////////////////////////////////////////////
//
// Beginning of implementation of class vPCA
//
/////////////////////////////////////////////////////////////


vPCA::vPCA(const char * filename)
{
  LoadFile(filename);
}


vPCA::~vPCA()
{
  CleanUp();
}


long vPCA::Initialize()
{
  return 1;
}


long vPCA::CleanUp()
{
  return 1;
}


long vPCA::LoadBinary(const char * filename)
{
  CleanUp();
  is_valid = 0;
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return 0;

  long items;
  items = vReadPCLongs(fp, &original_dimensions, 1);
  if (items != 1)
  {
    fclose(fp);
    return 0;
  }
  used_dimensions = original_dimensions;
  
  eigenvectors = vMatrix<double>(original_dimensions, original_dimensions);
  long success1, success2, success3;
  success1 = eigenvectors.ReadBandwise(fp);
  eigenvalues = vMatrix<double>(original_dimensions, 1);
  success2 = eigenvalues.ReadBandwise(fp);
  mean = vMatrix<double>(original_dimensions, 1);
  success3 = mean.ReadBandwise(fp);
  fclose(fp);
  if ((success1 <= 0) || (success2 <= 0) || (success3 <= 0))
  {
    return 0;
  }
  SetEigenvalueRoots();
  is_valid = 1;
  return 1;
}


// Format of file:
// original_dimensions (long)
// eigenvectors (original_dimension * original_dimension doubles)
// eigenvalues (original_dimension doubles)
// means (original_dimension doubles)
long vPCA::LoadText(const char * filename)
{
  CleanUp();
  is_valid = 0;
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return 0;

  fscanf(fp, "%li", &original_dimensions);
  used_dimensions = original_dimensions;

  eigenvectors = vMatrix<double>(original_dimensions, original_dimensions);
  long success1, success2, success3;
  success1 = eigenvectors.ReadTextBandwise(fp);
  eigenvalues = vMatrix<double>(original_dimensions, 1);
  success2 = eigenvalues.ReadTextBandwise(fp);
  mean = vMatrix<double>(original_dimensions, 1);
  success3 = mean.ReadTextBandwise(fp);
  fclose(fp);
  if ((success1 <= 0) || (success2 <= 0) || (success3 <= 0))
  {
    return 0;
  }
  SetEigenvalueRoots();
  is_valid = 1;
  return 1;
}


long vPCA::LoadFile(const char * filename)
{
  if (vStringEndsIn(filename, ".txt"))
  {
    return LoadText(filename);
  }
  else
  {
    return LoadBinary(filename);
  }
}


long vPCA::SaveBinary(const char * filename)
{
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) return 0;

  long items1;
  items1 = vWritePCLongs(fp, &original_dimensions, 1);
  if (items1 != 1)
  {
    fclose(fp);
    return 0;
  }
  long success1, success2, success3;
  success1 = eigenvectors.WriteBandwise(fp);
  success2 = eigenvalues.WriteBandwise(fp);
  success3 = mean.WriteBandwise(fp);
  fclose(fp);
  if ((success1 <= 0) || (success2 <= 0) || (success3 <= 0))
  {
    return 0;
  }
  return 1;
}


long vPCA::SaveText(const char * filename)
{
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) return 0;

  fprintf(fp, "%li\r\n", original_dimensions);
  long success1, success2, success3;
  success1 = eigenvectors.WriteText(fp);
  success2 = eigenvalues.WriteText(fp);
  success3 = mean.WriteBandwise(fp);
  fclose(fp);
  if ((success1 <= 0) || (success2 <= 0) || (success3 <= 0))
  {
    return 0;
  }
  return 1;
}


long vPCA::Save(const char * filename)
{
  if (vStringEndsIn(filename, ".txt"))
  {
    return SaveText(filename);
  }
  else
  {
    return SaveBinary(filename);
  }
}


long vPCA::SetEigenvalueRoots()
{
  eigenvalue_roots = vMatrix<double>(original_dimensions, 1);
  long i;
  for (i = 0; i < original_dimensions; i++)
  {
    eigenvalue_roots(i) = sqrt(eigenvalues(i));
  }
  return 1;
}


// Converts a column vector v to a new vector in the PCA space.
// result = eigenvectors' * (v - mean)
vMatrix<double> vPCA::PCA_Conversion(vMatrix<double> & v)
{
  if ((v.Rows() != original_dimensions) || 
      (v.Cols() != 1))
  {
    return vMatrix<double>(1, 1);
  }

  vMatrix<double> used_eigenvectors(original_dimensions, used_dimensions);
  eigenvectors.Copy(&used_eigenvectors, 0, original_dimensions - 1, 
                    0, used_dimensions - 1, 0, 0);
  vMatrix<double> used_eigenvectors_t = used_eigenvectors.Trans();
  vMatrix<double> result = used_eigenvectors_t * (v - mean);
  return result;
}


/////////////////////////////////////////////////////////////
//
// End of implementation of class vPCA
//
/////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////
// 
// Start of implementation of class vEdgeHistogram
//
////////////////////////////////////////////////////////////////


// This is the way views are generated by save_views:

// 12 views (indexed 0-11), each by rotating counterclockwise the previous 1
// Then, view 12 is view0 rotated clockwise 90 degrees.

// Orientation values used for edge histograms: 0 is horizontal.
// Each bin increment represents a clockwise rotation of the image
// by bin_step radians.

static const long frames_per_view = 12;

// The macro vStepDisplacement returns the bin displacement
// that occurs when we move from view 0 to view 1.
#define vStepDisplacement(bins) ((bins) - ((bins) / (frames_per_view * 2)))

// vNormalizedAngle(view) returns the number of stepwise
// counterclockwise rotations we need to apply to view 0 to
// obtain the view indexed by view (this is applicable to 
// views generated by the program save_views) Views 0-11 map
// to themselves, views 12-23 go to 36-47, views 24-35 go to
// themselves and views 36-47 go to 12-23.
#define vNormalizedAngle(i) \
(((i) % frames_per_view + (4 - (i)/frames_per_view) * frames_per_view) % (4 * frames_per_view))

// The macro vViewDisplacement returns the bin displacement that
// occurs when we move from view 0 to view i (i is between 0 and 48).
// So, the j-th bin of view 0 is equal to the bin indexed by
// (j + vViewDisplacement(bins, i)) % bins for frame i, 
// provided the number of
// bins in the histogram is the bins variable.
#define vViewDisplacement(bins, i) \
((vNormalizedAngle(i) * vStepDisplacement(bins)) % (bins))

vEdgeHistogram::vEdgeHistogram(long bins) : vMatrix<double>(1, bins)
{
  assert(bins > 0);
  bin_step = M_PI / (double) bins;
}


vEdgeHistogram::vEdgeHistogram(long bins, grayscale_image * original, 
                                 grayscale_image * edges) :
  vMatrix<double>(1, bins)
{
  assert(bins > 0);
  bin_step = M_PI / (double) bins;

  // Initialize bin values to 0.
  vWriteValue(this, (double) 0);

  // Get coordinates of edge pixels
  vector<long> rows_vector, cols_vector;
  vGetNonZeroPoints(edges, &rows_vector, &cols_vector);
  vArray2(uchar) values = original->Matrix2(0);

  long original_rows = original->Rows();
  long original_cols = original->Cols();
  long points = rows_vector.size();
  if (points == 0) return;
  long i;
  double horizontal = 0;
  double vertical = 0;
  // Go through the edge pixels, for each of them calculate the 
  // edge orientation at the edge pixel and increment the corresponding bin
  for (i = 0; i < points; i++)
  {
    long row = rows_vector[i];
    long col = cols_vector[i];
    if (row < 1) continue;
    if (col < 1) continue;
    if (row >= original_rows - 1) continue;
    if (col >= original_cols - 1) continue;
    vPixelGradient2(values, row, col, horizontal, vertical);
    if ((horizontal == 0) && (vertical == 0)) continue;
    double angle = vAngle(horizontal, vertical);
    // Convert from gradient to direction of least change
    angle = angle + M_PI / 2;
    long bin_index = BinIndex(angle);
    matrix[bin_index]++;
  }

  // Normalize bins so that their sum is 1.
  Normalize();
}


vEdgeHistogram::~vEdgeHistogram()
{
}


long vEdgeHistogram::Normalize()
{
  double sum = 0;
  long i;
  for (i = 0; i < cols; i++)
  {
    sum = sum + matrix[i];
  }
  if (sum == 0) return 0;
  for (i = 0; i < cols; i++)
  {
    matrix[i] = matrix[i] / sum;
  }
  return 1;
}


// Reads in original images from the video sequence in original_file
// and edge images from edge_file. For each frame in original_file,
// it reads the corresponding edge image from edge_file, calculates
// the edge histogram and saves it to target. The format of the output
// file is:
// bins (long)
// original_frames (long)
// histogram1 (written as floats)
// histogram2 (written as floats)
// ...
long vEdgeHistogram::SaveHistograms(long bins, const char * original_file, 
                                     const char * edge_file, 
                                     const char * target)
{
  // Open the input sequences (original and edges)
  vVideoFile * original_stream, * edge_stream;
  original_stream = edge_stream = 0;
  FILE * target_fp = 0;
  grayscale_image * original, * edges;
  original = edges = 0;

  long result, items, frame, original_rows, original_cols, original_frames;
  long edge_rows, edge_cols, edge_frames;
  result = 0;

  original_stream = vOpenReadVideoFile(original_file);
  if (original_stream == 0) 
  {
    vPrint("Can't open %s.\n", original_file);
    goto exit;
  }

  edge_stream = vOpenReadVideoFile(edge_file);
  if (edge_stream == 0)
  {
    vPrint("Can't open %s.\n", edge_file);
    goto exit;
  }

  // Verify that original and edge sequence have same
  // number of frames, width and height.
  original_frames = original_stream->NumberOfFrames();
  edge_frames = edge_stream->NumberOfFrames();
  if (original_frames != edge_frames)
  {
    vPrint("Error, original and edge sequences have different lengths.\n");
    goto exit;
  }


  original_rows = original_stream->Rows();
  edge_rows = edge_stream->Rows();
  if (original_rows != edge_rows)
  {
    vPrint("Error, original and edge sequences have different rows.\n");
    goto exit;
  }

  original_cols = original_stream->Cols();
  edge_cols = edge_stream->Cols();
  if (original_cols != edge_cols)
  {
    vPrint("Error, original and edge sequences have different cols.\n");
    goto exit;
  }

  // Open target file and write header (number of frames and bins).
  target_fp = fopen(target, vFOPEN_WRITE);
  if (target_fp == 0)
  {
    vPrint("Can't open target file %s\n", target);
    goto exit;
  }

  items = vWritePCLongs(target_fp, &bins, 1);
  if (items != 1)
  {
    vPrint("Error in writing to %s\n", target);
    goto exit;
  }
  items = vWritePCLongs(target_fp, &original_frames, 1);
  if (items != 1)
  {
    vPrint("Error in writing to %s\n", target);
    goto exit;
  }

  vPrint("\n");
  // We assume that the synthetic sequence has frames_per_view
  // orientations for
  // each view. Since the edge orientation histograms can be easily
  // steered, we only need to process one out of every 12 orientations.
  for (frame = 0; frame < original_frames; frame = frame + frames_per_view)
  {
    original = original_stream->ReadFrameGray(frame);
    edges = edge_stream->ReadFrameGray(frame);
    if ((original == 0) || (edges == 0))
    {
      vPrint("\nCouldn't read frame %li, (%li %li)\n",
              frame, original, edges);
      goto exit;
    }
    
    grayscale_image * thresholded_edges = (grayscale_image *) edges->threshold(1);
    v3dMatrix<double> * blurred_original = vBlur(original);
    grayscale_image * gray_blurred = blurred_original->ToGray();
    vEdgeHistogram histogram(bins, gray_blurred, thresholded_edges);
    // Save as floats, to save space
    v3dMatrix<float> float_histogram(&histogram);
    long success = float_histogram.WriteBandwise(target_fp);
//    histogram.Print("histogram: ");
    vdelete(thresholded_edges);
    vdelete(blurred_original);
    vdelete(gray_blurred);
    vdelete(edges);
    edges = 0;
    vdelete(original);
    original = 0;
    if (success == 0)
    {
      vPrint("\nError in writing histogram to file\n");
      goto exit;
    }
    vPrint("Processed frame %li\r", frame);
  }

  vPrint("\n");
  fclose(target_fp);
  target_fp = 0;
  result = 1;

exit:
  if (target_fp != 0) fclose(target_fp);
  vCloseVideoFile(original_stream);
  vCloseVideoFile(edge_stream);
  vdelete(original);
  vdelete(edges);
  return result;
}


// Each row in the result is a histogram corresponding to one frame.
vMatrix<double> * vEdgeHistogram::LoadHistograms(const char * filename)
{
  long bins = 0;
  long original_frames = 0;
  
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return 0;
  
  long items;
  items = vReadPCLongs(fp, &bins, 1);
  items += vReadPCLongs(fp, &original_frames, 1);
  if (items != 2)
  {
    fclose(fp);
    return 0;
  }

  long saved_frames = original_frames / frames_per_view;
  vMatrix<float> input_matrix(saved_frames, bins);
  long success = input_matrix.ReadBandwise(fp);
  fclose(fp);
  if (success <= 0)
  {
    return 0;
  }

  vMatrix<double> * result = new vMatrix<double>(saved_frames, bins);
  vArray(double) output = result->Matrix();
  vArray(float) input = input_matrix.Matrix();
  long i;
  long size = result->Size();
  for (i = 0; i < size; i++)
  {
    output[i] = input[i];
  }

  return result;
}


// Note: The code for the distances would be a lot prettier if I wrote
// separate functions to convert a histogram to its cumulative 
// equivalent, and other auxiliary functions, but it would also be 
// a lot slower.
double vEdgeHistogram::MinCumulativeDistance(vEdgeHistogram * histogram2)
{
  long bins = cols;
  if (bins != histogram2->NumberOfBins()) return -1;
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) other_matrix = histogram2->Matrix();
  long start;
  double min_distance = bins * 10;
  long min_index = -1;

  // Calculate all cumulative distances, select the mean.
  for (start = 0; start < bins; start++)
  {
    // Convert histograms into cumulative histograms with given start
    array1[0] = matrix[start];
    array2[0] = other_matrix[start];
    long bin;
    for (bin = 1; bin < bins; bin++)
    {
      long index = (start + bin) % bins;
      array1[bin] = array1[bin - 1] + matrix[index];
      array2[bin] = array2[bin - 1] + other_matrix[index];
    }

    // Calculate distance;
    double distance = 0;
    for (bin = 0; bin < bins; bin++)
    {
//      distance += vAbs(array1[bin] - array2[bin]);
      distance += 1 - Min(array1[bin], array2[bin]);
    }
    // Check if this distance is minimum found so far.
    if (distance < min_distance)
    {
      min_distance = distance;
      min_index = start;
    }
  }
  vdelete2(array1);
  vdelete2(array2);
  return min_distance;
}


double vEdgeHistogram::MeanCumulativeDistance(vEdgeHistogram * histogram2)
{
  long bins = cols;
  if (bins != histogram2->NumberOfBins()) return -1;
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) other_matrix = histogram2->Matrix();
  long start;
  double total_distance = 0;

  // Calculate all cumulative distances, select the mean.
  for (start = 0; start < bins; start++)
  {
    // Convert histograms into cumulative histograms with given start
    array1[0] = matrix[start];
    array2[0] = other_matrix[start];
    long bin;
    for (bin = 1; bin < bins; bin++)
    {
      long index = (start + bin) % bins;
      array1[bin] = array1[bin - 1] + matrix[index];
      array2[bin] = array2[bin - 1] + other_matrix[index];
    }

    // Calculate distance;
    double distance = 0;
    for (bin = 0; bin < bins; bin++)
    {
//      distance += vAbs(array1[bin] - array2[bin]);
      distance += 1 - Min(array1[bin], array2[bin]);
    }
    // Add distance to total_distance;
    total_distance += distance;
  }
  vdelete2(array1);
  vdelete2(array2);
  double average_distance = total_distance / (double) bins;
  return average_distance;
}

  
// The distance functions generate 48 different histograms for each
// histogram saved in training. Those 48 histograms correspond
// to different image plane rotations. The order in which they are
// generated corresponds to the order with which training views get saved
// by the program save_views. They return a matrix that contains the 
// distance between the test histogram and each of the generated
// training histograms (the distance is defined differently in each
// function).

vMatrix<double> 
vEdgeHistogram::MinCumulativeDistance2(vMatrix<double> * training,
                                        vMatrix<double> * test)
{
  long bins = training->Cols();
  long saved_frames = training->Rows();
  long frames = saved_frames * frames_per_view * 4;
  if (bins != test->Cols()) return vMatrix<double>();
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) matrix = vnew(double, bins);
  vArray2(double) matrix2 = training->Matrix2();
  vArray(double) other_matrix = test->Matrix();

  long frame = 0;
  vMatrix<double> result_matrix(1, frames);
  vArray(double) result = result_matrix.Matrix();

  long viewpoint;
  for (viewpoint = 0; viewpoint < saved_frames; viewpoint++)
  {
    vArray(double) training_histogram = matrix2[viewpoint];

    long view;
    // Now, process all 48 histograms generated from the training
    // histogram. Actually, only 24 histograms need to be processed,
    // since the edge histograms for an image are identical if the
    // image rotates by 180 degrees.
    // We consider the histograms in the following order:
    // 0, 12, 24, 36, 1, 13, 25, 37, ... The reason is that
    // the vMatchScores constructor assumes such an order
    // in the distances that it receives.
    for (view = 0; view < frames_per_view; view++)
    {
      long rotation;
      for (rotation = 0; rotation < 2; rotation++)
      {
        // First, rotate current orientation histogram accordingly
        long displacement = vViewDisplacement(bins, view + rotation * 12);
        long bin;
        for (bin = 0; bin < bins; bin++)
        {
          matrix[(bin+displacement) % bins] = training_histogram[bin];
        }

        // Now, calculate the distance between rotated histogram 
        // (stored in matrix) and test histogram (stored in 
        // other_matrix).
        long start;
        double min_distance = bins * 10;
        long min_index = -1;

        // Calculate all cumulative distances, select the mean.
        for (start = 0; start < bins; start++)
        {
          // Convert histograms into cumulative histograms with given start
          array1[0] = matrix[start];
          array2[0] = other_matrix[start];
          for (bin = 1; bin < bins; bin++)
          {
            long index = (start + bin) % bins;
            array1[bin] = array1[bin - 1] + matrix[index];
            array2[bin] = array2[bin - 1] + other_matrix[index];
          }

          // Calculate distance;
          double distance = 0;
          for (bin = 0; bin < bins; bin++)
          {
//          distance += vAbs(array1[bin] - array2[bin]);
            distance += 1 - Min(array1[bin], array2[bin]);
          }
          // Check if this distance is minimum found so far.
          if (distance < min_distance)
          {
            min_distance = distance;
            min_index = start;
          }
        }
//        vPrint("frames = %li, frame = %li\r", frames, frame);
        result[frame] = min_distance;
        result[frame+2] = min_distance;
        frame++;
      }
      frame = frame + 2;
    }
  }

  vdelete2(array1);
  vdelete2(array2);
  vdelete2(matrix);
  return result_matrix;
}


vMatrix<double> vEdgeHistogram::MeanCumulativeDistance2(vMatrix<double> * training,
                                                          vMatrix<double> * test)
{
  long bins = training->Cols();
  long saved_frames = training->Rows();
  long frames = saved_frames * frames_per_view * 4;
  if (bins != test->Cols()) return vMatrix<double>();
  vArray(double) array1 = vnew(double, bins);
  vArray(double) array2 = vnew(double, bins);
  vArray(double) matrix = vnew(double, bins);
  vArray2(double) matrix2 = training->Matrix2();
  vArray(double) other_matrix = test->Matrix();

  long frame = 0;
  vMatrix<double> result_matrix(1, frames);
  vArray(double) result = result_matrix.Matrix();

  // The next two lines is just for debugging
//  vWriteValue(&result_matrix, (double) 0);
//  saved_frames = 4;

  long viewpoint;
  for (viewpoint = 0; viewpoint < saved_frames; viewpoint++)
  {
    vArray(double) training_histogram = matrix2[viewpoint];

    long view;
    // Now, process all 48 histograms generated from the training
    // histogram. Actually, only 24 histograms need to be processed,
    // since the edge histograms for an image are identical if the
    // image rotates by 180 degrees.
    // We consider the histograms in the following order:
    // 0, 12, 24, 36, 1, 13, 25, 37, ... The reason is that
    // the vMatchScores constructor assumes such an order
    // in the distances that it receives.
    for (view = 0; view < frames_per_view; view++)
    {
      long rotation;
      for (rotation = 0; rotation < 2; rotation++)
      {
        double total_distance = 0;
        // First, rotate current orientation histogram accordingly
        long displacement = vViewDisplacement(bins, view + rotation * 12);
        long bin;
        for (bin = 0; bin < bins; bin++)
        {
          matrix[(bin+displacement) % bins] = training_histogram[bin];
        }

        // Now, calculate the distance between rotated histogram 
        // (stored in matrix) and test histogram (stored in 
        // other_matrix).
        long start;

        // Calculate all cumulative distances, select the mean.
        for (start = 0; start < bins; start++)
        {
          // Convert histograms into cumulative histograms with given start
          array1[0] = matrix[start];
          array2[0] = other_matrix[start];
          for (bin = 1; bin < bins; bin++)
          {
            long index = (start + bin) % bins;
            array1[bin] = array1[bin - 1] + matrix[index];
            array2[bin] = array2[bin - 1] + other_matrix[index];
          }

          // Calculate distance;
          double distance = 0;
          for (bin = 0; bin < bins; bin++)
          {
//            distance += vAbs(array1[bin] - array2[bin]);
            distance += 1 - Min(array1[bin], array2[bin]);
          }
          total_distance += distance;
        }
//        vPrint("frames = %li, frame = %li\r", frames, frame);
        double average_distance = total_distance / (double) bins;
        result[frame] = average_distance;
        result[frame+2] = average_distance;
        frame++;
      }
      frame = frame + 2;
    }
  }

  vdelete2(array1);
  vdelete2(array2);
  vdelete2(matrix);
  return result_matrix;
}
  


////////////////////////////////////////////////////////////////
// 
// End of implementation of class vEdgeHistogram
//
////////////////////////////////////////////////////////////////


// Returns a 1-dimensional matrix with the specified cols,
// whose entries are random.
vMatrix<ulong> vRandomVector(long cols)
{
  srand(time(0));
  vMatrix<ulong> result_matrix = vMatrix<ulong>(1, cols);
  vArray(ulong) matrix = result_matrix.Matrix();
  long i;
  for (i = 0; i < cols; i++)
  {
    matrix[i] = vRandomLong();
  }
  return result_matrix;
}


// in_rows and in_cols are arrays of length number. out_rows and out_cols
// are arrays with the same length. image_rows, image_cols are the 
// dimensions of the image from which point coordinates have been put to
// in_rows and in_cols. Those point coordinates are rotated by 90, 180,
// 270 or 0 degrees clockwise and the results are put in out_rows and 
// out_cols. The caller can pass the same arrays for in_rows and out_rows,
// or in_cols and out_cols.
long vRotatePointsRight(vArray(long) in_rows, vArray(long) in_cols, 
                         vArray(long) out_rows, vArray(long) out_cols, 
                         long number, long image_rows, long image_cols, 
                         long degrees)
{
  // If degrees is negative set it to a corresponding positive angle value.
  if (degrees < 0) degrees = degrees + (-degrees / 360 + 1) * 360;
  // Make degrees equal to the corresponding value between 0 and 360.
  degrees = degrees % 360;
  // In this function we can only rotate by 0, 90, 180 and 270 degres.
  if ((degrees % 90) != 0) return 0;

  long i;

  if (degrees == 0)
  {
    for (i = 0; i < number; i++)
    {
      out_rows[i] = in_rows[i];
      out_cols[i] = in_cols[i];
    }
  }
  
  else if (degrees == 90)
  {
    for (i = 0; i < number; i++)
    {
      long row = in_rows[i];
      long col = in_cols[i];
      out_rows[i] = col;
      out_cols[i] = image_rows - 1 - row;
    }
  }
 
  else if (degrees == 180)
  {
    for (i = 0; i < number; i++)
    {
      long row = in_rows[i];
      long col = in_cols[i];
      out_rows[i] = image_rows - 1 - row;
      out_cols[i] = image_cols - 1 - col;
    }
  }

  else if (degrees == 270)
  {
    for (i = 0; i < number; i++)
    {
      long row = in_rows[i];
      long col = in_cols[i];
      out_rows[i] = image_cols - 1 - col;
      out_cols[i] = row;
    }
  }

  return 1;
}


// vRefineHandSegmentation should be called whenever we have a clear
// idea of where the hand is in the image, and we have identified
// the pixels that are the most likely to be hand pixels by setting
// their value to 255 in first_binary. This 
// function identifies the largest connected component, fills up 
// holes in it, and makes sure all hand pixels have a value 
// of at least two in each
// of their R, G, and B color values (so that they don't get confused
// with their background).
color_image * vRefineHandSegmentation(v3dMatrix<uchar> * first_binary,
                                   color_image * source)
{
  long rows = source->Rows();
  long cols = source->Cols();
  long size = source->Size();
  if (rows != first_binary->Rows()) return 0;
  if (cols != first_binary->Cols()) return 0;
  if (first_binary->Bands() != 1) return 0;

  // Do connected component analysis on the binary skin image and find the
  // largest component
  vector<Label> labels;
  v3dMatrix<long> * connected = first_binary->ConnectedComponents(&labels);

  // Fill up holes in the connected components.
  // First we need to produce a label image where the background is label 1.
  // To do that, we invert the binary skin image and do connected component
  // analysis on the result.
  grayscale_image * inverse_first_binary = first_binary->ToGrayCopy();
  vInverseImage(inverse_first_binary);
  vector<Label> temp_labels;
  v3dMatrix<long> * background_component = 
    inverse_first_binary->ConnectedComponents(&temp_labels);

  vFillUpHoles(connected, background_component);
  vdelete(inverse_first_binary);
  vdelete(background_component);

  // Filling up holes may have connected components that were previously
  // not connected (this happens if one component was fully enclosed by
  // another but having background pixels separating them). So, we should
  // do connected component analysis again.
  grayscale_image * component_binary = (grayscale_image *) connected->threshold(1);
  labels.erase(labels.begin(), labels.end());
  vdelete(connected);
  connected = component_binary->ConnectedComponents(&labels);
  vdelete(component_binary);

  std::sort(labels.begin(), labels.end(), LabelMore());
  Label largest = labels[1];

  // Select the connected component that corresponds to the hand. If
  // row and col are valid pixel coordinates, select the component
  // that contains that pixel. If (row, col) is not a valid pixel,
  // then we choose as hand the component that contains the 
  // point (row, col), then we choose the largest component.
  long hand_id = largest.id;
  
  v3dMatrix<uchar> * second_binary = vSelectValue(connected, hand_id);
  // second_binary has intensity value 0 for background pixels and 255 for
  // hand pixels. third_binary will have intensity value 0 for background
  // pixels and 1 for hand pixels.
  v3dMatrix<uchar> * third_binary = 
    new v3dMatrix<uchar>(source->Rows(), source->Cols(), 1);
  function_image_divide(second_binary, 255, third_binary);

  v3dMatrix<uchar> * hand_binary = second_binary->ToRGB();
  vdelete(second_binary);
  vdelete(third_binary);
  vdelete(connected);
  color_image * hand_image = new color_image(source->Rows(), source->Cols());
  function_image_multiply(hand_binary, source, hand_image);

  // Finally, we should make sure that there is no pixel that is marked as
  // hand in hand_binary that has RGB value (0,0,0), because in that case
  // that pixel may be considered a background pixel in later processing
  // of the hand (like contour finding). So, to every pixel that is 
  // marked as hand in hand_binary we will make sure that it has 
  // intensity more than 1 in every band.

  vArray(uchar) binary = hand_binary->Matrix(0);
  vArray(uchar) R = hand_image->GetR();
  vArray(uchar) G = hand_image->GetR();
  vArray(uchar) B = hand_image->GetR();

  long i;
  for (i = 0; i < size; i++)
  {
    if (binary[i] == 0) continue;
    if (R[i] < 2) R[i] = 2;
    if (G[i] < 2) G[i] = 2;
    if (B[i] < 2) B[i] = 2;
  }

  vdelete(hand_binary);
  return hand_image;
}


// vSaveEdgeStatistics calculates and saves to a file the following:
// Suppose original_file stores a video sequence, and edge_file
// stores the edge images of the frames of original_file. Then, the
// question is, given an edge pixel A with edge orientation E1, how likely
// is it that there will be an edge pixel B with edge orientation E2
// at location (X,Y). This question can be answered by building a histogram
// where bins are indexed by E1, X, Y, E2, and where each bin contains the
// number of times that the combination E1, X, Y, E2 was found in the 
// sequence.
//
// spatial_subsampling is N if we want each bin to correspond to a NxN
// pixel area (X and Y indices).
// pixel1_bins is the number of discrete values into which values of E1
// will be converted.
// pixel2_bins is the number of discrete values into which values of E2
// will be converted. It is also the number of possible indices 
// corresponding to E2 in the histogram.
// [pixel1_start, ..., pixel1_end] specifies the discrete values of E1
// that will be used as indices in the histogram. If they are set
// to -1, they will acquire default values of 0 and pixel1_bins-1 
// respectively. Sometimes it is enough to keep track of only one of
// all possible discrete values of E1, because then we can use the 
// histogram to figure out the rest of the values. For example, my
// hand database contains 48 image plane rotations for each viewpoint,
// and if we keep edge statistics only for cases where E1 is 0,
// we can calculate based on them the statistics for all other values
// of E1.
long vSaveEdgeStatistics(const char * original_file, const char * edge_file,
                          const char * target_file,
                          long spatial_subsampling, 
                          long pixel1_bins, long pixel2_bins,
                          long pixel1_start, long pixel1_end,
                          long frames_to_process)
{
  vVideoFile * original_sequence =  vOpenReadVideoFile(original_file);
  vVideoFile * edge_sequence = vOpenReadVideoFile(edge_file);

  // Make sure the sequences both exist and agree with each other
  // in number of frames and frame size.
  if ((original_sequence == 0) || (edge_sequence == 0) ||
      (original_sequence->NumberOfFrames() != edge_sequence->NumberOfFrames()) ||
      (original_sequence->Rows() != edge_sequence->Rows()) ||
      (original_sequence->Cols() != edge_sequence->Cols()))
  {
    vdelete(original_sequence);
    vdelete(edge_sequence);
    return 0;
  }

  // If pixel1_start or pixel1_end are -1, set them to default values.
  if (pixel1_start == -1) pixel1_start = 0;
  if (pixel1_end == -1) pixel1_end = pixel1_bins - 1;

  long number_of_frames = original_sequence->NumberOfFrames();
  long rows = original_sequence->Rows();
  long cols = original_sequence->Cols();

  // If frames_to_process is -1 set it to a default value
  if (frames_to_process == -1) frames_to_process = number_of_frames;
  if (frames_to_process < 0) frames_to_process = 1;
  if (frames_to_process > number_of_frames) 
  {
    frames_to_process = number_of_frames;
  }

  // Create the vEdgeStatistics object that will hold the 
  // information that we will estimate.
  vEdgeStatistics edge_stats(rows, cols, spatial_subsampling,
                              pixel1_bins, pixel2_bins,
                              pixel1_start, pixel1_end);
  edge_stats.Zero();
  long frame;
  long failure = 0;
  // Read each frame of the sequence and process it.
  vPrint("\n");
  for (frame = 0; frame < frames_to_process; frame++)
  {
    // Read the original image and the edge image.
    grayscale_image * original = original_sequence->ReadFrameGray(frame);
    grayscale_image * edge_image = edge_sequence->ReadFrameGray(frame);
    // Make sure both images were read successfully.
    if ((original == 0) || (edge_image == 0))
    {
      vdelete(original);
      vdelete(edge_image);
      failure = 1;
      break;
    }
    v3dMatrix<double> * blurred = vBlur(original);
    grayscale_image * blurred_original = blurred->ToGray();

    long degrees;
    for (degrees = 0; degrees < 360; degrees += 90)
    {
      grayscale_image * rotated_blurred = 
        (grayscale_image *) vRotateRight(blurred_original, degrees);
      grayscale_image * rotated_edges = 
        (grayscale_image *) vRotateRight(edge_image, degrees);
      v3dMatrix<double> * orientations = vEdgeOrientations(rotated_blurred);
      long success = edge_stats.ProcessImage(orientations, rotated_edges);
      vdelete(orientations);
      vdelete(rotated_blurred);
      vdelete(rotated_edges);
      if (success <= 0) 
      {
        failure = 1;
        break;
      }
    }
    vdelete(original);
    vdelete(blurred);
    vdelete(blurred_original);
    vdelete(edge_image);
    vPrint("processed frame %li of %li\r", frame+1, frames_to_process);
  }

  vPrint("\n");
  vCloseVideoFile(original_sequence);
  vCloseVideoFile(edge_sequence);
  if (failure > 0) return 0;
  long success = edge_stats.Save(target_file);
  return success;
}


  long image_rows;
  long image_cols;

  long spatial_subsampling;
  long pixel1_bins;
  long pixel2_bins;
  long pixel1_start;
  long pixel1_end;


vEdgeStatistics::vEdgeStatistics(long in_image_rows, long in_image_cols,
                                   long in_spatial_subsampling, long in_pixel1_bins,
                                   long in_pixel2_bins, long in_pixel1_start,
                                   long in_pixel1_end)
{
  image_rows = in_image_rows;
  image_cols = in_image_cols;
  spatial_subsampling = in_spatial_subsampling;
  pixel1_bins = in_pixel1_bins;
  pixel2_bins = in_pixel2_bins;
  pixel1_start = in_pixel1_start;
  pixel1_end = in_pixel1_end;

  assert(image_rows > 0);
  assert(image_cols > 0);
  assert(spatial_subsampling > 0);
  assert(pixel1_bins > 0);
  assert(pixel2_bins > 0);
  assert(pixel1_start >= 0);
  assert(pixel1_end < pixel1_bins);

  bin_step1 = M_PI / (double) pixel1_bins;
  bin_step2 = M_PI / (double) pixel2_bins;

  dim2_high = image_rows / spatial_subsampling + 1;
  dim2_low = -dim2_high;
  dim3_high = image_cols / spatial_subsampling + 1;
  dim3_low = -dim3_high;

  array = new v4dArray<double>(pixel1_start, pixel1_end, 
                                 dim2_low, dim2_high,
                                 dim3_low, dim3_high, 0, pixel2_bins - 1);
  bins = array->Data4();
}


vEdgeStatistics::~vEdgeStatistics()
{
  vdelete(array);
}


long vEdgeStatistics::ImageRows()
{
  return image_rows;
}


long vEdgeStatistics::ImageCols()
{
  return image_cols;
}


long vEdgeStatistics::SpatialSubsampling()
{
  return spatial_subsampling;
}


long vEdgeStatistics::Pixel1Bins()
{
  return pixel1_bins;
}


long vEdgeStatistics::Pixel2Bins()
{
  return pixel2_bins;
}


long vEdgeStatistics::Pixel1Start()
{
  return pixel1_start;
}


long vEdgeStatistics::Pixel1End()
{
  return pixel1_end;
}


long vEdgeStatistics::Zero()
{
  return array->Zero();
}


long vEdgeStatistics::Save(const char * filename)
{
  long zero = 0;
  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) return 0;

  long items = 0;
  items += vWritePCLongs(fp, &zero, 1);
  items += vWritePCLongs(fp, &image_rows, 1);
  items += vWritePCLongs(fp, &image_cols, 1);
  items += vWritePCLongs(fp, &spatial_subsampling, 1);
  items += vWritePCLongs(fp, &pixel1_start, 1);
  items += vWritePCLongs(fp, &pixel1_end, 1);
  items += vWritePCLongs(fp, &pixel1_bins, 1);
  items += vWritePCLongs(fp, &pixel2_bins, 1);

  if (items != 8) 
  {
    fclose(fp);
    return 0;
  }

  long success = array->Write(fp);
  fclose(fp);
//  array->Print();
  return success;
}


vEdgeStatistics * vEdgeStatistics::Load(const char * filename)
{
  long zero = 0;
  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) return 0;

  long items = 0;
  long image_rows = 0, image_cols = 0, spatial_subsampling = 0;
  long pixel1_start = 0, pixel1_end = 0, pixel1_bins = 0, pixel2_bins = 0;
  items += vReadPCLongs(fp, &zero, 1);
  items += vReadPCLongs(fp, &image_rows, 1);
  items += vReadPCLongs(fp, &image_cols, 1);
  items += vReadPCLongs(fp, &spatial_subsampling, 1);
  items += vReadPCLongs(fp, &pixel1_start, 1);
  items += vReadPCLongs(fp, &pixel1_end, 1);
  items += vReadPCLongs(fp, &pixel1_bins, 1);
  items += vReadPCLongs(fp, &pixel2_bins, 1);

  if (items != 8) 
  {
    fclose(fp);
    return 0;
  }

  vEdgeStatistics * result = 
    new vEdgeStatistics(image_rows, image_cols, spatial_subsampling, 
                         pixel1_bins, pixel2_bins, pixel1_start, pixel1_end);
  if (result == 0) return 0;
  
  v4dArray<double> * array = result->array;
  long success1 = array->SkipHeader(fp);
  long success2 = array->ReadData(fp);
  fclose(fp);
//  array->Print();
  if ((success1 == 0) || (success2 == 0))
  {
    vdelete(result);
    return 0;
  }
  else 
  {
    return result;
  }
}


// Add votes to the appropriate bins using the orientation and
// edge information stored in orientations and edges respectively.
long vEdgeStatistics::ProcessImage(v3dMatrix<double> * orientations_image, 
                                    grayscale_image * edges_image)
{
  vArray2(double) orientations = orientations_image->Matrix2(0);
  vector<long> edge_rows, edge_cols;
  vGetNonZeroPoints(edges_image, &edge_rows, &edge_cols);
  // rel_row means "relative row", which will be the relative
  // coordinates of point2 with respect to point1.
  long point1, point2, row1, col1, row2, col2, rel_row, rel_col;
  double orientation1, orientation2;
  long points = edge_rows.size();
  for (point1 = 0; point1 < points; point1++)
  {
    row1 = edge_rows[point1];
    col1 = edge_cols[point1];
    orientation1 = orientations[row1][col1];
    long orientation_bin = Bin1(orientation1);
    if ((orientation_bin < pixel1_start) ||
        (orientation_bin > pixel1_end)) continue;
    for (point2 = 0; point2 < points; point2++)
    {
      row2 = edge_rows[point2];
      col2 = edge_cols[point2];
      orientation2 = orientations[row2][col2];
      rel_row = row2 - row1;
      rel_col = col2 - col1;
      // This line increments the bin indexed by 
      // orientation1, rel_row, rel_col, orientation2. How
      // the indexing works is defined by the two operators ()
      // defined in the declaration of the class vEdgeStatistics.
      double value1 = operator() (orientation1, rel_row, rel_col, orientation2);
      Data(orientation1, rel_row, rel_col, orientation2)++;
      double value2 = operator() (orientation1, rel_row, rel_col, orientation2);

//      long random = vRandomLong();
//      if (random % 1000 == 0)
//      {
//        vPrint("Before: value = %lf, %.1lf, %li, %li, %.1lf\n", 
//                 value1, orientation1, rel_row, rel_col, orientation2);
//        vPrint("After: value = %lf, %.1lf, %li, %li, %.1lf\n\n", 
//                value2, orientation1, rel_row, rel_col, orientation2);
//      }
    }
  }
    
  return 1;
}


// For each edge pixel in edges get the number of votes 
// it gets from the edge statistics histogram. The orientation
// of each edge pixel is stored in orientations.
vMatrix<double> * vEdgeStatistics::Vote(v3dMatrix<double> * orientation_image, 
                                          grayscale_image * edge_image)
{
  long rows = orientation_image->Rows();
  long cols = orientation_image->Cols();
  assert(edge_image->Rows() == rows);
  assert(edge_image->Cols() == cols);

  // Get the arrays for the orientations and the result
  vMatrix<double> * result_matrix = new vMatrix<double>(rows, cols);
  vWriteValue(result_matrix, (double) 0);
  vArray2(double) orientations = orientation_image->Matrix2(0);
  vArray2(double) result = result_matrix->Matrix2();

  // Get the edge points of the edge image
  vector<long> edge_rows, edge_cols;
  vGetNonZeroPoints(edge_image, &edge_rows, &edge_cols);
  long points = edge_rows.size();
  long point1, point2;
  // Process all points
  for (point1 = 0; point1 < points; point1++)
  {
    // Get position and orientation of point1
    long row1 = edge_rows[point1];
    long col1 = edge_cols[point1];
    double orientation1 = orientations[row1][col1];

    // Get votes for point1 from all other edge pixels
    for (point2 = 0; point2 < points; point2++)
    {
      // Get position and orientation of point2
      long row2 = edge_rows[point2];
      long col2 = edge_cols[point2];
      double orientation2 = orientations[row2][col2];
      // Get relative coordinates of point2 with respect to point1
      long rel_row = row2 - row1;
      long rel_col = col2 - col1;
      // Get the vote and add it to the votes for point1..
      double vote = operator() (orientation1, rel_row, rel_col, orientation2);
      result[row1][col1] += vote;
    }
  }

  return result_matrix;
}


// For each edge pixel in edges get the number of votes 
// it gets from the edge statistics histogram. The orientation
// of each edge pixel is stored in orientations. The difference
// betweent this function and Vote() is that this function is
// optimized for the database of hand images. In the 
// edge statistics for that database,
// the histogram only contains information for e1 = 0 
// (horizontal orientation). If the orientation of pixel1
// is e1 and e1 is not 0, then to obtain statistics for that pixel
// we need to rotate the image so that e1 becomes 0. This can be done
// by rotating, for every point2, its relative coordinates by an
// angle of -e1, and by rotating the orientation e2 of point2 also
// by an angle of -e1
vMatrix<double> * vEdgeStatistics::Vote2(v3dMatrix<double> * orientation_image, 
                                         grayscale_image * edge_image)
{
  long rows = orientation_image->Rows();
  long cols = orientation_image->Cols();
  assert(edge_image->Rows() == rows);
  assert(edge_image->Cols() == cols);

  // Get the arrays for the orientations and the result
  vMatrix<double> * result_matrix = new vMatrix<double>(rows, cols);
  vWriteValue(result_matrix, (double) 0);
  vArray2(double) orientations = orientation_image->Matrix2(0);
  vArray2(double) result = result_matrix->Matrix2();

  // Get the edge points of the edge image
  vector<long> edge_rows, edge_cols;
  vGetNonZeroPoints(edge_image, &edge_rows, &edge_cols);
  long points = edge_rows.size();
  long point1, point2;
  double rotated_row = 0;
  double rotated_col = 0;
  // Process all points
  for (point1 = 0; point1 < points; point1++)
  {
    // Get position and orientation of point1
    long row1 = edge_rows[point1];
    long col1 = edge_cols[point1];
    double orientation1 = orientations[row1][col1];

    // Get votes for point1 from all other edge pixels
    for (point2 = 0; point2 < points; point2++)
    {
      // Get position and orientation of point2
      long row2 = edge_rows[point2];
      long col2 = edge_cols[point2];
      double orientation2 = orientations[row2][col2];
      // Get relative coordinates of point2 with respect to point1
      double rel_row = row2 - row1;
      double rel_col = col2 - col1;

      // Rotate rel_row and rel_col by an angle of -orientation1;
      vRotatePoint(rel_col, rel_row, -orientation1, rotated_row, rotated_col);
      // Rotate orientation2 by -orientation1 (we add 2*M_PI to get back 
      // something non-negative).
      double rotated_orientation2 = orientation2 + 2*M_PI - orientation1;
      // Get the vote and add it to the votes for point1..
      double vote = operator() (0, (long) rotated_row, (long) rotated_col, 
                                rotated_orientation2);
      result[row1][col1] += vote;
    }
  }

  return result_matrix;
}


// For each pixel get the number of votes it gets from the edge 
// statistics histogram. The difference between this function
// and Vote2 is that here we get votes for every pixel in the
// image, whereas in Vote2 we get votes for every edge pixel.
vMatrix<double> * vEdgeStatistics::Vote4(v3dMatrix<double> * orientation_image, 
                                           grayscale_image * edge_image)
{
  long rows = orientation_image->Rows();
  long cols = orientation_image->Cols();
  assert(edge_image->Rows() == rows);
  assert(edge_image->Cols() == cols);

  // Get the arrays for the orientations and the result
  vMatrix<double> * result_matrix = new vMatrix<double>(rows, cols);
  vWriteValue(result_matrix, (double) 0);
  vArray2(double) orientations = orientation_image->Matrix2(0);
  vArray2(double) result = result_matrix->Matrix2();

  // Get the edge points of the edge image
  vector<long> edge_rows, edge_cols;
  vGetNonZeroPoints(edge_image, &edge_rows, &edge_cols);
  long points = edge_rows.size();
  long point1;
  double rotated_row = 0;
  double rotated_col = 0;
  // Process all points
  for (point1 = 0; point1 < points; point1++)
  {
    // Get position and orientation of point1
    long row1 = edge_rows[point1];
    long col1 = edge_cols[point1];
    double orientation1 = orientations[row1][col1];

    long row2, col2;
    for (row2 = 0; row2 < rows; row2++)
    {
      for (col2 = 0; col2 < cols; col2++)
      {
        double orientation2 = orientations[row2][col2];
        // Get relative coordinates of point2 with respect to point1
        double rel_row = row2 - row1;
        double rel_col = col2 - col1;

        // Rotate rel_row and rel_col by an angle of -orientation1;
        vRotatePoint(rel_col, rel_row, -orientation1, rotated_row, rotated_col);
        // Rotate orientation2 by -orientation1 (we add 2*M_PI to get back 
        // something non-negative).
        double rotated_orientation2 = orientation2 + 2*M_PI - orientation1;
        // Get the vote and add it to the votes for point1..
        double vote = operator() (0, (long) rotated_row, (long) rotated_col, 
                                  rotated_orientation2);
        result[row2][col2] += vote;
      }
    }
  }

  return result_matrix;
}


// It assumes that the input is a gray-scale binary image (pixels
// have values 0 or 255). It returns
// a binary image whose pixels have the value 255 for the pixels 
// belonging to the largest foreground connected component in binary
// (i.e. the largest connected component whose pixels have the 
// value 255).
grayscale_image * vLargestForegroundComponent(v3dMatrix<uchar> * binary)
{
  // Our second approximation for the foreground is the biggest connected 
  // component of foreground1.
  vector<Label> foreground_labels;
  v3dMatrix<long> * foreground_components = 
    binary->ConnectedComponents(&foreground_labels);
  std::sort(foreground_labels.begin(), foreground_labels.end(), LabelMore());
  grayscale_image * result = (grayscale_image *)
    vSelectValue(foreground_components, foreground_labels[1].id);
  vdelete(foreground_components);
  return result;
}


// This function assumes that the three arguments are binary images,
// with pixel values either 0 or 255. foreground1 and foreground2
// are two different estimates for the foreground of some image.
// (their pixels are non-zero if they are estimated to be at the 
// foreground). This function returns a binary image that reconciles
// the two estimates, by deciding the fate of pixels for which the 
// two estimates differ. Those pixels are assigned in a way that 
// sets the foreground boundary as close as possible to edge pixels.
grayscale_image *  vPushBoundaryToEdges(v3dMatrix<uchar> * foreground1, 
                                     v3dMatrix<uchar> * foreground2, 
                                     v3dMatrix<uchar> * edges)
{
  // First do some error checking.
  long rows = foreground1->Rows();
  long cols = foreground1->Cols();
  long bands = foreground1->Bands();
  if (bands != 1) return 0;
  if (rows != foreground2->Rows()) return 0;
  if (cols != foreground2->Cols()) return 0;
  if (bands != foreground2->Bands()) return 0;
  if (rows != edges->Rows()) return 0;
  if (cols != edges->Cols()) return 0;
  if (bands != edges->Bands()) return 0;

  grayscale_image * result = new grayscale_image(rows, cols);
  v3dMatrix<float> * dt_image = vCVDistanceTransform(edges);
  vArray2(uchar) estimate2a = foreground1->Matrix2(0);
  vArray2(uchar) estimate2b = foreground2->Matrix2(0);
  vArray2(float) dt2 = dt_image->Matrix2(0);
  vArray2(uchar) result2 = result->Matrix2(0);

  vector<long> point_rows, point_cols, new_rows, new_cols;
  long row, col;
  // First, put in each pixel in result a single value that is 0
  // if both estimates agree that the pixel is background, 255 if
  // both estimates agree that it is foreground, and 1 if the
  // two estimates disagree. Furthermore, keep track of all those
  // pixels. To make things easy, pixels on which the estimates 
  // disagree that are on the boundary of the image are
  // assigned to background.
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      uchar estimate1 = estimate2a[row][col];
      uchar estimate2 = estimate2b[row][col];
      if (estimate1 == estimate2) result2[row][col] = estimate1;
      else 
      {
        if ((row != 0) && (row != rows-1) &&
            (col != 0) && (col != cols-1))
        {
          new_rows.push_back(row);
          new_cols.push_back(col);
          result2[row][col] = 1;
        }
        else
        {
          result2[row][col] = 0;
        }
      }
    }
  }

  // Now keep processing the points saved in new_rows and new_cols
  // until we have processed all points. At every iteration we
  // can only process unclassified points that have at least
  // one 8-neighbor that is either definite foreground or definite
  // background.
  long old_size = rows * cols + 1;
  while(new_rows.size() != 0)
  {
    // number_of_points should eventually get to 0. If it
    // didn't decrease from the previous iteration, then
    // something is wrong with this algorithm.
    long number_of_points = new_rows.size();
    if (number_of_points >= old_size)
    {
      exit_error("Something is wrong\n");
    }
    vPrint("number_of_points = %li\n", number_of_points);
    // Put points stored in new_rows, new_cols into
    // point_rows and point_cols, and erase new_rows and new_cols.
    point_rows.erase(point_rows.begin(), point_rows.end());
    point_cols.erase(point_cols.begin(), point_cols.end());
    long i;
    for (i = 0; i < number_of_points; i++)
    {
      point_rows.push_back(new_rows[i]);
      point_cols.push_back(new_cols[i]);
    }
    new_rows.erase(new_rows.begin(), new_rows.end());
    new_cols.erase(new_cols.begin(), new_cols.end());

    for (i = 0; i < number_of_points; i++)
    {
      row = point_rows[i];
      col = point_cols[i];
      double sum = 0.0;
      double dt = dt2[row][col];
      if (dt < 1.1)
      {
        result2[row][col] = 255;
        continue;
      }
      long row2, col2;
      long to_process = 0;
      // Note that this for loop does not skip the
      // center pixel (i.e (row,col)), but that's
      // OK, for that pixel the factor is always 0.
      for (row2 = row-1; row2 <= row+1; row2++)
      {
        for (col2 = col-1; col2 <= col+1; col2++)
        {
          long factor = 1;
          switch(result2[row2][col2])
          {
          case 1:
            factor = 0;
            break;

          case 0: 
            factor = -1;
            to_process = 1;
            break;

          case 255:
            factor = 1;
            to_process = 1;
            break;

          default:
            exit_error("Something is wrong\n");
            break;
          }
          if (factor == 0) continue;
          double dt_diff = dt2[row2][col2] - dt;
          sum += factor * dt_diff;
        }
      }
      // If we didn't find any 8-neighbor who is 
      // definitely foreground or background, we 
      // can't process this pixel yet.
      if (to_process == 0)
      {
        new_rows.push_back(row);
        new_cols.push_back(col);
      }
      else
      {
        if (sum >= 0) result2[row][col] = 255;
        else result2[row][col] = 0;
      }
    }
  }

  vdelete(dt_image);
  return result;
}


// Trim from the foreground all pixels within distance length 
// of the boundary contour. We assume that foreground is a binary
// image (values are 0 or 255).
grayscale_image * vTrimBoundary(v3dMatrix<uchar> * foreground, long length)
{
  if (foreground->Bands() != 1) return 0;
  v3dMatrix<uchar> * background = vCopyImage(foreground);
  vInverseImage(background);
  v3dMatrix<float> * dt_image = vCVDistanceTransform(background);
  vdelete(background);

  long rows = foreground->Rows();
  long cols = foreground->Cols();
  grayscale_image * result_matrix = new grayscale_image(rows, cols);
  
  vArray(uchar) values = foreground->Matrix();
  vArray(uchar) result = result_matrix->Matrix();
  vArray(float) distances = dt_image->Matrix();
  long size = rows * cols;

  long i;
  for (i = 0; i < size; i++)
  {
    if (values[i] == 0)
    {
      result[i] = 0;
      continue;
    }
    else if ((distances[i]-0.001) < length) 
    // We subtract .001 to avoid rounding errors
    {
      result[i] = 0;
    }
    else
    {
      result[i] = 255;
    }
  }
  vdelete(dt_image);
  return result_matrix;
}



v3dMatrix<long> * vShadowScore(v3dMatrix<double> * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if (the_image->Bands() != 1) return 0;

  v3dMatrix<long> * result = new v3dMatrix<long>(rows, cols, 1);
  vArray2(double) color_image2 = the_image->Matrix2(0);
  vArray2(long) result2 = result->Matrix2(0);

  vWriteBorderValue(result, (long) 0);
  long row, col;
  double horizontal = 0, vertical = 0;
  for (row = 1; row < rows - 1; row++)
  {
    for (col = 1; col < cols - 1; col++)
    {
//      if ((row == 89) && (col == 49))
//      {
//        vPrint("nothing\n");
//      }
      vPixelGradient2(color_image2, row, col, horizontal, vertical);
      if ((horizontal == 0) && (vertical == 0))
      {
        result2[row][col] = 0;
      }
      double temp_sum = sqrt(horizontal * horizontal + vertical * vertical);
      double row_step = -vertical / temp_sum;
      double col_step = -horizontal / temp_sum;
      double previous_value = color_image2[row][col];
      double next_drow = row, next_dcol = col;
      while(1)
      {
        long next_row = (long) next_drow;
        long next_col = (long) next_dcol;
        if ((next_row < 1) || (next_col < 1) || 
            (next_row >= rows-1) || (next_col >= cols-1))
        {
          break;
        }
        double x = next_dcol - next_col;
        double y = next_drow - next_row;
        long bottom = next_row+1;
        long right = next_col+1;
        double top_left = color_image2[next_row][next_col];
        double top_right = color_image2[next_row][right];
        double bottom_left = color_image2[bottom][next_col];
        double bottom_right = color_image2[bottom][right];
        double next_value = vBilinearInterpolation(x, y, top_left, top_right,
                                                    bottom_left, bottom_right);
        if (next_value > previous_value) break;
        previous_value = next_value;
        next_drow += row_step;
        next_dcol += col_step;
      }
      long row_diff = (long) next_drow - row;
      long col_diff = (long) next_dcol - col;
//      if ((row == 89) && (col == 49))
//      {
//        vPrint("nothing\n");
//      }
      result2[row][col] = (long) sqrt((float) (row_diff * row_diff + col_diff * col_diff));
    }
  }
  return result;
}


v3dMatrix<long> * vLocalShadowScore(v3dMatrix<double> * the_image, 
                                       long row, long col)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  if ((row < 1) || (col < 1) || (row >= rows-1) || (col >= cols-1))
  {
    return 0;
  }
  if (the_image->Bands() != 1) 
  {
    return 0;
  }

  double horizontal = 0;
  double vertical = 0;

  v3dMatrix<long> * result = new v3dMatrix<long>(rows, cols, 1);
  vArray2(double) color_image2 = the_image->Matrix2(0);
  vArray2(long) result2 = result->Matrix2(0);

  vWriteValue(result, (long) 0);
  vPixelGradient2(color_image2, row, col, horizontal, vertical);
  if ((horizontal == 0) && (vertical == 0))
  {
    result2[row][col] = 0;
  }
  double temp_sum = sqrt(horizontal * horizontal + vertical * vertical);
  double row_step = -vertical / temp_sum;
  double col_step = -horizontal / temp_sum;
  double previous_value = color_image2[row][col];
  long row_diff, col_diff;
  double next_drow = row, next_dcol = col;
  while(1)
  {
    long next_row = (long) next_drow;
    long next_col = (long) next_dcol;
    if ((next_row < 1) || (next_col < 1) || 
        (next_row >= rows-1) || (next_col >= cols-1))
    {
      break;
    }
    double x = next_dcol - next_col;
    double y = next_drow - next_row;
    long bottom = next_row+1;
    long right = next_col+1;
    double top_left = color_image2[next_row][next_col];
    double top_right = color_image2[next_row][right];
    double bottom_left = color_image2[bottom][next_col];
    double bottom_right = color_image2[bottom][right];
    double next_value = vBilinearInterpolation(x, y, top_left, top_right,
                                                bottom_left, bottom_right);
    if (next_value > previous_value) break;
    row_diff = (long) next_drow - row;
    col_diff = (long) next_dcol - col;
    result2[next_row][next_col] = (long) sqrt((float) (row_diff * row_diff + col_diff * col_diff));
    previous_value = next_value;
    next_drow += row_step;
    next_dcol += col_step;
  }
  row_diff = (long) next_drow - row;
  col_diff = (long) next_dcol - col;
  result2[row][col] = (long) sqrt((float) (row_diff * row_diff + col_diff * col_diff));

  return result;
}



// Returns the angle (in radians) of the principal orientation of the 
// shape in binary (i.e. of the non-zero pixels). The location of the
// center of mass of the shape is placed into mean_row and mean_col.
double vPrincipalOrientation(grayscale_image * binary, 
                              double * mean_row_p, double * mean_col_p)
{
  vArray2(uchar) binary2 = binary->Matrix2(0);
  long row, col;
  long counter = 0;
  long row_sum, col_sum;
  row_sum = 0;
  col_sum = 0;
  long rows = binary->Rows();
  long cols = binary->Cols();
  // Find mean row and col.
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      if (binary2[row][col] == 0)
      {
        continue;
      }
      counter++;
      row_sum += row;
      col_sum += col;
    }
  }
  if (counter == 0)
  {
    *mean_row_p = -1;
    *mean_col_p = -1;
    return 0;
  }
  
  double mean_row = ((double) row_sum) / (double) counter;
  double mean_col = ((double) col_sum) / (double) counter;
  *mean_row_p = mean_row;
  *mean_col_p = mean_col;

  // Now build the covariance matrix
  vMatrix<double> covariance_matrix(2,2);
  vArray2(double) covariance = covariance_matrix.Matrix2();
  covariance[0][0] = covariance[0][1] = covariance[1][1] = 0;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      if (binary2[row][col] == 0) continue;
      double centered_row = ((double) row) - mean_row;
      double centered_col = ((double) col) - mean_col;
      covariance[0][0] += centered_col * centered_col;
      covariance[0][1] += centered_row * centered_col;
      covariance[1][1] += centered_row * centered_row;
    }
  }

  covariance[1][0] = covariance[0][1];
  vMatrix<double> eigenvectors = v2x2Eigenvectors(&covariance_matrix);
  // Handle the case where the covariance matrix is rank deficient.
  if ((eigenvectors(0, 0) == 0) && (eigenvectors(1, 0) == 0) &&
      (eigenvectors(0, 1) == 0) && (eigenvectors(1, 1) == 0))
  {
    *mean_row_p = -1;
    *mean_col_p = -1;
    return 0;
  }
  else if ((eigenvectors(0, 0) == 0) && (eigenvectors(1, 0) == 0))
  {
    double result = vAngle2(eigenvectors(0,1), eigenvectors(1,1));
    return result;
  }
  else if ((eigenvectors(0, 1) == 0) && (eigenvectors(1, 1) == 0))
  {
    double result = vAngle2(eigenvectors(0,0), eigenvectors(1,0));
    return result;
  }

  // Figure out which is the eigenvector that corresponds to the largest
  // eigenvalue. We can easily get the eigenvalues by multiplying
  // the eigenvectors by the covariance matrix and checking how
  // they got amplified.
  vMatrix<double> amplifications = covariance_matrix * eigenvectors;
  double eigenvalue1, eigenvalue2;
  if (vAbs(eigenvectors(0, 0)) > vAbs(eigenvectors(1, 0)))
  {
    eigenvalue1 = vAbs(amplifications(0, 0) / eigenvectors(0, 0));
  }
  else
  {
    eigenvalue1 = vAbs(amplifications(1, 0) / eigenvectors(1, 0));
  }

  if (vAbs(eigenvectors(0, 1)) > vAbs(eigenvectors(1, 1)))
  {
    eigenvalue2 = vAbs(amplifications(0, 1) / eigenvectors(0, 1));
  }
  else
  {
    eigenvalue2 = vAbs(amplifications(1, 1) / eigenvectors(1, 1));
  }

  double result;
  if (eigenvalue1 > eigenvalue2)
  {
    result = vAngle2(eigenvectors(0,0), eigenvectors(1,0));
  }
  else
  {
    result = vAngle2(eigenvectors(0,1), eigenvectors(1,1));
  }
  return result;
}


// Suppose we have an image of size rows x cols, and a line
// that goes through (row, col) whose orientation is specified
// by angle. Where will that line intersect the boundary of the
// image? This function computes the answer and puts the 
// two intersection points into (row1_p, col1_p) and
// (row2_p, col2_p). The function returns 1 if successful. It
// returns 0 when it fails. It fails when (row, col) is not
// inside the image.
long vBoundaryPoints(double row, double col, double angle, 
                      long rows, long cols, 
                      double * row1_p, double * col1_p,
                      double * row2_p, double * col2_p)
{
  if (row < 0) return 0;
  if (col < 0) return 0;
  if (row > rows - 1) return 0;
  if (col > cols - 1) return 0;

  // check if the tangent is defined for that angle (it is not
  // defined numerically if the angle is too close to PI/2 or to
  // 3*PI/2. If not, we can easily compute the answer.
  double distance1, distance2;
  distance1 = vAbs(vCircularDistance(angle, M_PI/2, 2*M_PI));
  distance2 = vAbs(vCircularDistance(angle, 3*M_PI/2, 2*M_PI));
  if ((distance1 < 0.0001) || (distance2 < 0.0001))
  {
    *row1_p = 0;
    *col1_p = col;
    *row2_p = rows-1;
    *col2_p = col;
    return 1;
  }

  // Check if the angle is almost horizontal, because then
  // we can also solve easily.
  distance1 = vAbs(vCircularDistance(angle, (double) 0, (double) M_PI));
  if (distance1 < 0.0001)
  {
    *row1_p = row;
    *col1_p = 0;
    *row2_p = row;
    *col2_p = cols-1;
    return 1;
  }

  // Otherwise the tangent is defined, and we can solve using the
  // line equation (y - row) = tangent * (x - col). We find
  // the intersection of the line with the four sides of the image
  // (all sides intersect the line, since the angle is not horizontal
  // or vertical).
  double tangent = tan(angle);

  // Intersection with top line
  double top_row = 0;
  double top_col = (top_row  - row + tangent * col) / tangent;
  if (vAbs(top_col) < 0.0001) top_col = 0;
  if (vAbs(top_col - (cols - 1)) < 0.0001) top_col = cols-1;

  double bottom_row = rows-1;
  double bottom_col = (bottom_row  - row + tangent * col) / tangent;
  if (vAbs(bottom_col) < 0.0001) bottom_col = 0;
  if (vAbs(bottom_col - (cols - 1)) < 0.0001) bottom_col = cols-1;

  double left_col = 0;
  double left_row = tangent * left_col - tangent * col + row;
  if (vAbs(left_row) < 0.0001) left_row = 0;
  if (vAbs(left_row - (rows - 1)) < 0.0001) left_row = rows - 1;

  double right_col = cols-1;
  double right_row = tangent * right_col - tangent * col + row;
  if (vAbs(bottom_row) < 0.0001) bottom_row = 0;

  double rows_array[4] = {top_row, right_row, bottom_row, left_row};
  double cols_array[4] = {top_col, right_col, bottom_col, left_col};

  long counter = 0;
  double result_rows[2], result_cols[2];
  long i;
  // Find the intersection points that are inside the image and return them.
  // Special care should be taken for cases where the line intersects
  // the image boundary on a corner (and therefore the intersection point
  // shows up twice in rows_array and cols_array).
  for (i = 0; i < 4; i++)
  {
    if ((rows_array[i] < 0) || (rows_array[i] > (rows-1)) ||
        (cols_array[i] < 0) || (cols_array[i] > (cols-1)))
    {
      continue;
    }
    if (counter == 1)
    {
      if ((rows_array[i] == result_rows[0]) &&
          (cols_array[i] == result_cols[0]))
      {
        continue;
      }
    }
    result_rows[counter] = rows_array[i];
    result_cols[counter] = cols_array[i];
    counter++;
    if (counter == 2)
    {
      break;
    }
  }

  if (counter == 1)
  {
    result_rows[1] = result_rows[0];
    result_cols[1] = result_cols[0];
  }

  if (counter < 1)
  {
    vPrint("Something went wrong in vBoundaryPoints. Arguments were:\n");
    exit_error("%lf %lf %lf %li %li\n", row, col, angle, rows, cols);
  }

  *row1_p = result_rows[0];
  *col1_p = result_cols[0];
  *row2_p = result_rows[1];
  *col2_p = result_cols[1];
  return 1;
}


// This is an interactive function that takes in row, col, angle,
// rows and cols from the user, and it calls vBoundaryPoints.
// This way we can easily check that vBoundaryPoints is correct.
// The testing stops when the user enters -1 for row.
long vTestBoundaryPoints()
{
  double row, col, angle;
  long rows, cols;
  row = col = angle = rows = cols = 0;
  while(1)
  {
    vPrint("Enter row, col, angle, rows, cols: ");
    vScan("%lf", &row);
    if (row == -1) break;
    vScan("%lf %lf %li %li\n", &col, &angle, &rows, &cols);
    double row1, row2, col1, col2;
    row1 = row2 = col1 = col2 = 0;
    vBoundaryPoints(row, col, angle, rows, cols, &row1, &col1, &row2, &col2);
    vPrint("(%.1lf, %.1lf), (%.1lf, %.1lf)\n\n", row1, col1, row2, col2);
  }
  return 1;
}


// Finds heuristically the cutoff point where the palm is separated
// from the rest of the arm. The heuristic algorithm is the following:
// - Find the principal axis of the shape
// - For each point P of the axis, count Proj(P), the number of pixels
//   from the shape that project on that axis
// - Find the point W of max width, i.e. the point W on the axis where
//   Proj(W) is maximized
// - Starting at W and going away from the center of the image, find
//   C, the point of least width, i.e. the point where Proj(C) is
//   minimized.
// - Draw a line perpendicilar to the principal axis that goes through
//   C, and find the first and last pixel of the shape that belong
//   to that line.
// - We return the locations of those two pixels. Basically, everything
//   on one side of the line connecting those pixels (the side that
//   is towards the center of the image) is the palm, everything else
//   is rejected.
// 
// We also store in hand_pixel_row and hand_pixel_col the coordinates
// of a point that is on the palm side of the cutoff line, so that
// the caller of vPalmCutpoint knows which way the palm lies.
/*
long vPalmCutpoint(grayscale_image * binary, long * row1_p, long * col1_p,
                    long * row2_p, long * col2_p,
                    long * hand_pixel_row, long * hand_pixel_col)
{
  double mean_row, mean_col;
  mean_row = mean_col = 0;
  // Get the principal orientation
  double orientation = vPrincipalOrientation(binary, &mean_row, &mean_col);
  double d_row1, d_col1, d_row2, d_col2;
  d_row1 = d_col1 = d_row2 = d_col2 = 0;
  long rows = binary->Rows();
  long cols = binary->Cols();

  // Get the intersection of principal axis with image boundary.
  long success = vBoundaryPoints(mean_row, mean_col, orientation, rows, cols,
                                  &d_row1, &d_col1, &d_row2, &d_col2);
  long row1 = vRound(d_row1);
  long col1 = vRound(d_col1);
  long row2 = vRound(d_row2);
  long col2 = vRound(d_col2);
  
  // For each point on the principal axis, find the number of hand pixels
  // that project there.
  // First, figure out the step at which we will sample points.
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(row1, col1, row2, col2, &row_step, &col_step, &steps);
  if (steps == 0) return 0;

  // So, we will consider steps points on the principal axis, and for
  // each of them we will count the number of hand pixels that get
  // projected there. The counters array will keep track of all those
  // counts.

  double perpendicular = orientation + M_PI/2;
  vArray(long) counters = vnew(long, steps);
  long step;
  for (step = 0; step < steps; step++)
  {
    double d_axis_point_row = d_row1 + step * row_step;
    double d_axis_point_col = d_col1 + step * col_step;
    long axis_point_row = vRound(d_axis_point_row);
    long axis_point_col = vRound(d_axis_point_col);
    long boundary_flag = 0;
    counters[step] = vCountProjectedPoints(binary, axis_point_row, 
                                            axis_point_col, perpendicular,
                                            &boundary_flag);
    if (boundary_flag != 0) 
    {
      counters[step] = 0;
    }
  }
    
  long max_width = -1;
  long max_width_index = -1;
  long first_non_zero = -1;
  long last_non_zero = -1;
  // Find the point of max width along the principal axis. Also, find
  // the first point of non-zero width and the last point of non-zero width.
  for (step = 0; step < steps; step++)
  {
    long counter = counters[step];
    if (counter > max_width)
    {
      max_width = counter;
      max_width_index = step;
    }
    if (counter > 0) 
    {
      last_non_zero = step;
      if (first_non_zero == -1)
      {
        first_non_zero = step;
      }
    }
  }

  // Figure out the cutoff point. First we must figure out if, from
  // max_width_index we should go up or down.
  long direction_step, limit;
  long difference1 = vAbs(first_non_zero - steps/2);
  long difference2 = vAbs(last_non_zero - steps/2);
  if (difference1 > difference2) 
  {
    direction_step = -1;
    limit = first_non_zero - 1;
    *hand_pixel_row = vRound(d_row1 + last_non_zero * row_step);
    *hand_pixel_col = vRound(d_col1 + last_non_zero * col_step);
  }
  else 
  {
    direction_step = 1;
    limit = last_non_zero + 1;
    *hand_pixel_row = vRound(d_row1 + first_non_zero * row_step);
    *hand_pixel_col = vRound(d_col1 + first_non_zero * col_step);
  }

  // From max_width point go towards direction_step and find the min
  // width;
  const long blur_window = 4;
  if ((direction_step == 1) && (max_width < 2*blur_window))
  {
    max_width = 2 * blur_window;
  }
  else if ((direction_step == -1) && (max_width > limit-2*blur_window-1))
  {
    max_width = limit-2*blur_window-1;
  }

  long cutoff_index = max_width_index;

  if (difference2 >= 2*blur_window)
  {
    // compute first and second derivatives of width
    long index;
    vArray(long) blurred_counters = vnew(long, steps);
    vArray(long) first_derivatives = vnew(long, steps);
    vArray(long) second_derivatives = vnew(long, steps); 
    for (index = blur_window; index < steps - blur_window; index++)
    {
      long index2;
      long sum = 0;
      for (index2 = -blur_window; index2 <= blur_window; index2++)
      {
        sum += counters[index+index2];
      }
      blurred_counters[index] = sum;
    }
    for (index = blur_window+1; index < steps - blur_window-1; index++)
    {
      first_derivatives[index] = blurred_counters[index+1] - blurred_counters[index-1];
    }

    for (index = blur_window+2; index < steps - 2*blur_window-2; index++)
    {
      second_derivatives[index] = 
        first_derivatives[index+1] - first_derivatives[index-1];
    }

    long max_second_deriv = -1000;
    long max_second_deriv_index = max_width_index;
    for (index = max_width_index+direction_step*2*blur_window; 
         index != limit-2*blur_window*direction_step; 
         index += direction_step)
    {
      long second_deriv = second_derivatives[index];
      double index_row = d_row1 + index * row_step;
      double index_col = d_col1 + index * col_step;
      vPrint("(%.0lf, %.0lf), deriv1 = %li, deriv2 = %li, blurred_counter = %li\n",
              index_row, index_col, 
              first_derivatives[index]*direction_step,
              second_derivatives[index], blurred_counters[index]);
      if ((second_deriv >= max_second_deriv) && 
          (counters[index+2*blur_window*direction_step] > 0) &&
          (first_derivatives[index]*direction_step > -3) &&
          (first_derivatives[index-direction_step]*direction_step < 0))
      {
        max_second_deriv = second_deriv;
        max_second_deriv_index = index;
        vPrint("Got it\n\n");
      }
    }
    cutoff_index = max_second_deriv_index;
    vdelete2(first_derivatives);
    vdelete2(second_derivatives);
    vdelete2(blurred_counters);
  }

  if (cutoff_index == max_width_index) cutoff_index = limit-blur_window*direction_step;
  // Now find the intersection between the perpendicular at the 
  // min_width_index point and the contour of the shape. If
  // the shape contour is intersected in more than two points, we
  // pick the pair of points that are the most distant.
  double cutoff_row, cutoff_col;
  cutoff_row = d_row1 + cutoff_index * row_step;
  cutoff_col = d_col1 + cutoff_index * col_step;
  vLineContourIntersection(binary, cutoff_row, cutoff_col, 
                            perpendicular, row1_p, col1_p, row2_p, col2_p);
                            
  vdelete2(counters);
  return 1;
}
*/



long vPalmCutpoint(grayscale_image * binary, long * row1_p, long * col1_p,
                    long * row2_p, long * col2_p,
                    long * hand_pixel_row, long * hand_pixel_col)
{
  const long blur_window = 4;
  const long two_windows = 2 * blur_window; 

  double mean_row, mean_col;
  mean_row = mean_col = 0;
  // Get the principal orientation
  double orientation = vPrincipalOrientation(binary, &mean_row, &mean_col);
  double d_row1, d_col1, d_row2, d_col2;
  d_row1 = d_col1 = d_row2 = d_col2 = 0;
  long rows = binary->Rows();
  long cols = binary->Cols();

  // Get the intersection of principal axis with image boundary.
  long success = vBoundaryPoints(mean_row, mean_col, orientation, rows, cols,
                                  &d_row1, &d_col1, &d_row2, &d_col2);
  long row1 = vRound(d_row1);
  long col1 = vRound(d_col1);
  long row2 = vRound(d_row2);
  long col2 = vRound(d_col2);
  
  // For each point on the principal axis, find the number of hand pixels
  // that project there.
  // First, figure out the step at which we will sample points.
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(row1, col1, row2, col2, &row_step, &col_step, &steps);
  if (steps <= two_windows * 2 + 6) return 0;

  // So, we will consider steps points on the principal axis, and for
  // each of them we will count the number of hand pixels that get
  // projected there. The counters array will keep track of all those
  // counts.

  double perpendicular = orientation + M_PI/2;
  vArray(long) counters = vnew(long, steps);
  long step;
  for (step = 0; step < steps; step++)
  {
    double d_axis_point_row = d_row1 + step * row_step;
    double d_axis_point_col = d_col1 + step * col_step;
    long axis_point_row = vRound(d_axis_point_row);
    long axis_point_col = vRound(d_axis_point_col);
    long boundary_flag = 0;
    counters[step] = vCountProjectedPoints(binary, axis_point_row, 
                                            axis_point_col, perpendicular,
                                            &boundary_flag);
    if (boundary_flag != 0) 
    {
      counters[step] = 0;
    }
  }
    
  long first_non_zero = -1;
  long last_non_zero = -1;
  // Find the point of max width along the principal axis. Also, find
  // the first point of non-zero width and the last point of non-zero width.
  for (step = 0; step < steps; step++)
  {
    if (counters[step] > 0) 
    {
      last_non_zero = step;
      if (first_non_zero == -1)
      {
        first_non_zero = step;
      }
    }
  }

  if (first_non_zero == -1)
  {
    vdelete2(counters);
    return 0;
  }


  // Figure out which end is the palm end. The heuristic is that the endpoint
  // of the axis that is closer to the center of the image belongs to the palm.
  // Another heuristic would be to see which endpoint is closer to the boundary
  // (that should be the arm end).
  double first_non_zero_row = d_row1 + first_non_zero * row_step;
  double first_non_zero_col = d_col1 + first_non_zero * col_step;
  double last_non_zero_row = d_row1 + last_non_zero * row_step;
  double last_non_zero_col = d_col1 + last_non_zero * col_step;
  double center_row = (rows - 1.0) / 2.0;
  double center_col = (cols - 1.0) / 2.0;

  double first_distance = vSquareDistance(first_non_zero_row, first_non_zero_col,
                                           center_row, center_col);
  double last_distance = vSquareDistance(last_non_zero_row, last_non_zero_col,
                                           center_row, center_col);
  
  double palm_end_row, palm_end_col, palm_end_index;
  double arm_end_row, arm_end_col, arm_end_index;

  if (first_distance < last_distance)
  {
    palm_end_row = first_non_zero_row;
    palm_end_col = first_non_zero_col;
    palm_end_index = first_non_zero;
    arm_end_row = last_non_zero_row;
    arm_end_col = last_non_zero_col;
    arm_end_index = last_non_zero;
  }
  else
  {
    palm_end_row = last_non_zero_row;
    palm_end_col = last_non_zero_col;
    palm_end_index = last_non_zero;
    arm_end_row = first_non_zero_row;
    arm_end_col = first_non_zero_col;
    arm_end_index = first_non_zero;
  }

  *hand_pixel_row = (long) palm_end_row;
  *hand_pixel_col = (long) palm_end_col;

  // One thing we must settle is, within what range should we look for
  // the max width of the palm and for the cutpoint?
  // min_cutpoint and max_cutpoint define that range, making
  // sure that only the portion of the axis with non-zero projections
  // is considered and that the first and second derivatives are 
  // well defined for the points that are considered.
  long min_index = Max<long>(first_non_zero, blur_window + 2);
  long max_index = Min<long>(last_non_zero, steps - blur_window - 3);

  // adjust palm_end_index and arm_end_index so they are within
  // [min_index, ..., max_index];
  if (palm_end_index < min_index) palm_end_index = min_index;
  else if (palm_end_index > max_index) palm_end_index = max_index;
  if (arm_end_index < min_index) arm_end_index = min_index;
  else if (arm_end_index > max_index) arm_end_index = max_index;
  
  // direction will be 1 if palm_index is less than arm_index,
  // and it will be -1 otherwise. In general, direction will
  // point to the way palm_index must be altered (incremented
  // or decremented) in order to get closer to arm_index.
  long direction;
  if (palm_end_index < arm_end_index) direction = 1;
  else direction = -1;

  // We blur counters so that little jitters in the contour
  // due to segmentation errors will not influence the 
  // calculations too much.
  vArray(long) blurred_counters = vnew(long, steps);
  vArray(long) first_derivatives = vnew(long, steps);
  vArray(long) second_derivatives = vnew(long, steps); 
  long index;
  for (index = blur_window; index < steps - blur_window; index++)
  {
    long index2;
    long sum = 0;
    for (index2 = -blur_window; index2 <= blur_window; index2++)
    {
      sum += counters[index+index2];
    }
    blurred_counters[index] = sum;
  }
  for (index = blur_window+1; index < steps - blur_window-1; index++)
  {
    first_derivatives[index] = blurred_counters[index+1] - blurred_counters[index-1];
  }

  for (index = blur_window+2; index < steps - blur_window-2; index++)
  {
    second_derivatives[index] = 
      first_derivatives[index+1] - first_derivatives[index-1];
  }


  // Figure out the position of maximum width of the palm.
  // The heuristic is as follows: We start at the palm end of
  // the axis, and we walk up to the middle. If widths are 
  // still increasing then and the middle is the max width, 
  // we walk up until we get a decrease.
  long middle_index = (long) ((palm_end_index + arm_end_index) * .6 + 1);
  long max_width = -1;
  long max_width_index = -1;
  // First find max_width up to middle_index;
  for (index = (long) palm_end_index; index != middle_index; index += direction)
  {
    long counter = counters[index];
    if (counter >= max_width)
    {
      max_width = counter;
      max_width_index = index;
    }
  }

  // If max_width_index is middle_index - 1, keep looking until widths
  // start decreasing
  if (max_width_index == middle_index - 1)
  {
    while(counters[index] >= max_width)
    {
      max_width = counters[index];
      max_width_index = index;
      index++;
    }
  }

  long max_second_deriv = -1000;
  long max_second_deriv_index = max_width_index;
  long starting_point = max_width_index + direction * two_windows;
  // Adjust the starting point 
     
  long cutoff_index = max_width_index;

  if (max_index > min_index)
  {
    if (starting_point < min_index) starting_point = min_index;
    if (starting_point > max_index) starting_point = max_index;

    long checked_points_limit = max_index - min_index+ 1;
    for (index = starting_point;
         index != arm_end_index;
         index += direction)
    {
      long second_deriv = second_derivatives[index];
      double index_row = d_row1 + index * row_step;
      double index_col = d_col1 + index * col_step;
      vPrint("(%.0lf, %.0lf), deriv1 = %li, deriv2 = %li, blurred_counter = %li\n",
              index_row, index_col, 
              first_derivatives[index]*direction,
              second_derivatives[index], blurred_counters[index]);
      if ((second_deriv >= max_second_deriv) && 
//          (counters[index+2*blur_window*direction] > 0) &&
          (first_derivatives[index]*direction> -3) &&
          (first_derivatives[index-direction]*direction < 0))
      {
        max_second_deriv = second_deriv;
        max_second_deriv_index = index;
        vPrint("Got it\n\n");
      }
    }   

    cutoff_index = max_second_deriv_index;
  }
  vdelete2(first_derivatives);
  vdelete2(second_derivatives);
  vdelete2(blurred_counters);

  if (cutoff_index == max_width_index) cutoff_index = (long) arm_end_index;
  // Now find the intersection between the perpendicular at the 
  // min_width_index point and the contour of the shape. If
  // the shape contour is intersected in more than two points, we
  // pick the pair of points that are the most distant.
  double cutoff_row, cutoff_col;
  cutoff_row = d_row1 + cutoff_index * row_step;
  cutoff_col = d_col1 + cutoff_index * col_step;
  vLineContourIntersection(binary, (long) cutoff_row, (long) cutoff_col, 
                            perpendicular, row1_p, col1_p, row2_p, col2_p);
                            
  vdelete2(counters);
  return 1;
}


// vKeepPalm uses vPalmCutpoint to figure out where the palm should
// be separated from the rest of the arm, and returns an image
// containing just the palm (zeros are put everywhere else).
grayscale_image * vKeepPalm(grayscale_image * binary)
{
  long rows = binary->Rows();
  long cols = binary->Cols();
  long row1, col1, row2, col2, hand_pixel_row, hand_pixel_col;
  row1 = col1 = row2 = col2 = 0;
  vPalmCutpoint(binary, &row1, &col1, &row2, &col2,
                 &hand_pixel_row, &hand_pixel_col);
  if ((row1 == row2) && (col1 == col2)) return binary->ToGrayCopy();
  double dx = col2 - col1;
  double dy = row2 - row1;
  double angle = vAngle2(dx, dy);
  double row3, col3, row4, col4;
  row3 = col3 = row4 = col4 = 0;
  vBoundaryPoints(row1, col1, angle, rows, cols, 
                   &row3, &col3, &row4, &col4);

  // Create a black image and draw the cutoff line on that image,
  // so that it cuts the image into two connected components.
  grayscale_image * boundary_sides = new grayscale_image(rows, cols);
  vWriteValue(boundary_sides, (uchar) 255);
  vDrawLine1(boundary_sides, (long) row3, (long) col3, (long) row4, (long) col4, (uchar) 0);
  vector<Label> labels;
  v3dMatrix<long> * components = boundary_sides->ConnectedComponents(&labels);
  long palm_label = components->Matrix2(0)[hand_pixel_row][hand_pixel_col];
  grayscale_image * palm = (grayscale_image *) vSelectValue(components, palm_label);
  vdelete(components);
  vdelete(boundary_sides);
  return palm;
}

  
long vPalmCutpoint2(grayscale_image * binary, long * row1_p, long * col1_p,
                     long * row2_p, long * col2_p,
                     long * hand_pixel_row, long * hand_pixel_col)
{
  const long blur_window = 4;
  const long two_windows = 2 * blur_window; 

  vAxisProjection projection(binary);
  long axis_points = projection.number_of_points;
  if (axis_points == 0) return 0;

  // Figure out which end is the palm end. The heuristic is that the endpoint
  // of the axis that is closer to the center of the image belongs to the palm.
  // Another heuristic would be to see which endpoint is closer to the boundary
  // (that should be the arm end).
  long first_row = projection.rows[0];
  long first_col = projection.cols[0];
  long last_row = projection.rows[axis_points-1];
  long last_col = projection.cols[axis_points-1];
  long rows = binary->Rows();
  long cols = binary->Cols();
  long center_row = (rows - 1) / 2;
  long center_col = (cols - 1) / 2;

  long first_distance = (long) vSquareDistance(first_row, first_col,
                                         center_row, center_col);
  long last_distance = (long) vSquareDistance(last_row, last_col,
                                        center_row, center_col);
  
  // Make sure that the points in the projection start from the palm.
  if (first_distance > last_distance)
  {
    projection.Reverse();
  }

  *hand_pixel_row = projection.rows[0];
  *hand_pixel_col = projection.cols[0];

  // Figure out the position of maximum width of the palm.
  // The heuristic is as follows: We start at the palm end of
  // the axis, and we walk up to the middle. If widths are 
  // still increasing then and the middle is the max width, 
  // we walk up until we get a decrease.
  long middle_index = axis_points / 2;
  long max_width = -1;
  long max_width_index = -1;
  // First find max_width up to middle_index;
  long index;
  for (index = 0; index != middle_index; index++)
  {
    long counter = (long) projection.counts[index];
    if (counter >= max_width)
    {
      max_width = counter;
      max_width_index = index;
    }
  }

  // If max_width_index is middle_index - 1, keep looking until widths
  // start decreasing
  if (max_width_index == middle_index - 1)
  {
    while(projection.counts[index] >= max_width)
    {
      max_width = (long) projection.counts[index];
      max_width_index = index;
      index++;
    }
  }

  long limit_index = (long) (max_width_index + .75 * max_width);
  if (limit_index >= axis_points) limit_index = axis_points - 1;

  // Find the min width between max_width_index and limit_index;
  long min_width = max_width;
  long min_index = max_width_index;
  for (index = max_width_index; index <= limit_index; index++)
  {
    long current_width = (long) projection.counts[index];
    if (current_width < min_width)
    {
      min_width = current_width;
      min_index = index;
    }
  }
  
  long cutoff_index = min_index;
  double cutoff_row, cutoff_col;
  cutoff_row = projection.rows[cutoff_index];
  cutoff_col = projection.cols[cutoff_index];
  double perpendicular = projection.orientation + M_PI/2;
  vLineContourIntersection(binary, (long) cutoff_row, (long) cutoff_col, 
                            perpendicular, row1_p, col1_p, row2_p, col2_p);
  return 1;
}


long vPalmCutpoint3(v3dMatrix<float> * values, long * row1_p, long * col1_p,
                     long * row2_p, long * col2_p,
                     long * hand_pixel_row, long * hand_pixel_col)
{
  const long blur_window = 4;
  const long two_windows = 2 * blur_window; 

  grayscale_image * binary = (grayscale_image *) values->threshold(1);
  vAxisProjection projection2(values);
  long axis_points = projection2.number_of_points;
  if (axis_points == 0) return 0;

  // Figure out which end is the palm end. The heuristic is that the half
  // where the hand is is the half that receives higher sums in the
  // axis projection.
  long half = axis_points / 2;
  float sum1 = 0, sum2 = 0;
  long i;
  for (i = 0; i < half; i++)
  {
    sum1 = sum1 + projection2.counts[i];
    sum2 = sum2 + projection2.counts[axis_points - i - 1];
  }
  float mean1 = sum1 / (float) half;
  float mean2 = sum2 / (float) half;

  vPrint("mean1 = %f, mean2 = %f\n", mean1, mean2);

  vAxisProjection projection(binary);
  // Make sure that the points in the projection start from the palm.
  if (mean1 < mean2)
  {
    projection.Reverse();
  }

  axis_points = projection.number_of_points;
  *hand_pixel_row = projection.rows[0];
  *hand_pixel_col = projection.cols[0];

  // Figure out the position of maximum width of the palm.
  // The heuristic is as follows: We start at the palm end of
  // the axis, and we walk up to the middle. If widths are 
  // still increasing then and the middle is the max width, 
  // we walk up until we get a decrease.
  long middle_index = axis_points / 2;
  long max_width = -1;
  long max_width_index = -1;
  // First find max_width up to middle_index;
  long index;
  for (index = 0; index != middle_index; index++)
  {
    long counter = (long) projection.counts[index];
    if (counter >= max_width)
    {
      max_width = counter;
      max_width_index = index;
    }
  }

  // If max_width_index is middle_index - 1, keep looking until widths
  // start decreasing
  if (max_width_index == middle_index - 1)
  {
    while(projection.counts[index] >= max_width)
    {
      max_width = (long) projection.counts[index];
      max_width_index = index;
      index++;
    }
  }

  long limit_index = (long) (max_width_index + .75 * max_width);
  if (limit_index >= axis_points) limit_index = axis_points - 1;

  // Find the min width between max_width_index and limit_index;
  long min_width = max_width;
  long min_index = max_width_index;
  for (index = max_width_index; index <= limit_index; index++)
  {
    long current_width = (long) projection.counts[index];
    if (current_width < min_width)
    {
      min_width = current_width;
      min_index = index;
    }
  }
  
  long cutoff_index = min_index;
  double cutoff_row, cutoff_col;
  cutoff_row = projection.rows[cutoff_index];
  cutoff_col = projection.cols[cutoff_index];
  double perpendicular = projection.orientation + M_PI/2;
  vLineContourIntersection(binary, (long) cutoff_row, (long) cutoff_col, 
                            perpendicular, row1_p, col1_p, row2_p, col2_p);
  return 1;
}


// vKeepPalm uses vPalmCutpoint to figure out where the palm should
// be separated from the rest of the arm, and returns an image
// containing just the palm (zeros are put everywhere else).
// cutoff is a segment into which this function writes the endpoints
// of the boundary between the palm and the rest of the arm.
grayscale_image * vKeepPalm2(grayscale_image * binary, vLineSegmentLong * cutoff)
{
  long rows = binary->Rows();
  long cols = binary->Cols();
  long row1, col1, row2, col2, hand_pixel_row, hand_pixel_col;
  row1 = col1 = row2 = col2 = 0;
  vPalmCutpoint2(binary, &row1, &col1, &row2, &col2,
                  &hand_pixel_row, &hand_pixel_col);
  if ((row1 == row2) && (col1 == col2)) return binary->ToGrayCopy();
  double dx = col2 - col1;
  double dy = row2 - row1;
  double angle = vAngle2(dx, dy);
  double row3, col3, row4, col4;
  row3 = col3 = row4 = col4 = 0;
  vBoundaryPoints(row1, col1, angle, rows, cols, 
                   &row3, &col3, &row4, &col4);

  // Create a black image and draw the cutoff line on that image,
  // so that it cuts the image into two connected components.
  grayscale_image * boundary_sides = new grayscale_image(rows, cols);
  vWriteValue(boundary_sides, (uchar) 255);
  vDrawLine1(boundary_sides, (long) row3, (long) col3, (long) row4, (long) col4, (uchar) 0);
  vector<Label> labels;
  v3dMatrix<long> * components = boundary_sides->ConnectedComponents(&labels);
  long palm_label = components->Matrix2(0)[hand_pixel_row][hand_pixel_col];
  grayscale_image * palm = (grayscale_image *) vSelectValue(components, palm_label);
  vdelete(components);
  vdelete(boundary_sides);
  cutoff->row1 = row1;
  cutoff->row2 = row2;
  cutoff->col1 = col1;
  cutoff->col2 = col2;
  return palm;
}


grayscale_image * vKeepPalm3(grayscale_image * binary, v3dMatrix<float> * values,
                          vLineSegmentLong * cutoff)
{
  long rows = binary->Rows();
  long cols = binary->Cols();
  long row1, col1, row2, col2, hand_pixel_row, hand_pixel_col;
  row1 = col1 = row2 = col2 = 0;
  vPalmCutpoint3(values, &row1, &col1, &row2, &col2,
                  &hand_pixel_row, &hand_pixel_col);
  if ((row1 == row2) && (col1 == col2)) return binary->ToGrayCopy();
  double dx = col2 - col1;
  double dy = row2 - row1;
  double angle = vAngle2(dx, dy);
  double row3, col3, row4, col4;
  row3 = col3 = row4 = col4 = 0;
  vBoundaryPoints(row1, col1, angle, rows, cols, 
                   &row3, &col3, &row4, &col4);

  // Create a black image and draw the cutoff line on that image,
  // so that it cuts the image into two connected components.
  grayscale_image * boundary_sides = new grayscale_image(rows, cols);
  vWriteValue(boundary_sides, (uchar) 255);
  vDrawLine1(boundary_sides, (long) row3, (long) col3, (long) row4, (long) col4, (uchar) 0);
  vector<Label> labels;
  v3dMatrix<long> * components = boundary_sides->ConnectedComponents(&labels);
  long palm_label = components->Matrix2(0)[hand_pixel_row][hand_pixel_col];
  grayscale_image * palm = (grayscale_image *) vSelectValue(components, palm_label);
  vdelete(components);
  vdelete(boundary_sides);
  cutoff->row1 = row1;
  cutoff->row2 = row2;
  cutoff->col1 = col1;
  cutoff->col2 = col2;
  return palm;
}



  
// This function draws a line that goes through row and col and makes
// an angle with the x axis specified by orientation and counts how 
// many non-zero pixels of binary belong to that line. The boundary
// flag is set to 0 if no non-zero boundary pixels belong to that line,
// and it is set to 1 otherwise. The reason we care about the boundary
// flag is that we use vCountProjectedPoints as an auxiliary function
// in finding where we should separate the palm from the rest of the
// hand, and we don't want to consider cutoff lines at corners of the
// image, where the number of projected pixels is small simply because
// part of the arm is not visible in the image.
long vCountProjectedPoints(grayscale_image * binary, double row, double col,
                            double orientation, long * boundary_flag)
{
  long rows = binary->Rows();
  long cols = binary->Cols();
  vArray2(uchar) binary2 = binary->Matrix2(0);
  double row1, col1, row2, col2;
  row1 = col1 = row2 = col2 = 0;
  vBoundaryPoints(row, col, orientation, rows, cols, 
                   &row1, &col1, &row2, &col2);
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(vRound(row1), vRound(col1), 
              vRound(row2), vRound(col2), &row_step, &col_step, &steps);

  long step;
  *boundary_flag = 0;
  long result = 0;
  for (step = 0; step < steps; step++)
  {
    long current_row = vRound(row1 + step * row_step);
    long current_col = vRound(col1 + step * col_step);
    if (binary->check_bounds(current_row, current_col) == 0)
    {
      continue;
    }
    if (binary2[current_row][current_col] == 0) continue;

    if ((current_row == 0) || (current_row == rows-1) ||
        (current_col == 0) || (current_col == cols-1))
    {
      *boundary_flag = 1;
    }
    result++;
  }
  return result;
}


// like vCountProjectedPoints, but here we sum up the values, as opposed
// to just counting the points with non-zero values.
double vSumProjectedPoints(v3dMatrix<float> * values, double row, double col,
                            double orientation, long * boundary_flag)
{
  long rows = values->Rows();
  long cols = values->Cols();
  vArray2(float) values2 = values->Matrix2(0);
  double row1, col1, row2, col2;
  row1 = col1 = row2 = col2 = 0;
  vBoundaryPoints(row, col, orientation, rows, cols, 
                   &row1, &col1, &row2, &col2);
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(vRound(row1), vRound(col1), 
              vRound(row2), vRound(col2), &row_step, &col_step, &steps);

  long step;
  *boundary_flag = 0;
  double result = 0;
  for (step = 0; step < steps; step++)
  {
    long current_row = vRound(row1 + step * row_step);
    long current_col = vRound(col1 + step * col_step);
    if (values->check_bounds(current_row, current_col) == 0)
    {
      continue;
    }

    if (values2[current_row][current_col] == (float) 0) 
    {
      continue;
    }

    if ((current_row == 0) || (current_row == rows-1) ||
        (current_col == 0) || (current_col == cols-1))
    {
      *boundary_flag = 1;
    }

    result = result + values2[current_row][current_col];
  }
  return result;
}


// This function considers the line going through (row, col) that makes
// the specified angle with the x axis, and finds the most distant
// pair of contour points in binary that belong to that line.
long vLineContourIntersection(grayscale_image * binary, long row, long col,
                               double angle, long * row1_p, long * col1_p,
                               long * row2_p, long * col2_p)
{
  long rows = binary->Rows();
  long cols = binary->Cols();
  vArray2(uchar) binary2 = binary->Matrix2(0);
  double row1, col1, row2, col2;
  row1 = col1 = row2 = col2 = 0;
  vBoundaryPoints(row, col, angle, rows, cols, 
                   &row1, &col1, &row2, &col2);
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(vRound(row1), vRound(col1), 
              vRound(row2), vRound(col2), &row_step, &col_step, &steps);

  long step;
  long result = 0;
  long first_row, first_col, last_row, last_col;
  first_row = first_col = last_row = last_col = -1;
  for (step = 0; step < steps; step++)
  {
    long current_row = vRound(row1 + step * row_step);
    long current_col = vRound(col1 + step * col_step);
    if (binary2[current_row][current_col] == 0) continue;

    if (first_row == -1)
    {
      first_row = current_row;
      first_col = current_col;
    }

    last_row = current_row;
    last_col = current_col;
  }
  *row1_p = first_row;
  *col1_p = first_col;
  *row2_p = last_row;
  *col2_p = last_col;
  return 1;
}


// Pick out_points at random fron in_rows and in_cols and put them
// in out_rows and out_cols. The i-th input point has row (*in_rows)[i]
// and col (*in_cols)[i]. The same input point cannot be picked twice.
// The algorithm is a bit inefficient, it could take a long time
// if we want to pick N points and we have N+1 points.
long vSamplePoints1(vector<long> * in_rows, vector<long> * in_cols, 
                    vector<long> * out_rows, vector<long> * out_cols,
                    long out_points)
{
  long in_points = in_rows->size();
  out_rows->erase(out_rows->begin(), out_rows->end());
  out_cols->erase(out_cols->begin(), out_cols->end());
  if (in_cols->size() != in_points) return 0;
  if (out_points >= in_points) return 0;
    
  // indices will keep track of the indices that have already been picked.
  uchar * indices = new uchar[in_points];

  long i;
  for (i = 0; i < in_points; i++)
  {
    indices[i] = 0;
  }
  for (i = 0; i < out_points; i++)
  {
    long index = vRandomLong2(0, in_points-1);
    if (indices[index] != 0) 
    {
      i--;
      continue;
    }
    indices[index] = 1;
    out_rows->push_back((*in_rows)[index]);
    out_cols->push_back((*in_cols)[index]);
  }
  vdelete2(indices);
  return 1;
}


long vSamplePoints2(vector<long> * in_rows, vector<long> * in_cols, 
                    vector<long> * out_rows, vector<long> * out_cols,
                    long sampling_rate)
{
  long in_points = in_rows->size();
  long out_points = in_points / sampling_rate;
  return vSamplePoints1(in_rows, in_cols, out_rows, out_cols, out_points);
}


// rows and cols contain coordinates of points. The i-th point
// has row = (*rows)[i] and col = (*cols)[i]. We want to save
// them in a file that is MATLAB-readable, as a matrix
// whose name is specified by the argument name, and whose
// i+1-th row will contain the row and col of the i-th point.
// This function was originally written to experiment with
// the shape context code I downloaded, which is in MATLAB.
long vSavePoints(vector<long> * rows, vector<long> * cols,
                  const char * name, const char * file)
{
  long number = rows->size();
  if (cols->size() != number)
  {
    vPrint("rows and cols have different size\n");
    return 0;
  }

  FILE * fp = fopen(file, vFOPEN_WRITE);
  if (fp == 0) 
  {
    vPrint("Can't open %s for writing\n", file);
    return 0;
  }

  long result, success;
  success = 1;
  result = fprintf(fp, "%s = [\r\n", name);
  if (result < 0) success = 0;

  long i;
  for (i = 0; i < number; i++)
  {
    result = fprintf(fp, "%li %li\r\n", (*cols)[i], (*rows)[i]);
    if (result < 0) success = 0;
  }
  result = fprintf(fp, "];\r\n");
  if (result < 0) success = 0;
  fclose(fp);
  return success;
}


// Start comments here, for code written for CVPR 2003

// Same functionality as vOrientedLines2, faster version. It is 
// faster because here we only check points whose value in
// dt_image is 0.
// dt_image is the distance transform of some edge image.
// For each edge pixel, i.e. a pixel with value 0 in the dt_image,
// consider a line segment of given length and orientation
// centered at the pixel, and find the number of pixels on that segment
// for which distances in the dt_image are below threshold. 
// A simple (maybe naive) way to detect
// lines of given orientation. Fast version.
// Result: A one-band matrix, that at position (i,j) contains the 
// number of votes received for the line centered at (i,j) that
// has the given orientation and length.
v3dMatrix<long> * vOrientedLines(v3dMatrix<float> * dt_image, 
                                    double orientation, long length, 
                                    long threshold)
{
  long rows = dt_image->Rows();
  long cols = dt_image->Cols();
  v3dMatrix<long> * votes_matrix = new v3dMatrix<long>(rows, cols, 1);
  vWriteValue(votes_matrix, (long) 0);
  vector<long> point_rows, point_cols;
  long size = dt_image->Size();
  vArray2(float) distances = dt_image->Matrix2(0);

  // Prepare arguments
  long row, col;
  for (row = 0; row < rows; row++)
  { 
    for (col = 0; col < cols; col++)
    {
      if (distances[row][col] == 0)
      {
        point_rows.push_back(row);
        point_cols.push_back(col);
      }
    }
  }
  // Call the function that does the main work.
  vOrientedLines7(dt_image, &point_rows, &point_cols, 
                   votes_matrix, orientation, length, threshold);
  return votes_matrix;
}


// Same functionality as vOrientedLines, slower version.
// It is slower because here we consider every single point as 
// a center for a line, as opposed to using only points that
// are 0 on the dt_image.
// dt_image is the distance transform of some edge image.
// For each edge pixel, i.e. a pixel with value 0 in the dt_image,
// consider a line segment of given length and orientation
// centered at the pixel, and find the number of pixels on that segment
// for which distances in the dt_image are below threshold. 
// A simple (maybe naive) way to detect
// lines of given orientation. Fast version.
// Result: A one-band matrix, that at position (i,j) contains the 
// number of votes received for the line centered at (i,j) that
// has the given orientation and length.
v3dMatrix<long> * vOrientedLines2(v3dMatrix<float> * dt_image, 
                                     double orientation, long length, 
                                     long threshold)
{
  static const long junk1 = (long) vPrecomputedSin::PrecomputeValues();
  static const long junk2 = (long) vPrecomputedCos::PrecomputeValues();

  // Get the gradient vector that corresponds to the orientation.
  double dx = vPrecomputedCos::Cos(orientation);
  double dy = vPrecomputedSin::Sin(orientation);

  // Figure out the steps we have to make in the x and y direction
  // to keep moving along the specified orientation and cover
  // all related pixels.
  double abs_dx = fabs(dx);
  double abs_dy = fabs(dy); 
  double step_x, step_y;

  if (abs_dx >= abs_dy) 
  {
    step_x = 1;
    step_y = dy / dx;
  }

  else
  {
    step_x = dx / dy;
    step_y = 1;
  }

  long half_length = length/2 + 1;
  long rows = dt_image->Rows();
  long cols = dt_image->Cols();
  vArray2(float) distances = dt_image->Matrix2(0);
  v3dMatrix<long> * votes_matrix = new v3dMatrix<long>(rows, cols, 1);
  vArray2(long) votes = votes_matrix->Matrix2(0);
  // Figure out the relative position of endpoints of the line with 
  // respect to the center of the line.
  double end_x = half_length * dx;
  double end_y = half_length * dy;

  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      // Get the current endopint.
      double current_x = col - end_x;
      double current_y = row - end_y;
      long counter = 0;
      long line_index;
      for (line_index = 0; line_index <= length; line_index++)
      {
        // Compute the current point on the line.
        current_x += step_x;
        current_y += step_y;
        long x = (long) current_x;
        long y = (long) current_y;
        if (!dt_image->check_bounds(y, x)) continue;
        // Get the vote
        if (distances[y][x] <= threshold) counter++;
      }
      votes[row][col] = counter;
    }
  }
  return votes_matrix;
}


// Like vOrientedLines, but here we are already given, in point_rows and
// point_cols the points that we should consider as centers for those lines,
// and in votes_matrix the image where we keep track of the votes (that
// we assume has already been initialized).
// The output argument is votes_matrix, where for each pixel in point_rows
// and point_cols we count the votes it receives for the specified orientation,
// length and threshold.
// Note: The votes are determined by drawing the specified line, and checking
// how many pixels with value in dt_image less than threshold are hit by
// the line.
long vOrientedLines7(v3dMatrix<float> * dt_image, vector<long> * point_rows,
                      vector<long> * point_cols, v3dMatrix<long> * votes_matrix,
                      double orientation, long length, long threshold)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  double dx = vPrecomputedCos::Cos(orientation);
  double dy = vPrecomputedSin::Sin(orientation);

  // Calculate the steps we have to take to go from one pixel of the line
  // to the next pixel of the line.
  double abs_dx = fabs(dx);
  double abs_dy = fabs(dy); 
  double step_x, step_y;
  double x_direction, y_direction;
  if (dx < 0) x_direction = -1;
  else x_direction = 1;
  if (dy < 0) y_direction = -1;
  else y_direction = 1;

  if (abs_dx >= abs_dy) 
  {
    step_x = x_direction;
    step_y = abs_dy / abs_dx * y_direction;
  }

  else
  {
    step_x = abs_dx / abs_dy * x_direction;
    step_y = y_direction;
  }

  long half_length = length/2 + 1;
  long rows = dt_image->Rows();
  long cols = dt_image->Cols();
  vArray2(float) distances = dt_image->Matrix2(0);
  vArray2(long) votes = votes_matrix->Matrix2(0);
  // Figure out the location of the endpoints of the line
  // from the center of the line.
  double end_x = half_length * dx;
  double end_y = half_length * dy;

  long point_index;
  long number_of_points = point_rows->size();
  // Go through all the points. Each point specifies
  // the center of a line, which is furthermore specified by
  // the arguments orientation and length.
  for (point_index = 0; point_index < number_of_points; point_index++)
  {
    // Get the current line center
    long row = (*point_rows)[point_index];
    long col = (*point_cols)[point_index];
    // Compute the line starting point.
    double current_x = col - end_x;
    double current_y = row - end_y;
    // counter will count the votes.
    long counter = 0;

    long line_index;
    // Go through the line points and count votes.
    for (line_index = 0; line_index <= length; line_index++)
    {
      // Get the current point on the line.
      current_x += step_x;
      current_y += step_y;
      long x = vRound(current_x);
      long y = vRound(current_y);
      if (!dt_image->check_bounds(y, x)) continue;
      // See if we get a vote or not.
      if (distances[y][x] <= threshold) counter++;
    }
    votes[row][col] = counter;
  }
  return 1;
}


// Arguments: 
// color_image: a gray scale image.
// edges: the corresponding edge image.
// orientation, length, threshold and tolerance are described below.
// For each edge pixel whose orientation is within a given
// tolerance of the specified orientation, consider 
// a line segment of given length and orientation
// centered at the pixel, and find the number of pixels on that segment
// that are within the given distance (expressed by threshold) from an edge
// pixel whose orientation is within a certain tolerance of orientation
// One difference between this function and vOrientedLines is that
// here a pixel cannot vote for a line unless its orientation is within
// the specified tolerance from the argument orientation.
// The other difference is that the only pixels that are considered as
// centers of lines rae the ones whose orientation is within the specified
// tolerance. So, fewer lines are considered, which makes it faster.
v3dMatrix<long> * vOrientedLines_c(grayscale_image * the_image, grayscale_image * edges,
                                      double orientation, long length, 
                                      long threshold, double tolerance)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  // Create the votes matrix, initialize it to 0.
  v3dMatrix<long> * votes_matrix = new v3dMatrix<long>(rows, cols, 1);
  vWriteValue(votes_matrix, (long) 0);

  // These vectors will hold the locations of all edge pixels whose
  // orientation is acceptable (within the given tolerance from the
  // given orientation).
  vector<long> point_rows, point_cols;

  // Get the orientation at each pixel.
  v3dMatrix<double> * orientations_image = vEdgeOrientations(the_image);
  vArray2(double) orientations2 = orientations_image->Matrix2(0);
  vArray2(uchar) edges2 = edges->Matrix2(0);
  // Find the edge pixels with compatible orientations.
  long row, col;
  for (row = 0; row < rows; row++)
  { 
    for (col = 0; col < cols; col++)
    {
      if (edges2[row][col] != 0)
      {
        double pixel_orientation = orientations2[row][col];
        double distance = vAbs(vCircularDistance(pixel_orientation, orientation, M_PI));
        if (distance <= tolerance)
        {
          point_rows.push_back(row);
          point_cols.push_back(col);
        }
      }
    }
  }

  vPrint("number of pixels = %li\n", (long) point_rows.size());

  // Create a new edge image, containing only pixels with acceptable
  // orientation.
  grayscale_image * selected_edges = new grayscale_image(rows, cols);
  vWriteValue(selected_edges, (uchar) 0);
  vWriteValue2(selected_edges, &point_rows, &point_cols, (uchar) 255);
  // Get the distance transform of the new image.
  v3dMatrix<float> * dt_image = vCVDistanceTransform(selected_edges);
  // Call the function that actually counts the votes.
  vOrientedLines7(dt_image, &point_rows, &point_cols, 
                   votes_matrix, orientation, length, threshold);
  vdelete(dt_image);
  vdelete(orientations_image);
  vdelete(selected_edges);
  return votes_matrix;
}


// This function is similar in spirit to vOrientedLines7. point_rows
// and point_cols specify centers of line segments that we should 
// evaluate (count the votes for), and orientation/length specify
// the properties of each line segment. votes_matrix is an output
// argument, recording the votes for the line segment centered at 
// each of the specified points.
// The main difference in this function with respect to vOrientedLines7
// is that here, instead of using a distance transform as an argument,
// we pass in a binary image. Any pixel that is non-zero in the image
// votes for every line segment that goes through it. Also, the
// center points are specified in point_rows and point_cols, they
// are NOT necessarily all the non-zero points in binary_matrix.
long vOrientedLines7b(grayscale_image * binary_matrix, vector<long> * point_rows,
                       vector<long> * point_cols, v3dMatrix<long> * votes_matrix,
                       double orientation, long length)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  double dx = vPrecomputedCos::Cos(orientation);
  double dy = vPrecomputedSin::Sin(orientation);

  // Compute the gradient vector corresponding to orientation.
  double abs_dx = fabs(dx);
  double abs_dy = fabs(dy); 
  double step_x, step_y;

  if (abs_dx >= abs_dy) 
  {
    step_x = 1;
    step_y = dy / dx;
  }

  else
  {
    step_x = dx / dy;
    step_y = 1;
  }
  
  // Compute the line endpoint locations with respect to the line center.
  long half_length = length/2 + 1;
  double end_x = half_length * dx;
  double end_y = half_length * dy;

  long rows = binary_matrix->Rows();
  long cols = binary_matrix->Cols();
  vArray2(uchar) binary = binary_matrix->Matrix2(0);
  vArray2(long) votes = votes_matrix->Matrix2(0);

  long point_index;
  long number_of_points = point_rows->size();
  // Consider every one of the points that should be used as 
  // line centers.
  for (point_index = 0; point_index < number_of_points; point_index++)
  {
    // Get the current center point.
    long row = (*point_rows)[point_index];
    long col = (*point_cols)[point_index];
    // Get the line start point.
    double current_x = col - end_x;
    double current_y = row - end_y;
    long counter = 0;
    long line_index;
    // Go through each point on the line
    for (line_index = 0; line_index <= length; line_index++)
    {
      // Compute the current point on the line.
      current_x += step_x;
      current_y += step_y;
      long x = (long) current_x;
      long y = (long) current_y;
      // Get the vote.
      if (!binary_matrix->check_bounds(y, x)) continue;
      if (binary[y][x] != 0) counter++;
    }
    votes[row][col] = counter;
  }
  return 1;
}


// Fit an ellipse to the non-zero pixels of the image, and return
// the width of that ellipse.
double vShapeWidth(grayscale_image * the_image)
{
  vector<long> rows, cols;
  vGetNonZeroPoints(the_image, &rows, &cols);
  vEllipse ellipse = vFitEllipse2(&rows, &cols);
  return ellipse.width;
}


// Fit an ellipse to the non-zero pixels of the image, and return
// the length of that ellipse.
double vShapeLength(grayscale_image * the_image)
{
  vector<long> rows, cols;
  vGetNonZeroPoints(the_image, &rows, &cols);
  vEllipse ellipse = vFitEllipse2(&rows, &cols);
  return ellipse.length;
}


// Fit an ellipse to the non-zero pixels of the image, and return
// the orientation of that ellipse (between 0 and M_PI).
double vShapeOrientation(grayscale_image * the_image)
{
  vector<long> rows, cols;
  vGetNonZeroPoints(the_image, &rows, &cols);
  vEllipse ellipse = vFitEllipse2(&rows, &cols);
  return ellipse.angle + M_PI/2;
}


// Input arguments:
// color_image: A gray-scale image that is treated as binary (zero 
// or non-zero).
// row, col, orientation: they specify a line of the given
// orientation that goes through the given row and col.
// Result: A line segment specifying the end points of 
// the projection of the non-zero portion of color_image onto the
// specified line.
vLineSegment vProjectionEndpoints(grayscale_image * the_image, 
                                   long row, long col, double orientation)
{
  vArray2(uchar) values = the_image->Matrix2(0);
  double start_row, start_col, end_row, end_col, row_step, col_step;
  start_row = start_col = end_row = end_col = row_step = col_step = 0;
  // Get the points where the line intersects the boundary of the image.
  vBoundaryPoints(row, col, orientation, the_image->Rows(), the_image->Cols(), 
                   &start_row, &start_col, &end_row, &end_col);
  long steps;
  // Get the gradient vector and number of steps required to go through
  // all pixels on the line.
  vLineSteps(start_row, start_col, end_row, end_col, &row_step, &col_step, &steps);
  double current_row, current_col;
  // Get the perpendicular orientation, to figure out the projections.
  double perpendicular = orientation + M_PI/2;
  long step;
  long junk = 0;
  // Declare the variables that will hold the coordinates of the 
  // projection endpoints.
  double first_row, first_col, last_row, last_col;
  first_row = first_col = last_row = last_col = -1;
  for (step = 0; step < steps; step++)
  {
    // Go through each point on the line.
    current_row = start_row + step * row_step;
    current_col = start_col + step * col_step;
    // Count the non-zero points in color_image that correspond to this line.
    // junk receives values for what is called a "boundary flag", for
    // which we don't care here.
    long points = vCountProjectedPoints(the_image, current_row, current_col,
                                         perpendicular, &junk);
    if (points == 0) continue;
    // Check if this is the first non-zero projection we have seen so far.
    if (first_row == -1)
    {
      first_row = current_row;
      first_col = current_col;
    }
    // Mark this as the last non-zero projection we have seen so far.
    last_row = current_row;
    last_col = current_col;
  }
  // Store the endpoints into the result.
  vLineSegment result;
  result.row1 = vRound(first_row);
  result.col1 = vRound(first_col);
  result.row2 = vRound(last_row);
  result.col2 = vRound(last_col);
  return result;
}


// We have a line with given angle, going through x0, y0. We position
// ourselves at (x0, y0) and look towards the direction of the angle.
// Every other point (x,y) in the plane is either right in front of us
// (i.e. it belongs to the line), or is on the left half plane, or on 
// the right half plane. This function returns 0 if (x,y) is on the line,
// -1 if it is on the left, and 1 if it is on the right.
long vFindHalfPlane(double x0, double y0, double angle, double x, double y)
{
  static const double two_pi = 2*M_PI;
  double dx = x - x0;
  double dy = y - y0;

  double angle2 = vAngle2(dx, dy);
  double difference = vCircularDistance(angle, angle2, two_pi);
  if (difference == 0)
  {
    return 0;
  }
  else if (difference < 0) return 1;
  else return -1;
}


// A simple conversion function, it converts a vector of doubles 
// into a vector of longs, by rounding each double.
long vDoubleToLongVector(vector<long> * target, vector<double> * source)
{
  long number = source->size();
  long i;
  for (i = 0; i < number; i++)
  {
    target->push_back(vRound((*source)[i]));
  }
  return 1;
}


// A simple conversion function, it converts a vector of longs 
// into a vector of doubles.
long vLongToDoubleVector(vector<double> * target, vector<long> * source)
{
  long number = source->size();
  long i;
  for (i = 0; i < number; i++)
  {
    target->push_back((double)(*source)[i]);
  }
  return 1;
}


// Inputs:
// the_image, edges: gray-scale image and corresponding edge image.
// angle, length, threshold, tolerance: used to specify
// the orientation and length of the segments we will look for,
// and the pixels that can vote for those segments.
// Output:
// The parameter results (which is assumed to be empty) will
// receive all the line segments that were found.
// This function gets line segments not only for the given orientation,
// but also for other nearby orientations (as specified by the
// variables angle_range and angle_steps). It counts votes
// using vOrientedLines_c, where only pixels of similar orientation
// can vote for a given line or be considered as centers of a line.
long vLineSegmentMatches(grayscale_image * the_image, grayscale_image * edges, 
                          double angle, double length, 
                          double threshold, double tolerance, 
                          vector<vLineSegmentMatch2> * results)
{
  // Figure out what angle range we will consider for orientations,
  // and how many samples we will use
  static const double angle_range = 0.3;
  static const long range_steps = 3;
  static const double step_width = angle_range / range_steps;
  static const long total_steps = range_steps * 2 + 1;

  // Get votes for every orientation.
  v3dMatrix<long> * votes_array[total_steps];
  long i;
  for (i = 0; i < total_steps; i++)
  {
    // Figure out the current orientation
    double current_angle = angle - angle_range + step_width * i;
    // Get the votes for the current orientation.
    votes_array[i] = vOrientedLines_c(the_image, edges, current_angle, 
                                       (long) length, (long) threshold, tolerance);
  }

  // Create and store line segments for all the segments for which
  // non-zero votes were counted.
  vector<vLineSegmentMatch2> angle_sorted, vote_sorted;
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long row, col;
  for (i = 0; i < total_steps; i++)
  {
    vArray2(long) votes = votes_array[i]->Matrix2(0);
    // Compute the current orientation.
    double current_angle = angle - angle_range + step_width * i;
    // Compute the gradient vector for the current orientation.
    double row_diff = vAbs(length/2 * sin(current_angle));
    double col_diff = vAbs(length/2 * cos(current_angle));
    // Go through all possible center points of lines (i.e.
    // all possible pixels).
    for (row = 0; row < rows; row++)
    {
      for (col = 0; col < cols; col++)
      {
        // check if the current pixel received any votes
        // for this orientation.
        if (votes[row][col] == 0) continue;
        // Create a vLineSegmentMatch2 object for the corresponding
        // line segment.
        vLineSegmentMatch2 match;
        match.segment.row1 = vRound(row - row_diff);
        match.segment.col1 = vRound(col - col_diff);
        match.segment.row2 = vRound(row + row_diff);
        match.segment.col2 = vRound(col + col_diff);
        match.angle_difference = vAbs(current_angle - angle);
        match.step = i;
        match.positive_votes = (float) votes[row][col];
        // Put the line segment into the vectors that will later
        // be sorted based on angle difference and on votes 
        angle_sorted.push_back(match);
        vote_sorted.push_back(match);
      }
    }
    vdelete(votes_array[i]);
  }

  // sort based on angle differences, and based on votes
  long number = angle_sorted.size();
  for (i = 0; i < number; i++)
  {
    angle_sorted[i].score = (float) angle_sorted[i].angle_difference;
    vote_sorted[i].score = -angle_sorted[i].positive_votes;
  }
  std::sort(angle_sorted.begin(), angle_sorted.end(), vLineSegmentMatchLess());
  std::sort(vote_sorted.begin(), vote_sorted.end(), vLineSegmentMatchLess());

  // Create images that will storethe results of sorting. Each image
  // will show at pixel (row, col, band) the rank (based on angle
  // difference or votes) of the segment centered at row, col, and
  // whose angle corresponded to the given band.
  // These images will be used to sum up the ranks of each segment under
  // each sorting criterion (angle difference or votes) so that
  // we can resort them.
  v3dMatrix<long> angle_ranks(rows, cols, total_steps);
  v3dMatrix<long> vote_ranks(rows, cols, total_steps);
  vArray3(long) angle_ranks3 = angle_ranks.Matrix3();
  vArray3(long) vote_ranks3 = vote_ranks.Matrix3();

  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch2 match = angle_sorted[i];
    long row = (match.segment.row1 + match.segment.row2) / 2;
    long col = (match.segment.col1 + match.segment.col2) / 2;
    long step = match.step;
    angle_ranks3[step][row][col] = i;

    match = vote_sorted[i];
    row = (match.segment.row1 + match.segment.row2) / 2;
    col = (match.segment.col1 + match.segment.col2) / 2;
    step = match.step;
    vote_ranks3[step][row][col] = i;
  }

  // Put all line segments into the results vector, and sort them
  // again, based on the sum of the two ranks (angles and votes).
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch2 match = angle_sorted[i];
    long row = (match.segment.row1 + match.segment.row2) / 2;
    long col = (match.segment.col1 + match.segment.col2) / 2;
    long step = match.step;
    match.score = (float) (angle_ranks3[step][row][col] + vote_ranks3[step][row][col]);
    results->push_back(match);
  }

  // Sort the results.
  std::sort(results->begin(), results->end(), vLineSegmentMatchLess());

  return 1;
}


// Inputs:
// orientations_image, edges: orientations and edges of some
// (not given here) image. edges is not binary, it contains
// the magnitude of the intensity gradient.
// edge_rows, edge_cols: pre-extracted edge pixels (so that we 
// don't have to search for them). In general, each pixel in
// edge_rows, edge_cols will be used as the center of line segments
// with given orientation and lengths, for which votes and saliency
// (edge strength) will be estimated.
// orientation: the orientation at which we will look for lines.
// params: in this function they specify how many and which lengths
// we should consider, and also the angle tolerance and distance tolerance
// that will determine the shape of the search region.
// Output:
// The parameter results receives all the segments that were found, together
// with information about their votes and saliency.
// In contrast to the previous vOrientedLines and vOrientedLines_c functions,
// here the search region is not a single line or rectangle, but it is the
// union of two triangles, as described in the CVPR 2003 paper. Also,
// in contrast to those previous functions, saliency is also computed here.
long vOrientedLines_d(v3dMatrix<double> * orientations_image, 
                       grayscale_image * edges, 
                       vector<long> * edge_rows, vector<long> * edge_cols,
                       double orientation, vLineSegmentSearchParams params,
                       vector<vLineSegmentMatch> * results)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  // Do some error checking
  if ((params.lengths <= 1) && (params.low_length != params.high_length)) return 0;

  // Figure out the length step
  double length_step = 0;
  if (params.lengths > 1) 
  {
    length_step = vRound((params.high_length - params.low_length) / (params.lengths - 1));
  }

  // Get the search region, over which we look for votes.
  vector<long> search_rows, search_cols, search_lengths;
  vGetSearchRegion(orientation, params.high_length, params.angle_tolerance,
                    params.distance_tolerance,
                    &search_rows, &search_cols, &search_lengths);

  long center_row, center_col;
  long number = edge_rows->size();
  long number2 = search_rows.size();

  // Create some prototype line segments, one for each possible length,
  // to save some time (otherwise we would have to compute these
  // things over and over again).
  long i, j, k;
  vArray(vLineSegmentMatch) current_matches = vnew(vLineSegmentMatch, params.lengths);
  vArray(vLineSegment) segments = vnew(vLineSegment, params.lengths);
  vArray(double) lengths = vnew(double, params.lengths);
  for (i = 0; i < params.lengths; i++)
  {
    lengths[i] = params.low_length + i * length_step;
    double half_length = lengths[i] / 2;
    // The row_diff and col_diff is fixed given length and orientation
    double row_diff = half_length * vPrecomputedSin::Sin(orientation);
    double col_diff = half_length * vPrecomputedCos::Cos(orientation);
    segments[i].row1 = -row_diff;
    segments[i].col1 = -col_diff;
    segments[i].row2 = row_diff;
    segments[i].col2 = col_diff;
    current_matches[i].orientation = (float) orientation;
    current_matches[i].length = (float) lengths[i];
  }

  vArray2(double) orientations2 = orientations_image->Matrix2(0);
  vArray2(uchar) edges2 = edges->Matrix2(0);

  // Calculate line votes centered at each edge point. We go through
  // all edge points (number is the number of edge pixels).
  for (i = 0; i < number; i++)
  {
    center_row = (*edge_rows)[i];
    center_col = (*edge_cols)[i];
    // Reset all counters to 0
    for (j = 0; j < params.lengths; j++)
    {
      current_matches[j].all_votes = 0;
      current_matches[j].positive_votes = 0;
      current_matches[j].strength = 0;
    }

    // Now go through all points in the search region
    for (k = 0; k < number2; k++)
    {
      // Compute the vote strength for the given point.
      long row = center_row + search_rows[k];
      long col = center_col + search_cols[k];
      if (edges->check_bounds(row, col) <= 0) continue;
      double edge = edges2[row][col];
      if (edge == 0) continue;
      double pixel_orientation = orientations2[row][col];
      double angle = vAbs(vCircularDistance(orientation, pixel_orientation, (double) M_PI));
      double vote_strength = vPrecomputedCos::Cos(2*angle);

      long length = search_lengths[k];

      // For all segments whose length is greater than the current length,
      // add in the votes for the current segment.
      for (j = params.lengths - 1; j >= 0; j--)
      {
        if (lengths[j] / 2 < length) break;
        current_matches[j].all_votes += (float) vote_strength;
        if (vote_strength > 0)
        {
          current_matches[j].positive_votes += (float) vote_strength;
          current_matches[j].strength += (float) edge * (float) vote_strength;
        }
      }
    }

    for (j = 0; j < params.lengths; j++)
    {
      current_matches[j].segment.row1 = (long) (center_row + segments[j].row1);
      current_matches[j].segment.row2 = center_row + (long) segments[j].row2;
      current_matches[j].segment.col1 = center_col + (long) segments[j].col1;
      current_matches[j].segment.col2 = center_col + (long) segments[j].col2;
      current_matches[j].center_row = center_row;
      current_matches[j].center_col = center_col;

      if (current_matches[j].positive_votes != 0)
      {
        current_matches[j].strength /= current_matches[j].positive_votes;
      }
      current_matches[j].vote_density = current_matches[j].all_votes / current_matches[j].length;

      current_matches[j].d = 0;
      current_matches[j].o = vRound(current_matches[j].orientation * 180 / M_PI);
      current_matches[j].l = vRound(current_matches[j].length);
      current_matches[j].v = vRound(current_matches[j].vote_density * 100);
      current_matches[j].s = vRound(current_matches[j].strength);

      if (current_matches[j].all_votes > 0)
      {
        results->push_back(current_matches[j]);
      }
    }
  }

  vdelete2(current_matches);
  vdelete2(segments);
  vdelete2(lengths);
  return 1;
}


// Same as vOrientedLines_d, but I will try to make this faster.
// 10/27/2002: There are a couple of bugs, that I am trying to fix
// on vOrientedLines_f
long vOrientedLines_e(v3dMatrix<double> * orientations_image, 
                       grayscale_image * edges, 
                       vector<long> * edge_rows, vector<long> * edge_cols,
                       double orientation, vLineSegmentSearchParams params,
                       vector<vLineSegmentMatch> * results)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  // Do some error checking
  if ((params.lengths <= 1) && (params.low_length != params.high_length)) return 0;

  // Figure out the length step
  double length_step = 0;
  if (params.lengths > 1) 
  {
    length_step = vRound((params.high_length - params.low_length) / (params.lengths - 1));
  }

  // Get the search region, over which we look for votes.
  vector<long> search_rows, search_cols, search_lengths;
  vGetSearchRegion(orientation, params.high_length, params.angle_tolerance,
                    params.distance_tolerance,
                    &search_rows, &search_cols, &search_lengths);

  long center_row, center_col;
  long number = edge_rows->size();
  long number2 = search_rows.size();
  vArray2(double) orientations2 = orientations_image->Matrix2(0);
  vArray2(uchar) edges2 = edges->Matrix2(0);

  long i, j, k;

  // Precompute vote strengths for all pixels.
  long rows = orientations_image->Rows();
  long cols = orientations_image->Cols();
  v3dMatrix<float> vote_strengths_matrix(rows, cols, 1);
  vArray2(float) vote_strengths = vote_strengths_matrix.Matrix2(0);
  for (i = 0; i < number; i++)
  {
    center_row = (*edge_rows)[i];
    center_col = (*edge_cols)[i];
    float pixel_orientation = (float) orientations2[center_row][center_col];
    float angle = vAbs(vCircularDistance((float) orientation, pixel_orientation, (float) M_PI));
    vote_strengths[center_row][center_col] = (float) vPrecomputedCos::Cos(2*angle);
  }

  // Create some prototype line segments, one for each possible length,
  // to save some time (otherwise we would have to compute these
  // things over and over again).
  vArray(vLineSegmentMatch) current_matches = vnew(vLineSegmentMatch, params.lengths);
  vArray(vLineSegment) segments = vnew(vLineSegment, params.lengths);
  vArray(double) lengths = vnew(double, params.lengths);
  for (i = 0; i < params.lengths; i++)
  {
    lengths[i] = params.low_length + i * length_step;
    double half_length = lengths[i] / 2;
    // The row_diff and col_diff is fixed given length and orientation
    double row_diff = half_length * vPrecomputedSin::Sin(orientation);
    double col_diff = half_length * vPrecomputedCos::Cos(orientation);
    segments[i].row1 = -row_diff;
    segments[i].col1 = -col_diff;
    segments[i].row2 = row_diff;
    segments[i].col2 = col_diff;
    current_matches[i].orientation = (float) orientation;
    current_matches[i].length = (float) lengths[i];
  }

  // Calculate line votes centered at each edge point. We go through
  // all edge points (number is the number of edge pixels).
  for (i = 0; i < number; i++)
  {
    center_row = (*edge_rows)[i];
    center_col = (*edge_cols)[i];
    float center_orientation = (float) orientations2[center_row][center_col];
    float orientation_diff = vCircularDistance((float) orientation, center_orientation, (float) M_PI);
    if (orientation_diff > .4) continue;

    current_matches[0].all_votes = 0;
    current_matches[0].positive_votes = 0;
    current_matches[0].strength = 0;

    long current_match = 0;
    long current_length = (long) lengths[current_match] / 2;
    // Now go through all points in the search region
    for (k = 0; k < number2; k++)
    {
      // Compute the vote strength for the given point.
      long row = center_row + search_rows[k];
      long col = center_col + search_cols[k];
      if (edges->check_bounds(row, col) <= 0) continue;
      float edge = edges2[row][col];
      if (edge == 0) continue;
      float vote_strength = vote_strengths[row][col];

      long length = vAbs(search_lengths[k]);

      // We know that search_lengths is an increasing sequence. If we have reached
      // a length that exceeds the range of the current match, we start updating
      // the next match (after we transfer to it all measurements made for this match).
      if (length > current_length)
      {
        while(length > current_length)
        {
          current_match++;
          if (current_match >= params.lengths) break;
          current_length = (long) lengths[current_match] / 2;
          current_matches[current_match].all_votes = current_matches[current_match-1].all_votes;
          current_matches[current_match].positive_votes = current_matches[current_match-1].positive_votes;
          current_matches[current_match].strength = current_matches[current_match-1].strength;
        }
        if (current_match >= params.lengths)
        {
          break;
        }
      }

      current_matches[current_match].all_votes += vote_strength;
      if (vote_strength > 0)
      {
        current_matches[current_match].positive_votes += vote_strength;
        current_matches[current_match].strength += edge * vote_strength;
      }
    }

    // In case we didn't initialize some matches, we do it now
    for (j = current_match + 1; j < params.lengths; j++)
    {
      current_matches[j].all_votes = current_matches[j-1].all_votes;
      current_matches[j].positive_votes = current_matches[j-1].positive_votes;
      current_matches[j].strength = current_matches[j-1].strength;
    }

    for (j = 0; j < params.lengths; j++)
    {
      if (current_matches[j].all_votes <= 0) continue;
      if ((j > 0) && 
          (current_matches[j].all_votes <= current_matches[j-1].all_votes))
      {
        continue;
      }
      
      current_matches[j].segment.row1 = center_row + (long) segments[j].row1;
      current_matches[j].segment.row2 = center_row + (long) segments[j].row2;
      current_matches[j].segment.col1 = center_col + (long) segments[j].col1;
      current_matches[j].segment.col2 = center_col + (long) segments[j].col2;
      current_matches[j].center_row = center_row;
      current_matches[j].center_col = center_col;

      if (current_matches[j].positive_votes <= 0)
      {
        exit_error("Error\n");
      }
      current_matches[j].strength /= current_matches[j].positive_votes;
      current_matches[j].vote_density = current_matches[j].all_votes / current_matches[j].length;

      current_matches[j].d = 0;
      current_matches[j].o = vRound(current_matches[j].orientation * 180 / M_PI);
      current_matches[j].l = vRound(current_matches[j].length);
      current_matches[j].v = vRound(current_matches[j].vote_density * 100);
      current_matches[j].s = vRound(current_matches[j].strength);
      results->push_back(current_matches[j]);
    }
  }

  vdelete2(current_matches);
  vdelete2(segments);
  vdelete2(lengths);
  return 1;
}


// Same as vOrientedLines_e, but here all pixels that belong to the same
// normal to the main search line are combined to give a vote between -1 and 1.
// In vOrientedLines_e each pixel had its own vote, and that had as a result
// vote densities exceeding 1, and a too high weight put on portions of the line
// that are far from the center.
long vOrientedLines_f(v3dMatrix<double> * orientations_image, 
                       grayscale_image * edges, 
                       vector<long> * edge_rows, vector<long> * edge_cols,
                       double orientation, vLineSegmentSearchParams params,
                       vector<vLineSegmentMatch> * results)
{
  static const double junk1 = vPrecomputedSin::PrecomputeValues();
  static const double junk2 = vPrecomputedCos::PrecomputeValues();

  // Do some error checking
  if ((params.lengths <= 1) && (params.low_length != params.high_length)) return 0;

  // Figure out the length step
  double length_step = 0;
  if (params.lengths > 1) 
  {
    length_step = (params.high_length - params.low_length) / (params.lengths - 1);
  }

  // Get the search region, over which we look for votes.
  vector<long> search_rows, search_cols, search_lengths;
  vGetSearchRegion(orientation, params.high_length, params.angle_tolerance,
                    params.distance_tolerance,
                    &search_rows, &search_cols, &search_lengths);

  long center_row, center_col;
  long number = edge_rows->size();
  long number2 = search_rows.size();
  vArray2(double) orientations2 = orientations_image->Matrix2(0);
  vArray2(uchar) edges2 = edges->Matrix2(0);

  long i, j, k;

  // Precompute vote strengths for all pixels.
  long rows = orientations_image->Rows();
  long cols = orientations_image->Cols();
  v3dMatrix<float> vote_strengths_matrix(rows, cols, 1);
  vArray2(float) vote_strengths = vote_strengths_matrix.Matrix2(0);
  for (i = 0; i < number; i++)
  {
    center_row = (*edge_rows)[i];
    center_col = (*edge_cols)[i];
    float pixel_orientation = (float) orientations2[center_row][center_col];
    float angle = vAbs(vCircularDistance((float) orientation, pixel_orientation, (float) M_PI));
    vote_strengths[center_row][center_col] = (float) vPrecomputedCos::Cos(2*angle);
  }

  // Create some prototype line segments, one for each possible length,
  // to save some time (otherwise we would have to compute these
  // things over and over again).
  vArray(vLineSegmentMatch) current_matches = vnew(vLineSegmentMatch, params.lengths);
  vArray(vLineSegment) segments = vnew(vLineSegment, params.lengths);
  vArray(double) lengths = vnew(double, params.lengths);
  for (i = 0; i < params.lengths; i++)
  {
    lengths[i] = params.low_length + length_step * (double) i;
    double half_length = lengths[i] / 2;
    // The row_diff and col_diff is fixed given length and orientation
    double row_diff = half_length * vPrecomputedSin::Sin(orientation);
    double col_diff = half_length * vPrecomputedCos::Cos(orientation);
    segments[i].row1 = -row_diff;
    segments[i].col1 = -col_diff;
    segments[i].row2 = row_diff;
    segments[i].col2 = col_diff;
    current_matches[i].orientation = (float) orientation;
    current_matches[i].length = (float) lengths[i];
  }

  // Calculate line votes centered at each edge point. We go through
  // all edge points (number is the number of edge pixels).
  for (i = 0; i < number; i++)
  {
    center_row = (*edge_rows)[i];
    center_col = (*edge_cols)[i];
    float center_orientation = (float) orientations2[center_row][center_col];
    float orientation_diff = vCircularDistance((float) orientation, center_orientation, (float) M_PI);
    if (orientation_diff > .4) continue;

    current_matches[0].all_votes = 0;
    current_matches[0].positive_votes = 0;
    current_matches[0].strength = 0;

    long current_match = 0;
    long current_length = (long) lengths[current_match] / 2;

    // These variables will count the corresponding
    // votes and strengths for pixels that have the same length.
    // Note that, because we use positive and negative lengths,
    // pixels on opposite sides of the line never have the same length
    double votes, positive_votes, strengths, abs_votes, abs_positive_votes;
    votes = positive_votes = strengths = abs_votes = abs_positive_votes = 0;
    // count the number of pixels that have the same length;
    long previous_length = -100000;
    long previous_length_valid = 0;

    // Now go through all points in the search region
    for (k = 0; k < number2; k++)
    {
      // Compute the vote strength for the given point.
      long row = center_row + search_rows[k];
      long col = center_col + search_cols[k];
      if (edges->check_bounds(row, col) <= 0) continue;
      float edge = edges2[row][col];
      if (edge == 0) continue;
      float vote_strength = vote_strengths[row][col];

      long length = search_lengths[k];
      long abs_length = vAbs(length);

      if (length != previous_length)
      {
        previous_length = length;
        if (previous_length_valid == 0)
        {
          previous_length_valid = 1;
        }
        else // Add the votes and strengths for the current length.
        {
          if (abs_votes == 0) abs_votes = 1;
          double average_vote = votes / abs_votes;
          current_matches[current_match].all_votes += (float) average_vote;
          double average_positive_vote, average_strength;
          if (abs_positive_votes > 0)
          {
            average_positive_vote = positive_votes / abs_positive_votes;
            average_strength = strengths / abs_positive_votes;
            current_matches[current_match].positive_votes += (float) average_positive_vote;
            current_matches[current_match].strength += 
              (float) average_strength * (float) average_positive_vote;
          }
        }
        // Reset the counters to 0
        votes = 0;
        positive_votes = 0;
        strengths = 0;
        abs_votes = 0;
        abs_positive_votes = 0;
      }

      // We know that search_lengths is an increasing sequence. If we have reached
      // a length that exceeds the range of the current match, we start updating
      // the next match (after we transfer to it all measurements made for this match).
      if (abs_length > current_length)
      {
        while(abs_length > current_length)
        {
          current_match++;
          if (current_match >= params.lengths) break;
          current_length = (long) lengths[current_match] / 2;
          current_matches[current_match].all_votes = current_matches[current_match-1].all_votes;
          current_matches[current_match].positive_votes = current_matches[current_match-1].positive_votes;
          current_matches[current_match].strength = current_matches[current_match-1].strength;
        }
        if (current_match >= params.lengths)
        {
          break;
        }
      }
      
      double vote_weight = vAbs(vote_strength);
      abs_votes += vote_weight;
      votes += vote_weight * vote_strength;
      if (vote_strength > 0)
      {
        abs_positive_votes += vote_weight;
        positive_votes += vote_weight * vote_strength;
        strengths += vote_weight * edge;
      }
    }

    // Add the info from the last length into the current match.
    if (current_match < params.lengths)
    {
      if (abs_votes == 0) abs_votes = 1;
      double average_vote = votes / abs_votes;
      current_matches[current_match].all_votes += (float) average_vote;
      double average_positive_vote, average_strength;
      if (abs_positive_votes > 0)
      {
        average_positive_vote = positive_votes / abs_positive_votes;
        average_strength = strengths / abs_positive_votes;
        current_matches[current_match].positive_votes += (float) average_positive_vote;
        current_matches[current_match].strength += 
          (float) average_strength * (float) average_positive_vote;
      }
    }
          
    // In case we didn't initialize some matches, we do it now
    for (j = current_match + 1; j < params.lengths; j++)
    {
      current_matches[j].all_votes = current_matches[j-1].all_votes;
      current_matches[j].positive_votes = current_matches[j-1].positive_votes;
      current_matches[j].strength = current_matches[j-1].strength;
    }

    for (j = 0; j < params.lengths; j++)
    {
      if (current_matches[j].all_votes <= 0) continue;
      if ((j > 0) && 
          (current_matches[j].all_votes <= current_matches[j-1].all_votes))
      {
        continue;
      }
      
      current_matches[j].segment.row1 = center_row + (long) segments[j].row1;
      current_matches[j].segment.row2 = center_row + (long) segments[j].row2;
      current_matches[j].segment.col1 = center_col + (long) segments[j].col1;
      current_matches[j].segment.col2 = center_col + (long) segments[j].col2;
      current_matches[j].center_row = center_row;
      current_matches[j].center_col = center_col;

      if (current_matches[j].positive_votes <= 0)
      {
        exit_error("Error\n");
      }
      current_matches[j].strength /= current_matches[j].positive_votes;
      current_matches[j].vote_density = current_matches[j].all_votes / current_matches[j].length;

      current_matches[j].d = 0;
      current_matches[j].o = vRound(current_matches[j].orientation * 180 / M_PI);
      current_matches[j].l = vRound(current_matches[j].length);
      current_matches[j].v = vRound(current_matches[j].vote_density * 100);
      current_matches[j].s = vRound(current_matches[j].strength);
      results->push_back(current_matches[j]);
    }
  }

  vdelete2(current_matches);
  vdelete2(segments);
  vdelete2(lengths);
  return 1;
}


// Inputs:
// orientation, high_length, angle_tolerance, distance_tolerance:
// These inputs define a search region that, when centered on the
// center of a line segment, it can be used to define the pixels
// that can vote for that segment. The region is as defined in CVPR 2003:
// It includes all pixels whose projection onto the segment is within
// high_length/2 of the center, and for which the angle between the 
// segment and the line connecting the pixel to the center is 
// within angle_tolerance. It also includes pixels that, in the direction
// perpendicular to the segment, are within distance_tolerance of 
// the pixels fitting the high_length and angle_tolerance criteria.
// 
// Outputs: search_rows and search_cols at index i,
// contain the row and col of the i-th search region point. 
// search_lengths[i] is the distance of the projection of that point
// into the corresponding line segment from the center of the line
// segment. The distance can be negative, if the projection falls
// onto the first half of the segment, and positive if the projection
// falls onto the second half of the segment. The points in the 
// search region are sorted in order of absolute value of the search length.
long vGetSearchRegion(double orientation, double high_length, 
                       double angle_tolerance, long distance_tolerance,
                       vector<long> * search_rows, vector<long> * search_cols, 
                       vector<long> * search_lengths)
{
  vector<double> rows, cols;
  // Get the line points corresponding to the given orientation
  // and high length, centered at point (0,0), and incrementing
  // in either rows or cols in steps no greater than 0.5.
  vLinePoints2(0, 0, orientation, high_length, .5, &rows, &cols);
  double perpendicular = orientation + M_PI/2;
  double tangent = vPrecomputedTan::Tan(angle_tolerance);
  long i;
  long number = rows.size();
  vector<long> all_rows, all_cols, all_lengths;
  // Go through each line segment point, and go along 
  // the direction normal to the line segment, to include
  // the additional points that should be in the search region.
  for (i = 0; i < number; i++)
  {
    long factor;
    // Figure out if we have crossed the center point yet or not.
    // The factor is -1 if not, 1 if yes.
    if (i < number/2) factor = -1;
    else factor = 1;

    vector<double> rows2, cols2;
    // Get the current point on the line.
    double row = rows[i];
    double col = cols[i];
    // Get the current distance from the center.
    double length = sqrt(row*row + col*col);
    // Figure out how much from the curren point along the 
    // normal to the segment we should go to include points
    // into the search region.
    double range = 2 * (tangent * length + distance_tolerance);
    // Get the line points along the normal that we should include.
    vLinePoints2(row, col, perpendicular, range, .5, &rows2, &cols2);
    long size = rows2.size();
    long j;
    // Add the line points along the normal into the search region.
    for (j = 0; j < size; j++)
    {
      all_rows.push_back(vRound(rows2[j]));
      all_cols.push_back(vRound(cols2[j]));
      all_lengths.push_back((long) (length * factor));
    }
  }

  // Because of the sampling method, points are repeated. We want to avoid
  // repetitions. We do that here.
  // Bounding distance is a loose upper bound on the maximum distance
  // between a point included in the search region and the center of
  // the segment.
  long bounding_distance = (long) (high_length/2 + tangent * (double) high_length + distance_tolerance + 5);
  // scratch will be used to find duplicate points in the search region.
  vMatrix<uchar> scratch = vMatrix<uchar>(-bounding_distance, bounding_distance,
                                            -bounding_distance, bounding_distance);
  vArray2(uchar) scratch2 = scratch.GetData();

  number = all_rows.size();
  // First, for each point in the search region, make the corresponding
  // entry in scratch 0.
  for (i = 0; i < number; i++)
  {
    long row = all_rows[i];
    long col = all_cols[i];
    scratch2[row][col] = 0;
  }

  // These vectors will hold the unduplicated search region points,
  // i.e. all the search region points, but without repetitions.
  vector<long> temp_rows, temp_cols, temp_lengths;

  long counter = 0;
  // Go through the search region points.
  for (i = 0; i < number; i++)
  {
    long row = all_rows[i];
    long col = all_cols[i];
    // Check if the point has been seen before.
    if (scratch2[row][col] != 0) continue;
    // If it is the first occurrence of this point, mark that
    // now we have seen it, and add it to the unduplicated points.
    scratch2[row][col] = 1;
    temp_rows.push_back(row);
    temp_cols.push_back(col);
    temp_lengths.push_back(all_lengths[i]);
    counter++;
  }

  // Rearrange points, so that shorter lengths come before longer lengths
  // count sort is used.
  long length_limit = (long) (high_length + 4);
  number = temp_lengths.size();
  vArray(long) length_counts = vnew(long, length_limit);
  vArray(long) length_indices = vnew(long, length_limit);
  for (i = 0; i < length_limit; i++)
  {
    length_counts[i] = 0;
  }

  // count occurrences of each length
  for (i = 0; i < number; i++)
  {
    long length = vAbs(temp_lengths[i]);
    length_counts[length]++;
  }

  // compute starting indices for each length
  length_indices[0] = 0;
  for (i = 1; i < length_limit; i++)
  {
    length_indices[i] = length_indices[i-1] + length_counts[i-1];
  }

  // Rearrange points and lengths in sorted order (into all_rows, all_cols, all_lengths)
  for (i = 0; i < number; i++)
  {
    long length = vAbs(temp_lengths[i]);
    long index = length_indices[length];
    length_indices[length]++;
    all_rows[index] = temp_rows[i];
    all_cols[index] = temp_cols[i];
    all_lengths[index] = temp_lengths[i];
  }
  // Here is the ending of count sort.

  // put results into search_rows, search_cols, search_lengths;
  for (i = 0; i < number; i++)
  {
    search_rows->push_back(all_rows[i]);
    search_cols->push_back(all_cols[i]);
    search_lengths->push_back(all_lengths[i]);
  }
  vdelete2(length_counts);
  vdelete2(length_indices);

  return 1;
}


// Inputs:
// the_image, edges: gray scale image and its corresponding edge image.
// orientation, length: they specify the length and orientation of
// line segments for which votes will be counted (using
// vOrientedLines_d, which is the first and worst version of
// vOrientedLines_f).
// distance_tolerance, angle_tolerance: they specify the search
// region on which votes will be counted for each segment.
// Result:
// A matrix that shows, for each each pixel, the sum of votes
// of all line segments (i.e. line segments of all lengths)
// centered at that pixel.
v3dMatrix<double> * vTestOrientedLinesD(grayscale_image * the_image, grayscale_image * edges,
                                           double orientation, double length,
                                           long distance_tolerance, double angle_tolerance)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  if (rows != edges->Rows()) return 0;
  if (cols != edges->Cols()) return 0;

  // Get the orientations and gradient norms needed to call vOrientedLines_d.
  v3dMatrix<double> * orientations = vEdgeOrientations(the_image);
  v3dMatrix<double> * gradient_norms = vGradientNorm0(the_image);
  v3dMatrix<double> * double_weighted_edges = vSelectPixels(gradient_norms, edges);
  grayscale_image * weighted_edges = double_weighted_edges->ToGrayBound();
  vdelete(gradient_norms);
  vdelete(double_weighted_edges);
  vector<vLineSegmentMatch> segments;

  // Prepare the search parameters.
  vLineSegmentSearchParams params(0, length, length, 1, angle_tolerance, distance_tolerance);
  vector<long> edge_rows, edge_cols;
  // Get the edge pixels.
  vGetNonZeroPoints(edges, &edge_rows, &edge_cols);
  // Get the line segments.
  vOrientedLines_d(orientations, weighted_edges, &edge_rows, &edge_cols, 
                    orientation, params, &segments);

  vdelete(orientations);
  vdelete(weighted_edges);
  // Create the result matrix.
  v3dMatrix<double> * result = new v3dMatrix<double>(rows, cols, 1);
  vWriteValue(result, (double) 0);
  long number = segments.size();
  vArray2(double) result2 = result->Matrix2(0);
  
  long i;
  for (i = 0; i < number; i++)
  {
    // Get the current segment.
    vLineSegmentMatch segment = segments[i];
    long row = (segment.segment.row1 + segment.segment.row2) / 2;
    long col = (segment.segment.col1 + segment.segment.col2) / 2;
    // Add the votes of the segment to the value stored at the pixel 
    // corresponding to the center of the segment.
    result2[row][col] += segment.all_votes;
  }

  return result;
}


// Inputs:
// the_image, edges: gray scale image, and edge pixels for that
// image. The edge image stores the intensity gradient norm
// for each edge pixel.
// params specifies the different sets of parameters that will
// be used to search for line segments.
// Output:
// The parameter results will receive all the line segments that
// were found.
// This function calles vOrientedLines_f for many different
// orientations, as specified in params.orientations.
long vGetLineSegments(grayscale_image * the_image, grayscale_image * edges, 
                       vLineSegmentSearchParams params, 
                       vector<vLineSegmentMatch> * results)
{
  if (params.IsValid() == 0) return 0;
  vector<long> edge_rows, edge_cols;
  // Extract the edge pixels.
  vGetNonZeroPoints(edges, &edge_rows, &edge_cols);
  // Extract the pixel orientations.
  v3dMatrix<double> * orientations_image = vEdgeOrientations(the_image);

  long i;
  // Extract line segments for each orientation.
  for (i = 0; i < params.orientations; i++)
  {
    // Get the current orientations.
    double orientation = M_PI / params.orientations * i;
    // Get the line segments for this orientation.
    vOrientedLines_f(orientations_image, edges,
                      &edge_rows, &edge_cols, orientation,
                      params, results);
    vPrint("i = %li, number = %li\n", i, (long) results->size());
  }

  vdelete(orientations_image);
  return 1;
}


// We sort line segments based on many different criteria,
// and then we sort them again based on the sum of the logs
// of the ranks of each segment under each criterion.
// The criteria are strength (saliency), vote density (votes/length)
// and length. segments is both an input and an output argument.
long vSortLineSegments(vector<vLineSegmentMatch> * segments)
{
  long number = segments->size();
  // These vectors will each store scores for all segments, that 
  // eventually will be sorted in different ways.
  vector<vPointerScore> v1(number), v2(number), v3(number);

  long i;
  // First, prepare the v1 vector, where segments will be sorted
  // based on the saliency (edge strength).
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    double score = -segment->strength;
    vPointerScore p_score(segment, score);
    v1[i] = p_score;
  }
  std::sort(v1.begin(), v1.end(), vPointerScoreLess());

  // Now prepare the v2 vector, where segments will be sorted
  // based on the vote density (votes divided by length).
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    double score = -segment->all_votes / segment->length;
    vPointerScore p_score(segment, score);
    v2[i] = p_score;
  }
  std::sort(v2.begin(), v2.end(), vPointerScoreLess());

  // Now prepare the v3 vector, where scores will be sorted
  // based on their length.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    double score = -segment->length;
    vPointerScore p_score(segment, score);
    v3[i] = p_score;
  }
  std::sort(v3.begin(), v3.end(), vPointerScoreLess());

  // Now, for each segment, we must sum up the logs of its ranks.
  // That score will be stored at the score slot of each segment.
  // First we make all the scores 0.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v1[i].pointer;
    segment->score = 0;
  }

  // Add the logs of the ranks of the segments as they appear in the v1 vector
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v1[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  // Add the logs of the ranks of the segments as they appear in the v2 vector
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v2[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  // Add the logs of the ranks of the segments as they appear in the v3 vector
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v3[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  // Sort the segments in increasing order of the sums of the logs
  // of their ranks.
  std::sort(segments->begin(), segments->end(), vLineSegmentMatchLess());
  
  return 1;
}


// Rank line segments based on criteria used in vSortLineSegments, and also
// based on additional criteria that have to do with how well each
// segment matches segment2. The segments vector is sorted, so it is
// both an input and an output argument.
// The criteria are differences in center location, orientation and length.
long vRankLineSegments(vector<vLineSegmentMatch> * segments, vLineSegment segment2)
{
  static const long junk = vPrecomputedLengths::PrecomputeValues(320, 320);
  // Sort the segments.
  vSortLineSegments(segments);

  long number = segments->size();
  // Create vectors that will be used to sort line segments
  // based on different criteria of how well they match segment2.
  vector<vPointerScore> v1(number), v2(number), v3(number);

  // Get the center, orientation and length of segment2.
  double dy = segment2.row2 - segment2.row1;
  double dx = segment2.col2 - segment2.col1;
  double length = sqrt(dy*dy + dx*dx);
  float orientation = (float) vAngle(dx, dy);
  double row = (segment2.row1 + segment2.row2) / 2;
  double col = (segment2.col1 + segment2.col2) / 2;

  long i;
  // Prepare vector v1, where segments will be sorted based on 
  // difference in length with respect to segment2.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    double score = vAbs(segment->length - length);
    segment->l = vRound(score);
    vPointerScore p_score(segment, score);
    v1[i] = p_score;
  }
  std::sort(v1.begin(), v1.end(), vPointerScoreLess());

  // Prepare vector v1, where segments will be sorted based on 
  // difference in orientation with respect to segment2.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    float score = vCircularDistance(segment->orientation, orientation, (float) M_PI);
    segment->o = vRound(score);
    vPointerScore p_score(segment, score);
    v2[i] = p_score;
  }
  std::sort(v2.begin(), v2.end(), vPointerScoreLess());

  // Prepare vector v1, where segments will be sorted based on 
  // distance in center location with respect to segment2.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = &((*segments)[i]);
    double segment_row = (segment->segment.row1 + segment->segment.row2) / 2;
    double segment_col = (segment->segment.col1 + segment->segment.col2) / 2;
    double row_diff = vAbs(segment_row - row);
    double col_diff = vAbs(segment_col - col);
    long score = vPrecomputedLengths::LengthR((long) row_diff, (long) col_diff);
    segment->d = score;
    vPointerScore p_score(segment, score);
    v3[i] = p_score;
  }
  std::sort(v3.begin(), v3.end(), vPointerScoreLess());

  // Add to the score of each segment (which contains the sums of
  // logs of ranks as computed by vSortLineSegments) the logs
  // of the ranks of the segment in vectors v1, v2 and v3.
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v1[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v2[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch * segment = (vLineSegmentMatch *) v3[i].pointer;
    segment->score += log((float) (i+1));
  }
  
  // Resort the segments.
  std::sort(segments->begin(), segments->end(), vLineSegmentMatchLess());

  return 1;
}


// For each segment in segments, compute its DOL difference to segment.
// DOL stands for Distance between centers, Orientation and Length.
long vLineSegmentDifferences(vector<vLineSegmentMatch> * segments, 
                              vLineSegment segment)
{
  static const long junk = vPrecomputedLengths::PrecomputeValues(320, 320);

  long number = segments->size();
  // Get the center and OL of segment.
  long dy = vAbs(vRound(segment.row2 - segment.row1));
  long dx = vAbs(vRound(segment.col2 - segment.col1));
  double length = vPrecomputedLengths::LengthR(dy, dx);
  float orientation = (float) vAngle(dx, dy);
  long row = (long) (segment.row1 + segment.row2) / 2;
  long col = (long) (segment.col1 + segment.col2) / 2;

  long i;
  // Go through all segments
  for (i = 0; i < number; i++)
  {
    // Compute the distance between center of current segment match
    // and center of segment.
    vLineSegmentMatch & segment2 = (*segments)[i];
    long segment_row = segment2.center_row;
    long segment_col = segment2.center_col;
    long row_diff = vAbs(segment_row - row);
    long col_diff = vAbs(segment_col - col);
    long distance = vPrecomputedLengths::LengthR(row_diff, col_diff);
    segment2.d = distance;

    // Compute orientation and length differences.
    long orientation_diff = (long) vCircularDistance(segment2.orientation, orientation, (float) M_PI);
    segment2.o = vAbs(orientation_diff);

    double length_diff = vAbs(segment2.length - length);
    segment2.l = vRound(length_diff);

  }

  return 1;
}



// It produces two images that can help visualize the line segments in
// segments. One image (*ranks_p) shows the rank of the segment (i.e.
// the index of the segment in segments). The other includes information
// about a segment in various bands. That information is the rank, orientation,
// length, strength and all_votes.
// Note that we have many (even 1000) segments defined at each pixel. Only
// the highest ranking segment is used here.
// k specifies how many segments we should use (for example, k=10 means
// that we should only use the top 10 segments). A value of k=-1 means
// that we should use all line segments.
long vLineSegmentRankImage(vector<vLineSegmentMatch> * segments, long rows, long cols,
                            v3dMatrix<long> ** ranks_p, v3dMatrix<long> ** info_p,
                            long k)
{
  // Create the results.
  v3dMatrix<long> *ranks = new v3dMatrix<long>(rows, cols, 1);
  vWriteValue(ranks, (long) 0);
  vArray3(long) ranks3 = ranks->Matrix3();

  v3dMatrix<long> * info = new v3dMatrix<long>(rows, cols, 8);
  vWriteValue(info, (long) 0);
  vArray3(long) info3 = info->Matrix3();
  
  long number;
  // Check if we need to set number to a default value or just to k.
  if (k <= -1)
  {
    number = segments->size();
  }
  else
  {
    number = k;
  }

  long i;
  for (i = 0; i < number; i++)
  {
    // Get the current segment.
    vLineSegmentMatch match = (*segments)[i];
    long row = match.center_row;
    long col = match.center_col;
    if (ranks->check_bounds(row, col) <= 0) continue;
    if (ranks3[0][row][col] != 0) continue;
    // Store information about this segment into the two result
    // matrices.
    ranks3[0][row][col] = number - i;
    info3[0][row][col] = i;
    info3[1][row][col] = number - i;
    info3[2][row][col] = vRound(match.score);
    info3[3][row][col] = match.d;
    info3[4][row][col] = match.o;
    info3[5][row][col] = match.l;
    info3[6][row][col] = match.v;
    info3[7][row][col] = match.s;
  }
  *ranks_p = ranks;
  *info_p = info;
  return 1;
}


// To account for the fact that input hand images have different bound box sizes
// for the hand, and we want all locations to correspond to a normalized 
// hand bounding box,
// of size 192x192 centered at 127*127, here we have to modify the search parameters
// for finding line segments. The user (or calling function) creates parameters
// that correspond to the normalized bounding box, and here we modify them to correspond
// to the bounding box of the input image (passed in as argument "box").
long vScaleSearchParams(vLineSegmentSearchParams * params, vBoundingBox * box, 
                         long new_size)
{
  double max_side = box->MaxSide();
  if ((max_side <= 0) || (new_size <= 0))
  {
    vPrint("vScaleSearchParams: impossible\n");
    exit(1);
    return 0;
  }
  double factor = max_side / ((double) new_size);
  params->low_length *= factor;
  params->high_length *= factor;

  return 1;
}


// We normalize segments extracted from a hand image where the hand has
// the given bounding box to segments corresponding to an image where
// the bounding box of the hand is centered at 
// (new_center_row, new_center_col) and the max side of the box is new_size.
long vNormalizeLineSegments(vector<vLineSegmentMatch> * segments, 
                             vBoundingBox * box, long new_size,
                             long new_center_row, long new_center_col)
{
  // We need to translate and scale the line segment, and also 
  // scale the length and the number of votes.

  double max_side = box->MaxSide();
  if (max_side <= 0) 
  {
    vPrint("vScaleSearchParams: impossible\n");
    exit(1);
    return 0;
  }
  
  double factor = ((double) new_size) / max_side;
  long length = segments->size();
  double old_center_row = box->CenterRow();
  double old_center_col = box->CenterCol();

  long i;
  for (i = 0; i < length; i++)
  {
    vLineSegmentMatch & match = (*segments)[i];
    vLineSegmentLong & segment = match.segment;
    match.length *= (float) factor;
    match.all_votes *= (float) factor;
    match.positive_votes *= (float) factor;
    match.l = vRound(match.length);
    double new_row1 = (segment.row1 - old_center_row) * factor + new_center_row;
    double new_col1 = (segment.col1 - old_center_col) * factor + new_center_col;
    double new_row2 = (segment.row2 - old_center_row) * factor + new_center_row;
    double new_col2 = (segment.col2 - old_center_col) * factor + new_center_col;
    
    segment.row1 = vRound(new_row1);
    segment.col1 = vRound(new_col1);
    segment.row2 = vRound(new_row2);
    segment.col2 = vRound(new_col2);
//    match.center_row = (segment.row1 + segment.row2) / 2;
//    match.center_col = (segment.col1 + segment.col2) / 2;
    match.center_row = vRound((match.center_row - old_center_row) * factor + new_center_row);
    match.center_col = vRound((match.center_col - old_center_col) * factor + new_center_col);
  }
  return 1;
}


long vLineSegmentMatch::Print1()
{
  vPrint("d = %li, o = %li, l = %li, v = %li, s = %li, score = %.1f\n",
          d, o, l, v, s, score);
  vPrint("orientation = %f\n", orientation);
  return 1;
}


vAxisProjection::vAxisProjection(grayscale_image * region)
{
  // Get the principal orientation
  orientation = vPrincipalOrientation(region, &center_row, &center_col);
  double d_row1, d_col1, d_row2, d_col2;
  d_row1 = d_col1 = d_row2 = d_col2 = 0;
  long image_rows = region->Rows();
  long image_cols = region->Cols();

  // Get the intersection of principal axis with image boundary.
  long success = vBoundaryPoints(center_row, center_col, orientation, 
                                  image_rows, image_cols,
                                  &d_row1, &d_col1, &d_row2, &d_col2);
  long row1 = vRound(d_row1);
  long col1 = vRound(d_col1);
  long row2 = vRound(d_row2);
  long col2 = vRound(d_col2);
  
  // For each point on the principal axis, find the number of hand pixels
  // that project there.
  // First, figure out the step at which we will sample points.
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(row1, col1, row2, col2, &row_step, &col_step, &steps);
  number_of_points = steps;
  rows = vnew(long, number_of_points);
  cols = vnew(long, number_of_points);
  counts = vnew(float, number_of_points);

  // So, we will consider number_of_points points on 
  // the principal axis, and for
  // each of them we will count the number of hand pixels that get
  // projected there. The counters array will keep track of all those
  // counts.

  double perpendicular = orientation + M_PI/2;
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    double d_axis_point_row = d_row1 + i * row_step;
    double d_axis_point_col = d_col1 + i * col_step;
    rows[i] = vRound(d_axis_point_row);
    cols[i] = vRound(d_axis_point_col);
    long boundary_flag = 0;
    counts[i] = (float) vCountProjectedPoints(region, rows[i], 
                                          cols[i], perpendicular,
                                          &boundary_flag);
  }

  // Find the first index with non-zero projection.
  for (i = 0; i < number_of_points; i++)
  {
    if (counts[i] != 0) break;
  }
  long first_non_zero = i;

  // Find the last index with non-zero projection
  for (i = number_of_points - 1; i >= 0; i--)
  {
    if (counts[i] != 0) break;
  }
  long last_non_zero = i;

  if (last_non_zero < first_non_zero)
  {
    vdelete2(rows);
    vdelete2(cols);
    vdelete2(counts);
    number_of_points = 0;
  }

  // Cut out the information about axis points that are 
  // outside the region of interest.
  Cut(first_non_zero, last_non_zero);
}


vAxisProjection::vAxisProjection(v3dMatrix<float> * region)
{
  grayscale_image * binary = (grayscale_image *) region->threshold(1);

  // Get the principal orientation
  orientation = vPrincipalOrientation(binary, &center_row, &center_col);
  double d_row1, d_col1, d_row2, d_col2;
  d_row1 = d_col1 = d_row2 = d_col2 = 0;
  long image_rows = region->Rows();
  long image_cols = region->Cols();

  // Get the intersection of principal axis with image boundary.
  long success = vBoundaryPoints(center_row, center_col, orientation, 
                                  image_rows, image_cols,
                                  &d_row1, &d_col1, &d_row2, &d_col2);
  long row1 = vRound(d_row1);
  long col1 = vRound(d_col1);
  long row2 = vRound(d_row2);
  long col2 = vRound(d_col2);
  
  // For each point on the principal axis, find the number of hand pixels
  // that project there.
  // First, figure out the step at which we will sample points.
  double row_step, col_step;
  long steps;
  row_step = col_step = steps = 0;
  vLineSteps(row1, col1, row2, col2, &row_step, &col_step, &steps);
  number_of_points = steps;
  rows = vnew(long, number_of_points);
  cols = vnew(long, number_of_points);
  counts = vnew(float, number_of_points);

  // So, we will consider number_of_points points on 
  // the principal axis, and for
  // each of them we will count the number of hand pixels that get
  // projected there. The counters array will keep track of all those
  // counts.

  double perpendicular = orientation + M_PI/2;
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    double d_axis_point_row = d_row1 + i * row_step;
    double d_axis_point_col = d_col1 + i * col_step;
    rows[i] = vRound(d_axis_point_row);
    cols[i] = vRound(d_axis_point_col);
    long boundary_flag = 0;
    counts[i] = (float) vSumProjectedPoints(region, rows[i], 
                                     cols[i], perpendicular,
                                     &boundary_flag);
  }

  // Find the first index with non-zero projection.
  for (i = 0; i < number_of_points; i++)
  {
    if (counts[i] != 0) break;
  }
  long first_non_zero = i;

  // Find the last index with non-zero projection
  for (i = number_of_points - 1; i >= 0; i--)
  {
    if (counts[i] != 0) break;
  }
  long last_non_zero = i;

  if (last_non_zero < first_non_zero)
  {
    vdelete2(rows);
    vdelete2(cols);
    vdelete2(counts);
    number_of_points = 0;
  }

  // Cut out the information about axis points that are 
  // outside the region of interest.
  Cut(first_non_zero, last_non_zero);
}


vAxisProjection::~vAxisProjection()
{
  vdelete2(rows);
  vdelete2(cols);
  vdelete2(counts);
}


// Return an image where, for each axis point, we draw a line
// perpendicular to the axis, centered at that point, 
// with length equal to the corresponding value in counts (the
// number of region pixels that project onto that pixel).
grayscale_image * vAxisProjection::DrawProjections()
{
  if (number_of_points == 0) return 0;
  // First figure out the size we need for the result image. 
  // It should fit the axis points and the longest possible 
  // projection.
  long max_row, max_col;
  max_row = max_col = 0;
  MaxRowCol(&max_row, &max_col);
  long max_index, max_width, temp1, temp2;
  max_index, max_width = temp1 = temp2 = 0;
  MaxWidth(&max_index, &temp1, &temp2, &max_width);
  long image_rows = max_row + max_width + 10;
  long image_cols = max_col + max_width + 10;

  // Create the result image.
  grayscale_image * result_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(result_image, (uchar) 0);

  // Draw lines for each projection (i.e. for each axis point).
  double perpendicular = orientation + M_PI / 2;
  long i;
  for (i = 0; i < number_of_points; i++)
  {
    long row = rows[i];
    long col = cols[i];
    long count = (long) counts[i];
    vDrawCenteredLine(result_image, row, col, perpendicular, 
                       count, (uchar) 255);
  }
  return result_image;
}


// Return the position at with at the axis point with the smallest 
// width
long vAxisProjection::MinWidth(long * indexp, long * rowp, 
                                long * colp, long * widthp)
{
  if (number_of_points == 0) return 0;
  long min_index, min_width;
  min_index = 0;
  min_width = (long) counts[0];
  long i;
  for (i = 1; i < number_of_points; i++)
  {
    long width = (long) counts[i];
    if (width < min_width)
    {
      min_width = width;
      min_index = i;
    }
  }

  *indexp = min_index;
  *rowp = rows[min_index];
  *colp = cols[min_index];
  *widthp = min_width;
  return 1;
}


// Return the position at with at the axis point with the biggest 
// width
long vAxisProjection::MaxWidth(long * indexp, long * rowp, 
                                long * colp, long * widthp)
{
  if (number_of_points == 0) return 0;
  long max_index, max_width;
  max_index = 0;
  max_width = (long) counts[0];
  long i;
  for (i = 1; i < number_of_points; i++)
  {
    long width = (long) counts[i];
    if (width > max_width)
    {
      max_width = width;
      max_index = i;
    }
  }

  *indexp = max_index;
  *rowp = rows[max_index];
  *colp = cols[max_index];
  *widthp = max_width;
  return 1;
}


// Returns a copy of this object.
vAxisProjection * vAxisProjection::Copy()
{
  if (number_of_points == 0) return 0;

  vAxisProjection * result = new vAxisProjection;
  result->number_of_points = number_of_points;
  result->orientation = orientation;
  result->length = length;
  result->center_row = center_row;
  result->center_col = center_col;

  result->rows = vnew(long, number_of_points);
  result->cols = vnew(long, number_of_points);
  result->counts = vnew(float, number_of_points);

  long i;
  for (i = 0; i < number_of_points; i++)
  {
    result->rows[i] = rows[i];
    result->cols[i] = cols[i];
    result->counts[i] = counts[i];
  }
  return result;
}

// Cuts the axis, keeping the region from startpoint to endpoint. 
// Information about the rest of the points is discarded.
long vAxisProjection::Cut(long start, long end)
{
  if (number_of_points == 0) return 0;
  if (start < 0) return 0;
  if (end < start) return 0;
  if (end >= number_of_points) return 0;
  long i;
  for (i = start; i <= end; i++)
  {
    long index = i-start;
    rows[index] = rows[i];
    cols[index] = cols[i];
    counts[index] = counts[i];
  }

  number_of_points = end - start + 1;
  return 1;
}

// The largest row and largest column occurring on the axis points.
long vAxisProjection::MaxRowCol(long * rowp, long * colp)
{
  if (number_of_points == 0) return 0;
  long max_row = rows[0];
  long max_col = cols[0];

  long i;
  for (i = 0; i < number_of_points; i++)
  {
    long row = rows[i];
    long col = cols[i];

    if (row > max_row) max_row = row;
    if (col > max_col) max_col = col;
  }

  *rowp = max_row;
  *colp = max_col;
  return 1;
}

  
// Reverses the arrays rows, cols and counts.
long vAxisProjection::Reverse()
{
  if (number_of_points == 0) return 0;
  long half = number_of_points / 2;
  long i;
  for (i = 0; i <= half; i++)
  {
    long index = number_of_points - i - 1;
    long temp_row = rows[index];
    long temp_col = cols[index];
    long temp_count = (long) counts[index];
    
    rows[index] = rows[i];
    cols[index] = cols[i];
    counts[index] = counts[i];

    rows[i] = temp_row;
    cols[i] = temp_col;
    counts[i] = (float) temp_count;
  }
  
  return 1;
}


// I have made an interface that lets me click twice to specify
// the palm cutoff line segment of a model image. To save time, 
// I have only labeled one out of every 12 frames (so, for each 
// set of 12 frames that are image-plane-rotated versions of each 
// other I have only labeled the first). The locations of the 
// cutoff segments of the remaining 11 frames can be derived by
// figuring out the appropriate translation and scale factor.
// This function does that, and returns the cutoff segments
// of all database views. Furthermore, for each frame of the database,
// it generates the cutoff segments that we get by rotating the 
// frame by 90, 180 and 270 degrees.
v3dMatrix<uchar> * vExtendCutoffs(v3dMatrix<uchar> * basic_cutoffs,
                                     vVideoFile * database)
{
  long basic_frames = basic_cutoffs->Rows();
  long database_frames = database->NumberOfFrames();
  long views = database_frames * 4;
  if (basic_frames != database_frames / 12) return 0;

  v3dMatrix<uchar> * result = new v3dMatrix<uchar>(views, 4, 1);
  vArray2(uchar) basic = basic_cutoffs->Matrix2(0);
  vArray2(uchar) extended = result->Matrix2(0);

  vPrint("\n");
  long base_frame;
  for (base_frame = 0; base_frame < database_frames; base_frame+= 12)
  {
    long basic_index = base_frame / 12;
    uchar base_row1 = basic[basic_index][0];
    uchar base_col1 = basic[basic_index][1];
    uchar base_row2 = basic[basic_index][2];
    uchar base_col2 = basic[basic_index][3];

    grayscale_image * base_image = database->ReadFrameGray(base_frame);
    double center_row, center_col, diameter;
    center_row = center_col = diameter = 0;
    vCenterOfMass(base_image, center_row, center_col);
//    center_row = 127;
//    center_col = 127;
    diameter = vDiameter(base_image);

    long rotated_frame;
    for (rotated_frame = 0; rotated_frame < 12; rotated_frame++)
    {
      grayscale_image * rotated = database->ReadFrameGray(base_frame + rotated_frame);
      long view;
      for (view = 0; view < 4; view++)
      {
        grayscale_image * current_frame = (grayscale_image *) vRotateRight(rotated, view * 90);
        double center_row2, center_col2, diameter2;
        center_row2 = center_col2 = diameter2 = 0;
        vCenterOfMass(current_frame, center_row2, center_col2);
//        center_row2 = 127;
//        center_col2 = 127;
        diameter2 = vDiameter(current_frame);
      
        double scale = diameter2 / diameter;
        double angle = -(rotated_frame * M_PI / 24) + view * M_PI / 2;

        // Now, for each point of the cutoff, apply the transformation
        // defined by the given centers, scale and rotation.
        double rotated_row1, rotated_col1, rotated_row2, rotated_col2;
        rotated_row1 = rotated_row2 = rotated_col1 = rotated_col2 = 0;
        vWarpPoint(base_row1, base_col1, &rotated_row1, &rotated_col1,
                    center_row, center_col, center_row2, center_col2, 
                    scale, angle);
        vWarpPoint(base_row2, base_col2, &rotated_row2, &rotated_col2,
                    center_row, center_col, center_row2, center_col2, 
                    scale, angle);
        long extended_index = (base_frame + rotated_frame) * 4 + view;
        extended[extended_index][0] = (uchar) rotated_row1;
        extended[extended_index][1] = (uchar) rotated_col1;
        extended[extended_index][2] = (uchar) rotated_row2;
        extended[extended_index][3] = (uchar) rotated_col2;
        vdelete(current_frame);
      }
      vdelete(rotated);
    }
    vdelete(base_image);
    vPrint("Done with frame %li of %li\r", base_frame, database_frames);
  }
  vPrint("\n");
  return result;
}


// Returns the maximum distance between two points in the image. We
// assume that the image is connected and we can just consider
// contour points.
double vDiameter(grayscale_image * binary)
{
  vContour contour(binary);
  vector<double> * xs = contour.GetXs();
  vector<double> * ys = contour.GetYs();

  long points = xs->size();
  long i, j;
  double max_distance = 0;
  for (i = 0; i < points; i++)
  {
    double row1 = (*ys)[i];
    double col1 = (*xs)[i];

    for (j = i+1; j < points; j++)
    {
      double row2 = (*ys)[j];
      double col2 = (*xs)[j];
      double row_diff = row2 - row1;
      double col_diff = col2 - col1;
      double distance = row_diff * row_diff + col_diff * col_diff;
      if (distance > max_distance) max_distance = distance;
    }
  }
  return sqrt(max_distance);
}



// Apply the transformation defined by t_row, t_col (for translation),
// scale and angle (for rotation) to the point (in_row, in_col). The
// results are put in out_rowp and out_colp. The two origins are 
// specified in (center_row1, center_col1) and (center_row2, center_col2);
// This is what we do:
// - Translate the point so that its center becomes (0,0) from 
// (center_row1, center_col1).
// - Rotate the point by angle
// - Scale the point by scale
// - Translate the point so that the center becomes (center_row2, center_col2)
// from (0,0)
long vWarpPoint(double in_row, double in_col, 
                 double * out_rowp, double * out_colp,
                 double center_row1, double center_col1,
                 double center_row2, double center_col2,
                 double scale, double angle)
{
  double row = in_row;
  double col = in_col;

  // Translate (make zero-centered)
  row -= center_row1;
  col -= center_col1;

  // rotate
  vRotatePoint(col, row, angle, col, row);

  // scale
  row *= scale;
  col *= scale;

  // Translate (make center_row2, center_col2 the origin)
  row += center_row2;
  col += center_col2;

  // Store result into out_rowp and out_colp;
  *out_rowp = row;
  *out_colp = col;

  return 1;
}


// Suppose we have a line segment, extracted from an image where
// the non-zero region has the given bounding box. Suppose that 
// we normalize the image, so that the non-zero region has the given
// "size", and it is centered at the center of a window with the
// given "window_size". How would the various properties of 
// the input line segment be affected? The updated (normalized) properties
// are stored in the result.
vLineSegmentLong vNormalizeSegment(vLineSegmentLong * input, vBoundingBox * box,
                                     long size, long window_size)
{
  // Compute scale of old bounding box and center of new bounding box.
  double scale = ((double) size) / (double) box->MaxSide();
  double center = (window_size - 1) / 2;
  
  double row1 = input->row1;
  double col1 = input->col1;
  double row2 = input->row2;
  double col2 = input->col2;

  // Translate points so that the center of the box becomes the origin
  row1 -= box->CenterRow();
  col1 -= box->CenterCol();
  row2 -= box->CenterRow();
  col2 -= box->CenterCol();

  // Scale points
  row1 *= scale;
  col1 *= scale;
  row2 *= scale;
  col2 *= scale;

  // Translate points so that origin moves to new center (center, center)
  row1 += center;
  col1 += center;
  row2 += center;
  col2 += center;

  vLineSegmentLong result;
  result.row1 = vRound(row1);
  result.col1 = vRound(col1);
  result.row2 = vRound(row2);
  result.col2 = vRound(col2);
  return result;
}


// We want to sample points uniformly from the space [0, ..., max_value] so
// that we have "samples" points in the sample, and the max distance
// between any point in [0,...,max_value] and its closest sample point
// is minimized.
// The desired spacing is max_value/samples, and the desired start point
// is max_value / (2*samples). Then, each sample point covers an area around it
// of radius max_value/(2*samples) and diameter max_value/samples, which is optimal.
vArray(long) vUniformSpacing(long max_value, long samples)
{
  if (samples == 0) return vZero(long);
  vArray(long) result = vnew(long, samples);
  double step = ((double) max_value) / (double) samples;
  double start = ((double) max_value) / (double) (2*samples);
  long i; 
  for (i = 0; i < samples; i++)
  {
    double value = start + ((double) i) * step;
    result[i] = vRound(value);
  }
  return result;
}


// Here, unlike vUniformSpacing, we deal with the case where our
// space is circular, and max_value is the same as 0. This is,
// for example, the case when we are dealing with orientations, where 
// M_PI, or 2*M_PI (depending on the case) are identified with 0.
// Notice that the way I use this function in vLineGrid, if I change
// the implementation here it may break vLineGrid.
// Right now it seems (I'm 99% sure) that the solution is the same
// as for vUniformSpacing. Basically, we can add an arbitrary
// number to all samples obtained from vUniformSpacing and we still
// get an optimal solution here. The solution that vUniformSpacing
// returns is the only one that will work for vLineGrid.
vArray(long) vCircularUniformSpacing(long max_value, long samples)
{
  return vUniformSpacing(max_value, samples);
}


// Compute the minimum enclosing circle for the non-zero
// region of color_image. Works with general_image arguments.
vCircleLong vMinEnclosingCircle(general_image * the_image)
{
  grayscale_image * binary = wrim(the_image).Binary();
  vCircleLong result = vMinEnclosingCircle(binary);
  vdelete(binary);
  return result;
}


// Compute the minimum enclosing circle for the non-zero
// region of color_image. Works for grayscale_image arguments.
vCircleLong vMinEnclosingCircle(grayscale_image * the_image)
{
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  long number = 0;
  vGetNonZeroPoints4(the_image, &rows, &cols, &number);

  vCircleLong circle = vMinEnclosingCircle3(rows, cols, number);
  vdelete2(rows);
  vdelete2(cols);

  return circle;
}


// We find the top-most, bottom-most, left-most and right-most coordinate
// values of non-zero pixels in input, and record them into the result's
// member variables. In a multi-band image, a pixel is non-zero if any
// of its bands is non-zero.
// The function would break down if negative coordinates are allowed,
// or coordinates over a billion are allowed.
vBoundingBox vFindBoundingBox(vArray(long) rows, vArray(long) cols, 
                                long number)
{
  vBoundingBox result;

  long top, bottom, left, right;
  top = 1000000000;
  bottom = -1;
  left = 1000000000;
  right = -1;

  long i;
  for (i = 0; i < number; i++)
  {
    long row = rows[i];
    long col = cols[i];
    if (row < top) top = row;
    if (row > bottom) bottom = row;
    if (col < left) left = col;
    if (col > right) right = col;
  }

  result.top = top;
  result.bottom = bottom;
  result.left = left;
  result.right = right;
  return result;
}


// Find the minimum enclosing circle for the set of points described
// by row and cols. The number of those points is given in "number".
// Maybe (but I haven't verified) this function can be made faster and
// still correct by starting at the bounding box center and doing
// a gradient descent search for the min max_distance. It would definitely
// be faster if we used a multiresolution method. At this point it
// can process 80 frames per second, which is good enough for my purposes.
vCircleLong vMinEnclosingCircle3(vArray(long) rows, vArray(long) cols,
                                   long number)
{
  // First, get the convex hull of the objects.
  vArray(long) hull_rows = vZero(long);
  vArray(long) hull_cols = vZero(long);
  long hull_number = 0;
  vConvexHull(rows, cols, number, &hull_rows, &hull_cols, &hull_number);

  // Now, get the bounding box of the convex hull
  vBoundingBox box = vFindBoundingBox(hull_rows, hull_cols, hull_number);
  long top = box.top;
  long bottom = box.bottom;
  long left = box.left;
  long right = box.right;

  // Find the point inside the bounding box that has the smallest 
  // distance to its farthest neighbor among the convex hull points.
  long min_row = -1;
  long min_col = -1;
  long min_radius = 100000000;
  long row, col;
  for (row = top; row <= bottom; row++)
  {
    for (col = left; col <= right; col++)
    {
      long max_distance = 0;
      long i;
      // Find the distance between point (row, col) and its 
      // farthest neighbor among hull points.
      for (i = 0; i < hull_number; i++)
      {
        long row_diff = row - hull_rows[i];
        long col_diff = col - hull_cols[i];
        long distance = row_diff * row_diff + col_diff * col_diff;
        if (distance > max_distance) max_distance = distance;
      }
      if (max_distance < min_radius)
      {
        min_radius = max_distance;
        min_row = row;
        min_col = col;
      }
    }
  }

  min_radius = vRound(sqrt((float) min_radius));
  vCircleLong result = vCircleLong(min_row, min_col, min_radius);
  vdelete2(hull_rows);
  vdelete2(hull_cols);
  return result;
}


// Returns a binary image with the specified size, that contains
// "number" non-zero pixels, each of which is no farther than
// "radius" distance from the center of the image.
// This function can be made more efficient: if the radius is small,
// we should pick random pixels in a narrower range.
grayscale_image * vRandomEdgeImage(ulong rows, ulong cols, ulong number,
                                ulong radius)
{
  long junk = vPrecomputedLengths::PrecomputeValues(rows, cols);
  if ((rows == 0) || (cols == 0)) return 0;

  grayscale_image * result = new grayscale_image(rows, cols);
  vWriteValue(result, (uchar) 0);
  vArray2(uchar) result2 = result->Matrix2(0);
  
  long min_side = Min(rows, cols);
  long min_radius = min_side / 2 - 1;
  if ((long) radius > min_radius) radius = (ulong) min_radius;
  long center_row = (rows-1) / 2;
  long center_col = (cols-1) / 2;

  long i;
  for (i = 0; i < (long) number; i++)
  {
    long current_row = vRandomLong2(0, rows-1);
    long current_col = vRandomLong2(0, cols-1);
    long row_diff = vAbs(current_row - center_row);
    long col_diff = vAbs(current_col - center_col);
    long length = vPrecomputedLengths::Length(row_diff, col_diff);
    if (length > (long) radius) 
    {
      i--;
      continue;
    }
    result2[current_row][current_col] = 255;
  }

  return result;
}


// Returns a gray scale image that is the distance transform
// of color_image. It calls the OpenCV function, and converts
// the result from float to uchar. Values over 255 are
// cropped to 255.
grayscale_image * vDistanceTransformUchar(general_image * the_image)
{
  v3dMatrix<float> * dt = vCVDistanceTransform(the_image);
  long rows = dt->Rows();
  long cols = dt->Cols();
  
  grayscale_image * result = new grayscale_image(rows, cols);
  vArray(uchar) result1 = result->Matrix(0);
  vArray(float) dt1 = dt->Matrix(0);

  long size = rows * cols;
  long i;
   
  for (i = 0; i < size; i++)
  {
    float value = dt1[i];
    if (value > 255)
    {
      result1[i] = 255;
    }
    else
    {
      result1[i] = (uchar) vRound(value);
    }
  }

  vdelete(dt);
  return result;
}


// I'm writing this function just out of curiosity to see the 
// Voronoi diagram for a set of polygons. Each region in the 
// diagram is the set of points that are closer to a specific
// polygon than to any other polygon. The output is an image
// where the regions are colored randomly, and each polygon
// is drawn in black.
color_image * vLineSetVoronoi(vLineSet * line_set)
{
  if (line_set->NumberOfPolygons() == 0) return 0;

  long top, bottom, left, right;
  top = bottom = left = right = 0;
  // Get the boundary of the line set, to figure out the size
  // of the result image.
  line_set->GetBounds(&top, &bottom, &left, &right);
  color_image * result = new color_image(bottom, right);
  vLineSetVoronoi2(line_set, result);
  return result;
}


long vLineSetVoronoi2(vLineSet * line_set, color_image * the_image)
{
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long number = line_set->NumberOfPolygons();
  if (number == 0) return 0;
  vArray(v3dMatrix<float> *) dts = vnew(v3dMatrix<float> *, number);
  long i;
  vector<uchar> colors1(1), colors3(3);
  colors1[0] = 255;
  colors3[0] = colors3[1] = colors3[2] = 0;
  // Compute the distance transform for each polygon.
  for (i = 0; i < number; i++)
  {
    vPolygon * polygon = line_set->GetPolygon(i);
    grayscale_image * polygon_image = new grayscale_image(rows, cols);
    vWriteValue(polygon_image, (uchar) 0);
    vDrawOpenPolygon(polygon_image, polygon, &colors1);
    dts[i] = vCVDistanceTransform(polygon_image);
    vdelete(polygon_image);
  }

  // Pick random colors for each region.
  const long high_color = 255;
  const long low_color = 125;
  vArray2(uchar) region_colors;
  vHyperArray2(&region_colors, number, 3);
  for (i = 0; i < number; i++)
  {
    long j;
    for (j = 0; j < 3; j++)
    {
      region_colors[i][j] = (uchar) vRandomLong2(low_color, high_color);
    }
  }

  // Now, for each pixel in the image, look up its value in all distance
  // transforms to figure out which polygon is the closest to the pixel.
  long row, col;
  vArray2(uchar) R = the_image->GetR2();
  vArray2(uchar) G = the_image->GetG2();
  vArray2(uchar) B = the_image->GetB2();

  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      float min_distance = (float) (rows*rows + cols*cols);
      long min_index = -1;
      for (i = 0; i < number; i++)
      {
        float distance = dts[i]->Matrix2(0)[row][col];
        if (distance < min_distance)
        {
          min_distance = distance;
          min_index = i;
        }
      }

      R[row][col] = region_colors[min_index][0];
      G[row][col] = region_colors[min_index][1];
      B[row][col] = region_colors[min_index][2];
    }
  }

  vDelete2(region_colors, number);
  for (i = 0; i < number; i++)
  {
    vPolygon * polygon = line_set->GetPolygon(i);
    // Draw each polygon.
    vDrawOpenPolygon(the_image, polygon, &colors3);
    vdelete(dts[i]);
  }
  vdelete2(dts);

  return 1;
}


// This function is useful for getting a frame exported by Poser (using,
// for example, save1_main.py) and retaining a smaller portion of the frame,
// centered on the hand. We assume that Poser has exported a realistic
// rendering, with green (0, 255, 0) in the background.
color_image * vPoserExtractHand(color_image * the_image,
                           long result_size, long bbox_size)
{
  // First delete the top 20 rows, that have the "LHand Camera" letters
  // on them.
  long rows1 = the_image->Rows();
  long cols1 = the_image->Cols();
  color_image * cropped = (color_image *) vCopyRectangle(the_image, 0, cols1-1, 20, rows1-1);

  // Set background to black (replace (0, 255, 0) with (0, 0, 0)).
  long rows2 = cropped->Rows();
  long cols2 = cropped->Cols();
  vArray3(uchar) croppedv = cropped->Matrix3();
  long row, col;
  for (row = 0; row < rows2; row++)
  {
    for (col = 0; col < cols2; col++)
    {
      uchar red = croppedv[0][row][col];
      uchar green = croppedv[1][row][col];
      uchar blue = croppedv[2][row][col];
      if ((red == 0) && (green == 255) && (blue == 0))
      {
        croppedv[1][row][col] = 0;
      }
    }
  }

  // find the bounding box
  const long side_limit = bbox_size;
  const long result_rows = result_size;
  const long result_cols = result_size;
  vBoundingBox box = vBinaryBoundingBox(cropped);
  if (box.MaxSide() > side_limit)
  {
    vPrint("box.MaxSide() > side_limit: %li > %li\n", 
           box.MaxSide(), side_limit);
    vdelete(cropped);
    return 0;
  }

  long target_top = vRound((result_rows - box.Height()) / 2.0);
  long target_left = vRound((result_cols - box.Width()) / 2.0);
  color_image * result = new color_image(result_rows, result_cols);
  vWriteValue(result, (uchar) 0);
  vCopyRectangle2(cropped, result, box.top, box.left, 
                   box.Height(), box.Width(), target_top, target_left);
  vdelete(cropped);
  return result;
}


