#include "stdafx.h"

#include <algorithm>
#include "interpreter2.h"
#include "contour.h"
#include "basics/drawing.h"
#include "basics/drawing_temp.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "gesture_data.h"
#include "hausdorff_matching.h"
#include "opencv_utils.h"
#include "opencv_utils2.h"
#include "quaternion.h"
#include "fingers.h"
#include "angle_sampler.h"
#include "enclosing_circle.h"
#include "quaternion.h"
#include "trajectory.h"
#include "chamfer_matching.h"
#include "learning/boost_map.h"

#include "basics/definitions.h"

using namespace std;

vInterpreter2::vInterpreter2(class_interpreter* in_interpreter) :
vInterpreter1 (in_interpreter)
{
}


vInterpreter2::~vInterpreter2()
{
  CleanUp();
}


void vInterpreter2::Initialize(vGestureInterfaceGlobals * in_globals)
{
  vInterpreter1::Initialize(in_globals);
  training_embeddings = 0;
  chamfer_index_im = 0;
  reference_images = 0;
  im_reference_images = 0;
  test_embedding = 0;
  reference_index = 0;
  embedding_dimensions = 200;
  step0_views = 20000;
  step1_views = 1000;
  reference_start = 0;
  camera_input = 0;
  ten_matches_index = 0;
  canny_threshold = 70;
  database_input = 0;
  database_edge_input = 2;
  segmentation_method = 0;
  online_segmentation = 0;
  online_ranking = 0;
  skin_threshold = 50;

  dolvs_start = -1;
  dolvs_period = -1;
  grid_fraction = 0.01;

  trajectory_cutoff = 500;
  scores_to_hold = 10;
  scores_to_use = 5;
  previous_score_frame = -1;
  top_to_print = 5;

  moments_file = 0;
  training_moments = 0;
  moment_index = 0;
  pca = 0;
  sequence_match_stats = 0;
  training_edge_histograms = 0;
  finger_info = 0;
  combined_scores = 0;
  edge_statistics = 0;
  hand_tracker = 0;
  model_straight_links = 0;
  line_segments = 0;
  all_model_lines = 0;
//  dolvs_pdf = vMatrix<long>(1,1);
  existential_DOLVS = 0;
  line_match_stats = 0;
  line_match_frame_stats = 0;
  point_sequence = 0;
  palm_cutoffs = 0;
  extended_cutoffs = 0;
  line_grid = 0;

  all_im_distances = 0;
  all_mi_distances = 0;
  all_cd_distances = 0;

  im_distances_file = vJoinPaths5(sequences_directory, "glove_data", 
                                  "training", "gw2003", 
                                  "training7.im_distances");
  mi_distances_file = vJoinPaths5(sequences_directory, "glove_data", 
                                  "training", "gw2003", 
                                  "training7.mi_distances");
  cd_distances_file = vJoinPaths5(sequences_directory, "glove_data", 
                                  "training", "gw2003", 
                                  "training7.cd_distances");
  cd_pair_file = vJoinPaths5(sequences_directory, "glove_data", 
                              "training", "gw2003", 
                              "training7.cd_pair");
  cd_pairs = 0;
  cd_pair_distances = 0;
  database = 0;
  previous_scores = 0;
  scores_holder = new vScoresHolder(scores_to_hold);
  trajectory_scores = 0;

  right_hand_frame = 0;
  original_video_frame = 0;
  right_hand_image = 0;
  original_hand_image = 0;
  hand_mask = 0;
  hand_edge_image = 0;
  hand_orientation_image = 0;
  hand_edge_scaled = 0;
  hand_orientation_scaled = 0;
  distance_transforms = vZero(v3dMatrix<float> *);
  seq_stats_file = 0;
}


void vInterpreter2::CleanUp()
{
  if (IsCameraInitialized() != 0)
  {
    camera.Uninitialize();
  }

  vdelete2(moments_file);
  vdelete(training_moments);
  vdelete(pca);
  vdelete(sequence_match_stats);
  vdelete(training_edge_histograms);
  vdelete(finger_info);
  vdelete(combined_scores);
  vdelete(edge_statistics);
  vdelete(hand_tracker);

  vdelete(training_embeddings);
  vdelete(chamfer_index_im);
  DeleteDistanceTransforms();
  vdelete(reference_images);
  vdelete(im_reference_images);
  vdelete(test_embedding);
  vdelete(model_straight_links);
  vdelete(line_segments);
  vdelete(all_model_lines);
  vdelete(existential_DOLVS);
  vdelete(line_match_stats);
  vdelete(line_match_frame_stats);
  vdelete(point_sequence);
  vdelete(palm_cutoffs);
  vdelete(extended_cutoffs);
  vdelete(line_grid);

  vdelete(all_mi_distances);
  vdelete(all_im_distances);
  vdelete(all_cd_distances);

  vdelete2(im_distances_file);
  vdelete2(mi_distances_file);
  vdelete2(cd_distances_file);
  vdelete2(cd_pair_file);

  vdelete(cd_pairs);
  vdelete(cd_pair_distances);
  vdelete(database);

  vdelete(previous_scores);
  vdelete(scores_holder);
  vdelete(trajectory_scores);

  vdelete(right_hand_frame);
  vdelete(original_video_frame);
  vdelete(right_hand_image);
  vdelete(original_hand_image);
  vdelete(hand_mask);
  vdelete(hand_edge_image);
  vdelete(hand_orientation_image);
  vdelete(hand_edge_scaled);
  vdelete(hand_orientation_scaled);
  vdelete2(seq_stats_file);
}


long vInterpreter2::ImageMax0()
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  double max = function_image_maximum(matrix);
  vPrint("\nmax = %lf\n", max);
  vdelete(matrix);
  return 1;
}


long vInterpreter2::ImageMin0()
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  double min = function_image_minimum(matrix);
  vPrint("\nmin = %lf\n", min);
  vdelete(matrix);
  return 1;
}


long vInterpreter2::BandMin0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return BandMin(band);
}


long vInterpreter2::BandMin(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  double min = function_image_minimum(matrix);
  vPrint("\nmin = %lf\n", min);
  vdelete(matrix);
  vdelete(band_matrix);
  return 1;
}



long vInterpreter2::BandMax0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return BandMax(band);
}


long vInterpreter2::BandMax(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  double max = function_image_maximum(matrix);
  vPrint("\nmax = %lf\n", max);
  vdelete(matrix);
  vdelete(band_matrix);
  return 1;
}

  
long vInterpreter2::CopyBand0()
{
  if (input == 0) return 0;
  long band = 0;
  vPrint("\nEnter band:\n");
  vScan("%li", &band);
  return CopyBand(band);
}



long vInterpreter2::CopyBand(long band)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * band_matrix = vCopyBand2(matrix, band);
  if (band_matrix == 0)
  {
    vPrint("Band not found\n");
    return 0;
  }
  ShowImage(band_matrix);
  vdelete(matrix);
  return 1;
}

  
long vInterpreter2::CountValue0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountValue(value);
}


long vInterpreter2::CountValue(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountValue(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::CountLequal0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountLequal(value);
}


long vInterpreter2::CountLequal(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountLequal(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::CountGequal0()
{
  if (input == 0) return 0;
  double value = 0;
  vPrint("\nEnter value:\n");
  vScan("%lf", &value);
  return CountGequal(value);
}


long vInterpreter2::CountGequal(double value)
{
  if (input == 0) return 0;
  v3dMatrix<double> * matrix = new v3dMatrix<double>(input->current_image);
  long number = vCountGequal(matrix, value);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::OuterContour()
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * gray = current->ToGray();
  color_image * color = current->ToRGBCopy();
  vContour contour;
  contour.OuterContour(gray);
  contour.SegmentImage3(color, 0, contour.NumberOfPoints() - 1, 255, 255, 255);
  ShowImage(color);

  if (current != gray) vdelete(gray);
  return 1;
}


long vInterpreter2::SmoothOuterContour0()
{
  if (input == 0) return 0;
  vPrint("Enter neighborhood size for smoothing: ");
  long neighborhood = 0;
  vScan("%li", &neighborhood);
  return SmoothOuterContour(neighborhood);
}


long vInterpreter2::SmoothOuterContour(long neighborhood)
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * gray = current->ToGray();
  vContour contour;
  contour.OuterContour(gray);
  grayscale_image * contour_image = (grayscale_image *) vCopyImage(gray);
  contour.ContourImage2(contour_image, 255);
  ShowImage(contour_image);

  // Smooth contour and display it.
  contour.Smooth1(neighborhood);
  contour_image = (grayscale_image *) vCopyImage(gray);
  contour.ContourImage2(contour_image, 255);
  ShowImage(contour_image);
  
  if (current != gray) vdelete(gray);
  return 1;
}


long vInterpreter2::ContourCuts0()
{
  if (input == 0) return 0;
  vPrint("Enter neighborhood size and threshold for cut detection: ");
  long neighborhood = 0;
  double threshold = 0.0;
  vScan("%li %lf", &neighborhood, &threshold);
  return ContourCuts(neighborhood, threshold);
}


long vInterpreter2::ContourCuts(long neighborhood, double threshold)
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * contour_image = current->ToGrayCopy();
  color_image * the_image1 = current->ToRGBCopy();

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->SegmentImage3(the_image1, 0, contour->NumberOfPoints() - 1,
                         255, 255, 255);
  ShowImage(the_image1);
  
  color_image * the_image2 = the_image1->ToRGBCopy();
  vector<long> indices, directions;
  vector<long> lengths;
  contour->CutPoints5(&indices, &lengths, &directions, neighborhood, threshold);
  long number_of_points = lengths.size();
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();
  long trash = indices.size();
  vector<uchar> cross_values(3);
  cross_values[0] = 0;
  cross_values[1] = 0;
  cross_values[2] = 255;

  long i;
  for (i = 0; i < number_of_points; i++)
  {
//    double value = maxima[i];
    long length = lengths[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) (*ys)[raw_index];
    long col = (long) (*xs)[raw_index];
    vPrint("%li, (%li, %li), %li\n", i, row, col, length);
    vDrawCrossv(the_image2, row, col, 1, 7, &cross_values);
  }
  ShowImage(the_image2);
  vdelete(contour_image);
    
  return 1;
}


long vInterpreter2::ContourRegions0()
{
  if (input == 0) return 0;
  vPrint("Enter neighborhood size and threshold for cut detection: ");
  long neighborhood = 0;
  double threshold = 0.0;
  vScan("%li %lf", &neighborhood, &threshold);
  return ContourRegions(neighborhood, threshold);
}


long vInterpreter2::ContourRegions(long neighborhood, double threshold)
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * contour_image = current->ToGray();
  if (contour_image == current)
  {
    contour_image = (grayscale_image *) vCopyImage(contour_image);
  }

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->ContourImage2(contour_image, 255);
  ShowImage(contour_image);
  
  grayscale_image * output_image = (grayscale_image *) vCopyImage(contour_image);
  vector<long> indices, directions;
  vector<long> lengths;
  contour->RegionBoundaries5(&indices, &lengths, &directions, neighborhood, threshold);
  long number_of_points = lengths.size();
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();
  long trash = indices.size();

  long i;
  for (i = 0; i < number_of_points; i++)
  {
//    double value = maxima[i];
    long length = lengths[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) (*ys)[raw_index];
    long col = (long) (*xs)[raw_index];
    vPrint("%li, (%li, %li), %li\n", i, row, col, length);
    vDrawCross(output_image, row, col, 0, 3, (uchar) 255);
  }
  ShowImage(output_image);

  return 1;
}


long vInterpreter2::ContourMaxima0()
{
  if (input == 0) return 0;
  vPrint("Enter neighborhood size and threshold for maxima detection: ");
  long neighborhood = 0;
  double threshold = 0.0;
  vScan("%li %lf", &neighborhood, &threshold);
  return ContourMaxima(neighborhood, threshold);
}


long vInterpreter2::ContourMaxima(long neighborhood, double threshold)
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * contour_image = current->ToGrayCopy();
  color_image * the_image1 = current->ToRGBCopy();

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;

  contour->SegmentImage3(the_image1, 0, contour->NumberOfPoints() - 1,
                         255, 255, 255);
  ShowImage(the_image1);
  color_image * the_image2 = the_image1->ToRGBCopy();

  vector<long> indices, directions;
  vector<double> maxima;
  contour->Maxima5(&indices, &maxima, &directions, neighborhood, threshold);
  long number_of_points = directions.size();
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();

  long i;
  long counter = 1;
  vPrint("\n");
  vector<uchar> cross_values(3);
  cross_values[0] = 0;
  cross_values[1] = 100;
  cross_values[2] = 255;
  for (i = 0; i < number_of_points; i++)
  {
    double value = maxima[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) (*ys)[raw_index];
    long col = (long) (*xs)[raw_index];
    long direction = directions[i];
    if (direction == -1)
    {
      vPrint("%li, (%li, %li), %lf\n", counter, row, col, value);
      vDrawCrossv(the_image2, row, col, 1, 7, &cross_values);

      vPrint("\n");
      counter++;
    }
  }
  ShowImage(the_image2);
  vdelete(contour_image);

  return 1;
}


// Collects statistics on the current input (using 
// vContour and its ElongationStatistics method)
// and saves the results to a file whose name
// derives from the input name.
long vInterpreter2::ElongationStatistics0()
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * gray_copy = current->ToGrayCopy();
  grayscale_image * contour_image = (grayscale_image *) vCopyImage(gray_copy);

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->ContourImage2(contour_image, 255);
  ShowImage(contour_image);
  
  grayscale_image * output_image = (grayscale_image *) vCopyImage(contour_image);
  vector<long> indices, directions;
  vector<double> maxima;
  contour->Maxima5(&indices, &maxima, &directions, 10, 0.3);
  long number_of_points = directions.size();
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();

  long i;
  long counter = 1;
  vPrint("\n");
  for (i = 0; i < number_of_points; i++)
  {
    double value = maxima[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) (*ys)[raw_index];
    long col = (long) (*xs)[raw_index];
    long direction = directions[i];
    if (direction == 1)
    {
      vPrint("%li, (%li, %li), %lf\n", counter, row, col, value);
      vDrawCross(output_image, row, col, 0, 3, (uchar) 255);

//      vElongation result = contour->MeasureElongation(index, 300, 150);
      vElongation result = contour->ElongationStatistics1(index);
      long j;
      vArray(double) extra = result.extra;
      for (j = 0; j < 10; j++)
      {
        vPrint("%lf ", extra[j]);
      }
      long start_index = 0;
      long end_index = 0;
      result.MaxSegmentIndices(&start_index, &end_index);
      grayscale_image * temp_image = (grayscale_image *) vCopyImage(gray_copy);
      contour->SegmentImage2(temp_image, start_index, end_index);
      ShowImage(temp_image);
      vPrint("\n");
      counter++;
      result.Print();
      vElongationBins bins(25, 25, 2, 5);
      bins.ProcessFingerElongation(&result);
    }
  }
  ShowImage(output_image);
  vdelete(gray_copy);
  return 1;
}


long vInterpreter2::FindFingers0()
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * gray_copy = current->ToGrayCopy();
  grayscale_image * contour_image = (grayscale_image *) vCopyImage(gray_copy);

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->ContourImage2(contour_image, 255);
  ShowImage(contour_image);
  
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();

  long i;
  vPrint("\n");

  vector<uchar> cross_values(3);
  cross_values[0] = 255;
  cross_values[1] = 255;
  cross_values[2] = 100;
  vector<uchar> cross_values2(3);
  cross_values2[0] = 0;
  cross_values2[1] = 0;
  cross_values2[2] = 0;
  color_image * temp_image = current->ToRGBCopy();
  vector<vElongation> elongations;
  contour->FindFingers1(&elongations);
  long number_of_elongations = elongations.size();
  for (i = 0; i < number_of_elongations; i++)
  {
    vElongation result = elongations[i];
	  double elongation = result.MaxElongation();
    long tip_row, tip_col, tip_index;
    tip_row = tip_col = tip_index = 0;
    result.StartingPoint(&tip_row, &tip_col, &tip_index);

    vPrint("\n%li, (%li, %li)\n", i, tip_row, tip_col);
    vArray(double) extra = result.extra;
    long j;
    for (j = 0; j < 10; j++)
    {
      vPrint("%lf ", extra[j]);
    }
    long start_index = 0;
    long end_index = 0;
    long start_row, start_col, end_row, end_col;
    start_row = start_col = end_row = end_col = 0;
    result.MaxSegment(&start_row, &start_col, &end_row, &end_col);
    result.MaxSegmentIndices(&start_index, &end_index);
    contour->SegmentImage3(temp_image, start_index, end_index, 0, 250, 150);
    vDrawCrossv(temp_image, tip_row, tip_col, 1, 4, &cross_values);
    vDrawCrossv(temp_image, start_row, start_col, 1, 4, &cross_values);
    vDrawCrossv(temp_image, end_row, end_col, 1, 4, &cross_values);
    vDrawCrossv(temp_image, (start_row + end_row) / 2, (start_col + end_col) / 2, 1, 4, &cross_values2);
    vPrint("\n");
  }
  
  ShowImage(temp_image);
  vdelete(gray_copy);
  return 1;
}


long vInterpreter2::FindFingers1(double elongation_threshold)
{
  if (input == 0) return 0;
  vector<vElongation> elongations;
  return FindFingers2(elongation_threshold, &elongations);
}


long vInterpreter2::FindFingers2(double elongation_threshold, 
                                  vector<vElongation> * elongations)
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * gray_copy = current->ToGrayCopy();
  grayscale_image * contour_image = (grayscale_image *) vCopyImage(gray_copy);

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->ContourImage2(contour_image, 255);
  contour->Smooth1(3);
  ShowImage(contour_image);
  
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();

  long i;
  vPrint("\n");

  vector<uchar> cross_values(3);
  cross_values[0] = 255;
  cross_values[1] = 150;
  cross_values[2] = 0;
  color_image * temp_image = current->ToRGBCopy();
  contour->FindFingers2(elongations, elongation_threshold);
  long number_of_elongations = elongations->size();
  for (i = 0; i < number_of_elongations; i++)
  {
    vElongation result = (*elongations)[i];
	  double elongation = result.MaxElongation();
    long tip_row, tip_col, tip_index;
    tip_row = tip_col = tip_index = 0;
    result.StartingPoint(&tip_row, &tip_col, &tip_index);

    vPrint("\n%li, (%li, %li)\n", i, tip_row, tip_col);
    vArray(double) extra = result.extra;
    long j;
    for (j = 0; j < 10; j++)
    {
      vPrint("%lf ", extra[j]);
    }
    long start_index = 0;
    long end_index = 0;
    result.MaxSegmentIndices(&start_index, &end_index);
    contour->SegmentImage3(temp_image, start_index, end_index, 0, 180, 180);
    vDrawCrossv(temp_image, tip_row, tip_col, 1, 7, &cross_values);
    vPrint("\n");
  }
  
  ShowImage(temp_image);
  vdelete(gray_copy);
  return 1;
}

  
long vInterpreter2::GrowSkeleton()
{
  if (input == 0) return 0;
  // Get and display unsmoothed contour
  general_image * current = input->current_image;
  grayscale_image * gray_copy = current->ToGrayCopy();
  grayscale_image * contour_image = (grayscale_image *) vCopyImage(gray_copy);

  vContour raw_contour;
  raw_contour.OuterContour(contour_image);

  // Smooth contour and display it.
  vContour * contour = &raw_contour;
  contour->ContourImage2(contour_image, 255);
  ShowImage(contour_image);
  
  grayscale_image * output_image = (grayscale_image *) vCopyImage(contour_image);
  vector<long> indices, directions;
  vector<double> maxima;
  contour->Maxima5(&indices, &maxima, &directions, 10, 0.3);
  long number_of_points = directions.size();
  vector<double> * xs = contour->GetXs();
  vector<double> * ys = contour->GetYs();

  long i;
  long counter = 1;
  vPrint("\n");
  for (i = 0; i < number_of_points; i++)
  {
    double value = maxima[i];
    long index = indices[i];
    long raw_index = index;
    long row = (long) (*ys)[raw_index];
    long col = (long) (*xs)[raw_index];
    long direction = directions[i];
    if (direction == 1)
    {
      vPrint("%li, (%li, %li), %lf\n", counter, row, col, value);
      vDrawCross(output_image, row, col, 0, 3, (uchar) 255);

//      vElongation result = contour->MeasureElongation(index, 300, 150);
      vMatrix<double> result = contour->GrowSkeleton(index, 300, gray_copy);
      long j;

      long start_index = 0;
      long end_index = 0;
      grayscale_image * temp_image = (grayscale_image *) vCopyImage(gray_copy);
//      contour->SegmentImage2(temp_image, start_index, end_index);
    
      // Draw skeleton
      long skeleton_rows = result.Rows();
      vArray2(double) skeleton = result.Matrix2();
      vArray2(uchar) gray2 = temp_image->Matrix2(0);
      for (j = 0; j < skeleton_rows; j++)
      {
        long row = (long) skeleton[j][4];
        long col = (long) skeleton[j][5];
        if (row == 0) break;
        gray2[row][col] = 255;
      }

      ShowImage(temp_image);
      vPrint("\n");
      counter++;
    }
  }
  ShowImage(output_image);
  vdelete(gray_copy);
  return 1;
}


long vInterpreter2::SaveFingerInfo0()
{
  if (input == 0) 
  {
    vPrint("no input is loaded\n");
    return 0;
  }

  long start_frame = 0;
  long end_frame = 0;
  vPrint("Input start frame, end_frame: ");
  vScan("%li %li", &start_frame, &end_frame);
  long result = SaveFingerInfo(start_frame, end_frame);
  return result;
}


long vInterpreter2::SaveFingerInfo(long start_frame, long end_frame)
{
  if (input == 0) 
  {
    vPrint("no input is loaded\n");
    return 0;
  }

  const char * path_name = input->path_name;
  long result = vSaveFingerInfo(path_name, 1, start_frame, end_frame, 70);
  if (result <= 0)
  {
    vPrint("Failed to save finger info for %s\n", path_name);
  }
  return result;
}


long vInterpreter2::LoadFingerInfo()
{
  if (input == 0) return 0;
  vdelete(finger_info);
  const char * path = input->path_name;
  char * fingers_file = vReplaceEnding2(path, ".fingers");
  finger_info = new vSequenceFingerInfo(fingers_file);
  long result = 1;
  if (finger_info->is_valid <= 0)
  {
    vPrint("Failed to load file %s\n", fingers_file);
    vdelete(finger_info);
    finger_info = 0;
    result = 0;
  }
  vdelete2(fingers_file);
  return result;
}
  

long vInterpreter2::FingerScores()
{
  if (input == 0) return 0;
  if (finger_info == 0) return 0;
  grayscale_image * gray = input->current_image->ToGray();
  grayscale_image * binary1 = (grayscale_image *) vScaleBoundingBox6(gray, 192, 128, 128, 256, 256);
  if (gray != input->current_image) vdelete(gray);
  ShowImage(binary1);
  vector<vElongation> elongations;
  FindFingers2(1, &elongations);
  vMatrix<double> distances = finger_info->Distances(&elongations);
  vdelete(match_scores);
  match_scores = new vMatchScores(&distances);
  UpdateMatchScores(input->current_frame);  
  return 1;
}


long vInterpreter2::FingerScore()
{
  if (input == 0) return 0;
  if (finger_info == 0) return 0;
  long frame = 0;
  long view = 0;
  vPrint("Enter frame, view: ");
  vScan("%li %li", &frame, &view);

  grayscale_image * gray = input->current_image->ToGray();
  grayscale_image * binary1 = (grayscale_image *) vScaleBoundingBox6(gray, 192, 128, 128, 256, 256);
  if (gray != input->current_image) vdelete(gray);
  ShowImage(binary1);
  vector<vElongation> elongations;
  FindFingers2(1, &elongations);
  long score_frame = frame * 4 + view;
  double distance = finger_info->Distance(score_frame, &elongations);
  vPrint("distance = %lf", distance);
  return 1;
}


long vInterpreter2::FillRegion0()
{
  if (input == 0) return 0;
  vPrint("Input row, col, value for region: ");
  long row, col, value;
  row = col = value = 0;
  vScan("%li %li %li", &row, &col, &value);
  return FillRegion(row, col, (char) value);
}


long vInterpreter2::FillRegion(long row, long col, uchar value)
{
  if (input == 0) return 0;
  general_image * current = input->current_image;
  grayscale_image * temp_image = current->ToGray();
  grayscale_image * the_image;
  if (temp_image == current)
  {
    the_image = (grayscale_image *) vCopyImage(temp_image);
  }
  else
  {
    the_image = temp_image;
  }
  long result = vFillRegion(the_image, row, col, value);
  ShowImage(the_image);
  return result;
}


long vInterpreter2::RandomFrame()
{
  if (input == 0) return 0;
  long number_of_frames = input->number_of_frames;
  if (number_of_frames <= 0) return 0;
  long random_frame = vRandomLong2(0, number_of_frames - 1);
  function_print("\nrandom frame = %li\n", random_frame);
  return JumpToFrame(random_frame);
}


long vInterpreter2::ToGray()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  ShowImage(gray);
  return 1;
}


long vInterpreter2::ToGrayBound()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayBoundCopy();
  ShowImage(gray);
  return 1;
}


long vInterpreter2::DefaultMomentsFile()
{
  vdelete2(moments_file);
  char * new_moments_file = vJoinPaths4(sequences_directory, "glove_data", 
                                         "training", "training7.moments");
  long result = SetMomentsFile(new_moments_file);
  vdelete2(new_moments_file);
  return result;
}


long vInterpreter2::InputMomentsFile()
{
  char input[1000];
  vPrint("Input new moments file: ");
  vScan("%s", input);
  char * new_moments_file = vJoinPaths4(sequences_directory, "glove_data", 
                                         "training", input);

  long result = SetMomentsFile(new_moments_file);
  vdelete2(new_moments_file);
  return result;
}


long vInterpreter2::SetMomentsFile(const char * new_file)
{
  vdelete(moments_file);
  moments_file = vCopyString(new_file);
  long file_size = vFilesize(new_file);
  if (file_size <= 0) 
  {
    vPrint("moments file %s not found\n", new_file);
    return 0;
  }
  const long number_of_moments = 14;

  long frames_size = file_size / (number_of_moments * sizeof(double));
  vdelete(training_moments);
  training_moments = new vMatrix<double>(frames_size, number_of_moments);
  long result = training_moments->ReadBandwise(moments_file, 0);
  if (result <= 0) 
  {
    vPrint("Error in reading moments from %s\n", moments_file);
    return result;
  }
  return 1;
}


long vInterpreter2::MomentScores()
{
  if (input == 0) return 0;
  if (training_moments == 0)
  {
    vPrint("No training moments have been loaded.\n");
    return 0;
  }

  long number = 0;
  long column = moment_index;
  double * moments = vImageMoments(input->current_image, &number);
  double moment = 0;
  long result = 0;
  if ((column >= 0) && (column < number))
  {
    moment = moments[column];
    vdelete(match_scores);
    match_scores = new vMatchScores(training_moments, column, moment);
    UpdateMatchScores(input->current_frame);
    result = 1;
  }
  vdelete2(moments);
  return result;
}


long vInterpreter2::SetMomentIndex0()
{
  long index = 0;
  vPrint("Input new moment index: ");
  vScan("%li", &index);
  return SetMomentIndex(index);
}


long vInterpreter2::SetMomentIndex(long index)
{
  if (moment_index < 0) return 0;
  moment_index = index;
  return 1;
}


long vInterpreter2::InputPCAFile()
{
  char pca_file[1000];
  vPrint("Enter pca file: ");
  vScan("%s", pca_file);
  char * new_moments_file = vJoinPaths4(sequences_directory, "glove_data", 
                                         "training", pca_file);
  long result = SetPCAFile(new_moments_file);
  vdelete2(new_moments_file);
  return result;
}


long vInterpreter2::SetPCAFile(const char * new_file)
{
  vPCA * new_pca = new vPCA(new_file);
  if (new_pca->IsValid() <= 0)
  {
    vPrint("Error in loading %s\n", new_file);
    vdelete(new_pca);
    return 0;
  }
  vdelete(pca);
  pca = new_pca;
  return 1;
}


long vInterpreter2::InputPCAUsedDimensions()
{
  if (pca == 0)
  {
    vPrint("Error, no pca object exists right now.\n");
    return 0;
  }
  long used_dimensions = 0;
  vPrint("Enter new number of PCA dimensions to use: ");
  vScan("%li", &used_dimensions);
  return SetPCAUsedDimensions(used_dimensions);
}


long vInterpreter2::SetPCAUsedDimensions(long new_dimensions)
{
  if (pca == 0)
  {
    vPrint("Error, no pca object exists right now.\n");
    return 0;
  }

  long result = pca->SetUsedDimensions(new_dimensions);
  if (result <= 0)
  {
    vPrint("Illegal dimensions passed to SetUsedDimensions: %li\n", 
            new_dimensions);
  }
  return result;
}

long vInterpreter2::PCAMomentScores()
{
  if (input == 0) return 0;
  if (pca == 0) 
  {
    vPrint("No pca object is loaded.\n");
    return 0;
  }

  if (training_moments == 0)
  {
    vPrint("No training moments have been loaded.\n");
    return 0;
  }

  long number;
  double * moments = vImageMoments(input->current_image, &number);
  vMatrix<double> features(number, 1);
  memcpy(vData(features.Matrix()), moments, sizeof(double) * number);
  vMatrix<double> pca_features = pca->PCA_Conversion(features);
  
  long result = 0;
  vdelete(match_scores);
  match_scores = new vMatchScores(training_moments, &pca_features, pca);
  UpdateMatchScores(input->current_frame);
  result = 1;

  vdelete2(moments);
  return result;
}


long vInterpreter2::PCAScoreStats()
{
  if (input == 0) return 0;
  if (match_scores == 0) return 0;
  if (pca == 0) return 0;
  long input_index = current_input_index;
  long frames = input->number_of_frames;
  long frame;
  vector<long> ground_matches(frames);
  vector<long> correct_matches(frames);
  for (frame = 0; frame < frames; frame++)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    PCAMomentScores();
    vMatchScore ground_score = match_scores->GroundMatch(0);
    vMatchScore correct_score = match_scores->CorrectMatch(0);
    ground_matches[frame] = ground_score.rank;
    correct_matches[frame] = correct_score.rank;
  }
  std::sort(ground_matches.begin(), ground_matches.end(), less<long>());
  vPrint("\nGround match ranks: ");
  for (frame = 0; frame < frames; frame++)
  {
    vPrint("%li ", ground_matches[frame]);
  }
  vPrint("\n");

  std::sort(correct_matches.begin(), correct_matches.end(), less<long>());
  vPrint("\nCorrect match ranks: ");
  for (frame = 0; frame < frames; frame++)
  {
    vPrint("%li ", correct_matches[frame]);
  }
  vPrint("\n\n");
  return 1;
}


long vInterpreter2::FilterScoresUsingMoments0()
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  if (pca == 0) return 0;
  long threshold = 0;
  vPrint("Input threshold: ");
  vScan("%li", &threshold);
  return FilterScoresUsingMoments(threshold);
}


// We go through scores and filter them using another set
// of scores, obtained by calling PCAMomentScores. We discard
// scores in the original set of scores whose rank is after
// the given threshold.
long vInterpreter2::FilterScoresUsingMoments(long threshold)
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  if (pca == 0) return 0;

  vMatchScore score = match_scores->GetScore(threshold);
  if (score.is_valid <= 0) return 0;

  vMatchScores * original_match_scores = match_scores;
  match_scores = 0;
  PCAMomentScores();
  original_match_scores->Filter(match_scores, threshold);
  vdelete(match_scores);
  match_scores = original_match_scores;
  return 1;
}


long vInterpreter2::FilterScoresUsingFingers0()
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  if (finger_info == 0) return 0;
  long threshold = 0;
  vPrint("Input threshold: ");
  vScan("%li", &threshold);
  return FilterScoresUsingFingers(threshold);
}


long vInterpreter2::FilterScoresUsingLogs0()
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  long threshold, add;
  double w0, w1, w2, w3;
  threshold = add = 0;
  w0 = w1 = w2 = w3 = 0;
  vPrint("Input threshold, w0, w1, w2, w3, add: ");
  vScan("%li %lf %lf %lf %lf %li", &threshold, &w0, &w1, &w2, &w3, &add);
  return FilterScoresUsingLogs(threshold, w0, w1, w2, w3, add);
}


// We go through scores and filter them using another set
// of scores, obtained by calling PCAMomentScores. We discard
// scores in the original set of scores whose rank is after
// the given threshold.
long vInterpreter2::FilterScoresUsingLogs(long threshold, double w0, double w1, double w2, double w3,
                                           long add)
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;

  vMatchScore score = match_scores->GetScore(threshold);
  if (score.is_valid <= 0) return 0;

  vMatchScores * original_match_scores = match_scores;
  match_scores = 0;
  CombineLogs2(w0, w1, w2, w3, add);
  original_match_scores->Filter(match_scores, threshold);
  vdelete(match_scores);
  match_scores = original_match_scores;
  return 1;
}


// We go through scores and filter them using another set
// of scores, obtained by calling PCAMomentScores. We discard
// scores in the original set of scores whose rank is after
// the given threshold.
long vInterpreter2::FilterScoresUsingFingers(long threshold)
{
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  if (finger_info == 0) return 0;

  vMatchScore score = match_scores->GetScore(threshold);
  if (score.is_valid <= 0) return 0;

  vMatchScores * original_match_scores = match_scores;
  match_scores = 0;
  FingerScores();
  original_match_scores->Filter(match_scores, threshold);
  vdelete(match_scores);
  match_scores = original_match_scores;
  return 1;
}


long vInterpreter2::PixelGradient0()
{
  if (input == 0) return 0;
  long row = 0;
  long col = 0;
  vPrint("Input row and col of pixel: ");
  vScan("%li %li", &row, &col);
  return PixelGradient(row, col);
}


long vInterpreter2::PixelGradient(long row, long col)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  vArray2(uchar) values = gray->Matrix2(0);
  long rows = gray->Rows();
  long cols = gray->Cols();
  long boundary_distance = 30;
  if ((row < boundary_distance) || (col < boundary_distance) || 
      (row > rows - boundary_distance) || (col > cols - boundary_distance))
  {
    vPrint("Can't estimate orientation, image size is %li x %li\n", 
            rows, cols);
    return 0;
  }
  
  double horizontal_vector = 0;
  double vertical_vector = 0;
  vPixelGradient(gray, row, col, horizontal_vector, vertical_vector);

  vdelete(gray);
  double start_row, start_col, end_row, end_col;
  double line_length = 20;
  if ((horizontal_vector == 0) && (vertical_vector == 0))
  {
    vPrint("No direction of change was found in the particular location.\n");
    return 0;
  }

  if (vAbs(horizontal_vector) < vAbs(vertical_vector))
  {
    start_row = row - line_length;
    end_row = row + line_length;
    start_col = col - line_length * (horizontal_vector / vertical_vector);
    end_col = col + line_length * (horizontal_vector / vertical_vector);
  }
  else
  {
    start_col = col - line_length;
    end_col = col + line_length;
    start_row = row - line_length * (vertical_vector / horizontal_vector);
    end_row = row + line_length * (vertical_vector / horizontal_vector);
  }
  DrawLine((long) start_row, (long) start_col, (long) end_row, (long) end_col, 200);
  return 1;
}


// Takes the current image, finds the edges, and for each
// out of every subsampling pixels in the edge image it 
// shows the gradient direction in the original images.
// Gradients are drawn onto the edge image.
long vInterpreter2::EdgePixelGradients0()
{
  if (input == 0) return 0;
  long subsampling = 0;
  vPrint("Enter subsampling: ");
  vScan("%li", &subsampling);
  return EdgePixelGradients(subsampling);
}


long vInterpreter2::EdgePixelGradients(long subsampling)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(gray, (uchar) canny_threshold);
  ShowImage(edges);
  grayscale_image * edges_copy = (grayscale_image *) vCopyImage(edges);
  vector<long> rows_vector, cols_vector;
  vGetNonZeroPoints(edges, &rows_vector, &cols_vector);

  vArray2(uchar) values = gray->Matrix2(0);
  long rows = gray->Rows();
  long cols = gray->Cols();
  long boundary_distance = 30;

  long pixels = rows_vector.size();
  long i;
  for (i = 0; i < pixels; i = i + subsampling)
  {
    long row = rows_vector[i];
    long col = cols_vector[i];
    if ((row < boundary_distance) || (col < boundary_distance) || 
        (row > rows - boundary_distance) || (col > cols - boundary_distance))
    {
      vPrint("Can't estimate orientation, image size is %li x %li\n", 
              rows, cols);
    }

    double horizontal_vector = 0;
    double vertical_vector = 0;
    vPixelGradient2(values, row, col, horizontal_vector, vertical_vector);
    double start_row, start_col, end_row, end_col;
    double line_length = 20;
    if ((horizontal_vector == 0) && (vertical_vector == 0))
    {
      vPrint("No direction of change was found in the particular location.\n");
    }

    if (vAbs(horizontal_vector) < vAbs(vertical_vector))
    {
      start_row = row - line_length;
      end_row = row + line_length;
      start_col = col - line_length * (horizontal_vector / vertical_vector);
      end_col = col + line_length * (horizontal_vector / vertical_vector);
    }
    else
    {
      start_col = col - line_length;
      end_col = col + line_length;
      start_row = row - line_length * (vertical_vector / horizontal_vector);
      end_row = row + line_length * (vertical_vector / horizontal_vector);
    }
    vDrawLine1(edges_copy, (long) start_row, (long) start_col, (long) end_row, (long) end_col, (uchar) 200);
  }
  vdelete(gray);
  ShowImage(edges_copy);
  return 1;
}


long vInterpreter2::EdgeHistogram0a()
{
  if (input == 0) return 0;
  long bins, low, high;
  bins = low = high = 0;
  vPrint("Enter bins, low, high: ");
  vScan("%li %li %li", &bins, &low, &high);
  return EdgeHistogram3a(bins, low, high);
}


long vInterpreter2::EdgeHistogram0b()
{
  if (input == 0) return 0;
  long input_seq, edge_seq, bins;
  input_seq = edge_seq = bins = 0;
  vPrint("Enter bins, source input, edge input: ");
  vScan("%li %li %li", &bins, &input_seq, &edge_seq);
  return EdgeHistogram3b(bins, input_seq, edge_seq);
}


long vInterpreter2::EdgeHistogram3a(long bins, long low, long high)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(gray, (uchar) low, (uchar) high, 3);
  v3dMatrix<double> * blurred = vBlur(gray);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(bins, blurred_gray, edges);
  long col;
  for (col = 0; col < bins; col++)
  {
    vPrint("(%li %lf) ", col, histogram(col));
  }
  vdelete(gray);
  vdelete(blurred);
  vdelete(blurred_gray);
  vdelete(edges);
  return 1;
}


long vInterpreter2::EdgeHistogram3b(long bins, long source_seq, long edge_seq)
{
  if ((source_seq < 0) || (edge_seq < 0) ||
      (source_seq >= number_of_inputs) || 
      (edge_seq >= number_of_inputs))
  {
    vPrint("Error, bad sequence numbers, %li %li\n",
            source_seq, edge_seq);
    return 0;
  }

  grayscale_image * original = CurrentImage(source_seq)->ToGrayCopy();
  v3dMatrix<double> * blurred_original = vBlur(original);
  grayscale_image * gray_blurred = blurred_original->ToGrayCopy();
  grayscale_image * edge_image = (grayscale_image *) CurrentImage(edge_seq)->threshold(1);
  vEdgeHistogram histogram(bins, gray_blurred, edge_image);
  long col;
  for (col = 0; col < bins; col++)
  {
    vPrint("(%li %lf) ", col, histogram(col));
  }
  vdelete(original);
  vdelete(edge_image);
  vdelete(gray_blurred);
  vdelete(blurred_original);
  return 1;
}

  
// Used to compare histogram of a test image with histogram of a training
// image.
long vInterpreter2::CompareEdgeHistograms0()
{
  if (input == 0) return 0;
  long bins, test_input, training_input, training_edges;
  bins = test_input = training_input = training_edges = 0;
  vPrint("Enter bins, test_input, training_input, training_edges: ");
  vScan("%li %li %li %li", &bins, &test_input, 
         &training_input, &training_edges);
  return CompareEdgeHistograms(bins, test_input, 
                               training_input, training_edges);
}


long vInterpreter2::CompareEdgeHistograms(long bins, long test_input, 
                                           long training_input, 
                                           long training_edges)
{
  if (bins <= 1) 
  {
    vPrint("Bad bin number %li\n", bins);
    return 0;
  }
  if ((test_input < 0) || (training_input < 0) || (training_edges < 0) ||
      (test_input >= number_of_inputs) || 
      (training_input >= number_of_inputs) ||
      (training_edges >= number_of_inputs))
  {
    vPrint("Bad input sequence indices: %li %li %li\n",
            test_input, training_input, training_edges);
  }

  // Get test histogram
  grayscale_image * test = CurrentImage(test_input)->ToGrayCopy();
  grayscale_image * test_edge = vCannyDetector(test, 40, 80, 3);
  v3dMatrix<double> * blurred_test = vBlur(test);
  grayscale_image * gray_blurred_test = blurred_test->ToGray();
  vEdgeHistogram test_histogram(bins, gray_blurred_test, test_edge);
  vdelete(test);
  vdelete(test_edge);
  vdelete(blurred_test);
  vdelete(gray_blurred_test);

  // Get training histogram
  grayscale_image * training = CurrentImage(training_input)->ToGrayCopy();
  v3dMatrix<double> * blurred_training = vBlur(training);
  grayscale_image * training_gray = blurred_training->ToGray();
  grayscale_image * training_edge = 
    (grayscale_image *) CurrentImage(training_edges)->threshold(1);
  vEdgeHistogram training_histogram(bins, training_gray, training_edge);
  vdelete(training);
  vdelete(blurred_training);
  vdelete(training_gray);
  vdelete(training_edge);

  // Print histograms, and estimate and print their distances
  test_histogram.Print("Test histogram");
  training_histogram.Print("Training histogram");
  double min_distance = 
    test_histogram.MinCumulativeDistance(&training_histogram);
  double mean_distance = 
    test_histogram.MeanCumulativeDistance(&training_histogram);
  vPrint("min_distance = %lf, mean_distance = %lf\n", 
          min_distance, mean_distance);
  
  return 1;
}
  

long vInterpreter2::SaveEdgeHistograms0()
{
  if (input == 0) return 0;
  long bins, original, edges;
  bins = original = edges = 0;
  vPrint("Enter bins, original and edges: ");
  vScan("%li %li %li", &bins, &original, &edges);
  return SaveEdgeHistograms(bins, original, edges);
}


// Calls vEdgeHistogram::SaveHistograms, with the files corresponding
// to the inputs indexed by original and edges.
long vInterpreter2::SaveEdgeHistograms(long bins, long original, 
                                        long edges)
{
  // Do some error checking
  if ((bins < 1) ||
      (original < 0) ||
      (edges < 0) ||
      (original >= number_of_inputs) ||
      (edges >= number_of_inputs))
  {
    vPrint("Can't save histograms, bad arguments\n");
    return 0;
  }
  
  class_sequence_information * original_input = (class_sequence_information *) inputs[original];
  class_sequence_information * edge_input = (class_sequence_information *) inputs[edges];
  const char * original_file = original_input->path_name;
  const char * edge_file = edge_input->path_name;
  char * original_simple = vSimpleName(original_file, 
                                        platform_directory_slash);
  char * target_simple = vReplaceEnding2(original_simple, ".edge_hist");
  char * target = vJoinPaths4(sequences_directory, "glove_data", "training",
                               target_simple);
  vdelete2(original_simple);
  vdelete2(target_simple);
  long result = vEdgeHistogram::SaveHistograms(bins, original_file, 
                                                edge_file, target);
  vdelete2(target);
  if (result <= 0) 
  {
    vPrint("Failed to save histograms\n");
    return 0;
  }

  vPrint("\nHistograms were saved successfully\n\n");
  return 1;
}


long vInterpreter2::LoadEdgeHistograms()
{
  if (input == 0) return 0;
  char * histogram_file = vReplaceEnding2(input->simple_name, ".edge_hist");
  char * histogram_path = vJoinPaths4(sequences_directory, "glove_data",
                                       "training", histogram_file);
  vdelete2(histogram_file);
  vdelete(training_edge_histograms);
  training_edge_histograms = vEdgeHistogram::LoadHistograms(histogram_path);
  if (training_edge_histograms == 0)
  {
    vPrint("Failed to load histograms from %s\n", histogram_path);
  }
  vdelete2(histogram_path);
  if (training_edge_histograms == 0) return 0;
  else return 1;
}


// We take edges from the unblurred image, but we compute the
// orientations from the blurred image
long vInterpreter2::MinEdgeHistogramScores()
{
  if (input == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  grayscale_image * current = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(current, 40, 80, 3);
  v3dMatrix<double> * blurred = vBlur(current);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edges);
  vMatrix<double> distances = 
    vEdgeHistogram::MinCumulativeDistance2(training_edge_histograms,
                                            &histogram);

  vdelete(match_scores);
  match_scores = new vMatchScores(&distances);
  UpdateMatchScores(input->current_frame);

  vdelete(current);
  vdelete(edges);
  vdelete(blurred);
  vdelete(blurred_gray);
  return 1;
}


long vInterpreter2::MeanEdgeHistogramScores()
{
  if (input == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  grayscale_image * current = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(current, 40, 80, 3);
  v3dMatrix<double> * blurred = vBlur(current);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edges);
  vMatrix<double> distances = 
    vEdgeHistogram::MeanCumulativeDistance2(training_edge_histograms,
                                             &histogram);

  vdelete(match_scores);
  match_scores = new vMatchScores(&distances);
  UpdateMatchScores(input->current_frame);

  vdelete(current);
  vdelete(edges);
  vdelete(blurred);
  vdelete(blurred_gray);
  return 1;
}


long vInterpreter2::SaveEdgeHistogramScores0()
{
  if (input == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  char buffer[500];
  long start = 0;
  long end = 0;
  vPrint("Input base filename to save scores to, start frame, end frame: ");
  vScan("%s %li %li", buffer, &start, &end);
  return SaveEdgeHistogramScores(buffer, start, end);
}


// Saves histogram scores for all frames in the current input,
// using MeanCumulativeDistance.
long vInterpreter2::SaveEdgeHistogramScores(const char * base, long start, long end)
{
  if (input == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  long error_flag = 0;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  long frame;
  if (start < 0) start = 0;
  if (end < 0) end = frames - 1;
  for (frame = start; frame <= end; frame++)
  {
    SetCurrentInput(input_index);
    long success = JumpToFrame(frame);
    if (success <= 0)
    {
      vPrint("\nError in reading frame %li\n", frame);
      error_flag = 1;
      break;
    }

    success = MinEdgeHistogramScores();
    if (success <= 0)
    {
      vPrint("\nError in calculating scores for frame %li\n", frame);
      error_flag = 1;
      break;
    }

    char buffer[20];
    sprintf(buffer, ".%li.bin", frame);

    char * score_base = vJoinPaths3(sequences_directory, "glove_data", "results\\");
    char * score_file = vMergeStrings3(score_base, base, buffer);
    match_scores->SetFilename(score_file);
    success = match_scores->WriteBinary();
    vPrint("scores_file = %s\n", score_file);
    vdelete2(score_base);
    vdelete2(score_file);
    if (success <= 0)
    {
      vPrint("\nError in saving scores for frame %li\n", frame);
      error_flag = 1;
      break;
    }
  
    vPrint("Processed frame %li of %li\n", frame, frames);
  }

  SetCurrentInput(input_index);
  if (error_flag != 0) return 0;
  else return 1;
}


long vInterpreter2::FrameMatchStats0()
{
  if (match_scores == 0) return 0;
  long option = 0;
  vPrint("Input option (1 for correct/ground, 2 for other): ");
  vScan("%li", &option);
  if (option == 1)
  {
    long correct = 0;
    long ground = 0;
    vPrint("Input number of correct and number of ground matches: ");
    vScan("%li %li\n", &correct, &ground);
    return FrameMatchStats1(correct, ground);
  }
  else if (option == 2)
  {
    long correct, ground, top;
    correct = ground = top = 0;
    vPrint("Enter correct, ground, top: ");
    vScan("%li %li %li", &correct, &ground, &top);
    return FrameMatchStats2(correct, ground, top);
  }
  else
  {
    vPrint("Invalid option %li\n", option);
    return 0;
  }
}


long vInterpreter2::FrameMatchStats1(long correct, long ground)
{
  if (match_scores == 0) return 0;
  vFrameMatchStats frame_stats(match_scores, correct, ground);
  if (frame_stats.IsValid() >= 1)
  {
    frame_stats.Print();
    return 1;
  }
  else
  {
    vPrint("Invalid frame stats\n");
    return 0;
  }
}


long vInterpreter2::FrameMatchStats2(long correct, long ground, long top)
{
  if (match_scores == 0) return 0;
  vFrameMatchStats frame_stats(match_scores, correct, ground, top);
  if (frame_stats.IsValid() >= 1)
  {
    frame_stats.Print();
    return 1;
  }
  else
  {
    vPrint("Invalid frame stats\n");
    return 0;
  }
}


long vInterpreter2::SequenceMatchStats0()
{
  char buffer[100];
  long correct = 0;
  long ground = 0;
  long start_frame = 0;
  long end_frame = 0;
  long period = 0;
  vPrint("Enter score script, correct, ground params, start, end frames, period: ");
  vScan("%s %li %li %li %li %li", buffer, &correct, &ground, 
         &start_frame, &end_frame, &period);
  return SequenceMatchStats(buffer, correct, ground, start_frame, end_frame, period);
}


long vInterpreter2::SequenceMatchStats(const char * score_script,
                                        long correct, long ground,
                                        long start_frame, long end_frame, long period)
{
  if (input == 0) return 0;
  long old_ground_distance = (long) ground_truth_distance;
  ground_truth_distance = 180;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  if (end_frame == -1) end_frame = input->number_of_frames - 1;
  long frame;
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();

  for (frame = start_frame; frame <= end_frame; frame+= period)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    base_interpreter->process_command_file1(score_script);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      SetGroundTruthDistance(old_ground_distance);
      return 0;
    }
    vFrameMatchStats * match_stats = new vFrameMatchStats(match_scores, 
                                                            4128, ground, 4000);
    match_stats->SetCorrectToPrint(correct);
    match_stats->SetGroundDistance((long) ground_truth_distance);
    sequence_match_stats->AddFrameStats(match_stats, frame);
    sequence_match_stats->SetGroundDistance(old_ground_distance);
    long median = sequence_match_stats->CorrectMedian(0);
    match_scores->SetGroundTruthDistance(old_ground_distance);
    vMatchScore highest_correct = match_scores->CorrectMatch(0);
    long rank = highest_correct.rank;
    vPrint("\n\nDone with frame %li, median = %li, rank = %li\n\n\n", 
            frame, median, rank);
  }
  SetGroundTruthDistance(old_ground_distance);
  sequence_match_stats->Print();
  SaveSequenceStats(score_script);
  return 1;
}


long vInterpreter2::TrackMatchStats0()
{
  char buffer[100];
  long correct = 0;
  long ground = 0;
  long start_frame = 0;
  long end_frame = 0;
  long period = 0;
  vPrint("Enter score script, correct, ground params, start, end frames, period: ");
  vScan("%s %li %li %li %li %li", buffer, &correct, &ground, 
         &start_frame, &end_frame, &period);
  return TrackMatchStats(buffer, correct, ground, start_frame, end_frame, period);
}


long vInterpreter2::TrackMatchStats(const char * score_script,
                                     long correct, long ground,
                                     long start_frame, long end_frame, long period)
{
  if (input == 0) return 0;
  long old_ground_distance = (long) ground_truth_distance;
  ground_truth_distance = 180;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  if (end_frame == -1) end_frame = input->number_of_frames - 1;
  long frame;
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();
  long counter = 0;
  EraseScores();

  for (frame = start_frame; frame <= end_frame; frame+= period)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    base_interpreter->process_command_file1(score_script);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      SetGroundTruthDistance(old_ground_distance);
      return 0;
    }

    SetCurrentInput(input_index);
    HoldScores();
    counter++;
    if (counter >= scores_to_use)
    {
      TrajectoryScores();
    }
    match_scores->SetGroundTruthDistance(old_ground_distance);
    vMatchScore score = match_scores->CorrectMatch(0);
    vPrint("\n\ncorrect match rank = %li\n", score.rank);
    match_scores->SetGroundTruthDistance(ground_truth_distance);

    vFrameMatchStats * match_stats = new vFrameMatchStats(match_scores, 
                                                            4128, ground, 5);
    match_stats->SetCorrectToPrint(correct);
    match_stats->SetGroundDistance((long) ground_truth_distance);
    sequence_match_stats->AddFrameStats(match_stats, frame);
    sequence_match_stats->SetGroundDistance(old_ground_distance);
    long median = sequence_match_stats->CorrectMedian(0);
    vPrint("Done with frame %li, median = %li\n\n\n", frame, median);
  }
  SetGroundTruthDistance(old_ground_distance);
  sequence_match_stats->Print();
  SaveSequenceStats(score_script);
  return 1;
}


long vInterpreter2::SequenceCorrectMean0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceCorrectMean(rank);
}

                                        
long vInterpreter2::SequenceCorrectMean(long rank)
{
  if (sequence_match_stats == 0) return 0;
  double mean = sequence_match_stats->CorrectMean(rank);
  vPrint("Correct mean of rank %li is %lf\n", rank, mean);
  return 1;
}


long vInterpreter2::SequenceCorrectMedian0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceCorrectMedian(rank);
}

                                        
long vInterpreter2::SequenceCorrectMedian(long rank)
{
  if (sequence_match_stats == 0) return 0;
  double median = sequence_match_stats->CorrectMedian(rank);
  vPrint("Correct median of rank %li is %lf\n", rank, median);
  return 1;
}


long vInterpreter2::SequenceGroundMean0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceGroundMean(rank);
}

                                        
long vInterpreter2::SequenceGroundMean(long rank)
{
  if (sequence_match_stats == 0) return 0;
  double mean = sequence_match_stats->GroundMean(rank);
  vPrint("Ground mean of rank %li is %lf\n", rank, mean);
  return 1;
}


long vInterpreter2::SequenceGroundMedian0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceGroundMedian(rank);
}

                                        
long vInterpreter2::SequenceGroundMedian(long rank)
{
  if (sequence_match_stats == 0) return 0;
  double Median = sequence_match_stats->GroundMedian(rank);
  vPrint("Ground median of rank %li is %lf\n", rank, Median);
  return 1;
}


long vInterpreter2::SequenceCorrectUnder0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceCorrectUnder(rank);
}


long vInterpreter2::SequenceCorrectUnder0b()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  long correct_index = 0;
  vPrint("Input correct_index, rank: ");
  vScan("%li %li", &correct_index, &rank);
  return SequenceCorrectUnder2(correct_index, rank);
}


long vInterpreter2::SequenceCorrectUnder(long rank)
{ 
  if (sequence_match_stats == 0) return 0;
  long correct_matches = sequence_match_stats->CorrectMatchesUnder(0, rank);
  double pcorrect_matches = sequence_match_stats->PcorrectMatchesUnder(0, rank);
  vPrint("(%.1lf) %li correct matches under rank %li\n", 
           pcorrect_matches * 100, correct_matches, rank);
  return 1;
}


long vInterpreter2::SequenceCorrectUnder2(long correct_index, long rank)
{ 
  if (sequence_match_stats == 0) return 0;
  long correct_matches = sequence_match_stats->CorrectMatchesUnder(correct_index, rank);
  vPrint("There are %li correct matches under rank %li\n", correct_matches, rank);
  return 1;
}


long vInterpreter2::SequenceCorrectWithin0()
{
  if (sequence_match_stats == 0) return 0;
  long index, low_rank, high_rank, low_distance, high_distance;
  index = low_rank = high_rank = low_distance = high_distance = 0;
  vPrint("Enter index, low_rank, high_rank, low_distance, high_distance: ");
  vScan("%li %li %li %li %li", 
         &index, &low_rank, &high_rank, &low_distance, &high_distance);
  return SequenceCorrectWithin(index, low_rank, high_rank, low_distance, high_distance);
}


long vInterpreter2::SequenceCorrectWithin(long index, long low_rank, long high_rank,
                                           long low_distance, long high_distance)
{
  if (sequence_match_stats == 0) return 0;
  long matches = sequence_match_stats->CorrectMatchesWithin(index, low_rank, high_rank,
                                                            low_distance, high_distance);
  vPrint("%li %li-matches of ranks [%li, %li], within distance [%li, %li]\n",
           matches, index, low_rank, high_rank, low_distance, high_distance);
  return 1;
}

  
long vInterpreter2::SequenceGroundUnder0()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  vPrint("Input rank: ");
  vScan("%li", &rank);
  return SequenceGroundUnder(rank);
}


long vInterpreter2::SequenceGroundUnder0b()
{
  if (sequence_match_stats == 0) return 0;
  long rank = 0;
  long ground_index = 0;
  vPrint("Input ground_index, rank: ");
  vScan("%li %li", &ground_index, &rank);
  return SequenceGroundUnder2(ground_index, rank);
}


long vInterpreter2::SequenceGroundUnder(long rank)
{
  if (sequence_match_stats == 0) return 0;
  long ground_matches = sequence_match_stats->GroundMatchesUnder(0, rank);
  vPrint("There are %li matches under rank %li\n", ground_matches, rank);
  return 1;
}


long vInterpreter2::SequenceGroundUnder2(long ground_index, long rank)
{
  if (sequence_match_stats == 0) return 0;
  long ground_matches = sequence_match_stats->GroundMatchesUnder(ground_index, rank);
  vPrint("There are %li matches under rank %li\n", ground_matches, rank);
  return 1;
}


long vInterpreter2::SequenceSelectAll()
{
  if (sequence_match_stats == 0) return 0;
  return sequence_match_stats->SelectAll();
}


long vInterpreter2::SequenceSelectViewpoint0()
{
  if (sequence_match_stats == 0) return 0;
  double x, y, tolerance;
  x = y = tolerance = 0;
  vPrint("Input x, y, tolerance: ");
  vScan("%lf %lf %lf", &x, &y, &tolerance);
  return SequenceSelectViewpoint(x, y, tolerance);
}


long vInterpreter2::SequenceSelectViewpoint(double x, double y, 
                                             double tolerance)
{
  if (sequence_match_stats == 0) return 0;
  return sequence_match_stats->SelectViewpoint(x, y, tolerance);
}


long vInterpreter2::SequenceSelectFrontDistance0()
{
  if (sequence_match_stats == 0) return 0;
  double distance, tolerance;
  distance = tolerance = 0;
  vPrint("Input distance, tolerance: ");
  vScan("%lf %lf", &distance, &tolerance);
  return SequenceSelectFrontDistance(distance, tolerance);
}


long vInterpreter2::SequenceSelectFrontDistance(double distance, 
                                                 double tolerance)
{
  if (sequence_match_stats == 0) return 0;
  return sequence_match_stats->SelectFrontDistance(distance, tolerance);
}


long vInterpreter2::SequenceSelectRange0()
{
  if (sequence_match_stats == 0) return 0;
  long start, end;
  start = end = 0;
  vPrint("\nEnter start, end:\n");
  vScan("%li %li", &start, &end);
  return SequenceSelectRange(start, end);
}


long vInterpreter2::SequenceSelectRange(long start, long end)
{
  if (sequence_match_stats == 0) return 0;
  return sequence_match_stats->SelectRange(start, end);
}


long vInterpreter2::SequenceSelectRangeDistance0()
{
  if (sequence_match_stats == 0) return 0;
  long start, end;
  double distance, tolerance;
  distance = tolerance = start = end = 0;
  vPrint("\nEnter start, end, distance, tolerance:\n");
  vScan("%li %li %lf %lf", &start, &end, &distance, &tolerance);
  return SequenceSelectRangeDistance(start, end, distance, tolerance);
}


long vInterpreter2::SequenceSelectRangeDistance(long start, long end, 
                                                 double distance, double tolerance)
{
  if (sequence_match_stats == 0) return 0;
  return sequence_match_stats->SelectRangeFrontDistance(start, end, 
                                                        distance, tolerance);
}


  
long vInterpreter2::PrintSequenceStats()
{
  if (sequence_match_stats == 0) return 0;  
  sequence_match_stats->SetTopToPrint(top_to_print);
  sequence_match_stats->Print();
  if (seq_stats_file == 0)
  {
    exit_error("Error: seq_stats_file = 0 in PrintSequenceStats()\n");
  }
  vPrint("seq_stats_file = %s\n", seq_stats_file);
  return 1;
}


long vInterpreter2::LogScoreStats0()
{
  if (match_scores == 0) 
  {
    vPrint("No scores loaded.\n");
    return 0;
  }
  long correct, ground, start_frame, end_frame, add;
  correct = ground = start_frame = end_frame = add = 0;
  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
    
  vPrint("correct, ground params, start, end frames: ");
  vScan("%li %li %li %li", &correct, &ground, &start_frame, &end_frame);
  vPrint("Enter w0, w1, w2, w3, add: ");
  vScan("%lf %lf %lf %lf %li", &w0, &w1, &w2, &w3, &add);
  return LogScoreStats(correct, ground, start_frame, end_frame,
                       w0, w1, w2, w3, add);
}


long vInterpreter2::LogScoreStats(long correct, long ground, long start, long end,
                                   double w0, double w1, double w2, double w3, long add)
{
  if (input == 0) return 0;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  long frame;
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();

  for (frame = start; frame <= end; frame++)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    CombineLogs2(w0, w1, w2, w3, add);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      return 0;
    }
    vFrameMatchStats * match_stats = new vFrameMatchStats(match_scores, 
                                                            correct, ground);
    sequence_match_stats->AddFrameStats(match_stats, frame);
    vPrint("Done with frame %li\n", frame);
  }
  sequence_match_stats->Print();
  return 1;
}

long vInterpreter2::RootScoreStats0()
{
  if (match_scores == 0) 
  {
    vPrint("No scores loaded.\n");
    return 0;
  }
  long correct, ground, start_frame, end_frame;
  correct = ground = start_frame = end_frame = 0;
  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
    
  vPrint("correct, ground params, start, end frames: ");
  vScan("%li %li %li %li", &correct, &ground, &start_frame, &end_frame);
  vPrint("Enter w0, w1, w2, w3, add: ");
  vScan("%lf %lf %lf %lf", &w0, &w1, &w2, &w3);
  return RootScoreStats(correct, ground, start_frame, end_frame,
                        w0, w1, w2, w3);
}


long vInterpreter2::RootScoreStats(long correct, long ground, long start, long end,
                                    double w0, double w1, double w2, double w3)
{
  if (input == 0) return 0;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  long frame;
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();

  for (frame = start; frame <= end; frame++)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    CombineRoots(w0, w1, w2, w3);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      return 0;
    }
    vFrameMatchStats * match_stats = new vFrameMatchStats(match_scores, 
                                                            correct, ground);
    sequence_match_stats->AddFrameStats(match_stats, frame);
    vPrint("Done with frame %li\n", frame);
  }
  sequence_match_stats->Print();
  return 1;
}


long vInterpreter2::SumScoreStats0()
{
  if (match_scores == 0) 
  {
    vPrint("No scores loaded.\n");
    return 0;
  }
  long correct, ground, start_frame, end_frame;
  correct = ground = start_frame = end_frame = 0;
  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
    
  vPrint("correct, ground params, start, end frames: ");
  vScan("%li %li %li %li", &correct, &ground, &start_frame, &end_frame);
  vPrint("Enter w0, w1, w2, w3, add: ");
  vScan("%lf %lf %lf %lf", &w0, &w1, &w2, &w3);
  return SumScoreStats(correct, ground, start_frame, end_frame,
                       w0, w1, w2, w3);
}


long vInterpreter2::SumScoreStats(long correct, long ground, long start, long end,
                                   double w0, double w1, double w2, double w3)
{
  if (input == 0) return 0;
  long frames = input->number_of_frames;
  long input_index = current_input_index;
  long frame;
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();

  for (frame = start; frame <= end; frame++)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    CombineSums(w0, w1, w2, w3);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      return 0;
    }
    vFrameMatchStats * match_stats = new vFrameMatchStats(match_scores, 
                                                            correct, ground);
    sequence_match_stats->AddFrameStats(match_stats, frame);
    vPrint("Done with frame %li\n", frame);
  }
  sequence_match_stats->Print();
  return 1;
}


long vInterpreter2::GroundTruthDistances()
{
  if (input == 0) return 0;
  char * ground_truth_file = vReplaceEnding2(input->path_name, ".gt");
  vector<vViewGroundTruth> * gts = vViewGroundTruth::Load(ground_truth_file);
  if (gts == 0) 
  {
    vPrint("Failed to load ground truths from %s.", ground_truth_file);
    vdelete2(ground_truth_file);
    return 0;
  }

  long result = vViewGroundTruth::SaveGroundTruthDistances(gts, ground_truth_file);
  vdelete2(ground_truth_file);
  vdelete(gts);
  if (result <= 0)
  {
    vPrint("Failed to save ground truths.\n");
    return result;
  }
  else
  {
    return 1;
  }
}


long vInterpreter2::SaveCombinedScores()
{
  // Sloppy code, this should be automatic
  char * filename = "g:\\users\\athitsos\\sequences\\glove_data\\results\\new_combined.bin";

  const char * edge_base = "big\\mean_edge";
  const char * cham_base = "big\\big_canny";
  if (input == 0) return 0;
  if (training_edge_histograms == 0) return 0;
  if (extended_cutoffs == 0) return 0;

  long current_index = current_input_index;
  long test_frames = input->number_of_frames;
  const long training_frames = 107328;
  const long scores = 6;
  vdelete(combined_scores);
  combined_scores = new vCombinedScores(scores, test_frames, training_frames);
  long frame;
  for (frame = 0; frame < test_frames; frame++)
  {
    SetCurrentInput(current_index);
    JumpToFrame(frame);
    grayscale_image * hand_image = input->current_image->ToGrayCopy();
    CannyDetector(canny_threshold);
    grayscale_image * edge_image = input->current_image->ToGrayCopy();
    JumpToFrame(frame);
    DOLVS_Stats2();
    long score_index = 0;

    // Get line scores
    SetCurrentInput(current_index);
    vdelete(line_match_stats);
    line_match_stats = vComputeLineMatchStats2(existential_DOLVS, line_grid,
                                                all_model_lines, grid_fraction);
    vMatchScores * line_scores = line_match_stats->MatchScores();
    combined_scores->EnterScores(line_scores, score_index++, frame);

    // Get chamfer scores
    SetCurrentInput(current_index);
    char * training_filename = inputs[database_edge_input]->path_name;
    vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                    line_scores, training_frames);
    combined_scores->EnterScores(chamfer_scores, score_index++, frame);

    
    // Get chamfer_im_index scores
    SetCurrentInput(current_index);
    ShowImage(edge_image->ToGrayCopy());
    long success = ChamferIMIndexScores();
    vMatchScores * chamfer_im_scores = match_scores;
    match_scores = 0;
    combined_scores->EnterScores(chamfer_im_scores, score_index++, frame);


    // Get chamfer1 scores
    SetCurrentInput(current_index);
    ShowImage(edge_image->ToGrayCopy());
    ScaleBoundingBox2(192, 256, 256);
    grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
    vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
    combined_scores->EnterScores(chamfer1_scores, score_index++, frame);


    // Get edge histogram scores
    SetCurrentInput(current_index);
    v3dMatrix<double> * blurred = vBlur(right_hand_image);
    grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
    vEdgeHistogram histogram(96, blurred_gray, edge_image);
    vdelete(blurred);
    vdelete(blurred_gray);
    vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           line_scores, training_frames);
    combined_scores->EnterScores(orientation_scores, score_index++, frame);
    

    // Get fast edge histogram scores
    SetCurrentInput(current_index);
    vMatchScores * fast_orientation_scores = 
      vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                                 line_scores, training_frames);
    combined_scores->EnterScores(fast_orientation_scores, score_index++, frame);

    
    vdelete(hand_image);
    vdelete(edge_image); 
    vdelete(chamfer_scores);
    vdelete(line_scores);
    vdelete(chamfer_im_scores);
    vdelete(chamfer1_scores);
    vdelete(orientation_scores);
    vdelete(fast_orientation_scores);
  }
  long result = combined_scores->Save(filename);
  if (result >= 1)
  {
    vPrint("combined scores saved successfully\n");
  }
  else
  {
    vPrint("error in saving combined scores\n");
  }
  return result;
}


long vInterpreter2::SaveCombinedScoresOld()
{
  char * filename1 = "C:\\users\\athitsos\\sequences\\glove_data\\results\\combined.bin";
  char * filename2 = "C:\\users\\athitsos\\sequences\\glove_data\\results\\combined2.bin";
  // This is not entirely correct.
  const char * edge_base = "big\\mean_edge";
  const char * cham_base = "big\\big_canny";
  if (input == 0) return 0;
  if (pca == 0) return 0;
  if (training_edge_histograms == 0) return 0;
  if (finger_info == 0) return 0;
  long current_index = current_input_index;
  long test_frames = input->number_of_frames;
  char * filename;
  if (test_frames == 28) filename = filename1;
  else filename = filename2;
  const long training_frames = 107328;
  const long scores = 4;
  vdelete(combined_scores);
  combined_scores = new vCombinedScores(scores, test_frames, training_frames);
  long frame;
  for (frame = 0; frame < test_frames; frame++)
  {
    // Get chamfer scores
    SetCurrentInput(current_index);
    SetScoreBase(cham_base, frame);
    combined_scores->EnterScores(match_scores, 0, frame);
      
    // Get edge histogram scores
    SetCurrentInput(current_index);
    SetScoreBase(edge_base, frame);
    combined_scores->EnterScores(match_scores, 1, frame);

    // Get finger scores
    SetCurrentInput(current_index);
    JumpToFrame(frame);
    FingerScores();
    combined_scores->EnterScores(match_scores, 2, frame);

    // Get PCA moment scores.
    SetCurrentInput(current_index);
    JumpToFrame(frame);
    PCAMomentScores();
    combined_scores->EnterScores(match_scores, 3, frame);
  }
  long result = combined_scores->Save(filename);
  if (result >= 1)
  {
    vPrint("combined scores saved successfully\n");
  }
  else
  {
    vPrint("error in saving combined scores\n");
  }
  return result;
}


long vInterpreter2::LoadCombinedScores()
{
  char filename[1000];
  sprintf(filename, "%s%s", sequences_directory, 
          "\\glove_data\\results\\new_combined.bin");

  const long test_frames = input->number_of_frames;
  const long training_frames = 107328;
  const long scores = 6;
  vdelete(combined_scores);
  combined_scores = new vCombinedScores(scores, test_frames, training_frames);

  long result = combined_scores->Load(filename);
  if (result >= 1)
  {
    vPrint("Read combined scores successfully\n");
  }
  else 
  {
    vPrint("Failed to read combined scores from %s\n", filename);
  }
  return result;
}


long vInterpreter2::LoadCombinedScoresOld()
{
  char filename1[1000];
  sprintf(filename1, "%s%s", sequences_directory, 
          "\\glove_data\\results\\combined.bin");
  char filename2[1000];
  sprintf(filename2, "%s%s", sequences_directory, 
          "\\glove_data\\results\\combined2.bin");

  const long test_frames = input->number_of_frames;
  char * filename;
  if (test_frames == 28) filename = filename1;
  else filename = filename2;
  const long training_frames = 107328;
  const long scores = 4;
  vdelete(combined_scores);
  combined_scores = new vCombinedScores(scores, test_frames, training_frames);

  long result = combined_scores->Load(filename);
  if (result >= 1)
  {
    vPrint("Read combined scores successfully\n");
  }
  else 
  {
    vPrint("Failed to read combined scores from %s\n", filename);
  }
  return result;
}


long vInterpreter2::CombineLogs0a()
{
  if (combined_scores == 0) return 0;

  double w0, w1, w2, w3, w4, w5;
  w0 = w1 = w2 = w3 = w4 = w5 = 0;
  vPrint("Enter weights w0, w1, w2, w3, w4, w5:\n");
  vScan("%lf %lf %lf %lf %lf %lf", &w0, &w1, &w2, &w3, &w4, &w5);
  return CombineLogs(w0, w1, w2, w3, w4, w5);
}


long vInterpreter2::CombineLogs(double w0, double w1, double w2, double w3, double w4, double w5)
{
  if (combined_scores == 0) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 6);
  weights.WriteN(6, w0, w1, w2, w3, w4, w5);
  vdelete(match_scores);
  match_scores = combined_scores->CombineLogs(&weights, frame);
  UpdateMatchScores(frame);
  return 1;
}


long vInterpreter2::FilterCombineLogs0()
{
  if (match_scores == 0) return 0;
  if (combined_scores == 0) return 0;

  double w0, w1, w2, w3, w4, w5;
  long number = 0;
  w0 = w1 = w2 = w3 = w4 = w5 = 0;
  vPrint("Enter number, and weights w0, w1, w2, w3, w4, w5:\n");
  vScan("%li %lf %lf %lf %lf %lf %lf", &number, &w0, &w1, &w2, &w3, &w4, &w5);
  return FilterCombineLogs(number, w0, w1, w2, w3, w4, w5);
}


// Keep the highest "number" scores from the current match scores, and 
// for those scores combine scores with this weight. Assuming the
// current scores are like a "step 1", this does step 2.
long vInterpreter2::FilterCombineLogs(long number, double w0, double w1, double w2, 
                                       double w3, double w4, double w5)
{
  if (combined_scores == 0) return 0;
  if (match_scores == 0) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;
  vMatrix<double> weights_matrix(1, 6);

  vMatchScores * selected = match_scores->KeepHighest(number);
  vMatchScores * scores0 = combined_scores->GetScores(0, frame);
  vMatchScores * filter0 = scores0->Select(selected);

  vMatchScores * scores1 = combined_scores->GetScores(1, frame);
  vMatchScores * filter1 = scores1->Select(selected);
  
  vMatchScores * scores2 = combined_scores->GetScores(2, frame);
  vMatchScores * filter2 = scores2->Select(selected);
  
  vMatchScores * scores3 = combined_scores->GetScores(3, frame);
  vMatchScores * filter3 = scores3->Select(selected);
  
  vMatchScores * scores4 = combined_scores->GetScores(4, frame);
  vMatchScores * filter4 = scores4->Select(selected);
  
  vMatchScores * scores5 = combined_scores->GetScores(5, frame);
  vMatchScores * filter5 = scores5->Select(selected);

  vdelete(scores0);
  vdelete(scores1);
  vdelete(scores2);
  vdelete(scores3);
  vdelete(scores4);
  vdelete(scores5);
  vdelete(selected);

  vMatchScores * combined = 
    filter0->CombineLogs(6, filter0, w0, filter1, w1, filter2, w2,
                         filter3, w3, filter4, w4, filter5, w5);
  vMatchScores * final_scores = combined->Merge(combined, match_scores);
  vdelete(combined);
  vdelete(match_scores);
  vdelete(filter0);
  vdelete(filter1);
  vdelete(filter2);
  vdelete(filter3);
  vdelete(filter4);
  vdelete(filter5);
  match_scores = final_scores;
  
  UpdateMatchScores(frame);
  return 1;
}


long vInterpreter2::CombineLogs0b()
{
  if (combined_scores == 0) return 0;

  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
  long add = 0;
  vPrint("Enter weights w0, w1, w2, w3, and parameter add\n");
  vScan("%lf %lf %lf %lf %li", &w0, &w1, &w2, &w3, &add);
  return CombineLogs2(w0, w1, w2, w3, add);
}


long vInterpreter2::CombineLogs2(double w0, double w1, double w2, double w3, long add)
{
  if (combined_scores == 0) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 4);
  weights.WriteN(4, w0, w1, w2, w3);
  vdelete(match_scores);
  match_scores = combined_scores->CombineLogs2(&weights, frame, add);
  UpdateMatchScores(frame);
  return 1;
}


long vInterpreter2::CombineRoots0()
{
  if (combined_scores == 0) return 0;

  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
  vPrint("Enter weights w0, w1, w2, w3\n");
  vScan("%lf %lf %lf %lf", &w0, &w1, &w2, &w3);
  return CombineRoots(w0, w1, w2, w3);
}


long vInterpreter2::CombineRoots(double w0, double w1, double w2, double w3)
{
  if (combined_scores == 0) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 4);
  weights.WriteN(4, w0, w1, w2, w3);
  vdelete(match_scores);
  match_scores = combined_scores->CombineRoots(&weights, frame);
  UpdateMatchScores(frame);
  return 1;
}


long vInterpreter2::CombineSums0()
{
  if (combined_scores == 0) return 0;

  double w0, w1, w2, w3;
  w0 = w1 = w2 = w3 = 0;
  vPrint("Enter weights w0, w1, w2, w3\n");
  vScan("%lf %lf %lf %lf", &w0, &w1, &w2, &w3);
  return CombineSums(w0, w1, w2, w3);
}


long vInterpreter2::CombineSums(double w0, double w1, double w2, double w3)
{
  if (combined_scores == 0) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 4);
  weights.WriteN(4, w0, w1, w2, w3);
  vdelete(match_scores);
  match_scores = combined_scores->CombineSums(&weights, frame);
  UpdateMatchScores(frame);
  return 1;
}


long vInterpreter2::CombineWithLogs0()
{
  if (combined_scores == 0) return 0;
  if (match_scores == 0) return 0;

  double w0, w1, w2, w3, w4;
  w0 = w1 = w2 = w3 = w4 = 0;
  vPrint("Enter weight for current scores and weights w1, w2, w3, w4\n");
  vScan("%lf %lf %lf %lf %lf", &w0, &w1, &w2, &w3, &w4);
  return CombineWithLogs(w0, w1, w2, w3, w4);
}


// The new version is more simple
/*long vInterpreter2::CombineWithLogs(double w0, double w1, double w2, 
                                     double w3, double w4)
{
  if (combined_scores == 0) return 0;
  if (match_scores == 0) return 0;
  if ((w0 < 0) || (w1 < 0) || (w2 < 0) || (w3 < 0) || (w4 < 0)) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 4);
  weights.WriteN(4, w1, w2, w3, w4);
  vMatchScores * current = match_scores;
  // The next line has a bug, it should be calling combine logs
  vMatchScores * combined = combined_scores->CombineSums(&weights, frame);
  match_scores = current->Combine1(combined, w0);
  vPrint("match scores have been updated\n");
  vdelete(current);
  vdelete(combined);
  
  UpdateMatchScores(frame);
  return 1;
}*/


long vInterpreter2::CombineWithLogs(double w0, double w1, double w2, 
                                     double w3, double w4)
{
  if (combined_scores == 0) return 0;
  if (match_scores == 0) return 0;
  if ((w0 < 0) || (w1 < 0) || (w2 < 0) || (w3 < 0) || (w4 < 0)) return 0;
  long frame = input->current_frame;
  if (frame >= combined_scores->test_frames) return 0;

  vMatrix<double> weights(1, 5);
  weights.WriteN(5, w1, w2, w3, w4, w0);
  vMatchScores * current = match_scores;
  combined_scores->EnterScores(current, 4, frame);
  match_scores = combined_scores->CombineLogs(&weights, frame);
  vPrint("match scores have been updated\n");
  vdelete(current);
  
  UpdateMatchScores(frame);
  return 1;
}
  
long vInterpreter2::RandomChamferDistances0()
{
  if (input == 0) return 0;
  vPrint("Enter database input, number of picks, and training frames\n");
  long  number_of_picks, training_frames;
  number_of_picks = training_frames = 0;
  vScan("%li %li", &number_of_picks, &training_frames);
  return RandomChamferDistances(number_of_picks, training_frames);
}


long vInterpreter2::RandomChamferDistances(long number_of_picks,
                                            long training_frames)
{
  if (database_edge_input < 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;

  class_sequence_information * info = (class_sequence_information *) inputs[database_edge_input];
  char * database_file = info->path_name;
  vRLE_Sequence database(database_file);
  if (database.IsValid() <= 0)
  {
    vPrint("Error, can't open %s\n", database_file);
    return 0;
  }

  char * target = vMergeStrings2(database_file, ".chamfer_index");
  long result = vRandomChamferDistances(&database, number_of_picks, target, training_frames);
  vdelete2(target);
  return result;
}


long vInterpreter2::MergeRandomDistances0()
{
  if (input == 0) return 0;
  long training_input = 0;
  long start, end;
  start = end = 0;
  vPrint("Enter training input, start, end: ");
  vScan("%li %li %li", &training_input, &start, &end);
  return MergeRandomDistances(training_input, start, end);
}


long vInterpreter2::MergeRandomDistances(long training_input, 
                                          long start, long end)
{
  if (training_input >= number_of_inputs) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[training_input];
  const char * training_base = info->path_name;
  char * distances_base = vMergeStrings2(training_base, ".chamfer_index");
  long success = vMergeRandomChamferDistances(distances_base, start, end);
  vdelete2(distances_base);
  if (success <= 0)
  {
    vPrint("Failed to merge distances\n");
  }
  else
  {
    vPrint("Merged distances successfully\n");
  }
  return success;
}


long vInterpreter2::LoadRandomEmbedding()
{
  long training_input = database_edge_input;
  if (training_input >= number_of_inputs) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[training_input];
  const char * training_base = info->path_name;
  char * distances_base = vMergeStrings2(training_base, ".chamfer_index");
  
  char * picks_file = vMergeStrings2(distances_base, ".random");
  vdelete2(distances_base);

  FILE * fp = fopen(picks_file, vFOPEN_READ);
  if (fp == 0) 
  {
    vPrint("Failed to open picks file %s\n", picks_file);
    return 0;
  }

  v3dMatrix<long> * reference_frames = v3dMatrix<long>::Read(fp);
  if (reference_frames == 0)
  {
    vPrint("Failed to read reference_frames\n");
    return 0;
  }
  v3dMatrix<long> * reference_rotations = v3dMatrix<long>::Read(fp);
  fclose(fp);
  vdelete2(picks_file);
  if (reference_rotations == 0)
  {
    vPrint("Failed to read reference_rotations\n");
    vdelete(reference_frames);
    return 0;
  }

  char * chamfer1_file = vReplaceEnding2(inputs[training_input]->path_name, ".chamfer1_index");
  vdelete(chamfer_index_im);
  chamfer_index_im = vLoadMatrixAsUshorts<float>(chamfer1_file);
  long success = 1;
  if (chamfer_index_im <= 0)
  {
    vPrint("Failed to load chamfer index im from %s\n", chamfer1_file);
    success = 0;
  }
  vdelete2(chamfer1_file);

  DeleteDistanceTransforms();
  vdelete(reference_images);
  long dimensions = reference_frames->Cols();
  dimensions = Min(dimensions, (long) 256);
  reference_images = new v3dMatrix<float>(dimensions, 5, 1);
  vArray2(float) references = reference_images->Matrix2(0);
  long i;
  for (i = 0; i < dimensions; i++)
  {
    long frame = reference_frames->Matrix()[i];
    long rotation = reference_rotations->Matrix()[i];
    references[i][0] = (float) frame;
    references[i][1] = (float) rotation;
    references[i][2] = (float) 1;
    references[i][3] = (float) (frame * 4 + rotation);
    references[i][4] = (float) i;
  }
  vdelete(reference_frames);
  vdelete(reference_rotations);
  vdelete(training_embeddings);
  training_embeddings = 0;

  vdelete(im_reference_images);
  im_reference_images = new v3dMatrix<float>(reference_images);

  long temp = LoadTrainingEmbedding();
  if (temp <= 0)
  {
    success = 0;
  }

//  training_embeddings = vLoadMatrixAsUshorts<float>(distances_base);
//  vdelete2(distances_base);
//  if (training_embeddings == 0)
//  {
//   vPrint("Failed to load random distances from %s\n", distances_base);
//  }

  return success;
}


// Loads in chamfer distances between reference images and all
// database images, assuming that reference_images has been
// initialized.
long vInterpreter2::LoadTrainingEmbedding()
{
  if (reference_images == 0) return 0;
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0) return 0;

  vRLE_Sequence * database_edges = (vRLE_Sequence *) sequence;
  vAllChamferDistances acd(database_edges);
  long training_images = database_edges->NumberOfFrames() * 4;

  // process the reference images.
  v3dMatrix<float> * temp_references = reference_images;
  reference_images = V_BoostMap::CleanUp(107328, temp_references);
  vdelete(temp_references);

  long dimensions = reference_images->Rows();
  vdelete(training_embeddings);
  training_embeddings = new v3dMatrix<float>(training_images, dimensions, 1);
  vArray2(float) embeddings = training_embeddings->Matrix2(0);
  vArray2(float) references = reference_images->Matrix2(0);

  long i;
  vPrint("\n");
  for (i = 0; i < dimensions; i++)
  {
    long frame = vRound(references[i][0]);
    long view = vRound(references[i][1]);
    v3dMatrix<float> * distances_matrix = acd.Distances(frame, view);
    if (distances_matrix->Cols() != training_images)
    {
      exit_error("Error: In LoadBoostMap, %li distances loaded\n",
                      distances_matrix->Cols());
    }
    vArray(float) distances = distances_matrix->Matrix();
    long j;
    for (j = 0; j < training_images; j++)
    {
      embeddings[j][i] = distances[j];
    }
    vPrint("Loaded dimension %li of %li\r", i+1, dimensions);
    vdelete(distances_matrix);
  }

  vPrint("\n");

  embedding_dimensions = dimensions;
  reference_start = 0;
  return 1;
}


// Calculate chamfer distances between current image and the reference images.
long vInterpreter2::ChamferDistances()
{
  if (input == 0) return 0;
  long training_input = database_edge_input;
  
  if ((training_input < 0) || (training_input >= number_of_inputs)) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[training_input];
  vVideoFile * training_sequence = info->sequence;
  if (strcmp(training_sequence->Type(), "vRLE_Sequence") != 0)
  {
    vPrint("Error, training input is not rle sequence\n");
    return 0;
  }

  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  vdelete(test_embedding);
  test_embedding = 0;

  grayscale_image * gray = input->current_image->ToGrayCopy();
  vMatrix<double> distances = 
    vDatabasePicksChamfer(gray, (vRLE_Sequence *) training_sequence, 
                           reference_images, reference_start, 
                           embedding_dimensions, -1, -1);
  test_embedding = new v3dMatrix<float>(&distances);
  vdelete(gray);
  return 1;
}


long vInterpreter2::SetDistanceIndex0()
{
  long index = 0;
  vPrint("Input new distance index: ");
  vScan("%li", &index);
  return SetDistanceIndex(index);
}


long vInterpreter2::SetDistanceIndex(long index)
{
  if (reference_index < 0) return 0;
  reference_index = index;
  return 1;
}


long vInterpreter2::SinglePickScores0()
{
  if (input == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if (test_embedding == 0) return 0;

  long number = test_embedding->Size();
  long column = reference_index;
  long result = 0;
  if ((column >= 0) && (column < number))
  {
    double distance = test_embedding->Matrix()[column];
    vdelete(match_scores);
    match_scores = new vMatchScores(training_embeddings, 
                                     column, distance);
    UpdateMatchScores(input->current_frame);
    result = 1;
  }
  return result;
}


long vInterpreter2::SetRandomDimensions0()
{
  vPrint("Input random start, random dimensions: ");
  long reference_start = 0;
  long random_dim = 0;
  vScan("%li %li", &reference_start, &random_dim);
  return SetRandomDimensions(reference_start, random_dim);
}


long vInterpreter2::SetRandomDimensions(long start, long dimensions)
{
  if(training_embeddings != 0)
  {
    if ((dimensions < 0) || (start < 0) || 
        (start+dimensions > training_embeddings->Cols()))
    {
      vPrint("Bad dimensions given:\n");
      return 0;
    }
  }

  reference_start = start;
  embedding_dimensions = dimensions;
  vPrint("reference_start = %li, embedding_dimensions = %li\n", start, dimensions);
  return 1;
}


long vInterpreter2::MultiPickScores0()
{
  if (input == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if (test_embedding == 0) return 0;

  vdelete(match_scores);
  match_scores = new vMatchScores(training_embeddings, test_embedding,
                                   reference_start, embedding_dimensions,
                                   reference_images);
  UpdateMatchScores(input->current_frame);

  return 1;
}


long vInterpreter2::SetDatabaseInputs0()
{
  long in_database_input, in_database_edge_input;
  in_database_input = in_database_edge_input = 0;
  vPrint("\nEnter database input, database edge input:\n");
  vScan("%li %li", &in_database_input, &in_database_edge_input);
  return SetDatabaseInputs(in_database_input, in_database_edge_input);
}


long vInterpreter2::SetDatabaseInputs(long in_database_input, 
                                       long in_database_edge_input)
{
  if ((in_database_input >= 0) && (in_database_input < number_of_inputs))
  {
    database_input = in_database_input;
  }

  if ((in_database_edge_input >= 0) && (in_database_edge_input < number_of_inputs))
  {
    database_edge_input = in_database_edge_input;
  }
  return 1;
}


long vInterpreter2::ChamferIndexScores0()
{
  ChamferDistances();
  return MultiPickScores0();
}


long vInterpreter2::ChamferIMIndexScores()
{
  if (input == 0) return 0;
  if (chamfer_index_im == 0) return 0;
  if (im_reference_images == 0) return 0;
  if (point_sequence == 0) return 0;

  grayscale_image * gray = input->current_image->ToGrayCopy();
  vMatrix<double> distances = 
    vDatabasePicksChamferIM(gray, point_sequence, 
                             im_reference_images, 0, 
                             200, -1, -1);
  vdelete(gray);

  vdelete(match_scores);
  match_scores = new vMatchScores(chamfer_index_im, &distances,
                                   0, 200);
  UpdateMatchScores(input->current_frame);
  return 1;
}


// RankViews ranks the views of the database in order of similarity
// to the current image. This is done online, no precomputed info
// about the current image is used. The difference between RankViews2
// and RankViews2 is that here we use two steps, i.e. a prefiltering
// step and a refining step.
long vInterpreter2::RankViews()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  long test_input_index = current_input_index;

  // Prefilter using moments and fingers.
  double fingers_weight= 0.3;
  double moments_weight = 0.2;
  double chamfer_index_weight = 0.5;

  // First get finger scores and moment scores.
  long success;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToGrayCopy());
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(hand_edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    finger_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                               moment_scores, moments_weight, 
                               finger_scores, fingers_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vMatchScores * filtered_finger_scores = finger_scores->Select(scores_to_keep);
  vMatchScores * filtered_moment_scores = moment_scores->Select(scores_to_keep);
//  vMatchScores * filtered_index_scores = chamfer_index_scores->Select(scores_to_keep);
  vdelete(finger_scores);
  vdelete(moment_scores);
  vdelete(chamfer_index_scores);
  
  // Get chamfer scores
  char training_filename[1000];
  sprintf(training_filename, "%s%s", sequences_directory,
          "\\glove_data\\training\\training7_edges.rle");
  vMatchScores * chamfer_scores = vChamferRanks(hand_edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(right_hand_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, hand_edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);
  vdelete(blurred_gray);

//  vMatchScores * new_scores =
//    vMatchScores::CombineLogs(4, chamfer_scores, (double) .4, 
//                               orientation_scores, (double) .1,
//                               filtered_finger_scores, (double) .4, 
//                               filtered_moment_scores, (double) .1);
  vMatchScores * new_scores =
    vMatchScores::CombineLogs(4, chamfer_scores, (double) .25, 
                               orientation_scores, (double) .25,
                               filtered_finger_scores, (double) .25, 
                               filtered_moment_scores, (double) .25);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(filtered_finger_scores);
  vdelete(filtered_moment_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);

  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);
  
  SetCurrentInput(test_input_index);
  ShowImage(original_hand_image->ToRGBCopy());
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// RankViews ranks the views of the database in order of similarity
// to the current image. This is done online, no precomputed info
// about the current image is used. The difference between RankViews
// and RankViews2 is that here we use only one step.
long vInterpreter2::RankViews2()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  long test_input_index = current_input_index;

  // Prefilter using moments and fingers.
  double fingers_weight= 0.3;
  double moments_weight = 0.2;
  double chamfer_index_weight = 0.5;

  // First get finger scores and moment scores.
  long success;
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    finger_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                               moment_scores, moments_weight, 
                               finger_scores, fingers_weight);
  vdelete(finger_scores);
  vdelete(moment_scores);
  vdelete(chamfer_index_scores);
  
  match_scores = prefilter_scores;

  vdelete(test_image);
  vdelete(edge_image);
  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// RankViews ranks the views of the database in order of similarity
// to the current image. This is done online, no precomputed info
// about the current image is used. 
long vInterpreter2::RankViews3()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  long test_input_index = current_input_index;

  // Prefilter using moments and fingers.
  double fingers_weight= 0.3;
  double moments_weight = 0.2;
  double chamfer_index_weight = 0.5;

  // First get finger scores and moment scores.
  long success;
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    finger_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                               moment_scores, moments_weight, 
                               finger_scores, fingers_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vMatchScores * filtered_finger_scores = finger_scores->Select(scores_to_keep);
  vMatchScores * filtered_moment_scores = moment_scores->Select(scores_to_keep);
//  vMatchScores * filtered_index_scores = chamfer_index_scores->Select(scores_to_keep);
  vdelete(finger_scores);
  vdelete(moment_scores);
  vdelete(chamfer_index_scores);
  
  // Get chamfer scores
  char training_filename[1000];
  sprintf(training_filename, "%s%s", sequences_directory,
          "\\glove_data\\training\\training7_edges.rle");
  vVideoFile * database = vOpenReadVideoFile(training_filename);
  assert(database != 0);
  vMatchScores * chamfer_scores = vFlexibleChamferRanks(edge_image, database, 
                                                          scores_to_keep, step1_views);

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);
  vdelete(blurred_gray);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(4, chamfer_scores, (double) .4, 
                               orientation_scores, (double) .1,
                               filtered_finger_scores, (double) .4, 
                               filtered_moment_scores, (double) .1);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(filtered_finger_scores);
  vdelete(filtered_moment_scores);
  vdelete(scores_to_keep);
  vdelete(test_image);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);
  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Like RankViews, but here we also use scores from line matching.
long vInterpreter2::RankViews4()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());


  // Prefilter using moments and fingers.
  double fingers_weight= 0.3;
  double moments_weight = 0.2;
  double chamfer_index_weight = 0.5;

  // First get finger scores and moment scores.
  long success;
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    finger_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                               moment_scores, moments_weight, 
                               finger_scores, fingers_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vMatchScores * filtered_finger_scores = finger_scores->Select(scores_to_keep);
  vMatchScores * filtered_moment_scores = moment_scores->Select(scores_to_keep);
//  vMatchScores * filtered_index_scores = chamfer_index_scores->Select(scores_to_keep);
  vdelete(finger_scores);
  vdelete(moment_scores);
  vdelete(chamfer_index_scores);
  
  // Get chamfer scores
  char training_filename[1000];
  sprintf(training_filename, "%s%s", sequences_directory,
          "\\glove_data\\training\\training7_edges.rle");
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);
  vdelete(blurred_gray);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();


//  vMatchScores * new_scores =
//    vMatchScores::CombineLogs(5, chamfer_scores, (double) .3, 
//                               orientation_scores, (double) .1,
//                               filtered_finger_scores, (double) .3, 
//                               filtered_moment_scores, (double) .1,
//                               line_scores, (double) .2);
  vMatchScores * new_scores =
    vMatchScores::CombineLogs(5, chamfer_scores, (double) .2, 
                               orientation_scores, (double) .2,
                               filtered_finger_scores, (double) .2, 
                               filtered_moment_scores, (double) .2,
                               line_scores, (double) .2);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(filtered_finger_scores);
  vdelete(filtered_moment_scores);
  vdelete(scores_to_keep);
  vdelete(test_image);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);
  vdelete(line_scores);
  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// step1: chamfer_index
// step2: chamfer
long vInterpreter2::RankViews5()
{
  if (input == 0) 
  {
    vPrint("no input\n");
    return 0;
  }
  if (input->current_image == 0) 
  {
    vPrint("no current image\n");
    return 0;
  }
  if (hand_edge_image == 0) 
  {
    vPrint("no current hand edge image\n");
    return 0;
  }

  long test_input_index = current_input_index;
  grayscale_image * edge_image = hand_edge_image->ToGrayCopy();

  // Prefilter using chamfer index.
  long success;
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  vMatchScores * prefilter_scores = chamfer_index_scores;
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  
  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vMatchScores * final_scores = vMatchScores::Merge(chamfer_scores,
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(chamfer_scores);

  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(test_input_index);
  if (original_hand_image != 0)
  {
    ShowImage(original_hand_image->ToRGBCopy());
  }
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Combining chamfer index scores and chamfer1 scores in step1.
long vInterpreter2::RankViews6()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  SetCurrentInput(test_input_index);
  ShowImage(test_image);
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Now combine scores and filter out the lowest ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  
  match_scores = prefilter_scores;
  vdelete(edge_image);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Combining chamfer_im_scores and chamfer1 scores in step1.
long vInterpreter2::RankViews6b()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  double chamfer_index_weight = 0.7;
  double chamfer1_weight = 0.3;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  SetCurrentInput(test_input_index);
  ShowImage(test_image);
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Now combine scores, and filter out the lowest ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight);
  match_scores = prefilter_scores;

  vdelete(edge_image);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer index and chamfer1.
// Step2: chamfer and line matching
long vInterpreter2::RankViews7()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  DOLVS_Stats();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  SetCurrentInput(test_input_index);
  ShowImage(test_image);
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  
  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(2, chamfer_scores, (double) .5, 
                               line_scores, .5);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer index and chamfer1.
// Step2: chamfer, line matching, orientation histograms
long vInterpreter2::RankViews8()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  DOLVS_Stats();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  SetCurrentInput(test_input_index);
  ShowImage(test_image);
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  
  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);
  vdelete(blurred_gray);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .45, 
                               line_scores, .45,
                               orientation_scores, .1);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: fast edge orientation scores.
long vInterpreter2::RankViews9()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  match_scores = orientation_scores;

  vdelete(test_image);
  vdelete(edge_image);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: chamfer, line matching, orientation histograms
long vInterpreter2::RankViews10()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  DOLVS_Stats();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .45, 
                               line_scores, .45,
                               orientation_scores, .1);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: chamfer, line matching, orientation histograms
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews11()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .45, 
                               line_scores, .45,
                               orientation_scores, .1);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer1, fast orientation histograms
// Step2: chamfer, line matching, orientation histograms
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews12()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

//  double chamfer_index_weight = 0.4;
//  double chamfer1_weight = 0.4;
  double chamfer_index_weight = 0.0;
  double chamfer1_weight = 0.8;
  double orientation_weight = .2;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .45, 
                               line_scores, .45,
                               orientation_scores, .1);
//  vMatchScores * new_scores =
//    vMatchScores::CombineLogs(2, chamfer_scores, (double) .45, 
//                               orientation_scores, (double) .1);

  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms, line scores
// No step2.
long vInterpreter2::RankViews13()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats2(existential_DOLVS, line_grid,
                                              all_model_lines, grid_fraction);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Now combine all  scores, 
  vdelete(match_scores);
  match_scores = 
    chamfer1_scores->CombineLogs(4, chamfer_index_scores, (double) .25,
                                 chamfer1_scores, (double) .25,
                                 orientation_scores, (double) .25,
                                 line_scores, (double) .25);

  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer1_scores);
  vdelete(chamfer_index_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer1, fast orientation histograms
// Step2: chamfer, slow orientation histograms
// This is the version that only uses previously-used methods,
// none of my methods. It is used to illustrate that my methods
// improve the performance of the system.
long vInterpreter2::RankViews14()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  ShowImage(test_image->ToGrayCopy());

  double chamfer1_weight = 0.8;
  double orientation_weight = .2;

  SetCurrentInput(test_input_index);

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get fast edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer1_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step1 scores, and filter out lowest 
  // ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);
  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(2, chamfer_scores, .45,
                               orientation_scores, .1);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  vdelete(match_scores);
  match_scores = final_scores;

  vdelete(chamfer_scores);
  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}

// Step0: We do palm scores, and we only keep the top step0_views.
// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: chamfer, line matching, orientation histograms
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews15()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (extended_cutoffs == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  // step 0, palm scores.
  vMatchScores * palm_scores = vPalmCutoffScores(extended_cutoffs,
                                                   &centered_palm_cutoff);
  vMatchScores * step0_scores = palm_scores->KeepHighest(step0_views);

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * selected_prefilter = prefilter_scores->Select(step0_scores);
  vdelete(prefilter_scores);
  vMatchScores * scores_to_keep = selected_prefilter->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .45, 
                               line_scores, .45,
                               orientation_scores, .1);
  vMatchScores * temp_scores = vMatchScores::Merge(new_scores, selected_prefilter);
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, 
                                                      palm_scores);
  match_scores = final_scores;

  vdelete(selected_prefilter);
  vdelete(palm_scores);
  vdelete(temp_scores);
  vdelete(step0_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: chamfer, chamfer_im_index, chamfer1, line matching, orientation histograms
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews16()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vMatchScores * chamfer_im_step2 = chamfer_index_scores->Select(scores_to_keep);
  vMatchScores * chamfer1_step2 = chamfer1_scores->Select(scores_to_keep);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid, 
                                              all_model_lines, &database_frames,
                                              &database_views, 0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(5, chamfer_scores, (double) .2, 
                               chamfer_im_step2, (double) .2,
                               chamfer1_step2, (double) .2,
                               line_scores, (double) .2,
                               orientation_scores, (double) .2);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(chamfer_im_step2);
  vdelete(chamfer1_step2);
  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: line matching, together step1 scores
// Step3: chamfer, line matching, orientation histograms
long vInterpreter2::RankViews17()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.4;
  double chamfer1_weight = 0.4;
  double orientation_weight = .2;

  long success;
  const long step0_views = 2000;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores1 = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep1 = prefilter_scores1->KeepHighest(step0_views);
  vMatchScores * chamfer_im_step2 = chamfer_index_scores->Select(scores_to_keep1);
  vMatchScores * chamfer1_step2 = chamfer1_scores->Select(scores_to_keep1);
  vMatchScores * orientations_step2 = orientation_scores->Select(scores_to_keep1);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get line scores
  vector<long> database_frames(step0_views);
  vector<long> database_views(step0_views);
  vArray(vMatchScore) score_sorted = scores_to_keep1->GetScores();
  long i;
  for (i = 0; i < step0_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid, 
                                              all_model_lines, &database_frames,
                                              &database_views, 0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();
  vMatchScores * prefilter_scores2 = 
    line_scores->CombineLogs(4, chamfer_im_step2, (double) 0.3,
                             chamfer1_step2, (double) 0.3,
                             line_scores, (double) 0.3,
                             orientations_step2, (double) 0.1);

  vMatchScores * scores_to_keep2 = prefilter_scores2->KeepHighest(step1_views);
  vMatchScores * lines_step3 = line_scores->Select(scores_to_keep2);
  vdelete(chamfer_im_step2);
  vdelete(chamfer1_step2);
  vdelete(orientations_step2);
  vdelete(scores_to_keep1);
  vdelete(line_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep2, step1_views);

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep2, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .6, 
                               lines_step3, (double) .4,
                               orientation_scores, (double) .0);
  vMatchScores * temp_scores = vMatchScores::Merge(new_scores, 
                                                     prefilter_scores2);
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, prefilter_scores1);
  match_scores = final_scores;

  vdelete(temp_scores);
  vdelete(prefilter_scores1);
  vdelete(prefilter_scores2);
  vdelete(new_scores);
  vdelete(scores_to_keep2);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(lines_step3);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Like RankViews11, but here all weights are equal, and 
// orientation histograms not used for indexing.
// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: chamfer, line matching, orientation histograms
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews18()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;
  double orientation_weight = 0;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .33, 
                               line_scores, (double) .33,
                               orientation_scores, (double) .33);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Like RankViews17, but no fast orientation histograms are used,
// and weights are all equal
// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: line matching, together step1 scores
// Step3: chamfer, line matching, orientation histograms
long vInterpreter2::RankViews19()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;
  double orientation_weight = 0;

  long success;
  const long step0_views = 4000;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores1 = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep1 = prefilter_scores1->KeepHighest(step0_views);
  vMatchScores * chamfer_im_step2 = chamfer_index_scores->Select(scores_to_keep1);
  vMatchScores * chamfer1_step2 = chamfer1_scores->Select(scores_to_keep1);
  vMatchScores * orientations_step2 = orientation_scores->Select(scores_to_keep1);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get line scores
  vector<long> database_frames(step0_views);
  vector<long> database_views(step0_views);
  vArray(vMatchScore) score_sorted = scores_to_keep1->GetScores();
  long i;
  for (i = 0; i < step0_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid, 
                                              all_model_lines, &database_frames,
                                              &database_views, 0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();
  vMatchScores * prefilter_scores2 = 
    line_scores->CombineLogs(4, chamfer_im_step2, (double) 0.33,
                             chamfer1_step2, (double) 0.33,
                             line_scores, (double) 0.33,
                             orientations_step2, (double) 0.0);

  vMatchScores * scores_to_keep2 = prefilter_scores2->KeepHighest(step1_views);
  vMatchScores * lines_step3 = line_scores->Select(scores_to_keep2);
  vdelete(chamfer_im_step2);
  vdelete(chamfer1_step2);
  vdelete(orientations_step2);
  vdelete(scores_to_keep1);
  vdelete(line_scores);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep2, step1_views);

  // Get edge orientation scores
  orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep2, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .33, 
                               lines_step3, (double) .33,
                               orientation_scores, (double) .33);
  vMatchScores * temp_scores = vMatchScores::Merge(new_scores, 
                                                     prefilter_scores2);
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, prefilter_scores1);
  match_scores = final_scores;

  vdelete(temp_scores);
  vdelete(prefilter_scores1);
  vdelete(prefilter_scores2);
  vdelete(new_scores);
  vdelete(scores_to_keep2);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(lines_step3);
  vdelete(orientation_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: previous stuff, plus line matching, for top step1_keep views.
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews20()
{
  const long step1_keep = 10000;
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (extended_cutoffs == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();

  double chamfer_index_weight = 0.33;
  double chamfer1_weight = 0.33;
  double orientation_weight = .33;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer_index_scores, 107328);
  vdelete(blurred_gray);

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_keep);
  vMatchScores * chamfer_index2 = chamfer_index_scores->Select(scores_to_keep);
  vMatchScores * chamfer1_2 = chamfer1_scores->Select(scores_to_keep);
  vMatchScores * orientation2 = orientation_scores->Select(scores_to_keep);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);
  vdelete(orientation_scores);

  // Get line scores.
  vector<long> database_frames(step1_keep);
  vector<long> database_views(step1_keep);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_keep; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid,
                                              all_model_lines, &database_frames,
                                              &database_views, grid_fraction);
  vMatchScores * line_scores = line_match_stats->MatchScores();
  vMatchScores * new_scores =
    vMatchScores::CombineLogs(4, chamfer_index2, (double) .25, 
                               chamfer1_2, (double) .25,
                               line_scores, (double) .25,
                               orientation2, (double) .25);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_index2);
  vdelete(chamfer1_2);
  vdelete(orientation2);
  vdelete(line_scores);
  vdelete(new_scores);
  
  vdelete(edge_image);
  vdelete(test_image);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer1, fast orientation histograms
long vInterpreter2::RankViews21()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  double chamfer1_weight = 0.5;
  double orientation_weight = 0.5;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanksFast(training_edge_histograms, &histogram, 
                               chamfer1_scores, 107328);
  vdelete(blurred_gray);

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vdelete(match_scores);
  match_scores = 
    chamfer1_scores->CombineLogs(2, chamfer1_scores, chamfer1_weight,
                                 orientation_scores, orientation_weight);

  vdelete(chamfer1_scores);
  vdelete(orientation_scores);
  vdelete(edge_image);
  vdelete(test_image);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer1,
long vInterpreter2::RankViews22()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;
  if (hand_edge_image == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  grayscale_image * edge_image = hand_edge_image;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vdelete(match_scores);
  match_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1
long vInterpreter2::RankViews23()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (chamfer_index_im == 0) return 0;
  if (im_reference_images == 0) return 0;

//  if (extended_cutoffs == 0) return 0;
//  if (training_edge_histograms == 0)
//  {
//    vPrint("No training edge histograms have been loaded.\n");
//    return 0;
//  }
  if (all_model_lines == 0) return 0;
  if (hand_edge_image == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  grayscale_image * edge_image = hand_edge_image;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  

  // Now combine all step-1 scores, and filter out low ranking frames.
  vdelete(match_scores);
  match_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, (double) 0.5,
                                 chamfer1_scores, (double) 0.5);

  vdelete(chamfer1_scores);
  vdelete(chamfer_index_scores);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);

  SetCurrentInput(test_input_index);
  ShowImage(original_hand_image->ToRGBCopy());
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, fast orientation histograms
// Step2: previous stuff, plus line matching, for top step1_keep views.
// Here, in contrast to previous RankViews functions, we read 
// the input hand image and the edge image from the image buffer
// of the current input.
long vInterpreter2::RankViews24()
{
  const long step1_keep = 10000;
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (extended_cutoffs == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();

  double chamfer_index_weight = 0.5;
  double chamfer1_weight = 0.5;

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, chamfer_index_weight,
                                 chamfer1_scores, chamfer1_weight);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_keep);
  vMatchScores * chamfer_index2 = chamfer_index_scores->Select(scores_to_keep);
  vMatchScores * chamfer1_2 = chamfer1_scores->Select(scores_to_keep);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);

  // Get line scores.
  vector<long> database_frames(step1_keep);
  vector<long> database_views(step1_keep);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_keep; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid,
                                              all_model_lines, &database_frames,
                                              &database_views, grid_fraction);
  vMatchScores * line_scores = line_match_stats->MatchScores();
  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_index2, (double) .33, 
                               chamfer1_2, (double) .33,
                               line_scores, (double) .33);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_index2);
  vdelete(chamfer1_2);
  vdelete(line_scores);
  vdelete(new_scores);
  
  vdelete(edge_image);
  vdelete(test_image);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_im_index, chamfer1, line scores
long vInterpreter2::RankViews25()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  DOLVS_Stats2();
  ShowImage(test_image->ToGrayCopy());

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats2(existential_DOLVS, line_grid,
                                              all_model_lines, grid_fraction);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Now combine all  scores, 
  vdelete(match_scores);
  match_scores = 
    chamfer1_scores->CombineLogs(3, chamfer_index_scores, (double) .33,
                                 chamfer1_scores, (double) .33,
                                 line_scores, (double) .33);

  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer1_scores);
  vdelete(chamfer_index_scores);
  vdelete(line_scores);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Here we assume that step1 has already been done, and there
// are match scores for that.
// Step2: chamfer, line matching, orientation histograms
// We also assume DOLVS_Stats2 has already been called.
long vInterpreter2::RankViews26()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;
  if (match_scores == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (line_grid == 0) return 0;
  if (hand_edge_image == 0) return 0;
  if (right_hand_image == 0) return 0;

  vMatchScores * step1_scores = match_scores;
  match_scores = 0;

  long test_input_index = current_input_index;
  grayscale_image * edge_image = hand_edge_image;

  // Keep the highest ranking scores.
  vMatchScores * scores_to_keep = step1_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid,
                                              all_model_lines,
                                              &database_frames, &database_views, 
                                              0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(right_hand_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vdelete(blurred_gray);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(3, chamfer_scores, (double) .33, 
                               line_scores, (double) .33,
                               orientation_scores, (double) .33);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      step1_scores);
  match_scores = final_scores;

  vdelete(step1_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_scores);
  vdelete(line_scores);
  vdelete(orientation_scores);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);

  SetCurrentInput(test_input_index);
  ShowImage(original_hand_image->ToRGBCopy());
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Here we assume that step1 has already been done, and there
// are match scores for that.
// Step2: chamfer, orientation histograms
long vInterpreter2::RankViews27()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;
  if (match_scores == 0) return 0;
  if (hand_edge_image == 0) return 0;
  if (right_hand_image == 0) return 0;

  vMatchScores * step1_scores = match_scores;
  match_scores = 0;

  long test_input_index = current_input_index;
  grayscale_image * edge_image = hand_edge_image;

  // Keep the highest ranking scores.
  vMatchScores * scores_to_keep = step1_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(right_hand_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vdelete(blurred_gray);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(2, chamfer_scores, (double) .50, 
                               orientation_scores, (double) .50);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      step1_scores);
  match_scores = final_scores;

  vdelete(step1_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Here we assume that step1 has already been done, and there
// are match scores for that.
// Step2: chamfer, 
long vInterpreter2::RankViews28()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;
  if (match_scores == 0) return 0;
  if (hand_edge_image == 0) return 0;
  if (right_hand_image == 0) return 0;

  vMatchScores * step1_scores = match_scores;
  match_scores = 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  grayscale_image * edge_image = hand_edge_image;

  // Keep the highest ranking scores.
  vMatchScores * scores_to_keep = step1_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vMatchScores * final_scores = vMatchScores::Merge(chamfer_scores, 
                                                      step1_scores);
  match_scores = final_scores;

  vdelete(step1_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_scores);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Here we assume that step1 has already been done, and there
// are match scores for that.
// Step2: chamfer, line matching
// We also assume DOLVS_Stats2 has already been called.
long vInterpreter2::RankViews29()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;
  if (match_scores == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (line_grid == 0) return 0;
  vMatchScores * step1_scores = match_scores;
  match_scores = 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  // Keep the highest ranking scores.
  vMatchScores * scores_to_keep = step1_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid,
                                              all_model_lines,
                                              &database_frames, &database_views, 
                                              0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();

  vMatchScores * new_scores =
    vMatchScores::CombineLogs(2, chamfer_scores, (double) .50, 
                               line_scores, (double) .50);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      step1_scores);
  match_scores = final_scores;

  vdelete(step1_scores);
  vdelete(new_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  vdelete(chamfer_scores);
  vdelete(line_scores);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// Like RankViews, but here we also use scores from line matching.
long vInterpreter2::RankViews30()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();
  DOLVS_Stats2();

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  

  // Get finger scores and moment scores.
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Now combine all step1 scores
  vMatchScores * prefilter_scores = 
    finger_scores->CombineLogs(4, chamfer_index_scores, (double) 0.25,
                               chamfer1_scores, (double) 0.25, 
                               finger_scores, (double) 0.25,
                               moment_scores, (double) 0.25);
  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);
  vMatchScores * filtered_finger_scores = finger_scores->Select(scores_to_keep);
  vMatchScores * filtered_moment_scores = moment_scores->Select(scores_to_keep);
//  vMatchScores * filtered_index_scores = chamfer_index_scores->Select(scores_to_keep);
  vdelete(finger_scores);
  vdelete(moment_scores);
  vdelete(chamfer_index_scores);
  vdelete(chamfer1_scores);

  // Get chamfer scores
  char training_filename[1000];
  sprintf(training_filename, "%s%s", sequences_directory,
          "\\glove_data\\training\\training7_edges.rle");
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           scores_to_keep, step1_views);
  vdelete(blurred_gray);

  vector<long> database_frames(step1_views);
  vector<long> database_views(step1_views);
  vArray(vMatchScore) score_sorted = scores_to_keep->GetScores();
  long i;
  for (i = 0; i < step1_views; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats3(existential_DOLVS, line_grid,
                                              all_model_lines,
                                              &database_frames, &database_views,
                                              0.05);
  vMatchScores * line_scores = line_match_stats->MatchScores();


  vMatchScores * new_scores =
    vMatchScores::CombineLogs(5, chamfer_scores, (double) .2, 
                               orientation_scores, (double) .2,
                               filtered_finger_scores, (double) .2, 
                               filtered_moment_scores, (double) .2,
                               line_scores, (double) .2);
  vMatchScores * final_scores = vMatchScores::Merge(new_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(new_scores);
  vdelete(filtered_finger_scores);
  vdelete(filtered_moment_scores);
  vdelete(scores_to_keep);
  vdelete(test_image);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);
  vdelete(line_scores);
  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  ShowImage(right_hand_image->ToRGBCopy());
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}



// Step0: palm scores
// Step1: chamfer_im_index, chamfer1
long vInterpreter2::RankViews31()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;
  if (extended_cutoffs == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }
  if (all_model_lines == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  // step 0, palm scores.
  vMatchScores * palm_scores = vPalmCutoffScores(extended_cutoffs,
                                                   &centered_palm_cutoff);
  vMatchScores * step0_scores = palm_scores->KeepHighest(step0_views);

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferIMIndexScores();
  vMatchScores * chamfer_index_scores = match_scores;
  match_scores = 0;

  ShowImage(edge_image->ToGrayCopy());
  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * edge_image2 = (grayscale_image *) input->current_image;
  vMatchScores * chamfer1_scores = vOneWayChamferRanks2(edge_image2, point_sequence);
  

  // Now combine all step-1 scores, and filter out low ranking frames.
  vMatchScores * prefilter_scores = 
    chamfer1_scores->CombineLogs(2, chamfer_index_scores, (double) 0.5,
                                 chamfer1_scores, (double) 0.5);
  vMatchScores * selected_prefilter = prefilter_scores->Select(step0_scores);
  vdelete(match_scores);
  match_scores = vMatchScores::Merge(selected_prefilter, palm_scores);

  vdelete(palm_scores);
  vdelete(step0_scores);
  vdelete(chamfer1_scores);
  vdelete(chamfer_index_scores);
  vdelete(prefilter_scores);
  vdelete(selected_prefilter);

  vdelete(edge_image);
  vdelete(test_image);

  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


// We use chamfer index scores only.
long vInterpreter2::RankViews32()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (hand_edge_image == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  grayscale_image * edge_image = hand_edge_image->ToGrayCopy();
  ShowImage(edge_image);

  long success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;

  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();

  return 1;
}


// Step1: chamfer_index.
// Step2: chamfer
long vInterpreter2::RankViews33()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (point_sequence == 0) return 0;

  long test_input_index = current_input_index;
  // Get the edge image and original hand image.
  ShowPreviousImage();
  grayscale_image * edge_image = input->current_image->ToGrayCopy();
  ShowNextImage();
  grayscale_image * test_image = input->current_image->ToGrayCopy();

  long success;
  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image->ToGrayCopy());
  success = ChamferDistances();
  if (success <= 0) return success;
  success = MultiPickScores0();
  if (success <= 0) return success;
  vMatchScores * prefilter_scores = match_scores;
  match_scores = 0;

  vMatchScores * scores_to_keep = prefilter_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  scores_to_keep, step1_views);
  vMatchScores * final_scores = vMatchScores::Merge(chamfer_scores, 
                                                      prefilter_scores);
  match_scores = final_scores;

  vdelete(prefilter_scores);
  vdelete(chamfer_scores);
  vdelete(scores_to_keep);
  vdelete(edge_image);
  vdelete(test_image);
  long training_index = database_input;
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


long vInterpreter2::ShowTopTenMatches()
{
  return ShowTenMatches(0);
}


long vInterpreter2::ShowTenMatches0()
{
  long start = 0;
  vPrint("Enter starting rank: ");
  vScan("%li", &start);
  return ShowTenMatches(start);
}


long vInterpreter2::ShowTenMatches(long start)
{
  if (input == 0) return 0;
  if (match_scores == 0) return 0;
  long rows = 256;
  rows = rows - 40;

  color_image * match_image = new color_image(2*rows, 5*rows);
  long row, col; 
  long index = start;
  long success = 1;
  for (row = 0; row < 2; row++)
  {
    for (col = 0; col < 5; col++)
    {
      vMatchScore score = match_scores->GetScore(index);
      if (score.is_valid == 0)
      {
        vPrint("Unable to get score of rank %li\n", index);
        success = 1;
      }
      long frame = score.frame;
      long view = score.view;
      color_image * rotated = ReadDatabaseFrame(frame, view);
      if (rotated == 0) 
      {
        success = 0;
        break;
      }
      if (left_hand == 1)
      {
        color_image * flipped = (color_image *) wrma(rotated).TopBottom_a();
        vdelete(rotated);
        rotated = flipped;
      }
      vCopyRectangle2(rotated, match_image, 20, 20, rows, rows, row * rows, col * rows);
      vdelete(rotated);
      index++;
    }
  }

  ten_matches_index = start;
  vPrint("\nten matches starting at %li\n", ten_matches_index);
  ShowImage(match_image);
  return 1;
}


long vInterpreter2::ShowNextTenMatches()
{
  ten_matches_index += 10;
  return ShowTenMatches(ten_matches_index);
}


long vInterpreter2::ShowPreviousTenMatches()
{
  ten_matches_index -= 10;
  if (ten_matches_index < 0)
  {
    ten_matches_index = 0;
  }
  return ShowTenMatches(ten_matches_index);
}


long vInterpreter2::ViewParams0()
{
  long frame, rotation;
  frame = rotation = 0;
  vPrint("Enter frame and rotation: ");
  vScan("%li %li", &frame, &rotation);
  return ViewParams(frame, rotation);
}


long vInterpreter2::ViewParams(long frame, long rotation)
{
  vFrameViewFinder view_finder;
  double x, y, z;
  long category;
  x = y = z = category = 0;
  view_finder.GetViewParams(frame, rotation, &category, &x, &y, &z);
  vPrint("%li class, (x, y, z) = (%.2lf, %.2lf, %.2lf)\n\n", category, x, y, z);
  JumpToFrame(frame);
  RotateRight(rotation);
  return 1;
}

  
long vInterpreter2::SetStep0Views0()
{
  long in_step0_views = 0;
  vPrint("Input value for step0_views: ");
  vScan("%li", &in_step0_views);
  return SetStep0Views(in_step0_views);
}


long vInterpreter2::SetStep0Views(long in_step0_views)
{
  if (in_step0_views <= 0) 
  {
    vPrint("%li is a bad value for step0_views\n", in_step0_views);
    return 0;
  }
  step0_views = in_step0_views;
  return 1;
}

  
long vInterpreter2::SetStep1Views0()
{
  long in_step1_views = 0;
  vPrint("Input value for step1_views: ");
  vScan("%li", &in_step1_views);
  return SetStep1Views(in_step1_views);
}


long vInterpreter2::SetStep1Views(long in_step1_views)
{
  if (in_step1_views <= 0) 
  {
    vPrint("%li is a bad value for step1_views\n", in_step1_views);
    return 0;
  }
  step1_views = in_step1_views;
  return 1;
}

  
long vInterpreter2::IsCameraInitialized()
{
  long result = camera.IsInitialized();
  return result;
}


long vInterpreter2::IsCameraRunning()
{
  long result = camera.IsRunning();
  return result;
}


long vInterpreter2::CheckCamera()
{
  long initialized = IsCameraInitialized();
  long running = IsCameraRunning();
  if (initialized == 0)
  {
    vPrint("Camera is not initialized\n");
  }
  else
  {
    if (running == 0)
    {
      vPrint("Camera is initialized but not running\n");
    }
    else
    {
      vPrint("Camera is initialized and running\n");
    }
  }
  return 1;
}


void * S_CameraFunction(void * arg)
{
  vInterpreter2 * interpreter = (vInterpreter2 *) arg;
//  static long frame = 0;
//  vPrint("frame = %li\r", frame);T
//  frame++;
  interpreter->GrabFrame();
  return 0;
}


long vInterpreter2::StartCamera()
{
  if (IsCameraRunning() != 0) return 1;
  if (IsCameraInitialized() == 0)
  {
	  camera.Initialize(320, 240, 0, window, 0);
  }

  camera.SetCameraFunction(S_CameraFunction);
  camera.SetCameraArgument(this);
  camera.Start();

  if (camera_input == 0)
  {
    class_sequence_information * info = new class_sequence_information(default_buffer_size);
    info->sequence = 0;
    info->path_name = vCopyString("camera input");
    info->simple_name = vCopyString("camera input");
    info->number_of_frames = 0;
    info->is_camera_input = 1;
    inputs.push_back(info);
    camera_input = info;

    number_of_inputs++;
    SetCurrentInput(number_of_inputs - 1);
  }
  else
  { 
    long camera_input_index = CameraInputIndex();
    SetCurrentInput(camera_input_index);
  }

  vPrint("Camera started\n");
  return 1;
}


long vInterpreter2::StopCamera()
{
  if (IsCameraRunning() == 0) return 0;
//  long camera_input_index = CameraInputIndex();
//  camera_input->is_camera_input = 0;
//  RemoveInput(camera_input_index);
//  camera_input = 0;
  camera.Stop();
  camera.SetCameraFunction(0);
  camera.SetCameraArgument(0);
  vPrint("Camera stopped\n");
  return 1;
}


long vInterpreter2::GrabFrame()
{
  if (camera_input == 0) return 0;
  // Get the current frame from the camera
  CImage & camera_frame = camera.GetFrame();

  // Convert the frame into an IplImage.
	IplImage* ipl_frame = camera_frame.GetImage();
  color_image * the_image = vIplTocolor_image(ipl_frame);
  long current_index = current_input_index;
  long camera_input_index = CameraInputIndex();
  if (current_index != camera_input_index)
  {
    SetCurrentInput(camera_input_index);
  }

  // If this is the first camera image that we show, set
  // it up to display at the bottom right corner
  if (camera_input->display_left == 0)
  {
    long window_width = window->Width();
    long window_height = window->Height();
    long image_width = the_image->Cols();
    long image_height = the_image->Rows();
    camera_input->display_left = window_width - image_width;
    camera_input->display_top = window_height - image_height;
  }
  
  ShowImage(the_image);
  if (current_index != camera_input_index)
  {
    SetCurrentInput(current_index);
  }

  return 1;
}


long vInterpreter2::SetCameraFormat()
{
  if (IsCameraInitialized() == 0) return 0;
  camera.VideoFormatDlg();
  return 1;
}


long vInterpreter2::SetCameraOptions()
{
  if (IsCameraInitialized() == 0) return 0;
  camera.VideoSourceDlg();
  return 1;
}


long vInterpreter2::CameraInputIndex()
{
  if (camera_input == 0) return -1;
  long i;
  long index = -1;
  for (i = 0; i < number_of_inputs; i++)
  {
    class_sequence_information * info = (class_sequence_information *) inputs[i];
    if (info == camera_input) 
    {
      index = i;
      break;
    }
  }
  if (index == -1)
  {
    // If we get here, something is wrong. If the camera input is not NULL,
    // then it should have been one of the entries in inputs.
    exit_error("Error, camera input is not null and not part of inputs.\n");
  }

  return index;
}


v3dMatrix<long> * vInterpreter2::CameraHand()
{
  if (input == 0) return 0;
  // Get input image
  color_image * the_image = input->current_image->ToRGBCopy();
  grayscale_image * gray = the_image->ToGrayCopy();
  // Get skin detection results
  vInterpreter1::InitializeSkinData();
  grayscale_image * skin = skin_table->DetectSkin(the_image);
  // Get undirected edges
  color_image * edges_color = (color_image *) vUndirectedEdges(the_image);
  grayscale_image * edges = edges_color->ToGrayCopy();
  vdelete(edges_color);
  long intensity_lo = 60;
  long intensity_hi = 100;
  long redness_lo = -50;
  long redness_hi = 30;
  long skin_lo = 0;
  long skin_hi = 20;
  long edge_lo = 0;
  long edge_hi = 12;

  long intensity_factor = 10000 / (intensity_hi - intensity_lo + 1);
  long redness_factor = 10000 / (redness_hi - redness_lo + 1);
  long skin_factor = 10000 / (skin_hi - skin_lo + 1);
  long edge_factor = 4000 / (edge_hi - edge_lo + 1);

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long size = rows * cols;
  v3dMatrix<long> * result_matrix = new v3dMatrix<long>(rows, cols, 1);
  vArray(long) result = result_matrix->Matrix();

  // Add intensity scores
  vArray(uchar) intensity = gray->Matrix();
  long i;
  for (i = 0; i < size; i++)
  {
    long current = intensity[i];
    if (current < intensity_lo) current = intensity_lo;
    if (current > intensity_hi) current = intensity_hi;
    current = current - intensity_lo;
    current = current * intensity_factor;
    result[i] = current;
  }

  // Add redness scores;
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) B = the_image->GetB();
  for (i = 0; i < size; i++)
  {
    long current = R[i] - B[i];
    if (current < redness_lo) current = redness_lo;
    if (current > redness_hi) current = redness_hi;
    current = current - redness_lo;
    current = current * redness_factor;
    result[i] = result[i] + current;
  }

  // Add skin scores
  intensity = skin->Matrix();
  for (i = 0; i < size; i++)
  {
    long current = intensity[i];
    if (current < skin_lo) current = skin_lo;
    if (current > skin_hi) current = skin_hi;
    current = current - skin_lo;
    current = current * skin_factor;
    result[i] = result[i] + current;
  }

  // Add edge scores
  intensity = edges->Matrix();
  for (i = 0; i < size; i++)
  {
    long current = intensity[i];
    if (current < edge_lo) current = edge_lo;
    if (current > edge_hi) current = edge_hi;
    current = current - edge_lo;
    current = current * edge_factor;
    result[i] = result[i] + current;
  }

  vdelete(the_image);
  vdelete(gray);
  vdelete(skin);
  vdelete(edges);

  vWriteBorderValue3(result_matrix, (long) 0, (long) 15);
  return result_matrix;
}



long vInterpreter2::ShowCameraHand()
{
  if (input == 0) return 0;
  color_image * input_image = input->current_image->ToRGBCopy();
  v3dMatrix<long> * camera_hand = CameraHand();
  v3dMatrix<uchar> * thresholded = camera_hand->threshold(15000);
  ShowImage(camera_hand);

  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = thresholded->ConnectedComponents(&labels);
  std::sort(labels.begin(), labels.end(), LabelMore());
  Label biggest = labels[0];
  if ((biggest.id == 0) && (labels.size() > 1))
  {
    biggest = labels[1];
  }
  v3dMatrix<uchar> * selected = vSelectValue(connected, biggest.id);
  vFillUpHoles2(selected, selected);
  long rows = input_image->Rows();
  long cols = input_image->Cols();
  color_image * segmented = new color_image(rows, cols);
  vArray(uchar) source_R = input_image->GetR();
  vArray(uchar) source_G = input_image->GetG();
  vArray(uchar) source_B = input_image->GetB();

  vArray(uchar) target_R = segmented->GetR();
  vArray(uchar) target_G = segmented->GetG();
  vArray(uchar) target_B = segmented->GetB();

  vArray(uchar) selection = selected->Matrix();
  long size = rows * cols;
  long i;
  for (i = 0; i < size; i++)
  {
    if (selection[i] > 0)
    {
      target_R[i] = source_R[i];
      target_G[i] = source_G[i];
      target_B[i] = source_B[i];
    }
    else
    {
      target_R[i] = target_G[i] = target_B[i] = 0;
    }
  }

  ShowImage(segmented);
  //KeepPalm();
  vdelete(connected);
  vdelete(selected);
  vdelete(thresholded);
  vdelete(input_image);
  
  return 1;
}


long vInterpreter2::ShowCameraHand2()
{
  if (input == 0) return 0;
  color_image * input_image = input->current_image->ToRGBCopy();
  v3dMatrix<long> * camera_hand = CameraHand();
  v3dMatrix<uchar> * thresholded = camera_hand->threshold(15000);
  ShowImage(camera_hand);
  
  color_image * hand_image = vRefineHandSegmentation(thresholded, input_image);
  ShowImage(right_hand_image);
  vdelete(input_image);
  vdelete(thresholded);
  return 1;
}


long vInterpreter2::SegmentHand_A()
{
  if (input == 0) return 0;
  if (input->current_image->Rows() < 40) return 0;
  if (input->current_image->Cols() < 40) return 0;
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * gray = color->ToGray();

  // Blur the input and get undirected edge values
  v3dMatrix<double> * blurred = vBlur(gray);
  vSmoothBorders(blurred);
  v3dMatrix<double> * edges = vUndirectedEdges(blurred);
  vSmoothBorders(edges);

  // We are now looking for a connected component that has a 
  // large overlap with the background (which is assumed to
  // be uniform and non-textured). That should be 
  // the largest connected component found after we threshold
  // the undirected edges (there should be very few edges
  // with value > 2 in the background). 
  v3dMatrix<uchar> * thresholded1 = edges->threshold(2);
  vInverseImage(thresholded1);
  vdelete(edges);
  vArray2(uchar) gray2 = thresholded1->Matrix2(0);

  vector<Label> labels;
  v3dMatrix<long> * components = thresholded1->ConnectedComponents(&labels);
  // image 0
  ShowImage(components);
  vdelete(thresholded1);

  std::sort(labels.begin(), labels.end(), LabelMore());

  // The first approximation for a background component should
  // be the largest connected component. However, we should also
  // make sure that that component corresponds to weak edges
  // (value 0 in the thresholded image).
  long background_index = 0;
  long number_of_labels = labels.size();
  while((background_index < number_of_labels - 1) &&
        (labels[background_index].value != 0))
  {
    background_index++;
  }
  Label background_label = labels[0];
  v3dMatrix<uchar> * background1 = vSelectValue(components, background_label.id);
  // image 1
  ShowImage(background1);

  // Similarly, the first approximation for the foreground should
  // be the second largest connected component. Maybe I should check
  // to ensure that the intensity value in this component is not one (we do get
  // edges in the hand image, and many pixels have a gradient that exceeds
  // the threshold).
  Label foreground_label = labels[1];
  v3dMatrix<uchar> * foreground1 = vSelectValue(components, foreground_label.id);
  //vWriteBorderValue3(foreground1, (uchar) 0, 4);
  //vdelete(components);
  // image 2
  ShowImage(foreground1);

  // The second approximation for the foreground is the largest 
  // connected component of foreground1
  grayscale_image * foreground2 = vLargestForegroundComponent(foreground1);
  //vdelete(foreground1);
  // image3
  ShowImage(foreground2);

  // Now get the mean color of the current foreground and background estimates.
  vector<double> background_median, foreground_median;
  long success1 = vRegionRobustMean(color, background1, &background_median);
  long success2 = vRegionRobustMean(color, foreground2, &foreground_median);

  if ((success1 <= 0) || (success2 <= 0))
  {
    exit_error("vRegionRobustMean failed\n");
  }

  // background_candidates is the inverse image of the current estimate
  // for the foreground.
  grayscale_image * background_candidates = (grayscale_image *)
    vCopyImage(foreground2);
  vInverseImage(background_candidates);

  grayscale_image * foreground_candidates = (grayscale_image *)
    vCopyImage(background1);
  //vdelete(background1);
  vInverseImage(foreground_candidates);

  // Our next approximation for the foreground is the set of pixels
  // that are close to the mean foreground color, and similarly our
  // next approximation for the background is the set of pixels
  // that are close to the mean background color.
  double color_distance = 0;
  long band;
  for (band = 0; band < 3; band++)
  {
    color_distance += vAbs(foreground_median[band] - background_median[band]);
  }

  grayscale_image * background3 = vSelectColorRange5(color, background_candidates, 
                                                  &background_median,
                                                  &foreground_median,
                                                  color_distance / 4.0);
  // image4
  ShowImage(background3);

  grayscale_image * foreground3 = vSelectColorRange5(color, foreground_candidates, 
                                                  &foreground_median,
                                                  &background_median,
                                                  color_distance / 2.0);
  //vWriteBorderValue3(foreground3, (uchar) 0, 4);
  // image5
  ShowImage(foreground3);
  vdelete(background_candidates);
  vdelete(foreground_candidates);

  // Our next approximation for the foreground is the largest connected component
  // of the current approximation
  grayscale_image * foreground4a = vLargestForegroundComponent(foreground3);
  vFillUpHoles2(foreground4a, foreground4a);
  // image6
  ShowImage(foreground4a);

  grayscale_image * inverse_background = (grayscale_image *) vCopyImage(background3);
  vInverseImage(inverse_background);
  //vdelete(background3);
  grayscale_image * foreground4b = vLargestForegroundComponent(inverse_background);
  vFillUpHoles2(foreground4b, foreground4b);
  //vWriteBorderValue3(foreground4b, (uchar) 0, 4);
  // image7
  ShowImage(foreground4b);
  vdelete(inverse_background);

  // Now we have an approximation for the foreground and an approximation
  // for the background. However, note that the approximation for the
  // foreground gives an implicit alternative approximation for the background
  // (as the inverse image of the foreground) and vice versa. Therefore, in
  // practice we have two competing approximations for foreground and 
  // background. We reconcile them by pushing the foreground/background boundary
  // to where there are strong edges.
  grayscale_image * canny_edges = vCannyDetector(gray, 70);
  vSmoothBorders(canny_edges);
  grayscale_image * foreground5 = 
    vPushBoundaryToEdges(foreground4a, foreground4b, canny_edges);
  vdelete(canny_edges);
  vSmoothBorders(foreground5);
  vFillUpHoles2(foreground5, foreground5);
  // image8
  ShowImage(foreground5);

  grayscale_image * foreground6 = vLargestForegroundComponent(foreground5);
  // image9
  ShowImage(foreground6);

  //grayscale_image * foreground7 = vTrimBoundary(foreground6, 1);
  //ShowImage(foreground7);
  grayscale_image * foreground8 = vLargestForegroundComponent(foreground6);
  //vWriteBorderValue3(foreground8, (uchar) 0, 4);
  // image10
  ShowImage(foreground8);

  v3dMatrix<uchar> * hand_image = vSelectPixels(color, foreground8);
  // image11
  ShowImage(right_hand_image);

  vdelete(color);
  vdelete(gray);
  vdelete(blurred);
  return 1;
}


long vInterpreter2::ColorDistances0a()
{
  if (input == 0) return 0;
  vPrint("Enter row, col: ");
  long row = 0, col = 0;
  vScan("%li %li", &row, &col);
  return ColorDistances1a(row, col);
}


long vInterpreter2::ColorDistances0b()
{
  if (input == 0) return 0;
  vPrint("Enter r, g, b");
  long r = 0, g = 0, b = 0;
  vScan("%li %li %li", &r, &g, &b);
  return ColorDistances1b((uchar) r, (uchar) g, (uchar) b);
}


long vInterpreter2::ColorDistances1a(long row, long col)
{
  if (input == 0) return 0;
  if (row < 0) return 0;
  if (col < 0) return 0;
  if (row >= input->current_image->Rows()) return 0;
  if (col >= input->current_image->Cols()) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  vArray3(uchar) matrix = the_image->Matrix3();
  uchar r = matrix[0][row][col];
  uchar g = matrix[1][row][col];
  uchar b = matrix[2][row][col];

  v3dMatrix<long> * distances = vColorDistances(the_image, r, g, b);
  ShowImage(distances);
  vdelete(the_image);
  return 1;
}


long vInterpreter2::ColorDistances1b(uchar r, uchar g, uchar b)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();

  v3dMatrix<long> * distances = vColorDistances(the_image, r, g, b);
  ShowImage(distances);
  vdelete(the_image);
  return 1;
}


long vInterpreter2::ConnectedComponents8()
{
  if (input == 0) return 0;
  grayscale_image * current = input->current_image->ToGrayCopy();
  vector<Label> label_objects;
  long_matrix connected = 
    vConnectedComponents8(current, &label_objects);
  vdelete(current);
  ShowImage(connected.copy());
  color_image * the_image = vLabelsToImage(&connected, &label_objects);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::ShadowScore()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * blurred = vBlur(gray);
  v3dMatrix<long> * score = vShadowScore(blurred);
  ShowImage(score);
  vdelete(blurred);
  vdelete(gray);
  return 1;
}


long vInterpreter2::LocalShadowScore0()
{
  long row = 0;
  long col = 0;
  vPrint("Enter row and col: ");
  vScan("%li %li", &row, &col);
  return LocalShadowScore(row, col);
}


long vInterpreter2::LocalShadowScore(long row, long col)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * blurred = vBlur(gray);
  v3dMatrix<long> * score = vLocalShadowScore(blurred, row, col);
  ShowImage(score);
  vdelete(blurred);
  vdelete(gray);
  return 1;
}


long vInterpreter2::EdgeOrientations()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations(gray);
  ShowImage(orientations);
  color_image * color_orientations = vCircularIntensity(orientations, 0, M_PI, -1);
  ShowImage(color_orientations);
  vdelete(gray);
  return 1;
}


long vInterpreter2::EdgeOrientations2()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations2(gray);
  ShowImage(orientations);
  color_image * color_orientations = vCircularIntensity(orientations, 0, 2*M_PI, -1);
  ShowImage(color_orientations);
  vdelete(gray);
  return 1;
}


long vInterpreter2::NonMaximaSuppression()
{
  if (input == 0) return 0;
  v3dMatrix<double> * non_maxima = vNonMaximaSuppression(input->current_image);
  grayscale_image * binary = (grayscale_image *) non_maxima->threshold(1);
  vdelete(non_maxima);
  v3dMatrix<double> * gradients = vGradientNorm0(input->current_image);
  ShowImage(binary);
  v3dMatrix<double> * selected = vSelectPixels(gradients, binary);
  ShowImage(gradients);
  ShowImage(selected);
  return 1;
}

  
long vInterpreter2::IntensityMaxima()
{
  if (input == 0) return 0;
  v3dMatrix<double> * maxima = vIntensityMaxima(input->current_image);
  v3dMatrix<double> * double_image = new v3dMatrix<double>(input->current_image);

  grayscale_image * binary = (grayscale_image *) maxima->threshold(1);
  ShowImage(binary);

  v3dMatrix<double> * selected = vSelectPixels(double_image, binary);
  ShowImage(selected);

  vdelete(maxima);
  vdelete(double_image);

  return 1;
}

  
long vInterpreter2::GradientNorms()
{
  if (input == 0) return 0;
  v3dMatrix<double> * gradient_norms = vGradientNorm0(input->current_image);
  ShowImage(gradient_norms);
  return 1;
}


long vInterpreter2::PrincipalOrientation()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  double mean_row = 0;
  double mean_col = 0;
  double angle = vPrincipalOrientation(binary, &mean_row, &mean_col);
  vDrawLineB(binary, (long) mean_row, (long) mean_col, angle, 50, (uchar) 192);
  vDrawLineB(binary, (long) mean_row, (long) mean_col, angle + M_PI, 50, (uchar) 128);
  ShowImage(binary);
  return 1;
}


// This function is used for classification of the data set
// of PETS 2002. Here we know that the images belong to one
// of 10 classes, so we only need to find the best match 
// from 10 training frames
long vInterpreter2::PETS_Classify()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_edge_histograms == 0)
  {
    vPrint("No training edge histograms have been loaded.\n");
    return 0;
  }

  grayscale_image * test_image = input->current_image->ToGrayCopy();
  long test_input_index = current_input_index;

  // Prefilter using moments and fingers.
  double fingers_weight= 0.6;
  double moments_weight = 0.4;

  // First get finger scores and moment scores.
  long success;
  success = FingerScores();
  if (success <= 0) return success;
  vMatchScores * finger_scores = match_scores;
  match_scores = 0;

  // Before getting moment scores, redisplay the original test image
  SetCurrentInput(test_input_index);
  ShowImage(test_image->ToGrayCopy());
  success = PCAMomentScores();
  if (success <= 0) return success;
  vMatchScores * moment_scores = match_scores;
  match_scores = 0;

  // Now combine finger scores and moment scores, and filter out low
  // ranking frames.
  vMatchScores * pets_filter1= vMatchScores::PETS_Filter1();
  vMatchScores * filtered_finger_scores = finger_scores->Select(pets_filter1);
  vMatchScores * filtered_moment_scores = moment_scores->Select(pets_filter1);
  vdelete(finger_scores);
  vdelete(moment_scores);

  grayscale_image * edge_image = vCannyDetector(test_image, 70, 140, 3);
  
  // Get chamfer scores
  char training_filename[1000];
  sprintf(training_filename, "%s%s", sequences_directory,
          "\\glove_data\\training\\training7_edges.rle");
  vMatchScores * chamfer_scores = vChamferRanks(edge_image, training_filename, 
                                                  pets_filter1, pets_filter1->NumberOfScores());

  // Get edge orientation scores
  v3dMatrix<double> * blurred = vBlur(test_image);
  grayscale_image * blurred_gray = (grayscale_image *) blurred->ToGray();
  vEdgeHistogram histogram(96, blurred_gray, edge_image);
  vdelete(blurred);
  vMatchScores * orientation_scores = 
    vEdgeOrientationRanks(training_edge_histograms, &histogram, 
                           pets_filter1, pets_filter1->NumberOfScores());
  vdelete(blurred_gray);

  vMatchScores * final_scores =
    vMatchScores::CombineLogs(4, chamfer_scores, (double) .4, 
                               orientation_scores, (double) .1,
                               filtered_finger_scores, (double) .4, 
                               filtered_moment_scores, (double) .1);
  match_scores = final_scores;

  vdelete(filtered_finger_scores);
  vdelete(filtered_moment_scores);
  vdelete(pets_filter1);
  vdelete(test_image);
  vdelete(edge_image);
  vdelete(chamfer_scores);
  vdelete(orientation_scores);
  ShowTopTenMatches();
  long training_index = current_input_index;
  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);

  return 1;
}


long vInterpreter2::OldViewDistance0()
{
  double x1, y1, z1, x2, y2, z2;
  x1 = y1 = z1 = x2 = y2 = z2 = 0;
  vPrint("Enter x1, y1, z1, x2, y2, z2: ");
  vScan("%lf %lf %lf %lf %lf %lf", &x1, &y1, &z1, &x2, &y2, &z2);
  return OldViewDistance(x1, y1, z1, x2, y2, z2);
}


long vInterpreter2::OldViewDistance(double x1, double y1, double z1, 
                                  double x2, double y2, double z2)
{
  double distance = vOldViewDistance(x1, y1, z1, x2, y2, z2);
  vPrint("Distance = %.2lf\n\n", distance);
  return 1;
}


long vInterpreter2::OldViewDistance2()
{
  double x1, y1, z1, x2, y2, z2;
  x1 = y1 = z1 = x2 = y2 = z2 = 0;
  vPrint("Enter x1, y1, z1, x2, y2, z2: ");
  vScan("%lf %lf %lf %lf %lf %lf", &x1, &y1, &z1, &x2, &y2, &z2);
  double distance = vOldViewDistance2(x1, y1, z1, x2, y2, z2);
  vPrint("Distance = %.2lf\n\n", distance);
  return 1;
}


long vInterpreter2::SaveEdgeStatistics0()
{
  if (input == 0) return 0;
  vPrint("Enter original input, edge_input, spatial_subsampling,\n");
  vPrint("pixel1_bins, pixel2_bins, pixel1_start, pixel1_end,\n");
  vPrint("frames_to_process\n");
  long original_input = 0, edge_input = 0, spatial_subsampling = 0;
  long pixel1_bins = 0, pixel2_bins = 0, pixel1_start = 0, pixel1_end = 0;
  long frames_to_process = 0;
  vScan("%li %li %li %li %li %li %li %li", &original_input, &edge_input,
         &spatial_subsampling, &pixel1_bins, &pixel2_bins,
         &pixel1_start, &pixel1_end, &frames_to_process);
  return SaveEdgeStatistics(original_input, edge_input, spatial_subsampling,
                            pixel1_bins, pixel2_bins, 
                            pixel1_start, pixel1_end, frames_to_process);
}


long vInterpreter2::SaveEdgeStatistics(long original_input, long edge_input,
                                        long spatial_subsampling, long pixel1_bins,
                                        long pixel2_bins, long pixel1_start,
                                        long pixel1_end, long frames_to_process)
{
  if ((original_input < 0) || (original_input >= number_of_inputs) ||
      (edge_input < 0) || (edge_input >= number_of_inputs))
  {
    return 0;
  }

  class_sequence_information * original_info = (class_sequence_information *) inputs[original_input];
  class_sequence_information * edge_info = (class_sequence_information*) inputs[edge_input];
  vVideoFile * original_video = original_info->sequence;
  vVideoFile * edge_video = edge_info->sequence;
  if ((original_video == 0) || (edge_video == 0)) return 0;
  const char * original_file = original_video->GetFilename();
  const char * edge_file = edge_video->GetFilename();
  char * target_file = vReplaceEnding2(original_file, ".edge_stats");

  long success = vSaveEdgeStatistics(original_file, edge_file, target_file,
                                      spatial_subsampling, pixel1_bins, 
                                      pixel2_bins, pixel1_start, pixel1_end,
                                      frames_to_process);
  vPrint("success = %li\n", success);
  vdelete2(target_file);
  return success;
}


long vInterpreter2::LoadEdgeStatistics0()
{
  if (input == 0) return 0;
  vdelete(edge_statistics);
  const char * original_file = input->path_name;
  if (strcmp(original_file, "default input") == 0) return 0;
  char * edge_stat_file = vReplaceEnding2(original_file, ".edge_stats");
  edge_statistics = vEdgeStatistics::Load(edge_stat_file);
  if (edge_statistics == 0)
  {
    vPrint("Failed to load edge statistics from %s\n", edge_stat_file);
    vdelete2(edge_stat_file);
    return 0;
  }
  else
  {
    vPrint("Loaded edge statistics\n");
    vdelete2(edge_stat_file);
    return 1;
  }
}


long vInterpreter2::EdgeStatisticsVote0a()
{
  if (input == 0) return 0;
  long original_input = 0, edge_input = 0;
  vPrint("Enter original input and edge_input: ");
  vScan("%li %li", &original_input, &edge_input);
  return EdgeStatisticsVote(original_input, edge_input);
}


long vInterpreter2::EdgeStatisticsVote(long original_input, long edge_input)
{
  if ((original_input < 0) || (original_input >= number_of_inputs) ||
      (edge_input < 0) || (edge_input >= number_of_inputs)) 
  {
    return 0;
  }
  if (edge_statistics == 0) return 0;
  class_sequence_information * original = (class_sequence_information *) inputs[original_input];
  class_sequence_information * edge = (class_sequence_information *) inputs[edge_input];
  grayscale_image * original_image = original->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations(original_image);

  grayscale_image * edge_image = edge->current_image->ToGrayCopy();
  vMatrix<double> * votes = edge_statistics->Vote(orientations, edge_image);
  ShowImage(votes);
  vdelete(original_image);
  vdelete(edge_image);
  vdelete(orientations);
  return 1;
}


long vInterpreter2::EdgeStatisticsVote0b()
{
  if (input == 0) return 0;
  long original_input = 0, edge_input = 0;
  vPrint("Enter original input and edge_input: ");
  vScan("%li %li", &original_input, &edge_input);
  return EdgeStatisticsVote2(original_input, edge_input);
}


long vInterpreter2::EdgeStatisticsVote2(long original_input, 
                                         long edge_input)
{
  if ((original_input < 0) || (original_input >= number_of_inputs) ||
      (edge_input < 0) || (edge_input >= number_of_inputs)) 
  {
    return 0;
  }
  if (edge_statistics == 0) return 0;
  class_sequence_information * original = (class_sequence_information *) inputs[original_input];
  class_sequence_information * edge = (class_sequence_information *) inputs[edge_input];
  grayscale_image * original_image = original->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations(original_image);

  grayscale_image * edge_image = edge->current_image->ToGrayCopy();
  vMatrix<double> * votes = edge_statistics->Vote2(orientations, edge_image);
  ShowImage(votes);
  vdelete(original_image);
  vdelete(edge_image);
  vdelete(orientations);
  return 1;
}


long vInterpreter2::EdgeStatisticsVote0d()
{
  if (input == 0) return 0;
  long original_input = 0, edge_input = 0;
  vPrint("Enter original input and edge_input: ");
  vScan("%li %li", &original_input, &edge_input);
  return EdgeStatisticsVote4(original_input, edge_input);
}


long vInterpreter2::EdgeStatisticsVote4(long original_input, 
                                         long edge_input)
{
  if ((original_input < 0) || (original_input >= number_of_inputs) ||
      (edge_input < 0) || (edge_input >= number_of_inputs)) 
  {
    return 0;
  }
  if (edge_statistics == 0) return 0;
  class_sequence_information * original = (class_sequence_information *) inputs[original_input];
  class_sequence_information * edge = (class_sequence_information *) inputs[edge_input];
  grayscale_image * original_image = original->current_image->ToGrayCopy();
  v3dMatrix<double> * orientations = vEdgeOrientations(original_image);

  grayscale_image * edge_image = edge->current_image->ToGrayCopy();
  vMatrix<double> * votes = edge_statistics->Vote4(orientations, edge_image);
  ShowImage(votes);
  vdelete(original_image);
  vdelete(edge_image);
  vdelete(orientations);
  return 1;
}


long vInterpreter2::SegmentHand_C()
{
  if (input == 0) return 0;
  if (input->current_image->Rows() < 40) return 0;
  if (input->current_image->Cols() < 40) return 0;
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * gray = color->ToGray();
  grayscale_image * edges = vCannyDetector(gray, 100);
  ShowImage(edges);

  v3dMatrix<float> * dt = vCVDistanceTransform(edges);
  grayscale_image * foreground1 = (grayscale_image *) dt->threshold(3);
  vInverseImage(foreground1);
  vFillUpHoles2(foreground1, foreground1);
  grayscale_image * foreground2 = vLargestForegroundComponent(foreground1);
  ShowImage(foreground2);
  vdelete(dt);
  vdelete(foreground1);

  vdelete(color);
  vdelete(gray);
  return 1;
}


long vInterpreter2::PalmCutpoint()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();

  long row1, col1, row2, col2, row3, col3;
  row1 = col1 = row2 = col2 = row3 = col3 = 0;
  vPalmCutpoint(binary, &row1, &col1, &row2, &col2, &row3, &col3);
  vDrawLine1(binary, row1, col1, row2, col2, (uchar) 255);
  vDrawCross(binary, row3, col3, 2, 5, (uchar) 255);
  ShowImage(binary);
  return 1;
}


long vInterpreter2::KeepPalm()
{
  if (input == 0) return 0;
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * binary = input->current_image->ToGrayCopy();
  grayscale_image * palm = vKeepPalm(binary);
  v3dMatrix<uchar> * color_palm = vSelectPixels(color, palm);
  ShowImage(palm);
  ShowImage(color_palm);
  vdelete(color);
  vdelete(binary);
  return 1;
}


long vInterpreter2::PalmCutpoint2()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();

  long row1, col1, row2, col2, row3, col3;
  row1 = col1 = row2 = col2 = row3 = col3 = 0;
  vPalmCutpoint2(binary, &row1, &col1, &row2, &col2, &row3, &col3);
  vDrawLine1(binary, row1, col1, row2, col2, (uchar) 255);
  vDrawCross(binary, row3, col3, 2, 5, (uchar) 255);
  ShowImage(binary);
  return 1;
}


long vInterpreter2::KeepPalm2()
{
  if (input == 0) return 0;
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * binary = input->current_image->ToGrayCopy();
  grayscale_image * palm = vKeepPalm2(binary, &palm_cutoff);
  v3dMatrix<uchar> * color_palm = vSelectPixels(color, palm);
  ShowImage(palm);
  ShowImage(color_palm);
  vdelete(color);
  vdelete(binary);
  return 1;
}


long vInterpreter2::KeepPalm3()
{
  if (input == 0) 
  {
    return 0;
  }
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * binary = input->current_image->ToGrayCopy();
  v3dMatrix<double> * temp_values = vGradientNorm0(binary);
  v3dMatrix<float> * values = new v3dMatrix<float>(temp_values);
  vdelete(temp_values);

  grayscale_image * palm = vKeepPalm3(binary, values, &palm_cutoff);
  v3dMatrix<uchar> * color_palm = vSelectPixels(color, palm);
  ShowImage(palm);
  ShowImage(color_palm);
  vdelete(color);
  vdelete(binary);
  vdelete(values);
  return 1;
}


long vInterpreter2::KeepPalm4()
{
  if ((input == 0) || (input->sequence == 0))
  {
    return 0;
  }

  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * binary = input->current_image->ToGrayCopy();
  
  ShowCurrentFrame();
  v3dMatrix<double> * temp_values = ComputeMotion();
  v3dMatrix<float> * values2 = new v3dMatrix<float>(temp_values);
  v3dMatrix<float> * values = vSelectPixels(values2, binary);

  vdelete(values2);
  ShowPreviousImage();
  ShowPreviousImage();

  grayscale_image * palm = vKeepPalm3(binary, values, &palm_cutoff);
  v3dMatrix<uchar> * color_palm = vSelectPixels(color, palm);
  ShowImage(palm);
  ShowImage(color_palm);
  vdelete(color);
  vdelete(binary);
  vdelete(values);
  return 1;
}


long vInterpreter2::SegmentHand_B()
{
  if (input == 0) return 0;
  if (input->current_image->Rows() < 40) return 0;
  if (input->current_image->Cols() < 40) return 0;
  color_image * color = input->current_image->ToRGBCopy();
  grayscale_image * gray = color->ToGray();
  long rows = color->Rows();
  long cols = color->Cols();
  long size = rows * cols;

  // Get canny edges with threshold 40 and 80.
  grayscale_image * edges1 = vCannyDetector(gray, 40);
  vWriteBorderValue3(edges1, (uchar) 0, 1);
  ShowImage(edges1);
  grayscale_image * edges2 = vCannyDetector(gray, 80);
  vWriteBorderValue3(edges2, (uchar) 0, 5);
  ShowImage(edges2);

  // We keep all 8-connected components in edges1
  // for which at least one pixel shows up in edges2.
  // First we need to get the connected components
  vector<Label> labels1;
  long_matrix connected1 = 
    vConnectedComponents8(edges1, &labels1);
  // Second we have to figure out which components in 
  // edges1 have at least one pixel in edges2.
  // label_flags[i] will be 0 iff component i in
  // edges1 has no pixel in edges2.
  long number_of_labels1 = labels1.size();
  vArray(ushort) label_flags = vnew(ushort, number_of_labels1);
  long i;
  for (i = 0; i < number_of_labels1; i++)
  {
    label_flags[i] = 0;
  }

  vArray(uchar) edge_array1 = edges1->Matrix();
  vArray(uchar) edge_array2 = edges2->Matrix();
  vArray(long) connected_array1 = connected1.Matrix();
 
  // Figure out which connected components of edges1 should
  // be kept. 
  for (i = 0; i < size; i++)
  {
    if (edge_array1[i] == 0) continue;
    if (edge_array2[i] == 0) continue;
    long label1 = connected_array1[i];
    label_flags[label1] = 1;
  }

  // Create the binary image that will contain the surviving
  // components of edges1.
  grayscale_image * edges3 = new grayscale_image(rows, cols);
  vArray(uchar) edge_array3 = edges3->Matrix();

  for (i = 0; i < size; i++)
  {
    long label1 = connected_array1[i];
    if (label_flags[label1] != 0)
    {
      edge_array3[i] = 255;
    }
    else
    {
      edge_array3[i] = 0;
    }
  }
  ShowImage(edges3);
  vdelete2(label_flags);

  vector<long> point_rows, point_cols;
  vGetNonZeroPoints(edges3, &point_rows, &point_cols);
  vector<Label> label_objects3;
  long_matrix connected3 = 
    vConnectedComponents8(edges3, &label_objects3);
  ShowImage(connected3.copy());
  long number_of_labels = label_objects3.size();

  vdelete(color);
  vdelete(gray);
  return 1;
}


// Uses the current match scores to initialize hand tracker
long vInterpreter2::InitializeHandTracker()
{
  if (match_scores == 0) return 0;
  if (hand_tracker == 0) hand_tracker = new vHandShapeTracker();
  return hand_tracker->InitializeEstimates(&(vHandShapeObservation(match_scores))); 
}


long vInterpreter2::HandTrackerPropagate()
{
  return hand_tracker->Propagate();
}


// Uses the current match scores to update hand tracker
long vInterpreter2::UpdateHandTracker()
{
  if (match_scores == 0) return 0;
  return hand_tracker->ProcessObservation(&(vHandShapeObservation(match_scores)));
}

// Gets match scores based on the current tracker estimates.
long vInterpreter2::HandTrackerScores()
{
  vMatchScores * tracking_scores = hand_tracker->ParticleScores();
  if (tracking_scores == 0) return 0;
  vdelete(match_scores);
  match_scores = tracking_scores;
  UpdateMatchScores(input->current_frame);
  return 1;
}


// Save non-zero image points in a MATLAB-readable file
long vInterpreter2::SavePoints0()
{
  char name[1000];
  char file[1000];
  long number = 0;
  if (input == 0) return 0;
  vPrint("Enter number, name and file: ");
  vScan("%li %s %s", &number, name, file);
  return SavePoints(number, name, file);
}


long vInterpreter2::SavePoints(long number, const char * name, const char * file)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGray();
  vector<long> rows, cols;
  vGetNonZeroPoints(gray, &rows, &cols);
  if (gray != input->current_image) 
  {
    vdelete(gray);
  }
  long number_of_points = rows.size();
  vector<long> out_rows, out_cols;
  long success = vSamplePoints1(&rows, &cols, &out_rows, &out_cols, number);
  long result = 1;
  if (success <= 0)
  {
    vPrint("Failed to sample points\n");
    return 0;
  }

  char * save_file = vJoinPaths5(code_directory, "..", "matlab", 
                                  "shape_context", file);
  success = vSavePoints(&out_rows, &out_cols, name, save_file);
  if (success <= 0)
  {
    vPrint("Failed to save points into %s\n", save_file);
    result = 0;
  }
  vdelete2(save_file);
  return result;
}


long vInterpreter2::CannyThreshold0()
{
  long new_threshold = 0;
  vPrint("Enter new threshold: ");
  vScan("%li", &new_threshold);
  return CannyThreshold(new_threshold);
}


long vInterpreter2::CannyThreshold(long new_threshold)
{
  if ((new_threshold < 0) || (new_threshold >= 255))
  {
    vPrint("%li is not a valid threshold\n", new_threshold);
    return 0;
  }
  canny_threshold = new_threshold;
  return 1;
}


long vInterpreter2::SetGroundTruthView0()
{
  if (match_scores == 0) return 0;
  long frame, view;
  frame = view = 0;
  vPrint("Enter frame and view: ");
  vScan("%li %li", &frame, &view);
  return SetGroundTruthView(frame, view);
}


long vInterpreter2::SetGroundTruthView(long frame, long view)
{
  if (match_scores == 0) return 0;
  return match_scores->SetGroundTruthView(frame, view);
}


long vInterpreter2::MatchDistance0()
{
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  vPrint("Enter frame1, view1, frame2, view2: ");
  vScan("%li %li %li %li", &frame1, &view1, &frame2, &view2);
  return MatchDistance(frame1, view1, frame2, view2);
}


long vInterpreter2::MatchDistance(long frame1, long view1, long frame2, long view2)
{
  vFrameViewFinder view_finder;
  double x1, y1, z1, x2, y2, z2;
  long category1, category2;
  x1 = y1 = z1 = category1 = 0;
  x2 = y2 = z2 = category2 = 0;
  view_finder.GetViewParams(frame1, view1, &category1, &x1, &y1, &z1);
  view_finder.GetViewParams(frame2, view2, &category2, &x2, &y2, &z2);
  double distance = vOldViewDistance(x1, y1, z1, x2, y2, z2);
  vPrint("\nDistance = %lf\n", distance);
  return 1;
}


long vInterpreter2::SaveSequenceStats0()
{
  if (sequence_match_stats == 0) return 0;
  char buffer[1000];
  vPrint("Enter filename: ");
  vScan("%s", buffer);
  return SaveSequenceStats(buffer);
}


long vInterpreter2::SaveSequenceStats(const char * filename)
{
  if (sequence_match_stats == 0) return 0;
  char * directory = vJoinPaths3(sequences_directory, "glove_data", 
                                  "sequence_stats");
  vIncreaseNumber(directory);
  long number = vGetNumber(directory);
  char * number_string = vLongToString(number);
  char * new_filename = vMergeStrings3(filename, ".", number_string);
  char * complete_name = vJoinPaths(directory, new_filename);
  long result = sequence_match_stats->Save(complete_name);
  if (result <= 0)
  {
    vPrint("Failed to save sequence statistics into %s", complete_name);
  }
  else
  {
    vPrint("Saved stats in %s\n", new_filename);
  }
  vdelete2(seq_stats_file);
  seq_stats_file = new_filename;
  vdelete2(complete_name);
  vdelete2(directory);
  vdelete2(number_string);

  return result;
}


long vInterpreter2::LoadSequenceStats0()
{
  char buffer[1000];
  vPrint("Enter filename: ");
  vScan("%s", buffer);
  return LoadSequenceStats(buffer);
}


long vInterpreter2::LoadSequenceStats(const char * filename)
{
  vdelete(sequence_match_stats);
  sequence_match_stats = new vSequenceMatchStats();
  char * complete_name = vJoinPaths4(sequences_directory, "glove_data", 
                                      "sequence_stats", filename);
  long success = sequence_match_stats->Load(complete_name);
  vdelete2(complete_name);
  if (success == 0)
  {
    vdelete(sequence_match_stats);
    sequence_match_stats = 0;
    vPrint("Failed to load sequence match stats from %s", filename);
    return 0;
  }
  else
  {
    sequence_match_stats->SetGroundDistance((long) ground_truth_distance);
    vdelete2(seq_stats_file);
    seq_stats_file = vCopyString(filename);
    return 1;
  }
}


long  vInterpreter2::SetGroundTruthDistance(double in_ground_truth_distance)
{
  ground_truth_distance = in_ground_truth_distance;
  if (sequence_match_stats != 0)
  {
    sequence_match_stats->SetGroundDistance((long) ground_truth_distance);
  }
  if (match_scores == 0) return 1;

  ushort success = match_scores->SetGroundTruthDistance(in_ground_truth_distance);
  if (success == 0)
  {
    vPrint("Failed to set test ground truth distance to %lf\n", 
            in_ground_truth_distance);
    return 0;
  }
  else
  {
    vPrint("Set ground truth distance to %lf\n", 
            in_ground_truth_distance);
    return 1;
  }
}


long  vInterpreter2::SetGroundTruthDistance0()
{
  vPrint("Input ground truth distance threshold: ");
  double distance = 0;
  vScan("%lf", &distance);
  return SetGroundTruthDistance(distance);
}


long vInterpreter2::OldRotationDistance0()
{
  double theta1, x1, y1, theta2, x2, y2;
  theta1 = x1 = y1 = theta2 = x2 = y2 = 0;
  vPrint("\nEnter theta1, x1, y1, theta2, x2, y2:\n");
  vScan("%lf %lf %lf %lf %lf %lf", &theta1, &x1, &y1, &theta2, &x2, &y2);
  return OldRotationDistance(theta1, x1, y1, theta2, x2, y2);
}


long vInterpreter2::OldRotationDistance(double theta1, double x1, double y1,
                                         double theta2, double x2, double y2)
{
  double distance = vOldRotationDistance(theta1, x1, y1, theta2, x2, y2);
  vPrint("distance = %lf\n", distance);
  return 1;
}


long vInterpreter2::ChamferIndexScoresb0()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;

  long dimensions, times;
  dimensions = times = 0;
  vPrint("\nEnter dimensions and times:\n");
  vScan("%li %li", &dimensions, &times);
  return ChamferIndexScoresb(dimensions, times);
}

long vInterpreter2::ChamferIndexScoresb(long dimensions, long times)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if ((dimensions < 0) || (times < 0))
  {
    return 0;
  }

  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * edge_image = vCannyDetector(test_image, (uchar) canny_threshold);
  long test_input_index = current_input_index;

  // Before getting chamfer_index scores, display the edge image
  SetCurrentInput(test_input_index);
  ShowImage(edge_image);
  
  vArray(vMatchScores *) scores_array = vnew(vMatchScores *, times);
  vArray(double) weights_array = vnew(double, times);
  double weight = 1.0 / times;

  long i;
  for (i = 0; i < times; i++)
  {
    SetRandomDimensions(i * dimensions, dimensions);

    vdelete(test_embedding);
    class_sequence_information * info = (class_sequence_information *) inputs[2];
    vMatrix<double> distances = 
      vDatabasePicksChamfer(edge_image, (vRLE_Sequence *) info->sequence, 
                             reference_images, reference_start, 
                             embedding_dimensions, -1, -1);
    test_embedding = new v3dMatrix<float>(&distances);

    scores_array[i] = new vMatchScores(training_embeddings, test_embedding,
                                        reference_start, embedding_dimensions);

    if (scores_array[i] == 0) return 0;
    weights_array[i] = weight;
  }

  // Now combine finger scores, moment scores and chamfer_index scores, and filter out
  // ranking frames.
  vMatchScores * prefilter_scores = 
    vMatchScores::CombineLogs2(times, scores_array, weights_array);
  
  vdelete(match_scores);
  match_scores = prefilter_scores;

  for (i = 0; i < times; i++)
  {
    vdelete(scores_array[i]);
  }
  vdelete2(scores_array);
  vdelete2(weights_array);
  
  long training_index = 0;
  vdelete2(test_image);
  SetCurrentInput(test_input_index);
  browse_image(5);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(training_index);
  ShowTopTenMatches();

  return 1;
}


long vInterpreter2::CopyInput0()
{
  if (input == 0) return 0;
  long input_index = 0;
  vPrint("\nInput index:\n");
  vScan("%li", &input_index);
  return CopyInput(input_index);
}


long vInterpreter2::CopyInput(long input_index)
{
  if (input == 0) return 0;
  if ((input_index < 0) || (input_index >= number_of_inputs)) return 0;
  general_image * the_image = inputs[input_index]->current_image->Copy();
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::Overlay0()
{
  if (input == 0) return 0;
  long input1, input2;
  input1 = input2 = 0;
  vPrint("\nEnter input1 and input2:\n");
  vScan("%li %li", &input1, &input2);
  return Overlay(input1, input2);
}


long vInterpreter2::Overlay(long input1, long input2)
{
  if (input == 0) return 0;
  if ((input1 < 0) || (input1 >= number_of_inputs) ||
      (input2 < 0) || (input2 >= number_of_inputs))
  {
    return 0;
  }
  general_image * the_image1 = inputs[input1]->current_image;
  general_image * the_image2 = inputs[input2]->current_image;
  color_image * overlay = vOverlay0(the_image1, the_image2);
  ShowImage(overlay);
  return 1;
}


long vInterpreter2::Randomize0()
{
  srand(time(0));
  return 1;
}


long vInterpreter2::Randomize1()
{
  vPrint("\nEnter seed:\n");
  long seed = 0;
  vScan("%li", &seed);
  srand(seed);
  return 1;
}


// I should start commenting for CVPR 2003 from here.
long vInterpreter2::OrientedLines0()
{
  if (input == 0) return 0;
  vPrint("\nEnter orientation, length, threshold\n");
  long length, threshold;
  double orientation;
  orientation = length = threshold = 0;
  vScan("%lf %li %li", &orientation, &length, &threshold);
  return OrientedLines(orientation, length, threshold);
}


long vInterpreter2::OrientedLines(double orientation, long length, long threshold)
{
  if (input == 0) return 0;
  v3dMatrix<float> * distances = vCVDistanceTransform(input->current_image);
  v3dMatrix<long> * votes = vOrientedLines(distances, orientation, length, threshold);
  ShowImage(distances);
  ShowImage(votes);
  return 1;
}

long vInterpreter2::OrientedLines0b()
{
  if (input == 0) return 0;
  vPrint("\nEnter orientation, length, threshold\n");
  long length, threshold;
  double orientation;
  orientation = length = threshold = 0;
  vScan("%lf %li %li", &orientation, &length, &threshold);
  return OrientedLines2(orientation, length, threshold);
}


long vInterpreter2::OrientedLines2(double orientation, long length, long threshold)
{
  if (input == 0) return 0;
  vPrint("Enter times: ");
  v3dMatrix<float> * distances = vCVDistanceTransform(input->current_image);
  v3dMatrix<long> * votes = vOrientedLines2(distances, orientation, length, threshold);
  ShowImage(distances);
  ShowImage(votes);
  return 1;
}


long vInterpreter2::OrientedLines0c()
{
  if (input == 0) return 0;
  vPrint("\nEnter orientation, length, distance_threshold, angle_tolerance\n");
  long length, threshold;
  double orientation, tolerance;
  orientation = tolerance = (double) 0;
  length = threshold = 0;
  vScan("%lf %li %li %lf", &orientation, &length, &threshold, &tolerance);
  return OrientedLines3(orientation, length, threshold, tolerance);
}


long vInterpreter2::OrientedLines3(double orientation, long length, long 
                                    threshold, double tolerance)
{
  if (input == 0) return 0;
  grayscale_image * current = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(current, (uchar) canny_threshold);
  v3dMatrix<long> * votes = vOrientedLines_c(current, edges, orientation, length, 
                                                threshold, tolerance);
  ShowImage(edges);
  ShowImage(votes);
  vdelete(current);
  return 1;
}

long vInterpreter2::OrientedLines0d()
{
  if (input == 0) return 0;
  vPrint("\nEnter orientation, length, distance_threshold, angle_tolerance\n");
  long length, threshold;
  double orientation, tolerance;
  orientation = tolerance = (double) 0;
  length = threshold = 0;
  vScan("%lf %li %li %lf", &orientation, &length, &threshold, &tolerance);
  return OrientedLines4(orientation, length, threshold, tolerance);
}


long vInterpreter2::OrientedLines4(double orientation, long length, long 
                                    threshold, double tolerance)
{
  if (input == 0) return 0;
  grayscale_image * current = input->current_image->ToGrayCopy();
  v3dMatrix<double> * double_edges = vNonMaximaSuppression(current);
  grayscale_image * edges = double_edges->ToGray();
  vdelete(double_edges);
  v3dMatrix<long> * votes = vOrientedLines_c(current, edges, orientation, length, 
                                                threshold, tolerance);
  ShowImage(edges);
  ShowImage(votes);
  vdelete(current);
  return 1;
}

  
long vInterpreter2::DilateImage()
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGray();
  grayscale_image * dilated = vDilateImage(the_image, 1, 255);
  if (the_image != input->current_image) vdelete(the_image);
  ShowImage(dilated);
  return 1;
}


long vInterpreter2::LinkBoundaries()
{
  if (input == 0) return 0;
  if (number_of_inputs <= database_edge_input) return 0;
  if (number_of_inputs <= database_input) return 0;
  color_image * the_image = inputs[database_input]->current_image->ToRGB();
  grayscale_image * edges = inputs[database_edge_input]->current_image->ToGray();
  vArray(vector<long>) rows;
  vArray(vector<long>) cols;
  long success = vLinkBoundaries(the_image, edges, &rows, &cols);
  color_image * boundaries = vLinkBoundariesImage(the_image, rows, cols);
  if (the_image != input->current_image)
  {
    vdelete(the_image);
  }
  if (edges != inputs[2]->current_image)
  {
    vdelete(edges);
  }
  ShowImage(boundaries);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


long vInterpreter2::FingerContours()
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  grayscale_image * edges = inputs[2]->current_image->ToGrayCopy();
  vArray(vector<long>) rows;
  vArray(vector<long>) cols;
  long success = vLinkBoundaries(the_image, edges, &rows, &cols);
  color_image * boundaries = vLinkBoundariesImage(the_image, rows, cols);
  ShowImage(boundaries);
  vector<vFingerContour> contours = vFingerContours(the_image, rows, cols);

  vdelete(the_image);
  vdelete(edges);
  vdelete2(rows);
  vdelete2(cols);

  long i;
  for (i = 0; i < (long) contours.size(); i++)
  {
    grayscale_image * contour_image = contours[i].contour->ContourImage();
    ShowImage(contour_image);
    vdelete(contours[i].contour);
  }
  return 1;
}


// The assumption here is that the database edge input and the
// database input are displaying the same database hand pose.
// The user specifies a link id.
long vInterpreter2::ContourLineVotes()
{
  if (number_of_inputs < database_input) return 0;
  if (number_of_inputs < database_edge_input) return 0;
  if (input == 0) return 0;

  // Let the user specify the link id.
  long link = 0;
  vPrint("Enter link\n");
  vScan("%li\n", &link);
  link = vAbs(link);
  link = link % 17; // Make sure the link id is between 0 and 16.

  // Read the database image and edge image.
  color_image * the_image = inputs[database_input]->current_image->ToRGBCopy();
  grayscale_image * edges = inputs[database_edge_input]->current_image->ToGrayCopy();

  // Get the boundaries of each link. rows[i], cols[i] are vectors 
  // containing the coordinates of the boundary points for the i-th link.
  vArray(vector<long>) rows;
  vArray(vector<long>) cols;
  long success = vLinkBoundaries(the_image, edges, &rows, &cols);
  color_image * boundaries = vLinkBoundariesImage(the_image, rows, cols);
  ShowImage(boundaries);

  grayscale_image * contour_image = new grayscale_image(the_image->Rows(), the_image->Cols());
  vWriteValue(contour_image, (uchar) 0);
  vWriteValue2(contour_image, &(rows[link]), &(cols[link]), (uchar) 255);
  grayscale_image * dilated = vDilateImage(contour_image, 1, 255);
  ShowImage(contour_image);
  ShowImage(dilated);
  
  v3dMatrix<double> * votes = vContourLineVotes(dilated, &(rows[link]), &(cols[link]));
  ShowImage(votes);
  v3dMatrix<double> * pure_votes = vCopyBand2(votes, 0);
  ShowImage(pure_votes);

  long max_point_index, max_orientation_index;
  max_point_index = max_orientation_index = 0;
  double max = function_image_maximum3(pure_votes, &max_point_index, &max_orientation_index);
  vArray3(double) votes3 = votes->Matrix3();
  double max_orientation = votes3[1][max_point_index][max_orientation_index];
  double max_row = votes3[2][max_point_index][max_orientation_index];
  double max_col = votes3[3][max_point_index][max_orientation_index];
  vDrawLineB(the_image, (long) max_row, (long) max_col, max_orientation, 30, (uchar) 255);
  vDrawCross(the_image, (long) max_row, (long) max_col, 1, 5, (uchar) 255);
  vPrint("\nmax_votes = %.0lf, orientation = %lf\n", max, max_orientation);
  ShowImage(the_image);

  vdelete(edges);
  vdelete2(rows);
  vdelete2(cols);

  return 1;
}


// The assumption here is that the database edge input and the
// database input are displaying the same database hand pose.
// The user specifies a link id.
// We first display the boundaries of every link, then the 
// contour of the specified link, 
long vInterpreter2::FingerLines()
{
  if (number_of_inputs < database_input) return 0;
  if (number_of_inputs < database_edge_input) return 0;
  if (input == 0) return 0;
  long link = 0;
  vPrint("Enter link\n");
  vScan("%li\n", &link);
  link = vAbs(link);
  link = link % 17;
  // Get the image with the link ids.
  color_image * the_image = inputs[database_input]->current_image->ToRGBCopy();
  // Get the database edge image.
  grayscale_image * edges = inputs[database_edge_input]->current_image->ToGrayCopy();
  // rows[i][j] and cols[i][j] will be the location of the j-th point on the 
  // boundary of the i-th link.
  vArray(vector<long>) rows;
  vArray(vector<long>) cols;
  // Identify the boundaries of every link and display them.
  long success = vLinkBoundaries(the_image, edges, &rows, &cols);
  color_image * boundaries = vLinkBoundariesImage(the_image, rows, cols);
  ShowImage(boundaries);

  // Create and show an image of the contour of the specified link.
  grayscale_image * contour_image = new grayscale_image(the_image->Rows(), the_image->Cols());
  vWriteValue(contour_image, (uchar) 0);
  vWriteValue2(contour_image, &(rows[link]), &(cols[link]), (uchar) 255);
  grayscale_image * dilated = vDilateImage(contour_image, 1, 255);
  ShowImage(contour_image);
  ShowImage(dilated);

  vector<vLineSegment> lines;
  vFingerLines(dilated, &(rows[link]), &(cols[link]), &lines);
  vDrawLine1(the_image, (long) lines[0].row1, (long) lines[0].col1, 
              (long) lines[0].row2, (long) lines[0].col2, (uchar) 255);
  vDrawCross(the_image, (long) lines[1].row1, (long) lines[1].col1, 1, 5, (uchar) 255);
  vDrawCross(the_image, (long) lines[1].row2, (long) lines[1].col2, 1, 5, (uchar) 255);
  ShowImage(the_image);
  
  vdelete(edges);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


long vInterpreter2::FingerLines2()
{
  if (input == 0) return 0;
  long link = 0;
  vPrint("Enter link\n");
  vScan("%li\n", &link);
  link = vAbs(link);
  link = link % 17;
  color_image * the_image = input->current_image->ToRGBCopy();
  grayscale_image * edges = inputs[2]->current_image->ToGrayCopy();
  vArray(vector<long>) rows;
  vArray(vector<long>) cols;
  long success = vLinkBoundaries(the_image, edges, &rows, &cols);
  color_image * boundaries = vLinkBoundariesImage(the_image, rows, cols);
  ShowImage(boundaries);

  grayscale_image * contour_image = new grayscale_image(the_image->Rows(), the_image->Cols());
  vWriteValue(contour_image, (uchar) 0);
  vWriteValue2(contour_image, &(rows[link]), &(cols[link]), (uchar) 255);
  grayscale_image * dilated = vDilateImage(contour_image, 1, 255);
  ShowImage(contour_image);
  ShowImage(dilated);

  vector<vLineSegment> lines;
  vFingerLines(dilated, &(rows[link]), &(cols[link]), &lines);
  vDrawLine1(the_image, (long) lines[0].row1, (long) lines[0].col1, 
              (long) lines[0].row2, (long) lines[0].col2, (uchar) 255);
  vDrawCross(the_image, (long) lines[1].row1, (long) lines[1].col1, 1, 5, (uchar) 255);
  vDrawCross(the_image, (long) lines[1].row2, (long) lines[1].col2, 1, 5, (uchar) 255);
  ShowImage(the_image);
  
  vdelete(edges);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


long vInterpreter2::SelectRange0()
{
  if (input == 0) return 0;
  long band;
  double low, high;
  band = 0;
  low = high = (double) 0;
  vPrint("\nEnter band, low and high\n");
  vScan("%li %lf %lf", &band, &low, &high);
  return SelectRange(band, low, high);
}


long vInterpreter2::SelectRange(long band, double low, double high)
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  general_image * selected = vSelectRange0(the_image, band, low, high);
  ShowImage(selected);
  return 1;
}


long vInterpreter2::LineEndpoints0()
{
  if (input == 0) return 0;
  long row, col;
  double orientation;
  row = col = 0;
  orientation = (double) 0;
  vPrint("\nEnter row, col, orientation\n");
  vScan("%li %li %lf", &row, &col, &orientation);
  return LineEndpoints(row, col, orientation);
}


long vInterpreter2::LineEndpoints(long row, long col, double orientation)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vLineSegment line = vProjectionEndpoints(the_image, row, col, orientation);
  vDrawLine1(the_image, (long) line.row1, (long) line.col1, (long) line.row2, (long) line.col2, (uchar) 255);
  vDrawCross(the_image, (long) line.row1, (long) line.col1, 2, 5, (uchar) 255);
  vDrawCross(the_image, (long) line.row2, (long) line.col2, 2, 5, (uchar) 255);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::FitEllipse0()
{
  if (input == 0) return 0;
  long band = 0;
  long low = 0;
  long high = 0;
  vPrint("\nEnter band, low, high\n");
  vScan("%li %li %li", &band, &low, &high);
  return FitEllipse3(band, low, high);
}


long vInterpreter2::FitEllipse()
{
  return FitEllipse3(0, 1, 255);
}


long vInterpreter2::FitEllipse3(long band, long low, long high)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  v3dMatrix<uchar> * selected = vSelectRange(the_image, band, (uchar) low, (uchar) high);
  grayscale_image * gray = selected->ToGrayCopy();
  vdelete(selected);
  vEllipse ellipse = vFitEllipse1(gray);
  vdelete(gray);
  vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                     ellipse.angle + M_PI/(double) 2, (long) ellipse.width, (uchar) 255);
  vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                     ellipse.angle, (long) ellipse.length, (uchar) 140);
  ShowImage(the_image);
  vPrint("length = %lf, width = %lf, angle = %lf\n", 
          ellipse.length, ellipse.width, ellipse.angle);
  vPrint("elongation = %lf\n", ellipse.length/ellipse.width);
  return 1;
}


long vInterpreter2::LinkIDs()
{
  if (input == 0) return 0;
  if (!predicate_color_image(input->current_image)) return 0;
  vLinkIDs((color_image *) input->current_image);
  DrawCurrentImage();
  return 1;
}


long vInterpreter2::StraightLinks()
{
  if (input == 0) return 0;
  if (predicate_color_image(input->current_image) == 0) return 0;
  color_image * the_image = (color_image *) input->current_image->ToRGBCopy();
  vector<vStraightLinks> links;
  vGoodLinksForLines(the_image, &links);
  grayscale_image * link_ids = vLinkIDsB(the_image);

  long number = links.size();
  long i;
  for (i = 0; i < number; i++)
  {
    grayscale_image * selected = 
      (grayscale_image *) vSelectRange(link_ids, 0, (uchar) links[i].link1, (uchar) links[i].link3);
    vEllipse ellipse = vFitEllipse1(selected);
    vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                       ellipse.angle + M_PI/(double) 2, (long) ellipse.width, (uchar) 255);
    vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                       ellipse.angle, (long) ellipse.length, (uchar) 200);
    if (links[i].line1.row1 != -1)
    {
      vDrawLine1(the_image, (long) links[i].line1.row1, (long) links[i].line1.col1, 
                  (long) links[i].line1.row2, (long) links[i].line1.col2, (uchar) (200+i));
    }
    if (links[i].line2.row1 != -1)
    {
      vDrawLine1(the_image, (long) links[i].line2.row1, (long) links[i].line2.col1, 
                  (long) links[i].line2.row2, (long) links[i].line2.col2, (uchar) (200+i));
    }
    vdelete(selected);
  }

  ShowImage(the_image);
  vdelete(link_ids);

  return 1;
}


// The next two "StraightLink" functions interface with a better
// version of the code than the previous function
long vInterpreter2::StraightLinks0()
{
  if (input == 0) return 0;
  if ((database_input < 0) || (database_input >= number_of_inputs) ||
      (database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view\n");
  vScan("%li %li", &frame, &view);
  return StraightLinks2(frame, view);
}


long vInterpreter2::StraightLinks2(long frame, long view)
{
  if (input == 0) return 0;
  if ((database_input < 0) || (database_input >= number_of_inputs) ||
      (database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  SetCurrentInput(database_edge_input);
  ViewRank(frame, view);
  grayscale_image * edges = (grayscale_image *) input->current_image->threshold(1);

  SetCurrentInput(database_input);
  ViewRank(frame, view);
  if (predicate_color_image(input->current_image) == 0) return 0;
  color_image * the_image = (color_image *) input->current_image->ToRGBCopy();
  
  grayscale_image * dilated = vDilateImage(edges, 1, 255);
  vdelete(edges);
  ShowImage(dilated);
  vdelete(model_straight_links);
  model_straight_links = new vector<vStraightLinks>;
  vGoodLinksForLines2(the_image, dilated, model_straight_links);
  grayscale_image * link_ids = vLinkIDsB(the_image);
  vector<vStraightLinks> * links = model_straight_links;

  long number = links->size();
  long i;
  for (i = 0; i < number; i++)
  {
    grayscale_image * selected = 
      (grayscale_image *) vSelectRange(link_ids, 0, (uchar) (*links)[i].link1, (uchar) (*links)[i].link3);
    vEllipse ellipse = vFitEllipse1(selected);
    long id1 = (*links)[i].link1;
    vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                       ellipse.angle + M_PI/(double) 2, (long) ellipse.width, (uchar) (200+id1));
    vDrawCenteredLine(the_image, (long) ellipse.center_row, (long) ellipse.center_col, 
                       ellipse.angle, (long) ellipse.length, (uchar) (200+id1));
    if ((*links)[i].line1.row1 != -1)
    {
      vDrawLine1(the_image, (long) (*links)[i].line1.row1, (long) (*links)[i].line1.col1, 
                  (long) (*links)[i].line1.row2, (long) (*links)[i].line1.col2, (uchar) (170+id1));
    }
    if ((*links)[i].line2.row1 != -1)
    {
      vDrawLine1(the_image, (long) (*links)[i].line2.row1, (long) (*links)[i].line2.col1, 
                  (long) (*links)[i].line2.row2, (long) (*links)[i].line2.col2, (uchar) (230+id1));
    }
    vdelete(selected);
  }

  ShowImage(the_image);
  vdelete(link_ids);

  return 1;
}


vLineSegment vInterpreter2::GetModelLine(long link_id, long line)
{
  vLineSegment result;
  result.row1 = -1;
  
  if (model_straight_links == 0) return result;
  long number = model_straight_links->size();
  long i;
  long index = -1;
  for (i = 0; i < number; i++)
  {
    if (((*model_straight_links)[i].link1 <= link_id) &&
        ((*model_straight_links)[i].link3 >= link_id))
    {
      index = i;
      break;
    }
  }
  if (index == -1)
  {
    vPrint("Model line not found for this link\n");
    return result;
  }
  if (line == 1)
  {
    result = (*model_straight_links)[index].line1;
  }
  else if (line == 2)
  {
    result = (*model_straight_links)[index].line2;
  }
  if (result.row1 < 0)
  {
    vPrint("The specified line is not valid\n");
  }

  return result;
}

  
long vInterpreter2::CircularDistance0()
{
  double x, y, period;
  x = y = period = 0;
  vPrint("\nEnter x, y, period\n");
  vScan("%lf %lf %lf", &x, &y, &period);
  double distance = vCircularDistance(x, y, period);
  vPrint("distance = %lf\n", distance);
  return 1;
}


long vInterpreter2::HalfPlane0()
{
  double x0, y0, angle, x, y;
  x0 = y0 = angle = x = y = 0;
  vPrint("\nEnter x0, y0, angle, x, y:\n");
  vScan("%lf %lf %lf %lf %lf", &x0, &y0, &angle, &x, &y);
  long result = vFindHalfPlane(x0, y0, angle, x, y);
  vPrint("half plane = %li\n", result);
  return result;
}


long vInterpreter2::FitLine0()
{
  if (input == 0) return 0;
  long method = 0;
  vPrint("\nEnter method:\n");
  vScan("%li", &method);
  return FitLine(method);
}


long vInterpreter2::FitLine(long method)
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  double row, col, angle;
  row = col = angle = 0;
  vector<long> long_rows, long_cols;
  vector<double> rows, cols;
  vGetNonZeroPoints(gray, &long_rows, &long_cols);
  vLongToDoubleVector(&rows, &long_rows);
  vLongToDoubleVector(&cols, &long_cols);
  long success = vFitLine2D(&rows, &cols, &row, &col, &angle, (CvDisType) method);
  if (success <= 0)
  {
    vPrint("Failed to fit line\n");
    return 0;
  }
  vDrawCenteredLine(gray, vRound(row), vRound(col), angle, 200, (uchar) 255);
  ShowImage(gray);
  return 1;
}


long vInterpreter2::LineSegmentMatches0()
{
  if (input == 0) return 0;
  vPrint("\nEnter orientation, length, threshold, tolerance\n");
  long length, threshold;
  double orientation, tolerance;
  orientation = tolerance = (double) 0;
  length = threshold = 0;
  vScan("%lf %li %li %lf", &orientation, &length, &threshold, &tolerance);
  return LineSegmentMatches(orientation, length, threshold, tolerance);
}


long vInterpreter2::LineSegmentMatches(double angle, long length, 
                                        long threshold, double tolerance)
{
  if (input == 0) return 0;
  grayscale_image * current = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(current, (uchar) canny_threshold);
  vector<vLineSegmentMatch2> matches;
  long success = vLineSegmentMatches(current, edges, angle, length, 
                                      threshold, tolerance, &matches);
  ShowImage(edges);
  vdelete(current);

  v3dMatrix<long> * ranks = new v3dMatrix<long>(edges->Rows(), edges->Cols(), 1);
  vWriteValue(ranks, (long) 0);
  vArray3(long) ranks3 = ranks->Matrix3();

  v3dMatrix<long> * info = new v3dMatrix<long>(edges->Rows(), edges->Cols(), 3);
  vWriteValue(info, (long) 0);
  vArray3(long) info3 = info->Matrix3();
  
  long number = matches.size();
  long i;
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch2 match = matches[i];
    long row = vRound((match.segment.row1 + match.segment.row2) / 2.);
    long col = vRound((match.segment.col1 + match.segment.col2) / 2.);
    long step = match.step;
    if (ranks3[0][row][col] != 0) continue;
    ranks3[0][row][col] = number - i;
    info3[0][row][col] = number - i;
    info3[1][row][col] = step;
    info3[2][row][col] = (long) match.positive_votes;
  }

  ShowImage(info);
  ShowImage(ranks);
  return 1;
}

  
long vInterpreter2::CountNonZero()
{
  if (input == 0) return 0;
  grayscale_image * thresholded = (grayscale_image *) input->current_image->threshold(0.000000000000001);
  vector<long> rows, cols;
  vGetNonZeroPoints(thresholded, &rows, &cols);
  long number = rows.size();
  vPrint("number = %li, percentage = %f%%\n", number, ((float) number) / ((float) thresholded->Size()) * 100.0f);
  vdelete(thresholded);
  return 1;
}


long vInterpreter2::TestLinePoints0()
{
  if (input == 0) return 0;
  long row, col, length;
  double angle;
  vPrint("\nEnter row, col, angle, length:\n");
  vScan("%li %li %lf %li", &row, &col, &angle, &length);
  return TestLinePoints(row, col, angle, length);
}


long vInterpreter2::TestLinePoints(long row, long col, double angle, long length)
{
  if (input == 0) return 0;
  if ((angle < 0) || (angle >= 2 * M_PI)) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vector<long> rows, cols;
  vLinePoints(row, col, angle, length, &rows, &cols);
  vWriteValue2(the_image, &rows, &cols, (uchar) 255);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::TestSearchRegion0()
{
  if (input == 0) return 0;
  double orientation, high_length, angle_tolerance;
  long distance_tolerance;
  orientation = high_length = angle_tolerance = distance_tolerance = 0;
  vPrint("\nEnter orientation, high_length, angle_tolerance, distance_tolerance:\n");
  vScan("%lf %lf %lf %li", &orientation, &high_length, 
         &angle_tolerance, &distance_tolerance);
  return TestSearchRegion(orientation, high_length, angle_tolerance, distance_tolerance);
}


long vInterpreter2::TestSearchRegion(double orientation, double high_length, 
                                      double angle_tolerance, long distance_tolerance)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vector<long> rows, cols, lengths;
  vGetSearchRegion(orientation, high_length, angle_tolerance, distance_tolerance, 
                    &rows, &cols, &lengths);
  long size = rows.size();
  long i;
  for (i = 0; i < size; i++)
  {
    rows[i] += the_image->Rows() / 2;
    cols[i] += the_image->Cols() / 2;
    vPrint("%li %li %li\n", rows[i], cols[i], lengths[i]);
  }
  vPrint("total number: %li\n", size);
  vWriteValue2(the_image, &rows, &cols, (uchar) 255);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::OrientedLinesD0()
{
  if (input == 0) return 0;
  double orientation, length, angle_tolerance;
  long distance_tolerance;
  orientation = length = angle_tolerance = distance_tolerance = 0;
  vPrint("\nEnter orientation, length, distance_tolerance, angle_tolerance:n");
  vScan("%lf %lf %li %lf", &orientation, &length, &distance_tolerance, &angle_tolerance);
  return OrientedLinesD(orientation, length, distance_tolerance, angle_tolerance);
}


long vInterpreter2::OrientedLinesD(double orientation, double length,
                                    long distance_tolerance, double angle_tolerance)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(the_image, (uchar) canny_threshold);
  v3dMatrix<double> * votes = vTestOrientedLinesD(the_image, edges, orientation, length,
                                                     distance_tolerance, angle_tolerance);
  ShowImage(edges);
  ShowImage(votes);
  vdelete(the_image);
  if (votes == 0)
  {
    vPrint("vTestOrientedLinesD failed\n");
    return 0;
  }
  else
  {
    return 1;
  }
}


// This is a corrected version, with respect to GetLineSegments02.
long vInterpreter2::GetLineSegments0()
{
  if (input == 0) return 0;
  long orientations, lengths, distance_tolerance;
  double low_length, high_length, angle_tolerance;
  vPrint("\nEnter orientations, low_length, hi_length, lengths, angle_thr, distance_thr:\n");
  vScan("%li %lf %lf %li %lf %li", 
         &orientations, &low_length, &high_length, &lengths, 
         &angle_tolerance, &distance_tolerance);
  return GetLineSegments(orientations, low_length, high_length, 
                         lengths, angle_tolerance, distance_tolerance);
}


long vInterpreter2::GetLineSegments(long orientations, double low_length, 
                                     double high_length,
                                     long lengths, double angle_tolerance, 
                                     long distance_tolerance)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(the_image, (uchar) canny_threshold);
  
  v3dMatrix<double> * gradient_norms = vGradientNorm0(the_image);
  v3dMatrix<double> * double_weighted_edges = vSelectPixels(gradient_norms, edges);
  grayscale_image * weighted_edges = double_weighted_edges->ToGrayBound();
  vdelete(gradient_norms);
  vdelete(double_weighted_edges);
  vdelete(edges);
  ShowImage(weighted_edges);

  vLineSegmentSearchParams params(orientations, low_length, high_length, 
                                   lengths, angle_tolerance, distance_tolerance);
  vdelete(line_segments);
  line_segments = new vector<vLineSegmentMatch>;
  long success = vGetLineSegments(the_image, weighted_edges, params, line_segments);
  vdelete(the_image);
  long number = line_segments->size();
  vPrint("number of matches found: %li\n", number);
  vPrint("size of vLineSegmentMatch = %li\n", (long) sizeof(vLineSegmentMatch));

  vSortLineSegments(line_segments);
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, weighted_edges->Rows(), 
                         weighted_edges->Cols(), &ranks, &info, -1);

  ShowImage(info);
  ShowImage(ranks);

  long i;
  while(1)
  {
    vPrint("Enter i: ");
    vScan("%li", &i);
    if (i < 0) break;
    if (i >= number) continue;
    vPrint("score = %lf\n", -(*line_segments)[i].score);
  }

  return 1;
}


// A modified version of GetLineSegments. Here, the segments are normalized,
// according to a scaling and translation that moves the hand image to the central
// 192x192 square of a 256x256 image.
long vInterpreter2::GetLineSegments03()
{
  if (input == 0) return 0;
  long orientations, lengths, distance_tolerance;
  double low_length, high_length, angle_tolerance;
  vPrint("\nEnter orientations, low_length, hi_length, lengths, angle_thr, distance_thr:\n");
  vScan("%li %lf %lf %li %lf %li", 
         &orientations, &low_length, &high_length, &lengths, 
         &angle_tolerance, &distance_tolerance);
  return GetLineSegments3(orientations, low_length, high_length, 
                         lengths, angle_tolerance, distance_tolerance);
}


long vInterpreter2::GetLineSegments3(long orientations, double low_length, 
                                      double high_length,
                                      long lengths, double angle_tolerance, 
                                      long distance_tolerance)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(the_image, 2);
  
  v3dMatrix<double> * gradient_norms = vGradientNorm0(the_image);
  v3dMatrix<double> * double_weighted_edges = vSelectPixels(gradient_norms, edges);
  grayscale_image * weighted_edges = double_weighted_edges->ToGrayBound();
  vdelete(gradient_norms);
  vdelete(double_weighted_edges);
  vdelete(edges);
  ShowImage(weighted_edges);

  vLineSegmentSearchParams params(orientations, low_length, high_length, 
                                   lengths, angle_tolerance, distance_tolerance);

  vBoundingBox box = vBinaryBoundingBox(the_image);
  vScaleSearchParams(&params, &box, 192);
  vdelete(line_segments);
  line_segments = new vector<vLineSegmentMatch>;
  long success = vGetLineSegments(the_image, weighted_edges, params, line_segments);
  vdelete(the_image);
  long number = line_segments->size();
  vPrint("number of matches found: %li\n", number);
  vPrint("size of vLineSegmentMatch = %li\n", (long) sizeof(vLineSegmentMatch));

  vNormalizeLineSegments(line_segments, &box, 192, 128, 128);
  
//  vSortLineSegments(line_segments);
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, weighted_edges->Rows(), 
                         weighted_edges->Cols(), &ranks, &info, -1);

  ShowImage(info);
  ShowImage(ranks);

  return 1;
}


long vInterpreter2::GetLineSegments04()
{
  if (input == 0) return 0;
  long orientations, lengths, distance_tolerance;
  double low_length, high_length, angle_tolerance;
  vPrint("\nEnter orientations, low_length, hi_length, lengths, angle_thr, distance_thr:\n");
  vScan("%li %lf %lf %li %lf %li", 
         &orientations, &low_length, &high_length, &lengths, 
         &angle_tolerance, &distance_tolerance);
  return GetLineSegments4(orientations, low_length, high_length, 
                          lengths, angle_tolerance, distance_tolerance);
}


// Line segments will be
// extracted from right_hand_frame (not currently displayed image),
// after we select from that frame the region of interest specified
// by hand_mask. The region
// of interest also depends on the selected segmentation method
// (fine segmentation or bounding box).
long vInterpreter2::GetLineSegments4(long orientations, double low_length, 
                                      double high_length,
                                      long lengths, double angle_tolerance, 
                                      long distance_tolerance)
{
  if (input == 0) return 0;
  if (right_hand_frame == 0) return 0;
  if (hand_mask == 0) return 0;

  grayscale_image * binary = hand_mask->ToGrayCopy();
  vBoundingBox box = vBinaryBoundingBox(hand_mask);
  // Read the current frame in the video sequence
  grayscale_image * temp_image = right_hand_frame->ToGrayCopy();
  // Blur the current frame
  v3dMatrix<double> * temp_blurred = vBlur(temp_image);
  grayscale_image * the_image = temp_blurred->ToGray();
  vdelete(temp_image);
  vdelete(temp_blurred);
  // Detect edges on the blurred image with the lowest threshold
  grayscale_image * edges = vCannyDetector(the_image, 2);
  
  // Prepare the region-of-interest mask (for bounding box segmentation
  // we must fill up the whole bounding box, otherwise binary is already
  // what we want).
  if (segmentation_method == 0)
  {
    vDrawFilledRectangle1(binary, box.top, box.left, 
                           box.bottom, box.right, (uchar) 255);
  }

  // Compute gradient norms, and select the region of interest
  v3dMatrix<double> * temp_gradient_norms = vGradientNorm0(the_image);
  v3dMatrix<double> * gradient_norms = vSelectPixels(temp_gradient_norms, binary);
  // Furthermore, from the gradient nomrs in the region of interest, 
  // select the ones that correspond to edge pixels (essentially 
  // that correspond to local maxima in the direction of the gradient).
  v3dMatrix<double> * double_weighted_edges = vSelectPixels(gradient_norms, edges);
  grayscale_image * weighted_edges = double_weighted_edges->ToGrayBound();
  vdelete(binary);
  vdelete(temp_gradient_norms);
  vdelete(gradient_norms);
  vdelete(double_weighted_edges);
  vdelete(edges);
  ShowImage(weighted_edges);

  vLineSegmentSearchParams params(orientations, low_length, high_length, 
                                   lengths, angle_tolerance, distance_tolerance);

  vScaleSearchParams(&params, &box, 192);
  vdelete(line_segments);
  line_segments = new vector<vLineSegmentMatch>;
  long success = vGetLineSegments(the_image, weighted_edges, params, line_segments);
  vdelete(the_image);
  long number = line_segments->size();
  vPrint("number of matches found: %li\n", number);

  vNormalizeLineSegments(line_segments, &box, 192, 128, 128);
  
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, 256, 256, &ranks, &info, -1);

  ShowImage(info);
  ShowImage(ranks);
  return 1;
}


// Here we order matches by sorting them separately based on their length, V, S,
// and based on their D, O, L with respect to the specified model line.
long vInterpreter2::MatchLineSegment0()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (input == 0) return 0;
  vPrint("\nEnter link id, line:\n");
  long link_id = 0;
  long line = 0;
  vScan("%li %li", &link_id, &line);
  return MatchLineSegment(link_id, line);
}


long vInterpreter2::MatchLineSegment(long link_id, long line)
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (input == 0) return 0;

  long number = model_straight_links->size();
  long i;
  long index = -1;
  for (i = 0; i < number; i++)
  {
    if (((*model_straight_links)[i].link1 <= link_id) &&
        ((*model_straight_links)[i].link3 >= link_id))
    {
      index = i;
      break;
    }
  }
  if (index == -1)
  {
    vPrint("Model line not found for this link\n");
    return 0;
  }
  vLineSegment line_segment;
  if (line == 1)
  {
    line_segment = (*model_straight_links)[index].line1;
  }
  else
  {
    line_segment = (*model_straight_links)[index].line2;
  }
  if (line_segment.row1 < 0)
  {
    vPrint("The specified line is not valid\n");
    return 0;
  }

  vSortLineSegments(line_segments);
  vRankLineSegments(line_segments, line_segment);
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, -1);
  ShowImage(info);
  vdelete(ranks);
  DrawLineSegment(0);
  return 1;
}


// Produce the two images of the top k line segments 
// (one image shows all the info, another shows the rank).
long vInterpreter2::LineSegmentImages0()
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;
  long k = 0;
  vPrint("Input number of segments to use for image:\n");
  vScan("%li", &k);
  return LineSegmentImages(k);
}


long vInterpreter2::LineSegmentImages(long k)
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, k);
  ShowImage(info);
  ShowImage(ranks);
  return 1;  
}


long vInterpreter2::DrawLineSegment0()
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  long rank = 0;
  vPrint("\nEnter rank:\n");
  vScan("%li", &rank);
  return DrawLineSegment(rank);
}


long vInterpreter2::DrawLineSegment(long rank)
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  if (rank < 0) return 0;
  if (rank >= (long) line_segments->size()) return 0;
  vLineSegmentMatch & segment_match = (*line_segments)[rank];
  vLineSegmentLong segment = segment_match.segment;
  vPrint("(%li, %li)-(%li, %li)\n", segment.row1, segment.col1, segment.row2, segment.col2);
  DrawLine(vRound(segment.row1), vRound(segment.col1), 
           vRound(segment.row2), vRound(segment.col2), (uchar) 255);
  segment_match.Print1();

  return 1;
}

  
// Count line segments whose dolvs are strictly better (or worse) than 
// the ones given here
long vInterpreter2::CountLineSegments0()
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  long score, d, o, l, v, s, direction;
  score = d = o = l = v = s = direction = 0;
  vPrint("Enter score, d, o, l, v, s, direction:\n");
  vScan("%li %li %li %li %li %li %li", &score, &d, &o, &l, &v, &s, &direction);
  return CountLineSegments(score, d, o, l, v, s, direction);
}


long vInterpreter2::CountLineSegments(long score, long d, long o, long l, 
                                       long v, long s, long direction)
{
  if (line_segments == 0) return 0;
  if (input == 0) return 0;
  long number = vCountLineSegments(line_segments, score, d, o, l, v, s, direction);
  vPrint("number of selected segments = %li\n", number);
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, number);
  ShowImage(info);
  ShowImage(ranks);
  
  return 1;
}


// Select line segments whose dolvs are strictly better (or worse) than 
// the ones given here, and count them
long vInterpreter2::SelectLineSegments0()
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  long score, d, o, l, v, s, direction;
  score = d = o = l = v = s = direction = 0;
  vPrint("Enter score, d, o, l, v, s, direction:\n");
  vScan("%li %li %li %li %li %li %li", &score, &d, &o, &l, &v, &s, &direction);
  return SelectLineSegments(score, d, o, l, v, s, direction);
}


long vInterpreter2::SelectLineSegments(long score, long d, long o, long l, long v, 
                                        long s, long direction)
{
  if (line_segments == 0) return 0;
  if (input == 0) return 0;
  long number = vSelectLineSegments(line_segments, score, d, o, l, v, s, direction);
  vPrint("number of selected segments = %li\n", number);
  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, number);
  ShowImage(info);
  ShowImage(ranks);
  
  return 1;
}


long vInterpreter2::MatchLineSegment02()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (input == 0) return 0;
  vPrint("\nEnter link id, line:\n");
  long link_id = 0;
  long line = 0;
  vScan("%li %li", &link_id, &line);
  return MatchLineSegment2(link_id, line);
}


// Here we order matches based on their frequency in a DOLVS histogram built
// using the specified model line.
long vInterpreter2::MatchLineSegment2(long link_id, long line)
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (input == 0) return 0;

  long number = model_straight_links->size();
  long i;
  long index = -1;
  for (i = 0; i < number; i++)
  {
    if (((*model_straight_links)[i].link1 <= link_id) &&
        ((*model_straight_links)[i].link3 >= link_id))
    {
      index = i;
      break;
    }
  }
  if (index == -1)
  {
    vPrint("Model line not found for this link\n");
    return 0;
  }
  vLineSegment line_segment;
  if (line == 1)
  {
    line_segment = (*model_straight_links)[index].line1;
  }
  else
  {
    line_segment = (*model_straight_links)[index].line2;
  }
  if (line_segment.row1 < 0)
  {
    vPrint("The specified line is not valid\n");
    return 0;
  }

  vDOLVS_Histogram * histogram = new vDOLVS_Histogram;
  histogram->MakeInputSummary(line_segments);
  histogram->ProcessLinkSegment(line_segment);
  histogram->MarkEntries();
  histogram->MakeCumulative();
  histogram->RankInputSegments(line_segments);
  vdelete(histogram);

  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, -1);
  ShowImage(info);
  vdelete(ranks);
  DrawLineSegment(0);
  return 1;
}


long vInterpreter2::AllModelLines0()
{
  if (input == 0) return 0;
  return AllModelLines(database_input, database_edge_input);
}


long vInterpreter2::AllModelLines(long database_images, long database_edges)
{
  if (input == 0) return 0;
  if (database_images < 0) return 0;
  if (database_images >= number_of_inputs) return 0;
  if (database_edges < 0) return 0;
  if (database_edges>= number_of_inputs) return 0;

  vVideoFile * database_video = inputs[database_images]->sequence;
  vVideoFile * edges_video = inputs[database_edges]->sequence;
  vdelete(all_model_lines);
  all_model_lines = vAllModelLines(database_video, edges_video);
  if (all_model_lines == 0)
  {
    vPrint("Failed to compute model lines\n");
    return 0;
  }
  else
  {
    return 1;
  }
}


long vInterpreter2::SaveModelLines0()
{
  if (input == 0) return 0;
  return SaveModelLines(database_input, database_edge_input);
}


long vInterpreter2::SaveModelLines(long database_images, long database_edges)
{
  if (input == 0) return 0;
  if (database_images < 0) return 0;
  if (database_images >= number_of_inputs) return 0;
  if (database_edges < 0) return 0;
  if (database_edges>= number_of_inputs) return 0;

  vVideoFile * database_video = inputs[database_images]->sequence;
  vVideoFile * edges_video = inputs[database_edges]->sequence;
  vdelete(all_model_lines);
  all_model_lines = vSaveAllModelLines(database_video, edges_video);
  if (all_model_lines == 0)
  {
    vPrint("Failed to compute model lines\n");
    return 0;
  }
  else
  {
    return 1;
  }
}


long vInterpreter2::LoadModelLines0()
{
  if (input == 0) return 0;
  return LoadModelLines(database_input);
}


long vInterpreter2::LoadModelLines(long database_images)
{
  if (input == 0) return 0;
  if (database_images < 0) return 0;
  if (database_images >= number_of_inputs) return 0;

  vVideoFile * database_video = inputs[database_images]->sequence;
  vdelete(all_model_lines);
  all_model_lines = vLoadAllModelLines(database_video);
  if (all_model_lines == 0)
  {
    vPrint("Failed to load model lines\n");
    return 0;
  }
  else
  {
    return 1;
  }
}


long vInterpreter2::ReadModelLines0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0)
  {
    vPrint("No model lines are loaded\n");
    return 0;
  }
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);
  return ReadModelLines(frame, view);
}


long vInterpreter2::ReadModelLines(long frame, long view)
{
  if (input == 0) return 0;
  if (all_model_lines == 0)
  {
    vPrint("No model lines are loaded\n");
    return 0;
  }
  vdelete(model_straight_links);
  model_straight_links = vReadModelLines(all_model_lines, frame * 4 + view);
  if (model_straight_links == 0)
  {
    vPrint("Failed to get model_straight_links\n");
    return 0;
  }
  return DrawModelStraightLinks();
}


long vInterpreter2::CountAllModelLines0()
{
  if (all_model_lines == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start and period:\n");
  vScan("%li %li", &start, &period);
  return CountAllModelLines(start, period);
}


long vInterpreter2::CountAllModelLines(long start, long period)
{
  if (all_model_lines == 0) return 0;
  if (period <= 0) return 0;
  long counter = 0;
  long bands = all_model_lines->Bands();
  long rows = all_model_lines->Rows();
  long total = bands*rows;
  vArray3(uchar) lines = all_model_lines->Matrix3();
  long index;
  for (index = start; index < total; index+= period)
  {
    long band = index / rows;
    long row = index % rows;
    if (lines[band][row][0] > 1)
    {
      counter++;
    }
  }

  vPrint("counter = %li\n", counter);
  return 1;
}

  
long vInterpreter2::DrawModelStraightLinks()
{
  if (input == 0) return 0;
  if (model_straight_links == 0) return 0;
  vector<vStraightLinks> * links = model_straight_links;
  v3dMatrix<uchar> * the_image;
  if ((input->current_image->Rows() < 256) ||
      (input->current_image->Cols() < 256))
  {
    the_image = new color_image(256, 256);
  }
  else 
  {
    the_image = input->current_image->ToRGBCopy();
  }
  
  long length = links->size();
  long i;
  for (i = 0; i < length; i++)
  {
    long id1 = (*links)[i].link1;
    if ((*links)[i].line1.row1 != -1)
    {
      vDrawLine1(the_image, (long) (*links)[i].line1.row1, (long) (*links)[i].line1.col1, 
                  (long) (*links)[i].line1.row2, (long) (*links)[i].line1.col2, (uchar) (170+id1));
    }
    if ((*links)[i].line2.row1 != -1)
    {
      vDrawLine1(the_image, (long) (*links)[i].line2.row1, (long) (*links)[i].line2.col1, 
                  (long) (*links)[i].line2.row2, (long) (*links)[i].line2.col2, (uchar) (230+id1));
    }
  }
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::ExistentialDOLVS0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start, period\n", start, period);
  vScan("%li %li\n", &start, &period);
  return ExistentialDOLVS(start, period);
}


long vInterpreter2::ExistentialDOLVS(long start, long period)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;

  if (existential_DOLVS == 0)
  {
    existential_DOLVS = new vDOLVS_Histogram();
  }
  long success = existential_DOLVS->ExistentialFrequencies(all_model_lines, line_segments, 
                                                           start, period);
  if (success <= 0)
  {
    vPrint("Failed to compute existential_DOLVS\n");
    return 0;
  }
  dolvs_start = start;
  dolvs_period = period;
  vPrint("Computed existential DOLVS histogram\n");
  DOLVS_PDF0();
  return 1;
}


long vInterpreter2::ExistentialDOLVS02()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start, period\n", start, period);
  vScan("%li %li\n", &start, &period);
  return ExistentialDOLVS2(start, period);
}


long vInterpreter2::ExistentialDOLVS2(long start, long period)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;

  if (existential_DOLVS == 0)
  {
    existential_DOLVS = new vDOLVS_Histogram();
  }
  long success = existential_DOLVS->ExistentialFrequencies2(all_model_lines, line_segments, 
                                                            start, period);
  if (success <= 0)
  {
    vPrint("Failed to compute existential_DOLVS\n");
    return 0;
  }
  dolvs_start = start;
  dolvs_period = period;
  vPrint("Computed existential DOLVS histogram\n");
  DOLVS_PDF2();
  return 1;
}


long vInterpreter2::DOLVS_PDF0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  dolvs_pdf = existential_DOLVS->PDF(all_model_lines, dolvs_start, dolvs_period);
  vPrint("Computed DOLVS PDF\n");
  return 1;
}


long vInterpreter2::DOLVS_PDF2()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  dolvs_pdf = existential_DOLVS->PDF2(all_model_lines, line_segments,
                                      dolvs_start, dolvs_period);
  vPrint("Computed DOLVS PDF\n");
  return 1;
}


long vInterpreter2::DOLVS_To_PDF()
{
  if (existential_DOLVS == 0) return 0;
  long max_value = existential_DOLVS->MaxValue();
  if (dolvs_pdf.Cols() <= max_value)
  {
    vPrint("Error, dolvs_pdf not appropriate\n");
    return 0;
  }
  existential_DOLVS->ToPDF(&dolvs_pdf);
  return 1;
}

  
long vInterpreter2::MatchLineSegment03()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;
  vPrint("\nEnter link id, line:\n");
  long link_id = 0;
  long line = 0;
  vScan("%li %li", &link_id, &line);
  return MatchLineSegment3(link_id, line);
}


long vInterpreter2::MatchLineSegment3(long link_id, long line)
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;

  vLineSegment model_segment = GetModelLine(link_id, line);
  if (model_segment.row1 < 0) return 0;

  existential_DOLVS->MakeInputSummary(line_segments);
  existential_DOLVS->ProcessLinkSegment(model_segment);
  existential_DOLVS->RankInputSegments(line_segments);

  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, -1);
  ShowImage(info);
  vdelete(ranks);
  DrawLineSegment(0);
  return 1;
}


// Here we order matches based on their existential frequencies in 
// the existential_DOLVS histogram which has been computed using
// a sampling of possible model lines. However, the score of each
// segment is not the frequency, but its corresponding probability
// in the dolvs_pdf table.
long vInterpreter2::MatchLineSegment04()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;
  vPrint("\nEnter link id, line:\n");
  long link_id = 0;
  long line = 0;
  vScan("%li %li", &link_id, &line);
  return MatchLineSegment4(link_id, line);
}


// 5/14/2003: This function does not seem to work correctly, combined
// with DOLVS_Stats2.
long vInterpreter2::MatchLineSegment4(long link_id, long line)
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;

  vLineSegment model_segment = GetModelLine(link_id, line);
  if (model_segment.row1 < 0) return 0;

  existential_DOLVS->ProcessLinkSegment(model_segment);
  existential_DOLVS->RankInputSegments(line_segments);
  long success = vFrequencyToPDF(dolvs_pdf, line_segments);
  if (success <= 0)
  {
    vPrint("Error: DOLVS_PDF table not appropriate\n");
  }

  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, -1);
  ShowImage(info);
  vdelete(ranks);
  DrawLineSegment(0);
  return 1;
}


long vInterpreter2::DOLVS_Stats()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;

  Blur();
  GetLineSegments3(24, 5, 100, 20, .1, 1);
  SelectLineSegments(-1, -1, -1, 21, 80, -1, 2);
  vSortLineSegments(line_segments);
  SampleLineSegments(2, 1, 1);
  ExistentialDOLVS(0, 401);
  DOLVS_To_PDF();
  return 1;
}


long vInterpreter2::DOLVS_Stats2()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (right_hand_frame == 0) return 0;

  GetLineSegments4(24, 5, 100, 20, .1, 1);
  SelectLineSegments(-1, -1, -1, 21, 50, -1, 2);
  vSortLineSegments(line_segments);
  SampleLineSegments(2, 1, 1);
  ExistentialDOLVS(0, 401);
  DOLVS_To_PDF();
  vdelete(line_grid);
  line_grid = new vLineGrid(10, 5, 5);
  line_grid->ComputeRankings(existential_DOLVS);
  vPrint("Computed line grid\n");
  return 1;
}


long vInterpreter2::SaveDOLVSStats0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;

  long start, period, start2, period2, matches;
  start = period = start2 = period2 = matches = 0;
  vPrint("\nEnter start, period, test frame start and period, number of matches:\n");
  vScan("%li %li %li %li %li", &start, &period, &start2, &period2, &matches);
  return SaveDOLVSStats(start, period, start2, period2, matches);
}


long vInterpreter2::SaveDOLVSStats(long start, long period, long start2, 
                                    long period2, long matches)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  long current_index = current_input_index;
  long number_of_frames = inputs[current_index]->number_of_frames;
  long frame;
  for (frame = start2; frame < number_of_frames; frame+=period2)
  {
    SetCurrentInput(current_index);
    JumpToFrame(frame);
    SaveDOLVSFrameStats(start, period, matches);
  }
  return 1;
}


long vInterpreter2::SaveDOLVSFrameStats0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  long start, period, matches;
  vPrint("\nEnter start, period, matches:\n");
  vScan("%li %li %li", &start, &period, &matches);
  return SaveDOLVSFrameStats(start, period, matches);
}


long vInterpreter2::SaveDOLVSFrameStats(long start, long period, long matches)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  ShowCurrentFrame();
  ShowCameraHand();
  Blur();
  GetLineSegments3(24, 5, 100, 20, .1, 1);
  SelectLineSegments(-1, -1, -1, 21, 80, -1, 2);
  ExistentialDOLVS(start, period);
  long result = SaveDOLVS0();
  //long result = LoadFrameDOLVS(start, period);
  DOLVS_To_PDF();

  if (result > 0)
  {
    result = SaveLineMatchFrameStats(start, period, matches);  
  }

  return result;
  
}


long vInterpreter2::SaveDOLVS0()
{
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;
  char * dolvs_file = vDOLVSFrameFile(input->path_name, input->current_frame, 
                                       dolvs_start, dolvs_period);
  long result = existential_DOLVS->Save(dolvs_file);
  if (result < 0)
  {
    vPrint("Failed to save DOLVS histogram into %s\n", dolvs_file);
  }
  else 
  {
    vPrint("Saved DOLVS histogram into %s\n", dolvs_file);
  }
  vdelete2(dolvs_file);
  if (dolvs_pdf.Size() <= 1) return 0;
  char * pdf_file = vDOLVS_PDF_File(input->path_name, input->current_frame,
                                     dolvs_start, dolvs_period);
  result = dolvs_pdf.Write(pdf_file);
  if (result < 0)
  {
    vPrint("Failed to save DOLVS pdf into %s\n", pdf_file);
  }
  else 
  {
    vPrint("Saved DOLVS pdf into %s\n", pdf_file);
  }
  vdelete2(pdf_file);

  return result;
}


long vInterpreter2::SaveLineMatchFrameStats0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  long start, period, matches;
  start = period = matches = 0;
  vPrint("\nEnter start, period, matches:\n");
  vScan("%li %li %li", &start, &period, &matches);
  return SaveLineMatchFrameStats(start, period, matches);
}


long vInterpreter2::SaveLineMatchFrameStats(long start, long period, 
                                             long matches)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  char * input_filename = input->path_name;
  long frame = input->current_frame;
  ShowCurrentFrame();
  ShowCameraHand();
  RankViews();
  if (match_scores == 0) return 0;
  if (match_scores->NumberOfScores() < matches) return 0;
  vector<long> database_frames(matches);
  vector<long> database_views(matches);
  vArray(vMatchScore) score_sorted = match_scores->GetScores();
  long i;
  for (i = 0; i < matches; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  return vSaveLineMatchFrameStats(existential_DOLVS,
                                   all_model_lines, start, period, frame,
                                   &database_frames, &database_views,
                                   line_segments, input_filename);
}


long vInterpreter2::LoadFrameDOLVS0()
{
  if (input == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start, period:\n");
  vScan("%li %li", &start, &period);
  return LoadFrameDOLVS(start, period);
}


long vInterpreter2::LoadFrameDOLVS(long start, long period)
{
  if (input == 0) return 0;
  if (existential_DOLVS == 0)
  {
    existential_DOLVS = new vDOLVS_Histogram();
  }
  char * filename = vDOLVSFrameFile(input->path_name, input->current_frame, 
                                     start, period);
  long success = existential_DOLVS->Load(filename);
  if (success <= 0)
  {
    vPrint("Failed to read DOLVS from %s\n", filename);
  }
  vdelete2(filename);
  filename = vDOLVS_PDF_File(input->path_name, input->current_frame, 
                              start, period);
  v3dMatrix<long> * pdf = v3dMatrix<long>::Read(filename);
  if (pdf == 0)
  {
    vPrint("Failed to read pdf from %s\n", filename);
  }
  else
  {
    dolvs_pdf = vMatrix<long>(pdf);
    vdelete(pdf);
  }
  vdelete2(filename);
  return success;
}

// Compares a DOLVS saved to disk to existential_DOLVS. Used to verify
// that there are no bugs in the saving/loading process.
long vInterpreter2::CompareDOLVS0()
{
  if (input == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start, period:\n");
  vScan("%li %li", &start, &period);
  return CompareDOLVS(start, period);
}


long vInterpreter2::CompareDOLVS(long start, long period)
{
  if (input == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  vDOLVS_Histogram * temp_DOLVS = new vDOLVS_Histogram();
  char * filename = vDOLVSFrameFile(input->path_name, input->current_frame, start, period);
  long success = temp_DOLVS->Load(filename);
  if (success <= 0)
  {
    vPrint("Failed to read DOLVS from %s\n", filename);
  }
  vdelete2(filename);
  long result = existential_DOLVS->Equal(temp_DOLVS);
  if (result == 1)
  {
    vPrint("The histograms are equal\n");
  }
  else
  {
    vPrint("The histograms are not equal\n");
  }
  vdelete(temp_DOLVS);
  return success;
}

long vInterpreter2::LoadLineMatchFrameStats0()
{
  if (input == 0) return 0;
  long start, period;
  start = period = 0;
  vPrint("\nEnter start, period:\n");
  vScan("%li %li", &start, &period);
  return LoadLineMatchFrameStats(start, period);
}


long vInterpreter2::LoadLineMatchFrameStats(long start, long period)
{
  if (input == 0) return 0;
  if (line_match_stats == 0) line_match_stats = new vLineMatchStats();
  long success = line_match_stats->Load(input->path_name, input->current_frame,
                                        start, period);
  if (success == 0)
  {
    vdelete(line_match_stats);
    line_match_stats = 0;
    return 0;
  }
  return 1;
}

long vInterpreter2::MatchModelLines0()
{
  if (existential_DOLVS == 0) return 0;
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view\n");
  vScan("%li %li", &frame, &view);
  return MatchModelLines(frame, view);
}


long vInterpreter2::MatchModelLines(long frame, long view)
{
  if (existential_DOLVS == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (database_input >= number_of_inputs) return 0;
  SetCurrentInput(database_input);
  ViewRank(frame, view);
  ReadModelLines(frame, view);
  vdelete(line_match_frame_stats);
  line_match_frame_stats = new vLineMatchFrameStats(existential_DOLVS, 
                                                     model_straight_links,
                                                     frame, view);
  if (line_match_frame_stats == 0)
  {
    vPrint("Failed to compute line_match_frame_stats\n");
    return 0;
  }
  else
  {
    line_match_frame_stats->Print();
    return 1;
  }
}


// Here we use the line grid.
long vInterpreter2::MatchModelLines0b()
{
  if (existential_DOLVS == 0) return 0;
  if (line_grid == 0) return 0;
  long frame, view;
  frame = view = 0;
  double fraction = 0;
  vPrint("\nEnter frame, view, fraction\n");
  vScan("%li %li %lf", &frame, &view, &fraction);
  return MatchModelLines2(frame, view, fraction);
}


long vInterpreter2::MatchModelLines2(long frame, long view, double fraction)
{
  if (existential_DOLVS == 0) return 0;
  if (line_grid == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (database_input >= number_of_inputs) return 0;
  SetCurrentInput(database_input);
  ViewRank(frame, view);
  ReadModelLines(frame, view);
  vdelete(line_match_frame_stats);
  line_match_frame_stats = new vLineMatchFrameStats(existential_DOLVS, 
                                                     line_grid,
                                                     model_straight_links,
                                                     fraction);
  if (line_match_frame_stats == 0)
  {
    vPrint("Failed to compute line_match_frame_stats\n");
    return 0;
  }
  else
  {
    line_match_frame_stats->Print();
    return 1;
  }
}


// Match model lines for the model image that is found
// by CorrectMatch(rank).
long vInterpreter2::MatchCorrectLines0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  long rank = 0;
  vPrint("\nEnter rank:\n");
  vScan("%li", &rank);
  return MatchCorrectLines(rank);
}


long vInterpreter2::MatchCorrectLines(long rank)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  SetCurrentInput(database_input);

  vMatchScore score = match_scores->CorrectMatch(rank);
  if (score.is_valid == 0)
  {
    vPrint("Failed to get correct truth match of rank %li\n", rank);
    return 0;
  }
  CorrectMatch(rank);
  long result = MatchModelLines(score.frame, score.view);
  return result;
}


// Match model lines for the model image that is found
// by GroundMatch(rank).
long vInterpreter2::MatchGroundLines0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  long rank = 0;
  vPrint("\nEnter rank:\n");
  vScan("%li", &rank);
  return MatchGroundLines(rank);
}


long vInterpreter2::MatchGroundLines(long rank)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  SetCurrentInput(database_input);

  vMatchScore score = match_scores->GroundMatch(rank);
  if (score.is_valid == 0)
  {
    vPrint("Failed to get ground truth match of rank %li\n", rank);
    return 0;
  }
  GroundMatch(rank);
  long result = MatchModelLines(score.frame, score.view);
  return result;
}


// Match model lines for the model image that is found
// by GotoMatch(rank).
long vInterpreter2::MatchMatchLines0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  long rank = 0;
  vPrint("\nEnter rank:\n");
  vScan("%li", &rank);
  return MatchMatchLines(rank);
}


long vInterpreter2::MatchMatchLines(long rank)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (match_scores == 0)
  {
    vPrint("No scores are currently loaded\n");
    return 0;
  }
  SetCurrentInput(database_input);

  vMatchScore score = match_scores->GetScore(rank);
  if (score.is_valid == 0)
  {
    vPrint("Failed to get match of rank %li\n", rank);
    return 0;
  }
  JumpToMatch(rank);
  long result = MatchModelLines(score.frame, score.view);
  return result;
}



long vInterpreter2::CompareMatchModelLines0()
{
  if (line_match_stats == 0) return 0;
  if (line_match_frame_stats == 0) return 0;
  long equality = line_match_stats->Compare(line_match_frame_stats);
  if (equality > 0) 
  {
    vPrint("line_match_stats agrees with line_match_frame_stats\n");
  }
  else 
  {
    vPrint("line_match_stats doesn't agree with line_match_frame_stats\n");
  }
  return 1;
}


long vInterpreter2::LineMatchStats0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  long number = 0;
  vPrint("\nEnter number\n");
  vScan("%li", &number);
  return LineMatchStats(number);
}


long vInterpreter2::LineMatchStats(long number)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  if (match_scores == 0) return 0;
  if (match_scores->NumberOfScores() < number) return 0;
  vector<long> database_frames(number);
  vector<long> database_views(number);
  vArray(vMatchScore) score_sorted = match_scores->GetScores();
  long i;
  for (i = 0; i < number; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }
  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats(existential_DOLVS, all_model_lines,
                                             &database_frames, &database_views);
  vMatchScores * temp_scores = line_match_stats->MatchScores();
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, match_scores);
  vdelete(match_scores);
  vdelete(temp_scores);
  match_scores = final_scores;

  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


long vInterpreter2::LineMatchScores0()
{
  if (input == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (line_grid == 0) return 0;
  if (all_model_lines == 0) return 0;

  vdelete(line_match_stats);
  line_match_stats = vComputeLineMatchStats2(existential_DOLVS, line_grid, all_model_lines,
                                              grid_fraction); 
  vdelete(match_scores);
  match_scores = line_match_stats->MatchScores();
  UpdateMatchScores(input->current_frame);
  if (database_input < number_of_inputs)
  {
    SetCurrentInput(database_input);
    ShowTopTenMatches();
  }
  return 1;
}


long vInterpreter2::DOLVS_Bin0()
{
  if (existential_DOLVS == 0) return 0;
  long d, o, l, v, s;
  d = o = l = v = s = 0;
  vPrint("\nEnter d, o, l, v, s:\n");
  vScan("%li %li %li %li %li", &d, &o, &l, &v, &s);
  return DOLVS_Bin(d, o, l, v, s);
}


long vInterpreter2::DOLVS_Bin(long d, long o, long l, long v, long s)
{
  if (existential_DOLVS == 0) return 0;
  long number = existential_DOLVS->BinValue(d, o, l, v, s);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::DOLVS_Entry0()
{
  if (existential_DOLVS == 0) return 0;
  long d, o, l, v, s;
  d = o = l = v = s = 0;
  vPrint("\nEnter d, o, l, v, s:\n");
  vScan("%li %li %li %li %li", &d, &o, &l, &v, &s);
  return DOLVS_Entry(d, o, l, v, s);
}


long vInterpreter2::DOLVS_Entry(long d, long o, long l, long v, long s)
{
  if (existential_DOLVS == 0) return 0;
  long number = existential_DOLVS->DOLVS(d, o, l, v, s);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::DOLVS_Value0()
{
  if (existential_DOLVS == 0) return 0;
  long value = 0;
  vPrint("\nEnter value:\n");
  vScan("%li", &value);
  return DOLVS_Value(value);
}



long vInterpreter2::DOLVS_Value(long value)
{
  if (existential_DOLVS == 0) return 0;
  long number = existential_DOLVS->CountValue(value);
  vPrint("number = %li\n", number);
  return 1;
}



long vInterpreter2::DOLVS_Lequal0()
{
  if (existential_DOLVS == 0) return 0;
  long value = 0;
  vPrint("\nEnter value:\n");
  vScan("%li", &value);
  return DOLVS_Lequal(value);
}


long vInterpreter2::DOLVS_Lequal(long value)
{
  if (existential_DOLVS == 0) return 0;
  long number = existential_DOLVS->CountLequal(value);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::DOLVS_Gequal0()
{
  if (existential_DOLVS == 0) return 0;
  long value = 0;
  vPrint("\nEnter value:\n");
  vScan("%li", &value);
  return DOLVS_Gequal(value);
}


long vInterpreter2::DOLVS_Gequal(long value)
{
  if (existential_DOLVS == 0) return 0;
  long number = existential_DOLVS->CountGequal(value);
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::PDF_Entry0()
{
  long frequency = 0;
  vPrint("\nEnter frequency:\n");
  vScan("%li", &frequency);
  return PDF_Entry(frequency);
}


long vInterpreter2::PDF_Entry(long frequency)
{
  if (frequency < 0) return 0;
  if (frequency >= dolvs_pdf.Cols())
  {
    vPrint("DOLVS_PDF only has entries up to %li\n", dolvs_pdf.Cols() - 1);
    return 0;
  }
  long probability = dolvs_pdf(frequency);
  vPrint("entry = %li\n", probability);
  return 1;
}


long vInterpreter2::PDF_Probability0()
{
  long frequency = 0;
  vPrint("\nEnter frequency:\n");
  vScan("%li", &frequency);
  return PDF_Probability(frequency);
}


long vInterpreter2::PDF_Probability(long frequency)
{
  if (frequency < 0) return 0;
  if (frequency >= dolvs_pdf.Cols())
  {
    vPrint("DOLVS_PDF only has entries up to %li\n", dolvs_pdf.Cols() - 1);
    return 0;
  }
  double entry = dolvs_pdf(frequency);
  double max = dolvs_pdf.Cols() - 1;
  double factor = 100.0 / max;
  double probability = entry * factor;
  vPrint("probability = %lf\n", probability);
  return 1;
}


long vInterpreter2::TestLineSegmentSamples0()
{
  if (line_segments == 0) return 0;
  long location_diff, orientation_diff, length_diff;
  location_diff = orientation_diff = length_diff = 0;
  vPrint("\nEnter location_diff, orientation_diff, length_diff:\n");
  vScan("%li %li %li", &location_diff, &orientation_diff, &length_diff);
  return TestLineSegmentSamples(location_diff, orientation_diff, length_diff);
}


long vInterpreter2::TestLineSegmentSamples(long location_diff, long orientation_diff, 
                                            long length_diff)
{
  if (line_segments == 0) return 0;
  vector<vLineSegmentMatch> samples;
  long success = vLineSegmentSamples(line_segments, &samples, location_diff, 
                                      orientation_diff, length_diff);
  return success;
}


long vInterpreter2::SampleLineSegments0()
{
  if (line_segments == 0) return 0;
  long location_diff, orientation_diff, length_diff;
  location_diff = orientation_diff = length_diff = 0;
  vPrint("\nEnter location_diff, orientation_diff, length_diff:\n");
  vScan("%li %li %li", &location_diff, &orientation_diff, &length_diff);
  return SampleLineSegments(location_diff, orientation_diff, length_diff);
}


long vInterpreter2::SampleLineSegments(long location_diff, long orientation_diff, 
                                        long length_diff)
{
  if (line_segments == 0) return 0;
  vector<vLineSegmentMatch> * samples = new vector<vLineSegmentMatch>;
  long success = vLineSegmentSamples(line_segments, samples, location_diff, 
                                      orientation_diff, length_diff);
  if (success > 0)
  {
    vdelete(line_segments);
    line_segments = samples;
    LineSegmentImages(-1);
  }
  else
  {
    vPrint("Failed to sample line segments\n");
  }
  return success;
}


long vInterpreter2::FindSegment0()
{
  if (line_segments == 0) return 0;
  long row, col, o, l;
  row = col = o = l = 0;
  vPrint("\nEnter row, col, o, l:\n");
  vScan("%li %li %li %li", &row, &col, &o, &l);
  return FindSegment(row, col, o, l);
}


long vInterpreter2::FindSegment(long row, long col, long o, long l)
{
  if (line_segments == 0) return 0;
  long number = line_segments->size();
  long i;
  for (i = 0; i < number; i++)
  {
    vLineSegmentMatch & match = (*line_segments)[i];
    long segment_row = match.center_row;
    long segment_col = match.center_col;
    long segment_o = vRound(match.orientation * 24 / M_PI);
    long segment_l = vRound((match.length - 5) / 5);
    if ((row == segment_row) && (col == segment_col) &&
        (o == segment_o) && (l == segment_l))
    {
      vPrint("rank = %li\n", i);
      match.Print1();
    }
  }
  return 1;
}


long vInterpreter2::AverageLineMatchScore0()
{
  if (existential_DOLVS == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (database_input < 0) return 0;
  if (database_input >= number_of_inputs) return 0;

  vPrint("\nEnter number of tries\n");
  long tries = 0;
  vScan("%li", &tries);
  return AverageLineMatchScore(tries);
}


long vInterpreter2::AverageLineMatchScore(long tries)
{
  if (existential_DOLVS == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (database_input < 0) return 0;
  if (database_input >= number_of_inputs) return 0;

  long frames = inputs[database_input]->number_of_frames;
  static const long views = 4;
  long i;
  double sum = 0;
  vector<double> scores;
  for (i = 0; i < tries; i++)
  {
    long frame = vRandomLong2(0, frames-1);
    long view = vRandomLong2(0, views-1);
    SetCurrentInput(database_input);
    ViewRank(frame, view);
    ReadModelLines(frame, view);
    vdelete(line_match_frame_stats);
    line_match_frame_stats = new vLineMatchFrameStats(existential_DOLVS, 
                                                       model_straight_links,
                                                       frame, view);
    if (line_match_frame_stats == 0)
    {
      vPrint("Error in AverageLineMatchScore\n");
      break;
    }
    sum = sum + line_match_frame_stats->average;
    scores.push_back(line_match_frame_stats->average);
  }
  std::sort(scores.begin(), scores.end(), less<double>());
  double average = sum / tries;
  for (i = 0; i < tries; i++)
  {
    vPrint("Score %li = %lf\n", i, scores[i]);
  }
  vPrint("\naverage = %lf\n", average);
  return 1;
}


long vInterpreter2::CountBestSegments0()
{
  if (line_segments == 0) return 0;
  vector<vLineSegmentMatch> best_segments;
  vSelectBestSegments(line_segments, &best_segments);
  long number = best_segments.size();
  vPrint("number = %li\n", number);
  return 1;
}


long vInterpreter2::SavePointSet()
{
  if ((database_edge_input < 0) ||
      (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  const char * path_name = inputs[database_edge_input]->path_name;
  vdelete(point_sequence);
  point_sequence = new vPointSetSequence(sequence);
  char * output_name = vReplaceEnding2(path_name, ".point_set");
  long success = point_sequence->Save(output_name);
  if (success <= 0)
  {
    vPrint("Failed to save points into %s\n", output_name);
  }
  vdelete(output_name);
  return 1;
}


long vInterpreter2::LoadPointSet()
{
 if ((database_edge_input < 0) ||
     (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  const char * path_name = inputs[database_edge_input]->path_name;
  char * filename = vReplaceEnding2(path_name, ".point_set");
  vPointSetSequence * temp_sequence = vPointSetSequence::Load(filename);
  long success;
  if (temp_sequence == 0)
  {
    vPrint("Failed to load points from %s\n", filename);
    success = 0;
  }
  else
  {
    vdelete(point_sequence);
    point_sequence = temp_sequence;
    success = 1;
  }
  vdelete(filename);
  return success;
}


long vInterpreter2::TestPointSet0()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  return TestPointSet(frame);
}


long vInterpreter2::TestPointSet(long frame)
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;

  long frames = point_sequence->NumberOfFrames();
  if ((frame < 0) || (frame >= frames))
  {
    return 0;
  }
  long rows = point_sequence->Rows();
  long cols = point_sequence->Cols();

  grayscale_image * point_image = new grayscale_image(rows, cols);
  vWriteValue(point_image, (uchar) 0);
  vArray(uchar) point_rows = point_sequence->PointRows(frame);
  vArray(uchar) point_cols = point_sequence->PointCols(frame);
  long length = point_sequence->Length(frame);
  vWriteValue3(point_image, point_rows, point_cols, length, (uchar) 255);
  ShowImage(point_image);
  return 1;
}

  
long vInterpreter2::SaveLeftoverSet()
{
  if ((database_edge_input < 0) ||
      (database_edge_input >= number_of_inputs) ||
      (database_input < 0) ||
      (database_input >= number_of_inputs) ||
      (all_model_lines == 0) || 
      (point_sequence == 0))
  {
    return 0;
  }
  
  vVideoFile * model_edges = inputs[database_edge_input]->sequence;
  vVideoFile * model_images = inputs[database_input]->sequence;
  if (strcmp(model_edges->Type(), "vRLE_Sequence") != 0) return 0;
  if (strcmp(model_images->Type(), "vRLE_Sequence") != 0) return 0;
  long success = point_sequence->MarkLeftovers(model_images, model_edges,
                                               all_model_lines);
  if (success == 0)
  {
    vPrint("Failed to mark leftovers\n");
    return 0;
  }

  const char * path_name = inputs[database_edge_input]->path_name;
  char * output_name = vReplaceEnding2(path_name, ".point_set");
  success = point_sequence->Save(output_name);
  if (success <= 0)
  {
    vPrint("Failed to save points into %s\n", output_name);
  }
  vdelete(output_name);
  return 1;
}


long vInterpreter2::TestLeftoverSet0()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  return TestLeftoverSet(frame);
}


long vInterpreter2::TestLeftoverSet(long frame)
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;

  long frames = point_sequence->NumberOfFrames();
  if ((frame < 0) || (frame >= frames))
  {
    return 0;
  }
  long rows = point_sequence->Rows();
  long cols = point_sequence->Cols();

  grayscale_image * point_image = new grayscale_image(rows, cols);
  vWriteValue(point_image, (uchar) 0);
  vArray(uchar) point_rows = point_sequence->PointRows(frame);
  vArray(uchar) point_cols = point_sequence->PointCols(frame);
  long length = point_sequence->LeftoverLength(frame);
  vWriteValue3(point_image, point_rows, point_cols, length, (uchar) 255);
  ShowImage(point_image);
  return 1;
}


long vInterpreter2::OneWayChamfer()
{
  if (input == 0) return 0;
  if (database_input < 0) return 0;
  if (database_input >= number_of_inputs) return 0;
  const char * training_file = inputs[database_edge_input]->path_name;
  Blur();
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image = (grayscale_image *) input->current_image;
  vMatchScores * temp_scores = vOneWayChamferRanks(edge_image, training_file);
  if (temp_scores == 0)
  {
    vPrint("Failed to compute match scores\n");
    return 0;
  }
  vdelete(match_scores);
  match_scores = temp_scores;
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


long vInterpreter2::OneWayChamfer2()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image = (grayscale_image *) input->current_image;
  vMatchScores * temp_scores = vOneWayChamferRanks2(edge_image, point_sequence);
  if (temp_scores == 0)
  {
    vPrint("Failed to compute match scores\n");
    return 0;
  }
  vdelete(match_scores);
  match_scores = temp_scores;
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


long vInterpreter2::TestOneWayChamfer()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame and view\n");
  vScan("%li %li", &frame, &view);
  Blur();
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edge_image = (grayscale_image *) input->current_image;
  v3dMatrix<float> * dt = vCVDistanceTransform(edge_image);
  long temp_index = current_input_index;
  SetCurrentInput(database_edge_input);
  ViewRank(frame, view);
  ThresholdCurrentImage(1);
  SetCurrentInput(temp_index);
  grayscale_image * database_edges = (grayscale_image *) inputs[database_edge_input]->current_image;
  vector<long> rows, cols;
  vGetNonZeroPoints(database_edges, &rows, &cols);
  long * rows1 = &(rows[0]);
  long * cols1 = &(cols[0]);
  long number1 = rows.size();
//  double distance1 = vChamferFromDT(rows1, cols1, number1, dt);
  double distance1 = vChamferFromDT(&rows, &cols, dt);
  vPrint("distance0 = %lf\n", distance1);
  vArray(uchar) rows2 = point_sequence->PointRows(frame);
  vArray(uchar) cols2 = point_sequence->PointCols(frame);
  long number2 = point_sequence->Length(frame);

  vArray(long) rows3 = vnew(long, 65536);
  vArray(long) cols3 = vnew(long, 65536);
  long number3 = 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  vRLE_Sequence * rle_seq = (vRLE_Sequence *) sequence;
  vRLE_Gray * training_rle = (vRLE_Gray *) rle_seq->ReadRLEFrame(frame);
  vRLENonZeroPoints(training_rle, rows3, cols3, &number3);
  vRotatePointsRight(rows3, cols3, rows3, cols3, number3, rle_seq->Rows(), 
                      rle_seq->Cols(), 90 * view);

  distance1 = vChamferFromDT(rows3, cols3, number3, dt);
  vPrint("distance1 = %lf\n", distance1);

  v3dMatrix<float> * dt2 = vRotateRight(dt, 90 * (4 - view));
  double distance2 = vChamferFromDT(rows2, cols2, number2, dt2);
  vPrint("distance2 = %lf\n", distance2);
  vdelete(dt);
  vdelete(dt2);
  vdelete(training_rle);
  vdelete2(rows3);
  vdelete2(cols3);

  return 1;
}


long vInterpreter2::Chamfer1TopScores0()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (match_scores == 0) return 0;

  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return Chamfer1TopScores(number);
}


long vInterpreter2::Chamfer1TopScores(long number)
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (match_scores == 0) return 0;
  if (match_scores->NumberOfScores() < number) return 0;

  vector<long> database_frames(number);
  vector<long> database_views(number);
  vArray(vMatchScore) score_sorted = match_scores->GetScores();
  long i;
  for (i = 0; i < number; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }

  Blur();
  ScaleBoundingBox2(192, 256, 256);
  CannyDetector(canny_threshold);
  grayscale_image * edges = (grayscale_image *) input->current_image;
  vMatchScores * temp_scores = vOneWayChamfer2(edges, point_sequence,
                                                 &database_frames, &database_views);
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, match_scores);
  vdelete(match_scores);
  vdelete(temp_scores);
  match_scores = final_scores;

  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


// The next functions are used to compute and display the 
// model leftover points, i.e. points that were not used
// for the fitting of lines.
long vInterpreter2::ShowLeftoverModelPoints0()
{
  if (input == 0) return 0;
  if ((database_input < 0) || (database_input >= number_of_inputs) ||
      (database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  if (model_straight_links == 0) return 0;

  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view\n");
  vScan("%li %li", &frame, &view);
  return ShowLeftoverModelPoints(frame, view);
}


long vInterpreter2::ShowLeftoverModelPoints(long frame, long view)
{
  if (input == 0) return 0;
  if ((database_input < 0) || (database_input >= number_of_inputs) ||
      (database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  if (model_straight_links == 0) return 0;

  SetCurrentInput(database_edge_input);
  ViewRank(frame, view);
  grayscale_image * edges = (grayscale_image *) input->current_image->threshold(1);

  SetCurrentInput(database_input);
  ViewRank(frame, view);
  if (predicate_color_image(input->current_image) == 0) return 0;
  color_image * the_image = (color_image *) input->current_image;
  
  ShowImage(edges);
  grayscale_image * dilated = vDilateImage(edges, 1, 255);
  ShowImage(dilated);
  grayscale_image * link_ids = vLinkIDsB(the_image);

  vector<long> leftover_rows, leftover_cols;
  vLeftoverModelPoints(link_ids, edges, dilated, model_straight_links,
                        &leftover_rows, &leftover_cols);
  grayscale_image * leftover_image = new grayscale_image(the_image->Rows(), the_image->Cols());
  vWriteValue(leftover_image, (uchar) 0);
  vWriteValue2(leftover_image, &leftover_rows, &leftover_cols, (uchar) 255);
  ShowImage(leftover_image);
  if ((point_sequence != 0) && (view == 0))
  {
    point_sequence->MarkLeftover(frame, &leftover_rows, &leftover_cols);
  }

  vdelete(link_ids);

  return 1;
}


long vInterpreter2::ShowLeftoverTestPoints0()
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;
  long number;
  vPrint("\nEnter number of segments:\n");
  vScan("%li", &number);
  vPrint("Enter indices:\n");
  vector<long> indices(number);
  long i;
  for (i = 0; i < number; i++)
  {
    vScan("%li", &(indices[i]));
  }
  return ShowLeftoverTestPoints(&indices);
}


long vInterpreter2::ShowLeftoverTestPoints(vector<long> * segments)
{
  if (input == 0) return 0;
  if (line_segments == 0) return 0;

  vector<vLineSegmentLong> selected;
  long number = segments->size();
  long all_number = line_segments->size();
  long i;
  for (i = 0; i < number; i++)
  {
    long index = (*segments)[i];
    if (index < 0) continue;
    if (index >= all_number) continue;
    selected.push_back((*line_segments)[index].segment);
  }
  grayscale_image * edges = input->current_image->ToGrayCopy();
  long image_rows = edges->Rows();
  long image_cols = edges->Cols();
  vArray(long) rows;
  vArray(long) cols;
  long size = 0;
  vGetNonZeroPoints4(edges, &rows, &cols, &size);

  vArray(long) leftover_rows;
  vArray(long) leftover_cols;
  long leftover_size = 0;
  vLeftoverPoints(rows, cols, size, &selected,
                   &leftover_rows, &leftover_cols, &leftover_size);

  grayscale_image * new_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(new_image, (uchar) 0);
  vWriteValue3(new_image, leftover_rows, leftover_cols, leftover_size,
                (uchar) 255);
  ShowImage(new_image);
  vdelete2(edges);
  vdelete2(rows);
  vdelete2(cols);
  vdelete2(leftover_rows);
  vdelete2(leftover_cols);
  return 1;
}


long vInterpreter2::LineChamferScores0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (point_sequence == 0) return 0;

  long number = 0;
  vPrint("\nEnter number\n");
  vScan("%li", &number);
  return LineChamferScores(number);
}


long vInterpreter2::LineChamferScores(long number)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (point_sequence == 0) return 0;

  vLineChamferInfo info;
  info.all_model_lines = all_model_lines;
  info.dolvs = existential_DOLVS;
  info.input_lines = line_segments;
  info.point_sets = point_sequence;

  if (match_scores == 0) return 0;
  if (match_scores->NumberOfScores() < number) return 0;
  vector<long> database_frames(number);
  vector<long> database_views(number);
  vArray(vMatchScore) score_sorted = match_scores->GetScores();
  long i;
  for (i = 0; i < number; i++)
  {
    database_frames[i] = score_sorted[i].frame;
    database_views[i] = score_sorted[i].view;
  }

  ScaleBoundingBox2(192, 256, 256);
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * edges = vCannyDetector(the_image, (uchar) canny_threshold);
  vdelete(the_image);
  vMatchScores * temp_scores = vLineChamferScores(edges, &database_frames,
                                                    &database_views, &info);
  vMatchScores * final_scores = vMatchScores::Merge(temp_scores, match_scores);
  vdelete(match_scores);
  vdelete(temp_scores);
  match_scores = final_scores;

  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


// ChamferIndexOutliers is used to give results to George Kollios 
// for a publication. He wants the results of non-hand images
// using the chamfer indexing scheme.
long vInterpreter2::ChamferIndexOutliers0()
{
  long training_input = database_edge_input;
  if (input == 0) return 0;
  if ((training_input < 0) || (training_input >= number_of_inputs)) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[training_input];
  vVideoFile * training_sequence = info->sequence;
  if (strcmp(training_sequence->Type(), "vRLE_Sequence") != 0)
  {
    vPrint("Error, training input is not rle sequence\n");
    return 0;
  }

  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;

  long start = 0;
  vPrint("\nEnter index of first input\n");
  vScan("%li", &start);
  return ChamferIndexOutliers(start);
}


long vInterpreter2::ChamferIndexOutliers(long start)
{
  long training_input = database_edge_input;
  if (input == 0) return 0;
  if ((training_input < 0) || (training_input >= number_of_inputs)) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[training_input];
  vVideoFile * training_sequence = info->sequence;
  if (strcmp(training_sequence->Type(), "vRLE_Sequence") != 0)
  {
    vPrint("Error, training input is not rle sequence\n");
    return 0;
  }

  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if ((start <= database_edge_input) || (start >= number_of_inputs))
  {
    return 0;
  }

  long i;
  long number_of_frames = number_of_inputs - start;
  vMatrix<double> result(number_of_frames, embedding_dimensions);
  for (i = start; i < number_of_inputs; i++)
  {
    SetCurrentInput(i);
    CannyDetector(canny_threshold);
    ScaleBoundingBox2(192, 256, 256);
    ChamferDistances();
    long row = i - start;
    long col;
    for (col = 0; col < embedding_dimensions; col++)
    {
      result(row, col) = test_embedding->Matrix()[col];
    }
  }
  
  long success = result.WriteText("g:\\users\\athitsos\\sequences\\outliers\\out_distances.txt");
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter2::SubtractBackground0()
{
  if (background_model == 0) return 0;
  if (input == 0) return 0;
  double threshold = 0;
  vPrint("\nEnter threshold:\n");
  vScan("%lf", &threshold);
  return SubtractBackground(threshold);
}


long vInterpreter2::SubtractBackground(double threshold)
{
  if (background_model == 0) return 0;
  if (input == 0) return 0;

  color_image * the_image = input->current_image->ToRGBCopy();
  color_image * foreground = background_model->SubtractBackground(the_image, threshold);
  ShowImage(foreground);
  vdelete(the_image);
  return 1;
}


// SelectPixels uses the current image as a binary mask, and 
// then applies this mask to the image in the ring buffer that
// is at the selected index, so that the product of the mask
// and the buffer image is displayed.
long vInterpreter2::SelectPixels0()
{
  if (input == 0) return 0;
  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return SelectPixels(index);
}


long vInterpreter2::SelectPixels(long index)
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  browse_image(index);
  if (input->current_image == 0) return 0;
  v3dMatrix<double> * the_image = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * selected = vSelectPixels(the_image, binary);
  ShowImage(selected);
  vdelete(binary);
  vdelete(the_image);
  return 1;
}


// SelectBBox uses the current image as a binary mask, finds the
// bounding box of the non-zero area, and 
// then selects the corresponding pixels (that belong to that box)
// from the image in the ring buffer that
// is at the selected index.
long vInterpreter2::SelectBoundingBox0()
{
  if (input == 0) return 0;
  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return SelectBoundingBox(index);
}


long vInterpreter2::SelectBoundingBox(long index)
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vBoundingBox box = vBinaryBoundingBox(binary);
  browse_image(index);
  v3dMatrix<double> * the_image = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * selected = vCopyRectangle(the_image, box.left, box.right,
                                                   box.top, box.bottom);
  ShowImage(selected);
  vdelete(binary);
  vdelete(the_image);
  return 1;
}




long vInterpreter2::FillUpHoles()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vFillUpHoles2(binary, binary);
  ShowImage(binary);
  return 1;
}


long vInterpreter2::DrawProjections()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vAxisProjection axis_projection(binary);
  grayscale_image * projections = axis_projection.DrawProjections();
  ShowImage(projections);
  axis_projection.Reverse();
  projections = axis_projection.DrawProjections();
  ShowImage(projections);
  vdelete(binary);
  return 1;
}


long vInterpreter2::SaveChamfer1Index()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;
  class_sequence_information * info = (class_sequence_information *) inputs[database_edge_input];
  const char * training_base = info->path_name;
  char * distances_base = vMergeStrings2(training_base, ".chamfer_index");
  
  char * picks_file = vMergeStrings2(distances_base, ".random");
  vdelete2(distances_base);

  FILE * fp = fopen(picks_file, vFOPEN_READ);
  if (fp == 0) 
  {
    vPrint("Failed to open picks file %s\n", picks_file);
    return 0;
  }

  v3dMatrix<long> * reference_frames = v3dMatrix<long>::Read(fp);
  v3dMatrix<long> * reference_rotations = v3dMatrix<long>::Read(fp);
  fclose(fp);
  vdelete(picks_file);
  
  vVideoFile * database_edges = inputs[database_edge_input]->sequence;
  if (strcmp(database_edges->Type(), "vRLE_Sequence") != 0)
  {
    vdelete(reference_frames);
    vdelete(reference_rotations);
    return 0;
  }
  long result = vSaveChamferIndexIM((vRLE_Sequence *) database_edges,
                                     point_sequence, reference_frames, 
                                     reference_rotations);
  vdelete(reference_frames);
  vdelete(reference_rotations);
  return result;
}


long vInterpreter2::ChamferMatching0()
{
  if (input == 0) return 0;
  long input2 = 0;
  vPrint("\nEnter input2:\n");
  vScan("%li", &input2);
  return ChamferMatching(input2);
}


// Chamfer distances between current input and input2.
// Nothing is normalized.
long vInterpreter2::ChamferMatching(long input2)
{
  if (input == 0) return 0;
  if ((input2 < 0) || (input2 >= number_of_inputs)) return 0;

  grayscale_image * image1 = input->current_image->ToGrayCopy();
  grayscale_image * image2 = inputs[input2]->current_image->ToGrayCopy();
  if ((image1->Rows() != image2->Rows()) || 
      (image1->Rows() != image2->Rows()))
  {
    vPrint("the two images have different sizes\n");
    return 0;
  }
  v3dMatrix<float> * dt1 = vCVDistanceTransform(image1);
  v3dMatrix<float> * dt2 = vCVDistanceTransform(image2);
  double distance1 = vChamferFromDT(image1, dt2);
  double distance2 = vChamferFromDT(image2, dt1);
  vPrint("distance1 = %lf\ndistance2 = %lf\nsum = %lf\n",
          distance1, distance2, distance1 + distance2);
  vdelete(image1);
  vdelete(image2);
  vdelete(dt1);
  vdelete(dt2);
  return 1;
}


// Testing for triples of database images whether they obey the triangle inequality.
// Stats gave violations 33 out of 10000 times.
long vInterpreter2::ChamferTriangleInequality()
{
  if (input == 0) return 0;
  long times = 0;
  vPrint("\nInput times:\n");
  vScan("%li", &times);

  long number_of_frames = input->number_of_frames;
  long views = 4;

  long i;
  long counter = 0;
  for (i = 0; i < times; i++)
  {
    long frame1 = vRandomLong2(0, number_of_frames-1);
    long frame2 = vRandomLong2(0, number_of_frames-1);
    long frame3 = vRandomLong2(0, number_of_frames-1);
    long view1 = vRandomLong2(0, views-1);
    long view2 = vRandomLong2(0, views-1);
    long view3 = vRandomLong2(0, views-1);

    JumpToFrame(frame1);
    RotateRight(view1);
    ThresholdCurrentImage(1);
    vPrint("i = %li, counter = %li\n", i, counter);
    vPrint("frame1 = %li, view1 = %li\n", frame1, view1);
    grayscale_image * gray1 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt1 = vCVDistanceTransform(gray1);

    JumpToFrame(frame2);
    RotateRight(view2);
    ThresholdCurrentImage(1);
    vPrint("frame2 = %li, view2 = %li\n", frame2, view2);
    grayscale_image * gray2 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt2 = vCVDistanceTransform(gray2);

    JumpToFrame(frame3);
    RotateRight(view3);
    ThresholdCurrentImage(1);
    vPrint("frame3 = %li, view3 = %li\n", frame3, view3);
    grayscale_image * gray3 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt3 = vCVDistanceTransform(gray3);

    double distance1 = vChamferFromDT(gray1, dt2);
    double distance2 = vChamferFromDT(gray2, dt3);
    double distance3 = vChamferFromDT(gray1, dt3);
    vPrint("distance1 = %.1lf, distance2 = %.1lf, distance3 = %.1lf\n",
            distance1, distance2, distance3);

    if (distance1 + distance2 < distance3)
    {
      vPrint("Triangle inequality violated\n");
      counter++;
    }

    vdelete(gray1);
    vdelete(dt1);
    vdelete(gray2);
    vdelete(dt2);
    vdelete(gray3);
    vdelete(dt3);
  }
  vPrint("\nThe triangle inequality was violated %li out of %li times\n\n",
          counter, times);
  return 1;
}


// Testing for triples of database images whether they obey the triangle inequality.
// This is different than the first test, because here we test the distances
// A->B, C->B, and A->C, as opposed to testing the distances A->B, B->C, A->C.
// Maybe this test is more appropriate for the technique I described in the 
// submission to CVPR 2003. Stats gave violations 200 out of 10000 times.
long vInterpreter2::ChamferTriangleInequality2()
{
  if (input == 0) return 0;
  long times = 0;
  vPrint("\nInput times:\n");
  vScan("%li", &times);

  long number_of_frames = input->number_of_frames;
  long views = 4;

  long i;
  long counter = 0;
  for (i = 0; i < times; i++)
  {
    long frame1 = vRandomLong2(0, number_of_frames-1);
    long frame2 = vRandomLong2(0, number_of_frames-1);
    long frame3 = vRandomLong2(0, number_of_frames-1);
    long view1 = vRandomLong2(0, views-1);
    long view2 = vRandomLong2(0, views-1);
    long view3 = vRandomLong2(0, views-1);

    JumpToFrame(frame1);
    RotateRight(view1);
    ThresholdCurrentImage(1);
    vPrint("i = %li, counter = %li\n", i, counter);
    vPrint("frame1 = %li, view1 = %li\n", frame1, view1);
    grayscale_image * gray1 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt1 = vCVDistanceTransform(gray1);

    JumpToFrame(frame2);
    RotateRight(view2);
    ThresholdCurrentImage(1);
    vPrint("frame2 = %li, view2 = %li\n", frame2, view2);
    grayscale_image * gray2 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt2 = vCVDistanceTransform(gray2);

    JumpToFrame(frame3);
    RotateRight(view3);
    ThresholdCurrentImage(1);
    vPrint("frame3 = %li, view3 = %li\n", frame3, view3);
    grayscale_image * gray3 = input->current_image->ToGrayCopy();
    v3dMatrix<float> * dt3 = vCVDistanceTransform(gray3);

    double distance1 = vChamferFromDT(gray1, dt2);
    double distance2 = vChamferFromDT(gray3, dt2);
    double distance3 = vChamferFromDT(gray1, dt3);
    vPrint("distance1 = %.1lf, distance2 = %.1lf, distance3 = %.1lf\n",
            distance1, distance2, distance3);

    if (distance3 < vAbs(distance1 - distance2))
    {
      vPrint("Triangle inequality violated\n");
      counter++;
    }

    vdelete(gray1);
    vdelete(dt1);
    vdelete(gray2);
    vdelete(dt2);
    vdelete(gray3);
    vdelete(dt3);
  }
  vPrint("\nThe triangle inequality was violated %li out of %li times\n\n",
          counter, times);
  return 1;
}


long vInterpreter2::TestChamfer1Index()
{
  if (reference_images == 0) return 0;
  if (database_edge_input >= number_of_inputs) return 0;
  char * output_file = vReplaceEnding2(inputs[database_edge_input]->path_name,
                                        ".chamfer1_index");
  v3dMatrix<float> * matrix = vLoadMatrixAsUshorts<float>(output_file);
  if (matrix == 0)
  {
    vPrint("Failed to load result from %s\n", output_file);
  }
  vdelete2(output_file);

  long i;
  for (i = 0; i < 4; i++)
  {
    vPrint("frame = %li, view = %li\n",
            reference_images->Matrix2(0)[i][0],
            reference_images->Matrix2(0)[i][1]);
  }

  while(1)
  {
    long frame1, view1;
    frame1 = view1 = 0;
    vPrint("Enter frame1, view1\n");
    vScan("%li %li", &frame1, &view1);
    if (frame1 < 0) break;
    long index = frame1 * 4 + view1;
    vArray2(float) entries = matrix->Matrix2(0);
    for (i = 0; i < 4; i++)
    {
      vPrint("Entry %li = %f\n", i, entries[index][i]);
    }
  }

  vdelete(matrix);
  return 1;
}


// Sets the prefered method of hand segmentation to
// bounding box segmentation
long vInterpreter2::BoundingBoxSegmentation()
{
  segmentation_method = 0;
  return 1;
}


// Sets the prefered method of hand segmentation to
// fine segmentation
long vInterpreter2::FineSegmentation()
{
  segmentation_method = 1;
  return 1;
}


  
// This function assumes that the currently displayed image
// is a binary image indicating the location of the segmented 
// hand. Based on that image, it uses the current frame
// of the video sequence to extract the hand pixels (producing
// a segmented hand image), and also to extract the edge image
// of the hand. In the edge image it artificially adds a line
// for the palm cutoff point (which, this function assumes, is
// stored in the palm_cutoff member variable of vInterpreter2).
long vInterpreter2::HandAndEdgeImages()
{
  if (input == 0) return 0;
  if (original_video_frame == 0) 
  {
    if (input->sequence == 0) return 0;
    original_video_frame = input->sequence->ReadFrame(input->current_frame);
  }

  if ((original_video_frame != 0) && (input->sequence != 0))
  {
    vdelete(original_video_frame);
    original_video_frame = input->sequence->ReadFrame(input->current_frame);
  }

  if (input == 0) return 0;

  vdelete(right_hand_image);
  vdelete(original_hand_image);
  vdelete(hand_mask);
  vdelete(right_hand_frame);

  grayscale_image * temp_hand_mask = input->current_image->ToGrayCopy();
  vBoundingBox bbox = vBinaryBoundingBox(temp_hand_mask);
  if (segmentation_method == 0)
  {
    // If segmentation_method is 0, then we want to segment the whole
    // bounding box of the hand. The disadvantage here is that we may include
    // more noise. The advantage is that if the system works well in this case
    // it is a very positive experimental result, and it is more ready for 
    // real-life applications, where fine segmentation may be hard.
    vDrawFilledRectangle1(temp_hand_mask, bbox.top, bbox.left, 
                           bbox.bottom, bbox.right, (uchar) 255);
  }
  original_hand_image = (color_image *) vSelectPixels(original_video_frame,
                                                   temp_hand_mask);
  vLineSegmentLong palm_line;

  if (left_hand == 0)
  {
    hand_mask = temp_hand_mask->ToGrayCopy();
    right_hand_frame = (color_image *) original_video_frame->ToRGBCopy();
    palm_line = palm_cutoff;
  }
  else
  {
    hand_mask = (grayscale_image *) vSwitchTopBottom(temp_hand_mask);
    right_hand_frame = (color_image *) vSwitchTopBottom(original_video_frame);
    palm_line = LineTopBottom(palm_cutoff, original_video_frame->Rows(),
                              original_video_frame->Cols());
  }
  vdelete(temp_hand_mask);
  right_hand_image = (color_image *) vSelectPixels(right_hand_frame, hand_mask);

  centered_palm_cutoff = vNormalizeSegment(&palm_cutoff, &bbox, 192, 256);

  grayscale_image * full_gray_image = right_hand_frame->ToGrayCopy();
  grayscale_image * full_edge_image = vCannyDetector(full_gray_image, (uchar) canny_threshold);

  vdelete(hand_edge_image);
  hand_edge_image = (grayscale_image *) vSelectPixels(full_edge_image, hand_mask);
  vdelete(full_edge_image);

  // Before we get orientations, we should blur the image.
//  v3dMatrix<double> * blurred_gray_temp = vBlur(full_gray_image);
//  grayscale_image * blurred_gray = blurred_gray_temp->ToGray();
//  vdelete(blurred_gray_temp);

//  v3dMatrix<double> * all_orientations = vEdgeOrientations(blurred_gray);
//  vdelete(full_gray_image);
//  vdelete(blurred_gray);

  v3dMatrix<double> * all_orientations = vEdgeOrientations(full_gray_image);
  vPrint("old orientation, no blurring\n");
  vdelete(full_gray_image);

  vdelete(hand_orientation_image);
  hand_orientation_image = vSelectPixels(all_orientations, hand_edge_image);
  vdelete(all_orientations);

  // Draw palm cutoff line.
  if (palm_cutoff.row1 >= 0)
  {
    vDrawLine1(hand_edge_image, palm_line.row1, palm_line.col1, 
                palm_line.row2, palm_line.col2, (uchar) 255);
    double orientation = vAngle(palm_line.col2 - palm_line.col1,
                                 palm_line.row2 - palm_line.row1);
    vDrawLine1(hand_orientation_image, palm_line.row1, palm_line.col1, 
                palm_line.row2, palm_line.col2, orientation);
  }

  ShowImage(hand_edge_image->ToGrayCopy());
  ShowImage(original_hand_image->ToRGBCopy());

  vdelete(hand_edge_scaled);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  hand_edge_scaled = (grayscale_image *) vSparseScaleBoundingBox7(hand_edge_image, bbox, 192, 128, 128, 256, 256);

  vdelete(hand_orientation_scaled);
  hand_orientation_scaled = vSparseScaleBoundingBox7(hand_orientation_image, bbox, 192, 
                                                      128, 128, 256, 256);

  return 1;
}


long vInterpreter2::SetEdgeImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;
  vdelete(hand_edge_image);
  hand_edge_image = input->current_image->ToGrayCopy();
  return 1;
}


// A way to find the hand based on skin detection
long vInterpreter2::SkinHand()
{
  if (input == 0) return 0;
  vdelete(original_video_frame);
  original_video_frame = input->current_image->ToRGBCopy();

  DetectSkin();
  ThresholdCurrentImage(skin_threshold);
  Blur();
  Blur();
  Blur();
  BiggestComponent();
  FillUpHoles();
  KeepPalm2();
  SelectPixels(10);
  HandAndEdgeImages();

  return 1;
}


long vInterpreter2::LineNumberStats0()
{
  if (all_model_lines == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  long number_of_frames;
  number_of_frames = 0;
  vPrint("\nEnter number_of_frames:\n");
  vScan("%li", &number_of_frames);
  return LineNumberStats(number_of_frames);
}


long vInterpreter2::LineNumberStats(long number_of_views)
{
  if (all_model_lines == 0) return 0;
  if (existential_DOLVS == 0) return 0;

  return  vLineNumberStatistics(all_model_lines, existential_DOLVS, 
                                 number_of_views);  
}


long vInterpreter2::StartLabelingCutoffs()
{
  if (database_input >= number_of_inputs) return 0;
  SetCurrentInput(database_input);
  JumpToFrame(0);
  SetSequenceStep(12);
  mouse_click_response = CutoffLabelResponse;
  if (palm_cutoffs == 0) 
  {
    palm_cutoffs = new v3dMatrix<uchar>(2236, 4, 1);
    vWriteValue(palm_cutoffs, (uchar) 0);
  }
  DrawCurrentCutoff();
  return 1;
}


long vInterpreter2::StopLabelingCutoffs()
{
  mouse_click_response = pixel_info_response;
  return 1;
}


long vInterpreter2::ProcessCutoffClick(long row, long col)
{
  static long click_state = 0;
  static long previous_row = -1000;
  static long previous_col = -1000;

  long row_diff = (row - previous_row);
  long col_diff = (col - previous_col);
  long square = row_diff * row_diff + col_diff * col_diff;
  if (square < 100) return 0;
  previous_row = row;
  previous_col = col;

  if (palm_cutoffs == 0) return 0;
  if (database_input >= number_of_inputs) return 0;
  SetCurrentInput(database_input);
  long current_frame = input->current_frame;
  long remainder = current_frame % 12;
  vPrint("current frame = %li, remainder = %li, click_state = %li\n",
          current_frame, remainder, click_state);
  if (remainder != 0)
  {
    current_frame = current_frame - remainder;
    JumpToFrame(current_frame);
  }
  vArray2(uchar) lines = palm_cutoffs->Matrix2(0);
  long index = current_frame / 12;

  vPrint("current frame = %li, index = %li\n", current_frame, index);
  switch(click_state)
  {
  case 0:
    lines[index][0] = (uchar) row;
    lines[index][1] = (uchar) col;
    break;

  case 1:
    lines[index][2] = (uchar) row;
    lines[index][3] = (uchar) col;
    DrawCurrentCutoff();
    break;

  case 2:
    JumpToFrame(current_frame + 12);
    index = input->current_frame / 12;
    DrawCurrentCutoff();
    long success = palm_cutoffs->Write("G:\\users\\athitsos\\sequences\\glove_data\\training\\training7.cutoffs");
    if (success == 0)
    {
      vPrint("Failed to save to file\n");
    }
    else
    {
      vPrint("Saved cutoffs\n");
    }
    break;
  }

  click_state = (click_state + 1) % 3;
  return 1;
}


long vInterpreter2::LoadCutoffs()
{
  vdelete(palm_cutoffs);
  palm_cutoffs = v3dMatrix<uchar>::Read("G:\\users\\athitsos\\sequences\\glove_data\\training\\training7.cutoffs");
  if (palm_cutoffs == 0)
  {
    vPrint("Failed to read palm cutoffs\n");
    return 0;
  }
  return 1;
}


long vInterpreter2::DrawCurrentCutoff()
{
  if (input == 0) return 0;
  if (palm_cutoffs == 0) return 0;
  vArray2(uchar) lines = palm_cutoffs->Matrix2(0);
  long index = input->current_frame / 12;
  if (lines[index][0] != 0)
  {
    DrawLine(lines[index][0], lines[index][1], 
             lines[index][2], lines[index][3], 255);
  }
  return 1;
}


long vInterpreter2::NextCutoff()
{
  if (input == 0) return 0;
  NextInSequence();
  DrawCurrentCutoff();
  return 1;
}


long vInterpreter2::PreviousCutoff()
{
  if (input == 0) return 0;
  PreviousInSequence();
  DrawCurrentCutoff();
  return 1;
}


long vInterpreter2::Diameter()
{
  if (input == 0) return 0;
  grayscale_image * image = input->current_image->ToGrayCopy();
  double diameter = vDiameter(image);
  vPrint("\ndiameter = %lf\n", diameter);
  vdelete(image);
  return 1;
}


long vInterpreter2::ExtendCutoffs()
{
  if (database_input >= number_of_inputs) return 0;

  vdelete(extended_cutoffs);
  extended_cutoffs = vExtendCutoffs(palm_cutoffs, 
                                     inputs[database_input]->sequence);
  long success = extended_cutoffs->Write("G:\\users\\athitsos\\sequences\\glove_data\\training\\training7.x_cutoffs");
  if (success == 0)
  {
    vPrint("Failed to save to file\n");
    return 0;
  }
  else
  {
    vPrint("Saved extended cutoffs\n");
    return 1;
  }
}


long vInterpreter2::LoadExtendedCutoffs()
{
  vdelete(extended_cutoffs);
  extended_cutoffs = v3dMatrix<uchar>::Read("G:\\users\\athitsos\\sequences\\glove_data\\training\\training7.x_cutoffs");
  if (extended_cutoffs == 0)
  {
    vPrint("Failed to read palm cutoffs\n");
    return 0;
  }
  return 1;
}


long vInterpreter2::DrawExtendedCutoff0()
{
  if (extended_cutoffs == 0) return 0;
  long view = 0;
  vPrint("\nEnter view:\n");
  vScan("%li", &view);
  return DrawExtendedCutoff(view);
}


long vInterpreter2::DrawExtendedCutoff(long view)
{
  if (extended_cutoffs == 0) return 0;
  long frame = input->current_frame;
  long views = extended_cutoffs->Rows();
  if (frame * 4 + view >= views) return 0;
  ViewRank(frame, view);
  vArray2(uchar) extended = extended_cutoffs->Matrix2(0);
  long index = frame * 4 + view;
  long row1 = extended[index][0];
  long col1 = extended[index][1];
  long row2 = extended[index][2];
  long col2 = extended[index][3];
  DrawLine(row1, col1, row2, col2, 255);
  return 1;
}


long vInterpreter2::PalmScores()
{
  if (input == 0) return 0;
  if (extended_cutoffs == 0) return 0;

  vdelete(match_scores);
  match_scores = vPalmCutoffScores(extended_cutoffs, &centered_palm_cutoff);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


long vInterpreter2::TestCutoffCentering()
{
  if (input == 0) return 0;
  grayscale_image * binary = input->current_image->ToGrayCopy();
  vBoundingBox box = vBinaryBoundingBox(binary);
  centered_palm_cutoff = vNormalizeSegment(&palm_cutoff, &box, 192, 256);
  vdelete(binary);
  ScaleBoundingBox2(192, 256, 256);
  DrawLine(centered_palm_cutoff.row1, centered_palm_cutoff.col1, 
           centered_palm_cutoff.row2, centered_palm_cutoff.col2, 255);
  return 1;
}


long vInterpreter2::TestLineGrid0()
{
  long d_samples, o_samples, l_samples;
  d_samples = o_samples = l_samples = 0;
  vPrint("\nEnter d_samples, o_samples, l_samples:\n");
  vScan("%li %li %li", &d_samples, &o_samples, &l_samples);
  return TestLineGrid(d_samples, o_samples, l_samples);
}


long vInterpreter2::TestLineGrid(long d_samples, long o_samples, 
                                  long l_samples)
{
  vLineGrid grid(d_samples, o_samples, l_samples);
  grid.PrintValues();
  return 1;
}


long vInterpreter2::GridFraction0()
{
  double new_grid_fraction = 0;
  vPrint("\nEnter new grid fraction:\n");
  vScan("%lf", &new_grid_fraction);
  return GridFraction(new_grid_fraction);
}


long vInterpreter2::GridFraction(double new_grid_fraction)
{
  if ((new_grid_fraction < 0) || (new_grid_fraction > 1))
  {
    vPrint("Bad grid fraction value: %lf\n", new_grid_fraction);
    return  0;
  }
  grid_fraction = new_grid_fraction;
  return 1;
}


long vInterpreter2::DrawLineMatch()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (input == 0) return 0;
  if (number_of_inputs < 2) return 0;

  long test_input = 1;

  long link_id, line, r, g, b;
  link_id = line = r = g = b = 0;
  vPrint("\nEnter link_id, line, r, g, b:\n");
  vScan("%li %li %li %li %li", &link_id, &line, &r, &g, &b);

  vLineSegment model_segment = GetModelLine(link_id, line);
  if (model_segment.row1 < 0) return 0;

  vector<uchar> values;
  values.push_back((uchar) r);
  values.push_back((uchar) g);
  values.push_back((uchar) b);

  SetCurrentInput(test_input);
  color_image * input_image = input->current_image->ToRGBCopy();
  SetCurrentInput(database_input);
  color_image * model_image = input->current_image->ToRGBCopy();

  // Replace black background with white background;
  vArray2(uchar) R2 = model_image->GetR2();
  vArray2(uchar) G2 = model_image->GetG2();
  vArray2(uchar) B2 = model_image->GetB2();
  long rows = model_image->Rows();
  long cols = model_image->Cols();
  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      if ((R2[row][col] == 0) &&
          (G2[row][col] == 0) &&
          (B2[row][col] == 0))
      {
        R2[row][col] = 255;
        G2[row][col] = 255;
        B2[row][col] = 255;
      }
    }
  }

  existential_DOLVS->MakeInputSummary(line_segments);
  existential_DOLVS->ProcessLinkSegment(model_segment);
  existential_DOLVS->RankInputSegments(line_segments);

  vLineSegmentMatch & segment_match = (*line_segments)[0];
  vLineSegmentLong segment = segment_match.segment;
  vDrawLine(input_image, 
             segment.row1, segment.col1, 
             segment.row2, segment.col2, &values);
  vDrawLine(input_image, 
             segment.row1+1, segment.col1, 
             segment.row2+1, segment.col2, &values);
  vDrawLine(input_image, 
             segment.row1-1, segment.col1, 
             segment.row2-1, segment.col2, &values);
  vDrawLine(input_image, 
             segment.row1, segment.col1+1, 
             segment.row2, segment.col2+1, &values);
  vDrawLine(input_image, 
             segment.row1, segment.col1-1, 
             segment.row2, segment.col2-1, &values);
  vDrawLine(input_image, 
             segment.row1+2, segment.col1, 
             segment.row2+2, segment.col2, &values);
  vDrawLine(input_image, 
             segment.row1-2, segment.col1, 
             segment.row2-2, segment.col2, &values);
  vDrawLine(input_image, 
             segment.row1, segment.col1+2, 
             segment.row2, segment.col2+2, &values);
  vDrawLine(input_image, 
             segment.row1, segment.col1-2, 
             segment.row2, segment.col2-2, &values);
  SetCurrentInput(test_input);
  ShowImage(input_image);

  vDrawLine(model_image, 
             (long) model_segment.row1, (long) model_segment.col1, 
             (long) model_segment.row2, (long) model_segment.col2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1-1, (long) model_segment.col1, 
             (long) model_segment.row2-1, (long) model_segment.col2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1+1, (long) model_segment.col1, 
             (long) model_segment.row2+1, (long) model_segment.col2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1, (long) model_segment.col1-1, 
             (long) model_segment.row2, (long) model_segment.col2-1, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1, (long) model_segment.col1+1, 
             (long) model_segment.row2, (long) model_segment.col2+1, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1-2, (long) model_segment.col1, 
             (long) model_segment.row2-2, (long) model_segment.col2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1+2, (long) model_segment.col1, 
             (long) model_segment.row2+2, (long) model_segment.col2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1, (long) model_segment.col1-2, 
             (long) model_segment.row2, (long) model_segment.col2-2, &values);
  vDrawLine(model_image, 
             (long) model_segment.row1, (long) model_segment.col1+2, 
             (long) model_segment.row2, (long) model_segment.col2+2, &values);


  SetCurrentInput(database_input);
  ShowImage(model_image);

  return 1;
}


long vInterpreter2::FrontDistance()
{
  if (sequence_match_stats == 0) return 0;
  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  double front_distance = sequence_match_stats->GetFrontDistance(frame);
  vPrint("front_distance = %lf\n", front_distance);
  return 1;
}
// I should end commenting for CVPR 2003 here.


// Saves all chamfer distances between every shape/viewpoint combination
// in the database and every database image. It assumes that the
// directed distances have already been computed (using the functions
// Save_All_IM_Distances() and Save_All_MI_Distances() to compute and 
// save the distances, or Load_All_IM_Distances() and Load_All_MI_Distances()
// to load them from disk.
long vInterpreter2::Save_All_CD_Distances()
{
  // check if directed distances have been computed.
  if (all_im_distances == 0) return 0;
  if (all_mi_distances == 0) return 0;

  long rows = all_im_distances->Rows();
  long cols = all_im_distances->Cols();
  vdelete(all_cd_distances);
  // Create the new matrix that will store all chamfer distances.
  all_cd_distances = new v3dMatrix<uchar>(rows, cols, 1);

  vArray2(uchar) cd = all_cd_distances->Matrix2(0);
  vArray2(uchar) im = all_im_distances->Matrix2(0);
  vArray2(uchar) mi = all_mi_distances->Matrix2(0);
  
  // Add the directed distances to obtain the undirected distance.
  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      long im_distance = im[row][col];
      long mi_distance = mi[row][col];
      double im_double = uchar_to_double_mi_convert(im_distance);
      double mi_double = uchar_to_double_mi_convert(mi_distance);
      double cd_double = im_double + mi_double;
      cd[row][col] = double_to_uchar_cd_convert(cd_double);
    }
  }

  long success = all_cd_distances->Write(cd_distances_file);
  if (success <= 0)
  {
    vPrint("Failed to save all_cd_distances to %s\n", cd_distances_file);
  }
  else
  {
    vPrint("Saved all_cd_distances to %s\n", cd_distances_file);
  }
  return success;
  //  return 0;
}


// Saves all directed chamfer distances from each
// image (shape/viewpoint/image-plane-orientation combination)
// in the database to each shape/viewpoint combination in the 
// database. We assume that database_edge_input is set
// to the right sequence.
long vInterpreter2::Save_All_MI_Distances()
{
  if (number_of_inputs <= database_edge_input) return 0;
  if (point_sequence == 0) return 0;

  // Get the video sequence of database edge images.
  vVideoFile * database_edges = inputs[database_edge_input]->sequence;
  vdelete(all_mi_distances);
  // Create the matrix that will store all distances, and compute
  // those distances.
  all_mi_distances = vAll_MI_Distances(database_edges, point_sequence);
  if (all_mi_distances == 0)
  {
    vPrint("Failed to compute all_mi_distances\n");
    return 0;
  }
  else 
  {
    vPrint("Computed all_mi_distances\n");
  }

  // Save the distances to the appropriate file.
  long success = all_mi_distances->Write(mi_distances_file);
  if (success <= 0)
  {
    vPrint("Failed to save all_mi_distances to %s\n", mi_distances_file);
  }
  else
  {
    vPrint("Saved all_mi_distances to %s\n", mi_distances_file);
  }
  return success;
//  return 0;
}


// Similar to Save_All_IM_Distances, but here it saves distances
// from each shape/viewpoint combination to each database image.
long vInterpreter2::Save_All_IM_Distances()
{
  if (number_of_inputs <= database_edge_input) return 0;
  if (point_sequence == 0) return 0;

  vVideoFile * database_edges = inputs[database_edge_input]->sequence;
  vdelete(all_im_distances);
  all_im_distances = vAll_IM_Distances(database_edges, point_sequence);
  if (all_im_distances == 0)
  {
    vPrint("Failed to compute all_im_distances\n");
    return 0;
  }
  else 
  {
    vPrint("Computed all_im_distances\n");
  }

  long success = all_im_distances->Write(im_distances_file);
  if (success <= 0)
  {
    vPrint("Failed to save all_im_distances to %s\n", im_distances_file);
  }
  else
  {
    vPrint("Saved all_im_distances to %s\n", im_distances_file);
  }
  return success;
//  return 1;
}


// Load precomputed chamfer distances (computed with Save_All_CD_Distances)
// from every shape/viewpoint combination to every database image.
long vInterpreter2::Load_All_CD_Distances()
{
  vdelete(all_cd_distances);
  all_cd_distances = v3dMatrix<uchar>::Read(cd_distances_file);
  if (all_cd_distances == 0)
  {
    vPrint("Failed to read all_cd_distances from %s\n", cd_distances_file);
    return 0;
  }
  else
  {
    vPrint("Read all_cd_distances");
    return 1;
  }
}


// Load precomputed chamfer distances (computed with Save_All_CD_Distances)
// from every shape/viewpoint combination to every database image. This
// function loads directed chamfer distances, see comments on 
// Save_All_MI_Distances.
long vInterpreter2::Load_All_MI_Distances()
{
  vdelete(all_mi_distances);
  all_mi_distances = v3dMatrix<uchar>::Read(mi_distances_file);
  if (all_mi_distances == 0)
  {
    vPrint("Failed to read all_mi_distances from %s\n", mi_distances_file);
    return 0;
  }
  else
  {
    vPrint("Read all_mi_distances");
    return 1;
  }
}


// Load precomputed chamfer distances (computed with Save_All_CD_Distances)
// from every shape/viewpoint combination to every database image. This
// function loads directed chamfer distances, see comments on 
// Save_All_IM_Distances.
long vInterpreter2::Load_All_IM_Distances()
{
  vdelete(all_im_distances);
  all_im_distances = v3dMatrix<uchar>::Read(im_distances_file);
  if (all_im_distances == 0)
  {
    vPrint("Failed to read all_im_distances from %s\n", im_distances_file);
    return 0;
  }
  else
  {
    vPrint("Read all_im_distances");
    return 1;
  }
}


// Used to verify that the distances stored in all_cd_distances
// are correct. The user specifies a shape/viewpoint combination
// with frame1, and a database image with frame2/view2, and
// the system prints out the stored distance.
long vInterpreter2::Show_CD_Entry()
{
  if (all_cd_distances == 0) return 0;
  vPrint("\nInput frame1, frame2, view2: ");
  long frame1, frame2, view2;
  frame1 = frame2 = view2 = 0;
  vScan("%li %li %li", &frame1, &frame2, &view2);
  vArray2(uchar) distances = all_cd_distances->Matrix2(0);
  long row = frame1 / 12;
  long col = frame2 * 4 + view2;
  long uchar_distance = distances[row][col];
  double distance = uchar_to_double_cd_convert(uchar_distance);
  vPrint("uchar = %li, double = %.2lf\n", uchar_distance, distance);
  return 1;
}


// Same as Show_CD_Entry, but for directed distances from 
// database image to shape/viewpoint combination.
long vInterpreter2::Show_MI_Entry()
{
  if (all_mi_distances == 0) return 0;
  vPrint("\nInput frame1, frame2, view2: ");
  long frame1, frame2, view2;
  frame1 = frame2 = view2 = 0;
  vScan("%li %li %li", &frame1, &frame2, &view2);
  vArray2(uchar) distances = all_mi_distances->Matrix2(0);
  long row = frame1 / 12;
  long col = frame2 * 4 + view2;
  long uchar_distance = distances[row][col];
  double distance = uchar_to_double_mi_convert(uchar_distance);
  vPrint("uchar = %li, double = %.2lf\n", uchar_distance, distance);
  return 1;
}


// Same as Show_CD_Entry, but for directed distances from 
// shape/viewpoint combination to database image.
long vInterpreter2::Show_IM_Entry()
{
  if (all_im_distances == 0) return 0;
  vPrint("\nInput frame1, frame2, view2: ");
  long frame1, frame2, view2;
  frame1 = frame2 = view2 = 0;
  vScan("%li %li %li", &frame1, &frame2, &view2);
  vArray2(uchar) distances = all_im_distances->Matrix2(0);
  long row = frame1 / 12;
  long col = frame2 * 4 + view2;
  long uchar_distance = distances[row][col];
  double distance = uchar_to_double_mi_convert(uchar_distance);
  vPrint("uchar = %li, double = %.2lf\n", uchar_distance, distance);
  return 1;
}



long vInterpreter2::TestIndexAdjustment()
{
  long frame, view, index;
  frame = view = index = 0;
  vPrint("\nEnter frame, view, index:\n");
  vScan("%li %li %li", &frame, &view, &index);
  long adjusted = vTestIndexAdjustment(frame, view, index);
  vPrint("adjusted_index = %li\n", adjusted);
  return 1;
}


// This function is used to display the precomputed chamfer distance
// between two database images. It is used to verify that we can correctly
// look up this distance based on the distance between the shape/viewpoint
// combination frame3/view3 corresponding to frame1/view1 and the database 
// image frame4/view4 that is as far from frame3/view3 as frame2/view2
// is from frame1/view1.
long vInterpreter2::Show_CD_Entry4()
{
  if (all_cd_distances == 0) return 0;
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  vPrint("\nEnter frame1, view1, frame2, view2: ");
  vScan("%li %li %li %li", &frame1, &view1, &frame2, &view2);
  vMatrix<double> distances = vGet_CD_Distances(frame1, view1, 
                                                  all_cd_distances);
  ulong index = frame2 * 4 + view2;
  if (index >= (ulong) distances.Cols()) return 0;
  double distance = distances(index);
  vPrint("distance = %.2lf\n", distance);
  return 1;
}


// Assuming that the current input is the database edge input,
// this function computes directly and displays the chamfer distance
// between two database images.
long vInterpreter2::ChamferMatching40()
{
  if (input == 0) return 0;
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  vPrint("\nEnter frame1, view1, frame2, view2: ");
  vScan("%li %li %li %li", &frame1, &view1, &frame2, &view2);
  
  return ChamferMatching4(frame1, view1, frame2, view2);
}


long vInterpreter2::ChamferMatching4(long frame1, long view1, 
                                      long frame2, long view2)
{
  if (input == 0) return 0;
  JumpToFrame(frame1);
  RotateRight(view1);
  ThresholdCurrentImage(1);
  grayscale_image * gray1 = input->current_image->ToGrayCopy();

  JumpToFrame(frame2);
  RotateRight(view2);
  ThresholdCurrentImage(1);
  grayscale_image * gray2 = input->current_image->ToGrayCopy();

  v3dMatrix<float> * dt1 = vCVDistanceTransform(gray1);
  v3dMatrix<float> * dt2 = vCVDistanceTransform(gray2);
  double distance1 = vChamferFromDT(gray1, dt2);
  double distance2 = vChamferFromDT(gray2, dt1);
  vPrint("distance1 = %lf\ndistance2 = %lf\nsum = %lf\n",
          distance1, distance2, distance1 + distance2);

  vdelete(gray1);
  vdelete(dt1);
  vdelete(gray2);
  vdelete(dt2);
  return 1;
}


// Create pairs of a shape/viewpoint combination and a database image 
// combination. Notice that the number of database images is 4 times
// the number of database frames (for each database frame, we can 
// obtain four images by rotating the frame by 0, 90, 180 and 270 degrees).
// For each of those pairs, we look up the chamfer distance between the
// corresponding images, as stored in all_cd_distances.
long vInterpreter2::Make_CD_Pairs()
{
  if (all_cd_distances == 0) return 0;
  vdelete(cd_pairs);
  vdelete(cd_pair_distances);
  cd_pairs = vRandom_CD_Pairs(1000000);
  cd_pair_distances = vCD_Pair_Distances(cd_pairs, all_cd_distances);
  return 1;
}


// We save the distances between random pairs of database images (see
// comments on Make_CD_Pairs for more information) on a file.
long vInterpreter2::Save_CD_Pairs()
{
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;
  FILE * fp = fopen(cd_pair_file, vFOPEN_WRITE);
  if (fp == 0)
  { 
    vPrint("Failed to open %s\n", cd_pair_file);
    return 0;
  }
  long success;
  success = cd_pairs->Write(fp);
  if (success <= 0)
  {
    vPrint("Failed to save cd_pairs\n");
    fclose(fp);
    return 0;
  }
  success = cd_pair_distances->Write(fp);
  fclose(fp);
  if (success <= 0)
  {
    vPrint("Failed to save cd_pair_distances\n");
    return 0;
  }
  vPrint("Saved cd_pairs and cd_pair_distances\n");
  return 1;
}


// We load precomputed distances between database image pairs
// that were computed with Make_CD_Pairs and saved on disk 
// with Save_CD_Pairs.
long vInterpreter2::Load_CD_Pairs()
{
  vdelete(cd_pairs);
  vdelete(cd_pair_distances);
  FILE * fp = fopen(cd_pair_file, vFOPEN_READ);
  if (fp == 0)
  { 
    vPrint("Failed to open %s\n", cd_pair_file);
    return 0;
  }
  cd_pairs = v3dMatrix<long>::Read(fp);
  if (cd_pairs <= 0)
  {
    vPrint("Failed to load cd_pairs\n");
    fclose(fp);
    return 0;
  }

  cd_pair_distances = v3dMatrix<double>::Read(fp);
  fclose(fp);
  if (cd_pair_distances <= 0)
  {
    vPrint("Failed to load cd_pair_distances\n");
    return 0;
  }
  vPrint("Loaded cd_pairs and cd_pair_distances\n");
  vPrint("cd_pair_distances: %li rows, %li cols\n", 
          cd_pair_distances->Rows(), cd_pair_distances->Cols());
  vPrint("cd_pairs: %li rows, %li cols\n", 
          cd_pairs->Rows(), cd_pairs->Cols());
  return 1;
}


// This function is used to verify that the chamfer distances
// stored in cd_pair_distances are correct. The user enters
// an index and the information and distance for the 
// corresponding pair is displayed.
long vInterpreter2::Show_CD_Pair()
{
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  long index = 0;
  vPrint("\nEnter index: ");
  vScan("%li", &index);

  vArray2(long) pairs = cd_pairs->Matrix2(0);
  vArray(double) distances = cd_pair_distances->Matrix();
  if ((index < 0) || (index >= cd_pairs->Cols()))
  {
    vPrint("index %li is bad\n", index);
    return 0;
  }

  frame1 = pairs[0][index];
  view1 = pairs[1][index];
  frame2 = pairs[2][index];
  view2 = pairs[3][index];
  double distance = distances[index];
  vPrint("cd pair %li: (%li, %li) and (%li, %li). Distance = %.2lf\n",
          index, frame1, view1, frame2, view2, distance);

  return 1;
}


// Calculates the average distortion obtained by using a reference
// set consisting of a single database image, which is specified
// by the user. This is used to play around and see what kinds
// of average distortions we get for different database images.
long vInterpreter2::CalculateSTD()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long rows = cd_pair_distances->Rows();
  long cols = cd_pair_distances->Cols();
  vMatrix<double> estimated_distances(rows, cols);
  vWriteValue(&estimated_distances, (double) 0);

  long candidate = 0;
  long factor = 0;
  vPrint("\nEnter candidate, factor: ");
  vScan("%li %li", &candidate, &factor);

  double mean = 0;
  double std = 0;
  vCalculateSTD(all_cd_distances, cd_pairs, cd_pair_distances,
                 &estimated_distances, candidate, &mean, &std,
                 factor);

  vPrint("mean = %lf, std = %lf\n", mean, std);
  return 1;
}


// Calculate the average distortions obtained by using each
// possible candidate reference image as a single reference
// image. This is used to experiment and see what kind
// of average distortions we get.
long vInterpreter2::CalculateSTDs()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long rows = cd_pair_distances->Rows();
  long cols = cd_pair_distances->Cols();
  vMatrix<double> estimated_distances(rows, cols);
  vWriteValue(&estimated_distances, (double) 0);

  long factor = 0;
  vPrint("\nEnter factor: ");
  vScan("%li", &factor);


  vMatrix<double> distortions = 
    vCalculateSTDs(all_cd_distances, cd_pairs, cd_pair_distances,
                           &estimated_distances, factor);

  long min_std_index, max_std_index;
  min_std_index = max_std_index = 0;
  double min_std = color_imageRowMin3(&distortions, 1, &min_std_index);
  double max_std = color_imageRowMax3(&distortions, 1, &max_std_index);
  vPrint("std[%li] = %lf\nstd[%li] = %lf\n", min_std_index, min_std,
          max_std_index, max_std);
  long min_mean_index, max_mean_index;
  min_mean_index = max_mean_index = 0;
  double min_mean = color_imageRowMin3(&distortions, 0, &min_mean_index);
  double max_mean = color_imageRowMax3(&distortions, 0, &max_mean_index);
  vPrint("mean[%li] = %lf\nmean[%li] = %lf\n", min_mean_index, min_mean,
          max_mean_index, max_mean);
  return 1;
}


// Chooses a set of reference images greedily, so that they minimize
// average distortion, and saves information about the images chosen
// and the distortions attained to a file.
long vInterpreter2::ChooseBasis1()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, factor;
  number = factor = 0;
  vPrint("\nEnter size of basis, factor: ");
  vScan("%li %li", &number, &factor);
  vMatrix<double> basis = vChooseBasis1(all_cd_distances, cd_pairs, cd_pair_distances,
                                          factor, number);
  return 1;
}


// Chooses a set of reference images randomly,
// and saves information about the images chosen
// and the average distortions attained to a file.
long vInterpreter2::ChooseBasis2()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, factor;
  number = factor = 0;
  vPrint("\nEnter size of basis, factor: ");
  vScan("%li %li", &number, &factor);
  vMatrix<double> basis = vChooseBasis2(all_cd_distances, cd_pairs, cd_pair_distances,
                                          factor, number);
  return 1;
}


// Chooses a set of reference images greedily, so that they minimize
// worst-case distortion, and saves information about the images chosen
// and the worst-case distortions attained to a file.
long vInterpreter2::ChooseBasis3()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, factor;
  number = factor = 0;
  vPrint("\nEnter size of basis, factor: ");
  vScan("%li %li", &number, &factor);
  vMatrix<double> basis = vChooseBasis3(all_cd_distances, cd_pairs, cd_pair_distances,
                                          factor, number);
  return 1;
}


// Chooses a set of reference images randomly (like ChooseBasis2), 
// and saves information about the images chosen
// and the worst-case distortions attained to a file.
long vInterpreter2::ChooseBasis4()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, factor;
  number = factor = 0;
  vPrint("\nEnter size of basis, factor: ");
  vScan("%li %li", &number, &factor);
  vMatrix<double> basis = vChooseBasis4(all_cd_distances, cd_pairs, cd_pair_distances,
                                          factor, number);
  return 1;
}


// Chooses a set of reference images greedily, so that they maximize
// classification accuracy on a test set, and saves information about 
// the images chosen and classification accuracies attained to a file.
// The test set is specified indirectly, it is the video sequence on which
// SaveReferenceImages was called.
long vInterpreter2::ChooseBasis5()
{
  if (all_cd_distances == 0) return 0;
  if (input == 0) return 0;

  long basis_size, frames_to_use;
  basis_size = frames_to_use = 0;
  vPrint("\nEnter size of basis, frames to use: ");
  vScan("%li %li", &basis_size, &frames_to_use);

  v3dMatrix<float> * reference_distances = 
    v3dMatrix<float>::Read("g:\\users\\athitsos\\sequences\\glove_data\\training\\gw2003\\reference_distances");
  vMatrix<double> basis = vChooseBasis5(all_cd_distances, reference_distances,
                                          input->path_name,
                                          frames_to_use, basis_size);
  vdelete(reference_distances);
  return 1;
}


// Chooses a set of reference Timages greedily, so that they minimize
// average distortion, and saves information about the images chosen
// and the distortions attained to a file.
long vInterpreter2::ChooseBasis6()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, factor, shape;
  number = factor = 0;
  vPrint("\nEnter size of basis, factor, shape: ");
  vScan("%li %li %li", &number, &factor, &shape);
  vMatrix<double> basis = vChooseBasis6(all_cd_distances, cd_pairs, cd_pair_distances,
                                          factor, number, shape);
  return 1;
}


// Chooses a set of reference images randomly,
// and saves information about the images chosen
// and the average distortions attained to a file.
// The difference between this function and ChooseBasis2
// is that here we only consider as candidate reference
// images database images that come from a particular shape.
// This allows bases chosen using this function to be 
// compared with bases chosen using ChooseBasis6.
long vInterpreter2::ChooseBasis7()
{
  if (all_cd_distances == 0) return 0;
  if (cd_pairs == 0) return 0;
  if (cd_pair_distances == 0) return 0;

  long number, shape;
  number = shape = 0;
  vPrint("\nEnter size of basis, shape: ");
  vScan("%li %li", &number, &shape);
  vMatrix<double> basis = vChooseBasis7(all_cd_distances, cd_pairs, cd_pair_distances,
                                          number, shape);
  return 1;
}


// Create a set of reference images to be used for computing approximate
// chamfer distances, and compute and store all the necessary information
// to compute scores based on those distances using ChamferIndexScores0.
long vInterpreter2::MakeLipschitzBasis()
{
  if (database_edge_input == 0) return 0;

  long basis, index;
  basis = index = 0;
  vPrint("\nEnter basis, index: ");
  vScan("%li %li", &basis, &index);
  reference_start = 0;
  embedding_dimensions = index+1;
  vdelete(training_embeddings);
  training_embeddings = 0;
  DeleteDistanceTransforms();
  vdelete(reference_images);
  reference_images = 0;

//  if (all_cd_distances == 0) return 0;
//  long success = vMakeLipschitzBasis(all_cd_distances, basis, index,
//                                      &training_embeddings,
//                                      &reference_images);

  long success = vMakeLipschitzBasis3(basis, index, &reference_images);
  LoadTrainingEmbedding();

  return success;
}


// This function should be called before calling ChooseBasis5(),
// to precompute chamfer distances between a set of test images
// and all candidate reference images.
long vInterpreter2::SaveReferenceDistances()
{
  if (point_sequence == 0) return 0;
  if (input == 0) return 0;
  vVideoFile * sequence = input->sequence;
  vMatrix<float> distances = vSaveReferenceDistances(sequence, point_sequence);
  distances.Write("g:\\backup_distances");
  distances.Write("g:\\users\\athitsos\\sequences\\glove_data\\training\\gw2003\\reference_distances");
  distances.Write("r:\\backup_distances");
  vPrint("Saved distances successfully\n");
  return 1;
}


long vInterpreter2::OldSampleSpherePoints()
{
  double degrees = 0;
  vPrint("\nEnter degrees: ");
  vScan("%lf",  &degrees);
  vector<vPoint3> points;
  vOldSampleSpherePoints(degrees, &points);
  vPrint("Number of points = %li\n", (long) points.size());
  return 1;
}


// Samples points from the surface of a sphere, prints the points
// and their number.
long vInterpreter2::OldSampleSpherePoints2()
{
  double double1 = 0;
  vector<vPoint3> samples;
  vPrint("\nEnter degrees: ");
  vScan("%lf", &double1);
  vOldSampleSpherePoints(double1, &samples);
  long i;
  long size = samples.size();
  for (i = 0; i < size; i++)
  {
    vPoint3 & result = samples[i];
    vPrint("%li: (%lf, %lf, %lf)\n", i, result.x, result.y, result.z);
  }
  vPrint("size = %li\n", size);
  return 1;
}


// Samples points from the surface of a sphere, shows their number.
// Points are represented by 2 coordinates (on sphere surface).
long vInterpreter2::OldSampleViewpoints()
{
  double degrees = 0;
  vPrint("\nEnter degrees: ");
  vScan("%lf",  &degrees);
  vector<vPoint2> points;
  vOldSampleViewpoints(degrees, &points);
  vPrint("Number of points = %li\n", (long) points.size());
  return 1;
}


// Samples points from the surface of a sphere, prints the points
// and their number.
// Points are represented by 2 coordinates (on sphere surface).
long vInterpreter2::OldSampleViewpoints2()
{
  double double1 = 0;
  vector<vPoint2> samples;
  vPrint("\nEnter degrees: ");
  vScan("%lf", &double1);
  vOldSampleViewpoints(double1, &samples);
  long i;
  long size = samples.size();
  for (i = 0; i < size; i++)
  {
    vPoint2 & result = samples[i];
    vPrint("%li: (%lf, %lf)\n", i, result.col, result.row);
  }
  vPrint("size = %li\n", size);
  return 1;
}


// Samples points from the surface of a sphere, prints the points.
// Points are represented by 2 coordinates (on sphere surface).
// Same as SampleViewpoints2, but the printing format here is 
// different (appropriate to be saved as a text file, to be used
// by save_views).
long vInterpreter2::OldSampleViewpoints3()
{
  double double1 = 0;
  vector<vPoint2> samples;
  vPrint("\nEnter degrees: ");
  vScan("%lf", &double1);
  vOldSampleViewpoints(double1, &samples);
  long i;
  long size = samples.size();
  vPrint("\n");
  for (i = 0; i < size; i++)
  {
    vPoint2 & result = samples[i];
    vPrint("%lf %lf\n", result.col, result.row);
  }
  return 1;
}


// Samples points from the surface of a sphere, shows their number.
// Points are represented by 2 coordinates (on sphere surface).
long vInterpreter2::PoserSampleViewpoints()
{
  double degrees = 0;
  vPrint("\nEnter degrees: ");
  vScan("%lf",  &degrees);
  vector<vPoint2> points;
  vPoserSampleViewpoints(degrees, &points);
  vPrint("Number of points = %li\n", (long) points.size());
  return 1;
}


// Samples points from the surface of a sphere, prints the points
// and their number.
// Points are represented by 2 coordinates (on sphere surface).
long vInterpreter2::PoserSampleViewpoints2()
{
  double double1 = 0;
  vector<vPoint2> samples;
  vPrint("\nEnter degrees: ");
  vScan("%lf", &double1);
  vPoserSampleViewpoints(double1, &samples);
  long i;
  long size = samples.size();
  for (i = 0; i < size; i++)
  {
    vPoint2 & result = samples[i];
    vPrint("%li: (%lf, %lf)\n", i, result.col, result.row);
  }
  vPrint("size = %li\n", size);
  return 1;
}


// Samples points from the surface of a sphere, prints the points.
// Points are represented by 2 coordinates (on sphere surface).
// Same as SampleViewpoints2, but the printing format here is 
// different (appropriate to be saved as a text file, to be used
// by save_views).
long vInterpreter2::PoserSampleViewpoints3()
{
  double double1 = 0;
  vector<vPoint2> samples;
  vPrint("\nEnter degrees: ");
  vScan("%lf", &double1);
  vPoserSampleViewpoints(double1, &samples);
  long i;
  long size = samples.size();
  vPrint("\n");
  for (i = 0; i < size; i++)
  {
    vPoint2 & result = samples[i];
    vPrint("%lf %lf\n", result.col, result.row);
  }
  return 1;
}


// My algorithm
long vInterpreter2::EnclosingCircle()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  color_image * color = input->current_image->ToRGBCopy();
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  long number = 0;
  vGetNonZeroPoints4(gray, &rows, &cols, &number);
  vCircleLong circle = vMinEnclosingCircle3(rows, cols, number);
  ShowImage(color);
  DrawCircle(circle.center_row, circle.center_col, circle.radius, 255);
  vPrint("center = (%li, %li), radius = %li\n", 
          circle.center_row, circle.center_col, circle.radius);
  vdelete(gray);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


// OpenCV's buggy algorithm, do not rely on it.
long vInterpreter2::EnclosingCircle2()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  color_image * color = input->current_image->ToRGBCopy();
  vArray(double) rows = vZero(double);
  vArray(double) cols = vZero(double);
  long number = 0;
  vGetNonZeroPoints4(gray, &rows, &cols, &number);
  vCircle circle = vcvMinEnclosingCircle(rows, cols, number);
  ShowImage(color);
  DrawCircle((long) circle.center_row, (long) circle.center_col, (long) circle.radius, 255);
  vPrint("center = (%.1lf, %.1lf), radius = %.1lf\n", 
          circle.center_row, circle.center_col, circle.radius);
  vdelete(gray);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


// Used to measure the speed of enclosing circle.
long vInterpreter2::EnclosingCircleLoop()
{
  if (input == 0) return 0;
  long repetitions = 0;
  vPrint("\nEnter number of repetitions: ");
  vScan("%li", &repetitions);
  grayscale_image * gray = input->current_image->ToGrayCopy();
  color_image * color = input->current_image->ToRGBCopy();
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  long number = 0;
  vGetNonZeroPoints4(gray, &rows, &cols, &number);
  vCircleLong circle;
  long i;
  for (i = 0; i < repetitions; i++)
  {
    circle = vMinEnclosingCircle3(rows, cols, number);
  }
  ShowImage(color);
  DrawCircle(circle.center_row, circle.center_col, circle.radius, 255);
  vPrint("center = (%li, %li), radius = %li\n", 
          circle.center_row, circle.center_col, circle.radius);
  vdelete(gray);
  vdelete2(rows);
  vdelete2(cols);
  return 1;
}


long vInterpreter2::ConvexHull()
{
  if (input == 0) return 0;
  grayscale_image * gray = input->current_image->ToGrayCopy();
  vArray(long) rows = vZero(long);
  vArray(long) cols = vZero(long);
  vArray(long) hull_rows = vZero(long);
  vArray(long) hull_cols = vZero(long);
  long number = 0;
  long hull_number = 0;

  vGetNonZeroPoints4(gray, &rows, &cols, &number);
  vConvexHull(rows, cols, number, &hull_rows, &hull_cols, &hull_number);
  vPrint("\nnumber of hull vertices = %li\n", hull_number);

  vector<uchar> cross_values(3);
  color_image * color = input->current_image->ToRGBCopy();
  cross_values[0] = 0;
  cross_values[1] = 0;
  cross_values[2] = 255;

  long i;
  for (i = 0; i < hull_number; i++)
  {
    vDrawCrossv(color, hull_rows[i], hull_cols[i], 1, 7, &cross_values);
  }

  ShowImage(color);
  vdelete2(rows);
  vdelete2(cols);
  vdelete2(hull_rows);
  vdelete2(hull_cols);
  vdelete(gray);
  return 1;
}


long vInterpreter2::RotateImage()
{
  if (input == 0) return 0;
  if (input->current_image == 0)
  {
    return 0;
  }
  long center_row, center_col;
  double degrees;
  center_row = center_col = 0;
  degrees = (double) 0;
  vPrint("\nEnter center_row, center_col, degrees: ");
  vScan("%li %li %lf", &center_row, &center_col, &degrees);
  if ((center_row < 0) || (center_col < 0))
  {
    center_row = input->current_image->Rows() / 2;
    center_col = input->current_image->Cols() / 2;
  }

  double_image * the_image = new double_image (input->current_image);
  v3dMatrix<float> * sources = vRotationTemplatef(the_image->Rows(), the_image->Cols(),
                                                    center_row, center_col, degrees);
  ShowImage(sources);
  v3dMatrix<double> * rotated = rotate_image_exact_degrees(the_image, center_row, center_col, degrees);
  vdelete(the_image);
  ShowImage(rotated);
  return 1;
}


// Save bounding box information for each frame in a video.
// This is probably a one-use routine, I need it to check
// bounding boxes on a sequence of synthetic hand images
// (viewpoints_2.5.rle)
long vInterpreter2::SaveBoundingBoxes()
{
  if (input == 0) return 0;
  vVideoFile * sequence = input->sequence;
  long frames = sequence->NumberOfFrames();
  const char * filename = sequence->GetFilename();

  char * output_name = vReplaceEnding2(filename, "_bbox.txt");
  FILE * fp = fopen(output_name, vFOPEN_WRITE);
  if (fp == 0) 
  {
    vPrint("Failed to open %s\n", output_name);
    vdelete2(output_name);
    return 0;
  }

  long frame;
  vPrint("\n");
  for (frame = 0; frame < frames; frame++)
  {
    grayscale_image * the_image = sequence->ReadFrameGray(frame);
    if (the_image == 0)
    {
      vPrint("\n\nError on frame %li\n\n", frame);
      break;
    }
    vBoundingBox box = vBinaryBoundingBox(the_image);
    fprintf(fp, "%li %li %li %li %li\r\n", frame, 
            box.top, box.bottom, box.left, box.right);
    vdelete(the_image);
    vPrint("Done with frame %li\r", frame);
  }

  vPrint("\n");
  vPrint("Done\n");
  fclose(fp);
  vdelete2(output_name);
  return 1;
}


// Save minimum enclosign circle information for each frame in a video.
// This is probably a one-use routine, I need it to check
// MECs on a sequence of synthetic hand images
// (viewpoints_2.5.rle)
long vInterpreter2::SaveMECs()
{
  if (input == 0) return 0;
  vVideoFile * sequence = input->sequence;
  long frames = sequence->NumberOfFrames();
  const char * filename = sequence->GetFilename();

  char * output_name = vReplaceEnding2(filename, "_mec.txt");
  FILE * fp = fopen(output_name, vFOPEN_WRITE);
  if (fp == 0) 
  {
    vPrint("Failed to open %s\n", output_name);
    vdelete2(output_name);
    return 0;
  }

  long frame;
  vPrint("\n");
  for (frame = 0; frame < frames; frame++)
  {
    grayscale_image * the_image = sequence->ReadFrameGray(frame);
    if (the_image == 0)
    {
      vPrint("\n\nError on frame %li\n\n", frame);
      break;
    }
    vCircleLong circle = vMinEnclosingCircle(the_image);
    fprintf(fp, "%li %li %li %li\r\n", frame, 
            circle.center_row, circle.center_col, circle.radius);
    vdelete(the_image);
    vPrint("Done with frame %li\r", frame);
  }

  vPrint("\n");
  vPrint("Done\n");
  fclose(fp);
  vdelete2(output_name);
  return 1;
}


long vInterpreter2::NormalizeMEC1()
{
  exit_error("error: this function must be reimplemented\n");
  if (input == 0) return 0;
  long radius = 0;
  vPrint("\nEnter radius: ");
  vScan("%li", &radius);

  general_image * the_image = input->current_image;
//  vImageWrap * wrap = wrim(the_image);
//  general_image * normalized = wrap->NormalizeMEC1(radius);

  
  
  //  general_image * normalized = wrim(the_image).NormalizeMEC1(radius);
//  ShowImage(normalized);
  return 1;
}


long vInterpreter2::NormalizeMEC3()
{
  exit_error("error: this function must be reimplemented\n");
  if (input == 0) return 0;
  long radius = 0, center_row = 0, center_col = 0;
  vPrint("\nEnter center_row, center_col, radius: ");
  vScan("%li %li %li", &center_row, &center_col, &radius);

  general_image * the_image = input->current_image;
//  general_image * normalized = wrim(the_image)->NormalizeMEC3(center_row, center_col, radius);





  //    general_image * normalized = wrim(the_image).NormalizeMEC3(center_row, center_col, radius);
  //ShowImage(normalized);
  return 1;
}


long vInterpreter2::Binary()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;

//  vImageWrap wrap(the_image);
//  grayscale_image * binary = wrap.Binary();
  grayscale_image * binary = wrim(the_image).Binary();
  ShowImage(binary);
  return 1;
}


long vInterpreter2::OldQuaternionViewTest()
{
  double x_angle, y_angle, z_angle;
  x_angle = y_angle = z_angle = 0;

  vPrint("\nEnter x_angle, y_angle, z_angle: ");
  vScan("%lf %lf %lf", &x_angle, &y_angle, &z_angle);

  vQuaternion q = vOldViewQuaternion(x_angle, y_angle, z_angle);
  q.Print("q = ");
  double new_x, new_y, new_z;
  new_x = new_y = new_z = 0;
  vOldQuaternionView(q, &new_x, &new_y, &new_z);
  vPrint("(%.1lf, %.1lf, %.1lf)", new_x, new_y, new_z);
  return 1;
}


long vInterpreter2::TrajectorySampleTest()
{
  vHandPose pose1, pose2;
  pose1.shape_id = pose2.shape_id = 0;
  long samples = 0;
  vPrint("\nEnter samples, orientation1, orientation2\n");
  vScan("%li %lf %lf %lf %lf %lf %lf", &samples,
         &pose1.camera_x, &pose1.camera_y, &pose1.orientation,
         &pose2.camera_x, &pose2.camera_y, &pose2.orientation);

  vector<vHandPose> * trajectory = vLinearTrajectory(pose1, pose2, samples);
  if (trajectory == 0) 
  {
    vPrint("error\n");
    return 0;
  }

  long i;
  long number = trajectory->size();
  for (i = 0; i < number; i++)
  {
    vHandPose pose = (*trajectory)[i];
    vPrint("%li, %li, (%.1lf %.1lf %.1lf)\n", i, pose.shape_id,
            pose.camera_x, pose.camera_y, pose.orientation);
  }
  vdelete(trajectory);
  return 1;
}


long vInterpreter2::TimeSlowTrajectorySampling()
{
  vHandPose pose1, pose2;
  pose1.shape_id = pose2.shape_id = 0;
  long samples = 0;
  long times = 0;
  vPrint("\nEnter times, samples, orientation1, orientation2\n");
  vScan("%li %li %lf %lf %lf %lf %lf %lf", &times, &samples,
         &pose1.camera_x, &pose1.camera_y, &pose1.orientation,
         &pose2.camera_x, &pose2.camera_y, &pose2.orientation);

  long i;
  for (i = 0; i < times - 1; i++)
  {
    vector<vHandPose> * trajectory = vLinearTrajectorySlow(pose1, pose2, samples);
    if (trajectory == 0) 
    {
      vPrint("error\n");
      return 0;
    }
    vdelete(trajectory);
  }

  vector<vHandPose> * trajectory = vLinearTrajectorySlow(pose1, pose2, samples);
  long number = trajectory->size();
  for (i = 0; i < number; i++)
  {
    vHandPose pose = (*trajectory)[i];
    vPrint("%li, %li, (%.1lf %.1lf %.1lf)\n", i, pose.shape_id,
            pose.camera_x, pose.camera_y, pose.orientation);
  }
  vdelete(trajectory);
  return 1;
}


long vInterpreter2::TimeTrajectorySampling()
{
  vHandPose pose1, pose2;
  pose1.shape_id = pose2.shape_id = 0;
  long samples = 0;
  long times = 0;
  vPrint("\nEnter times, samples, orientation1, orientation2\n");
  vScan("%li %li %lf %lf %lf %lf %lf %lf", &times, &samples,
         &pose1.camera_x, &pose1.camera_y, &pose1.orientation,
         &pose2.camera_x, &pose2.camera_y, &pose2.orientation);

  long i;
  for (i = 0; i < times - 1; i++)
  {
    vector<vHandPose> * trajectory = vLinearTrajectory(pose1, pose2, samples);
    if (trajectory == 0) 
    {
      vPrint("error\n");
      return 0;
    }
    vdelete(trajectory);
  }

  vector<vHandPose> * trajectory = vLinearTrajectory(pose1, pose2, samples);
  long number = trajectory->size();
  for (i = 0; i < number; i++)
  {
    vHandPose pose = (*trajectory)[i];
    vPrint("%li, %li, (%.1lf %.1lf %.1lf)\n", i, pose.shape_id,
            pose.camera_x, pose.camera_y, pose.orientation);
  }
  vdelete(trajectory);
  return 1;
}


long vInterpreter2::CreateDatabase()
{
  if (input == 0) return 0;
  vVideoFile * sequence = input->sequence;
  const char * type = sequence->Type();
  if (strcmp(type, "vRLE_Sequence") != 0)
  {
    vPrint("Error, the input is not of type vRLE_Sequence\n");
    return 0;
  }
  vdelete(database);
  database = vOldDatabase2::Load((vRLE_Sequence *) sequence);
  if (database == 0)
  {
    vPrint("Error in creating database\n");
    return 0;
  }
  else
  {
    vPrint("Created database successfully\n");
    return 1;
  }
}


long vInterpreter2::PoseImage()
{
  if (input == 0) return 0;
  if (database == 0) return 0;
  vHandPose pose;
  vPrint("\nEnter shape_id, x, y, z: ");
  vScan("%li %lf %lf %lf", &(pose.shape_id), 
         &(pose.camera_x), &(pose.camera_y), &(pose.orientation));
  color_image * pose_image = database->PoseImage(pose);
  ShowImage(pose_image);
  return 1;
}


long vInterpreter2::DatabaseImage3()
{
  if (input == 0) return 0;
  if (database == 0) return 0;

  long shape, viewpoint, view;
  shape = viewpoint = view = 0;
  vPrint("\nEnter shape, viewpoint, view:\n");
  vScan("%li %li %li", &shape, &viewpoint, &view);
  color_image * the_image = database->GetImage3(shape, viewpoint, view);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::DatabaseImage2()
{
  if (input == 0) return 0;
  if (database == 0) return 0;

  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);
  color_image * the_image = database->GetImage2(frame, view);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::PoseEdges()
{
  if (input == 0) return 0;
  if (database == 0) return 0;
  vHandPose pose;
  vPrint("\nEnter shape_id, x, y, z: ");
  vScan("%li %lf %lf %lf", &(pose.shape_id), 
         &(pose.camera_x), &(pose.camera_y), &(pose.orientation));
  grayscale_image * pose_image = database->PoseEdges(pose);
  ShowImage(pose_image);
  return 1;
}


long vInterpreter2::DatabaseEdges3()
{
  if (input == 0) return 0;
  if (database == 0) return 0;

  long shape, viewpoint, view;
  shape = viewpoint = view = 0;
  vPrint("\nEnter shape, viewpoint, view:\n");
  vScan("%li %li %li", &shape, &viewpoint, &view);
  grayscale_image * the_image = database->GetEdges3(shape, viewpoint, view);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::DatabaseEdges2()
{
  if (input == 0) return 0;
  if (database == 0) return 0;

  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", frame, &view);
  grayscale_image * the_image = database->GetEdges2(frame, view);
  ShowImage(the_image);
  return 1;
}


long vInterpreter2::DatabaseRotate()
{
  if (input == 0) return 0;
  if (database == 0) return 0;
  
  long view = 0;
  vPrint("\nEnter view:\n");
  vScan("%li", &view);

  color_image * the_image = input->current_image->ToRGBCopy();
  color_image * rotated = database->RotateImage(the_image, view);
  vdelete(the_image);
  ShowImage(rotated);
  return 1;
}


long vInterpreter2::DatabasePose2()
{
  if (database == 0) return 0;
  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);
  vHandPose pose = database->GetPose2(frame, view);
  pose.Print();
  return 1;
}


long vInterpreter2::DatabasePose()
{
  if (database == 0) return 0;
  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  vHandPose pose = database->GetPose(frame);
  pose.Print();
  return 1;
}


long vInterpreter2::DatabasePoseFrame()
{
  if (database == 0) return 0;
  vHandPose pose;
  vPrint("\nEnter shape_id, 3D orientation\n");
  pose.Scan();
  long frame, view;
  frame = view = 0;
  database->GetPoseFrame(pose, &frame, &view);
  vPrint("frame = %li, view = %li\n", frame, view);
  return 1;
}


long vInterpreter2::Database1PoseFrame()
{
  vHandPose pose;
  vPrint("\nEnter shape_id, 3D orientation\n");
  pose.Scan();
  long frame, view;
  frame = view = 0;
  vOldDatabase1 database1;
  database1.GetPoseFrame(pose, &frame, &view);

  vFrameViewFinder view_finder;
  double x, y, z;
  long category;
  x = y = z = category = 0;
  view_finder.GetViewParams(frame, view, &category, &x, &y, &z);
  double distance = vOldViewDistance2(pose.camera_x, pose.camera_y, pose.orientation,
                                    x, y, z);

  vPrint("frame = %li, view = %li, distance = %.2lf\n", frame, view, distance);
  return 1;
}


long vInterpreter2::SetTrajectoryCutoff()
{
  long new_cutoff = 0;
  vPrint("\nEnter new cutoff:\n");
  vScan("%li", &new_cutoff);
  if (new_cutoff <= 0) return 0;
  trajectory_cutoff = new_cutoff;
  return 1;
}

  
long vInterpreter2::SetPreviousScores()
{
  if (match_scores == 0) return 0;
  vdelete(previous_scores);
  previous_scores = match_scores;
  match_scores = 0;
  previous_score_frame = input->current_frame;
  return 1;
}


// This is a first attempt at generating trajectories. Assuming that
// previous_scores and previous_frame holds some previous match scores
// and the frame at which they were generated, and that match_scores
// holds scores for the current frame, we generate trajectories that
// join the top estimates of the previous frame with the top estimates
// of the current frame.
long vInterpreter2::CountTrajectories()
{
  if (previous_scores == 0) return 0;
  if (match_scores == 0) return 0;
  long frames = input->current_frame - previous_score_frame + 1;
  if (frames < 0) return 0;
  if (frames > 100) return 0;
  vector<vTrajectory> * trajectories = 
    vCandidateTrajectories(previous_scores, match_scores, frames, trajectory_cutoff);
  long count = trajectories->size();
  vPrint("%li trajectories were created\n", count);
  vdelete(trajectories);
  return 1;
}


long vInterpreter2::OldTestFastPoseFrame1()
{
  static vOldDatabase1 database1;
  vHandPose pose;
  vPrint("\nEnter shape_id, x, y, z:\n");
  pose.Scan();
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  database1.GetPoseFrame(pose, &frame1, &view1);
  database1.FastPoseFrame(pose, &frame2, &view2);
  vHandPose pose1 = database1.GetPose2(frame1, view1);
  vHandPose pose2 = database1.GetPose2(frame2, view2);
  double distance1 = vOldPoseDistance(pose, pose1);
  double distance2 = vOldPoseDistance(pose, pose2);
  double distance3 = vOldPoseDistance(pose1, pose2);
    
  vPrint("slow: (%li, %li), distance = %.2lf\n",
          frame1, view1, distance1);
  vPrint("fast: (%li, %li), distance = %.2lf\n",
          frame2, view2, distance2);
  vPrint("distance3 = %.2lf\n\n", distance3);
  return 1;
}


long vInterpreter2::OldTestFastPoseFrame2()
{
  if (database == 0) return 0;
  vHandPose pose;
  vPrint("\nEnter shape_id, x, y, z:\n");
  pose.Scan();
  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  database->GetPoseFrame(pose, &frame1, &view1);
  database->FastPoseFrame(pose, &frame2, &view2);
  vHandPose pose1 = database->GetPose2(frame1, view1);
  vHandPose pose2 = database->GetPose2(frame2, view2);
  double distance1 = vOldPoseDistance(pose, pose1);
  double distance2 = vOldPoseDistance(pose, pose2);
  double distance3 = vOldPoseDistance(pose1, pose2);
    
  vPrint("slow: (%li, %li), distance = %.2lf\n",
          frame1, view1, distance1);
  vPrint("fast: (%li, %li), distance = %.2lf\n",
          frame2, view2, distance2);
  vPrint("distance3 = %.2lf\n\n", distance3);
  return 1;
}


long vInterpreter2::ResizeScoresHolder()
{
  long new_size = 0;
  vPrint("\nEnter new size:\n");
  vScan("%li", &new_size);
  if (new_size <= 0) return 0;
  if (new_size > 100) 
  {
    vPrint("Sizes over 100 are not allowed\n");
    return 0;
  }
  scores_to_hold = new_size;
  if (scores_holder == 0) scores_holder = new vScoresHolder(scores_to_hold);
  else scores_holder->Resize(scores_to_hold);
  return 1;
}


long vInterpreter2::HoldScores()
{
  if (scores_holder == 0) return 0;
  if (match_scores == 0) return 0;
  if (input == 0) return 0;
  vMatchScores * copy_scores = new vMatchScores(match_scores);
  long frame = input->current_frame;
  long result = scores_holder->AddScores(copy_scores, frame);
  return result;
}


long vInterpreter2::TrajectoryScores()
{
  if (scores_holder == 0) return 0;
  vdelete(match_scores);
  match_scores = scores_holder->TrajectoryScores(scores_to_use, trajectory_cutoff);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


long vInterpreter2::SetScoresToUse()
{
  long new_size = 0;
  vPrint("\nEnter new size:\n");
  vScan("%li", &new_size);
  if (new_size <= 0) return 0;
  if (new_size > 100) 
  {
    vPrint("Sizes over 100 are not allowed\n");
    return 0;
  }
  scores_to_use = new_size;
  return 1;
}


long vInterpreter2::EraseScores()
{
  if (scores_holder == 0) return 0;
  scores_holder->EraseScores();
  return 1;
}


// Get a vTrajectoryScores object.
long vInterpreter2::TrajectoryScores2()
{
  if (scores_holder == 0) return 0;
  vTrajectoryScores * new_scores = 
    scores_holder->TrajectoryScores2(scores_to_use, trajectory_cutoff);
  if (new_scores != 0)
  {
    vdelete(trajectory_scores);
    trajectory_scores = new_scores;
    return 1;
  }
  else
  {
    return 0;
  }
}


// Show a trajectory (from vTrajectoryScores) specified by its rank.
long vInterpreter2::ShowTrajectory()
{
  if (input == 0) return 0;
  if (trajectory_scores == 0) return 0;

  long rank = 0;
  vPrint("\nEnter rank\n");
  vScan("%li", &rank);
  if (rank < 0) return 0;
  if (rank >= (long) trajectory_scores->scores.size())
  {
    vPrint("Only %li scores are available\n", trajectory_scores->scores.size());
    return 0;
  }

  vTrajectoryScore & score = trajectory_scores->scores[rank];
  score.Print();
  vTrajectory & trajectory = score.trajectory;
  long frames = trajectory.frames->size();
  long i;
  for (i = 0; i < frames; i++)
  {
    long frame = (*trajectory.frames)[i];
    long view = (*trajectory.views)[i];
    color_image * the_image = input->sequence->ReadFrame(frame);
    if (frame == 0) continue;
    color_image * rotated = (color_image *) vRotateRight(the_image, view * 90);
    ShowImage(rotated);
    vdelete(the_image);
  }
  return 1;
}


// Show a trajectories (from vTrajectoryScores) with ranks within
// a specified range.
long vInterpreter2::ShowTrajectories()
{
  if (input == 0) return 0;
  if (trajectory_scores == 0) return 0;

  long low, high;
  low = high = 0;
  vPrint("\nEnter low and high\n");
  vScan("%li %li", &low, &high);
  if (low < 0) return 0;
  if (high>= (long) trajectory_scores->scores.size())
  {
    vPrint("Only %li scores are available\n", trajectory_scores->scores.size());
    return 0;
  }

  long i;
  for (i = low; i <= high; i++)
  {
    vTrajectoryScore & score = trajectory_scores->scores[i];
    vPrint("%li: rank = %li, rotation = %.1f, score = %.1lf, (%li, %li)\n",
            i, score.rank, score.trajectory.angle, score.score,
            (*score.trajectory.frames)[0],
            (*score.trajectory.views)[0]);
  }
  return 1;
}


// Find trajectories (from vTrajectoryScores) that end in the specified
// frame/view.
long vInterpreter2::FindTrajectories()
{
  if (trajectory_scores == 0) return 0;

  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);
  long number_of_trajectories = trajectory_scores->scores.size();
  long i;
  long counter = 0;
  for (i = 0; i < number_of_trajectories; i++)
  {
    vTrajectoryScore & score = trajectory_scores->scores[i];
    long frame2 = score.trajectory.frame;
    long view2 = score.trajectory.view;
    if ((frame2 == frame) && (view2 == view))
    {
      vPrint("%li: rank = %li, rotation = %.1f, score = %.1lf, (%li, %li)\n",
              counter, score.rank, score.trajectory.angle, score.score,
              (*score.trajectory.frames)[0],
              (*score.trajectory.views)[0]);
    }
    counter++;
  }
  return 1;
}


long vInterpreter2::TrajectoryAngleCutoff()
{
  if (scores_holder == 0) return 0;
  double cutoff = 0;
  vPrint("\nEnter new trajectory angle cutoff:\n");
  vScan("%lf", &cutoff);
  scores_holder->SetAngleCutoff(cutoff);
  return 1;
}


// The general strategy to find hands is like this:
// - Find the face (as a rectangle, the way the face detector does)
// - Find skin in the face
// - Look for pixels elsewhere in the image that have color similar to 
// the skin of the face
// In addition to what FindHands does, here we identify the two largest 
// connected components that are not associated with the face, and we draw
// bounding boxes around them.
long vInterpreter2::FindHands2()
{
  // Do some error checking
  if (input == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }
  
  if (input->current_image == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }

  general_image * the_image1 = input->current_image;

  // Check if we know the color of the face. If we don't, try to find a face
  // and calculate its color. If no face is found then we fail.
  if (input->face_color == 0)
  {
    long success = SetFaceColor0();
    if (success == 0)
    {
      vPrint("Couldn't find face, can't proceed with finding hands\n");
      return 0;
    }
  }
    
  color_image * current_image = the_image1->ToRGBCopy();

  // Find regions in the image where color is similar to face skin color.
  v3dMatrix<long> * color_distances;
  color_distances = vGridDistances(current_image, input->face_color);

  ShowImage(color_distances);
  grayscale_image * binary = (grayscale_image *) color_distances->threshold((double) hand_threshold);
  vInverseImage(binary);
  ShowImage(binary);

  grayscale_image * filled_holes = new grayscale_image(binary->Rows(), binary->Cols());
  vFillUpHoles2(binary, filled_holes);
  ShowImage(filled_holes);

  // Find the hands (largest connected components that do not overlap 
  // with face).
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = filled_holes->ConnectedComponents(&labels);
  vInterpreter1::ShowImage(connected);

  std::sort(labels.begin(), labels.end(), LabelMore());
  long i;
  long rank = 0;
  long hand_ids[] = {-1, -1};
  for (i = 0; i < (long) labels.size(); i++)
  {
    Label current_label = labels[i];
    if (current_label.id == 0) continue;
    if (vBoxesOverlap(current_label, face_box)) continue;
    hand_ids[rank] = current_label.id;
    vPrint("id = %li, size = %li\n", current_label.id, current_label.count);
    rank++;
    if (rank >= 2) break;
  }

  vBoundingBox boxes[2];
  v3dMatrix<uchar> * hands[2];
  for (i = 0; i < rank; i++)
  {
    long id = hand_ids[i];
    hands[i] = vSelectValue(connected, id);
    boxes[i] = vBinaryBoundingBox(hands[i]);
    vDrawRectangle1(current_image, boxes[i].top, boxes[i].left, 
                     boxes[i].bottom, boxes[i].right, (uchar) 255);
  }

  // We hope that the right hand will appear on the left side 
  // of the image. We want to display the right hand first.
  if (boxes[0].left <= boxes[1].left)
  {
    ShowImage(hands[0]);
    ShowImage(hands[1]);
  }
  else
  {
    ShowImage(hands[1]);
    ShowImage(hands[0]);
  }

  ShowImage(current_image);
  return 1;
}


// The general strategy to find hands is like this:
// - Find the face (as a rectangle, the way the face detector does)
// - Find skin in the face
// - Look for pixels elsewhere in the image that have color similar to 
// the skin of the face
// The difference between this function and FindHands2 is that here 
// we display each hand as a separate image, and we segment the 
// hand using fine segmentation or box segmentation.
long vInterpreter2::FindHands3()
{
  // Do some error checking
  if (input == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }
  
  if (input->current_image == 0)
  {
    vPrint("No input frame\n");
    return 0;
  }

  general_image * the_image1 = input->current_image;

  // Try to find a face and calculate its color. If no face is 
  // found then we fail.
  long success = SetFaceColor0();
  if (success == 0)
  {
    vPrint("Couldn't find face, can't proceed with finding hands\n");
    return 0;
  }
    
  color_image * current_image = the_image1->ToRGBCopy();

  // Find regions in the image where color is similar to face skin color.
  v3dMatrix<long> * color_distances;
  color_distances = vGridDistances(current_image, input->face_color);

  ShowImage(color_distances);
  grayscale_image * binary = (grayscale_image *) color_distances->threshold((double) hand_threshold);
  vInverseImage(binary);
  ShowImage(binary);

  grayscale_image * filled_holes = new grayscale_image(binary->Rows(), binary->Cols());
  vFillUpHoles2(binary, filled_holes);
  ShowImage(filled_holes);

  // Find the hands (largest connected components that do not overlap 
  // with face).
  vector<Label> labels;
  v3dMatrix<long> * connected;
  connected = filled_holes->ConnectedComponents(&labels);
  vInterpreter1::ShowImage(connected);

  std::sort(labels.begin(), labels.end(), LabelMore());
  long i;
  long rank = 0;
  long hand_ids[] = {-1, -1};
  for (i = 0; i < (long) labels.size(); i++)
  {
    Label current_label = labels[i];
    if (current_label.id == 0) continue;
    if (vBoxesOverlap(current_label, face_box)) continue;
    hand_ids[rank] = current_label.id;
    vPrint("id = %li, size = %li\n", current_label.id, current_label.count);
    rank++;
    if (rank >= 2) break;
  }

  vBoundingBox boxes[2];
  v3dMatrix<uchar> * images[2];
  v3dMatrix<uchar> * edges[2];
  grayscale_image * gray = current_image->ToGrayCopy();
  grayscale_image * current_edges = vCannyDetector(gray, (uchar) canny_threshold);
  for (i = 0; i < rank; i++)
  {
    long id = hand_ids[i];
    grayscale_image * hand = (grayscale_image *) vSelectValue(connected, id);
    boxes[i] = vBinaryBoundingBox(hand);
    if (segmentation_method == 0) // bounding box segmentation
    {
      vDrawFilledRectangle1(hand, boxes[i].top, boxes[i].left, boxes[i].bottom, boxes[i].right,
                             (uchar) 255);
    }

    images[i] = vSelectPixels(current_image, hand);
    edges[i] = vSelectPixels(current_edges, hand);
    vdelete(hand);
    vDrawRectangle1(current_image, boxes[i].top, boxes[i].left, 
                     boxes[i].bottom, boxes[i].right, (uchar) 255);
  }
  vdelete(gray);
  vdelete(current_edges);

  // We hope that the right hand will appear on the left side 
  // of the image. We want to display the right hand first.
  if (boxes[0].left <= boxes[1].left)
  {
    ShowImage(edges[0]);
    ShowImage(images[0]);
    ShowImage(edges[1]);
    ShowImage(images[1]);
  }
  else
  {
    ShowImage(edges[1]);
    ShowImage(images[1]);
    ShowImage(edges[0]);
    ShowImage(images[0]);
  }

  ShowImage(current_image);
  return 1;
}


long vInterpreter2::FeaturesToTrack()
{
  if (input == 0) return 0;
  double quality, min_distance;
  quality = min_distance = 0;
  vPrint("\nEnter quality, min_distance:\n");
  vScan("%lf %lf", &quality, &min_distance);
  
  color_image * the_image = input->current_image->ToRGBCopy();
  grayscale_image * gray = input->current_image->ToGrayCopy();
  features_to_track.erase(features_to_track.begin(), features_to_track.end());
  vGoodFeaturesToTrack(gray, &features_to_track, quality, min_distance);
  vdelete(gray);

  // Draw features;
  long i;
  vPrint("%li features found\n", features_to_track.size());
  vector<uchar> values(3);
  values[0] = 255;
  values[1] = 255;
  for (i = 0; i < (long) features_to_track.size(); i++)
  {
    values[2] = (uchar) i;
    vDrawCrossv(the_image, (long) features_to_track[i].row, (long) features_to_track[i].col,
                 0, 2, &values);
  }

  ShowImage(the_image);
  return 1;
}


long vInterpreter2::OpticalFlow()
{
  if (input == 0) return 0;
  if (features_to_track.size() == 0) return 0;

  long window_size, pyramids;
  window_size = pyramids = 0;
  vPrint("\nEnter window size, pyramids:\n");
  vScan("%li %li", &window_size, &pyramids);

  ShowPreviousImage();
  grayscale_image * gray1 = input->current_image->ToGrayCopy();
  color_image * the_image1 = input->current_image->ToRGBCopy();
  ShowNextImage();
  grayscale_image * gray2 = input->current_image->ToGrayCopy();
  color_image * the_image2 = input->current_image->ToRGBCopy();
  color_image * the_image3 = input->current_image->ToRGBCopy();

  vector<uchar> values(3);
  values[0] = 255;
  values[1] = 255;

  vector<vPoint2> new_locations;
  vector<long> status;
  vector<double> error;

  vOpticalFlowPyrLK(gray1, gray2, &features_to_track, &new_locations,
                     window_size, pyramids, &status, &error);
  vdelete(gray1);
  vdelete(gray2);

  long size2 = new_locations.size();
  vPrint("%li new_locations found\n", size2);
  long i;
  for (i = 0; i < (long) features_to_track.size(); i++)
  {
    long row1 = vRound(features_to_track[i].row);
    long col1 = vRound(features_to_track[i].col);
    long row2 = vRound(new_locations[i].row);
    long col2 = vRound(new_locations[i].col);
    
    vPrint("%li: (%li, %li) -> (%li, %li), error = %.2lf, status = %li\n",
            i, row1, col1, row2, col2, error[i], status[i]);
 

    values[2] = (uchar) i;
    vDrawCrossv(the_image1, row1, col1, 0, 2, &values);
    vDrawCrossv(the_image2, row2, col2, 0, 2, &values);
    vDrawCrossv(the_image3, row2, col2, 0, 2, &values);
    vDrawLine(the_image3, row1, col1, row2, col2, &values);
  }

  ShowImage(the_image1);
  ShowImage(the_image2);
  ShowImage(the_image3);
  return 1;
}


long vInterpreter2::EraseFeatures()
{
  features_to_track.erase(features_to_track.begin(), features_to_track.end());
  return 1;
}


long vInterpreter2::AddFeature0()
{
  if (input == 0) return 0;

  long row, col;
  row = col = 0;
  vPrint("\nEnter row, col:\n");
  vScan("%li %li", &row, &col);
  return AddFeature(row, col);
}


long vInterpreter2::AddFeature(long row, long col)
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  if (the_image == 0) return 0;

  if (the_image->check_bounds(row, col) <= 0)
  {
    vPrint("Error, coordinates out of bound\n");
    return 0;
  }

  // Check if the feature location is already included
  long size = features_to_track.size();
  long i;
  for (i = 0; i < size; i++)
  {
    vPoint2 & point = features_to_track[i];
    if ((row == vRound(point.row)) &&
        (col == vRound(point.col)))
    {
      return 0;
    }
  }

  features_to_track.push_back(vPoint2(row, col));
  color_image * rgb = the_image->ToRGBCopy();
  vDrawCross(rgb, row, col, 0, 2, (uchar) 255);
  ShowImage(rgb);
  return 1;
}


long vInterpreter2::AddFeatures()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  if (the_image == 0) return 0;

  vector<long> row_vector, col_vector;
  grayscale_image * gray = the_image->ToGrayCopy();
  vGetNonZeroPoints(gray, &row_vector, &col_vector);
  vdelete(gray);

  long number = row_vector.size();
  long j;
  long size = features_to_track.size();

  for (j = 0; j < number; j++)
  {
    long row = row_vector[j];
    long col = col_vector[j];
    // Check if the feature location is already included
    long i;
    long flag = 0;
    for (i = 0; i < size; i++)
    {
      vPoint2 & point = features_to_track[i];
      if ((row == vRound(point.row)) &&
          (col == vRound(point.col)))
      {
        flag = 1;
        break;
      }
    }

    if (flag != 0) continue;
    features_to_track.push_back(vPoint2(row, col));
  }

  return 1;
}


// Almost identical to OpticalFlow, but here the new locations
// are written over the old feature locations.
long vInterpreter2::UpdateFeatures()
{
  if (input == 0) return 0;
  if (features_to_track.size() == 0) return 0;

  long window_size, pyramids;
  window_size = pyramids = 0;
  vPrint("\nEnter window size, pyramids:\n");
  vScan("%li %li", &window_size, &pyramids);

  ShowPreviousImage();
  grayscale_image * gray1 = input->current_image->ToGrayCopy();
  color_image * the_image1 = input->current_image->ToRGBCopy();
  ShowNextImage();
  grayscale_image * gray2 = input->current_image->ToGrayCopy();
  color_image * the_image2 = input->current_image->ToRGBCopy();
  color_image * the_image3 = input->current_image->ToRGBCopy();

  vector<uchar> values(3);
  values[0] = 255;
  values[1] = 255;

  vector<vPoint2> new_locations;
  vector<long> status;
  vector<double> error;

  vOpticalFlowPyrLK(gray1, gray2, &features_to_track, &new_locations,
                     window_size, pyramids, &status, &error);
  vdelete(gray1);
  vdelete(gray2);

  long size2 = new_locations.size();
  vPrint("%li new_locations found\n", size2);
  long i;
  for (i = 0; i < (long) features_to_track.size(); i++)
  {
    long row1 = vRound(features_to_track[i].row);
    long col1 = vRound(features_to_track[i].col);
    long row2 = vRound(new_locations[i].row);
    long col2 = vRound(new_locations[i].col);
    
    vPrint("%li: (%li, %li) -> (%li, %li), error = %.2lf, status = %li\n",
            i, row1, col1, row2, col2, error[i], status[i]);
 

    values[2] = (uchar) i;
    vDrawCrossv(the_image1, row1, col1, 0, 2, &values);
    vDrawCrossv(the_image2, row2, col2, 0, 2, &values);
    vDrawCrossv(the_image3, row2, col2, 0, 2, &values);
    vDrawLine(the_image3, row1, col1, row2, col2, &values);
    features_to_track[i] = vPoint2(new_locations[i].row, 
                                    new_locations[i].col);
  }

  ShowImage(the_image1);
  ShowImage(the_image2);
  ShowImage(the_image3);
  return 1;
}


long vInterpreter2::MouseAddFeatures()
{
  if (input == 0) return 0;
  mouse_click_response = AddFeaturesResponse;
  return 1;
}


long vInterpreter2::AddAllFeatures()
{
  if (input == 0) return 0;
  general_image * the_image = input->current_image;
  long rows = the_image->Rows();
  long cols = the_image->Cols();

  long row, col;
  for (row = 0; row < rows; row++)
  {
    for (col = 0; col < cols; col++)
    {
      features_to_track.push_back(vPoint2(row, col));
    }
  }
  return 1;
}


// This function is called to adjust palm cutoff in the case
// where left_hand is 1, and we flip the whole image upside down.
vLineSegmentLong vInterpreter2::LineTopBottom(vLineSegmentLong line, 
                                                long rows, long cols)
{
  vLineSegmentLong result;
  result.row1 = rows - line.row1 - 1;
  result.row2 = rows - line.row2 - 1;
  result.col1 = line.col1;
  result.col2 = line.col2;
  return result;
}

  
long vInterpreter2::OnlineMethods0()
{
  long segmentation = 0, ranking = 0;
  vPrint("\nEnter segmentation, ranking\n");
  vScan("%li %li", &segmentation, &ranking);
  return OnlineMethods(segmentation, ranking);
}


long vInterpreter2::OnlineMethods(long in_segmentation, long in_ranking)
{
  online_segmentation = in_segmentation;
  online_ranking = in_ranking;

  return 1;
}


long vInterpreter2::SkinThreshold0()
{
  long threshold = 0;
  vPrint("\nEnter threshold:\n");
  vScan("%li", &threshold);
  return SkinThreshold(threshold);
}


long vInterpreter2::SkinThreshold(long threshold)
{
  skin_threshold = threshold;
  return 1;
}


long vInterpreter2::SetTopToPrint0()
{
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return SetTopToPrint(number);
}


long vInterpreter2::SetTopToPrint(long number)
{
  top_to_print = number;
  return 1;
}


long vInterpreter2::DeleteDistanceTransforms()
{
  if (reference_images == 0) return 0;
  if (vIsZero(distance_transforms)) return 0;
  long dimensions = reference_images->Rows();

  long i;
  for (i = 0; i < dimensions; i++)
  {
    vdelete(distance_transforms[i]);
  }
  vdelete2(distance_transforms);
  distance_transforms = vZero(v3dMatrix<float> *);
  return 0;
}
