#include "stdafx.h"


#include "interpreter3.h"
#include "basics/image_algorithms.h"
#include "basics/image_templates.h"
#include "gestures/algorithms.h"
#include "gestures/algo_templates.h"
#include "avi_file.h"
#include "gestures/hausdorff_matching.h"
#include "opencv_utils.h"
#include "gestures/chamfer_matching.h"
#include "gestures/projections.h"
#include "learning/BoostMap_module.h"
#include "learning/mnist_set.h"

#include "basics/definitions.h"


 

vInterpreter3::vInterpreter3(class_interpreter* in_interpreter) :
vInterpreter2(in_interpreter),
bm(((class_BoostMap_module *) base_interpreter->get_module ("BoostMap_module"))->bm),
bm_data(((class_BoostMap_module *) base_interpreter->get_module ("BoostMap_module"))->bm_data)
{
}


vInterpreter3::~vInterpreter3()
{
  CleanUp();
}


void vInterpreter3::Initialize(vGestureInterfaceGlobals * in_globals)
{
  vInterpreter2::Initialize(in_globals);

  edge_summary = 0;
  vs_histogram = 0;
  edge_orientations = 0;
  hand_database = 0;

  // CVPR 2004
  cbi = 0;
  sbi = 0;
  acd = 0;
  boost_map = 0;
  sm_info = 0;
  sm_embeddings = 0;
  fm_info = 0;
  fm_embeddings = 0;
  ceb = 0;
  ceb_embeddings = 0;
  
  // KDD 2004
  training_mnist = 0;
  test_mnist = 0;
//  bm = 0;
//  bm_data = 0;
  fm = 0;

  x_scale = y_scale = z_scale = (double) 1.0;
  canonical_image_size = 500;
  canonical_bbox_size = 400;
  foreground_threshold = (double) 10;
  acd_input1 = database_edge_input;
  acd_input2 = database_edge_input;
}


// An interesting thing with the CleanUp functions of each interpreter
// (i.e. interpreter1, 2, 3), is that each of them gets called by
// the destructor of that class. I don't understand why that happens.
// I would expect that, since the CleanUp functions are all virtual,
// only the CleanUp function of the lowest subclass (interpreter3
// currently) should be called. I don't know if the way things work 
// now is according to the C++ standard or if it is a Virtual Studio bug.
void vInterpreter3::CleanUp()
{
  vdelete(edge_summary);
  vdelete(vs_histogram);
  vdelete(edge_orientations);
  vdelete(hand_database);
  vdelete(acd);
  vdelete(sm_info);
  vdelete(sm_embeddings);
  vdelete(fm_info);
  vdelete(fm_embeddings);
  vdelete(ceb);
  vdelete(ceb_embeddings);
  vdelete(training_mnist);
  vdelete(test_mnist);
  vdelete(fm);
  
  DeleteBM();
  DeleteCBI();
  DeleteSBI();

//  vdelete(bm);
//  vdelete(bm_data);
}


vRLE_Sequence * vInterpreter3::DatabaseEdges()
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  if (point_sequence == 0) return 0;

  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0) return 0;

  vRLE_Sequence * database_edges = (vRLE_Sequence *) sequence;
  return database_edges;
}
  
long vInterpreter3::WaitSeconds0()
{
  long seconds = 0;
  vPrint("\nEnter seconds:\n");
  vScan("%li", &seconds);
  return WaitSeconds(seconds);
}


long vInterpreter3::WaitSeconds(long seconds)
{
  long argument = seconds * 1000;
  Sleep(argument);
  return 1;
}
long vInterpreter3::DoubleSize()
{
  if (input == 0) return 0;
  v3dMatrix<double> * current = new v3dMatrix<double>(input->current_image);
  v3dMatrix<double> * double_size = vDoubleSize(current);
  ShowImage(double_size);
  vdelete(current);
  return 1;
}


long vInterpreter3::SequenceDoubleSize()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vVideoFile * sequence = input->sequence;
  long rows = sequence->Rows();
  long cols = sequence->Cols();
  long frames = sequence->NumberOfFrames();
  const char * filename = sequence->GetFilename();
  char * output_name = vReplaceEnding2(filename, "_double.avi");

  vAVI_Stream * output = new vAVI_Stream(output_name, 2*rows, 2*cols, 3);
  if (output->IsValid() <= 0)
  {
    vPrint("Failed to write to %s\n", output_name);
    vdelete2(output_name);
    vdelete(output);
    return 0;
  }

  long i;
  vPrint("\n");
  for (i = 0; i < frames; i++)
  {
    color_image * the_image = sequence->ReadFrame(i);
    if (the_image == 0) break;
    v3dMatrix<double> * the_image2 = new v3dMatrix<double>(the_image);
    v3dMatrix<double> * double_image2 = vDoubleSize(the_image2);
    color_image * double_color_image = double_image2->ToRGBCopy();
    output->write_frame(double_color_image);
    vPrint("Done with frame %li of %li\r", i, frames);
    vdelete(the_image2);
    vdelete(the_image);
    vdelete(double_image2);
    vdelete(double_color_image);
  }

  vPrint("\n");
  vdelete(output);
  return 1;
}


long vInterpreter3::CorrectToPrint()
{
  if (sequence_match_stats == 0) return 0;
  long number = 0;
  vPrint("\nEnter new number of correct matches to print:\n");
  vScan("%li", &number);
  sequence_match_stats->SetCorrectToPrint(number);
  return 1;
}


long vInterpreter3::RandomEdges()
{
  if (input == 0) return 0;
  long rows, cols, number, radius;
  rows = cols = number = radius = 0;
  vPrint("\nEnter rows, cols, number, radius:\n");
  vScan("%li %li %li %li", &rows, &cols, &number, &radius);

  grayscale_image * the_image = vRandomEdgeImage(rows, cols, number, radius);
  ShowImage(the_image);
  return 1;
}


// We assume that the current input is the database edges.
// We save the distance transform of every 12th image, so
// that we use only one image per viewpoint. The image
// is normalized so that its MEC has radius 96.
long vInterpreter3::LoadEdgeSummary()
{
  if (database_edge_input >= number_of_inputs) return 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (sequence == 0) return 0;

  vdelete(edge_summary);
  edge_summary = new vDatabaseEdgeSummary1(sequence);
  if (edge_summary->IsValid() <= 0)
  {
    vdelete(edge_summary);
    edge_summary = 0;
    return 0;
  }
  return 1;
}


long vInterpreter3::SaveDTs()
{
  if (database_edge_input >= number_of_inputs) return 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (sequence == 0) return 0;
  
  vDatabaseEdgeSummary1 summary;
  long success = summary.SaveDTs(sequence);
  if (success <= 0)
  {
    vPrint("Failed to save points\n");
  }
  else
  {
    vPrint("Saved points successfully\n");
  }
  return success;
}


long vInterpreter3::SaveDatabasePoints()
{
  if (database_edge_input >= number_of_inputs) return 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (sequence == 0) return 0;

  vDatabaseEdgeSummary1 summary;
  long success = summary.SavePoints(sequence);
  if (success <= 0)
  {
    vPrint("Failed to save points\n");
  }
  else
  {
    vPrint("Saved points successfully\n");
  }
  return success;
}


long vInterpreter3::TestPrepareChamfer()
{
  if (input == 0) return 0;
  if (edge_summary == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  
  grayscale_image * dt_array[48];
  vector<long> rows_array[48];
  vector<long> cols_array[48];

  edge_summary->PrepareForChamfer(the_image, dt_array, rows_array, cols_array);
  long i;
  for (i = 0; i < 48; i+=2)
  {
    ShowImage(dt_array[i]->ToGrayCopy());
    grayscale_image * new_image = new grayscale_image(dt_array[i]->Rows(), dt_array[i]->Cols());
    vWriteValue(new_image, (uchar) 0);
    vWriteValue2(new_image, &(rows_array[i]), &(cols_array[i]), (uchar) 255);
    ShowImage(new_image);
  }

  for (i = 0; i < 48; i++)
  {
    vdelete(dt_array[i]);
  }

  vdelete(the_image);
  return 1;
}


long vInterpreter3::QuickChamferScores()
{
  if (input == 0) return 0;
  if (edge_summary == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();

  vdelete(match_scores);
  match_scores = edge_summary->ChamferDistanceScores(the_image);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}

long vInterpreter3::ReplaceColor0()
{
  if (input == 0) return 0;
  long r1, g1, b1, r2, g2, b2;
  r1 = g1 = b1 = r2 = g2 = b2 = 0;
  vPrint("\nEnter r1, g1, b1, r2, g2, b2:\n");
  vScan("%li %li %li %li %li %li", &r1, &g1, &b1, &r2, &g2, &b2);
  return ReplaceColor((uchar) r1, (uchar) g1, (uchar) b1, 
                      (uchar) r2, (uchar) g2, (uchar) b2);
}


long vInterpreter3::ReplaceColor(uchar r1, uchar g1, uchar b1,
                                  uchar r2, uchar g2, uchar b2)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();

  long size = the_image->Size();

  long i;
  for (i = 0; i < size; i++)
  {
    if ((R[i] == r1) && (G[i] == g1) && (B[i] == b1))
    {
      R[i] = r2;
      G[i] = g2;
      B[i] = b2;
    }
  }
  ShowImage(the_image);
  return 1;
}


// This function is used to generate some data for George Kollios,
// for the class project of a student. We generate a distance
// matrix with pairwise chamfer distances between all database images
// in a subset of the database.
long vInterpreter3::DistanceMatrix()
{
  const long start_frame = 26000;
  const long frames = 250;
  const long images = 250 * 4;
  const long views_per_frame = 4;

  if (number_of_inputs <= database_edge_input) return 0;
  SetCurrentInput(database_edge_input);
  vRLE_Sequence * edges = (vRLE_Sequence *) input->sequence;
  class_pointer (vRLE_Gray *) rles = vnew(vRLE_Gray *, images);

  vMatrix<double> distance1_matrix(images, images);
  vMatrix<double> distance2_matrix(images, images);
  vArray2(double) distance1 = distance1_matrix.Matrix2();
  vArray2(double) distance2 = distance2_matrix.Matrix2();
  

  long i, j;
  for (i = 0; i < images; i++)
  {
    long frame = start_frame + i/4;
    long view = i%4;
    vRLE_Gray * rle = (vRLE_Gray *) edges->ReadRLEFrame(frame);
    grayscale_image * gray = rle->ToGrayImage();
    grayscale_image * thresholded = (grayscale_image *) gray->threshold(1);
    grayscale_image * rotated = (grayscale_image *) vRotateRight(thresholded, 90 * view);
    rles[i] = new vRLE_Gray(rotated);
    vdelete(rle);
    vdelete(gray);
    vdelete(rotated);
    vdelete(thresholded);
  }

  vPrint("Read all images\n");
  vArray(long) rows = vnew(long, 65536);
  vArray(long) cols = vnew(long, 65536);
  long number = 0;

  for (i = 0; i < images; i++)
  {
    vRLE_Gray * rle1 = rles[i];
    grayscale_image * gray1 = rle1->ToGrayImage();
    v3dMatrix<float> * dt1 = vCVDistanceTransform(gray1);
    vdelete(gray1);
    for (j = 0; j < images; j++)
    {
      vRLE_Gray * rle2 = rles[j];
      long number;
      vRLENonZeroPoints(rle2, rows, cols, &number);
      distance1[i][j] = vChamferFromDT(rows, cols, number, dt1);
    }
    
    vPrint("Processed image %li, distance = %.2lf\r", i, distance1[i][0]);
    vdelete(dt1);
  }
  vPrint("\n");

  vdelete2(rows);
  vdelete2(cols);
  for (i = 0; i < images; i++)
  {
    vdelete(rles[i]);

    for (j = 0; j < images; j++)
    {
      distance2[i][j] = distance1[i][j] + distance1[j][i];
    }
  }

  vdelete2(rles);
  long success1 = distance1_matrix.WriteText("d:\\users\\athitsos\\distance1_matrix.txt");
  long success2 = distance2_matrix.WriteText("d:\\users\\athitsos\\distance2_matrix.txt");
  vPrint("success1 = %li, success2 = %li\n", success1, success2);
  return 1;
}


// For debugging
long vInterpreter3::CheckDistanceMatrix()
{
  v3dMatrix<double> * distance1_matrix = v3dMatrix<double>::ReadText("c:\\users\\athitsos\\distance1_matrix.txt");
  v3dMatrix<double> * distance2_matrix = v3dMatrix<double>::ReadText("c:\\users\\athitsos\\distance2_matrix.txt");

  vArray2(double) distance1 = distance1_matrix->Matrix2(0);
  vArray2(double) distance2 = distance2_matrix->Matrix2(0);

  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  while(frame1 >= 0)
  {
    vScan("%li %li %li %li\n", &frame1, &view1, &frame2, &view2);
    long base1 = frame1 - 26000;
    long base2 = frame2 - 26000;
    long i = base1 * 4 + view1;
    long j = base2 * 4 + view2;
    vPrint("i = %li, j = %li\n", i, j);
    if (distance1_matrix->check_bounds(i, j) <= 0)
    {
      vPrint("out of bounds\n");
      continue;
    }
    double value1 = distance1[i][j];
    double value2 = distance1[j][i];
    double value3 = distance2[i][j];
    double value4 = distance2[j][i];
    vPrint("%.2lf, %.2lf, %.2lf, %.2lf\n\n\n", value1, value2, value3, value4);
  }

  vdelete(distance1_matrix);
  vdelete(distance2_matrix);
  return 1;
}


// This function is used in conjunction with DistanceMatrix. DistanceMatrix
// generates chamfer distances between pairs of database images. 
// GroundDistanceMatrix generates ground truth distances between pairs of 
// database images.
long vInterpreter3::OldGroundDistanceMatrix()
{
  const long start_frame = 26000;
  const long frames = 250;
  const long images = 250 * 4;
  const long views_per_frame = 4;

  vMatrix<double> distance1_matrix(images, images);
  vArray2(double) distance1 = distance1_matrix.Matrix2();
  
  v3dMatrix<double> x_matrix(1, images, 3);

  vArray(double) x = x_matrix.Matrix(0);
  vArray(double) y = x_matrix.Matrix(1);
  vArray(double) z = x_matrix.Matrix(2);

  vFrameViewFinder finder;
  long junk = 0;

  long i, j;
  for (i = 0; i < images; i++)
  {
    long frame1 = start_frame + i/4;
    long view1 = i%4;
    finder.GetViewParams(frame1, view1, &junk, &(x[i]), &(y[i]), &(z[i]));

    for (j = 0; j <= i; j++)
    {
      distance1[i][j] = vOldViewDistance(x[i], y[i], z[i], x[j], y[j], z[j]);
      distance1[j][i] = distance1[i][j];
    }
    
    vPrint("Processed image %li, distance = %.2lf\r", i, distance1[i][0]);
  }
  vPrint("\n");

  long success1 = distance1_matrix.WriteText("d:\\users\\athitsos\\ground_truth.txt");
  long success2 = x_matrix.WriteText("d:\\users\\athitsos\\ground_truth2.txt");
  vPrint("success1 = %li, %li\n", success1, success2);
  return 1;
}


// For debugging
long vInterpreter3::CheckOldGroundDistanceMatrix()
{
  v3dMatrix<double> * distance1_matrix = v3dMatrix<double>::ReadText("d:\\users\\athitsos\\ground_truth.txt");
  v3dMatrix<double> * x_matrix = v3dMatrix<double>::ReadText("d:\\users\\athitsos\\ground_truth2.txt");

  vArray2(double) distance1 = distance1_matrix->Matrix2(0);

  long frame1, view1, frame2, view2;
  frame1 = view1 = frame2 = view2 = 0;
  while(frame1 >= 0)
  {
    vScan("%li %li %li %li\n", &frame1, &view1, &frame2, &view2);
    long base1 = frame1 - 26000;
    long base2 = frame2 - 26000;
    long i = base1 * 4 + view1;
    long j = base2 * 4 + view2;
    vPrint("i = %li, j = %li\n", i, j);
    if (distance1_matrix->check_bounds(i, j) <= 0)
    {
      vPrint("out of bounds\n");
      continue;
    }
    double value1 = distance1[i][j];
    double value2 = distance1[j][i];
    double x1 = x_matrix->Matrix(0)[i];
    double y1 = x_matrix->Matrix(1)[i];
    double z1 = x_matrix->Matrix(2)[i];
    double x2 = x_matrix->Matrix(0)[j];
    double y2 = x_matrix->Matrix(1)[j];
    double z2 = x_matrix->Matrix(2)[j];
    vPrint("%.2lf, %.2lf\n\n\n", value1, value2);
    vPrint("x1, y1, z1 = (%.2lf, %.2lf, %.2lf)\n", x1, y1, z1);
    vPrint("x2, y2, z2 = (%.2lf, %.2lf, %.2lf)\n", x2, y2, z2);
  }

  vdelete(distance1_matrix);
  vdelete(x_matrix);
  return 1;
}


long vInterpreter3::LipschitzVectors0()
{
  if (training_embeddings == 0) return 0;

  long dimensions = 0;
  vPrint("\nEnter dimensions:\n");
  vScan("%li", &dimensions);
  return LipschitzVectors(dimensions);
}

  
// George Kollios gave me a 10-dimensional embedding of the images 
// used in DistanceMatrix() to generate the distance matrix. That
// embedding was produced using Isomap. LipschitzVectors produces
// an n-dimensional embedding of the same images. The argument 
// "dimensions" specifies n. We assume that an appropriate 
// reference set has already been loaded. The goal is to compare
// the accuracy of the Lipschitz embedding with the accuracy of the
// Isomap embedding. 
long vInterpreter3::LipschitzVectors(long dimensions)
{
  const long start_frame = 26000;
  const long frames = 250;
  const long images = 250 * 4;
  const long views_per_frame = 4;

  if (training_embeddings == 0) return 0;

  vMatrix<double> vectors_matrix(images, dimensions);
  vArray2(double) vectors = vectors_matrix.Matrix2();
  vArray2(float) distances = training_embeddings->Matrix2(0);

  long i, j;
  for (i = 0; i < images; i++)
  {
    long frame = start_frame + i/views_per_frame;
    long view = i%views_per_frame;
    long index = frame * views_per_frame + view;

    for (j = 0; j < dimensions; j++)
    {
      vectors[i][j] = distances[index][j];
    }
  }

  char filename[1000];
  sprintf(filename, "d:\\users\\athitsos\\lipschitz_vectors_%li.txt", dimensions);
  long success1 = vectors_matrix.WriteText(filename);
  vPrint("success1 = %li\n", success1);
  return 1;
}


// For debugging
long vInterpreter3::CheckLipschitzVectors()
{
  return 1;
}


//-------------------------------------------------------------------

// The next functions are used for matching lines and evaluating 
// image similarity based on line matching scores.

// Construct a votes/saliency histogram for input lines.
long vInterpreter3::VS_Stats()
{
  if (input == 0) return 0;

  GetLineSegments4(24, 5, 100, 20, .1, 1);
  vdelete(vs_histogram);
  vs_histogram = new VS_Histogram(line_segments);
  ShowImage(vs_histogram->Copy());

  SelectLineSegments(-1, -1, -1, 21, 50, -1, 2);
  vSortLineSegments(line_segments);
  SampleLineSegments(2, 1, 1);
  ExistentialDOLVS(0, 401);
  DOLVS_To_PDF();

  return 1;
}


// Match an individual model line segment to input line segments
// using the VS histogram.
long vInterpreter3::MatchLineSegmentVS0()
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (vs_histogram == 0) return 0;
  if (input == 0) return 0;
  vPrint("\nEnter link id, line:\n");
  long link_id = 0;
  long line = 0;
  vScan("%li %li", &link_id, &line);
  return MatchLineSegmentVS(link_id, line);
}


long vInterpreter3::MatchLineSegmentVS(long link_id, long line)
{
  if (model_straight_links == 0) return 0;
  if (line_segments == 0) return 0;
  if (vs_histogram == 0) return 0;
  if (input == 0) return 0;

  static long junk = vPrecomputedAngles::PrecomputeValues(256, 256);
  vLineSegment model_segment = GetModelLine(link_id, line);
  if (model_segment.row1 < 0) return 0;

  long row = vRound((model_segment.row1 + model_segment.row2)/2);
  long col = vRound((model_segment.col1 + model_segment.col2)/2);
  long drow = (long) (model_segment.row1 - model_segment.row2);
  long dcol = (long) (model_segment.col1 - model_segment.col2);
  long orientation = vRound(vAngle(dcol, drow) * 180 / M_PI);
  long length = vRound(sqrt((float) (drow*drow + dcol*dcol)));

  vArray(long) rankings = vs_histogram->FrequencyRanking(line_segments, row, col, orientation, length);
  std::sort(line_segments->begin(), line_segments->end(), vLineSegmentMatchLess());

  v3dMatrix<long> * ranks;
  v3dMatrix<long> * info;
  vLineSegmentRankImage(line_segments, input->current_image->Rows(), 
                         input->current_image->Cols(), &ranks, &info, -1);
  ShowImage(info);
  DrawLineSegment(0);
  vdelete2(rankings);
  return 1;
}


// Find the best match for a pair of model lines
long vInterpreter3::LinePairMatch0()
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (model_straight_links == 0) return 0;

  long link1, side1, link2, side2;
  link1 = side1 = link2 = side2 = 0;

  vPrint("\nEnter link1, side1, link2, side2:\n");
  vScan("%li %li %li %li", &link1, &side1, &link2, &side2);
  return LinePairMatch(link1, side1, link2, side2);
}


long vInterpreter3::LinePairMatch(long link1, long side1, long link2, long side2)
{
  if (input == 0) return 0;
  if (all_model_lines == 0) return 0;
  if (line_segments == 0) return 0;
  if (existential_DOLVS == 0) return 0;
  if (model_straight_links == 0) return 0;

  vector<long> indices1, indices2;
  vector<float> scores;

  vLineSegment model_segment1 = GetModelLine(link1, side1);
  vLineSegment model_segment2 = GetModelLine(link2, side2);

  existential_DOLVS->LinePairMatches(model_segment1, model_segment2,
                                     &indices1, &indices2, &scores);

  color_image * current = input->current_image->ToRGBCopy();
  
  long i = 0;
  while (i >= 0)
  {
    color_image * the_image = current->ToRGBCopy();
    
    long index1 = indices1[i];
    long index2 = indices2[i];
    float score = scores[i];
    vPrint("index1 = %li, index2 = %li, score = %f\n",
             index1, index2, score);

    vLineSegmentMatch line1 = (*line_segments)[index1];
    vLineSegmentMatch line2 = (*line_segments)[index2];

    vector<uchar> green_color(3);
    vector<uchar> blue_color(3);
    green_color[1] = blue_color[2] = 255;
    green_color[0] = green_color[2] = blue_color[0] = blue_color[1] = 100;

    vDrawLine(the_image, line1.segment.row1, line1.segment.col1,
               line1.segment.row2, line1.segment.col2, &green_color);
    vDrawLine(the_image, line2.segment.row1, line2.segment.col1,
               line2.segment.row2, line2.segment.col2, &blue_color);

    ShowImage(the_image);
    vScan("%li", &i);
  }

  vdelete(current);
  return 1;
}





// Saves the edge orientations of all database images.
long vInterpreter3::SaveEdgeOrientations()
{
  if (point_sequence == 0) return 0;
  if (number_of_inputs <= database_input) return 0;

  vVideoFile * database = inputs[database_input]->sequence;
  if (database == 0) return 0;
  vdelete(edge_orientations);
  edge_orientations = new vPointSetOrientation(database, point_sequence);
  long success = edge_orientations->Save(database->GetFilename());
  vPrint("success = %li\n", success);
  return 1;
}


long vInterpreter3::LoadEdgeOrientations()
{
  if (point_sequence == 0) return 0;
  if (number_of_inputs <= database_input) return 0;

  vVideoFile * database = inputs[database_input]->sequence;
  if (database == 0) return 0;
  vdelete(edge_orientations);
  edge_orientations = vPointSetOrientation::Load(database->GetFilename());
  if (edge_orientations == 0)
  {
    vPrint("Failed to load edge orientations\n");
  }
  else
  {
    vPrint("Loaded edge orientations\n");
  }
  return 1;
}


long vInterpreter3::ShowOrientations0()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (edge_orientations == 0) return 0;
  
  long frame = 0;
  long view = 0;

  vPrint("\nEnter frame and view:\n");
  vScan("%li %li", &frame, &view);
  return ShowOrientations(frame, view);
}

  
long vInterpreter3::ShowOrientations(long frame, long view)
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (edge_orientations == 0) return 0;
  
  vArray(float) orientations;
  long length = edge_orientations->GetOrientations(frame, view, 
                                                   &orientations);
  vArray(long) rows;
  vArray(long) cols;
  point_sequence->GetPoints(frame, view, &rows, &cols);

  v3dMatrix<double> * orientation_image = new v3dMatrix<double>(256, 256, 1);
  vWriteValue(orientation_image, (double) 0);
  vArray2(double) orientations2 = orientation_image->Matrix2(0);

  long i;
  for (i = 0; i < length; i++)
  {
    long row = rows[i];
    long col = cols[i];
    double orientation = orientations[i];
    orientations2[row][col] = orientation;
  }

  ShowImage(orientation_image);
  return 1;
}


// This function is used to verify that the orientations saved 
// using SaveEdgeOrientations are similar to the orientations
// computed directly from the database images.
long vInterpreter3::VerifyOrientations()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (edge_orientations == 0) return 0;
  if (number_of_inputs <= database_input) return 0;
  if (number_of_inputs <= database_edge_input) return 0;

  long frame, view;
  frame = view = 0;
  vPrint("\nEnter frame and view:\n");
  vScan("%li %li", &frame, &view);

  SetCurrentInput(database_input);
  JumpToFrame(frame);
  RotateRight(view);
  Blur();
  Blur();
  Blur();
  EdgeOrientations();
  ShowPreviousImage();
  if (input->current_image->TypeName() != DoubleType)
  {
    vPrint("Bad type\n");
    return 1;
  }
  v3dMatrix<double> * orientations1 = (v3dMatrix<double> *) input->current_image;
  ShowOrientations(frame, view);
  v3dMatrix<double> * orientations2 = (v3dMatrix<double> *) input->current_image;

  SetCurrentInput(database_edge_input);
  JumpToFrame(frame);
  RotateRight(view);
  grayscale_image * edges = input->current_image->ToGrayCopy();

  SetCurrentInput(database_input);
  v3dMatrix<double> * diff = new v3dMatrix<double>(256, 256, 1);
  vWriteValue(diff, (double) 0);
  vArray(double) diff_values = diff->Matrix(0);

  long size = diff->Size();
  long i;
  vArray(double) o1 = orientations1->Matrix(0);
  vArray(double) o2 = orientations2->Matrix(0);
  vArray(uchar) e1 = edges->Matrix(0);

  for (i = 0; i < size; i++)
  {
    if (e1[i] == 0) continue;
    diff_values[i] = vAbs(vCircularDistance(o1[i], o2[i], M_PI));
  }

  ShowImage(diff);
  vdelete(edges);
  return 1;
}


// Used to see the result of scaling edges and orientations the way
// we do in preparation for the distance/orientation transform.
long vInterpreter3::TestEdgesOrientations()
{
  if (input == 0) return 0;
  if (right_hand_frame == 0) return 0;
  if (right_hand_image == 0) return 0;
  if (hand_mask == 0) return 0;

  grayscale_image * gray_frame = right_hand_frame->ToGrayCopy();
  grayscale_image * all_edges = vCannyDetector(gray_frame, (uchar) canny_threshold);
  grayscale_image * edges = (grayscale_image *) vSelectPixels(all_edges, hand_mask);
  vdelete(all_edges);
  v3dMatrix<double> * all_orientations = vEdgeOrientations(gray_frame);
  vdelete(gray_frame);
  v3dMatrix<double> * orientations = vSelectPixels(all_orientations, edges);
  vdelete(all_orientations);

  // Draw palm cutoff line.
  vDrawLine1(edges, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, (uchar) 255);
  double orientation = vAngle(palm_cutoff.col2 - palm_cutoff.col1,
                               palm_cutoff.row2 - palm_cutoff.row1);
  vDrawLine1(orientations, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, orientation);

  grayscale_image * scaled_edges = (grayscale_image *) vSparseScaleBoundingBox6(edges, 192, 128, 128, 256, 256);
  vBoundingBox box = vBinaryBoundingBox(edges);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  v3dMatrix<double> * scaled_orientations = vSparseScaleBoundingBox7(orientations, box, 192, 128, 128, 256, 256);
  ShowImage(scaled_edges);
  ShowImage(scaled_orientations);
  vdelete(edges);
  vdelete(orientations);
  return 1;
}

  
// Used to see the results of vDO_DistanceTransform();
long vInterpreter3::Test_DO_Transform()
{
  if (input == 0) return 0;
  if (right_hand_frame == 0) return 0;
  if (right_hand_image == 0) return 0;
  if (hand_mask == 0) return 0;

  grayscale_image * gray_frame = right_hand_frame->ToGrayCopy();
  grayscale_image * all_edges = vCannyDetector(gray_frame, (uchar) canny_threshold);
  grayscale_image * edges = (grayscale_image *) vSelectPixels(all_edges, hand_mask);
  vdelete(all_edges);
  v3dMatrix<double> * all_orientations = vEdgeOrientations(gray_frame);
  vdelete(gray_frame);
  v3dMatrix<double> * orientations = vSelectPixels(all_orientations, edges);
  vdelete(all_orientations);

  // Draw palm cutoff line.
  vDrawLine1(edges, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, (uchar) 255);
  double orientation = vAngle(palm_cutoff.col2 - palm_cutoff.col1,
                               palm_cutoff.row2 - palm_cutoff.row1);
  vDrawLine1(orientations, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, orientation);

  grayscale_image * scaled_edges = (grayscale_image *) vSparseScaleBoundingBox6(edges, 192, 128, 128, 256, 256);
  vBoundingBox box = vBinaryBoundingBox(edges);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  v3dMatrix<double> * scaled_orientations = vSparseScaleBoundingBox7(orientations, box, 192, 128, 128, 256, 256);
  ShowImage(scaled_edges);
  ShowImage(scaled_orientations);
  v3dMatrix<float> * do_dt = vDO_DistanceTransform(scaled_edges, scaled_orientations);

  long row, col;
  orientation = row = col = 0;
  while(orientation >= 0)
  {
    vPrint("\nEnter orientation, row, col:\n");
    vScan("%lf %li %li", &orientation, &row, &col);

    if (do_dt->check_bounds(row, col) <= 0) break;
    float bands = (float) do_dt->Bands();
    long band = (long) (orientation / M_PI * bands);
    if (band >= (long) bands) band = ((long) bands) - 1;
    if (band < 0) band = 0;
  
    vPrint("Entry = %lf\n", do_dt->Matrix3()[band][row][col]);
  }

  vdelete(do_dt);
  vdelete(edges);
  vdelete(orientations);
  return 1;
}


// scores based on an extension of the chamfer distance, that takes into
// account edge orientation in addition to edge location.
long vInterpreter3::Chamfer_DO_MI_Scores()
{
  if (input == 0) return 0;
  if (right_hand_frame == 0) return 0;
  if (right_hand_image == 0) return 0;
  if (hand_mask == 0) return 0;
  if (point_sequence == 0) return 0;
  if (edge_orientations == 0) return 0;

  long test_input_index = current_input_index;

  grayscale_image * gray_frame = right_hand_frame->ToGrayCopy();
  grayscale_image * all_edges = vCannyDetector(gray_frame, (uchar) canny_threshold);
  grayscale_image * edges = (grayscale_image *) vSelectPixels(all_edges, hand_mask);
  vdelete(all_edges);
  v3dMatrix<double> * all_orientations = vEdgeOrientations(gray_frame);
  vdelete(gray_frame);
  v3dMatrix<double> * orientations = vSelectPixels(all_orientations, edges);
  vdelete(all_orientations);


  // Draw palm cutoff line.
  vDrawLine1(edges, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, (uchar) 255);
  double orientation = vAngle(palm_cutoff.col2 - palm_cutoff.col1,
                               palm_cutoff.row2 - palm_cutoff.row1);
  vDrawLine1(orientations, palm_cutoff.row1, palm_cutoff.col1, 
              palm_cutoff.row2, palm_cutoff.col2, orientation);


  grayscale_image * scaled_edges = (grayscale_image *) vSparseScaleBoundingBox6(edges, 192, 128, 128, 256, 256);
  vBoundingBox box = vBinaryBoundingBox(edges);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  v3dMatrix<double> * scaled_orientations = vSparseScaleBoundingBox7(orientations, box, 192, 128, 128, 256, 256);
  ShowImage(scaled_edges);
  ShowImage(scaled_orientations);
  v3dMatrix<float> * do_dt = vDO_DistanceTransform(scaled_edges, scaled_orientations);

  vdelete(match_scores);
  match_scores = vChamfer_DO_MI3(do_dt, point_sequence, edge_orientations);

  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  vdelete(do_dt);
  vdelete(edges);
  vdelete(orientations);
  
  return 1;
}


// scores based on an extension of the chamfer distance, that takes into
// account edge orientation in addition to edge location.
long vInterpreter3::Chamfer_DO_Scores()
{
  if (input == 0) return 0;
  if (point_sequence == 0) return 0;
  if (edge_orientations == 0) return 0;
  if (match_scores == 0) return 0;
  if (hand_edge_scaled == 0) return 0;
  if (hand_orientation_scaled == 0) return 0;

  long test_input_index = current_input_index;

  vMatchScores * step1_scores = match_scores;
  // Keep the highest ranking scores.
  vMatchScores * scores_to_keep = step1_scores->KeepHighest(step1_views);

  // Get chamfer scores
  char * training_filename = inputs[database_edge_input]->path_name;
  vMatchScores * chamfer_do_scores = vChamfer_DO_Scores6b(hand_edge_scaled, hand_orientation_scaled, 
                                                            point_sequence, edge_orientations,
                                                            scores_to_keep, step1_views);

  vMatchScores * final_scores = vMatchScores::Merge(chamfer_do_scores, 
                                                      step1_scores);
  match_scores = final_scores;

  SetCurrentInput(test_input_index);
  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();

  vdelete(step1_scores);
  vdelete(scores_to_keep);
  vdelete(chamfer_do_scores);
  
  return 1;
}


// This function initializes the same structures as SkinHand(),
// but it simply assumes that the current input image is the
// hand image. This is the right thing for presegmented sequences,
// like hands1.avi.
long vInterpreter3::TrivialSegmentation()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vdelete(right_hand_image);
  vdelete(original_hand_image);
  vdelete(hand_mask);
  vdelete(right_hand_frame);
  vdelete(original_video_frame);
  
  original_hand_image = input->current_image->ToRGBCopy();
  original_video_frame = original_hand_image->ToRGBCopy();
  if (left_hand == 0)
  {
    right_hand_image = original_hand_image->ToRGBCopy();
    right_hand_frame = original_video_frame->ToRGBCopy();
  }
  else
  {
    right_hand_image = (color_image *) vSwitchTopBottom(original_hand_image);
    right_hand_frame = (color_image *) vSwitchTopBottom(original_hand_image);
  }


  hand_mask = (grayscale_image *) right_hand_image->threshold(1);
  vBoundingBox bbox = vBinaryBoundingBox(hand_mask);
  grayscale_image * full_gray_image = right_hand_frame->ToGrayCopy();
  vdelete(hand_edge_image);
  hand_edge_image = (grayscale_image *) vCannyDetector(full_gray_image, (uchar) canny_threshold);

  v3dMatrix<double> * all_orientations = vEdgeOrientations(full_gray_image);
  vdelete(hand_orientation_image);
  vdelete(full_gray_image);
  hand_orientation_image = vSelectPixels(all_orientations, hand_edge_image);
  vdelete(all_orientations);

  // Draw palm cutoff line.
  if (palm_cutoff.row1 != -1)
  {
    vDrawLine1(hand_edge_image, palm_cutoff.row1, palm_cutoff.col1, 
                palm_cutoff.row2, palm_cutoff.col2, (uchar) 255);
    double orientation = vAngle(palm_cutoff.col2 - palm_cutoff.col1,
                                 palm_cutoff.row2 - palm_cutoff.row1);
    vDrawLine1(hand_orientation_image, palm_cutoff.row1, palm_cutoff.col1, 
                palm_cutoff.row2, palm_cutoff.col2, orientation);
  }

  vdelete(hand_edge_scaled);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  hand_edge_scaled = (grayscale_image *) vSparseScaleBoundingBox7(hand_edge_image, bbox, 192, 128, 128, 256, 256);

  vdelete(hand_orientation_scaled);
  hand_orientation_scaled = vSparseScaleBoundingBox7(hand_orientation_image, bbox, 192, 
                                                      128, 128, 256, 256);
  ShowImage(vCopyImage(hand_edge_scaled));
  ShowImage(vCopyImage(hand_orientation_scaled));

  return 1;
}


// Picks a random bounding box, of size canonical_bbox_size.
long vInterpreter3::RandomSegmentation()
{
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vdelete(right_hand_frame);
  right_hand_frame = input->current_image->ToRGBCopy();
  long rows = right_hand_frame->Rows();
  long cols = right_hand_frame->Cols();
  long top_limit = rows - canonical_bbox_size;
  long left_limit = cols - canonical_bbox_size;
  long random_top, random_left;

  if (top_limit <= 0) random_top = 0;
  else random_top = vRandomLong2(0, top_limit);
  if (left_limit <= 0) random_left = 0;
  else random_left = vRandomLong2(0, left_limit);


  vdelete(hand_mask);
  hand_mask = new grayscale_image(rows, cols);
  vWriteValue(hand_mask, (uchar) 0);
  vDrawFilledRectangle1(hand_mask, random_top, random_left, 
                         random_top + canonical_bbox_size - 1,
                         random_left + canonical_bbox_size - 1,
                         (uchar) 255);
  
  vBoundingBox bbox = vBinaryBoundingBox(hand_mask);

  vdelete(right_hand_image);
  right_hand_image = (color_image *) vSelectPixels(right_hand_frame, hand_mask);

  grayscale_image * full_gray_image = right_hand_frame->ToGrayCopy();
  grayscale_image * full_edge_image = vCannyDetector(full_gray_image, (uchar) canny_threshold);
  
  vdelete(hand_edge_image);
  hand_edge_image = (grayscale_image *) vSelectPixels(full_edge_image, hand_mask);
  vdelete(full_edge_image);

  v3dMatrix<double> * all_orientations = vEdgeOrientations(full_gray_image);
  vdelete(full_gray_image);
  vdelete(hand_orientation_image);
  hand_orientation_image = vSelectPixels(all_orientations, hand_edge_image);
  vdelete(all_orientations);

  vdelete(hand_edge_scaled);
  // We call vSparseScaleBoundingBox7 to ensure that the same transformation is
  // applied to orientations as was applied to edges.
  hand_edge_scaled = (grayscale_image *) vSparseScaleBoundingBox7(hand_edge_image, bbox, 192, 128, 128, 256, 256);

  vdelete(hand_orientation_scaled);
  hand_orientation_scaled = vSparseScaleBoundingBox7(hand_orientation_image, bbox, 192, 
                                                      128, 128, 256, 256);
  ShowImage(vCopyImage(hand_edge_scaled));
  ShowImage(vCopyImage(hand_orientation_scaled));

  return 1;
}


long vInterpreter3::NormalizeIntensity()
{
  if (input == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  color_image * normalized = vNormalizeIntensity(current);
  ShowImage(normalized);
  vdelete(current);
  return 1;
}


long vInterpreter3::WriteRange0()
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;
  
  long low, high;
  low = high = 0;
  vPrint("\nEnter start, end:\n");
  vScan("%li %li", &low, &high);
  return WriteRange(low, high);
}


long vInterpreter3::WriteRange(long low, long high)
{
  if (current_output == -1) return 0;
  vVideoFile * output = GetOutput(current_output);
  if (output == 0) return 0;
  if (input == 0) return 0;
  if (input->sequence == 0) return 0;

  vVideoFile * in_sequence = input->sequence;
  long color_bytes = Min(in_sequence->BytesPerPixel(), output->BytesPerPixel());

  long frame;
  vPrint("\n");
  for (frame = low; frame <= high; frame++)
  {
    long success = 0;
    if (strcmp(output->Type(), "class_image_stream") == 0)
    {
      general_image * current = in_sequence->ReadImage(frame);
      if (current == 0)
      {
        break;
      }
      class_image_stream* output_stream = (class_image_stream*) output;
      success = output_stream->save_image(current);
      vdelete(current);
    }
    else if (color_bytes == 1)
    {
      grayscale_image * the_image = in_sequence->ReadFrameGray(frame);
      if (the_image == 0) break;
      success = output->write_frameGray(the_image);
      vdelete(the_image);
    }
    else if (color_bytes == 3)
    {
      color_image * the_image = in_sequence->ReadFrame(frame);
      if (the_image == 0) break;
      success = output->write_frame(the_image);
      vdelete(the_image);
    }
    if (success > 0)
    {
      vPrint("Successfully wrote frame %li\r", frame);
    }
    else
    {
      vPrint("\nfailed to save frame %li\n", frame);
    }
  }
  vPrint("\n");
  return 1;
}


long vInterpreter3::InputFaceColor0()
{
  long r, g;
  r = g = 0;
  vPrint("\nEnter r, g:\n");
  vScan("%li %li", &r, &g);
  return InputFaceColor(r, g);
}


long vInterpreter3::InputFaceColor(long r, long g)
{
  if (input == 0) return 0;
  vdelete(input->face_color);
  input->face_color = new vSearchGrid(256, 256);
  input->face_color->SetCenter((ushort) r, (ushort) g);
  return 1;
}


long vInterpreter3::OldSphereSurfaceTo3D0()
{
  double x, y;
  x = y = 0;
  vPrint("\nEnter x, y:\n");
  vScan("%lf %lf", &x, &y);
  return OldSphereSurfaceTo3D(x, y);
}


long vInterpreter3::OldSphereSurfaceTo3D(double x, double y)
{
  vPoint3 point = vOldSphereSurfaceTo3D(x, y);
  vPrint("point = (%lf, %lf, %lf)\n", point.x, point.y, point.z);
  return 1;
}

long vInterpreter3::OldThreeDToSphereSurface0()
{
  double x, y, z;
  x = y = z = 0;
  vPrint("\nEnter x, y, z:\n");
  vScan("%lf %lf %lf", &x, &y, &z);
  return OldThreeDToSphereSurface(x, y, z);
}


long vInterpreter3::OldThreeDToSphereSurface(double x, double y, double z)
{
  vPoint3 point = vOld3DToSphereSurface(x, y, z);
  vPrint("viewpoint = (%lf, %lf)\n", point.x, point.y);
  return 1;
}


// These functions are useful to convert BMP files generated by Poser
// to RLE sequences containing many images. Poser (using my
// save_viewpoints.py script) generates images named with a 
// base_number1_number2.bmp template, where number1 is handshape
// and number2 is viewpoint number.
long vInterpreter3::BMP_To_RLE0()
{
  char base[1000];
  vPrint("\nEnter base:\n");
  vScan("%s", base);
  return BMP_To_RLE(base);
}


// This function is used to process the images that are generated 
// by Poser, where each link has a different color, and convert
// them into an RLE sequence.
long vInterpreter3::BMP_To_RLE(const char * base)
{
  long number1 = 0;
  long number2 = 0;

  char filename[1000];
  char output_name[1000];
  sprintf(output_name, "%s.rle", base);
  sprintf(filename, "%s_%li_%li.bmp", base, number1, number2);
  
  color_image * the_image = vReadBMP(filename);
  if (the_image == 0) 
  {
    vPrint("no images found\n");
    return 0;
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vdelete(the_image);

  vRLE_Sequence * output = new vRLE_Sequence(output_name, rows, cols);
  vColorMap * colormap = vHandDatabase1::DefaultColorMap();
  vPrint("\n");
  while(1)
  {
    sprintf(filename, "%s_%li_%li.bmp", base, number1, number2);
    color_image * the_image = vReadBMP(filename);
    if (the_image == 0)
    {
      if (number2 == 0) break;
      else 
      {
        number1++;
        number2 = 0;
        continue;
      }
    }

    grayscale_image * gray = vLinkIDsB(the_image);
    vDrawFilledRectangle1(gray, 0, 0, 15, 200, (uchar) 0);
    output->write_frameGray2(gray, 2, colormap);
    vdelete(gray);
    vdelete(the_image);
    vPrint("%s      \r", filename);
    number2++;
  }
  vPrint("\n");
  output->FinishWriting();
  vdelete(output);
  vdelete(colormap);
  return 1;
}


long vInterpreter3::BMP_To_RLE0b()
{
  char base[1000];
  vPrint("\nEnter base:\n");
  vScan("%s", base);
  return BMP_To_RLE2(base);
}


// This function is used to process the images that are generated 
// by Poser, using the cartoon with lines option, and convert
// them into an RLE sequence.
long vInterpreter3::BMP_To_RLE2(const char * base)
{
  long number1 = 0;
  long number2 = 0;

  char filename[1000];
  char output_name[1000];
  sprintf(output_name, "%s.rle", base);
  sprintf(filename, "%s_%li_%li.bmp", base, number1, number2);
  
  color_image * the_image = vReadBMP(filename);
  if (the_image == 0) 
  {
    vPrint("no images found\n");
    return 0;
  }

  long rows = the_image->Rows();
  long cols = the_image->Cols();
  vdelete(the_image);

  vRLE_Sequence * output = new vRLE_Sequence(output_name, rows, cols);
  vPrint("\n");
  while(1)
  {
    sprintf(filename, "%s_%li_%li.bmp", base, number1, number2);
    color_image * the_image = vReadBMP(filename);
    if (the_image == 0)
    {
      if (number2 == 0) break;
      else 
      {
        number1++;
        number2 = 0;
        continue;
      }
    }

    vDrawFilledRectangle1(the_image, 0, 0, 15, 200, (uchar) 0);
    grayscale_image * gray = (grayscale_image *) the_image->threshold(76);
    vInverseImage(gray);
    output->write_frameGray2(gray, 0, 0);
    vdelete(gray);
    vdelete(the_image);
    vPrint("%s      \r", filename);
    number2++;
  }
  vPrint("\n");
  output->FinishWriting();
  vdelete(output);
  return 1;
}


// These functions are used to look directly at values stored at the 
// skin table.
long vInterpreter3::DebugSkinTable0()
{
  if (skin_table == 0) return 0;
  long r, g, b;
  r = g = b = 0;
  vPrint("\nEnter r, g, b:\n");
  vScan("%li %li %li", &r, &g, &b);
  return DebugSkinTable(r, g, b);
}


long vInterpreter3::DebugSkinTable(long r, long g, long b)
{
  if (skin_table == 0) return 0;

  // The next line will crash the program if the skin table is not
  // of type vSkinTable<float>
  vSkinTable<double> * skin_tablef = (vSkinTable<double> *) skin_table;
  double skin = skin_tablef->ReadSkinValue(r, g, b);
  double non_skin = skin_tablef->ReadNonSkinValue(r, g, b);
  vPrint("skin = %lf, non_skin = %lf\n", skin, non_skin);
  return 1;
}


// Load the specified hand database.
long vInterpreter3::SetHandDatabase0()
{
  char name[1000];
  vPrint("\nEnter database name\n");
  vScan("%s", name);
  return SetHandDatabase(name);
}


long vInterpreter3::SetHandDatabase(const char * name)
{
  vHandDatabase1 * database = new vHandDatabase1(name);
  if (database->IsValid() <= 0)
  {
    vPrint("\nFailed to load database %s\n", name);
    return 0;
  }
  else
  {
    vdelete(hand_database);
    hand_database = database;
    vPrint("\nLoaded database %s\n", name);
    return 1;
  }
}


// Reload the hand database (this is necessary if we have 
// created new sequences that the database now needs to load).
long vInterpreter3::ReloadHandDatabase()
{
  if (hand_database == 0) return 0;
  const char * name = hand_database->Name();
  vHandDatabase1 * reloaded_database = new vHandDatabase1(name);
  if (reloaded_database->IsValid() <= 0)
  {
    vPrint("Failed to reload database\n");
    vdelete(reloaded_database);
    return 0;
  }
  vdelete(hand_database);
  hand_database = reloaded_database;
  return 1;
}


// Prints the name of the currently loaded hand database.
long vInterpreter3::DatabaseName()
{
  if (hand_database == 0) return 0;
  const char * name = hand_database->Name();
  if (name == 0) return 0;
  vPrint("The name of the hand database is %s\n", name);
  return 1;
}

  
// Color the forearm as background.
long vInterpreter3::RemoveForearm()
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  color_image * result = hand_database->RemoveForearm(the_image);
  ShowImage(result);
  vdelete(the_image);
  return 1;
}


// Create the database image sequence, from the original
// poser sequence.
long vInterpreter3::CreateImageSequence()
{
  if (hand_database == 0) return 0;
  long success = hand_database->CreateImageSequence();
  if (success > 0)
  {
    vPrint("image sequence created successfully\n");
  }
  else
  {
    vPrint("failed to create image sequence\n");
  }
  return 1;
}


long vInterpreter3::SaveDatabaseBoundingBoxes()
{
  if (hand_database == 0) return 0;
  long success = hand_database->SaveBoundingBoxes();
  if (success > 0)
  {
    vPrint("bounding boxes saved successfully\n");
  }
  else
  {
    vPrint("failed to save bounding boxes\n");
  }
  return 1;
}


// Applies the articulated edge extraction as specified in 
// the database with that name. This is just for debugging.
long vInterpreter3::ArticulatedEdges()
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;
  color_image * current = input->current_image->ToRGBCopy();
  grayscale_image * edges = hand_database->ArticulatedEdges(current);
  ShowImage(edges);
  vdelete(current);
  return 1;
}


// This function is used to verify that the SaveBoundingBoxes
// function in vHandDatabase1 does the right thing.
long vInterpreter3::ReadBoundingBox0()
{
  if (hand_database == 0) return 0;
  long frame = 0;
  vPrint("\nInput frame\n");
  vScan("%li", &frame);
  return ReadBoundingBox(frame);
}


long vInterpreter3::ReadBoundingBox(long frame)
{
  if (hand_database == 0) return 0;
  vBoundingBox box = hand_database->BoundingBox(frame);
  box.Print();
  return 1;
}


// This function is used to display the results of 
// vHandDatabase1::EdgeImage
long vInterpreter3::DatabaseEdgeImage0()
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;
  long frame = 0;
  vPrint("\nEnter frame (-1 for current frame)\n");
  vScan("%li", &frame);
  if (frame == -1)
  {
    return DatabaseEdgeImage();
  }
  else
  {
    return DatabaseEdgeImage1(frame);
  }
}


// This version works on the currently displayed frame, assuming
// it comes from the big image sequence.
long vInterpreter3::DatabaseEdgeImage()
{
  if (input == 0) return 0;
  long frame = input->current_frame;
  return DatabaseEdgeImage1(frame);
}


long vInterpreter3::DatabaseEdgeImage1(long frame)
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;
  grayscale_image * edges = hand_database->EdgeImage(frame);
  ShowImage(edges);
  return 1;
}


// Recreate, based on Poser, a sequence of edge images 
// similar to the edge images I
// had generated using VirtualHand.
long vInterpreter3::RecreateOldEdgeSequence()
{
  if (hand_database == 0) return 0;
  long success = hand_database->RecreateOldEdgeSequence();
  if (success <= 0)
  {
    vPrint("Failed to create old edge sequence\n");
  }
  else
  {
    vPrint("Created old edge sequence\n");
  }
  return 1;
}


// Create normalized cartoon sequence based on the unnormalized
// sequence generated by Poser.
long vInterpreter3::CreateCartoonSequence()
{
  if (hand_database == 0) return 0;
  long success = hand_database->CreateCartoonSequence();
  if (success <= 0)
  {
    vPrint("Failed to create cartoon sequence\n");
  }
  else
  {
    vPrint("Created cartoon sequence\n");
  }
  return 1;
}


long vInterpreter3::SaveOldFormat0()
{
  if (input == 0) return 0;
  char name[1000];
  vPrint("\nEnter name:\n");
  vScan("%s", name);
  return SaveOldFormat(name);
}


long vInterpreter3::SaveOldFormat(const char * name)
{
  if (input == 0) return 0;
  vVideoFile * sequence = input->sequence;
  if (sequence == 0) return 0;

  char * full_name = vJoinPaths4(sequences_directory, "glove_data", 
                                  "training", name);
  long rows = 256;
  long cols = 256;
  long out_center = 128;
  long bbox_size = 192;

  vRLE_Sequence out_sequence(full_name, rows, cols);
  if (out_sequence.IsValid() <= 0)
  {
    vPrint("Failed to initialize %s\n", full_name);
    vdelete2(full_name);
    return 0;
  }
  vdelete2(full_name);

  float angle_step = 7.5;
  long rotations = 12;

  long frames = sequence->NumberOfFrames();
  long in_rows = sequence->Rows();
  long in_cols = sequence->Cols();
  long in_center_row = in_rows / 2;
  long in_center_col = in_cols / 2;

  long error_flag = 0;

  long i;
  vPrint("\n");
  for (i = 0; i < frames; i++)
  {
    grayscale_image * temp_in_frame = sequence->ReadFrameGray(i);
    if (temp_in_frame == 0)
    {
      vPrint("Failed to read frame %li\n", i);
      error_flag = 1;
      break;
    }

    grayscale_image * in_frame = (grayscale_image *) wrim(temp_in_frame).TopBottom();
    vdelete(temp_in_frame);

    long j;
    for (j = 0; j < rotations; j++)
    {
      float degrees = -angle_step * (float) j;
      grayscale_image * rotated = (grayscale_image *) vRotateImage(in_frame, in_center_row, 
                                                            in_center_col, degrees);
      grayscale_image * scaled = (grayscale_image *) vScaleBoundingBox6(rotated, bbox_size,
                                                                 out_center, out_center,
                                                                 rows, cols);
      long success = out_sequence.write_frameGray(scaled);
      if (success <= 0)
      {
        vPrint("Failed to write frame %li %li\n", i, j);
        error_flag = 1;
        break;
      }
      vdelete(rotated);
      vdelete(scaled);
    }
    vdelete(in_frame);
    if (error_flag > 0) break;
    vPrint("processed frame %li of %li\r", i+1, frames);
  }

  vPrint("\n");
  long success = out_sequence.FinishWriting();
  if (success <= 0) 
  {
    error_flag = 1;
  }

  if (error_flag > 0)
  {
    vPrint("Errors were encountered in converting\n");
    return 0;
  }
  return 1;
}


// The next functions are used to test and debug vScaleImageDown8
long vInterpreter3::ScaleImageA0()
{
  if (input == 0) return 0;
  long bbox_size = 0;
  long image_size = 0;
  vPrint("\nEnter bbox size and image size:\n");
  vScan("%li %li", &bbox_size, &image_size);
  return ScaleImageA(bbox_size, image_size);
}


long vInterpreter3::ScaleImageA(long bbox_size, long image_size)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vBoundingBox bbox = vBinaryBoundingBox(the_image);
  long center_row = bbox.CenterRow();
  long center_col = bbox.CenterCol();
  long new_center_row = image_size/2;
  long new_center_col = image_size/2;
  double max_side = bbox.MaxSide();
  double scaling = ((double) bbox_size) / max_side;
  grayscale_image * result = (grayscale_image *)
    vScaleImageDown8(the_image, scaling, center_row, center_col,
                      new_center_row, new_center_col, 
                      image_size, image_size);
  ShowImage(result);
  vdelete(the_image);
  return 1;
}


long vInterpreter3::ScaleImageB0()
{
  if (input == 0) return 0;
  double scaling = 0;
  long center_row, center_col, new_center_row, new_center_col, new_rows, new_cols;
  center_row = center_col = new_center_row = new_center_col = new_rows = new_cols = 0;
  vPrint("\nEnter scaling, center_row, center_col, new_center_row, new_center_col, new_rows, new_cols:\n");
  vScan("%lf %li %li %li %li %li %li", &scaling, &center_row, &center_col,
         &new_center_row, &new_center_col, &new_rows, &new_cols);
  return ScaleImageB(scaling, center_row, center_col, new_center_row,
                     new_center_col, new_rows, new_cols);
}


long vInterpreter3::ScaleImageB(double scaling, long center_row, long center_col,
                                 long new_center_row, long new_center_col,
                                 long new_rows, long new_cols)
{
  if (input == 0) return 0;
  grayscale_image * the_image = input->current_image->ToGrayCopy();
  grayscale_image * result = (grayscale_image *)
    vScaleImageDown8(the_image, scaling, center_row, center_col,
                      new_center_row, new_center_col, 
                      new_rows, new_cols);
  ShowImage(result);
  vdelete(the_image);
  return 1;
}


// Create sequence of DT transforms for a database edge sequence.
// inputs: database name, and ending specifying the particular
// edge sequence we want to work on (cartoon edges, articulated edges,
// or whatever else).
long vInterpreter3::CreateDTSequence0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return CreateDTSequence(ending);
}


long vInterpreter3::CreateDTSequence(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->CreateDistanceTransform(ending);
  if (success > 0)
  {
    vPrint("created distance transform sequence successfully\n");
  }
  else
  {
    vPrint("failed to create distance transform sequence\n");
  }
  return success;
}


// Create point sequence for a database edge sequence.
// inputs: database name, and ending specifying the particular
// edge sequence we want to work on (cartoon edges, articulated edges,
// or whatever else).
long vInterpreter3::CreatePointSequence0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return CreatePointSequence(ending);
}


long vInterpreter3::CreatePointSequence(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->CreatePointSequence(ending);
  if (success > 0)
  {
    vPrint("created point sequence successfully\n");
  }
  else
  {
    vPrint("failed to create point sequence\n");
  }
  return success;
}


long vInterpreter3::DatabaseLoadDT0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseLoadDT(ending);
}


long vInterpreter3::DatabaseLoadDT(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->LoadDistanceTransform(ending);
  if (success <= 0)
  {
    vPrint("Failed to load distance transform\n");
  }
  else
  {
    vPrint("Loaded distance transform successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseSetCurrentDT0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseSetCurrentDT(ending);
}


long vInterpreter3::DatabaseSetCurrentDT(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->SetCurrentDistanceTransform(ending);
  if (success <= 0)
  {
    vPrint("Failed to set current DT\n");
  }
  else
  {
    vPrint("Set current DT successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseClearDT0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseClearDT(ending);
}


long vInterpreter3::DatabaseClearDT(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->ClearDistanceTransform(ending);
  if (success <= 0)
  {
    vPrint("Failed to clear distance transform\n");
  }
  else
  {
    vPrint("Cleared distance transform successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseClearAllDTs()
{
  if (hand_database == 0) return 0;
  long success = hand_database->ClearAllDistanceTransforms();
  if (success <= 0)
  {
    vPrint("Failed to clear all distance transforms\n");
  }
  else
  {
    vPrint("Cleared all distance transforms successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseGetDT0()
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  return DatabaseGetDT(frame);
}


long vInterpreter3::DatabaseGetDT(long frame)
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  grayscale_image * dt = hand_database->GetDistanceTransform(frame);
  ShowImage(dt);
  return 1;
}


long vInterpreter3::DatabaseLoadPointSequence0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseLoadPointSequence(ending);
}


long vInterpreter3::DatabaseLoadPointSequence(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->LoadPointSequence(ending);
  if (success <= 0)
  {
    vPrint("Failed to load point sequence\n");
  }
  else
  {
    vPrint("Loaded point sequence successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseSetCurrentPointSequence0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseSetCurrentPointSequence(ending);
}


long vInterpreter3::DatabaseSetCurrentPointSequence(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->SetCurrentPointSequence(ending);
  if (success <= 0)
  {
    vPrint("Failed to set current point sequence\n");
  }
  else
  {
    vPrint("Current point sequence set successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseClearPointSequence0()
{
  if (hand_database == 0) return 0;
  char ending[1000];
  vPrint("\nEnter ending:\n");
  vScan("%s", ending);
  return DatabaseClearPointSequence(ending);
}


long vInterpreter3::DatabaseClearPointSequence(const char * ending)
{
  if (hand_database == 0) return 0;
  long success = hand_database->ClearPointSequence(ending);
  if (success <= 0)
  {
    vPrint("Failed to clear point sequence\n");
  }
  else
  {
    vPrint("Cleared point sequence successfully\n");
  }
  return success;
}


long vInterpreter3::DatabaseClearAllPointSequences()
{
  if (hand_database == 0) return 0;
  long success = hand_database->ClearAllPointSequences();
  if (success <= 0)
  {
    vPrint("Failed to clear all point sequences\n");
  }
  else
  {
    vPrint("Cleared all point sequences successfully\n");
  }
  return success;
}


long vInterpreter3::DatabasePointImage0()
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  long frame = 0;
  vPrint("\nEnter frame:\n");
  vScan("%li", &frame);
  return DatabasePointImage(frame);
}


long vInterpreter3::DatabasePointImage(long frame)
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  grayscale_image * dt = hand_database->GetPointImage(frame);
  ShowImage(dt);
  return 1;
}


long vInterpreter3::DatabaseRotationTemplate0()
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return DatabaseRotationTemplate(index);
}


long vInterpreter3::DatabaseRotationTemplate(long index)
{
  if (hand_database == 0) return 0;
  if (input == 0) return 0;

  v3dMatrix<short> * rotation_template = hand_database->RotationTemplate(index);
  ShowImage(rotation_template);
  return 1;
}


// Test the vWarpImage and vWarpImageSparse functions, by getting
// rotation templates and applying them to the current image.
long vInterpreter3::TestWarpImage0()
{
  if (input == 0) return 0;
  long degrees = 0;
  vPrint("\nEnter degrees:\n");
  vScan("%li", &degrees);
  return TestWarpImage(degrees);
}


long vInterpreter3::TestWarpImage(long degrees)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long center_row = rows / 2;
  long center_col = cols / 2;

  v3dMatrix<short> * rotation_template = 
    vRotationTemplate(rows, cols, center_row, center_col, degrees);
  ShowImage(rotation_template);
  v3dMatrix<uchar> * warped = vWarpImage(the_image, rotation_template);
  ShowImage(warped);
  vdelete(the_image);
  return 1;
}


long vInterpreter3::TestWarpImageSparse0()
{
  if (input == 0) return 0;
  long degrees = 0;
  vPrint("\nEnter degrees:\n");
  vScan("%li", &degrees);
  return TestWarpImageSparse(degrees);
}


long vInterpreter3::TestWarpImageSparse(long degrees)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  long rows = the_image->Rows();
  long cols = the_image->Cols();
  long center_row = rows / 2;
  long center_col = cols / 2;

  v3dMatrix<short> * rotation_template = 
    vRotationTemplate(rows, cols, center_row, center_col, degrees);
  ShowImage(rotation_template);
  v3dMatrix<uchar> * warped = vWarpImageSparse(the_image, rotation_template);
  ShowImage(warped);
  vdelete(the_image);
  return 1;
}


long vInterpreter3::CroppedUchar()
{
  if (input == 0) return 0;
  v3dMatrix<double> * the_image = 
    new v3dMatrix<double>(input->current_image);

  v3dMatrix<uchar> * cropped = vCroppedUchar(the_image);
  ShowImage(cropped);
  vdelete(the_image);
  return 1;
}


long vInterpreter3::TestChamferFeatures0()
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return TestChamferFeatures(index);
}


long vInterpreter3::TestChamferFeatures(long index)
{
  if (input == 0) return 0;
  if (hand_database == 0) return 0;

  long number_of_orientations = hand_database->NumberOfOrientations();
  if (index < 0) return 0;
  if (index >= number_of_orientations) return 0;

  grayscale_image * the_image = input->current_image->ToGrayCopy();
  vChamferFeatures features = hand_database->ChamferFeatures(the_image);
  grayscale_image * point_image = features.point_sequence->PointImage(index);
  ShowImage(point_image);
  grayscale_image * dt = features.distance_transforms[index]->ToGrayCopy();
  ShowImage(dt);
  features.EraseData();
  vdelete(the_image);
  return 1;
}


// Tests the function vFrameViewFinder::ConvertPoser1Pose.
long vInterpreter3::ConvertPose0()
{
  long frame = 0;
  long view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);

  return ConvertPose(frame, view);
}


long vInterpreter3::ConvertPose(long frame, long view)
{
  vFrameViewFinder finder;
  long new_frame = 0;
  long new_view = 0;

  finder.ConvertPoser1Pose(frame, view, &new_frame, &new_view);
  vPrint("(%li, %li) -> (%li, %li)\n", frame, view, new_frame, new_view);
  
  return 1;
}


// Tests the function vFrameViewFinder::ConvertToPoser1Pose.
long vInterpreter3::Poser1Pose0()
{
  long frame = 0;
  long view = 0;
  vPrint("\nEnter frame, view:\n");
  vScan("%li %li", &frame, &view);

  return Poser1Pose(frame, view);
}


long vInterpreter3::Poser1Pose(long frame, long view)
{
  vFrameViewFinder finder;
  long new_frame = 0;
  long new_rotation = 0;
  float new_angle = 0;

  finder.ConvertToPoser1Pose(frame, view, 
                             &new_frame, &new_rotation, &new_angle);
  vPrint("(%li, %li) -> (%li, %li), angle = %f\n", 
          frame, view, new_frame, new_rotation, new_angle);
  
  return 1;
}


  
// Chamfer scores using the database "poser1".
long vInterpreter3::Poser1ChamferScores()
{
  if (hand_database == 0) return 0;
  if (strcmp(hand_database->Name(), "poser1") != 0) return 0;
  if (input == 0) return 0;
  if (hand_edge_image == 0) return 0;
  if (number_of_inputs <= database_input) return 0;

  vChamferFeatures database_features = hand_database->DatabaseChamferFeatures();
  if ((database_features.point_sequence == 0) ||
      (predicate_zero (database_features.distance_transforms)))
  {
    vPrint("hand database has no point seq or dt. Use load_dt, load_point_sequence.\n");
    return 0;
  }
  
  grayscale_image * edge_image;
  edge_image = hand_edge_image->ToGrayCopy();

  vChamferFeatures query_features = hand_database->ChamferFeatures(edge_image);

  vMatrix<float> distances = vChamferDistances2(query_features, database_features);
  query_features.EraseData();
  vPoseSet all_poses = hand_database->AllPoses(hand_database->NumberOfFrames(),
                                                hand_database->NumberOfOrientations());
  vdelete(match_scores);
  match_scores = new vMatchScores(&distances, all_poses);
  vdelete(edge_image);

  UpdateMatchScores(input->current_frame);
  SetCurrentInput(database_input);
  ShowTopTenMatches();
  return 1;
}


// Set the current image as the current hand edge image.
long vInterpreter3::SetHandEdgeImage()
{
  if (input == 0) return 0;
  vdelete(hand_edge_image);
  hand_edge_image = input->current_image->ToGrayCopy();
  return 1;
}


long vInterpreter3::OldLineOrientation0()
{
  double camera_x1, camera_y1, camera_z1, angle1;
  double camera_x2, camera_y2, camera_z2, angle2;

  camera_x1 =  camera_y1 =  camera_z1 =  angle1 = 0;
  camera_x2 =  camera_y2 =  camera_z2 =  angle2 = 0;

  vPrint("\nEnter viewpoint1, angle1, viewpoint2, angle2:\n");
  vScan("%lf %lf %lf %lf %lf %lf %lf %lf", 
         &camera_x1, &camera_y1, &camera_z1, &angle1,
         &camera_x2, &camera_y2, &camera_z2, &angle2);

  return OldLineOrientation(camera_x1, camera_y1, camera_z1, angle1,
                         camera_x2, camera_y2, camera_z2, angle2);
}


long vInterpreter3::OldLineOrientation(double camera_x1, double camera_y1, 
                                     double camera_z1, double angle1,
                                     double camera_x2, double camera_y2, 
                                     double camera_z2, double angle2)
{
  vQuaternion viewpoint1 = vOldViewQuaternion(camera_x1, camera_y1, camera_z1);
  vQuaternion viewpoint2 = vOldViewQuaternion(camera_x2, camera_y2, camera_z2);

  double angle1_radians = angle1 / 180 * M_PI;
  double angle2_radians = angle2 / 180 * M_PI;

  vQuaternion orientation = vOldLineOrientation(viewpoint1, angle1_radians,
                                               viewpoint2, angle2_radians);

  return 1;
}


// These functions are for debugging LineOrientation.
long vInterpreter3::VerifyMultiplication()
{
  double x1, y1, z1, x2, y2, z2;
  x1 = y1 = z1 = x2 = y2 = z2 = 0;
  
  vPrint("\nEnter (x1, y1, z1) and (x2, y2, z2):\n");
  vScan("%lf %lf %lf %lf %lf %lf", &x1, &y1, &z1, &x2, &y2, &z2);
  
  vQuaternion quat1 = vOldViewQuaternion(x1, y1, z1);
  vQuaternion quat2 = vOldViewQuaternion(x2, y2, z2);
  vQuaternion result = vQuatMultiply(quat1, quat2);

  quat1.Print("quat1 = ");
  quat2.Print("quat2 = ");

  double q1 = quat1(0);
  x1 = quat1(1);
  y1 = quat1(2);
  z1 = quat1(3);

  double q2 = quat2(0);
  x2 = quat2(1);
  y2 = quat2(2);
  z2 = quat2(3);

  double r0 = q1*q2 - x1*x2 - y1*y2 - z1*z2;
  double r1 = q1*x2 + q2*x1 + y1*z2 - z1*y2;
  double r2 = q1*y2 + q2*y1 + z1*x2 - x1*z2;
  double r3 = q1*z2 + q2*z1 + x1*y2 - y1*x2;

  vPrint("result0 = %lf, r0 = %lf\n", result(0), r0);
  vPrint("result1 = %lf, r1 = %lf\n", result(1), r1);
  vPrint("result2 = %lf, r2 = %lf\n", result(2), r2);
  vPrint("result3 = %lf, r3 = %lf\n", result(3), r3);

  return 1;
}


long vInterpreter3::VerifyInverse()
{
  double x1, y1, z1;
  x1 = y1 = z1 = 0;
  
  vPrint("\nEnter (x1, y1, z1):\n");
  vScan("%lf %lf %lf", &x1, &y1, &z1);
  
  vQuaternion quat1 = vOldViewQuaternion(x1, y1, z1);
  vQuaternion result = vQuatInverse(quat1);

  quat1.Print("quat1 = ");

  double q1 = quat1(0);
  x1 = quat1(1);
  y1 = quat1(2);
  z1 = quat1(3);

  double r0 = q1;
  double r1 = -x1;
  double r2 = -y1;
  double r3 = -z1;

  vPrint("result0 = %.3lf, r0 = %.3lf\n", result(0), r0);
  vPrint("result1 = %.3lf, r1 = %.3lf\n", result(1), r1);
  vPrint("result2 = %.3lf, r2 = %.3lf\n", result(2), r2);
  vPrint("result3 = %.3lf, r3 = %.3lf\n", result(3), r3);

  return 1;
}


long vInterpreter3::VerifyRotation()
{
  double x1, y1, z1, x2, y2, z2;
  x1 = y1 = z1 = x2 = y2 = z2 = 0;
  
  vPrint("\nEnter (x1, y1, z1) and (x2, y2, z2):\n");
  vScan("%lf %lf %lf %lf %lf %lf", &x1, &y1, &z1, &x2, &y2, &z2);
  
  vQuaternion quat1 = vQuaternion();
  quat1(1) = x1;
  quat1(2) = y1;
  quat1(3) = z1;
  vQuaternion quat2 = vOldViewQuaternion(x2, y2, z2);
  vQuaternion result = vQuatRotate(quat1, quat2);

  quat1.Print("quat1 = ");
  quat2.Print("quat2 = ");

  vPrint("result0 = %.lf\n", result(0));
  vPrint("result1 = %.lf\n", result(1));
  vPrint("result2 = %.lf\n", result(2));
  vPrint("result3 = %.lf\n", result(3));

  return 1;
}


long vInterpreter3::VerifyRotationMatrix()
{
  double x1, y1, z1, x2, y2, z2;
  x1 = y1 = z1 = x2 = y2 = z2 = 0;
  
  vPrint("\nEnter point (x1, y1, z1) and viewpoint (x2, y2, z2):\n");
  vScan("%lf %lf %lf %lf %lf %lf", &x1, &y1, &z1, &x2, &y2, &z2);
  
  vQuaternion quat1 = vQuaternion();
  quat1(1) = x1;
  quat1(2) = y1;
  quat1(3) = z1;
  vQuaternion quat2 = vOldViewQuaternion(x2, y2, z2);
  vQuaternion result = vQuatRotate(quat1, quat2);

  quat1.Print("quat1 = ");
  quat2.Print("quat2 = ");

  double q1 = quat1(0);
  x1 = quat1(1);
  y1 = quat1(2);
  z1 = quat1(3);

  double q2 = quat2(0);
  x2 = quat2(1);
  y2 = quat2(2);
  z2 = quat2(3);

  vMatrix<double> quaternion_matrix = vQuaternionMatrix(quat2);
  quaternion_matrix.Print("quaternion_matrix = ");
  vMatrix<double> point(3,1);
  point.WriteN(3, x1, y1, z1);
  point.Print("point = ");
  vMatrix<double> rotated = quaternion_matrix * point;
  rotated.Print("rotated = ");  

  double r0 = 0;
  double r1 = rotated(0);
  double r2 = rotated(1);
  double r3 = rotated(2);

  vPrint("result0 = %.3lf, r0 = %.3lf\n", result(0), r0);
  vPrint("result1 = %.3lf, r1 = %.3lf\n", result(1), r1);
  vPrint("result2 = %.3lf, r2 = %.3lf\n", result(2), r2);
  vPrint("result3 = %.3lf, r3 = %.3lf\n", result(3), r3);

  return 1;
}


long vInterpreter3::OldVerifyPlaneIntersection0()
{
  double camera_x1, camera_y1, camera_z1, angle1;
  double camera_x2, camera_y2, camera_z2, angle2;

  camera_x1 = camera_y1 = camera_z1 = angle1 = 0;
  camera_x2 = camera_y2 = camera_z2 = angle2 = 0;

  vPrint("\nEnter viewpoint1, angle1, viewpoint2, angle2:\n");
  vScan("%lf %lf %lf %lf %lf %lf %lf %lf",
         &camera_x1, &camera_y1, &camera_z1, &angle1,
         &camera_x2, &camera_y2, &camera_z2, &angle2);

  return OldVerifyPlaneIntersection(camera_x1, camera_y1, camera_z1, angle1,
                                 camera_x2, camera_y2, camera_z2, angle2);
}


long vInterpreter3::OldVerifyPlaneIntersection(double camera_x1, double camera_y1,
                                             double camera_z1, double angle1,
                                             double camera_x2, double camera_y2,
                                             double camera_z2, double angle2)
{
  vQuaternion viewpoint1 = vOldViewQuaternion(camera_x1, camera_y1, camera_z1);
  vQuaternion viewpoint2 = vOldViewQuaternion(camera_x2, camera_y2, camera_z2);

  double angle1_radians = angle1 / 180.0 * M_PI;
  double angle2_radians = angle2 / 180.0 * M_PI;

  vOldVerifyPlaneIntersection(viewpoint1, angle1_radians, 
                            viewpoint2, angle2_radians);

  return 1;
}


long vInterpreter3::OldVerifyPlaneIntersection20()
{
  double camera_x1, camera_y1, camera_z1, angle1;
  double camera_x2, camera_y2, camera_z2, angle2;

  camera_x1 = camera_y1 = camera_z1 = angle1 = 0;
  camera_x2 = camera_y2 = camera_z2 = angle2 = 0;

  vPrint("\nEnter viewpoint1, angle1, viewpoint2, angle2:\n");
  vScan("%lf %lf %lf %lf %lf %lf %lf %lf",
         &camera_x1, &camera_y1, &camera_z1, &angle1,
         &camera_x2, &camera_y2, &camera_z2, &angle2);

  return OldVerifyPlaneIntersection2(camera_x1, camera_y1, camera_z1, angle1,
                                  camera_x2, camera_y2, camera_z2, angle2);
}


long vInterpreter3::OldVerifyPlaneIntersection2(double camera_x1, double camera_y1,
                                              double camera_z1, double angle1,
                                              double camera_x2, double camera_y2,
                                              double camera_z2, double angle2)
{
  vQuaternion viewpoint1 = vOldViewQuaternion(camera_x1, camera_y1, camera_z1);
  vQuaternion viewpoint2 = vOldViewQuaternion(camera_x2, camera_y2, camera_z2);

  double angle1_radians = angle1 / 180.0 * M_PI;
  double angle2_radians = angle2 / 180.0 * M_PI;

  vOldVerifyPlaneIntersection2(viewpoint1, angle1_radians, 
                             viewpoint2, angle2_radians);

  return 1;
}


long vInterpreter3::LineVoronoi()
{
  if (input == 0) return 0;
  if (line_set == 0) return 0;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  color_image * result = new color_image(rows, cols);
  vLineSetVoronoi2(line_set, result);
  ShowImage(result);
  return 1;
}


long vInterpreter3::DominanceTransform()
{
  if (input == 0) return 0;
  v3dMatrix<long> * dominance_transform = 
    vDominanceTransform(input->current_image);
  ShowImage(dominance_transform);
  return 1;
}


// Draws circles for the top number points (with highest values)
// on current image. The circles are drawn on a copy of the
// image with the specified buffer index.
long vInterpreter3::TopCircles0()
{
  if (input == 0) return 0;
  long number = 0, buffer_index = 0;
  vPrint("\nEnter number, buffer_index\n");
  vScan("%li %li", &number, &buffer_index);
  return TopCircles(number, buffer_index);
}


long vInterpreter3::TopCircles(long number, long buffer_index)
{
  if (input == 0) return 0;
  long buffer_size = input->buffer_size;
  long temp_index = (input->buffer_index + buffer_size - number) % buffer_size;
  general_image * output_original = input->image_buffer[temp_index];
  if (output_original == 0) return 0;
  color_image * output = output_original->ToRGBCopy();
  v3dMatrix<long> * current = new v3dMatrix<long>(input->current_image);
  vDrawTopCircles(current, output, number);
  ShowImage(output);
  vdelete(current);
  return 1;
}


// Draws a random quadrilateral, so that we can test the 
// vQuadPixels function.
long vInterpreter3::RandomQuadrilateral()
{
  if (input == 0) return 0;
  long top_left_row = vRandomLong2(100, 200);
  long top_left_col = vRandomLong2(100, 200);

  long top_right_row = vRandomLong2(100, 200);
  long top_right_col = vRandomLong2(200, 300);

  long bottom_left_row = vRandomLong2(200, 300);
  long bottom_left_col = vRandomLong2(100, 200);

  long bottom_right_row = vRandomLong2(200, 300);
  long bottom_right_col = vRandomLong2(200, 300);

  class_pointer (long) rows = vnew (long, 4);
  class_pointer (long) cols = vnew (long, 4);

  rows[0] = top_left_row;
  cols[0] = top_left_col;

  rows[1] = top_right_row;
  cols[1] = top_right_col;

  rows[2] = bottom_right_row;
  cols[2] = bottom_right_col;

  rows[3] = bottom_left_row;
  cols[3] = bottom_left_col;

  DrawQuadrilateral(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return 1;
}


long vInterpreter3::DrawQuadrilateral0()
{
  if (input == 0) return 0;
  class_pointer (long) rows = vnew (long, 4);
  class_pointer (long) cols = vnew (long, 4);

  vPrint("\nEnter four points:\n");
  vScan("%li %li %li %li %li %li %li %li", 
         &(rows[0]), &(cols[0]), &(rows[1]), &(cols[1]),
         &(rows[2]), &(cols[2]), &(rows[3]), &(cols[3]));
  long result = DrawQuadrilateral(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::DrawQuadrilateral(class_pointer (long) rows, class_pointer (long) cols)
{
  if (input == 0) return 0;

  long min_row, max_row, min_col, max_col;
  min_row = max_row = rows[0];
  min_col = max_col = cols[0];
  long i;
  for (i = 1; i < 4; i++)
  {
    if (rows[i] < min_row) min_row = rows[i];
    else if (rows[i] > max_row) max_row = rows[i];
    if (cols[i] < min_col) min_col = cols[i];
    else if (cols[i] > max_col) max_col = cols[i];
  }

  grayscale_image * the_image = new grayscale_image(max_row + 50, max_col + 50);
  vWriteValue(the_image, (uchar) 0);
  vector<vPoint> pixels;
  vQuadPixels(rows, cols, &pixels);
  vWriteValue4(the_image, &pixels, (uchar) 160);
  ShowImage(the_image);

  grayscale_image * copy = the_image->ToGrayCopy();
  for (i = 0; i < 4; i++)
  {
    vDrawCross(copy, rows[i], cols[i], 1, 3, (uchar) 255);
  }

  ShowImage(copy);
  return 1;
}


long vInterpreter3::DrawQuadrilateral0b()
{
  if (input == 0) return 0;
  class_pointer (long) rows = vnew (long, 4);
  class_pointer (long) cols = vnew (long, 4);

  vPrint("\nEnter four points:\n");
  vScan("%li %li %li %li %li %li %li %li", 
         &(rows[0]), &(cols[0]), &(rows[1]), &(cols[1]),
         &(rows[2]), &(cols[2]), &(rows[3]), &(cols[3]));
  long result = DrawQuadrilateral2(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


// This version is for debugging, so we can compare drawing a quadrilateral
// in the inefficient way, or using the efficient triangle renderings.
long vInterpreter3::DrawQuadrilateral2(class_pointer (long) rows, class_pointer (long) cols)
{
  if (input == 0) return 0;

  long min_row, max_row, min_col, max_col;
  min_row = max_row = rows[0];
  min_col = max_col = cols[0];
  long i;
  for (i = 1; i < 4; i++)
  {
    if (rows[i] < min_row) min_row = rows[i];
    else if (rows[i] > max_row) max_row = rows[i];
    if (cols[i] < min_col) min_col = cols[i];
    else if (cols[i] > max_col) max_col = cols[i];
  }

  grayscale_image * the_image = new grayscale_image(max_row + 50, max_col + 50);
  vWriteValue(the_image, (uchar) 0);
  vArray2(uchar) values = the_image->Matrix2(0);

  vector<vPoint> pixels(300000);
  long number;
  number = vTrianglePixels(rows, cols, &pixels);
  for (i = 0; i < number; i++)
  {
    long row = pixels[i].row;
    long col = pixels[i].col;
    values[row][col] = 160;
  }

  long row1 = rows[1];
  long col1 = cols[1];
  rows[1] = rows[3];
  cols[1] = cols[3];
  vTrianglePixels(rows, cols, &pixels);
  number = vTrianglePixels(rows, cols, &pixels);
  for (i = 0; i < number; i++)
  {
    long row = pixels[i].row;
    long col = pixels[i].col;
    values[row][col] = 160;
  }
  rows[1] = row1;
  cols[1] = col1;
  
  ShowImage(the_image);

  grayscale_image * copy = the_image->ToGrayCopy();
  for (i = 0; i < 4; i++)
  {
    vDrawCross(copy, rows[i], cols[i], 1, 3, (uchar) 255);
  }

  ShowImage(copy);
  return 1;
}


long vInterpreter3::DrawTriangle0()
{
  if (input == 0) return 0;
  class_pointer (long) rows = vnew (long, 4);
  class_pointer (long) cols = vnew (long, 4);

  vPrint("\nEnter three points:\n");
  vScan("%li %li %li %li %li %li", 
         &(rows[0]), &(cols[0]), &(rows[1]), &(cols[1]),
         &(rows[2]), &(cols[2]));
  long result = DrawTriangle(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::DrawTriangle(class_pointer (long) rows, class_pointer (long) cols)
{
  if (input == 0) return 0;

  long min_row, max_row, min_col, max_col;
  min_row = max_row = rows[0];
  min_col = max_col = cols[0];
  long i;
  for (i = 1; i < 3; i++)
  {
    if (rows[i] < min_row) min_row = rows[i];
    else if (rows[i] > max_row) max_row = rows[i];
    if (cols[i] < min_col) min_col = cols[i];
    else if (cols[i] > max_col) max_col = cols[i];
  }

  grayscale_image * the_image = new grayscale_image(max_row + 50, max_col + 50);
  vWriteValue(the_image, (uchar) 0);
  vector<vPoint> pixels(300000);
  vTrianglePixels(rows, cols, &pixels);
  vWriteValue4(the_image, &pixels, (uchar) 160);
  ShowImage(the_image);

  grayscale_image * copy = the_image->ToGrayCopy();
  for (i = 0; i < 3; i++)
  {
    vDrawCross(copy, rows[i], cols[i], 1, 3, (uchar) 255);
  }

  ShowImage(copy);
  return 1;
}


  
// Selects a color from the current image (assuming it is a color image).
long vInterpreter3::SelectColor0()
{
  if (input == 0) return 0;
  long r, g, b;
  r = g = b = 0;

  vPrint("\nEnter r, g, b\n");
  vScan("%li %li %li", &r, &g, &b);
  return SelectColor((uchar) r, (uchar) g, (uchar) b);
}


long vInterpreter3::SelectColor(uchar r, uchar g, uchar b)
{
  if (input == 0) return 0;

  vector<uchar> color(3);
  color[0] = r;
  color[1] = g;
  color[2] = b;

  color_image * current = input->current_image->ToRGBCopy();
  grayscale_image * result = vSelectColor(current, &color);
  ShowImage(result);
  vdelete(current);
  return 1;
}


// Counts the pixels having the specified color in the current image 
// (assuming it is a color image).
long vInterpreter3::CountColor0()
{
  if (input == 0) return 0;
  long r, g, b;
  r = g = b = 0;

  vPrint("\nEnter r, g, b\n");
  vScan("%li %li %li", &r, &g, &b);
  return CountColor((uchar) r, (uchar) g, (uchar) b);
}


long vInterpreter3::CountColor(uchar r, uchar g, uchar b)
{
  if (input == 0) return 0;
  color_image * the_image = input->current_image->ToRGBCopy();
  vArray(uchar) R = the_image->GetR();
  vArray(uchar) G = the_image->GetG();
  vArray(uchar) B = the_image->GetB();

  long size = the_image->Size();

  long counter = 0;
  long i;
  for (i = 0; i < size; i++)
  {
    if ((R[i] == r) && (G[i] == g) && (B[i] == b))
    {
      counter = counter+1;
    }
  }

  vPrint("%li pixels\n", counter);
  vdelete(the_image);
  return 1;
}


long vInterpreter3::CompareInputs0()
{
  if (input == 0) return 0;
  vPrint("\nEnter input1 and input2\n");
  long input1, input2;
  input1 = input2 = 0;
  vScan("%li %li", &input1, &input2);
  return CompareInputs(input1, input2);
}


long vInterpreter3::CompareInputs(long input1, long input2)
{
  if ((input1 < 0) || (input2 < 0) ||
      (input1 >= number_of_inputs) || (input2 >= number_of_inputs))
  {
    return 0;
  }

  double_image * image1 = new double_image(inputs[input1]->current_image);
  double_image * image2 = new double_image(inputs[input2]->current_image);
  long result = color_imageEqual(image1, image2);
  if (result > 0)
  {
    vPrint("\nImages are equal\n");
  }
  else
  {
    vPrint("\nImages are different, result = %li\n", result);
  }
  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter3::compare_images()
{
  if (input == 0) return 0;

  vPrint("\nEnter index1 and index2\n");
  long index1, index2;
  index1 = index2 = 0;
  vScan("%li %li", &index1, &index2);
  if ((index1 < 0) || (index2 < 0) ||
      (index1 >= input->buffer_size) || (index2 >= input->buffer_size))
  {
    return 0;
  }

  long buffer_size = input->buffer_size;
  long temp1 = (input->buffer_index + buffer_size - index1) % buffer_size;
  long temp2 = (input->buffer_index + buffer_size - index2) % buffer_size;
  double_image * image1 = new double_image(input->image_buffer[temp1]);
  double_image * image2 = new double_image(input->image_buffer[temp2]);
  long result = color_imageEqual(image1, image2);
  if (result > 0)
  {
    vPrint("\nImages are equal\n");
  }
  else
  {
    vPrint("\nImages are different, result = %li\n", result);
  }
  vdelete(image1);
  vdelete(image2);
  return 1;
}


long vInterpreter3::HandPointsSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandPointsSlow(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandPointsSlow(const char * shape, double roll, 
                                double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawPoints5(size, bbox_size, 
                                            roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandPointsSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandPointsSlow(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandPointsSlow(const char * shape, double roll, 
                                    double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawPoints5(size, bbox_size, 
                                              roll, pitch, yaw);
//    color_image * result = new color_image(500, 500);
//    vWriteValue(result, (uchar) 0);
    vdelete(result);
  }
  return 1;
}


long vInterpreter3::PreprocessPolygons()
{
  vHandModel hand_model;
  long success = hand_model.PreprocessPolygons();
  return success;
}


long vInterpreter3::HandPolygonsSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandPolygonsSlow(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandPolygonsSlow(const char * shape, double roll, 
                                  double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawPolygons5(size, bbox_size, 
                                            roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandPolygonsSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandPolygonsSlow(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandPolygonsSlow(const char * shape, double roll, 
                                    double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawPolygons5(size, bbox_size, 
                                              roll, pitch, yaw);
//    color_image * result = new color_image(500, 500);
//    vWriteValue(result, (uchar) 0);
    vdelete(result);
  }
  return 1;
}


  
long vInterpreter3::HandLinksSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandLinksSlow(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandLinksSlow(const char * shape, double roll, 
                               double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

//  char order[1000];
//  vPrint("\nEnter order:\n");
//  vScan("%s", order);
//  hand_shape.AltWorldMatrices(order[0], order[1], order[2]);

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawLinks5(size, bbox_size, 
                                            roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandLinksSlow0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandLinksSlow(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandLinksSlow(const char * shape, double roll, 
                                    double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawLinks5(size, bbox_size, 
                                              roll, pitch, yaw);
//    color_image * result = new color_image(500, 500);
//    vWriteValue(result, (uchar) 0);
    vdelete(result);
  }
  return 1;
}


long vInterpreter3::HandPoints0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandPoints(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandPoints(const char * shape, double roll, 
                                double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawPoints5(size, bbox_size, 
                                            (float) roll, (float) pitch, (float) yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandPoints0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandPoints(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandPoints(const char * shape, double roll, 
                                    double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawPoints5(size, bbox_size, 
                                              (float) roll, (float) pitch, (float) yaw);
//    color_image * result = new color_image(500, 500);
//    vWriteValue(result, (uchar) 0);
    vdelete(result);
  }
  return 1;
}


long vInterpreter3::HandPolygons0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandPolygons(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandPolygons(const char * shape, double roll, 
                                  double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawPolygons5(size, bbox_size, 
                                            (float) roll, (float) pitch, (float) yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandPolygons0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandPolygons(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandPolygons(const char * shape, double roll, 
                                    double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawPolygons5(size, bbox_size, 
                                              (float) roll, (float) pitch, (float) yaw);
//    color_image * result = new color_image(500, 500);
//    vWriteValue(result, (uchar) 0);
    vdelete(result);
  }
  return 1;
}


  
long vInterpreter3::HandLinks0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw:\n");
  vScan("%s %lf %lf %lf", buffer, &roll, &pitch, &yaw);
  return HandLinks(buffer, roll, pitch, yaw);
}


long vInterpreter3::HandLinks(const char * shape, double roll, 
                               double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.DrawLinks5(size, bbox_size, 
                                           (float) roll, (float) pitch, (float) yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::TimeHandLinks0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeHandLinks(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeHandLinks(const char * shape, double roll, 
                                   double pitch, double yaw, long times)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    color_image * result = hand_shape.DrawLinks5(size, bbox_size, 
                                             (float) roll, (float) pitch, (float) yaw);
    vdelete(result);
  }

  vPrint("\nDone\n");
  return 1;
}


long vInterpreter3::TestScaleX0()
{
  if (input == 0) return 0;
  char buffer[1000];
  double roll, pitch, yaw, x_scale;
  roll = pitch = yaw = x_scale = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, x_scale:\n");
  vScan("%s %lf %lf %lf %lf", buffer, &roll, &pitch, &yaw, &x_scale);
  return TestScaleX(buffer, roll, pitch, yaw, x_scale);
}


long vInterpreter3::TestScaleX(const char * shape, double roll, 
                                double pitch, double yaw, double x_scale)
{
  if (input == 0) return 0;
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  hand_shape.CreateDensePoints(vRound(canonical_bbox_size * sqrt((float) 2)));

  return 1;
}

long vInterpreter3::AltHandPoints0()
{
  if (input == 0) return 0;
  char order[1000];
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter order, shape, roll, pitch, yaw:\n");
  vScan("%s %s %lf %lf %lf", order, buffer, &roll, &pitch, &yaw);
  return AltHandPoints(order, buffer, roll, pitch, yaw);
}


long vInterpreter3::AltHandPoints(const char * order, const char * shape, 
                                   double roll, double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  const long size = 500;
  const long bbox_size = 400;
  long success = hand_shape.AltWorldMatrices(order[0], order[1], order[2]);
  if (success == 0)
  {
    vPrint("Failed to set world matrices\n");
  }
  else
  {
    vPrint("Set world matrices successfully\n");
  }

  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.AltDrawPoints5(size, bbox_size, 
                                            roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::AltHandPolygons0()
{
  if (input == 0) return 0;
  char order[1000];
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter order, shape, roll, pitch, yaw:\n");
  vScan("%s %s %lf %lf %lf", order, buffer, &roll, &pitch, &yaw);
  return AltHandPolygons(order, buffer, roll, pitch, yaw);
}


long vInterpreter3::AltHandPolygons(const char * order, const char * shape, 
                                     double roll, double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  const long size = 500;
  const long bbox_size = 400;
  long success = hand_shape.AltWorldMatrices(order[0], order[1], order[2]);
  if (success == 0)
  {
    vPrint("Failed to set world matrices\n");
  }
  else
  {
    vPrint("Set world matrices successfully\n");
  }

  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.AltDrawPolygons5(size, bbox_size, 
                                                 roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::AltHandLinks0()
{
  if (input == 0) return 0;
  char order[1000];
  char buffer[1000];
  double roll, pitch, yaw;
  roll = pitch = yaw = 0;
  vPrint("\nEnter order, shape, roll, pitch, yaw:\n");
  vScan("%s %s %lf %lf %lf", order, buffer, &roll, &pitch, &yaw);
  return AltHandLinks(order, buffer, roll, pitch, yaw);
}


long vInterpreter3::AltHandLinks(const char * order, const char * shape, 
                                  double roll, double pitch, double yaw)
{
  if (input == 0) return 0;
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  const long size = 500;
  const long bbox_size = 400;
  long success = hand_shape.AltWorldMatrices(order[0], order[1], order[2]);
  if (success == 0)
  {
    vPrint("Failed to set world matrices\n");
  }
  else
  {
    vPrint("Set world matrices successfully\n");
  }

  hand_shape.SetScale(x_scale, y_scale, z_scale);
  color_image * result = hand_shape.AltDrawLinks5(size, bbox_size, 
                                              roll, pitch, yaw);
  ShowImage(result);
  return 1;
}


long vInterpreter3::SetScale0()
{
  double x, y, z;
  x = y = z = 0;
  vPrint("\nEnter x scale, y scale, z scale:\n");
  vScan("%lf %lf %lf", &x, &y, &z);
  return SetScale(x, y, z);
}


long vInterpreter3::SetScale(double x, double y, double z)
{
  x_scale = (float) x;
  y_scale = (float) y;
  z_scale = (float) z;
  return 1;
}


long vInterpreter3::SetCanonicalSizes0()
{
  long size, bbox_size;
  size = bbox_size = 0;
  vPrint("\nEnter canonical image size, canonical bounding box size:\n");
  vScan("%li %li", &size, &bbox_size);
  return SetCanonicalSizes(size, bbox_size);
}


long vInterpreter3::SetCanonicalSizes(long size, long bbox_size)
{
  if ((size > 0) && (size <= 2000) && (bbox_size > 0) && (bbox_size < size - 5)) 
  {
    canonical_image_size = size;
    canonical_bbox_size = bbox_size;
  }
  return 1;
}



long vInterpreter3::ShapeMatrices0()
{
  char order[1000];
  char shape[1000];
  vPrint("\nEnter order, shape:\n");
  vScan("%s %s", order, shape);
  return ShapeMatrices(order, shape);
}


long vInterpreter3::ShapeMatrices(const char * order, const char * shape)
{
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long success = hand_shape.AltWorldMatrices(order[0], order[1], order[2]);
  if (success == 0)
  {
    vPrint("Failed to set world matrices\n");
  }
  else
  {
    vPrint("Set world matrices successfully\n");
  }

  hand_shape.PrintMatrices();
  return 1;
}


long vInterpreter3::LinkMatrices0()
{
  char order[1000];
  char shape[1000];
  long link_id = 0;
  vPrint("\nEnter order, shape, link_id:\n");
  vScan("%s %s %li", order, shape, &link_id);
  return LinkMatrices(order, shape, link_id);
}


long vInterpreter3::LinkMatrices(const char * order, const char * shape, 
                                  long link_id)
{
  vHandShapeSlow hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long success = hand_shape.AltWorldMatrices(order[0], order[1], order[2]);
  if (success == 0)
  {
    vPrint("Failed to set world matrices\n");
  }
  else
  {
    vPrint("Set world matrices successfully\n");
  }

  hand_shape.PrintLinkMatrices(link_id);
  return 1;
}


long vInterpreter3::SideLengthStats()
{
  vHandModel model;
  model.SideLengthStats();
  return 1;
}


long vInterpreter3::CutTriangle0()
{
  if (input == 0) return 0;
  long max_side = 0;
  class_pointer (long) rows = vnew (long, 3);
  class_pointer (long) cols = vnew (long, 3);
  vPrint("\nEnter three 2D points, and length of max side:\n");
  vScan("%li %li %li %li %li %li %li", &(rows[0]), &(cols[0]),
         &(rows[1]), &(cols[1]), &(rows[2]), &(cols[2]), &max_side);
  long result = CutTriangle(rows, cols, max_side);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::CutTriangle(class_pointer (long) rows, class_pointer (long) cols, long max_side)
{
  if (input == 0) return 0;
  long image_rows = input->current_image->Rows();
  long image_cols = input->current_image->Cols();
  vPrecomputedLengths::PrecomputeValues(image_rows, image_cols);

  v3dMatrix<long> result_matrix(100000, 3, 2);
  matrix_pointer (long) result_rows = result_matrix.Matrix2(0);
  matrix_pointer (long) result_cols = result_matrix.Matrix2(1);
  long number = vCutTriangle(rows, cols, result_rows, result_cols, max_side);

  vPrint("number = %li\n", number);

  color_image * the_image1 = new color_image(image_rows, image_cols);
  vWriteValue(the_image1, (uchar) 0);

  color_image * the_image2 = new color_image(image_rows, image_cols);
  vWriteValue(the_image2, (uchar) 0);

  vDrawTriangle1(rows, cols, the_image1, 255);
  ShowImage(the_image1);
  vDrawTriangles1(number, result_rows, result_cols, the_image2, 255);
  ShowImage(the_image2);

  return 1;
}


long vInterpreter3::TrianglePoints3D0()
{
  if (input == 0) return 0;
  float x1, y1, z1, x2, y2, z2, x3, y3, z3, max_distance;
  x1 = y1 = z1 = x2 = y2 = z2 = x3 = y3 = z3 = max_distance = (float) 0;
  vPrint("\nEnter three 3D vertices and max_distance:\n");
  vScan("%f %f %f %f %f %f %f %f %f %f", 
          &x1, &y1, &z1, &x2, &y2, &z2, &x3, &y3, &z3, &max_distance);

  vPoint3f vertices[3];
  vertices[0] = vPoint3f(x1, y1, z1);
  vertices[1] = vPoint3f(x2, y2, z2);
  vertices[2] = vPoint3f(x3, y3, z3);
  
  return TrianglePoints3D(vertices, max_distance);
}


long vInterpreter3::TrianglePoints3D(vPoint3f * vertices, float max_distance)
{
  if (input == 0) return 0;

  vector<vPoint3f> points;
  long number = vTrianglePixels3D(vertices, &points, max_distance);
  long i;
  for (i = 0; i < number ;i++)
  {
    vPrint("%4li  ", i);
    points[i].Print("");
  }

  return 1;
}


long vInterpreter3::TrianglePixels0()
{
  if (input == 0) return 0;
  class_pointer (long) rows = vnew (long, 3);
  class_pointer (long) cols = vnew (long, 3);
  vPrint("\nEnter three pixels:\n");
  vScan("%li %li %li %li %li %li", &(rows[0]), &(cols[0]),
         &(rows[1]), &(cols[1]), &(rows[2]), &(cols[2]));
  long result = TrianglePixels(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::TrianglePixels(class_pointer (long) rows, class_pointer (long) cols)
{
  if (input == 0) return 0;
  vector<vPoint> pixels(300000);
  long number = vTrianglePixels(rows, cols, &pixels);
  
  long image_rows = input->current_image->Rows();
  long image_cols = input->current_image->Cols();
  grayscale_image * the_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(the_image, (uchar) 0);
  vArray2(uchar) matrix2 = the_image->Matrix2(0);
  long i;
  for (i = 0; i < number; i++)
  {
    long row = pixels[i].row;
    long col = pixels[i].col;
    if (the_image->check_bounds(row, col) > 0)
    {
      matrix2[row][col] = 255;
    }
  }

  ShowImage(the_image);
  return 1;
}


long vInterpreter3::TimeTrianglePixels0()
{
  long times = 0;
  class_pointer (long) rows = vnew (long, 3);
  class_pointer (long) cols = vnew (long, 3);

  vPrint("\nEnter three pixels and times:\n");
  vScan("%li %li %li %li %li %li %li", &(rows[0]), &(cols[0]),
         &(rows[1]), &(cols[1]), &(rows[2]), &(cols[2]), &times);
  long result = TimeTrianglePixels(rows, cols, times);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::TimeTrianglePixels(class_pointer (long) rows, class_pointer (long) cols, long times)
{
  vector<vPoint> pixels(300000);
  long i;
  for (i = 0; i < times; i++)
  {
    long number = vTrianglePixels(rows, cols, &pixels);
  }

  vPrint("\nDone\n");
  
  return 1;
}


long vInterpreter3::TrianglePixels0b()
{
  if (input == 0) return 0;
  class_pointer (long) rows = vnew (long, 3);
  class_pointer (long) cols = vnew (long, 3);
  vPrint("\nEnter three pixels:\n");
  vScan("%li %li %li %li %li %li", &(rows[0]), &(cols[0]),
         &(rows[1]), &(cols[1]), &(rows[2]), &(cols[2]));
  long result = TrianglePixels2(rows, cols);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::TrianglePixels2(class_pointer (long) rows, class_pointer (long) cols)
{
  if (input == 0) return 0;
  vector<vPoint> pixels;
  long number = vTrianglePixelsSlow(rows, cols, &pixels);
  
  long image_rows = input->current_image->Rows();
  long image_cols = input->current_image->Cols();
  grayscale_image * the_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(the_image, (uchar) 0);
  vArray2(uchar) matrix2 = the_image->Matrix2(0);
  long i;
  for (i = 0; i < number; i++)
  {
    long row = pixels[i].row;
    long col = pixels[i].col;
    if (the_image->check_bounds(row, col) > 0)
    {
      matrix2[row][col] = 255;
    }
  }

  ShowImage(the_image);
  return 1;
}


long vInterpreter3::TimeTrianglePixels0b()
{
  long times = 0;
  class_pointer (long) rows = vnew (long, 3);
  class_pointer (long) cols = vnew (long, 3);

  vPrint("\nEnter three pixels and times:\n");
  vScan("%li %li %li %li %li %li %li", &(rows[0]), &(cols[0]),
         &(rows[1]), &(cols[1]), &(rows[2]), &(cols[2]), &times);
  long result =  TimeTrianglePixels2(rows, cols, times);
  delete_pointer (rows);
  delete_pointer (cols);
  return result;
}


long vInterpreter3::TimeTrianglePixels2(class_pointer (long) rows, class_pointer (long) cols, long times)
{
  long i;
  for (i = 0; i < times; i++)
  {
    vector<vPoint> pixels;
    long number = vTrianglePixelsSlow(rows, cols, &pixels);
  }

  vPrint("\nDone\n");
  
  return 1;
}


long vInterpreter3::TestLinePoints50()
{
  if (input == 0) return 0;

  long row1, col1, row2, col2;
  row1 = col1 = row2 = col2 = 0;
  
  vPrint("\nEnter row1, col1, row2, col2\n");
  vScan("%li %li %li %li", &row1, &col1, &row2, &col2);
  return TestLinePoints5(row1, col1, row2, col2);
}


long vInterpreter3::TestLinePoints5(long row1, long col1, long row2, long col2)
{
  if (input == 0) return 0;

  vector<vPoint> pixels;

  vLinePoints5(row1, col1, row2, col2, &pixels);
  long image_rows = input->current_image->Rows();
  long image_cols = input->current_image->Cols();
  grayscale_image * the_image = new grayscale_image(image_rows, image_cols);
  vWriteValue(the_image, (uchar) 0);

  vWriteValue4(the_image, &pixels, (uchar) 255);
  ShowImage(the_image);

  long i;
  for (i = 0; i < (long) pixels.size(); i++)
  {
    vPrint("%5li (%5li, %5li)\n", i, pixels[i].row, pixels[i].col);
  }

  return 1;
}

long vInterpreter3::TestLineCols0()
{
  if (input == 0) return 0;
  long row = 0, col = 0;
  vPrint("\nEnter row, col:\n");
  vScan("%li %li", &row, &col);
  return TestLineCols(row, col);
}


long vInterpreter3::TestLineCols(long row, long col)
{
  if (input == 0) return 0;

  static const long max_size = 128;
  static long junk = precomputed_line_columns::PrecomputeValues(max_size);
  if ((row < -max_size) || (col < -max_size) || (row >= max_size) || (col >= max_size)) return 0;

  if (input == 0) return 0;
  vArray(short) left_cols = precomputed_line_columns::LeftColsR(row, col);
  vArray(short) right_cols = precomputed_line_columns::RightColsR(row, col);

  long image_rows = 300;
  long image_cols = 300;
  color_image * the_image = new color_image(image_rows, image_cols);
  vArray2(uchar) R = the_image->GetR2();
  vArray2(uchar) G = the_image->GetG2();
  vWriteValue(the_image, (uchar) 0);

  long increment;
  if (row == 0)
  {
    increment = 1;
  }
  else
  {
    increment = row/function_absolute (row);
  }


  long i;
  long counter = 0;
  for (i = 150; i != 150+row+ increment ; i+= increment)
  {
    long left_col = vAbs(150+left_cols[counter]);
    long right_col = vAbs(150+right_cols[counter]);
    counter ++;
    long horizontal;
    for (horizontal = left_col; horizontal <= right_col; horizontal ++)
    {
      R[i][horizontal] = 255;
      G[i][horizontal] = 255;
    }
  }

  ShowImage(the_image);

  return 1;
}


long vInterpreter3::test_line_pixels()
{
  if (input == 0) return 0;
  long first_vertical = 0, first_horizontal = 0, second_vertical = 0, second_horizontal = 0;
  vPrint("\nEnter first row, first column, second row, second column:\n");
  vScan("%li %li %li %li", & first_vertical, & first_horizontal, & second_vertical, & second_horizontal);
  long horizontal_difference = second_horizontal - first_horizontal;
  long vertical_difference = second_vertical - first_vertical;

  static const long max_size = 50;
  static long junk = precomputed_line_columns::PrecomputeValues(max_size);
  if ((vertical_difference < -max_size) || (horizontal_difference < -max_size) || 
      (vertical_difference >= max_size) || (horizontal_difference >= max_size)) return 0;
  
  long_matrix line_pixels = function_line_pixels (first_vertical, first_horizontal, second_vertical, second_horizontal);

  long image_rows = 300;
  long image_cols = 300;
  color_image * the_image = new color_image(image_rows, image_cols);
  vArray2(uchar) R = the_image->GetR2();
  vArray2(uchar) G = the_image->GetG2();
  vWriteValue(the_image, (uchar) 0);

  long counter;
  for (counter = 0; counter < line_pixels.horizontal(); counter ++)
  {
    long vertical = line_pixels (0, counter);
    long horizontal = line_pixels (1, counter);
    R[vertical] [horizontal] = 255;
    G[vertical] [horizontal] = 255;
  }

  ShowImage(the_image);

  return 1;
}


long vInterpreter3::OnlineSegmentation()
{
  switch(online_segmentation)
  {
  case 0:
    ShowCameraHand();
    TrivialSegmentation();
    break;

  case 1:
    SkinHand();
    break;

  case 2:
    ForegroundHand();
    break;

  default:
    SkinHand();
    break;
  }

  return 1;
}


long vInterpreter3::OnlineRanking()
{
  long index = 0;
 
  if ((hand_edge_image == 0) || (right_hand_image == 0))
  {
    return 0;
  }
  ShowImage(hand_edge_image->ToGrayCopy());

  switch(online_ranking)
  {
  case 0:
    RankViews();
    break;

  case 1:
    index = current_input_index;
    CEB_Scores();
    SetCurrentInput(index);
    RankViews28();
    break;

  case 2:
    index = current_input_index;
    CEB_Scores();
    SetCurrentInput(index);
    DOLVS_Stats2();
    RankViews26();
    break;

  default:
    RankViews();
    break;
  }

  return 1;
}


long vInterpreter3::CameraBackground()
{
  if (camera_input == 0) return 0;
  if (camera_input->current_image == 0) return 0;

  long camera_input_index = CameraInputIndex();
  SetCurrentInput(camera_input_index);
  vSequenceStatistics<uchar> stats(input->current_image->Rows(), 
				                            input->current_image->Cols(), 3);
  long buffer_size = camera_input->buffer_size;
  long num_frames = Min((long) 20, buffer_size);

  vPrint("\n");
  long current_frame;
  for (current_frame = -num_frames + 1; 
       current_frame <= 0; 
       current_frame++)
  {
    long index = (input->buffer_index + buffer_size + current_frame) % buffer_size;
    general_image * the_image = input->image_buffer[index];
    if (the_image == 0) 
    {
      vPrint("not enough images in the buffer\n");
      return 0;
    }
    if ((the_image->TypeName() != UcharType) ||
        (the_image->Bands() != 3))
    {
      vPrint("Non-RGB image found\n");
      return 0;
    }
    color_image * rg_image = vNormalizeIntensity((color_image *) the_image);
    stats.ProcessValues(rg_image);
    vdelete(rg_image);
    vPrint("%li   \r", current_frame);
  }

  vPrint("\n");
  v3dMatrix<double> * means = stats.GetMeans();

  for (current_frame = -num_frames + 1; 
       current_frame <= 0; 
       current_frame++)
  {
    long index = (input->buffer_index + buffer_size + current_frame) % buffer_size;
    general_image * the_image = input->image_buffer[index];
    color_image * rg_image = vNormalizeIntensity((color_image *) the_image);
    stats.ProcessDeviations(rg_image);
    vdelete(rg_image);
    vPrint("%li   \r", current_frame);
  }

  vPrint("\n");
  ShowImage(vCopyImage(means));
  v3dMatrix<double> * deviations = stats.GetDeviations();
  ShowImage(vCopyImage(deviations));
  vdelete(background_model);
  background_model = new vBackgroundModel(&stats);

  return 1;
}


long vInterpreter3::ShowBackground()
{
  if (input == 0) return 0;
  if (background_model == 0) return 0;
  v3dMatrix<double> * mean = background_model->BackgroundMean();
  ShowImage(vCopyImage(mean));
  v3dMatrix<double> * std = background_model->BackgroundStd();
  ShowImage(vCopyImage(std));

  return 1;
}


long vInterpreter3::ForegroundThreshold0()
{
  double threshold = 0;
  vPrint("\nEnter threshold:\n");
  vScan("%lf", &threshold);
  return ForegroundThreshold(threshold);
}


long vInterpreter3::ForegroundThreshold(double threshold)
{
  foreground_threshold = threshold;
  return 1;
}


long vInterpreter3::ForegroundHand()
{
  if (input == 0) return 0;
  if (background_model == 0) return 0;
  vdelete(original_video_frame);
  original_video_frame = input->current_image->ToRGBCopy();

  color_image * rg = vNormalizeIntensity(original_video_frame);
  ShowImage(rg);
  SubtractBackground(foreground_threshold);
  BiggestComponent();
  FillUpHoles();
  KeepPalm2();
  SelectPixels(7);
  HandAndEdgeImages();

  return 1;
}


long vInterpreter3::PrecomputedTriangle0()
{
  if (input == 0) return 0;
  long rows[3];
  long cols[3];

  vPrint("\nEnter three points:\n");
  vScan("%li %li %li %li %li %li", 
         &(rows[0]), &(cols[0]), &(rows[1]), &(cols[1]),
         &(rows[2]), &(cols[2]));
  return PrecomputedTriangle(rows, cols);
}


long vInterpreter3::PrecomputedTriangle(long * rows, long * cols)
{
  static long junk = vPrecomputedTriangles::PrecomputeValues(5);
  if (input == 0) return 0;

  long min_row, max_row, min_col, max_col;
  min_row = max_row = rows[0];
  min_col = max_col = cols[0];
  long i;
  for (i = 1; i < 3; i++)
  {
    if (rows[i] < min_row) min_row = rows[i];
    else if (rows[i] > max_row) max_row = rows[i];
    if (cols[i] < min_col) min_col = cols[i];
    else if (cols[i] > max_col) max_col = cols[i];
  }

  grayscale_image * the_image = new grayscale_image(max_row + 50, max_col + 50);
  vWriteValue(the_image, (uchar) 0);
  vArray2(uchar) values = the_image->Matrix2(0);

  long i1 = rows[1] - rows[0];
  long j1 = cols[1] - cols[0];
  long i2 = rows[2] - rows[0];
  long j2 = cols[2] - cols[0];

  vArray(vPoint) pixels = vPrecomputedTriangles::PixelsR(i1, j1, i2, j2);
  long number = pixels[0].col;
  for (i = 1; i < number; i++)
  {
    long row = pixels[i].row + rows[0];
    long col = pixels[i].col + cols[0];
    values[row][col] = 160;
  }

  ShowImage(the_image);

  grayscale_image * copy = the_image->ToGrayCopy();
  for (i = 0; i < 3; i++)
  {
    vDrawCross(copy, rows[i], cols[i], 1, 3, (uchar) 255);
  }

  ShowImage(copy);
  return 1;
}


long vInterpreter3::TimeVertexOperations0()
{
  char buffer[1000];
  double roll, pitch, yaw;
  long times = 0;
  roll = pitch = yaw = 0;
  vPrint("\nEnter shape, roll, pitch, yaw, times:\n");
  vScan("%s %lf %lf %lf %li", buffer, &roll, &pitch, &yaw, &times);
  return TimeVertexOperations(buffer, roll, pitch, yaw, times);
}


long vInterpreter3::TimeVertexOperations(const char * shape, double roll, 
                                          double pitch, double yaw, long times)
{
  vHandShape hand_shape(shape);
  if (hand_shape.IsValid() <= 0)
  {
    vPrint("couldn't load shape\n");
    return 0;
  }

  long size = canonical_image_size;
  long bbox_size = canonical_bbox_size;
  long i;
  for (i = 0; i < times; i++)
  {
    hand_shape.SetScale(x_scale, y_scale, z_scale);
    vMatrix<float> camera_points = hand_shape.CameraPoints((float) roll, (float) pitch, (float) yaw);
    vMatrix<long> pixels_matrix = hand_shape.CameraToPixel(camera_points, size, bbox_size);
  }

  vPrint("\nDone\n");
  return 1;
}


long vInterpreter3::TimeImage0()
{
  long times = 0;
  vPrint("\nEnter times:\n");
  vScan("%li", &times);
  return TimeImage(times);
}


long vInterpreter3::TimeImage(long times)
{
  long i;
  long size = canonical_image_size;
  for (i = 0; i < times; i++)
  {
    grayscale_image link_id_matrix(size, size);
    vWriteValue(&link_id_matrix, (uchar) 0);
    v3dMatrix<float> depth_matrix(size, size, 1);
    vWriteValue(&depth_matrix, (float) -1000000);
  }

  vPrint("\nDone\n");
  return 1;
}


/////////////////////////////////////////////////////////////////////
//                                                                 //
//           Start of functions written for CVPR 2004.             //
//                                                                 //
/////////////////////////////////////////////////////////////////////


// Create a boost map for 2D points.
long vInterpreter3::PointBM0()
{
  long points = 0, triples = 0;
  vPrint("\nEnter points, triples:\n");
  vScan("%li %li", &points, &triples);
  return PointBM(points, triples);
}


long vInterpreter3::PointBM(long points, long triples)
{
  DeleteBM();
  boost_map = new vPlaneBoostMap(points, triples);
  return 1;
}


long vInterpreter3::DeleteBM()
{
  if (boost_map == 0) return 0;
  if (cbi != 0)
  {
    if ((cbi->BoostMap() == boost_map) || (cbi->SparseMap() == boost_map) ||
        (cbi->FastMap() == boost_map))
    {
      vPrint("current boost map comes from cbi object, use DeleteCBI()\n");
      return 0;
    }
  }
  if (sbi != 0)
  {
    if ((sbi->BoostMap() == boost_map) || (sbi->SparseMap() == boost_map) ||
        (sbi->FastMap() == boost_map))
    {
      vPrint("current boost map comes from sbi object, use DeleteSBI()\n");
      return 0;
    }
  }
  if (strcmp(boost_map->ClassName(), "V_PlaneBoostMap") == 0)
  {
    vPlaneBoostMap * temp = (vPlaneBoostMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_BoostMap3") == 0)
  {
    V_BoostMap3 * temp = (V_BoostMap3 *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_BoostMap2") == 0)
  {
    V_BoostMap2 * temp = (V_BoostMap2 *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_SparseMap") == 0)
  {
    V_SparseMap * temp = (V_SparseMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_PlaneSparseMap") == 0)
  {
    vPlaneSparseMap * temp = (vPlaneSparseMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_ChamferSparseMap") == 0)
  {
    vChamferSparseMap * temp = (vChamferSparseMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_FastMap") == 0)
  {
    V_FastMap * temp = (V_FastMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_PlaneFastMap") == 0)
  {
    vPlaneFastMap * temp = (vPlaneFastMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_ChamferFastMap") == 0)
  {
    vChamferFastMap * temp = (vChamferFastMap *) boost_map;
    vdelete(temp);
  }
  
  else if (strcmp(boost_map->ClassName(), "V_BoostMap") == 0)
  {
    vdelete(boost_map);
  }

  else
  {
    exit_error("Unknown subclass of class_BoostMap\n");
  }
  
  boost_map = 0;
  return 1;
}


long vInterpreter3::BM_PointImage()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_PlaneBoostMap") != 0) 
  {
    return 0;
  }
  if (input == 0) return 0;

  vPlaneBoostMap * plane_bm = (vPlaneBoostMap *) boost_map;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  color_image * the_image = plane_bm->PointImage(rows, cols);
  if (the_image == 0)
  {
    vPrint("Failed to get point image\n");
  }
  else
  {
    ShowImage(the_image);
  }
  return 1;
}


long vInterpreter3::BM_BasisImage()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_PlaneBoostMap") != 0) 
  {
    return 0;
  }
  if (input == 0) return 0;

  vPlaneBoostMap * plane_bm = (vPlaneBoostMap *) boost_map;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  color_image * the_image = plane_bm->BasisImage(rows, cols);
  if (the_image == 0)
  {
    vPrint("Failed to get point image\n");
  }
  else
  {
    ShowImage(the_image);
  }
  return 1;
}


long vInterpreter3::BM_PrintSummary()
{
  if (boost_map == 0) return 0;
  boost_map->PrintSummary();
  return 1;
}


long vInterpreter3::BM_PrintClassifier()
{
  if (boost_map == 0) return 0;
  boost_map->PrintClassifier();
  return 1;
}


long vInterpreter3::BM_PrintAll()
{
  if (boost_map == 0) return 0;
  boost_map->PrintAll();
  return 1;
}


long vInterpreter3::BM_Step0()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0)
  {
    return BM_Step(0);
  }
  char buffer[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", buffer);
  return BM_Step(buffer);
}


long vInterpreter3::BM_Step(const char * filename)
{
  if (boost_map == 0) return 0;
  float training_margin = boost_map->NextStep();
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") == 0)
  {
    V_BoostMap3 * bm = (V_BoostMap3 *) boost_map;
    bm->SaveClassifier(filename);
  }

  float training_error = boost_map->TrainingError();
  float validation_error = boost_map->ValidationError();
  long steps_done = boost_map->StepsDone();
  vPrint("step %li: training error/margin = %f, %f, validation = %f\n", 
          steps_done, training_error, training_margin, validation_error);
  return 1;
}


long vInterpreter3::BM_Steps0()
{
  if (boost_map == 0) return 0;
  long steps = 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0)
  {
    vPrint("\nEnter steps:\n");
    vScan("%li", &steps);
    return BM_Steps(0, steps);
  }
  else
  {
    char buffer[1000];
    vPrint("\nEnter filename, steps:\n");
    vScan("%s %li",  buffer, &steps);
    return BM_Steps(buffer, steps);
  }
}


long vInterpreter3::BM_Steps(char * buffer, long steps)
{
  if (boost_map == 0) return 0;
  long i;
  float min_validation_error = 100000;
  for (i = 0; i < steps; i++)
  {
    BM_Step(buffer);
    float validation_error = boost_map->ValidationError();
    if (validation_error < min_validation_error)
    {
      min_validation_error = validation_error;
    }
  }

  vPrint("\nmin_validation_error = %f\n", min_validation_error);
  return 1;
}


long vInterpreter3::BM_Random0()
{
  if (boost_map == 0) return 0;
  if (input == 0) return 0;
  long number = 0, times = 0;
  vPrint("\nEnter number, times:\n");
  vScan("%li %li", &number, &times);
  return BM_Random(number, times);
}


long vInterpreter3::BM_Random(long number, long times)
{
  if (boost_map == 0) return 0;
  float min_error = 10;
  long i;
  vector<float> errors(times);
  vPrint("\n");
  for (i = 0; i < times; i++)
  {
    boost_map->PickRandom(number);
    float error = boost_map->TrainingError();
    if (error < min_error)
    {
      min_error = error;
    }
    errors[i] = error;
    vPrint("done with %li of %li\r", i+1, times);
  }
  
  vPrint("\n");
  BM_PrintSummary();

  std::sort(errors.begin(), errors.end(), less<float>());
  for (i = 0; i < times; i++)
  {
    vPrint("%li, %f\n", i, errors[i]);
  }

  vPrint("times = %li, min_error = %f\n", times, min_error);

  return 1;
}


long vInterpreter3::BM_Training0()
{
  if (boost_map == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return BM_Training(number);
}


long vInterpreter3::BM_Training(long number)
{
  if (boost_map == 0) return 0;
  long success = boost_map->ChooseTrainingSet(number);
  BM_PrintSummary();
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter3::BM_Validation0()
{
  if (boost_map == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return BM_Validation(number);
}


long vInterpreter3::BM_Validation(long number)
{
  if (boost_map == 0) return 0;
  long success = boost_map->ChooseValidationSet(number);
  BM_PrintSummary();
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter3::BM_AllowNegative0()
{
  if (boost_map == 0) return 0;
  long value = 0;
  vPrint("\nEnter allow_negative value:\n");
  vScan("%li", &value);
  return BM_AllowNegative(value);
}


long vInterpreter3::BM_AllowNegative(long value)
{
  if (boost_map == 0) return 0;
  boost_map->SetAllowNegative(value);
  return 1;
}

  
long vInterpreter3::BM_AllowRemovals0()
{
  if (boost_map == 0) return 0;
  long value = 0;
  vPrint("\nEnter allow_removals value:\n");
  vScan("%li", &value);
  return BM_AllowRemovals(value);
}


long vInterpreter3::BM_AllowRemovals(long value)
{
  if (boost_map == 0) return 0;
  boost_map->SetAllowRemovals(value);
  return 1;
}


long vInterpreter3::BM_AllowLipschitz0()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  long value = 0;
  vPrint("\nEnter allow_lipschitz value:\n");
  vScan("%li", &value);
  return BM_AllowLipschitz(value);
}


long vInterpreter3::BM_AllowLipschitz(long value)
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  V_BoostMap3 * bm = (V_BoostMap3 *) boost_map;
  bm->SetAllowLipschitz(value);
  return 1;
}


long vInterpreter3::BM_AllowProjections0()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  long value = 0;
  vPrint("\nEnter allow_projections value:\n");
  vScan("%li", &value);
  return BM_AllowProjections(value);
}


long vInterpreter3::BM_AllowProjections(long value)
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  V_BoostMap3 * bm = (V_BoostMap3 *) boost_map;
  bm->SetAllowProjections(value);
  return 1;
}


long vInterpreter3::BM_Load0()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  long dimensions = 0;
  char buffer[1000];
  vPrint("\nEnter filename, dimensions:\n");
  vScan("%s %li", buffer, &dimensions);
  return BM_Load(buffer, dimensions);
}


long vInterpreter3::BM_Load(const char * filename, long dimensions)
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) return 0;
  V_BoostMap3 * bm = (V_BoostMap3 *) boost_map;
  bm->LoadClassifier2(filename, dimensions);
  return 1;
}

  
long vInterpreter3::MakeCBI0()
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  if (point_sequence == 0) return 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }

  long sampling = 0, training = 0;
  vPrint("\nEnter sampling, training:\n");
  vScan("%li %li", &sampling, &training);
  return MakeCBI(sampling, training);
}


long vInterpreter3::MakeCBI(long sampling, long training)
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  if (point_sequence == 0) return 0;
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }

  vdelete(cbi);
  vRLE_Sequence * database_edges = (vRLE_Sequence *) sequence;
  cbi = new vChamferBoostInfo(database_edges, point_sequence, 
                               sampling, training);
  return 1;
}


long vInterpreter3::DeleteCBI()
{
  if (cbi == 0) return 0;
  V_BoostMap * bm = cbi->BoostMap();
  V_SparseMap * sm = cbi->SparseMap();
  V_FastMap * fm = cbi->FastMap();
  if (boost_map == bm)
  {
    boost_map = 0;
  }

  else if (boost_map == sm)
  {
    boost_map = 0;
  }

  else if (boost_map == fm)
  {
    boost_map = 0;
  }

  vdelete(cbi);
  cbi = 0;
  return 1;
}


long vInterpreter3::SaveCBI()
{
  if (cbi == 0) return 0;
  long success = cbi->Save();
  vPrint("success = %li\n", success);
  return 1;
}


long vInterpreter3::LoadCBI0()
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }

  long sampling = 0;
  vPrint("\nEnter sampling:\n");
  vScan("%li", &sampling);
  return LoadCBI(sampling);
}

long vInterpreter3::LoadCBI(long sampling)
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }
  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0)
  {
    return 0;
  }

  DeleteCBI();
  cbi = new vChamferBoostInfo(sequence->GetFilename(), sampling);
  if (cbi == 0)
  {
    vPrint("Failed to load cbi with sampling %li\n", sampling);
  }
  else if (cbi->IsValid() <= 0)
  {
	vdelete(cbi);
	cbi = 0;
  }
  else
  {
    vPrint("Loaded cbi\n");
  }
  return 1;
}


long vInterpreter3::CBI_Indices()
{
  if (cbi == 0) return 0;
  cbi->PrintIndices();
  return 1;
}


long vInterpreter3::CBI_Distances()
{
  if (cbi == 0) return 0;
  cbi->PrintDistances();
  return 1;
}


long vInterpreter3::CBI_Distance0()
{
  if (cbi == 0) return 0;
  long i = 0, j = 0;
  vPrint("\nEnter i, j:\n");
  vScan("%li %li", &i, &j);
  return CBI_Distance(i, j);
}


long vInterpreter3::CBI_Distance(long i, long j)
{
  if (cbi == 0) return 0;
  vMatrix<float> distances = cbi->Distances();

  long frame1 = 0, rotation1 = 0, frame2 = 0, rotation2 = 0;
  long success1 = cbi->GetPose(i, &frame1, &rotation1);
  long success2 = cbi->GetPose(j, &frame2, &rotation2);
  if (success1 < 0)
  {
    vPrint("bad index %li\n", i);
    return 0;
  }

  if (success2 < 0)
  {
    vPrint("bad index %li\n", i);
    return 0;
  }

  float distance = distances(i, j);
  float max_distance = cbi->MaxDistance();
  float scaled_distance = distance * max_distance;
  vPrint("%li - %li: (%li, %li) - (%li, %li): %f, %f\n",
          i, j, frame1, rotation1, frame2, rotation2, distance, scaled_distance);
  
  SetCurrentInput(database_edge_input);
  ChamferMatching4(frame1, rotation1, frame2, rotation2);
  return 1;
}


long vInterpreter3::CBI_Triples()
{
  if (cbi == 0) return 0;
  cbi->PrintTriples();
  return 1;
}


long vInterpreter3::CBI_BM()
{
  if (cbi == 0) return 0;
  DeleteBM();
  boost_map = 0;
  long success = cbi->MakeBoostMap();
  if (success > 0) 
  {
    boost_map = cbi->BoostMap();
  }
  return success;
}


long vInterpreter3::CBI_Steps0()
{
  if (cbi == 0) return 0;
  long steps = 0;
  char base[1000];
  vPrint("\nEnter base, steps:\n");
  vScan("%s %li", base, &steps);
  return CBI_Steps(base, steps);
}


long vInterpreter3::CBI_Steps(char * base, long steps)
{
  if (cbi == 0) return 0;
  V_BoostMap * bm = cbi->BoostMap();
  long i;
  for (i = 0; i < steps; i++)
  {
    bm->NextStep();
    bm->PrintSummary();
    long success = cbi->SaveClassifier(base);
    vPrint("success = %li\n", success);

//    long done = bm->StepsDone();
//    bm->PickRandom(done);
//    bm->PrintSummary();
  }

  return 1;
}


long vInterpreter3::CBI_Random0()
{
  if (cbi == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return CBI_Random(number);
}


long vInterpreter3::CBI_Random(long number)
{
  if (cbi == 0) return 0;
  V_BoostMap * bm = cbi->BoostMap();
  bm->PickRandom(number);
  bm->PrintSummary();
  return 1;
}

  
long vInterpreter3::CBI_LoadBM0()
{
  if (cbi == 0) return 0;
  long steps = 0;
  char base[1000];
  vPrint("\nEnter base, steps:\n");
  vScan("%s %li", base, &steps);
  return CBI_LoadBM(base, steps);
}


long vInterpreter3::CBI_LoadBM(char * base, long steps)
{
  if (cbi == 0) return 0;
  long result = cbi->LoadClassifier(base, steps);
  return result;
}


long vInterpreter3::CBI_Training0()
{
  if (cbi == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return CBI_Training(number);
}


long vInterpreter3::CBI_Training(long number)
{
  if (cbi == 0) return 0;
  long result = cbi->SetTrainingNumber(number);
  vPrint("cbi training number= %li\n", result);
  return 1;
}


long vInterpreter3::CBI_Validation0()
{
  if (cbi == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return CBI_Validation(number);
}


long vInterpreter3::CBI_Validation(long number)
{
  if (cbi == 0) return 0;
  long result = cbi->SetValidationNumber(number);
  vPrint("cbi validation number= %li\n", result);
  return 1;
}


long vInterpreter3::CBI_SM0()
{
  if (cbi == 0) return 0;
  long number = 0;
  vPrint("\nEnter number of candidates:\n");
  vScan("%li", &number);
  return CBI_SM(number);
}


long vInterpreter3::CBI_SM(long number)
{
  if (cbi == 0) return 0;
  DeleteBM();
  boost_map = 0;
  long success = cbi->MakeSparseMap(number);
  if (success > 0) 
  {
    boost_map = cbi->SparseMap();
  }
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter3::CBI_FM()
{
  if (cbi == 0) return 0;
  DeleteBM();
  boost_map = 0;
  long success = cbi->MakeFastMap();
  if (success > 0) 
  {
    boost_map = cbi->FastMap();
  }
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter3::TimeExp0()
{
  long times = 0;
  float argument = (float) 0;
  vPrint("\nEnter times, argument\n");
  vScan("%li %f", &times, &argument);
  return TimeExp(times, argument);
}


long vInterpreter3::TimeExp(long times, float argument)
{
  static const float e = (float) 2.71828182845905;
  
  float result;
  long i;
  vPrint("times = %li, argument = %f\n", times, argument);
  for (i = times -1; i >= 0; i--)
  {
    result = pow(e, argument + i / (float) times);
  }
  vPrint("\nresult = %f\n", result);
  return 1;
}


long vInterpreter3::TimePrecomputedExp0()
{
  long times = 0;
  float argument = (float) 0;
  vPrint("\nEnter times, argument\n");
  vScan("%li %f", &times, &argument);
  return TimePrecomputedExp(times, argument);
}


long vInterpreter3::TimePrecomputedExp(long times, float argument)
{
  float result;
  long i;
  vPrint("times = %li, argument = %f\n", times, argument);
  for (i = times -1; i >= 0; i--)
  {
    result = vPrecomputedExp::Exp(argument + i / (float) times);
  }
  vPrint("\nresult = %f\n", result);
  return 1;
}


long vInterpreter3::ACD_CheckInputs()
{
  vVideoFile * sequence;

  if ((acd_input1 < 0) || (acd_input1 >= number_of_inputs))
  {
    return 0;
  }
  sequence = inputs[acd_input1]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0) return 0;

  if ((acd_input2 < 0) || (acd_input2 >= number_of_inputs))
  {
    return 0;
  }

  sequence = inputs[acd_input2]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0) return 0;
  return 1;
}


long vInterpreter3::ACD_SetInputs0()
{
  long input1 = 0, input2 = 0;
  vPrint("\nEnter input1, input2:\n");
  vScan("%li %li", &input1, &input2);
  return ACD_SetInputs(input1, input2);
}


long vInterpreter3::ACD_SetInputs(long input1, long input2)
{
  vdelete(acd);
  acd = 0;
  if (input1 < 0) return 0;
  if (input2 < 0) return 0;
  acd_input1 = input1;
  acd_input2 = input2;
  if (ACD_CheckInputs() <= 0) return 0;
  
  vRLE_Sequence * edges1 = (vRLE_Sequence *) inputs[acd_input1]->sequence;
  vRLE_Sequence * edges2 = (vRLE_Sequence *) inputs[acd_input2]->sequence;

  if (acd_input1 == acd_input2)
  {
    acd = new vAllChamferDistances(edges1);
  }
  else
  {
    acd = new vAllChamferDistances(edges1, edges2);
    if (strcmp(edges1->GetFilename(), edges2->GetFilename()) > 0)
    {
      long temp = acd_input1;
      acd_input1 = acd_input2;
      acd_input2 = temp;
    }
  }

  vPrint("acd_input1 = %li, acd_input2 = %li\n",
          acd_input1, acd_input2);

  return 1;
}


long vInterpreter3::ACD_SaveDirected0()
{
  if (ACD_CheckInputs() <= 0) return 0;
  if (acd == 0) return 0;

  long start = 0, end = 0, order = 0;
  vPrint("\nEnter start, end, order:\n");
  vScan("%li %li %li", &start, &end, &order);
  return ACD_SaveDirected(start, end, order);
}


long vInterpreter3::ACD_SaveDirected(long start, long end, long order)
{
  if (ACD_CheckInputs() <= 0) return 0;
  if (acd == 0) return 0;

  vRLE_Sequence * edges1 = (vRLE_Sequence *) inputs[acd_input1]->sequence;
  vRLE_Sequence * edges2 = (vRLE_Sequence *) inputs[acd_input2]->sequence;

  vRLE_Sequence * arg_sequence = edges1;
  vRLE_Sequence * arg_points = edges2;
  if (order == 2)
  {
    arg_sequence = edges2;
    arg_points = edges1;
  }

  const char * path_name = arg_points->GetFilename();
  char * filename = vReplaceEnding2(path_name, ".point_set");
  vPointSetSequence * points = vPointSetSequence::Load(filename);
  if (points == 0)
  {
    vPrint("Error, didn't find point sequence in %s\n", filename);
    vdelete2(filename);
    return 0;
  }

  vdelete2(filename);
  long success = acd->SaveDirectedDistances(arg_sequence, points,
                                            start, end);
  if (success <= 0)
  {
    vPrint("Failed to save distances\n");
  }
  else
  {
    vPrint("Saved distances successfully\n");
  }

  vdelete(points);
  return success;
}


long vInterpreter3::ACD_TestDirected0()
{
  if (ACD_CheckInputs() <= 0) return 0;
  
  long frame1 = 0, view1 = 0, frame2 = 0, view2 = 0, order = 0;
  vPrint("\nEnter frame1, view1, frame2, view2, order:\n");
  vScan("%li %li %li %li %li", &frame1, &view1, &frame2, &view2, &order);

  return ACD_TestDirected(frame1, view1, frame2, view2, order);
}


long vInterpreter3::ACD_TestDirected(long frame1, long view1, 
                                      long frame2, long view2, long order)
{
  if (ACD_CheckInputs() <= 0) return 0;
  if (acd == 0) return 0;
  if (point_sequence == 0) return 0;

  long current = current_input_index;
  if (acd_input1 != acd_input2)
  {
    SetCurrentInput(acd_input2);
    ViewRank(frame2, view2);
    ThresholdCurrentImage(1);
  
    SetCurrentInput(acd_input1);
    ViewRank(frame1, view1);
    ThresholdCurrentImage(1);

    ChamferMatching(acd_input2);
  }
  else
  {
    ChamferMatching4(frame1, view1, frame2, view2);
  }
  SetCurrentInput(current);

  v3dMatrix<float> * distances;
  long index;

  if (order == 2)
  {
    distances = acd->DirectedDistances(frame2, view2, order);
    index = frame1 * 4 + view1;
  }
  else
  {
    distances = acd->DirectedDistances(frame1, view1, order);
    index = frame2 * 4 + view2;
  }

  if (distances == 0)
  {
    vPrint("Failed to read distances\n");
    return 1;
  }

  float distance = distances->Matrix()[index];
  vPrint("distance = %f\n", distance);
  vdelete(distances);

  return 1;
}


long vInterpreter3::ACD_Save0()
{
  if (acd == 0) return 0;

  long start = 0, end = 0, chunk = 0;
  vPrint("\nEnter start, end, chunk:\n");
  vScan("%li %li %li", &start, &end, &chunk);
  return ACD_Save(start, end, chunk);
}


long vInterpreter3::ACD_Save(long start, long end, long chunk)
{
  if (acd == 0) return 0;
  long success = acd->SaveDistances(start, end, chunk);

  if (success <= 0)
  {
    vPrint("Failed to save distances\n");
  }
  else
  {
    vPrint("Saved distances successfully\n");
  }

  return 1;
}


long vInterpreter3::ACD_Test0()
{
  if (ACD_CheckInputs() <= 0) return 0;
  if (acd == 0) return 0;

  long frame1 = 0, view1 = 0, frame2 = 0, view2 = 0;
  vPrint("\nEnter frame1, view1, frame2, view2:\n");
  vScan("%li %li %li %li", &frame1, &view1, &frame2, &view2);

  return ACD_Test(frame1, view1, frame2, view2);
}


long vInterpreter3::ACD_Test(long frame1, long view1, 
                              long frame2, long view2)
{
  if (acd == 0) return 0;
  if (ACD_CheckInputs() == 0) return 0;

  long current = current_input_index;
  SetCurrentInput(acd_input2);
  if (acd_input1 != acd_input2)
  {
    ViewRank(frame2, view2);
    ThresholdCurrentImage(1);
  
    SetCurrentInput(acd_input1);
    ViewRank(frame1, view1);
    ThresholdCurrentImage(1);

    ChamferMatching(acd_input2);
  }
  else
  {
    ChamferMatching4(frame1, view1, frame2, view2);
  }
  SetCurrentInput(current);

  v3dMatrix<float> * distances = acd->Distances(frame1, view1);
  if (distances == 0)
  {
    vPrint("Failed to read distances for %li %li\n", frame1, view1);
    return 1;
  }

  long index = frame2 * 4 + view2;
  if (index >= distances->Size())
  {
    vPrint("Index %li is too large, only %li entries available\n",
           index, distances->Size());
  }
  else
  {
    float distance = distances->Matrix()[index];
    vPrint("distance = %f\n", distance);
  }

  vdelete(distances);

  return 1;
}


long vInterpreter3::ACD_NN0()
{
  if (acd == 0) return 0;
  long frame = 0, view = 0, k = 0;
  vPrint("\nEnter frame, view, k:\n");
  vScan("%li %li %li", &frame, &view, &k);
  return ACD_NN(frame, view, k);
}


long vInterpreter3::ACD_NN(long frame, long view, long k)
{
  if (acd == 0) return 0;
  long nn_frame = 0, nn_view = 0;
  float distance = acd->NearestNeighbor(frame, view, k,
                                        &nn_frame, &nn_view);
  vPrint("NN of (%li, %li): (%li, %li), %6.2f\n",
          frame, view, nn_frame, nn_view, distance);
  return 1;
}


long vInterpreter3::ACD_ApproximateRank0()
{
  if (acd == 0) return 0;

  long frame1 = 0, view1 = 0, frame2 = 0, view2 = 0;
  vPrint("\nEnter frame1, view1, frame2, view2:\n");
  vScan("%li %li %li %li", &frame1, &view1, &frame2, &view2);
  return ACD_ApproximateRank(frame1, view1, frame2, view2);
}


long vInterpreter3::ACD_ApproximateRank(long frame1, long view1,
                                         long frame2, long view2)
{
  if (acd == 0) return 0;
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;

  long result = acd->ApproximateRank(frame1, view1, frame2, view2,
                                     reference_images, training_embeddings,
                                     reference_start, embedding_dimensions);
  vPrint("result = %li\n", result);
  return 1;
}


long vInterpreter3::ACD_ApproximateLoss0()
{
  if (DatabaseEdges() == 0) return 0;
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;

  long frame = 0, view = 0, k = 0;
  vPrint("\nEnter frame, view, k:\n");
  vScan("%li %li %li", &frame, &view, &k);
  return ACD_ApproximateLoss(frame, view, k);
}


long vInterpreter3::ACD_ApproximateLoss(long frame1, long view1, long k)
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (acd == 0) return 0;
  
  long result = acd->ApproximateLoss(frame1, view1, k, reference_images,
                                     training_embeddings, reference_start,
                                     embedding_dimensions);
  vPrint("result = %li\n", result);
  return 1;
}

long vInterpreter3::ACD_LossStats0()
{
  if (acd == 0) return 0;
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;

  long k = 0, times = 0, print_all = 0;
  vPrint("\nEnter k, times, print_all:\n");
  vScan("%li %li %li", &k, &times, &print_all);
  return ACD_LossStats(k, times, print_all);
}


long vInterpreter3::ACD_LossStats(long k, long times, long print_all)
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (times <= 0) return 0;
  if (acd == 0) return 0;
  
  vector<long> ranks;
  acd->ApproximateLossStats(k, reference_images, training_embeddings,
                            reference_start, embedding_dimensions,
                            times, &ranks);

  long i;
  float sum = 0;
  for (i = 0; i < times; i++)
  {
    if (print_all != 0)
    {
      vPrint("%8li: %li\n", i, ranks[i]);
    }
    sum = sum + ranks[i];
  }

  long half = times / 2;
  long median = ranks[half];
  float mean = sum / times;
  vPrint("%li times, median = %li, mean = %f\n",
          times, median, mean);

  return 1;
}

  
long vInterpreter3::LoadBoostMap0()
{
  long dimensions = 0;
  char base[1000];
  vPrint("\nEnter base, dimensions:\n");
  vScan("%s %li", base, &dimensions);
  return LoadBoostMap(base, dimensions);
}


long vInterpreter3::LoadBoostMap(const char * base, long dimensions)
{
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  vVideoFile * sequence = inputs[database_edge_input]->sequence;
  if (strcmp(sequence->Type(), "vRLE_Sequence") != 0) return 0;

  vRLE_Sequence * database_edges = (vRLE_Sequence *) sequence;

  char * filename = vChamferBoostInfo::ClassifierFilename(base, dimensions);
  DeleteDistanceTransforms();
  vdelete(reference_images);
  reference_images = v3dMatrix<float>::ReadText(filename);
  if  (reference_images == 0)
  {
    vPrint("Failed to load reference images from %s\n", filename);
  }
  vdelete2(filename);

  long success = LoadTrainingEmbedding();
  if (success <= 0)
  {
    vPrint("Failed to load embedding\n");
    vdelete(reference_images);
    reference_images = 0;
  }
  else
  {
    embedding_dimensions = reference_images->Rows();
    reference_start = 0;
  }
  return success;
}


long vInterpreter3::BM_Scores()
{
  if (input == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if (vIsZero(distance_transforms) == 0)
  {
    vdelete(test_embedding);
    grayscale_image * current = input->current_image->ToGrayCopy();
    test_embedding = vChamferEmbedding(current,
                                        distance_transforms, point_sequence,
                                        reference_images, reference_start,
                                        embedding_dimensions);
    vdelete(current);
  }
  else
  {
    ChamferDistances();
  }
 
  vdelete(match_scores);
  match_scores = new vMatchScores(training_embeddings, test_embedding,
                                   reference_start, embedding_dimensions,
                                   reference_images);
  UpdateMatchScores(input->current_frame);
  return 1;
}


long vInterpreter3::PrintReferenceImages()
{
  if (reference_images == 0) return 0;
  vMatrix<float> temp(reference_images);
  temp.Print("reference_images");
  return 1;
}


// Compute the embedding of the current image, and print it.
long vInterpreter3::ShowEmbedding()
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  grayscale_image * edges = input->current_image->ToGrayCopy();
  ChamferDistances();
  if (test_embedding == 0) return 0;

  vArray2(float) references = reference_images->Matrix2(0);
  vArray(float) embedding = test_embedding->Matrix();

  long i;
  for (i = reference_start; i < reference_start + embedding_dimensions; i++)
  {
    double coordinate = embedding[i - reference_start];
    double weight = references[i][2];
    double scaled_coordinate = coordinate * weight;
    vPrint("%4li %5.2f:   %8.2lf    %8.2lf\n", 
             i, weight, coordinate, scaled_coordinate);
  }

  vdelete(edges);
  return 1;
}


// Display side-by-side the precomputed embedding for the test image and
// the matches of the specified ranks.
long vInterpreter3::CompareEmbeddings0()
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (test_embedding == 0) return 0;
  if (match_scores == 0) return 0;

  long rank1 = 0, rank2 = 0;
  vPrint("\nEnter rank1, rank2:\n");
  vScan("%li %li", &rank1, &rank2);

  return CompareEmbeddings(rank1, rank2);
}


long vInterpreter3::CompareEmbeddings(long rank1, long rank2)
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (test_embedding == 0) return 0;
  if (match_scores == 0) return 0;

  vMatchScore score1 = match_scores->GetScore(rank1);
  vMatchScore score2 = match_scores->GetScore(rank2);

  long index1 = score1.frame * 4 + score1.view;
  long index2 = score2.frame * 4 + score2.view;

  vArray2(float) references = reference_images->Matrix2(0);
  vArray(float) test_coords = test_embedding->Matrix();
  vArray2(float) training_coords = training_embeddings->Matrix2(0);

  long i;
  float sum1 = 0, sum2 = 0;
  for (i = reference_start; i < reference_start + embedding_dimensions; i++)
  {
    float weight = references[i][2];
    float test_coord = test_coords[i];
    float rank1_coord = training_coords[index1][i];
    float rank2_coord = training_coords[index2][i];
    float diff1 = weight * vAbs(test_coord - rank1_coord);
    float diff2 = weight * vAbs(test_coord - rank2_coord);
    sum1 = sum1 + diff1;
    sum2 = sum2 + diff2;

    vPrint("%4li %5.2f: %6.2f %6.2f %6.2f - %6.2f %6.2f - %6.2f %6.2f\n",
            i, weight, test_coord, rank1_coord, rank2_coord,
            diff1, diff2, sum1, sum2);
  }

  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  grayscale_image * test_image = input->current_image->ToGrayCopy();
  grayscale_image * temp_rank1_image = 
    inputs[database_edge_input]->sequence->ReadFrameGray(score1.frame);
  grayscale_image * temp_rank2_image = 
    inputs[database_edge_input]->sequence->ReadFrameGray(score2.frame);

  grayscale_image * rank1_image = 
    (grayscale_image *) vRotateRight(temp_rank1_image, score1.view * 90);
  grayscale_image * rank2_image = 
    (grayscale_image *) vRotateRight(temp_rank2_image, score2.view * 90);

  vdelete(temp_rank1_image);
  vdelete(temp_rank2_image);

  v3dMatrix<float> * test_dt = vCVDistanceTransform(test_image);
  v3dMatrix<float> * rank1_dt = vCVDistanceTransform(rank1_image);
  v3dMatrix<float> * rank2_dt = vCVDistanceTransform(rank2_image);

  float test_rank1 = (float) vChamferFromDT(test_image, rank1_dt);
  float rank1_test = (float) vChamferFromDT(rank1_image, test_dt);
  float test_rank1_total = test_rank1 + rank1_test;

  float test_rank2 = (float) vChamferFromDT(test_image, rank2_dt);
  float rank2_test = (float) vChamferFromDT(rank2_image, test_dt);
  float test_rank2_total = test_rank2 + rank2_test;

  vPrint("\n");
  vPrint("test->rank1 = %6.2f, rank1->test = %6.2f, total = %6.2f\n",
          test_rank1, rank1_test, test_rank1_total);
  vPrint("test->rank2 = %6.2f, rank2->test = %6.2f, total = %6.2f\n",
          test_rank2, rank2_test, test_rank2_total);

  vdelete(test_image);
  vdelete(rank1_image);
  vdelete(rank2_image);

  vdelete(test_dt);
  vdelete(rank1_dt);
  vdelete(rank2_dt);

  return 1;
}


long vInterpreter3::PositiveWeights()
{
  if (reference_images == 0) return 0;
  vArray2(float) references = reference_images->Matrix2(0);
  long number = reference_images->Rows();
  long i;
  for (i = 0; i < number; i++)
  {
    references[i][2] = vAbs(references[i][2]);
  }

  vPrint("Made weights positive\n");
  return 1;
}


long vInterpreter3::TestKthSmallest0()
{
  long k = 0, size = 0, range = 0;
  vPrint("\nEnter k, size, range:\n");
  vScan("%li %li %li", &k, &size, &range);
  return TestKthSmallest(k, size, range);
}


long vInterpreter3::TestKthSmallest(long k, long size, long range)
{
  if (range < 0) return 0;
  vector<long> numbers(size);
  vector<long> sorted_numbers(size);
  long i;
  for (i = 0; i < size; i++)
  {
    numbers[i] = vRandomLong2(0, range);
    sorted_numbers[i] = numbers[i];
  }

  long index = 0;
  long kth = kth_smallest_ca(k, &numbers, &index);
  std::sort(sorted_numbers.begin(), sorted_numbers.end(), less<long>());

  for (i = 0; i < size; i++)
  {
    vPrint("%4li %10li %10li\n", i, numbers[i], sorted_numbers[i]);
  }

  vPrint("correct answer = %li\n", sorted_numbers[k-1]);
  vPrint("index = %li, kth = %li\n", index, kth);
  return 1;
}


long vInterpreter3::TestSample0()
{
  long low = 0, high = 0, number = 0;
  vPrint("\nEnter low, high, number:\n");
  vScan("%li %li %li", &low, &high, &number);
  return TestSample(low, high, number);
}


long vInterpreter3::TestSample(long low, long high, long number)
{
  vMatrix<long> samples = vSampleWithoutReplacement(low, high, number);
  samples.PrintInt("samples"); 
  long min = function_image_minimum(&samples);
  long max = function_image_maximum(&samples);
  long size = samples.Size();
  vPrint("size = %li, min = %li, max = %li\n", size, min, max);
  return 1;
}


// Create a SparseMap for 2D points.
long vInterpreter3::PointSM0()
{
  long points = 0, triples = 0;
  vPrint("\nEnter points, triples:\n");
  vScan("%li %li", &points, &triples);
  return PointSM(points, triples);
}


long vInterpreter3::PointSM(long points, long triples)
{
  DeleteBM();
  boost_map = new vPlaneSparseMap(points, triples);
  return 1;
}


long vInterpreter3::SM_Features0()
{
  if (boost_map == 0) return 0;
  if ((strcmp(boost_map->ClassName(), "V_SparseMap") != 0) &&
      (strcmp(boost_map->ClassName(), "V_PlaneSparseMap") != 0))
  {
    return 0;
  }

  long in_features = 0;
  vPrint("\nEnter features:\n");
  vScan("%li", &in_features);
  return SM_Features(in_features);
}


long vInterpreter3::SM_Features(long in_features)
{
  if ((strcmp(boost_map->ClassName(), "V_SparseMap") != 0) &&
      (strcmp(boost_map->ClassName(), "V_PlaneSparseMap") != 0))
  {
    return 0;
  }

  V_SparseMap * sparse_map = (V_SparseMap *) boost_map;
  sparse_map->SetFeaturesToUse(in_features);
  long features_to_use = sparse_map->FeaturesToUse();
  long number_of_features = sparse_map->NumberOfFeatures();
  vPrint("Using %li of %li features\n", features_to_use, number_of_features);
  float error = sparse_map->ComputeValidationError();
  vPrint("validation_error = %f\n", error);
  return 1;
}


long vInterpreter3::SparseMapInfo()
{
  if (boost_map == 0) return 0;
  if (acd == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_ChamferSparseMap") != 0)
  {
    return 0;
  }

  vdelete(sm_info);
  sm_info = new class_SparseMapInfo();
  vChamferSparseMap * csm = (vChamferSparseMap *) boost_map;

  csm->ReferenceImages(&(sm_info->sets));
  SparseMapInfoB();
  SM_TrainingEmbedding();
  return 1;
}


long vInterpreter3::SparseMapInfoB()
{
  if (sm_info == 0) return 0;

  vMatrix<long> image_to_index_matrix(1, 107328);
  vArray(long) image_to_index = image_to_index_matrix.Matrix();
  vWriteValue(&image_to_index_matrix, (long) -1);
  long number_of_sets = sm_info->sets.size();

  long i;
  long counter = 0;
  for (i = 0; i < number_of_sets; i++)
  {
    v3dMatrix<long> i_images_matrix = sm_info->sets[i];
    vArray3(long) i_images = i_images_matrix.Matrix3();

    long rows = i_images_matrix.Rows();
    long cols = i_images_matrix.Cols();
    vMatrix<long> i_indices_matrix(rows, cols);
    vArray2(long) i_indices = i_indices_matrix.Matrix2();

    long row, col;
    for (row = 0; row < rows; row++)
    {
      for (col = 0; col < cols; col++)
      {
        long frame = i_images[0][row][col];
        long rotation = i_images[1][row][col];
        long id = frame * 4 + rotation;
        long index = image_to_index[id];
        if (index == -1)
        {
          index = counter;
          image_to_index[id] = index;
          counter++;
        }
        i_indices[row][col] = index;
      }
    }
    sm_info->indices.push_back(i_indices_matrix);
  }

  sm_info->flattened_images = vMatrix<long>(counter, 2);
  vArray2(long) flattened = sm_info->flattened_images.Matrix2();
  DeleteDistanceTransforms();
  vdelete(reference_images);
  reference_images = new v3dMatrix<float>(counter, 5, 1);
  reference_start = 0;
  embedding_dimensions = number_of_sets;
  vArray2(float) references = reference_images->Matrix2(0);
  
  counter = 0;
  for (i = 0; i < image_to_index_matrix.Size(); i++)
  {
    long index = image_to_index[i];
    if (index < 0) continue;
    long frame = i / 4;
    long rotation = i % 4;

    flattened[index][0] = frame;
    flattened[index][1] = rotation;

    references[index][0] = (float) frame;
    references[index][1] = (float) rotation;
    references[index][2] = (float) 1.0; // weight
    references[index][3] = (float) (frame * 4 + rotation);
    references[index][4] = (float) index;
    counter++;
  }

  // Now go through the sets of indices, and fill in the
  // image_to_sets information.
  sm_info->image_to_sets = vector<vector<long> >(counter);
  for (i = 0; i < number_of_sets; i++)
  {
    vMatrix<long> i_indices_matrix = sm_info->indices[i];
    vArray2(long) i_indices = i_indices_matrix.Matrix2();

    long rows = i_indices_matrix.Rows();
    long cols = i_indices_matrix.Cols();
    long row, col;
    for (row = 0; row < rows; row++)
    {
      long set_index = rows * i + row;
      for (col = 0; col < cols; col++)
      {
        long index = i_indices[row][col];
        sm_info->image_to_sets[index].push_back(set_index);
      }
    }
  }

  // We need to delete cbi to make memory for the huge
  // embeddings.
  // DeleteCBI();
  EmbeddingDTs();
  return 1;
}


long vInterpreter3::SM_TrainingEmbedding()
{
  if (sm_info == 0) return 0;
  if (acd == 0) return 0;

  long number_of_sets = sm_info->sets.size();
  if (number_of_sets == 0) return 0;

  long rows_per_set = sm_info->sets[0].Rows();
  if (rows_per_set == 0) return 0;
  long dimensions = number_of_sets * rows_per_set;

  vdelete(sm_embeddings);
  sm_embeddings = new v3dMatrix<float>(107328, dimensions, 1);
  vWriteValue(sm_embeddings, (float) 1000000000); // 1 billion
  vArray2(float) sm_array = sm_embeddings->Matrix2(0);

  long reference_image_number = reference_images->Rows();
  vArray2(float) references = reference_images->Matrix2(0);
  long i;
  vPrint("\n");
  for (i = 0; i < reference_image_number; i++)
  {
    long frame = vRound(references[i][0]);
    long rotation = vRound(references[i][1]);
    long index = vRound(references[i][4]);
    v3dMatrix<float> * distances_matrix = acd->Distances(frame, rotation);
    if (distances_matrix == 0) 
    {
      vPrint("Failed to get distances for (%li, %li)\n", frame, rotation);
      return 0;
    }
    vArray(float) distances = distances_matrix->Matrix();
    vector<long> & sets = sm_info->image_to_sets[index];
    long number_of_sets = sets.size();
    long j, k;
    for (j = 0; j < 107328; j++)
    {
      float distance = distances[j];
      for (k = 0; k < number_of_sets; k++)
      {
        long set_index = sets[k];
        float min_distance = sm_array[j][set_index];
        if (distance < min_distance)
        {
          sm_array[j][set_index] = distance;
        }
      }
    }
    vdelete(distances_matrix);
    vPrint("processed reference image %li of %li\r", 
            i+1, reference_image_number);
  }

  vPrint("\n");

  return 1;
}


long vInterpreter3::SM_Scores()
{
  if (input == 0) return 0;
  if (sm_info == 0) return 0;
  if (reference_images == 0) return 0;
  if (sm_embeddings == 0) return 0;
  if (vIsZero(distance_transforms) != 0) return 0;
  if (point_sequence == 0) return 0;

  long number_of_sets = sm_info->sets.size();
  if (number_of_sets == 0) return 0;

  long rows_per_set = sm_info->sets[0].Rows();
  if (rows_per_set == 0) return 0;
  long dimensions = number_of_sets * rows_per_set;

  grayscale_image * current = input->current_image->ToGrayCopy();
  v3dMatrix<float> * temp_embedding = vChamferEmbedding(current,
                                                           distance_transforms, point_sequence,
                                                           reference_images, 0,
                                                           reference_images->Rows());
  vdelete(current);
  vArray(float) temp_array = temp_embedding->Matrix();

  vdelete(test_embedding);
  test_embedding = new v3dMatrix<float>(1, dimensions, 1);
  vArray(float) embedding = test_embedding->Matrix();
  
  long dimension_index = 0;
  long j;
  for (j = 0; j < number_of_sets; j++)
  {
    vArray2(long) indices = sm_info->indices[j].Matrix2();
    long cols = sm_info->indices[j].Cols();
    if (cols <= 0)
    {
      exit_error("Error: cols <= 0: %li\n", cols);
    }

    long row, col;
    for (row = 0; row < rows_per_set; row++)
    {
      float min_distance = (float) 1000000000; // one billion
      for (col = 0; col < cols; col++)
      {
        long index = indices[row][col];
        float distance = temp_array[index];
        if (distance < min_distance)
        {
          min_distance = distance;
        }
      }

      embedding[dimension_index] = min_distance;
      dimension_index++;
    }
  }

  vdelete(temp_embedding);
  vdelete(match_scores);
  match_scores = new vMatchScores(sm_embeddings, test_embedding,
                                   reference_start, embedding_dimensions,
                                   reference_images);
  UpdateMatchScores(input->current_frame);
  return 1;
}


long vInterpreter3::SM_SaveEmbedding0()
{
  if (sm_info == 0) return 0;
  if (reference_images == 0) return 0;
  if (sm_embeddings == 0) return 0;

  long number_of_sets = sm_info->sets.size();
  if (number_of_sets == 0) return 0;

  long rows_per_set = sm_info->sets[0].Rows();
  if (rows_per_set == 0) return 0;

  char buffer[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", buffer);
  return SM_SaveEmbedding(buffer);
}


long vInterpreter3::SM_SaveEmbedding(const char * name)
{
  if (sm_info == 0) return 0;
  if (reference_images == 0) return 0;
  if (sm_embeddings == 0) return 0;

  long number_of_sets = sm_info->sets.size();
  if (number_of_sets == 0) return 0;

  long rows_per_set = sm_info->sets[0].Rows();
  if (rows_per_set == 0) return 0;

  long success = 1;

  char * directory = sm_info->Directory();
  char * filename = vJoinPaths(directory, name);
  vdelete2(directory);

  FILE * fp = fopen(filename, vFOPEN_WRITE);
  if (fp == 0) 
  {
    vPrint("Failed to save sm_info to %s\n", filename);
    vdelete2(filename);
    return 0;
  }

  long temp;
  temp = vWritePCLongs(fp, &number_of_sets, 1);
  if (temp != 1) 
  {
    success = 0;
  }

  long i;
  for (i = 0; i < number_of_sets; i++)
  {
    temp = sm_info->sets[i].Write(fp);
    if (temp <= 0)
    {
      success = 0;
    }
  }

  temp = sm_embeddings->Write(fp);
  if (temp <= 0)
  {
    success = 0;
  }
  fclose(fp);

  if (success <= 0)
  {
    vPrint("Failed to save sm_info to %s\n", filename);
  }
  else
  {
    vPrint("Saved sm_info to %s\n", filename);
  }

  vdelete2(filename);
  return success;
}


long vInterpreter3::SM_LoadEmbedding0()
{
  char buffer[1000];
  vPrint("\nEnter name:\n");
  vScan("%s", buffer);
  return SM_LoadEmbedding(buffer);
}


long vInterpreter3::SM_LoadEmbedding(const char * name)
{
  vdelete(sm_info);
  vdelete(sm_embeddings);
  sm_info = new class_SparseMapInfo;
  sm_embeddings = 0;

  char * directory = sm_info->Directory();
  char * filename = vJoinPaths(directory, name);
  vdelete2(directory);

  long success = 1;

  FILE * fp = fopen(filename, vFOPEN_READ);
  if (fp == 0) 
  {
    vPrint("Failed to load sm_info from %s\n", filename);
    vdelete2(filename);
    return 0;
  }

  long temp;
  long number_of_sets = 0;
  temp = vReadPCLongs(fp, &number_of_sets, 1);
  if (temp != 1) 
  {
    success = 0;
  }

  long i;
  sm_info->sets = vector<v3dMatrix<long> >(number_of_sets);
  for (i = 0; i < number_of_sets; i++)
  {
    v3dMatrix<long> * i_images = v3dMatrix<long>::Read(fp);
    if (i_images == 0)
    {
      success = 0;
    }
    else
    {
      sm_info->sets[i] = v3dMatrix<long>(i_images);
    }
    vdelete(i_images);
  }

  sm_embeddings = v3dMatrix<float>::Read(fp);
  if (sm_embeddings == 0)
  {
    success = 0;
  }
  fclose(fp);

  if (success <= 0)
  {
    vPrint("Failed to load sm_info from %s\n", filename);
  }
  else
  {
    vPrint("Loaded sm_info from %s\n", filename);
    SparseMapInfoB();
  }

  vdelete2(filename);
  return success;
}


long vInterpreter3::SM_ReferenceSet0()
{
  if (sm_info == 0) return 0;
  long dimension = 0;
  vPrint("\nEnter dimension:\n");
  vScan("%li", &dimension);
  return SM_ReferenceSet(dimension);
}


long vInterpreter3::SM_ReferenceSet(long dimension)
{
  if (sm_info == 0) return 0;
  long number_of_matrices = sm_info->sets.size();
  if (number_of_matrices == 0) return 0;
  long rows_per_matrix = sm_info->sets[0].Rows();
  long number_of_sets = number_of_matrices * rows_per_matrix;
  if ((dimension < 0) || (dimension >= number_of_sets))
  {
    vPrint("There are %li reference sets\n", number_of_sets);
    return 0;
  }

  long i = dimension / rows_per_matrix;
  long row = dimension % rows_per_matrix;
  v3dMatrix<long> i_sets_matrix = sm_info->sets[i];
  vMatrix<long> i_indices_matrix = sm_info->indices[i];
  vArray3(long) i_sets = i_sets_matrix.Matrix3();
  vArray2(long) i_indices = i_indices_matrix.Matrix2();

  long col;
  vPrint("reference set %li:\n", dimension);
  long cols = i_indices_matrix.Cols();
  for (col = 0; col < cols; col++)
  {
    long frame = i_sets[0][row][col];
    long rotation = i_sets[1][row][col];
    long index = i_indices[row][col];
    vPrint("%6li: (%6li %2li) --> %5li\n", col, frame, rotation, index);
  }
  return 1;
}


// Look at a specific coordinate of the SparseMap embedding
// for a particular database image, and all the distances
// to images in the corresponding reference set.
long vInterpreter3::SM_TrainingCoordinate0()
{
  if (sm_embeddings == 0) return 0;
  long frame = 0, rotation = 0;
  vPrint("\nEnter frame, rotation:\n");
  vScan("%li %li", &frame, &rotation);
  return SM_TrainingCoordinate(frame, rotation);
}

long vInterpreter3::SM_TrainingCoordinate(long frame, long rotation)
{
  if (sm_embeddings == 0) return 0;

  vArray2(float) sm = sm_embeddings->Matrix2(0);
  long cols = sm_embeddings->Cols();
  long index = frame * 4 + rotation;
  long i;
  vPrint("SM embedding for (%li, %li):\n", frame, rotation);
  for (i = 0; i < cols; i++)
  {
    vPrint("%li: %f\n", i, sm[index][i]);
  }

  return 1;
}


long vInterpreter3::SM_TestCoordinate0()
{
  if (sm_info == 0) return 0;
  if (predicate_zero (distance_transforms)) return 0;
  if (reference_images == 0) return 0;
  if (test_embedding == 0) return 0;
  if (point_sequence == 0) return 0;
  if (input == 0) return 0;

  long dimension = 0;
  vPrint("\nEnter dimension:\n");
  vScan("%li", &dimension);
  return SM_TestCoordinate(dimension);
}


long vInterpreter3::SM_TestCoordinate(long dimension)
{
  if (sm_info == 0) return 0;
  if (predicate_zero (distance_transforms)) return 0;
  if (reference_images == 0) return 0;
  if (test_embedding == 0) return 0;
  if (point_sequence == 0) return 0;
  if (input == 0) return 0;
  if (dimension < 0) return 0;

  long number_of_matrices = sm_info->sets.size();
  long rows_per_matrix = sm_info->sets[0].Rows();
  long number_of_sets = number_of_matrices * rows_per_matrix;
  if ((dimension < 0) || (dimension >= number_of_sets))
  {
    vPrint("There are %li reference sets\n", number_of_sets);
    return 0;
  }

  long sm_dimensions = sm_info->sets.size();
  sm_dimensions = sm_dimensions * sm_dimensions;
  if (dimension >= sm_dimensions)
  {
    vPrint("sm_dimensions = %li\n", sm_dimensions);
    return 0;
  }

  long dimensions = reference_images->Rows();
  grayscale_image * current = input->current_image->ToGrayCopy();
  v3dMatrix<float> * embedding = 
    vChamferEmbedding(current, distance_transforms, point_sequence,
                       reference_images, 0, dimensions);

  long i = dimension / rows_per_matrix;
  long row = dimension % rows_per_matrix;
  v3dMatrix<long> i_sets_matrix = sm_info->sets[i];
  vMatrix<long> i_indices_matrix = sm_info->indices[i];
  vArray3(long) i_sets = i_sets_matrix.Matrix3();
  vArray2(long) i_indices = i_indices_matrix.Matrix2();

  long col;
  vPrint("reference set %li:\n", dimension);
  float min_distance = 1000000000; // one billion
  long min_index = -1;
  long cols = i_indices_matrix.Cols();
  for (col = 0; col < cols; col++)
  {
    long frame = i_sets[0][row][col];
    long rotation = i_sets[1][row][col];
    long index = i_indices[row][col];
    float distance = embedding->Matrix()[index];
    vPrint("%6li: (%6li %2li) --> %5li, distance = %f\n", 
            col, frame, rotation, index, distance);
    if (distance < min_distance)
    {
      min_distance = distance;
      min_index = col;
    }
  }

  vPrint("min_distance found = %f, at column %li\n", 
          min_distance, min_index);
  float stored_distance = test_embedding->Matrix()[dimension];
  vPrint("stored_distance = %f\n", stored_distance);

  vdelete(current);
  return 1;
}


long vInterpreter3::ShowOriginalFrame()
{
  if (original_video_frame == 0) return 0;
  if (input == 0) return 0;
  ShowImage(original_video_frame->ToRGBCopy());
  return 1;
}


long vInterpreter3::TimeDT0()
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  long times = 0;
  vPrint("\nEnter times:\n");
  vScan("%li", &times);
  return TimeDT(times);
}


long vInterpreter3::TimeDT(long times)
{
  if (input == 0) return 0;
  if (input->current_image == 0) return 0;

  grayscale_image * gray = input->current_image->ToGrayCopy();

  long i;
  for (i = 0; i < times; i++)
  {
    v3dMatrix<float> * dt = vCVDistanceTransform(gray);
    vdelete(dt);
  }
  vPrint("done\n");
  return 1;
}


long vInterpreter3::EmbeddingDTs()
{
  if (reference_images == 0) return 0;
  if ((database_edge_input < 0) || (database_edge_input >= number_of_inputs))
  {
    return 0;
  }

  if (point_sequence == 0) 
  {
    LoadPointSet();
    if (point_sequence == 0)
    {
      vPrint("Failed to load point_sequence\n");
      return 0;
    }
  }
  DeleteDistanceTransforms();
  long dimensions = reference_images->Rows();
  distance_transforms = vnew(v3dMatrix<float> *, dimensions);
  vArray2(float) references = reference_images->Matrix2(0);

  SetCurrentInput(database_edge_input);

  long i;
  vPrint("\n");
  for (i = 0; i < dimensions; i++)
  {
    long frame = vRound(references[i][0]);
    long rotation = vRound(references[i][1]);
    grayscale_image * gray = input->sequence->ReadFrameGray(frame);
    if (gray == 0)
    {
      distance_transforms[i] = 0;
      continue;
    }

    grayscale_image * rotated = (grayscale_image *) vRotateRight(gray, 90 * rotation);
    distance_transforms[i] = vCVDistanceTransform(rotated);
    
    vdelete(gray);
    vdelete(rotated);
    vPrint("Done with frame %li of %li\r", i+1, dimensions);
  }

  vPrint("\n");
  return 1;
}


long vInterpreter3::PrintTestEmbedding()
{
  if (test_embedding == 0) return 0;
  long dimensions = test_embedding->Cols();
  vArray(float) entries = test_embedding->Matrix();

  long i;
  vPrint("test embedding\n");
  for (i = 0; i < dimensions; i++)
  {
    vPrint("%li: %f\n", i, entries[i]);
  }
  return 1;
}


// Create a FastMap for 2D points.
long vInterpreter3::PointFM0()
{
  long points = 0, triples = 0;
  vPrint("\nEnter points, triples:\n");
  vScan("%li %li", &points, &triples);
  return PointFM(points, triples);
}


long vInterpreter3::PointFM(long points, long triples)
{
  DeleteBM();
  boost_map = new vPlaneFastMap(points, triples);
  return 1;
}


long vInterpreter3::FM_PointImage()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_PlaneFastMap") != 0) 
  {
    return 0;
  }
  if (input == 0) return 0;

  vPlaneFastMap * plane_fm = (vPlaneFastMap *) boost_map;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  color_image * the_image = plane_fm->PointImage(rows, cols);
  if (the_image == 0)
  {
    vPrint("Failed to get point image\n");
  }
  else
  {
    ShowImage(the_image);
  }
  return 1;
}


long vInterpreter3::FM_BasisImage()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_PlaneFastMap") != 0) 
  {
    return 0;
  }
  if (input == 0) return 0;

  vPlaneFastMap * plane_fm = (vPlaneFastMap *) boost_map;
  general_image * current = input->current_image;
  long rows = current->Rows();
  long cols = current->Cols();
  color_image * the_image = plane_fm->BasisImage(rows, cols);
  if (the_image == 0)
  {
    vPrint("Failed to get point image\n");
  }
  else
  {
    ShowImage(the_image);
  }
  return 1;
}


long vInterpreter3::FM_Pivots()
{
  if (boost_map == 0) return 0;
  if (cbi == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_ChamferFastMap") != 0) 
  {
    return 0;
  }

  vMatrix<long> indices = cbi->Indices();
  vChamferFastMap * cfm = (vChamferFastMap *) boost_map;
  cfm->FindPivotImages(&indices);
  vMatrix<long> pivots = cfm->PivotImages();
  pivots.Print("pivots");
  return 1;
}


long vInterpreter3::FM_LoadEmbedding0()
{
  char buffer[1000];
  vPrint("\nEnter name:\n");
  vScan("%s", buffer);
  return FM_LoadEmbedding(buffer);
}


long vInterpreter3::FM_LoadEmbedding(const char * name)
{
  vdelete(fm_info);
  fm_info = new class_FastMapInfo;
  char * directory = fm_info->Directory();
  char * filename = vJoinPaths(directory, name);
  vdelete2(directory);

  // Read the pairs of pivot points.
  v3dMatrix<float> * temp = v3dMatrix<float>::ReadText(filename);
  if (temp == 0)
  {
    vPrint("Failed to read pivot images from %s\n", filename);
    vdelete2(filename);
    return 0;
  }
  vdelete2(filename);

  // Save pivot points into fm_info, and identify the unique
  // images.
  vMatrix<long> image_to_index_matrix(1, 107328);
  vArray(long) image_to_index = image_to_index_matrix.Matrix();
  vWriteValue(&image_to_index_matrix, (long) -1);
  long rows = temp->Rows();
  vArray2(float) temp_pivots = temp->Matrix2(0);

  fm_info->pivots_matrix = vMatrix<long>(rows, 4);
  fm_info->indices_matrix = vMatrix<long>(rows, 2);
  vArray2(long) pivots = fm_info->pivots_matrix.Matrix2();
  vArray2(long) indices = fm_info->indices_matrix.Matrix2();

  vector<long> unique_frames;
  vector<long> unique_rotations;

  long i;
  long counter = 0;
  for (i = 0; i < rows; i++)
  {
    long col;
    for (col = 0; col <= 2; col = col + 2)
    {
      long frame = vRound(temp_pivots[i][col]);
      long rotation = vRound(temp_pivots[i][col+1]);
      long id = frame * 4 + rotation;
      long index = image_to_index[id];
      if (index == -1)
      {
        index = counter;
        image_to_index[id] = index;
        counter++;
        unique_frames.push_back(frame);
        unique_rotations.push_back(rotation);
      }
      pivots[i][col] = frame;
      pivots[i][col+1] = rotation;
      indices[i][col/2] = index;
    }
  }

  vdelete(temp);
  // Store information into reference images.
  DeleteDistanceTransforms();
  vdelete(reference_images);
  reference_images = new v3dMatrix<float>(counter, 5, 1);
  reference_start = 0;
  embedding_dimensions = counter;
  vArray2(float) references = reference_images->Matrix2(0);
  
  for (i = 0; i < counter; i++)
  {
    long frame = unique_frames[i];
    long rotation = unique_rotations[i];

    references[i][0] = (float) frame;
    references[i][1] = (float) rotation;
    references[i][2] = (float) 1.0; // weight
    references[i][3] = (float) (frame * 4 + rotation);
    references[i][4] = (float) i;
  }

  LoadTrainingEmbedding();
  EmbeddingDTs();

  // compute pivot distances;
  long dimensions = fm_info->pivots_matrix.Rows();
  fm_info->pivot_distances_matrix = vMatrix<float>(2*dimensions, 2*dimensions);
  vArray2(float) pivot_distances = fm_info->pivot_distances_matrix.Matrix2();
  vArray2(float) lipschitz = training_embeddings->Matrix2(0);

  long j;
  for (i = 0; i < dimensions; i++)
  {
    for (j = 0; j < dimensions; j++)
    {
      long index1 = indices[i][0];
      long frame2 = pivots[j][0];
      long rotation2 = pivots[j][1];
      long id2 = frame2 * 4 + rotation2;
      float distance = lipschitz[id2][index1];
      pivot_distances[2*i][2*j] = distance;

      index1 = indices[i][0];
      frame2 = pivots[j][2];
      rotation2 = pivots[j][3];
      id2 = frame2 * 4 + rotation2;
      distance = lipschitz[id2][index1];
      pivot_distances[2*i][2*j+1] = distance;

      index1 = indices[i][1];
      frame2 = pivots[j][0];
      rotation2 = pivots[j][1];
      id2 = frame2 * 4 + rotation2;
      distance = lipschitz[id2][index1];
      pivot_distances[2*i+1][2*j] = distance;

      index1 = indices[i][1];
      frame2 = pivots[j][2];
      rotation2 = pivots[j][3];
      id2 = frame2 * 4 + rotation2;
      distance = lipschitz[id2][index1];
      pivot_distances[2*i+1][2*j+1] = distance;
    }
  }

  fm_info->optimized_pivot_distances = 
    V_FastMap::OptimizedPivotDistances(fm_info->pivot_distances_matrix);

  return 1;
}


// Compute the training embedding using FastMap
long vInterpreter3::FM_TrainingEmbedding()
{
  if (reference_images == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (fm_info == 0) return 0;

  long dimensions = fm_info->pivots_matrix.Rows();
  vArray2(float) pivot_distances = fm_info->pivot_distances_matrix.Matrix2();
  vArray2(long) indices = fm_info->indices_matrix.Matrix2();
  vArray2(float) lipschitz = training_embeddings->Matrix2(0);

  // Compute the FastMap embeddings of all database images.
  vdelete(fm_embeddings);
  fm_embeddings = new v3dMatrix<float>(107328, dimensions, 1);
  vArray2(float) fm = fm_embeddings->Matrix2(0);
  
  vMatrix<float> distances_to_pivots_matrix(dimensions, 2);
  vArray2(float) distances_to_pivots = distances_to_pivots_matrix.Matrix2();

  vPrint("\n");
  long i;
  for (i = 0; i < 107328; i++)
  {
    // First get the distances of the image to the pivots
    long j;
    for (j = 0; j < dimensions; j++)
    {
      long index1 = indices[j][0];
      long index2 = indices[j][1];
      distances_to_pivots[j][0] = lipschitz[i][index1];
      distances_to_pivots[j][1] = lipschitz[i][index2];
    }

//    vMatrix<float> intrapivot(&(fm_info->pivot_distances_matrix));
//    vMatrix<float> fm_embedding_matrix = 
//      class_FastMap::Embedding(distances_to_pivots_matrix,
//                           intrapivot);
    vMatrix<float> fm_embedding_matrix = 
      V_FastMap::EmbeddingB(distances_to_pivots_matrix,
                            fm_info->optimized_pivot_distances);
    vArray(float) fm_embedding = fm_embedding_matrix.Matrix();
    for (j = 0; j < dimensions; j++)
    {
      fm[i][j] = fm_embedding[j];
    }
    if (((i+1) % 1000) == 0)
    {
      vPrint("done with fm embedding %li of 107328\r", i+1);
    }
  }

  reference_start = 0;
  embedding_dimensions = dimensions;
  vPrint("\n");
  return 1;
}


long vInterpreter3::FM_TestEmbedding()
{
  if (input == 0) return 0;
  if (fm_info == 0) return 0;
  if (vIsZero(distance_transforms) != 0) return 0;
  if (point_sequence == 0) return 0;

  long dimensions = fm_info->pivots_matrix.Rows();
  if (dimensions == 0) return 0;

  grayscale_image * current = input->current_image->ToGrayCopy();
  v3dMatrix<float> * temp_embedding = 
    vChamferEmbedding(current, distance_transforms, point_sequence,
                       reference_images, 0, reference_images->Rows());

  vdelete(current);
  vArray(float) temp_array = temp_embedding->Matrix();

  vMatrix<float> distances_to_pivots_matrix(dimensions, 2);
  vArray2(float) distances_to_pivots = distances_to_pivots_matrix.Matrix2();
  vArray2(long) indices = fm_info->indices_matrix.Matrix2();
  
  long j;
  for (j = 0; j < dimensions; j++)
  {
    long index1 = indices[j][0];
    long index2 = indices[j][1];
    distances_to_pivots[j][0] = temp_array[index1];
    distances_to_pivots[j][1] = temp_array[index2];
  }

//  vMatrix<float> intrapivot(&(fm_info->pivot_distances_matrix));
//  vMatrix<float> fm_embedding_matrix = 
//    class_FastMap::Embedding(distances_to_pivots_matrix,
//                         intrapivot);
  vMatrix<float> fm_embedding_matrix = 
    V_FastMap::EmbeddingB(distances_to_pivots_matrix,
                          fm_info->optimized_pivot_distances);

  vdelete(temp_embedding);
  vdelete(test_embedding);
  test_embedding = new v3dMatrix<float>(&fm_embedding_matrix);
  return 1;
}


long vInterpreter3::FM_Scores()
{
  if (input == 0) return 0;
  if (fm_info == 0) return 0;
  if (fm_embeddings == 0) return 0;
  if (vIsZero(distance_transforms) != 0) return 0;
  if (point_sequence == 0) return 0;

  FM_TestEmbedding();
  vdelete(match_scores);
  match_scores = new vMatchScores(fm_embeddings, test_embedding,
                                   embedding_dimensions);
  UpdateMatchScores(input->current_frame);
  return 1;
}


long vInterpreter3::FM_PrintInfo()
{
  if (fm_info == 0) return 0;
  fm_info->pivot_distances_matrix.Print("pivot_distances");
  fm_info->indices_matrix.Print("indices");
  fm_info->pivots_matrix.Print("pivots");
  return 1;
}


long vInterpreter3::FM_TrainingCoords0()
{
  if (fm_embeddings == 0) return 0;
  long frame = 0, rotation = 0;
  vPrint("\nEnter frame, rotation\n");
  vScan("%li %li", &frame, &rotation);
  return FM_TrainingCoords(frame, rotation);
}


long vInterpreter3::FM_TrainingCoords(long frame, long rotation)
{
  if (fm_embeddings == 0) return 0;
  if (frame >= 107328 / 4) return 0;
  if (rotation >= 4) return 0;
  if ((frame < 0) || (rotation < 0)) return 0;

  vArray2(float) fm = fm_embeddings->Matrix2(0);
  long index = frame * 4 + rotation;
  long cols = fm_embeddings->Cols();

  long i;
  for (i = 0; i < cols; i++)
  {
    vPrint("%5li  %f\n", i, fm[index][i]);
  }

  return 1;
}


long vInterpreter3::FM_TestCoords()
{
  if (input == 0) return 0;
  if (reference_images == 0) return 0;

  grayscale_image * current = input->current_image->ToGrayCopy();

  v3dMatrix<float> * lipschitz_matrix = 
    vChamferEmbedding(current, distance_transforms, point_sequence,
                       reference_images, 0, reference_images->Rows());
  vArray(float) lipschitz = lipschitz_matrix->Matrix();
  vArray(float) fm = test_embedding->Matrix();

  long i;
  vPrint("lipschitz embedding:\n");
  for (i = 0; i < lipschitz_matrix->Size(); i++)
  {
    vPrint("%5li  %f\n", i, lipschitz[i]);
  }

  if (test_embedding == 0) return 1;
  vPrint("\nFastMap embedding:\n");
  for (i = 0; i < test_embedding->Size(); i++)
  {
    vPrint("%5li  %f\n", i, fm[i]);
  }

  vdelete(current);
  vdelete(lipschitz_matrix);
  return 1;
}


long vInterpreter3::FM_LineProjection0()
{
  float ai = 0, bi = 0, ab = 0;
  vPrint("\nEnter ai, bi, ab:\n");
  vScan("%f %f %f", &ai, &bi, &ab);
  
  return FM_LineProjection(ai, bi, ab);
}


long vInterpreter3::FM_LineProjection(float ai, float bi, float ab)
{
  float result = V_FastMap::LineProjection3(ai, bi, ab);
  vPrint("result = %f\n", result);
  return 1;
}


long vInterpreter3::FM_Distances()
{
  if (boost_map == 0) return 0;
  const char * name = boost_map->ClassName();
  if ((strcmp(name, "V_FastMap") != 0) &&
      (strcmp(name, "V_PlaneFastMap") != 0) &&
      (strcmp(name, "V_ChamferFastMap") != 0))
  {
    vPrint("no fast map object is available\n");
  }

  V_FastMap * fm = (V_FastMap *) boost_map;
  vMatrix<long> distances = fm->RequiredDistances();
  distances.Print("distances");
  return 1;
}

  
long vInterpreter3::MakeSBI0()
{
  long training = 0;
  vPrint("\nEnter training number:\n");
  vScan("%li", &training);
  return MakeSBI(training);
}


long vInterpreter3::MakeSBI(long training)
{
  DeleteSBI();
  sbi = new vSignBoostInfo(training);
  return 1;
}


long vInterpreter3::DeleteSBI()
{
  if (sbi == 0) return 0;
  V_BoostMap * bm = sbi->BoostMap();
  V_SparseMap * sm = sbi->SparseMap();
  V_FastMap * fm = sbi->FastMap();
  if (boost_map == bm)
  {
    boost_map = 0;
  }

  else if (boost_map == sm)
  {
    boost_map = 0;
  }

  else if (boost_map == fm)
  {
    boost_map = 0;
  }

  vdelete(sbi);
  sbi = 0;
  return 1;
}


long vInterpreter3::SBI_Print()
{
  if (sbi == 0) return 0;
  sbi->Print();
  return 1;
}


long vInterpreter3::SBI_BM()
{
  if (sbi == 0) return 0;
  DeleteBM();
  sbi->MakeBoostMap();
  boost_map = sbi->BoostMap();
  return 1;
}


long vInterpreter3::SBI_Training0()
{
  if (sbi == 0) return 0;
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return SBI_Training(number);
}


long vInterpreter3::SBI_Training(long number)
{
  if (sbi == 0) return 0;
  long result = sbi->SetTrainingNumber(number);
  vPrint("sbi training number= %li\n", result);
  return 1;
}


long vInterpreter3::SBI_SM0()
{
  if (sbi == 0) return 0;
  long candidates = 0;
  vPrint("\nEnter number of candidates\n");
  vScan("%li", &candidates);
  return SBI_SM(candidates);
}


long vInterpreter3::SBI_SM(long candidates)
{
  if (sbi == 0) return 0;
  DeleteBM();
  sbi->MakeSparseMap(candidates);
  boost_map = sbi->SparseMap();
  class_SparseMap * sm = (class_SparseMap *) boost_map;
  long dimensions = sm->Dimensions();
  vPrint("dimensions = %li\n", dimensions);
  return 1;
}


long vInterpreter3::SBI_FM()
{
  if (sbi == 0) return 0;
  DeleteBM();
  sbi->MakeFastMap();
  boost_map = sbi->FastMap();
  return 1;
}


long vInterpreter3::SBI_BM_Stats0()
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  long max_dimensions = 0, max_k = 0;
  vPrint("\nEnter max dimensions, max_k:\n");
  vScan("%li %li", &max_dimensions, &max_k);
  return SBI_BM_Stats(max_dimensions, max_k);
}


long vInterpreter3::SBI_BM_Stats(long max_dimensions, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  approximate_stats = sbi->ApproximateRankStatsBM(max_dimensions, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}



long vInterpreter3::SBI_FM_Stats0()
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  long max_dimensions = 0, max_k = 0;
  vPrint("\nEnter max dimensions, max_k:\n");
  vScan("%li %li", &max_dimensions, &max_k);
  return SBI_FM_Stats(max_dimensions, max_k);
}


long vInterpreter3::SBI_FM_Stats(long max_dimensions, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  approximate_stats = sbi->ApproximateRankStatsFM(max_dimensions, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_BM_Stats0d()
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_BM_Stats4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_BM_Stats4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  approximate_stats = 
    sbi->ApproximateRankStatsBM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_FM_Stats0d()
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_FM_Stats4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_FM_Stats4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  approximate_stats = 
    sbi->ApproximateRankStatsFM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_SM_Stats0d()
{
  if (sbi == 0) return 0;
  if (sbi->SparseMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_SM_Stats4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_SM_Stats4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->SparseMap() == 0) return 0;

  approximate_stats = 
    sbi->ApproximateRankStatsSM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::KthRankStats0()
{
  long k = 0;
  vPrint("\nEnter k:\n");
  vScan("%li", &k);
  return KthRankStats(k);
}


long vInterpreter3::KthRankStats(long k)
{
  vMatrix<float> kth_ranks = vColumnKth(&approximate_stats, k);
  kth_ranks.Print("kth_ranks");
  return 1;
}


long vInterpreter3::SBI_BM_AHRCM0d()
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_BM_AHRCM4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_BM_AHRCM4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->BoostMap() == 0) return 0;

  approximate_stats = 
    sbi->HRCM_ApproxStatsBM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_FM_AHRCM0d()
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_FM_AHRCM4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_FM_AHRCM4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->FastMap() == 0) return 0;

  approximate_stats = 
    sbi->HRCM_ApproxStatsFM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_SM_AHRCM0d()
{
  if (sbi == 0) return 0;
  if (sbi->SparseMap() == 0) return 0;

  long min_dimensions = 0, max_dimensions = 0, min_k = 0, max_k = 0;
  vPrint("\nEnter min_dimensions, max dimensions, min_k, max_k:\n");
  vScan("%li %li %li %li", &min_dimensions, &max_dimensions, &min_k, &max_k);
  return SBI_SM_AHRCM4(min_dimensions, max_dimensions, min_k, max_k);
}


long vInterpreter3::SBI_SM_AHRCM4(long min_dimensions, long max_dimensions, 
                                   long min_k, long max_k)
{
  if (sbi == 0) return 0;
  if (sbi->SparseMap() == 0) return 0;

  approximate_stats = 
    sbi->HRCM_ApproxStatsSM4(min_dimensions, max_dimensions, min_k, max_k);
  if (approximate_stats.Bands() == 1)
  {
    vMatrix<float> temp_stats(&approximate_stats);
    temp_stats.Print("stats");
  }
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SBI_HRCM0()
{
  if (sbi == 0) return 0;

  long min_k = 0, max_k = 0;
  vPrint("\nEnter min_k, max_k:\n");
  vScan("%li %li", &min_k, &max_k);
  return SBI_HRCM(min_k, max_k);
}


long vInterpreter3::SBI_HRCM(long min_k, long max_k)
{
  if (sbi == 0) return 0;

  approximate_stats = 
    sbi->HRCM_Stats2(min_k, max_k);
  vMatrix<float> means = vColumnMeans(&approximate_stats);
  vMatrix<float> medians = vColumnMedians(&approximate_stats);
  means.Print("means");
  medians.Print("medians");
  return 1;
}


long vInterpreter3::SbiDataset()
{
  if (sbi == 0)
  {
    MakeSBI(100);
  }

  char * sbi_name = "sbi";
  vMatrix<long> training_labels = sbi->TrainingLabels();
  v3dMatrix<float> training_labelsf(&training_labels);
  vMatrix<float> training_labelsf2(&training_labelsf);
  BoostMap_dataset::SaveTrainingLabels(sbi_name, training_labelsf2);

  vMatrix<long> test_labels = sbi->TestLabels();
  v3dMatrix<float> test_labelsf(&test_labels);
  vMatrix<float> test_labelsf2(&test_labelsf);
  BoostMap_dataset::SaveTestLabels(sbi_name, test_labelsf2);

  char * path;
  vMatrix<float> training_distances = sbi->TrainingDistances();
  path = BoostMap_dataset::TrainTrainDistancesPath(sbi_name);
  long success = training_distances.Write(path);
  if (success == 0)
  {
    vPrint("failed to save training distances to %s\n", path);
  }
  vdelete2(path);

  vMatrix<float> test_distances = sbi->TestDistances();
  path = BoostMap_dataset::TestTrainDistancesPath(sbi_name);
  success = test_distances.Write(path);
  if (success == 0)
  {
    vPrint("failed to save test distances to %s\n", path);
  }
  vdelete2(path);

//  BoostMap_dataset dataset("sbi1", 880, 300, 400, 180);
//  BoostMap_dataset dataset2("sbi2", 880, 1, 878, 1);
//  BoostMap_dataset dataset3("sbi3", 880, 450, 400, 30);
  BoostMap_dataset dataset4("sbi4", 880, 587, 290, 3);
//  dataset.MakeAllDistances(training_distances);
//  dataset2.MakeAllDistances(training_distances);
//  dataset3.MakeAllDistances(training_distances);
  dataset4.MakeAllDistances(training_distances);

  vMatrix<float> pdistances = sbi->TrainingPdistances();
//  dataset.MakeAllPdistances(pdistances);
//  dataset2.MakeAllPdistances(pdistances);
//  dataset3.MakeAllPdistances(pdistances);
  dataset4.MakeAllPdistances(pdistances);

  return 1;
}



long vInterpreter3::CEB_Create()
{
  if (boost_map == 0) return 0;
  if (strcmp(boost_map->ClassName(), "V_BoostMap3") != 0) 
  {
    return 0;
  }
  if (cbi == 0) return 0;

  V_BoostMap3 * bm3 = (V_BoostMap3 *) boost_map;
  vMatrix<float> classifiers = bm3->ClassifierMatrix();
  vMatrix<float> pivot_distances = cbi->PairDistances(classifiers);
  vMatrix<long> indices_to_poses = cbi->Indices();

  vdelete(ceb);
  ceb = new vChamferEmbeddingBM(classifiers, pivot_distances, 
                                 indices_to_poses);
  return 1;
}


long vInterpreter3::CEB_Print()
{
  if (ceb == 0) return 0;
  ceb->Print();
  return 1;
}


long vInterpreter3::CEB_Save0()
{
  if (ceb == 0) return 0;
  char filename[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", filename);
  return CEB_Save(filename);
}


long vInterpreter3::CEB_Save(const char * filename)
{
  if (ceb == 0) return 0;
  long dimensions = ceb->Dimensions();
  if (ceb->Dimensions() <= 0) return 0;

  char * number_string = vLongToString(dimensions);
  char * name = vMergeStrings4(filename, "_", number_string, ".txt");
  long success = ceb->Save(name);
  if (success == 0)
  {
    vPrint("failed to save ceb to %s\n", name);
  }
  else
  {
    vPrint("Saved ceb successfully to %s\n", name);
  }
  vdelete2(number_string);
  vdelete2(name);
  return success;
}


long vInterpreter3::CEB_Load0()
{
  char filename[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", filename);
  return CEB_Load(filename);
}


long vInterpreter3::CEB_Load(const char * filename)
{
  char * name = vMergeStrings2(filename, ".txt");
  vdelete(ceb);
  ceb = new vChamferEmbeddingBM(name);

  if (ceb->IsValid() <= 0)
  {
    vPrint("failed to load ceb from %s\n", name);
    vdelete(ceb);
    ceb = 0;
    vdelete2(name);
    return 0;
  }
  else
  {
    vPrint("Loaded ceb successfully from %s\n", name);
  }

  vdelete2(name);

  return 1;
}


long vInterpreter3::CEB_Embeddings()
{
  if (ceb == 0) return 0;
  vdelete(reference_images);
  vMatrix<float> temp_references = ceb->UniqueReferenceImages();
  reference_images = new v3dMatrix<float>(&temp_references);
  LoadTrainingEmbedding();
  
  long dimensions = ceb->Dimensions();
  vdelete(ceb_embeddings);
  long number = training_embeddings->Rows();
  ceb_embeddings = new v3dMatrix<float>(number, dimensions, 1);

  long i;
  vArray2(float) lipschitz = training_embeddings->Matrix2(0);
  vArray2(float) ceb_matrix = ceb_embeddings->Matrix2(0);
  long lipschitz_dimensions = training_embeddings->Cols();
  vMatrix<float> training_embedding(1, lipschitz_dimensions);
  vArray(float) training = training_embedding.Matrix();
  vPrint("\n");
  for (i = 0; i < number; i++)
  {
    long j;
    // Get training embedding.
    for (j = 0; j < lipschitz_dimensions; j++)
    {
      training[j] = lipschitz[i][j];
    }

    // Adjust training embedding
    vMatrix<float> adjusted_embedding(1, 2*dimensions);
    vArray(float) adjusted = adjusted_embedding.Matrix();
    vMatrix<long> unique_indices = ceb->UniqueIndices();
    long unique_dimensions = training_embedding.Size();
    if (2*dimensions != unique_indices.Size())
    {
      exit_error("Error: dimensions != unique_indices.Size(): %li %li\n",
                      dimensions, unique_indices.Size());
    }

    long k;
    for (k = 0; k < 2*dimensions; k++)
    {
      long index = unique_indices(k);
      if (index < 0)
      {
        continue;
      }
      if (index >= unique_dimensions)
      {
        exit_error("Error: index >= unique_dimensions: %li %li\n",
                        index, unique_dimensions);
      }
      adjusted[k] = training[index];
    }
    // Get ceb embedding
    vMatrix<float> ceb_embedding = ceb->Embedding(adjusted_embedding);
    vArray(float) ceb = ceb_embedding.Matrix();
    for (j = 0; j < dimensions; j++)
    {
      ceb_matrix[i][j] = ceb[j];
    }
    if (i % 1000 == 0)
    {
      vPrint("frame %li\r", i);
    }
  }

  vPrint("\n");

  return 1;
}


long vInterpreter3::CEB_Scores()
{
  if (input == 0) return 0;
  if (training_embeddings == 0) return 0;
  if (ceb == 0) return 0;
  if (ceb_embeddings == 0) return 0;
  if (reference_images == 0) return 0;
  if (vIsZero(distance_transforms) == 0)
  {
    vdelete(test_embedding);
    grayscale_image * current = input->current_image->ToGrayCopy();
    test_embedding = vChamferEmbedding(current,
                                        distance_transforms, point_sequence,
                                        reference_images, reference_start,
                                        embedding_dimensions);
    vdelete(current);
  }
  else
  {
    ChamferDistances();
  }
 
  // Before we call the vMatchScores constructor, we need to construct
  // a weight matrix. Usually (in BM_Scores, for example) we pass in
  // reference images, which contains weights on column 2. However, here
  // we want to pass in the weights of the classifiers, not those of the
  // images.
  vMatrix<float> classifiers = ceb->UniqueClassifiers();
  vMatrix<float> weights_matrix(ceb->Dimensions(), 3);
  long dimensions = ceb->Dimensions();
  long i;
  for (i = 0; i < dimensions; i++)
  {
    weights_matrix(i, 2) = classifiers(i, 3);
  }

  vMatrix<float> adjusted_embedding(1, 2*dimensions);
  vArray(float) adjusted = adjusted_embedding.Matrix();
  vMatrix<long> unique_indices = ceb->UniqueIndices();
  long unique_dimensions = test_embedding->Size();
  if (2*dimensions != unique_indices.Size())
  {
    exit_error("Error: dimensions != unique_indices.Size(): %li %li\n",
                    dimensions, unique_indices.Size());
  }

  for (i = 0; i < 2*dimensions; i++)
  {
    long index = unique_indices(i);
    if (index < 0)
    {
      continue;
    }
    if (index >= unique_dimensions)
    {
      exit_error("Error: index >= unique_dimensions: %li %li\n",
                      index, unique_dimensions);
    }
    adjusted[i] = test_embedding->Matrix()[index];
  }

  vMatrix<float> test_ceb_embedding = ceb->Embedding(adjusted_embedding);

  vdelete(match_scores);
  match_scores = new vMatchScores(ceb_embeddings, &test_ceb_embedding,
                                   0, ceb->Dimensions(),
                                   &weights_matrix);
  UpdateMatchScores(input->current_frame);
  return 1;
}


// Based on the current sequence_match_stats object, which is
// supposed to know for each frame the exact nearest neighbor,
// figure out the rank of that neighbor at each frame as computed
// bo score_script.
long vInterpreter3::NN_RankStats0()
{
  if (input == 0) return 0;
  if (sequence_match_stats == 0) return 0;

  char score_script[1000];
  long start_frame = 0, end_frame = 0;
  vPrint("\nEnter score_script, start_frame, end_frame:\n");
  vScan("%s %li %li", score_script, &start_frame, &end_frame);
  return NN_RankStats(score_script, start_frame, end_frame);
}


long vInterpreter3::NN_RankStats(const char * score_script, long start_frame, 
                                  long end_frame)
{
  if (input == 0) return 0;
  if (sequence_match_stats == 0) return 0;

  long frames = input->number_of_frames;
  if (sequence_match_stats->NumberOfFrames() != frames)
  {
    vPrint("sequence_match_stats has %li frames instead of %li\n",
            sequence_match_stats->NumberOfFrames(), frames);
    return 0;
  }

  long input_index = current_input_index;
  if (start_frame < 0) start_frame = 0;
  if (start_frame >= frames)
  {
    start_frame = frames - 1;
  }
  if (end_frame < 0) end_frame = input->number_of_frames - 1;
  if (end_frame >= frames) end_frame = frames-1;


  vector<long> results;
  long frame;
  for (frame = start_frame; frame <= end_frame; frame = frame + 1)
  {
    SetCurrentInput(input_index);
    JumpToFrame(frame);
    base_interpreter->process_command_file1(score_script);
    if (match_scores == 0)
    {
      vPrint("Failed to calculate match_scores.\n");
      return 0;
    }

    SetCurrentInput(input_index);
    UpdateMatchScores(input->current_frame);
    
    vFrameMatchStats * frame_stats = sequence_match_stats->FrameStats(frame);
    if (frame_stats == 0)
    {
      exit_error("Error: frame_stats = 0\n");
    }

    vMatchScore top_score = frame_stats->TopScore(0);
    if (top_score.rank != 0)
    {
      exit_error("Error: top score rank = %li != 0\n",
                      top_score.rank);
    }

    long top_frame = top_score.frame;
    long top_view = top_score.view;
    vMatchScore current_score = match_scores->GetRank(top_frame, top_view);
    long current_rank = current_score.rank;
    results.push_back(current_rank);
    long current_size = results.size();

    vPrint("\n\nDone with frame %li, rank = %li\n\n\n", 
            frame, current_rank);
  }

  long size = results.size();
  long i;
  for (i = 0; i < size; i++)
  {
    vPrint("%6li  %6li\n", i, results[i]);
  }

  return 1;
}


long vInterpreter3::CombineSequenceStats0()
{
  char in_file1[1000];
  char in_file2[1000];
  char out_file[1000];

  vPrint("\nEnter in_file1, in_file2, out_file:\n");
  vScan("%s %s %s", in_file1, in_file2, out_file);
  return CombineSequenceStats(in_file1, in_file2, out_file);
}


long vInterpreter3::CombineSequenceStats(const char * in_file1, const char * in_file2, 
                                          const char * out_file)
{
  LoadSequenceStats(in_file1);
  if (sequence_match_stats == 0) 
  {
    vPrint("Failed to read %s\n", in_file1);
    return 0;
  }

  vSequenceMatchStats * in_stats1 = sequence_match_stats;
  sequence_match_stats = 0;

  LoadSequenceStats(in_file2);
  if (sequence_match_stats == 0) 
  {
    vPrint("Failed to read %s\n", in_file2);
    vdelete(in_stats1);
    return 0;
  }

  vSequenceMatchStats * in_stats2 = sequence_match_stats;
  sequence_match_stats = in_stats1;

  long number1 = in_stats1->NumberOfFrames();
  long number2 = in_stats2->NumberOfFrames();

  long i;
  for (i = 0; i < number2; i++)
  {
    vFrameMatchStats * frame_stats = in_stats2->FrameStats2(i);
    sequence_match_stats->AddFrameStats(frame_stats, i+number1);
  }

  in_stats1->Print();
  SaveSequenceStats(out_file);
  in_stats2->RemoveAllStats();
  vdelete(in_stats2);

  return 1;
}


long vInterpreter3::WriteBigDistances0()
{
  long number = 0;
  vPrint("\nEnter number:\n");
  vScan("%li", &number);
  return WriteBigDistances(number);
}


long vInterpreter3::WriteBigDistances(long number)
{
  long success = vWriteBigDistancesFile(number);
  vPrint("success = %li\n", success);
  return success;
}


long vInterpreter3::ReadBigDistances0()
{
  long number = 0, frame = 0;
  vPrint("\nEnter number, frame:\n");
  vScan("%li %li", &number, &frame);
  return ReadBigDistances(number, frame);
}


long vInterpreter3::ReadBigDistances(long number, long frame)
{
  vMatrix<long> samples(1, number);
  vMatrix<float> distances = vReadBigDistancesFile(samples, frame);
  return 1;
}


/////////////////////////////////////////////////////////////////////
//                                                                 //
//           End of functions written for CVPR 2004.               //
//                                                                 //
/////////////////////////////////////////////////////////////////////

// Test the function vComplement.
long vInterpreter3::Complement0()
{
  long number = 0, start = 0, end = 0;
  vPrint("\nEnter number of entries, start, end:\n");
  vScan("%li %li %li", &number, &start, &end);
  return Complement(number, start, end);
}


long vInterpreter3::Complement(long number, long start, long end)
{
  vMatrix<long> matrix = vSampleWithoutReplacement(start, end, number);
  vMatrix<long> complement = vComplement(&matrix, start, end);
  matrix.PrintInt("matrix");
  complement.PrintInt("complement");
  return 1;
}
  
  
// Test the function vPermutation.
long vInterpreter3::Permutation0()
{
  long start = 0, end = 0;
  vPrint("\nEnter low value, high value:\n");
  vScan("%li %li", &start, &end);
  return Permutation(start, end);
}


long vInterpreter3::Permutation(long start, long end)
{
  vMatrix<long> matrix = vPermutation(start, end);
  matrix.PrintInt("permutation");
  return 1;
}
  


/////////////////////////////////////////////////////////////////////
//                                                                 //
//           Start of functions written for KDD 2004.              //
//                                                                 //
/////////////////////////////////////////////////////////////////////


long vInterpreter3::MnistLoad0()
{
  long dataset = 0;
  vPrint("\nEnter 0 (training) or 1 (test)\n");
  vScan("%li", &dataset);
  return MnistLoad3(dataset, -1, -1);

}


long vInterpreter3::MnistLoad(long dataset)
{
  return MnistLoad3(dataset, -1, -1);
}

  
long vInterpreter3::MnistLoad30()
{
  long dataset = 0, start = 0, end = 0;
  vPrint("\nEnter 0 (training) or 1 (test), start, end:\n");
  vScan("%li %li %li", &dataset, &start, &end);
  return MnistLoad3(dataset, start, end);
}


long vInterpreter3::MnistLoad3(long dataset, long start, long end)
{
  if ((dataset < 0) || (dataset > 1))
  {
    return 0;
  }
  if (dataset == 0)
  {
    vdelete(training_mnist);
    training_mnist = new vMnistSet(dataset, start, end);
  }
  else if (dataset == 1)
  {
    vdelete(test_mnist);
    test_mnist = new vMnistSet(dataset, start, end);
  }

  return 1;
}


long vInterpreter3::MnistShow0()
{
  if ((training_mnist == 0) && (test_mnist == 0))
  {
    vPrint("No mnist set is loaded\n");
    return 0;
  }

  long index = 0;
  vPrint("\nEnter index:\n");
  vScan("%li", &index);
  return MnistShow(index);
}


long vInterpreter3::MnistShow(long number)
{
  if (number < 0)
  {
    return 0;
  }

  if ((training_mnist == 0) && (test_mnist == 0))
  {
    return 0;
  }

  if (input == 0)
  {
    default_input(300, 300);
  }

  vPrint("index = %li:\n", number);
  
  if ((training_mnist != 0) && (number < training_mnist->Number()))
  {
    grayscale_image * training_image = training_mnist->Digit(number);
    long training_label = training_mnist->Label(number);
    ShowImage(training_image->ToGrayCopy());
    vPrint("training_label = %li\n", training_label);
  }

  if ((test_mnist != 0) && (number < test_mnist->Number()))
  {
    grayscale_image * test_image = test_mnist->Digit(number);
    long test_label = test_mnist->Label(number);
    ShowImage(test_image->ToGrayCopy());
    vPrint("test_label = %li\n", test_label);
  }

  return 1;
}


long vInterpreter3::MnistDataset20()
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  char name[1000];
  long number = 0;
  vPrint("\nEnter name, number:\n");
  vScan("%s %li", name, &number);
  return MnistDataset2(name, number);
}


long vInterpreter3::MnistDataset2(const char * name, long number)
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  vdelete(bm_data);
  vMnistBmSet mnist_set(training_mnist);
  bm_data = mnist_set.BmDataset(name, number);
  return 1;
}


long vInterpreter3::MnistDataset40()
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  char name[1000];
  long candidates = 0, training = 0, validation = 0;
  vPrint("\nEnter name, candidates, training, validation:\n");
  vScan("%s %li %li %li", name, &candidates, &training, &validation);
  return MnistDataset4(name, candidates, training, validation);
}


long vInterpreter3::MnistDataset4(const char * name, long candidates, 
                                  long training, long validation)
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  vdelete(bm_data);
  vMnistBmSet mnist_set(training_mnist);
  bm_data = mnist_set.BmDataset(name, candidates, training, validation);
  return 1;
}


long vInterpreter3::MnistCandidates()
{
  if (training_mnist == 0)
  {
    return 0;
  }
  if (bm_data == 0) 
  {
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.CandCandDistances(bm_data);
  mnist_set.CandTrainDistances(bm_data);
  mnist_set.CandValDistances(bm_data);
  return 1;
}


long vInterpreter3::MnistTraining()
{
  if (training_mnist == 0)
  {
    return 0;
  }
  if (bm_data == 0) 
  {
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.TrainingDistances(bm_data);
  return 1;
}


long vInterpreter3::MnistValidation()
{
  if (training_mnist == 0)
  {
    return 0;
  }
  if (bm_data == 0) 
  {
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.ValidationDistances(bm_data);
  return 1;
}


long vInterpreter3::MnistWholeDataset()
{
  if (training_mnist == 0)
  {
    return 0;
  }
  if (bm_data == 0) 
  {
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.CandCandDistances(bm_data);
  mnist_set.CandTrainDistances(bm_data);
  mnist_set.CandValDistances(bm_data);
  mnist_set.TrainingDistances(bm_data);
  mnist_set.ValidationDistances(bm_data);
  mnist_set.TrainingPdistances(bm_data);
  mnist_set.ValidationPdistances(bm_data);
  return 1;
}

long vInterpreter3::MnistLabels()
{
  if ((training_mnist == 0) || (test_mnist == 0))
  {
    vPrint("Both training and test mnist dataset should be loaded.\n");
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.ClassLabels("mnist", test_mnist);
  vPrint("\nDone\n");
  return 1;
}


long vInterpreter3::MnistTestTrain()
{
  if ((training_mnist == 0) || (test_mnist == 0))
  {
    vPrint("Both training and test mnist dataset should be loaded.\n");
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.TestTrainDistances("mnist", test_mnist);
  vPrint("\nDone\n");
  return 1;
}


long vInterpreter3::MnistTrainTrain()
{
  if (training_mnist == 0)
  {
    vPrint("training mnist dataset should be loaded.\n");
    return 0;
  }

  vMnistBmSet mnist_set(training_mnist);
  mnist_set.TrainTrainDistances("mnist");
  vPrint("\nDone\n");
  return 1;
}





long vInterpreter3::MnistChamfer0()
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  long index1 = 0, index2 = 0;
  vPrint("\nenter index1, index2:\n");
  vScan("%li %li", &index1, &index2);
  return MnistChamfer(index1, index2);
}


long vInterpreter3::MnistChamfer(long index1, long index2)
{
  if (training_mnist == 0) 
  {
    return 0;
  }

  grayscale_image * digit1 = training_mnist->Digit(index1);
  grayscale_image * digit2 = training_mnist->Digit(index2);

  if ((digit1 == 0) || (digit2 == 0))
  {
    vPrint("bad digits: %li %li\n", index1, index2);
  }
  else
  {
    if (input == 0)
    {
      default_input(300, 300);
    }

    ShowImage(digit1->ToGrayCopy());
    ShowImage(digit2->ToGrayCopy());
    grayscale_image * edges1 = vCannyDetector(digit1, (uchar) canny_threshold);
    grayscale_image * edges2 = vCannyDetector(digit2, (uchar) canny_threshold);
    ShowImage(edges1);
    ShowImage(edges2);
    v3dMatrix<float> * dt1 = vCVDistanceTransform(edges1);
    v3dMatrix<float> * dt2 = vCVDistanceTransform(edges2);

    float distance, distance1, distance2;
    distance1 = (float) vChamferFromDT(edges1, dt2);
    distance2 = (float) vChamferFromDT(edges2, dt1);
    distance = distance1 + distance2;

    vPrint("distance1 = %f, distance2 = %f, distance = %f\n",
           distance1, distance2, distance);

    vdelete(dt1);
    vdelete(dt2);
  }

  return 1;
}




char * vInterpreter3::EmbeddingDirectory2()
{
  char * result = vJoinPaths4(data_directory, "experiments", 
                               "bm_datasets", "fastmap");
  return result;
}


long vInterpreter3::FastMapFilterRefine()
{
  long dimensions = 0, max_k = 0, to_keep = 0;
  vPrint("\nenter dataset, test/training embedding files, dimensions, to_keep, max_k:\n");
  char dataset[1000], test_name[1000], train_name[1000];
  vScan("%s %s %s %li %li %li", 
        dataset, test_name, train_name, &dimensions, &to_keep, &max_k);
  
  char * directory = EmbeddingDirectory2();
  char * test_file = vJoinPaths(directory, test_name);
  char * train_file = vJoinPaths(directory, train_name);
  
  vdelete2(directory);

  vMatrix<float> test_embedding2;
  vMatrix<float> train_embedding2;

  long problems = 0;

  test_embedding2 = vMatrix<float>::Read(test_file);
  if (test_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", test_file);
    problems = 1;
  }

  train_embedding2 = vMatrix<float>::Read(train_file);
  if (train_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", train_file);
    problems = 1;
  }

  vdelete2(test_file);
  vdelete2(train_file);

  if (problems != 0)
  {
    return 0;
  }

  long test_size = test_embedding2.Rows();
  long training_size = train_embedding2.Rows();
  vMatrix<float> test_embedding(test_size, dimensions);
  vMatrix<float> train_embedding(training_size, dimensions);

  test_embedding2.Copy(&test_embedding, 0, test_size-1, 0, dimensions-1, 0, 0);
  train_embedding2.Copy(&train_embedding, 0, training_size-1, 0, dimensions-1, 0, 0);

  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 1.0);

  vMatrix<float> test_labels = BoostMap_dataset::LoadTestLabels(dataset);
  vMatrix<float> train_labels = BoostMap_dataset::LoadTrainingLabels(dataset);

  char * distances_file = BoostMap_dataset::TestTrainDistancesPath(dataset);
  class_file * fp = BoostMap_dataset::OpenObjectDistancesFile(distances_file);
  vdelete2(distances_file);

  if (fp == 0)
  {
    vPrint("failed to open distances file\n");
    return 0;
  }

  vMatrix<float> result = BoostMap_dataset::FilterRefineL2(test_embedding, train_embedding,
                                                     weights, test_labels, train_labels,
                                                     max_k, 0, to_keep, fp);
  
  fclose(fp);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    long i;
    long min_index = -1;
    float min_error = (float) 1000;
    for (i = 0; i <= max_k; i++)
    {
      float error = result(i);
      vPrint("%li-nn error: %f\n", i, error);
      if (i != 0)
      { 
        if (error < min_error)
        {
          min_error = error;
          min_index = i;
        }
      }
    }
    vPrint("\nbest k: %li. best knn-error: %f\n", min_index, min_error);
  }

  return 1;
}


long vInterpreter3::FastMapKnn()
{
  long dimensions = 0, max_k = 0;
  vPrint("\nenter dataset, test/training embedding files, dimensions, max_k:\n");
  char dataset[1000], test_name[1000], train_name[1000];
  vScan("%s %s %s %li %li", dataset, test_name, train_name, &dimensions, &max_k);
  
  char * directory = EmbeddingDirectory2();
  char * test_file = vJoinPaths(directory, test_name);
  char * train_file = vJoinPaths(directory, train_name);
  
  vdelete2(directory);

  vMatrix<float> test_embedding2;
  vMatrix<float> train_embedding2;

  long problems = 0;

  test_embedding2 = vMatrix<float>::Read(test_file);
  if (test_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", test_file);
    problems = 1;
  }

  train_embedding2 = vMatrix<float>::Read(train_file);
  if (train_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", train_file);
    problems = 1;
  }

  vdelete2(test_file);
  vdelete2(train_file);

  if (problems != 0)
  {
    return 0;
  }

  long test_size = test_embedding2.Rows();
  long training_size = train_embedding2.Rows();
  vMatrix<float> test_embedding(test_size, dimensions);
  vMatrix<float> train_embedding(training_size, dimensions);

  test_embedding2.Copy(&test_embedding, 0, test_size-1, 0, dimensions-1, 0, 0);
  train_embedding2.Copy(&train_embedding, 0, training_size-1, 0, dimensions-1, 0, 0);

  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 1.0);

  vMatrix<float> test_labels = BoostMap_dataset::LoadTestLabels(dataset);
  vMatrix<float> train_labels = BoostMap_dataset::LoadTrainingLabels(dataset);

  vMatrix<float> result = BoostMap_dataset::KnnError7L2(test_embedding, train_embedding,
                                                  weights, test_labels, 
                                                  train_labels, max_k, 0);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    long i;
    long min_index = -1;
    float min_error = (float) 1000;
    for (i = 0; i <= max_k; i++)
    {
      float error = result(i);
      vPrint("%li-nn error: %f\n", i, error);
      if (i != 0)
      { 
        if (error < min_error)
        {
          min_error = error;
          min_index = i;
        }
      }
    }
    vPrint("\nbest k: %li. best knn-error: %f\n", min_index, min_error);
  }

  return 1;
}


long vInterpreter3::FastMapIndexErrors()
{
  long dimensions = 0, max_k = 0;
  vPrint("\nenter dataset, test/training embedding files, dimensions, max_k:\n");
  char dataset[1000], test_name[1000], train_name[1000];
  vScan("%s %s %s %li %li", dataset, test_name, train_name, &dimensions, &max_k);
  
  char * directory = EmbeddingDirectory2();
  char * test_file = vJoinPaths(directory, test_name);
  char * train_file = vJoinPaths(directory, train_name);
  
  vdelete2(directory);

  vMatrix<float> test_embedding2;
  vMatrix<float> train_embedding2;

  long problems = 0;

  test_embedding2 = vMatrix<float>::Read(test_file);
  if (test_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", test_file);
    problems = 1;
  }

  train_embedding2 = vMatrix<float>::Read(train_file);
  if (train_embedding2.IsValid() <= 0)
  {
    vPrint("failed to read from %s\n", train_file);
    problems = 1;
  }

  vdelete2(test_file);
  vdelete2(train_file);

  if (problems != 0)
  {
    return 0;
  }

  long test_size = test_embedding2.Rows();
  long training_size = train_embedding2.Rows();
  vMatrix<float> test_embedding(test_size, dimensions);
  vMatrix<float> train_embedding(training_size, dimensions);

  test_embedding2.Copy(&test_embedding, 0, test_size-1, 0, dimensions-1, 0, 0);
  train_embedding2.Copy(&train_embedding, 0, training_size-1, 0, dimensions-1, 0, 0);

  vMatrix<float> weights(1, dimensions);
  vWriteValue(&weights, (float) 1.0);

  char * distances_file = BoostMap_dataset::TestTrainDistancesPath(dataset);
  class_file * fp = BoostMap_dataset::OpenObjectDistancesFile(distances_file);
  vdelete2(distances_file);
  if (fp == 0)
  {
    vPrint("failed to open distances file\n");
    return 0;
  }

  vMatrix<float> result = BoostMap_dataset::IndexErrors6L2(test_embedding, train_embedding,
                                                     weights, fp, 0, max_k);
  fclose(fp);

  if (result.IsValid() == 0)
  {
    vPrint("invalid result\n");
  }
  else
  {
    v3dMatrix<float> temp_ranks = copy_vertical_line(&result, max_k);

    vector<float> ranks;
    vector_from_matrix(&temp_ranks, &ranks);
    std::sort(ranks.begin(), ranks.end(), less<float>());
    long size = ranks.size();
    float number = 100;
    float i;
    for (i = 0; vRound(i) <= vRound(number); i = i + (float) 1.0)
    {
      float percentile = i * ((float) 100.0 / number);
      long index = vRound(((float) size) / number * i);
      if (index == size)
      {
        index--;
      }
      float entry = ranks[index];

      vPrint("%li: %5.2f, %f\n", vRound(index), percentile, entry);
    }
  }

  char * output = BoostMap_dataset::IndexErrorOutputPath2(dataset, "fastmap", 
                                                    dimensions);

  long success = result.Write(output);
  if (success <= 0)
  {
    vPrint("failed to save result to %s\n", output);
  }
  else
  {
    vPrint("saved result to %s\n", output);
  }
  vdelete2(output);

  return 1;
}


char * vInterpreter3::EmbeddingDirectory()
{
  char * result = vJoinPaths4(data_directory, "experiments", 
                               "bm_datasets", "embeddings");
  return result;
}


long vInterpreter3::FmMake20()
{
  char dataset[1000];
  char sample_name[1000];
  vPrint("\nenter dataset, sample_name\n");
  vScan("%s %s", dataset, sample_name);
  return FmMake2(dataset, sample_name);
}


long vInterpreter3::FmMake2(const char * dataset, const char * sample_name)
{
  class_FastMap * new_fm = new class_FastMap(dataset, sample_name);
  if (new_fm->IsValid() > 0)
  {
    vPrint("created new fastmap\n");
    FmDelete();
    fm = new_fm;
  }
  else
  {
    vPrint("failed to make fastmap\n");
  }
  return 1;
}


long vInterpreter3::FmMake30()
{
  char dataset[1000];
  char sample_name[1000];
  char saved_file[1000];
  vPrint("\nenter dataset, sample_name, saved_file\n");
  vScan("%s %s %s", dataset, sample_name, saved_file);
  return FmMake3(dataset, sample_name, saved_file);
}


long vInterpreter3::FmMake3(const char * dataset, const char * sample_name,
                            const char * saved_file)
{
  class_FastMap * new_fm = new class_FastMap(dataset, sample_name, saved_file);
  if (new_fm->IsValid() > 0)
  {
    vPrint("created new fastmap\n");
    FmDelete();
    fm = new_fm;
  }
  else
  {
    vPrint("failed to make fastmap\n");
  }
  return 1;
}


long vInterpreter3::FmDelete()
{
  vdelete(fm);
  fm = 0;
  return 1;
}


long vInterpreter3::FmNextStep()
{
  if (fm != 0)
  {
    fm->NextStep();
  }

  return 1;
}


long vInterpreter3::FmNextSteps0()
{
  if (fm == 0)
  {
    return 0;
  }

  long steps = 0;
  vPrint("\nenter steps:\n");
  vScan("%li", &steps);

  return FmNextSteps(steps);
}


long vInterpreter3::FmNextSteps(long steps)
{
  if (fm == 0)
  {
    return 0;
  }

  long i;
  for (i = 0; i < steps; i++)
  {
    FmNextStep();
  }

  return 1;
}


long vInterpreter3::FmEmbedTest()
{
  if (fm == 0)
  {
    return 0;
  }

  fm->EmbedTestSet();
  return 1;
}


long vInterpreter3::FmEmbedTraining()
{
  if (fm == 0)
  {
    return 0;
  }

  fm->EmbedTrainingSet();
  return 1;
}


long vInterpreter3::FmSave0()
{
  if (fm == 0)
  {
    return 0;
  }

  char filename[1000];
  vPrint("\nenter filename:\n");
  vScan("%s", filename);
  return FmSave(filename);
}


long vInterpreter3::FmSave(const char * filename)
{
  if (fm == 0)
  {
    return 0;
  }

  fm->Save(filename);
  return 1;
}


long vInterpreter3::FmPrintSample0()
{
  if (fm == 0)
  {
    return 0;
  }

  long i = 0;
  vPrint("\nenter i:\n");
  vScan("%li", &i);
  return FmPrintSample(i);
}


long vInterpreter3::FmPrintSample(long i)
{
  if (fm == 0)
  {
    return 0;
  }

  fm->PrintSampleObject(i);
  return 1;
}


long vInterpreter3::FmPrintTraining0()
{
  if (fm == 0)
  {
    return 0;
  }

  long i = 0;
  vPrint("\nenter i:\n");
  vScan("%li", &i);
  return FmPrintTraining(i);
}


long vInterpreter3::FmPrintTraining(long i)
{
  if (fm == 0)
  {
    return 0;
  }

  fm->PrintTrainingObject(i);
  return 1;
}


long vInterpreter3::FmPrintPivots()
{
  if (fm == 0)
  {
    return 0;
  }

  fm->PrintPivots();
  return 1;
}


long vInterpreter3::FmTest()
{
  long dimensions = 0, points = 0, triples = 0;

  vPrint("\nenter dimensions, points, triples:\n");
  vScan("%li %li %li", &dimensions, &points, &triples);

  vMatrix<float> data(points, dimensions);
  long row, col;
  for (row = 0; row < points; row++)
  {
    for (col = 0; col < dimensions; col++)
    {
      double pick = vRandomDouble(0.0, 1.0);
      float casted = (float) pick;
      data(row, col) = casted;
    }
  }

  vPrint("picked points\n");
  vMatrix<float> distances(points, points);
  for (row = 0; row < points; row++)
  {
    for (col = 0; col < points; col++)
    {
      float distance = 0.0;
      long i;
      for (i = 0; i < dimensions; i++)
      {
        float x = data(row, i);
        float y = data(col, i);

        float diff = x - y;
        distance = distance + diff * diff;  // for L2 original space
        distance = distance + vAbs(diff);  // for L1 original space
      }
      distance = sqrt(distance);
      distances(row, col) = distance;
    }
    vPrint("done with row %li of %li\r", row+1, points);
  }
  vPrint("\n");
  DeleteBM();
  boost_map = new V_FastMap(distances, triples);
  vPrint("done\n");
  return 1;
}


// SaveEdgeImages is used to save the database of edge
// images, so that I can give it to other people or just
// put it on the web. For the time being, I will
// just save it all as an uncompressed AVI file.
long vInterpreter3::SaveEdgeImages0()
{
  if (input == 0) 
  {
    return 0;
  }
  char filename[1000];
  vPrint("\nEnter filename:\n");
  vScan("%s", filename);
  return SaveEdgeImages(filename);
}

long vInterpreter3::SaveEdgeImages(const char * filename)
{
  if (input == 0) 
  {
    return 0;
  }
  const long step = 12;
  vVideoFile * sequence = input->sequence;
  
  long rows = sequence->Rows();
  long cols = sequence->Cols();
  long frames = sequence->NumberOfFrames();
  if (frames % step != 0)
  {
    exit_error("Error: frames = %li, step = %li\n", frames, step);
  }

  bitmap_sequence bmp_sequence(filename, rows, cols, 1);
  if (bmp_sequence.IsValid() <= 0)
  {
    vPrint("Failed to open %s for writing\n", filename);
    return 0;
  }

  long frame;
  vPrint("\n");
  for (frame = 0; frame < frames; frame = frame + step)
  {
    grayscale_image * gray = sequence->ReadFrameGray(frame);
    if (gray == 0)
    {
      vPrint("failed to load frame %li\n", frame);
      break;
    }

    vArray(uchar) data = gray->Matrix();
    long size = rows * cols;
    long j;
    for (j = 0; j < size; j++)
    {
      if (data[j] != 0)
      {
        data[j] = data[j] + (uchar) 200;
      }
    }
    long success = bmp_sequence.write_frameGray(gray);
    vdelete(gray);
    if (success == 0)
    {
      vPrint("failed to write frame %li\n", frame);
      break;
    }
    
    vPrint("saved frame %li of %li\r", frame, frames);
  }
  vPrint("\n");
  bmp_sequence.FinishWriting();

  return 1;
}


// ConvertSequence takes a sequence of frames, from the current
// input, and converts each frame by applying to it a given script,
// and then saves the output of that frame to the output sequence.
long vInterpreter3::ConvertSequence0()
{
  if (input == 0) 
  {
    vPrint("no input sequence available\n");
    return 0;
  }

  if (current_output == -1) 
  {
    vPrint("no output sequence available\n");
    return 0;
  }

  char filename[1000];
  long start = 0, end = 0, step = 0;
  vPrint("\nEnter script, start, end, step:\n");
  vScan("%s %li %li %li", filename, &start, &end, &step);
  return ConvertSequence(filename, start, end, step);
}


long vInterpreter3::ConvertSequence(const char * script_file, long start, 
                                    long end, long step)
{
  if (input == 0) 
  {
    vPrint("no input sequence available\n");
    return 0;
  }

  if (current_output == -1) 
  {
    vPrint("no output sequence available\n");
    return 0;
  }

  vVideoFile * sequence = input->sequence;
  if (sequence == 0)
  {
    vPrint("no sequence associated with current input\n");
    return 0;
  }

  long frames = sequence->NumberOfFrames();
  vVideoFile * output = GetOutput(current_output);
  if (start == -1)
  {
    start = 0;
  }
  if (end == -1)
  {
    end = frames-1;
  }
  if (step == -1)
  {
    step = 1;
  }

  long bytes_per_pixel = output->BytesPerPixel();
  vPrint("\n");
  long i;
  for (i = start; i <= end; i = i + step)
  {
    JumpToFrame(i);
    base_interpreter->process_command_file1(script_file);
    long success = 0;

    if (bytes_per_pixel == 1)
    {
      grayscale_image * the_image = input->current_image->ToGrayCopy();
      success = output->write_frameGray(the_image);
      vdelete(the_image);
    }
    else if (bytes_per_pixel == 3)
    {
      color_image * the_image = input->current_image->ToRGBCopy();
      success = output->write_frame(the_image);
      vdelete(the_image);
    }
    else 
    {
      exit_error("bad bytes per pixel = %li, should be 1 or 3\n",
                      bytes_per_pixel);
    }

    if (success <= 0)
    {
      vPrint("failed to write frame %li\n", i);
      break;
    }

    vPrint("wrote frame %li\r", i);
  }
  vPrint("\n");
  return 1;
}


long vInterpreter3::PrintClassificationError(vMatrix<float> result)
{
  long max_k = result.Size() - 1;

  long i;
  long min_index = -1;
  float min_error = (float) 1000;
  for (i = 0; i <= max_k; i++)
  {
    float error = result(i);
    vPrint("%li-nn error: %f\n", i, error);
    if (i != 0)
    { 
      if (error < min_error)
      {
        min_error = error;
        min_index = i;
      }
    }
  }
  vPrint("1-nn error: %f\n", result(1));
  vPrint("\nbest k: %li. best knn-error: %f\n", min_index, min_error);

  return 1;
}
